# -*- coding: utf-8 -*-
class BannersController < ApplicationController
  class_desc :display_name => 'バナー管理'

  # GET /banners
  # GET /banners.xml
  def index
    @condition = prepare_session_condition do |condition|
      condition || Banner::SearchForm.new(params[:condition])
    end

    @condition.page = params[:page]
    @banners = Banner.paginate(
      @condition.to_paginate_options(
        :include => :displays))

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @banners }
    end
  end

  # GET /banners/1
  # GET /banners/1.xml
  def show
    @banner = Banner.find(params[:id], :include => [:displays, :conditions])
    @banner.setup_displays_for

    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @banner }
    end
  end

  # GET /banners/new
  # GET /banners/new.xml
  def new
    @banner = Banner.new
    @banner.setup_displays_for
    @banner.setup_condition_at_least_1

    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @banner }
    end
  end

  # GET /banners/1/edit
  def edit
    @banner = Banner.find(params[:id])
    @banner.setup_displays_for
  end

  # POST /banners
  # POST /banners.xml
  def create
    @banner = Banner.new(params[:banner])

    respond_to do |format|
      if @banner.save
        flash[:notice] = 'バナーを登録しました。'
        format.html { redirect_to(@banner) }
        format.xml  { render :xml => @banner, :status => :created, :location => @banner }
      else
        format.html { render :action => "new" }
        format.xml  { render :xml => @banner.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /banners/1
  # PUT /banners/1.xml
  def update
    @banner = Banner.find(params[:id])

    respond_to do |format|
      if @banner.update_attributes(params[:banner])
        flash[:notice] = 'バナーを更新しました。'
        format.html { redirect_to(@banner) }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @banner.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /banners/1
  # DELETE /banners/1.xml
  def destroy
    @banner = Banner.find(params[:id])
    @banner.destroy

    respond_to do |format|
      format.html { redirect_to(banners_url) }
      format.xml  { head :ok }
    end
  end
end
