# -*- coding: utf-8 -*-
class CampaignConditionsController < ApplicationController
  class_desc :display_name => 'キャンペーン条件管理(Ajax)', 
    :auth_as => 'campaigns', :invisible => true

  include NestedAjax::RenderExt

  # GET /campaign_conditions
  # GET /campaign_conditions.xml
  def index
    @campaign_conditions = CampaignCondition.all

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @campaign_conditions }
    end
  end

  # GET /campaign_conditions/1
  # GET /campaign_conditions/1.xml
  def show
    @campaign_condition = CampaignCondition.find(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @campaign_condition }
    end
  end

  # GET /campaign_conditions/new
  # GET /campaign_conditions/new.xml
  def new
    @campaign_condition = CampaignCondition.new(params[:campaign_condition])

    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @campaign_condition }
    end
  end

  # GET /campaign_conditions/1/edit
  def edit
    @campaign_condition = CampaignCondition.find(params[:id])
  end

  # POST /campaign_conditions
  # POST /campaign_conditions.xml
  def create
    @campaign_condition = CampaignCondition.new(params[:campaign_condition])

    respond_to do |format|
      if @campaign_condition.save
        flash[:notice] = 'CampaignCondition was successfully created.'
        format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'show', :id => @campaign_condition.id) }
        format.xml  { render :xml => @campaign_condition, :status => :created, :location => @campaign_condition }
      else
        format.html { render_if_xhr(:action => 'new') || render(:action => "new") }
        format.xml  { render :xml => @campaign_condition.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /campaign_conditions/1
  # PUT /campaign_conditions/1.xml
  def update
    @campaign_condition = CampaignCondition.find(params[:id])
    
    respond_to do |format|
      if @campaign_condition.update_attributes(params[:campaign_condition])
        # belongs_to, has_one, has_many などの関連のキャッシュをクリアしておきます
        @campaign_condition.clear_association_cache
        # composed_of などの集約のキャッシュをクリアしておきます
        @campaign_condition.clear_aggregation_cache
        flash[:notice] = 'CampaignCondition was successfully updated.'
        format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'show', :id => @campaign_condition.id) }
        format.xml  { head :ok }
      else
        format.html { render_if_xhr(:action => 'edit') || render(:action => "edit") }
        format.xml  { render :xml => @campaign_condition.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /campaign_conditions/1
  # DELETE /campaign_conditions/1.xml
  def destroy
    @campaign_condition = CampaignCondition.find(params[:id])
    @campaign_condition.destroy
    flash[:notice] = 'CampaignCondition was successfully deleted.'
    respond_to do |format|
      # いきなりザクッと消したい場合はこんな感じで。ただし例外発生時に何も分からなくなるので、
      # 例外のメッセージをinlineでrenderする必要があるかもしれませんので、ご注意を。
      # format.html { render_if_xhr(:inline => '') || redirect_to(party_emails_url) }
      # エフェクトをかけたい場合はこんな感じ。
      format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'index') }
      format.xml  { head :ok }
    end
  end


  desc :auth_as => :index
  # GET /campaign_conditions/name
  # GET /campaign_conditions/names.xml
  # GET /campaign_conditions/names.json
  def names
    campaign_conditions = CampaignCondition.find_with_name(params[:name])
    result = campaign_conditions.map{|campaign_condition|
      [campaign_condition.name_for_nested_ajax, campaign_condition.id] }
    respond_to do |format|
      format.html { render :text => auto_complete_html(result)}
      format.json { render :text => result.map{|(name, id)| {:name => name, :id => id} }.to_json }
      format.xml  { render :xml => result.map{|(name, id)| {:name => name, :id => id} } }
    end
  end

end
