# -*- coding: utf-8 -*-
class CampaignEffectsController < ApplicationController
  class_desc :display_name => 'キャンペーン効果管理(Ajax)', :auth_as => 'campaigns', :invisible => true

  include NestedAjax::RenderExt

  # GET /campaign_effects
  # GET /campaign_effects.xml
  def index
    @campaign_effects = CampaignEffect.all

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @campaign_effects }
    end
  end

  # GET /campaign_effects/1
  # GET /campaign_effects/1.xml
  def show
    @campaign_effect = CampaignEffect.find(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @campaign_effect }
    end
  end

  # GET /campaign_effects/new
  # GET /campaign_effects/new.xml
  def new
    @campaign_effect = CampaignEffect.new(params[:campaign_effect])

    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @campaign_effect }
    end
  end

  # GET /campaign_effects/1/edit
  def edit
    @campaign_effect = CampaignEffect.find(params[:id])
  end

  # POST /campaign_effects
  # POST /campaign_effects.xml
  def create
    @campaign_effect = CampaignEffect.new(params[:campaign_effect])

    respond_to do |format|
      if @campaign_effect.save
        flash[:notice] = 'CampaignEffect was successfully created.'
        format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'show', :id => @campaign_effect.id) }
        format.xml  { render :xml => @campaign_effect, :status => :created, :location => @campaign_effect }
      else
        format.html { render_if_xhr(:action => 'new') || render(:action => "new") }
        format.xml  { render :xml => @campaign_effect.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /campaign_effects/1
  # PUT /campaign_effects/1.xml
  def update
    @campaign_effect = CampaignEffect.find(params[:id])
    
    respond_to do |format|
      if @campaign_effect.update_attributes(params[:campaign_effect])
        # belongs_to, has_one, has_many などの関連のキャッシュをクリアしておきます
        @campaign_effect.clear_association_cache
        # composed_of などの集約のキャッシュをクリアしておきます
        @campaign_effect.clear_aggregation_cache
        flash[:notice] = 'CampaignEffect was successfully updated.'
        format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'show', :id => @campaign_effect.id) }
        format.xml  { head :ok }
      else
        format.html { render_if_xhr(:action => 'edit') || render(:action => "edit") }
        format.xml  { render :xml => @campaign_effect.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /campaign_effects/1
  # DELETE /campaign_effects/1.xml
  def destroy
    @campaign_effect = CampaignEffect.find(params[:id])
    @campaign_effect.destroy
    flash[:notice] = 'CampaignEffect was successfully deleted.'
    respond_to do |format|
      # いきなりザクッと消したい場合はこんな感じで。ただし例外発生時に何も分からなくなるので、
      # 例外のメッセージをinlineでrenderする必要があるかもしれませんので、ご注意を。
      # format.html { render_if_xhr(:inline => '') || redirect_to(party_emails_url) }
      # エフェクトをかけたい場合はこんな感じ。
      format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'index') }
      format.xml  { head :ok }
    end
  end

  desc :auth_as => :index
  # GET /campaign_effects/name
  # GET /campaign_effects/names.xml
  # GET /campaign_effects/names.json
  def names
    campaign_effects = CampaignEffect.find_with_name(params[:name])
    result = campaign_effects.map{|campaign_effect|
      [campaign_effect.name_for_nested_ajax, campaign_effect.id] }
    respond_to do |format|
      format.html { render :text => auto_complete_html(result)}
      format.json { render :text => result.map{|(name, id)| {:name => name, :id => id} }.to_json }
      format.xml  { render :xml => result.map{|(name, id)| {:name => name, :id => id} } }
    end
  end

end
