# -*- coding: utf-8 -*-
class CampaignsController < ApplicationController
  class_desc :display_name => 'キャンペーン管理'

  include NestedAjax::RenderExt

  # GET /campaigns
  # GET /campaigns.xml
  def index
    @condition = prepare_session_condition do |condition|
      condition || Campaign::SearchConditionForm.new(params[:condition])
    end

    @condition.page = params[:page] if params[:page]
    @campaigns = Campaign.paginate(@condition.to_paginate_options)
    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @campaigns }
    end
  end

  # GET /campaigns/1
  # GET /campaigns/1.xml
  def show
    @campaign = Campaign.find(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @campaign }
    end
  end

  # GET /campaigns/new
  # GET /campaigns/new.xml
  def new
    @campaign = Campaign.new(params[:campaign])
    @campaign.effects << CampaignEffect.new() if @campaign.effects.empty?

    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @campaign }
    end
  end

  # GET /campaigns/1/edit
  def edit
    @campaign = Campaign.find(params[:id])
  end

  # POST /campaigns
  # POST /campaigns.xml
  def create
    @campaign = Campaign.new(params[:campaign])

    respond_to do |format|
      if @campaign.save
        flash[:notice] = 'キャンペーンを新規登録しました。'
        format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'show', :id => @campaign.id) }
        format.xml  { render :xml => @campaign, :status => :created, :location => @campaign }
      else
        format.html { render_if_xhr(:action => 'new') || render(:action => "new") }
        format.xml  { render :xml => @campaign.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /campaigns/1
  # PUT /campaigns/1.xml
  def update
    @campaign = Campaign.find(params[:id])
    @campaign.mark_targets_for_destruction(params[:campaign] ? params[:campaign][:targets_modified] : false)
    
    respond_to do |format|
      if @campaign.update_attributes(params[:campaign])
        # belongs_to, has_one, has_many などの関連のキャッシュをクリアしておきます
        @campaign.clear_association_cache
        # composed_of などの集約のキャッシュをクリアしておきます
        @campaign.clear_aggregation_cache
        flash[:notice] = 'キャンペーンを更新しました。'
        format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'show', :id => @campaign.id) }
        format.xml  { head :ok }
      else
        format.html { render_if_xhr(:action => 'edit') || render(:action => "edit") }
        format.xml  { render :xml => @campaign.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /campaigns/1
  # DELETE /campaigns/1.xml
  def destroy
    @campaign = Campaign.find(params[:id])
    @campaign.destroy
    flash[:notice] = 'キャンペーンを削除しました。'
    respond_to do |format|
      # いきなりザクッと消したい場合はこんな感じで。ただし例外発生時に何も分からなくなるので、
      # 例外のメッセージをinlineでrenderする必要があるかもしれませんので、ご注意を。
      # format.html { render_if_xhr(:inline => '') || redirect_to(party_emails_url) }
      # エフェクトをかけたい場合はこんな感じ。
      format.html { render_if_xhr(:action => 'show') || redirect_to(:action => 'index') }
      format.xml  { head :ok }
    end
  end

  desc :auth_as => :index
  # GET /campaigns/name
  # GET /campaigns/names.xml
  # GET /campaigns/names.json
  def names
    campaigns = Campaign.find_with_name(params[:name])
    result = campaigns.map{|campaign|
      [campaign.name_for_nested_ajax, campaign.id] }
    respond_to do |format|
      format.html { render :text => auto_complete_html(result)}
      format.json { render :text => result.map{|(name, id)| {:name => name, :id => id} }.to_json }
      format.xml  { render :xml => result.map{|(name, id)| {:name => name, :id => id} } }
    end
  end

end
