# -*- coding: utf-8 -*-
class CmsProjectsController < ApplicationController
  class_desc :display_name => 'プロジェクト管理'

  # GET /cms_projects
  # GET /cms_projects.xml
  def index
    @condition = prepare_session_condition do |condition|
      condition || CmsProject::SearchCondition.new(params[:condition])
    end
    per_page = @condition.disp_count.to_i if @condition
    per_page = (per_page.to_i == 0) ? 20 : per_page
    @cms_projects = CmsProject.paginate(
      @condition.to_find_options(
        :page => params[:page], :per_page => per_page, :order => "project_code DESC"))
  end

  # GET /cms_projects/1
  # GET /cms_projects/1.xml
  def show
    @cms_project = CmsProject.find(params[:id])
    @cms_project_assigns = CmsProjectAssign.find(:all, :include => :goldberg_user, :conditions => ['cms_project_id = ?', @cms_project.id], :order => 'role')

    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @cms_project }
    end
  end

  # GET /cms_projects/new
  # GET /cms_projects/new.xml
  def new
    @cms_project = params[:id] ? CmsProject.find(params[:id]) : CmsProject.new
    @cms_project.project_code = nil

    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @cms_project }
    end
  end

  desc :auth_as => :create
  def new_assign
    @assign = CmsProjectAssign.new
    responseText = %{
      <% fields_for("cms_project[assigns_attributes][#{params[:index]}]", @assign) do |assign_form| %>
      <table id="new_assign" style="">
       <%= render(:partial => 'assign', :locals => {:f => assign_form, :index => #{params[:index]}}) %>
      </table>
      <% end %>

      <script type="text/javascript">
      //<![CDATA[
        var dest_table = $("member_table");
        $$("#new_assign tr").each(function(row){
          row.parentNode.removeChild(row);
          dest_table.appendChild(row);
        });
      //]]>
      </script>
    }.split(/$/).map(&:strip).join("\n")
    render(:inline => responseText)
  end

  # GET /cms_projects/1/edit
  def edit
    @cms_project = CmsProject.find(params[:id])
  end

  # POST /cms_projects
  # POST /cms_projects.xml
  def create
    @cms_project = CmsProject.new(params[:cms_project].merge(:check_flg => true))
    @cms_project.initialize_status
    if @cms_project.assigns_attributes
      @cms_project.assigns_attributes.each_value do |assigns_attribute|
        @cms_project.assigns << CmsProjectAssign.new(assigns_attribute)
      end
    end
    if @cms_project.save
      FileUtils.makedirs(File.expand_path(File.join(EcRider.config.ec_admin_root, 'public', 'projects', @cms_project.id.to_s, 'public')))
      redirect_to(@cms_project)
    else
      render(:action => "new")
    end
  end

  # PUT /cms_projects/1
  # PUT /cms_projects/1.xml
  def update
    @cms_project = CmsProject.find(params[:id])
    @cms_project.attributes = params[:cms_project]
    @cms_project.check_flg = true

    CmsProject.transaction do
      if @cms_project.assigns
        @cms_project.assigns.each {|assign| assign.destroy}
        @cms_project.assigns.clear
      end
      if @cms_project.assigns_attributes
        @cms_project.assigns_attributes.each_value do |assigns_attribute|
          @cms_project.assigns << CmsProjectAssign.new(assigns_attribute)
        end
      end
      @cms_project.save!
    end
    redirect_to(:action => 'show', :id => @cms_project.id)
  rescue
    render :action => "edit"
  end

  # DELETE /cms_projects/1
  # DELETE /cms_projects/1.xml
  def destroy
    @cms_project = CmsProject.find(params[:id])
    CmsProject.transaction do
      @cms_project.assigns.each do |assign|
        assign.destroy
      end
      @cms_project.destroy
      FileUtils.rm_rf(File.expand_path(File.join(EcRider.config.ec_admin_root, 'public', 'projects', params[:id])))
    end

    respond_to do |format|
      format.html { redirect_to(cms_projects_url) }
      format.xml  { head :ok }
    end
  end
end
