# -*- coding: utf-8 -*-
require 'tmpdir'
require 'pathname'
require 'fileutils'
require 'zip/zipfilesystem'

class CmsPublicContentsController < ApplicationController
  class_desc :display_name => '公開コンテンツ管理'

  protect_from_forgery :except => [:realtime_preview]
  
  include EcRider::CloudFront
  before_filter :setup_base_path

  public
  #
  def index
    files = []
    files << File.join(@base_path, 'views', '/')
    cms_files = Dir.glob(File.join(@base_path, 'views', 'cms', "*")).sort
    cms_files.delete_if{|f| File.basename(f) =~ /^_top_|^_bottom_|^_left_|^_right_|^_default_/}
    files << cms_files
    
    EcRider::CmsSetup.ecr_controllers.each do |controller|
      files << File.join(@base_path, 'views', controller)
      files << Dir.glob(File.join(@base_path, 'views', controller, "*")).sort
    end
    
    @real_file_pathes = files.flatten
    
    @real_file_pathes = @real_file_pathes.map{|path| path.gsub(@base_path, '')}
    @file_to_bind_in_opened_projects = CmsBind.filename_map(
      {:project_status_keys => [:opened], 
      :find_options => {:include => [{:cms_page => [:cms_project, :titles]}, {:cms_template => :cms_project}, :items]}}
    )
  end

  desc :auth_as => :index
  #
  # パラメータのパスにあるファイルを非公開にします。
  #
  def stop
    unless params[:path].nil?
      real_path = File.expand_path(File.join(@base_path, params[:path]))
      if File.exist?(real_path)
        if File.directory?(real_path)
          Dir.foreach(real_path) do |path|
            if File.directory?("#{real_path}/#{path}")
              Dir.foreach("#{real_path}/#{path}") do |sub_path|
                if  File.extname(sub_path) != nil && File.exist?("#{real_path}/#{path}/#{sub_path}") && File.extname(sub_path) == '.erb'
                  File::rename("#{real_path}/#{path}/#{sub_path}", "#{real_path}/#{path}/#{sub_path}_bak")
                end
              end
            else
              if  File.extname(path) != nil && File.exist?("#{real_path}/#{path}") && File.extname(path) == '.erb'
                File::rename("#{real_path}/#{path}", "#{real_path}/#{path}_bak")
              end
            end
          end
        else
          File.rename(real_path, "#{real_path}_bak")
        end
      end
      # ec_frontのビューをリロードする
      CmsPublish.send_reload_request(:production)
    end
    redirect_to :action => 'index'
  end

  desc :auth_as => :index
  #
  # パラメータのパスにあるファイルを公開にします。
  #
  def public
    unless params[:path].nil?
      real_path = File.expand_path(File.join(@base_path, params[:path]))
      if File.exist?(real_path)
        if File.directory?(real_path)
          Dir.foreach(real_path) do |path|
            if File.directory?("#{real_path}/#{path}")
              Dir.foreach("#{real_path}/#{path}") do |sub_path|
                Fil::rename(real_path, real_path.gsub("_bak", "")) if File.extname(sub_path) == '.erb_bak'
              end
            else
              File::rename(real_path, real_path.gsub("_bak", "")) if File.extname(path) == '.erb_bak'
            end
          end
        else
          File::rename(real_path, real_path.gsub("_bak", ""))
        end
      end
      # ec_frontのビューをリロードする
      CmsPublish.send_reload_request(:production)
    end
    redirect_to :action => 'index'
  end

  private

  def setup_base_path
    # @base_path は、ビューでも使われているインスタンス変数です
    @base_path = File.expand_path(File.join(EcRider.config.ec_front_root, 'app'))
  end

  #
  # ファイルパスの配列をディレクトリ－ファイルの親子関係を意識してソートします
  #
  def filetree_sort(real_file_pathes)
    # ディレクトリ-ファイルの親子関係を整理する
    files = {}
    real_file_pathes.sort.each do |real_file_path|
      if File.directory?(real_file_path)
        files[real_file_path] ||= []
      else
        dirname = File.dirname(real_file_path)
        files[dirname] ||= []
        files[dirname] << real_file_path
      end
    end

    # ディレクトリ名でソートし、各ディレクトリ内をファイル名でソートする
    result = []
    files.keys.sort.each do |dirname|
      result << dirname
      result += files[dirname].sort
    end
    return result
  end
  
end
