# -*- coding: utf-8 -*-
require 'tmpdir'
require 'pathname'
require 'fileutils'
require 'zip/zipfilesystem'


class CmsRealFilesController < ApplicationController
  class_desc :display_name => 'ファイルマネージャ'

  include EcRider::CloudFront
  before_filter :setup_base_path

  public
  def index
    @condition = prepare_session_condition do |condition|
      condition || CmsFile::CmsRealFileForm.new(params[:condition])
    end
    # ec_front/public/cms/*/**に一致するファイルパスを検索します
    files = Dir.glob(File.join(@base_path, 'public', '**',"*#{@condition.dir_name}*", '**', "*#{@condition.file_name}*.#{@condition.extension}*"))
    # 検索条件に当てはまるファイルがある場合、ルートディレクトリを表示します
    if files.size > 0
      files << File.join(@base_path, 'public')
      @real_file_pathes = filetree_sort(files)
      @real_file_pathes.delete_if {|path| /^\S*\-\d{14}\.\S*$/ =~ path}
      @real_file_pathes = @real_file_pathes.map{|path| path.gsub(@base_path, '')}
    end
  end

  def destroy
    FileUtils.rm_rf(File.join(@base_path, params[:path]))
    delete_from_s3(File.join(@base_path, params[:path]))
    redirect_to :action => 'index'
  end


  #
  # パラメータのパスにあるファイルをダウンロードします。
  #
  def show
    send_file( File.expand_path(File.join(@base_path, params[:path] )), :disposition => 'inline')
  end

  private

  def setup_base_path
    # @base_path は、ビューでも使われているインスタンス変数です
    @base_path = File.expand_path(EcRider.config.ec_front_root)
  end

  #
  # ファイルパスの配列をディレクトリ－ファイルの親子関係を意識してソートします
  #
  def filetree_sort(real_file_pathes)
    # ディレクトリ-ファイルの親子関係を整理する
    files = {}
    real_file_pathes.sort.each do |real_file_path|
      if File.directory?(real_file_path)
        files[real_file_path] ||= []
      else
        dirname = File.dirname(real_file_path)
        files[dirname] ||= []
        files[dirname] << real_file_path
      end
    end

    # ディレクトリ名でソートし、各ディレクトリ内をファイル名でソートする
    result = []
    files.keys.sort.each do |dirname|
      result << dirname
      result += files[dirname].sort
    end
    return result
  end

end
