# -*- coding: utf-8 -*-

# このrequireは app/models_admin/cms_version.rb に記述されるべきです。
require 'git'

class CmsVersionsController < ApplicationController
  class_desc :display_name => 'バージョン管理'
  
  around_filter :setup_cms_version

  def index
    @log_search_form = prepare_session_condition(:log_search_form) do |condition|
      condition || CmsVersion::LogSearchForm.new(params[:log_search_form])
    end
    @log_search_form.page = params[:page]
    @git_commits = @cms_version.filtered_log(@log_search_form)
    @git_branches = @cms_version.sorted_branche_names
  end

  def show
    redirect_to :action => 'index' unless params[:id]
    @git_commit = @cms_version.find_commit(params[:id])
    @git_diff = @git_commit.diff_parent.to_a rescue []
  end

  def new
    @git_commit = @cms_version.new_commit(flash[:invalid_commit])
    @git_commit.valid?
    if flash[:invalid_commit] && @git_commit.empty?
      @git_commit.errors.add_to_base("バージョン管理の対象にするファイルを選択してください。")
    end
  end

  def create
    @git_commit = @cms_version.new_commit(params[:git_commit])
    unless @git_commit.valid? && !@git_commit.empty?
      flash[:invalid_commit] = params[:git_commit]
      redirect_to :action => 'new'
      return
    end
    @git_commit.apply
    flash[:notice] = "コミットしました"
    redirect_to :action => 'index'
  end

  desc :display_name => 'ファイルのロールバック', :category_name => 'update'
  def revert
    @cms_version.revert(params[:id], params[:path])
    # ec_frontのビューをリロードする
    CmsPublish.send_reload_request(:production)

    redirect_to :action => :index
  end

  desc :display_name => 'ロールバック', :category_name => 'update'
  def reset_hard
    @cms_version.reset_hard_with_backup_branch(params[:id])
    # ec_frontのビューをリロードする
    CmsPublish.send_reload_request(:production)
    
    flash[:notice] = "バージョンを戻しました"
    redirect_to :action => 'index'
  end

  desc :display_name => 'ブランチの切り替え', :category_name => 'update'
  def checkout
    @cms_version.checkout(params[:id])
    # ec_frontのビューをリロードする
    CmsPublish.send_reload_request(:production)
    
    flash[:notice] = "ブランチを切り替えました"
    redirect_to :action => 'index'
  end

  desc :category_name => :destroy, :display_name => 'ブランチの削除'
  def branch_delete
    @cms_version.delete_branch(params[:id])
    flash[:notice] = "ブランチを削除しました"
    redirect_to :action => 'index'
  end
  
  private

  def setup_cms_version
    if EcRider.config.cms_git_dir.blank?
      render :text => "You must set environment variable ECR_CMS_GIT_DIR", :layout => true
    elsif !File.exist?(EcRider.config.cms_git_dir)
      render :text => "#{EcRider.config.cms_git_dir} doesn't exist. Please check environment variable ECR_CMS_GIT_DIR", :layout => true
    else
      # CmsVersionはgitのリポジトリにアクセスするためのクラス
      # ec_adminでしか使わないので app/models_adminにあります。
      @cms_version = CmsVersion.new(EcRider.config.cms_git_dir)
      yield
    end
  end
end
