class CsvFileDownloadController < ApplicationController
  class_desc :display_name => 'CSVファイル出力'
  
  def index
    @csv_download_pattern = CsvDownloadPattern.new
    @csv_download_pattern.download_type = params[:download_type] if params[:download_type]
    @csv_download_pattern.items_attributes = {:item_list => "", :fixed_value_list => ""}
    reset_condition
  end
  
  def edit
    @csv_download_pattern = CsvDownloadPattern.find(params[:id].to_i, :include => [:items, :user_condition, :order_condition, :product_condition])
    @condition = @csv_download_pattern.set_detail_for_edit
    render :action => "index"
  end
  
  def update
    @csv_download_pattern = CsvDownloadPattern.find(
      params[:csv_download_pattern][:id].to_i, 
      :include => [:items, :user_condition, :order_condition, :product_condition, :accounting_condition]
    )
    CsvDownloadPattern.transaction do
      @csv_download_pattern.destroy_item_and_condition
      @csv_download_pattern.attributes = params[:csv_download_pattern]
      @csv_download_pattern.set_condition(params[:condition])
      @csv_download_pattern.save!
    end
    flash[:notice] = '出力情報を更新しました。'
    reset_condition(params[:condition])
    render :action => "index"
  rescue
    render :action => "index"
  end
  
  def destroy
    @csv_download_pattern = CsvDownloadPattern.find(
      params[:id].to_i, 
      :include => [:items, :user_condition, :order_condition, :product_condition]
    )
    if @csv_download_pattern.uneditable?
      render :text => 'undeletable' ,:layout => false
    else
      CsvDownloadPattern.transaction do
        @csv_download_pattern.destroy_item_and_condition
        @csv_download_pattern.destroy
      end
    end
  end
  
  def create
    @csv_download_pattern = CsvDownloadPattern.new(params[:csv_download_pattern])
    @csv_download_pattern.set_condition(params[:condition])
    if @csv_download_pattern.save
      flash[:notice] = '出力情報を作成しました。'
    end
    reset_condition(params[:condition])
    render :action => "index"
  end
  
  desc :display_name => "CSVファイル出力"
  def download
    CsvDownloadPattern.transaction do
      @csv_download_pattern = CsvDownloadPattern.find(params[:id].to_i, :include => [:items, :user_condition, :order_condition, :product_condition, :accounting_condition])
      csv = @csv_download_pattern.generate
      file_name = @csv_download_pattern.file_name.concat("_#{Time.now.strftime("%Y%m%d%H%M%S")}.csv")

      # 履歴用ファイルを保存
      FileUtils.mkdir_p(EcRider.config.csv_dir)
      open(File.join(EcRider.config.csv_dir, file_name), "w", 0666) do |file|
        file.puts(csv)
      end
      CsvFileDownloadHistory.create(:csv_download_pattern_id => params[:id].to_i, :file_name => file_name, :character_code => @csv_download_pattern.character_code)
      # 経理情報出力の場合、出力したデータに出力済みフラグを追加
      if @csv_download_pattern.download_type_key == :accounting
        @csv_download_pattern.rows.each do |accounting|
          accounting.output_data_flg = true
          accounting.save!
        end
      end
      send_data(csv, :type => "text/csv charset=#{@csv_download_pattern.character_key.to_s}", :filename => file_name)
    end
  end

  private
  def reset_condition(params_condition = {})
    case @csv_download_pattern.download_type_key
      when :user, :user_address
        @condition = User::SearchConditionForm.new(params_condition)
      when :product, :product_sku, :product_display, :product_price, :product_component
        @condition = Product::SearchCondition.new(params_condition)
      when :order, :order_shipping, :order_item
        @condition = Order::SearchInfo.new(params_condition)
      when :accounting
        @condition = Accounting::SearchInfo.new(params_condition)
    end
  end
end