class DistributorController < ApplicationController
  class_desc :display_name => "物流情報管理"
  #
  # = 検索画面表示アクション =
  #
  def index
    @condition = prepare_session_condition do |condition|
      condition || Distributor::SearchInfo.new(params[:condition])
    end
    if @condition.valid?
      per_page = @condition.disp_count ? @condition.disp_count.to_i : 20
      @distributors = Distributor.paginate(
        @condition.to_find_options(
          :page => params[:page], :per_page => per_page, :order => "id DESC"))
    end
  end
  
  #
  # = 物流情報新規登録表示アクション =
  #
  def new
    if flash[:distributor]
      @distributor = flash[:distributor]
      @supported_locales = I18n.supported_locales
    else
      @distributor = Distributor.new()
      for i in 1..10
        @distributor.delivery_times << DistributorDeliveryTime.new(:display_order => i)
      end
      @supported_locales = I18n.supported_locales
      @distributor.delivery_times.each do |d_time|
        @supported_locales.each do |locale|
          d_time.delivery_time_displays << DistributorDeliveryTimeDisplay.new(:locale => locale.to_s)
        end
      end
    end
  end

  def edit
    if flash[:distributor]
      @distributor = flash[:distributor]
    else
      @distributor = Distributor.find(params[:id])
    end
      @supported_locales = I18n.supported_locales
  end

  def create
    distributor = Distributor.new(params[:distributor])
    # バリデーション
    unless distributor.valid?
      flash[:distributor] = distributor
      return redirect_to :action => :new, :page => params[:page]
    end
    distributor.save!
    return redirect_to :action => :index, :page => params[:page]
  end

  def update
    distributor = Distributor.find(params[:id])
    distributor.attributes = params[:distributor]
    # バリデーション
    unless distributor.valid?
      flash[:distributor] = distributor
      return redirect_to :action => :edit, :page => params[:page], :id => params[:id]
    end
    distributor.save!
    return redirect_to :action => :index, :page => params[:page]
  end

  def show
    @distributor = Distributor.find(params[:id])
    @countries = Country.find(:all, :conditions => ['distributor_id = ?', params[:id]])
    @supported_locales = I18n.supported_locales
  end

  def destroy
    Distributor.destroy(params[:id])
    redirect_to :action => 'index', :page => params[:page]
  end

end
