# -*- coding: utf-8 -*-
require 'goldberg'

module Goldberg
  class RolesController < ApplicationController
    class_desc :display_name => '権限管理'
    
    # skip_filter :goldberg_security_up

    before_filter :prepare_site_action_groups, :only => [:show, :new, :edit]
    before_filter :keep_page_no

    private

    def prepare_site_action_groups
      @site_action_groups = Goldberg::SiteActionGroup.find(:all, :include => :actions, :order => "display_name asc")
    end
    
    def keep_page_no
      @page = params[:page]
    end

    def redirect_to_index
      options = {:action => :index}
      options[:page] = @page unless @page.blank?
      redirect_to(options)
    end

    # expire_fragment メソッドでなぜか必要なので実装してあります。
    def supported_locales
      I18n.supported_locales
    end



    public
    
    # GET /goldberg/roles
    def index
      @condition = prepare_session_condition do |condition|
        condition || Goldberg::Role::SearchCondition.new(params[:condition])
      end
      @condition.page = params[:page]
      @roles = Goldberg::Role.paginate(
        @condition.to_paginate_options(:order => "updated_at DESC"))
    end

    # GET /goldberg/roles/1
    def show
      @role = Goldberg::Role.find(params[:id], :include => :role_actions)
    end

    # GET /goldberg/roles/new
    def new
      @role = Goldberg::Role.new
    end

    # POST /goldberg/roles/1
    def create
      @role = Goldberg::Role.new(params[:role])
      Goldberg::Role.transaction do
        if @role.save
          redirect_to_index
        else
          render :action => :new
        end
      end
    end

    # GET /goldberg/roles/1/edit
    def edit
      @role = Goldberg::Role.find(params[:id], :include => :role_actions)
    end

    # PUT /goldberg/roles/1
    def update
      @role = Goldberg::Role.find(params[:id], :include => :role_actions)
      Goldberg::Role.transaction do
        if @role.update_attributes(params[:role])
          # app/views/layouts/application.rhtml で使っている cacheを失効させます。
          expire_fragment(:action => :menu_bar, :role_id => @role_id)
          redirect_to_index
        else
          render :action => :edit
        end
      end
    end
    
    # DELETE /goldberg/roles/1
    def destroy
      @role = Goldberg::Role.find(params[:id])
      Goldberg::Role.transaction do
        # app/views/layouts/application.rhtml で使っている cacheを失効させます。
        expire_fragment(:action => :menu_bar, :role_id => @role_id)
        @role.destroy
      end
      redirect_to_index
    end
    
  end
end
