# -*- coding: utf-8 -*-
require 'digest/sha1'

module Goldberg
  class UsersController < ApplicationController
    class_desc :display_name => 'システム利用者管理'

    #
    # = 一覧表示 =
    #
    def index
	    @condition = prepare_session_condition do |condition|
 	      condition || Goldberg::User::SearchCondition.new(params[:condition])
 	    end
      per_page = @condition.disp_count.to_i if @condition
 	    per_page = (per_page.to_i == 0) ? 20 : per_page

      @goldberg_users = Goldberg::User.paginate(@condition.to_find_options(
 	        :page => params[:page], :per_page => per_page, :order => "id DESC"))
    end

    #
    # = 詳細画面表示 =
    #
    def show
      @goldberg_user = Goldberg::User.find(params[:id])
      @page_no = params[:page]
    end

    #
    # = 更新画面表示 =
    #
    def edit
      if flash[:goldberg_user]
        #既に表示済み(エラー時)
        @goldberg_user = session[:tmp_goldberg_user] = flash[:goldberg_user]
      else
        @goldberg_user = session[:tmp_goldberg_user] = Goldberg::User.find(params[:id])
      end
      @page_no = params[:page]
    end

    #
    # = 更新 =
    #
    def update
      session[:tmp_goldberg_user].attributes = params[:goldberg_user]
      goldberg_user = session[:tmp_goldberg_user]

      unless goldberg_user.valid? #バリデーション
        flash[:goldberg_user] = goldberg_user
        redirect_to :action => :edit, :page => params[:page]
        return
      end

      # 更新
      Goldberg::User.transaction do
        goldberg_user.save!
      end

      session[:tmp_goldberg_user] = nil
      redirect_to :action => :index, :page => params[:page]
    end

    #
    # = 削除 =
    #
    def destroy
      Goldberg::User.find(params[:id]).destroy
      redirect_to :action => :index, :page => params[:page]
    end

    #
    # = 登録画面表示 =
    #
    def new
      if flash[:goldberg_user]
        # バリデーションエラーの場合
        @goldberg_user = flash[:goldberg_user]
      else
        @goldberg_user = Goldberg::User.new()
      end
    end

    #
    # = 新規登録 =
    #
    def create
      goldberg_user = Goldberg::User.new(params[:goldberg_user])
      unless goldberg_user.valid? #バリデーション
        flash[:goldberg_user] = goldberg_user
        redirect_to :action => :new
        return
      end

      Goldberg::User.transaction do #新規登録
        goldberg_user.save!
      end

      session[:tmp_goldberg_user] = nil
      redirect_to :action => :index, :page => params[:page]
    end

  end
end
