# -*- coding: utf-8 -*-
class InquiryController < ApplicationController
  class_desc :display_name => '問い合わせ管理'

  def index
    @condition = prepare_session_condition do |condition|
      condition || Inquiry::SearchInfo.new(params[:condition])
    end
    if @condition.valid?
      per_page = @condition.disp_count ? @condition.disp_count.to_i : 20
      @inquiries = Inquiry.paginate(
        @condition.to_find_options(
          :page => params[:page], :per_page => per_page, :include => [:inquiry_memos], :order => "inquiries.created_at DESC"))
    else
      flash[:condition] = @condition
    end
  end

  desc :auth_as => :index, :display_name => '問合せ詳細'
  def detail
    @mail_deliveries = MailDelivery.find(:all,
      :conditions => MailDelivery.create_condition_for_inquiry(params[:id]), 
      :order => 'deliveried_at desc')
    @inquiry = Inquiry.find(params[:id])
  end

  def update
    target = Inquiry.find(params[:id])
    # 商品選択で商品が選択された場合のみ、DBに保存されている商品関連データを削除します。
    target.old_product_mark_for_destruction(params[:inquiry][:targets_modified])
    
    target.attributes = params[:inquiry]
    unless target.valid?
      flash[:error] = target.errors.zip
      flash[:id] = params[:id]
      redirect_to :action =>'edit', :id => params[:id], :page => params[:page]
      return
    end
    Inquiry.transaction do
      target.save!
    end

    if params[:send_mail]
      send_mail(target)
    else
      @inquiry = target
      redirect_to :action =>:detail, :id => params[:id], :page => params[:page]
    end
  end

  def edit
    @inquiry = Inquiry.find(params[:id])
    @products = InquiryProduct.create_product_list(@inquiry.inquiry_products)
    # バリデーションエラーで戻った場合
    if flash[:mail_form]
      @mail_form = flash[:mail_form]
    # 通常の画面遷移
    else
      @mail_form = MailForm.new()
      @mail_form.from = ShopSetting[:shop_mail]
      @mail_form.to = @inquiry.email

      template = MailTemplate.find_valid_template(:reply, @inquiry.locale)
      options = {:first_name => @inquiry.name_first, :last_name => @inquiry.name_last, :user_name => @inquiry.fullname,
                 :email_address => @inquiry.email, :inquiry_content => @inquiry.inquiry, :answer_content => @inquiry.answer}
      pc_mobile_div = UserEmail.check_by_emall(@inquiry.email)
      @mail_form.subject = template.get_mail_subject(pc_mobile_div, options)
      @mail_form.body = template.get_mail_body(pc_mobile_div, options)
      @mail_form.locale = template.locale
    end
  end

  desc :category_name => :update, :display_name => "問合せ返信メール送信"
  def send_mail(inquiry)
    @mail_form = MailForm.new(params[:mail_form])
    @inquiry = inquiry
    unless @mail_form.valid?
      flash[:mail_form] = @mail_form
      redirect_to :action => :edit, :id => params[:id], :page => params[:page]
      return
    end

    # 複数あて先変換
    to_list = []
    to_list = @mail_form.to_split
    #フリーメール送信
    AdminMailer.send_free_mail(:reply, @mail_form.from, to_list, @mail_form.subject, @mail_form.body, @mail_form.locale, @inquiry.user, nil, @inquiry.id)
    flash[:notice] = "メールを送信しました。"

    redirect_to :action => :detail, :id => params[:id], :page => params[:page]
  end

  desc :category_name => :update, :display_name => "問合せメモ登録"
  def regist_inquiry_memo
    inquiry_memo = InquiryMemo.new(params[:inquiry_memo])
    unless inquiry_memo.valid?
      flash[:inquiry_memo] = inquiry_memo
      redirect_to :action => :edit, :id => params[:id], :page => params[:page]
      return
    end

    inquiry_memo.inquiry_id = params[:id]
    inquiry_memo.creator_name = current_user.name
    inquiry_memo.save!

    redirect_to :action => :edit, :id => params[:id], :page => params[:page]
  end

  desc :category_name => :destroy, :display_name => "問合せメモ削除"
  def delete_memo
    begin
      memo = InquiryMemo.find params[:memo_id]
      memo.destroy!

    rescue ActiveRecord::RecordNotFound
      # 何もしない
      logger.warn i18n_message(:EM070003)
    end
    redirect_to :action => :edit, :id => params[:id], :page => params[:page]
  end

  desc :category_name => :destroy, :display_name => 'お問合せ削除'
  def delete
    if params[:destroy_all_test]
      inquiry_ids = Inquiry.find(:all, :conditions => ["test_flg = ?", true]).map{|user| user.id}
    else
      inquiry_ids =[params[:id]]
    end
    inquiry_ids.each do |inquiry_id|
      Inquiry.find(inquiry_id).destroy
    end
    redirect_to :action => 'index', :page => params[:page]
  end

  desc :auth_as => :update
  # 回答内容をメール本文へ反映
  def marge_mail_template
    @inquiry = Inquiry.find(params[:id])

    # 通常の遷移のケース
    @mail_form = MailForm.new()
    @mail_form.from = ShopSetting[:shop_mail]
    @mail_form.to = params[:mail_form][:to]
    template = MailTemplate.find_valid_template(:reply, @inquiry.locale)
    options = {:first_name => @inquiry.name_first, :last_name => @inquiry.name_last, :user_name => @inquiry.fullname,
                 :email_address => @inquiry.email, :inquiry_content => @inquiry.inquiry, :answer_content => params[:inquiry][:answer]}
    pc_mobile_div = UserEmail.check_by_emall(@inquiry.email)
    @mail_form.subject = params[:mail_form][:subject]
    @mail_form.body = template.get_mail_body(pc_mobile_div, options)
    render :partial => 'edit_mail_form', :id => params[:id], :page => params[:page]
  end

end
