class InventoriesController < ApplicationController
  class_desc :display_name => "在庫管理"

  # GET /inventories
  # GET /inventories.xml
  def index
    @condition = prepare_session_condition do |condition|
      condition || ProductSkuInventory::SearchForm.new(params[:condition])
    end

    unless params[:clear]
      @product_skus = ProductSku.paginate(
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => @condition.to_paginate_condition,
        :per_page => @condition.get_per_page, :page => params[:page])
      @product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(@product_skus)
      @product_skus = ProductComponent.set_component_parent_sku_id(@product_skus)
    end
    
    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @product_skus }
    end
  end

  # GET /inventories/1/edit
  def edit
    @product_sku = ProductSku.find(params[:id])
    @inventory = ProductSkuInventory::CreateForm.new(:in_out_div => 'in')
  end
  
  def update
    @inventory = ProductSkuInventory::CreateForm.new(params[:product_sku_inventory_create_form])
    @product_sku = ProductSku.find(@inventory.product_sku_id)
    respond_to do |format|
      if @inventory.valid? && @inventory.create_inventory_history
        format.html { redirect_to(:action => 'show', :id => @inventory.product_sku_id) }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @inventory.errors, :status => :unprocessable_entity }
      end
    end
  end

  def show
    @product_sku = ProductSku.find(params[:id])
    @inventories = ProductSkuInventory.paginate(:conditions => {:product_sku_id => @product_sku.id}, :per_page => 50, :page => params[:history_page], :order => 'id desc')
    @stock_count_at_first_record = @inventories.first.get_stock_count if @inventories.first
    
    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @inventories }
    end
  end

end
