# -*- coding: utf-8 -*-
class NameMasterController < ApplicationController
  class_desc :display_name => '項目名称管理'

  desc :category_name => :index, :display_name => '検索'
  # 検索画面表示
  def search_index
    session[:tmp_kbn_code] = nil
    session[:kbn_code_condition] = nil
    render :action => :name_master_search
  end

  desc :auth_as => :search_index
  # 検索
  def search
    session[:tmp_kbn_code] = nil
    if params[:condition]
      session[:kbn_code_condition] = trim_values_indifferent(params[:condition])
    end

    @condition = KbnCode::SearchInfo.new(session[:kbn_code_condition])
    @kbn_codes = KbnCode.paginate(@condition.to_paginate_options(:page => params[:page]))
    render :action => :name_master_search
  end

  desc :auth_as => :search_index
  # 詳細
  def name_master_detail
    @kbn_code = KbnCode.find(params[:id])
    @code_names = KbnCodeName.find(:all, :conditions => ['kbn_code_id = ?', params[:id]])
    @page_no = params[:page]
  end

  desc :auth_as => :update
  # 編集
  def update_form
    @kbn_code = KbnCode.find(params[:id])
    @code_names = []
    # バリデーションエラーで戻った場合
    if flash[:code_names]
      @code_names = flash[:code_names]
      @code_names.each do |kbn_code_name|
        unless kbn_code_name.valid?
          @kbn_code_name = kbn_code_name
        end
      end
    # 初期表示
    else
      I18n.supported_locales.each do |locale|
        kbn_code_name = KbnCodeName.find(:first, :conditions => ['kbn_code_id = ? and locale = ?', params[:id], locale.to_s])
        kbn_code_name ||= KbnCodeName.new({:locale => locale.to_s})
        @code_names << kbn_code_name
      end
    end
  end

  # 更新
  def update
    code_names = []
    validate = true
    I18n.supported_locales.each do |locale|
      kbn_code_name = KbnCodeName.find(:first, :conditions => ['kbn_code_id = ? and locale = ?', params[:id], locale.to_s])
      kbn_code_name ||= KbnCodeName.new({:kbn_code_id => params[:id], :locale => locale.to_s})
      kbn_code_name.name = params["name_#{locale}".intern]
      code_names << kbn_code_name
      unless kbn_code_name.valid?
        validate = false
      end
    end
    unless validate
      flash[:code_names] = code_names
      redirect_to :action => 'update_form', :id => params[:id]
      return
    end
    KbnCode.transaction do
      code_names.each do |kbn_code_name|
        kbn_code_name.save
      end
    end
    redirect_to :action => :search
  end

  desc :category_name => :destroy, :display_name => '削除'
  # 削除
  def delete
    KbnCode.find(params[:id]).destroy
    redirect_to :action => :search, :page => params[:page]
  end

  def new
    @kbn_code = KbnCode.new
  end

  def create
    begin
      KbnCode.transaction do
        @kbn_code = KbnCode.new(params[:kbn_code])
        @kbn_code.kbn_name = KbnCode.name_div_name_by_id(@kbn_code.kbn)
        @kbn_code.save!
        I18n.supported_locales.each do |locale|
          @kbn_code_name = KbnCodeName.new({:kbn_code_id => @kbn_code.id, :locale => locale.to_s, :name => params["name_#{locale}".intern]})
          @kbn_code_name.save!
        end
      end
      redirect_to :action => :search
    rescue ActiveRecord::RecordInvalid
      render :action => 'new'
    end
  end
end
