# -*- coding: utf-8 -*-
require 'csv'
class OrderController < ApplicationController
  include EcRider::PluginModule::Extendable

  class_desc :display_name => '注文管理'

  include EcRider::I18nSupport

  desc :invisible => true, :auth_as => :order_search
  def top
    session[:order_condition] = nil
    session[:user_search_cond] = nil
    session[:user_out_search_cond] = nil
  end

  desc :auth_as => :order_search
  # 検索画面表示
  def search_index
    # バリデーションエラーで戻った場合
    if flash[:error]
      @condition = Order::SearchInfo.new(flash[:condition_attr])
      flash[:error].each{|errors| errors.each {|attr, msg| @condition.errors.add(attr, msg) unless attr.empty?}}
      render(:action => 'order_search')
      return
    end

    clear_sessions
    session[:order_condition] = nil

    #検索条件の初期設定
    @condition = Order::SearchInfo.new
    @condition.delivery_to_native_flg = true
    @condition.delivery_to_foreign_flg = true

    render(:action => 'order_search')
  end

  desc :category_name => :index, :display_name => "注文検索"
  # 検索
  def order_search
    clear_sessions

    if params[:condition]
      session[:order_condition] = trim_values_indifferent(params[:condition])
    end
    #不正な画面遷移
    unless (params[:condition] || session[:order_condition])
      flash[:error] = "検索情報がクリアされている為、初期状態に戻しました。"
      redirect_to :action => :search_index
      return
    end

    #検索条件取得
    @condition = Order::SearchInfo.new(session[:order_condition])
    unless @condition.valid?
      flash[:error] = @condition.errors.zip
      flash[:condition_attr] = @condition.attributes
      redirect_to :action => :search_index
      return
    end

    render_order_search
  end


  desc :auth_as => :order_search, :display_name => "タブ選択"
  def index_tab
    #TOPページ（ダッシュボード）からの遷移用
    if params[:order_condition]
      session[:order_condition] = params[:order_condition]
    end
    condition_attrs = session[:order_condition]
    session[:order_condition] = condition_attrs
    unless condition_attrs
      flash[:error] = "検索情報がクリアされている為、初期状態に戻しました。"
      return redirect_to(:action => :search_index)
    end
    @condition = Order::SearchInfo.new(condition_attrs)
    @condition.status_tab = params[:status_tab]
    @condition.tab_inclusion = params[:inclusion]
    render_order_search
  end

  private

  def render_order_search
    page = params[:page] ? params[:page].to_i : 1
    per_page = @condition.disp_count ? @condition.disp_count.to_i : 20
    if status_keys = @condition.selected_status_div_keys
      total_count = 0
      @condition.order_counts_by_tab.each {|key, value| total_count += value if status_keys.include?(key)}
    else
      total_count = @condition.order_counts_by_tab[Order::SearchInfo.status_tab_key_by_id(params[:status_tab])]
    end
    @orders = WillPaginate::Collection.new(page, per_page, total_count)

    order_shippings = Order.find_any_condition(@condition, per_page, @orders.offset)
    @orders.replace(order_shippings)

    if @condition.display_summary == "1"
      total_amount_and_shipping_cost = Order.total_amount_and_shipping_cost(session[:order_condition])
      @total_amount = total_amount_and_shipping_cost[:total_amount]
      @total_shipping_cost = total_amount_and_shipping_cost[:shipping_cost]
    end

    if request.xml_http_request?
      # CSVファイル画面のダウンロード情報を注文情報(xxx)を選んだ際のCSVの出力に使用されます。
      render(:partial => "/order/order_search_result", :layout => false, :locals => {:invoked => 'csv'})
    else
      render :action => 'order_search'
    end
  end

  public

  desc :auth_as => :order_search
  # 詳細
  def order_detail
    @order = Order.find(params[:id])
    @order_shippings = OrderShipping.find(:all, :conditions => ['order_id = ?', @order.id])
    @order_items = OrderItem.find(:all, :conditions => ['order_shipping_id = ?', @order_shippings[0].id])
    @user = User.find(:first, :conditions => ["id = ?", @order.user_id])
    if @user
      @user_address = @user.user_address
      @user_email = @user.user_email
    end
    @payment_error_history = PaymentErrorHistory.find(:first, :conditions => {:order_id => @order.id})
    @page_no = params[:page]
    @sent_mails = MailDelivery.find(:all, :conditions => ["order_id = ?", params[:id]], :order => "deliveried_at desc")
  end

  desc :category_name => :index, :display_name => "請求書兼納品書"
  # 請求書兼納品書
  def print
    @order = Order.find(params[:id], :include => [{:order_shippings => :order_items}, {:user => [:user_addresses, :user_email]}])

    if @order.locale == 'ja'
      render :layout => false, :action => 'print_ja'
    else
      i18n_tmp_block('en') do
        render :layout => false, :action => 'print_en'
      end
    end
  end

  desc :auth_as => :order_send_mail
  # メール送信フォームを表示
  def order_mail
    @order = Order.find(params[:id], :include => :order_shippings)
    @order_shipping = OrderShipping.find(params[:order_shipping_id])
    @user = User.find(@order.user_id)
    @user_address = @user.user_address
    @mail_type_div = params[:mail_type_div] ? params[:mail_type_div] : MailTemplate.mail_type_div_id_by_key(:order_reply)

    # バリデーションエラーで戻った場合
    if flash[:mail_form]
      @mail_form = flash[:mail_form]
    else
      @mail_form = MailForm.new()
      @mail_form.from = @order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
      @mail_form.to = @order_shipping.email

      # メールテンプレートを取得
      template = MailTemplate.find_valid_template(MailTemplate.mail_type_div_key_by_id(@mail_type_div), @order.locale)
      # メールテンプレートにバインドする変数を設定
      options = {:first_name => @user.name_first, :last_name => @user.name_last, :user_name => @user.user_address.fullname,
                 :email_address => @order_shipping.email, :order => @order, :order_shipping => @order_shipping}
      case MailTemplate.mail_type_div_key_by_id(@mail_type_div)
      when :order_reply
        options[:inquiry_content] = @order.free_request
      end
      # メールテンプレートに注文情報をバインドする
      pc_mobile_div = UserEmail.check_by_emall(@order_shipping.email)
      @mail_form.subject = template.get_mail_subject(pc_mobile_div, options)
      @mail_form.body = template.get_mail_body(pc_mobile_div, options)
      @mail_form.locale = template.locale
    end
  end

  desc :category_name => :create, :display_name => "メール送信"
  # メール送信
  def order_send_mail
    @mail_form = MailForm.new(params[:mail_form])
    unless @mail_form.valid?
      flash[:mail_form] = @mail_form
      redirect_to :action => :order_mail, :id => params[:id], :order_shipping_id => params[:order_shipping_id], :page => params[:page], :mail_type_div => params[:mail_type_div]
      return
    end

    unless params[:id].blank?
      @order = Order.find(params[:id], :include => [{:order_shippings => :order_items}, {:user => [:user_addresses, :user_email]}])
    end

    mail_type_div_key = MailTemplate.mail_type_div_key_by_id(params[:mail_type_div])
    to_list = @mail_form.to_split
    AdminMailer.send_free_mail(mail_type_div_key, @mail_form.from, to_list, @mail_form.subject, @mail_form.body, @mail_form.locale, @order.user, @order)

    render :action => "confirm_send_mail"
  end

  desc :auth_as => :update
  def edit
    # バリデーションエラーで戻った場合
    if flash[:order_shipping] || flash[:order] || flash[:order_items]
      @order_shipping = flash[:order_shipping]
      @order = flash[:order]
      @order_items = flash[:order_items]
      @page_no = params[:page]
      @after_total_amount = params[:after_total_amount]
      flash[:error_info] = flash[:error_info]
    else
      @order_shipping = OrderShipping.find_by_id(params[:id])
      @after_total_amount = @order_shipping.total_amount
      @order = @order_shipping.order
      @order_items = @order_shipping.order_items
      @page_no = params[:page]
    end
  end

  desc :auth_as => :create
  # 新規注文登録画面
  # 事前条件：購入対象者のユーザ情報が入っていること
  def new
    # バリデーションエラーで戻った場合
    if flash[:order_form] || flash[:order_shipping] || flash[:order] || flash[:order_items] || flash[:shipping_address]
      @user = flash[:user]
      @order_form = flash[:order_form]
      @order_shipping = flash[:order_shipping]# || OrderShipping.new
      @order = flash[:order]
      @order_items = flash[:order_items]# || {}
      @page_no = params[:page]
      @after_total_amount = params[:after_total_amount]
      @shipping_address = flash[:shipping_address]
    else
      #初期表示
      @user = User.find(params[:user_id])
      @order_form = OrderForm.new
      @order = Order.new
      @order.user_id = @user.id
      @order_shipping = OrderShipping.new
      @order_items = {}

      # 配送先に、ここで編集データをセットします
      user_address = @user.user_and_shipping_addresses.first
      @order_shipping.name_first = user_address.name_first
      @order_shipping.name_last = user_address.name_last
      if I18n.locale.to_s == 'ja'
        @order_shipping.kana_name_first = user_address.kana_name_first
        @order_shipping.kana_name_last = user_address.kana_name_last
      end
      @order_shipping.country_code = user_address.country_code
      @order_shipping.zip_code = user_address.zip_code
      @order_shipping.address_1 = user_address.address_1
      @order_shipping.address_2 = user_address.address_2
      @order_shipping.address_3 = user_address.address_3
      @order_shipping.tel_num = user_address.tel_num
      @page_no = params[:page]
    end
  end

  # 注文
  # 事前条件：購入対象者のユーザ情報が入っていること
  def create
    return redirect_to(:controller => 'order', :action => 'new', :user_id => params[:user_id]) if params[:order_form].nil?

    # 注文情報を作成
    user = User.find(params[:user_id].to_s)
    order_form = OrderForm.new(params[:order_form])
    order_form.campaign_attached_at = Time.parse(params[:campaign_attached_at]) if order_form.campaign_attached_div_key == :select
    order_shipping = OrderShipping.new(params[:order_shipping])
    
    # 注文商品を作成してバリデーションチェック
    order_items = OrderItem.create_order_items(params[:order_item])
    flash[:error_info] = "注文商品は最低１商品登録してください。" if order_items.size <= 0
    order_item_no_error = true
    order_items.each do |order_item|
      order_item_no_error = false unless order_item.valid?
    end

    # 配送先住所を取得する 
    shipping_address = UserAddress.new()
    shipping_address.merge_only_need(params[:order_shipping])
    shipping_address.address_div_key = :ship_to
    shipping_address.locale = I18n.locale.to_s

    # 入力チェック
    order_shipping.validates_for
    order_form.valid?
    shipping_address.valid?
    error_count = order_shipping.errors.count + order_form.errors.count + shipping_address.errors.count
    unless (error_count <= 0 && order_item_no_error && flash[:error_info].nil?)
      flash[:order_form] = order_form
      flash[:user] = user
      flash[:order_shipping] = order_shipping
      flash[:shipping_address] = shipping_address
      flash[:order] = Order.new()
      flash[:order_items] = order_items
      return redirect_to :action => 'new', :page => params[:page], :after_total_amount => params[:after_total_amount]
    end

    #アリペイ連携の場合
    #if params[:order_form] && params[:order_form].normal_payment_type == OrderShipping.payment_type_div_id_by_key(:alipay)
    #  redirect_to :controller => 'sps_payment', :action => 'start_register'
    #  return
    #end
    
    #--- ここまでがバリデーションチェック、ここからは注文登録 ---#

    # 価格計算用にカートを作成する
    cart = Cart.new()
    cart.user = user
    cart.campaign_attached_at = order_form.campaign_attached_at
    cart.add_items(params[:order_item])
    cart.calculate

    # カートから注文を作成
    order = Order.create_new_order(user, shipping_address, order_form, cart)
    order.user_id = user.id

    # 注文情報の登録
    create_order(order)
    order_shipping = order.order_shippings.first
    order_shipping.order = order
    order_shipping.given_point = cart.total_given_point if cart
    order_shipping.arrival_date = order_form.delivery_date + " 00:00:00" if order_form.delivery_date
    
    # Thanksメール送信用の引数
    order_shipping.send_thanks_mail_args = [user]

    # 各決済サービスに決済要求
    order_shipping.payment_arg = order_form, user
    # order_shipping.credit_card_payment if order_shipping.payment_type_div_key == :credit

    # 注文のステータスを進める
    context = order_shipping.process_status_div
    error_messages = nil
    
    case order_shipping.status_div_key
    when :settlement_failed then
      # クレジット処理結果がエラーの場合
      error_messages = context.exceptions.last.message
    when :stock_error then
      # 在庫数が不足している場合
      error_messages = context.exceptions.last.message
    when :internal_error then
      # その他の例外
      deliver_exception_notification(context.exceptions.last)
      return render_500
    end

    # 在庫エラー、決済エラーの場合は新規登録画面へ戻る
    if error_messages
        flash[:order_form] = order_form
        flash[:user] = user
        flash[:order_shipping] = OrderShipping.new(params[:order_shipping])
        flash[:order] = Order.new()
        flash[:order_items] = order_items
        flash[:error_info] = error_messages
        return redirect_to :controller => 'order', :action => 'new'
    end
    
    redirect_to :controller => :order, :action => :order_search, :condition => {:test_flg => CmsPublish.target_id_by_key(:production)}
  end

  desc :auth_as => :update
  # 注文編集時商品選択ダイアログで選択されたものを表示するときに使用するアクション(商品SKU用)
  # 商品SKUダイアログから渡されたproduct_sku_idsパラメータから商品情報を取得し、@order_itemsオブジェクトを生成します。
  def show_order_item_list
    clear_selected_ids_from_session
    @order_items = []
    if params[:product_sku_ids]
      selected_product_sku_ids = params[:product_sku_ids].map(&:to_i)
      product_skus = ProductSku.find(:all, :conditions => ["id in (?)", selected_product_sku_ids])
      shipping = OrderShipping.find_by_id(params[:shipping_id])
      user_rank_id = shipping.order.user.rank.id if (!shipping.order.user.nil? && !shipping.order.user.rank.nil?)
      locale = shipping.order.locale
      total_shipping_cost_point = 0
      product_skus.each do |psku|
        product = psku.product
        total_shipping_cost_point = total_shipping_cost_point + psku.shipping_cost_point.to_i
        # 注文者のロケールにあわせたproduct_displayを取得
        product_display = Product.with_displays(locale).find(:first, :conditions => ['products.id = ?', psku.product.id])

        unit_price = (user_rank_id ? psku.price_for(user_rank_id).to_i : nil) || psku.price.to_i
        total_amount = ((unit_price * 1).to_i + product.shipping_cost_price.to_i)
        sub_total_amount = (unit_price * 1) # カートの小計の仕様と違い、割引額は小計に含まない仕様です。

        @order_items << OrderItem.new({
            :order_shipping_id => shipping.id,
            :product_name => psku.product_name_1,
            :order_product_name_1 => psku.product_name_for_admin_ja,
            :order_product_name_2 => product.order_product_name_2,
            :course_div => product.course_div,
            :unit_price => unit_price,
            :total_amount => total_amount,
            :qty => 1,
            :product_sku_id => psku.id,
            :subtotal_amount => sub_total_amount
        })
      end
    end

    # 送料再計算
    shipping_cost = ShippingCost.calc_shipping_cost(shipping, total_shipping_cost_point, false)
    response_text = <<-EOS
      <%= render :partial => 'edit_order_item_list' %>
      <script type="text/javascript">
        SumTotalAmounts(#{@order_items.size}, #{shipping_cost});
      </script>
    EOS
    render :inline => response_text
  end

  desc :auth_as => :update
  #
  # 注文の郵便番号、国、都道府県、明細の商品SKU、個数(と金額?)によってキャンペーンを適用しなおして
  # 計算を行った後、送料も計算します。
  #
  def recalculate
    #
    # 編集で期待しているパラメータ
    # params =
    #   { "order_shipping_id"=>"11",
    #     "order_shipping"=>{"country_code"=>"JP", "zip_code"=>"1000003", "address_1"=>"東京都", "shipping_cost"=>"100"},
    #     "order_item"=>{
    #       "0"=>{"qty"=>"2", "product_sku_id"=>"265"},
    #       "1"=>{"qty"=>"3", "product_sku_id"=>"267"},
    #       "2"=>{"qty"=>"1", "product_sku_id"=>"269"}
    #     }
    #   }
    #
    # 新規登録で期待しているパラメータ
    # 編集で期待しているパラメータとほぼ同じで、order_shipping_idがuser_idとなります
    #  
    cart = Cart.new
    if params[:order_shipping_id].nil? || params[:order_shipping_id].blank?
    #新規注文登録の場合は、OrderShippingが登録されていないため、パラメータを利用する
      @order_shipping = OrderShipping.new
      @order_shipping.attributes =  params[:order_shipping]
      cart.user = User.find(params[:user_id])
      #cart.campaign_attached_at = Date.new #TODO 指定日付かどうかの判定も行う。
      if !params[:campaign_attached_div].nil? && params[:campaign_attached_div] == OrderForm.campaign_attached_div_id_by_key(:select)
        # 日付指定の場合
        cart.campaign_attached_at = Time.parse(params[:campaign_attached_at])
      else
        #日付指定でない場合場合
        cart.campaign_attached_at = Time.now
      end
    else
      @order_shipping = OrderShipping.find(params[:order_shipping_id], :include => {:order => :user})
      cart.user = @order_shipping.order.user
      cart.campaign_attached_at = @order_shipping.order.campaign_attached_at_or_ordered_at # 注文日時でキャンペーンを検索するための設定
    end
    shipping_cost_point = 0 # 送料ポイント
    order_item_hash = (params[:order_item] || {}).inject({}){|dest, (key, value)| dest[key.to_i] = value; dest}
    # 何度もProductSkuにfindしないようにHashにしまってしまいます。
    product_sku_ids = order_item_hash.values.map{|order_item| order_item[:product_sku_id].to_i}
    product_skus = ProductSku.find(:all,
      :conditions => ["id in (?)", product_sku_ids],
      :include => [:prices, {:product => :product_displays}])
    product_sku_hash = product_skus.inject({}){|d, sku| d[sku.id] = sku; d}
    # カートに商品を追加します。
    order_item_hash.keys.sort.each do |idx|
      order_item = order_item_hash[idx]
      ps = product_sku_hash[order_item[:product_sku_id].to_i]
      qty = order_item[:qty].to_i
      # プレゼント品は一旦カートから削除する
      cart.add(ps, :normal, qty) unless ps.product.present_flg
      shipping_cost_point += (ps.shipping_cost_point.to_i * qty)
    end
    # 送料再計算のためにorder_shippingに国コード、郵便番号、都道府県を指定します。
    @recalc_results = {}
    addressable = params[:order_shipping]
    @order_shipping.country_code = addressable[:country_code]
    @order_shipping.zip_code = addressable[:zip_code]
    @order_shipping.address_1 = addressable[:address_1]
    
    #新規注文登録からの遷移の場合、支払い方法の選択をする
    unless params[:payment_type].nil?
      #支払い方法が代引きの場合、手数料を加算する
      @recalc_results[:cod_charge] = params[:payment_type] == OrderShipping.payment_type_div_id_by_key(:cod) ?
         (ShopSetting.get_setting(:cod_charge, I18n.locale) || 0).to_i : 0
    else
      @recalc_results[:cod_charge] = @order_shipping.cod_charge || 0
    end

    @recalc_results[:shipping_cost] =  ShippingCost.calc_shipping_cost(@order_shipping, shipping_cost_point, cart.free_shipping_cost?) || 0
    # カート再計算
    if @order_shipping && @order_shipping.order && !@order_shipping.order.id.blank?
      cart.secured_stocks = ProductSkuSecureInventory.find_by_order(@order_shipping.order.id)
    end
    cart.calculate
    @recalc_results[:total_given_point] = cart.total_given_point # 付与ポイント
    @recalc_results[:subtotal_amount] = cart.subtotal_amount # 小計
    @recalc_results[:total_amount] = cart.total_amount # 合計(送料を除く)
    # 計算されたカートから明細群を作成します。
    order_locale =  !@order_shipping.order.nil? ? @order_shipping.order.locale : 'JP'
    @order_items = []
    cart.item_list(false).each do |cart_item|
      product_sku = cart_item.product_sku
      product = product_sku.product
      @order_items << OrderItem.new(
        :order_shipping_id => @order_shipping.id,
        :product_name => product_sku.product_name_1,
        :order_product_name_1 => product_sku.product_name_for_admin_ja,
        :order_product_name_2 => product.order_product_name_2,
        :course_div => cart_item.course_div,
        :unit_price => cart_item.unit_price,
        :total_amount => cart_item.total_amount,
        :qty => cart_item.qty,
        :product_sku_id => product_sku.id,
        :subtotal_amount => cart_item.subtotal_amount,
        :discount_amount => cart_item.subtotal_amount - cart_item.total_amount
        )
    end
    @recalc_results[:applied_campaign_list] = cart.applied_campaigns_for_cart_and_items
    render :partial => 'edit_order_item_list'
  end
  private
  # 選択済みの商品／商品SKUのIDを格納しているセッションを削除します
  def clear_selected_ids_from_session
    session[:selected_product_sku_ids] = nil
  end

  public

  def update
    # バリデーションチェック
    order_shipping = OrderShipping.find_by_id(params[:order_shipping][:id])
    # 更新前注文情報を退避(後に比較する箇所があるため)
    before_order_shipping = OrderShipping.new(order_shipping.attributes)
    # 編集内容をインスタンスへ反映する
    order_shipping.attributes = params[:order_shipping]
    order_shipping.order.attributes = params[:order]

    # 注文商品を作成してバリデーションチェック
    order_items = OrderItem.create_order_items(params[:order_item])
    flash[:error_info] = "注文商品は最低１商品登録してください。" if order_items.size <= 0
    order_item_no_error = true
    order_items.each do |order_item|
      order_item_no_error = false unless order_item.valid?
    end

    unless (order_shipping.validates_for && order_item_no_error && flash[:error_info].nil?)
      flash[:order_shipping] = order_shipping
      flash[:order] = order_shipping.order
      flash[:order_items] = order_items
      redirect_to :action => 'edit', :id => order_shipping.id, :page => params[:page], :after_total_amount => params[:after_total_amount]
      return
    end

    # 注文情報更新
    begin
      # キャンペーン計算用にカートを作成する
      cart = Cart.new()
      cart.user = order_shipping.order.user
      cart.campaign_attached_at = order_shipping.order.campaign_attached_at_or_ordered_at
      cart.add_items(params[:order_item])
      cart.calculate

      if params[:order_shipping][:arrival_date]
        order_shipping.arrival_date = params[:order_shipping][:arrival_date] + " 00:00:00"
      end

      OrderShipping.transaction do
        # 受注金額が変更された場合
        if order_shipping.total_amount != params[:after_total_amount]
          order_shipping.total_amount = params[:after_total_amount]
          # TODO ペイメントへの変更処理
        end

        # 注文商品の紐付け、更新
        order_shipping.update_order_items(order_items) #商品構成情報もこの中で更新する
        order_shipping.order.save!
        order_shipping.save!

        # ポイント調整処理
        UserPoint.update_inactive_point(order_shipping.order.id, cart.given_point)

        # 紐付くキャンペーンの更新
        order_shipping.order.update_campaigns(cart)

        # TODO 同送処理

        #ギフト指定のフラグを更新する
        order_shipping.order.update_attributes(
          :gift_flg => params[:order][:gift_flg],
          :send_invoice_separately_flg => params[:order][:send_invoice_separately_flg],
          :write_sender_flg => params[:order][:write_sender_flg]
          )
      end
      redirect_to :action => 'order_detail', :id => order_shipping.order_id, :page => params[:page]

    # 在庫が不足している場合
    rescue OrderItem::OutOfStockError => stock_error
      flash[:error_info] = stock_error.message
      redirect_to :action => 'edit', :id => order_shipping.id, :page => params[:page]
      return
    end
  end

  desc :auth_as => :update, :display_name => "ステータスを入金待ちに戻す"
  def back_to_undelivery_unpaid
    order_shipping = OrderShipping.find(params[:id])
    order_shipping.back_to_undelivery_unpaid
    @order_shipping = order_shipping
    render :inline => order_shipping.status_div_name
  end

  desc :auth_as => :update, :display_name => "ステータスを配送指示済みに戻す"
  def back_to_deliver_request
    order_shipping = OrderShipping.find(params[:id])
    order_shipping.back_to_deliver_request
    @order_shipping = order_shipping
    render :inline => order_shipping.status_div_name
  end

  desc :auth_as => :update, :display_name => "キャンセル"
  def cancel
    order_shipping = OrderShipping.find(params[:id])
    order_shipping.cancel
    @order_shipping = order_shipping
    render :inline => order_shipping.status_div_name
  end

  desc :auth_as => :update, :display_name => "返品"
  # 返品
  # ポイント削除、経理データ登録を行います。
  # 在庫の調整は行いません。
  def returned
    order_shipping = OrderShipping.find(params[:id])
    order_shipping.returned_request
    @order_shipping = order_shipping
    render :inline => order_shipping.status_div_name
  end

  desc :category_name => :destroy, :display_name => "削除"
  # 削除

  # 本番データは論理削除、テストデータは物理削除します。
  def delete
    if params[:destroy_all_test]
      order_ids = Order.find(:all, :conditions => ["test_flg = ?", true]).map{|user| user.id}
    elsif params[:test_id]
      order_ids =[params[:test_id]]
    else
      order_ids =[params[:id]]
    end

    if params[:destroy_all_test] || params[:test_id]
      # テストデータは物理削除
      order_ids.each do |order_id|
        ActiveRecord::Base.transaction do
          # 注文見出しと注文を削除
          order_shippings = OrderShipping.find(:all, :conditions => ["order_id = ?", order_id])

          order_shippings.each do|order_shipping|
            order_items = OrderItem.find(:all, :conditions => ["order_shipping_id = ?", order_shipping.id])
            order_items.each {|order_item|order_item.destroy!} if order_items # 注文を削除

            order_shipping.delete_send_address # 個人情報を削除
            order_shipping.destroy!
          end if order_shippings

          # EC注文見出しを削除
          order = Order.find(order_id)
          order.destroy! if order
        end
      end
    else
      # 本番データは論理削除
      ActiveRecord::Base.transaction do
        # 注文見出しと注文を削除
        order_shippings = OrderShipping.find(:all, :conditions => ["order_id = ?", params[:id]])

        order_shippings.each do|order_shipping|
          order_items = OrderItem.find(:all, :conditions => ["order_shipping_id = ?", order_shipping.id])
          order_items.each {|order_item|order_item.destroy_with_stock_control} if order_items # 注文を削除

          order_shipping.delete_send_address # 個人情報を削除
          order_shipping.destroy
        end if order_shippings

        # EC注文見出しを削除
        order = Order.find(params[:id])
        order.destroy if order
      end
    end

    redirect_to :action => 'order_search', :page => params[:page]
  end

  desc :auth_as => :order_search
  def mail_detail
    @mail_delivery = MailDelivery.find_by_id(params[:mail_id])
  end

  private
  # sessionsクリア =
  def clear_sessions
    session[:tmp_order] = nil
    session[:tmp_order_shipping] = nil
    session[:tmp_order_items] = nil
  end


  public

  desc :auth_as => :update
  # 注文ステータス一括更新
  def update_status
    status_tab_key = Order::SearchInfo.status_tab_key_by_id(params[:status_tab])
    entry = OrderShipping.status_div_entry_by_key(status_tab_key)
    if entry.nil? || entry[:event_to_next].nil?
      raise ArgumentError, "Illegal status_div #{params[:origin_status]}"
    end

    # 必要なメールアドレスが店舗設定に設定されているかチェック
    not_set_mail_keys= []
    if [:undelivery_unpaid, :undelivery, :deliver_request, :delivered, :cancel_request].include?(status_tab_key) && ShopSetting[:shop_mail].nil?
      not_set_mail_keys << :shop_mail
    end
    if status_tab_key == :undelivery && ShopSetting[:shipment_instruction_mail].nil?
      not_set_mail_keys << :shipment_instruction_mail
    elsif status_tab_key == :deliver_request && ShopSetting[:shipment_complete_mail].nil?
      not_set_mail_keys << :shipment_complete_mail
    end
    unless not_set_mail_keys.empty?
      mail_name = not_set_mail_keys.map{|mail_key| ShopSetting.setting_div_name_by_key(mail_key)}.join(',')
      flash[:status_update_error] = "店舗情報に　#{mail_name}　が設定されていません。"
      redirect_to(:action => :index_tab, :page => params[:page],
        :status_tab => params[:status_tab], :inclusion => params[:inclusion])
      return
    end

    order_shippings = OrderShipping.find(:all, :conditions => ["id in (?)", params[:selected_order_shipping_ids]])
    order_shippings.each do |order_shipping|
      # 残念ながら注文ステータスによっては送信するメールをビューでrenderしており、helperのメソッドなどを使用しています。
      # 本来はビューとは無関係のERBは使用するべきです。作り直す時間が無いのでステータスを遷移する前に送信するメールを
      # コントローラで作成しています。
      prepare_order_shipping_before_update_status(order_shipping)
      order_shipping.send(entry[:event_to_next])
    end
    redirect_to(:action => :index_tab, :page => params[:page],
      :status_tab => params[:status_tab], :inclusion => params[:inclusion])
  end

  private

  def prepare_order_shipping_before_update_status(order_shipping)
    case order_shipping.status_div_key
    when :undelivery then
      prepare_delivery_request_mail(order_shipping)
    end
  end

  def prepare_delivery_request_mail(order_shipping)
    @order = Order.find(order_shipping.order_id, :include => :order_shippings)
    @order_shipping = order_shipping
    @user = User.find(@order.user_id)
    @user_address = @user.user_address
    order_shipping.shipment_content = render_to_string(:partial => 'shipment_instruction')
    if @order.locale == 'ja'
      order_shipping.invoice_content = render_to_string(:layout => false, :action => 'print_ja')
    else
      i18n_tmp_block('en') do
        order_shipping.invoice_content = render_to_string(:layout => false, :action => 'print_en')
      end
    end
  end

  def create_order(order)
    Order.transaction do
      user = User.find(order.user_id)
      order.save!
      if user.member_div = :member
        after_create_order_for_member(order)
      else
      # 非会員の場合
        after_create_order_for_non_member(order)
      end
    end
  end

  # 会員が注文した後に呼び出されます
  # プラグインで拡張するためのメソッド
  def after_create_order_for_member(order)
  end

  # 非会員が注文した後に呼び出されます
  # プラグインで拡張するためのメソッド
  def after_create_order_for_non_member(order)
  end
end
