# -*- coding: utf-8 -*-
class ProductDialogController < ApplicationController
  class_desc :display_name => "商品選択ダイアログ", :invisible => true, :auth_as => 'product'

  # ダイアログ内の検索ボタン押下時などの処理
  #
  def index
    @product_dialog = Product::DialogForm.new(
      (params[:product_dialog] || {}).update({:page => params[:page]}))
    if @product_dialog.selected_product_ids && !@product_dialog.selected_product_ids.empty?
      session[:selected_product_ids] = @product_dialog.selected_product_ids
    end
    if @product_dialog.selected_product_sku_ids && !@product_dialog.selected_product_sku_ids.empty?
      session[:selected_product_sku_ids] = @product_dialog.selected_product_sku_ids
    end
    
    @products = Product.paginate(@product_dialog.to_paginate_options(:include => :skus))
    if params[:result_only]
      render :partial => 'list'
    else
      render :action => 'index', :layout => false
    end
  end

  desc :auth_as => :index
  # ダイアログで選択されたものを表示するときに使用するアクション
  #
  def index_for_client
    clear_selected_ids_from_session
    @sku_mode = !!params[:product_sku_ids]
    if @sku_mode
      @selected_product_sku_ids = params[:product_sku_ids].map(&:to_i)
      @products = Product.find(:all,
        :include => :skus,
        :joins => "inner join product_skus cond_product_skus on cond_product_skus.product_id = products.id",
        :conditions => ["cond_product_skus.id in (?)", @selected_product_sku_ids ],
        :order => "products.product_code asc")
    else
      @selected_product_sku_ids = []
      @products = Product.find(:all,
        :conditions => ["products.id in (?)", params[:product_ids] ],
        :order => "products.product_code asc")
    end
    render :partial => 'index_for_client'
  end
  
  desc :auth_as => :index
  # ダイアログで選択されたものを表示するときに使用するアクション(商品SKU用)
  # ※表示したいレイアウトがindex_for_clientで表示するレイアウトと違うので追加しました。
  def index_for_product_sku
    clear_selected_ids_from_session
    selected_product_sku_ids = params[:product_sku_ids].map(&:to_i) if params[:product_sku_ids]
    @product_skus = ProductSku.find(:all, :conditions => ["id in (?)", selected_product_sku_ids])
    render :partial => 'index_for_product_sku'
  end

  desc :auth_as => :index
  # ダイアログで選択されたものを表示するときに使用するアクション（商品SKU１つだけの場合）
  def show_product_sku
    clear_selected_ids_from_session
    selected_product_sku_id = params[:product_sku_id].to_i if params[:product_sku_id]
    product_sku = ProductSku.find(:first, :conditions => ["id = ?", selected_product_sku_id])
    display_text = product_sku ? "[#{product_sku.sku_code}] #{product_sku.product_name_for_admin_ja} #{product_sku.name_for_admin}" : ''
    render :text => display_text
  end

  desc :auth_as => :index
  # ダイアログで選択されたものを表示するときに使用するアクション(関連商品グループ用)
  # ※表示したいレイアウトがindex_for_clientで表示するレイアウトと違うので追加しました。
  def index_for_product_group
    clear_selected_ids_from_session
    selected_product_sku_ids = params[:product_sku_ids].map(&:to_i) if params[:product_sku_ids]
    @product_skus = ProductSku.find(:all, :conditions => ["id in (?)", selected_product_sku_ids])
    render :partial => 'index_for_product_group'
  end

  private
  # 選択済みの商品／商品SKUのIDを格納しているセッションを削除します
  #
  def clear_selected_ids_from_session
    session[:selected_product_ids] = nil
    session[:selected_product_sku_ids] = nil
  end

end
