# -*- coding: utf-8 -*-
class ProductDisplaysController < ApplicationController
  class_desc :display_name => "商品表示名管理", :invisible => true, :auth_as => 'product'

  def bulk_update_product_id
    product_displays = ProductDisplay.find(:all, :conditions => ["id in (?)", params[:product_display_ids]])
    ProductDisplay.transaction do
      product_displays.each do |product_display|
        old_product = product_display.product
        product_display.product_id = params[:product_id]
        product_display.save!
        if old_product.product_displays.count == 0
          new_product = Product.find(product_display.product_id)
          new_product.stock_count += old_product.stock_count
          old_product.stock_count = 0
          new_product.save!
          old_product.save!
        end
      end
    end
    redirect_to :action => :index
  end

  # GET /product_displays
  # GET /product_displays.xml
  def index
    @product_displays = ProductDisplay.find(:all, :include => :product, :order => "products.product_code asc, products.display_order asc")

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @product_displays }
    end
  end

  # GET /product_displays/1
  # GET /product_displays/1.xml
  def show
    # セッションがあればセッションの中身を表示する
    @product_display = session[:product_form][:product_displays][params[:locale]] if session[:product_form]
    @product_display ||= ProductDisplay.find(params[:id])
    @product = @product_display.product

    respond_to do |format|
      format.html { request.xhr? ? render(:partial => 'show') : render(:action => 'show') }
      format.xml  { render :xml => @product_display }
    end
  end

  # GET /product_displays/new
  # GET /product_displays/new.xml
  def new
    @product = request.xhr? ? Product.find(params[:product_id]) : nil
    
    # セッション内のproduct_displayを更新し、次のタブに表示する内容を用意する
    update_session_product_display(params[:product_display])
    @product_display = get_product_display_from_session(ProductDisplay.new(:product => @product, :locale => params[:locale]))

    respond_to do |format|
      format.html { request.xhr? ? render(:partial => 'form') : render(:action => 'new') }
      format.xml  { render :xml => @product_display }
    end
  end

  # GET /product_displays/1/edit
  def edit
    # セッション内のproduct_displayを更新し、次のタブに表示する内容を用意する
    update_session_product_display(params[:product_display])
    @product_display = get_product_display_from_session(ProductDisplay.find(params[:id]))
    @product = @product_display.product
    request.xhr? ? render(:partial => 'form') : render(:action => 'edit')
  end

  # POST /product_displays
  # POST /product_displays.xml
  def create
    @product_display = ProductDisplay.new(params[:product_display])

    respond_to do |format|
      if @product_display.save
        flash[:notice] = 'ProductDisplay was successfully created.'
        format.html { redirect_to(@product_display) }
        format.xml  { render :xml => @product_display, :status => :created, :location => @product_display }
      else
        format.html { render :action => "new" }
        format.xml  { render :xml => @product_display.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /product_displays/1
  # PUT /product_displays/1.xml
  def update
    @product_display = ProductDisplay.find(params[:id])

    respond_to do |format|
      if @product_display.update_attributes(params[:product_display])
        flash[:notice] = 'ProductDisplay was successfully updated.'
        format.html { redirect_to(@product_display) }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @product_display.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /product_displays/1
  # DELETE /product_displays/1.xml
  def destroy
    @product_display = ProductDisplay.find(params[:id])
    @product_display.destroy

    respond_to do |format|
      format.html { redirect_to(product_displays_url) }
      format.xml  { head :ok }
    end
  end
  
  private
  # 引数に渡したproduct_displayがsessionに存在すればsessionにあるproduct_displayを返す
  # 引数に渡したproduct_displayがsessionに存在しなければ、sessionに登録して返す
  def get_product_display_from_session(product_display)
    result = session[:product_form][:product_displays][product_display.locale]
    unless result
      session[:product_form][:product_displays][product_display.locale] = product_display
      result = product_display
    end
    return result
  end
  
  # session内のproduct_displayを、引数に渡した値で更新する
  def update_session_product_display(params_product_display)
    locale = params_product_display[:locale]
    product_display = session[:product_form][:product_displays][locale]
    
    if product_display.id || edit_product_display?(params_product_display)
      product_display.attributes = params[:product_display]
      session[:product_form][:product_displays][locale] = product_display
    else
      # product_displayが未登録で、編集されていなければsessionから削除する
      session[:product_form][:product_displays].delete(locale)
    end
  end
  
  # product_displayの入力項目が１つ以上入力されていたらtrueを返す
  def edit_product_display?(params_product_display)
    return false unless params_product_display
    params_product_display.each do |key, value|
      return true unless key == 'locale' || key == 'product_id' || value.blank?
    end
    return false
  end
end
