# -*- coding: utf-8 -*-
class ProductGroupsController < ApplicationController
  class_desc :display_name => "商品グループ管理"

  # GET /product_groups
  def index
    @condition = prepare_session_condition do |condition|
      condition || ProductGroup::SearchCondition.new(params[:condition])
    end
    @condition.page = params[:page]
    @product_groups = ProductGroup.paginate(
      @condition.to_paginate_options(:order => "id DESC"))
  end

  # GET /product_groups/1
  def show
    @product_group = ProductGroup.find(params[:id])
  end

  # GET /product_groups/new
  def new
    # 商品SKU選択ダイアログのチェックボックス表示用の情報をクリアしておく
    session[:selected_product_ids] = nil
    session[:selected_product_sku_ids] = nil
    
    @product_group = ProductGroup.new
  end

  # GET /product_groups/1/edit
  def edit
    @product_group = ProductGroup.find(params[:id])
  end

  # POST /product_groups
  def create
    @product_group = ProductGroup.new(params[:product_group])

    if @product_group.save
      flash[:notice] = 'ProductGroup was successfully created.'
      redirect_to(@product_group)
    else
      render :action => "new" 
    end
  end

  # PUT /product_groups/1
  def update
    @product_group = ProductGroup.find(params[:id])

    begin
      ProductGroup.transaction do
        # 登録済みのSKUはここで削除します。
        @product_group.skus.destroy_all if @product_group.skus.size > 0

        @product_group.attributes = params[:product_group]
        @product_group.save!
      end
      #flash[:notice] = 'ProductGroup was successfully updated.'
      redirect_to(@product_group)
    rescue ActiveRecord::RecordInvalid
      render :action => "edit"
    end
  end

  # DELETE /product_groups/1
  def destroy
    @product_group = ProductGroup.find(params[:id])
    @product_group.destroy
    redirect_to(product_groups_url)
  end
end
