# -*- coding: utf-8 -*-
class ProductImagesController < ApplicationController
  class_desc :display_name => "商品画像管理", :invisible => true, :auth_as => 'product'

  desc :auth_as => :update
  def sort
    product_id = params[:id]
    @product = Product.find(product_id.to_i)
    
    params[:sortable_product_images].each_with_index { |product_image_id, index|
      ProductImage.find(product_image_id.to_i).update_attributes(:display_order => index + 1)
    }
    @product_images = ProductImage.find(:all, :conditions => ["parent_id IS NULL AND product_id = ?", @product.id], :order => "display_order")
    render :partial => 'index'
  end

  # GET /product_images
  def index
    @product_images = ProductImage.find(:all, :conditions => ["parent_id IS NULL"], :order => "display_order")
  end

  # GET /product_images/1
  def show
    @product_image = ProductImage.find(params[:id])
  end

  # GET /product_images/new
  def new
    @product_image = ProductImage.new
  end

  # GET /product_images/1/edit
  def edit
    @product_image = ProductImage.find(params[:id])
  end

  # POST /product_images
  def create
    @product_image = ProductImage.new(params[:product_image])
    @product_image.main_flg = true if ProductImage.count(:conditions => ["product_id = ?" , @product_image.product_id]) == 0
    
    unless @product_image.save
      flash[:product_image] = @product_image
    end
    @product = Product.find(@product_image.product_id)
    @product_images = @product.images
    responseText = %{
      <%= javascript_include_tag '/ec_rider/javascripts/prototype.js' %>
      <%= javascript_include_tag '/ec_rider/javascripts/effects.js' %>
      <%= javascript_include_tag '/ec_rider/javascripts/dragdrop.js' %>
      <%= javascript_include_tag '/ec_rider/javascripts/controls.js' %>
      <div id="partial_content">
        <table id="sortable_product_images" width="100%">
          <%= render :partial => "product_images/index" %>
        </table>
      </div>

      <script type="text/javascript">
      //<![CDATA[
        var content = document.getElementById("partial_content");
        window.parent.updateImageList(content.innerHTML);
      //]]>
      </script>
    }.split(/$/).map(&:strip).join("\n")
    render(:inline => responseText)
  end

  # PUT /product_images/1
  def update
    @product_image = ProductImage.find(params[:id])

    if @product_image.update_attributes(params[:product_image])
      flash[:notice] = 'ProductImage was successfully updated.'
      redirect_to(@product_image)
    else
      render :action => "edit"
    end
  end

  desc :auth_as => :update
  #
  # = 商品画像のメイン画像を更新するアクション =
  #
  def update_main_flg
    product_id = params[:product_id].to_i
    main_product_image_id = params[:main_product_image_id].to_i

    ProductImage.find(:all, :conditions => ["product_id = ?", product_id]).each do |pi|
      # メイン画像を更新する
      if pi.id == main_product_image_id
        pi.update_attributes(:main_flg => true)
      else
        pi.update_attributes(:main_flg => false)
      end
    end

    render :nothing => true
  end
  
  # DELETE /product_images/1
  def destroy
    @product_image = ProductImage.find(params[:id])
    
    ProductImage.transaction do
      # 表示順とメイン画像を更新
      @product_image.update_same_product_id_images
      # 商品画像を削除
      @product_image.destroy
    end
 
    @product = Product.find(@product_image.product_id)
    @product_images = @product.images
    responseText = %{
      <table id="sortable_product_images" width="100%">
        <%= render :partial => "product_images/index" %>
      </table>
    }.split(/$/).map(&:strip).join("\n")
    render(:inline => responseText)
  end
end
