# -*- coding: utf-8 -*-
class ProductSkusController < ApplicationController
  class_desc :display_name => "商品SKU管理"
  
  # ANY /product_skus/1
  def show
    # 商品SKU
    @product_sku = ProductSku.find(params[:id])
    # 商品
    @product = Product.find(@product_sku.product_id)
    @product_components = @product_sku.components

    # 商品SKUがリクエストされた場合（確認画面）
    product_sku = params[:product_sku]
    if product_sku
      # 商品構成が登録されていれば削除
      @product_components = []
      
      # リクエストパラメータをセッションとインスタンスに設定
      session[:tmp_product_sku] = product_sku
      @product_sku.attributes = product_sku if @product_sku.attributes

      ## 入力チェック
      unless @product_sku.valid?
        render(:action => :edit)
        return
      end

      unless product_sku[:components_attributes].nil?
#        @product_sku.components_attributes = product_sku[:components_attributes]
        @product_components = []
        product_sku[:components_attributes].size.times do |i|
          @product_components << ProductComponent.new(product_sku[:components_attributes][i.to_s])
        end
      end

    end
  end

  # GET /product_skus/new
  def new
    # 商品SKU選択ダイアログのチェックボックス表示用の情報をクリアしておく
    session[:selected_product_ids] = nil
    session[:selected_product_sku_ids] = nil
    
    # 商品
    @product = Product.find(params[:product_id])
    # 商品SKU
    @product_sku = ProductSku.new
    @product_sku.primary_sku_flg = true unless @product.primary_sku
    # 商品売価
    UserRank.all.each do |user_rank|
      @product_sku.prices << ProductPrice.new(:user_rank_id => user_rank.id)
    end

    # 商品SKUのバリエーション項目
    variation_item_ids = params[:variation_item_ids]
    variation_item_ids.each_with_index do |variation_item_id, index|
      eval("@product_sku.variation_item_#{index + 1}_id = variation_item_id")
    end
  end

  # GET /product_skus/1/edit
  def edit
    @product_sku = ProductSku.find(params[:id])
    if session[:tmp_product_sku]
      @product_sku = ProductSku.new(session[:tmp_product_sku])
      @product_sku.id = params[:id] 
    end
    @product = Product.find(@product_sku.product_id)

    # 確認画面表示に用いる一時情報を削除
    session[:tmp_product_sku] = nil if params[:clear]
  end

  # POST /product_skus
  def create
    @product_sku = ProductSku.new(params[:product_sku])
    @product = Product.find(@product_sku.product_id)

    begin
      ProductSku.transaction do
        @product_sku.save!
        @product_sku.update_composable_count
      end
      redirect_to :controller => :product, :action => :show, :id => @product_sku.product_id
    rescue
      render :action => :new
    end
  end

  # PUT /product_skus/1
  def update
    @product_sku = ProductSku.find(params[:id])
    @product = Product.find(@product_sku.product_id)

    begin
      ProductSku.transaction do
        # 登録済みの商品売価、商品構成はここで削除
        @product_sku.prices.delete_all if @product_sku.prices.size > 0
        @product_sku.components.delete_all if @product_sku.components.size > 0
        @product_sku.attributes = params[:product_sku] if params[:product_sku]
        @product_sku.save!
        @product_sku.update_composable_count
      end

      # 確認画面表示に用いる一時情報を削除
      session[:tmp_product_sku] = nil
      
      redirect_to(:controller => :product, :action => :show, :id => @product_sku.product_id)
    rescue ActiveRecord::RecordInvalid
      render :action => :edit
    end
  end

  # DELETE /product_skus/1
  def destroy
    @product_sku = ProductSku.find(params[:id])
    @product_sku.destroy
    flash[:sku_error_messages] = @product_sku.errors.full_messages
    redirect_to(:controller => :product, :action => :show, :id => @product_sku.product_id)
  end
end
