# -*- coding: utf-8 -*-
class ProductVariationsController < ApplicationController
  class_desc :display_name => "商品バリエーション管理"

  # GET /product_variations
  def index
    @condition = prepare_session_condition do |condition|
      condition || ProductVariation::SearchCondition.new(params[:condition])
    end
    per_page = @condition.disp_count.to_i if @condition
    per_page = (per_page.to_i == 0) ? 20 : per_page

    @product_variations = ProductVariation.paginate(
      @condition.to_find_options(
        :page => params[:page], :per_page => per_page, :order => "product_variations.id DESC", :include => :items))
  end

  # GET /product_variations/1
  def show
    @product_variation = ProductVariation.find(params[:id], :include => [:displays, :items])
  end

  # GET /product_variations/new
  def new
    @product_variation = ProductVariation.new
    I18n.supported_locales.each do |locale|
      @product_variation.displays << ProductVariationDisplay.new(:locale => locale.to_s)
    end
  end

  # GET /product_variations/1/edit
  def edit
    @product_variation = ProductVariation.find(params[:id], :include => [:displays, :items])
  end

  # POST /product_variations
  def create
    @product_variation = ProductVariation.new(params[:product_variation])

    if @product_variation.save
      flash[:notice] = 'ProductVariation was successfully created.'
      redirect_to(@product_variation)
    else
      render :action => "new"
    end
  end

  # PUT /product_variations/1
  def update
    @product_variation = ProductVariation.find(params[:id])

    if @product_variation.update_attributes(params[:product_variation])
      flash[:notice] = 'ProductVariation was successfully updated.'
      redirect_to(@product_variation)
    else
      render :action => "edit"
    end
  end

  # DELETE /product_variations/1
  def destroy
    @product_variation = ProductVariation.find(params[:id])
    @product_variation.destroy

    redirect_to(product_variations_url)
  end
end
