class ShippingCostsController < ApplicationController
  class_desc :display_name => '送料管理'

  def index
    @condition = prepare_session_condition do |condition|
      condition || Country::SearchCondition.new(params[:condition])
    end
    if @condition.valid?
      per_page = @condition.disp_count ? @condition.disp_count.to_i : 20
      @countries = Country.paginate_by_sql(
        @condition.get_finder_sql, {:page => params[:page], :per_page => per_page})
    end
    @boundaries = ShippingCostBoundary.all
  end

  desc :auth_as => :update
  def edit_point
    # セッション初期化
    if params[:session_crear]
      session[:boundary_points] = nil
      session[:shipping_cost_countries] = nil
      session[:sipping_costs] = nil
      session[:updatable_country_codes] = nil
    end

    if session[:boundary_points]
      @boundary_points = session[:boundary_points]
      @countries = Country.set_ship_restriction_able(session[:updatable_country_codes])
      session[:boundary_points] = nil
      session[:shipping_cost_countries] = nil
      session[:updatable_country_codes] = nil
    else
      if flash[:error]
        @boundary_points = flash[:boundary_points]
        @countries = Country.set_ship_restriction_able(flash[:countries])
      else
        @boundary_points = ShippingCostBoundary.find(:all, :order => 'id').map{|boundary|boundary.point}
        @countries = Country.all
      end
    end
  end

  desc :auth_as => :update
  def edit_shipping_cost
    if session[:boundary_points]
      @boundary_points = session[:boundary_points]
      @countries = session[:shipping_cost_countries]
      @prefectures = session[:shipping_cost_prefectures]
      @shipping_costs = session[:sipping_costs]
      session[:sipping_costs] = nil
    else
      points = []
      for i in 0..18
        unless params[:boundaries][i.to_s][:point].blank?
          points << params[:boundaries][i.to_s][:point].to_i
        end
      end
      # ポイントから新しいShippingCostBoundaryを生成して配列に格納
      boundary_points = ShippingCostBoundary.sort_boundaries(points)
      # 今まであった閾値のIDを取得
      boundary_ids = ShippingCostBoundary.get_boundary_ids(boundary_points)
      valid_success_flg = true
      error_message = ""
      # バリデーション
      if params[:countries].nil? || !valid_success_flg
        error_message = "<li>配送可能国は必ず１種類以上 設定してください。</li>" if params[:countries].nil?
        flash[:error] = error_message
        flash[:boundary_points] = boundary_points
        flash[:countries] = params[:countries] || []
        return redirect_to :action => 'edit_point', :page => params[:page]
      end
      # 更新する国コードをセッションに保存
      session[:updatable_country_codes] = params[:countries]
      boundary_points << "" # ←｢それ以上｣の閾値を閾値無しで登録するため空の閾値を追加しています。
      @boundary_points = session[:boundary_points] = boundary_points
      session[:shipping_cost_countries] = @countries = Country.find_by_sql(Country.get_sql_for_shipping_costs_countries(boundary_ids, params[:countries]))
      session[:shipping_cost_prefectures] = @prefectures = Country.find_by_sql(Country.get_shipping_costs_sql_for_prefectures(boundary_ids))
      @shipping_costs = []
      @countries.each do |country|
        boundary_points.each_with_index do |scb, index|
          @shipping_costs << ShippingCost.new(:shipping_cost => country["shipping_cost_" + index.to_s]) unless country.code == "JP"
        end
      end
      @prefectures.each do |pre|
        boundary_points.each_with_index do |scb, index|
          @shipping_costs << ShippingCost.new(:shipping_cost => pre["shipping_cost_" + index.to_s])
        end
      end
    end
  end

  desc :auth_as => :index
  def confirm
    @boundary_points = session[:boundary_points]
    size = params[:shipping_costs].size
    shipping_costs = []
    valid_success_flg = true
    error_message = []
    for i in 0..(size -1)
      shipping_cost = ShippingCost.new(params[:shipping_costs][i.to_s])
      unless shipping_cost.valid?
        error_message << "<li>" + shipping_cost.errors.full_messages.join + "</li>"
        valid_success_flg = false
      end
      shipping_costs << shipping_cost
    end
    @shipping_costs = session[:sipping_costs] = shipping_costs
    unless valid_success_flg
      flash[:error] = error_message.uniq.join
      return redirect_to :action => 'edit_shipping_cost', :page => params[:page]
    end
  end

  def update
    boundary_points = session[:boundary_points]
    countries = Country.set_ship_restriction_able(session[:updatable_country_codes])
    shipping_costs = session[:sipping_costs]
    ShippingCostBoundary.transaction do
      ShippingCostBoundary.replace_boundary_points(boundary_points, shipping_costs)
      countries.each do |country|
        country.save!
      end
      session[:boundary_points] = nil
      session[:shipping_cost_boundaries] = nil
      session[:shipping_cost_countries] = nil
      session[:shipping_cost_prefectures] = nil
      session[:sipping_costs] = nil
      session[:updatable_country_codes] = nil
    end
    return redirect_to :action => 'index', :page => params[:page]
  end
  desc :auth_as => :update
  def edit_country
    @country = Country.find(params[:id])
  end

  desc :auth_as => :update
  def update_country
    country = Country.find(params[:id])
    country.update_attributes(params[:country])
    return redirect_to :action => 'index', :page => params[:page]
  end

  desc :auth_as => :index
  def show_country
    @country = Country.find(params[:id])
  end

end
