# -*- coding: utf-8 -*-
class TestEnvironmentSetupController < ApplicationController
  class_desc :display_name => "テスト環境設定管理"

  def index
    find_values_for_display
    @cms_publish = CmsPublish.new
    @cms_dead_link_check = CmsDeadLinkCheck.new
    @cms_dead_link_check.cms_publish_id = @last_cms_publish.id if @last_cms_publish
  end

  desc :display_name => 'テスト環境作成'
  def create
    @cms_publish = CmsPublish.new(params[:cms_publish].merge(
      :target => CmsPublish.target_id_by_key(:test),
      :status_key => :waiting))

    # 対象日時が公開期間外のプロジェクトを取り除く
    @cms_publish.projects.delete_if { |p| (@cms_publish.designated_at < p.started_at) || (@cms_publish.designated_at > p.finished_at) }

    @cms_publish.use_only_view = true
    if @cms_publish.save
      redirect_to :action => 'index'
    else
      find_values_for_display
      render :action => 'index'
    end
  end

  desc :display_name => "リンク切れチェック"
  def create_dead_link_check
    link_check = CmsDeadLinkCheck.new(params[:cms_dead_link_check].merge(
      :status_key => :waiting))
    link_check.save!
    find_values_for_display
    render :nothing => true
  end

  private
  def find_values_for_display
    @last_cms_publish = CmsPublish.find(:first, :order => 'cms_publishes.created_at desc', 
      :include => :dead_link_checks,
      :conditions => ["target = ?",CmsPublish.target_id_by_key(:test)])
    if @last_cms_publish && !@last_cms_publish.dead_link_checks.empty?
      @last_cms_link_check = CmsDeadLinkCheck.find(@last_cms_publish.dead_link_checks.first.id, 
        :include => {:resources => [
          :links, 
          {:cms_bind => {:cms_page => :titles}}, 
          :cms_template,
          :cms_file,
          {:product => :product_displays},
          {:product_image => :product},
          {:product_display => :product},
          :banner,
          {:banner_display => :banner},
          :shop_setting
        ]}
      )
    end
    @shop_setting = ShopSetting.get_setting(:shop_mail_test, 'ja')
    @cms_projects = CmsProject.find(:all, :order => 'cms_projects.id',
      :conditions => ["status_cd not in (? , ?)", CmsProject.status_id_by_key(:opened), CmsProject.status_id_by_key(:closed)])
  end
end
