# -*- coding: utf-8 -*-
class UserController < ApplicationController
  include EcRider::PluginModule::Extendable
  class_desc :display_name => "会員管理"

  # 検索条件を初期化してindexを開く
  def init_index
    session[:user_search_cond] = nil
    redirect_to(:action => 'index')
  end

  # 検索画面
  def index
    # 検索条件があれば検索する
    if params[:condition]
      session[:user_search_cond] = trim_values(params[:condition])
    end
    if session[:user_search_cond]
      @condition = User::SearchConditionForm.new(session[:user_search_cond])
      return unless @condition.valid?
      @users = User.paginate(
        :page => params[:page],
        :per_page => @condition.disp_count,
        :include => [:user_address, :user_email],
        :conditions => @condition.build_condition)
    end
    # CSVファイル出力の中から会員検索が呼ばれた場合
    if request.xml_http_request?
      render :partial => "/user/user_search_result", :layout => false, :locals => {:invoked => 'csv'}
    end
  end

  # 詳細
  # 
  def show
    @user = User.find params[:id]
    @user_address = @user.user_address
    @page_no = params[:page]
    render :action => :show_test_user if @user.test?
  end

  #
  # 編集
  #
  def edit
    @page_no = params[:page]
    # バリデーションエラーで戻った場合
    if flash[:user]
      @user = flash[:user]
      @user_address = flash[:user_address]
      @sub_address = flash[:sub_address]
      @shipping_addresses = flash[:shipping_address]
      @error_messages = []
      [@user, @user_address, @sub_address].each do |object|
        unless object.nil?
          object.errors.full_messages.each do |message|
            @error_messages << message
          end
        end
      end
      unless @shipping_addresses.nil?
        @shipping_addresses.each_with_index do |shipping_address, index|
            shipping_address.errors.full_messages.each do |message|
              @error_messages << "配送先#{index + 1}：#{message}"
            end
        end
      end
    else
      # 初期表示
      @user =  User.find(params[:id])
      @user_address = @user.user_address
      @sub_address = @user.sub_address
      @shipping_addresses = @user.shipping_addresses
    end
    user_email_instance_for_edit
  end

  # 更新
  #
  def update
    @user = User.find(params[:id])
    @user_address = @user.user_address
    @shipping_addresses = []
    @sub_address = UserAddress.hash_to_sub_address(params[:sub_address])
    user_email_instance_for_update
    begin
      ActiveRecord::Base.transaction do
        # パラメータの更新
        @user.original_member_div = @user.member_div
        @user.merge_only_need(params[:user])
        @user_address.attributes = params[:user_address]
        @user.mirror_from_address(@user_address)
        update_parame_user_email_instance_for_update
        update_param_sub_andshipping_address_instance_for_update
        # DB登録
        @user.save!
        @user_address.save!
        save_user_email_for_update
        save_sub_and_shipping_address_for_update
      end
    rescue Exception
      # バリデーション
      flash_user_for_update
      return redirect_to :action => :edit, :page => params[:page], :id => params[:id]
    end
    return redirect_to :action => 'index', :page => params[:page]
  end

  # 新規
  def new
    # バリデーションエラーで戻った場合
    if flash[:user]
      @user = flash[:user]
      @user_address = flash[:user_address]
      @error_messages = []
      [@user, @user_address].each do |object|
        unless object.nil?
          object.errors.full_messages.each do |message|
            @error_messages << message
          end
        end
      end
    else
      # 初期表示
      @user =  User.new(:sex_key => :male, :dm_send_div_key => :prefer, :member_div_key=>:membership,
        :pc_mail_magazine_send_div_key => :prefer_html, :mobile_mail_magazine_send_div_key => :prefer_text)
      @user_address = UserAddress.new(:locale => I18n.locale, :country_code => ShopSetting.get_setting(:default_address_country, I18n.locale))
    end
    user_email_instance_for_new
  end

  # 新規登録
  def create
    # インスタンス変数にセットします。
    @user = User.new(params[:user])
    @user_address = UserAddress.new(params[:user_address])
    @user.user_address = @user_address
    @user.mirror_from_address(@user_address)
    @user.regist_div_key = :pc
    user_email_instance_for_create
    # バリデーションチェックを行います。
    @user_address.valid?
    @user.valid?
    valid_user_email_for_create
    # エラーカウントを行います。
    if invalid_error_count_for_create > 0
      flash_user_for_create
      return redirect_to :action=> :new
    end    
    # 登録処理
    begin
      User.transaction do
        @user.save!
        @user.user_address.address_div_key = :user
        @user.user_address.locale = I18n.locale.to_s
        @user.user_address.save!
        save_user_email_for_create
      end
    rescue
      flash_user_for_create
      return redirect_to :action=> :new
    end
    send_signup_mail
    return redirect_to :action => :index
  end

  # テストユーザー編集
  #
  desc :auth_as => :update
  def edit_test_user
    @page_no = params[:page]
    # バリデーションエラーで戻った場合
    if flash[:change_user_point]
      @change_point = flash[:change_user_point]
      @error_messages = @change_point.errors.full_messages
    # 初期表示
    else
      @user =  User.find(params[:id])
    end
  end

  # テストユーザー更新
  #
  desc :auth_as => :update
  def update_test_user
    # 有効期限を設定
    point_setting = PointSetting.find(:first)
    point = params[:user_point]
    rank = params[:user_rank]
    unless point.nil? || point[:change_point].to_i == 0
      if (UserPoint.total_point_for_user(params[:id]).to_i + point[:change_point].to_i) < 0
        return redirect_to :action => 'edit_test_user', :page => params[:page], :id => params[:id]
      end
      if point[:change_point].to_i > 0
        user_point = UserPoint.new(:given_point => point[:change_point].to_i)
      else
        user_point = UserPoint.new(:used_point => point[:change_point].to_i.abs)
      end
      user_point.user_id = params[:id]
      user_point.term_date = point_setting.create_term_date(Date.today)
      unless user_point.valid?
        flash[:change_user_point] = point[:change_point]
        return redirect_to :action => 'edit_test_user', :page => params[:page], :id => params[:id]
      end
      # 有効期限がポイント最終付与時の場合全ポイントの有効期限を付け替え
      unless user_point.term_date.nil?
        UserPoint.term_date_update_for_last_point(params[:id], user_point.term_date)
      end
      user_point.save!
    end

    if rank
      user = User.find(params[:id])
      user.user_rank_id = rank[:name]
      user.save!
    end
    return redirect_to :action => 'show', :page => params[:page], :id => params[:id]
  end

  desc :category_name => :destroy, :display_name => '会員削除'
  def delete
    if params[:destroy_all_test]
      user_ids = User.find(:all, :conditions => ["test_flg = ?", true]).map(&:id)
    else
      user_ids =[params[:id]]
    end
    user_ids.each do |user_id|
      User.find(user_id).destroy
    end
    redirect_to :action => 'index', :page => params[:page]
  end

  desc :category_name => :index, :display_name => "会員購入履歴"
  def order_history
    @user = User.find(params[:id])
    @orders = Order.find(:all,
      :include => :order_shippings,
      :conditions => ["orders.user_id = ? and order_shippings.id is not null", @user.id],
      :order => "orders.created_at DESC")
  end

  desc :auth_as => :mail_detail
  # メール一覧
  def mail_history
    @sent_mails = MailDelivery.find(:all, :conditions => ["user_id = ?", params[:id]], :order => "deliveried_at DESC")
  end

  desc :category_name => :index, :display_name => "配信済みメール照会"
  def mail_detail
    @mail_delivery = MailDelivery.find_by_id(params[:mail_id])
  end

  protected
    # 登録情報編集時のバリデーションエラーメッセージをセットする
    def user_email_instance_for_edit
      if flash[:user_email]
        @user_email =flash[:user_email]
        @user_email.errors.full_messages.each do |message|
          @error_messages << message
        end
      else
        @user_email = @user.user_email
      end
    end
    # Emailをインスタンス変数に設定
    def user_email_instance_for_update
      @user_email = @user.user_email
    end
    # Emailのパラメータを更新
    def update_parame_user_email_instance_for_update
      if !params[:user_email].nil? && ! params[:user_email][:email].blank?
        if @user_email # 更新
          @user_email.attributes = params[:user_email]
        else # 新規作成
          @user_email = UserEmail.new(params[:user_email])
          @user_email.pc_mobile_div_key = :pc
          @user.user_email = @user_email # DBが更新されます。
          @user_email.user = @user
        end
      else
        @user_email.destroy if @user_email # 削除
        @user_email = nil
        @user.user_email = nil
      end
    end
    # 海外配送用住所と複数配送先のインスタンスを設定
    def update_param_sub_andshipping_address_instance_for_update
      # 海外配送用住所
      @sub_address.set_sub_address_from_address(@user_address) if @sub_address
      # 複数配送先
      if shipping_address_hash = params[:shipping_address]
        shipping_address_hash.each_pair do |shipping_address_id, shipping_address_data|
          shipping_address = UserAddress.find(shipping_address_id)
          shipping_address.attributes = shipping_address_data
          @shipping_addresses << shipping_address
        end
      end
    end
    # Email情報をDBに登録
    def save_user_email_for_update
      @user_email.save! if @user_email
    end
    # 海外配送用住所と複数配送先をDBに登録
    def save_sub_and_shipping_address_for_update
      # 海外配送用住所
      if @user.sub_address
        # 更新
        if @sub_address
          @user.sub_address.attributes = params[:sub_address]
          @user.sub_address.set_sub_address_from_address(@user_address)
          @user.sub_address.save!
        # 削除
        else
          @user.sub_address.destroy
        end
      else
        # 新規作成
        if @sub_address
          @sub_address.user_id = @user.id
          @sub_address.save!
        end
      end
      # 複数配送先更新
      @shipping_addresses.each{|shipping_address| shipping_address.save!}
    end
    # バリデーション結果をセット
    def flash_user_for_update
      @user.valid?; @user_address.valid?
      @user_email.valid? if @user_email
      @sub_address.valid? if @sub_address
      @shipping_addresses.each{|shipping_address| shipping_address.valid? }
      flash[:user] = @user
      flash[:user_email] = @user_email
      flash[:user_address] = @user_address
      flash[:sub_address] = @sub_address
      flash[:shipping_address] = @shipping_addresses
    end

    # 新規登録時のバリデーションエラーメッセージをセットする
    def user_email_instance_for_new
      if flash[:user_email]
        @user_email = flash[:user_email]
        @user_email.errors.full_messages.each do |message|
          @error_messages << message
        end
      else
        @user_email = UserEmail.new
      end
    end
    # Emailをインスタンス変数に設定
    def user_email_instance_for_create
      if !params[:user_email].nil? && !params[:user_email][:email].blank?
        @user_email = UserEmail.new(params[:user_email])
        @user_email.pc_mobile_div_key = :pc
        @user_email.mail_address_pc_mobile_div = UserEmail.check_by_emall(@user_email.email)
        @user.user_email = @user_email
        @user_email.user = @user
      end
    end
    # Emailの入力チェックの実行
    def valid_user_email_for_create
      @user_email.valid? if @user_email
    end
    # 入力チェックエラー件数の取得
    def invalid_error_count_for_create
      @user.errors.length + @user_address.errors.length + invalid_error_count_with_user_email_for_create
    end
    # Emailの入力チェックエラー件数の取得
    def invalid_error_count_with_user_email_for_create
      @user_email.nil? ? 0 : @user_email.errors.length
    end
    # flashにユーザ情報を設定
    def flash_user_for_create
      flash[:user] = @user
      flash[:user_address] = @user_address
      flash[:user_email] = @user_email
    end
    # Email情報をDBに登録
    def save_user_email_for_create
      @user_email.save! if @user_email
    end
    # 会員登録完了メール送信（WEB会員の場合に送信します）
    def send_signup_mail
      if @user.member_div_key == :membership
        AdminMailer.send_signup_mail(@user.id.to_s, @user.user_email.email)
      end
    end
end
