# -*- coding: utf-8 -*-
# Methods added to this helper will be available to all templates in the application.
require "date"

module ApplicationHelper
  include UserHelper
  include EcRider::I18nSupport
  include TopHelper

  # エラーメッセージなどのi18nサポート
  include EcRider::I18nSupport

  # JSサポート付きの入力フィールドなどのモジュール
  include EcRider::JsValidationHelper

  # プラグインで拡張されるヘルパー
  include ApplicationHelperExtention

  #
  # Goldberg::Userの情報取得
  #
  def current_user
    # コントローラのcurrent_userはpublicじゃないので、__send__で取得します。
    controller.__send__(:current_user)
  end

  # このメソッドは余計な引数を必要とするので、非推奨です。
  # cycle('even', 'odd') を使用してください。
  def odd_or_even(cnt)
    # logger.warn("'odd_or_even' helper method is DEPRECATED! Use even_odd")
    (cnt % 2 == 0) ? 'even' : 'odd' # 偶数がeven、奇数がoddです
  end

  #
  # 日付フォーマット
  #
  def format_date(date)
    date ? date.strftime('%Y年%m月%d日') : nil
  end
  def format_datetime(date)
    date ? date.strftime('%Y/%m/%d %H:%M') : nil
  end

  #
  # バリデーションエラーメッセージ
  # error_messages_forの代替に使用
  #
  def template_error_messages_for(*params)
    #    options = params.last.is_a?(Hash) ? params.pop.symbolize_keys : {}
    objects = params.collect {|object_name| instance_variable_get("@#{object_name}") }.compact
    count   = objects.inject(0) {|sum, object| sum + object.errors.count }
    unless count.zero?
      error_messages = objects.map {|object| object.errors.full_messages.map {|msg| content_tag(:li, msg) } }
      render :partial => "common/error_messages_for", :locals => {:messages => error_messages}
    end
  end

  #
  # バリデーションエラーメッセージ
  # キーが「locale」、値が「モデルのオブジェクト」型のハッシュ用
  #
  def template_error_messages_for_locales(hash_name)
    objects = instance_variable_get("@#{hash_name}")
    return if objects.nil? || objects.empty?

    error_messages = []
    objects.each do |locale, object|
      error_messages += object.errors.full_messages.map {|msg| content_tag(:li, "#{locale_for_display(locale)}: #{msg}")}
    end
    render :partial => "common/error_messages_for", :locals => {:messages => error_messages}
  end

  #
  # = カレンダー付のテキストフィールド =
  # type: ymd => 年月日、 md => 月日
  #
  def calendar_text_field(object, method, html_options = {}, type = 'ymd')
    # IEの場合、カレンダーが他のカレンダー付きテキストフィールドの裏側に隠れてしまうため、
    # 先に描画したものの方が手前に表示されるようにz-indexを調整する（１画面９個までに対応）
    @calendar_count ||= 10
    @calendar_count = [@calendar_count-1, 1].max
    if obj = instance_variable_get("@#{object.to_s}")
      val = obj.send(method.to_s)
      if val && (val.respond_to?(:strftime))
        unless html_options.has_key?(:value)
          html_options[:value] = val.strftime('%Y/%m/%d')
        end
      end
    end
    form_id = object.to_s.sub(/\[\]$/,"") + "_" + method.to_s
    container_id = form_id + "_Container"
    cal_obj_name = form_id + "_Container_cal"

    icon_tag = image_tag("/ec_rider/images/calendar_icon.gif", {:width=>'15px', :height=>'15px', :border=>'0'})
    result = <<-EOS
      <div style='position:relative; display: inline; z-index: #{@calendar_count};'>
        #{text_field(object, method, html_options)}
        <span style='margin-left:4px;'>
          #{link_to(icon_tag, "javascript:void(0)", :id => "#{form_id}_cal")}
        </span>
        <ifreme><div id='#{container_id}' style='display:none; position:absolute; left:10px; top:20px; z-index:100;'></div></ifreme>
        <script>
          YAHOO.util.Event.addListener('#{form_id}_cal', 'click', getInitDoc('#{form_id}', '#{container_id}', '#{type}'));
          YAHOO.util.Event.addListener('#{form_id}_cal', 'click', getHandleFocus('#{container_id}'), YAHOO.myCalendar.#{cal_obj_name}, true);
        </script>
      </div>
    EOS
    return result
  end

  #
  # ３桁ごとにカンマを入れる
  #
  def fix_num_format(number)
    number.to_s.reverse.gsub(/[0-9]{3}/){|s| s + ","}.gsub(/,$/ ,"").reverse
  end

  #
  # link_to_remoteのbutton版
  #
  def submit_to(name, id, url)
    button_to_function(name, "$('#{id}').action='#{url_for(url)}';$('#{id}').submit();")
  end

  #
  # 現在のページ情報を表示する
  #
  def paginate_info(collection)
    "全<strong>#{collection.total_entries}</strong>件中<strong>#{collection.offset + 1}</strong>～<strong>#{collection.offset + collection.length}</strong>件を表示しています。"
  end

  #
  # 表示件数のオプションを返します
  #
  def admin_search_disp_cnt_list
    [["20", 20], ["50", 50], ["100", 100]]
  end

  #
  # セレクトボックス用の都道府県名リストを返す
  #
  def prefecture_names_for_select
    [["▼都道府県を選択",""]] + Prefecture.prefecture_names(I18n.locale.to_s).each{|name| [name, name]}
  end

  #
  # 言語選択のオプションを返します
  #
  def locale_list
    I18n.supported_locales.map{|locale| [locale_for_display(locale), locale]}
  end

  #
  # 住所を表示します
  #   :include_country => true 海外の場合には国を表示する
  #   :include_zipcode => true 郵便番号を表示する
  #   :inline => true 一行で表示する
  #
  def print_address(model, options = {})
    result = ''
    if model.is_japanese_format?
      separator = options[:inline] ? ' ' : '<br/>'
      result << "〒#{h(model.zip_code)}#{separator}" if options[:include_zipcode]
      result << h(model.address_1)
      result << h(model.fulladdress)
    else
      separator = options[:inline] ? ', ' : '<br/>'
      result << h(model.fulladdress)
      result << separator
      result << h(model.address_1)
      if !model.fulladdress.blank? || !model.address_1.blank?
        result << ", #{model.country.name}" if options[:include_country]
      end
      result << "#{separator}zipcode:#{h(model.zip_code)}" if options[:include_zipcode] && !model.zip_code.blank?
    end
    return result
  end

  #
  # = 改行コード(\n)を<br/>に置換します。 =
  #
  def add_br_tag(str)
    return nil unless str
    str.split(/$/).map{|line| h(line)}.join("<br/>")
  end

  # hidden_fieldをまとめて出力します。
  def hidden_fields(form, *args)
    args.map{|name| form.hidden_field(name)}.join("\n")
  end

  # 非表示のtext_areaをまとめて出力します。
  def hidden_text_areas(form, *args)
    args.map{|name| form.text_area(name, :class => :invisible)}.join("\n")
  end

  # noticeを強調表示するJSを出力します。
  def notice_effect_js(div_id)
    js = <<-"EOS"
      <script>
      Event.observe(window, 'load', function(){
        var div = $('#{div_id}');
        if (div) {
          new Effect.Grow(div);
        }
      });
      </script>
    EOS
    return js
  end

  # noticeを表示するdivとjsを出力します。
  def notice_div_with_effect(options = nil)
    return nil unless flash[:notice]
    options = {
      :id => 'notice_msg_' << Time.now.to_i.to_s,
      :class => 'notice_msg'
    }.update(options || {})
    div_id = options[:id]
    result = content_tag(:div, h(flash[:notice]), options)
    result << notice_effect_js(div_id)
    flash[:notice] = nil
    return result
  end
  
  #
  # = 改行コード(\n)を<br/>に置換し、自動改行用の<wbr/>タグをはさみます。 =
  # 値にHTMLを含むものは使えません。
  # 
  def add_wbr_tag(str)
    return nil unless str
    #Symbolが渡されてもいいようにto_sします。
    str.to_s.split(/$/).map{|line| line.split('').map{|c| h(c)}.join('<wbr/>') }.join("<br/>")
  end
end
