module CampaignConditionsHelper

  # 指定された変数（value_name）の値にしたがって、
  # 指定された行（row_id）のキャンペーン条件演算子の選択肢を切り替えるjavascriptを生成する
  def script_for_change_condition_operator_field(value_name, row_id)
    result = "switch (#{value_name}) {"
    CampaignCondition.type_keys().each do |type_key|
      result << "  case '#{CampaignCondition.type_id_by_key(type_key)}':"
      result << "    change_operator_field('#{CampaignCondition.type_entry_by_key(type_key)[:operator]}', #{row_id});"
      result << "    break;"
    end
    result << "}"
  end

  # 指定された変数（value_name）の値にしたがって、
  # 指定された行（row_id）のキャンペーン条件値の入力欄を切り替えるjavascriptを生成する
  def script_for_change_condition_value_field(value_name, row_id)
    result = "switch (#{value_name}) {"
    CampaignCondition.type_keys().each do |type_key|
      result << "  case '#{CampaignCondition.type_id_by_key(type_key)}':"
      result << "    change_value_field('#{CampaignCondition.type_entry_by_key(type_key)[:input]}', #{row_id});"
      result << "    break;"
    end
    result << "}"
  end

  # 指定された変数（value_name）の値にしたがって、
  # 指定された場所（target_field_id）にキャンペーン効果のsuffixを表示するjavascriptを生成する
  def script_for_display_condition_suffix(target_field_id, value_name)
    result = "switch (#{value_name}) {"
    CampaignCondition.type_keys().each do |type_key|
      result << "  case '#{CampaignCondition.type_id_by_key(type_key)}':"
      result << "    $('#{target_field_id}').innerHTML = '#{CampaignCondition.type_entry_by_key(type_key)[:suffix]}';"
      result << "    break;"
    end
    result << "}"
  end

  # javascriptのメソッド（change_operator_field）を生成します
  def change_operator_field_function
    result = "function change_operator_field(type, num) {"
    CampaignCondition.type_entries.map{|v| v[:operator]}.uniq.each do |operator|
      result << "  display_operator_field('#{operator.to_s}', num, (type == '#{operator.to_s}'));"
    end
    result << "}"
  end

  # javascriptのメソッド（change_value_field）を生成します
  def change_value_field_function
    result = "function change_value_field(type, num) {"
    CampaignCondition.type_entries.map{|v| v[:input]}.uniq.each do |input|
      result << "  display_value_field('#{input.to_s}', num, (type == '#{input.to_s}'));"
    end
    result << "}"
  end

  # 年の選択肢リストを生成する
  def year_list(this_year = Date.today.year)
    list = (this_year-99..this_year).map{|v|v.to_s}.reverse
    ['today']+list
  end

  # 月の選択肢リストを生成する
  def month_list
    list = (1..12).map{|v|v.to_s}
    ['today']+list
  end

  # 日の選択肢リストを生成する
  def date_list
    list = (1..31).map{|v|v.to_s}
    ['today']+list
  end
end
