# -*- coding: utf-8 -*-
module CmsPagesHelper
  include EcRider::PluginModule::Extendable
  include CmsHelper

  # 編集リンクを返します。
  # PJ編集対象の場合はeditアクション、PJ編集対象外の場合はnewアクションへのリンクを返します。
  def print_edit_link(cms_object)
    if cms_object.current_project?(@cms_project.id)
      if cms_object.cms_project.edittable?
        link_to('編集', :action => 'edit', :id => cms_object.id)
      end
    else
      if cms_object.cms_project.status_key == :opened || cms_object.cms_project.status_key == :closed
        link_to('編集', :action => 'new', :source_id => cms_object.id)
      end
    end
  end

  def print_destroy_link(cms_page)
    if cms_page.current_project?(@cms_project.id)
      if cms_page.cms_project.edittable?
        link_to('削除', {:action => "destroy", :id => cms_page.id}, {:confirm => '削除してよろしいですか？', :method => :delete})
      end
    else
      if cms_page.cms_project.status_key == :opened && !@delete_ids.include?(cms_page.id)
        link_to('公開時に削除', {:action => 'create', :source_id => cms_page.id}, {:method => :post, :confirm => 'プロジェクト公開期間はこのページは削除されます。よろしいですか？'})
      elsif cms_page.cms_project.status_key == :opened && @delete_ids.include?(cms_page.id)
        link_to('削除取消', {:action => 'undo_destroy', :id => cms_page.id}, :method => :delete)
      end
    end
  end

  # プレビューリンクを返します。
  # 全ての言語分表示し、デプロイ可能な場合は操作可能にします。
  def print_preview_link(cms_page, cms_project_id)
    if cms_page.cms_shop_page?
      result = '<div>'
      I18n.supported_locales.each do |locale|
        result << link_to_if(cms_page.previewable?(locale.to_s), "プレビュー(#{locale})", 
                  {:controller => 'cms_pages', :action => 'preview', :id => cms_page.id, :cms_project_id => cms_project_id, :locale => locale.to_s},
                  {:target => 'blank'})
        result << "<br/>"
      end if cms_page.terminal_key == :pc
      CmsTemplateContent.carrier_options.each do |name, locale|
        result << link_to_if(cms_page.previewable?(locale.to_s), "プレビュー(#{name})",
                  {:controller => 'cms_pages', :action => 'preview', :id => cms_page.id, :cms_project_id => cms_project_id, :locale => locale.to_s},
                  {:target => 'blank'})
        result << "<br/>"
      end if cms_page.terminal_key == :mobile
      result << '</div>'
      result
    end
  end

  # 新規登録リンクを作成します
  def print_new_link
    link_to('PCページ新規作成', :controller => 'cms_pages', :action => 'setup')
  end
end
