module CmsTemplatesHelper
  include EcRider::PluginModule::Extendable
  
  # 編集リンクを返します。
  # PJ編集対象の場合はeditアクション、PJ編集対象外の場合はnewアクションへのリンクを返します。
  def print_edit_link(cms_template)
    if cms_template.current_project?(@cms_project.id)
      if cms_template.cms_project.edittable?
        link_to('編集', :action => 'edit', :id => cms_template.id)
      end
    else
      if cms_template.cms_project.status_key == :opened || cms_template.cms_project.status_key == :closed
        link_to('編集', :action => 'new', :source_id => cms_template.id)
      end
    end
  end

  # 削除リンク作成
  def print_destroy_link(cms_template, default_template_codes)
    if cms_template.current_project?(@cms_project.id)
      if cms_template.cms_project.edittable? && cms_template.cms_project.id != CmsProject::BASE_PROJECT_ID
        link_to('削除', cms_template, :confirm => "このテンプレートを使用しているページも削除されます。\n削除してよろしいですか？", :method => :delete)
      end
    else
      unless default_template_codes.position_id(cms_template.template_code, cms_template.terminal_key)
        if cms_template.cms_project.status_key == :opened && !@delete_ids.include?(cms_template.id)
          link_to('公開時に削除', {:action => 'create', :source_id => cms_template.id}, {:method => :post, :confirm => 'プロジェクト公開期間はこのテンプレートは削除されます。よろしいですか？'})
        elsif cms_template.cms_project.status_key == :opened && @delete_ids.include?(cms_template.id)
          link_to('削除取消', {:action => 'destroy', :source_id => cms_template.id}, :method => :delete)
        end
      end
    end
  end

  # 新規登録リンク作成
  def print_new_link
    link_to('PCテンプレート新規作成', :controller => 'cms_templates', :action => 'new')
  end

  # デフォルトテンプレート表示位置を表示します
  def print_default_template_position(default_template_codes, template_code, terminal=:pc, prefix='')
    name = default_template_codes.position_name(template_code, terminal)
    return '' unless name
    "#{prefix}#{name}"
  end

  # デフォルトテンプレート表示位置選択セレクトボックス
  def select_default_template_position(default_template_codes, template_code, terminal=:pc, new_position_id=nil)
    position_id = default_template_codes.position_id(template_code, terminal)
    new_position_id ||= position_id
    result = hidden_field(:cms_default_template, :current_position, :value => position_id)

    if default_template_codes.editable?(template_code, terminal)
      options = (terminal == :pc) ? CmsTemplate.default_template_position_options : CmsTemplate.default_template_position_mobile_options
      result << select(:cms_default_template, :new_position, options, :include_blank => true, :selected => new_position_id)
    else
      result << default_template_codes.position_name(template_code, terminal)
      result << hidden_field(:cms_default_template, :new_position, :value => new_position_id)
    end
    result
  end
end
