# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module IncludeShippingFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:同送先一覧を出力します。]
        [引数:なし]
        [使用例:
           {{{
              <ec:each_included_shippings>
                <%= ec.included_shipping_address_radio -%>
                <%= ec.included_shipping_ordered_at -%>
                <%= ec.included_shipping_name_last -%>  <%= ec.included_shipping_name_first -%>
                <%= ec.included_shipping_zipcode -%>
                <%= ec.included_shipping_address -%>
                <%= ec.included_shipping_telnum -%>
              </ec:end>
           }}}
        ]
      EOS
      def each_included_shippings(*args, &block)
        @included_shipping_addresses = @template.instance_variable_get(:@included_shipping_addresses)
        _each_with_index(:@included_shipping_addresses, :@included_shipping_address, :@included_shipping_address_index) do
          @template.concat(@template.capture(&block))
        end
        @template.concat(included_shipping_form_include_order_id)
      end

      desc <<-EOS
        [概要:同送先の住所を選択するラジオボタンを出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:同送先の住所を選択するラジオボタン]
        [使用例:{{{<ec:iincluded_shipping_address_radio/>}}}]
      EOS
      def included_shipping_address_radio(options = {})
        raise UsageError, "ec:included_shipping_order_dateはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        options = {:class => 'include_address_radio_buttons'}.update(options || {})
        js_func = @template.javascript_tag(<<-"EOS")
          (function(){
            Event.observe("order_form_send_address_index_#{@included_shipping_address.order_id}", 'click', function(event){
              $('order_form_include_order_id').value = #{@included_shipping_address.order_id}

              // 同送指定した場合、配送オプションを指定できない
              new Effect.Fade('shipping_option_area');
              $('gift_check').checked  = false;
              $('send_invoice_separately_check').checked  = false;
              $('write_sender_check').checked  = false;
            });
          })();
        EOS
        @order_form.radio_button(:send_address_index, @included_shipping_address.order_id, options) << js_func
      end

      private
      def included_shipping_form_include_order_id
        raise UsageError, "ec:each_included_shippingsはec:order_formの中で使用してください" unless @order_form
        @order_form.hidden_field(:include_order_id)
      end

    end
  end
end
