# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module InquiryFormTag
      extend MethodDescription

      desc <<-EOS
        [概要：お問い合わせ用のフォームを出力します。]
      EOS
      def inquiry_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@inquiry)
        @template.form_for(:inquiry, :url => {:controller => 'inquiry', :action => :confirm}, :html => {:multipart => true}) do |f|
          @inquiry_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @inquiry_form = nil
          end
        end
      end

      desc <<-EOS
        [概要：お問い合わせ時のエラーメッセージがあればそれを表示します。お問い合わせ用のフォーム内で使用してください。]
      EOS
      def inquiry_form_error_messages
        @template.template_error_messages_for(:inquiry)
      end

      desc <<-EOS
        [概要：メールアドレスを入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_email(options = {})
        raise UsageError, "ec:inquiry_form_emailはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:maxLength => 50,:size => 35,:class=>:default, :style => "ime-mode: inactive"}.update(options || {})
        validate_options = { :empty => true, :realtime => true, :update => 'email_msg', :error => I18n.t(:"activerecord.attributes.inquiry.email")}
        @inquiry_form.text_field_valid(:email, options, validate_options)
      end

      desc <<-EOS
        [概要：お名前(姓)を入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_name_last(options = {})
        raise UsageError, "ec:inquiry_form_name_lastはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validate_options = { :empty => true, :realtime => true, :update => 'kanji_name_msg', :error => I18n.t(:"activerecord.attributes.inquiry.name_last")}
        @inquiry_form.text_field_valid(:name_last, options, validate_options)
      end

      desc <<-EOS
        [概要：お名前(名)を入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_name_first(options = {})
        raise UsageError, "ec:inquiry_form_name_firstはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validate_options = { :empty => true, :realtime => true, :update => 'kanji_name_msg', :error => I18n.t(:"activerecord.attributes.inquiry.name_first")}
        @inquiry_form.text_field_valid(:name_first, options, validate_options)
      end

      desc <<-EOS
        [概要：お名前(カナ姓)を入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_kana_name_last(options = {})
        raise UsageError, "ec:inquiry_form_kana_name_lastはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validate_options = { :empty => true, :realtime => true, :update => 'kana_name_msg', :error => "フリガナ（性）"}
        @inquiry_form.text_field_valid(:kana_name_last, options, validate_options)
      end

      desc <<-EOS
        [概要：お名前(カナ名)を入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_kana_name_first(options = {})
        raise UsageError, "ec:inquiry_form_kana_name_firstはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validate_options = { :empty => true, :realtime => true, :update => 'kana_name_msg', :error => "フリガナ（名）"}
        @inquiry_form.text_field_valid(:kana_name_first, options, validate_options)
      end

      desc <<-EOS
        [概要：電話番号を入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_tel_num(options = {})
        raise UsageError, "ec:inquiry_form_tel_numはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:maxLength => 20,:size => 20,:class=>:default,:style => "ime-mode: disabled"}.update(options || {})
        validate_options = { :empty => true, :onebyte => true, :realtime => true, :update => 'tel_num_msg', :error => I18n.t(:"activerecord.attributes.inquiry.tel_num")}
        @inquiry_form.text_field_valid(:tel_num, options, validate_options)
      end

      desc <<-EOS
        [概要：お問い合わせ内容を入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_inquiry(options = {})
        raise UsageError, "ec:inquiry_form_inquiryはec:inquiry_formの中で使用してください" unless @inquiry_form
        options = {:size => "35x15", :style => "ime-mode: active ; width:350px; height:200px;"}.update(options || {})
        @inquiry_form.text_area(:inquiry, options)
      end

      desc <<-EOS
        [概要：お問い合わせ種別のセレクトタグを出力します。お問い合わせフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_inquiry_type_select(options = {})
        raise UsageError, "ec:inquiry_form_inquiryはec:inquiry_formの中で使用してください" unless @inquiry_form
        @inquiry_form.select(:inquiry_div, Inquiry.inquiry_div_name_by_id(@inquiry_type), options)
      end

      desc <<-EOS
        [概要：商品に関するお問い合わせの場合、このタグで囲んだ部分を表示します。]
        [引数 (オプション)：なし]
      EOS
      def if_product_inquiry(&block)
        raise UsageError, "ここでは使用できません。" unless @inquiry_form
        inquiry = @template.instance_variable_get(:@inquiry)
        if inquiry.inquiry_div_key == :product && !@template.instance_variable_get(:@inquiry_product).nil?
          begin
            @template.concat(@template.capture(&block))
          ensure
            ""
          end
        end
      end

      desc <<-EOS
        [概要：商品に関するお問い合わせでない場合、このタグで囲んだ部分を表示します。]
        [引数 (オプション)：なし]
      EOS
      def unless_product_inquiry(&block)
        raise UsageError, "ここでは使用できません。" unless @inquiry_form
        inquiry = @template.instance_variable_get(:@inquiry)
        if inquiry.inquiry_div_key != :product
          begin
            @template.concat(@template.capture(&block))
          ensure
            ""
          end
        end
      end

      desc <<-EOS
        [概要：商品コードを表示するタグを出力します。お問い合わせ用のフォーム、またはお問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_product_code(options = {})

        raise UsageError, "ec:inquiry_form_product_codeはec:inquiry_formまたは、inquiry_confirmation_form、if_product_inquiryの中で使用してください" unless @inquiry_form
        product = @template.instance_variable_get(:@inquiry_product)
        unless product.nil?
          product.product_code
        end
      end

      desc <<-EOS
        [概要：商品名1を表示するタグを出力します。お問い合わせ用のフォーム、またはお問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_form_product_name(options = {})
        raise UsageError, "ec:inquiry_form_inquiryはec:inquiry_formまたは、inquiry_confirmation_form、if_product_inquiryの中で使用してください" unless @inquiry_form
        product = @template.instance_variable_get(:@inquiry_product)
        unless product.nil?
          product.product_name_1
        end
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面用のフォームを出力します。]
      EOS
      def inquiry_confirmation_form(&block)
        @inquiry = @template.instance_variable_get(:@inquiry)
        raise UsageError, "ここでは使用できません。" unless @inquiry

        @template.form_for(:inquiry, :url => {:controller => 'inquiry', :action => :update_information}, :html => {:multipart => true}) do |f|
          @inquiry_form = f
          begin
            tmp_images = @template.instance_variable_get(:@tmp_images)
            @template.concat(@template.capture(&block))
            @template.concat(@template.hidden_field_tag("tmp_image_dir", tmp_images[:tmp_image_dir]))
          ensure
            @inquiry_form = nil
            @inquiry = nil
          end
        end
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力されたメールアドレスを出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_email(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_emailはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        ERB::Util.h(@inquiry.email)
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力された名前を出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_fullname(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_fullnameはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        ERB::Util.h(@inquiry.fullname)
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力された名前(カナ姓)を出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_kana_name_last(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_kana_name_lastはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        ERB::Util.h(@inquiry.kana_name_last)
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力された名前(カナ名)を出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_kana_name_first(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_kana_name_firstはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        ERB::Util.h(@inquiry.kana_name_first)
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力された電話番号を出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_tel_num(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_tel_numはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        ERB::Util.h(@inquiry.tel_num)
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力されたお問い合わせ内容を出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_inquiry(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_inquiryはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        @template.add_br_tag(@inquiry.inquiry)
      end

      desc <<-EOS
        [概要：お問い合わせ確認画面で入力されたお問い合わせ種類を出力します。お問い合わせ確認画面用のフォーム内でのみ使用できます。]
      EOS
      def inquiry_confirmation_form_inquiry_type(options = {})
        raise UsageError, "ec:inquiry_confirmation_form_inquiry_typeはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        @inquiry.inquiry_div_name
      end

      desc <<-EOS
        [概要：商品詳細から遷移した場合のみ商品詳細画面に遷移するボタンを出力します。]
      EOS
      def button_to_product_detail(*args, &block)
        product = @template.instance_variable_get(:@inquiry_product)
        if product.nil? || product == ""
          ""
        else
          options = args.shift || {}
          detail_url = product_info(product.product_code, 'detail_url')
          button = @template.button_to_url(@template.capture(&block), detail_url, options)
          @template.concat(button)
        end
      end

      desc <<-EOS
        [概要：画像URLを入力するinputタグを出力します。お問い合わせ用のフォーム内でのみ使用できます。]
        [引数:display_number 画像URL入力欄の番号を指定します。]
        [使用例:
          {{{
          <ec:inquiry_form_image_list display_number=1 />
          }}}
        ]
      EOS
      def inquiry_form_image(*args)
        raise UsageError, "ec:inquiry_form_name_lastはec:inquiry_formの中で使用してください" unless @inquiry_form
        display_number, options = extract(args, :display_number => "1")
        options = {:maxLength => 200,:size => 80, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        if display_number.to_i >= 1 && display_number.to_i <= 5
          image_name = "image_url_#{display_number}".to_sym
          @inquiry_form.file_field(image_name, options)
        end
      end

      desc <<-EOS
        [概要:問い合わせ確認 問い合わせフォームで選択した画像を表示します。]
        [引数:display_number 表示画像の番号を指定します。]
        [戻り値:問い合わせフォームで選択した画像。]
        [使用例:
          {{{
          <ec:inquiry_confirmation_form_image display_number=1 />
          }}}
        ]
      EOS
      def inquiry_confirmation_form_image(*args)
        raise UsageError, "ec:inquiry_confirmation_form_inquiryはec:inquiry_confirmation_formの中で使用してください" unless @inquiry_form
        display_number, options = extract(args, :display_number => 1)
        image_name = "image_url_#{display_number}".to_sym
        tmp_images = @template.instance_variable_get(:@tmp_images)
        if tmp_images && tmp_images[:tmp_image_dir] && @inquiry[image_name]
          @template.image_tag("/tmp_inquiry_images/#{tmp_images[:tmp_image_dir]}/#{@inquiry[image_name]}", options)
        end
      end
    end
  end
end
