# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module ResignationFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:退会用のフォームを出力します。]
      EOS
      def resignation_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@resigned_reason)
        @template.form_for(:resigned_reason, :url => {:controller => 'resign', :action => :confirm}) do |f|
          @resigned_reason = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @resigned_reason = nil
          end
        end
      end

      desc <<-EOS
        [概要:退会時のエラーメッセージがあればそれを表示します。退会用のフォーム内で使用してください。]
      EOS
      def resignation_form_error_messages
        @template.template_error_messages_for(:resigned_reason)
      end

      desc <<-EOS
        [概要:退会理由を選択する欄を出力します。退会用のフォーム内で使用してください。]
      EOS
      def resignation_form_reason_choice(&block)
        raise UsageError, "ec:resignation_form_resign_reasonはec:resignation_formの中で使用してください" unless @resigned_reason
        @resigned_reason.radio_button_group(:ms_resigned_reason_id) do |b|
          b.each do
            @resigned_reason_choice = b
            begin
              @template.concat(@template.capture(&block))
            ensure
            @resigned_reason_choice = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:退会理由のinputラジオボタンタグを出力します。退会用のフォーム内で使用してください。]
      EOS
      def resignation_form_resign_reason_radio(options = {})
        raise UsageError, "ec:payment_form_gender_radioはec:resignation_form_reason_choiceの中で使用してください" unless @resigned_reason_choice
        options = {}.update(options || {})
        @resigned_reason_choice.radio_button(options)
      end

      desc <<-EOS
        [概要:退会理由のラジオボタンに表示するラベルを出力します。退会用のフォーム内で使用してください。]
      EOS
      def resignation_form_resign_reason_label(options = {})
        raise UsageError, "ec:payment_form_card_kind_radio_labelはec:resignation_form_reason_choiceの中で使用してください" unless @resigned_reason_choice
        @resigned_reason_choice.label
      end

      desc <<-EOS
        [概要:退会理由自由記述のtextareaタグを出力します。退会用のフォーム内で使用してください。]
      EOS
      def resignation_form_reason_free_text(options = {})
        raise UsageError, "ec:resignation_form_reason_free_textはec:resignation_formの中で使用してください" unless @resigned_reason
        options = {:size => "50x10", :style => "ime-mode: active"}.update(options || {})
        @resigned_reason.text_area(:reason, options)
      end

      desc <<-EOS
        [概要:退会確認用のフォームを出力します。]
      EOS
      def resignation_confirmation_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@resigned_reason)
        @template.form_for(:resigned_reason, @template.instance_variable_get(:@resigned_reason), :url => {:controller => 'resign', :action => :update_information}) do |f|
          @resigned_reason = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @resigned_reason = nil
          end
        end
      end

      desc <<-EOS
        [概要:退会理由の選択値を出力します。退会確認用のフォーム内で使用してください。]
      EOS
      def resignation_confirmation_form_reason_choice
        raise UsageError, "ec:resignation_confirmation_form_reason_choiceはec:resignation_confirmation_formの中で使用してください" unless @resigned_reason
        @resigned_reason.object.ms_resigned_reason_id_name
      end

      desc <<-EOS
        [概要:退会理由の自由記述欄の入力内容を出力します。退会確認用のフォーム内で使用してください。]
      EOS
      def resignation_confirmation_form_reason_free_text
        raise UsageError, "ec:resignation_confirmation_form_reason_free_textはec:resignation_confirmation_formの中で使用してください" unless @resigned_reason
        @resigned_reason.object.reason
      end
    end
  end
end
