# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module SecretQuestionFormTag
      extend MethodDescription

      desc <<-EOS
        [概要：秘密の質問変更用のフォームを出力します。]
      EOS
      def secret_question_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user)
        @template.form_for(:user, :url => {:controller => 'account', :action => :confirm_security_answer}) do |f|
          @user = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user = nil
          end
        end
      end

      desc <<-EOS
        [概要：秘密の質問変更時のエラーメッセージがあればそれを表示します。秘密の質問変更用のフォーム内で使用してください。]
      EOS
      def secret_question_form_error_messages
        @template.template_error_messages_for(:user)
      end

      desc <<-EOS
        [概要：新しい秘密の質問を入力するinputタグを出力します。秘密の質問変更用のフォーム内でのみ使用できます。]
      EOS
      def secret_question_form_remember_question(options = {})
        raise UsageError, "ec:secret_question_form_remember_questionはec:secret_question_formの中で使用してください" unless @user
        options = {:maxLength => 20, :class => :default, :style => "ime-mode: active"}.update(options || {})
        @user.text_field(:remember_question, options)
      end

      desc <<-EOS
        [概要：新しい秘密の質問の回答を入力するinputタグを出力します。秘密の質問変更用のフォーム内でのみ使用できます。]
      EOS
      def secret_question_form_remember_answer(options = {})
        raise UsageError, "ec:secret_question_form_remember_answerはec:secret_question_formの中で使用してください" unless @user
        options = {:maxLength => 20, :class => :default, :style => "ime-mode: active"}.update(options || {})
        @user.text_field(:remember_answer, options)
      end

      desc <<-EOS
        [概要：秘密の質問変更確認用のフォームを出力します。]
      EOS
      def secret_question_confirmation_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user)
        @template.form_for(:user, :url => {:controller => 'account', :action => :update_security_answer}) do |f|
          @user_confirm = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user_confirm = nil
          end
        end
      end

      desc <<-EOS
        [概要：秘密の質問変更画面で入力された秘密の質問を出力します。秘密の質問変更確認用のフォーム内でのみ使用できます。]
      EOS
      def secret_question_confirmation_form_remember_question(options = {})
        raise UsageError, "ec:usecret_question_confirmation_form_remember_answerはec:secret_question_confirmation_formの中で使用してください" unless @user_confirm
        ERB::Util.h(@template.instance_variable_get(:@user).remember_question)
      end

      desc <<-EOS
        [概要：秘密の質問変更画面で入力された秘密の質問の回答を出力します。秘密の質問変更確認用のフォーム内でのみ使用できます。]
      EOS
      def secret_question_confirmation_form_remember_answer(options = {})
        raise UsageError, "ec:usecret_question_confirmation_form_remember_answerはec:secret_question_confirmation_formの中で使用してください" unless @user_confirm
        ERB::Util.h(@template.instance_variable_get(:@user).remember_answer)
      end
    end
  end
end
