# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module ShopTag
      extend MethodDescription

      SHOP_TAG_BIND_OPTION_METHODS = [
        :shop_name, :shop_zip_code, :shop_address, :shop_tel_num, :shop_mail
      ]

      desc <<-EOS
        [概要:店舗名称を表示します。]
        [引数:なし]
        [戻り値:店舗名称に登録した文字列。]
        [使用例:{{{<ec:shop_name/>}}}]
      EOS
      def shop_name(*args, &block)
        ShopSetting.get_setting(:shop_name, I18n.locale)
      end

      desc <<-EOS
        [概要:店舗郵便番号を表示します。]
        [引数:なし]
        [戻り値:店舗郵便番号に登録した文字列。]
        [使用例:{{{<ec:shop_zip_code/>}}}]
      EOS
      def shop_zip_code(*args, &block)
        ShopSetting.get_setting(:shop_zip_code, I18n.locale)
      end

      desc <<-EOS
        [概要:店舗住所を表示します。]
        [引数:なし]
        [戻り値:店舗住所に登録した文字列。]
        [使用例:{{{<ec:shop_address/>}}}]
      EOS
      def shop_address(*args, &block)
        ShopSetting.get_setting(:shop_address, I18n.locale)
      end

      desc <<-EOS
        [概要:店舗電話番号を表示します。]
        [引数:なし]
        [戻り値:店舗電話番号に登録した文字列。]
        [使用例:{{{<ec:shop_tel_num/>}}}]
      EOS
      def shop_tel_num(*args, &block)
        ShopSetting.get_setting(:shop_tel_num, I18n.locale)
      end

      desc <<-EOS
        [概要:店舗ロゴを表示します。]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:店舗設定画面で登録した店舗ロゴ画像。]
        [使用例:{{{<ec:shop_logo/>}}}]
      EOS
      def shop_logo(*args, &block)
        src = ShopSetting.get_setting(:shop_logo, I18n.locale)
        if src
          options = args.shift || {}
          shop_name = ShopSetting.get_setting(:shop_name, I18n.locale)
          options[:alt] ||= shop_name
          @template.image_tag(src, options)
        end
      end

      desc <<-EOS
        [概要:店舗ロゴをリンクつきで表示します。]
        [引数 href(必須):遷移先のURLを指定します。]
        [引数 (オプション):画像のHTMLオプションを指定します。]
        [戻り値:店舗設定画面で登録した店舗ロゴ画像（リンクつき）。]
        [使用例:{{{<ec:shop_logo_link/>}}}]
      EOS
      def shop_logo_link(*args, &block)
        src = ShopSetting.get_setting(:shop_logo, I18n.locale)
        if src
          options = args.shift || {}
          href = options.delete(:href)
          shop_name = ShopSetting.get_setting(:shop_name, I18n.locale)
          options[:alt] ||= shop_name
          link_to(@template.image_tag(src, options), {:href => href, :escape => false})
        end
      end

      desc <<-EOS
        [概要:店舗メールアドレスを表示します。]
        [引数:なし]
        [戻り値:店舗メールアドレスに登録した文字列。]
        [使用例:{{{<ec:shop_mail/>}}}]
      EOS
      def shop_mail(*args, &block)
        ShopSetting.get_setting(:shop_mail, I18n.locale)
      end

    end
  end
end
