# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module SignupFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:個人情報の収集についてのお知らせのテキストページを挿入します。店舗情報設定画面で指定したページを挿入します。]
        [引数:なし]
        [戻り値:個人情報の収集についてのお知らせテキストページの登録内容]
        [使用例:
           {{{
           <ec:include_policy_text/>
           }}}
        ]
      EOS
      def include_policy_text
        @template.render(:file => ShopSetting.policy_confirmation_page_path)
      end

      desc <<-EOS
        [概要:個人情報の収集についてのフォームを出力します。このフォームを使用することで会員種別毎に登録する場合、新規会員登録用フォームに遷移する際に入力された会員種別情報を引き継ぎます。]
      EOS
      def policy_form(&block)
        @template.form_tag(:controller => 'account', :action => 'signup') do
          begin
            @member_category_name = @template.instance_variable_get(:@member_category_name)
            @member_category_div = @template.instance_variable_get(:@member_category_div)
            @template.concat(@template.hidden_field_tag(:member_category_div, @member_category_div)) if @member_category_div
            @template.concat(@template.capture(&block))
          ensure
            @member_category_name = nil
          end
        end
      end

      desc <<-EOS
        [概要:期待しない会員種別で登録しようとした場合、エラーメッセージを表示します。]
        [使用例:
          {{{
          <ec:member_category_error_messages/>
          　　==> <div class="alert_msg"><ul><li>URLが正しくありません</li></ul></div>
          }}}
        ]
      EOS
      def member_category_error_messages
        if @template.instance_variable_get(:@member_category_error)
          content_tag(:div, content_tag(:ul, content_tag(:li, @template.instance_variable_get(:@member_category_error))), :class=>'alert_msg')
        end
      end

      desc <<-EOS
        [概要:新規会員登録用のフォームを出力します。]
      EOS
      def signup_form(&block)
        unless @template.instance_variable_get(:@user_address) && @template.instance_variable_get(:@user_profile) && @template.instance_variable_get(:@user_email)
          raise UsageError, "ここでは使用できません。"
        end
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => 'account', :action => :confirm_signup}, :html => {:id => "signup_form" }) do |f|
          @template.fields_for(:user_en_address, @template.instance_variable_get(:@user_en_address)) do |f1|
            @template.fields_for(:user_email, @template.instance_variable_get(:@user_email)) do |f2|
              @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f3|
                @user_address = f
                @user_en_address = f1
                @user_email = f2
                @user_profile = f3
                begin
                  @template.concat(@template.capture(&block))
                ensure
                  @user_address = nil
                  @user_en_address = nil
                  @user_email = nil
                  @user_profile = nil
                end
              end
            end
          end
        end
      end

      desc <<-EOS
        [概要:新規会員登録のエラーメッセージがあればそれを表示します。新規会員登録用のフォーム内で使用してください。]
      EOS
      def signup_form_error_messages
        @template.template_error_messages_for(:user_address, :user_en_address, :user_email, :user_profile)
      end

      # 店舗設定から取得する実装が必要です。
      def shippable_to_oversea?
        @user_profile.object.user_address ? @user_profile.object.shippable_to_oversea? : true
      end

      desc <<-EOS
        [概要:普通会員ではない場合(特別会員専用URLから遷移した場合など)に中身を実行します。存在しない会員種別の会員の場合、表示されません。]
      EOS
      def if_customized_member
        if !@user_profile.nil? || @member_category_name
          member_category_div = @user_profile.object.member_category_div unless @user_profile.nil?
          if !(member_category_div == User.member_category_div_id_by_key(:general)) || @member_category_name
            yield
          end
        end
      end

      desc <<-EOS
        [概要:普通会員の場合(特別会員専用URLから遷移した場合など)に中身を実行します。]
      EOS
      def unless_customized_member
        if !@user_profile.nil? || @member_category_name
          member_category_div = @user_profile.object.member_category_div unless @user_profile.nil?
          unless (member_category_div == User.member_category_div_id_by_key(:general)) || @member_category_name
            yield
          end
        end
      end

      desc <<-EOS
        [概要:普通会員ではない場合(特別会員専用URLから遷移した場合など)に項目管理で設定した会員種別名を表示します。普通会員の場合は何も表示しません。]
      EOS
      def member_category_name
        if @user_profile && @user_profile.object.member_category_div
          @template.concat(@user_profile.object.member_category_div_name)
          @user_profile.hidden_field(:member_category_div)
        elsif !@member_category_name.nil?
          @member_category_name
        end
      end

      desc <<-EOS
        [概要:海外配送を行っている場合は中身を実行します。]
      EOS
      def if_shippable_to_oversea
        shippable_to_oversea = shippable_to_oversea?
        if shippable_to_oversea
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:海外配送を行っていない場合は中身を実行します。]
      EOS
      def unless_shippable_to_oversea
        shippable_to_oversea = shippable_to_oversea?
        unless shippable_to_oversea
          yield if block_given?
        end
      end

      #
      #@user_address
      #

      desc <<-EOS
        [概要:住所入力欄を挿入します。]
      EOS
      def include_user_address_fields
        raise UsageError, "ec:signup_form_main_address_last_nameはec:signup_formの中で使用してください" unless @user_address
        @template.render(:partial => "/account/user_address")
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の姓を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_last_name　住所フォーム内で使用]
        [同様のタグ:account_form_last_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_last_name　非会員用住所フォーム内で使用]
        ]
      EOS
      def signup_form_main_address_last_name(options = {})
        raise UsageError, "ec:signup_form_main_address_last_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validation_option = {:empty => true, :realtime => true, :update => 'name_msg', :error => I18n.t(:"activerecord.attributes.user_address.name_last")}
        @user_address.text_field_valid(:name_last, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の名を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_first_name　住所フォーム内で使用]
        [同様のタグ:account_form_first_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_first_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_first_name(options = {})
        raise UsageError, "ec:signup_form_main_address_first_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validation_option = {:empty => true, :realtime => true, :update => 'name_msg', :error => I18n.t(:"activerecord.attributes.user_address.name_first")}
        @user_address.text_field_valid(:name_first, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の名前のバリデーションエラーメッセージを表示するspanタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_name_error_message　住所フォーム内で使用]
        [同様のタグ:account_form_name_error_message　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_name_error_message　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_name_error_message(options = {})
        raise UsageError, "ec:signup_form_main_address_name_error_messageはec:signup_formの中で使用してください" unless @user_address
        options = {:class => "error_msg"}.update(options || {})
        html_options = 'id="name_msg"'
        options.each_pair {|key, value|
          html_options << " #{key}=\"#{value}\""
        }
        "<span #{html_options}></span>"
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所のカナ姓を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_kana_last_name　住所フォーム内で使用]
        [同様のタグ:account_form_kana_last_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_kana_last_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_kana_last_name(options = {})
        raise UsageError, "ec:signup_form_main_address_kana_last_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validation_option = {:empty => true, :realtime => true, :update => 'kana_name_msg', :error => I18n.t(:"activerecord.attributes.user_address.kana_name_last")}
        @user_address.text_field_valid(:kana_name_last, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所のカナ名を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_kana_first_name　住所フォーム内で使用]
        [同様のタグ:account_form_kana_first_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_kana_first_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_kana_first_name(options = {})
        raise UsageError, "ec:signup_form_main_address_kana_first_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        validation_option = {:empty => true, :realtime => true, :update => 'kana_name_msg', :error => I18n.t(:"activerecord.attributes.user_address.kana_name_first")}
        @user_address.text_field_valid(:kana_name_first, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の名前のフリガナのバリデーションエラーメッセージを表示するspanタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_kana_name_error_message　住所フォーム内で使用]
        [同様のタグ:account_form_kana_name_error_message　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_kana_name_error_message　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_kana_name_error_message(options = {})
        raise UsageError, "ec:signup_form_main_address_kana_name_error_messageはec:signup_formの中で使用してください" unless @user_address
        options = {:class => "error_msg"}.update(options || {})
        html_options = 'id="kana_name_msg"'
        options.each_pair {|key, value|
          html_options << " #{key}=\"#{value}\""
        }
        "<span #{html_options}></span>"
      end

      # 別名を定義します。
      method_names = %w(last_name first_name name_error_message kana_last_name kana_first_name kana_name_error_message)

      src_form_name = 'signup_form_main_address'
      alias_form_names = %w(user_address account_form nonmember_address_form)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の国を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_country　住所フォーム内で使用]
      EOS
      def signup_form_main_address_country(options = {})
        raise UsageError, "ec:signup_form_main_address_countryはec:signup_formの中で使用してください" unless @user_address

        select_id = options[:id] || 'user_address_country_code'
        js_func = @template.javascript_tag(<<-"MSG")
            (function(){
              var country_select = $("#{select_id}");
              var last_value = "#{@user_address.object.country_code}";
              function update_area_and_inputs(class_name, enabled) {
                if (enabled){
                  $$("." + class_name).each(function(area){area.show();});
                  $$("." + class_name + " input").each(function(input){input.enable();});
                  $$("." + class_name + " select").each(function(select){select.enable();});
                }else{
                  $$("." + class_name).each(function(area){area.hide();});
                  $$("." + class_name + " input").each(function(input){input.disable();});
                  $$("." + class_name + " select").each(function(select){select.disable();});
                }
              }
              function update_addresses_by_country(){
                if (country_select.value == 'JP'){
                  update_area_and_inputs("main_address_country_pane_JP", true);
                  update_area_and_inputs("main_address_country_pane_CN", false);
                  update_area_and_inputs("main_address_country_pane_others", false);
                } else if(country_select.value == 'CN'){
                  update_area_and_inputs("main_address_country_pane_JP", false);
                  update_area_and_inputs("main_address_country_pane_CN", true);
                  update_area_and_inputs("main_address_country_pane_others", false);
                } else{
                  update_area_and_inputs("main_address_country_pane_JP", false);
                  update_area_and_inputs("main_address_country_pane_CN", false);
                  update_area_and_inputs("main_address_country_pane_others", true);
                }
              }
              Event.observe(country_select, "change", update_addresses_by_country);
              Event.observe(country_select, "keypress", function(){
                if (last_value != country_select.value) {
                  update_addresses_by_country();
                  last_value = country_select.value;
                }
              });
              Event.observe(window, "load", update_addresses_by_country);
            })();
        MSG

        if @user_profile.object.not_decided_address || shippable_to_oversea?
          @user_address.select(:country_code,  Country.ship_to_country_list, {}, options) << js_func
        else
          "Japan" << @user_address.hidden_field(:country_code) << js_func
        end
      end


      desc <<-EOS
        [概要:メイン住所によって表示が切り替わるタグを出力します]
        [引数 country(任意):どの国名に対して領域を表示するのかを指定します。指定しなかった場合はothersが指定されたものとして、
          メイン住所の国に合致するものがなかった場合に適用されます]
        [引数 tag_name(任意):表示する領域として使用するHTMLのタグ名を指定します。指定しなかった場合はdivが適用されます]
        [同様のタグ:user_address_country_pane　住所フォーム内で使用]
      EOS
      def signup_form_main_address_country_pane(options = {}, &block)
        raise UsageError, "ec:signup_form_main_address_japanese_zip_codeはec:signup_formの中で使用してください" unless @user_address
        options = {:tag_name => 'div', :country => "others"}.update(options || {})
        country_code = options.delete(:country)
        options[:class] = "main_address_country_pane_#{country_code} " << (options[:class] || '')
        options[:style] = "display:none; " <<(options[:style] || '') if country_code.downcase != @user_address.object.country_code.downcase
        tag_name = options.delete(:tag_name)
        @template.concat(@template.tag(tag_name, options, true))
        @template.concat(@template.capture(&block))
        @template.concat("</#{tag_name}>")
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の日本用郵便番号を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_japanese_zip_code　住所フォーム内で使用]
      EOS
      def signup_form_main_address_japanese_zip_code(options = {})
        raise UsageError, "ec:signup_form_main_address_japanese_zip_codeはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 7,:size => 7, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        validation_option = {:numeric => true, :realtime => true, :update => 'zip_code_msg', :error =>I18n.t(:"activerecord.attributes.user_address.zip_code")}
        @user_address.text_field_valid(:zip_code, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の海外用郵便番号を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_zip_code　住所フォーム内で使用]
      EOS
      def signup_form_main_address_zip_code(options = {})
        raise UsageError, "ec:signup_form_main_address_zip_codeはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 15,:size => 15, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        @user_address.text_field(:zip_code, options)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の日本用郵便番号検索のinput(ボタン)タグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_zip_to_address_button　住所フォーム内で使用]
      EOS
      def signup_form_main_address_zip_to_address_button(*args, &block)
        options = args.shift || {}
        text = @template.capture(&block) if block_given?

        js_msg = <<-"MSG"
        str=$('zip_code_msg').innerHTML;
        str=str.replace(/#{I18n.t(:"activerecord.attributes.user_address.zip_code")}#{I18n.t(:"activerecord.errors.messages.EM990007")}/, '');
        errmsg = str + '#{I18n.t(:"activerecord.attributes.user_address.zip_code")}#{I18n.t(:"activerecord.errors.messages.EM990007")}';
        res = eval(request.responseText);
        $('user_address_address_1').value = res[0];
        $('user_address_address_2').value = res[1] + res[2];
        if(res[0] == ''){ error_msg('zip_code_msg',errmsg); }
        MSG

        result = @template.button_to_remote("郵便番号で住所表示",
          :url => {:controller=>:common, :action=>:to_address},
          :with => "'id=' + $F('user_address_zip_code') + '&session_id_validation=#{@session_id_validation}'",
          :success => js_msg) if text
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の日本用郵便番号のバリデーションエラーメッセージを表示するspanタグを出力します。
          新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_japanese_zip_code_error_message　住所フォーム内で使用]
      EOS
      def signup_form_main_address_japanese_zip_code_error_message(options = {})
        raise UsageError, "ec:signup_form_main_address_japanese_zip_code_error_messageはec:signup_formの中で使用してください" unless @user_address
        options = {:class => "error_msg"}.update(options || {})
        html_options = 'id="zip_code_msg"'
        options.each_pair {|key, value|
          html_options << " #{key}=\"#{value}\""
        }
        "<span #{html_options}></span>"
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の都道府県を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_japanese_prefecture　住所フォーム内で使用]
      EOS
      def signup_form_main_address_japanese_prefecture(options = {})
        raise UsageError, "ec:signup_form_main_address_japanese_prefectureはec:signup_formの中で使用してください" unless @user_address
        @user_address.select(:address_1, @template.prefecture_names_for_select)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の都道府県や州を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_prefecture　住所フォーム内で使用]
      EOS
      def signup_form_main_address_prefecture(options = {})
        raise UsageError, "ec:signup_form_main_address_prefectureはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 30, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        validation_option = {:empty => true, :realtime => true, :update => 'address_msg', :error =>I18n.t(:"activerecord.attributes.user_address.address_1")}
        @user_address.text_field_valid(:address_1, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の市区町村番地を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_address_1　住所フォーム内で使用]
      EOS
      def signup_form_main_address_address_1(options = {})
        raise UsageError, "ec:signup_form_main_address_address_1はec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 60,:size => 60,:class=>:default,:style => "ime-mode: active"}.update(options || {})
        validation_option = {:empty => true, :realtime => true, :update => 'address_msg', :error =>I18n.t(:"activerecord.models.user_address")}
        @user_address.text_field_valid(:address_2, options, validation_option)
      end

      desc <<-EOS
        [概要:新規会員登録用住所のマンション名以下を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_address_2　住所フォーム内で使用]
      EOS
      def signup_form_main_address_address_2(options = {})
        raise UsageError, "ec:signup_form_main_address_address_2はec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 60,:size => 60,:class=>:default,:style => "ime-mode: active"}.update(options || {})
        @user_address.text_field(:address_3, options)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の住所のバリデーションエラーメッセージを表示するspanタグを出力します。
          新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_address_error_message　住所フォーム内で使用]
      EOS
      def signup_form_main_address_address_error_message(options = {})
        raise UsageError, "ec:signup_form_main_address_address_error_messageはec:signup_formの中で使用してください" unless @user_address
        options = {:class => "error_msg"}.update(options || {})
        html_options = 'id="address_msg"'
        options.each_pair {|key, value|
          html_options << " #{key}=\"#{value}\""
        }
        "<span #{html_options}></span>"
      end

      desc <<-EOS
        [概要:新規会員登録用電話番号を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_tel_num　住所フォーム内で使用]
      EOS
      def signup_form_main_address_tel_num(options = {})
        raise UsageError, "ec:signup_form_main_address_tel_numはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 20,:size => 20,:class=>:default,:style => "ime-mode: disabled"}.update(options || {})
        validation_option = {:empty => true, :onebyte => true, :realtime => true, :update => 'tel_num_msg', :error => I18n.t(:"activerecord.attributes.user_address.tel_num")}
        @user_address.text_field_valid(:tel_num, options, validation_option)
      end

      # 別名を定義します。
      method_names = %w(country country_pane japanese_zip_code zip_code zip_to_address_button japanese_zip_code_error_message japanese_prefecture prefecture address_1 address_2 address_error_message tel_num)

      src_form_name = 'signup_form_main_address'
      alias_form_names = %w(user_address)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      #
      #@user_en_address
      #
      desc <<-EOS
        [概要:新規会員登録用英語住所の姓を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_english_address_last_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_english_address_last_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_english_address_last_name(options = {})
        raise UsageError, "ec:signup_form_english_address_last_nameはec:signup_formの中で使用してください" unless @user_en_address
        options = {:maxLength => 30,:size => 30, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        @user_en_address.text_field(:name_last, options)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の名を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_english_address_first_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_english_address_first_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_english_address_first_name(options = {})
        raise UsageError, "ec:signup_form_english_address_first_nameはec:signup_formの中で使用してください" unless @user_en_address
        options = {:maxLength => 30,:size => 30, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        @user_en_address.text_field(:name_first, options)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の都道府県を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_english_address_prefecture　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_english_address_prefecture　非会員用住所フォーム内で使用]
      EOS
      def signup_form_english_address_prefecture(options = {})
        raise UsageError, "ec:signup_form_english_address_prefectureはec:signup_formの中で使用してください" unless @user_en_address
        options = {:maxLength => 30,:size => 30, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        @user_en_address.text_field(:address_1, options)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の市区町村番地を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_english_address_address_1　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_english_address_address_1　非会員用住所フォーム内で使用]
      EOS
      def signup_form_english_address_address_1(options = {})
        raise UsageError, "ec:signup_form_en_address_address_1はec:signup_formの中で使用してください" unless @user_en_address
        options = {:maxLength => 60,:size => 60, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        @user_en_address.text_field(:address_2, options)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所のマンション以下を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_english_address_address_2　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_english_address_address_2　非会員用住所フォーム内で使用]
      EOS
      def signup_form_english_address_address_2(options = {})
        raise UsageError, "ec:signup_form_english_address_address_2はec:signup_formの中で使用してください" unless @user_en_address
        options = {:maxLength => 60,:size => 60, :class=>:default, :style => "ime-mode: disabled"}.update(options || {})
        @user_en_address.text_field(:address_3, options)
      end

      # 別名を定義します。
      method_names = %w(last_name first_name prefecture address_1 address_2)

      src_form_name = 'signup_form_english_address'
      alias_form_names = %w(account_form_english_address nonmember_address_form_english_address)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      #
      #@user_email
      #
      desc <<-EOS
        [概要:新規会員登録用メールアドレスを入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_email　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_email　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_email(options = {})
        raise UsageError, "ec:signup_form_profile_emailはec:signup_formの中で使用してください" unless @user_email
        options = {:maxLength => 50, :size => 35,
          :class => :default, :style => "ime-mode: inactive"}.update(options || {})
        validation_option = {:empty => true, :email => true, :realtime => true, :update => 'email_msg', :error => I18n.t(:"activerecord.attributes.user_email.email")}
        @user_email.text_field_valid(:email, options, validation_option) <<
          @user_email.hidden_field(:pc_mobile_div, :value => UserEmail.pc_mobile_div_id_by_key(:pc))
      end

      desc <<-EOS
        [概要:新規会員登録用メールアドレスのバリデーションエラーメッセージを表示するspanタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_email_error_message　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_email_error_message　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_email_error_message(options = {})
        raise UsageError, "ec:signup_form_profile_email_error_messageはec:signup_formの中で使用してください" unless @user_email
        options = {:class => "error_msg"}.update(options || {})
        html_options = 'id="email_msg"'
        options.each_pair {|key, value|
          html_options << " #{key}=\"#{value}\""
        }
        "<span #{html_options}></span>"
      end

      # 別名を定義します。
      method_names = %w(email email_error_message)

      src_form_name = 'signup_form_profile'
      alias_form_names = %w(account_form_profile nonmember_address_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      #
      #@user_profile
      #
      desc <<-EOS
        [概要:新規会員登録用性別を入力する欄を出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_gender　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_gender　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_gender(&block)
        raise UsageError, "ec:signup_form_profile_genderはec:signup_formの中で使用してください" unless @user_profile
        @user_profile.radio_button_group(:sex) do |b|
          b.each do
            @user_profile_gender = b
            begin
              @template.concat(@template.capture(&block))
            ensure
              @user_profile_gender = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:新規会員登録用性別をのinputラジオボタンタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_gender_radio　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_gender_radio　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_gender_radio(options = {})
        raise UsageError, "ec:payment_form_gender_radioはec:payment_form_card_kindの中で使用してください" unless @user_profile_gender
        options = {}.update(options || {})
        @user_profile_gender.radio_button(options)
      end

      desc <<-EOS
        [概要:新規会員登録用性別のラジオボタンに表示するラベルを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_gender_label　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_gender_label　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_gender_label(options = {})
        raise UsageError, "ec:payment_form_card_kind_radio_labelはec:payment_form_card_kindの中で使用してください" unless @user_profile_gender
        @user_profile_gender.label
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の年を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_birth_year　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_birth_year　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_birth_year(options = {})
        raise UsageError, "ec:signup_form_profile_birth_yearはec:signup_formの中で使用してください" unless @user_profile
        options = {:prefix => 'user_profile', :field_name => 'birth_year',
          :start_year => Date.today.year, :end_year => Date.today.year-99, :include_blank => true}.update(options || {})
        @template.select_year(@user_profile.object.birth_year.to_i, options)
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の月を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_birth_month　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_birth_month　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_birth_month(options = {})
        raise UsageError, "ec:signup_form_profile_birth_monthはec:signup_formの中で使用してください" unless @user_profile
        options = {:prefix => 'user_profile', :field_name => 'birth_month',
          :include_blank => true, :use_month_numbers => true}.update(options || {})
        @template.select_month(@user_profile.object.birth_month.to_i, options)
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の日を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_birth_day　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_birth_day　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_birth_day(options = {})
        raise UsageError, "ec:signup_form_profile_birth_dayはec:signup_formの中で使用してください" unless @user_profile
        options = {:prefix => 'user_profile', :field_name => 'birth_day',
          :include_blank => true}.update(options || {})
        @template.select_day(@user_profile.object.birth_day.to_i, options)
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日のバリデーションエラーメッセージを表示するspanタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_birth_error_message　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_birth_error_message　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_birth_error_message(options = {})
        raise UsageError, "ec:signup_form_profile_birth_error_messageはec:signup_formの中で使用してください" unless @user_email
        options = {:class => "error_msg"}.update(options || {})
        html_options = 'id="birth_msg"'
        options.each_pair {|key, value|
          html_options << " #{key}=\"#{value}\""
        }
        "<span #{html_options}></span>"
      end

      desc <<-EOS
        [概要:新規会員登録用職業を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_gender　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_gender　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_job(options = {})
        raise UsageError, "ec:signup_form_profile_jobはec:signup_formの中で使用してください" unless @user_profile
        @user_profile.select(:job_div,  User.job_div_options, {:include_blank => true}, options)
      end

      desc <<-EOS
        [概要:新規会員登録用性別を入力するselectタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_gender_select　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_gender_select　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_gender_select(options = {})
        raise UsageError, "ec:signup_form_profile_gender_selectはec:signup_formの中で使用してください" unless @user_profile
        @user_profile.select(:sex,  User.sex_options, {:include_blank => true}, options)
      end

      desc <<-EOS
        [概要:新規会員登録用PCメールマガジン購読区分を入力する欄を出力します。新規会員登録用のフォーム内で使用してください。]
        [引数 default_checked(任意)メールマガジン配信 区分が登録されていない場合の初期値を設定します   0:HTMLメール 1:テキストメール 99:受け取らない]
        [同様のタグ:account_form_profile_pc_mail_magazine　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_pc_mail_magazine　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_pc_mail_magazine(*args, &block)
        default_checked, option = extract(args, :default_checked => :not_prefer)
        raise UsageError, "ec:signup_form_profile_pc_mail_magazineはec:signup_formの中で使用してください" unless @user_profile
        @user_profile.object.pc_mail_magazine_send_div ||= default_checked
        @user_profile.radio_button_group(:pc_mail_magazine_send_div) do |b|
          b.each do
            @user_profile_pc_mail_magazine = b
            begin
              @template.concat(@template.capture(&block))
            ensure
              @user_profile_pc_mail_magazine = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:新規会員登録用PCメールマガジン購読区分のinputラジオボタンタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_pc_mail_magazine_radio　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_pc_mail_magazine_radio　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_pc_mail_magazine_radio(options = {}, &block)
        raise UsageError, "ec:payment_form_pc_mail_magazine_radioはec:signup_form_profile_pc_mail_magazineの中で使用してください" unless @user_profile_pc_mail_magazine
        options = {}.update(options || {})
        @user_profile_pc_mail_magazine.radio_button(options)
      end

      desc <<-EOS
        [概要:新規会員登録用PCメールマガジン購読区分のラジオボタンに表示するラベルを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_pc_mail_magazine_label　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_pc_mail_magazine_label　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_pc_mail_magazine_label(options = {})
        raise UsageError, "ec:signup_form_profile_pc_mail_magazine_labelはec:signup_form_profile_pc_mail_magazineの中で使用してください" unless @user_profile_pc_mail_magazine
        @user_profile_pc_mail_magazine.label
      end

      # 別名を定義します。
      method_names = %w(gender gender_radio gender_label gender_select birth_year birth_month birth_day birth_error_message job pc_mail_magazine pc_mail_magazine_radio pc_mail_magazine_label)

      src_form_name = 'signup_form_profile'
      alias_form_names = %w(account_form_profile nonmember_address_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用パスワードを入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:nonmember_address_form_profile_password　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_password(options = {})
        raise UsageError, "ec:signup_form_profile_passwordはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 20, :size => 50}.update(options || {})
        @user_profile.password_field(:password, options)
      end

      desc <<-EOS
        [概要:新規会員登録用パスワードの確認を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:nonmember_address_form_profile_password_confirmation　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_password_confirmation(options = {})
        raise UsageError, "ec:signup_form_profile_password_confirmationはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 20, :size => 50}.update(options || {})
        @user_profile.password_field(:password_confirmation, options)
      end

      desc <<-EOS
        [概要:新規会員登録用秘密の質問を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:nonmember_address_form_profile_remember_question　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_remember_question(options = {})
        raise UsageError, "ec:signup_form_profile_remember_questionの中で使用してください" unless @user_profile
        options = {:maxLength => 20, :size => 70,
          :class => :default, :style => "ime-mode: active"}.update(options || {})
        @user_profile.text_field(:remember_question, options)
      end

      desc <<-EOS
        [概要:新規会員登録用秘密の質問の答えを入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:nonmember_address_form_profile_remember_answer　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_remember_answer(options = {})
        raise UsageError, "ec:signup_form_profile_remember_answerはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 50, :size => 35,
          :class => :default, :style => "ime-mode: active"}.update(options || {})
        @user_profile.text_field(:remember_answer, options)
      end

      # 別名を定義します。
      method_names = %w(password password_confirmation remember_question remember_answer)

      src_form_name = 'signup_form_profile'
      alias_form_names = %w(nonmember_address_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end


      desc <<-EOS
        [概要:登録内容の確認用のフォームを出力します。]
      EOS
      def signup_confirmation_form(&block)
        unless @template.instance_variable_get(:@user_address) && @template.instance_variable_get(:@user_email) && @template.instance_variable_get(:@user_profile)
          raise UsageError, "ここでは使用できません。"
        end
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => 'account', :action => :create_user}, :html => {:id => "signup_confirmation_form" }) do |f|
          @template.fields_for(:user_en_address, @template.instance_variable_get(:@user_en_address)) do |f1|
            @template.fields_for(:user_email, @template.instance_variable_get(:@user_email)) do |f2|
              @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f3|
                @user_address = f
                @user_en_address = f1
                @user_email = f2
                @user_profile = f3
                begin
                  @template.concat(@template.capture(&block))
                ensure
                  @user_address = nil
                  @user_en_address = nil
                  @user_email = nil
                  @user_profile = nil
                end
              end
            end
          end
        end
      end

      desc <<-EOS
        [概要:住所が日本語フォーマットの場合は中身を実行します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def if_signup_address_japanese_format
        if @template.instance_variable_get(:@user_address).is_japanese_format?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:住所が日本語フォーマットで場合は中身を実行します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def unless_signup_address_japanese_format
        unless @template.instance_variable_get(:@user_address).is_japanese_format?
          yield if block_given?
        end
      end
      
      desc <<-EOS
        [概要:英語住所の入力がある場合は中身を実行します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def if_english_address_input
        if @template.instance_variable_get(:@user_en_address)
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:英語住所の入力がない場合は中身を実行します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def unless_english_address_input
        unless @template.instance_variable_get(:@user_en_address)
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:生年月日の入力がある場合は中身を実行します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def if_birth_day_input
        if @user_profile.object.birth_year && @user_profile.object.birth_month && @user_profile.object.birth_day
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:生年月日の入力がない場合は中身を実行します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def unless_birth_day_input
        unless @user_profile.object.birth_year && @user_profile.object.birth_month && @user_profile.object.birth_day
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:新規会員登録メイン住所の姓の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_last_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_last_name
        raise UsageError, "ec:signup_confirmation_form_main_address_last_nameはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.name_last)
      end

      desc <<-EOS
        [概要:新規会員登録メイン住所の名の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_first_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_first_name
        raise UsageError, "ec:signup_confirmation_form_main_address_first_nameはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.name_first)
      end

      desc <<-EOS
        [概要:新規会員登録メイン住所の姓名の入力内容をフルネームで出力します。日本語の場合は姓名、英語の場合は名姓で表示します。
          登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_full_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_full_name
        raise UsageError, "ec:signup_confirmation_form_main_address_full_nameはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.fullname)
      end

      desc <<-EOS
        [概要:新規会員登録メイン住所のカナ姓の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_kana_last_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_kana_last_name
        raise UsageError, "ec:signup_confirmation_form_main_address_kana_last_nameはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_profile.object.kana_name_last)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所のカナ名の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_kana_first_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_kana_first_name
        raise UsageError, "ec:signup_confirmation_form_main_address_kana_first_nameはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_profile.object.kana_name_first)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の国の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_country　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_country
        raise UsageError, "ec:signup_confirmation_form_main_address_countryはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_profile.object.user_address.country.name)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の郵便番号の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_zip_code　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_zip_code
        raise UsageError, "ec:signup_confirmation_form_main_address_japanese_zip_codeはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.zip_code)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の都道府県の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_prefecture　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_prefecture
        raise UsageError, "ec:signup_confirmation_form_main_address_japanese_prefectureはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.address_1)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の市区町村番地の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_address_1　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_address_1
        raise UsageError, "ec:signup_confirmation_form_main_address_address_1はec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.address_2)
      end

      desc <<-EOS
        [概要:新規会員登録用住所のマンション名以下の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_address_2　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_address_2
        raise UsageError, "ec:signup_confirmation_form_main_address_address_2はec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.address_3)
      end

      desc <<-EOS
        [概要:新規会員登録用住所の都道府県以下の入力内容をフォーマットして出力します。日本と日本以外を判断してフォーマットします。
          登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_full_address　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_full_address
        raise UsageError, "ec:signup_confirmation_form_main_address_address_2はec:signup_confirmation_formの中で使用してください" unless @user_address
        @template.print_address(@user_address.object, :include_zipcode => true)
      end

      desc <<-EOS
        [概要:新規会員登録用電話番号の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_main_address_tel_num　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_main_address_tel_num
        raise UsageError, "ec:signup_confirmation_form_main_address_tel_numはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.tel_num)
      end

      # 別名を定義します。
      method_names = %w(last_name first_name kana_last_name kana_first_name full_name country zip_code prefecture address_1 address_2 full_address tel_num)

      src_form_name = 'signup_confirmation_form_main_address'
      alias_form_names = %w(account_confirmation_form_main_address)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の姓の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_last_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_last_name
        raise UsageError, "ec:signup_confirmation_form_english_address_last_nameはec:signup_confirmation_formの中で使用してください" unless @user_en_address
        ERB::Util.h(@user_en_address.object.name_last)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の名の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_first_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_first_name
        raise UsageError, "ec:signup_confirmation_form_english_address_first_nameはec:signup_confirmation_formの中で使用してください" unless @user_en_address
        ERB::Util.h(@user_en_address.object.name_first)
      end

      desc <<-EOS
        [概要:新規会員登録英語住所の姓名の入力内容をフルネームで出力します。日本語の場合は姓名、英語の場合は名姓で表示します。
          登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_full_name　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_full_name
        raise UsageError, "ec:signup_confirmation_form_english_address_full_nameはec:signup_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_en_address.object.fullname)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の都道府県の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_prefecture　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_prefecture
        raise UsageError, "ec:signup_confirmation_form_english_address_prefectureはec:signup_confirmation_formの中で使用してください" unless @user_en_address
        ERB::Util.h(@user_en_address.object.address_1)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の市区町村番地の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_address_1　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_address_1
        raise UsageError, "ec:signup_confirmation_form_en_address_address_1はec:signup_confirmation_formの中で使用してください" unless @user_en_address
        ERB::Util.h(@user_en_address.object.address_2)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所のマンション以下の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_address_2　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_address_2
        raise UsageError, "ec:signup_confirmation_form_english_address_address_2はec:signup_confirmation_formの中で使用してください" unless @user_en_address
        ERB::Util.h(@user_en_address.object.address_3)
      end

      desc <<-EOS
        [概要:新規会員登録用英語住所の都道府県以下の入力内容をフォーマットして出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_english_address_full_address　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_english_address_full_address
        raise UsageError, "ec:signup_confirmation_form_english_address_address_2はec:signup_confirmation_formの中で使用してください" unless @user_address
        @template.print_address(@user_en_address.object, :include_zipcode => true)
      end

      # 別名を定義します。
      method_names = %w(last_name first_name full_name prefecture address_1 address_2 full_address)

      src_form_name = 'signup_confirmation_form_english_address'
      alias_form_names = %w(account_confirmation_form_english_address)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用メールアドレスの入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_birth_email　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_email
        raise UsageError, "ec:signup_confirmation_form_profile_emailはec:signup_confirmation_formの中で使用してください" unless @user_email
        ERB::Util.h(@user_email.object.email)
      end

      desc <<-EOS
        [概要:新規会員登録用性別の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_birth_gender　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_gender
        raise UsageError, "ec:signup_confirmation_form_profile_genderはec:signup_confirmation_formの中で使用してください" unless @user_profile
        @user_profile.object.sex_name
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の年の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_birth_year　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_birth_year
        raise UsageError, "ec:signup_confirmation_form_profile_birth_yearはec:signup_confirmation_formの中で使用してください" unless @user_profile
        ERB::Util.h(@user_profile.object.birth_year)
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の月の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_birth_month　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_birth_month
        raise UsageError, "ec:signup_confirmation_form_profile_birth_monthはec:signup_confirmation_formの中で使用してください" unless @user_profile
        ERB::Util.h(@user_profile.object.birth_month)
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の日の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_birth_day　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_birth_day
        raise UsageError, "ec:signup_confirmation_form_profile_birth_dayはec:signup_confirmation_formの中で使用してください" unless @user_profile
        ERB::Util.h(@user_profile.object.birth_day)
      end

      desc <<-EOS
        [概要:新規会員登録用職業の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_job　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_job
        raise UsageError, "ec:signup_confirmation_form_profile_jobはec:signup_confirmation_formの中で使用してください" unless @user_profile
        @user_profile.object.job_div_name
      end

      desc <<-EOS
        [概要:新規会員登録用PCメールマガジン配信区分の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_pc_mail_magazine　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_pc_mail_magazine
        raise UsageError, "ec:signup_confirmation_form_profile_pc_mail_magazineはec:signup_confirmation_formの中で使用してください" unless @user_profile
        @user_profile.object.pc_mail_magazine_send_div_name
      end

      # 別名を定義します。
      method_names = %w(email gender birth_year birth_month birth_day job pc_mail_magazine)

      src_form_name = 'signup_confirmation_form_profile'
      alias_form_names = %w(account_confirmation_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用秘密の質問を入力するinputタグを出力します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def signup_confirmation_form_profile_remember_question
        raise UsageError, "ec:signup_confirmation_form_profile_remember_questionの中で使用してください" unless @user_profile
        ERB::Util.h(@user_profile.object.remember_question)
      end

      desc <<-EOS
        [概要:新規会員登録用秘密の質問の答えを入力するinputタグを出力します。登録内容確認用のフォーム内で使用してください。]
      EOS
      def signup_confirmation_form_profile_remember_answer
        raise UsageError, "ec:signup_confirmation_form_profile_remember_answerはec:signup_confirmation_formの中で使用してください" unless @user_profile
        ERB::Util.h(@user_profile.object.remember_answer)
      end
    end
  end
end
