# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module UserLoginFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:ログイン用のフォームを出力します。]
      EOS
      def user_login_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user_login_form)
        # このメソッドは、ec_adminからも実行されるので、jpmobileがインストールされていないかもしれません。
        # ですので、mobile?メソッドが存在するかどうかを確認する必要があります。
        req = @template.__send__(:request)
        url = (req.respond_to?(:mobile?) && req.mobile?) && false ? # FIXME mobileのログインアクションパス
          {:controller => 'mobile_sessions', :action => :create} :
          {:controller => 'account', :action => :authenticate}
        @template.form_for(:user_login_form, :url => url) do |f|
          user_login_form_backup = @user_login_form
          @user_login_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user_login_form = user_login_form_backup
          end
        end
      end

      desc <<-EOS
        [概要:ログイン時のエラーメッセージがあればそれを表示します。ログイン用のフォーム内で使用してください。]
        [使用例:
          {{{
          <ec:user_login_form_error_messages/>
          　　==> <div class="alert_msg"><ul><li>パスワードが正しくありません</li></ul></div>
          }}}
        ]
      EOS
      def user_login_form_error_messages
        if @template.instance_variable_get(:@login_error_notice)
          content_tag(:div, content_tag(:ul, content_tag(:li, @template.instance_variable_get(:@login_error_notice))),:class=> 'alert_msg')
        else
          @template.template_error_messages_for(:user_login_form)
        end
      end

      desc <<-EOS
        [概要:ログイン用メールアドレスを入力するinputタグを出力します。ログイン用のフォーム内でのみ使用できます。]
      EOS
      def user_login_form_email(options = {})
        raise UsageError, "ec:user_login_form_emailはec:user_login_formの中で使用してください" unless @user_login_form
        options = {:maxLength => 50, :size => 35,
          :class => :default, :style => "ime-mode: inactive"}.update(options || {})
        @user_login_form.text_field(:email, options)
      end

      desc <<-EOS
        [概要:ログイン用パスワードを入力するinputタグを出力します。ログイン用のフォーム内でのみ使用できます。]
      EOS
      def user_login_form_password(options = {})
        raise UsageError, "ec:user_login_form_passwordはec:user_login_formの中で使用してください" unless @user_login_form
        options = {:maxLength => 20, :size => 50}.update(options || {})
        @user_login_form.password_field(:password, options)
      end

      desc <<-EOS
        [概要:「次回からの入力を省略」のチェックボックスを出力します。ログイン用のフォーム内で使用してください。]
      EOS
      def user_login_form_remember_me(options = {})
        @template.check_box_tag("remember_me", 1, true, options)
      end

      desc <<-EOS
        [概要:メールアドレス設定用のフォームを出力します。]
      EOS
      def add_mailaddress_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user_email)
        @template.form_for(:user_email, @template.instance_variable_get(:@user_email), :url => {:controller => 'account', :action => :confirm_mailaddress}) do |f|
          @user_email_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user_email_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:パスワード再設定画面から遷移した場合はタグの中身を実行します。]
        [使用例:
          {{{
          <ec:if_reminder_to_add_mailaddress_form>
            パスワードを変更しました。
          </ec:if_reminder_to_add_mailaddress_form>
          }}}
        ]
      EOS
      def if_reminder_to_add_mailaddress(&block)
        if @template.instance_variable_get(:@from_reminder)
          yield
        end
      end

      desc <<-EOS
        [概要:メールアドレス設定時のエラーメッセージがあればそれを表示します。メールアドレス設定用のフォーム内で使用してください。]
        [使用例:
          {{{
          <ec:add_mailaddress_form_error_messages/>
          　　==> <div class="alert_msg"><ul><li>メールアドレスが正しくありません</li></ul></div>
          }}}
        ]
      EOS
      def add_mailaddress_form_error_messages
        @template.template_error_messages_for(:user_email)
      end

      desc <<-EOS
        [概要:メールアドレス設定用メールアドレスを入力するinputタグを出力します。メールアドレス設定用のフォーム内でのみ使用できます。]
      EOS
      def add_mailaddress_form_email(options = {})
        raise UsageError, "ec:add_mailaddress_form_emailはec:add_mailaddress_formの中で使用してください" unless @user_email_form
        options = {:maxLength => 50, :size => 35,
          :class => :default, :style => "ime-mode: inactive"}.update(options || {})
        @user_email_form.text_field(:email, options)
      end

      desc <<-EOS
        [概要:遷移元の画面に戻るボタンを表示します。]
        [戻り値:画面遷移リンク。]
        [使用例:{{{<ec:add_mailaddress_form_button_to_back/>}}}]
      EOS
      def add_mailaddress_form_button_to_back(*args, &block)
        back_to_url = @template.instance_variable_get(:@back_to_url)
        raise UsageError, "ec:add_mailaddress_form_button_to_backはメールアドレス設定画面で使用してください" unless back_to_url
        options = args.shift || {}
        result = @template.button_to_url(@template.capture(&block), back_to_url, options) if block_given?
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:メールアドレス設定確認用のフォームを出力します。]
      EOS
      def add_mailaddress_confirmation_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user_email)
        @template.form_for(:user_email, @template.instance_variable_get(:@user_email), :url => {:controller => 'account', :action => :update_mailaddress}) do |f|
          @user_email_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user_email_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:設定したメールアドレスを表示します。メールアドレス設定確認用のフォーム内でのみ使用できます。]
      EOS
      def add_mailaddress_confirmation_form_email(options = {})
        raise UsageError, "ec:add_mailaddress_confirmation_form_emailはec:add_mailaddress_confirmation_formの中で使用してください" unless @user_email_form
        @user_email_form.object.email
      end
    end
  end
end
