# -*- coding: utf-8 -*-
module EcRider
  module JsValidationHelper
    module FormBuilderExt
      def text_field_valid(*args, &block)
        @template.text_field_valid(object_name, *args, &block)
      end
    end

    #
    #  == text_field_jsvalidation (object_name, method, validate, options={} ) ==
    #  object_name,method,optionsはtextfiledに直接渡されます
    #
    #   バリデート引数が設定されているときはその条件によるバリデーションを実行します。
    #   値がtrueの場合、バリデーションを実行します。
    #
    #     :realtime           # リアルタイム実行をするかしないか (デフォルトはリアルタイム実行)
    #     :update             # エラー表示先のid属性値 指定しないとエラーメッセージ表示用のspanを生成する
    #                         # alertが指定されるとalertを実行
    #
    #     :empty              # 空欄の場合エラー
    #     :numeric            # 数字
    #     :alpha              # アルファベット
    #     :onebyte            # 半角
    #     :towbyte            # 全角
    #     :email              # メールアドレス
    #     :url                # URL
    #     :max                # 最大文字数
    #     :min                # 最小文字数
    #     :other              # 任意の関数(javascript)
    #
    #     :red_notifier       # エラー発生時にフォームの背景色を赤くする。false => OFF
    #
    #     :error              # エラーメッセージ、　:error + @errorsが出力される
    #                         # false指定の場合は表示しない
    #
    #     :onFailer           # 成功時に実行するjavascriptを指定
    #     :onSucssese　       # 失敗時に実行するjavascriptを指定
    def text_field_valid(object_name, method, options={} , validate ={})
      #バリデーション方法が指定されないとき通常のテキストフィールドを出力
      return text_field(object_name, method, options) if validate == {}

      #object_nameの末尾が[]の場合idがobject_name_id_methodとなる
      if object_name.to_s.dup.sub!(/\[\]$/,"")
        object = self.instance_variable_get('@' + Regexp.last_match.pre_match)
        id = Regexp.last_match.pre_match + "_" + object.id.to_s + "_" + method.to_s
      else
        id = object_name.to_s + "_" + method.to_s
      end

      error_msg = validate[:error] || id

      generete_error_message = generate_function_generate_error_message(validate[:update]||"validate_" + id,error_msg)

      # JSのif(...){ } の...で使用される評価のためのスクリプト
      evaluation_for_if = []
      evaluation_for_if << "(($F(\'#{id}\') != '')#{generete_error_message.call(:empty)})" if validate[:empty]
      evaluation_for_if << "(validate.isNum($F(\'#{id}\'))#{generete_error_message.call(:numeric)})" if validate[:numeric]
      evaluation_for_if << "(($F(\'#{id}\').length >= #{validate[:min]})#{generete_error_message.call(:min)})" if validate[:min]
      evaluation_for_if << "(($F(\'#{id}\').length <= #{validate[:max]}#{generete_error_message.call(:max)})" if validate[:max]
      evaluation_for_if << "(validate.isEMailAddr($F(\'#{id}\'))#{generete_error_message.call(:email)})" if validate[:email]
      evaluation_for_if << "(validate.isURL($F(\'#{id}\'))#{generete_error_message.call(:url)})" if validate[:url]
      evaluation_for_if << validate[:other] if !validate[:other].blank?

      #なにかあった時にやること # このコメント意味分からん
      success_script = ""
      failure_script = ""

      unless validate[:red_notifier] == false
        success_script << "Element.addClassName($('#{id}'),'default');Element.removeClassName($('#{id}'),'error');"
        failure_script << "Element.addClassName($('#{id}'),'error');Element.removeClassName($('#{id}'),'default');"
      end

      success_script << "$('#{validate[:update]||"validate_" + id}').innerHTML = '';" if error_msg

      success_script << "return true;"
      failure_script << "return false;"

      validate_script = 
        "if(" << evaluation_for_if.join(" && ") << "){" <<
          success_script <<
        "}else{" <<
          failure_script <<
        "}"

      update_area = nil
      if validate["update"].nil?
        update_area = "<span id=\"validate_#{object_name}_#{method}\" ></span>"
      end

      text_field(object_name, method, options).sub(/\/>$/, "onblur=\"#{validate_script}\" />#{update_area}")
    end

    #エラー時の処理用の
    def generate_function_generate_error_message(id,message)
      errors = {
        :empty   => i18n_message(:EM990001),
        :numeric  => i18n_message(:EM990003),
        :alpha    => i18n_message(:EM990028),
        :onebyte  => i18n_message(:EM990029),
        :towbyte  => i18n_message(:EM990005),
        :max      => i18n_message(:EM990030),
        :min      => i18n_message(:EM990031),
        :email   => i18n_message(:EM990007)
      }
      errors.default = "invalid"
      if message
        #lambda{|type| " || writeErrorMessage('#{id}','#{message + errors[type]}')"}
        lambda{|type| " || error_msg('#{id}','#{message + errors[type]}')"}
      else
        lambda{|type| ""}
      end
    end

  end
end
