# -*- coding: utf-8 -*-
module OrderHelper
  include EcRider::I18nSupport
  include ProductDialogHelper::Client

  # 購入数のセレクトボックス用選択肢を返す
  def order_count_list(product_sku)
    Array.new(product_sku.orderable_count){|i| i+1 }
  end

  # メールテンプレートのセレクトボックス用選択肢を返す
  def mail_template_list(order_shipping)
    MailTemplate.mail_type_div_options(:order_reply, :free)
  end


  #
  # 店舗のロゴを表示する
  #
  def shop_logo(locale)
    image_url = ShopSetting.get_setting(:shop_logo, locale, :value => :url)
    if image_url.blank?
      return ''
    else
      shop_name = ShopSetting.get_setting(:shop_name, locale)
      return image_tag(image_url, :alt => shop_name) 
    end
  end

  #
  # 注文に含まれる商品情報をカテゴリごとにまとめる
  #
  def grouped_products(order_shipping)
    list = {}
    order_shipping.order_items.each do |order_item|
      key = order_item.product_sku.product.category_1_id 
      list[key] ||= {:weight => 0, :amount => 0, :count => 0}
      list[key][:weight] += (order_item.product_sku.weight * order_item.qty) if order_item.product_sku
      list[key][:amount] += order_item.subtotal_amount
      list[key][:count] += order_item.qty
    end
    return list
  end

  #
  # 原産国のリストを生成する
  #
  def origin_list(order_shipping)
    list = []
    order_shipping.order_items.each do |order_item|
      product = order_item.product_sku.product
      list << product.origin_name if product
    end
    return list.compact.sort.uniq
  end

  #
  # 商品名のリストを生成する
  #
  def product_list
    Product.find_product_list_name_and_code.map do |product_hash|
      [ "%s （%s）" %
          [product_hash['order_product_name_1'],
           locale_for_display(product_hash['locale'])],
        product_hash['product_code']]
    end
  end

  #
  # バリデーションエラーメッセージ
  # メッセージの配列を返します。
  #
  def template_error_messages_for_order_item(order_items, error_info = "")
    error_messages = []
    unless error_info.blank?
      error_messages << "<li>#{error_info}</li>"
    end
    order_items.each_with_index do |order_item, index|
      count = 0
      count += order_item.errors.count
      unless count.zero?
        error_messages << order_item.errors.full_messages.map {|msg| content_tag(:li, "注文商品#{index + 1}:" + msg) }
        
      end
    end
    return error_messages
  end

  # 注文ステータスが選択されているかどうかで異なるHTML要素のクラス名を返します。
  # 注文の検索条件オブジェクト@conditionの検索対象となる注文ステータスに引数のステータス(のキー)が
  # 含まれていたら"selected"、含まれていなかったら"unselected"を返します。
  # 注文一覧の注文ステータスのタブで使用します。
  def status_selected_class_name(status_div_key)
    @condition.selected_tab_keys.include?(status_div_key) ? 'selected' : 'unselected'
  end

  # 注文ステータスを一括で更新可能かどうかを返します
  def bulk_status_updatable?
    return false unless @condition.tab_inclusion_key == :selected_one
    status_entry = OrderShipping.status_div_entry_by_key(@condition.status_tab_key)
    return false unless status_entry
    return !!status_entry[:event_to_next]
  end

  # 注文ステータスのカラムを表示するかどうかを判断します。
  def show_status_column?
    return true if @condition.status_tab_entry[:show_status_column]
    @condition.tab_inclusion_key != :selected_one
  end
  
  # 適用済みキャンペーンの一覧を表示する。
  def print_campaigns(campaigns)
    result = ''
    campaigns.each do |campaign|
      result << "[#{h(campaign.code)}] #{h(campaign.name)}<br/>"
    end
    result
  end
end
