module TagReferencesHelper
  include EcRider::PluginModule::Extendable

  # 指定されたタグのdescriptionを返す
  def sorted_descriptions(tag_name)
    base_descriptions = []
    if EcRider::EcHelper.constants.include?(tag_name.to_s)
      base_descriptions = EcRider::EcHelper.const_get(tag_name).desc_sort
    end
    add_descriptions = []
    module_names = added_tag_helpers[tag_name.to_s] || []
    module_names.each do |module_name|
      add_descriptions = add_descriptions + module_name.constantize.desc_sort
    end
    return unique_descriptions(base_descriptions + add_descriptions)
  end

  # タグリファレンスのメニューリンクを表示する
  def tag_reference_menu_link(tag_name, title)
    "<li><a href=\"javascript:switch_tag('#{to_class_name(tag_name)}')\">#{title}</a></li>"
  end

  # タグリファレンスのメニュー（その他）を表示する
  # プラグインによる拡張用
  def render_tag_reference_menu_other
  end

  # タグリファレンスの本文（その他）を表示する
  # プラグインによる拡張用
  def render_tag_reference_other
  end

  # タグリファレンスの本文を表示する
  def render_reference(tag_name, comment='')
    result = ''
    result << "<div class=\"tag #{to_class_name(tag_name)}\">"
    result << '   <table class="standard_table">'
    result << '     <tbody>'
    sorted_descriptions(tag_name).each do |desc|
      if desc[1]
        result << '   <tr>'
        result << "     <th width=\"18%\">#{add_wbr_tag(desc[0])}</th>"
        result << "     <td><table class=\"description\">#{desc[1]}</table></td>"
        result << '   </tr>'
      end
    end
    result << '    </tbody>'
    result << '  </table>'
    result << comment
    result << '</div>'
  end

  # 後から読み込まれたモジュールを優先して重複するタグを取り除く
  def unique_descriptions(descriptions)
    result = []
    tag_names = []
    descriptions.reverse.each do |desc|
      next if tag_names.include?(desc.first)
      result << desc
      tag_names << desc.first
    end
    result.sort_by{|desc| desc.first.to_s }
  end

  private

  # PC用に拡張されたタグの一覧をハッシュで返す
  def added_tag_helpers
    return @added_tag_helpers if @added_tag_helpers
    @added_tag_helpers = {}
    EcRider::PluginModule::TagExtender.added_ec_helpers.each do |module_name|
      plugin_tag_name = /.*::([^:]+)/.match(module_name).to_a[1] #ヘルパー名だけ取得する
      plugin_tag_name.sub!('TagPc', 'Tag')
      @added_tag_helpers[plugin_tag_name] ||= []
      @added_tag_helpers[plugin_tag_name] << module_name
    end
    @added_tag_helpers
  end

  def to_class_name(tag_name)
    tag_name.to_s.tableize.gsub(/_tags/, '')
  end
end
