# -*- coding: utf-8 -*-
require 'nkf'
require 'cgi'
require 'iconv'
class AdminMailer < ActionMailer::Base

  @@default_charset = 'iso-2022-jp'
  @@encode_subject  = false

  #
  # = フリーメール送信 =
  #
  def self.send_free_mail(mail_type_div, from, to, subject, body, locale, user=nil, order=nil, inquiry_id=nil)
    return unless to #メールアドレスが登録されていなければメールを送信しない
    #フリーメール
    email = AdminMailer.deliver_free_mail(from, to, subject, body, locale)
    mail_address = UserEmail.find_by_email(to)
    user ||= mail_address.user unless mail_address.blank?
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(email, subject, body, (user.nil? ? nil : user.id ), mail_type_div)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id if order
    mail_delivery.inquiry_id = inquiry_id if inquiry_id
    mail_delivery.save!
  end

  #
  # = 配送指示メール送信 =
  #
  def self.send_shipment_mail(mail_type_div, from, to, subject, body, locale, user=nil, order=nil, file_content='')
    #添付ファイル付きフリーメール（出荷指示メール）
    email = AdminMailer.deliver_shipment_mail(from, to, subject, body, locale, 'text/html', file_content, 'invoice.html')
    mail_address = UserEmail.find_by_email(to)
    user ||= mail_address.user unless mail_address.blank?
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(email, subject, body, (user.nil? ? nil : user.id ), mail_type_div)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id if order
    mail_delivery.save!
  end

  #
  # = キャンセル処理完了メール送信 =
  #
  def self.send_cancel_complete_mail(order, email)
    return unless email # メールアドレスが登録されていなければメールを送信しない
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:cancel_complete, order.locale)
    user = User.find(order.user_id)
    options = {:order => order, :user => user}

    subject = template.get_mail_subject(pc_mobile_div, options)
    body = template.get_mail_body(pc_mobile_div, options)
    #キャンセル処理完了メール
    tmail = AdminMailer.deliver_cancel_complete(order, subject, body, template.locale, email)
    #メール配信履歴作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, order.user_id, :cancel_complete)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id
    mail_delivery.save!
  end

  #
  # = キャンセル処理完了メール =
  #
  def cancel_complete(order, subject, body, locale, email, sent_at = Time.now)
    @charset = locale_to_charset(locale)
    @subject    = subject
    @body       = {:body => body}
    @recipients = email
    @from       = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = フリーメール =
  #
  def free_mail(from, recipients, subject, body, locale, sent_at = Time.now)
    @charset = locale_to_charset(locale)
    @subject    = subject
    @body       = {:body => body}
    @recipients = recipients
    @from       = from
    @bcc        = from
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = 添付ファイル付きフリーメール =
  #
  def shipment_mail(from, recipients, subject, body, locale, file_content_type='text/plain', file_content='', filename='attachment', sent_at = Time.now)
    @charset = locale_to_charset(locale)
    @subject    = subject
    @body       = {:body => body}
    @recipients = recipients
    @from       = from
    @bcc        = from
    @sent_on    = sent_at
    @headers    = {}
    self.attachment(:content_type => file_content_type, :body => file_content, :filename => filename)
  end

  #
  # = プロジェクト確認、承認、差し戻しメール =
  #
  def project_approval_mail(project, comment, sent_at = Time.now)
    case project.status_key
      when :editing
        template = MailTemplate.find_valid_template(:project_passback)
      when :requesting_check
        template = MailTemplate.find_valid_template(:project_check_request)
      when :requesting_approval
        template = MailTemplate.find_valid_template(:project_approval_request)
    end

    options = {:project_code => project.project_code, :project_name => project.project_name, :approval_comment => comment,
      :project_period => "#{project.started_at.strftime('%Y/%m/%d  %H:%M')} ～ {project.finished_at.strftime('%Y/%m/%d  %H:%M')}"}

    @subject    = base64(template.get_mail_subject(nil, options))
    @body       = {:body => template.get_mail_body(nil, options)}
    @recipients = project.approval_mail_to
    @from       = ShopSetting[:shop_mail]
    @bcc        = nil
    @sent_on    = sent_at
    @headers    = {}
  end


  #
  # = サンクスメールの送信 =
  #
  def self.send_thanks_mail(user, order_id)
    return unless user.member_div_key == :membership

    order = Order.find(order_id)
    email = order.site_type_div_key == :pc ? user.user_email.email : user.user_email_mobile.email
    pc_mobile_div = UserEmail.check_by_emall email
    mail_type_div = nil
    order.order_shippings.each do |order_shipping|
      if order_shipping.payment_type_div_key == :bank_transfer
        #銀行振り込み決済が存在する場合は注文受付（銀行振り込み）メール
        mail_type_div = :thanks_bank_transfer
        break
      end
    end
    mail_type_div ||= :thanks
    template = MailTemplate.find_valid_template(mail_type_div)
    options = {:order => order, :user => order.user}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)

    #THANKSメールの送付
    thanks_tmail = AdminMailer.deliver_thanks(order_id, email)

    #THANKSメール配信情報作成
    thanks_mail_delivery = MailDelivery.create_front_mail_history(thanks_tmail, subject, body, user.id.to_s, :thanks)
    thanks_mail_delivery.site_div_key = :jisha
    thanks_mail_delivery.order_id = order_id
    thanks_mail_delivery.save!
  end

  #
  # = サンクスメール =
  #
  def thanks(order_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    order = Order.find(order_id)
    mail_type_div = nil
    order.order_shippings.each do |order_shipping|
      if order_shipping.payment_type_div_key == :bank_transfer
        #銀行振り込み決済が存在する場合は注文受付（銀行振り込み）メール
        mail_type_div = :thanks_bank_transfer
        break
      end
    end
    mail_type_div ||= :thanks
    template = MailTemplate.find_valid_template(mail_type_div)
    options = {:order => order, :user => order.user}
    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = ユーザ登録メールの送信 =
  #
  def self.send_signup_mail(user_id, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user)
    options = {:user => User.find(user_id.to_i), :email_address => email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body = template.get_mail_body(pc_mobile_div, options)
    #ユーザ登録メール
    tmail = AdminMailer.deliver_signup(user_id, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user_id, :regist_user)
    mail_delivery.site_div_key = :jisha
    mail_delivery.save!
  end

  #
  # = ユーザ登録メール =
  #
  def signup(user_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user)
    options = {:user => User.find(user_id.to_i), :email_address => email}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)

    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = 返品完了メール送信 =
  #
  def self.send_returned_mail(order, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:returned, order.locale)
    user = User.find(order.user_id)
    options = {:ec_order_chapter => order, :user => user}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body = template.get_mail_body(pc_mobile_div, options)
    #返信処理完了メール
    tmail = AdminMailer.deliver_returned(order, subject, body, template.locale, email)
    #メール配信履歴作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, order.user_id, :returned)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id
    mail_delivery.save!
  end

  #
  # = 返品完了メール =
  #
  def returned(order, subject, body, locale, email, sent_at = Time.now)
    @charset = locale_to_charset(locale)
    @subject    = subject
    @body       = {:body => body}
    @recipients = email
    @from       = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = order.test_flg ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  private
  #
  # = BASE64エンコード変換 =
  # 対応キャラクタセットはiso-2022-jp, iso-8859-1, GB2312, Big5。
  # エンコード失敗時に例外Iconv::Failureが発生します。
  #
  def base64(text, charset="iso-2022-jp", convert=true)
    charset = charset.downcase unless charset.nil?
    if convert
      if charset == 'iso-2022-jp' || charset == 'iso-8859-1' || charset == 'gb2312' || charset == 'big5'
        text = Iconv.conv(charset, 'utf-8', text)
      end
    end
    text = [text].pack('m').delete("\r\n")
    "=?#{charset}?B?#{text}?="
  end

  #
  # = ロケール文字列→キャラクタセット文字列変換 =
  #
  def locale_to_charset(locale)
    case locale
    when 'en'
      return 'iso-8859-1'
    when 'zh_CN'
      return "gb2312"
    when 'zh_TW'
      return "big5"
    else
      return "iso-2022-jp"
    end
  end

  #
  #
  # = 本文の文字コード変換 =
  # @localeの内容に応じて自動的に変換文字コードを判別します。
  # @localeがnilの場合はiso-2022-jpで送信します。（デフォルト動作）
  #
  #def create! (*)
  #  super
  #  @mail.subject = base64(@subject, charset)
  #  @mail.body = NKF::nkf('-j', @mail.body) or Iconv.conv(charset, 'UTF-8', part.body)
  #  return @mail
  #end
  def create! (*)
    super #@mailの生成
    part = @mail.parts.empty? ? @mail : @mail.parts.first
    retry_charset = 'utf-8' #エンコードに失敗した場合はUTF-8(無変換)で送信する
    begin #Subjectのエンコード
      @mail.subject = base64(@subject, @charset)
    rescue Iconv::Failure
      @charset = retry_charset
      retry
    end
    is_retry = false #本文エンコード失敗時にtrue
    begin #本文のエンコード
      if part.content_type == "text/plain"
        if @charset == 'utf-8'
          @mail.subject = base64(@subject, @charset, false) if is_retry
          part.charset = charset
        elsif @charset == 'iso-8859-1'
          part.charset = charset
          part.body = Iconv.conv(charset, 'utf-8', part.body)
          part.content_transfer_encoding = '7bit'
        elsif @charset == 'gb2312' || @charset == 'big5'
          part.charset = charset
          part.body = Iconv.conv(charset, 'utf-8', part.body)
        else
          #default
          part.charset = 'iso-2022-jp'
          part.body = Iconv.conv(charset, 'utf-8', part.body)
          part.content_transfer_encoding = '7bit'  # 申請の「請」などが無くなってしまう。
        end
      end
    rescue Iconv::Failure
      @charset = retry_charset
      is_retry = true
      retry
    end
    return @mail   # メソッドチェインを期待した変更があったら怖いので
  end

end
