# -*- coding: utf-8 -*-
class Accounting < ActiveRecord::Base
  include EcRider::PluginModule::Extendable

  belongs_to :order
  
  attr_accessor :order_accounting

  selectable_attr :debt_account_div do
    entry '01', :order_cash,        '注文（現金）', :code => 100, :output_name => "現金"
    entry '02', :order_receivable,  '注文（売掛金）', :code => 135, :output_name => "売掛金"
    entry '03', :cancel_cash,       'キャンセル（現金）', :code => 500, :output_name => "売上高" #経理データ出力時には「キャンセル（売上）」になる
    entry '04', :cancel_receivable, 'キャンセル（売掛金）', :code => 500, :output_name => "売上高" #経理データ出力時には「キャンセル（売上）」になる
    entry '05', :point_summary,     'ポイント集計', :code => 999, :output_name => "引当金繰入額"
    entry '06', :point_expired,     'ポイント失効'
    entry '07', :point_created,     'ポイント付与'
    entry '08', :point_updated,     'ポイント変更'
    entry '09', :point_destroyed,   'ポイント削除'
    entry '10', :pay_back_cash,   '返品（現金）', :code => 500, :output_name => "売上高"
    entry '11', :pay_back_receivable,   '返品（売掛金）', :code => 500, :output_name => "売上高"
    update_with_plugins(:Accounting, :added_entries_for_debt_account_div)
  end
  # credit_account_divをプラグインで拡張するためのメソッドです
  def self.added_entries_for_debt_account_div(mod); end

  selectable_attr :credit_account_div do
    entry '01', :order_cash,        '注文（現金）', :code => 500, :output_name => "売上高" #経理データ出力時には「注文（売上）」になる
    entry '02', :order_receivable,  '注文（売掛金）', :code => 500, :output_name => "売上高"#経理データ出力時には「注文（売上）」になる
    entry '03', :cancel_cash,       'キャンセル（現金）', :code => 100, :output_name => "現金"
    entry '04', :cancel_receivable, 'キャンセル（売掛金）', :code => 135, :output_name => "売掛金"
    entry '05', :point_summary,     'ポイント集計', :code => 999, :output_name => "ポイント引当金"
    entry '06', :point_expired,     'ポイント失効'
    entry '07', :point_created,     'ポイント付与'
    entry '08', :point_updated,     'ポイント変更'
    entry '09', :point_destroyed,   'ポイント削除'
    entry '10', :pay_back_cash,   '返品（現金）', :code => 100, :output_name => "現金"
    entry '11', :pay_back_receivable,   '返品（売掛金）', :code => 135, :output_name => "売掛金"
    update_with_plugins(:Accounting, :added_entries_for_credit_account_div)
  end
  # credit_account_divをプラグインで拡張するためのメソッドです
  def self.added_entries_for_credit_account_div(mod); end

  
  # 注文経理データ登録
  def self.create_for_order(order_shipping)
    return if order_shipping.order.test_flg
    account = Accounting.new(
      :order_id => order_shipping.order_id,
      :debt_sales => order_shipping.total_amount,
      :debt_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :order_cash : :order_receivable,
      :credit_sales => order_shipping.total_amount,
      :credit_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :order_cash : :order_receivable,
      :given_point => order_shipping.order.given_point,
      :used_point => order_shipping.order.used_point
    )
    user_point = UserPoint.find(:first, :conditions => ['order_id = ?', order_shipping.order.id],:order => "id desc")
    account.user_point_id = user_point.id unless user_point.nil?
    account.culc_previous_point
    account.save!
  end
  
  # キャンセル経理データ登録
  def self.create_for_cancel(order_shipping)
    return if order_shipping.order.test_flg
    account = Accounting.new(
      :order_id => order_shipping.order_id,
      :debt_sales => order_shipping.total_amount,
      :debt_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :cancel_cash : :cancel_receivable,
      :credit_sales => order_shipping.total_amount,
      :credit_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :cancel_cash : :cancel_receivable,
      :given_point => 0 - order_shipping.order.given_point,
      :used_point => 0 - order_shipping.order.used_point
    )
    user_point = UserPoint.find(:first, :conditions => ['order_id = ?', order_shipping.order.id],:order => "id desc")
    account.user_point_id = user_point.id unless user_point.nil?
    account.culc_previous_point
    account.save!
  end
  
  # ポイント集計経理データ登録
  def self.create_for_point_summary
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg
    account = Accounting.new(
      :debt_account_div_key => :point_summary,
      :credit_account_div_key => :point_summary,
      :given_point => 0,
      :used_point => 0
    )
    account.culc_previous_point(true)
    converted_price = account.previous_point * point_setting.exchange_rate
    account.debt_sales = converted_price
    account.credit_sales = converted_price
    account.save!
  end
  
  # ポイント期限切れ経理データ登録
  def self.create_for_point_expire(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg && find_order_accountings(user_point.order_id)
    account = Accounting.new(
      :user_point_id => user_point.id,
      :debt_account_div_key => :point_expired,
      :credit_account_div_key => :point_expired,
      :given_point => user_point.given_point,
      :used_point => 0
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price.abs
    account.credit_sales = converted_price.abs
    account.save!
  end
  
  # ポイント追加経理データ登録
  def self.create_for_point_create(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg
    account = Accounting.new(
      :user_point_id => user_point.id,
      :debt_account_div_key => :point_created,
      :credit_account_div_key => :point_created,
      :given_point => user_point.given_point,
      :used_point => 0
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price.abs
    account.credit_sales = converted_price.abs
    account.save!
  end
  
  # ポイント変更経理データ登録
  def self.create_for_point_update(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg && accountings = find_order_accountings(user_point.order_id, user_point.id)
    # 今までのポイントから差分を引くため、すべての注文に紐づくaccountingデータからポイントを計上する
    before_given_point = 0
    before_used_point = 0
    accountings.each do |before_account|
      before_given_point = before_given_point + before_account.given_point
      before_used_point  = before_used_point + before_account.used_point
    end
    account = Accounting.new(
      :order_id => user_point.order_id,
      :user_point_id => user_point.id,
      :debt_account_div_key => :point_updated,
      :credit_account_div_key => :point_updated,
      :given_point => user_point.given_point - before_given_point,
      :used_point => user_point.used_point - before_used_point
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price
    account.credit_sales = converted_price
    account.save!
  end
  
  # ポイント削除経理データ登録
  #
  def self.create_for_point_destroy(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg && accountings = find_order_accountings(user_point.order_id, user_point.id)
    # 今までのポイントから差分を引くため、すべての注文に紐づくaccountingデータからポイントを計上する
    before_given_point = 0
    before_used_point = 0
    accountings.each do |before_account|
      before_given_point = before_given_point + before_account.given_point
      before_used_point  = before_used_point + before_account.used_point
    end
    account = Accounting.new(
      :order_id => user_point.order_id,
      :debt_account_div_key => :point_destroyed,
      :credit_account_div_key => :point_destroyed,
      :given_point => 0 - before_given_point,
      :used_point => 0 - before_used_point
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price
    account.credit_sales = converted_price
    account.save!
  end
  
  # 経理データ登録可能な場合はtrueを返します
  # 経理データが登録されていて経理ステータスが
  # :cancel_cash, :cancel_receivable, :point_expired, :point_destroyedを含ない場合と、
  # order_idに紐づく注文経理データがなく、user_point_idに紐づくデータがある場合(管理側でポイント新規追加したもの)
  # accountingを返却します
  def self.find_order_accountings(order_id, user_point_id = nil)
    accountings = Accounting.find(:all, :conditions => ["order_id = ?", order_id], :order => "id desc")
    if !accountings.empty? && ![:cancel_cash, :cancel_receivable, :point_expired, :point_destroyed].include?(accountings[0].debt_account_div_key)
      return accountings
    end
    return if user_point_id.nil?
    accountings = Accounting.find(:all, :conditions => ["user_point_id = ? AND order_id is null", user_point_id], :order => "id desc")
    if !accountings.empty?
      return accountings
    end
  end

  # 経理上のポイント残高をセットします
  def culc_previous_point(monthly = false)
    if monthly
      condition = ["debt_account_div != ? and created_at < ?", Accounting.debt_account_div_id_by_key(:point_summary), Time.now.beginning_of_month]
    else
      condition = ["debt_account_div != ?", Accounting.debt_account_div_id_by_key(:point_summary)]
    end
    if previous_accounting = Accounting.find(:first, :conditions => condition, :order => 'id desc')
      if previous_accounting.previous_point &&  previous_accounting.previous_point > 0
        last_point = previous_accounting.previous_point
      end
    end
    self.previous_point = (last_point || 0) + self.given_point - self.used_point
  end

  # 借方コードを返却します
  def debt_account_div_code
    return self.debt_account_div_entry[:code]
  end

  # 貸方コードを返却します
  def credit_account_div_code
    return self.credit_account_div_entry[:code]
  end

  # 借方名称を返却します
  def debt_account_div_output_name
    return self.debt_account_div_entry[:output_name]
  end

  # 貸方名称を返却します
  def credit_account_div_output_name
    return self.credit_account_div_entry[:output_name]
  end

  # 摘要を返却します
  def remark
    if self.debt_account_div_key == :point_summary
      return "ポイント集計"
    else
      # TODO 現在は注文と配送先が１対１のため配列の最初を返しています。
      # 複数配送先を対応する場合は作り変える必要があります。
      status_name = ""
      case self.order.order_shippings[0].status_div_key
      when :delivered, :deliver_advised
        status_name = "注文"
      when :canceled
        status_name = "キャンセル"
  #    when :canceled
  #      status_name = "返品"
      end
      return "#{self.order.ec_order_code}(#{status_name})"
    end
  end
  #
  # = 商品Q&A検索入力フォーム(管理画面用) =
  #
  class SearchInfo < EcRider::ConditionForm
    ATTRIBUTE_NAMES = [:start_date, :end_date, :none_output_data, :previous_point]
    attr_accessor *ATTRIBUTE_NAMES

    #
    # = 検索条件を作成します。 =
    #
    def to_find_options(options = nil)

      where = Condition.block do |c|
        c.filter(:ignore_blank)
        # 範囲日時(開始)の条件
        c.and("accountings.created_at", ">=", Time.parse(start_date + " 00:00:00")) unless start_date.nil? || start_date.empty?
        # 範囲日時(終了)の条件
        c.and("accountings.created_at", "<=", Time.parse(end_date + " 23:59:59")) unless end_date.nil? || end_date.empty?

        # 出力済みデータを表示しない
        c.and("accountings.output_data_flg", "!=", 1 ) if !none_output_data.nil? && none_output_data == "1"

        # ポイント集計を出力する
        # チェックがない場合はポイント集計を表示しない
        c.and("accountings.credit_account_div", "!=", Accounting.debt_account_div_id_by_key(:point_summary) ) if previous_point.nil? || previous_point == "0"
        # ポイント編集、追加、削除、有効期限切れは表示しない。
        c.and do |c1|
          c1.and("accountings.credit_account_div", "!=", Accounting.debt_account_div_id_by_key(:point_expired))
          c1.and("accountings.credit_account_div", "!=", Accounting.debt_account_div_id_by_key(:point_created))
          c1.and("accountings.credit_account_div", "!=", Accounting.debt_account_div_id_by_key(:point_updated))
          c1.and("accountings.credit_account_div", "!=", Accounting.debt_account_div_id_by_key(:point_destroyed))
        end
      end
      result = {:conditions => where}
      result.update(options) if options
      return result
    end
  end

end
