# -*- coding: utf-8 -*-
class BannerCondition < ActiveRecord::Base
  belongs_to :banner

  validates_presence_of :type_cd
  validates_presence_of :operator_cd
  validates_presence_of :value
  validates_length_of   :value,  :in => 0..30, :allow_nil => true
  validates_presence_of :weight_point
  validates_length_of   :weight_point,  :in => 0..11, :allow_nil => true
  validates_numericality_of :weight_point


  selectable_attr :type_cd do
    entry '01', :user_age       , '年齢'
    entry '02', :user_occupation, '職業'
    entry '03', :user_gender    , '性別'
    entry '04', :user_birthday  , '生年月日'
    entry '05', :user_prefecture, '都道府県'
    entry '06', :user_point     , '保有ポイント'
    entry '07', :user_country   , '国'
#    entry '08', :shipping_date  , '発送日', :accessor => ""
    entry '09', :system_date    , 'システム日付'
#    entry '10', :search_key_word    , '検索ワード'
    entry '11', :random    , 'ランダム表示'
    # entry '10', :whether        , '気象情報（天気）'
    # entry '11', :temperature    , '気象情報（気温）'
  end

  selectable_attr :operator_cd do
    entry '01', :lt     , 'より小さい', :ruby => "<"
    entry '02', :gt     , 'より大きい', :ruby => ">"
    entry '03', :lte    , '以下', :ruby => "<="
    entry '04', :gte    , '以上', :ruby => ">="
    entry '05', :eql    , '等しい', :ruby => "=="
    entry '06', :not_eql, '以外', :ruby => "!="
  end

  # 検索条件に一致する場合、重みポイントを返却します。
  def calc_weight_point(user)
    self.evaluation_banner_condition(user) ? self.weight_point : 0
  end

  # バナー条件評価
  def evaluation_banner_condition(user)
    case self.type_key
    when :user_age
      return false if user.birthday.nil?
      target = user.age.to_i
      value  = self.value.to_i
    when :user_occupation
      target = user.job_div
      value  = self.value || ""
    when :user_gender
      target = user.sex
      value  = self.value || ""
    when :user_birthday
      return false if user.birthday.nil?
      target = user.birthday.strftime('%Y/%m/%d')
      value  = self.value || ""
    when :user_prefecture
      return false if user.user_address.nil?
      target = user.user_address.address_1
      value  = self.value || ""
    when :user_point
      return false if user.id.nil?
      target = UserPoint.total_point_for_user(user.id).to_s
      value  = self.value || ""
    when :user_country
      return false if user.user_address.nil?
      target = user.user_address.country_code
      value  = self.value || ""
    when :system_date
      target = Date.today.strftime('%Y/%m/%d')
      value  = self.value || ""
    else
      target = ""
      value = ""
    end

    case self.operator_key
    when :lt
      return target < value
    when :gt
      return target > value
    when :lte
      return target <= value
    when :gte
      return target >= value
    when :eql
      return target == value
    when :not_eql
      return target != value
    end
  end

  def display_value_for_type
    case self.type_key
    when :user_occupation
      result = User.job_div_name_by_id(self.value)
    when :user_gender
      result = User.sex_name_by_id(self.value)
    when :user_country
      country = Country.find_by_code(self.value)
      result = country.name
    else
      result = self.value
    end
    return result
  end
end
