# -*- coding: utf-8 -*-
class CmsDeadLinkResource < ActiveRecord::Base
  include EcRider::I18nSupport
  
  belongs_to :cms_dead_link_check, :foreign_key=> "cms_dead_link_check_id"
  has_many :links, :class_name => 'CmsDeadLink', :foreign_key => 'cms_dead_link_resource_id', 
    :order => "cms_dead_links.url"
  belongs_to :cms_bind, :class_name => 'CmsBind', :foreign_key => 'cms_bind_id'
  belongs_to :cms_template, :class_name => 'CmsTemplate', :foreign_key => 'cms_template_id'
  belongs_to :cms_file, :class_name => 'CmsFile', :foreign_key => 'cms_file_id'
  belongs_to :product, :class_name => 'Product', :foreign_key => 'product_id'
  belongs_to :product_image, :class_name => 'ProductImage', :foreign_key => 'product_image_id'
  belongs_to :product_display, :class_name => 'ProductDisplay', :foreign_key => 'product_display_id'
  belongs_to :banner, :class_name => 'Banner', :foreign_key => 'banner_id'
  belongs_to :banner_display, :class_name => 'BannerDisplay', :foreign_key => 'banner_display_id'
  belongs_to :shop_setting, :class_name => 'ShopSetting', :foreign_key => 'shop_setting_id'
  
  selectable_attr :resource do
    entry '1', :cms_bind, 'ページ'
    entry '2', :cms_template, 'デフォルトテンプレート'
    entry '3', :cms_file, 'ファイル'
    entry '4', :product, '商品詳細URL'
    entry '5', :product_image, '商品画像'
    entry '6', :product_display, '商品表示'
    entry '7', :banner, 'バナー'
    entry '8', :banner_display, 'バナー表示'
    entry '9', :shop_setting, '店舗設定'
  end
  
  def resource_display
    case resource_key
      when :cms_bind
        title = cms_bind.cms_page.titles.select {|item| item.locale == 'ja' }.first
        "#{title.name}　（#{locale_for_display(locale)}）"
      when :cms_template
        "#{cms_template.name}　（#{locale_for_display(locale)}）"
      when :cms_file
        cms_file.public_file_path
      when :product
        display = product.product_displays.select {|item| item.locale == 'ja' }.first
        if display
        "商品コード：#{product.product_code}　商品名：#{display.product_name_1}"
        else
        "商品コード：#{product.product_code}"
        end
      when :product_image
        display = product_image.product.product_displays.select {|item| item.locale == 'ja' }.first
        "商品コード：#{product_image.product.product_code}　商品名：#{display.product_name_1}"
      when :product_display
        "商品コード：#{product_display.product.product_code}　商品名：#{product_display.product_name_1}" <<
          "　（#{locale_for_display(product_display.locale)}）"
      when :banner
        "バナーコード：#{banner.code}"
      when :banner_display
        "バナーコード：#{banner_display.banner.code}　（#{locale_for_display(banner_display.locale)}）"
      when :shop_setting
        setting_locale = shop_setting.locale.blank? ? 'ja' : shop_setting.locale
        "#{shop_setting.setting_div_name}　（#{locale_for_display(setting_locale)}）"
    end
  end
end
