class CsvFileDownloadHistory < ActiveRecord::Base
  belongs_to :pattern, :class_name => "CsvDownloadPattern", :foreign_key => 'csv_download_pattern_id'
  
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :outputed_at_from, :outputed_at_to, :file_name, :disp_count
    
    # 開始日の入力制限(yyyy/MM/dd)
    validates_format_of :outputed_at_from, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :outputed_at_from_unempty?

    # 終了日の入力制限(yyyy/MM/dd))
    validates_format_of :outputed_at_to,   :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :outputed_at_to_unempty?

    def outputed_at_from_unempty?; outputed_at_from && !outputed_at_from.empty?; end
    def outputed_at_to_unempty?; outputed_at_to && !outputed_at_to.empty?; end

    #
    # バリデーション
    #
    def validate
      # 日付の存在チェック
      if outputed_at_from_unempty?
        _date = outputed_at_from.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:outputed_at_from, :EM990007)
        end
      end
      if outputed_at_to_unempty?
        _date = outputed_at_to.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:outputed_at_to, :EM990007)
        end
      end
    end

    def create_condition(options = nil)
      Condition.block do |c|
        c.and "file_name", "like", "%#{file_name}%" unless file_name.blank?
        c.and "created_at", ">=", Time.parse(outputed_at_from + " 00:00:00") unless outputed_at_from.blank?
        c.and "created_at", "<=", Time.parse(outputed_at_to + " 23:59:59") unless outputed_at_to.blank?
      end
    end
  end
end
