# -*- coding: utf-8 -*-
require 'ec_rider/selectable_attr_ext'

class KbnCode < ActiveRecord::Base
  versioned
  acts_as_paranoid
  has_many :kbn_code_names, :class_name => 'KbnCodeName', :foreign_key => 'kbn_code_id', :dependent => :destroy

  # バリデーション
  validates_presence_of :kbn, :code
  validates_uniqueness_of :code, :scope => :kbn

  # 名称マスタ区分
  selectable_attr :name_div do
    entry '04', :member_category, '会員種別'
    entry '05', :blacklist, '注意顧客種別'
    entry '51', :job, '職業区分'
    entry '911', :reasons, '退会理由'
    entry '1001', :stockout_display, '在庫切れ時の表示'
  end

  #
  # 検索フォーム
  #
  class SearchInfo < ActiveForm
    attr_accessor :kbn, :kbn_name, :code, :name, :disp_count

    def create_condition
      Condition.block do |c|
        # 区分の条件
        c.and "kbn", "like", "%#{kbn}%" unless kbn.blank?
        # 区分名称の条件
        c.and "kbn_name", "like", "%#{kbn_name}%" unless kbn_name.blank?
        # コードの条件
        c.and "code", "like", "%#{code}%" unless code.blank?
        # コード名称の条件
        c.and "kbn_code_names.name", "like", "%#{name}%" unless name.blank?
        # 国コードの条件
        c.and "kbn_code_names.locale", "=", "ja"
      end
    end

    def to_paginate_options(options = {})
      result = {
        :page => nil,
        :per_page => per_page,
        :include => :kbn_code_names,
        :order => "kbn_codes.kbn, kbn_codes.code"
      }
      conditions = create_condition
      result[:conditions] = conditions if conditions
      result.update(options || {})
    end

    def per_page
      (disp_count.to_i == 0) ? 20 : disp_count
    end
  end

  def self.find_names_by_code(code)
    options = []
    kbns = self.find(:all, :conditions => ['kbn =? ', code])
    kbns.each do |kbn|
      kbn.kbn_code_names.each do |names|
        options << [names.name,kbn.code] if names.locale == 'ja'
      end
    end
    return options
  end
end
