# -*- coding: utf-8 -*-
class Product < ActiveRecord::Base

  #
  # 商品／商品SKU選択ダイアログで使用するフォーム
  #
  class DialogForm < EcRider::ConditionForm
    include SelectableAttr::Base

    # 検索には使用しませんが、画面の表示に使用します。
    attr_accessor :dialog_name
    attr_accessor :sku
    attr_accessor :present_only
    attr_accessor :product_ids, :product_sku_ids
    attr_accessor :hide_thumbnail # 検索には使用しませんが、画面の表示に使用します。
    attr_reader :selected_product_ids
    attr_reader :selected_product_sku_ids

    # 以下は検索には使用します
    attr_accessor :search_word

    selectable_attr :present_cd do
      entry '0', :all, "" # 指定なし（全ての商品）
      entry '1', :present_only, "プレゼント品のみ"
      entry '2', :except_present, "プレゼント品を除く"
    end

    def sku?
      ActiveRecord::ConnectionAdapters::Column.value_to_boolean(sku)
    end
    
    def present_only?
      ActiveRecord::ConnectionAdapters::Column.value_to_boolean(present_only)
    end

    # ビューでidが含まれているかどうかの判断する場合でも、nilかどうかの判断をしなくても良くなるように
    # 追加しましたが、self.attriburesが正しく取得できなくなったのでコメントアウトしています。
    #    def selected_product_ids
    #      @selected_product_ids ||= []
    #    end

    def selected_product_ids=(value)
      value = value.to_s.split(',').select{|val| !val.blank?} unless value.is_a?(Array)
      @selected_product_ids = value
    end

    # ビューでidが含まれているかどうかの判断する場合でも、nilかどうかの判断をしなくても良くなるように
    # 追加しましたが、self.attriburesが正しく取得できなくなったのでコメントアウトしています。
    #    def selected_product_sku_ids
    #      @selected_product_sku_ids ||= []
    #    end

    def selected_product_sku_ids=(value)
      value = value.to_s.split(',').select{|val| !val.blank?} unless value.is_a?(Array)
      @selected_product_sku_ids = value
    end

    def to_find_options(options = nil)
      joins = []
      where = Condition.block do |c|
        unless search_word.blank?
          c.and do |c1|
            c1.or("products.product_code", "like", "%#{search_word}%")
            c1.or("products.order_product_name_1", "like", "%#{search_word}%")
            c1.or("products.order_product_name_2", "like", "%#{search_word}%")
            c1.or("products.kana_product_name", "like", "%#{search_word}%")
          end
        end
        if present_only?
          c.and("products.present_flg", 1)
        else
          case present_key
          when :present_only
            c.and("products.present_flg", 1)
          when :except_present
            c.and do |c2|
              c2.or('products.present_flg', 0)
              c2.or('sql', 'products.present_flg IS NULL')
            end
          end
        end
      end
      #標準順
      order_condition = 'products.display_order is null, products.display_order, products.product_code'

      result = {}
      result[:conditions] = where unless where.blank?
      joins.uniq!
      result[:joins] = joins.join(" ") unless joins.empty?
      result[:order] = order_condition
      result.update(options) if options
      return result
    end
  end

end
