# -*- coding: utf-8 -*-
class Product < ActiveRecord::Base
  
  #
  # ec:fetch_productsで使用する検索オブジェクト
  #
  class FinderForm < ActiveForm
    attr_accessor :category1, :category2, :category3
    attr_accessor :name1, :name2
    attr_accessor :product_code
    attr_accessor :price

    alias_attribute :category_name_1, :category1
    alias_attribute :category_name_2, :category2
    alias_attribute :category_name_3, :category3

    alias_attribute :product_name_1, :name1
    alias_attribute :product_name_2, :name2

    alias_attribute :product_price, :price

    def to_find_options(options = nil)
      @wheres, @params, @joins = [], [], []
      build_for_category(1)
      build_for_category(2)
      build_for_category(3)
      build_product_name(1)
      build_product_name(2)
      build_product_code
      # build_price
      build_visible
      result = {}
      unless @wheres.empty?
        result[:conditions] = [@wheres.map{|w| "(#{w})"}.join(" AND ")] + @params
      end
      result[:joins] = @joins.join(" ") unless @joins.empty?
      result[:order] = 'products.display_order is null, products.display_order, products.product_code'
      result
    end

    private
    def build_for_category(index)
      attr = "category#{index}"
      value = send(attr)
      return if value.blank?
      column = "category_#{index}_id"
      name_div_key = "product_category_#{index}".to_sym
      table_alias = "nm4#{attr}"
      @joins << "inner join kbn_codes #{table_alias} on #{table_alias}.code = products.#{column}"
      @wheres << "#{table_alias}.kbn = ?"
      @params << KbnCode.name_div_id_by_key(name_div_key)
      add_build_string("#{table_alias}.kbn_name", value)
    end

    def build_product_name(index)
      value = send("name#{index}")
      return if value.blank?
      column = "product_name_#{index}"
      table_alias = "cond_product_displays"
      @joins << "inner join product_displays #{table_alias} on #{table_alias}.product_id = products.id"
      add_build_string("#{table_alias}.#{column}", value)
    end

    def build_product_code
      value = send(:product_code)
      return if value.blank?
      add_build_string("products.product_code", value)
    end

    #    def build_price
    #      value = send(:product_price)
    #      return if value.blank?
    #      add_build_integer("products.price", value)
    #    end

    def build_visible
      @wheres << "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))"
      @wheres << 'invisible_flg is null' unless EcRider.config.ec_test
      now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
      @params << now
      @params << now
    end

    private
    def add_build_string(column, values)
      wheres, params = [], []
      values.split(/,/).each do |value|
        value.strip!
        next if value.empty?
        if /^\=/ =~ value
          wheres << "#{column} = ?"
          params << value.gsub(/^\=/, '')
        else
          wheres << "#{column} like ?"
          val = (value !~ /^\*/ && value !~ /\*$/) ? "*#{value}*" : value
          params << val.gsub(/^\*|\*$/, '%')
        end
      end
      add_wheres_and_params(@wheres, @params, wheres, params, " OR ")
    end

    def add_build_integer(column, values)
      wheres, params = [], []
      values.split(/\,/).each do |value|
        value.gsub!(/\s/, '')
        ope_values = value.scan(/([\=\<\>]*)([\d]+)/)
        next if ope_values.empty?
        _wheres, _params = [], []
        ope_values.each do |(operator, value)|
          ope = ''
          ope = '<' if operator =~ /\</
          ope << '>' if operator =~ /\>/
          ope << '=' if operator =~ /\=/
          ope << '=' if operator.blank?
          _wheres << "#{column} #{ope} ?"
          _params << value.to_i
        end
        add_wheres_and_params(wheres, params, _wheres, _params, " AND ")
      end
      add_wheres_and_params(@wheres, @params, wheres, params, " OR ")
    end

    def add_wheres_and_params(dest_wheres, dest_params, src_wheres, src_params, connector)
      return if src_wheres.empty?
      if src_wheres.size > 1
        dest_wheres << src_wheres.map{|w| "(#{w})"}.join(connector)
      else
        dest_wheres << src_wheres.first
      end
      src_params.each{|param| dest_params << param }
    end

  end

end
