# -*- coding: utf-8 -*-
class ProductGroup < ActiveRecord::Base
  versioned

  validates_presence_of :name
  validates_numericality_of :priority, :only_integer =>true, :greater_than_or_equal_to => 0, :if => :priority

  has_many :skus, :class_name => "ProductGroupSku", :dependent => :destroy, :order => :product_sku_id
  accepts_nested_attributes_for :skus, :allow_destroy => true


  #
  # = 有効な関連商品グループSKU =
  #
  def availables
    # 商品SKUまたは商品自体が削除されているものは、除外しています。
    self.skus.delete_if{|sku| (sku.product_sku.nil? || sku.product_sku.product.nil?)}
  end

  # =SearchCondition=
  # 検索条件を保持するためのクラス
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :name
    attr_accessor :sku_code
    attr_accessor :product_name

    def to_find_options(options = nil)
      where = Condition.block do |c|
        c.filter(:ignore_blank)
        # 関連商品グループ名の条件
        c.and "product_groups.name", "like", "%#{name}%"
        # SKUコードの条件
        c.and "product_skus.sku_code", "like", "%#{sku_code}%"
        # 商品名の条件
        c.and "products.order_product_name_1", "like", "%#{product_name}%"
      end
      result = {:conditions => where}
      result[:order] = "products.display_order is null, products.display_order, products.product_code"#"product_groups.id desc"

      # 「SKUコード」もしくは「商品名」で検索する場合は、必要なテーブルを結合する
      # 「関連商品グループ名」で検索する場合は、テーブルの結合は不要です
      unless sku_code.blank? && product_name.blank?
        result[:select] = "product_groups.*, product_skus.sku_code, products.order_product_name_1"
        result[:joins] = <<-"LEFT_OUTER_JOIN"
          LEFT OUTER JOIN product_group_skus
          on product_groups.id = product_group_skus.product_group_id
          LEFT OUTER JOIN product_skus
          on product_skus.id = product_group_skus.product_sku_id
          LEFT OUTER JOIN products
          on products.id = product_skus.product_id
        LEFT_OUTER_JOIN
      end
      
      result.update(options) if options
      return result
    end
  end

end
