# -*- coding: utf-8 -*-
class ProductVariation < ActiveRecord::Base
  versioned

  include LocaleDisplayable

  has_many :displays, :class_name => "ProductVariationDisplay", :foreign_key => 'product_variation_id',
    :dependent => :destroy
  accepts_nested_attributes_for :displays

  has_many :items, :class_name => "ProductVariationItem", :foreign_key => 'product_variation_id',
    :dependent => :destroy
  accepts_nested_attributes_for :items

  validates_presence_of :name_for_admin
  validates_uniqueness_of :name_for_admin

  #
  # = [バリエーション名, バリエーションID]の配列を返す =
  #
  def self.options
    _options = []
    find(:all, :order => 'id').each do |variaion|
      _options << [variaion.name_for_admin, variaion.id]
    end
    return _options
  end

  # =バリエーションが削除可能かどうかを判定する
  def editable_or_deletable_variation?
    result = Product.find(:all, 
      :conditions => ["variation_1_id = ? or variation_2_id = ? or variation_3_id = ?", self.id, self.id, self.id ]
    )
    return result.empty?
  end

  #
  # =一覧の検索条件=
  #
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :variation_name, :variation_item

    def to_find_options(options = nil)
      joins = []
      where = Condition.block do |c|
        c.and "product_variations.name_for_admin", "like", "%#{variation_name}%" unless variation_name.blank?
        c.and "product_variation_items.code", "like", "%#{variation_item}%" unless variation_item.blank?
      end
      result = {:conditions => where}
      result[:joins] = joins.uniq.join(" ")
      result.update(options) if options
      return result
    end
  end

end
