# -*- coding: utf-8 -*-
class ProductVariationItem < ActiveRecord::Base
  versioned

  include LocaleDisplayable

  has_many :displays, :class_name => "ProductVariationItemDisplay", :foreign_key => 'product_variation_item_id',
    :dependent => :destroy
  accepts_nested_attributes_for :displays

  belongs_to :product_variation, :class_name => 'ProductVariation'

  # 表示順、コードの必須チェック
  validates_presence_of :display_order
  
  # コードの一意チェック
  validates_uniqueness_of :code, :scope => :product_variation_id

  selectable_attr :code do
    ProductVariationItem.all.each do |variation_item|
      entry variation_item.id.to_s, nil, variation_item.code
    end
  end

  # 入力チェック
  def validate
    # コードは「00」以外の英数2桁で必須
    if (self.code.blank? || self.code.length != 2 || self.code == "00" || self.code =~ /[^\w\w]/)
      errors.add(:code, 'は「00」以外の英数2文字で入力してください。')
    end
  end

  # 商品バリエーション項目の管理用名称は、表示用名称の日本語の名称を使用します。
  def name_for_admin
    unless new_record?
      ja_display = displays.find(:first, :conditions => "locale = 'ja'")
      return ja_display.name if ja_display
    end
    displays.each do |display|
      return display.name if display.locale == 'ja'
    end
    nil
  end
  
  # バリエーションIDとバリエーション項目コードからバリエーション項目を返す
  def self.find_by_variation_id_and_code(variation_id, variation_item_code)
    return nil unless variation_id && variation_item_code
    ProductVariationItem.find(:first, :conditions => ["product_variation_id = ? AND code = ?", variation_id, variation_item_code])
  end
  
  # 渡されたidから名称とコードの配列を返す
  def self.options_by_ids(ids)
    where = 'product_variation_items.id in (?) AND product_variation_item_displays.locale = ?'
    order = 'product_variation_items.display_order, product_variation_items.id'
    items = ProductVariationItem.find(:all, :conditions => [where, ids, I18n.locale.to_s], :order => order, :include => :displays)
    items.map{|item| [item.displays.first.name, item.code] }
  end
end
