# -*- coding: utf-8 -*-
class ReminderForm < ActiveForm
  include ActiveRecord::Validations

  attr_accessor :email, :remember_question, :remember_answer,  :password, :password2, :back_to_url, :completion_url_controller, :completion_url_action

  #未入力バリデーション
  validates_presence_of :email, :message => :EM990001
  validates_presence_of :remember_question, :message => :EM990007
  validates_presence_of :remember_answer, :message => :EM990007

  #文字数バリデーション
  validates_length_of      :password,              :minimum => 4, :message => :EM990009
  validates_length_of      :password_confirmation, :minimum => 4, :message => :EM990009

  #パスワード確認
  validates_confirmation_of :password,              :message => :EM990008
  # パスワード複雑さバリデーション
  validates_format_of :password, :with => /^[a-zA-Z\d]*([a-zA-Z]\d|\d[a-zA-Z])[a-zA-Z\d]*$/, :message => :EM990042
  
  #メールアドレスフォーマット
  validates_format_of      :email,  :with => /^([a-zA-Z0-9])+([a-zA-Z0-9\._-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/, :message => :EM990007, :if => :email_check?

  # メールアドレスが入力されていたらtrueを返す
  def email_check?
    return !email.blank?
  end
  
  # 完了後の遷移先がある場合フォームにセットします。
  def set_completion_url(url)
    unless url.nil?
      unless (url[:controller].nil? || url[:action].nil?) || (url[:controller].empty? || url[:action].empty?)
        self.completion_url_controller = url[:controller]
        self.completion_url_action = url[:action]
      end
    end
  end
  # 完了後の遷移先がある場合trueを返却します。
  def other_redirect?
    unless (self.completion_url_controller.nil? || self.completion_url_action.nil?)
      unless (self.completion_url_controller.empty? || self.completion_url_action.empty?)
        return true
      end
    end
    return false
  end

  # 秘密の質問をメールアドレスから取得
  def set_remember_question
    user = User.find(:first,
      :joins => "LEFT OUTER JOIN user_emails ON user_emails.user_id = users.id  AND (user_emails.deleted_at IS NULL OR user_emails.deleted_at > '#{Time.now.strftime('%Y-%m-%d %H:%M:%S')}')" ,
      :conditions => ["user_emails.email = ? and member_div = ?",  self.email, User.member_div_id_by_key(:membership)]
    )
    self.remember_question = user.remember_question
  end
end
