# -*- coding: utf-8 -*-
class ShippingCostBoundary < ActiveRecord::Base
  versioned

  has_many :shipping_costs, :dependent => :destroy
  accepts_nested_attributes_for :shipping_costs

  validates_numericality_of :point, :only_integer => true, :allow_nil => true

  # 配列で渡された閾値を昇順に並び替えます。
  def self.sort_boundaries(input_boundary_points = [])
    input_boundary_points = input_boundary_points.reject{|x| x.blank?}
    input_boundary_points.collect! {|point| point.to_i}
    input_boundary_points.uniq!
    input_boundary_points.sort!
    return input_boundary_points
  end

  # 入力された閾値と現在の閾値に同様の数値があった場合
  # その閾値マスタのIDを配列で返却します。
  def self.get_boundary_ids(points = [])
    ids = []
    points.each do |point|
      boundary = ShippingCostBoundary.find(:first, :conditions =>["point = ? ", point])
      if boundary.nil?
        ids << ""
      else
        ids << boundary.id
      end
    end
    # 閾値設定値以上の値を取得
    boundary = ShippingCostBoundary.find(:first, :conditions =>"point is null")
    ids << boundary.id unless boundary.nil?
    return ids
  end

  # 閾値マスタを渡された閾値に置き換え、
  # 渡されたshipping_costsとともにsaveします。
  def self.replace_boundary_points(points = [], shipping_costs = [])
    shipping_cost_boundaries = self.find(:all, :order => 'id')
    new_shipping_cost_boundaries = []
    points.each_with_index do |point, index|
      if shipping_cost_boundaries[index].nil?
        new_shipping_cost_boundaries << self.new(:point => point)
      else
        shipping_cost_boundaries[index].point = point
        new_shipping_cost_boundaries << shipping_cost_boundaries[index]
      end
    end
    # 前の閾値の方が多い場合、削除する
    if shipping_cost_boundaries.size > points.size
      shipping_cost_boundaries.each_with_index do |boundary, index|
        if index + 1 > points.size
          boundary.destroy
        end
      end
    end
    # 送料を紐付けて閾値マスタをsave
    ShippingCost.destroy_all
    shipping_costs.each do |shipping_cost|
      new_shipping_cost_boundaries[shipping_cost.boundary_index.to_i].shipping_costs << shipping_cost
    end
    new_shipping_cost_boundaries.each do |new_boundary|
      new_boundary.save!
    end
  end
end
