class UserRankCondition < ActiveRecord::Base
  selectable_attr :type_cd do
    entry '01', :user_age,            '年齢',                 :check_method => "age"
    entry '02', :order_amount,        '購入金額',             :check_method => "total_purchase_for_distance"
    entry '03', :order_count,         '購入回数',             :check_method => "order_count_for_distance"
    entry '04', :member_elapsed_days, '会員登録からの日数',   :check_method => "count_day_of_member_created"
    entry '05', :order_elapsed_days,  '最終注文日からの日数', :check_method => "count_day_of_ordered"
  end

  selectable_attr :operator_cd do
    entry '01', :lt     , 'より小さい', :ruby => "<"
    entry '02', :gt     , 'より大きい', :ruby => ">"
    entry '03', :lte    , '以下',       :ruby => "<="
    entry '04', :gte    , '以上',       :ruby => ">="
    entry '05', :eql    , '等しい',     :ruby => "=="
    entry '06', :not_eql, '以外',       :ruby => "!="
  end

  validates_presence_of :type_cd, :operator_cd, :value, :if => :not_all_inputed?
  validates_numericality_of :value, :only_integer => true, :allow_nil => true, :if => :not_all_inputed?
  validates_length_of   :value, :maximum => 120, :allow_nil => true

  # :type_cd, :operator_cd, :valueがすべて未入力の場合falseを返します。
  def not_all_inputed?
    unless (type_cd.nil? || type_cd.blank?) && (operator_cd.nil? || operator_cd.blank?) && (value.nil? || value.blank?)
      return true
    end
    return false
  end
  
  # 評価用メソッド
  # user_value(例:３) operator(例:>)  value(例：４)の場合
  # 3 > 4　の条件式で評価します。
  def judge(user)
    return true unless not_all_inputed?
    user_value = user.send(self.type_entry[:check_method]) # 各type_cd会員の値を取得します。
    operator = self.operator_entry[:ruby] # ruby演算子を取得
    return user_value.to_i.send(operator, self.value.to_i) unless self.operator_key == :not_eql
    return user_value.to_i != self.value.to_i # !=はsendができないので別定義しています。
  end
end
