# -*- coding: utf-8 -*-
class CmsDefaultTemplate < ActiveRecord::Base
  belongs_to :cms_project, :foreign_key=> "cms_project_id"

  selectable_attr :default_template_position do
    entry '0', :top, '上', :mobile => true, :require => true
    entry '1', :bottom, '下', :mobile => true, :require => true
    entry '2', :left, '左'
    entry '3', :right, '右'
  end

  selectable_attr :terminal do
    entry '0', :pc, 'PC'
    entry '1', :mobile, '携帯'
  end

  # 指定されたプロジェクトが公開された際に使用されるデフォルトテンプレート情報を返す
  def self.primary_default_template_codes(project_ids, preferred_project_id = nil)
    query = <<-"EOS"
      SELECT d.*
      FROM cms_default_templates d
      JOIN cms_projects p ON p.id = d.cms_project_id AND p.id IN (?)
      WHERE NOT EXISTS (
        SELECT *
        FROM cms_default_templates sd
        JOIN cms_projects sp ON sp.id = sd.cms_project_id AND sp.id IN (?)
        WHERE (
          sp.priority > p.priority
          OR (sp.priority = p.priority AND sp.started_at > p.started_at)
          OR (sp.priority = p.priority AND sp.started_at = p.started_at AND sp.id > p.id)
        ) AND sd.default_template_position = d.default_template_position AND sd.terminal = d.terminal
      )
    EOS

    default_templates = CmsDefaultTemplate.find_by_sql([query, project_ids, project_ids])
    default_template_codes = CmsDefaultTemplateList.new(default_templates)

    # 優先されるプロジェクト（preferred_project_id）でデフォルトテンプレートが指定されていればそれを使う
    if preferred_project_id
      preferred_default_templates = CmsDefaultTemplate.find(:all, :conditions => ["cms_project_id = ?", preferred_project_id])
      default_template_codes.overwrite(preferred_default_templates)
    end
    default_template_codes
  end

  def self.update_default_template_position(project_id, template_code, terminal, current_position, new_position)
    current_entry = CmsDefaultTemplate.default_template_position_entry_by_id(current_position)
    new_entry = CmsDefaultTemplate.default_template_position_entry_by_id(new_position)
    # 変更していない場合（何もしない）
    if current_entry[:key] == new_entry[:key]
      return
    # 『上下』から変更した場合
    elsif current_entry[:require]
      create_or_update_default_template(project_id, terminal, current_entry[:id], template_code)
    # 『指定なし』から変更した場合
    elsif !current_entry[:key]
      create_or_update_default_template(project_id, terminal, new_entry[:id], template_code)
    # 『左右』から『上下左右』に変更した場合
    elsif !current_entry[:require] && new_entry[:key]
      # 変更前のデフォルトテンプレート情報を削除する
      attributes = {:cms_project_id => project_id, :terminal => terminal, :default_template_position => current_entry[:id]}
      default_template = CmsDefaultTemplate.find(:first, :conditions => attributes)
      default_template.destroy
      # 新しいデフォルトテンプレート情報を登録する
      create_or_update_default_template(project_id, terminal, new_entry[:id], template_code)
    # 『左右』から『指定なし』に変更した場合
    elsif !current_entry[:require] && !new_entry[:key]
      create_or_update_default_template(project_id, terminal, current_entry[:id], '')
    end
  end

  def self.create_or_update_default_template(project_id, terminal, default_template_position, template_code)
    attributes = {:cms_project_id => project_id, :terminal => terminal, :default_template_position => default_template_position}
    default_template = CmsDefaultTemplate.find(:first, :conditions => attributes)
    default_template ||= CmsDefaultTemplate.new(attributes)
    default_template.cms_template_code = template_code
    default_template.save!
  end

  class CmsDefaultTemplateList
    attr_accessor :default_template_hash
    def initialize(default_templates)
      overwrite(default_templates)
    end
    def overwrite(default_templates)
      @default_template_hash ||= {}
      default_templates.each do |template|
        next unless template.default_template_position_key
        @default_template_hash[template.terminal_key] ||= {}
        @default_template_hash[template.terminal_key][template.default_template_position_key] = template.cms_template_code
      end
    end
    def values
      @default_template_hash.values.map{|positions_hash| positions_hash.values }.flatten
    end
    def position_key(template_code, terminal=:pc)
      @default_template_hash[terminal].index(template_code)
    end
    def position_id(template_code, terminal=:pc)
      key = position_key(template_code, terminal)
      return nil unless key
      CmsDefaultTemplate.default_template_position_id_by_key(key)
    end
    def position_name(template_code, terminal=:pc)
      key = position_key(template_code, terminal)
      return nil unless key
      CmsDefaultTemplate.default_template_position_name_by_key(key)
    end
    def editable?(template_code, terminal=:pc)
      key = position_key(template_code, terminal)
      return false if key && CmsDefaultTemplate.default_template_position_entry_by_key(key)[:require]
      return true
    end
  end
end
