# -*- coding: utf-8 -*-
require 'goldberg'

module Goldberg
  class Role < ActiveRecord::Base
    versioned

    include Goldberg::Model

    has_many :users, :class_name => 'Goldberg::User'

    validates_presence_of :name
    validates_uniqueness_of :name

    has_many :role_actions, :class_name => "Goldberg::RoleAction", :foreign_key => 'role_id'
    has_many :actions, :through => :role_actions

    def has_users?
      self.users.count > 0
    end

    def can_access?(action_or_action_id)
      return false if action_or_action_id.nil?
      action_id = action_or_action_id.respond_to?(:new_record?) ? action_or_action_id.id : action_or_action_id.to_i
      @allowed_action_ids ||= self.role_actions.map{|ra| ra.site_action_id}
      @allowed_action_ids.include?(action_id)
    end


    #
    # = 権限検索の入力フォーム =
    #
    class SearchCondition < EcRider::ConditionForm
      attr_accessor :name, :disp_count
      
      def to_find_options(options = nil)
        where = Condition.block do |c|
          c.filter(:ignore_blank)
          # 権限名の条件
          c.and "goldberg_roles.name", "like", "%#{name}%" unless name.blank?
        end
        result = {:conditions => where}
        result.update(options) if options
        return result
      end
    end

  end
end
