# -*- coding: utf-8 -*-
require 'fileutils'

module Goldberg
  class SiteActionGroup < ActiveRecord::Base
    include Goldberg::Model

    has_many :actions, :class_name => 'Goldberg::SiteAction', 
      :foreign_key => 'site_action_group_id',
      :order => 'name', :dependent => :destroy
    
    validates_presence_of :name

    def allow_update?
      # goldberg関係は変更不可です。
      /^goldberg\// !~ name.to_s
    end

    def base_name
      display_name.blank? ? nil : display_name.sub(/管理$/, '')
    end



    def [](action_name)
      return nil if action_name.blank?
      named_actions_cache[action_name.to_s]
    end

    def categorised_actions
      unless @categorised_actions
        @categorised_actions = HashWithIndifferentAccess.new
        self.actions.each do |action|
          values = @categorised_actions[action.category_name.to_s] ||= []
          values << action
        end
      end
      @categorised_actions
    end

    def categorised_actions
      categorised_actions_cache
    end

    def has_category?(category_name)
      !categorised_actions_cache[category_name].empty?
    end
    
    
    private

    def named_actions_cache
      unless @named_actions_cache
        @named_actions_cache = {}
        self.actions.each{|action|@named_actions_cache[action.name.to_s] = action}
      end
      @named_actions_cache
    end
    
    # def clear_actions_cache
    #   @named_actions_cache = nil
    # end

    def categorised_actions_cache
      unless @categorised_actions
        @categorised_actions = HashWithIndifferentAccess.new
        Goldberg::SiteAction.category_name_ids.each do |category|
          @categorised_actions[category] = []
        end
        self.actions.each do |action|
          key = action.category_name.to_s
          key = 'other' unless Goldberg::SiteAction.category_name_ids.include?(key)
          @categorised_actions[key] << action
        end
      end
      @categorised_actions
    end

  end
end
