# -*- coding: utf-8 -*-
$ecr_application_name = "ec_admin".freeze
$ecr_application_name_in_repository = $ecr_application_name

# config/deploy_common.rbの読み込み
# requireを使うとスクリプトを実行するコンテキストが変わってしまうのでNGです。
deploy_common = File.join(File.dirname(__FILE__), 'deploy', 'deploy_common.rb')
instance_eval(IO.read(deploy_common), deploy_common, __LINE__)

#
# ec_admin 固有の設定など
#

namespace :mongrel do
  desc "Setup mongrel_cluster.yml to /etc/mongrel_cluster"
  task :copy_mongrel_cluster_yml do
    sudo "mkdir -p /etc/mongrel_cluster"
    sudo "cp config/mongrel_cluster_#{$ecr_application_name}.yml /etc/mongrel_cluster/#{$ecr_application_name}.yml"
  end
end
after "deploy:setup", "mongrel:copy_mongrel_cluster_yml"

namespace :cms do
  desc "Setup shared dir"
  task :setup_shared_dir do
    # product_images_for_previewを使う機能は未実装なのでコメントアウトします。
    #   mkdir -p #{shared_path}/public/ec_rider/product_images_for_preview && 
    run <<-EOS
      mkdir -p #{shared_path}/public/ec_rider/inquiry_images && 
      mkdir -p #{shared_path}/public/projects && 
      mkdir -p #{shared_path}/tmp/csv_file_download &&
      mkdir -p #{shared_path}/tmp/csv_file_upload &&
      mkdir -p #{shared_path}/tmp/product_image_import
    EOS
  end


  desc "Make symlinks"
  task :symlinks do
    run(<<-"EOS")
      if [ -d #{shared_path}/public/projects/1 ]; then
        echo "Initial project 1 already exist."
      else
        echo "Initial project doesn't exist, so execute: mv #{release_path}/public/projects/1 #{shared_path}/public/projects."
        mkdir -p #{shared_path}/public/projects
        mv #{release_path}/public/projects/1 #{shared_path}/public/projects
      fi
    EOS
    rm_rf_and_ln_nfs([ # 順番を保持するために一つのHashにまとめないでください。
      # banner_imagesはec_adminとec_frontでパスが異なるので注意
      {"$ECR_CMS_GIT_DIR/public/banner_images"  => "#{release_path}/public/ec_rider/banner_images"},
      # product_imagesはプラグインの制約でpublicに対してec_adminもec_frontお同じパスでなければなりません。
      {"$ECR_CMS_GIT_DIR/public/ec_rider/product_images" => "#{release_path}/public/ec_rider/product_images"},
      {"#{shared_path}/public/ec_rider/inquiry_images"   => "#{release_path}/public/ec_rider/inquiry_images"},
      {"#{shared_path}/public/projects"                  => "#{release_path}/public/projects"},
      {"#{shared_path}/tmp/csv_file_download"            => "#{release_path}/tmp/csv_file_download"},
      {"#{shared_path}/tmp/csv_file_upload"              => "#{release_path}/tmp/csv_file_upload"},
      {"#{shared_path}/tmp/product_image_import"         => "#{release_path}/tmp/product_image_import"},
    ])
  end

end

after "deploy:setup", "cms:setup_shared_dir"
after "deploy:update_code", "cms:symlinks"
