# -*- coding: utf-8 -*-
require 'net/http'

class Ec2Metadata
  class ResponseError < StandardError
  end
  class InvalidKey < StandardError
  end

  KEYS = %w(ami-id ami-launch-index instance-id hostname local-hostname local-ipv4 public-hostname public-ipv4 security-groups)

  HOST = '169.254.169.254'

  def connect
    result = nil
    Net::HTTP.start(HOST, 80) do |http|
      result = yield(http)
    end
    result
  end

  def [](key)
    actual_key = key.to_s.gsub(/_/, '-')
    raise InvalidKey, "Unsupported key: #{key.inspect}" unless KEYS.include?(actual_key)
    connect{|http| get(http, actual_key)}
  end

  KEYS.each do |key|
    class_eval(<<-"EOS")
      def #{key.gsub(/-/, '_')}
        self[:#{key}]
      end
    EOS
  end

  def to_hash
    connect do |http|
      KEYS.inject({}) do |d, key|
        d[key] = self[key]
        d
      end
    end
  end

  private

  def get(http, key)
    res = http.get("/latest/meta-data/#{key}")
    case res.code
    when /20\d/ then
      s = res.body
    else
      raise ResponseError, "#{res.code}: #{res.message}"
    end
  end


  # ローカルの特定のURLでEC2のメタ情報を取得します
  def ec2_metadata
    result = {}
    Net::HTTP.start('169.254.169.254', 80) {|http|
      KEYS.each do |meta_data_key|
        # リクエストはハイフン区切りですが、内部ではアンダースコア区切りを使っています。
        # その方がRails的には扱いやすいので。
        # puts "GET http://169.254.169.254/latest/meta-data/#{meta_data_key} ..."
        response = http.get("/latest/meta-data/#{meta_data_key}")
        s = response.body
        s = nil if (/404/m =~ s && /Not Found/ =~ s)
        result[meta_data_key.gsub(/-/, '_').to_sym] = s
      end
    }
    # puts YAML.dump(result)
    return result
  end

end
