#!/usr/bin/env ruby
# -*- coding: utf-8 -*-
KCODE="u"

require 'erb'
require 'fileutils'
require 'yaml'
require 'optparse'

opt = OptionParser.new(<<EOS)
Usage: ruby zabbix-host-registrar.rb [options]
EOS

arg_opts = {
  :config => File.join(File.dirname(__FILE__), 'zabbix-register.yml')
}

opt.separator("  Command options")
opt.on('-A', '--add-host'   ){ arg_opts[:command] = :add_host }
opt.on('-R', '--remove-host'){ arg_opts[:command] = :remove_host }
opt.on('-S', '--show-config'){ arg_opts[:command] = :show_config }
opt.on('-U', '--update-agent-config'){ arg_opts[:command] = :update_agent_config }

opt.separator("  General options")
opt.on('-d', '--debug') {|v| arg_opts[:debug] = true}
opt.on('-c', "--config=#{arg_opts[:config]}") {|v| arg_opts[:config] = v }

opt.parse!(ARGV)

# コマンドが指定されてなかったらヘルプを表示して抜けます
unless arg_opts[:command]
  puts opt.help
  exit(0)
end

require 'logger'
require File.join(File.dirname(__FILE__), 'ec2_metadata')
require File.join(File.dirname(__FILE__), 'zabbix_frontend_client')

class ZabbixAutoRegister

  def initialize(optinos = {})
    @config_filepath = optinos[:config]
    @config = load_config(@config_filepath)
    @debug = optinos[:debug] || @config[:debug]
    @server_config = @config[:server]
    @registration_config = @config[:registration]
    if log_filepath = @config[:log_filepath]
      @logger = Logger.new(log_filepath)
    end
  end

  def execute(command)
    @client = ZabbixFrontendClient.new(
      @server_config['url'],
      @server_config['username'],
      @server_config['password'],
      { :debug => @debug,
        :logger => @logger,
        :server_version => @server_config['version'],
      })
    send(command)
  end

  private

  def load_config(filepath)
    config = YAML.load(load_config_file(filepath))
    config.inject({}){|dest, (key, value)| dest[key.to_sym] = value; dest}
  end

  def load_config_file(filepath)
    puts "configuration loading from #{filepath}"
    # ERB内で使用可能な変数群
    ec2_metadata = Ec2Metadata.new
    ec2_default_display_name = ec2_security_group_and_instnance_id(ec2_metadata) 
    #
    ERB.new(IO.read(filepath)).result(binding)
  end
  
  def show_config
    puts YAML.dump(@config).gsub(/^/, '|')
  end

  def add_host
    @client.login
    @client.index
    @client.add_host(@registration_config)
  end

  def remove_host
    @client.login
    @client.index
    @client.hosts
    @client.remove_host(@registration_config)
  end

  ZABBIX_AGENT_CONFIGS = %w(/etc/zabbix/zabbix_agent.conf /etc/zabbix/zabbix_agentd.conf)

  def update_agent_config
    visibility = @config[:visibility]
    server_ipv4 = @server_config["#{visibility}_ipv4"]
    hostname = @registration_config['host']
    ZABBIX_AGENT_CONFIGS.each do |config_filepath|
      FileUtils.cp(config_filepath, "%s.%s" % [config_filepath, Time.now.strftime("%Y%m%d-%H%M%S")])
      config = IO.read(config_filepath)
      replaced = config.
        gsub(/^Server\=(.*?)$/){ "# Server=#{$1}\nServer=#{server_ipv4}" }.
        gsub(/^Hostname\=(.*?)$/){ "# Hostname=#{$1}\nHostname=#{hostname}" }
      File.open(config_filepath, "w"){|f| f.puts(replaced)}
    end
  end

  def ec2_security_group_and_instnance_id(d)
    group_names = (d['security-groups'] || '').split(/$/).
      map{|s| s.strip }.compact.reject{|s| s == 'default'}.sort.join("_")
    "#{d['instance-id']}_#{group_names}".gsub(/[\-\.\s\n]/, '_')[0, 30]
  end
  
end

register = ZabbixAutoRegister.new(arg_opts)
register.execute(arg_opts[:command])
