# -*- coding: utf-8 -*-
class SchemaMigration < ActiveRecord::Base ; end
#
# = STEP2との統合を行うマイグレーションファイルです。 =
#
# ・マイグレーションの実行結果が既に存在していれば、何もしません。
# ・マイグレーションの実行結果がまだ存在してなければ、テーブルを１から作ります。
class MergeStep2BrancheOrMigrate < ActiveRecord::Migration

  def self.up
    # テーブルを１から作る場合のみ実行されます。
    if SchemaMigration.count == 0
      if agree("!!!WARNING!!! This migrate will initialize tables Are you sure you want to \ncontinue? [yn] ")
        initialize_tables
      else
        say("The task canceled...")
        exit!
      end
    end
  end

  def self.down
    # テーブルが１から作った場合のみ実行されます。
    if SchemaMigration.count == 1
      if agree("!!!WARNING!!! This migrate will destroy any data in the database. Are you sure you want to \ncontinue? [yn]  ")
        Rake::Task["db:drop"].invoke
        Rake::Task["db:create"].invoke
        exit!
      else
        say("The task canceled...")
        exit!
      end
    end
  end

  #
  # = コンソールに同意付きのメッセージを表示します =
  #
  def self.agree(message)
    say(message)
    continue = STDIN.gets.chomp
    continue == 'y' ? true : false
  end

  #
  # = テーブルの初期化 =
  #
  def self.initialize_tables
    create_table "accountings", :force => true, :comment => '経理用履歴テーブル' do |t|
      t.integer  "order_id",                           :comment => "注文"
      t.integer  "credit_sales",       :null => false, :comment => "貸方金額"
      t.string   "credit_account_div", :null => false, :comment => "貸方勘定項目区分"
      t.integer  "debt_sales",         :null => false, :comment => "借方金額"
      t.string   "debt_account_div",   :null => false, :comment => "借方勘定項目区分"
      t.integer  "given_point",                        :comment => "ポイント付与"
      t.integer  "used_point",                         :comment => "ポイント利用"
      t.integer  "previous_point",                     :comment => "ポイント残高"
      t.datetime "created_at",                         :comment => ""
      t.datetime "updated_at",                         :comment => ""
      t.integer  "creator_id",                         :comment => "データ登録者"
      t.integer  "updater_id",                         :comment => "データ更新者"
      t.integer  "deleter_id",                         :comment => "データ削除者"
      t.integer  "user_point_id",                      :comment => "会員ポイント履歴"
    end

    add_index "accountings", ["creator_id"], :name => "index_accountings_on_creator_id"
    add_index "accountings", ["deleter_id"], :name => "index_accountings_on_deleter_id"
    add_index "accountings", ["order_id"], :name => "index_accountings_on_order_id"
    add_index "accountings", ["updater_id"], :name => "index_accountings_on_updater_id"

    create_table "banner_conditions", :force => true, :comment => 'バナー条件' do |t|
      t.integer  "banner_id",                                  :null => false, :comment => "バナー"
      t.string   "type_cd",      :limit => 2,                  :null => false, :comment => "項目(年齢／職業／性別／生年月日／気温／システム日付など)"
      t.string   "operator_cd",  :limit => 2,                  :null => false, :comment => "演算子"
      t.string   "value",        :limit => 100,                                :comment => "値"
      t.integer  "weight_point",                :default => 0, :null => false, :comment => "重みポイント"
      t.datetime "created_at",                                                 :comment => ""
      t.datetime "updated_at",                                                 :comment => ""
      t.integer  "creator_id",                                                 :comment => "データ登録者"
      t.integer  "updater_id",                                                 :comment => "データ更新者"
      t.integer  "deleter_id",                                                 :comment => "データ削除者"
    end

    add_index "banner_conditions", ["banner_id"], :name => "index_banner_conditions_on_banner_id"
    add_index "banner_conditions", ["creator_id"], :name => "index_banner_conditions_on_creator_id"
    add_index "banner_conditions", ["deleter_id"], :name => "index_banner_conditions_on_deleter_id"
    add_index "banner_conditions", ["updater_id"], :name => "index_banner_conditions_on_updater_id"

    create_table "banner_displays", :force => true, :comment => 'バナー表示' do |t|
      t.integer  "banner_id",                      :null => false, :comment => "バナー"
      t.string   "locale",            :limit => 5, :null => false, :comment => "ロケール"
      t.string   "image_public_path",                              :comment => "(publicからの)ファイルパス[バナータイプが画像の時は必須]"
      t.string   "image_alt",                                      :comment => "画像の説明(imgタグのalt属性の値)"
      t.string   "text_category_cd",  :limit => 2,                 :comment => "(テキストの)分類[バナータイプがテキストの時は必須]"
      t.text     "text_description",                               :comment => "テキスト(の記述)"
      t.datetime "created_at",                                     :comment => ""
      t.datetime "updated_at",                                     :comment => ""
      t.integer  "creator_id",                                     :comment => "データ登録者"
      t.integer  "updater_id",                                     :comment => "データ更新者"
      t.integer  "deleter_id",                                     :comment => "データ削除者"
    end

    add_index "banner_displays", ["banner_id"], :name => "index_banner_displays_on_banner_id"
    add_index "banner_displays", ["creator_id"], :name => "index_banner_displays_on_creator_id"
    add_index "banner_displays", ["deleter_id"], :name => "index_banner_displays_on_deleter_id"
    add_index "banner_displays", ["updater_id"], :name => "index_banner_displays_on_updater_id"

    create_table "banners", :force => true, :comment => 'バナー' do |t|
      t.string   "code",                :limit => 10,                  :null => false, :comment => "バナーコード"
      t.string   "type_cd",             :limit => 1,  :default => "1", :null => false, :comment => "バナータイプ（画像／テキスト）"
      t.integer  "display_order",                                                      :comment => "表示優先度"
      t.datetime "publish_started_at",                                                 :comment => "公開終了日時"
      t.datetime "publish_finished_at",                                                :comment => "公開終了日時"
      t.string   "link_url",                                                           :comment => "リンク先URL"
      t.datetime "created_at",                                                         :comment => ""
      t.datetime "updated_at",                                                         :comment => ""
      t.integer  "creator_id",                                                         :comment => "データ登録者"
      t.integer  "updater_id",                                                         :comment => "データ更新者"
      t.integer  "deleter_id",                                                         :comment => "データ削除者"
    end

    add_index "banners", ["creator_id"], :name => "index_banners_on_creator_id"
    add_index "banners", ["deleter_id"], :name => "index_banners_on_deleter_id"
    add_index "banners", ["updater_id"], :name => "index_banners_on_updater_id"

    create_table "campaign_conditions", :force => true, :comment => 'キャンペーン条件' do |t|
      t.integer  "campaign_id",                      :null => false, :comment => "キャンペーン条件"
      t.integer  "no",                               :null => false, :comment => "順番"
      t.string   "type_cd",           :limit => 2,   :null => false, :comment => "項目種"
      t.string   "operator_cd",       :limit => 2,   :null => false, :comment => "演算子"
      t.string   "value",             :limit => 100,                 :comment => "値"
      t.string   "connector_cd",      :limit => 2,   :null => false, :comment => "接続演算子"
      t.integer  "brace_begin_count",                :null => false, :comment => "開始括弧数"
      t.integer  "brace_end_count",                  :null => false, :comment => "終了括弧数"
      t.datetime "created_at",                                       :comment => ""
      t.datetime "updated_at",                                       :comment => ""
      t.integer  "creator_id",                                       :comment => "データ登録者"
      t.integer  "updater_id",                                       :comment => "データ更新者"
      t.integer  "deleter_id",                                       :comment => "データ削除者"
    end

    add_index "campaign_conditions", ["campaign_id"], :name => "index_campaign_conditions_on_campaign_id"
    add_index "campaign_conditions", ["creator_id"], :name => "index_campaign_conditions_on_creator_id"
    add_index "campaign_conditions", ["deleter_id"], :name => "index_campaign_conditions_on_deleter_id"
    add_index "campaign_conditions", ["updater_id"], :name => "index_campaign_conditions_on_updater_id"

    create_table "campaign_effects", :force => true, :comment => 'キャンペーン効果' do |t|
      t.integer  "campaign_id",            :null => false, :comment => "キャンペーン"
      t.string   "type_cd",                :null => false, :comment => "効果種別"
      t.string   "operator_cd",            :null => false, :comment => "演算子"
      t.string   "value",                                  :comment => "値"
      t.datetime "created_at",                             :comment => ""
      t.datetime "updated_at",                             :comment => ""
      t.datetime "point_started_at",                       :comment => "ポイント有効期間開始日時(type_keyが:pointの時のみ必須)"
      t.datetime "point_finished_at",                      :comment => "ポイント有効期間終了日時(type_keyが:pointの時のみ必須)"
      t.integer  "present_product_sku_id",                 :comment => "プレゼント商品SKU(type_keyが:presentの時のみ必須)"
      t.integer  "creator_id",                             :comment => "データ登録者"
      t.integer  "updater_id",                             :comment => "データ更新者"
      t.integer  "deleter_id",                             :comment => "データ削除者"
    end

    add_index "campaign_effects", ["campaign_id"], :name => "index_campaign_effects_on_campaign_id"
    add_index "campaign_effects", ["creator_id"], :name => "index_campaign_effects_on_creator_id"
    add_index "campaign_effects", ["deleter_id"], :name => "index_campaign_effects_on_deleter_id"
    add_index "campaign_effects", ["present_product_sku_id"], :name => "index_campaign_effects_on_present_product_sku_id"
    add_index "campaign_effects", ["updater_id"], :name => "index_campaign_effects_on_updater_id"

    create_table "campaign_target_product_skus", :force => true, :comment => 'キャンペーン対象商品SKU' do |t|
      t.integer  "campaign_id",    :null => false, :comment => "キャンペーン"
      t.integer  "product_sku_id", :null => false, :comment => "商品SKU"
      t.datetime "created_at",                     :comment => ""
      t.datetime "updated_at",                     :comment => ""
      t.integer  "creator_id",                     :comment => "データ登録者"
      t.integer  "updater_id",                     :comment => "データ更新者"
      t.integer  "deleter_id",                     :comment => "データ削除者"
    end

    add_index "campaign_target_product_skus", ["campaign_id"], :name => "index_campaign_target_product_skus_on_campaign_id"
    add_index "campaign_target_product_skus", ["product_sku_id"], :name => "index_campaign_target_product_skus_on_product_sku_id"

    create_table "campaigns", :force => true, :comment => 'キャンペーン' do |t|
      t.string   "code",          :limit => 10,                :null => false, :comment => "コード"
      t.string   "name",          :limit => 60,                :null => false, :comment => "名称"
      t.datetime "started_at",                                 :null => false, :comment => "開始日時"
      t.datetime "finished_at",                                :null => false, :comment => "終了日時"
      t.datetime "created_at",                                                 :comment => ""
      t.datetime "updated_at",                                                 :comment => ""
      t.integer  "priority",                    :default => 1, :null => false, :comment => "優先度 ※1～999（数値が大きいほど優先度高）"
      t.integer  "creator_id",                                                 :comment => "データ登録者"
      t.integer  "updater_id",                                                 :comment => "データ更新者"
      t.integer  "deleter_id",                                                 :comment => "データ削除者"
      t.string   "main_page_url",                                              :comment => "ページ"
    end

    add_index "campaigns", ["started_at", "finished_at", "priority"], :name => "index_campaigns_for_cms_show"

    create_table "cms_bind_contents", :force => true, :comment => 'CMSバインド内容' do |t|
      t.integer  "cms_bind_item_id",              :comment => "CMSバインド項目"
      t.string   "locale",           :limit => 5, :comment => "ロケール"
      t.text     "descriptions",                  :comment => "内容"
      t.datetime "created_at",                    :comment => ""
      t.datetime "updated_at",                    :comment => ""
      t.integer  "creator_id",                    :comment => "データ登録者"
      t.integer  "updater_id",                    :comment => "データ更新者"
      t.integer  "deleter_id",                    :comment => "データ削除者"
    end

    add_index "cms_bind_contents", ["cms_bind_item_id"], :name => "index_cms_bind_contents_on_cms_bind_item_id"

    create_table "cms_bind_items", :force => true, :comment => 'CMSバインド項目' do |t|
      t.integer  "cms_bind_id", :comment => "CMSバインド"
      t.string   "no",          :comment => "バインド番号"
      t.string   "code",        :comment => "コード"
      t.datetime "created_at",  :comment => ""
      t.datetime "updated_at",  :comment => ""
      t.integer  "creator_id",  :comment => "データ登録者"
      t.integer  "updater_id",  :comment => "データ更新者"
      t.integer  "deleter_id",  :comment => "データ削除者"
    end

    add_index "cms_bind_items", ["cms_bind_id"], :name => "index_cms_bind_items_on_cms_page_bind_id"

    create_table "cms_binds", :force => true, :comment => 'CMSバインド' do |t|
      t.integer  "cms_page_id",                  :comment => "CMSページ"
      t.integer  "cms_template_id",              :comment => "CMSテンプレート"
      t.string   "position",        :limit => 2, :comment => "表示位置"
      t.datetime "created_at",                   :comment => ""
      t.datetime "updated_at",                   :comment => ""
      t.integer  "creator_id",                   :comment => "データ登録者"
      t.integer  "updater_id",                   :comment => "データ更新者"
      t.integer  "deleter_id",                   :comment => "データ削除者"
    end

    add_index "cms_binds", ["cms_page_id"], :name => "index_cms_binds_on_cms_page_id"
    add_index "cms_binds", ["cms_template_id"], :name => "index_cms_binds_on_cms_template_id"
    add_index "cms_binds", ["creator_id"], :name => "index_cms_binds_on_creator_id"
    add_index "cms_binds", ["deleter_id"], :name => "index_cms_binds_on_deleter_id"
    add_index "cms_binds", ["updater_id"], :name => "index_cms_binds_on_updater_id"

    create_table "cms_dead_link_checks", :force => true, :comment => 'リンク切れチェック' do |t|
      t.string   "status_cd",                :limit => 2, :comment => "ステータス"
      t.datetime "checked_at",                            :comment => "チェック実施日時"
      t.datetime "created_at",                            :comment => ""
      t.datetime "updated_at",                            :comment => ""
      t.integer  "cms_publish_id",                        :comment => "CMS公開"
      t.boolean  "external_site_ignore_flg",              :comment => "外部サイトへのリンクチェックを行わないフラグ"
      t.integer  "creator_id",                            :comment => "データ登録者"
      t.integer  "updater_id",                            :comment => "データ更新者"
      t.integer  "deleter_id",                            :comment => "データ削除者"
    end

    add_index "cms_dead_link_checks", ["cms_publish_id"], :name => "index_cms_dead_link_checks_on_cms_publish_id"
    add_index "cms_dead_link_checks", ["creator_id"], :name => "index_cms_dead_link_checks_on_creator_id"
    add_index "cms_dead_link_checks", ["deleter_id"], :name => "index_cms_dead_link_checks_on_deleter_id"
    add_index "cms_dead_link_checks", ["updater_id"], :name => "index_cms_dead_link_checks_on_updater_id"

    create_table "cms_dead_link_resources", :force => true, :comment => 'リンク切れチェック対象' do |t|
      t.integer  "banner_id",              :comment => "バナーID"
      t.string   "resource",               :comment => "チェック対象"
      t.datetime "created_at",             :comment => ""
      t.datetime "updated_at",             :comment => ""
      t.integer  "cms_dead_link_check_id", :comment => "リンク切れチェック"
      t.string   "file_nmae",              :comment => "ページファイル名"
      t.integer  "cms_bind_id",            :comment => "CMSバインド"
      t.integer  "product_image_id",       :comment => "商品画像"
      t.integer  "product_display_id",     :comment => "商品説明"
      t.integer  "banner_display_id",      :comment => "バナー説明"
      t.integer  "shop_setting_id",        :comment => "店舗設定"
      t.integer  "product_id",             :comment => "商品"
      t.string   "locale",                 :comment => "ロケール"
      t.integer  "cms_file_id",            :comment => "ファイル"
      t.integer  "cms_template_id",        :comment => "テンプレート"
      t.integer  "creator_id",             :comment => "データ登録者"
      t.integer  "updater_id",             :comment => "データ更新者"
      t.integer  "deleter_id",             :comment => "データ削除者"
    end

    add_index "cms_dead_link_resources", ["banner_display_id"], :name => "index_cms_dead_link_resources_on_banner_display_id"
    add_index "cms_dead_link_resources", ["banner_id"], :name => "index_cms_dead_link_resources_on_banner_id"
    add_index "cms_dead_link_resources", ["cms_bind_id"], :name => "index_cms_dead_link_resources_on_cms_bind_id"
    add_index "cms_dead_link_resources", ["cms_dead_link_check_id"], :name => "index_cms_dead_link_resources_on_cms_dead_link_check_id"
    add_index "cms_dead_link_resources", ["cms_file_id"], :name => "index_cms_dead_link_resources_on_cms_file_id"
    add_index "cms_dead_link_resources", ["cms_template_id"], :name => "index_cms_dead_link_resources_on_cms_template_id"
    add_index "cms_dead_link_resources", ["creator_id"], :name => "index_cms_dead_link_resources_on_creator_id"
    add_index "cms_dead_link_resources", ["deleter_id"], :name => "index_cms_dead_link_resources_on_deleter_id"
    add_index "cms_dead_link_resources", ["product_display_id"], :name => "index_cms_dead_link_resources_on_product_display_id"
    add_index "cms_dead_link_resources", ["product_id"], :name => "index_cms_dead_link_resources_on_product_id"
    add_index "cms_dead_link_resources", ["product_image_id"], :name => "index_cms_dead_link_resources_on_product_image_id"
    add_index "cms_dead_link_resources", ["shop_setting_id"], :name => "index_cms_dead_link_resources_on_shop_setting_id"
    add_index "cms_dead_link_resources", ["updater_id"], :name => "index_cms_dead_link_resources_on_updater_id"

    create_table "cms_dead_links", :force => true, :comment => 'リンク切れURL' do |t|
      t.string   "url",                       :comment => "リンクURL"
      t.datetime "created_at",                :comment => ""
      t.datetime "updated_at",                :comment => ""
      t.integer  "cms_dead_link_resource_id", :comment => "リンク切れチェック対象"
      t.string   "response_status",           :comment => "HTTPレスポンスステータス"
      t.integer  "creator_id",                :comment => "データ登録者"
      t.integer  "updater_id",                :comment => "データ更新者"
      t.integer  "deleter_id",                :comment => "データ削除者"
    end

    add_index "cms_dead_links", ["cms_dead_link_resource_id"], :name => "index_cms_dead_links_on_cms_dead_link_resource_id"

    create_table "cms_files", :force => true, :comment => 'CMSファイル' do |t|
      t.datetime "created_at",        :comment => ""
      t.datetime "updated_at",        :comment => ""
      t.integer  "cms_project_id",    :comment => "CMSプロジェクト"
      t.boolean  "delete_at_publish", :comment => "プロジェクト公開時に削除"
      t.string   "public_file_path",  :comment => "publicからのパス"
      t.integer  "creator_id",        :comment => "データ登録者"
      t.integer  "updater_id",        :comment => "データ更新者"
      t.integer  "deleter_id",        :comment => "データ削除者"
    end

    add_index "cms_files", ["cms_project_id"], :name => "index_cms_files_on_cms_project_id"

    create_table "cms_layouts", :force => true, :comment => 'CMSレイアウト' do |t|
      t.string   "file_name",             :comment => ""
      t.text     "user_contents",         :comment => ""
      t.datetime "created_at",            :comment => ""
      t.datetime "updated_at",            :comment => ""
      t.boolean  "default_layout_top",    :comment => "デフォルトレイアウト(上部)"
      t.boolean  "default_layout_bottom", :comment => "デフォルトレイアウト(下部)"
      t.integer  "creator_id",            :comment => "データ登録者"
      t.integer  "updater_id",            :comment => "データ更新者"
      t.integer  "deleter_id",            :comment => "データ削除者"
    end

    create_table "cms_page_titles", :force => true, :comment => 'CMSページ名' do |t|
      t.string   "locale",      :limit => 5, :comment => "ロケール"
      t.string   "name",                     :comment => "ページ名"
      t.datetime "created_at",               :comment => ""
      t.datetime "updated_at",               :comment => ""
      t.integer  "cms_page_id",              :comment => "CMSページ"
      t.integer  "creator_id",               :comment => "データ登録者"
      t.integer  "updater_id",               :comment => "データ更新者"
      t.integer  "deleter_id",               :comment => "データ削除者"
    end

    add_index "cms_page_titles", ["cms_page_id"], :name => "index_cms_page_titles_on_cms_page_id"

    create_table "cms_pages", :force => true, :comment => 'CMSページ' do |t|
      t.integer  "cms_project_id",    :comment => "CMSプロジェクト"
      t.string   "controller",        :comment => "コントローラ名"
      t.boolean  "stop_public",       :comment => "緊急非公開"
      t.datetime "created_at",        :comment => ""
      t.datetime "updated_at",        :comment => ""
      t.string   "terminal",          :comment => "端末(0: PC, 1:携帯)"
      t.string   "action",            :comment => "アクション"
      t.boolean  "delete_at_publish", :comment => "プロジェクト公開時に削除"
      t.integer  "creator_id",        :comment => "データ登録者"
      t.integer  "updater_id",        :comment => "データ更新者"
      t.integer  "deleter_id",        :comment => "データ削除者"
    end

    add_index "cms_pages", ["cms_project_id"], :name => "index_cms_pages_on_cms_project_id"

    create_table "cms_project_approvals", :force => true, :comment => 'プロジェクト承認' do |t|
      t.integer  "cms_project_id",   :comment => "CMSプロジェクト"
      t.integer  "goldberg_user_id", :comment => "メンバー"
      t.string   "result_cd",        :comment => "結果"
      t.string   "comment",          :comment => "コメント"
      t.datetime "created_at",       :comment => ""
      t.datetime "updated_at",       :comment => ""
      t.integer  "creator_id",       :comment => "データ登録者"
      t.integer  "updater_id",       :comment => "データ更新者"
      t.integer  "deleter_id",       :comment => "データ削除者"
    end

    add_index "cms_project_approvals", ["cms_project_id"], :name => "index_cms_project_approvals_on_cms_project_id"
    add_index "cms_project_approvals", ["creator_id"], :name => "index_cms_project_approvals_on_creator_id"
    add_index "cms_project_approvals", ["deleter_id"], :name => "index_cms_project_approvals_on_deleter_id"
    add_index "cms_project_approvals", ["goldberg_user_id"], :name => "index_cms_project_approvals_on_goldberg_user_id"
    add_index "cms_project_approvals", ["updater_id"], :name => "index_cms_project_approvals_on_updater_id"

    create_table "cms_project_assigns", :force => true, :comment => 'CMSプロジェクト' do |t|
      t.integer  "cms_project_id",                :comment => "CMSプロジェクト"
      t.integer  "goldberg_user_id",              :comment => "管理者"
      t.string   "role",             :limit => 1, :comment => "ロール(0: 承認者, 1:確認者, 2:作成責任者 3:作成者)"
      t.datetime "created_at",                    :comment => ""
      t.datetime "updated_at",                    :comment => ""
      t.integer  "creator_id",                    :comment => "データ登録者"
      t.integer  "updater_id",                    :comment => "データ更新者"
      t.integer  "deleter_id",                    :comment => "データ削除者"
    end

    add_index "cms_project_assigns", ["cms_project_id"], :name => "index_cms_project_assigns_on_cms_project_id"
    add_index "cms_project_assigns", ["creator_id"], :name => "index_cms_project_assigns_on_creator_id"
    add_index "cms_project_assigns", ["deleter_id"], :name => "index_cms_project_assigns_on_deleter_id"
    add_index "cms_project_assigns", ["goldberg_user_id"], :name => "index_cms_project_assigns_on_goldberg_user_id"
    add_index "cms_project_assigns", ["updater_id"], :name => "index_cms_project_assigns_on_updater_id"

    create_table "cms_projects", :force => true, :comment => 'CMSプロジェクト' do |t|
      t.integer  "priority",                     :default => 999, :comment => "優先度"
      t.string   "acceptance_type",                               :comment => "承認経路"
      t.string   "project_code",                                  :comment => "プロジェクトコード"
      t.string   "project_name",                                  :comment => "プロジェクト名"
      t.datetime "started_at",                                    :comment => "プロジェクトの公開開始日時"
      t.datetime "finished_at",                                   :comment => "プロジェクトの公開終了日時"
      t.string   "status_cd",       :limit => 1,                  :comment => "公開ステータス(0: 未公開, 1:公開, 2:公開終了)"
      t.datetime "created_at",                                    :comment => ""
      t.datetime "updated_at",                                    :comment => ""
      t.integer  "creator_id",                                    :comment => "データ登録者"
      t.integer  "updater_id",                                    :comment => "データ更新者"
      t.integer  "deleter_id",                                    :comment => "データ削除者"
    end

    add_index "cms_projects", ["creator_id"], :name => "index_cms_projects_on_creator_id"
    add_index "cms_projects", ["deleter_id"], :name => "index_cms_projects_on_deleter_id"
    add_index "cms_projects", ["project_code"], :name => "index_cms_projects_on_project_code"
    add_index "cms_projects", ["project_name"], :name => "index_cms_projects_on_project_name"
    add_index "cms_projects", ["updater_id"], :name => "index_cms_projects_on_updater_id"

    create_table "cms_published_projects", :force => true, :comment => 'プロジェクト公開関連' do |t|
      t.integer  "cms_project_id",                                               :comment => "プロジェクト"
      t.integer  "cms_publish_id",                                               :comment => "公開"
      t.datetime "created_at",                                                   :comment => ""
      t.datetime "updated_at",                                                   :comment => ""
      t.string   "calc_type_cd",   :limit => 1, :default => "0", :null => false, :comment => "計算種(0:追加, 1:削除, 2:保持)"
      t.integer  "creator_id",                                                   :comment => "データ登録者"
      t.integer  "updater_id",                                                   :comment => "データ更新者"
      t.integer  "deleter_id",                                                   :comment => "データ削除者"
    end

    add_index "cms_published_projects", ["cms_project_id"], :name => "index_cms_published_projects_on_cms_project_id"
    add_index "cms_published_projects", ["cms_publish_id"], :name => "index_cms_published_projects_on_cms_publish_id"
    add_index "cms_published_projects", ["creator_id"], :name => "index_cms_published_projects_on_creator_id"
    add_index "cms_published_projects", ["deleter_id"], :name => "index_cms_published_projects_on_deleter_id"
    add_index "cms_published_projects", ["updater_id"], :name => "index_cms_published_projects_on_updater_id"

    create_table "cms_publishes", :force => true, :comment => 'CMS公開情報' do |t|
      t.integer  "cms_project_id",                  :comment => "CMSプロジェクト"
      t.string   "target",             :limit => 1, :comment => "公開対象(0: 本番, 1:テスト)"
      t.datetime "designated_at",                   :comment => "対象日時"
      t.datetime "published_at",                    :comment => "公開実施日時"
      t.string   "status_cd",          :limit => 1, :comment => "公開ステータス(0: 未公開, 1:公開, 2:失敗)"
      t.datetime "created_at",                      :comment => ""
      t.datetime "updated_at",                      :comment => ""
      t.boolean  "cloud_front_upload",              :comment => "公開完了後のクラウドフロントへのアプロード"
      t.integer  "creator_id",                      :comment => "データ登録者"
      t.integer  "updater_id",                      :comment => "データ更新者"
      t.integer  "deleter_id",                      :comment => "データ削除者"
    end

    add_index "cms_publishes", ["cms_project_id"], :name => "index_cms_publishes_on_cms_project_id"
    add_index "cms_publishes", ["creator_id"], :name => "index_cms_publishes_on_creator_id"
    add_index "cms_publishes", ["deleter_id"], :name => "index_cms_publishes_on_deleter_id"
    add_index "cms_publishes", ["updater_id"], :name => "index_cms_publishes_on_updater_id"

    create_table "cms_template_contents", :force => true, :comment => 'CMSテンプレート内容' do |t|
      t.integer  "cms_template_id",              :comment => "CMSテンプレート"
      t.string   "locale",          :limit => 5, :comment => "ロケール"
      t.text     "descriptions",                 :comment => "内容"
      t.datetime "created_at",                   :comment => ""
      t.datetime "updated_at",                   :comment => ""
      t.integer  "creator_id",                   :comment => "データ登録者"
      t.integer  "updater_id",                   :comment => "データ更新者"
      t.integer  "deleter_id",                   :comment => "データ削除者"
    end

    add_index "cms_template_contents", ["cms_template_id"], :name => "index_cms_template_contents_on_cms_template_id"

    create_table "cms_templates", :force => true, :comment => 'CMSテンプレート' do |t|
      t.integer  "cms_project_id",                          :comment => "CMSプロジェクト"
      t.string   "name",                                    :comment => "テンプレート名"
      t.string   "display_position",                        :comment => "表示可能位置(0: 上, 1:下, 2:左, 3:右, 4:中央, 5:右または左)"
      t.boolean  "delete_at_publish",                       :comment => "プロジェクト公開時に削除"
      t.datetime "created_at",                              :comment => ""
      t.datetime "updated_at",                              :comment => ""
      t.string   "default_template_position",               :comment => "デフォルトテンプレート表示位置(0: 上, 1:下, 2:左, 3:右, 4:中央)"
      t.string   "terminal",                                :comment => "端末(0: PC, 1:携帯)"
      t.string   "template_code",             :limit => 24, :comment => "テンプレート識別コード"
      t.string   "development_code",          :limit => 35, :comment => "開発用テンプレート識別コード"
      t.integer  "creator_id",                              :comment => "データ登録者"
      t.integer  "updater_id",                              :comment => "データ更新者"
      t.integer  "deleter_id",                              :comment => "データ削除者"
    end

    add_index "cms_templates", ["cms_project_id"], :name => "index_cms_templates_on_cms_project_id"
    add_index "cms_templates", ["creator_id"], :name => "index_cms_templates_on_creator_id"
    add_index "cms_templates", ["deleter_id"], :name => "index_cms_templates_on_deleter_id"
    add_index "cms_templates", ["updater_id"], :name => "index_cms_templates_on_updater_id"

    create_table "countries", :force => true do |t|
      t.string   "code",                 :limit => 3, :comment => "国コード（3桁）"
      t.string   "name",                              :comment => "国名（英語）"
      t.integer  "shipping_cost",                     :comment => "送料"
      t.datetime "created_at",                        :comment => ""
      t.datetime "updated_at",                        :comment => ""
      t.string   "ship_restriction_div", :limit => 2, :comment => "配送制限区分（0:配送可能、1：配送不可）"
      t.integer  "creator_id",                        :comment => "データ登録者"
      t.integer  "updater_id",                        :comment => "データ更新者"
      t.integer  "deleter_id",                        :comment => "データ削除者"
      t.integer  "distributor_id",                    :comment => "物流ID"
      t.string   "use_language_div",                  :comment => "対応言語"
      t.string   "name_order_div",                    :comment => "姓名の並び順区分"
      t.string   "address_order_div",                 :comment => "住所の並び順区分"
    end

    add_index "countries", ["creator_id"], :name => "index_countries_on_creator_id"
    add_index "countries", ["deleter_id"], :name => "index_countries_on_deleter_id"
    add_index "countries", ["distributor_id"], :name => "index_countries_on_distributor_id"
    add_index "countries", ["updater_id"], :name => "index_countries_on_updater_id"

    create_table "csv_download_pattern_items", :force => true, :comment => 'CSVダウンロードパターン項目' do |t|
      t.integer  "csv_download_pattern_id", :comment => "CSVダウンロードパターン"
      t.string   "code",                    :comment => "項目コード"
      t.string   "value",                   :comment => "固定値"
      t.datetime "created_at",              :comment => ""
      t.datetime "updated_at",              :comment => ""
      t.integer  "display_order",           :comment => "表示順"
      t.string   "name",                    :comment => "項目名"
      t.integer  "creator_id",              :comment => "データ登録者"
      t.integer  "updater_id",              :comment => "データ更新者"
      t.integer  "deleter_id",              :comment => "データ削除者"
    end

    add_index "csv_download_pattern_items", ["creator_id"], :name => "index_csv_download_pattern_items_on_creator_id"
    add_index "csv_download_pattern_items", ["csv_download_pattern_id"], :name => "index_csv_download_pattern_items_on_csv_download_pattern_id"
    add_index "csv_download_pattern_items", ["deleter_id"], :name => "index_csv_download_pattern_items_on_deleter_id"
    add_index "csv_download_pattern_items", ["updater_id"], :name => "index_csv_download_pattern_items_on_updater_id"

    create_table "csv_download_pattern_order_conditions", :force => true, :comment => 'CSVダウンロードパターン注文情報条件' do |t|
      t.integer  "csv_download_pattern_id", :comment => "CSVダウンロードパターン"
      t.date     "order_date_start",        :comment => "注文日from"
      t.date     "order_date_end",          :comment => "注文日to"
      t.string   "order_code",              :comment => "オンライン受注番号"
      t.string   "status_div",              :comment => "注文の進捗"
      t.string   "name_first",              :comment => "注文者（フリガナ）"
      t.string   "tel_num",                 :comment => "電話番号"
      t.string   "delivery_kanji_name",     :comment => "お届け先（フリガナ）"
      t.string   "delivery_tel_num",        :comment => "お届け先電話番号"
      t.string   "product_code",            :comment => "商品"
      t.integer  "qty_from",                :comment => "購入数量from"
      t.integer  "qty_to",                  :comment => "購入数量to"
      t.string   "email",                   :comment => "メールアドレス"
      t.string   "payment_type_div",        :comment => "支払方法"
      t.string   "free_request_div",        :comment => "自由記入欄"
      t.string   "send_difference_div",     :comment => "お届先相違"
      t.boolean  "test_flg",                :comment => "本番注文／テスト注文"
      t.datetime "created_at",              :comment => ""
      t.datetime "updated_at",              :comment => ""
      t.integer  "creator_id",              :comment => "データ登録者"
      t.integer  "updater_id",              :comment => "データ更新者"
      t.integer  "deleter_id",              :comment => "データ削除者"
      t.time     "shipping_date_start",     :comment => "配送日from"
      t.time     "shipping_date_end",       :comment => "配送日to"
      t.string   "order_shipping_code",     :comment => "配送番号"
      t.integer  "total_amount",            :comment => "受注金額from"
      t.integer  "upper_total_amount",      :comment => "受注金額to"
      t.string   "send_count_div",          :comment => "お届先数区分"
      t.integer  "disp_count",              :comment => "表示件数"
      t.boolean  "delivery_to_native_flg",  :comment => "国内配送フラグ"
      t.boolean  "delivery_to_foreign_flg", :comment => "海外配送フラグ"
    end

    add_index "csv_download_pattern_order_conditions", ["csv_download_pattern_id"], :name => "index_csv_download_pattern_order_conditions_on_01"

    create_table "csv_download_pattern_product_conditions", :force => true, :comment => 'CSVダウンロードパターン商品情報条件' do |t|
      t.integer  "csv_download_pattern_id", :comment => "CSVダウンロードパターン"
      t.string   "product_code",            :comment => "商品コード"
      t.string   "product_name",            :comment => "商品名"
      t.string   "variation",               :comment => "バリエーション"
      t.datetime "created_at",              :comment => ""
      t.datetime "updated_at",              :comment => ""
      t.integer  "creator_id",              :comment => "データ登録者"
      t.integer  "updater_id",              :comment => "データ更新者"
      t.integer  "deleter_id",              :comment => "データ削除者"
    end

    add_index "csv_download_pattern_product_conditions", ["csv_download_pattern_id"], :name => "index_csv_download_pattern_product_conditions_on_01"

    create_table "csv_download_pattern_user_conditions", :force => true, :comment => 'CSVダウンロードパターン会員情報条件' do |t|
      t.integer  "csv_download_pattern_id", :comment => "CSVダウンロードパターン"
      t.string   "user_code",               :comment => "会員コード"
      t.string   "name",                    :comment => "会員名"
      t.string   "address",                 :comment => "住所"
      t.string   "email",                   :comment => "メールアドレス"
      t.string   "sex",                     :comment => "性別"
      t.string   "tel_num",                 :comment => "電話番号"
      t.string   "member_div",              :comment => "顧客区分"
      t.datetime "regist_date_from",        :comment => "登録日from"
      t.datetime "regist_date_to",          :comment => "登録日to"
      t.boolean  "test_flg",                :comment => "本番会員／テスト会員"
      t.datetime "created_at",              :comment => ""
      t.datetime "updated_at",              :comment => ""
      t.integer  "creator_id",              :comment => "データ登録者"
      t.integer  "updater_id",              :comment => "データ更新者"
      t.integer  "deleter_id",              :comment => "データ削除者"
    end

    add_index "csv_download_pattern_user_conditions", ["csv_download_pattern_id"], :name => "index_csv_download_pattern_user_conditions_on_01"

    create_table "csv_download_patterns", :force => true, :comment => 'CSVダウンロードパターン' do |t|
      t.string   "download_type",  :limit => 2, :comment => "ダウンロード種類"
      t.string   "name",                        :comment => "パターン名"
      t.string   "character_code",              :comment => "文字コード"
      t.string   "file_name",                   :comment => "ファイル名"
      t.datetime "created_at",                  :comment => ""
      t.datetime "updated_at",                  :comment => ""
      t.integer  "creator_id",                  :comment => "データ登録者"
      t.integer  "updater_id",                  :comment => "データ更新者"
      t.integer  "deleter_id",                  :comment => "データ削除者"
    end

    add_index "csv_download_patterns", ["creator_id"], :name => "index_csv_download_patterns_on_creator_id"
    add_index "csv_download_patterns", ["deleter_id"], :name => "index_csv_download_patterns_on_deleter_id"
    add_index "csv_download_patterns", ["updater_id"], :name => "index_csv_download_patterns_on_updater_id"

    create_table "csv_file_download_histories", :force => true, :comment => 'CSVダウンロード履歴' do |t|
      t.integer  "csv_download_pattern_id", :comment => "CSVダウンロードパターン"
      t.integer  "goldberg_user_id",        :comment => "管理者"
      t.string   "file_name",               :comment => "publicからのファイルパス"
      t.datetime "created_at",              :comment => ""
      t.datetime "updated_at",              :comment => ""
      t.integer  "creator_id",              :comment => "データ登録者"
      t.integer  "updater_id",              :comment => "データ更新者"
      t.integer  "deleter_id",              :comment => "データ削除者"
      t.string   "character_code",          :comment => "文字コード"
    end

    add_index "csv_file_download_histories", ["csv_download_pattern_id"], :name => "index_csv_file_download_histories_on_csv_download_pattern_id"

    create_table "csv_file_upload_histories", :force => true do |t|
      t.integer  "upload_id",     :comment => "アップロード情報"
      t.datetime "processed_at",  :comment => "処理日時"
      t.integer  "input_count",   :comment => "入力件数"
      t.integer  "success_count", :comment => "登録件数"
      t.integer  "error_count",   :comment => "エラー件数"
      t.text     "csv_file_path", :comment => "CSVファイルパス"
      t.integer  "creator_id",    :comment => "データ登録者"
      t.integer  "updater_id",    :comment => "データ更新者"
      t.integer  "deleter_id",    :comment => "データ削除者"
    end

    create_table "csv_order_output_items", :force => true do |t|
      t.integer  "order_shipping_id",                  :comment => ""
      t.datetime "output_at",                          :comment => ""
      t.integer  "supervisor_id",                      :comment => ""
      t.datetime "created_at",                         :comment => ""
      t.datetime "updated_at",                         :comment => ""
      t.datetime "deleted_at",                         :comment => ""
      t.integer  "lock_version",        :default => 0, :comment => ""
      t.integer  "csv_order_output_id",                :comment => ""
      t.integer  "creator_id",                         :comment => "データ登録者"
      t.integer  "updater_id",                         :comment => "データ更新者"
      t.integer  "deleter_id",                         :comment => "データ削除者"
    end

    add_index "csv_order_output_items", ["csv_order_output_id"], :name => "index_order_extra_histories_on_order_extra_history_id"
    add_index "csv_order_output_items", ["order_shipping_id"], :name => "index_csv_order_output_items_on_order_shipping_id"
    add_index "csv_order_output_items", ["supervisor_id"], :name => "index_csv_order_output_items_on_supervisor_id"

    create_table "csv_order_outputs", :force => true do |t|
      t.datetime "output_at",                    :comment => ""
      t.integer  "supervisor_id",                :comment => ""
      t.datetime "created_at",                   :comment => ""
      t.datetime "updated_at",                   :comment => ""
      t.datetime "deleted_at",                   :comment => ""
      t.integer  "lock_version",  :default => 0, :comment => ""
      t.integer  "creator_id",                   :comment => "データ登録者"
      t.integer  "updater_id",                   :comment => "データ更新者"
      t.integer  "deleter_id",                   :comment => "データ削除者"
    end

    create_table "distributor_delivery_time_displays", :force => true, :comment => 'お届け時間指定名称' do |t|
      t.integer  "distributor_delivery_time_id", :comment => "お届け時間ID"
      t.string   "name",                         :comment => "お届け時間名称"
      t.string   "locale",                       :comment => "言語"
      t.datetime "created_at",                   :comment => ""
      t.datetime "updated_at",                   :comment => ""
      t.integer  "creator_id",                   :comment => "データ登録者"
      t.integer  "updater_id",                   :comment => "データ更新者"
      t.integer  "deleter_id",                   :comment => "データ削除者"
    end

    add_index "distributor_delivery_time_displays", ["distributor_delivery_time_id"], :name => "index_distributor_delivery_time_displays_on_01"

    create_table "distributor_delivery_times", :force => true, :comment => 'お届け時間指定' do |t|
      t.integer  "distributor_id", :comment => "物流マスタID"
      t.integer  "display_order",  :comment => "表示順"
      t.string   "code",           :comment => "お届け時間指定コード"
      t.datetime "created_at",     :comment => ""
      t.datetime "updated_at",     :comment => ""
      t.integer  "creator_id",     :comment => "データ登録者"
      t.integer  "updater_id",     :comment => "データ更新者"
      t.integer  "deleter_id",     :comment => "データ削除者"
    end

    add_index "distributor_delivery_times", ["distributor_id"], :name => "index_distributor_delivery_times_on_distributor_id"

    create_table "distributors", :force => true, :comment => '物流マスタ' do |t|
      t.string   "name",                   :comment => "物流名称"
      t.string   "url",                    :comment => "配送状況問い合わせURL"
      t.integer  "deliverable_term_start", :comment => "お届け日指定開始日"
      t.integer  "deliverable_term_end",   :comment => "お届け日指定終了日"
      t.datetime "created_at",             :comment => ""
      t.datetime "updated_at",             :comment => ""
      t.integer  "creator_id",             :comment => "データ登録者"
      t.integer  "updater_id",             :comment => "データ更新者"
      t.integer  "deleter_id",             :comment => "データ削除者"
    end

    create_table "goldberg_role_actions", :force => true, :comment => 'ロールが使用可能なアクション' do |t|
      t.integer  "role_id",        :comment => "ロール"
      t.integer  "site_action_id", :comment => "アクション"
      t.datetime "created_at",     :comment => ""
      t.datetime "updated_at",     :comment => ""
      t.integer  "creator_id",     :comment => "データ登録者"
      t.integer  "updater_id",     :comment => "データ更新者"
      t.integer  "deleter_id",     :comment => "データ削除者"
    end

    add_index "goldberg_role_actions", ["role_id"], :name => "index_goldberg_role_actions_on_role_id"
    add_index "goldberg_role_actions", ["site_action_id"], :name => "index_goldberg_role_actions_on_site_action_id"

    create_table "goldberg_roles", :force => true, :comment => 'Goldbergロール' do |t|
      t.string   "name",                            :null => false, :comment => ""
      t.string   "description",     :default => "",                 :comment => ""
      t.integer  "default_page_id",                                 :comment => ""
      t.datetime "created_at",                                      :comment => ""
      t.datetime "updated_at",                                      :comment => ""
      t.datetime "deleted_at",                                      :comment => ""
      t.integer  "creator_id",                                      :comment => "データ登録者"
      t.integer  "updater_id",                                      :comment => "データ更新者"
      t.integer  "deleter_id",                                      :comment => "データ削除者"
    end

    add_index "goldberg_roles", ["default_page_id"], :name => "fk_role_default_page_id"

    create_table "goldberg_site_action_groups", :force => true, :comment => 'アクショングループ' do |t|
      t.string   "name",                        :null => false, :comment => "グループ名(=コントローラ名)"
      t.datetime "created_at",                                  :comment => ""
      t.datetime "updated_at",                                  :comment => ""
      t.datetime "deleted_at",                                  :comment => ""
      t.string   "display_name", :limit => 100,                 :comment => "表示名称"
      t.integer  "creator_id",                                  :comment => "データ登録者"
      t.integer  "updater_id",                                  :comment => "データ更新者"
      t.integer  "deleter_id",                                  :comment => "データ削除者"
    end

    create_table "goldberg_site_actions", :force => true, :comment => 'アクション' do |t|
      t.integer  "site_action_group_id",                :null => false, :comment => "アクショングループ(=コントローラ)"
      t.string   "name",                                :null => false, :comment => "アクション名"
      t.datetime "created_at",                                          :comment => ""
      t.datetime "updated_at",                                          :comment => ""
      t.datetime "deleted_at",                                          :comment => ""
      t.string   "display_name",         :limit => 100,                 :comment => "表示名称"
      t.string   "category_name",        :limit => 10,                  :comment => "カテゴリ"
      t.integer  "creator_id",                                          :comment => "データ登録者"
      t.integer  "updater_id",                                          :comment => "データ更新者"
      t.integer  "deleter_id",                                          :comment => "データ削除者"
    end

    add_index "goldberg_site_actions", ["creator_id"], :name => "index_goldberg_site_actions_on_creator_id"
    add_index "goldberg_site_actions", ["deleter_id"], :name => "index_goldberg_site_actions_on_deleter_id"
    add_index "goldberg_site_actions", ["site_action_group_id"], :name => "index_goldberg_site_actions_on_site_action_group_id"
    add_index "goldberg_site_actions", ["updater_id"], :name => "index_goldberg_site_actions_on_updater_id"

    create_table "goldberg_users", :force => true, :comment => 'Goldbergユーザー' do |t|
      t.string   "login",                           :null => false, :comment => "表示用名称"
      t.string   "crypted_password",                :null => false, :comment => ""
      t.integer  "role_id",                         :null => false, :comment => ""
      t.string   "salt",                                            :comment => ""
      t.string   "email",                                           :comment => ""
      t.datetime "created_at",                                      :comment => ""
      t.datetime "updated_at",                                      :comment => ""
      t.datetime "deleted_at",                                      :comment => ""
      t.integer  "lock_version",     :default => 0,                 :comment => ""
      t.string   "name",                            :null => false, :comment => "表示用名称"
      t.integer  "creator_id",                                      :comment => "データ登録者"
      t.integer  "updater_id",                                      :comment => "データ更新者"
      t.integer  "deleter_id",                                      :comment => "データ削除者"
    end

    add_index "goldberg_users", ["creator_id"], :name => "index_goldberg_users_on_creator_id"
    add_index "goldberg_users", ["deleter_id"], :name => "index_goldberg_users_on_deleter_id"
    add_index "goldberg_users", ["role_id"], :name => "fk_user_role_id"
    add_index "goldberg_users", ["updater_id"], :name => "index_goldberg_users_on_updater_id"

    create_table "inquiries", :force => true, :comment => '問合せ' do |t|
      t.integer  "user_id",                                          :comment => ""
      t.string   "inquiry_div",     :limit => 5,                     :comment => ""
      t.string   "name_last",       :limit => 30,                    :comment => ""
      t.string   "name_first",      :limit => 30,                    :comment => "漢字名前（名）"
      t.string   "kana_name_last",  :limit => 30,                    :comment => ""
      t.string   "kana_name_first", :limit => 30,                    :comment => ""
      t.string   "tel_num",         :limit => 20,                    :comment => "電話番号"
      t.string   "zip_code",        :limit => 15,                    :comment => "郵便番号"
      t.string   "address_1",       :limit => 45,                    :comment => ""
      t.string   "address_2",       :limit => 90,                    :comment => ""
      t.string   "address_3",       :limit => 90,                    :comment => ""
      t.string   "email",           :limit => 100,                   :comment => ""
      t.string   "inquiry",         :limit => 4000,                  :comment => ""
      t.datetime "created_at",                                       :comment => ""
      t.datetime "updated_at",                                       :comment => ""
      t.datetime "deleted_at",                                       :comment => ""
      t.integer  "lock_version",                    :default => 0,   :comment => ""
      t.string   "status_div",      :limit => 6,    :default => "0", :comment => ""
      t.string   "country_code",    :limit => 3,                     :comment => "国"
      t.string   "locale",          :limit => 5,                     :comment => "言語コード"
      t.string   "answer",          :limit => 4000,                  :comment => "回答内容"
      t.boolean  "test_flg",                                         :comment => "テスト環境フラグ"
      t.string   "image_url_1",                                      :comment => "画像URL1"
      t.string   "image_url_2",                                      :comment => "画像URL2"
      t.string   "image_url_3",                                      :comment => "画像URL3"
      t.string   "image_url_4",                                      :comment => "画像URL4"
      t.string   "image_url_5",                                      :comment => "画像URL5"
      t.integer  "creator_id",                                       :comment => "データ登録者"
      t.integer  "updater_id",                                       :comment => "データ更新者"
      t.integer  "deleter_id",                                       :comment => "データ削除者"
    end

    add_index "inquiries", ["country_code"], :name => "index_inquiries_on_country_code"
    add_index "inquiries", ["creator_id"], :name => "index_inquiries_on_creator_id"
    add_index "inquiries", ["deleter_id"], :name => "index_inquiries_on_deleter_id"
    add_index "inquiries", ["updater_id"], :name => "index_inquiries_on_updater_id"
    add_index "inquiries", ["user_id"], :name => "index_inquiries_on_user_id"

    create_table "inquiry_memos", :force => true, :comment => '問合せメモ' do |t|
      t.integer  "inquiry_id",                                  :null => false, :comment => ""
      t.string   "creator_name", :limit => 30,                  :null => false, :comment => ""
      t.datetime "created_at",                                                  :comment => ""
      t.datetime "updated_at",                                                  :comment => ""
      t.datetime "deleted_at",                                                  :comment => ""
      t.integer  "lock_version",                 :default => 0,                 :comment => ""
      t.string   "memo",         :limit => 4000,                :null => false, :comment => ""
      t.integer  "creator_id",                                                  :comment => "データ登録者"
      t.integer  "updater_id",                                                  :comment => "データ更新者"
      t.integer  "deleter_id",                                                  :comment => "データ削除者"
    end

    add_index "inquiry_memos", ["inquiry_id"], :name => "index_inquiry_memos_on_inquiry_id"

    create_table "inquiry_products", :force => true, :comment => 'お問合せ対象商品' do |t|
      t.integer  "product_id", :comment => "対象商品ID"
      t.datetime "created_at", :comment => ""
      t.datetime "updated_at", :comment => ""
      t.integer  "inquiry_id", :comment => "お問合せ番号"
      t.integer  "creator_id", :comment => "データ登録者"
      t.integer  "updater_id", :comment => "データ更新者"
      t.integer  "deleter_id", :comment => "データ削除者"
    end

    add_index "inquiry_products", ["inquiry_id"], :name => "index_inquiry_products_on_inquiry_id"
    add_index "inquiry_products", ["product_id"], :name => "index_inquiry_products_on_product_id"

    create_table "jp_address", :force => true do |t|
      t.string  "jiscode",     :limit => 5, :null => false, :comment => ""
      t.string  "zipcode_old", :limit => 5, :null => false, :comment => ""
      t.string  "zipcode",     :limit => 7, :null => false, :comment => ""
      t.string  "prefecture",               :null => false, :comment => ""
      t.string  "city",                     :null => false, :comment => ""
      t.string  "address",                                  :comment => ""
      t.integer "creator_id",                               :comment => "データ登録者"
      t.integer "updater_id",                               :comment => "データ更新者"
      t.integer "deleter_id",                               :comment => "データ削除者"
    end

    add_index "jp_address", ["jiscode"], :name => "index_jp_address_on_jiscode"
    add_index "jp_address", ["zipcode"], :name => "index_jp_address_on_zipcode"
    add_index "jp_address", ["zipcode_old"], :name => "index_jp_address_on_zipcode_old"

    create_table "kbn_code_names", :force => true do |t|
      t.integer  "kbn_code_id",              :comment => "名称マスタID"
      t.string   "locale",      :limit => 5, :comment => "言語コード"
      t.string   "name",                     :comment => "名称"
      t.datetime "created_at",               :comment => ""
      t.datetime "updated_at",               :comment => ""
      t.integer  "creator_id",               :comment => "データ登録者"
      t.integer  "updater_id",               :comment => "データ更新者"
      t.integer  "deleter_id",               :comment => "データ削除者"
    end

    add_index "kbn_code_names", ["kbn_code_id"], :name => "index_ms_code_names_on_name_master_id"

    create_table "kbn_codes", :force => true, :comment => '名称マスタ' do |t|
      t.string   "kbn",          :limit => 5,                 :comment => ""
      t.string   "kbn_name",     :limit => 40,                :comment => ""
      t.string   "code",         :limit => 5,                 :comment => ""
      t.integer  "creator_id",                                :comment => ""
      t.integer  "updater_id",                                :comment => ""
      t.datetime "created_at",                                :comment => ""
      t.datetime "updated_at",                                :comment => ""
      t.datetime "deleted_at",                                :comment => ""
      t.integer  "lock_version",               :default => 0, :comment => ""
      t.integer  "deleter_id",                                :comment => "データ削除者"
    end

    add_index "kbn_codes", ["creator_id"], :name => "index_kbn_codes_on_creator_id"
    add_index "kbn_codes", ["deleter_id"], :name => "index_kbn_codes_on_deleter_id"
    add_index "kbn_codes", ["updater_id"], :name => "index_kbn_codes_on_updater_id"

    create_table "mail_deliveries", :force => true, :comment => '配信メール履歴' do |t|
      t.integer  "user_id",                                    :comment => ""
      t.string   "mail_div",      :limit => 15,                :comment => "メール区分(10種類のメールがあります)"
      t.datetime "deliveried_at",                              :comment => ""
      t.string   "header",                                     :comment => ""
      t.string   "subject",                                    :comment => ""
      t.datetime "created_at",                                 :comment => ""
      t.datetime "updated_at",                                 :comment => ""
      t.datetime "deleted_at",                                 :comment => ""
      t.integer  "lock_version",                :default => 0, :comment => ""
      t.string   "site_div",      :limit => 5,                 :comment => ""
      t.text     "mail_body",                                  :comment => ""
      t.string   "message_id",                                 :comment => ""
      t.string   "to_address",                                 :comment => ""
      t.string   "from_address",                               :comment => ""
      t.integer  "order_id",                                   :comment => ""
      t.integer  "inquiry_id",                                 :comment => "お問合せID"
      t.integer  "creator_id",                                 :comment => "データ登録者"
      t.integer  "updater_id",                                 :comment => "データ更新者"
      t.integer  "deleter_id",                                 :comment => "データ削除者"
    end

    add_index "mail_deliveries", ["creator_id"], :name => "index_mail_deliveries_on_creator_id"
    add_index "mail_deliveries", ["deleter_id"], :name => "index_mail_deliveries_on_deleter_id"
    add_index "mail_deliveries", ["inquiry_id"], :name => "index_mail_deliveries_on_inquiry_id"
    add_index "mail_deliveries", ["order_id"], :name => "index_mail_deliveries_on_ec_order_chapter_id"
    add_index "mail_deliveries", ["updater_id"], :name => "index_mail_deliveries_on_updater_id"
    add_index "mail_deliveries", ["user_id"], :name => "index_mail_deliveries_on_user_id"

    create_table "mail_templates", :force => true, :comment => 'メールテンプレート' do |t|
      t.string   "name",                                                 :comment => ""
      t.string   "mail_type_div",           :limit => 15,                :comment => ""
      t.string   "pc_subject_template",                                  :comment => ""
      t.text     "pc_text",                                              :comment => ""
      t.text     "pc_html",                                              :comment => ""
      t.string   "mobile_subject_template",                              :comment => ""
      t.text     "mobile_text",                                          :comment => ""
      t.text     "mobile_html",                                          :comment => ""
      t.datetime "created_at",                                           :comment => ""
      t.integer  "creator_id",                                           :comment => ""
      t.datetime "updated_at",                                           :comment => ""
      t.integer  "updater_id",                                           :comment => ""
      t.datetime "deleted_at",                                           :comment => ""
      t.integer  "lock_version",                          :default => 0, :comment => ""
      t.string   "content_type_div",        :limit => 15,                :comment => "メール本文形式(1:テキストのみ, 2:テキストとHTML形式)"
      t.string   "use_div",                 :limit => 2,                 :comment => "使用区分"
      t.string   "locale",                  :limit => 5,                 :comment => "言語コード"
      t.integer  "deleter_id",                                           :comment => "データ削除者"
    end

    add_index "mail_templates", ["creator_id"], :name => "index_mail_templates_on_creator_id"
    add_index "mail_templates", ["deleter_id"], :name => "index_mail_templates_on_deleter_id"
    add_index "mail_templates", ["updater_id"], :name => "index_mail_templates_on_updater_id"

    create_table "order_items", :force => true, :comment => '注文商品' do |t|
      t.integer  "order_shipping_id",                                  :comment => ""
      t.string   "product_name",         :limit => 120,                :comment => "商品名1"
      t.string   "order_product_name_1", :limit => 120,                :comment => "管理用商品名（日本語）"
      t.string   "order_product_name_2", :limit => 120,                :comment => "管理用商品名（英語）"
      t.string   "course_div",           :limit => 5,                  :comment => "コース区分(1:都度購入,2:定期購入)"
      t.integer  "unit_price",                                         :comment => ""
      t.integer  "shipping_cost",                                      :comment => ""
      t.integer  "total_amount",                                       :comment => "明細毎の合計"
      t.integer  "qty",                  :limit => 3,                  :comment => "数量"
      t.datetime "created_at",                                         :comment => ""
      t.datetime "updated_at",                                         :comment => ""
      t.datetime "deleted_at",                                         :comment => ""
      t.integer  "lock_version",                        :default => 0, :comment => ""
      t.integer  "product_sku_id",                                     :comment => "商品SKUのID"
      t.integer  "subtotal_amount",                                    :comment => "小計(単価*数量)"
      t.integer  "discount_amount",                                    :comment => "割引額"
      t.integer  "creator_id",                                         :comment => "データ登録者"
      t.integer  "updater_id",                                         :comment => "データ更新者"
      t.integer  "deleter_id",                                         :comment => "データ削除者"
      t.boolean  "out_of_stock_flg",                                   :comment => "在庫不足フラグ"
    end

    add_index "order_items", ["creator_id"], :name => "index_order_items_on_creator_id"
    add_index "order_items", ["deleter_id"], :name => "index_order_items_on_deleter_id"
    add_index "order_items", ["order_shipping_id"], :name => "index_orders_on_order_chapter_id"
    add_index "order_items", ["product_sku_id"], :name => "index_order_items_on_product_sku_id"
    add_index "order_items", ["updater_id"], :name => "index_order_items_on_updater_id"

    create_table "order_shippings", :force => true, :comment => '注文見出し' do |t|
      t.integer  "order_id",                                             :comment => ""
      t.string   "order_code",             :limit => 11,                 :comment => ""
      t.string   "status_div",             :limit => 2,                  :comment => ""
      t.string   "address_name",           :limit => 60,                 :comment => ""
      t.string   "name_last",              :limit => 30,                 :comment => ""
      t.string   "name_first",             :limit => 30,                 :comment => ""
      t.string   "kana_name_last",         :limit => 30,                 :comment => ""
      t.string   "kana_name_first",        :limit => 30,                 :comment => ""
      t.string   "tel_num",                :limit => 20,                 :comment => "電話番号"
      t.string   "zip_code",               :limit => 15,                 :comment => "郵便番号"
      t.string   "address_1",              :limit => 45,                 :comment => ""
      t.string   "address_2",              :limit => 90,                 :comment => ""
      t.string   "address_3",              :limit => 90,                 :comment => ""
      t.string   "email",                  :limit => 100,                :comment => ""
      t.datetime "arrival_date",                                         :comment => ""
      t.string   "arrival_hour",           :limit => 5,                  :comment => ""
      t.string   "payment_type_div",       :limit => 5,                  :comment => ""
      t.integer  "total_amount",                                         :comment => "総合計"
      t.string   "delivery_div",           :limit => 5,                  :comment => ""
      t.integer  "shipping_cost",                                        :comment => ""
      t.datetime "created_at",                                           :comment => ""
      t.datetime "updated_at",                                           :comment => ""
      t.datetime "deleted_at",                                           :comment => ""
      t.integer  "lock_version",                          :default => 0, :comment => ""
      t.string   "inquiry_number",         :limit => 20,                 :comment => ""
      t.integer  "complete_sent_mail_flg", :limit => 2,                  :comment => ""
      t.string   "delivery_code",                                        :comment => "発送伝票番号"
      t.string   "csv_output_div",         :limit => 2,                  :comment => "CSV出力区分"
      t.text     "delivery_content",                                     :comment => "配送に関する備考"
      t.string   "country_code",           :limit => 3,                  :comment => "国"
      t.integer  "subtotal_amount",                                      :comment => "小計(明細の合計)"
      t.integer  "discount_amount",                                      :comment => "割引額"
      t.integer  "creator_id",                                           :comment => "データ登録者"
      t.integer  "updater_id",                                           :comment => "データ更新者"
      t.integer  "deleter_id",                                           :comment => "データ削除者"
      t.integer  "distributor_id",                                       :comment => "物流ID"
      t.integer  "cod_charge",                                           :comment => "代金引換手数料"
    end

    add_index "order_shippings", ["country_code"], :name => "index_order_chapters_on_country_code"
    add_index "order_shippings", ["created_at", "deleted_at", "status_div"], :name => "index_order_chapters_on_created_at_and_deleted_at_and_status_div"
    add_index "order_shippings", ["creator_id"], :name => "index_order_shippings_on_creator_id"
    add_index "order_shippings", ["deleter_id"], :name => "index_order_shippings_on_deleter_id"
    add_index "order_shippings", ["order_id"], :name => "index_order_chapters_on_ec_order_chapter_id"
    add_index "order_shippings", ["updater_id"], :name => "index_order_shippings_on_updater_id"

    create_table "orders", :force => true, :comment => 'ECサイト注文見出し' do |t|
      t.string   "ec_order_code",               :limit => 11,                    :comment => ""
      t.integer  "user_id",                                                      :comment => ""
      t.datetime "ordered_at",                                                   :comment => ""
      t.string   "site_type_div",               :limit => 5,                     :comment => "購入サイト区分(1:PCサイト,2:モバイルサイト)"
      t.string   "free_request",                :limit => 4000,                  :comment => ""
      t.datetime "created_at",                                                   :comment => ""
      t.datetime "updated_at",                                                   :comment => ""
      t.datetime "deleted_at",                                                   :comment => ""
      t.integer  "lock_version",                                :default => 0,   :comment => ""
      t.integer  "gmo_credit_card_id",                                           :comment => ""
      t.string   "on_line_payment_exe_div",     :limit => 5,    :default => "1", :comment => ""
      t.string   "locale",                      :limit => 5,                     :comment => "言語コード"
      t.boolean  "send_invoice_separately_flg",                                  :comment => "請求書・納品書を別送する"
      t.boolean  "write_sender_flg",                                             :comment => "送状に送り主を明記する"
      t.integer  "regular_order_id",                                             :comment => "定期購入ID"
      t.integer  "regular_order_count",                                          :comment => "何回目の定期購入か"
      t.integer  "include_order_id",                                             :comment => "同送先の注文ID"
      t.integer  "include_regular_order_id",                                     :comment => "同送先の定期購入ID"
      t.boolean  "gift_flg",                                                     :comment => "ギフトフラグ"
      t.boolean  "test_flg",                                                     :comment => "テスト環境フラグ"
      t.integer  "creator_id",                                                   :comment => "データ登録者"
      t.integer  "updater_id",                                                   :comment => "データ更新者"
      t.integer  "deleter_id",                                                   :comment => "データ削除者"
      t.string   "sender_name_first",                                            :comment => "送り主 名"
      t.string   "sender_name_last",                                             :comment => "送り主 姓"
      t.string   "sender_country_code",                                          :comment => "送り主 国"
      t.string   "sender_zip_code",                                              :comment => "送り主 郵便番号"
      t.string   "sender_address_1",                                             :comment => "送り主 都道府県"
      t.string   "sender_address_2",                                             :comment => "送り主 市区町村町域"
      t.string   "sender_address_3",                                             :comment => "送り主 番地・マンション名"
      t.string   "sender_tel_num",                                               :comment => "送り主 電話番号"
    end

    add_index "orders", ["creator_id"], :name => "index_orders_on_creator_id"
    add_index "orders", ["deleter_id"], :name => "index_orders_on_deleter_id"
    add_index "orders", ["gmo_credit_card_id"], :name => "index_ec_order_chapters_on_credit_info_id"
    add_index "orders", ["include_order_id"], :name => "index_orders_on_include_order_id"
    add_index "orders", ["updater_id"], :name => "index_orders_on_updater_id"
    add_index "orders", ["user_id"], :name => "index_ec_order_chapters_on_user_id"

    create_table "payment_code_seqs", :force => true, :comment => '決済管理番号採番テーブル' do |t|
      t.integer "last_payment_code",                :null => false, :comment => "最終決済管理番号"
      t.integer "lock_version",      :default => 0,                 :comment => ""
      t.integer "creator_id",                                       :comment => "データ登録者"
      t.integer "updater_id",                                       :comment => "データ更新者"
      t.integer "deleter_id",                                       :comment => "データ削除者"
    end

    create_table "point_settings", :force => true, :comment => 'ポイント設定' do |t|
      t.boolean  "available_flg",          :comment => "ポイント制度使用フラグ"
      t.boolean  "terminable_flg",         :comment => "有効期限フラグ"
      t.string   "term_div",               :comment => "有効期限基準日区分"
      t.integer  "term_month",             :comment => "有効期限指定月"
      t.integer  "term_day",               :comment => "有効期限指定日"
      t.string   "point_period_div",       :comment => "有効期間"
      t.integer  "issue_price",            :comment => "ポイント発行"
      t.integer  "exchange_rate",          :comment => "利用時加算"
      t.boolean  "exchangeable_price_flg", :comment => "利用条件フラグ(お金)"
      t.integer  "exchangeable_price",     :comment => "利用条件(お金)"
      t.boolean  "exchangeable_point_flg", :comment => "利用条件フラグ(ポイント)"
      t.integer  "exchangeable_point",     :comment => "利用条件(ポイント)"
      t.datetime "created_at",             :comment => ""
      t.datetime "updated_at",             :comment => ""
      t.integer  "creator_id",             :comment => "データ登録者"
      t.integer  "updater_id",             :comment => "データ更新者"
      t.integer  "deleter_id",             :comment => "データ削除者"
    end

    add_index "point_settings", ["creator_id"], :name => "index_point_settings_on_creator_id"
    add_index "point_settings", ["deleter_id"], :name => "index_point_settings_on_deleter_id"
    add_index "point_settings", ["updater_id"], :name => "index_point_settings_on_updater_id"

    create_table "prefecture_displays", :force => true, :comment => '都道府県名称マスタ' do |t|
      t.integer  "prefecture_id", :comment => "都道府県マスタID"
      t.string   "name",          :comment => "県名称"
      t.string   "locale",        :comment => "言語"
      t.datetime "created_at",    :comment => ""
      t.datetime "updated_at",    :comment => ""
      t.integer  "creator_id",    :comment => "データ登録者"
      t.integer  "updater_id",    :comment => "データ更新者"
      t.integer  "deleter_id",    :comment => "データ削除者"
    end

    add_index "prefecture_displays", ["prefecture_id"], :name => "index_prefecture_displays_on_prefecture_id"

    create_table "prefectures", :force => true, :comment => '都道府県マスタ' do |t|
      t.string   "code",       :comment => "都道府県コード"
      t.datetime "created_at", :comment => ""
      t.datetime "updated_at", :comment => ""
      t.integer  "creator_id", :comment => "データ登録者"
      t.integer  "updater_id", :comment => "データ更新者"
      t.integer  "deleter_id", :comment => "データ削除者"
    end

    create_table "product_categories", :force => true, :comment => '商品カテゴリ' do |t|
      t.integer  "parent_id",                                    :comment => "親カテゴリ"
      t.string   "name_for_admin", :limit => 60, :null => false, :comment => "管理用名称"
      t.datetime "created_at",                                   :comment => ""
      t.datetime "updated_at",                                   :comment => ""
      t.integer  "display_order",                                :comment => "表示順序（parent毎）"
      t.string   "staff_emails",                                 :comment => "担当者メールアドレス"
      t.string   "url",                                          :comment => "URL"
      t.string   "code",           :limit => 10, :null => false, :comment => "カテゴリコード"
      t.integer  "creator_id",                                   :comment => "データ登録者"
      t.integer  "updater_id",                                   :comment => "データ更新者"
      t.integer  "deleter_id",                                   :comment => "データ削除者"
    end

    add_index "product_categories", ["parent_id", "display_order", "id"], :name => "index_product_categories_on_parent_id_and_display_order_and_id"
    add_index "product_categories", ["parent_id"], :name => "index_product_categories_on_parent_id"

    create_table "product_category_displays", :force => true, :comment => '商品カテゴリ名称' do |t|
      t.integer  "product_category_id",               :null => false, :comment => "商品カテゴリ"
      t.string   "locale",              :limit => 5,  :null => false, :comment => "言語コード"
      t.string   "name",                :limit => 60, :null => false, :comment => "名称"
      t.datetime "created_at",                                        :comment => ""
      t.datetime "updated_at",                                        :comment => ""
      t.integer  "creator_id",                                        :comment => "データ登録者"
      t.integer  "updater_id",                                        :comment => "データ更新者"
      t.integer  "deleter_id",                                        :comment => "データ削除者"
    end

    add_index "product_category_displays", ["product_category_id"], :name => "index_product_category_displays_on_product_category_id"

    create_table "product_components", :force => true, :comment => '商品構成' do |t|
      t.datetime "created_at",       :comment => ""
      t.datetime "updated_at",       :comment => ""
      t.integer  "product_sku_id",   :comment => "商品SKU"
      t.integer  "count",            :comment => "数量"
      t.integer  "component_sku_id", :comment => "子商品SKUのID"
      t.integer  "creator_id",       :comment => "データ登録者"
      t.integer  "updater_id",       :comment => "データ更新者"
      t.integer  "deleter_id",       :comment => "データ削除者"
    end

    add_index "product_components", ["component_sku_id"], :name => "index_product_components_on_component_sku_id"
    add_index "product_components", ["product_sku_id"], :name => "index_product_components_on_product_sku_id"

    create_table "product_displays", :force => true, :comment => '商品表示マスタ' do |t|
      t.integer  "product_id",                          :comment => "商品マスタ"
      t.string   "locale",              :limit => 5,    :comment => "言語コード"
      t.string   "product_name_1",      :limit => 120,  :comment => "商品名1"
      t.string   "product_name_2",      :limit => 120,  :comment => "商品名2"
      t.string   "product_outline",     :limit => 4000, :comment => "商品概要"
      t.string   "product_explanation", :limit => 4000, :comment => "商品説明"
      t.string   "normal_explanation",  :limit => 400,  :comment => "通常商品の説明"
      t.string   "regular_explanation", :limit => 400,  :comment => "定期商品の説明"
      t.string   "recommendation",      :limit => 400,  :comment => "推奨メッセージ"
      t.string   "recommendation_2",    :limit => 400,  :comment => "推奨メッセージ2"
      t.datetime "created_at",                          :comment => ""
      t.datetime "updated_at",                          :comment => ""
      t.string   "product_keyword",                     :comment => "検索キーワード"
      t.integer  "creator_id",                          :comment => "データ登録者"
      t.integer  "updater_id",                          :comment => "データ更新者"
      t.integer  "deleter_id",                          :comment => "データ削除者"
    end

    add_index "product_displays", ["locale"], :name => "index_product_displays_on_locale"
    add_index "product_displays", ["product_id", "locale"], :name => "index_product_displays_on_product_id_and_locale"
    add_index "product_displays", ["product_id"], :name => "index_product_displays_on_product_id"

    create_table "product_faq_displays", :force => true, :comment => '商品Q&A内容' do |t|
      t.integer  "product_faq_id",              :comment => "商品Q&AID"
      t.string   "locale",         :limit => 5, :comment => "ロケール"
      t.string   "inquiry_user",                :comment => "質問者"
      t.string   "inquiry",                     :comment => "質問内容"
      t.string   "answer_user",                 :comment => "回答者"
      t.string   "answer",                      :comment => "回答内容"
      t.datetime "created_at",                  :comment => ""
      t.datetime "updated_at",                  :comment => ""
      t.integer  "creator_id",                  :comment => "データ登録者"
      t.integer  "updater_id",                  :comment => "データ更新者"
      t.integer  "deleter_id",                  :comment => "データ削除者"
    end

    add_index "product_faq_displays", ["product_faq_id"], :name => "index_product_faq_displays_on_product_faq_id"

    create_table "product_faq_relation_products", :force => true, :comment => '商品Q&A対象商品' do |t|
      t.integer  "product_faq_id", :comment => "商品Q&AID"
      t.integer  "product_id",     :comment => "商品ID"
      t.datetime "created_at",     :comment => ""
      t.datetime "updated_at",     :comment => ""
      t.integer  "creator_id",     :comment => "データ登録者"
      t.integer  "updater_id",     :comment => "データ更新者"
      t.integer  "deleter_id",     :comment => "データ削除者"
    end

    add_index "product_faq_relation_products", ["product_faq_id"], :name => "index_product_faq_relation_products_on_product_faq_id"
    add_index "product_faq_relation_products", ["product_id"], :name => "index_product_faq_relation_products_on_product_id"

    create_table "product_faqs", :force => true, :comment => '商品Q&A' do |t|
      t.integer  "display_order", :comment => "表示順"
      t.date     "answer_date",   :comment => "回答日"
      t.boolean  "invisible_flg", :comment => "非表示フラグ"
      t.datetime "created_at",    :comment => ""
      t.datetime "updated_at",    :comment => ""
      t.integer  "creator_id",    :comment => "データ登録者"
      t.integer  "updater_id",    :comment => "データ更新者"
      t.integer  "deleter_id",    :comment => "データ削除者"
    end

    add_index "product_faqs", ["invisible_flg", "display_order"], :name => "index_product_faqs_on_invisible_flg_and_display_order"
    add_index "product_faqs", ["invisible_flg"], :name => "index_product_faqs_on_invisible_flg"

    create_table "product_group_skus", :force => true, :comment => '関連商品グループSKU' do |t|
      t.integer  "product_sku_id",   :comment => "商品SKU"
      t.integer  "product_group_id", :comment => "関連商品グループ"
      t.datetime "created_at",       :comment => ""
      t.datetime "updated_at",       :comment => ""
      t.integer  "creator_id",       :comment => "データ登録者"
      t.integer  "updater_id",       :comment => "データ更新者"
      t.integer  "deleter_id",       :comment => "データ削除者"
    end

    add_index "product_group_skus", ["product_group_id"], :name => "index_product_group_skus_on_product_group_id"
    add_index "product_group_skus", ["product_sku_id"], :name => "index_product_group_skus_on_product_sku_id"

    create_table "product_groups", :force => true, :comment => '商品グループ' do |t|
      t.datetime "created_at", :comment => ""
      t.datetime "updated_at", :comment => ""
      t.string   "name",       :comment => "関連商品グループ名"
      t.integer  "priority",   :comment => "優先度"
      t.integer  "creator_id", :comment => "データ登録者"
      t.integer  "updater_id", :comment => "データ更新者"
      t.integer  "deleter_id", :comment => "データ削除者"
    end

    create_table "product_images", :force => true, :comment => '商品画像' do |t|
      t.integer  "product_id",    :comment => "商品"
      t.integer  "display_order", :comment => "表示順序"
      t.integer  "parent_id",     :comment => "元画像"
      t.string   "thumbnail",     :comment => "サムネイルの種類"
      t.string   "filename",      :comment => "ファイル名"
      t.string   "content_type",  :comment => "コンテンツの種類"
      t.integer  "size",          :comment => "ファイルサイズ"
      t.integer  "width",         :comment => "画像の幅"
      t.integer  "height",        :comment => "画像の高さ"
      t.datetime "created_at",    :comment => ""
      t.datetime "updated_at",    :comment => ""
      t.boolean  "main_flg",      :comment => "メイン画像フラグ"
      t.integer  "creator_id",    :comment => "データ登録者"
      t.integer  "updater_id",    :comment => "データ更新者"
      t.integer  "deleter_id",    :comment => "データ削除者"
    end

    add_index "product_images", ["parent_id"], :name => "index_product_images_on_parent_id"
    add_index "product_images", ["product_id"], :name => "index_product_images_on_product_id"

    create_table "product_prices", :force => true, :comment => '商品売価' do |t|
      t.integer  "product_sku_id",         :comment => "商品SKU"
      t.integer  "user_rank_id",           :comment => "会員ランク"
      t.datetime "created_at",             :comment => ""
      t.datetime "updated_at",             :comment => ""
      t.integer  "standard_price",         :comment => "標準売価"
      t.integer  "regular_purchase_price", :comment => "定期購入売価"
      t.integer  "creator_id",             :comment => "データ登録者"
      t.integer  "updater_id",             :comment => "データ更新者"
      t.integer  "deleter_id",             :comment => "データ削除者"
    end

    add_index "product_prices", ["product_sku_id", "user_rank_id"], :name => "index_product_prices_on_product_sku_id_and_user_rank_id"
    add_index "product_prices", ["product_sku_id"], :name => "index_product_prices_on_product_sku_id"
    add_index "product_prices", ["user_rank_id"], :name => "index_product_prices_on_user_rank_id"

    create_table "product_sku_inventories", :force => true, :comment => '商品在庫' do |t|
      t.integer  "product_sku_id", :null => false, :comment => "商品SKU"
      t.integer  "count",          :null => false, :comment => "入出庫数"
      t.string   "comment",                        :comment => "コメント"
      t.datetime "created_at",                     :comment => ""
      t.datetime "updated_at",                     :comment => ""
      t.integer  "creator_id",                     :comment => "データ登録者"
      t.integer  "updater_id",                     :comment => "データ更新者"
      t.integer  "deleter_id",                     :comment => "データ削除者"
    end

    add_index "product_sku_inventories", ["product_sku_id"], :name => "index_product_sku_inventories_on_product_sku_id"

    create_table "product_sku_secure_inventories", :force => true, :comment => '引き当て済み在庫' do |t|
      t.integer  "product_sku_id",                    :null => false, :comment => "商品SKU"
      t.integer  "count",                             :null => false, :comment => "引き当て数"
      t.integer  "order_item_id",                     :null => false, :comment => "注文商品"
      t.datetime "deleted_at",                                        :comment => ""
      t.datetime "created_at",                                        :comment => ""
      t.datetime "updated_at",                                        :comment => ""
      t.integer  "creator_id",                                        :comment => "データ登録者"
      t.integer  "updater_id",                                        :comment => "データ更新者"
      t.integer  "deleter_id",                                        :comment => "データ削除者"
      t.boolean  "paid_flg",       :default => false,                 :comment => "決済済みフラグ"
    end

    add_index "product_sku_secure_inventories", ["order_item_id"], :name => "index_product_sku_secure_inventories_on_order_item_id"
    add_index "product_sku_secure_inventories", ["product_sku_id"], :name => "index_product_sku_secure_inventories_on_product_sku_id"

    create_table "product_skus", :force => true, :comment => '商品SKU' do |t|
      t.integer  "product_id",                                            :null => false, :comment => "商品"
      t.integer  "variation_item_1_id",                                                   :comment => "バリエーション項目1"
      t.integer  "variation_item_2_id",                                                   :comment => "バリエーション項目2"
      t.integer  "variation_item_3_id",                                                   :comment => "バリエーション項目3"
      t.datetime "created_at",                                                            :comment => ""
      t.datetime "updated_at",                                                            :comment => ""
      t.string   "sku_code",             :limit => 22,                                    :comment => "SKUコード(商品コード-枝番1-枝番2-枝番3)"
      t.boolean  "primary_sku_flg",                    :default => false,                 :comment => "代表SKUフラグ"
      t.datetime "deleted_at",                                                            :comment => ""
      t.string   "jan_code",                                                              :comment => "JANコード"
      t.string   "shelf_no",                                                              :comment => "棚番（あくまで目安です）"
      t.integer  "maker_price",                                                           :comment => "メーカー希望価格"
      t.integer  "lead_time",                                                             :comment => "出荷リードタイム"
      t.integer  "order_limit",                                                           :comment => "購入数上限"
      t.integer  "weight",                                                                :comment => "重量（ｇ）"
      t.integer  "usage_days",                                                            :comment => "何日分（サプリメントとかの消耗品）"
      t.integer  "safety_stock_count",                                                    :comment => "安全在庫数"
      t.integer  "shipping_cost_point",                                                   :comment => "送料ポイント"
      t.integer  "includable_count",                                                      :comment => "同梱可能数"
      t.boolean  "regular_purchase_flg",                                                  :comment => "定期購入可能フラグ"
      t.string   "stockout_display_cd",                                                   :comment => "在庫切れ時の表示"
      t.boolean  "parent_stockable_flg",                                                  :comment => "親商品に在庫を持つかどうか"
      t.integer  "creator_id",                                                            :comment => "データ登録者"
      t.integer  "updater_id",                                                            :comment => "データ更新者"
      t.integer  "deleter_id",                                                            :comment => "データ削除者"
    end

    add_index "product_skus", ["creator_id"], :name => "index_product_skus_on_creator_id"
    add_index "product_skus", ["deleter_id"], :name => "index_product_skus_on_deleter_id"
    add_index "product_skus", ["product_id"], :name => "index_product_skus_on_product_id"
    add_index "product_skus", ["sku_code", "deleted_at"], :name => "index_product_skus_on_sku_code_and_deleted_at"
    add_index "product_skus", ["updater_id"], :name => "index_product_skus_on_updater_id"
    add_index "product_skus", ["variation_item_1_id"], :name => "index_product_skus_on_variation_item_1_id"
    add_index "product_skus", ["variation_item_2_id"], :name => "index_product_skus_on_variation_item_2_id"
    add_index "product_skus", ["variation_item_3_id"], :name => "index_product_skus_on_variation_item_3_id"

    create_table "product_variation_displays", :force => true, :comment => '商品バリエーション名称' do |t|
      t.integer  "product_variation_id",                               :comment => "商品バリエーション"
      t.string   "locale",               :limit => 5,  :null => false, :comment => "言語コード"
      t.string   "name",                 :limit => 60, :null => false, :comment => "名称"
      t.datetime "created_at",                                         :comment => ""
      t.datetime "updated_at",                                         :comment => ""
      t.integer  "creator_id",                                         :comment => "データ登録者"
      t.integer  "updater_id",                                         :comment => "データ更新者"
      t.integer  "deleter_id",                                         :comment => "データ削除者"
    end

    add_index "product_variation_displays", ["product_variation_id"], :name => "index_product_variation_displays_on_product_variation_id"

    create_table "product_variation_item_displays", :force => true, :comment => '商品バリエーション項目名称' do |t|
      t.integer  "product_variation_item_id",                               :comment => "商品バリエーション項目"
      t.string   "locale",                    :limit => 5,  :null => false, :comment => "言語コード"
      t.string   "name",                      :limit => 60, :null => false, :comment => "名称"
      t.datetime "created_at",                                              :comment => ""
      t.datetime "updated_at",                                              :comment => ""
      t.integer  "creator_id",                                              :comment => "データ登録者"
      t.integer  "updater_id",                                              :comment => "データ更新者"
      t.integer  "deleter_id",                                              :comment => "データ削除者"
    end

    add_index "product_variation_item_displays", ["product_variation_item_id"], :name => "index_product_variation_item_displays_on_01"

    create_table "product_variation_items", :force => true, :comment => '商品バリエーション項目' do |t|
      t.integer  "product_variation_id", :null => false, :comment => "商品バリエーション"
      t.integer  "display_order",        :null => false, :comment => "表示順"
      t.string   "code",                 :null => false, :comment => "コード"
      t.datetime "created_at",                           :comment => ""
      t.datetime "updated_at",                           :comment => ""
      t.integer  "creator_id",                           :comment => "データ登録者"
      t.integer  "updater_id",                           :comment => "データ更新者"
      t.integer  "deleter_id",                           :comment => "データ削除者"
    end

    add_index "product_variation_items", ["creator_id"], :name => "index_product_variation_items_on_creator_id"
    add_index "product_variation_items", ["deleter_id"], :name => "index_product_variation_items_on_deleter_id"
    add_index "product_variation_items", ["product_variation_id"], :name => "index_product_variation_items_on_product_variation_id"
    add_index "product_variation_items", ["updater_id"], :name => "index_product_variation_items_on_updater_id"

    create_table "product_variations", :force => true, :comment => '商品バリエーション' do |t|
      t.string   "name_for_admin", :limit => 60, :null => false, :comment => "管理用名称"
      t.datetime "created_at",                                   :comment => ""
      t.datetime "updated_at",                                   :comment => ""
      t.integer  "creator_id",                                   :comment => "データ登録者"
      t.integer  "updater_id",                                   :comment => "データ更新者"
      t.integer  "deleter_id",                                   :comment => "データ削除者"
    end

    create_table "products", :force => true, :comment => '商品マスタ' do |t|
      t.string   "product_code",         :limit => 13,                 :comment => ""
      t.string   "order_product_name_1", :limit => 120,                :comment => "管理用商品名（日本語）"
      t.string   "order_product_name_2", :limit => 120,                :comment => "管理用商品名（英語）"
      t.string   "kana_product_name",    :limit => 120,                :comment => "管理用商品名（カナ）"
      t.string   "course_div",           :limit => 5,                  :comment => "コース区分"
      t.datetime "created_at",                                         :comment => ""
      t.datetime "updated_at",                                         :comment => ""
      t.datetime "deleted_at",                                         :comment => ""
      t.integer  "lock_version",                        :default => 0, :comment => ""
      t.string   "detail_url",                                         :comment => "詳細画面URL"
      t.integer  "display_order",                                      :comment => "表示順"
      t.string   "origin",               :limit => 4,                  :comment => "原産国"
      t.integer  "category_1_id",                                      :comment => "カテゴリ1"
      t.integer  "category_2_id",                                      :comment => "カテゴリ2"
      t.integer  "category_3_id",                                      :comment => "カテゴリ3"
      t.integer  "category_4_id",                                      :comment => "カテゴリ4"
      t.integer  "category_5_id",                                      :comment => "カテゴリ5"
      t.integer  "variation_1_id",                                     :comment => "バリエーション1"
      t.integer  "variation_2_id",                                     :comment => "バリエーション2"
      t.integer  "variation_3_id",                                     :comment => "バリエーション3"
      t.boolean  "present_flg",                                        :comment => "プレゼント品フラグ"
      t.datetime "sale_started_at",                                    :comment => "販売開始日時"
      t.datetime "publish_started_at",                                 :comment => "公開開始日時"
      t.datetime "publish_finished_at",                                :comment => "公開終了日時"
      t.boolean  "invisible_flg",                                      :comment => "公開フラグ"
      t.integer  "creator_id",                                         :comment => "データ登録者"
      t.integer  "updater_id",                                         :comment => "データ更新者"
      t.integer  "deleter_id",                                         :comment => "データ削除者"
    end

    add_index "products", ["category_1_id"], :name => "index_products_on_category_1_id"
    add_index "products", ["category_2_id"], :name => "index_products_on_category_2_id"
    add_index "products", ["category_3_id"], :name => "index_products_on_category_3_id"
    add_index "products", ["category_4_id"], :name => "index_products_on_category_4_id"
    add_index "products", ["category_5_id"], :name => "index_products_on_category_5_id"
    add_index "products", ["creator_id"], :name => "index_products_on_creator_id"
    add_index "products", ["deleted_at", "display_order"], :name => "index_products_on_deleted_at_and_display_order"
    add_index "products", ["deleted_at"], :name => "index_products_on_deleted_at"
    add_index "products", ["deleter_id"], :name => "index_products_on_deleter_id"
    add_index "products", ["display_order", "product_code"], :name => "index_products_on_display_order_and_product_code"
    add_index "products", ["origin"], :name => "index_products_on_origin"
    add_index "products", ["updater_id"], :name => "index_products_on_updater_id"
    add_index "products", ["variation_1_id"], :name => "index_products_on_variation_1_id"
    add_index "products", ["variation_2_id"], :name => "index_products_on_variation_2_id"
    add_index "products", ["variation_3_id"], :name => "index_products_on_variation_3_id"

    create_table "resigned_reasons", :force => true, :comment => '退会理由' do |t|
      t.integer  "user_id",                                              :comment => ""
      t.integer  "ms_resigned_reason_id",                                :comment => ""
      t.string   "reason",                :limit => 4000,                :comment => ""
      t.datetime "created_at",                                           :comment => ""
      t.datetime "updated_at",                                           :comment => ""
      t.datetime "deleted_at",                                           :comment => ""
      t.integer  "lock_version",                          :default => 0, :comment => ""
      t.string   "sex",                   :limit => 5,                   :comment => "性別"
      t.integer  "age",                   :limit => 3,                   :comment => "年齢"
      t.string   "job_div",               :limit => 5,                   :comment => "職業区分"
      t.datetime "admission_at",                                         :comment => "入会日"
      t.datetime "resigened_at",                                         :comment => "退会日"
      t.integer  "creator_id",                                           :comment => "データ登録者"
      t.integer  "updater_id",                                           :comment => "データ更新者"
      t.integer  "deleter_id",                                           :comment => "データ削除者"
    end

    add_index "resigned_reasons", ["creator_id"], :name => "index_resigned_reasons_on_creator_id"
    add_index "resigned_reasons", ["deleter_id"], :name => "index_resigned_reasons_on_deleter_id"
    add_index "resigned_reasons", ["updater_id"], :name => "index_resigned_reasons_on_updater_id"
    add_index "resigned_reasons", ["user_id"], :name => "index_resigned_reasons_on_user_id"

    create_table "sessions", :force => true, :comment => 'セッション情報' do |t|
      t.string   "session_id",                       :comment => ""
      t.text     "data",       :limit => 2147483647, :comment => ""
      t.datetime "updated_at",                       :comment => ""
    end

    add_index "sessions", ["session_id"], :name => "index_sessions_on_session_id"
    add_index "sessions", ["updated_at"], :name => "index_sessions_on_updated_at"

    create_table "shipping_cost_boundaries", :force => true, :comment => '送料閾値マスタ' do |t|
      t.integer  "code",       :comment => "閾値コード"
      t.integer  "point",      :comment => "閾値"
      t.datetime "created_at", :comment => ""
      t.datetime "updated_at", :comment => ""
      t.integer  "creator_id", :comment => "データ登録者"
      t.integer  "updater_id", :comment => "データ更新者"
      t.integer  "deleter_id", :comment => "データ削除者"
    end

    create_table "shipping_costs", :force => true, :comment => '送料' do |t|
      t.string   "area_type_div",             :comment => "地域区分"
      t.string   "area_code",                 :comment => "地域コード"
      t.integer  "shipping_cost",             :comment => "送料"
      t.integer  "shipping_cost_boundary_id", :comment => "閾値マスタID"
      t.datetime "created_at",                :comment => ""
      t.datetime "updated_at",                :comment => ""
      t.integer  "creator_id",                :comment => "データ登録者"
      t.integer  "updater_id",                :comment => "データ更新者"
      t.integer  "deleter_id",                :comment => "データ削除者"
    end

    add_index "shipping_costs", ["area_code"], :name => "index_shipping_costs_on_area_code"
    add_index "shipping_costs", ["creator_id"], :name => "index_shipping_costs_on_creator_id"
    add_index "shipping_costs", ["deleter_id"], :name => "index_shipping_costs_on_deleter_id"
    add_index "shipping_costs", ["shipping_cost_boundary_id"], :name => "index_shipping_costs_on_shipping_cost_boundary_id"
    add_index "shipping_costs", ["updater_id"], :name => "index_shipping_costs_on_updater_id"

    create_table "shop_settings", :force => true, :comment => '店舗情報マスタ' do |t|
      t.string   "setting_div", :limit => 4,    :comment => "設定区分"
      t.string   "locale",      :limit => 5,    :comment => "言語コード"
      t.string   "value",       :limit => 2000, :comment => "値"
      t.datetime "created_at",                  :comment => ""
      t.datetime "updated_at",                  :comment => ""
      t.integer  "creator_id",                  :comment => "データ登録者"
      t.integer  "updater_id",                  :comment => "データ更新者"
      t.integer  "deleter_id",                  :comment => "データ削除者"
    end

    add_index "shop_settings", ["creator_id"], :name => "index_shop_settings_on_creator_id"
    add_index "shop_settings", ["deleter_id"], :name => "index_shop_settings_on_deleter_id"
    add_index "shop_settings", ["setting_div", "locale"], :name => "index_shop_settings_on_setting_div_and_locale"
    add_index "shop_settings", ["updater_id"], :name => "index_shop_settings_on_updater_id"

    create_table "user_addresses", :force => true, :comment => '住所' do |t|
      t.integer  "user_id",                                      :comment => ""
      t.string   "name_last",       :limit => 30,                :comment => ""
      t.string   "name_first",      :limit => 30,                :comment => ""
      t.string   "kana_name_last",  :limit => 30,                :comment => ""
      t.string   "kana_name_first", :limit => 30,                :comment => ""
      t.string   "tel_num",         :limit => 20,                :comment => "電話番号"
      t.string   "zip_code",        :limit => 15,                :comment => "郵便番号"
      t.string   "address_1",       :limit => 45,                :comment => ""
      t.string   "address_2",       :limit => 90,                :comment => ""
      t.string   "address_3",       :limit => 90,                :comment => ""
      t.datetime "created_at",                                   :comment => ""
      t.datetime "updated_at",                                   :comment => ""
      t.datetime "deleted_at",                                   :comment => ""
      t.integer  "lock_version",                  :default => 0, :comment => ""
      t.string   "address_div",                                  :comment => "アドレス種類区分（0:会員の住所・1:配送先の住所）"
      t.string   "country_code",    :limit => 3,                 :comment => "国"
      t.string   "locale",          :limit => 5,                 :comment => "言語コード"
      t.integer  "creator_id",                                   :comment => "データ登録者"
      t.integer  "updater_id",                                   :comment => "データ更新者"
      t.integer  "deleter_id",                                   :comment => "データ削除者"
    end

    add_index "user_addresses", ["country_code"], :name => "index_addresses_on_country_code"
    add_index "user_addresses", ["creator_id"], :name => "index_user_addresses_on_creator_id"
    add_index "user_addresses", ["deleter_id"], :name => "index_user_addresses_on_deleter_id"
    add_index "user_addresses", ["updater_id"], :name => "index_user_addresses_on_updater_id"
    add_index "user_addresses", ["user_id"], :name => "index_addresses_on_user_id"

    create_table "user_emails", :force => true, :comment => 'メールアドレス' do |t|
      t.integer  "user_id",                                                  :comment => ""
      t.string   "email",                      :limit => 100,                :comment => ""
      t.string   "pc_mobile_div",              :limit => 5,                  :comment => ""
      t.datetime "created_at",                                               :comment => ""
      t.datetime "updated_at",                                               :comment => ""
      t.datetime "deleted_at",                                               :comment => ""
      t.integer  "lock_version",                              :default => 0, :comment => ""
      t.integer  "undeliverable_flg",          :limit => 2,   :default => 0, :comment => ""
      t.string   "mail_address_pc_mobile_div", :limit => 15,                 :comment => ""
      t.integer  "creator_id",                                               :comment => "データ登録者"
      t.integer  "updater_id",                                               :comment => "データ更新者"
      t.integer  "deleter_id",                                               :comment => "データ削除者"
    end

    add_index "user_emails", ["creator_id"], :name => "index_user_emails_on_creator_id"
    add_index "user_emails", ["deleter_id"], :name => "index_user_emails_on_deleter_id"
    add_index "user_emails", ["updater_id"], :name => "index_user_emails_on_updater_id"
    add_index "user_emails", ["user_id"], :name => "index_email_addresses_on_user_id"

    create_table "user_points", :force => true, :comment => '会員ポイント履歴' do |t|
      t.integer  "user_id",                        :comment => "会員ID"
      t.integer  "order_id",                       :comment => "注文ID"
      t.datetime "term_date",                      :comment => "ポイント有効期限"
      t.integer  "given_point", :default => 0,     :comment => "発行ポイント"
      t.integer  "used_point",  :default => 0,     :comment => "利用ポイント"
      t.string   "remark",                         :comment => "備考欄"
      t.datetime "created_at",                     :comment => ""
      t.datetime "updated_at",                     :comment => ""
      t.boolean  "expired_flg", :default => false, :comment => "有効期限切れフラグ"
      t.integer  "creator_id",                     :comment => "データ登録者"
      t.integer  "updater_id",                     :comment => "データ更新者"
      t.integer  "deleter_id",                     :comment => "データ削除者"
    end

    add_index "user_points", ["creator_id"], :name => "index_user_points_on_creator_id"
    add_index "user_points", ["deleter_id"], :name => "index_user_points_on_deleter_id"
    add_index "user_points", ["order_id"], :name => "index_user_points_on_order_id"
    add_index "user_points", ["updater_id"], :name => "index_user_points_on_updater_id"
    add_index "user_points", ["user_id"], :name => "index_user_points_on_user_id"

    create_table "user_rank_conditions", :force => true, :comment => '会員ランク条件' do |t|
      t.integer  "user_rank_id", :comment => "会員ランクID"
      t.integer  "no",           :comment => "順番"
      t.string   "type_cd",      :comment => "項目種"
      t.string   "operator_cd",  :comment => "演算子"
      t.string   "connector_cd", :comment => "接続演算子"
      t.string   "value",        :comment => "値"
      t.datetime "created_at",   :comment => ""
      t.datetime "updated_at",   :comment => ""
    end

    create_table "user_rank_displays", :force => true, :comment => '会員ランク表示' do |t|
      t.integer  "user_rank_id", :comment => "会員ランクID"
      t.string   "locale",       :comment => "ロケール"
      t.string   "name",         :comment => "表示名"
      t.datetime "created_at",   :comment => ""
      t.datetime "updated_at",   :comment => ""
    end

    create_table "user_ranks", :force => true, :comment => '会員ランク' do |t|
      t.string   "name",                                                 :comment => "会員ランク名"
      t.datetime "created_at",                                           :comment => ""
      t.datetime "updated_at",                                           :comment => ""
      t.boolean  "no_membership_flg", :default => false, :null => false, :comment => "非会員ランクフラグ"
      t.integer  "creator_id",                                           :comment => "データ登録者"
      t.integer  "updater_id",                                           :comment => "データ更新者"
      t.integer  "deleter_id",                                           :comment => "データ削除者"
      t.string   "code",                                                 :comment => "コード"
      t.string   "available_div",                                        :comment => "使用区分"
      t.integer  "point_rate",                                           :comment => ""
    end

    add_index "user_ranks", ["creator_id"], :name => "index_user_ranks_on_creator_id"
    add_index "user_ranks", ["deleter_id"], :name => "index_user_ranks_on_deleter_id"
    add_index "user_ranks", ["no_membership_flg"], :name => "index_user_ranks_on_no_membership_flg"
    add_index "user_ranks", ["updater_id"], :name => "index_user_ranks_on_updater_id"

    create_table "users", :force => true, :comment => '会員' do |t|
      t.string   "user_code",                     :limit => 12,                                    :comment => ""
      t.string   "name_last",                     :limit => 30,                    :null => false, :comment => ""
      t.string   "name_first",                    :limit => 30,                    :null => false, :comment => ""
      t.string   "kana_name_last",                                                                 :comment => "フリガナ（姓）"
      t.string   "kana_name_first",                                                                :comment => "フリガナ（名）"
      t.string   "tel_num",                       :limit => 20,                    :null => false, :comment => "電話番号"
      t.integer  "birth_year",                                                                     :comment => ""
      t.integer  "birth_month",                   :limit => 2,                                     :comment => ""
      t.integer  "birth_day",                     :limit => 2,                                     :comment => ""
      t.datetime "resigened_at",                                                                   :comment => ""
      t.string   "sex",                           :limit => 5,                                     :comment => ""
      t.string   "member_div",                    :limit => 5,                     :null => false, :comment => "ユーザー区分(会員・非会員)"
      t.string   "login",                                                                          :comment => ""
      t.string   "crypted_password",                                                               :comment => ""
      t.string   "salt",                                                                           :comment => ""
      t.string   "remember_answer",               :limit => 90,                                    :comment => ""
      t.string   "remember_token",                                                                 :comment => ""
      t.string   "remember_token_expires_at",                                                      :comment => ""
      t.string   "regist_div",                    :limit => 5,                                     :comment => ""
      t.string   "job_div",                       :limit => 5,                                     :comment => ""
      t.integer  "dm_send_div",                   :limit => 2,                                     :comment => ""
      t.integer  "csv_output_div",                :limit => 2,                                     :comment => ""
      t.datetime "created_at",                                                                     :comment => ""
      t.datetime "updated_at",                                                                     :comment => ""
      t.datetime "deleted_at",                                                                     :comment => ""
      t.integer  "lock_version",                                :default => 0,                     :comment => ""
      t.string   "remember_question",                                                              :comment => "秘密の質問"
      t.string   "pc_mail_magazine_send_div",     :limit => 5,                                     :comment => "PCメールマガジン配信区分"
      t.string   "mobile_mail_magazine_send_div", :limit => 5,                                     :comment => "モバイルメールマガジン配信区分"
      t.boolean  "test_flg",                                                                       :comment => "テスト環境フラグ"
      t.integer  "user_rank_id",                                                                   :comment => "会員ランク"
      t.string   "member_category_div",           :limit => 5,  :default => "000", :null => false, :comment => "会員種別"
      t.integer  "login_challenge_count",                                                          :comment => "ログイン試行回数"
      t.datetime "locked_at",                                                                      :comment => "ログインロック日時"
      t.integer  "creator_id",                                                                     :comment => "データ登録者"
      t.integer  "updater_id",                                                                     :comment => "データ更新者"
      t.integer  "deleter_id",                                                                     :comment => "データ削除者"
    end

    add_index "users", ["creator_id"], :name => "index_users_on_creator_id"
    add_index "users", ["deleter_id"], :name => "index_users_on_deleter_id"
    add_index "users", ["updater_id"], :name => "index_users_on_updater_id"
    add_index "users", ["user_rank_id"], :name => "index_users_on_user_rank_id"

    create_table "versions", :force => true do |t|
      t.integer  "versioned_id",   :comment => ""
      t.string   "versioned_type", :comment => ""
      t.text     "changes",        :comment => ""
      t.integer  "number",         :comment => ""
      t.datetime "created_at",     :comment => ""
      t.integer  "creator_id",     :comment => ""
    end

    add_index "versions", ["created_at"], :name => "index_versions_on_created_at"
    add_index "versions", ["creator_id"], :name => "index_versions_on_creator_id"
    add_index "versions", ["number"], :name => "index_versions_on_number"
    add_index "versions", ["versioned_id"], :name => "index_versions_on_versioned_id"
    add_index "versions", ["versioned_type", "versioned_id"], :name => "index_versions_on_versioned_type_and_versioned_id"

    ActiveRecord::Base.connection.execute(<<-EOS)
    CREATE
    or REPLACE VIEW `product_sku_current_inventories` AS (select `psi`.`product_sku_id` AS `product_sku_id`,
    sum(`psi`.`count`) AS `stock_count`,
    coalesce((select sum(`pssi`.`count`) AS `SUM(count)`
    from
    `product_sku_secure_inventories` `pssi`
    where
    (isnull(`pssi`.`deleted_at`)
    and (`pssi`.`product_sku_id` = `psi`.`product_sku_id`)
    and (`pssi`.`paid_flg` = 1))),0) AS `secure_stock_count`,
    coalesce((select sum(`pssi`.`count`) AS `SUM(count)`
    from
    `product_sku_secure_inventories` `pssi`
    where
    (isnull(`pssi`.`deleted_at`)
    and (`pssi`.`product_sku_id` = `psi`.`product_sku_id`)
    and (`pssi`.`paid_flg` = 0))),0) AS `unpaid_secure_stock_count`
    from
    `product_sku_inventories` `psi` group by `psi`.`product_sku_id`)
    EOS

  end

end
