# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper
    module EzloginFormTag
      unloadable
      
      extend ::EcRider::EcHelper::MethodDescription

      desc <<-EOS
        [概要:かんたんログインフォーム。]
        [引数:なし]
        [使用例:
           {{{
           <ec:ezlogin_form>
             <ec:submit>かんたんログイン</ec:submit>
           </ec:ezlogin_form>
           }}}
        ]
      EOS
      def ezlogin_form(&block)
        # utnはdocomoで必要
        @template.form_for(:user_mobile, :url => {:controller => 'account', :action => :authenticate_ez}, :html=>{:utn => 'utn'}) do |f|
          @ezlogin_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @ezlogin_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:かんたんログイン設定フォーム。]
        [引数:なし]
        [使用例:
           {{{
           <ec:ezlogin_config_form>
             <ec:if_mobile_unique_id_get>
             </ec:if_mobile_unique_id_get>
             <ec:submit>設定する</ec:submit>
           </ec:ezlogin_config_form>
           }}}
        ]
      EOS
      def ezlogin_config_form(&block)
        # utnはdocomoで必要
        @template.form_for(:user_mobile, :url => {:controller => 'member', :action => :complete_config_ezlogin}, :html=>{:utn => 'utn'}) do |f|
          @ezlogin_config_form = f
          begin
            @template.concat(@template.hidden_field_tag("mobile_unique_id", @template.instance_variable_get(:@mobile_unique_id)))
            @template.concat(@template.capture(&block))
          ensure
            @ezlogin_config_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:携帯電話の固有IDが取得できたならば、中身を実行します。。ec:ezlogin_config_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:ezlogin_config_form>
             <ec:if_mobile_unique_id_get>
             </ec:if_mobile_unique_id_get>
           </ec:ezlogin_config_form>
           }}}
        ]
      EOS
      def if_mobile_unique_id_get
        raise ::EcRider::EcHelper::UsageError, "ec:if_mobile_unique_id_getはec:ezlogin_config_formの中で使用してください" unless @ezlogin_config_form
        unless @template.instance_variable_get(:@mobile_unique_id).nil?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:携帯電話の固有IDが取得できなかったならば、中身を実行します。。ec:ezlogin_config_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:ezlogin_config_form>
             <ec:if_mobile_unique_id_unknown>
             </ec:if_mobile_unique_id_unknown>
           </ec:ezlogin_config_form>
           }}}
        ]
      EOS
      def if_mobile_unique_id_unknown
        raise ::EcRider::EcHelper::UsageError, "ec:if_mobile_unique_id_getはec:ezlogin_config_formの中で使用してください" unless @ezlogin_config_form
        if @template.instance_variable_get(:@mobile_unique_id).nil?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:かんたんログイン設定解除フォーム。]
        [引数:なし]
        [使用例:
           {{{
           <ec:ezlogin_config_cancel_form>
             <ec:if_mobile_unique_id_get>
             </ec:if_mobile_unique_id_get>
             <ec:submit>設定する</ec:submit>
           </ec:ezlogin_config_cancel_form>
           }}}
        ]
      EOS
      def ezlogin_config_cancel_form(&block)
        # utnはdocomoで必要
        @template.form_for(:user_mobile, :url => {:controller => 'member', :action => :complete_cancel_config_ezlogin}, :html=>{:utn => 'utn'}) do |f|
          @ezlogin_config_cancel_form = f
          begin
            @template.concat(@template.hidden_field_tag("mobile_unique_id", @template.instance_variable_get(:@mobile_unique_id)))
            @template.concat(@template.capture(&block))
          ensure
            @ezlogin_config_cancel_form = nil
          end
        end
      end

    end
  end
end