# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper

    module OrderFormTag
      extend ::EcRider::EcHelper::MethodDescription

      desc <<-EOS
        [概要:お届け先指定・お支払方法用のカード種類を出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_card_kind>
            <ec:order_form_card_kind_radio />
            <ec:order_form_card_kind_label />
          </ec:order_form_card_kind>
          }}}
        ]
      EOS
      def order_form_card_kind(&block)
        raise ::EcRider::EcHelper::UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        raise ::EcRider::EcHelper::UsageError, "ec:order_form_card_kindはec:order_formの中で使用してください" unless @order_form
        checked_card_kind = @order_form.object.card_kind
        @order_form.radio_button_group(:card_kind) do |b|
          b.each do
            @order_form_card_kind = b
            begin
              @template.concat(@template.capture(&block))
            ensure
              @order_form_card_kind = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:お支払い方法のセレクトボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数 セレクトボックスのHTML属性(任意):]
        [戻り値:お支払い方法のセレクトボックス]
        [使用例:
          {{{
          <ec:order_form_payment_type />
          }}}
        ]
      EOS
      def order_form_payment_type(*args, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:order_form_shipping_address_radio_buttonはec:order_formの中で使用してください" unless @order_form
        options = {:class=>'payment_type'}
        payment_type_list = OrderShipping.select_form_list
        @order_form.select(:normal_payment_type, payment_type_list, {}, options)
      end

      desc <<-EOS
        [概要：お届け先指定・お支払方法用のカード番号を入力する入力フィールドとその入力例を出力します。注文用のフォーム内でのみ使用できます。]
        [使用例：]
      EOS
      def order_form_card_no(*args, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:order_form_card_noはec:order_formの中で使用してください" unless @order_form
        options = {:size => 20}.update(options || {})
        @order_form.text_field_valid(:no, options)
      end

      desc <<-EOS
        [概要:配送希望時間を入力するセレクトボックスを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_form_hope_delivery_time />
          }}}
        ]
      EOS
      def order_form_delivery_time_select(*args, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:order_form_delivery_dateはec:order_formの中で使用してください" unless @order_form
        selects = @order_form.object.available_delivery_time
        displays = []
        selects.each do |dt|
          unless dt.code.blank?
            @order_form_delivery_time = dt
            display = @order_form_delivery_time.displays.select{|t| t.locale == I18n.locale.to_s}
            displays << [display.first.name, dt.code]
          end
        end
        @order_form.select(:delivery_time, displays, {}, {})
      end

      desc <<-EOS
        [概要:配送先を選択するリンクを出力します。お届け先指定・お支払方法画面からの配送一覧画面で使用してください。]
        [引数:なし]
        [使用例:
          {{{
          <ec:order_shipping_select_link />
          }}}
        ]
      EOS
      def order_shipping_select_link(*args, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:order_shipping_select_linkはec:shipping_listの中で使用してください" unless @shipping_address
        return "" unless session[:cart]
        controller = @template.send(:params)[:controller]
        text, options = extract(args, :text => nil)
        text = @template.capture(&block) if block_given?
        link_to(text, {:controller => controller, :action => 'payment', :send_address_index=> @shipping_address_index}, options)
      end
    end

  end
end

