# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper

    # PC、モバイル両方に対するタグは～Tagという名前にします
    module ProductTag
      unloadable
      extend ::EcRider::EcHelper::MethodDescription

      # 商品バリエーション
      (1..3).each do |index|
        desc <<-EOS
          [概要:商品SKUバリエーション名#{index}を表示します。]
          [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
          [戻り値:在庫数]
          [使用例:
            {{{
            <ec:product_variation_item_name_#{index} product_code="001"/>
            }}}
          ]
        EOS
        module_eval(<<-"EOS", __FILE__, __LINE__)
          def product_variation_item_name_#{index}(*args, &block)
            product_code, options = extract(args, :product_code => "")
            product_sku_info(product_code, "variation_item_#{index}_name")
          end
        EOS
      end

      desc <<-EOS
        [概要: 商品の画像を、largeサイズの画像へのリンクつきで表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 index(任意):{{{何番目の商品画像を表示するかを指定します。指定しない場合はメイン画像が表示されます。}}}]
        [引数 type(任意):商品画像のサイズを指定します。指定しない場合はアップロードしたままのサイズで表示します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数 expansion_type(任意):商品画像押下後の遷移後のサイズを指定します。指定しない場合はアップロードしたままのサイズで表示します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数 (オプション): HTMLオプションを指定します。]
        [戻り値: 詳細画面へのリンクつきの商品画像。]
        [使用例:{{{<ec:product_image_link product_code="001"/>}}}]
      EOS
      def product_expansion_image_link(*args, &block)
        expansion_type, index, product_code, type, options = extract(args, :expansion_type => "large", :index => nil, :product_code => "", :type => "small")
        image = @template.instance_variable_get(:@product_image)
        if index.nil? && image.nil?
          image =  product_info(product_code, 'main_image')
        end
        if index
          product_id = product_info(product_code, 'id')
          image = ProductImage.find(:first, :conditions => ["product_id = ? and display_order = ?", product_id, index])
        end
        if image
          src = options["src"] ? options["src"] : image.public_filename(type)
          expansion_src = options["src"] ? options["src"] : image.public_filename(expansion_type)
          options["alt"] ||= product_info(product_code, 'product_name_1') if product_info(product_code, 'product_name_1')
          link_to(@template.image_tag(src, options), {:href => expansion_src, :escape => false})
        else
          return ''
        end
      end

      desc <<-EOS
        [概要:商品検索フォームを出力します。]
        [引数 render_name(必須):商品検索結果を表示する画面のURLを指定します。]
        [引数 product_separate_value(任意):商品検索結果の１ページの表示件数を指定します。何も指定しない場合は5が適用されます。]
        [使用例:
          {{{
            <ec:product_search_form render_name="list">
              <ec:submit>検索</ec:submit>
            </ec:product_search_form_mobile>
          　「検索ボタンを押すと、listのページに遷移し、検索結果を表示する」検索フォーム。
          }}}
        ]
      EOS
      def product_search_form(*args, &block)
        per_page, render_name, options = extract(args, :per_page => 5, :render_name => nil)
        raise ::EcRider::EcHelper::UsageError, "ec:product_search_formを使用する時は遷移先のrender_nameオプションを設定してください" unless render_name
        @template.form_for(:condition, :url => {:controller =>'cms', :action => render_name},:html => {:method => :get}) do |f|
          @product_search_form = f
          @template.concat(@template.hidden_field_tag('condition[per_page]', per_page))
          begin
            @template.concat(@template.capture(&block))
          ensure
            @product_search_form = nil
          end
        end
      end

      desc <<-EOS
        [概要: 商品の画像を、詳細画面へのリンクつきで表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 index(任意):{{{何番目の商品画像を表示するかを指定します。指定しない場合はメイン画像が表示されます。}}}]
        [引数 type(任意):商品画像のサイズを指定します。デフォルトはmediumになっています。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。, ""→オリジナルサイズの画像を表示します。)]
        [引数 (オプション): HTMLオプションを指定します。]
        [戻り値: 詳細画面へのリンクつきの商品画像。]
        [使用例:{{{<ec:product_image_link product_code="001"/>}}}]
      EOS
      def product_image_link(*args, &block)
        image_display_div, index, product_code, type, options = extract(args, :image_display_div => "0", :index => nil, :product_code => "", :type => :medium)
        if @template.send(:params)[:condition] && @template.send(:params)[:condition][:image_display_div]
          image_display_div = @template.send(:params)[:condition][:image_display_div]
        end
        if image_display_div != "0"
          return ''
        end
        image =  product_info(product_code, 'main_image')
        if index
          product_id = product_info(product_code, 'id')
          image = ProductImage.find(:first, :conditions => ["product_id = ? and display_order = ?", product_id, index])
        end
        if image
          src = options["src"] ? options["src"] : image.public_filename(type)
          options["alt"] ||= product_info(product_code, 'product_name_1') if product_info(product_code, 'product_name_1')
          detail_url = product_info(product_code, 'detail_url')
          link_to(@template.image_tag(src, options), {:href => detail_url, :escape => false})
        else
          return ''
        end
      end

      desc <<-EOS
        [概要:商品画像を表示します。]
        [引数 product_code(準必須): {{{商品コードを指定します。商品詳細ページ、<ec:each_products>～</ec:each_products>の中では不要です。}}}]
        [引数 index(任意):{{{何番目の商品画像を表示するかを指定します。指定しない場合はメイン画像が表示されます。<ec:each_product_images>の中では使用できません。}}}]
        [引数 type(任意):商品画像のサイズを指定します。デフォルトはmediumになっています。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。, ""→オリジナルサイズの画像を表示します。)]
        [引数 (オプション):HTMLオプションを指定します。]
        [戻り値:商品画像に登録した画像。]
        [使用例:
          {{{
          <ec:product_image product_code="001"/>
          }}}
        ]
      EOS
      def product_image(*args, &block)
        image_display_div, index, product_code, type, options = extract(args, :image_display_div => "0", :index => nil, :product_code => "", :type => :medium)
        if @template.send(:params)[:condition] && @template.send(:params)[:condition][:image_display_div]
          image_display_div = @template.send(:params)[:condition][:image_display_div]
        end
        if image_display_div != "0"
          return ''
        end
        image = @template.instance_variable_get(:@product_image)
        if index.nil? && image.nil?
          image =  product_info(product_code, 'main_image')
        end
        if index
          product_id = product_info(product_code, 'id')
          image = ProductImage.find(:first, :conditions => ["product_id = ? and display_order = ?", product_id, index])
        end
        if image
          options["alt"] ||= product_info(product_code, 'product_name_1') if product_info(product_code, 'product_name_1')
          src = options["src"]
          src ||= image.blank? ? nil : image.public_filename(type)
          return @template.image_tag(src, options) if src
        else
          return ''
        end
      end

      desc <<-EOS
        [概要:{{{商品検索フォームからの検索結果を出力する場合に使用します。このタグ内に<ec:each_products>を記述すると検索結果の一覧を表示できます。}}}]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
          <ec:searched_products_mobile>
              <ec:product_crumbs_display/>
              <ec:each_products>
                  <ec:product_name_1/>
              </ec:each_products>
          </ec:searched_products_mobile>
          　　⇒ 検索条件にヒットした商品の商品名を表示します。
          }}}
        ]
      EOS
      def searched_products(*args, &block)
        # 商品検索パラメータが有る場合、検索商品をインスタンスに格納
        params = @template.send(:params)
        condition_hash = {}
        if params[:category_code]
          condition_hash[:category_code] = params[:category_code]
        elsif params[:condition]
          condition_hash = params[:condition]
        end
        # 検索条件がない場合でも、表示件数の条件を追加する
        condition_hash[:per_page] ||= !params[:per_page].nil? ? params[:per_page].to_i : 5

        condition_hash[:user_rank_id] = UserRank.find(:first, :conditions => ["no_membership_flg IS TRUE"]).id
        # ログイン済みの会員の場合は会員ランクに合わせた商品売価を取得する
        condition_hash[:user_rank_id] = get_user_rank_id()
        condition_hash[:order] = params[:order] if params[:order]
        @condition = Product::CategorySearchForm.new(condition_hash)
        @per_page = condition_hash[:per_page].to_i
        @condition_page = params[:page]
        search_products = Product.paginate(@condition.to_find_options(:page => @condition_page, :per_page => @per_page))
        @template.instance_variable_set("@condition", @condition)
        @template.instance_variable_set("@search_products", search_products)
        @template.instance_variable_set("@products", search_products)
        begin
          yield
        ensure
          @products = nil
          @template.instance_variable_set("@products", nil)
          @template.instance_variable_set("@condition", nil)
          @template.instance_variable_set("@search_products", nil)
        end
      end

      desc <<-EOS
        [概要:商品検索結果の表示件数を切り替えるセレクトボックスを表示します。]
        [引数:select_count(任意)セレクトボックスで選ぶ表示件数の件数を｢○,○｣形式で設定します。何も指定しなかった場合、5,10を選択肢にします。]
        [使用例:
          {{{
          <ec:product_pagination_count_per_page/>
          <ec:product_pagination_count_per_page select_count="20,50,100"/>
          }}}
        ]
      EOS
      def product_pagination_count_per_page(*args)
        select_count, options = extract(args, :select_count => "5, 10")
        condition_params = @template.send(:params)[:condition]
        @template.concat(@template.hidden_field_tag('condition[condition_div]', condition_params[:condition_div]))
        @template.concat(@template.hidden_field_tag('condition[category_code]', condition_params[:category_code]))
        @template.concat(@template.hidden_field_tag('condition[keyword]', condition_params[:keyword]))
        selected = !condition_params[:per_page].nil? ? condition_params[:per_page] : 5
        @product_search_form.select(:per_page, select_count.split(/\s*,\s*/),{:selected => selected}, options)
      end

      desc <<-EOS
        [概要:検索条件（商品画像表示、非表示）をセレクトボックスで表示します。商品検索フォーム内でのみ使用できます。]
        [引数 image_display(必須):商品画像を表示させる旨の文言を指定します。]
        [引数 no_image_display(必須):商品画像を表示させない旨の文言を指定します。]
      EOS
      def product_image_display_select(*args)
        raise ::EcRider::EcHelper::UsageError, "ec:product_search_form_condition_selectはec:product_search_formの中で使用してください" unless @product_search_form
        image_display, no_image_display, options = extract(args, :image_display => Product::CategorySearchForm.image_display_div_name_by_key(:visible), :no_image_display => Product::CategorySearchForm.image_display_div_name_by_key(:invisible))
        condition_params = @template.send(:params)[:condition]
        if condition_params && !@template.send(:params)[:condition][:image_display_div].nil?
          selected = condition_params[:image_display_div].to_i
        else
          selected = 0
        end
        @template.concat(@template.hidden_field_tag('condition[condition_div]', condition_params[:condition_div]))
        @template.concat(@template.hidden_field_tag('condition[category_code]', condition_params[:category_code]))
        @template.concat(@template.hidden_field_tag('condition[keyword]', condition_params[:keyword]))
        @product_search_form.select(:image_display_div, [[image_display, 0], [no_image_display, 1]], {:selected => selected}, options)
      end
    end
  end
end