# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper

    # PC、モバイル両方に対するタグは～Tagという名前にします
    module ShippingFormTag
      unloadable
      extend ::EcRider::EcHelper::MethodDescription

      desc <<-EOS
        [概要:配送先入力用のフォームを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_form>
             <ec:shipping_form_error_messages/>
             <ec:include_user_address_fields/>
           </ec:shipping_form>
           }}}
        ]
      EOS
      def shipping_form(&block)
        unless @template.instance_variable_get(:@user_address)
          raise ::EcRider::EcHelper::UsageError, "ここでは使用できません。"
        end
        controller = @template.send(:params)[:controller]
        action = @template.send(:params)[:action]
        action = action == 'new_shipping_address' ? 'confirm_address' : 'confirm_modify_change'
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => controller, :action => action}, :html => {:id => "shipping_form" }) do |f|
          @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f2|
            @user_address = f
            @user_profile = f2
            begin
              edit_target_index = @template.instance_variable_get(:@edit_target_index)
              @template.concat(@template.capture(&block))
              @template.concat(@template.hidden_field_tag("edit_target_index", edit_target_index)) if edit_target_index
            ensure
              @user_address = nil
              @user_profile = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:配送先入力確認用のフォームを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_full_name/>
             <ec:if_address_japanese_format>
               <ec:shipping_confirmation_form_kana_last_name/>
               <ec:shipping_confirmation_form_kana_first_name/>
             </ec:if_address_japanese_format>
             <ec:shipping_confirmation_form_country/>
             <ec:shipping_confirmation_form_zip_code/>
             <ec:shipping_confirmation_form_prefecture/>
             <ec:shipping_confirmation_form_full_address/>
             <ec:shipping_confirmation_form_tel_num/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form(&block)
        unless @template.instance_variable_get(:@user_address)
          raise ::EcRider::EcHelper::UsageError, "ここでは使用できません。"
        end
        controller = @template.send(:params)[:controller]
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => controller, :action => :update_shipping}, :html => {:id => "shipping_confirmation_form" }) do |f|
          @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f2|
            @user_address = f
            @user_profile = f2
            begin
              edit_target_index = @template.instance_variable_get(:@edit_target_index)
              @template.concat(@template.capture(&block))
              @template.concat(@template.hidden_field_tag(:edit_target_index, edit_target_index)) if edit_target_index
            ensure
              @user_address = nil
              @user_profile = nil
            end
          end
        end
      end

    end
  end
end