# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper

    # PC、モバイル両方に対するタグは～Tagという名前にします
    module ShippingTag
      unloadable
      extend ::EcRider::EcHelper::MethodDescription

      desc <<-EOS
        [概要:配送先一覧を出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_list>
             <ec:order_form_shipping_radio_button />
             <ec:shipping_fullname />
             <ec:shipping_countryname />
             <ec:shipping_zipcode />
             <ec:shipping_address />
             <ec:shipping_telnum />
             <ec:shipping_cost />
           </ec:shipping_list>
           }}}
        ]
      EOS
      def shipping_list(&block)
        @shipping_addresses = @template.instance_variable_get(:@shipping_addresses)
        @send_address_index = @template.instance_variable_get(:@send_address_index)
        raise UsageError, "配送先が登録されていません" unless @shipping_addresses
        _each_with_index(:@shipping_addresses, :@shipping_address, :@shipping_address_index) do
          # 支払い方法画面での配送表示の際、使用する配送先のみ表示します
          if !@send_address_index || @send_address_index == @shipping_address_index
            @template.concat(@template.capture(&block))
          end
        end
        @template.concat(@template.hidden_field_tag(:send_address_index, @send_address_index)) if @send_address_index
      end

      desc <<-EOS
        [概要:配送先編集ボタンを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_shipping_edit />
           }}}
        ]
      EOS
      def button_to_shipping_edit(options={}, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:button_to_shipping_editはec:shipping_listの中で使用してください" unless @shipping_address
        controller = @template.send(:params)[:controller]
        @template.form_for(:user_address, @shipping_address, :url => {:controller => controller, :action => :change_address}, :html => {:id => "shipping_for_edit_form" }) do |f|
          user_address = f
          begin
            @template.concat(user_address.hidden_field(:id))
            @template.concat(@template.hidden_field_tag(:id, @shipping_address_index))
            options = {:class => "zero_button"}.update(options || {})
            submit(options) do
              @template.capture(&block)
            end
          ensure
            user_address = nil
          end
        end
      end

      desc <<-EOS
        [概要:配送先削除ボタンを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_shipping_destoroy />
           }}}
        ]
      EOS
      def button_to_shipping_destroy(options={}, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:button_to_shipping_destroyはec:shipping_listの中で使用してください" unless @shipping_address
        return unless @shipping_address.address_div_key == :ship_to
        controller = @template.send(:params)[:controller]
        @template.form_for(:user_address, @shipping_address, :url => {:controller => controller, :action => :delete_address}, :html => {:id => "shipping_for_destroy_form" }) do |f|
          user_address = f
          begin
            @template.concat(user_address.hidden_field(:id))
            @template.concat(@template.hidden_field_tag(:id, @shipping_address_index))
            submit(options) do
              @template.capture(&block)
            end
          ensure
            user_address = nil
          end
        end
      end

    end
  end
end