# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper

    # PC、モバイル両方に対するタグは～Tagという名前にします
    module SignupFormTag
      unloadable
      extend ::EcRider::EcHelper::MethodDescription

      def request
        @template.send(:request)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の姓を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_last_name　住所フォーム内で使用]
        [同様のタグ:account_form_last_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_last_name　非会員用住所フォーム内で使用]
        ]
      EOS
      def signup_form_main_address_last_name(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_last_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        @user_address.text_field_valid(:name_last, options)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の名を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_first_name　住所フォーム内で使用]
        [同様のタグ:account_form_first_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_first_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_first_name(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_first_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(options || {})
        @user_address.text_field_valid(:name_first, options)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所のカナ姓を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_kana_last_name　住所フォーム内で使用]
        [同様のタグ:account_form_kana_last_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_kana_last_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_kana_last_name(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_kana_last_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength=>30, :size=>20, :class=>:default}.update(MobileInputMode.katakana(request)).update(options || {})
        @user_address.text_field_valid(:kana_name_last, options)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所のカナ名を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_kana_first_name　住所フォーム内で使用]
        [同様のタグ:account_form_kana_first_name　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_kana_first_name　非会員用住所フォーム内で使用]
      EOS
      def signup_form_main_address_kana_first_name(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_kana_first_nameはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 30,:size => 20, :class=>:default, :style => "ime-mode: active"}.update(MobileInputMode.katakana(request)).update(options || {})
        @user_address.text_field_valid(:kana_name_first, options)
      end

      # 別名を定義します。
      method_names = %w(last_name first_name kana_last_name kana_first_name)

      src_form_name = 'signup_form_main_address'
      alias_form_names = %w(user_address account_form nonmember_address_form)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の日本用郵便番号を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_japanese_zip_code　住所フォーム内で使用]
      EOS
      def signup_form_main_address_japanese_zip_code(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_japanese_zip_codeはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 7,:size => 7, :class=>:default}.update(MobileInputMode.numeric(request)).update(options || {})
        @user_address.text_field_valid(:zip_code, options)
      end

      desc <<-EOS
        [概要:新規会員登録用メイン住所の市区町村番地を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_address_1　住所フォーム内で使用]
      EOS
      def signup_form_main_address_address_1(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_address_1はec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 60,:size => 20,:class=>:default,:style => "ime-mode: active"}.update(options || {})
        @user_address.text_field_valid(:address_2, options)
      end

      desc <<-EOS
        [概要:新規会員登録用電話番号を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:user_address_tel_num　住所フォーム内で使用]
      EOS
      def signup_form_main_address_tel_num(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_main_address_tel_numはec:signup_formの中で使用してください" unless @user_address
        options = {:maxLength => 20,:size => 20,:class=>:default}.update(MobileInputMode.numeric(request)).update(options || {})
        @user_address.text_field_valid(:tel_num, options)
      end


      # 別名を定義します。
      method_names = %w(japanese_zip_code address_1 tel_num)

      src_form_name = 'signup_form_main_address'
      alias_form_names = %w(user_address)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      #@user_email
      #
      desc <<-EOS
        [概要:新規モバイル会員登録用メールアドレスを入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_email　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_email　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_email(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_emailはec:signup_formの中で使用してください" unless @user_email
        options = {:maxLength => 50, :size => 20, :class => :default}.update(MobileInputMode.alphabet(request)).update(options || {})
        @user_email.text_field_valid(:email, options) <<
          @user_email.hidden_field(:pc_mobile_div, :value => UserEmail.pc_mobile_div_id_by_key(:mobile))
      end

      # 別名を定義します。
      method_names = %w(email)

      src_form_name = 'signup_form_profile'
      alias_form_names = %w(account_form_profile nonmember_address_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用生年月日の年を入力するinputtタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_birth_year　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_birth_year　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_birth_year(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_birth_yearはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 4, :size => 4, :class => :default}.update(MobileInputMode.numeric(request)).update(options || {})
        @user_profile.text_field_valid(:birth_year, options)
      end

      desc <<-EOS
        [概要:新規会員登録用モバイルメールマガジン購読区分を入力する欄を出力します。新規会員登録用のフォーム内で使用してください。]
        [引数 default_checked(任意)メールマガジン配信 区分が登録されていない場合の初期値を設定します   0:HTMLメール 1:テキストメール 99:受け取らない]
        [同様のタグ:account_form_profile_mobile_mail_magazine　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_mobile_mail_magazine　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_mobile_mail_magazine(*args, &block)
        default_checked, option = extract(args, :default_checked => :not_prefer)
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_mobile_mail_magazineはec:signup_formの中で使用してください" unless @user_profile
        @user_profile.object.mobile_mail_magazine_send_div ||= default_checked
        @user_profile.radio_button_group(:mobile_mail_magazine_send_div) do |b|
          b.each do
            @user_profile_mobile_mail_magazine = b
            begin
              @template.concat(@template.capture(&block))
            ensure
              @user_profile_mobile_mail_magazine = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:新規会員登録用モバイルメールマガジン購読区分のinputラジオボタンタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_mobile_mail_magazine_radio　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_mobile_mail_magazine_radio　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_mobile_mail_magazine_radio(options = {}, &block)
        raise ::EcRider::EcHelper::UsageError, "ec:payment_form_mobile_mail_magazine_radioはec:signup_form_profile_mobile_mail_magazineの中で使用してください" unless @user_profile_mobile_mail_magazine
        options = {}.update(options || {})
        @user_profile_mobile_mail_magazine.radio_button(options)
      end

      desc <<-EOS
        [概要:新規会員登録用モバイルメールマガジン購読区分のラジオボタンに表示するラベルを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_mobile_mail_magazine_label　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_mobile_mail_magazine_label　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_mobile_mail_magazine_label(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_mobile_mail_magazine_labelはec:signup_form_profile_mobile_mail_magazineの中で使用してください" unless @user_profile_mobile_mail_magazine
        @user_profile_mobile_mail_magazine.label
      end

     desc <<-EOS
        [概要:新規モバイル会員登録用ログインIDを入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:account_form_profile_login_id　会員情報変更フォーム内で使用]
        [同様のタグ:nonmember_address_form_profile_login_id　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_login_id(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_login_idはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 20, :size => 20,:class => :default}.update(MobileInputMode.alphabet(request)).update(options || {})
        @user_profile.text_field_valid(:login_id, options)
      end

      # 別名を定義します。
      method_names = %w(birth_year mobile_mail_magazine mobile_mail_magazine_radio mobile_mail_magazine_label login_id)

      src_form_name = 'signup_form_profile'
      alias_form_names = %w(account_form_profile nonmember_address_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用パスワードを入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:nonmember_address_form_profile_password　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_password(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_passwordはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 20, :size => 20}.update(MobileInputMode.alphabet(request)).update(options || {})
        @user_profile.password_field(:password, options)
      end

      desc <<-EOS
        [概要:新規会員登録用パスワードの確認を入力するinputタグを出力します。新規会員登録用のフォーム内で使用してください。]
        [同様のタグ:nonmember_address_form_profile_password_confirmation　非会員用住所フォーム内で使用]
      EOS
      def signup_form_profile_password_confirmation(options = {})
        raise ::EcRider::EcHelper::UsageError, "ec:signup_form_profile_password_confirmationはec:signup_formの中で使用してください" unless @user_profile
        options = {:maxLength => 20, :size => 20}.update(MobileInputMode.alphabet(request)).update(options || {})
        @user_profile.password_field(:password_confirmation, options)
      end

      # 別名を定義します。
      method_names = %w(password password_confirmation)

      src_form_name = 'signup_form_profile'
      alias_form_names = %w(nonmember_address_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用入会規約同意のチェックボックスを出力します。新規会員登録用のフォーム内で使用してください。]
      EOS
      def signup_form_enrollment_agreement_check(options = {})
        @user_profile.check_box(:enrollment_agreement, options) <<
        @user_profile.hidden_field(:enrollment_agreement_check, :value => "1")
      end

      desc <<-EOS
        [概要:新規会員登録時の会員登録完了画面において、次の画面に進むためのリンクを出力します。会員登録完了画面で使用してください。]
      EOS
      def link_to_next_for_completion_signup(*args, &block)
        text, options = extract(args, :text => '続ける')
        text = @template.capture(&block) if block_given?
        link_url = @template.instance_variable_get(:@link_url_for_completion_signup)
        raise ::EcRider::EcHelper::UsageError, "ec:link_to_next_for_completion_signupは会員登録完了画面で使用してください" unless link_url
        link_to(text, {:href => "#{link_url}"})
      end

      desc <<-EOS
        [概要:新規会員登録用ログインIDの入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_login_id　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_login_id
        raise ::EcRider::EcHelper::UsageError, "ec:signup_confirmation_form_profile_login_idはec:signup_confirmation_formの中で使用してください" unless @user_profile
        @user_profile.object.login_id
      end

      desc <<-EOS
        [概要:新規会員登録用PCメールマガジン配信区分の入力内容を出力します。登録内容確認用のフォーム内で使用してください。]
        [同様のタグ:account_confirmation_form_profile_mobile_mail_magazine　会員情報変更確認フォーム内で使用]
      EOS
      def signup_confirmation_form_profile_mobile_mail_magazine
        raise ::EcRider::EcHelper::UsageError, "ec:signup_confirmation_form_profile_mobile_mail_magazineはec:signup_confirmation_formの中で使用してください" unless @user_profile
        @user_profile.object.mobile_mail_magazine_send_div_name
      end


      # 別名を定義します。
      method_names = %w(login_id mobile_mail_magazine)

      src_form_name = 'signup_confirmation_form_profile'
      alias_form_names = %w(account_confirmation_form_profile)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

      desc <<-EOS
        [概要:新規会員登録用のフォームを出力します。]
      EOS
      def order_signup_form(&block)
        unless @template.instance_variable_get(:@user_address) && @template.instance_variable_get(:@user_profile) && @template.instance_variable_get(:@user_email)
          raise ::EcRider::EcHelper::UsageError, "ここでは使用できません。"
        end
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => 'order', :action => :payment}, :html => {:id => "order_signup_form" }) do |f|
          @template.fields_for(:user_en_address, @template.instance_variable_get(:@user_en_address)) do |f1|
            @template.fields_for(:user_email, @template.instance_variable_get(:@user_email)) do |f2|
              @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f3|
                @user_address = f
                @user_en_address = f1
                @user_email = f2
                @user_profile = f3
                begin
                  @template.concat(@template.capture(&block))
                ensure
                  @user_address = nil
                  @user_en_address = nil
                  @user_email = nil
                  @user_profile = nil
                end
              end
            end
          end
        end
      end
    end
  end
end