module EcrMobile
  module UserExt
    unloadable
    def self.included(mod)
      mod.module_eval do
        has_one :mobile, :class_name => '::UserMobile', :foreign_key => 'user_id'
        attr_accessor :enrollment_agreement,        # 会員規約同意のチェックボックスの値を格納
                      :enrollment_agreement_check     # 会員規約同意のバリデーションチェックを行うかどうか（"1":行う、"0":行わない）
        validates_numericality_of :birth_year, :only_integer => true, :greater_than_or_equal_to => Date.today.year-99,
          :less_than_or_equal_to => Date.today.year, :allow_nil => true
      end
      mod.extend(ClassMethods)
    end

    def validate_with_ecr_mobile
      # 会員規約に同意したかどうか
      errors.add(:base, :MOBILE_EM990001) if self.enrollment_agreement_check == "1" && self.enrollment_agreement != "1"
      validate_without_ecr_mobile
    end

    # メールアドレスの相関チェックにモバイルのメールアドレスも追加
    def invalid_email_with_ecr_mobile?
      no_exist_mobile_mail = user_email_mobile.nil? || user_email_mobile.email.nil? || user_email_mobile.email.blank?
      invalid_email_without_ecr_mobile? && no_exist_mobile_mail
    end

    module ClassMethods
      # Authenticates a user by their user_mobile and unique_id.  Returns the user or nil.
      def authenticate_ez(unique_id)
        where = ['user_mobiles.unique_id = ?', 'users.member_div = ?']
        param = [unique_id, User.member_div_id_by_key(:membership)]
        where << (EcRider.config.ec_test ? 'users.test_flg = 1' : '(users.test_flg = 0 OR users.test_flg IS NULL)')

        find(:first, :include => :mobile, :conditions => [where.join(' AND ')]+param)
      end
    end
  end
end