# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'
describe EcRider::EcHelper do

  fixtures :products, :product_displays, :product_prices, :product_groups, :product_group_skus,
  :product_skus, :product_categories, :product_category_displays, :product_images,
  :product_variations, :product_variation_items, :product_variation_item_displays,
  :product_sku_inventories, :product_sku_secure_inventories,
  :users, :countries,
  :user_ranks, :user_rank_displays, :user_rank_conditions,
  :point_settings, :shop_settings,
  :campaigns, :campaign_target_product_skus, :campaign_effects, :campaign_conditions,
  :product_faqs, :product_faq_displays, :product_faq_relation_products,
  :cms_publishes

  before do
    request.stub!(:mobile?).and_return(true)
    request.stub_chain(:mobile, :supports_cookie?).and_return(false)
    @template.extend(ApplicationHelper)
    @template.output_buffer = ""
  end

  def ec
    helper.ec(:mobile)
  end

  describe "ec:product_variation_item_name" do
    it "<ec:product_variation_item_name_1> 取得している商品SKUのバリエーションを表示します" do
      product_sku = ProductSku.find(283)
      @template.instance_variable_set(:@each_product_sku, product_sku)
      ec.product_variation_item_name_1.should == "S"
      ec.product_variation_item_name_2.should == "赤"
      ec.product_variation_item_name_3.should == nil
    end
  end

  describe "ec:product_expansion_image_link" do
    it "<ec.product_expansion_image_link> 拡大画像を表示するための画像リンクを表示します。" do
      # 画像
      ec.product_expansion_image_link(:product_code => "0000000000001").should have_tag('a[href*=/product00001_large.gif]') do
        with_tag('img[src*=/product00001_small.gif][alt="商品1の商品名1"]')
      end
      ec.product_expansion_image_link(:expansion_type=>"medium", :product_code => "0000000000001").should have_tag('a[href*=/product00001_medium.gif]') do
        with_tag('img[src*=/product00001_small.gif][alt="商品1の商品名1"]')
      end
      ec.product_expansion_image_link(:index => 2, :product_code => "0000000000001").should have_tag('a[href*=/product00002_large.gif]') do
        with_tag('img[src*=/product00002_small.gif][alt="商品1の商品名1"]')
      end
    end
    it "indexが異常な値を指している際、タグが表示されていなければok" do
      ec.product_expansion_image_link(:product_code => "0000000000001", :index => "99").should == ""
    end
    it "imageがnilでなかった場合、そのイメージが取得できていればok" do
      pending "納期優先にするため、現状pending"
    end
  end

  describe "product_image_linkのテスト" do
    it "<ec.product_image_link>～</ec.product_image_link> 画像をリンク付きで表示させます。" do
      ec.product_image_link(:product_code => "0000000000001").should have_tag('img[src*=/product00001_medium.gif][alt="商品1の商品名1"]')
      ec.product_image_link(:product_code => "0000000000001", :type => "small").should have_tag('img[src*=/product00001_small.gif][alt="商品1の商品名1"]')
      ec.product_image_link(:product_code => "0000000000001", :type => "large").should have_tag('img[src*=/product00001_large.gif][alt="商品1の商品名1"]')
      ec.product_image_link(:product_code => "0000000000001", :image_display_div => "1").should == ""
      ec.product_image_link(:product_code => "0000000000001", :image_display_div => "0").should have_tag('img[src*=/product00001_medium.gif][alt="商品1の商品名1"]')
    end
  end

  describe "product_imageのテスト" do
    it "商品画像を表示させます。" do
      ec.product_image(:product_code => "0000000000001", :type => "small").should have_tag('img[src*=/product00001_small.gif][alt="商品1の商品名1"]')
      ec.product_image(:product_code => "0000000000001", :type => "small").should have_tag('img[src*=/product00001_small.gif][alt="商品1の商品名1"]')
      ec.product_image(:product_code => "0000000000001", :type => "large").should have_tag('img[src*=/product00001_large.gif][alt="商品1の商品名1"]')
      ec.product_image(:product_code => "0000000000001", :image_display_div => "1").should == ""
      ec.product_image(:product_code => "0000000000001", :image_display_div => "0").should have_tag('img[src*=/product00001_medium.gif][alt="商品1の商品名1"]')
    end
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "product_pagination_count_per_pageのテスト" do
    it "商品検索結果の表示件数を切り替えるセレクトボックスを表示" do
      params[:condition] = {:category_code => '',:keyword => '',:condition_div => '0'}
      ec.product_search_form(:render_name => 'testpage') do
        concat(ec.product_pagination_count_per_page)
        ec.submit{"test"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag("input[name=?]","condition[per_page]")
      @template.output_buffer.should have_tag("input[name=?]","condition[condition_div]")
      @template.output_buffer.should have_tag("input[name=?]","condition[category_code]")
      @template.output_buffer.should have_tag("input[name=?]","condition[keyword]")
      @template.output_buffer.should have_tag('select') do |select|
        select.should have_tag("option[value=?]","5")
        select.should have_tag("option[value=?]","10")
      end
    end
  end

  describe "product_image_display_selectのテスト" do
    it "検索条件（商品画像表示、非表示）をセレクトボックスで表示" do
      params[:condition] = {:category_code => '',:keyword => '',:condition_div => '0', :display_div => '0'}
      ec.product_search_form(:render_name => 'testpage') do
        concat(ec.product_image_display_select)
        @template.output_buffer
      end
      @template.output_buffer.should have_tag("input[name=?]","condition[per_page]")
      @template.output_buffer.should have_tag("input[name=?]","condition[condition_div]")
      @template.output_buffer.should have_tag("input[name=?]","condition[category_code]")
      @template.output_buffer.should have_tag("input[name=?]","condition[keyword]")
      @template.output_buffer.should have_tag('select') do |select|
        select.should have_tag("select[name=?]","condition[image_display_div]")
        select.should have_tag("option[value=?]","0")
        select.should have_tag("option[value=?]","1")
      end
    end
  end
end
