# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'
describe EcRider::EcHelper do
  fixtures :users, :user_addresses, :user_emails, :countries, :prefectures, :prefecture_displays, :shipping_costs, :shop_settings, :product_skus, :campaigns, :products, :shipping_cost_boundaries

  def ec
    helper.ec(:mobile)
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  before(:each) do
    request.stub!(:mobile?).and_return(true)
    request.stub_chain(:mobile, :supports_cookie?).and_return(true) # テストを簡略化するため、クッキーを使用する設定にする。
    @template.extend(ApplicationHelper)
    @template.output_buffer = ""
  end

  describe "shipping_tag" do

    before(:each) do
      @template.output_buffer = ""
      @user = User.find(1006, :include => [:user_address, :user_email])
      @shipping_addresses = @user.user_and_shipping_addresses
      @template.instance_variable_set(:@shipping_addresses, @shipping_addresses)
      @template.extend(ApplicationHelper)
      @template.instance_variable_set(:@shipping_address_index, 1)
    end

    describe "ec:shipping_list" do
      it "<ec:shipping_list>～</ec:shipping_list>" do
        ec.shipping_list do
          ec.shipping_fullname
        end
        @template.output_buffer.should =~ /空川 結衣/
        @template.output_buffer.should =~ /空川 助左衛門/
      end
    end

    describe "ec:shipping_fullname" do
      it "<ec:shipping_fullname/>(kana)" do
        ec.shipping_list do
          ec.shipping_fullname
        end
        @template.output_buffer.should =~ /空川 結衣/
        @template.output_buffer.should =~ /空川 助左衛門/
        @template.output_buffer.should =~ /ソラカワ/
      end
    end

    describe "with countries" do
      fixtures :countries

      describe "ec:shipping_countryname" do
        it "<ec:shipping_countryname/>" do
          ec.shipping_list do
            ec.shipping_countryname
          end
          @template.output_buffer.should =~ /Japan/
        end
      end


      describe "ec:shipping_address" do
        it "<ec:shipping_address/>" do
          ec.shipping_list do
            ec.shipping_address
          end
          @template.output_buffer.should == '東京都足立区 神明４－１－６　マリンマンション１４０５号' + '大阪府大阪市北区 曽根崎新地４－２－７３　田中マンション３０５号'
        end
      end

      describe "ec:shipping_telnum" do
        it "<ec:shipping_telnum/>" do
          ec.shipping_list do
            ec.shipping_telnum
          end
          @template.output_buffer.should =~ /000-111-2224/
          @template.output_buffer.should =~ /000-111-2225/
        end
      end

      describe "ec:button_to_shipping_edit" do
        it "<ec:button_to_shipping_edit/>" do
          params[:controller] = 'order'
          ec.shipping_list do
            ec.button_to_shipping_edit{'編集'}
          end
          @template.output_buffer.should have_tag('form') do
            with_tag('input[type=?]', 'submit')
            with_tag('input[type=?][name=?]', 'hidden', 'user_address[id]')
            with_tag('input[type=?][name=?]', 'hidden', 'id')
          end
        end
      end

      describe "ec:button_to_shipping_destoroy" do
        it "<ec:button_to_shipping_destoroy/>" do
          params[:controller] = 'order'
          ec.shipping_list do
            ec.button_to_shipping_destroy{'削除'} || ''
          end
          @template.output_buffer.should have_tag('form') do
            with_tag('input[type=?]', 'submit')
            with_tag('input[type=?][name=?]', 'hidden', 'user_address[id]')
            with_tag('input[type=?][name=?]', 'hidden', 'id')
          end
        end
      end

    end
  end
end