# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'
describe EcRider::EcHelper do

  def ec
    helper.ec(:mobile)
  end

  before(:each) do
    request.stub!(:mobile?).and_return(true)
    request.stub_chain(:mobile, :supports_cookie?).and_return(true) # テストを簡略化するため、クッキーを使用する設定にする。
    @template.extend(ApplicationHelper)
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:signup_form中のタグ" do
    before(:each) do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
    end

    it "normal usage" do
      ec.signup_form do
        concat(ec.signup_form_error_messages || '')
        concat(ec.signup_form_main_address_last_name)
        concat(ec.signup_form_main_address_first_name)
        concat(ec.signup_form_main_address_kana_last_name)
        concat(ec.signup_form_main_address_kana_first_name)
        concat(ec.signup_form_main_address_japanese_zip_code)
        concat(ec.signup_form_main_address_address_1)
        concat(ec.signup_form_main_address_address_2)
        concat(ec.signup_form_main_address_tel_num)
        concat(ec.signup_form_profile_email)
        concat(ec.signup_form_profile_birth_year)
        concat(ec.signup_form_profile_login_id)
        concat(ec.signup_form_profile_password)
        concat(ec.signup_form_profile_password_confirmation)
        concat(ec.signup_form_profile_remember_question)
        concat(ec.signup_form_profile_remember_answer)
        ec.signup_form_profile_mobile_mail_magazine(:default_checked => :prefer_text) do
          @template.concat(ec.signup_form_profile_mobile_mail_magazine_radio)
          @template.concat(ec.signup_form_profile_mobile_mail_magazine_label)
        end
        concat(ec.signup_form_enrollment_agreement_check)
        @template.output_buffer
      end

      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("input#user_address_name_last")
        with_tag("input#user_address_name_first")
        with_tag("input#user_address_kana_name_last")
        with_tag("input#user_address_kana_name_first")
        with_tag("input#user_address_address_2")
        with_tag("input#user_address_address_3")
        with_tag("input#user_address_tel_num")
        with_tag("input#user_email_email")
        with_tag("input#user_profile_birth_year")
        with_tag("input#user_profile_login_id")
        with_tag("input#user_profile_password[type=?]", 'password')
        with_tag("input#user_profile_password_confirmation[type=?]", 'password')
        with_tag("input#user_profile_remember_question")
        with_tag("input#user_profile_remember_answer")
        with_tag("input#user_profile_mobile_mail_magazine_send_div_0[type=?][value=?]", 'radio', '0')
        with_tag("label[for=?]", 'user_profile_mobile_mail_magazine_send_div_0', 'HTML(絵や写真入のメール)を受け取る')
        with_tag("input#user_profile_mobile_mail_magazine_send_div_99[type=?][value=?]", 'radio', '99')
        with_tag("label[for=?]", 'user_profile_mobile_mail_magazine_send_div_1', 'テキストメール(文字だけのメール)を受け取る')
        with_tag("input#user_profile_mobile_mail_magazine_send_div_99[type=?][value=?]", 'radio', '99')
        with_tag("label[for=?]", 'user_profile_mobile_mail_magazine_send_div_99', '受け取らない')
        with_tag("input#user_profile_enrollment_agreement[type=?]", 'checkbox')
        with_tag("input#user_profile_enrollment_agreement_check[type=?]", 'hidden')
      end
    end
  end


  describe "ec:signup_confirmation_form中のタグ" do
    it "normal usage" do
      user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト", :kana_name_first => "ジロウ", :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000", :address_1 => "福岡県", :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP', :locale => 'ja'}
      user_email = {:email => "test@test.jp"}
      user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970",
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A',
        :kana_name_last => "テスト",  :kana_name_first => "ジロウ",
        :login_id => 'loginid', :mobile_mail_magazine_send_div => "99"}

      @template.instance_variable_set(:@user_address, UserAddress.new(user_address))
      @template.instance_variable_set(:@user_email, UserEmail.new(user_email))
      @template.instance_variable_set(:@user_profile, User.new(user_profile.merge(:user_address => UserAddress.new(user_address.merge(:country => Country.new(:code=>'JP', :name=>'日本'))))))

      ec.signup_confirmation_form do
        ec.signup_confirmation_form_profile_login_id.should == "loginid"
        ec.signup_confirmation_form_profile_mobile_mail_magazine.should == "受け取らない"
        @template.output_buffer
      end
    end
  end


  describe "フォーム不要のタグ" do
    it "normal usage" do
      @template.instance_variable_set(:@link_url_for_completion_signup, "/next_page")
      concat(ec.link_to_next_for_completion_signup)

      @template.output_buffer.should have_tag("a[href=?]", '/next_page')
    end

    it "should raise error" do
      @template.instance_variable_set(:@link_url_for_completion_signup, nil)
      lambda{ec.link_to_next_for_completion_signup}.should raise_error(EcRider::EcHelper::UsageError)
    end
  end


  describe "ec:order_signup_formタグ" do
    before(:each) do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
    end

    it "should no error" do
      ec.order_signup_form do
        @template.output_buffer
      end
    end

    it "should raise error (user_address is nil)" do
      @template.instance_variable_set(:@user_address, nil)
      lambda{ec.order_signup_form}.should raise_error(EcRider::EcHelper::UsageError)
    end

    it "should raise error (user_address is nil)" do
      @template.instance_variable_set(:@user_email, nil)
      lambda{ec.order_signup_form}.should raise_error(EcRider::EcHelper::UsageError)
    end

    it "should raise error (user_address is nil)" do
      @template.instance_variable_set(:@user_profile, nil)
      lambda{ec.order_signup_form}.should raise_error(EcRider::EcHelper::UsageError)
    end
  end

end
