# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe User, :type => :model do

  describe "test authenticate_ez" do
    it "user_mobiles に unique_id が見つからない場合、nilを返す" do
      User.authenticate_ez("uniqueXXXX").should be_nil
    end

    it "user_mobiles に unique_id が見つかったが、ひもづくusersが存在しない場合、nilを返す" do
      user_mobile = UserMobile.find_by_user_id(1008)
      user_mobile.user_id = 0 # usersとひもづかなくする
      user_mobile.save!
      User.authenticate_ez("unique1008").should be_nil
    end

    it "user_mobiles に unique_id が見つかったが、ひもづくusersが退会者の場合、nilを返す" do
      user = User.find(1008)
      user.member_div_key = :resigned
      user.save!
      User.authenticate_ez("unique1008").should be_nil
    end

    it "user_mobiles に unique_id が見つかり、ひもづくusersが退会者でない場合、usersのインスタンスを返す" do
      user = User.authenticate_ez("unique1008")
      user.id.should == 1008
    end
  end

  describe "Validation check" do
    before(:each) do
      @param = {:name_last => "意思湾", :name_first => '太郎',
        :kana_name_last => 'カイモノ', :kana_name_first => 'タロウ', :tel_num => '1234-56-7890',
        :sex => '1', :password => 'pass', :password_confirmation => 'pass', :remember_answer => 'テスト',
        :member_div => '1'
        }
    end

    it "「会員規約に同意」ボタンが会員登録画面にあり、会員規約に同意した場合" do
      @param[:enrollment_agreement_check] = "1"
      @param[:enrollment_agreement] = "1"
      test_user = User.new(@param)
      test_user.errors_on(:base).should == []
    end

    it "「会員規約に同意」ボタンが会員登録画面にあり、会員規約に同意しない場合" do
      @param[:enrollment_agreement_check] = "1"
      @param[:enrollment_agreement] = "0"
      test_user = User.new(@param)
      test_user.errors_on(:base).should == ["会員規約の同意が必要です。"]
    end

    it "「会員規約に同意」ボタンが会員登録画面にない場合" do
      @param[:enrollment_agreement_check] = nil
      @param[:enrollment_agreement] = nil
      test_user = User.new(@param)
      test_user.errors_on(:base).should == []
    end

    it "生年月日（年）が数値以外の場合" do
      @param[:birth_year] = "abc"
      test_user = User.new(@param)
      test_user.errors_on(:birth_year).should == ["は数値で入力してください。"]
    end

    it "生年月日（年）が小さすぎる場合" do
      @param[:birth_year] = (Date.today.year - 100).to_s
      test_user = User.new(@param)
      test_user.errors_on(:birth_year).should == ["は#{Date.today.year - 99}以上の値にしてください。"]
    end

    it "生年月日（年）が範囲内（下限）の場合" do
      @param[:birth_year] = (Date.today.year - 99).to_s
      test_user = User.new(@param)
      test_user.errors_on(:birth_year).should == []
    end

    it "生年月日（年）が範囲内（上限）の場合" do
      @param[:birth_year] = Date.today.year.to_s
      test_user = User.new(@param)
      test_user.errors_on(:birth_year).should == []
    end

    it "生年月日（年）が大きすぎる場合" do
      @param[:birth_year] = (Date.today.year + 1).to_s
      test_user = User.new(@param)
      test_user.errors_on(:birth_year).should == ["は#{Date.today.year}以下の値にしてください。"]
    end

    it "生年月日（年）が空白の場合" do
      @param[:birth_year] = ""
      test_user = User.new(@param)
      test_user.errors_on(:birth_year).should == []
    end
  end

end