# -*- coding: utf-8 -*-
module EcrMobileAdmin
  module EcRider
    module CmsSetupExt

      def self.included(mod)
        mod.extend(ClassMethods)
      end

      module ClassMethods
        #
        # ec_rider提供ページのコントローラを追加した場合は、ここにビューのフォルダ名を追加します
        # ※コントローラ名で無いことに注意。ビューのフォルダが増えなければ追加する必要はありません。
        #
        def ecr_controllers_with_ecr_mobile_admin
          ecr_controllers_without_ecr_mobile_admin
        end
      end

      # XML（CMSの初期データ）を追加しデフォルトプロジェクトとして登録する場合は、
      # このINITIAL_DATAハッシュに
      # （key:アクション名アクション名（View名）, value: テンプレート名、ページ名）の組み合わせで追加してください。
      INITIAL_DATA = {
        'モバイル用_上部' => ['cms', '_default_top', :top],
        'モバイル用_下部' => ['cms', '_default_bottom', :bottom],
        'モバイル用_商品詳細' => ['cms', 'show'],
        'モバイル用_カートに入れる' => ['cms', '_show_cart_in'],
        'モバイル用_お問い合わせ' => ['inquiry', 'inquiry'],
        'モバイル用_お問い合わせ確認' => ['inquiry', 'confirm'],
        'モバイル用_お問い合わせ完了' => ['inquiry', 'completion'],
        'モバイル用_購入履歴' => ['member', 'history'],
        'モバイル用_購入履歴詳細' => ['member', 'history_order_detail'],
        'モバイル用_ポイント状況' => ['member', 'point_history'],
        'モバイル用_注文キャンセル依頼確認' => ['member', 'confirm_cancel'],
        'モバイル用_商品一覧' => ['cms','list'],
        'モバイル用_商品検索結果一覧' => ['cms','search_list'],
        'モバイル用_配送先指定／支払方法' => ['order','payment'],
        'モバイル用_注文確認' => ['order','confirmation_order'],
        'モバイル用_カート' => ['cart','cart_index'],
        'モバイル用_カート商品一覧' => ['cart','_cart_list'],
        'モバイル用_カート合計' => ['cart','_cart_total_prices'],
        'モバイル用_カート商品なし' => ['cart','cart_no_input'],
        'モバイル用_トップページ' => ['cms','index'],
        'モバイル用_ログイン' => ['account','login'],
        'モバイル用_携帯メールアドレス登録' => ['account','add_mailaddress'],
        'モバイル用_携帯メールアドレス確認' => ['account','confirm_mailaddress'],
        'モバイル用_かんたんログイン設定' => ['member','config_ezlogin'],
        'モバイル用_かんたんログイン設定完了' => ['member','complete_config_ezlogin'],
        'モバイル用_かんたんログイン設定解除' => ['member','cancel_config_ezlogin'],
        'モバイル用_かんたんログイン設定解除完了' => ['member','complete_cancel_config_ezlogin'],
        'モバイル用_会員登録' => ['account', 'signup'],
        'モバイル用_住所入力フォーム' => ['account', '_user_address'],
        'モバイル用_利用規約' => ['account', 'policy_information'],
        'モバイル用_会員情報確認' => ['account', 'confirm_signup'],
        'モバイル用_会員登録完了' => ['account', 'completion_signup'],
        'モバイル用_注文者情報登録' => ['order', 'confirmation_cart'],
        'モバイル用_利用規約（注文時）' => ['order', 'agreement'],
        'モバイル用_配送一覧' => ['member', 'show_addresses'],
        'モバイル用_配送登録' => ['member', 'new_shipping_address'],
        'モバイル用_配送登録確認' => ['member', 'confirm_address'],
        'モバイル用_配送編集' => ['member', 'change_address'],
        'モバイル用_配送編集確認' => ['member', 'confirm_modify_change'],
        'モバイル用_不正IPアクセスエラー' => ['cms', 'invalid_ip'],
        'モバイル用_会員情報変更' => ['account', 'change_information'],
        'モバイル用_会員情報変更確認' => ['account', 'completion_update'],
        'モバイル用_会員情報変更完了' => ['account', 'confirm_update_information'],
        'モバイル用_携帯キャリア決済移動ページ' => ['order', 'relocate_settlement'],
        'モバイル用_パスワード再設定メールアドレス入力' => ['reminder', 'index'],
        'モバイル用_パスワード再設定' => ['reminder', 'reminder_question'],
        'モバイル用_パスワード再設定完了' => ['reminder', 'reminder_completion'],
        'モバイル用_パスワード変更' => ['account', 'change_password'],
        'モバイル用_パスワード変更完了' => ['account', 'change_password_completion'],
        'モバイル用_注文時配送一覧' => ['order', 'show_addresses'],
        'モバイル用_注文時配送登録' => ['order', 'new_shipping_address'],
        'モバイル用_注文時配送登録確認' => ['order', 'confirm_address'],
        'モバイル用_注文時配送編集' => ['order', 'change_address'],
        'モバイル用_注文時配送編集確認' => ['order', 'confirm_modify_change'],
        'モバイル用_秘密の質問編集' => ['account', 'change_security_answer'],
        'モバイル用_秘密の質問編集確認' => ['account', 'confirm_security_answer'],
        'モバイル用_秘密の質問編集完了' => ['account', 'completion_security_answer']
      }


      #
      # DBにプロジェクト情報を読み込みます。
      # プラグインで拡張して使われます。
      #
      def db_data_load_for_plugins_with_ecr_mobile_admin
        db_data_load_for_plugins_without_ecr_mobile_admin

        # プラグインで追加するXML（CMSの初期データ）をスキャンして、ハッシュを作成します。
        xml_paths = {}
        cms_directory = File.dirname(__FILE__) + '/../../../initial_data/cms/'
        Dir::glob("#{cms_directory}/**/*.xml").each {|path|
          # ハッシュのキー名を文字列操作して取り出しています。
          # "s/ecr_mobile_admin/lib/ecr_mobile_admin/ec_rider/../../../initial_data/cms//cms_pages/index_mobile.xml"
          # ⇒ "cms_pages/index.xml"
          tmp = path.split('/')
          file_name = tmp.pop
          cms_dir = tmp.pop
          # cms_layouts, cms_pagesの場合はcms
          cms_dir = "cms" if cms_dir == "cms_pages" || cms_dir == "cms_layouts"
          key = "#{cms_dir}/#{file_name}".gsub!('_mobile', '')

          xml_paths[key] = path
        }

        # デフォルトプロジェクトとして登録します
        INITIAL_DATA.each_with_index do | (page_name, controller_and_action), index|
          target = controller_and_action.dup
          controller_name, action_name = target[0], target[1]
          position_key = (target.size == 3) ? position_key : :center
          file_path = xml_paths["#{controller_name}/#{action_name}.xml"]
          # 引数:
          #  terminal_key   :mobileで固定
          #  controller     コントローラ
          #  action         アクション名（View名）
          #  name           テンプレート名、ページ名
          #  code           テンプレートコード（プラグインコード＋連番で24文字以内）
          #  locale         ロケール（モバイルプラグインでは、docomoで固定）
          #  file_path      XML（CMSの初期データ）のファイルパス
          if ::EcRider::CmsSetup::InitialData.setup(:mobile, controller_name, action_name, page_name.dup,  "mobile_#{index}", file_path, 'docomo', position_key)
            ::EcRider::CmsSetup::InitialData.add_locale(:mobile, controller_name, action_name, page_name.dup,  "mobile_#{index}", file_path, 'au')
            ::EcRider::CmsSetup::InitialData.add_locale(:mobile, controller_name, action_name, page_name.dup,  "mobile_#{index}", file_path, 'softbank')
          end
        end

        # デフォルトテンプレートを設定します
        top = CmsTemplate.find(:first, :conditions => ['development_code = ? and terminal = 1', 'cms/_default_top'])
        CmsDefaultTemplate.create_or_update_default_template(1, top.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), top.template_code)

        bottom = CmsTemplate.find(:first, :conditions => ['development_code = ? and terminal = 1', 'cms/_default_bottom'])
        CmsDefaultTemplate.create_or_update_default_template(1, bottom.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:bottom), bottom.template_code)

        # 上部、下部テンプレートの表示位置を設定します
        top.display_position_key = :top
        top.save!

        bottom.display_position_key = :bottom
        bottom.save!
      end

    end
  end
end
