module EcrMobileFront
  module AccountControllerExt
    unloadable

    def self.included(mod)
      mod.module_eval do
        # IP制限を行うフィルターを追加
        #  参照：EcrMobileFront::ApplicationControllerExt#check_valid_ip
        before_filter :check_valid_ip, :only => [:authenticate_ez]
      end
    end

    # ユーザが携帯メールアドレスを持っているかどうか調べる
    def valid_mailaddress_with_ecr_mobile_front(current_user)
      if request.mobile?
        current_user.have_mobile_mailaddress?
      else
        valid_mailaddress_without_ecr_mobile_front(current_user)
      end
    end

    #
    # = 携帯メールアドレス確認 =
    #
    def confirm_mailaddress_with_ecr_mobile_front
      if request.mobile?
        redirect_to(:controller => '/account', :action => 'login') unless session[:add_mailaddress]
        user = session[:add_mailaddress][:target_user]

        @user_email = UserEmail.new(params[:user_email])
        # ここを変更しただけ
        @user_email.pc_mobile_div_key = :mobile
        @user_email.user = user

        unless @user_email.valid?
          flash[:user_email] = @user_email
          redirect_to :action => :add_mailaddress
        end
        session[:add_mailaddress][:user_email] = @user_email
      else
        confirm_mailaddress_without_ecr_mobile_front
      end
    end

    #
    # かんたんログイン
    # 携帯電話の固有IDからユーザ情報を取得し認証する
    # jp_mobileによるIP制限を行う
    #  参照：ApplicationControllerExt#check_valid_ip
    def authenticate_ez
      redirect_to(:action => 'login') unless request.mobile?

      # ログインする
      auth_user = User.authenticate_ez(request.mobile.ident)
      user_locked = auth_user ? auth_user.locked? : false
      current_user = auth_user unless user_locked

      check_logged_in_and_redirect(auth_user, current_user, user_locked)
    end

    #
    # ユーザがメールアドレスを持っているかどうか調べる
    #
    def have_mailaddress?(current_user)
      return !!(current_user.user_email_mobile && current_user.user_email_mobile.email)
    end

    # user_profileからメールアドレスを取得する
    def get_user_email_from_user_profile_with_ecr_mobile_front(user_profile)
      if request.mobile?
        user_profile.user_email_mobile
      else
        get_user_email_from_user_profile_without_ecr_mobile_front(user_profile)
      end
    end

    #
    # 会員登録確認画面
    #
    def confirm_signup_with_ecr_mobile_front
      return confirm_signup_without_ecr_mobile_front unless request.mobile?

      return redirect_to(:action => 'signup') unless request.post?

      # モバイルは日本語しか対応しない
      params[:user_address]["country_code"] = "JP"

      # 「確認」ボタンが押されたかどうかを判別するフラグ
      submit_flg = true

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        submit_flg = false
        params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
      end

      # 「入会規約を読む」ボタンが押された場合
      if params.include?("policy")
        submit_flg = false
        redirect_to(:action => 'policy')
      end

      # 以下本体と同様の処理

      @user_email = UserEmail.new(params[:user_email])
      @user_email.pc_mobile_div_key = :mobile

      @user_address = UserAddress.new(params[:user_address])
      @user_address.locale = I18n.locale.to_s
      @user_address.address_div_key = :user

      if @user_en_address = UserAddress.hash_to_sub_address(params[:user_en_address])
        @user_en_address.set_sub_address_from_address(@user_address)
      end

      @user_profile = User.new(params[:user_profile])
      @user_profile.member_div_key = :membership
      @user_profile.regist_div_key = :mobile
      @user_profile.user_email_mobile = @user_email
      @user_profile.user_address = @user_address
      @user_profile.user_addresses << @user_en_address if @user_en_address
      @user_profile.mirror_from_address(@user_address)
      @user_profile.pc_mail_magazine_send_div = User.pc_mail_magazine_send_div_id_by_key(:not_prefer) if @user_profile.pc_mail_magazine_send_div.nil?
      @user_profile.login = @user_address.fullname
      @user_profile.remember_question_check = true

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        session[:signup] = @user_profile
        redirect_to(:action => 'signup')
      end

      if submit_flg
        @user_email.user = @user_profile # ←メールアドレス重複チェック内でuser_email.userを取得するため必要な処理です。.
        @user_profile.valid?
        @user_address.valid?
        @user_email.valid?
        @user_en_address.valid? if @user_en_address
        error_count = @user_profile.errors.length + @user_address.errors.length + @user_email.errors.length
        error_count += @user_en_address.errors.length if @user_en_address
        if error_count > 0
          @user_profile.not_decided_address = true
          flash[:user_profile] = @user_profile
          flash[:user_address] = @user_address
          flash[:user_email] = @user_email
          flash[:user_en_address] = @user_en_address
          redirect_back_or_default :action => :signup
        end
      end
      session[:signup] = @user_profile

    end


    #
    # 個人情報保護方針
    #
    def policy_with_ecr_mobile_front
      return policy_without_ecr_mobile_front unless request.mobile?

      render :action => :policy_information
    end


    #
    # 登録
    #
    def create_user_with_ecr_mobile_front
      return create_user_without_ecr_mobile_front unless request.mobile?

      return redirect_to(:action => 'signup') unless request.post?
      @user =  session[:signup]
      if logged_in?
        return redirect_to(:controller => :member, :action => :index, :id => @user.member_category_div)
      end

      # トランザクションの範囲が広すぎるのでは？
      # メール送信に失敗すると会員登録は取り消されてしまいます。
      # 少なくともredirect_toしている辺りはtransactionの外で実行するべきです。
      User.transaction do
        @user.save!

        @user.user_address.address_div_key = :user
        @user.user_address.locale = I18n.locale.to_s
        @user.user_address.save!

        self.current_user = @user
        # 会員登録完了メール送信
        FrontSiteMailer.send_signup_mail(@user.id.to_s, @user.user_email_mobile.email)
      end

      session[:signup] = nil
      redirect_to(:action => :completion_signup)
    end


    #
    # 会員登録完了
    #
    def completion_signup
      @link_url_for_completion_signup = session[:forward_url].nil? ? "/member/index/#{self.current_user.member_category_div}" : session.delete(:forward_url)
    end

    #
    # 登録情報変更
    #
    def change_information_with_ecr_mobile_front
      return change_information_without_ecr_mobile_front unless request.mobile?

      session[:account] = nil
      session[:delete_en_address] = nil
      @user_profile = flash[:user_profile] || self.current_user
      @user_address = flash[:user_address] || @user_profile.user_address
      # モバイルのメールアドレスを取得する。
      # モバイル版のメールアドレスがある前提です。
      @user_email = flash[:user_email] || @user_profile.user_email_mobile
      @user_en_address = flash[:user_en_address] || @user_profile.sub_address

      #複数メールアドレスに未対応
      @user_profile.not_decided_address = true
    end

    #
    # 変更内容確認
    #
    def confirm_update_information_with_ecr_mobile_front
      return confirm_update_information_without_ecr_mobile_front unless request.mobile?

      return redirect_to(:action => 'change_information') unless request.post?

      # モバイルは日本語しか対応しない
      params[:user_address]["country_code"] = "JP"

      # 「確認」ボタンが押されたかどうかを判別するフラグ
      submit_flg = true

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        submit_flg = false
        params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
      end

      session[:account] ||= {}

      @user_profile = self.current_user
      @user_profile.attributes = params[:user_profile]
      @user_profile.user_address.attributes = params[:user_address]

      @user_profile.user_email_mobile.attributes = params[:user_email]
      @user_profile.user_email_mobile.pc_mobile_div_key = :mobile
      @user_profile.mirror_from_address(@user_profile.user_address)

      if @user_en_address = UserAddress.hash_to_sub_address(params[:user_en_address])
        @user_en_address.set_sub_address_from_address(@user_profile.user_address)
        @user_profile.user_addresses << @user_en_address
      end

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        # 入力されている情報を  引き継ぐためにflashに渡す
        flash[:user_profile] = @user_profile
        flash[:user_address] = @user_profile.user_address
        flash[:user_email] = @user_profile.user_email_mobile
        flash[:user_en_address] = @user_en_address
        redirect_to(:action => 'change_information')
      end

      if submit_flg
        @user_address = @user_profile.user_address
        # 変更するメールアドレスはモバイル用
        @user_email = @user_profile.user_email_mobile
        @user_profile.valid?
        @user_address.valid?
        @user_email.valid?
        @user_en_address.valid? if @user_en_address
        error_count = @user_profile.errors.length + @user_address.errors.length + @user_email.errors.length
        error_count += @user_en_address.errors.length if @user_en_address
        if error_count > 0
          flash[:user_profile] = @user_profile
          flash[:user_address] = @user_address
          flash[:user_email] = @user_email
          flash[:user_en_address] = @user_en_address
          redirect_back_or_default :action => :change_information
        end
      end
      session[:account][:user_profile] = @user_profile
      session[:account][:user_email] = @user_profile.user_email_mobile
      session[:account][:user_address] = @user_profile.user_address
      session[:account][:user_en_address] = @user_profile.sub_address
    end

    #
    # 登録情報変更確認
    #
    def update_information_with_ecr_mobile_front
      return update_information_without_ecr_mobile_front unless request.mobile?
      return redirect_to(:action => 'change_information') unless request.post?
      User.transaction do
        session[:account][:user_profile].save!
        session[:account][:user_email].save!
        session[:account][:user_address].save!
        if session[:delete_en_address]
          session[:account][:user_en_address].destroy
        elsif session[:account][:user_en_address]
          session[:account][:user_en_address].save!
        end
        session[:account] = nil
        session[:delete_en_address] = nil
      end

      self.current_user.reload
      # PC版との違いは変更後の遷移先です
      render :action => :completion_update
    end

    # 秘密の質問変更完了時の遷移先変更
    def set_redirect_to_after_update_seciruty_answer_with_ecr_mobile_front
      if request.mobile?
        render(:action => :completion_security_answer)
      else
        set_redirect_to_after_update_seciruty_answer_without_ecr_mobile_front
      end
    end

    # パスワード変更を行います。
    def set_redirect_to_after_update_password_with_ecr_mobile_front
      if request.mobile? && request.post?
         return redirect_to(:action => :change_password_completion)
      else
        set_redirect_to_after_update_password_without_ecr_mobile_front
      end
    end
  end
end
