module EcrMobileFront
  module MemberControllerExt
    unloadable

    def self.included(mod)
      mod.module_eval do
        # IP制限を行うフィルターを追加
        #  参照：EcrMobileFront::ApplicationControllerExt#check_valid_ip
        before_filter :check_valid_ip, :only => [:complete_config_ezlogin]
      end
    end

    #
    # 会員ページ
    # モバイルアクセスの時、会員メニューはトップページへ
    #
    def index_with_ecr_mobile_front
      redirect_to(:controller => 'cms', :action => 'index') if request.mobile?
      
      index_without_ecr_mobile_front
    end

    # かんたんログイン設定画面表示
    # 携帯アクセス以外は受け付けない
    # かんたんログイン設定状況をチェックし、設定画面 or 解除画面を表示する
    def config_ezlogin
      redirect_to(:action => 'index') unless request.mobile?
      
      @mobile_unique_id = request.mobile.ident
      @user_mobile = UserMobile.find(:first, :conditions => ["user_id = ?", self.current_user.id])
      @user_mobile = UserMobile.new unless @user_mobile
      
      render :cancel_config_ezlogin if @mobile_unique_id && @user_mobile.unique_id == @mobile_unique_id
      
    end
    
    # かんたんログイン設定完了
    # 携帯アクセス以外は受け付けない
    # postされた識別IDと異なれば受け付けない
    def complete_config_ezlogin
      redirect_to(:action => 'index') unless request.mobile?
      redirect_to(:action => 'index') unless request.mobile.ident == params[:mobile_unique_id]
      
      user_mobile = UserMobile.find(:first, :conditions => ["user_id = ?", self.current_user.id])
      unless user_mobile
        user_mobile = UserMobile.new
        user_mobile.user_id = self.current_user.id
      end
      user_mobile.unique_id = request.mobile.ident
      user_mobile.save
      
    end
    
    # かんたんログイン設定解除完了
    # 携帯アクセス以外は受け付けない
    # postされた識別IDと異なれば受け付けない
    def complete_cancel_config_ezlogin
      redirect_to(:action => 'index') unless request.mobile?
      redirect_to(:action => 'index') unless request.mobile.ident == params[:mobile_unique_id]
      
      user_mobile = UserMobile.find(:first, :conditions => ["user_id = ?", self.current_user.id])
      user_mobile.destroy
    end

    # 配送先一覧
    def show_addresses_with_ecr_mobile_front
      session[:address] = nil if request.mobile?
      show_addresses_without_ecr_mobile_front
    end

    #
    # = 新規配送先の追加フォームを表示します。 =
    #
    def new_shipping_address
      @error_flg = flash[:address_error] || nil
      @user_address = flash[:address]
      @user_address ||= session[:address]
      @user_address ||= UserAddress.new(:country_code => 'JP')
      @user_profile = current_user
    end

    #
    # 配送先追加確認
    #
    def confirm_address_with_ecr_mobile_front
      return confirm_address_without_ecr_mobile_front unless request.mobile?
      return redirect_to(:show_addresses) unless request.post?
      # モバイルは日本語しか対応しない
      params[:user_address]["country_code"] = "JP"

      # 「確認」ボタンが押されたかどうかを判別するフラグ
      submit_flg = true

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        submit_flg = false
        params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
        flash[:address] = UserAddress.new(params[:user_address])
        redirect_to(:action => 'new_shipping_address')
        return
      end

      if submit_flg
        @user_address = UserAddress.new(params[:user_address])
        @user_address.locale = I18n.locale.to_s
        unless @user_address.valid?
          flash[:error] = @user_address.errors.zip
          flash[:address] = @user_address
          redirect_back_or_default(:action => :new_shipping_address)
          return
        end
        session[:address] = @user_address
      end
      @user_profile = current_user
    end


    #
    # 配送先編集
    #
    def change_address_with_ecr_mobile_front
      return change_address_without_ecr_mobile_front unless request.mobile?
      return redirect_to(:action => 'show_addresses') unless (params[:user_address] && params[:user_address][:id]) || session[:change_address]

      if session[:change_address]
        @user_address = session[:change_address]
      else
        @user_address = session[:change_address]
        @user_address ||= UserAddress.find(params[:user_address][:id])
      end

      # セキュリティ対策
      # パラメータ改ざんされ、別会員の配送先住所が閲覧できないように自分の登録した配送先情報かをチェックします。
      if current_user.users_data?("user_address", @user_address.id)
        flash[:address_error] = true
        return redirect_to :action => :show_addresses
      end
      session[:change_address] = @user_address
      @user_profile = current_user
    end

    #
    # 配送先編集確認
    #
    def confirm_modify_change_with_ecr_mobile_front
      return confirm_modify_change_without_ecr_mobile_front unless request.mobile?
      return redirect_to(:action => 'show_addresses') unless request.post?
      # モバイルは日本語しか対応しない
      params[:user_address]["country_code"] = "JP"
      # 「確認」ボタンが押されたかどうかを判別するフラグ
      submit_flg = true

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        submit_flg = false
        params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
        address_form = session[:change_address]
        address_form.merge_only_need(params[:user_address])
        session[:change_address] = address_form
        redirect_to(:action => 'change_address')
        return
      end

      if submit_flg
        address_form = session[:change_address]
        address_form.merge_only_need(params[:user_address])
        unless address_form.valid?
          session[:change_address] = address_form
          flash[:error] = address_form.errors.zip
          redirect_back_or_default(:action => :change_address, :id => address_form.id)
          return
        end
        session[:change_address] = address_form
      end
      @user_address =  address_form
    end

  end
end
