module EcrMobileFront
  module UrlForExtention
    unloadable
    
    def self.included(klass)
      klass.class_eval do
        alias_method_chain :url_for_with_ec_rider, :ecr_mobile_front
      end
    end

    # リンクURLが文字列で渡されたときに、jpmobileがsession_idを付加してくれないため、
    # URLが文字列でもsession_idを付加できるようにurl_forを拡張します。
    def url_for_with_ec_rider_with_ecr_mobile_front(options = {})
      url = url_for_with_ec_rider_without_ecr_mobile_front(options)

      if (options.kind_of?(String) || options[:href] ||
            (url.kind_of?(String) && options[session_key].nil?)) and apply_trans_sid?
        unless options =~ /#{session_key}/
          parsed_url = URI.parse(url)
          if parsed_url.query
            parsed_url.query += "&#{session_key}=#{session_id}"
          else
            parsed_url.query = "#{session_key}=#{session_id}"
          end
          url = parsed_url.to_s
        end
      end

      url
    end

    private

    # session_keyを返す。
    def session_key
      ActionController::Base.session_options[:key] unless @controller
      # rspecのときだけ、request.session_optionsは空でないが、
      # request.session_options[:key]の場合があるため、暫定対応を行います。
      opts1 = @controller.request.session_options
      opts2 = ActionController::Base.session_options
      (opts1 && opts1[:key]) || (opts2 && opts2[:key])
#      (@controller.request.session_options || ActionController::Base.session_options)[:key]
    end

    # session_idを返す
    def session_id
      session.session_id unless @controller
      @controller.request.session_options[:id] rescue session.session_id
    end

    # trans_sidを適用すべきかを返す。
    def apply_trans_sid?
      return false unless @controller
      return false if @controller.trans_sid_mode == :none
      return true if @controller.trans_sid_mode == :always
      if @controller.trans_sid_mode == :mobile
        if @controller.request.mobile?
          return !@controller.request.mobile.supports_cookie?
        else
          return false
        end
      end
      return false
    end
  end
end
