# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe AccountController do
  fixtures :users, :user_emails, :user_addresses, :mail_templates,
    :mail_deliveries, :kbn_codes, :kbn_code_names, :user_mobiles

  before(:each) do
    request.stub!(:mobile?).and_return(true)
    # デフォルトはDocomoですが、日本語の結果を返すテストをするため、Softbankにします。
    # 照合しやすいように、文字コードがutf-8のSoftbankを指定。
    request.stub!(:mobile).and_return(
      mock_model(Jpmobile::Mobile::Softbank, {:supports_cookie? => false}))
    request.session_options[:key] = "session_id"
    ActionController::Base.session_options[:key] = "session_id"
  end

  describe "test confirm_signup" do
    before do
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "ジロウ",  :kana_name_first => "テスト",   :name_last => "0011",
        :name_first => "0012", :zip_code => "0000000",  :address_1 => "1111",  :address_2 => "1112",
        :address_3 => "1-1-1", :country_code => 'JP'}
      @user_email = {:email => "test@test.jp", :pc_mobile_div_key => :mobile}
      @user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970",
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A'}
      @user_en_address = {:name_last => "test", :name_first => "jiro",
        :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}
    end

    it "confirm_signup with get method" do
      get :confirm_signup
      response.should redirect_to(:action => 'signup', :lang => 'ja', :session_id => request.session_options[:id])
    end

    it "confirm_signup invalid" do
      post :confirm_signup, :user_address=>{}
      response.should redirect_to(:action => 'signup', :lang => 'ja', :session_id => request.session_options[:id])
      flash[:user_profile].should_not be_nil
      flash[:user_profile].errors.length.should == 5
      flash[:user_address].should_not be_nil
      flash[:user_address].errors.length.should == 19
      flash[:user_email].should_not be_nil
      flash[:user_email].errors.length.should == 2
      flash[:user_en_address].should be_nil
    end

    it "confirm_signup valid" do
      post :confirm_signup, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      controller.instance_variable_get(:@user_profile).attributes.should ==
        User.new(@user_profile.merge(:member_div_key => :membership, :regist_div_key => :mobile, :tel_num => "000-0000-0000",  :kana_name_last => "ジロウ",
          :kana_name_first => "テスト",   :name_last => "0011", :name_first => "0012", :login => '0011 0012', :pc_mail_magazine_send_div => 99)).attributes
      controller.instance_variable_get(:@user_email).attributes.should ==
        UserEmail.new(@user_email.merge(:user_id => nil)).attributes
      controller.instance_variable_get(:@user_address).attributes.should ==
        UserAddress.new(@user_address.merge(:locale => 'ja', :address_div_key => :user)).attributes
      controller.instance_variable_get(:@user_en_address).attributes.should ==
        UserAddress.new(@user_en_address.merge(:address_div_key => :user_en, :country_code => 'JP',
          :tel_num => "000-0000-0000", :zip_code => "0000000", :locale => 'en')).attributes
      response.should be_success
      response.should render_template(:confirm_signup_mobile_softbank)
    end

    it "confirm_signup with get zip_to_address" do
      UserAddress.stub!(:zip_to_address).and_return(["愛知県", "名古屋市中区丸の内"])
      post :confirm_signup, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address, :zip_to_address=>"郵便番号で住所表示"
      response.should redirect_to(:action => 'signup', :lang => 'ja', :session_id => request.session_options[:id])
      user_address = session[:signup].user_address
      controller.instance_variable_get(:@user_profile).user_address.should == user_address
      user_address.zip_code.should == "0000000"
      user_address.address_1.should == "愛知県"
      user_address.address_2.should == "名古屋市中区丸の内"
    end
  end
  
  describe "test policy" do
    it "normal case" do
      get :policy
      response.should render_template(:policy_information_mobile_softbank)
    end
  end
  
  describe "test create_user" do
    it "create_user with get method" do
      get :create_user
      response.should redirect_to(:action => 'signup', :lang => 'ja', :session_id => request.session_options[:id])
    end

    # TODO テストケース不足

  end
  
  describe "completion_signup" do
    it "forward_url is not nil" do
      forward_url = "/xxx/yyy"
      session[:forward_url] = forward_url
      get "completion_signup"
      response.should render_template(:completion_signup_mobile_softbank)
      session[:forward_url].should be_nil
      controller.instance_variable_get(:@link_url_for_completion_signup).should == forward_url
    end

    it "forward_url is nil" do
      controller.stub!(:current_user).and_return(User.new(:member_category_div_key => :general))
      session[:forward_url] = nil
      get "completion_signup"
      response.should render_template(:completion_signup_mobile_softbank)
      session[:forward_url].should be_nil
      controller.instance_variable_get(:@link_url_for_completion_signup).should == "/member/index/#{User.member_category_div_id_by_key(:general)}"
    end
  end
  
  describe "check_valid_ipフィルターが有効なとき、" do
    before(:each) do
      EcRider.config.stub!(:valid_ip_check).and_return(true)
    end
    describe "不正なIPからアクセスがあると、" do
      before(:each) do
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo,
            {:ident => 'unique1008', :valid_ip? => false, :supports_cookie? => false}))
      end
      it "エラー画面（レスポンスコード=400）に遷移する。" do
        post :authenticate_ez
        response.should_not be_success
        response.code.should == "400"
        response.should render_template("cms/invalid_ip")
      end
    end
    describe "有効なIPからアクセスがあると、" do
      before(:each) do
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo,
            {:ident => 'unique1008', :valid_ip? => true, :supports_cookie? => false}))
      end
      it "正常なレスポンスを返す。" do
        post :authenticate_ez
        response.should redirect_to "http://test.host/ja/account/add_mailaddress?session_id=#{request.session_options[:id]}"
      end
    end
  end
  
  describe "test change_information" do
    before do
      session[:user] = 1001
    end

    it "change_information with flash" do
      flash[:user_profile] = mock_user = mock_model(User)
      flash[:user_address] = mock_address = mock_model(UserAddress)
      flash[:user_email] = mock_email = mock_model(UserEmail)
      flash[:user_en_address] = mock_sub_address = mock_model(UserAddress)
      mock_user.should_receive(:not_decided_address=).with(true).once

      get :change_information
      response.should render_template('change_information_mobile_softbank')
      session[:account].should == nil
      controller.instance_variable_get(:@user_profile).should == mock_user
      controller.instance_variable_get(:@user_address).should == mock_address
      controller.instance_variable_get(:@user_email).should == mock_email
      controller.instance_variable_get(:@user_en_address).should == mock_sub_address
    end

    it "change_information without flash mobile only" do
      mock_user = User.find(1001)
      mock_user.user_email.pc_mobile_div_key.should == :pc
      mock_user.user_email.pc_mobile_div_key = :mobile
      mock_user.user_email.save!
      mock_user = User.find(1001)
      mock_user.user_email.should be_nil
      mock_user.user_email_mobile.should_not be_nil
      get :change_information
      response.should render_template('change_information_mobile_softbank')
      session[:account].should == nil
      controller.instance_variable_get(:@user_profile).id.should == 1001
      controller.instance_variable_get(:@user_address).id.should == mock_user.user_address.id
      controller.instance_variable_get(:@user_email).id.should == mock_user.user_email_mobile.id
      controller.instance_variable_get(:@user_en_address).should be_nil
    end

    it "change_information without flash mobile and pc" do
      mock_user = User.find(1001)
      mock_user.user_email.pc_mobile_div_key.should == :pc
      mock_user.user_email_mobile = UserEmail.new(:email => "test@test.jp", :pc_mobile_div => "2")
      mock_user.user_email_mobile.save!
      mock_user.user_email.should_not be_nil
      mock_user.user_email_mobile.should_not be_nil
      get :change_information
      response.should render_template('change_information_mobile_softbank')
      session[:account].should == nil
      controller.instance_variable_get(:@user_profile).id.should == 1001
      controller.instance_variable_get(:@user_address).id.should == mock_user.user_address.id
      controller.instance_variable_get(:@user_email).id.should_not == mock_user.user_email.id
      controller.instance_variable_get(:@user_email).id.should == mock_user.user_email_mobile.id
      controller.instance_variable_get(:@user_en_address).should be_nil
    end
  end
  
  describe "test confirm_update_information" do
    before do
      session[:user] = 1001
      mock_user = User.find(1001)
      mock_user.regist_div_key = :mobile
      mock_user.user_email.pc_mobile_div_key = :mobile
      mock_user.save!
      mock_user.user_email.save!
      @mock_user = User.find(1001)
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",  :kana_name_first => "ジロウ",   :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000",  :address_1 => "福岡県",  :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP'}
      @user_email = {:email => "test@test.jp", :pc_mobile_div => "2"} # モバイル用のメールアドレスが入力される前提
      @user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970"}
      @user_en_address = {:name_last => "test", :name_first => "jiro",
        :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}
    end

    it "confirm_update_information with get method" do
      get :confirm_update_information
      response.should redirect_to(:action => 'change_information', :lang => 'ja', :session_id => request.session_options[:id])
    end

    it "confirm_update_information with get zip_to_address" do
      UserAddress.stub!(:zip_to_address).and_return(["愛知県", "名古屋市中区丸の内"])
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address, :zip_to_address=>"郵便番号で住所表示"
      response.should redirect_to(:action => 'change_information', :lang => 'ja', :session_id => request.session_options[:id])
      controller.instance_variable_get(:@user_profile).id.should == 1001
      user_address = flash[:user_address]
      controller.instance_variable_get(:@user_profile).user_address.should == user_address
      user_address.zip_code.should == "0000000"
      user_address.address_1.should == "愛知県"
      user_address.address_2.should == "名古屋市中区丸の内"
    end

    it "confirm_update_information invalid mobile mail" do
      @user_address[:name_first] = ''
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should redirect_to(:action => 'change_information', :lang => 'ja', :session_id => request.session_options[:id])
      session[:account][:user_profile].id.should == @mock_user.id
      session[:account][:user_email].id.should == @mock_user.user_email_mobile.id
      session[:account][:user_email].pc_mobile_div_key.should == :mobile
      session[:account][:user_address].id.should == @mock_user.user_address.id
      session[:account][:user_en_address].id.should == @mock_user.sub_address.id
      flash[:user_address].errors.length.should == 2
    end

    it "confirm_update_information add sub_address" do
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should render_template('confirm_update_information_mobile_softbank')
      @mock_user.attributes = @user_profile.merge(:tel_num => "000-0000-0000",  :kana_name_last => "テスト", :kana_name_first => "ジロウ",
        :name_last => "てすと", :name_first => "じろう")
      controller.instance_variable_get(:@user_profile).attributes.should == @mock_user.attributes
      mock_email_mobile = @mock_user.user_email_mobile
      mock_email_mobile.attributes = @user_email
      controller.instance_variable_get(:@user_email).attributes.should == mock_email_mobile.attributes
      mock_address = @mock_user.user_address
      mock_address.attributes = @user_address
      controller.instance_variable_get(:@user_address).attributes.should == mock_address.attributes
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
      controller.instance_variable_get(:@user_en_address).name_last.should == "test"
      controller.instance_variable_get(:@user_en_address).tel_num.should == "000-0000-0000"
    end

    it "confirm_update_information change sub_address" do
      session[:user] = 1008
      mock_user = User.find(1008)
      mock_user.regist_div_key = :mobile
      mock_user.user_email.pc_mobile_div_key = :mobile
      mock_user.save!
      mock_user.user_email.save!
      @mock_user = User.find(1008)
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should render_template('confirm_update_information_mobile_softbank')
      @mock_user.attributes = @user_profile.merge(:tel_num => "000-0000-0000",  :kana_name_last => "テスト", :kana_name_first => "ジロウ",
        :name_last => "てすと", :name_first => "じろう")
      controller.instance_variable_get(:@user_profile).attributes.should == @mock_user.attributes
      mock_email_mobile = @mock_user.user_email_mobile
      mock_email_mobile.attributes = @user_email
      controller.instance_variable_get(:@user_email).attributes.should == mock_email_mobile.attributes
      mock_address = @mock_user.user_address
      mock_address.attributes = @user_address
      controller.instance_variable_get(:@user_address).attributes.should == mock_address.attributes
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
      controller.instance_variable_get(:@user_en_address).name_last.should == "test"
      controller.instance_variable_get(:@user_en_address).tel_num.should == "000-0000-0000"
    end
  end
  
  describe "test update_information" do
    before do
      session[:user] = 1001
      user = User.find(1001)
      user.user_email.pc_mobile_div_key = :mobile
      user.user_email.save!
      user = User.find(1008)
      user.user_email.pc_mobile_div_key = :mobile
      user.user_email.save!
    end

    it "update_information with get method" do
      get :update_information
      response.should redirect_to(:action => 'change_information', :lang => 'ja', :session_id => request.session_options[:id])
    end

    it "update_information add sub_address" do
      mock_user = User.find(1001)
      mock_user.name_first = 'test1'
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email_mobile
      session[:account][:user_address] = mock_user.user_address
      mock_sub_address = UserAddress.new(UserAddress.find(1012).attributes)
      mock_sub_address.user_id = 1001
      session[:account][:user_en_address] = mock_sub_address
      address_count = UserAddress.count(:conditions => {:user_id => 1001})
      post :update_information
      response.should  render_template('completion_update_mobile_softbank')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1001}).should == address_count + 1
      User.find(1001).name_first.should == 'test1'
      User.find(1001).sub_address.should_not be_nil
      User.find(1008).test_flg.should be_nil
    end

    it "update_information delete sub_address" do
      mock_user = User.find(1008)
      mock_user.name_first = 'test1'
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email_mobile
      session[:account][:user_address] = mock_user.user_address
      session[:account][:user_en_address] = mock_user.sub_address
      session[:delete_en_address] = true

      address_count = UserAddress.count(:conditions => {:user_id => 1008})
      post :update_information
      response.should  render_template('completion_update_mobile_softbank')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1008}).should == address_count - 1
      User.find(1008).name_first.should == 'test1'
      User.find(1008).sub_address.should be_nil
      User.find(1008).test_flg.should be_nil
    end

    it "update_information change sub_address" do
      mock_user = User.find(1008)
      mock_user.name_first = 'test1'
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email_mobile
      session[:account][:user_address] = mock_user.user_address
      session[:account][:user_en_address] = mock_user.sub_address

      address_count = UserAddress.count(:conditions => {:user_id => 1008})
      post :update_information
      response.should render_template('completion_update_mobile_softbank')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1008}).should == address_count
      User.find(1008).name_first.should == 'test1'
      User.find(1008).sub_address.should_not be_nil
      User.find(1008).test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      mock_user = User.find(1008)
      mock_user.name_first = 'test1'
      mock_user.test_flg = true
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email_mobile
      session[:account][:user_address] = mock_user.user_address
      session[:account][:user_en_address] = mock_user.sub_address

      address_count = UserAddress.count(:conditions => {:user_id => 1008})
      post :update_information
      response.should render_template('completion_update_mobile_softbank')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1008}).should == address_count
      User.find(1008).name_first.should == 'test1'
      User.find(1008).sub_address.should_not be_nil
      User.find(1008).test_flg.should be_true
    end
  end

  describe "PC会員が携帯会員になるフロー" do
    before(:each) do
      request.stub!(:mobile).and_return(
        mock_model(Jpmobile::Mobile::Softbank, {:supports_cookie? => true}))
    end

    def user_has_not_mobile_email_address
      @user1 ||= User.find(1002)
    end

    def user_has_mobile_email_address
      if @user2.nil?
        @user2 = User.find(1002)
        @user2.user_email = UserEmail.new(:email => 'sample@docomo.ne.jp', :pc_mobile_div => 2)
        @user2.save!
      end
      @user2 ||= User.find(@user2.id)
    end

    def mock_user_login_form(stub)
      @mock_login_form ||= mock_model(UserLoginForm, stub)
    end

    describe "ログイン認証" do
      it "携帯メールアドレスが登録済みの場合、会員画面に遷移する" do
        UserLoginForm.stub!(:new).and_return(
          mock_user_login_form(
            :email => "sample.docomo.ne.jp",
            :authenticate => user_has_mobile_email_address)
        )

        post :authenticate
        response.should redirect_to(:controller => 'member', :lang => 'ja', :session_id => request.session_options[:id])
      end

      it "携帯メールアドレスが未登録の場合、携帯メールアドレス登録画面に遷移する" do
        UserLoginForm.stub!(:new).and_return(
          mock_user_login_form(
            :email => "sample.docomo.ne.jp",
            :authenticate => user_has_not_mobile_email_address)
        )

        post :authenticate
        response.should redirect_to(:action => 'add_mailaddress', :lang => 'ja', :session_id => request.session_options[:id])
      end
    end
    
    describe "携帯メールアドレス登録フォーム表示" do
      it "携帯メールアドレスが登録画面が表示される" do
        session[:add_mailaddress] = {:target_user => user_has_not_mobile_email_address, :back_to_url => '/test', :remember_me => false}

        get :add_mailaddress
        response.should render_template("account/add_mailaddress_mobile_softbank")
      end
    end
    
    describe "携帯メールアドレス確認" do
      it "携帯メールアドレス確認画面を表示する" do
        session[:add_mailaddress] = {:target_user => user_has_not_mobile_email_address, :back_to_url => '/test', :remember_me => false}
        controller.stub!(:current_user).and_return(user_has_not_mobile_email_address)

        post :confirm_mailaddress, :user_email => {:email => 'sampel@docomo.ne.jp'}
        response.should render_template("account/confirm_mailaddress_mobile_softbank")
      end
    end
        
    describe "携帯メールアドレス登録" do
      it "登録される" do
        session[:add_mailaddress] = {
          :target_user => user_has_not_mobile_email_address,
          :back_to_url => '/test',
          :remember_me => false,
          :user_email => UserEmail.new(:user_id => 1002, :email => 'sampel@docomo.ne.jp', :pc_mobile_div => '2')
        }
        
        post :update_mailaddress
        response.should redirect_to(:controller => :member, :lang => 'ja', :session_id => request.session_options[:id])
      end

      it "携帯メールアドレスが入力チェックでエラー" do
        session[:add_mailaddress] = {:target_user => user_has_not_mobile_email_address, :back_to_url => '/test', :remember_me => false}
        session[:add_mailaddress][:user_email] = mock_model(UserEmail, {:save => false, :user_id => 1002})

        post :update_mailaddress
        response.should redirect_to(:action => :add_mailaddress, :lang => 'ja', :session_id => request.session_options[:id])
      end
    end
  end

  describe "update_security_answer" do
    before do
      session[:user] = 1001
      request.stub!(:mobile?).and_return(true)
      request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Softbank, {:supports_cookie? => false}))
    end
    it "update_security_answer should be success" do
      session[:user] = 1001
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      post :update_security_answer
      response.should render_template('account/completion_security_answer_mobile_softbank')
      User.find(1001).remember_question.should == 'QQQ'
      User.find(1001).remember_answer.should == 'AAA'
      User.find(1001).test_flg.should be_nil
    end

    it "update_security_answer should be success false" do
      session[:user] = 1001
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      controller.stub!(:current_user).and_return(User.new(:id => '不正なID'))
      post :update_security_answer
      response.should redirect_to(:action => :change_security_answer, :lang => 'ja', :session_id => request.session_options[:id])
      User.find(1001).test_flg.should be_nil
    end

    describe "update_password のテスト" do
      before do
        session[:user] = 1001
      end
      it "パスワードが正しい場合、パスワード変更完了画面に遷移すればOK" do
        session[:user] = 1001
        post :update_password, :user => {:password => 'pass2', :password_confirmation => 'pass2'}
        response.should redirect_to(:action => :change_password_completion, :lang => 'ja', :session_id => request.session_options[:id])
        controller.instance_variable_get(:@user).id = 1001
        controller.instance_variable_get(:@user).password.should == 'pass2'
      end

      it "パスワードが不正（英字不組合わせ）な場合、パスワード設定画面に戻ればOK" do
        session[:user] = 1001
        post :update_password, :user => {:password => 'pass', :password_confirmation => 'pass'}
        response.should redirect_to(:action => :change_password, :lang => 'ja', :session_id => request.session_options[:id])
      end

      it "パスワード未入力の場合、パスワード設定画面に戻ればOK" do
        post :update_password, :user => {:password => '', :password_confirmation => ''}
        response.should redirect_to(:action => :change_password, :lang => 'ja', :session_id => request.session_options[:id])
      end

      it "パスワードが不正（4文字未満）の場合、パスワード設定画面に戻ればOK" do
        post :update_password, :user => {:password => 'a12', :password_confirmation => 'a12'}
        response.should redirect_to(:action => :change_password, :lang => 'ja', :session_id => request.session_options[:id])
      end
    end
  end
end