# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe InquiryController do
  fixtures :users, :user_addresses, :user_emails, :inquiries, :shop_settings,
    :products, :product_displays, :inquiry_products

  describe "Get 'index'" do
    before do
      user = users(:users_00003)
      user.user_email_mobile = UserEmail.new(:pc_mobile_div_key => :mobile, :email => 'email_address_mobile@ec-one.com')
      user.user_email_mobile.save!
    end

    describe "mobile" do
      before do
        request.stub!(:mobile?).and_return(true)
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo, {:supports_cookie? => false}))
        request.session_options[:key] = "session_id"
        ActionController::Base.session_options[:key] = "session_id"
      end

      it "#210 attributes set. for mobile" do
        session[:user] = users(:users_00003).id
        flash[:inquiry] = nil
        session[:inquiry] = nil
        get(:index)
        controller.instance_variable_get(:@inquiry).email.should == "email_address_mobile@ec-one.com"
        controller.instance_variable_get(:@inquiry).name_last.should == "山田"
        response.should be_success
      end

      it "#210 attributes set. inquiry type product for mobile" do
        session[:user] = users(:users_00003).id
        flash[:inquiry] = nil
        session[:inquiry] = nil
        get(:index, :product_code => '0000000009051')
        controller.instance_variable_get(:@inquiry_product).should_not == nil
        controller.instance_variable_get(:@inquiry_product).id.should == 1
        controller.instance_variable_get(:@inquiry).email.should == "email_address_mobile@ec-one.com"
        controller.instance_variable_get(:@inquiry).name_last.should == "山田"
        response.should be_success
      end
    end

    describe "pc" do
      before do
        request.stub!(:mobile?).and_return(false)
      end

      it "#210 attributes set." do
        session[:user] = users(:users_00003).id
        flash[:inquiry] = nil
        session[:inquiry] = nil
        get(:index)
        controller.instance_variable_get(:@inquiry).email.should == "email_address_01002@ec-one.com"
        controller.instance_variable_get(:@inquiry).name_last.should == "山田"
        response.should be_success
      end

      it "#210 attributes set. inquiry type product" do
        session[:user] = users(:users_00003).id
        flash[:inquiry] = nil
        session[:inquiry] = nil
        get(:index, :product_code => '0000000009051')
        controller.instance_variable_get(:@inquiry_product).should_not == nil
        controller.instance_variable_get(:@inquiry_product).id.should == 1
        controller.instance_variable_get(:@inquiry).email.should == "email_address_01002@ec-one.com"
        controller.instance_variable_get(:@inquiry).name_last.should == "山田"
        response.should be_success
      end
    end
  end
end