# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe MemberController do
  fixtures :users, :user_emails, :user_addresses, :mail_templates,
    :mail_deliveries, :kbn_codes, :kbn_code_names, :user_mobiles

  before(:each) do
    request.stub!(:mobile?).and_return(true)
    request.stub!(:mobile).and_return(
      mock_model(Jpmobile::Mobile::Softbank, {:supports_cookie? => false}))
    request.session_options[:key] = "session_id"
    ActionController::Base.session_options[:key] = "session_id"
  end

  describe "check_valid_ipフィルターが有効なとき、" do
    before(:each) do
      EcRider.config.stub!(:valid_ip_check).and_return(true)
    end
    describe "不正なIPからアクセスがあると、" do
      before(:each) do
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo,
          {:ident => 'unique1008', :valid_ip? => false, :supports_cookie? => false}))
      end
      it "エラー画面（レスポンスコード=400）に遷移する。" do
        post :complete_config_ezlogin, :mobile_unique_id => 'unique1008'
        response.should_not be_success
        response.code.should == "400"
        response.should render_template("cms/invalid_ip")
      end
    end
    describe "有効なIPからアクセスがあると、" do
      before(:each) do
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo,
          {:ident => 'unique1008', :valid_ip? => true, :supports_cookie? => false}))
      end
      it "正常なレスポンスを返す。" do
        post :complete_config_ezlogin, :mobile_unique_id => 'unique1008'
        response.should redirect_to "http://test.host/ja?session_id=#{request.session_options[:id]}"
      end
    end
  end

  describe "confirm_modify_change" do
    before do
      session[:user] = User.find(1000)
    end
    it "confirm_modify_change with get" do
      get :confirm_modify_change
      response.should redirect_to(:action => :show_addresses, :lang => 'ja', :session_id => request.session_options[:id])
    end
    it "confirm_modify_change with post" do
      session[:change_address] = UserAddress.find(:first)
      post :confirm_modify_change, :user_address => {}
      response.should render_template('member/confirm_modify_change_mobile_softbank')
    end
    it "confirm_modify_change with zip_to_address" do
      UserAddress.stub!(:zip_to_address).and_return(["愛知県", "名古屋市中区丸の内"])
      session[:change_address] = UserAddress.find(:first)
      post :confirm_modify_change, :user_address => {:zip_code => '0000000'}, :zip_to_address => 'test'
      response.should redirect_to(:action => :change_address, :lang => 'ja', :session_id => request.session_options[:id])
    end
  end

  describe "change_address" do
    before do
      session[:user] = User.find(1000)
    end
    it "change_address without parameter" do
      get :change_address
      response.should redirect_to(:action => :show_addresses, :lang => 'ja', :session_id => request.session_options[:id])
    end
    it "change_address with parameter" do
      get :change_address, :user_address => {:id => 1000}
      response.should render_template('member/change_address_mobile_softbank')
    end
    it "change_address with session" do
      session[:change_address] = UserAddress.find(1000)
      get :change_address
      response.should render_template('member/change_address_mobile_softbank')
    end
  end

end