require File.dirname(__FILE__) + '/../spec_helper'
describe ApplicationHelper, :type => :helper do
  include ApplicationHelper

  describe "url_for without session_id" do
    it "args is hash" do
      helper.url_for(:controller => 'cart', :action => 'index').should == '/cart'
    end
    it "args is string" do
      helper.url_for('http://www.testtesttest.com/').should == 'http://www.testtesttest.com/'
    end
    it "args is string with url_parameter" do
      helper.url_for('http://www.testtesttest.com/?id=10').should == 'http://www.testtesttest.com/?id=10'
    end
    it "args is hash_href with url_parameter" do
      helper.url_for(:href => 'http://www.testtesttest.com/?id=10').should == 'http://www.testtesttest.com/?id=10'
    end
    it "args is hash_href with url_parameter escape_false" do
      helper.url_for(:href => 'http://www.testtesttest.com/?id=10', :escape => false).should == 'http://www.testtesttest.com/?id=10'
    end
  end

  describe "url_for with session_id always" do
    before do
      @helper = helper
      app = ActionController::Base.new()
      app.stub!(:trans_sid_mode).and_return(:always)
      app.request.stub!(:session_options).and_return({:key => 'session_id', :id => 'testsessionid12345'})
      @helper.instance_variable_set(:@controller, app)
    end
    it "args is hash" do
      pending "元のurl_forを呼ぶ際にnil.rewriteでエラーになる"
      @helper.url_for(:controller => 'cart', :action => 'index').should == '/cart?session_id=testsessionid12345'
    end
    it "args is string" do
      @helper.url_for('http://www.testtesttest.com/').should == 'http://www.testtesttest.com/?session_id=testsessionid12345'
    end
    it "args is string with url_parameter" do
      @helper.url_for('http://www.testtesttest.com/?id=10').should == 'http://www.testtesttest.com/?id=10&session_id=testsessionid12345'
    end
    it "args is hash_href with url_parameter" do
      helper.url_for(:href => 'http://www.testtesttest.com/?id=10').should == 'http://www.testtesttest.com/?id=10&session_id=testsessionid12345'
    end
    it "args is hash_href with url_parameter escape_false" do
      helper.url_for(:href => 'http://www.testtesttest.com/?id=10', :escape => false).should == 'http://www.testtesttest.com/?id=10&session_id=testsessionid12345'
    end
  end

  describe "button_to_url and button_to_remote" do
    describe "for mobile" do
      before(:each) do
        request.stub!(:mobile?).and_return(true)
        request.stub_chain(:mobile, :supports_cookie?).and_return(true)
      end

      it "モバイルプラグインでは、button_to_urlをリンクで描画する" do
        tag = helper.button_to_url('テスト', 'test/index', {:class => 'test1'})
        expected = %{<a href=\"test/index\" class=\"test1\">テスト</a>}
        tag.should == expected
      end

      it "モバイルプラグインでは、button_to_remoteをリンクで描画する" do
        tag =  helper.button_to_remote('テスト', {:url => 'test/index'}, {:class => 'test1'})
        expected = %{<a class=\"test1\" href=\"#\" onclick=\"new Ajax.Request('test/index', {asynchronous:true, evalScripts:true}); return false;\">テスト</a>}
        tag.should == expected
      end
    end

    describe "for pc" do
      before(:each) do
        request.stub!(:mobile?).and_return(false)
        request.stub_chain(:mobile, :supports_cookie?).and_return(false)
      end

      it "モバイルプラグインでは、button_to_urlをリンクで描画する" do
        tag = helper.button_to_url('テスト', 'test/index', {:class => 'test1'})
        expected = %{<input class=\"test1\" onclick=\"location.href='test/index';\" type=\"button\" value=\"テスト\" />}
        tag.should == expected
      end

      it "モバイルプラグインでは、button_to_remoteをリンクで描画する" do
        tag =  helper.button_to_remote('テスト', {:url => 'test/index'}, {:class => 'test1'})
        expected = %{<input class=\"test1\" onclick=\"new Ajax.Request('test/index', {asynchronous:true, evalScripts:true});\" type=\"button\" value=\"テスト\" />}
        tag.should == expected
      end
    end
  end

end