# -*- coding: utf-8 -*-
# The module 'Goldberg' is the namespace for Goldberg's libraries.  It
# also provides a number of class methods for convenient access to
# information about the system from Goldberg's perspective (the
# current user and the System Settings).
module Goldberg

  class << self

    # Accessor to attach the current user (if logged in)
    # attr_accessor :user
    def user; get_on_thread(:user); end
    def user=(value); set_on_thread(:user, value); end

    def clear!
      set_on_thread(:user, nil)
      set_on_thread(:role, nil)
      set_on_thread(:accessible_actions, nil)
      @logger = nil
    end

    def set_user(find_user_id)
      Goldberg.user = find_user_id ? Goldberg::User.find(find_user_id) : nil
      if Goldberg.user
        set_on_thread(:role, Goldberg.user.role)
      else
        # 一般ユーザのロールを取得しています
        set_on_thread(:role, Goldberg::Role.find(3)) # TODO マジックナンバーを消したい！店舗情報でデフォルトロール？を持ってくるようにするとか
      end

      role = get_on_thread(:role)
      if role
        # テストでGoldberg::Roleのstubを使用すると role.respond_to?(:name) が falseを返すことがあります。
        # そのチェックの為に以下のコードを残しておきます。
        unless role.respond_to?(:name)
          puts "role.class.name => #{role.class.name}"
          puts "role.inspect => #{role.inspect}"
          puts "  " << caller.join("\n  ")
        end
        logger.info "Logging in user as role #{role.name}"
      else
        logger.error "Something went seriously wrong with the role"
      end
    end

    def action_authorised?(controller_name, action_name)
      return false unless Goldberg.user
      role = set_on_thread_if_nil(:role, Goldberg.user.role)
      return false unless role
      # Goldberg::RoleAction.count(:conditions => [
      #     "goldberg_role_actions.role_id = ? AND goldberg_site_action_groups.name = ? AND goldberg_site_actions.name = ?", 
      #     role.id, controller_name, action_name],
      #   :joins => (
      #     " inner join goldberg_site_actions on goldberg_site_actions.id = goldberg_role_actions.site_action_id" <<
      #     " inner join goldberg_site_action_groups on goldberg_site_action_groups.id = goldberg_site_actions.site_action_group_id")
      #   ) > 0
      accessible_actions_for_role.include?([controller_name, action_name])
    end

    private

    def accessible_actions_for_role
      set_on_thread_if_nil(:accessible_actions) do
        role = get_on_thread(:role)
        controllers_and_actions = Goldberg::RoleAction.connection.select_all(
          " select goldberg_site_action_groups.name controller_name, goldberg_site_actions.name action_name" <<
          " from goldberg_site_actions" <<
          " inner join goldberg_site_action_groups on goldberg_site_action_groups.id = goldberg_site_actions.site_action_group_id" <<
          " inner join goldberg_role_actions on goldberg_role_actions.site_action_id = goldberg_site_actions.id" <<
          " where goldberg_role_actions.role_id = #{role.id}"
          )
        controllers_and_actions.map do |record|
          [record['controller_name'], record['action_name']]
        end
      end
    end

    def get_on_thread(key)
      Thread.current[actual_key_on_thread(key)]
    end

    def set_on_thread(key, value = nil)
      result = block_given? ? yield : value
      Thread.current[actual_key_on_thread(key)] = result
      result
    end
    
    def set_on_thread_if_nil(key, value = nil)
      actual_key = actual_key_on_thread(key)
      result = Thread.current[actual_key]
      return result unless result.nil?
      result = block_given? ? yield : value
      Thread.current[actual_key] = result
      result
    end
    
    def actual_key_on_thread(key)
      "#{self.name}_#{self.object_id}_#{key.to_s}"
    end


    def logger
      @logger ||= ActiveRecord::Base.logger
    end

  end

end
