EmojiEditor = Class.create();
EmojiEditor.Strategies = {}
EmojiEditor.Strategies.ecrider = {
    prepareForTextToPreview: function(text){
        return text.gsub(/<ec:emoji\s+code=[\'\"][^<>]*[\"\']\/>/m, function(tag){
            var emoji_code = tag[0].gsub(/\D/, '');
            return "<img class='dropped' src='/ec_rider/images/mobile/emoji/" + emoji_code + ".gif'/>";
        });
    },

    droppedToText: function(img){
        return "<ec:emoji code='" + img.src.split(/\//).last().gsub(/\D/, '') + "'/>";
    }
}
EmojiEditor.prototype = {
    initialize: function(text_area, preview_area, options){
        this.text_area = $(text_area);
        this.preview_area = $(preview_area);
        this.emoji_type = "ecrider";
        this.options = Object.extend({
            emoji_image_path: '/ec_rider/images/mobile/emoji',
            emoji_table_columns: 12,
            emoji_gabage_bin_name: 'ゴミ箱'
        }, options || {});
    },

    textToPreview: function(){
        var htmlet = EmojiEditor.Strategies[this.emoji_type].prepareForTextToPreview(this.text_area.value);
        this.preview_area.innerHTML = htmlet;
        htmlet = this._childNodesToDroppable(this.preview_area);
        this.preview_area.innerHTML = htmlet;
        _handler = this._emojiDropped;
        $$("#" + this.preview_area.id +" span.emoji_char").each(function(span){
            Droppables.add(span, {
                accept: 'emoji',
                hoverclass: 'emoji_char_hover',
                onDrop: _handler
            });
        });
        $$("#" + this.preview_area.id + " img.dropped").each(function(img){
            new Draggable(img, {revert: true});
        });

    },

    previewToText: function(){
        this.text_area.value = this._chideNodesToText(this.preview_area);
    },

    buildEmojiTable: function(dest){
        dest = dest ? $(dest) : document.body;
        var emoji_image_path = this.options['emoji_image_path'];
        var emoji_table_columns = this.options.emoji_table_columns;
        this.emoji_table = document.createElement('table');
        this.emoji_table.id = 'emoji_table';
        this.emoji_table.style.display = 'none';
        this.emoji_table.cellspacing = 2;
        this.emoji_table.cellpadding = 2;
        this.emoji_table.border = 0;
        dest.appendChild(this.emoji_table);

        var gabage_bin_tr = document.createElement('tr');
        this.emoji_table.appendChild(gabage_bin_tr);
        var gabage_bin_td = document.createElement('td');
        gabage_bin_td.innerHTML = this.options['emoji_gabage_bin_name'] || 'delete emoji';
        gabage_bin_td.className = "emoji_gabage_bin";
        gabage_bin_td.setAttribute('align', 'center');
        gabage_bin_td.setAttribute('colspan', emoji_table_columns);
        gabage_bin_tr.appendChild(gabage_bin_td);
         Droppables.add(gabage_bin_td, {
             accept: 'dropped',
             hoverclass: 'emoji_gobage_bin_hover',
             onDrop: this._droppedToGabageBin.bind(this)
         });

        var tr = null;
        for(var i=1; i<253; i++){
            if (!tr || ((i % emoji_table_columns) == 1)) {
                tr = document.createElement('tr');
                this.emoji_table.appendChild(tr);
            }
            var td = document.createElement('td');
            tr.appendChild(td);
            var img = document.createElement('img');
            img.className = 'emoji';
            img.src = emoji_image_path + '/' + i + ".gif";
            img.alt = i;
            img.width = 12;
            img.height = 12;
            td.appendChild(img);
        }
        var footer_tr = document.createElement('tr');
        this.emoji_table.appendChild(footer_tr);
        var footer_td = document.createElement('td');
        footer_td.innerHTML = 'Images from <a href="http://www.dspt.net/tools/emoji/" target="blank">http://www.dspt.net/tools/emoji/</a>';
        footer_td.setAttribute('colspan', emoji_table_columns);
        footer_tr.appendChild(footer_td);
        new Effect.Appear(this.emoji_table);
        return this.emoji_table;
    },

    prepareEmojiTable: function(){
        if (!this.emoji_table)
            return;
        $$("#" + this.emoji_table.id + " img").each(function(img){
            new Draggable(img, {revert: true});
        });
    },


    _nodeToDroppable: function(node){
        if (node.nodeType == 1){ // タグの場合
            if (node.childNodes.length > 0){
                var dest = this._nodeToString(node, true);
                dest = dest + this._childNodesToDroppable(node);
                dest = dest + "</" + node.tagName.toLowerCase() + ">";
            } else {
                var dest = this._nodeToString(node, false);
                dest = "<span class='emoji_char'>" + dest + "</span>";
            }
            return dest;
        } else if(node.nodeType == 2) {
            return '';
        } else {
            var dest = "<span class='emoji_char dropped'>";
            dest = dest + node.data.split('').join("</span><span class='emoji_char'>");
            dest = dest + "</span>";
            return dest;
        }
    },

    _childNodesToDroppable: function(src_parent_element){
        var result = '';
        var childNodes = src_parent_element.childNodes;
        var length = childNodes.length;
        for(var i=0; i < length; i++){
            result = result + this._nodeToDroppable(childNodes[i]);
        }
        return result;
    },

    _emojiDropped: function(dragged, acceptance, event) {
        var span = document.createElement('span');
        span.className = "emoji_char"
        var img = document.createElement('img');
        img.src = dragged.src;
        img.alt = dragged.alt;
        img.className = "dropped";
        span.appendChild(img);
        acceptance.parentNode.insertBefore(span, acceptance);
        new Draggable(img, {revert: true});
    },

    _droppedToGabageBin: function(dropped, gabage_bin, event){
        Element.remove(dropped.parentNode);
    },

    _nodeToText: function(node){
        if (node.nodeType == 1){ // タグの場合
            if (node.tagName.match(/SPAN/i) && Element.hasClassName(node, 'emoji_char')){
                if (node.childNodes.length == 1 && node.firstChild.tagName == 'IMG' && Element.hasClassName(node.firstChild, 'dropped')) {
                    return EmojiEditor.Strategies[this.emoji_type].droppedToText(node.firstChild);
                } else {
                    return node.innerHTML;
                }
            } else if(node.childNodes.length < 1) {
                return node.outerHTML;
            } else {
                var dest = this._nodeToString(node, true);
                dest = dest + this._chideNodesToText(node);
                dest = dest + "</" + node.tagName.toLowerCase() + ">";
                return dest;
            }
        } else if(node.nodeType == 2) {
            //何もしません。
        } else {
            return node.data;
        }
    },

    _chideNodesToText: function(parent_element){
        result = "";
        var childNodes = parent_element.childNodes;
        var length = childNodes.length;
        for(var i=0; i < length; i++){
            result = result + this._nodeToText(childNodes[i]);
        }
        return result;
    },

    _nodeToString: function(node, opened){
        var dest = "<" + node.tagName.toLowerCase();
        var attrs = node.attributes;
        var attrs_length = node.attributes.length;
        var attr_array = [];
        for(var j = 0; j < attrs_length; j++){
            var attr = attrs[j];
            attr_array.push(attr.name + '="' + attr.nodeValue + '"');
        }
        if (attr_array.length > 0)
            dest = dest + " " + attr_array.sort().join(" ");
        if (!opened)
            dest = dest + "/";
        dest = dest + ">";
        return dest;
    }

}
