require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe BannerConditionsController do

  before(:each) do
    login_as("admin")
  end

  def mock_banner_condition(stubs={})
    @mock_banner_condition ||= mock_model(BannerCondition, stubs)
  end
  
  describe "GET new" do
    it "assigns a new banner_condition as @banner_condition" do
      BannerCondition.stub!(:new).and_return(mock_banner_condition)
      get :new
      assigns[:banner_condition].should equal(mock_banner_condition)
    end
  end

  describe "GET edit" do
    it "assigns the requested banner_condition as @banner_condition" do
      BannerCondition.stub!(:find).with("37").and_return(mock_banner_condition)
      get :edit, :id => "37"
      assigns[:banner_condition].should equal(mock_banner_condition)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created banner_condition as @banner_condition" do
        BannerCondition.stub!(:new).with({'these' => 'params'}).and_return(mock_banner_condition(:save => true))
        post :create, :banner_condition => {:these => 'params'}
        assigns[:banner_condition].should equal(mock_banner_condition)
      end

      it "redirects to the created banner_condition" do
        BannerCondition.stub!(:new).and_return(mock_banner_condition(:save => true))
        post :create, :banner_condition => {}
        response.should redirect_to(banner_condition_url(mock_banner_condition))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved banner_condition as @banner_condition" do
        BannerCondition.stub!(:new).with({'these' => 'params'}).and_return(mock_banner_condition(:save => false))
        post :create, :banner_condition => {:these => 'params'}
        assigns[:banner_condition].should equal(mock_banner_condition)
      end

    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested banner_condition" do
        BannerCondition.should_receive(:find).with("37").and_return(mock_banner_condition)
        mock_banner_condition.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :banner_condition => {:these => 'params'}
      end

      it "assigns the requested banner_condition as @banner_condition" do
        BannerCondition.stub!(:find).and_return(mock_banner_condition(:update_attributes => true))
        put :update, :id => "1"
        assigns[:banner_condition].should equal(mock_banner_condition)
      end

      it "redirects to the banner_condition" do
        BannerCondition.stub!(:find).and_return(mock_banner_condition(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(banner_condition_url(mock_banner_condition))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested banner_condition" do
        BannerCondition.should_receive(:find).with("37").and_return(mock_banner_condition)
        mock_banner_condition.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :banner_condition => {:these => 'params'}
      end

      it "assigns the banner_condition as @banner_condition" do
        BannerCondition.stub!(:find).and_return(mock_banner_condition(:update_attributes => false))
        put :update, :id => "1"
        assigns[:banner_condition].should equal(mock_banner_condition)
      end

    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested banner_condition" do
      BannerCondition.should_receive(:find).with("37").and_return(mock_banner_condition)
      mock_banner_condition.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the banner_conditions list" do
      BannerCondition.stub!(:find).and_return(mock_banner_condition(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(banner_conditions_url)
    end
  end

end
