require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe BannerDisplaysController do

  before(:each) do
    login_as("admin")
  end

  def mock_banner_display(stubs={})
    @mock_banner_display ||= mock_model(BannerDisplay, stubs)
  end
  
  describe "GET index" do
    it "assigns all banner_displays as @banner_displays" do
      BannerDisplay.stub!(:find).with(:all).and_return([mock_banner_display])
      get :index
      assigns[:banner_displays].should == [mock_banner_display]
    end
  end

  describe "GET show" do
    it "assigns the requested banner_display as @banner_display" do
      BannerDisplay.stub!(:find).with("37").and_return(mock_banner_display)
      get :show, :id => "37"
      assigns[:banner_display].should equal(mock_banner_display)
    end
  end

  describe "GET new" do
    it "assigns a new banner_display as @banner_display" do
      BannerDisplay.stub!(:new).and_return(mock_banner_display)
      get :new
      assigns[:banner_display].should equal(mock_banner_display)
    end
  end

  describe "GET edit" do
    it "assigns the requested banner_display as @banner_display" do
      BannerDisplay.stub!(:find).with("37").and_return(mock_banner_display)
      get :edit, :id => "37"
      assigns[:banner_display].should equal(mock_banner_display)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created banner_display as @banner_display" do
        BannerDisplay.stub!(:new).with({'these' => 'params'}).and_return(mock_banner_display(:save => true))
        post :create, :banner_display => {:these => 'params'}
        assigns[:banner_display].should equal(mock_banner_display)
      end

      it "redirects to the created banner_display" do
        BannerDisplay.stub!(:new).and_return(mock_banner_display(:save => true))
        post :create, :banner_display => {}
        response.should redirect_to(banner_display_url(mock_banner_display))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved banner_display as @banner_display" do
        BannerDisplay.stub!(:new).with({'these' => 'params'}).and_return(mock_banner_display(:save => false))
        post :create, :banner_display => {:these => 'params'}
        assigns[:banner_display].should equal(mock_banner_display)
      end

    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested banner_display" do
        BannerDisplay.should_receive(:find).with("37").and_return(mock_banner_display)
        mock_banner_display.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :banner_display => {:these => 'params'}
      end

      it "assigns the requested banner_display as @banner_display" do
        BannerDisplay.stub!(:find).and_return(mock_banner_display(:update_attributes => true))
        put :update, :id => "1"
        assigns[:banner_display].should equal(mock_banner_display)
      end

      it "redirects to the banner_display" do
        BannerDisplay.stub!(:find).and_return(mock_banner_display(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(banner_display_url(mock_banner_display))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested banner_display" do
        BannerDisplay.should_receive(:find).with("37").and_return(mock_banner_display)
        mock_banner_display.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :banner_display => {:these => 'params'}
      end

      it "assigns the banner_display as @banner_display" do
        BannerDisplay.stub!(:find).and_return(mock_banner_display(:update_attributes => false))
        put :update, :id => "1"
        assigns[:banner_display].should equal(mock_banner_display)
      end

    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested banner_display" do
      BannerDisplay.should_receive(:find).with("37").and_return(mock_banner_display)
      mock_banner_display.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the banner_displays list" do
      BannerDisplay.stub!(:find).and_return(mock_banner_display(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(banner_displays_url)
    end
  end

end
