require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignConditionsController do

  before(:each) do
    login_as("admin")
  end


  def mock_campaign_condition(stubs={})
    @mock_campaign_condition ||= mock_model(CampaignCondition, stubs)
  end
  
  describe "GET index" do
    it "assigns all campaign_conditions as @campaign_conditions" do
      CampaignCondition.stub!(:find).with(:all).and_return([mock_campaign_condition])
      get :index
      assigns[:campaign_conditions].should == [mock_campaign_condition]
    end
  end

  describe "GET show" do
    it "assigns the requested campaign_condition as @campaign_condition" do
      CampaignCondition.stub!(:find).with("37").and_return(mock_campaign_condition)
      get :show, :id => "37"
      assigns[:campaign_condition].should equal(mock_campaign_condition)
    end
  end

  describe "GET new" do
    it "assigns a new campaign_condition as @campaign_condition" do
      CampaignCondition.stub!(:new).and_return(mock_campaign_condition)
      get :new
      assigns[:campaign_condition].should equal(mock_campaign_condition)
    end
  end

  describe "GET edit" do
    it "assigns the requested campaign_condition as @campaign_condition" do
      CampaignCondition.stub!(:find).with("37").and_return(mock_campaign_condition)
      get :edit, :id => "37"
      assigns[:campaign_condition].should equal(mock_campaign_condition)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created campaign_condition as @campaign_condition" do
        CampaignCondition.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign_condition(:save => true))
        post :create, :campaign_condition => {:these => 'params'}
        assigns[:campaign_condition].should equal(mock_campaign_condition)
      end

      it "redirects to the created campaign_condition" do
        CampaignCondition.stub!(:new).and_return(mock_campaign_condition(:save => true))
        post :create, :campaign_condition => {}
        response.should redirect_to(campaign_condition_url(mock_campaign_condition))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved campaign_condition as @campaign_condition" do
        CampaignCondition.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign_condition(:save => false))
        post :create, :campaign_condition => {:these => 'params'}
        assigns[:campaign_condition].should equal(mock_campaign_condition)
      end

      it "re-renders the 'new' template" do
        CampaignCondition.stub!(:new).and_return(mock_campaign_condition(:save => false))
        post :create, :campaign_condition => {}
        response.should render_template('new')
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested campaign_condition" do
        CampaignCondition.should_receive(:find).with("37").and_return(mock_campaign_condition)
        mock_campaign_condition.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign_condition => {:these => 'params'}
      end

      it "assigns the requested campaign_condition as @campaign_condition" do
        CampaignCondition.stub!(:find).and_return(mock_campaign_condition(:update_attributes => true))
        mock_campaign_condition.should_receive(:clear_aggregation_cache).once
        mock_campaign_condition.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        assigns[:campaign_condition].should equal(mock_campaign_condition)
      end

      it "redirects to the campaign_condition" do
        CampaignCondition.stub!(:find).and_return(mock_campaign_condition(:update_attributes => true))
        mock_campaign_condition.should_receive(:clear_aggregation_cache).once
        mock_campaign_condition.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        response.should redirect_to(campaign_condition_url(mock_campaign_condition))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested campaign_condition" do
        CampaignCondition.should_receive(:find).with("37").and_return(mock_campaign_condition)
        mock_campaign_condition.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign_condition => {:these => 'params'}
      end

      it "assigns the campaign_condition as @campaign_condition" do
        CampaignCondition.stub!(:find).and_return(mock_campaign_condition(:update_attributes => false))
        put :update, :id => "1"
        assigns[:campaign_condition].should equal(mock_campaign_condition)
      end

      it "re-renders the 'edit' template" do
        CampaignCondition.stub!(:find).and_return(mock_campaign_condition(:update_attributes => false))
        put :update, :id => "1"
        response.should render_template('edit')
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested campaign_condition" do
      CampaignCondition.should_receive(:find).with("37").and_return(mock_campaign_condition)
      mock_campaign_condition.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the campaign_conditions list" do
      CampaignCondition.stub!(:find).and_return(mock_campaign_condition(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(campaign_conditions_url)
    end
  end

  describe "GET names" do
    it "return name and id" do
      CampaignCondition.stub!(:find_with_name).with('searching string').
        and_return([mock_campaign_condition(:id => 1)])
      mock_campaign_condition.should_receive(:name_for_nested_ajax).and_return("Ivica Osim")
      get :names, :name => 'searching string'
    end
  end

end
