require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignEffectsController do

  def login_as_admin
    login_as("admin")
  end

  before(:each) do
    login_as_admin
  end

  def mock_campaign_effect(stubs={})
    @mock_campaign_effect ||= mock_model(CampaignEffect, stubs)
  end
  
  describe "GET index" do
    it "assigns all campaign_effects as @campaign_effects" do
      CampaignEffect.stub!(:find).with(:all).and_return([mock_campaign_effect])
      get :index
      assigns[:campaign_effects].should == [mock_campaign_effect]
    end
  end

  describe "GET show" do
    it "assigns the requested campaign_effect as @campaign_effect" do
      CampaignEffect.stub!(:find).with("37").and_return(mock_campaign_effect)
      get :show, :id => "37"
      assigns[:campaign_effect].should equal(mock_campaign_effect)
    end
  end

  describe "GET new" do
    it "assigns a new campaign_effect as @campaign_effect" do
      CampaignEffect.stub!(:new).and_return(mock_campaign_effect)
      get :new
      assigns[:campaign_effect].should equal(mock_campaign_effect)
    end
  end

  describe "GET edit" do
    it "assigns the requested campaign_effect as @campaign_effect" do
      CampaignEffect.stub!(:find).with("37").and_return(mock_campaign_effect)
      get :edit, :id => "37"
      assigns[:campaign_effect].should equal(mock_campaign_effect)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created campaign_effect as @campaign_effect" do
        CampaignEffect.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign_effect(:save => true))
        post :create, :campaign_effect => {:these => 'params'}
        assigns[:campaign_effect].should equal(mock_campaign_effect)
      end

      it "redirects to the created campaign_effect" do
        CampaignEffect.stub!(:new).and_return(mock_campaign_effect(:save => true))
        post :create, :campaign_effect => {}
        response.should redirect_to(campaign_effect_url(mock_campaign_effect))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved campaign_effect as @campaign_effect" do
        CampaignEffect.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign_effect(:save => false))
        post :create, :campaign_effect => {:these => 'params'}
        assigns[:campaign_effect].should equal(mock_campaign_effect)
      end

      it "re-renders the 'new' template" do
        CampaignEffect.stub!(:new).and_return(mock_campaign_effect(:save => false))
        post :create, :campaign_effect => {}
        response.should render_template('new')
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested campaign_effect" do
        CampaignEffect.should_receive(:find).with("37").and_return(mock_campaign_effect)
        mock_campaign_effect.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign_effect => {:these => 'params'}
      end

      it "assigns the requested campaign_effect as @campaign_effect" do
        CampaignEffect.stub!(:find).and_return(mock_campaign_effect(:update_attributes => true))
        mock_campaign_effect.should_receive(:clear_aggregation_cache).once
        mock_campaign_effect.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        assigns[:campaign_effect].should equal(mock_campaign_effect)
      end

      it "redirects to the campaign_effect" do
        CampaignEffect.stub!(:find).and_return(mock_campaign_effect(:update_attributes => true))
        mock_campaign_effect.should_receive(:clear_aggregation_cache).once
        mock_campaign_effect.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        response.should redirect_to(campaign_effect_url(mock_campaign_effect))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested campaign_effect" do
        CampaignEffect.should_receive(:find).with("37").and_return(mock_campaign_effect)
        mock_campaign_effect.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign_effect => {:these => 'params'}
      end

      it "assigns the campaign_effect as @campaign_effect" do
        CampaignEffect.stub!(:find).and_return(mock_campaign_effect(:update_attributes => false))
        put :update, :id => "1"
        assigns[:campaign_effect].should equal(mock_campaign_effect)
      end

      it "re-renders the 'edit' template" do
        CampaignEffect.stub!(:find).and_return(mock_campaign_effect(:update_attributes => false))
        put :update, :id => "1"
        response.should render_template('edit')
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested campaign_effect" do
      CampaignEffect.should_receive(:find).with("37").and_return(mock_campaign_effect)
      mock_campaign_effect.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the campaign_effects list" do
      CampaignEffect.stub!(:find).and_return(mock_campaign_effect(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(campaign_effects_url)
    end
  end

  describe "GET names" do
    it "return name and id" do
      CampaignEffect.stub!(:find_with_name).with('searching string').
        and_return([mock_campaign_effect(:id => 1)])
      mock_campaign_effect.should_receive(:name_for_nested_ajax).and_return("Ivica Osim")
      get :names, :name => 'searching string'
    end
  end

end
