require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignTargetProductSkusController do

  before(:each) do
    login_as("admin")
  end

  def mock_campaign_target_product_sku(stubs={})
    @mock_campaign_target_product_sku ||= mock_model(CampaignTargetProductSku, stubs)
  end
  
  describe "GET index" do
    it "assigns all campaign_target_product_skus as @campaign_target_product_skus" do
      CampaignTargetProductSku.stub!(:find).with(:all).and_return([mock_campaign_target_product_sku])
      get :index
      assigns[:campaign_target_product_skus].should == [mock_campaign_target_product_sku]
    end
  end

  describe "GET show" do
    it "assigns the requested campaign_target_product_sku as @campaign_target_product_sku" do
      CampaignTargetProductSku.stub!(:find).with("37").and_return(mock_campaign_target_product_sku)
      get :show, :id => "37"
      assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
    end
  end

  describe "GET new" do
    it "assigns a new campaign_target_product_sku as @campaign_target_product_sku" do
      CampaignTargetProductSku.stub!(:new).and_return(mock_campaign_target_product_sku)
      get :new
      assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
    end
  end

  describe "GET edit" do
    it "assigns the requested campaign_target_product_sku as @campaign_target_product_sku" do
      CampaignTargetProductSku.stub!(:find).with("37").and_return(mock_campaign_target_product_sku)
      get :edit, :id => "37"
      assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created campaign_target_product_sku as @campaign_target_product_sku" do
        CampaignTargetProductSku.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign_target_product_sku(:save => true))
        post :create, :campaign_target_product_sku => {:these => 'params'}
        assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
      end

      it "redirects to the created campaign_target_product_sku" do
        CampaignTargetProductSku.stub!(:new).and_return(mock_campaign_target_product_sku(:save => true))
        post :create, :campaign_target_product_sku => {}
        response.should redirect_to(campaign_target_product_sku_url(mock_campaign_target_product_sku))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved campaign_target_product_sku as @campaign_target_product_sku" do
        CampaignTargetProductSku.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign_target_product_sku(:save => false))
        post :create, :campaign_target_product_sku => {:these => 'params'}
        assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested campaign_target_product_sku" do
        CampaignTargetProductSku.should_receive(:find).with("37").and_return(mock_campaign_target_product_sku)
        mock_campaign_target_product_sku.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign_target_product_sku => {:these => 'params'}
      end

      it "assigns the requested campaign_target_product_sku as @campaign_target_product_sku" do
        CampaignTargetProductSku.stub!(:find).and_return(mock_campaign_target_product_sku(:update_attributes => true))
        mock_campaign_target_product_sku.should_receive(:clear_aggregation_cache).once
        mock_campaign_target_product_sku.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
      end

      it "redirects to the campaign_target_product_sku" do
        CampaignTargetProductSku.stub!(:find).and_return(mock_campaign_target_product_sku(:update_attributes => true))
        mock_campaign_target_product_sku.should_receive(:clear_aggregation_cache).once
        mock_campaign_target_product_sku.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        response.should redirect_to(campaign_target_product_sku_url(mock_campaign_target_product_sku))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested campaign_target_product_sku" do
        CampaignTargetProductSku.should_receive(:find).with("37").and_return(mock_campaign_target_product_sku)
        mock_campaign_target_product_sku.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign_target_product_sku => {:these => 'params'}
      end

      it "assigns the campaign_target_product_sku as @campaign_target_product_sku" do
        CampaignTargetProductSku.stub!(:find).and_return(mock_campaign_target_product_sku(:update_attributes => false))
        put :update, :id => "1"
        assigns[:campaign_target_product_sku].should equal(mock_campaign_target_product_sku)
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested campaign_target_product_sku" do
      CampaignTargetProductSku.should_receive(:find).with("37").and_return(mock_campaign_target_product_sku)
      mock_campaign_target_product_sku.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the campaign_target_product_skus list" do
      CampaignTargetProductSku.stub!(:find).and_return(mock_campaign_target_product_sku(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(campaign_target_product_skus_url)
    end
  end

  describe "GET names" do
    it "return name and id" do
      CampaignTargetProductSku.stub!(:find_with_name).with('searching string').
        and_return([mock_campaign_target_product_sku(:id => 1)])
      mock_campaign_target_product_sku.should_receive(:name_for_nested_ajax).and_return("Ivica Osim")
      get :names, :name => 'searching string'
    end
  end

end
