require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignsController do

  def login_as_admin
    login_as("admin")
  end

  before(:each) do
    login_as_admin
  end

  def mock_campaign(stubs={})
    @mock_campaign ||= mock_model(Campaign, stubs)
  end
  
  describe "GET index" do
    it "assigns all campaigns as @campaigns" do
      default_options = {
        :include => :effects,
        :per_page => 20,
        :page => 1,
        :order => "started_at desc, campaigns.id desc"
      }
      Campaign.stub!(:paginate).with(default_options).and_return([mock_campaign])
      get :index
      assigns[:campaigns].should == [mock_campaign]
    end
  end

  describe "GET show" do
    it "assigns the requested campaign as @campaign" do
      Campaign.stub!(:find).with("37").and_return(mock_campaign)
      get :show, :id => "37"
      assigns[:campaign].should equal(mock_campaign)
    end
  end

  describe "GET new" do
    it "assigns a new campaign as @campaign" do
      Campaign.stub!(:new).and_return(mock_campaign)
      get :new
      assigns[:campaign].should equal(mock_campaign)
    end
  end

  describe "GET edit" do
    it "assigns the requested campaign as @campaign" do
      Campaign.stub!(:find).with("37").and_return(mock_campaign)
      get :edit, :id => "37"
      assigns[:campaign].should equal(mock_campaign)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created campaign as @campaign" do
        Campaign.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign(:save => true))
        post :create, :campaign => {:these => 'params'}
        assigns[:campaign].should equal(mock_campaign)
      end

      it "redirects to the created campaign" do
        Campaign.stub!(:new).and_return(mock_campaign(:save => true))
        post :create, :campaign => {}
        response.should redirect_to(campaign_url(mock_campaign))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved campaign as @campaign" do
        Campaign.stub!(:new).with({'these' => 'params'}).and_return(mock_campaign(:save => false))
        post :create, :campaign => {:these => 'params'}
        assigns[:campaign].should equal(mock_campaign)
      end

      it "re-renders the 'new' template" do
        Campaign.stub!(:new).and_return(mock_campaign(:save => false))
        post :create, :campaign => {}
        response.should render_template('new')
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested campaign" do
        Campaign.should_receive(:find).with("37").and_return(mock_campaign(:mark_targets_for_destruction => true))
        mock_campaign.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign => {:these => 'params'}
      end

      it "assigns the requested campaign as @campaign" do
        Campaign.stub!(:find).and_return(mock_campaign(:update_attributes => true, :mark_targets_for_destruction => true))
        mock_campaign.should_receive(:clear_aggregation_cache).once
        mock_campaign.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        assigns[:campaign].should equal(mock_campaign)
      end

      it "redirects to the campaign" do
        Campaign.stub!(:find).and_return(mock_campaign(:update_attributes => true, :mark_targets_for_destruction => true))
        mock_campaign.should_receive(:clear_aggregation_cache).once
        mock_campaign.should_receive(:clear_association_cache).once
        put :update, :id => "1"
        response.should redirect_to(campaign_url(mock_campaign))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested campaign" do
        Campaign.should_receive(:find).with("37").and_return(mock_campaign(:mark_targets_for_destruction => true))
        mock_campaign.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :campaign => {:these => 'params'}
      end

      it "assigns the campaign as @campaign" do
        Campaign.stub!(:find).and_return(mock_campaign(:update_attributes => false, :mark_targets_for_destruction => true))
        put :update, :id => "1"
        assigns[:campaign].should equal(mock_campaign)
      end

      it "re-renders the 'edit' template" do
        Campaign.stub!(:find).and_return(mock_campaign(:update_attributes => false, :mark_targets_for_destruction => true))
        put :update, :id => "1"
        response.should render_template('edit')
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested campaign" do
      Campaign.should_receive(:find).with("37").and_return(mock_campaign)
      mock_campaign.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the campaigns list" do
      Campaign.stub!(:find).and_return(mock_campaign(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(campaigns_url)
    end
  end

  describe "GET names" do
    it "return name and id" do
      Campaign.stub!(:find_with_name).with('searching string').
        and_return([mock_campaign(:id => 1)])
      mock_campaign.should_receive(:name_for_nested_ajax).and_return("Ivica Osim")
      get :names, :name => 'searching string'
    end
  end

end
