# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe CmsPagesController do
  fixtures :cms_pages, :cms_binds, :cms_bind_items, :cms_bind_contents, :cms_templates, :cms_page_titles, :cms_projects,
    :cms_template_contents, :cms_files, :cms_default_templates

  before do
    login_as("admin")
    session[:project_id] = cms_projects(:cms_projects_00001).id
  end

  describe "GET index" do
    it "should get index" do
      get :index
      response.should be_success
      assigns(:cms_pages).should_not be_nil
    end

    it "should search" do
      get :index, :condition => {:front_url => 'fashion'}
      response.should be_success
    end

    it "should update session[:id]" do
      session[:project_id] = 1
      session[:project_id].should == 1
      get :index, :project_id => 2
      response.should be_success
      session[:project_id].should == "2"
     end
   end

   describe "GET show" do
     it "should get show" do
       get :show, :id => "4"
       response.should be_success
     end
   end

   describe "GET setup" do
     it "session[:cms_pages_new]がnil" do
       session[:cms_pages_new] = nil
       get :setup
       response.should be_success
       assigns(:page_options).should_not be_nil
     end

     it "session[:cms_pages_new]が存在する" do
       cms_page = CmsPage.new(:cms_project_id => "1", :action => 'new_action')
       I18n.supported_locales.each do |locale|
         cms_page.titles << CmsPageTitle.new(:locale => locale.to_s, :name => "title#{locale}")
       end
       CmsBind.position_ids.each do |position|
         cms_page.binds << CmsBind.new(:position => position, :items => [])
       end
       session[:cms_pages_new] = cms_page
       session[:cms_pages_new].should_not be_nil

       get :setup
       response.should be_success
       assigns(:cms_page).titles.each do |title|
         title.name.should_not be_nil
       end
     end
   end

   describe "GET new" do
     it "PJ編集対象以外のページを編集（コピーして登録）する場合" do
       session[:project_id] = cms_projects(:cms_projects_00001).id
       get :new, :source_id => "4"
       response.should be_success
       assigns(:cms_page).should_not be_nil
       assigns(:cms_page).binds.should_not be_nil
       assigns(:cms_page).titles.should_not be_nil
       session[:cms_pages_new].should_not be_nil
     end

     it "コピーしない場合" do
       get(:new, :cms_page => {:terminal => '',
           :titles_attributes => {:zh_TW=>{:name=>"",:locale=>"zh_TW"},
             :ja=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
             :ko=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
             :zh_CN=>{:name=>"",:locale=>"zh_CN"},
             :th=>{:name=>"",:locale=>"th"},
             :en=>{:name=>"",:locale=>"en"}}})
       response.should be_success
       assigns(:cms_page).should_not be_nil
       assigns(:cms_page).binds.should_not be_nil
       assigns(:cms_page).titles.should_not be_nil
       session[:cms_pages_new].should_not be_nil
     end

     it "コピーする場合" do
       get(:new, :cms_page => {:terminal => '',
           :org_id => '4',
           :titles_attributes => {:zh_TW=>{:name=>"",:locale=>"zh_TW"},
             :ja=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
             :ko=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
             :zh_CN=>{:name=>"",:locale=>"zh_CN"},
             :th=>{:name=>"",:locale=>"th"},
             :en=>{:name=>"",:locale=>"en"}}})
       response.should be_success
       assigns(:cms_page).should_not be_nil
       assigns(:cms_page).binds.should_not be_nil
       assigns(:cms_page).titles.should_not be_nil
       session[:cms_pages_new].should_not be_nil
     end
   end

   describe "GET edit" do
     it "should get edit" do
       get :edit, :id => cms_pages(:cms_pages_001).id
       response.should be_success
     end
   end

   describe "POST create" do
     it "should create cms_page" do
       lambda {
         post(:create, :cms_page => {:action => 'new_action',
             :binds_attributes => {"0"=>{:position=>"0",
                 :items_attributes=>{"0"=>{:no=>"01", :code=>"shop_tel_num",:input_type=>"select"},
                   "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"}},
                 :cms_template_id=>"15"},
               "1"=>{:position=>"1",:cms_template_id=>"2"},
               "2"=>{:position=>"2",:cms_template_id=>"3"},
               "3"=>{:position=>"3",:cms_template_id=>""},
               "4"=>{:position=>"4",
                 :items_attributes=>{"0"=>{:no=>"01",:code=>"shop_tel_num",:input_type=>"tab",
                     :contents_attributes=>{"zh-CN"=>{:descriptions=>"再入荷しました！！cn",:locale=>"zh-CN"},
                       "ja"=>{:descriptions=>"再入荷しました！！ja",:locale=>"ja"},
                       "zh-TW"=>{:descriptions=>"再入荷しました！！tw", :locale=>"zh-TW"},
                       "ko"=>{:descriptions=>"再入荷しました！！ko",:locale=>"ko"},
                       "th"=>{:descriptions=>"再入荷しました！！th",:locale=>"th"},
                       "en"=>{:descriptions=>"再入荷しました！！en",:locale=>"en"}}},
                   "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"},
                   "2"=>{:no=>"03",:code=>"shop_tel_num",:input_type=>"tab",
                     :contents_attributes=>{"zh-CN"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-CN"},
                       "ja"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ja"},
                       "zh-TW"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-TW"},
                       "ko"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ko"},
                       "th"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"th"},
                       "en"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"en"}}}},
                 :cms_template_id=>"16"}},
             :titles_attributes => {"zh_TW"=>{:name=>"",:locale=>"zh_TW"},
               "ja"=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
               "ko"=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
               "zh_CN"=>{:name=>"",:locale=>"zh_CN"},
               "th"=>{:name=>"",:locale=>"th"},
               "en"=>{:name=>"",:locale=>"en"}}})
       }.should change(CmsPage, :count).by(1)
     end

     it "should render new" do
       post(:create, :cms_page => {:action => '',
           :binds_attributes => {"0"=>{:position=>"0",
               :items_attributes=>{"0"=>{:no=>"01", :code=>"shop_tel_num",:input_type=>"select"},
                 "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"}},
               :cms_template_id=>"15"},
             "1"=>{:position=>"1",:cms_template_id=>"2"},
             "2"=>{:position=>"2",:cms_template_id=>"3"},
             "3"=>{:position=>"3",:cms_template_id=>""},
             "4"=>{:position=>"4",
               :items_attributes=>{"0"=>{:no=>"01",:code=>"shop_tel_num",:input_type=>"tab",
                   :contents_attributes=>{"zh-CN"=>{:descriptions=>"再入荷しました！！cn",:locale=>"zh-CN"},
                     "ja"=>{:descriptions=>"再入荷しました！！ja",:locale=>"ja"},
                     "zh-TW"=>{:descriptions=>"再入荷しました！！tw", :locale=>"zh-TW"},
                     "ko"=>{:descriptions=>"再入荷しました！！ko",:locale=>"ko"},
                     "th"=>{:descriptions=>"再入荷しました！！th",:locale=>"th"},
                     "en"=>{:descriptions=>"再入荷しました！！en",:locale=>"en"}}},
                 "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"},
                 "2"=>{:no=>"03",:code=>"shop_tel_num",:input_type=>"tab",
                   :contents_attributes=>{"zh-CN"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-CN"},
                     "ja"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ja"},
                     "zh-TW"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-TW"},
                     "ko"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ko"},
                     "th"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"th"},
                     "en"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"en"}}}},
               :cms_template_id=>"16"}},
           :titles_attributes => {"zh_TW"=>{:name=>"",:locale=>"zh_TW"},
             "ja"=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
             "ko"=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
             "zh_CN"=>{:name=>"",:locale=>"zh_CN"},
             "th"=>{:name=>"",:locale=>"th"},
             "en"=>{:name=>"",:locale=>"en"}}})
       response.should render_template "new"
     end

     it "PJ編集対象以外のページを編集（コピーして登録）する場合" do
       lambda {
         post(:create, :cms_page => {:action => 'new_action',
             :binds_attributes => {"0"=>{:position=>"0",
                 :items_attributes=>{"0"=>{:no=>"01", :code=>"shop_tel_num",:input_type=>"select"},
                   "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"}},
                 :cms_template_id=>"15"},
               "1"=>{:position=>"1",:cms_template_id=>"2"},
               "2"=>{:position=>"2",:cms_template_id=>"3"},
               "3"=>{:position=>"3",:cms_template_id=>""},
               "4"=>{:position=>"4",
                 :items_attributes=>{"0"=>{:no=>"01",:code=>"shop_tel_num",:input_type=>"tab",
                     :contents_attributes=>{"zh-CN"=>{:descriptions=>"再入荷しました！！cn",:locale=>"zh-CN"},
                       "ja"=>{:descriptions=>"再入荷しました！！ja",:locale=>"ja"},
                       "zh-TW"=>{:descriptions=>"再入荷しました！！tw", :locale=>"zh-TW"},
                       "ko"=>{:descriptions=>"再入荷しました！！ko",:locale=>"ko"},
                       "th"=>{:descriptions=>"再入荷しました！！th",:locale=>"th"},
                       "en"=>{:descriptions=>"再入荷しました！！en",:locale=>"en"}}},
                   "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"},
                   "2"=>{:no=>"03",:code=>"shop_tel_num",:input_type=>"tab",
                     :contents_attributes=>{"zh-CN"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-CN"},
                       "ja"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ja"},
                       "zh-TW"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-TW"},
                       "ko"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ko"},
                       "th"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"th"},
                       "en"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"en"}}}},
                 :cms_template_id=>"16"}},
             :titles_attributes => {"zh_TW"=>{:name=>"",:locale=>"zh_TW"},
               "ja"=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
               "ko"=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
               "zh_CN"=>{:name=>"",:locale=>"zh_CN"},
               "th"=>{:name=>"",:locale=>"th"},
               "en"=>{:name=>"",:locale=>"en"}}})
       }.should change(CmsPage, :count).by(1)
     end
   end

  describe "GET preview" do
    it "should preview" do
      pending
      login_as_admin
      cms_page = cms_pages_doubles(:one)
      cms_page.header_id = 996332878
      cms_page.footer_id = 996332880
      CmsPagesDouble.stub!(:find).and_return(cms_page)
      get :preview, :id => cms_pages_doubles(:one).id
      response.should redirect_to(:controller => 'cms', :action => 'show', :id => cms_pages_doubles(:one).id)
    end
  end

  describe "PUT update" do
    it "should update cms_pages" do
      put(:update, :id => cms_pages(:cms_pages_001).id,
        :cms_page => {:action => 'new_action',
          :binds_attributes => {"0"=>{:position=>"0",
              :items_attributes=>{"0"=>{:no=>"01", :code=>"shop_tel_num",:input_type=>"select"},
                "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"}},
              :cms_template_id=>"15"},
            "1"=>{:position=>"1",:cms_template_id=>"2"},
            "2"=>{:position=>"2",:cms_template_id=>"3"},
            "3"=>{:position=>"3",:cms_template_id=>""},
            "4"=>{:position=>"4",
              :items_attributes=>{"0"=>{:no=>"01",:code=>"shop_tel_num",:input_type=>"tab",
                  :contents_attributes=>{"zh-CN"=>{:descriptions=>"再入荷しました！！cn",:locale=>"zh-CN"},
                    "ja"=>{:descriptions=>"再入荷しました！！ja",:locale=>"ja"},
                    "zh-TW"=>{:descriptions=>"再入荷しました！！tw", :locale=>"zh-TW"},
                    "ko"=>{:descriptions=>"再入荷しました！！ko",:locale=>"ko"},
                    "th"=>{:descriptions=>"再入荷しました！！th",:locale=>"th"},
                    "en"=>{:descriptions=>"再入荷しました！！en",:locale=>"en"}}},
                "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"},
                "2"=>{:no=>"03",:code=>"shop_tel_num",:input_type=>"tab",
                  :contents_attributes=>{"zh-CN"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-CN"},
                    "ja"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ja"},
                    "zh-TW"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-TW"},
                    "ko"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ko"},
                    "th"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"th"},
                    "en"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"en"}}}},
              :cms_template_id=>"16"}},
          :titles_attributes => {"zh_TW"=>{:name=>"",:locale=>"zh_TW"},
            "ja"=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
            "ko"=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
            "zh_CN"=>{:name=>"",:locale=>"zh_CN"},
            "th"=>{:name=>"",:locale=>"th"},
            "en"=>{:name=>"",:locale=>"en"}}})
      response.should redirect_to(:action => 'show', :id => cms_pages(:cms_pages_001).id)
    end

    it "should render edit" do
      put(:update, :id => cms_pages(:cms_pages_001).id,
        :cms_page => {:action => 'new_action',
          :binds_attributes => {"0"=>{:position=>"0",
              :items_attributes=>{"0"=>{:no=>"01", :code=>"",:input_type=>""},
                "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"}},
              :cms_template_id=>"15"},
            "1"=>{:position=>"1",:cms_template_id=>"2"},
            "2"=>{:position=>"2",:cms_template_id=>"3"},
            "3"=>{:position=>"3",:cms_template_id=>""},
            "4"=>{:position=>"4",
              :items_attributes=>{"0"=>{:no=>"01",:code=>"shop_tel_num",:input_type=>"tab",
                  :contents_attributes=>{"zh-CN"=>{:descriptions=>"再入荷しました！！cn",:locale=>"zh-CN"},
                    "ja"=>{:descriptions=>"再入荷しました！！ja",:locale=>"ja"},
                    "zh-TW"=>{:descriptions=>"再入荷しました！！tw", :locale=>"zh-TW"},
                    "ko"=>{:descriptions=>"再入荷しました！！ko",:locale=>"ko"},
                    "th"=>{:descriptions=>"再入荷しました！！th",:locale=>"th"},
                    "en"=>{:descriptions=>"再入荷しました！！en",:locale=>"en"}}},
                "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"},
                "2"=>{:no=>"03",:code=>"shop_tel_num",:input_type=>"tab",
                  :contents_attributes=>{"zh-CN"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-CN"},
                    "ja"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ja"},
                    "zh-TW"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh-TW"},
                    "ko"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ko"},
                    "th"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"th"},
                    "en"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"en"}}}},
              :cms_template_id=>"16"}},
          :titles_attributes => {"zh_TW"=>{:name=>"",:locale=>"zh_TW"},
            "ja"=>{:name=>"フィギュア商品詳細ja",:locale=>"ja"},
            "ko"=>{:name=>"フィギュア商品詳細ko",:locale=>"ko"},
            "zh_CN"=>{:name=>"",:locale=>"zh_CN"},
            "th"=>{:name=>"",:locale=>"th"},
            "en"=>{:name=>"",:locale=>"en"}}})
      response.should render_template "edit"
    end
  end

  describe "DELETE destroy" do
    it "should destroy cms_page" do
      lambda {
        delete :destroy, :id => cms_pages(:cms_pages_001).id
      }.should change(CmsPage, :count).by(-1)

      assert_redirected_to cms_pages_path
    end
  end

  describe "POST realtime_preview" do
    it "should success realtime_preview" do
      contents = 'aaaaaa<br/><b>bbbbbbbbb</b><ec:link href="/test_url" text="test"/>'
      post :realtime_preview, :user_content => contents
    end
  end
end
