require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CmsProjectsController do
  fixtures :cms_projects, :cms_project_assigns, :goldberg_users

  def login_as_admin
    login_as("admin")
  end

  describe "GET index" do
    it "should get index" do
      login_as_admin
      get :index
      response.should be_success
      assigns(:cms_projects).should_not be_nil
    end

    it "should search" do
      login_as_admin
      get(:index, :condition => {:project_name => 'project1'})
      response.should be_success
      assigns(:cms_projects).size.should == 1
    end
  end

  describe "GET show" do
    it "should get show" do
      login_as_admin
      get :show, :id => cms_projects(:cms_projects_00001).id
      response.should be_success
    end
  end

  describe "GET new" do
    it "should get new" do
      login_as_admin
      get :new
      assert_response :success
      response.should be_success
    end
  end

  describe "GET new_assign" do
    it "should get new_assign" do
      login_as_admin
      get :new_assign
    end
  end

  describe "GET edit" do
    it "should get edit" do
      login_as_admin
      get :edit, :id => cms_projects(:cms_projects_00001).id
      response.should be_success
    end
  end

  describe "POST create" do
    it "should create cms_project" do
      login_as_admin
      lambda {
        post :create, :cms_project => {
          :priority => 10, :acceptance_type => '0', :project_code => "NEW", :project_name => "projectnew",
          :started_at => DateTime.now, :finished_at => DateTime.now
          }
      }.should change(CmsProject, :count).by(1)
      CmsProject.find(:first, :conditions => ["project_name = ?", "projectnew"]).status_key.should == :approved
    end

    it "should create cms_project with assign" do
      login_as_admin
      lambda {
        post :create, :cms_project => {
          :priority => 10, :acceptance_type => '0', :project_code => "NEW", :project_name => "projectnew",
          :started_at => DateTime.now, :finished_at => DateTime.now,
          :assigns_attributes => {:"0" => {:role => '2' , :goldberg_user_id => "1"}}}
      }.should change(CmsProject, :count).by(1)
    end

    it "should render new" do
      login_as_admin
      post :create, :cms_project => {}
      response.should render_template "new"
    end
  end

  describe "PUT update" do
    it "should update cms_project with assign" do
      login_as_admin
      put :update, :id => cms_projects(:cms_projects_00001).id,
        :cms_project => {:project_name => "PROJECT1", :assigns_attributes => {:"0" => {:role => '2' , :goldberg_user_id => "1"}}}
      response.should redirect_to(:action => 'show')
    end

    it "should render edit" do
      login_as_admin
      put :update, :id => cms_projects(:cms_projects_00001).id, :cms_project => {:project_name => nil}
      response.should render_template "edit"
    end
  end

  describe "DELETE destroy" do
    it "should destroy cms_projects" do
      FileUtils.should_receive(:rm_rf).with(
        File.expand_path(File.join(EcRider.config.ec_admin_root, 'public', 'projects', cms_projects(:cms_projects_00001).id.to_s))
      )
      login_as_admin
      lambda {
        delete :destroy, :id => cms_projects(:cms_projects_00001).id
      }.should change(CmsProject, :count).by(-1)

      assert_redirected_to cms_projects_path
    end
  end

end
