# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe CountriesController do
  fixtures :countries
  
  before do
    login_as("admin")
  end

  it "should get index" do
    get :index
    response.should redirect_to(:action => 'list')
  end
  
  describe "GET list" do
    
    before do
      flash[:error] = nil
    end
    
    it "valid parameters and no session" do
      session[:countries_search_cond] = nil
      get :list, :page => "1", :disp_count => "20", :condition => {  :name => "apan" }
      session[:countries_search_cond].name.should == "apan"
      controller.instance_variable_get(:@countries).should_not == nil
      flash[:error].should == nil
      response.should be_success
    end
    
    it "no parameters and valid session" do
      session[:countries_search_cond] = Country::SearchCondition.new({ :name_top => "J", :name => "apan" })
      get :list, :page => "1", :disp_count => "20"
      session[:countries_search_cond].name_top.should == "J"
      session[:countries_search_cond].name.should == "apan"
      controller.instance_variable_get(:@countries).should_not == nil
      flash[:error].should == nil
      response.should be_success
    end
    
    it "invalid parametersn" do
      session[:countries_search_cond] = nil
      get :list, :page => "1", :disp_count => "20", :condition => {  :name_top => "1" }
      flash[:error].should_not == nil
      response.should be_success
    end
  end

  it "should show country" do
    country = Country.find(:first)
    get :show, :id => country.id
    response.should render_template('show')
    controller.instance_variable_get(:@country).id.should == country.id
  end

  it "should get edit" do
    country = Country.find(:first)
    get :edit, :id => country.id
    response.should render_template('edit')
    controller.instance_variable_get(:@country).id.should == country.id
  end

  it "should update country" do
    country = Country.find(:first)
    put :update, :id => country.id, :country => {:shipping_cost => '750'}
    response.should redirect_to(:action => 'index')
    Country.find(country.id).shipping_cost.should == 750
  end
  
  it "should update country invalid" do
    country = Country.find(:first)
    put :update, :id => country.id, :country => {:ship_restriction_div => '1111111111'}
    pending # 500が返される
    #    response.should render_template('edit')
  end
end
