require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CsvFileDownloadController do
  fixtures :csv_download_patterns, :csv_download_pattern_items, :csv_download_pattern_user_conditions, :csv_download_pattern_accounting_conditions
      
  before(:each) do
    login_as("admin")
  end
  
  describe "GET index" do
    it "種類未選択" do
      get :index
      response.should be_success
      assigns[:csv_download_pattern].should_not be_nil
    end
    
    it "会員情報（住所単位）" do
      get :index, :download_type => CsvDownloadPattern.download_type_id_by_key(:user_address)
      response.should be_success
      assigns[:csv_download_pattern].should_not be_nil
      assigns[:condition].is_a?(User::SearchConditionForm)
    end
    
    it "商品情報（SKU単位）" do
      get :index, :download_type => CsvDownloadPattern.download_type_id_by_key(:product_sku)
      response.should be_success
      assigns[:csv_download_pattern].should_not be_nil
      assigns[:condition].is_a?(Product::SearchCondition)
    end
    
    it "注文情報（注文単位）" do
      get :index, :download_type => CsvDownloadPattern.download_type_id_by_key(:order)
      response.should be_success
      assigns[:csv_download_pattern].should_not be_nil
      assigns[:condition].is_a?(Order::SearchInfo)
    end

    it "経理情報" do
      get :index, :download_type => CsvDownloadPattern.download_type_id_by_key(:accounting)
      response.should be_success
      assigns[:csv_download_pattern].should_not be_nil
      assigns[:condition].is_a?(Accounting::SearchInfo)
    end
  end

  describe "GET edit" do
    it "編集" do
      get :edit, :id => "1"
      response.should be_success
      pattern = assigns[:csv_download_pattern]
      pattern.name.should == "全会員"
      pattern.items_attributes.length.should_not be_nil
      assigns[:condition].is_a?(User::SearchConditionForm)
    end
  end

  describe "POST create" do
    it "登録" do
      lambda {
        put(:create, 
          :csv_download_pattern => {
            :name => '  非会員',
            :character_code => '1', 
            :download_type => '1', 
            :file_name => 'no_member',
            :items_attributes => {
              :fixed_value_list => '"会員種別:正会員""追加項目１:2009"', 
               :item_list => '101:会員コード,102:氏名漢字（姓）,103:氏名フリガナ（姓）,104:氏名漢字（名）,105:氏名フリガナ（名）,' << '
                106:電話番号,107:国,108:郵便番号,109:都道府県（住所１）,110:市区町村（住所２）,111:番地・マンション名（住所３）,121:会員種別,000:追加項目１,'}},
          :condition => {
            :address => nil,
            :email => nil,
            :member_div => '1',
            :name => nil,
            :regist_date_from => nil,
            :regist_date_to => nil,
            :sex => nil,
            :tel_num => nil,
            :test_flg => nil,
            :user_code => nil})
      }.should change(CsvDownloadPattern, :count).by(1)
    end
    
    it "バリデーションエラー" do
      lambda {
        put(:create, 
          :csv_download_pattern => {
            :name => '  非会員',
            :character_code => '1', 
            :download_type => '1', 
            :file_name => 'all_member',
            :items_attributes => {
              :fixed_value_list => '"会員種別:正会員""追加項目１:2009"', 
               :item_list => '101:会員コード,102:氏名漢字（姓）,103:氏名フリガナ（姓）,104:氏名漢字（名）,105:氏名フリガナ（名）,' << '
                106:電話番号,107:国,108:郵便番号,109:都道府県（住所１）,110:市区町村（住所２）,111:番地・マンション名（住所３）,121:会員種別,000:追加項目１,'}},
          :condition => {
            :address => nil,
            :email => nil,
            :member_div => '1',
            :name => nil,
            :regist_date_from => nil,
            :regist_date_to => nil,
            :sex => nil,
            :tel_num => nil,
            :test_flg => nil,
            :user_code => nil})
      }.should change(CsvDownloadPattern, :count).by(0)
    end
  end

  describe "PUT update" do
    it "更新" do
      lambda {
        put(:update, :id => csv_download_patterns(:csv_download_patterns_00001).id,
          :csv_download_pattern => {
            :name => '  全会員',
            :character_code => '1', 
            :download_type => '1', 
            :file_name => 'all_user',
            :id => '1',
            :items_attributes => {
              :fixed_value_list => '', 
               :item_list => '101:会員コード,102:氏名漢字（姓）,103:氏名フリガナ（姓）,104:氏名漢字（名）,105:氏名フリガナ（名）,'}},
          :condition => {
            :address => nil,
            :email => nil,
            :member_div => '1',
            :name => nil,
            :regist_date_from => nil,
            :regist_date_to => nil,
            :sex => nil,
            :tel_num => nil,
            :test_flg => nil,
            :user_code => nil})
      }.should change(CsvDownloadPatternItem, :count).by(-8)
    end
  
    it "バリデーションエラー" do
      lambda {
        put(:update, :id => csv_download_patterns(:csv_download_patterns_00001).id,
          :csv_download_pattern => {
            :name => '  全会員',
            :character_code => '1', 
            :download_type => '1', 
            :file_name => 'あいう',
            :id => '1',
            :items_attributes => {
              :fixed_value_list => '"会員種別:正会員""追加項目１:2009"', 
               :item_list => '101:会員コード,102:氏名漢字（姓）,103:氏名フリガナ（姓）,104:氏名漢字（名）,105:氏名フリガナ（名）,'}},
          :condition => {
            :address => nil,
            :email => nil,
            :member_div => '1',
            :name => nil,
            :regist_date_from => nil,
            :regist_date_to => nil,
            :sex => nil,
            :tel_num => nil,
            :test_flg => nil,
            :user_code => nil})
      }.should change(CsvDownloadPatternItem, :count).by(0)
    end
  end

  describe "DELETE destroy" do
    it "削除" do
      lambda {
        delete :destroy, :id => csv_download_patterns(:csv_download_patterns_00001).id
      }.should change(CsvDownloadPattern, :count).by(-1)
    end

    it "削除 ひな形CSV(削除不可)を削除しようとした場合 削除されないこと" do
      csv_pattern = CsvDownloadPattern.find_by_id(csv_download_patterns(:csv_download_patterns_00001))
      csv_pattern.file_name = "user_bulk_upload_template"
      csv_pattern.save_with_validation(false)
      lambda {
        delete :destroy, :id => csv_download_patterns(:csv_download_patterns_00001).id
      }.should change(CsvDownloadPattern, :count).by(0)
    end
  end

  describe "GET download" do
    before(:each) do
      @tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_csv"))
      Dir.glob(File.join(@tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      FileUtils.mkdir_p(@tmp_dir)
      EcRider.config.stub!(:csv_dir).and_return(@tmp_dir)
    end
    
    it "ダウンロード" do
      lambda {
        get :download, :id => csv_download_patterns(:csv_download_patterns_00001).id
      }.should change(CsvFileDownloadHistory, :count).by(1)
      Dir.glob(File.join(@tmp_dir, '*.csv')).length.should == 1
    end

    it "ダウンロード" do
      lambda {
        get :download, :id => csv_download_patterns(:csv_download_patterns_00015).id
      }.should change(CsvFileDownloadHistory, :count).by(1)
      Dir.glob(File.join(@tmp_dir, '*.csv')).length.should == 1
      accountings = Accounting.find(:all)
      accountings.each do |accounting|
        accounting.output_data_flg.should be_true
      end
    end
  end

end
