require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CsvFileUploadController do
  before(:each) do
    login_as("admin")
  end

  def mock_csv_file_upload(stubs={})
    @mock_csv_file_upload ||= mock_model(CsvFileUpload, stubs)
  end

  def mock_user_csv_file_upload(stubs={})
    stubs = stubs.update(:uploader => CsvFileUpload::BaseCsvUploader.new({"upload_id" => "1"}))
    mock_model(CsvFileUpload, stubs)
  end

  def mock_csv_file_upload_history(stubs={})
    @mock_csv_file_upload ||= mock_model(CsvFileUploadHistory, stubs)
  end
  
  describe "GET index" do
    it "assigns all csv_file_upload_histories as @csv_file_upload_histories" do
      CsvFileUploadHistory.stub!(:paginate).and_return([mock_csv_file_upload_history])
      get :index
      assigns[:csv_file_upload_histories].should == [mock_csv_file_upload_history]
    end
  end
  
  #  describe "GET show" do
  #    it "assigns the requested csv_file_upload as @csv_file_upload" do
  #      CsvFileUpload.stub!(:find).with("37").and_return(mock_csv_file_upload)
  #      get :show, :id => "37"
  #      assigns[:csv_file_upload].should equal(mock_csv_file_upload)
  #    end
  #  end
  
  describe "GET new" do
    it "assigns a new csv_file_upload as @csv_file_upload" do
      CsvFileUpload.stub!(:new).and_return(mock_user_csv_file_upload)
      get :new
      assigns[:file_field_labels].should == [nil]
    end
  end
  
  #  describe "GET edit" do
  #    it "assigns the requested csv_file_upload as @csv_file_upload" do
  #      CsvFileUpload.stub!(:find).with("37").and_return(mock_csv_file_upload)
  #      get :edit, :id => "37"
  #      assigns[:csv_file_upload].should equal(mock_csv_file_upload)
  #    end
  #  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created csv_file_upload as @csv_file_upload" do
        CsvFileUpload.stub!(:new).with({'these' => 'params'}).and_return(mock_csv_file_upload(:save => true))
        post :create, :csv_file_upload => {:these => 'params'}
        assigns[:csv_file_upload].should equal(mock_csv_file_upload)
      end

      it "redirects to the created csv_file_upload" do
        CsvFileUpload.stub!(:new).and_return(
          mock_csv_file_upload(
            :uploader => mock_model(
              CsvFileUpload::BaseCsvUploader, {:validate? => true, :errors= => true, :upload => true}
            )
          )
        )
        post :create, :csv_file_upload => {}
        response.should redirect_to(:action => 'index')
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved csv_file_upload as @csv_file_upload" do
        CsvFileUpload.stub!(:new).with({'these' => 'params'}).and_return(mock_csv_file_upload(:save => false))
        post :create, :csv_file_upload => {:these => 'params'}
        assigns[:csv_file_upload].should equal(mock_csv_file_upload)
      end

      it "re-renders the 'index' template" do
        CsvFileUpload.stub!(:new).and_return(
          mock_csv_file_upload(
            :uploader => mock_model(
              CsvFileUpload::BaseCsvUploader, {:validate? => false, :errors= => true}
            )
          )
        )
        post :create, :csv_file_upload => {}
        response.should render_template('index')
      end
    end
    
  end

  #  describe "PUT update" do
  #
  #    describe "with valid params" do
  #      it "updates the requested csv_file_upload" do
  #        CsvFileUpload.should_receive(:find).with("37").and_return(mock_csv_file_upload)
  #        mock_csv_file_upload.should_receive(:update_attributes).with({'these' => 'params'})
  #        put :update, :id => "37", :csv_file_upload => {:these => 'params'}
  #      end
  #
  #      it "assigns the requested csv_file_upload as @csv_file_upload" do
  #        CsvFileUpload.stub!(:find).and_return(mock_csv_file_upload(:update_attributes => true))
  #        put :update, :id => "1"
  #        assigns[:csv_file_upload].should equal(mock_csv_file_upload)
  #      end
  #
  #      it "redirects to the csv_file_upload" do
  #        CsvFileUpload.stub!(:find).and_return(mock_csv_file_upload(:update_attributes => true))
  #        put :update, :id => "1"
  #        response.should redirect_to(csv_file_upload_url(mock_csv_file_upload))
  #      end
  #    end
  #
  #    describe "with invalid params" do
  #      it "updates the requested csv_file_upload" do
  #        CsvFileUpload.should_receive(:find).with("37").and_return(mock_csv_file_upload)
  #        mock_csv_file_upload.should_receive(:update_attributes).with({'these' => 'params'})
  #        put :update, :id => "37", :csv_file_upload => {:these => 'params'}
  #      end
  #
  #      it "assigns the csv_file_upload as @csv_file_upload" do
  #        CsvFileUpload.stub!(:find).and_return(mock_csv_file_upload(:update_attributes => false))
  #        put :update, :id => "1"
  #        assigns[:csv_file_upload].should equal(mock_csv_file_upload)
  #      end
  #
  #      it "re-renders the 'edit' template" do
  #        CsvFileUpload.stub!(:find).and_return(mock_csv_file_upload(:update_attributes => false))
  #        put :update, :id => "1"
  #        response.should render_template('edit')
  #      end
  #    end
  #
  #  end
  #
  #  describe "DELETE destroy" do
  #    it "destroys the requested csv_file_upload" do
  #      CsvFileUpload.should_receive(:find).with("37").and_return(mock_csv_file_upload)
  #      mock_csv_file_upload.should_receive(:destroy)
  #      delete :destroy, :id => "37"
  #    end
  #
  #    it "redirects to the csv_file_upload_histories list" do
  #      CsvFileUpload.stub!(:find).and_return(mock_csv_file_upload(:destroy => true))
  #      delete :destroy, :id => "1"
  #      response.should redirect_to(:action => "index")
  #    end
  #  end

end
