require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CsvFileUploadHistoryController do
  before(:each) do
    login_as("admin")
  end

  def mock_csv_file_upload_history(stubs={})
    @mock_csv_file_upload_history ||= mock_model(CsvFileUploadHistory, stubs)
  end
  
  describe "GET index" do
    it "assigns all csv_file_upload_histories as @csv_file_upload_histories" do
      CsvFileUploadHistory.stub!(:find).with(:all).and_return([mock_csv_file_upload_history])
      get :index
      assigns[:csv_file_upload_histories].should == [mock_csv_file_upload_history]
    end
  end

  describe "GET show" do
    it "assigns the requested csv_file_upload_history as @csv_file_upload_history" do
      CsvFileUploadHistory.stub!(:find).with("37").and_return(mock_csv_file_upload_history)
      get :show, :id => "37"
      assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
    end
  end

  describe "GET new" do
    it "assigns a new csv_file_upload_history as @csv_file_upload_history" do
      CsvFileUploadHistory.stub!(:new).and_return(mock_csv_file_upload_history)
      get :new
      assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
    end
  end

  describe "GET edit" do
    it "assigns the requested csv_file_upload_history as @csv_file_upload_history" do
      CsvFileUploadHistory.stub!(:find).with("37").and_return(mock_csv_file_upload_history)
      get :edit, :id => "37"
      assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created csv_file_upload_history as @csv_file_upload_history" do
        CsvFileUploadHistory.stub!(:new).with({'these' => 'params'}).and_return(mock_csv_file_upload_history(:save => true))
        post :create, :csv_file_upload_history => {:these => 'params'}
        assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
      end

      it "redirects to the created csv_file_upload_history" do
        CsvFileUploadHistory.stub!(:new).and_return(mock_csv_file_upload_history(:save => true))
        post :create, :csv_file_upload_history => {}
        response.should redirect_to(csv_file_upload_history_url(mock_csv_file_upload_history))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved csv_file_upload_history as @csv_file_upload_history" do
        CsvFileUploadHistory.stub!(:new).with({'these' => 'params'}).and_return(mock_csv_file_upload_history(:save => false))
        post :create, :csv_file_upload_history => {:these => 'params'}
        assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
      end
    end
    
  end

  describe "PUT update" do

    describe "with valid params" do
      it "updates the requested csv_file_upload_history" do
        CsvFileUploadHistory.should_receive(:find).with("37").and_return(mock_csv_file_upload_history)
        mock_csv_file_upload_history.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :csv_file_upload_history => {:these => 'params'}
      end

      it "assigns the requested csv_file_upload_history as @csv_file_upload_history" do
        CsvFileUploadHistory.stub!(:find).and_return(mock_csv_file_upload_history(:update_attributes => true))
        put :update, :id => "1"
        assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
      end

      it "redirects to the csv_file_upload_history" do
        CsvFileUploadHistory.stub!(:find).and_return(mock_csv_file_upload_history(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(csv_file_upload_history_url(mock_csv_file_upload_history))
      end
    end

    describe "with invalid params" do
      it "updates the requested csv_file_upload_history" do
        CsvFileUploadHistory.should_receive(:find).with("37").and_return(mock_csv_file_upload_history)
        mock_csv_file_upload_history.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :csv_file_upload_history => {:these => 'params'}
      end

      it "assigns the csv_file_upload_history as @csv_file_upload_history" do
        CsvFileUploadHistory.stub!(:find).and_return(mock_csv_file_upload_history(:update_attributes => false))
        put :update, :id => "1"
        assigns[:csv_file_upload_history].should equal(mock_csv_file_upload_history)
      end
    end

  end

  describe "DELETE destroy" do
    it "destroys the requested csv_file_upload_history" do
      CsvFileUploadHistory.should_receive(:find).with("37").and_return(mock_csv_file_upload_history)
      mock_csv_file_upload_history.should_receive(:destroy)
      delete :destroy, :id => "37"
    end

    it "redirects to the csv_file_upload_histories list" do
      CsvFileUploadHistory.stub!(:find).and_return(mock_csv_file_upload_history(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(:action => "index")
    end
  end

end
