require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe DistributorController do
  fixtures :distributors, :distributor_delivery_times, :distributor_delivery_time_displays

  before(:each) do
    # adminでログイン
    login_as("admin")
  end

  describe "GET index" do
    it "index no session" do
      session[:distributor_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
      controller.instance_variable_get(:@distributors).size.should == 2
    end

    it "index session" do
      flash[:error] = nil
      get :index, :condition => {:distributor_name => '業者'}
      response.should be_success
      controller.instance_variable_get(:@distributors).size.should == 1
    end
  end
  describe "GET show" do
    it "success" do
      get :show ,:id => "7", :page => "1"
      controller.instance_variable_get(:@distributor).id.should == 7
      response.should be_success
    end
  end

  describe "GET new" do
    it "new 通常遷移" do
      get :new, :page => "1"
      response.should be_success
      controller.instance_variable_get(:@distributor).should_not == nil
    end
    it "new バリデーション不正遷移" do
      distributor = Distributor.new()
      distributor.deliverable_term_start = "error"
      flash[:distributor] = distributor
      get :new, :page => "1"
      controller.instance_variable_get(:@distributor).should == distributor
      response.should be_success
    end
  end

  describe "GET edit" do
    it "success" do
      I18n.stub!(:supported_locales).and_return(['ja', 'en', 'ko', 'th', 'zh_CN'])
      flash[:distributor] = nil
      get :edit, :id => "8", :page => 1
      response.should be_success
      controller.instance_variable_get(:@distributor).id.should == 8
    end
    it "invalid" do
      I18n.stub!(:supported_locales).and_return(['ja', 'en', 'ko', 'th', 'zh_CN'])
      distributor = Distributor.find(8)
      distributor.deliverable_term_start = "error"
      flash[:distributor] = distributor
      get :edit, :id => "8", :page => 1
      response.should be_success
      controller.instance_variable_get(:@distributor).id.should == 8
    end
  end

  describe "GET create" do
        displays = {
          '0' => {:name => '日本', :locale => 'ja'},
          '1' => {:name => '英語', :locale => 'en'},
          '2' => {:name => '韓国', :locale => 'ko'},
          '3' => {:name => 'タイ', :locale => 'th'},
          '4' => {:name => '中国', :locale => 'zh_CN'}
        }
        delebery_times = {
          '0' => {:code => 1, :display_order => 1, :delivery_time_displays_attributes => displays},
          '1' => {:code => 2, :display_order => 2, :delivery_time_displays_attributes => displays},
          '2' => {:code => 3, :display_order => 3, :delivery_time_displays_attributes => displays},
          '3' => {:code => 4, :display_order => 4, :delivery_time_displays_attributes => displays},
          '4' => {:code => 5, :display_order => 5, :delivery_time_displays_attributes => displays},
          '5' => {:code => 6, :display_order => 6, :delivery_time_displays_attributes => displays},
          '6' => {:code => 7, :display_order => 7, :delivery_time_displays_attributes => displays},
          '7' => {:code => 8, :display_order => 8, :delivery_time_displays_attributes => displays},
          '8' => {:code => 9, :display_order => 9, :delivery_time_displays_attributes => displays},
          '9' => {:code => 10, :display_order => 10, :delivery_time_displays_attributes => displays}
        }

    it "save success" do
      distributor = {
        :name => "aa",
        :url => "",
        :deliverable_term_start => 3,
        :deliverable_term_end => 5,
        :delivery_times_attributes => delebery_times
      }

      get :create, :distributor => distributor , :page => 1
      response.should redirect_to :action => 'index', :page => 1
    end

    it "valid error delebery_times" do
      distributor = {
        :name => "aa",
        :url => "",
        :deliverable_term_start => "aaa",
        :deliverable_term_end => 5,
        :delivery_times_attributes => delebery_times
      }
      get :create, :distributor => distributor , :page => 1
      response.should redirect_to :action => 'new', :page => 1
      flash[:distributor].should_not == nil
    end

    it "valid error display" do
      delebery_times['0'][:code] = 'てすと'
      distributor = {
        :name => "aa",
        :url => "",
        :deliverable_term_start => 2,
        :deliverable_term_end => 5,
        :delivery_times_attributes => delebery_times
      }
      get :create, :distributor => distributor, :page => 1
      response.should redirect_to :action => 'new', :page => 1
      flash[:distributor].should_not == nil
    end
  end

  describe "GET update" do
    distributor = {"name"=>"物流業者１", "url"=>"http://ec-one.com",
      "delivery_times_attributes" => {
        "6"=>{"code"=>"7", "id"=>"77", "display_order"=>"7",
          "delivery_time_displays_attributes" => {
            "0"=>{"name"=>"日本語7", "id"=>"457", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語7", "id"=>"458", "locale"=>"ko"},
            "2"=>{"name"=>"中国語7", "id"=>"459", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"460", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語7", "id"=>"461", "locale"=>"en"},
            "5"=>{"name"=>"タイ語7", "id"=>"462", "locale"=>"th"}}},
        "7"=>{"code"=>"8", "id"=>"78", "display_order"=>"8",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語8", "id"=>"463", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語8", "id"=>"464", "locale"=>"ko"},
            "2"=>{"name"=>"中国語8", "id"=>"465", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"466", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語8", "id"=>"467", "locale"=>"en"},
            "5"=>{"name"=>"タイ語8", "id"=>"468", "locale"=>"th"}}},
        "8"=>{"code"=>"9", "id"=>"79", "display_order"=>"9",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語9", "id"=>"469", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語9", "id"=>"470", "locale"=>"ko"},
            "2"=>{"name"=>"中国語9", "id"=>"471", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"472", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語9", "id"=>"473", "locale"=>"en"},
            "5"=>{"name"=>"タイ語9", "id"=>"474", "locale"=>"th"}}},
        "9"=>{"code"=>"10", "id"=>"80", "display_order"=>"10",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語10", "id"=>"475", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語10", "id"=>"476", "locale"=>"ko"},
            "2"=>{"name"=>"中国語10", "id"=>"477", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"478", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語10", "id"=>"479", "locale"=>"en"},
            "5"=>{"name"=>"タイ語10", "id"=>"480", "locale"=>"th"}}},
        "0"=>{"code"=>"1", "id"=>"71", "display_order"=>"1",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語1", "id"=>"421", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語1", "id"=>"422", "locale"=>"ko"},
            "2"=>{"name"=>"中国語1", "id"=>"423", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"424", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語1", "id"=>"425", "locale"=>"en"},
            "5"=>{"name"=>"タイ語1", "id"=>"426", "locale"=>"th"}}},
        "1"=>{"code"=>"2", "id"=>"72", "display_order"=>"2",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語2", "id"=>"427", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語2", "id"=>"428", "locale"=>"ko"},
            "2"=>{"name"=>"中国語2", "id"=>"429", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"430", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語2", "id"=>"431", "locale"=>"en"},
            "5"=>{"name"=>"タイ語2", "id"=>"432", "locale"=>"th"}}},
        "2"=>{"code"=>"3", "id"=>"73", "display_order"=>"3",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語3", "id"=>"433", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語3", "id"=>"434", "locale"=>"ko"},
            "2"=>{"name"=>"中国語3", "id"=>"435", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"436", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語3", "id"=>"437", "locale"=>"en"},
            "5"=>{"name"=>"タイ語3", "id"=>"438", "locale"=>"th"}}},
        "3"=>{"code"=>"4", "id"=>"74", "display_order"=>"4",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語4", "id"=>"439", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語4", "id"=>"440", "locale"=>"ko"},
            "2"=>{"name"=>"中国語4", "id"=>"441", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"442", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語4", "id"=>"443", "locale"=>"en"},
            "5"=>{"name"=>"タイ語4", "id"=>"444", "locale"=>"th"}}},
        "4"=>{"code"=>"5", "id"=>"75", "display_order"=>"5",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語5", "id"=>"445", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語5", "id"=>"446", "locale"=>"ko"},
            "2"=>{"name"=>"中国語5", "id"=>"447", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"448", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語5", "id"=>"449", "locale"=>"en"},
            "5"=>{"name"=>"タイ語5", "id"=>"450", "locale"=>"th"}}},
        "5"=>{"code"=>"6", "id"=>"76", "display_order"=>"6",
          "delivery_time_displays_attributes"=>{
            "0"=>{"name"=>"日本語6", "id"=>"451", "locale"=>"ja"},
            "1"=>{"name"=>"韓国語6", "id"=>"452", "locale"=>"ko"},
            "2"=>{"name"=>"中国語6", "id"=>"453", "locale"=>"zh_CN"},
            "3"=>{"name"=>"", "id"=>"454", "locale"=>"zh_TW"},
            "4"=>{"name"=>"英語6", "id"=>"455", "locale"=>"en"},
            "5"=>{"name"=>"タイ語6", "id"=>"456", "locale"=>"th"}}}},
      "deliverable_term_start"=>"10", "deliverable_term_end"=>"31"}
    it "save success" do
      get :update, :distributor => distributor , :page => 1, :id => 8
      response.should redirect_to :action => 'index', :page => 1
    end

    it "valid error delebery_times" do
      distributor["deliverable_term_start"] = "2aaa"
      get :update, :distributor => distributor , :page => 1, :id => 7
      response.should redirect_to :action => 'edit', :page => 1, :id => 7
      flash[:distributor].id.should == 7
      flash[:distributor].should_not == nil
    end

    it "valid error display" do
      distributor["delivery_times_attributes"]["6"]["code"] = "test"
      get :update, :distributor => distributor, :page => 1, :id => 8
      response.should redirect_to :action => 'edit', :page => 1, :id => 8
      flash[:distributor].should_not == nil
    end
  end
  describe "GET destroy" do
    it "success destroy" do
      get :destroy ,:id => "7", :page => "1"
      response.should redirect_to :action => 'index', :page => 1
    end
  end

end
