# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

# refs #737 goldberg/rolesコントローラのspecでは、内部で権限管理のための
# Goldberg::Role.findを行っているため、スタブ／モックを使用していると、予想しない
# 動作をしてしまいます。
# ですので、このスペックではスタブ／モックを使用しないでください。
describe Goldberg::RolesController do
  fixtures :goldberg_role_actions, :goldberg_roles, :goldberg_site_action_groups, :goldberg_site_actions, :goldberg_users
  
  before do
    login_as("admin")
  end

  def mock_role(stubs={})
    @mock_role ||= mock_model(Goldberg::Role, stubs)
  end
  
  describe "GET index" do
    it "assigns all roles as @roles" do
      get :index
      assigns[:roles].map(&:id).should == [1,3,2]
    end
  end

  describe "GET show" do
    it "assigns the requested role as @role" do
      get :show, :id => 1 # システム管理者
      assigns[:role].should == Goldberg::Role.find(1)
    end
  end

  describe "GET new" do
    it "assigns a new role as @role" do
      Goldberg::Role.stub!(:new).and_return(mock_role)
      get :new
      assigns[:role].should equal(mock_role)
    end
  end

  describe "GET edit" do
    it "assigns the requested role as @role" do
      get :edit, :id => "3" # 一般ユーザー
      assigns[:role].should == Goldberg::Role.find(3)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created role as @role" do
        Goldberg::Role.stub!(:new).with({'these' => 'params'}).and_return(mock_role(:save => true))
        post :create, :role => {:these => 'params'}
        assigns[:role].should equal(mock_role)
      end

      it "redirects to the created role" do
        Goldberg::Role.stub!(:new).and_return(mock_role(:save => true))
        post :create, :role => {}
        response.should redirect_to(:action => :index)
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved role as @role" do
        Goldberg::Role.stub!(:new).with({'these' => 'params'}).and_return(mock_role(:save => false))
        post :create, :role => {:these => 'params'}
        assigns[:role].should equal(mock_role)
      end

      it "re-renders the 'new' template" do
        Goldberg::Role.stub!(:new).and_return(mock_role(:save => false))
        post :create, :role => {}
        response.should render_template('new')
      end
    end
    
  end

  describe "PUT update" do
    before(:each) do
      action_paths = %w(goldberg/auth/login cms_real_files/destroy cms_projects/update cms_projects/destroy cms_projects/index)
      @action_ids = action_paths.inject({}) do |dest, path|
        action = Goldberg::SiteAction.find_by_path(path)
        puts "path '#{path}' not found" unless action
        action.should_not be_nil
        dest[path] = action.id
        dest
      end
    end
    
    describe "with valid params" do
      it "updates the requested role" do
        role_id = Goldberg::Role.find_by_name("一般ユーザー").id
        put :update, :id => role_id, :role => {:name => 'neo admin', 
          :action_ids => [@action_ids["goldberg/auth/login"], @action_ids["cms_real_files/destroy"], 
            @action_ids["cms_projects/update"], @action_ids["cms_projects/destroy"], @action_ids["cms_projects/index"]]}
        Goldberg::Role.find(role_id).role_actions.map(&:site_action_id).sort.should == [
          @action_ids["cms_projects/index"], @action_ids["cms_projects/destroy"], @action_ids["cms_projects/update"],
          @action_ids["cms_real_files/destroy"], @action_ids["goldberg/auth/login"]].sort
        assigns[:role].id.should == role_id
        response.should redirect_to(:action => :index)
      end
    end
    
    describe "with invalid params" do
      it "updates the requested role" do
        Goldberg::Role.find(3).name.should == "一般ユーザー"
        put :update, :id => "3", :role => {:name => ''}
        Goldberg::Role.find(3).name.should == "一般ユーザー"
      end

      it "re-renders the 'edit' template" do
        role_id = Goldberg::Role.find_by_name("一般ユーザー").id
        put :update, :id => role_id, :role => {:name => '', 
          :action_ids => [@action_ids["goldberg/auth/login"], @action_ids["cms_real_files/destroy"], 
            @action_ids["cms_projects/update"], @action_ids["cms_projects/destroy"], @action_ids["cms_projects/index"]]}
        assigns[:role].id.should == role_id
        response.should be_success
        response.should render_template('edit')
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested role" do
      role_id = Goldberg::Role.find_by_name("一般ユーザー").id
      delete :destroy, :id => role_id
      Goldberg::Role.find_by_id(role_id).should == nil
      response.should redirect_to(roles_url)
    end
  end



end
