# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Goldberg::UserEditController do

  before do
    login_as("admin")

    # システム管理者を１人作成する
    Goldberg::User.delete_all
    Goldberg::User.create!(
      :login => 'admin',
      :role_id => '1',
      :name => "admin",
      :email => 'admin@ec-one.com',
      :password => 'password',
      :password_confirmation => 'password'
    )
    # セッションに保持する
    session[:user] = Goldberg::User.find_by_login("admin").id
  end

  describe "Get 'edit'" do
    it "セッションからログインユーザのIDを取得し、ユーザオブジェクトがインスタンス変数にセットされる" do
      get(:edit)
      controller.instance_variable_get(:@goldberg_user).login.should == 'admin'
      controller.instance_variable_get(:@goldberg_user).name.should == 'admin'
      controller.instance_variable_get(:@goldberg_user).email.should == 'admin@ec-one.com'
    end
  end

  describe "Post 'update'" do
    it "システム管理者の登録情報がを更新される" do
      before_user = Goldberg::User.first

      post(:update,
        :goldberg_user => {:name => "admin2", :email => "admin2@ec-one.com", :password => "password2", :password_confirmation => "password2"})
      response.should redirect_to(:controller => "goldberg/auth", :action=> 'login')

      after_user = Goldberg::User.first
      after_user.name.should == 'admin2'
      after_user.email.should == 'admin2@ec-one.com'
      after_user.crypted_password.should_not == before_user.crypted_password
    end

    it "パスワードが未入力の場合は、パスワードを更新しない" do
      before_user = Goldberg::User.first

      post(:update,
        :goldberg_user => {:name => "admin3", :email => "admin3@ec-one.com"})
      response.should redirect_to(:controller => "goldberg/auth", :action=> 'login')

      after_user = Goldberg::User.first
      after_user.name.should == 'admin3'
      after_user.email.should == 'admin3@ec-one.com'
      after_user.crypted_password.should == before_user.crypted_password
    end

    it "システム利用者名が未入力の場合、アカウントと同じ名前が登録される" do
      before_user = Goldberg::User.first

      post(:update,
        :goldberg_user => {:name => "", :email => "admin4@ec-one.com", :password => "", :password_confirmation => ""})

      after_user = Goldberg::User.first
      after_user.name.should == 'admin'
      after_user.email.should == 'admin4@ec-one.com'
      after_user.crypted_password.should == before_user.crypted_password
    end

    it "メールアドレスが未入力の場合、バリデーションエラーになる" do
      post(:update,
        :goldberg_user => {:name => "admin5", :email => "", :password => "", :password_confirmation => ""})
      response.should render_template('edit')
    end
  end
  
end
