# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Goldberg::UsersController do

  before do
    login_as("admin")
  end

  describe "Get 'index'" do
    it "should get index" do
      get :index
      session[:tmp_goldberg_user].should == nil
      session[:goldberg_user_condition].should == nil
      response.should be_success
      response.should render_template(:index)
    end
  end

  describe "Post 'index'" do
    it "The pattern that a parameter is set" do
      post(:index, :condition => {:login => "admin"}, :page => 1)
      session[:goldberg_user_condition].should_not nil
      controller.instance_variable_get(:@condition).should_not nil
      controller.instance_variable_get(:@goldberg_users).should_not nil
      response.should be_success
      response.should render_template(:index)
    end
    it "The pattern that a parameter is not set" do
      post(:index, :page => 1)
      session[:goldberg_user_condition].should == nil
      controller.instance_variable_get(:@condition).should_not == nil
      controller.instance_variable_get(:@goldberg_users).should_not == nil
      response.should be_success
      response.should render_template(:index)
    end
  end

  describe "Get 'show'" do
    it "show" do
      get(:show, :id => "1", :page => 1)
      controller.instance_variable_get(:@goldberg_user).should_not == nil
      controller.instance_variable_get(:@page_no).should == "1"
    end
  end

  describe "Get 'edit'" do
    it "When data are set by a flash" do
      flash[:goldberg_user] = Goldberg::User.find("1")
      get(:edit, :page => 1)
      controller.instance_variable_get(:@goldberg_user).id.should == 1
      controller.instance_variable_get(:@page_no).should == "1"
    end
    it "When data are not set by a flash" do
      get(:edit, :page => 1, :id => "1")
      controller.instance_variable_get(:@goldberg_user).id.should == 1
      controller.instance_variable_get(:@page_no).should == "1"
    end
  end

  describe "Post 'update'" do
    it "validation error" do
      session[:tmp_goldberg_user] = Goldberg::User.find("1")
      post(:update,
        :goldberg_user => {:name => "admin", :email => "ec-one.com", :password => "password", :password_confirmation => "password", :role_id => "1"})
      response.should redirect_to(:action => :edit)
    end
    it "should get update" do
      session[:tmp_goldberg_user] = Goldberg::User.find("1")
      post(:update,
        :goldberg_user => {:name => "admin", :email => "adminadmin@ec-one.com", :password => "password", :password_confirmation => "password", :role_id => "1"})
      session[:tmp_goldberg_user].should nil
      response.should redirect_to(:action => :index)
    end
  end

  describe "Get 'destroy'" do
    it "should get destroy" do
      get(:destroy, :id => "1")
      response.should redirect_to(:action => :index)
    end
  end

  describe "Post 'new'" do
    it "When there is already an error" do
      flash[:goldberg_user] = "error"
      post(:new)
      controller.instance_variable_get(:@goldberg_user).should == "error"
      response.should be_success
      response.should render_template(:new)
    end
    it "When there is not an error" do
      post(:new)
      controller.instance_variable_get(:@goldberg_user).id.should == nil
      response.should be_success
      response.should render_template(:new)
    end
  end

  describe "Post 'create'" do
    it "validation error" do
      session[:tmp_goldberg_user] = Goldberg::User.find("1")
      post(:create,
        :goldberg_user => {:name => "admin", :email => "ec-one.com", :password => "password", :password_confirmation => "password", :role_id => "1"})
      response.should redirect_to(:action => :new)
    end
    it "should get create" do
      session[:tmp_goldberg_user] = Goldberg::User.find("1")
      post(:create,
        :goldberg_user => {:login => "adminadmin", :email => "adminadmin@ec-one.com", :password => "password", :password_confirmation => "password", :role_id => "1"})
      session[:tmp_goldberg_user].should == nil
      response.should redirect_to(:action => :index)
    end
  end

end
