# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe InquiryController do
  fixtures :inquiries, :inquiry_memos, :mail_templates, :inquiry_products, :products,
    :mail_deliveries

  before do
    # adminでログイン
    login_as("admin")
  end

  describe "GET index" do
    it "index no session" do
      session[:inquiry_search_cond] = nil
      flash[:error] = nil
      get :index, :id => "1"
      response.should be_success
      response.should render_template("index")
    end

    it "index with session" do
      now_date = Time.now.strftime('%Y/%m/%d')
      cond = { :date_from => now_date, :date_to => now_date }
      session[:inquiry_search_cond] = cond
      get :index, :id => "1"
      response.should be_success
      response.should render_template("index")
    end

    it "with valid condition" do
      now_date = Time.now.strftime('%Y/%m/%d')
      cond = { :date_from => now_date, :date_to => now_date}
      get :index, :condition => cond
      flash[:error].should == nil
      response.should be_success
      response.should render_template("index")
    end

    it "with invalid condition" do
      get :index, :condition => {:date_from => 'ERROR'}
      response.should be_success
      flash[:condition].should_not == nil
      response.should render_template("index")
    end
  end

  describe "GET detail" do
    it ":@mail_deliveries not null" do
      get :detail, :id => 2
      response.should be_success
      response.should render_template("detail")
      mail_dliverries = controller.instance_variable_get(:@mail_deliveries)
      mail_dliverries.size.should == 2
      mail_dliverries[0].id.should == 2
      mail_dliverries[1].id.should == 8
      controller.instance_variable_get(:@inquiry).id.should == 2
    end

    it ":@mail_deliveries null" do
      get :detail, :id => 1
      response.should be_success
      response.should render_template("detail")
      controller.instance_variable_get(:@mail_deliveries).size.should == 0
      controller.instance_variable_get(:@inquiry).id.should == 1
    end
  end

  describe "POST update" do
    it "with error" do
      mf = { :from => "m-user@ec-one.com" , :to => "m-user@ec-one.com" , :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 1 , :page => 1, :mail_form => mf, :inquiry => { :status_div => nil }
      response.should redirect_to(:action =>'edit', :id => 1, :page => 1)
      flash[:error].should_not == nil
      flash[:id].should == "1"
    end

    it "update normal" do
      mf = { :from => "m-user@ec-one.com" , :to => "m-user@ec-one.com" , :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 2 , :page => 1, :mail_form => mf, :inquiry => {
          :status_div => "1", :answer => '回答します。'
      }
      response.should redirect_to(:action =>'detail' , :id => 2, :page => 1)
      controller.instance_variable_get(:@inquiry).id.should == 2
    end

    it "update mail_send" do
      mf = { :from => "m-user@ec-one.com" , :to => "m-user@ec-one.com" , :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 2 , :page => 1, :send_mail => 'send_mail', :mail_form => mf, :inquiry => {
          :status_div => "1", :answer => '回答します。'
      }
      response.should redirect_to(:action =>'detail' , :id => 2, :page => 1)
      flash[:notice].should_not == nil
      controller.instance_variable_get(:@inquiry).id.should == 2
    end

    it "update mail_send, then locale is not ja" do
      mf = { :from => "m-user@ec-one.com" , :to => "m-user@ec-one.com, m-user@ec-one.com" , :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com", :locale => 'zh-CN' }
#      inquiry = Inquiry.find(1)
#      inquiry.name_last = 'Yamada'
#      inquiry.name_first = 'Taro'
#      inquiry.address_1 = 'beijing'
#      inquiry.address_2 = 'street'
#      inquiry.inquiry = 'content'
#      inquiry.locale = 'zh-CN'
#      inquiry.targets_modified = 1
      post :update, :id => 1 , :page => 1, :send_mail => 'send_mail', :mail_form => mf,
        :inquiry => {
          :name_last => 'Yamada', :name_first => 'Taro', :inquiry => 'content',
          :status_div => "1", :answer => '回答します。', :locale => 'zh-CN'
      }
      flash[:mail_form] = nil
      response.should redirect_to(:action =>'detail' , :id => 1, :page => 1)
      mf = controller.instance_variable_get(:@mail_form)
      mf.body.should_not == nil
      mf.locale.should == 'zh-CN'
    end

    it "update mail_send invalid mail_form" do
      mf = { :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 2 , :page => 1, :send_mail => 'send_mail', :mail_form => mf, :inquiry => {
          :status_div => "1", :answer => '回答します。'
      }
      response.should redirect_to :action => :edit, :id => 2, :page => 1
      flash[:mail_form].should_not == nil
    end

    it "関連商品が追加された場合" do
      add_inquiry_products = {
        '0' => {:product_id => '100048'},
        '1' => {:product_id => '2'}
      }

      mf = { :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 3 , :page => 1, :send_mail => 'send_mail', :mail_form => mf, :inquiry => {
          :status_div => "1", :answer => '回答します。', :inquiry_products_attributes => add_inquiry_products,
          :targets_modified => 1
      }

      response.should redirect_to :action => :edit, :id => 3, :page => 1
      inquiry = Inquiry.find('3' ,:include => 'inquiry_products', :order => 'inquiry_products.product_id')
      inquiry.inquiry_products[0].product_id.should == 2
      inquiry.inquiry_products[1].product_id.should == 100048
    end

    it "関連商品が変更なしの場合" do
      mf = { :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 3 , :page => 1, :send_mail => 'send_mail', :mail_form => mf, :inquiry => {
          :status_div => "1", :answer => '回答します。'
      }
      response.should redirect_to :action => :edit, :id => 3, :page => 1
      inquiry = Inquiry.find('3',:include => 'inquiry_products', :order => 'inquiry_products.product_id')
      inquiry.inquiry_products[0].product_id.should == 100048
    end

    it "関連商品が削除された場合" do
      mf = { :subject => "m-user@ec-one.com" , :body => "m-user@ec-one.com" }
      post :update, :id => 3 , :page => 1, :send_mail => 'send_mail', :mail_form => mf, :inquiry => {
          :status_div => "1", :answer => '回答します。',
          :targets_modified => 1
      }
      response.should redirect_to :action => :edit, :id => 3, :page => 1
      inquiry = Inquiry.find('3')
      inquiry.inquiry_products.size.should == 0
    end


  end

  describe "POST edit" do
    it "with flash[:mail_form]" do
      flash[:mail_form] = "dummy"
      post :edit, :id => 1
      response.should be_success
      controller.instance_variable_get(:@mail_form).should == "dummy"
    end

    it "success" do
      post :edit, :id => 2, :page => 1
      response.should be_success
      controller.instance_variable_get(:@inquiry).id.should == 2
      controller.instance_variable_get(:@mail_form).subject.should == "お問い合わせの件"
    end
  end

  describe "POST regist_inquiry_memo" do
    it "success" do
      post :regist_inquiry_memo, :id => 2, :page => 1, :inquiry_memo => { :memo => "memo" }, :req_page => "detail"
      response.should redirect_to(:action => "edit", :id => 2, :page => 1)
    end
    it "fail" do
      post :regist_inquiry_memo, :id => 2, :page => 1, :inquiry_memo => {  }, :req_page => "mail"
      response.should redirect_to(:action => "edit", :id => 2, :page => 1)
    end
  end

  describe "delete_memo" do
    it "ok" do
      post :delete_memo, :memo_id => 1, :id => 1, :page => 1
      response.should redirect_to :action => 'edit', :id => 1, :page => 1
    end

    it "no record raise ActiveRecord::RecordNotFound" do
      post :delete_memo, :memo_id => 999, :id => 1, :page => 1
      response.should redirect_to :action => 'edit', :id => 1, :page => 1
    end
  end

  describe "delete" do
    it "ok" do
      post :delete, :id => 1, :page => 1
      response.should redirect_to :action => 'index', :page => 1
    end

    it "テスト問合せを全て削除 テスト問合せがない場合" do
      lambda {
        delete :delete, :destroy_all_test => true, :page => '1'
      }.should change(Inquiry, :count).by(0)
      response.should redirect_to(:action => 'index', :page => '1')
    end

    it "テスト問合せを全て削除 テスト問合せがある場合" do
      Inquiry.update(inquiries(:inquiries_00001).id, :test_flg => true)
      Inquiry.update(inquiries(:inquiries_00002).id, :test_flg => true)
      lambda {
        delete :delete, :destroy_all_test => true, :page => '1'
      }.should change(Inquiry, :count).by(-2)
      response.should redirect_to(:action => 'index', :page => '1')
    end
  end

  describe "marge_mail_template" do
    it "ok" do
      inq = Inquiry.find(1)
      inq.answer = 'test answer'
      mf = { :from => "m-user@ec-one.com" , :to => "test to" , :subject => "test subject" , :body => "m-user@ec-one.com" }
      post :marge_mail_template, :id => 1, :page => 1, :inquiry => inq, :mail_form => mf
      controller.instance_variable_get(:@inquiry).id.should == 1
      controller.instance_variable_get(:@mail_form).to.should == "test to"
      controller.instance_variable_get(:@mail_form).subject.should == "test subject"
      body = "テスト テスト様\r\n\
\r\n\
お問い合わせいただいた件について回答いたします。\r\n\
\r\n\
[お問い合わせ内容]\r\n\
------------------------------------------\r\n\
問い合わせ\r\n\
------------------------------------------\r\n\
\r\n\
[回答]\r\n\
------------------------------------------\r\n\
test answer \r\n\
\r\n\
\r\n\
\r\n\
------------------------------------------"
      controller.instance_variable_get(:@mail_form).body.should == body
      response.should render_template 'inquiry/_edit_mail_form'
    end
  end
end
