# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe MailTemplatesController do
  fixtures :mail_templates, 
    :orders, :order_shippings, :order_items,
    :products, :product_displays, :product_skus, :product_variations, :product_variation_items 
  
  before do
    # adminでログイン
    login_as("admin")
  end
  
  describe "POST new" do
    
    before do
      session[:mail_template_mail_template] = MailTemplate.new(:content_type_div => MailTemplate.content_type_div_id_by_key(:text), :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks))
    end
    
    it "mail_template from flash" do
      flash[:mail_template] = MailTemplate.new(:content_type_div => MailTemplate.content_type_div_id_by_key(:text), :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks))
      post :new
      controller.instance_variable_get(:@mail_template).should_not be_nil
      response.should be_success
    end
    
    it "mail_template from param(session)" do
      flash[:mail_template] = nil
      post :new, :use_session => { :aaa => "AAA" }
      controller.instance_variable_get(:@mail_template).should_not be_nil
      response.should be_success
    end
    
    it "mail_template from param(session)" do
      flash[:mail_template] = nil
      post :new
      session[:mail_template_mail_template].should be_nil
      controller.instance_variable_get(:@mail_template).should_not be_nil
      response.should be_success
    end
    
  end
  
  describe "POST new_pc_template" do
    
    it "with flash" do
      flash[:mail_template] = MailTemplate.new(:content_type_div => MailTemplate.content_type_div_id_by_key(:text), :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks))
      post :new_pc_template, :mail_template => {    }
      response.should be_success
    end
    
    it "with flash" do
      flash[:mail_template] = nil
      post :new_pc_template, :mail_template => {    }
      response.should_not be_success
    end
    
    it "validation fail" do
      flash[:mail_template] = nil
      post :new_pc_template, :mail_template => {
        :name => "AAA",
      }
      controller.instance_variable_get(:@mail_template).should_not be_nil
      response.should redirect_to :action => :new
    end
    
    it "validation ok" do
      flash[:mail_template] = nil
      post :new_pc_template, :mail_template => {
        :name => "AAA",
        :pc_subject_template => "YUE",
        :pc_text => "jcneocneocneo",
        :mobile_subject_template => "YUE",
        :mobile_text => "jcneocneocneo",
        :content_type_div => MailTemplate.content_type_div_id_by_key(:text),
        :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks)
      }
      controller.instance_variable_get(:@mail_template).should_not be_nil
      session[:mail_template_mail_template].should_not be_nil
      response.should be_success
    end
  end
  
  describe "POST confirm_create" do
    
    it "validate ok" do
      mt = {
        :name => "AAA",
        :pc_subject_template => "YUE",
        :pc_text => "jcneocneocneo",
        :mobile_subject_template => "YUE",
        :mobile_text => "jcneocneocneo",
        :content_type_div => MailTemplate.content_type_div_id_by_key(:text),
        :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks)
      }
      session[:mail_template_mail_template] = MailTemplate.new(mt)
      post :confirm_create, :mail_template => mt
      response.should be_success
    end
    
    it "validate fail" do
      mt = {
        :pc_subject_template => "YUE",
        :pc_text => "jcneocneocneo",
        :mobile_subject_template => "YUE",
        :mobile_text => "jcneocneocneo",
        :content_type_div => MailTemplate.content_type_div_id_by_key(:text),
        :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks)
      }
      session[:mail_template_mail_template] = MailTemplate.new(mt)
      post :confirm_create, :mail_template => mt
      response.should redirect_to :action => :new_pc_template
    end
  end
  
  describe "POST create" do
    
    it "validate ok" do
      mt = {
        :name => "AAA",
        :pc_subject_template => "YUE",
        :pc_text => "jcneocneocneo",
        :mobile_subject_template => "YUE",
        :mobile_text => "jcneocneocneo",
        :content_type_div => MailTemplate.content_type_div_id_by_key(:text),
        :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks),
      }
      session[:mail_template_mail_template] = MailTemplate.new(mt)
      post :create, :mail_template => mt
      response.should redirect_to :action => :search
    end
    
    it "validate fail" do
      mt = {
        :pc_subject_template => "YUE",
        :pc_text => "jcneocneocneo",
        :mobile_subject_template => "YUE",
        :mobile_text => "jcneocneocneo",
        :content_type_div => MailTemplate.content_type_div_id_by_key(:text),
        :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks)
      }
      session[:mail_template_mail_template] = MailTemplate.new(mt)
      post :create, :mail_template => mt
      response.should redirect_to :action => :new
    end
  end
  
  describe "POST edit" do
    
    it "validation error" do
      mt = {
        :pc_subject_template => "YUE",
        :pc_text => "jcneocneocneo",
        :mobile_subject_template => "YUE",
        :mobile_text => "jcneocneocneo",
        :content_type_div => MailTemplate.content_type_div_id_by_key(:text),
        :mail_type_div => MailTemplate.mail_type_div_id_by_key(:thanks)
      }
      flash[:mail_template] = mt
      post :edit
      response.should be_success
      controller.instance_variable_get(:@mail_template).should == mt
    end
    
    it "load only" do
      flash[:mail_template] = nil
      post :edit, :id => 1
      response.should be_success
      controller.instance_variable_get(:@mail_template).should_not be_nil
    end
    
    it "load with parameter" do
      flash[:mail_template] = nil
      post :edit, :id => 1, :mail_template => { :name => "JNIDNOJDNO" }
      response.should be_success
      controller.instance_variable_get(:@mail_template).should_not be_nil
    end
  end
  
  describe "POST confirm_update" do
    
    it "validation fail" do
      flash[:mail_template] = nil
      post :confirm_update, :id => 1, :mail_template => { :content_type_div => "AA" }
      response.should redirect_to :action => :edit, :id => 1
      flash[:mail_template].should_not be_nil
    end
    
    it "ok" do
      flash[:mail_template] = nil
      post :confirm_update, :id => 1, :mail_template => { :name => "JNIDNOJDNO" }
      response.should be_success
    end
  end
  
  describe "POST update" do
    
    it "validation fail" do
      flash[:mail_template] = nil
      post :update, :id => 1, :mail_template => { :content_type_div => "AA" }
      response.should redirect_to :action => :edit, :id => 1
      flash[:mail_template].should_not be_nil
    end
    
    it "ok" do
      flash[:mail_template] = nil
      post :update, :id => 1, :mail_template => { :name => "JNIDNOJDNO" }
      response.should redirect_to :action => :search
    end
  end
  
  describe "POST delete" do
    
    it "delete" do
      post :delete, :id => 1
      response.should redirect_to :action => :search
    end
  end
  
  describe "GET preview" do
    
    it "preview." do
      pending "MailTemplateのresultメソッドがないのでエラーになります。のではずしてあります"
      get :preview, :id => 2
      response.should be_success
    end
  end
  
  describe "GET show" do
    
    it "show." do
      get :show, :id => 1
      response.should be_success
    end
  end
  
  describe "POST change_mail_type" do
    
    it "change_mail_type" do
      post :change_mail_type, :value => '1'
      response.should be_success
    end
  end
  
  describe "GET search" do
    
    it "" do
      session[:mail_template_search_cond] = nil
      get :search, :condition => {  }
      response.should be_success
    end
  end
  
end
