# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe MsShopSettingsController do
  fixtures :shop_settings

 # fixturesで作成されるkbn_codesメソッドがNameMasterオブジェクトを返してくれないので、
 # NameMasterを返すようにメソッドを上書きします。
 # クラス名とテーブル名の対応が取れるまでの一時的な措置です。
  def shop_settings(name)
    ids = {
      :shop_settings_00005 => 5
    }
    ShopSetting.find(ids[name])
  end


  before do
    login_as("admin")

    session[:tmp_shop_settings] = nil
  end

  describe "get index" do
    it "should get index without condition" do
      get :index
      response.should be_success
      controller.instance_variable_get(:@shop_settings).should be_nil
    end

    it "should get index with params[:condition]" do
      get :index, :condition => {}, :page => '1'
      response.should be_success
      controller.instance_variable_get(:@shop_settings).length.should == 17
    end
  end

  it "should get show shop_setting" do
    get :show, :id => shop_settings(:shop_settings_00005).id
    response.should be_success
    controller.instance_variable_get(:@shop_setting).should == shop_settings(:shop_settings_00005)
  end

  it "should get new" do
    get :new
    response.should be_success
    controller.instance_variable_get(:@shop_setting).setting_div_key.should == :shop_name
  end

  it "should post change_setting_div" do
    post :change_setting_div, :shop_setting => {:setting_div => ShopSetting.setting_div_id_by_key(:shop_address)}
    response.should be_success
    response.should render_template('ms_shop_settings/_form_value')
    controller.instance_variable_get(:@shop_setting).setting_div_key.should == :shop_address
  end

  it "should get edit" do
    get :edit, :id => shop_settings(:shop_settings_00005).id
    response.should be_success
    controller.instance_variable_get(:@shop_setting).should == shop_settings(:shop_settings_00005)
  end

  describe "post create" do
    it "should redirect to index shop_setting successful" do
      post :create, :shop_setting => {:setting_div => '0', :value => 'test'}
      response.should redirect_to(ms_shop_settings_path)
    end

    it "should redirect to new shop_setting without setting_div" do
      post :create
      response.should render_template(:new)
    end
  end

  describe "put update" do
    it "should redirect to index shop_setting successful" do
      put :update, :id => shop_settings(:shop_settings_00005).id, :shop_setting => {:value => 'testtest'}
      response.should redirect_to(ms_shop_settings_path)
    end

    it "should redirect to edit shop_setting without setting_div" do
      put :update, :id => shop_settings(:shop_settings_00005).id, :shop_setting => {:setting_div => ''}
      response.should render_template(:edit)
    end
  end

  it "should destroy shop_setting" do
    delete :destroy, :id => shop_settings(:shop_settings_00005).id
    response.should redirect_to(ms_shop_settings_path)
  end

  describe "init_index" do
    it "init_index " do
      get :init_index
      session[:tmp_shop_settings].should == nil
      response.should redirect_to(:action => "index")
    end
  end
end
