# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe OrderController do
  include EcRider::I18nSupport

  fixtures :accountings,
    :countries, :prefectures, :prefecture_displays, 
    :distributors, :distributor_delivery_times, :distributor_delivery_time_displays,
    :shipping_costs, :shipping_cost_boundaries, 
    :kbn_codes, :kbn_code_names, 
    :mail_templates, :mail_deliveries, 
    :orders, :order_shippings, :order_items, 
    :point_settings,
    :products, :product_skus, :product_sku_inventories, :product_sku_secure_inventories,
    :product_displays, :product_components, :product_prices,
    :product_categories, 
    :product_groups, :product_group_skus,
    :shop_settings, 
    :users, :user_addresses, :user_emails, 
    :user_ranks, :user_rank_conditions, :user_rank_displays,
    :user_points,
    :campaigns, :campaign_effects, :campaign_conditions, :campaign_target_product_skus

  before do
    # adminでログイン
    login_as("admin")
  end

  describe "get search_index" do
    before do
      session[:tmp_order] = "test"
      session[:tmp_order_shipping] = "test"
      session[:tmp_order_items] = "test"
      session[:order_condition] = "test"
    end

    it "should get search_index" do
      get :search_index
      response.should render_template(:order_search)
      order_search = Order::SearchInfo.new()
      order_search.delivery_to_native_flg = true
      order_search.delivery_to_foreign_flg = true
      controller.instance_variable_get(:@condition).attributes.should == order_search.attributes

      session[:tmp_order].should be_nil
      session[:tmp_order_shipping].should be_nil
      session[:tmp_order_items].should be_nil
      session[:order_condition].should be_nil
    end

    it "should get search_index with flash" do
      search_info = Order::SearchInfo.new(:order_date_start => '2009/01/32')
      search_info.valid?
      flash[:error] = 'test'
      flash[:condition_attr] = search_info.attributes

      get :search_index
      response.should render_template(:order_search)
      controller.instance_variable_get(:@condition).errors.length.should == 1
    end
  end

  describe "get order_search" do
    it "should get order_search" do
      get :order_search
      response.should redirect_to(:action => :search_index)
    end

    it "should get oder_search with parameter" do
      get :order_search, :condition => {:name_first => 'タムラ', :display_summary => '1'}
      response.should render_template(:order_search)
      controller.instance_variable_get(:@condition).attributes.should == {"name_first" => 'タムラ', "order_counts_by_tab"=>{:all=>2, :undelivery=>2}, "display_summary" => '1'}
      controller.instance_variable_get(:@orders).length.should == 1
      controller.instance_variable_get(:@orders).first.ec_order_code.should == '00000000007'
      controller.instance_variable_get(:@total_amount).should == 194040
      controller.instance_variable_get(:@total_shipping_cost).should == 0
    end

    it "should get order_search with session" do
      session[:order_condition] = {:delivery_kanji_name => 'タムラ', :display_summary => '1'}
      get :order_search
      response.should render_template(:order_search)
      controller.instance_variable_get(:@condition).attributes.should == {"delivery_kanji_name" => 'タムラ', "order_counts_by_tab"=>{:all=>1, :undelivery=>1}, "display_summary" => '1'}
      controller.instance_variable_get(:@orders).length.should == 1
      controller.instance_variable_get(:@orders).first.ec_order_code.should == '00000000007'
      controller.instance_variable_get(:@total_amount).should == 97020
      controller.instance_variable_get(:@total_shipping_cost).should == 0
    end

    it "should get order_chaoer with invalid" do
      get :order_search, :condition => {:order_date_start => '2008/01/32'}
      flash[:error].should_not be_nil
      flash[:condition_attr].should_not be_nil
    end
  end

  describe "get index_tab" do
    it "配送指示済のみ" do
      get :index_tab, :inclusion => 0, :status_tab => 1, :order_condition => {:disp_count => '20'}
      controller.instance_variable_get(:@orders).total_entries.should == 1
    end
    
    it "配送指示済以前" do
      get :index_tab, :inclusion => 1, :status_tab => 1, :order_condition => {:disp_count => '20'}
      controller.instance_variable_get(:@orders).total_entries.should == 11
    end
    
    it "配送指示済以後" do
      get :index_tab, :inclusion => 2, :status_tab => 1, :order_condition => {:disp_count => '20'}
      controller.instance_variable_get(:@orders).total_entries.should == 1
    end
  end

  describe "get order_detail" do
    fixtures(:all)

    it "should get order_detail with user" do
      get :order_detail, :id => '7', :page => '1'
      response.should render_template(:order_detail)
      controller.instance_variable_get(:@order).ec_order_code.should == '00000000007'
      controller.instance_variable_get(:@order_shippings).length.should == 2
      controller.instance_variable_get(:@order_items).length.should == 1
      controller.instance_variable_get(:@user).id.should == 1007
      controller.instance_variable_get(:@user_address).id.should == 1009
      controller.instance_variable_get(:@user_email).id.should == 1007
      controller.instance_variable_get(:@page_no).should == '1'
      controller.instance_variable_get(:@sent_mails).length.should == 3
    end

    it "should get order_detail with user exist payment_error_history" do
      get :order_detail, :id => '1', :page => '1'
      response.should render_template(:order_detail)
      controller.instance_variable_get(:@payment_error_history).id.should == 1
    end
  end

  describe "get print" do
    it "should display invoice in japanese" do
      get :print, :id => orders(:orders_00006).id
      response.should render_template(:print_ja)
      controller.instance_variable_get(:@order).id.should == orders(:orders_00006).id
    end

    it "should display invoice in english" do
      e1 = orders(:orders_00006)
      e1.locale = 'en'
      e1.save

      get :print, :id => orders(:orders_00006).id
      response.should render_template(:print_en)
      controller.instance_variable_get(:@order).id.should == orders(:orders_00006).id
    end
  end

  describe "get order_mail" do
    it "should get order_mail" do
      get :order_mail, :id => '7', :order_shipping_id => '10'
      response.should render_template(:order_mail)
      controller.instance_variable_get(:@order).id.should == 7
      controller.instance_variable_get(:@order_shipping).id.should == 10
      controller.instance_variable_get(:@user).id.should == 1007
      controller.instance_variable_get(:@user_address).id.should == 1009
      controller.instance_variable_get(:@mail_type_div).should == MailTemplate.mail_type_div_id_by_key(:order_reply)
      actual_mail_form = controller.instance_variable_get(:@mail_form)
      actual_mail_form.should_not == nil
      actual_mail_form.locale.should == 'ja'
    end

    it "should get order_mail, when locale is not ja" do
      ec_order_chapter = Order.find(7)
      ec_order_chapter.locale = 'zh_CN'
      ec_order_chapter.save!
      params[:order_shipping_id]
      get :order_mail, :id => '7', :order_shipping_id => '10'
      response.should render_template(:order_mail)
      controller.instance_variable_get(:@order).id.should == 7
      controller.instance_variable_get(:@order_shipping).id.should == 10
      controller.instance_variable_get(:@user).id.should == 1007
      controller.instance_variable_get(:@user_address).id.should == 1009
      controller.instance_variable_get(:@mail_type_div).should == MailTemplate.mail_type_div_id_by_key(:order_reply)
      actual_mail_form = controller.instance_variable_get(:@mail_form)
      actual_mail_form.should_not == nil
      actual_mail_form.locale.should == 'zh_CN'
    end

    it "テスト注文はテスト環境用店舗メールアドレス" do
      order = Order.find(7)
      order.test_flg = true
      order.save!
      params[:order_shipping_id]
      get :order_mail, :id => '7', :order_shipping_id => '10'
      response.should render_template(:order_mail)
      controller.instance_variable_get(:@order).id.should == 7
      actual_mail_form = controller.instance_variable_get(:@mail_form).from.should == "test_tenpo_address@ecrider.com"
    end
  end

  describe "post order_send_mail" do
    it "should get order_send_mail with order" do
      mail_form = {
        :from => 'test_from@test.com',
        :to => 'test_to@test.com',
        :subject => 'test_subject',
        :body => 'test_message',
        :locale => 'ja'}

      post :order_send_mail, :id => '7', :page => '1', :mail_type_div => MailTemplate.mail_type_div_id_by_key(:order_reply), :mail_form => mail_form, :order_shipping_id => 1
      response.should render_template('confirm_send_mail')
    end

    it "should get order_send_mail_with invalid" do
      mail_form = {
        "from" => 'test_from@test.com',
        "to" => 'test_to@test.com',
        "body" => '',
        "locale" => 'ja'}

      post :order_send_mail, :id => '7', :page => '1', :mail_type_div => MailTemplate.mail_type_div_id_by_key(:order_reply), :mail_form => mail_form
      flash[:mail_form].attributes.should == mail_form
      response.should redirect_to(:action => :order_mail, :id => '7', :page => '1', :mail_type_div => MailTemplate.mail_type_div_id_by_key(:order_reply))
    end
  end

  describe "get edit" do
    it "初期表示" do
      get :edit, :id => '10', :page => '1'
      response.should render_template(:edit)
      controller.instance_variable_get(:@order_shipping).id.should == 10
      controller.instance_variable_get(:@order_items).length.should == 1
      controller.instance_variable_get(:@order).id.should == 7
      controller.instance_variable_get(:@page_no).should == '1'
    end

    it "バリデーションエラー時" do
      order_shipping = OrderShipping.find(10, :include => [:order, :order_items])
      edited_order = {
        :qty => "5",
        :course_div => "1",
        :total_amount => "10000",
        :shipping_cost => "500"
      }
      tmp_order = order_shipping.order
      tmp_order_shipping = order_shipping
      tmp_order_shipping.arrival_date = '2007/12/20'
      tmp_order_items = order_shipping.order_items

      flash[:order_shipping] = tmp_order_shipping
      flash[:order] = tmp_order
      flash[:order_items] = tmp_order_items

      get :edit, :id => 0, :order => edited_order, :page => '1'
      response.should render_template(:edit)
      controller.instance_variable_get(:@order_shipping).should == tmp_order_shipping
      controller.instance_variable_get(:@order_items).should == tmp_order_items
      controller.instance_variable_get(:@order).should == tmp_order
      controller.instance_variable_get(:@page_no).should == '1'
    end
  end

  describe "post update" do
    before do
      @order_shipping = OrderShipping.find(7, :include => [:order, :order_items])
      session[:tmp_order] = @order_shipping.order
      session[:tmp_order_shipping] = @order_shipping
      session[:tmp_order_shipping].arrival_date = '2007/12/20'
    end

    it "注文商品が１つもない場合" do
      post :update, :page => '1', :order_shipping => {:id => 1, :delivery_content => ('a' * 20001)},
        :order_item_list_size => -1, :order =>{}
      flash[:error_info].should_not be_blank
      response.should redirect_to(:action => 'edit',:id => 1,  :page => '1')
      order_item_1 = OrderItem.find(8)
      order_item_2 = OrderItem.find(9)
      order_items = {"0" => order_item_1.attributes, "1" => order_item_2.attributes}
      order_items["0"][:delete_flg] = 1
      order_items["1"][:delete_flg] = 1
      post :update, :page => '1', :order_shipping => {:id => 1},
        :order_item_list_size => 1, :order_item => order_items, :order =>{}
      flash[:error_info].should_not be_blank
      response.should redirect_to(:action => 'edit',:id => 1,  :page => '1')
    end

    it "注文商品のバリデーションエラーの場合" do
      order_item_1 = OrderItem.find(8)
      order_item_2 = OrderItem.find(9)
      order_items = {"0" => order_item_1.attributes, "1" => order_item_2.attributes}
      order_items["0"][:subtotal_amount] = "test"
      order_items["1"][:qty] = 9999999999999999999999
      post :update, :page => '1', :order_shipping => {:id => 1},
        :order_item_list_size => 1, :order_item => order_items, :order =>{}
      response.should redirect_to(:action => 'edit',:id => 1,  :page => '1')
    end

    it "should not update order_shipping with invalid" do
      order_items ={"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      post :update, :page => '1', :order_shipping => {:id => 1,:name_last => "", :delivery_content => ('a' * 20001)},
        :order_item_list_size => 1, :order_item => order_items, :order =>{}
      response.should redirect_to(:action => 'edit',:id => 1,  :page => '1')
    end

    it "redirect_toで指定するIDは、order_shipping.order_idではなくorder_shipping.id" do
      order_items ={"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      post :update, :page => '1', :order_shipping => {:id => 8,:name_last => "", :delivery_content => ('a' * 20001)},
        :order_item_list_size => 1, :order_item => order_items, :order =>{}
      response.should redirect_to(:action => 'edit',:id => 8,  :page => '1')
    end

    it "should update order_shipping" do
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      order_shipping = OrderShipping.find(1)
      order_shipping.unsecure_stocks
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.save!
      order_shipping.secure_stocks(:before_pay)
      order_shipping.order_items.length.should == 2
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      post :update, :page => '1', :order_shipping => {:id => 1, :status_div => OrderShipping.status_div_id_by_key(:undelivery)},
        :order_item_list_size => 1, :order_item => order_items,
       :order => {:gift_flg => 1, :send_invoice_separately_flg => 0, :write_sender_flg => 1}
      response.should redirect_to(:action => 'order_detail', :id => order_shipping.order_id, :page => params[:page])

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      order = Order.find_by_id(order_shipping.order_id)
      order.gift_flg.should == true
      order.send_invoice_separately_flg.should == false
      order.write_sender_flg.should == true
    end

    it "注文商品に変更がない場合" do
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}

      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.save
      order_shipping.status_div_key.should == :undelivery
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      post :update, :page => '1', :order_shipping => {:id => 1, :arrival_date => '2009/01/01'},
        :order_item_list_size => 1, :order_item => order_items, :order => {}
      response.should redirect_to(:action => 'order_detail', :id => order_shipping.order_id, :page => '1')

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149
      
      order_shipping = OrderShipping.find(1)
      order_shipping.status_div_key.should == :undelivery
    end

    it "キャンペーン適用状態が変わった場合" do
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}

      order = Order.find(1)
      order.campaign_attached_at = '2010-01-12 10:00:00'
      order.save!
      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.save
      order_shipping.status_div_key.should == :undelivery

      OrderCampaign.count().should == 0
      OrderItemCampaign.count().should == 0

      post :update, :page => '1', :order_shipping => {:id => 1, :arrival_date => '2009/01/01'},
        :order_item_list_size => 1, :order_item => order_items, :order => {}
      response.should redirect_to(:action => 'order_detail', :id => order_shipping.order_id, :page => '1')

      order_shipping = OrderShipping.find(1)
      order_shipping.status_div_key.should == :undelivery

      OrderCampaign.count().should == 2
      OrderItemCampaign.count().should == 0
    end

    it "注文商品の個数を変更した場合" do
      order_item_1 = OrderItem.find(8)
      order_item_1.qty = 2
      order_item_2 = OrderItem.find(9)
      order_item_2.qty = 2

      order_items = {"0" => order_item_1.attributes, "1" => order_item_2.attributes}

      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.save
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      post :update, :page => '1', :order_shipping => {:id => 1},
        :order_item_list_size => 1, :order_item => order_items, :order => {}
      response.should redirect_to(:action => 'order_detail', :id => order_shipping.order_id, :page => '1')

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 22

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 148
    end

    it "注文商品が削除された場合" do
      order_item_1 = OrderItem.find(8)
      order_item_2 = OrderItem.find(9)
      order_item_2.qty = 2
      order_items = {"0" => order_item_1.attributes, "1" => order_item_2.attributes}
      order_items["0"][:delete_flg] = 1

      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.save
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      post :update, :page => '1', :order_shipping => {:id => 1},
        :order_item_list_size => 1, :order_item => order_items, :order => {}
      response.should redirect_to(:action => 'order_detail', :id => order_shipping.order_id, :page => '1')

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 24

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 148
    end

    it "注文商品を変更した場合" do
      order_items = {"0" => OrderItem.find(1).attributes, "1" => OrderItem.find(9).attributes}
      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.save!
      p1 = ProductSku.find(265)
      p1.stock_count.should == 20
      p1.orderable_count.should == 19

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      post :update, :page => '1', :order_shipping => {:id => 1, :status_div => OrderShipping.status_div_id_by_key(:undelivery)},
        :order_item_list_size => 1, :order_item => order_items,
       :order => {:gift_flg => 1, :send_invoice_separately_flg => 0, :write_sender_flg => 1}
      response.should redirect_to(:action => 'order_detail', :id => order_shipping.order_id, :page => params[:page])
      p1 = ProductSku.find(265)
      p1.stock_count.should == 20
      p1.orderable_count.should == 18

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 24

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      order = Order.find_by_id(order_shipping.order_id)
      order.gift_flg.should == true
      order.send_invoice_separately_flg.should == false
      order.write_sender_flg.should == true
    end

    it "注文商品商品の在庫がない場合" do
      order_item = OrderItem.find(8)
      order_item.qty = 30
      order_items = {"0" => order_item.attributes, "1" => OrderItem.find(9).attributes}

      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.status_div_key = :undelivery_unpaid
      order_shipping.save
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149
      
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => 8})
      secures[0].paid_flg = false
      secures[0].save!

      post :update, :page => '1', :order_shipping => {:id => 1, :status_div => OrderShipping.status_div_id_by_key(:undelivery)},
        :order_item_list_size => 1, :order_item => order_items, :order => {}
      response.should redirect_to(:action => 'edit',:id => 1,  :page => '1')
      flash[:error_info].should_not == nil

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149
      order_shipping.status_div_key.should == :undelivery_unpaid
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => 8})
      secures[0].paid_flg.should be_false
    end

  end

  describe "DELETE destroy" do
    it "destroy order" do
    end
  end

  describe "DELETE destroy" do
    it "destroy order" do
      lambda {
        delete :delete, :id => '7', :page => '2'
      }.should change(Order, :count).by(-1)
      response.should redirect_to(:action => 'order_search', :page => '2')
    end

    it "テスト注文を１件削除" do
      Order.update(orders(:orders_00007).id, :test_flg => true)
      lambda {
        delete :delete, :test_id => '7', :page => '1'
      }.should change(Order, :count).by(-1)
      response.should redirect_to(:action => 'order_search', :page => '1')
    end

    it "テスト注文を全て削除 テスト注文がない場合" do
      lambda {
        delete :delete, :destroy_all_test => true, :page => '1'
      }.should change(Order, :count).by(0)
      response.should redirect_to(:action => 'order_search', :page => '1')
    end

    it "テスト注文を全て削除 テスト注文がある場合" do
      Order.update(orders(:orders_00007).id, :test_flg => true)
      Order.update(orders(:orders_00005).id, :test_flg => true)
      lambda {
        delete :delete, :destroy_all_test => true, :page => '1'
      }.should change(Order, :count).by(-2)
      response.should redirect_to(:action => 'order_search', :page => '1')
    end
  end

  it "get mail_detail" do
    mail = MailDelivery.find(:first)
    get :mail_detail, :mail_id => mail.id
    response.should render_template(:mail_detail)
    controller.instance_variable_get(:@mail_delivery).should == mail
  end

  describe "show_order_item_list" do
    it "正常" do
      get :show_order_item_list, :product_sku_ids => [265, 266], :shipping_id => 1
      result = controller.instance_variable_get(:@order_items)
      result.size.should == 2
      result[0].product_sku.id.should == 265
      result[1].product_sku.id.should == 266
    end
  end

  describe "back_to_undelivery_unpaid" do
    it "本番注文　在庫仮引当に戻す" do
      shipping = OrderShipping.find(4)
      OrderShipping.find(4).status_div_key.should == :undelivery
      ProductSkuSecureInventory.find(:first, :conditions => ["order_item_id = ?", shipping.order_items[0].id]).paid_flg.should be_true
      
      post(:back_to_undelivery_unpaid, :id => "4")
      OrderShipping.find(4).status_div_key.should == :undelivery_unpaid
      ProductSkuSecureInventory.find(:first, :conditions => ["order_item_id = ?", shipping.order_items[0].id]).paid_flg.should be_false
    end
    
    it "テスト注文　在庫仮引当はない" do
      ProductSkuSecureInventory.destroy_all
      ProductSku.update_all(:secure_stock_count => 0)
      shipping = OrderShipping.find(4)
      shipping.order.test_flg = true
      shipping.order.save!
      OrderShipping.find(4).status_div_key.should == :undelivery
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
      
      post(:back_to_undelivery_unpaid, :id => "4")
      OrderShipping.find(4).status_div_key.should == :undelivery_unpaid
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
    end
  end

  describe "back_to_deliver_request" do
    it "本番注文　在庫と在庫引当を戻す" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :delivered
      shipping.save!
      before_sku_inventory_count = ProductSkuInventory.count
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
      post(:back_to_deliver_request, :id => "9")
      shipping = OrderShipping.find(9)
      shipping.status_div_key.should == :deliver_request
      ProductSkuInventory.count.should == before_sku_inventory_count + 3
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 1
    end

    it "テスト注文　在庫と在庫引当はない" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :delivered
      shipping.save!
      shipping.order.test_flg = true
      shipping.order.save!
      before_sku_inventory_count = ProductSkuInventory.count
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
      post(:back_to_deliver_request, :id => "9")
      shipping = OrderShipping.find(9)
      shipping.status_div_key.should == :deliver_request
      ProductSkuInventory.count.should == before_sku_inventory_count
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
    end
  end

  describe "cancel" do
    it "本番注文　在庫引当を取り消す" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :deliver_request
      shipping.save!
      shipping.secure_stocks
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 1
      post(:cancel, :id => "9")
      OrderShipping.find(9).status_div_key.should == :canceled
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "cancel complete mail subject pc!"
      mail.from.should == ["test_webadministrator@ecrider.com"]
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
    end

    it "テスト注文　在庫引当はない" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :deliver_request
      shipping.save!
      shipping.order.test_flg = true
      shipping.order.save!
      shipping.secure_stocks
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
      post(:cancel, :id => "9")
      OrderShipping.find(9).status_div_key.should == :canceled
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "cancel complete mail subject pc!"
      mail.from.should == ["test_tenpo_address@ecrider.com"]
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
    end
  end

  describe "returned" do
    it "配送済→返品" do
      order_shipping = OrderShipping.find(2)
      order_shipping.status_div_key = :delivered
      order_shipping.save!
      Accounting.create_for_order(order_shipping)
      post(:returned, :id => "2")
      OrderShipping.find(2).status_div_key.should == :returned
      accountings = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id], :order => 'id')
      accountings.length.should == 2
      accountings[0].order_id.should == order_shipping.order_id
      accountings[0].debt_sales.should == 15750
      accountings[0].debt_account_div_key.should == :order_receivable
      accountings[0].credit_sales.should == 15750
      accountings[0].credit_account_div_key.should == :order_receivable
      accountings[0].given_point.should == 1000
      accountings[0].used_point.should == 0
      accountings[0].previous_point.should == 1000
      accountings[1].order_id.should == order_shipping.order_id
      accountings[1].debt_sales.should == 15750
      accountings[1].debt_account_div_key.should == :cancel_receivable
      accountings[1].credit_sales.should == 15750
      accountings[1].credit_account_div_key.should == :cancel_receivable
      accountings[1].given_point.should == -1000
      accountings[1].used_point.should == 0
      accountings[1].previous_point.should == 0
      UserPoint.count(:conditions => ["order_id = ?", order_shipping.order_id]).should == 0
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
      mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
      NKF.nkf('-w', mail.subject).should == "返品処理完了のご連絡"
      NKF.nkf('-w', mail.body).should == "山田 太郎 様\n\n返品処理が完了しました。\n\n今後も宜しくお願い致します。\n"
    end

    it "配送案内済→返品" do
      order_shipping = OrderShipping.find(2)
      order_shipping.status_div_key = :deliver_advised
      order_shipping.save!
      Accounting.create_for_order(order_shipping)
      post(:returned, :id => "2")
      OrderShipping.find(2).status_div_key.should == :returned
      accountings = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id], :order => 'id')
      accountings.length.should == 2
      accountings[0].order_id.should == order_shipping.order_id
      accountings[0].debt_sales.should == 15750
      accountings[0].debt_account_div_key.should == :order_receivable
      accountings[0].credit_sales.should == 15750
      accountings[0].credit_account_div_key.should == :order_receivable
      accountings[0].given_point.should == 1000
      accountings[0].used_point.should == 0
      accountings[0].previous_point.should == 1000
      accountings[1].order_id.should == order_shipping.order_id
      accountings[1].debt_sales.should == 15750
      accountings[1].debt_account_div_key.should == :cancel_receivable
      accountings[1].credit_sales.should == 15750
      accountings[1].credit_account_div_key.should == :cancel_receivable
      accountings[1].given_point.should == -1000
      accountings[1].used_point.should == 0
      accountings[1].previous_point.should == 0
      UserPoint.count(:conditions => ["order_id = ?", order_shipping.order_id]).should == 0
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
      mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
      NKF.nkf('-w', mail.subject).should == "返品処理完了のご連絡"
      NKF.nkf('-w', mail.body).should == "山田 太郎 様\n\n返品処理が完了しました。\n\n今後も宜しくお願い致します。\n"
    end
  end
  
  describe "update_status" do
    it "入金待ち→配送準備中　本番注文" do
      shipping = OrderShipping.find(4)
      shipping.status_div_key = :undelivery_unpaid
      shipping.save!
      shipping.secure_stocks(:before_pay)
      ProductSkuSecureInventory.find(:first, :conditions => ["order_item_id = ?", shipping.order_items[0].id]).paid_flg.should be_false
      
      post(:update_status, :status_tab => "11", :selected_order_shipping_ids => ["4"])
      OrderShipping.find(4).status_div_key.should == :undelivery
      ProductSkuSecureInventory.find(:first, :conditions => ["order_item_id = ?", shipping.order_items[0].id]).paid_flg.should be_true
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "receipt payment pc!"
      mail.from.should == ["test_webadministrator@ecrider.com"]
    end
    
    it "入金待ち→配送準備中　テスト注文" do
      ProductSkuSecureInventory.destroy_all
      ProductSku.update_all(:secure_stock_count => 0)
      shipping = OrderShipping.find(4)
      shipping.status_div_key = :undelivery_unpaid
      shipping.save!
      shipping.order.test_flg = true
      shipping.order.save!
      shipping.secure_stocks(:before_pay)
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
      
      post(:update_status, :status_tab => "11", :selected_order_shipping_ids => ["4"])
      OrderShipping.find(4).status_div_key.should == :undelivery
      ProductSkuSecureInventory.count(:conditions => ["order_item_id = ?", shipping.order_items[0].id]).should == 0
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "receipt payment pc!"
      mail.from.should == ["test_tenpo_address@ecrider.com"]
    end
  
    it "配送準備中→配送指示済　本番注文" do
      OrderShipping.find(2).status_div_key.should == :undelivery
      OrderShipping.find(4).status_div_key.should == :undelivery
      post(:update_status, :status_tab => "0", :selected_order_shipping_ids => ["2", "4"])
      OrderShipping.find(2).status_div_key.should == :deliver_request
      OrderShipping.find(4).status_div_key.should == :deliver_request
      ActionMailer::Base.deliveries.size.should == 2
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "商品の出荷をお願いします"
      mail.to.should == ["test_delivery_service@ecrider.com"]
      mail.from.should == ["test_webadministrator@ecrider.com"]
      NKF.nkf('-w', ActionMailer::Base.deliveries[1].subject).should == "商品の出荷をお願いします"
    end
  
    it "配送準備中→配送指示済　テスト注文" do
      ProductSkuSecureInventory.destroy_all
      ProductSku.update_all(:secure_stock_count => 0)
      shipping = OrderShipping.find(2)
      shipping.status_div_key.should == :undelivery
      shipping.order.test_flg = true
      shipping.order.save!
      post(:update_status, :status_tab => "0", :selected_order_shipping_ids => ["2"])
      OrderShipping.find(2).status_div_key.should == :deliver_request
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "商品の出荷をお願いします"
      mail.to.should == ["test_tenpo_address@ecrider.com"]
      mail.from.should == ["test_tenpo_address@ecrider.com"]
    end
  
    it "配送指示済→配送済　本番注文" do
      OrderShipping.find(9).status_div_key.should == :deliver_request
      post(:update_status, :status_tab => "1", :selected_order_shipping_ids => ["9"])
      order_shipping = OrderShipping.find(9)
      order_shipping.status_div_key.should == :delivered
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "出荷完了しました"
      mail.to.should == ["test_shipment_complete_for_shop@ecrider.com"]
      mail.from.should == ["test_webadministrator@ecrider.com"]
      Accounting.count(:conditions => ["order_id = ?", order_shipping.order_id]).should == 1
    end
  
    it "配送指示済→配送済　テスト注文" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key.should == :deliver_request
      shipping.order.test_flg = true
      shipping.order.save!
      post(:update_status, :status_tab => "1", :selected_order_shipping_ids => ["9"])
      order_shipping = OrderShipping.find(9)
      order_shipping.status_div_key.should == :delivered
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "出荷完了しました"
      mail.to.should == ["test_tenpo_address@ecrider.com"]
      mail.from.should == ["test_tenpo_address@ecrider.com"]
      Accounting.count(:conditions => ["order_id = ?", order_shipping.order_id]).should == 0
    end
  
    it "配送済→配送案内済　本番注文" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :delivered
      shipping.save!
      post(:update_status, :status_tab => "2", :selected_order_shipping_ids => ["9"])
      OrderShipping.find(9).status_div_key.should == :deliver_advised
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "出荷完了しました"
      mail.from.should == ["test_webadministrator@ecrider.com"]
    end
  
    it "配送済→配送案内済　テスト注文" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :delivered
      shipping.save!
      shipping.order.test_flg = true
      shipping.order.save!
      post(:update_status, :status_tab => "2", :selected_order_shipping_ids => ["9"])
      OrderShipping.find(9).status_div_key.should == :deliver_advised
      ActionMailer::Base.deliveries.size.should == 1
      mail = ActionMailer::Base.deliveries[0]
      NKF.nkf('-w', mail.subject).should == "出荷完了しました"
      mail.from.should == ["test_tenpo_address@ecrider.com"]
    end

    it "キャンセル依頼中→キャンセル完了　引き当てあり" do
      shipping = OrderShipping.find(2)
      shipping.status_div_key = :cancel_request
      shipping.save!
      shipping = OrderShipping.find(4)
      shipping.status_div_key = :cancel_request
      shipping.save!
      post(:update_status, :status_tab => "10", :selected_order_shipping_ids => ["2", "4"])
      OrderShipping.find(2).status_div_key.should == :canceled
      OrderShipping.find(4).status_div_key.should == :canceled
      ActionMailer::Base.deliveries.size.should == 2
      NKF.nkf('-w', ActionMailer::Base.deliveries[0].subject).should == "cancel complete mail subject pc!"
      NKF.nkf('-w', ActionMailer::Base.deliveries[1].subject).should == "cancel complete mail subject pc!"
    end
  
    it "キャンセル依頼中→キャンセル完了　引き当てなし" do
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :cancel_request
      shipping.save!
      post(:update_status, :status_tab => "10", :selected_order_shipping_ids => ["9"])
      order_shipping = OrderShipping.find(9)
      order_shipping.status_div_key.should == :canceled
      ActionMailer::Base.deliveries.size.should == 1
      NKF.nkf('-w', ActionMailer::Base.deliveries[0].subject).should == "cancel complete mail subject pc!"
    end
  
    it "配送準備中→配送指示済　出荷指示メール送信先メールアドレス未設定" do
      ShopSetting.destroy(10)
      OrderShipping.find(2).status_div_key.should == :undelivery
      OrderShipping.find(4).status_div_key.should == :undelivery
      post(:update_status, :status_tab => "0", :selected_order_shipping_ids => ["2", "4"])
      OrderShipping.find(2).status_div_key.should == :undelivery
      OrderShipping.find(4).status_div_key.should == :undelivery
      ActionMailer::Base.deliveries.size.should == 0
      flash[:status_update_error].should == "店舗情報に　出荷指示メール送信先メールアドレス　が設定されていません。"
    end
  
    it "配送指示済→配送済　店舗メールアドレスと配送完了メール（店舗向け）送信先メールアドレス未設定" do
      ShopSetting.destroy(9)
      ShopSetting.destroy(21)
      OrderShipping.find(9).status_div_key.should == :deliver_request
      post(:update_status, :status_tab => "1", :selected_order_shipping_ids => ["9"])
      order_shipping = OrderShipping.find(9)
      order_shipping.status_div_key.should == :deliver_request
      ActionMailer::Base.deliveries.size.should == 0
      Accounting.count(:conditions => ["order_id = ?", order_shipping.order_id]).should == 0
      flash[:status_update_error].should == "店舗情報に　店舗メールアドレス,配送完了メール（店舗向け）送信先メールアドレス　が設定されていません。"
    end
  
    it "キャンセル依頼中→キャンセル完了　店舗メールアドレス未設定" do
      ShopSetting.destroy(9)
      shipping = OrderShipping.find(9)
      shipping.status_div_key = :cancel_request
      shipping.save!
      post(:update_status, :status_tab => "10", :selected_order_shipping_ids => ["9"])
      order_shipping = OrderShipping.find(9)
      order_shipping.status_div_key.should == :cancel_request
      ActionMailer::Base.deliveries.size.should == 0
      flash[:status_update_error].should == "店舗情報に　店舗メールアドレス　が設定されていません。"
    end
  end

  describe "post create" do
    before do
      @order_form = {:arrival_date=>"", :normal_payment_type=>"1", :write_sender_flg=>"0", :free_request=>"", :send_invoice_separately_flg=>"0",
       :campaign_attached_div=>"0", :gift_flg=>"0", :arrival_hour=>""}
      @order_shipping = {:kana_name_first=>"テスト", :name_last=>"タロウ", :country_code=>"JP", :zip_code=>"8700001", :used_point=>"",
       :discount_amount=>"0", :address_1=>"愛知県", :tel_num=>"0975-12-1234", :cod_charge=>"", :address_2=>"北区中町", :address_3=>"",
        :name_first=>"テスト", :kana_name_last=>"タロウ", :shipping_cost=>"500", :subtotal_amount=>"51450"}
      @no_member_user = User.find(1001).clone
      @no_member_user.member_div = User.member_div_id_by_key(:no_membership)
      @no_member_user.save!
      user_address = UserAddress.find(:first, :conditions=>['user_id=1001']).clone
      user_address.user_id = @no_member_user.id
      user_address.save!
      user_email = UserEmail.find(:first, :conditions=>['user_id=1001']).clone
      user_email.user_id = @no_member_user.id
      user_email.save!
    end
    it "注文商品が１つもない場合" do
       post :create, :page => '1', :user_id=>'1001', :order_shipping=>@order_shipping,
        :order_item_list_size => -1, :order =>"", :order_form=> @order_form, :order_item=>{}
      flash[:error_info].should == "注文商品は最低１商品登録してください。"
      response.should redirect_to(:action => 'new', :page => '1')
    end

    it "注文商品のバリデーションエラーの場合" do
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      order_items["0"][:qty] = 9999999999999999999999
      post :create, :page => '1',  :user_id=>'1001', :order_shipping => @order_shipping,
        :order_item_list_size => 1, :order_item => order_items, :order =>"", :order_form=>@order_form
      response.should redirect_to(:action => 'new', :page => '1')
    end

    it "[正常系]会員　銀行振り込み" do
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23
      before_secure_count = ProductSkuSecureInventory.count
      @order_form[:gift_flg] = 1
      @order_form[:send_invoice_separately_flg] = 0
      @order_form[:write_sender_flg] = 1
      @order_form[:normal_payment_type] = OrderShipping.payment_type_div_id_by_key(:bank_transfer)

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      post :create, :page => '1', :user_id => '1001', :order_form=>@order_form, :order_shipping => @order_shipping, :order_item_list_size => 1,
       :order_item => order_items, :order => ""
      response.should redirect_to(:action => 'order_search', :condition => {:test_flg => 1})

      order = Order.find(:first, :order => 'id desc')
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid
      
      # 在庫は仮引当
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 1
      order_item.product_sku_secure_inventories.first.paid_flg.should be_false
      order_item.product_sku_id.should == 273
      product_sku = ProductSku.find(273)
      product_sku.stock_count.should == 150
      product_sku.orderable_count == 149
      ProductSkuSecureInventory.count.should == before_secure_count + 2
      
      # ポイントは付与する
      order.user_points.length.should == 1
      order.user_points.first.given_point.should == 25

      # 顧客区分は会員のまま
      order.user.member_div_key.should == :membership
      
      # メールは1通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
    end

    it "[正常系]会員　アリペイ決済" do
      @order_form[:normal_payment_type] = OrderShipping.payment_type_div_id_by_key(:alipay)
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23
      post :create, :page => '1', :user_id => '1001', :order_form=>@order_form, :order_shipping => @order_shipping, :order_item_list_size => 1,
       :order_item => order_items, :order => ""
      #TODO 注文登録・ペイメント連携・未実装
      # response.should redirect_to(:action => 'search_index')
    end

    it "[正常系]非会員　銀行振り込み　" do
      order_items = {"0" => OrderItem.find(8).attributes, "1" => OrderItem.find(9).attributes}
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23
      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149

      before_secure_count = ProductSkuSecureInventory.count
      @order_form[:gift_flg] = 1
      @order_form[:send_invoice_separately_flg] = 0
      @order_form[:write_sender_flg] = 1
      @order_form[:normal_payment_type] = OrderShipping.payment_type_div_id_by_key(:bank_transfer)

      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      post :create, :page => '1', :user_id => @no_member_user.id, :order_form=>@order_form, :order_shipping => @order_shipping, :order_item_list_size => 1,
       :order_item => order_items, :order => ""

      # 本番注文
      order = Order.find(:first, :order => 'id desc')
      order.test?.should be_false
      
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid

      # 在庫は仮引当
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 1
      order_item.product_sku_secure_inventories.first.paid_flg.should be_false
      order_item.product_sku_id.should == 273
      product_sku = ProductSku.find(273)
      product_sku.stock_count.should == 150
      product_sku.orderable_count == 149
      ProductSkuSecureInventory.count.should == before_secure_count + 2
      
      # ポイントは付与しない
      order.user_points.length.should == 0

      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      
      # 非会員には注文完了メールは送らない
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      response.should redirect_to(:action => 'order_search', :condition => {:test_flg => 1})
    end

    it "注文商品商品の在庫がない場合" do
      order_item = OrderItem.find(8)
      order_item.qty = 30
      order_items = {"0" => order_item.attributes, "1" => OrderItem.find(9).attributes}

      order_shipping = OrderShipping.find(1)
      order_shipping.order_items = [OrderItem.find(8), OrderItem.find(9)]
      order_shipping.status_div_key = :undelivery_unpaid
      order_shipping.save
      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149
      
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => 8})
      secures[0].paid_flg = false
      secures[0].save!

      post :update, :page => '1', :order_shipping => {:id => 1, :status_div => OrderShipping.status_div_id_by_key(:undelivery)},
        :order_item_list_size => 1, :order_item => order_items, :order => {}
      response.should redirect_to(:action => 'edit',:id => 1,  :page => '1')
      flash[:error_info].should_not == nil

      p1 = ProductSku.find(266)
      p1.stock_count.should == 50
      p1.orderable_count.should == 23

      p2 = ProductSku.find(273)
      p2.stock_count.should == 150
      p2.orderable_count.should == 149
      order_shipping.status_div_key.should == :undelivery_unpaid
      secures = ProductSkuSecureInventory.find(:all, :conditions => {:order_item_id => 8})
      secures[0].paid_flg.should be_false
    end

  end
end

