# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe PaymentsController do
  fixtures :users, :user_emails, :user_addresses, :payment_error_histories

  before do
    # adminでログイン
    login_as("admin")
  end

  describe "GET init_index" do
    it "init_index" do
      get :init_index
      response.should redirect_to(:action => 'index')
    end
  end

  describe "GET user_detail" do
    it "get user detail" do
      get :user_detail, :id => 1007
      controller.instance_variable_get(:@user).should_not be_nil
      controller.instance_variable_get(:@user_address).should_not be_nil
      response.should be_success
    end
  end

  describe "GET index" do
    it "index new" do
      session[:payments_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
    end

    it "index with session" do
      now_date = Time.now.strftime('%Y/%m/%d')
      session[:payments_search_cond] = { :exe_date_from => now_date, :exe_time_from => '00:00:00', :exe_date_to => now_date, :exe_time_to => '23:59:59' }
      flash[:error] = nil
      get :index
      response.should be_success
    end

    it "index error" do
      now_date = Time.now.strftime('%Y/%m/%d')
      session[:payments_search_cond] = { :exe_date_from => now_date, :exe_time_from => '00:00:00', :exe_date_to => now_date, :exe_time_to => '23:59:59' }
      flash[:error] = "AA"
      get :index
      response.should be_success
    end

    it "index reload" do
      session[:payments_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
    end
  end

  describe "GET search" do
    it "search" do
      now_date = Time.now.strftime('%Y/%m/%d')
      get :search, :condition => { :exe_date_from => now_date, :exe_time_from => '00:00:00', :exe_date_to => now_date, :exe_time_to => '23:59:59' }
      response.should be_success
      controller.instance_variable_get(:@condition).should_not == nil
    end

    it "search with condition error" do
      get :search, :condition => { :exe_time_from => 'ERROR' }
      response.should redirect_to :action => 'index'
    end
  end

  describe "delete" do
    it "テストデータを削除 テストデータが１件の場合" do
      # 削除元は各決済の履歴テーブルになるため、各決済プラグインにてテストしています。
#      lambda {
#        delete :destroy, :destroy_all_test => true, :page => '1'
#      }.should change(PaymentErrorHistory, :count).by(-1)
#      response.should redirect_to(:action => 'search', :page => '1')
    end
  end

  describe "POST send_mail" do
    it "ok" do
      post :send_mail, :id => "1", :page => 1, :mail_form => {:from => "test@ecrider.com", :to => "test@ecrider.com", :subject => "メールタイトル", :body => "本文" }
      response.should be_success
    end

    it "ng" do
      post :send_mail , :id => "1", :page => 1, :mail_form => { :subject => "メールタイトル", :body => "本文" }
      response.should redirect_to(:action => :mail, :id => "1", :page => 1)
    end
  end

end
