require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductDisplaysController do
  fixtures :products, :product_displays


  before(:each) do
    login_as("admin")
  end

  def mock_product_display(stubs={})
    unless @mock_product_display
      @mock_product_display = mock_model(ProductDisplay, stubs)
      # @mock_product_display.stub!(:product)
    end
    @mock_product_display
  end
  
  describe "responding to GET index" do

    it "should expose all product_displays as @product_displays" do
      ProductDisplay.should_receive(:find).
        with(:all, :include => :product, :order => "products.product_code asc, products.display_order asc").
        and_return([mock_product_display])
      get :index
      assigns[:product_displays].should == [mock_product_display]
    end

    describe "with mime type of xml" do
  
      it "should render all product_displays as xml" do
        request.env["HTTP_ACCEPT"] = "application/xml"
        ProductDisplay.should_receive(:find).
          with(:all, :include => :product, :order => "products.product_code asc, products.display_order asc").
          and_return(product_displays = mock("Array of ProductDisplays"))
        product_displays.should_receive(:to_xml).and_return("generated XML")
        get :index
        response.body.should == "generated XML"
      end
    
    end

  end

  describe "responding to GET show" do

    it "should expose the requested product_display as @product_display" do
      ProductDisplay.should_receive(:find).with("37").and_return(mock_product_display)
      mock_product_display.should_receive(:product).and_return(mock_model(Product))
      get :show, :id => "37"
      assigns[:product_display].should equal(mock_product_display)
    end
    
    describe "with mime type of xml" do

      it "should render the requested product_display as xml" do
        request.env["HTTP_ACCEPT"] = "application/xml"
        ProductDisplay.should_receive(:find).with("37").and_return(mock_product_display)
        mock_product_display.should_receive(:product).and_return(mock_model(Product))
        mock_product_display.should_receive(:to_xml).and_return("generated XML")
        get :show, :id => "37"
        response.body.should == "generated XML"
      end

    end
    
  end

  describe "responding to GET new" do
  
    it "should expose a new product_display as @product_display" do
      product = Product.find(:first)
      product_display = ProductDisplay.new(:product_id => product.id, :locale => 'th')
      session[:product_form] = {:product_displays => {'th' => product_display}}

      request.stub!(:xhr?).and_return(true)
      get :new, :product_display => {'locale' => 'th'}, :product_id => product.id, :locale => 'ja'
      assigns[:product_display].attributes.should == ProductDisplay.new(:locale => 'ja', :product_id => product.id).attributes
    end

  end

  describe "responding to GET edit" do
  
    it "should expose the requested product_display as @product_display" do
      product_display = ProductDisplay.new(:product_id => Product.find(:first), :locale => 'th')
      session[:product_form] = {:product_displays => {'th' => product_display}}
      
      ja_mock_product_display = ProductDisplay.find(:first)
      ja_mock_product_display.locale = 'ja'
      ProductDisplay.should_receive(:find).with("37").and_return(ja_mock_product_display)
      
      get :edit, :id => "37", :product_display => {'locale' => 'th', :product_name_1 => 'AAA'}
      assigns[:product_display].should equal(ja_mock_product_display)
    end

  end

  describe "responding to POST create" do

    describe "with valid params" do
      
      it "should expose a newly created product_display as @product_display" do
        ProductDisplay.should_receive(:new).with({'these' => 'params'}).and_return(mock_product_display(:save => true))
        post :create, :product_display => {:these => 'params'}
        assigns(:product_display).should equal(mock_product_display)
      end

      it "should redirect to the created product_display" do
        ProductDisplay.stub!(:new).and_return(mock_product_display(:save => true))
        post :create, :product_display => {}
        response.should redirect_to(product_display_url(mock_product_display))
      end
      
    end
    
    describe "with invalid params" do

      it "should expose a newly created but unsaved product_display as @product_display" do
        ProductDisplay.stub!(:new).with({'these' => 'params'}).and_return(mock_product_display(:save => false))
        post :create, :product_display => {:these => 'params'}
        assigns(:product_display).should equal(mock_product_display)
      end
    end
    
  end

  describe "responding to PUT udpate" do

    describe "with valid params" do

      it "should update the requested product_display" do
        ProductDisplay.should_receive(:find).with("37").and_return(mock_product_display)
        mock_product_display.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_display => {:these => 'params'}
      end

      it "should expose the requested product_display as @product_display" do
        ProductDisplay.stub!(:find).and_return(mock_product_display(:update_attributes => true))
        put :update, :id => "1"
        assigns(:product_display).should equal(mock_product_display)
      end

      it "should redirect to the product_display" do
        ProductDisplay.stub!(:find).and_return(mock_product_display(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(product_display_url(mock_product_display))
      end

    end
    
    describe "with invalid params" do

      it "should update the requested product_display" do
        ProductDisplay.should_receive(:find).with("37").and_return(mock_product_display)
        mock_product_display.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_display => {:these => 'params'}
      end

      it "should expose the product_display as @product_display" do
        ProductDisplay.stub!(:find).and_return(mock_product_display(:update_attributes => false))
        put :update, :id => "1"
        assigns(:product_display).should equal(mock_product_display)
      end

    end

  end

  describe "responding to DELETE destroy" do

    it "should destroy the requested product_display" do
      ProductDisplay.should_receive(:find).with("37").and_return(mock_product_display)
      mock_product_display.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "should redirect to the product_displays list" do
      ProductDisplay.stub!(:find).and_return(mock_product_display(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(product_displays_url)
    end

  end

end
