require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe ProductFaqController do
  fixtures :product_faqs, :product_faq_relation_products, :product_faq_displays, :products, :inquiries

    before(:each) do
      # adminでログイン
      login_as("admin")
    end

  describe "GET index" do
    it "index no session" do
      session[:product_faq_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
      controller.instance_variable_get(:@product_faqs).size.should == 6
    end

    it "index session" do
      cond = ProductFaq::SearchInfo.new()
      cond.product_code = '10000000'
      session[:product_faq_search_cond] = cond

      flash[:error] = nil
      get :index
      response.should be_success
      controller.instance_variable_get(:@product_faqs).size.should == 6
    end
  end
  describe "GET show" do
    it "success" do
      get :show ,:id => "1", :page => "1"
      controller.instance_variable_get(:@product_faq).id.should == 1
      controller.instance_variable_get(:@relation_product).size.should == 1
      controller.instance_variable_get(:@product_faq_displays).size.should == 6
      response.should be_success
    end
  end
  describe "GET destroy" do
    it "success destroy" do
      get :destroy ,:id => "1", :page => "1"
      response.should redirect_to :action => 'index', :page => 1
    end
  end

  describe "GET new" do
    it "inquiry parameter non" do
      get :new
      response.should be_success
      controller.instance_variable_get(:@product_faq).should_not == nil
    end
    it "inquiry parameter no product" do
      get :new, :inquiry_id => "1", :page => "1"
      response.should be_success
      inquiry = Inquiry.find(1)
      controller.instance_variable_get(:@product_faq).should_not == nil
      product_faq = controller.instance_variable_get(:@product_faq)

      display_ja = ProductFaqDisplay.new
      product_faq.displays.each do |d|
        if d.locale == 'ja'
          display_ja = d
        end
      end
      display_ja.inquiry_user.should == inquiry.fullname
      display_ja.inquiry.should      == inquiry.inquiry
      display_ja.answer.should       == inquiry.answer
    end
    it "inquiry parameter" do
      get :new, :inquiry_id => "2", :page => "1"
      response.should be_success
      inquiry = Inquiry.find(2)
      controller.instance_variable_get(:@product_faq).should_not == nil
      product_faq = controller.instance_variable_get(:@product_faq)

      display_ja = ProductFaqDisplay.new
      product_faq.displays.each do |d|
        if d.locale == 'ja'
          display_ja = d
        end
      end
      display_ja.inquiry_user.should == inquiry.fullname
      display_ja.inquiry.should      == inquiry.inquiry
      display_ja.answer.should       == inquiry.answer
    end
    it "invalid" do
      pfaq = ProductFaq.new()
      pfaq.answer_date_text = "error"
      flash[:product_faq] = pfaq
      get :new
      controller.instance_variable_get(:@product_faq).answer_date_text.should == "error"
      controller.instance_variable_get(:@invalid_product_faq_displays).should == nil
      controller.instance_variable_get(:@invalid_product_faq).should == nil
      response.should be_success
    end
    it "invalid" do
      pfaq = ProductFaq.new()
      pfaq.answer_date_text = "error"
      flash[:product_faq] = pfaq
      flash[:invalid_product_faq_displays] = "error_display"
      get :new
      controller.instance_variable_get(:@product_faq).answer_date_text.should == "error"
      controller.instance_variable_get(:@invalid_product_faq_displays).should == "error_display"
      controller.instance_variable_get(:@invalid_product_faq).should == nil
      response.should be_success
    end
  end

  describe "GET edit" do
    it "success" do
      flash[:product_faq] = nil
      get :edit, :id => "2"
      response.should be_success
      controller.instance_variable_get(:@product_faq).id.should == 2
    end
    it "invalid" do
      pfaq = ProductFaq.new()
      pfaq.answer_date_text = "error"
      flash[:product_faq] = pfaq
      get :edit
      response.should be_success
      controller.instance_variable_get(:@product_faq).answer_date_text.should == "error"
      controller.instance_variable_get(:@invalid_product_faq_displays).should == nil
      controller.instance_variable_get(:@invalid_product_faq).should == nil
    end
    it "invalid" do
      pfaq = ProductFaq.new()
      pfaq.answer_date_text = "error"
      flash[:product_faq] = pfaq
      flash[:invalid_product_faq_displays] = "error_display"
      get :edit
      response.should be_success
      controller.instance_variable_get(:@product_faq).answer_date_text.should == "error"
      controller.instance_variable_get(:@invalid_product_faq_displays).should == "error_display"
      controller.instance_variable_get(:@invalid_product_faq).should == nil
    end
  end

  describe "GET create" do
    faqd = {
            '0' => {:inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "ja"
                  },
            '1' => {:inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "en"
                  },
            '2' => {:inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "ko"
                  },
            '3' => {:inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "th"
                  },
            '4' => {:inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "zh_TW"
                  },
            '5' => {:inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "zh_CN"
                  }
          }

    order = {
      '0' => {:product_id => '1'}
    }

    it "save success" do
      faq = {
           :answer_date_text => "2009/07/10",
           :display_order => "",
           :invisible_flg => "0",
           :displays_attributes => faqd,
           :relation_products_attributes => order
          }

      get :create, :product_faq => faq , :page => 1
      response.should redirect_to :action => 'index', :page => 1
    end

    it "valid error product_faq" do
      faq = {
             :answer_date_text => "aa",
             :display_order => "",
             :invisible_flg => "0",
             :displays_attributes => faqd,
             :relation_products_attributes => order
            }
      get :create, :product_faq => faq , :page => 1
      response.should redirect_to :action => 'new', :page => 1
      flash[:product_faq].answer_date_text.should == 'aa'
      flash[:invalid_product_faq_displays].should_not == nil
    end

    it "valid error display" do
      faqd['0'][:inquiry] = nil
      faq = {
           :answer_date_text => "2009/07/10",
           :display_order => "",
           :invisible_flg => "0",
           :displays_attributes => faqd,
           :relation_products_attributes => order
          }
      get :create, :product_faq => faq, :page => 1
      response.should redirect_to :action => 'new', :page => 1
      flash[:product_faq].answer_date_text.should == '2009/07/10'
      flash[:invalid_product_faq_displays].should_not == nil
    end

    it "valid error common" do
      faqd.each_key do |key|
        faqd[key][:inquiry] = ""
        faqd[key][:inquiry_user] = ""
        faqd[key][:answer] = ""
        faqd[key][:answer_user] = ""
      end
      faq = {
           :answer_date_text => "2009/07/10",
           :display_order => "",
           :invisible_flg => "0",
           :displays_attributes => faqd,
           :relation_products_attributes => order
          }
      get :create, :product_faq => faq, :page => 1
      response.should redirect_to :action => 'new', :page => 1
      flash[:product_faq].answer_date_text.should == '2009/07/10'
      flash[:invalid_product_faq_displays].should_not == nil
    end
  end

  describe "GET update" do
    faqd = {
            '1' => {
                  :id => '1',
                  :inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "ja"
                  },
            '2' => {
                  :id => '2',
                  :inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "en"
                  },
            '3' => {
                  :id => '3',
                  :inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "ko"
                  },
            '4' => {
                  :id => '4',
                  :inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "th"
                  },
            '5' => {
                  :id => '5',
                  :inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "zh_TW"
                  },
            '6' => {
                  :id => '6',
                  :inquiry => "inquiry",
                  :inquiry_user => "inquiry_user",
                  :answer => "answer",
                  :answer_user => "answer_user",
                  :locale => "zh_CN"
                  }
          }

    order = {
      '0' => {:product_id => '1'}
    }

    it "save success" do
      faq = {
           :answer_date_text => "2009/07/10",
           :display_order => "",
           :invisible_flg => "0",
           :displays_attributes => faqd,
           :relation_products_attributes => order
          }

      get :update, :product_faq => faq , :page => 1, :id => 1
      response.should redirect_to :action => 'index', :page => 1
    end

    it "valid error product_faq" do
      faq = {
             :answer_date_text => "aa",
             :display_order => "",
             :invisible_flg => "0",
             :displays_attributes => faqd,
             :relation_products_attributes => order
            }
      get :update, :product_faq => faq , :page => 1, :id => 1
      response.should redirect_to :action => 'edit', :page => 1
      flash[:product_faq].answer_date_text.should == 'aa'
      flash[:invalid_product_faq_displays].should_not == nil
    end

    it "valid error display" do
      faqd['1'][:inquiry] = nil
      faq = {
           :answer_date_text => "2009/07/10",
           :display_order => "",
           :invisible_flg => "0",
           :displays_attributes => faqd,
           :relation_products_attributes => order
          }
      get :update, :product_faq => faq, :page => 1, :id => 1
      response.should redirect_to :action => 'edit', :page => 1
      flash[:product_faq].answer_date_text.should == '2009/07/10'
      flash[:invalid_product_faq_displays].should_not == nil
    end

    it "valid error common" do
    faqd2 = {
            '1' => {
                  :id => '1',
                  :inquiry => "",
                  :inquiry_user => "",
                  :answer => "",
                  :answer_user => "",
                  :locale => "ja"
                  },
            '2' => {
                  :id => '2',
                  :inquiry => "",
                  :inquiry_user => "",
                  :answer => "",
                  :answer_user => "",
                  :locale => "en"
                  },
            '3' => {
                  :id => '3',
                  :inquiry => "",
                  :inquiry_user => "",
                  :answer => "",
                  :answer_user => "",
                  :locale => "ko"
                  },
            '4' => {
                  :id => '4',
                  :inquiry => "",
                  :inquiry_user => "",
                  :answer => "",
                  :answer_user => "",
                  :locale => "th"
                  },
            '5' => {
                  :id => '5',
                  :inquiry => "",
                  :inquiry_user => "",
                  :answer => "",
                  :answer_user => "",
                  :locale => "zh_TW"
                  },
            '6' => {
                  :id => '6',
                  :inquiry => "",
                  :inquiry_user => "",
                  :answer => "",
                  :answer_user => "",
                  :locale => "zh_CN"
                  }
          }
      faq = {
           :answer_date_text => "2009/07/10",
           :display_order => "",
           :invisible_flg => "0",
           :displays_attributes => faqd2,
           :relation_products_attributes => order
          }
      get :update, :product_faq => faq, :page => 1, :id => 1

      flash[:product_faq].answer_date_text.should == '2009/07/10'
      flash[:invalid_product_faq_displays].should_not == nil
      response.should redirect_to :action => 'edit', :page => 1
    end
  end
end
