require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductGroupsController do

  before(:each) do
    login_as("admin")
  end

  def mock_product_group(stubs={})
    @mock_product_group ||= mock_model(ProductGroup, stubs)
  end
  
  describe "GET index" do
    it "assigns all product_groups as @product_groups" do
      ProductGroup.stub!(:paginate).and_return([mock_product_group])
      get :index
      assigns[:product_groups].should == [mock_product_group]
    end
  end
  
  describe "GET show" do
    it "assigns the requested product_group as @product_group" do
      ProductGroup.stub!(:find).with("37").and_return(mock_product_group)
      get :show, :id => "37"
      assigns[:product_group].should equal(mock_product_group)
    end
  end
  
  describe "GET new" do
    it "assigns a new product_group as @product_group" do
      ProductGroup.stub!(:new).and_return(mock_product_group)
      get :new
      assigns[:product_group].should equal(mock_product_group)
    end
  end
  
  describe "GET edit" do
    it "assigns the requested product_group as @product_group" do
      ProductGroup.stub!(:find).with("37").and_return(mock_product_group)
      get :edit, :id => "37"
      assigns[:product_group].should equal(mock_product_group)
    end
  end
  
  describe "POST create" do

    describe "with valid params" do
      it "assigns a newly created product_group as @product_group" do
        ProductGroup.stub!(:new).with({'these' => 'params'}).and_return(mock_product_group(:save => true))
        post :create, :product_group => {:these => 'params'}
        assigns[:product_group].should equal(mock_product_group)
      end

      it "redirects to the created product_group" do
        ProductGroup.stub!(:new).and_return(mock_product_group(:save => true))
        post :create, :product_group => {}
        response.should redirect_to(product_group_url(mock_product_group))
      end
    end

    describe "with invalid params" do
      it "assigns a newly created but unsaved product_group as @product_group" do
        ProductGroup.stub!(:new).with({'these' => 'params'}).and_return(mock_product_group(:save => false))
        post :create, :product_group => {:these => 'params'}
        assigns[:product_group].should equal(mock_product_group)
      end

      it "re-renders the 'new' template" do
        ProductGroup.stub!(:new).and_return(mock_product_group(:save => false))
        post :create, :product_group => {}
        response.should render_template('new')
      end
    end

  end

  describe "PUT update" do
    before(:each) do
      ProductGroup.create!({
          :name => "関連商品１",
          :priority => 1,
          :skus_attributes => {
            "1" => {:product_sku_id => 80},
            "2" => {:product_sku_id => 6},
            "3" => {:product_sku_id => 15},
          }
        })
      @product_group1 = ProductGroup.find_by_name("関連商品１")
    end
    
    it "assigns the requested product_group as @product_group" do
      ProductGroup.stub!(:find).and_return(mock_product_group())
      put :update, :id => "1"
      assigns[:product_group].should equal(mock_product_group)
    end
    
    it "redirects to the product_group" do
      ProductGroup.stub!(:find).and_return(@product_group1)
      put :update, :id => "1"
      response.should redirect_to(product_group_url(@product_group1))
    end

    it "re-renders the 'edit' template" do
      ProductGroup.stub!(:find).and_return(@product_group1)
      put :update, :id => "1", :product_group => {:name => nil}
      response.should render_template('edit')
    end
  end

  describe "DELETE destroy" do
    it "destroys the requested product_group" do
      ProductGroup.should_receive(:find).with("37").and_return(mock_product_group)
      mock_product_group.should_receive(:destroy)
      delete :destroy, :id => "37"
    end

    it "redirects to the product_groups list" do
      ProductGroup.stub!(:find).and_return(mock_product_group(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(product_groups_url)
    end
  end

end
