require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductImagesController do
  fixtures :products, :product_images

  before(:each) do
    login_as("admin")
  end

  def mock_product_image(stubs={})
    @mock_product_image ||= mock_model(ProductImage, stubs)
  end

  describe "GET sort" do
    before(:each) do
      ProductImage.create!(:product_id => "1", :filename => "image1", :display_order => 1, :content_type => "image/jpeg", :size => 777)
      ProductImage.create!(:product_id => "1", :filename => "image2", :display_order => 2, :content_type => "image/jpeg", :size => 777)
      ProductImage.create!(:product_id => "1", :filename => "image3", :display_order => 3, :content_type => "image/jpeg", :size => 777)
    end

    it "assigns the requested product_image as @product_image" do
      pending
      post :sort, :id => "1", :sortable_product_images => [3, 2, 1]
      product_images = assigns[:product_images]
      product_images.map(&:filename).should == ["image3", "image2", "image1"]
    end
  end

  describe "GET index" do
    it "assigns the requested product_image as @product_image" do
      ProductImage.stub!(:find).with(:all, :conditions => ["parent_id IS NULL"],
        :order => "display_order").and_return([mock_product_image])
      get :index
      assigns[:product_images].should == [mock_product_image]
    end
  end

  describe "GET show" do
    it "assigns the requested product_image as @product_image" do
      ProductImage.stub!(:find).with("37").and_return(mock_product_image)
      get :show, :id => "37"
      assigns[:product_image].should equal(mock_product_image)
    end
  end

  describe "GET new" do
    it "assigns a new product_image as @product_image" do
      ProductImage.stub!(:new).and_return(mock_product_image)
      get :new
      assigns[:product_image].should equal(mock_product_image)
    end
  end

  describe "GET edit" do
    it "assigns the requested product_image as @product_image" do
      ProductImage.stub!(:find).with("37").and_return(mock_product_image)
      get :edit, :id => "37"
      assigns[:product_image].should equal(mock_product_image)
    end
  end

  describe "POST create" do
    describe "with valid params" do
      it "assigns a newly created product_image as @product_image" do
        ProductImage.stub!(:new).with({'these' => 'params'}).and_return(mock_product_image(:save => true))
        post :create, :product_image => {:these => 'params'}
        assigns[:product_image].should equal(mock_product_image)
      end

      it "redirects to the created product_image" do
        pending
        ProductImage.stub!(:new).and_return(
          mock_product_image(:product_id => 10001, :display_order => 0, :save => true)
        )
        post :create, :product_image => {}
        response.should redirect_to(:controller => :product, :action => :edit, :id => 10001)
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved product_image as @product_image" do
        ProductImage.stub!(:new).with({'these' => 'params'}).and_return(mock_product_image(:save => false))
        post :create, :product_image => {:these => 'params'}
        assigns[:product_image].should equal(mock_product_image)
      end

      it "re-renders the 'new' template" do
        pending
        ProductImage.stub!(:new).and_return(
          mock_product_image(:product_id => 10001, :display_order => 0, :save => false)
        )
        post :create, :product_image => {}
        response.should redirect_to(:controller => :product, :action => :edit, :id => 10001)
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested product_image" do
        ProductImage.should_receive(:find).with("37").and_return(mock_product_image)
        mock_product_image.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_image => {:these => 'params'}
      end

      it "assigns the requested product_image as @product_image" do
        ProductImage.stub!(:find).and_return(mock_product_image(:update_attributes => true))
        put :update, :id => "1"
        assigns[:product_image].should equal(mock_product_image)
      end

      it "redirects to the product_image" do
        ProductImage.stub!(:find).and_return(mock_product_image(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(product_image_url(mock_product_image))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested product_image" do
        ProductImage.should_receive(:find).with("37").and_return(mock_product_image)
        mock_product_image.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_image => {:these => 'params'}
      end

      it "assigns the product_image as @product_image" do
        ProductImage.stub!(:find).and_return(mock_product_image(:update_attributes => false))
        put :update, :id => "1"
        assigns[:product_image].should equal(mock_product_image)
      end
    end
    
  end

  describe "POST update_mail_flg" do
    before(:each) do
      ProductImage.create!(:product_id => "1", :filename => "main_image", :display_order => 1, :content_type => "image/jpeg", :size => 777, :main_flg => true)
      ProductImage.create!(:product_id => "1", :filename => "not_main_image", :display_order => 1, :content_type => "image/jpeg", :size => 777, :main_flg => false)
    end

    it "updates the requested main_flg" do
      not_main_image = ProductImage.find_by_filename("not_main_image")

      post :update_main_flg, :product_id => "1", :main_product_image_id => not_main_image.id
      updated_not_main_image = ProductImage.find_by_filename("not_main_image")
      updated_main_image = ProductImage.find_by_filename("main_image")

      updated_not_main_image.main_flg.should == true
      updated_main_image.main_flg.should == false
    end
  end

  describe "DELETE destroy" do
    before(:each) do
      ProductImage.create!(:product_id => "1", :filename => "image_1", :display_order => 1, :content_type => "image/jpeg", :size => 777, :main_flg => false)
      @image_2 = ProductImage.create!(:product_id => "1", :filename => "image_2", :display_order => 2, :content_type => "image/jpeg", :size => 777, :main_flg => false)
      @image_3 = ProductImage.create!(:product_id => "1", :filename => "image_3", :display_order => 3, :content_type => "image/jpeg", :size => 777, :main_flg => true)
      ProductImage.create!(:product_id => "1", :filename => "image_4", :display_order => 4, :content_type => "image/jpeg", :size => 777, :main_flg => false)
      ProductImage.create!(:product_id => "1", :filename => "image_5", :display_order => 5, :content_type => "image/jpeg", :size => 777, :main_flg => false)
    end
  
    it "メイン画像ではない画像を削除" do
      delete :destroy, :id => @image_2.id
      images = ProductImage.find(:all, :conditions => ["product_id = 1"], :order => "display_order")
      images.length.should == 4
      images[0].filename.should == "image_1"
      images[0].display_order.should == 1
      images[0].main_flg.should be_false
      images[1].filename.should == "image_3"
      images[1].display_order.should == 2
      images[1].main_flg.should be_true
      images[2].filename.should == "image_4"
      images[2].display_order.should == 3
      images[2].main_flg.should be_false
      images[3].filename.should == "image_5"
      images[3].display_order.should == 4
      images[3].main_flg.should be_false
    end
  
    it "メイン画像を削除" do
      delete :destroy, :id => @image_3.id
      images = ProductImage.find(:all, :conditions => ["product_id = 1"], :order => "display_order")
      images.length.should == 4
      images[0].filename.should == "image_1"
      images[0].display_order.should == 1
      images[0].main_flg.should be_true
      images[1].filename.should == "image_2"
      images[1].display_order.should == 2
      images[1].main_flg.should be_false
      images[2].filename.should == "image_4"
      images[2].display_order.should == 3
      images[2].main_flg.should be_false
      images[3].filename.should == "image_5"
      images[3].display_order.should == 4
      images[3].main_flg.should be_false
    end
  end

end
