require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductSkusController do
  fixtures :products, :user_ranks, :product_skus, :product_sku_inventories, :product_prices

  before(:each) do
    login_as("admin")
  end

  def mock_product_sku(stubs={})
    @mock_product_sku ||= mock_model(ProductSku, stubs)
  end

  describe "GET show" do
    it "assigns the requested product_sku as @product_sku" do
      ProductSku.stub!(:find).with("37").and_return(mock_product_sku)
      get :show, :id => "37"
      assigns[:product_sku].should equal(mock_product_sku)
    end

    it "assigns the requested product_sku as @product_sku and session" do
      ProductSku.stub!(:find).with("37").and_return(mock_product_sku)
      get :show, :id => "37", :product_sku => ProductSku.new
      assigns[:product_sku].should equal(mock_product_sku)
    end
  
    it "re-renders the 'edit' template" do
      ProductSku.stub!(:find).and_return(mock_product_sku(:product_id => 1, :valid? => false, :components => [], :attributes => nil))
      get :show, :id => "37", :product_sku => {:these => 'params'}
      response.should render_template('edit')
    end
  end
  
  describe "GET new" do
    it "assigns a new product_sku as @product_sku" do
      Product.stub!(:find).and_return(Product.find(:first))
      ProductSku.stub!(:new).and_return(mock_product_sku(:prices => []))
      get :new, :product_id => 1, :variation_item_ids => [1,2]
      assigns[:product_sku].should equal(mock_product_sku)
    end
  end
  
  describe "GET edit" do
    before(:each) do
      session[:tmp_product_sku] = mock_product_sku(:product_id => 1)
    end
  
    it "assigns the requested product_sku as @product_sku" do
      ProductSku.stub!(:find).with("38").and_return(mock_product_sku(:product_id => 1))
      get :edit, :id => "38"
      assigns[:product_sku].should equal(mock_product_sku)
    end
  end
  
  describe "POST create" do
  
    describe "with valid params" do
      it "assigns a newly created product_sku as @product_sku" do
        ProductSku.stub!(:new).and_return(mock_product_sku(:product_id => 1, :save! => true, :update_composable_count => true))
        post :create, :product_sku => {:product_id => 1}
        assigns[:product_sku].should equal(mock_product_sku)
      end
  
      it "redirects to the created product_sku" do
        ProductSku.stub!(:new).and_return(mock_product_sku(:id => 10001, :product_id => 1, :save! => true, :update_composable_count => true))
        post :create, :product_sku => {:product_id => 1}
        response.should redirect_to(:controller => :product, :action => :show, :id => 1)
      end
    end
  
    describe "with invalid params" do
      it "assigns a newly created but unsaved product_sku as @product_sku" do
        mock_sku = mock_product_sku(:save => true, :update_composable_count => true)
        ProductSku.stub!(:new).with({'these' => 'params'}).and_return(mock_sku)
        post :create, :product_sku => {:these => 'params'}
        assigns[:product_sku].should equal(mock_product_sku)
      end
  
      it "re-renders the 'new' template" do
        mock_sku = mock_model(ProductSku, {:prices => [], :components => [],:product_id => 1})
        mock_sku.errors.stub!(:full_messages).and_return([])
        mock_sku.stub!(:save!).and_raise(ActiveRecord::RecordInvalid.new(mock_sku))
        ProductSku.stub!(:new).and_return(mock_sku)

        post :create, :product_sku => {:product_id => 1}
        response.should render_template('new')
      end
    end
  
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested product_sku" do
        ProductSku.should_receive(:find).with("37").and_return(mock_product_sku(:prices => [], :components => [], :product_id => 1))
        mock_product_sku.should_receive(:attributes=).with({'these' => 'params'})
        put :update, :id => "37", :product_sku => {:these => 'params'}
      end
    
      it "assigns the requested product_sku as @product_sku" do
        ProductSku.stub!(:find).and_return(mock_product_sku)
        put :update, :id => "1"
        assigns[:product_sku].should equal(mock_product_sku)
      end
    
      it "redirects to the product_sku" do
        ProductSku.stub!(:find).and_return(mock_product_sku(:prices => [], :components => [], :product_id => 1, :save! => true, :update_composable_count => true))
        put :update, :id => "1"
        response.should redirect_to(:controller => :product, :action => :show, :id => 1)
      end
    end

    describe "with invalid params" do
      it "updates the requested product_sku" do
        ProductSku.should_receive(:find).with("37").and_return(mock_product_sku(:prices => [], :components => [], :product_id => 1))
        mock_product_sku.should_receive(:attributes=).with({'these' => 'params'})
        put :update, :id => "37", :product_sku => {:these => 'params'}
      end
      
      it "assigns the product_sku as @product_sku" do
        mock_sku = mock_product_sku(:prices => [], :components => [], :save! => true, :update_composable_count => true)
        ProductSku.stub!(:find).and_return(mock_sku)
        put :update, :id => "1", :product_sku => {}
        assigns[:product_sku].should equal(mock_product_sku)
      end

      it "re-renders the 'edit' template" do
        product_sku = mock_model(ProductSku, {:prices => [], :components => [],:product_id => 1})
        product_sku.errors.stub!(:full_messages).and_return([])
        error = ActiveRecord::RecordInvalid.new(product_sku)
        product_sku.stub!(:save!).and_raise(error)
        
        ProductSku.stub!(:find).and_return(product_sku)
 
        put :update, :id => "1"
        response.should render_template(:edit)
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested product_sku" do
      ProductSku.should_receive(:find).with("37").and_return(mock_product_sku)
      mock_product_sku.should_receive(:destroy)
      delete :destroy, :id => "37"
    end

    it "redirects to the product_skus list" do
      mock_errors = mock(:errors)
      mock_errors.should_receive(:full_messages).and_return(["errror!"])
      ProductSku.stub!(:find).and_return(mock_product_sku(:product_id => 1, :destroy => true, :errors => mock_errors))
      delete :destroy, :id => "1"
      response.should redirect_to(:controller => :product, :action => :show, :id => 1)
    end
  end

end
