require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductVariationItemsController do

  before(:each) do
    login_as("admin")
  end

  def mock_product_variation_item(stubs={})
    @mock_product_variation_item ||= mock_model(ProductVariationItem, stubs)
  end
  
  describe "GET new" do
    it "assigns a new product_variation_item as @product_variation_item" do
      ProductVariationItem.stub!(:new).and_return(mock_product_variation_item)
      get :new
      assigns[:product_variation_item].should equal(mock_product_variation_item)
    end
  end

  describe "GET edit" do
    it "assigns the requested product_variation_item as @product_variation_item" do
      ProductVariationItem.stub!(:find).with("37").and_return(mock_product_variation_item)
      get :edit, :id => "37"
      assigns[:product_variation_item].should equal(mock_product_variation_item)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created product_variation_item as @product_variation_item" do
        ProductVariationItem.stub!(:new).with({'these' => 'params'}).and_return(mock_product_variation_item(:save => true))
        post :create, :product_variation_item => {:these => 'params'}
        assigns[:product_variation_item].should equal(mock_product_variation_item)
      end

      it "redirects to the created product_variation_item" do
        ProductVariationItem.stub!(:new).and_return(mock_product_variation_item(:save => true))
        post :create, :product_variation_item => {}
        response.should be_success

        #        response.should redirect_to(product_variation_item_url(mock_product_variation_item))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved product_variation_item as @product_variation_item" do
        ProductVariationItem.stub!(:new).with({'these' => 'params'}).and_return(mock_product_variation_item(:save => false))
        post :create, :product_variation_item => {:these => 'params'}
        assigns[:product_variation_item].should equal(mock_product_variation_item)
      end

      it "re-renders the 'new' template" do
        ProductVariationItem.stub!(:new).and_return(mock_product_variation_item(:save => false))
        post :create, :product_variation_item => {}
        response.should be_success
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested product_variation_item" do
        ProductVariationItem.should_receive(:find).with("37").and_return(mock_product_variation_item)
        mock_product_variation_item.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_variation_item => {:these => 'params'}
      end

      it "assigns the requested product_variation_item as @product_variation_item" do
        ProductVariationItem.stub!(:find).and_return(mock_product_variation_item(:update_attributes => true))
        put :update, :id => "1"
        assigns[:product_variation_item].should equal(mock_product_variation_item)
      end

      it "redirects to the product_variation_item" do
        ProductVariationItem.stub!(:find).and_return(mock_product_variation_item(:update_attributes => true))
        put :update, :id => "1"
        response.should be_success
      end
    end
    
    describe "with invalid params" do
      it "updates the requested product_variation_item" do
        ProductVariationItem.should_receive(:find).with("37").and_return(mock_product_variation_item)
        mock_product_variation_item.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_variation_item => {:these => 'params'}
      end

      it "assigns the product_variation_item as @product_variation_item" do
        ProductVariationItem.stub!(:find).and_return(mock_product_variation_item(:update_attributes => false))
        put :update, :id => "1"
        assigns[:product_variation_item].should equal(mock_product_variation_item)
      end

      it "re-renders the 'edit' template" do
        ProductVariationItem.stub!(:find).and_return(mock_product_variation_item(:update_attributes => false))
        put :update, :id => "1"
        response.should be_success
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested product_variation_item" do
      ProductVariationItem.should_receive(:find).with("37").and_return(mock_product_variation_item)
      mock_product_variation_item.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  end

end
