require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductVariationsController do

  before(:each) do
    login_as("admin")
  end

  def mock_product_variation(stubs={})
    @mock_product_variation ||= mock_model(ProductVariation, stubs)
  end
  
  describe "GET index" do
    it "assigns all product_variations as @product_variations" do
      mock_product_variation = mock('product_variations', :size => 1)
      ProductVariation.stub!(:paginate).and_return([mock_product_variation])
      get :index
      assigns[:product_variations].should == [mock_product_variation]
    end
  end

  describe "GET show" do
    it "assigns the requested product_variation as @product_variation" do
      ProductVariation.stub!(:find).with("37", :include => [:displays, :items]).and_return(mock_product_variation)
      get :show, :id => "37"
      assigns[:product_variation].should equal(mock_product_variation)
    end
  end

  describe "GET new" do
    it "assigns a new product_variation as @product_variation" do
      ProductVariation.stub!(:new).and_return(mock_product_variation)
      get :new
      assigns[:product_variation].should equal(mock_product_variation)
    end
  end

  describe "GET edit" do
    it "assigns the requested product_variation as @product_variation" do
      ProductVariation.stub!(:find).with("37", :include => [:displays, :items]).and_return(mock_product_variation)
      get :edit, :id => "37"
      assigns[:product_variation].should equal(mock_product_variation)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created product_variation as @product_variation" do
        ProductVariation.stub!(:new).with({'these' => 'params'}).and_return(mock_product_variation(:save => true))
        post :create, :product_variation => {:these => 'params'}
        assigns[:product_variation].should equal(mock_product_variation)
      end

      it "redirects to the created product_variation" do
        ProductVariation.stub!(:new).and_return(mock_product_variation(:save => true))
        post :create, :product_variation => {}
        response.should redirect_to(product_variation_url(mock_product_variation))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved product_variation as @product_variation" do
        ProductVariation.stub!(:new).with({'these' => 'params'}).and_return(mock_product_variation(:save => false))
        post :create, :product_variation => {:these => 'params'}
        assigns[:product_variation].should equal(mock_product_variation)
      end

      it "re-renders the 'new' template" do
        ProductVariation.stub!(:new).and_return(mock_product_variation(:save => false))
        post :create, :product_variation => {}
        response.should render_template('new')
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested product_variation" do
        ProductVariation.should_receive(:find).with("37").and_return(mock_product_variation)
        mock_product_variation.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_variation => {:these => 'params'}
      end

      it "assigns the requested product_variation as @product_variation" do
        ProductVariation.stub!(:find).and_return(mock_product_variation(:update_attributes => true))
        put :update, :id => "1"
        assigns[:product_variation].should equal(mock_product_variation)
      end

      it "redirects to the product_variation" do
        ProductVariation.stub!(:find).and_return(mock_product_variation(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(product_variation_url(mock_product_variation))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested product_variation" do
        ProductVariation.should_receive(:find).with("37").and_return(mock_product_variation)
        mock_product_variation.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_variation => {:these => 'params'}
      end

      it "assigns the product_variation as @product_variation" do
        ProductVariation.stub!(:find).and_return(mock_product_variation(:update_attributes => false))
        put :update, :id => "1"
        assigns[:product_variation].should equal(mock_product_variation)
      end

      it "re-renders the 'edit' template" do
        ProductVariation.stub!(:find).and_return(mock_product_variation(:update_attributes => false))
        put :update, :id => "1"
        response.should render_template('edit')
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested product_variation" do
      ProductVariation.should_receive(:find).with("37").and_return(mock_product_variation)
      mock_product_variation.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the product_variations list" do
      ProductVariation.stub!(:find).and_return(mock_product_variation(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(product_variations_url)
    end
  end

end
