require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe ShippingCostsController do
  fixtures :countries, :prefectures, :shipping_cost_boundaries, :shipping_costs
    before(:each) do
      # adminでログイン
      login_as("admin")
    end

  describe "GET index" do
    it "index no session" do
      session[:country_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
      controller.instance_variable_get(:@countries).size.should == 20
    end

    it "index session" do
      cond = Country::SearchCondition.new()
      cond.name = 'Ja'
      flash[:error] = nil
      get :index ,:condition => {:name => 'Ja'}
      response.should be_success
      controller.instance_variable_get(:@countries).size.should == 5
    end
  end
  describe "GET edit_point" do
    it "indexから遷移した場合" do
      get :edit_point, :session_crear => "true"
      response.should be_success
      controller.instance_variable_get(:@boundary_points).should_not == nil
      controller.instance_variable_get(:@countries).should_not == nil
    end
    it "戻るボタンで遷移した場合" do
      boundaries = []
      boundaries << ShippingCostBoundary.new(:point => 100)
      boundaries << ShippingCostBoundary.new(:point => 200)
      session[:boundary_points] = boundaries
      session[:updatable_country_codes] = ['CN', 'JP']
      get :edit_point
      response.should be_success
      controller.instance_variable_get(:@boundary_points).size.should == 2
      controller.instance_variable_get(:@countries).should_not == nil
    end
    it "バリデーションエラーで遷移した場合" do
      flash[:error] = "test"
      flash[:boundary_points] = [100,200,300]
      flash[:countries] = ['CN', 'JP']
      get :edit_point
      response.should be_success
      controller.instance_variable_get(:@boundary_points).size.should == 3
      controller.instance_variable_get(:@countries).should_not == nil
    end
  end

  describe "GET edit_shipping_cost" do
    it "edit_pointから遷移した場合バリデーション正常" do
      boundaries = {
        "0" => ShippingCostBoundary.new(:point => "100"), "1" => ShippingCostBoundary.new(:point => "400"),
        "2" => ShippingCostBoundary.new(:point => "200"), "3" => ShippingCostBoundary.new(:point => "100"),
        "4" => ShippingCostBoundary.new(:point => "100"), "5" => ShippingCostBoundary.new(:point => ""),
        "6" => ShippingCostBoundary.new(:point => ""), "7" => ShippingCostBoundary.new(:point => "100"),
        "8" => ShippingCostBoundary.new(:point => "100"), "9" => ShippingCostBoundary.new(:point => "100"),
        "10" => ShippingCostBoundary.new(:point => "100"), "11" => ShippingCostBoundary.new(:point => "100"),
        "12" => ShippingCostBoundary.new(:point => "300"), "13" => ShippingCostBoundary.new(:point => ""),
        "14" => ShippingCostBoundary.new(:point => "100"), "15" => ShippingCostBoundary.new(:point => "100"),
        "16" => ShippingCostBoundary.new(:point => "100"), "17" => ShippingCostBoundary.new(:point => "100"),
        "18" => ShippingCostBoundary.new(:point => "")
      }
      get :edit_shipping_cost, :boundaries => boundaries, :countries => ['CN', 'JP'], :page => 1
      response.should be_success
      controller.instance_variable_get(:@boundary_points).size.should == 5
      controller.instance_variable_get(:@countries).should_not == nil
    end
    it "edit_pointから遷移した場合バリデーション不正（選択国なし）" do
      boundaries = {
        "0" => ShippingCostBoundary.new(:point => "100"), "1" => ShippingCostBoundary.new(:point => "100"),
        "2" => ShippingCostBoundary.new(:point => "100"), "3" => ShippingCostBoundary.new(:point => "100"),
        "4" => ShippingCostBoundary.new(:point => "100"), "5" => ShippingCostBoundary.new(:point => "100"),
        "6" => ShippingCostBoundary.new(:point => "100"), "7" => ShippingCostBoundary.new(:point => "100"),
        "8" => ShippingCostBoundary.new(:point => "100"), "9" => ShippingCostBoundary.new(:point => "100"),
        "10" => ShippingCostBoundary.new(:point => "100"), "11" => ShippingCostBoundary.new(:point => "100"),
        "12" => ShippingCostBoundary.new(:point => "100"), "13" => ShippingCostBoundary.new(:point => "100"),
        "14" => ShippingCostBoundary.new(:point => "100"), "15" => ShippingCostBoundary.new(:point => "100"),
        "16" => ShippingCostBoundary.new(:point => "100"), "17" => ShippingCostBoundary.new(:point => "100"),
        "18" => ShippingCostBoundary.new(:point => "100")
      }
      get :edit_shipping_cost, :boundaries => boundaries, :page => 1
      response.should redirect_to :action => 'edit_point', :page => 1
    end
    it "戻るボタンで遷移した場合" do
      boundaries = []
      boundaries << ShippingCostBoundary.new(:point => 100)
      boundaries << ShippingCostBoundary.new(:point => 200)
      session[:boundary_points] = boundaries
      session[:shipping_cost_countries] = Country.all
      shipping_costs = []
      shipping_costs << ShippingCost.new(:shipping_cost => 1000)
      shipping_costs << ShippingCost.new(:shipping_cost => 2000)
      session[:shipping_costs] = shipping_costs
      get :edit_shipping_cost
      response.should be_success
      controller.instance_variable_get(:@boundary_points).size.should == 2
      controller.instance_variable_get(:@countries).should_not == nil
    end
  end

  describe "GET confirm" do
    it "バリデーション正常" do
      boundaries = [ShippingCostBoundary.new(:point => 100), ShippingCostBoundary.new(:point => 200), ShippingCostBoundary.new(:point => 300)]
      shipping_costs = {
        "0" => {"shipping_cost" => "100", "boundary_index"=>"0", "area_code"=>"10", "area_type_div"=>"1"},
        "1" => {"shipping_cost" => "200", "boundary_index"=>"1", "area_code"=>"10", "area_type_div"=>"1"},
        "2" => {"shipping_cost" => "300", "boundary_index"=>"2", "area_code"=>"10", "area_type_div"=>"1"}
      }
      session[:boundary_points] = boundaries
      get :confirm, :shipping_costs => shipping_costs, :page => 1
      response.should be_success
      controller.instance_variable_get(:@shipping_costs).size.should == 3
    end
    it "バリデーション不正" do
      boundaries = [ShippingCostBoundary.new(:point => 100), ShippingCostBoundary.new(:point => 200), ShippingCostBoundary.new(:point => 300)]
      shipping_costs = {
        "0" => {"shipping_cost" => "aaa", "boundary_index"=>"0", "area_code"=>"10", "area_type_div"=>"1"},
        "1" => {"shipping_cost" => "200", "boundary_index"=>"1", "area_code"=>"10", "area_type_div"=>"1"},
        "2" => {"shipping_cost" => "", "boundary_index"=>"2", "area_code"=>"10", "area_type_div"=>"1"}
      }
      session[:boundary_points] = boundaries
      get :confirm, :shipping_costs => shipping_costs, :page => 1
      response.should redirect_to :action => 'edit_shipping_cost', :page => 1
    end
  end

  describe "GET update" do
    it "update" do
      shipping_costs = []
      shipping_costs << ShippingCost.new("shipping_cost" => "200", "boundary_index"=>"0", "area_code"=>"CN", "area_type_div"=>"1")
      shipping_costs << ShippingCost.new("shipping_cost" => "300", "boundary_index"=>"1", "area_code"=>"CN", "area_type_div"=>"1")
      shipping_costs << ShippingCost.new("shipping_cost" => "400", "boundary_index"=>"2", "area_code"=>"CN", "area_type_div"=>"1")
      shipping_costs << ShippingCost.new("shipping_cost" => "200", "boundary_index"=>"0", "area_code"=>"10", "area_type_div"=>"2")
      shipping_costs << ShippingCost.new("shipping_cost" => "300", "boundary_index"=>"1", "area_code"=>"10", "area_type_div"=>"2")
      shipping_costs << ShippingCost.new("shipping_cost" => "400", "boundary_index"=>"2", "area_code"=>"10", "area_type_div"=>"2")
      session[:boundary_points] = [100, 200, 300]
      session[:updatable_country_codes] = ['CN', 'JP']
      session[:sipping_costs] = shipping_costs
      get :update, :page => 1
      response.should redirect_to :action => 'index', :page => 1
      session[:boundary_points].should == nil
      session[:updatable_country_codes].should == nil
      session[:sipping_costs].should == nil
      ShippingCost.all.size.should == 6
      result_boundary = ShippingCostBoundary.all
      result_boundary.size.should == 3
      result_boundary[0].shipping_costs.size.should == 2
      result_boundary[1].shipping_costs.size.should == 2
      result_boundary[2].shipping_costs.size.should == 2
    end
  end

  describe "GET edit_country" do
    it "edit_country" do
      get :edit_country, :id => 'JP' , :page => 1
      response.should be_success
      controller.instance_variable_get(:@country).id.should == 'JP'
    end
  end
  describe "GET show_country" do
    it "show_country" do
      get :show_country, :id => 'JP' , :page => 1
      response.should be_success
      controller.instance_variable_get(:@country).id.should == 'JP'
    end
  end
  describe "GET update_country" do
    it "update_country" do
      get :update_country, :id => 'JP' , :page => 1, :country => {:use_language_div => '1', :name_order_div => '2'}
      response.should redirect_to :action => 'index', :page => 1
      result = Country.find(:first, :conditions => ['code = ?', 'JP'])
      result.use_language_div.should == "1"
      result.name_order_div.should == "2"
    end
  end
end
