# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe UserController do
  fixtures :users, :user_emails, :user_addresses, :mail_deliveries, 
    :user_points, :user_ranks, :point_settings, :shop_settings, :mail_templates,
    :goldberg_users, :goldberg_site_actions, :goldberg_site_action_groups,
    :goldberg_roles, :goldberg_role_actions

 # include AuthenticatedTestHelper
  include EcRider::PluginModule::RSpecExt

  before do
    login_as("admin")

    session[:user_search_cond] = nil
    flash[:error] = nil
  end

  describe "GET index" do
    it "should get index" do
      get :index
      response.should be_success
      response.should render_template(:index)
      controller.instance_variable_get(:@users).should be_nil
    end

    it "should get index with session" do
      session[:user_search_cond] = {:name => 'てすと'}

      get :index
      response.should be_success
      response.should render_template(:index)
      controller.instance_variable_get(:@users).length.should == 1
    end

    it "should get search with invalid" do
      get :index, :condition => {:regist_date_from => '2009/01/32'}
      response.should render_template(:index)
      controller.instance_variable_get(:@users).should be_nil
    end
  end

  it "GET show" do
    user = User.find(1008)
    get :show, :id => user.id, :page => '1'
    response.should render_template(:show)
    controller.instance_variable_get(:@user).should == user
    controller.instance_variable_get(:@user_address).should == user.user_address
    controller.instance_variable_get(:@page_no).should == '1'
  end

  describe "GET edit" do

    it "should render edit" do
      user = User.find(1008)
      get :edit, :id => user.id, :page => '1'
      response.should render_template(:edit)
      controller.instance_variable_get(:@user).should == user
      controller.instance_variable_get(:@user_address).should == user.user_address
      controller.instance_variable_get(:@user_email).should == user.user_email
      controller.instance_variable_get(:@page_no).should == '1'
    end

    it "invalid subaddressがnilの場合" do
      user = User.new
      user.valid?
      email = UserEmail.new
      email.valid?
      user_address = UserAddress.new
      user_address.valid?
      shipping_address = []
      shipping_address << UserAddress.new()
      shipping_address[0].valid?

      flash[:user] = user
      flash[:user_email] = email
      flash[:user_address] = user_address
      flash[:sub_address] = nil
      flash[:shipping_address] = shipping_address

      get :edit, :id => user.id, :page => '1'
      response.should render_template(:edit)
      controller.instance_variable_get(:@user).should == user
      controller.instance_variable_get(:@user_address).should == user_address
      controller.instance_variable_get(:@user_email).should == email
      controller.instance_variable_get(:@sub_address).should == nil
      controller.instance_variable_get(:@shipping_addresses).should == shipping_address
      controller.instance_variable_get(:@page_no).should == '1'
      flash[:user].should == user
      flash[:user_email].should == email
      flash[:user_address].should == user_address
      flash[:shipping_address].should == shipping_address
    end

    it "invalid ship_toaddressがnilの場合" do
      user = User.new
      user.valid?
      email = UserEmail.new
      email.valid?
      user_address = UserAddress.new
      user_address.valid?

      flash[:user] = user
      flash[:user_email] = email
      flash[:user_address] = user_address
      flash[:sub_address] = user_address
      flash[:shipping_address] = nil

      get :edit, :id => user.id, :page => '1'
      response.should render_template(:edit)
      controller.instance_variable_get(:@user).should == user
      controller.instance_variable_get(:@user_address).should == user_address
      controller.instance_variable_get(:@user_email).should == email
      controller.instance_variable_get(:@sub_address).should == user_address
      controller.instance_variable_get(:@page_no).should == '1'
      flash[:user].should == user
      flash[:user_email].should == email
      flash[:user_address].should == user_address
      flash[:shipping_address].should == nil
    end
  end

  describe "put update" do
    before do
      @user_address = {
        :tel_num => "000-0000-0000",
        :kana_name_first => "テスト",
        :kana_name_last => "ジロウ",
        :name_first => "てすと",
        :name_last => "じろう",
        :zip_code => "0000000",
        :address_1 => "福岡県",
        :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１",
        :country_code => 'JP' }
      @user_email = {:email => "test@test.jp" }
      @user = {
        :birth_month => "1",
        :dm_send_div => "0",
        :job_div => "1",
        :birth_day => "23",
        :sex => "1",
        :birth_year => "1970" }
      @sub_address = {
        :tel_num => "000-0000-0000",
        :kana_name_first => "TEST",
        :kana_name_last => "2ROU",
        :name_first => "TEST",
        :name_last => "JIRO",
        :zip_code => "0000000",
        :address_1 => "A1",
        :address_2 => "A2",
        :address_3 => "A3" }

    end

    it "should update user" do
      post :update, :user_address => @user_address, :user_email => @user_email, :user => @user, :id => '1008', :page => '1'
      response.should redirect_to(:action => 'index', :page => '1')
    end

    it "should update user with invalid" do
      @user_address[:tel_num] = 'aaa-aaaa-aaaa'
      post :update, :user_address => @user_address, :user_email => @user_email, :user => @user, :id => '1008', :page => '1'
      response.should redirect_to(:action => :edit, :page => '1', :id => '1008')
      flash[:user].is_a?(User).should == true
      flash[:user_email].is_a?(UserEmail).should == true
      flash[:user_address].is_a?(UserAddress).should == true
      flash[:user_address].errors.length.should == 2
    end

    it "should update user with sub_address" do
      post :update, :user_address => @user_address, :user_email => @user_email, :user => @user, :id => '1008', :page => '1', :sub_address => @sub_address
      response.should redirect_to(:action => 'index', :page => '1')
    end

    it "should update user no sub_address" do
      post :update, :user_address => @user_address, :user_email => @user_email, :user => @user, :id => '1000', :page => '1', :sub_address => @sub_address
      response.should redirect_to(:action => 'index', :page => '1')
    end

    it "should update user shipping_address" do
      user = User.find(1005)
      shipping_addresses = user.shipping_addresses
      ship_to = {shipping_addresses[0].id.to_s => shipping_addresses[0].attributes }

      post :update, :user_address => @user_address, :user_email => @user_email, :user => @user, :id => '1000', :page => '1', :sub_address => @sub_address, :shipping_address => ship_to
      response.should redirect_to(:action => 'index', :page => '1')
    end
    it "should update user shipping_address invalid" do
      user = User.find(1005)
      shipping_addresses = user.shipping_addresses
      shipping_addresses[0].kana_name_last = 'invalid'
      ship_to = {shipping_addresses[0].id.to_s => shipping_addresses[0].attributes }
      post :update, :user_address => @user_address, :user_email => @user_email, :user => @user, :id => user.id, :page => '1', :sub_address => @sub_address, :shipping_address => ship_to
      response.should redirect_to(:action => 'edit', :page => '1', :id => '1005')
      flash[:user].is_a?(User).should == true
      flash[:user_email].is_a?(UserEmail).should == true
      flash[:user_address].is_a?(UserAddress).should == true
      flash[:shipping_address].should_not == nil
    end
  end


  describe "test_user" do
    before do
      User.update(users(:users_00001).id, :test_flg => true)
    end

    after do
      User.update(users(:users_00001).id, :test_flg => false)
    end

    it "should render show test user" do
      user = User.find(1000)
      get :show, :id => user.id, :page => '1'
      response.should render_template(:show_test_user)
      controller.instance_variable_get(:@user).should == user
      controller.instance_variable_get(:@user_address).should == user.user_address
      controller.instance_variable_get(:@page_no).should == '1'
    end

    it "should render edit test user" do
      user = User.find(1000)
      get :edit_test_user, :id => user.id, :page => '1'
      response.should render_template(:edit_test_user)
      controller.instance_variable_get(:@user).should == user
      controller.instance_variable_get(:@user).test_flg.should be_true
      controller.instance_variable_get(:@page_no).should == '1'
    end
  end

  describe "update_test_user" do
    before do
      User.update(users(:users_00001).id, :test_flg => true)
    end

    after do
      User.update(users(:users_00001).id, :test_flg => false)
    end

    it "should render update test user add point" do
      user = User.find(1000)
      UserPoint.total_point_for_user(1000).should == 0
      post :update_test_user, :id => '1000', :page => '1', :user_rank => {:name => '2'}, :user_point => {:change_point => '50'}
      user = User.find(1000)
      user.user_rank_id.should == 2
      UserPoint.total_point_for_user(1000).should == 50
      user.test_flg.should be_true
    end

    it "should render update test user point error" do
      user = User.find(1000)
      UserPoint.total_point_for_user(1000).should == 0
      post :update_test_user, :id => '1000', :page => '1', :user_rank => {:name => '2'}, :user_point => {:change_point => '-50'}
      user = User.find(1000)
      user.user_rank_id.should == 1
      UserPoint.total_point_for_user(1000).should == 0
      user.test_flg.should be_true
    end
  end

  describe "delete" do
    it "should delete user" do
      mock_object = User.find('1008')
      User.should_receive(:find).with('1008').and_return(mock_object)
      mock_object.should_receive(:destroy)

      delete :delete, :id => '1008', :page => '1'
      response.should redirect_to(:action => 'index', :page => '1')
    end

    it "テスト会員を全て削除 テスト会員がない場合" do
      lambda {
        delete :delete, :destroy_all_test => true, :page => '1'
      }.should change(User, :count).by(0)
      response.should redirect_to(:action => 'index', :page => '1')
    end

    it "テスト会員を全て削除 テスト会員がある場合" do
      User.update(users(:users_00001).id, :test_flg => true)
      User.update(users(:users_00002).id, :test_flg => true)
      lambda {
        delete :delete, :destroy_all_test => true, :page => '1'
      }.should change(User, :count).by(-2)
      response.should redirect_to(:action => 'index', :page => '1')
    end
  end

  describe "get order_history" do
    it "should render order_history" do
      get :order_history , :id => '1008'
      response.should render_template(:order_history)
      controller.instance_variable_get(:@user).should == User.find('1008')
      controller.instance_variable_get(:@orders).should == []
    end
  end

  describe "get mail_history" do
    it "get mail_history" do
      get :mail_history, :id => '1008'
      response.should render_template(:mail_history)
      controller.instance_variable_get(:@sent_mails).length.should == 1
    end
  end

  describe "get mail_detail" do
    it "get mail_detail" do
      mail = MailDelivery.find(:first)
      get :mail_detail, :mail_id => mail.id
      response.should render_template(:mail_detail)
      controller.instance_variable_get(:@mail_delivery).should == mail
    end
  end

  describe "get init_index" do
    it "init_index" do
      get :init_index
      response.should redirect_to(:action => 'index')
    end
  end

#  describe "get init_output" do
#    it "init_output" do
#      get :init_output
#      response.should redirect_to(:action => 'user_output')
#    end
#  end

  describe "new" do
    it "new with flash" do
      flash[:user] = User.new
      flash[:user_address] = UserAddress.new
      flash[:user_email] = UserEmail.new
      post :new
      response.should be_success
      response.should render_template(:new)
      controller.instance_variable_get(:@user).should == flash[:user] 
      controller.instance_variable_get(:@user_email).should == flash[:user_email]
      controller.instance_variable_get(:@user_address).should == flash[:user_address]
    end

    it "new init" do
      post :new
      response.should be_success
      response.should render_template(:new)
      controller.instance_variable_get(:@user).sex_key.should == :male
      controller.instance_variable_get(:@user).dm_send_div_key.should == :prefer
      controller.instance_variable_get(:@user_email).class.should == UserEmail
      controller.instance_variable_get(:@user_address).country_code.should == 'JP'
      controller.instance_variable_get(:@user).member_category_div_key.should == :general
      controller.instance_variable_get(:@user_email).class.should == UserEmail
      controller.instance_variable_get(:@user_address).country_code.should == 'JP'
    end
  end

  describe "test create" do
    before do
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",  :kana_name_first => "ジロウ",   :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000",  :address_1 => "福岡県",  :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP', :locale => 'ja'}
      @user_email = {:email => "test@test.jp"}
      @user = {:birth_month => "1", :dm_send_div => "0", :job_div => "1", :member_div_key => :membership,
        :birth_day => "23", :sex => "1", :birth_year => "1970", :pc_mail_magazine_send_div_key => :not_prefer,
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A'}
    end

    it "create with get method" do
      get :create
      response.should redirect_to(:action => 'new')
    end

    it "create invalid" do
      post :create
      response.should redirect_to(:action => 'new')
      flash[:user].should_not be_nil
      flash[:user].errors.length.should == 0
      flash[:user_address].should_not be_nil
      flash[:user_address].errors.length.should == 13
      flash[:user_email].should be_nil
    end

    it "create invalid with address" do
      @user_address[:kana_name_last] = nil
      post :create, :user => @user, :user_address => @user_address, :user_email => @user_email
      response.should redirect_to(:action => 'new')

      flash[:user].errors.length.should == 0
      flash[:user_address].errors.length.should == 2
      flash[:user_email].errors.length.should  == 0
    end

    it "WEB会員の場合emailが必須" do
      @user_email[:email] = nil
      post :create, :user => @user, :user_address => @user_address, :user_email => @user_email
      response.should redirect_to(:action => 'new')
      flash[:user].errors.length.should == 1
      flash[:user_address].errors.length.should == 0
      flash[:user_email].should be_nil
    end

    it "WEB会員の場合の場合パスワード、秘密の質問が必須" do
      @user[:password] = nil
      @user[:member_div_key] = :membership
      post :create, :user => @user, :user_address => @user_address, :user_email => @user_email
      response.should redirect_to(:action => 'new')
      flash[:user].errors.length.should == 2
      flash[:user_address].errors.length.should == 0
      flash[:user_email].errors.length.should  == 0
    end

    it "membership create_user should be success" do
      user_count = User.count()
      email_count = UserEmail.count()
      address_count = UserAddress.count()
      
      post :create, :user => @user, :user_address => @user_address, :user_email => @user_email
      User.count().should == user_count + 1
      UserEmail.count().should == email_count + 1
      UserAddress.count().should == address_count + 1
      response.should redirect_to(:action => 'index')
      #response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja')
      user = User.find(:first, :include => :user_email, :conditions => ["user_emails.email = ?", 'test@test.jp'])
      user.test_flg.should be_nil
      MailDelivery.count(:conditions => ["user_id = ? and mail_div = ?", user.id, MailTemplate.mail_type_div_id_by_key(:regist_user)]).should == 1
    end

    it "offline_membership create_user should be success" do
      @user[:member_div_key] = :offline_membership
      @user_email[:email] = nil
      user_count = User.count()
      email_count = UserEmail.count()
      address_count = UserAddress.count()
      
      post :create, :user => @user, :user_address => @user_address, :user_email => @user_email
      User.count().should == user_count + 1
      UserEmail.count().should == email_count
      UserAddress.count().should == address_count + 1
      response.should redirect_to(:action => 'index')
      #response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja')
      user = User.find(controller.instance_variable_get(:@user).id, :include => :user_email)
      user.test_flg.should be_nil
      user.user_email.should be_nil
      MailDelivery.count(:conditions => ["user_id = ? and mail_div = ?", user.id, MailTemplate.mail_type_div_id_by_key(:regist_user)]).should == 0
    end
  end

end
