# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe UserPointController do
  fixtures :users, :user_points, :orders, :point_settings, :accountings

 # include AuthenticatedTestHelper
  before do
    login_as("admin")
  end

  describe "index" do
    it "should get index" do
      get :index
      response.should be_success
      response.should render_template(:index)
      controller.instance_variable_get(:@user_points).should_not == nil
    end

    it "should get index with session" do
      cond = UserPoint::SearchInfo.new
      cond.date_search_div = '0'
      cond.member_name = 'テスト'
      session[:user_point_search_cond] = cond
      get :index
      response.should be_success
      response.should render_template(:index)
    end

    it "should get search with invalid" do
      get :index, :condition => {:member_name => 'てすと'}
      response.should render_template(:index)
      controller.instance_variable_get(:@user_points).length.should == 0
    end

    it "should get search with invalid" do
      get :index, :condition => {:date_from => 'invalid'}
      response.should render_template(:index)
      controller.instance_variable_get(:@user_points).should be_nil
    end
  end

  describe "show" do
    it "get show" do
        get :show, :user_id => '1001'
        response.should render_template(:show)
        user_points = controller.instance_variable_get(:@user_points)
        user_points.each do |user_point|
          user_point.user_id.should == 1001
        end
    end
  end
  describe "edit" do
    it "正常：初期表示の場合" do
      get :edit, :user_id => '1001'
      response.should render_template(:edit)
      user_points = controller.instance_variable_get(:@user_points)
      user_points.each do |user_point|
        user_point.user_id.should == 1001
      end
    end

    it "正常：ポイント編集のバリデーション不正から遷移した場合" do
      flash[:user_points] = "error"
      get :edit, :user_id => '1001'
      user_points = controller.instance_variable_get(:@user_points)
      user_points.should == "error"
    end
    it "正常：新規ポイント追加のバリデーション不正から遷移した場合" do
      user_point = UserPoint.new()
      user_point.given_point = "validate_error"
      flash[:create_user_point] = user_point
      get :edit, :user_id => '1001'
      user_points = controller.instance_variable_get(:@user_points)
      user_points.each do |user_point|
        user_point.user_id.should == 1001
        user_point.inactive_flg == 0
      end
      controller.instance_variable_get(:@create_point).should_not == nil
    end
  end

  describe "update" do
    it "正常：更新" do
     params_user_point = {
                     "1"=>{"remark"=>"その他", "used_point"=>"", "given_point"=>"500"},
                     "2"=>{"remark"=>"その他", "used_point"=>"333", "given_point"=>"1000"},
                     "3"=>{"remark"=>"その他", "used_point"=>"100", "given_point"=>""},
                     "4"=>{"remark"=>"その他", "used_point"=>"333", "given_point"=>"1000"}
                     }
      get :update, :user_id => '1001',:user_points => params_user_point, :page =>1
      response.should redirect_to :action => 'show', :page => 1, :user_id =>1001
      user_point_1 = UserPoint.find(1)
      user_point_1.remark.should == "その他"
      user_point_1.used_point.should == 0
      user_point_1.given_point.should == 500
      user_point_2 = UserPoint.find(2)
      user_point_2.remark.should == "その他"
      user_point_2.used_point.should == 333
      user_point_2.given_point.should == 1000
      user_point_3 = UserPoint.find(3)
      user_point_3.remark.should == "その他"
      user_point_3.used_point.should == 100
      user_point_3.given_point.should == 0
      Accounting.count.should == 0
    end
    
    it "正常：更新　注文ステータスが配送済みの場合は経理テーブルに登録する" do
     params_user_point = {
                     "1"=>{"remark"=>"その他", "used_point"=>"", "given_point"=>"500"},
                     "2"=>{"remark"=>"その他", "used_point"=>"333", "given_point"=>"1000"},
                     "3"=>{"remark"=>"その他", "used_point"=>"", "given_point"=>"500"},
                     "4"=>{"remark"=>"その他", "used_point"=>"333", "given_point"=>"1000"}
                     }
      get :update, :user_id => '1001',:user_points => params_user_point, :page =>1
      response.should redirect_to :action => 'show', :page => 1, :user_id =>1001
      order_shipping = OrderShipping.find(1)
      order_shipping.status_div_key = :delivered
      order_shipping.save!
      Accounting.create_for_order(order_shipping)
      order_shipping = OrderShipping.find(2)
      order_shipping.status_div_key = :delivered
      order_shipping.save!
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all)
      accounting.length.should == 2
      accounting[0].order_id.should_not be_nil
      accounting[1].order_id.should_not be_nil
    end
    
    it "正常：バリデーション不正の場合" do
     params_user_point = {
                     "1"=>{"remark"=>"その他", "used_point"=>"100", "given_point"=>"aaa"},
                     "2"=>{"remark"=>"その他", "used_point"=>"bbb", "given_point"=>"1000"},
                     "3"=>{"remark"=>"その他", "used_point"=>"100", "given_point"=>""},
                     "4"=>{"remark"=>"その他", "used_point"=>"333", "given_point"=>"1000"}
                     }
      get :update, :user_id => '1001',:user_points => params_user_point, :page =>1
      response.should redirect_to :action => 'edit', :page => 1, :user_id =>1001
      flash[:user_points].should_not == nil
      Accounting.count.should == 0
    end
  end

  describe "destroy" do
    it "正常：削除(一覧から遷移の場合)" do
      size = UserPoint.find(:all).size
      get :destroy, :id => '1', :page => 1
      response.should redirect_to :action => 'index', :page => 1
      UserPoint.find(:all).size.should == size -1
      Accounting.count.should == 0
    end
    it "正常：削除(編集から遷移の場合)" do
      size = UserPoint.find(:all).size
      get :destroy, :id => '1', :edit => true, :page => 1, :user_id => 1001
      response.should redirect_to :action => 'edit', :page => 1, :user_id => 1001
      UserPoint.find(:all).size.should == size -1
      Accounting.count.should == 0
    end
  end

  describe "create" do
    it "正常：ポイント新規追加" do
      size = UserPoint.find(:all).size
      params = {:given_point => '500', :remark => ""}
      get :create, :id => '1', :create_point => params, :user_id => 1001, :page => 1
      response.should redirect_to :action => 'edit', :page => 1, :user_id => 1001
      UserPoint.find(:all).size.should == size + 1
      Accounting.count.should == 1
    end
    it "正常：バリデーションエラーの場合" do
      size = UserPoint.find(:all).size
      params = {:given_point => 'aaaaaa', :remark => ""}
      get :create, :id => '1', :create_point => params, :user_id => 1001, :page => 1
      response.should redirect_to :action => 'edit', :page => 1, :user_id => 1001
      UserPoint.find(:all).size.should_not == size + 1
      Accounting.count.should == 0
    end

    it "正常：バリデーションエラーの場合 未入力の場合" do
      size = UserPoint.find(:all).size
      params = {:given_point => nil, :remark => ""}
      get :create, :id => '1', :create_point => params, :user_id => 1001, :page => 1
      response.should redirect_to :action => 'edit', :page => 1, :user_id => 1001
      UserPoint.find(:all).size.should_not == size + 1
      Accounting.count.should == 0
    end
  end
end
