require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe UserRanksController do
  before(:each) do
    login_as("admin")
  end

  def mock_user_rank(stubs={})
    @mock_user_rank ||= mock_model(UserRank, stubs)
  end
  
  describe "GET index" do
    it "assigns all user_ranks as @user_ranks" do
      UserRank.should_receive(:paginate).
        with(:page => 1, :per_page => 20, :order => "code asc, created_at asc").
        and_return([mock_user_rank])
      get :index
      assigns[:user_ranks].should == [mock_user_rank]
    end
  end

  describe "GET show" do
    it "assigns the requested user_rank as @user_rank" do
      UserRank.stub!(:find).with("37").and_return(mock_user_rank)
      get :show, :id => "37"
      assigns[:user_rank].should equal(mock_user_rank)
    end
  end

  describe "GET new" do
    it "assigns a new user_rank as @user_rank" do
      UserRank.stub!(:new).and_return(mock_user_rank)
      get :new
      assigns[:user_rank].should equal(mock_user_rank)
    end
  end

  describe "GET edit" do
    it "assigns the requested user_rank as @user_rank" do
      UserRank.stub!(:find).with("37").and_return(mock_user_rank)
      get :edit, :id => "37"
      assigns[:user_rank].should equal(mock_user_rank)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created user_rank as @user_rank" do
        UserRank.stub!(:new).with({'these' => 'params'}).and_return(mock_user_rank(:save => true))
        post :create, :user_rank => {:these => 'params'}
        assigns[:user_rank].should equal(mock_user_rank)
      end

      it "redirects to the created user_rank" do
        UserRank.stub!(:new).and_return(mock_user_rank(:save => true))
        post :create, :user_rank => {}
        response.should redirect_to(user_rank_url(mock_user_rank))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved user_rank as @user_rank" do
        UserRank.stub!(:new).with({'these' => 'params'}).and_return(mock_user_rank(:save => false))
        post :create, :user_rank => {:these => 'params'}
        assigns[:user_rank].should equal(mock_user_rank)
      end

      it "re-renders the 'new' template" do
        UserRank.stub!(:new).and_return(mock_user_rank(:save => false))
        post :create, :user_rank => {}
        response.should render_template('new')
      end
    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested user_rank" do
        UserRank.should_receive(:find).with("37").and_return(mock_user_rank)
        mock_user_rank.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :user_rank => {:these => 'params'}
      end

      it "assigns the requested user_rank as @user_rank" do
        UserRank.stub!(:find).and_return(mock_user_rank(:update_attributes => true))
        put :update, :id => "1"
        assigns[:user_rank].should equal(mock_user_rank)
      end

      it "redirects to the user_rank" do
        UserRank.stub!(:find).and_return(mock_user_rank(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(user_rank_url(mock_user_rank))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested user_rank" do
        UserRank.should_receive(:find).with("37").and_return(mock_user_rank)
        mock_user_rank.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :user_rank => {:these => 'params'}
      end

      it "assigns the user_rank as @user_rank" do
        UserRank.stub!(:find).and_return(mock_user_rank(:update_attributes => false))
        put :update, :id => "1"
        assigns[:user_rank].should equal(mock_user_rank)
      end

      it "re-renders the 'edit' template" do
        UserRank.stub!(:find).and_return(mock_user_rank(:update_attributes => false))
        put :update, :id => "1"
        response.should render_template('edit')
      end
    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested user_rank" do
      UserRank.should_receive(:find).with("37").and_return(mock_user_rank)
      mock_user_rank.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the user_ranks list" do
      UserRank.stub!(:find).and_return(mock_user_rank(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(user_ranks_url)
    end
  end

end
