# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  # fixtures :products, :product_displays, :name_masters, :ms_code_names, :ms_shop_settings, :users

  def ec
    helper.ec
  end


  describe '<ec:each values="1,2,3..." key="name">～</ec:each>' do
    it "required values are input" do
      dest = []
      ec.each(:values => "a,b,c,d,e", :key => "num") do
        dest << helper.instance_variable_get(:@num)
      end
      dest.should == ['a', 'b', 'c', 'd', 'e']
    end
    
    it "required arguments values and key aren't set" do
      dest = []
      lambda{ec.each{ dest << helper.instance_variable_get(:@num) } }.should raise_error(
        EcRider::EcHelper::ArgumentError)
    end

    it "required argument values aren't set" do
      dest = []
      lambda{ec.each(:key => "num"){ dest << helper.instance_variable_get(:@num) } }.should raise_error(
        EcRider::EcHelper::ArgumentError)
    end
  end

  describe '<ec:if_equal value="3" key="index">～</ec:if_equal>' do
    it "@num=value" do 
      helper.instance_variable_set(:@num, 1)
      ec.if_equal(:key => "num", :value => "3"){ flunk }
      helper.instance_variable_set(:@num, 3)
      invoked = false
      ec.if_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
    end
  end

  describe "ec:if_greater_equal" do
    it '<ec:if_greater_equal value="3" key="index">～</ec:if_greater_equal>' do
      # @num>=valueの場合
      helper.instance_variable_set(:@num, 2)
      invoked = false
      ec.if_greater_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_false
      helper.instance_variable_set(:@num, 3)
      invoked = false
      ec.if_greater_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
      helper.instance_variable_set(:@num, 4)
      invoked = false
      ec.if_greater_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
    end
  end

  describe "ec:if_greater_than" do
    it '<ec:if_greater_than value="3" key="index">～</ec:if_greater_than> ' do
      # @num>valueの場合
      helper.instance_variable_set(:@num, 2)
      invoked = false
      ec.if_greater_than(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_false
      helper.instance_variable_set(:@num, 3)
      invoked = false
      ec.if_greater_than(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_false
      helper.instance_variable_set(:@num, 4)
      invoked = false
      ec.if_greater_than(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
    end
  end

  describe "ec:if_greater_equal" do
    it '<ec:if_less_equal value="3" key="index">～</ec:if_less_equal>' do
      # @num<=valueの場合
      helper.instance_variable_set(:@num, 2)
      invoked = false
      ec.if_less_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
      helper.instance_variable_set(:@num, 3)
      invoked = false
      ec.if_less_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
      helper.instance_variable_set(:@num, 4)
      invoked = false
      ec.if_less_equal(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_false
    end
  end

  describe "ec:if_greater_than" do
    it '<ec:if_less_than value="3" key="index">～</ec:if_less_than> ' do
      # @num<valueの場合
      helper.instance_variable_set(:@num, 2)
      invoked = false
      ec.if_less_than(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_true
      helper.instance_variable_set(:@num, 3)
      invoked = false
      ec.if_less_than(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_false
      helper.instance_variable_set(:@num, 4)
      invoked = false
      ec.if_less_than(:key => "num", :value => "3"){ invoked = true }
      invoked.should be_false
    end
  end


  # 何でも記述できるec:anyタグと
  # ec:anyタグに属性を追加できるattrタグ
  describe "ec:any and ec:attr" do
    before(:each) do
      @template.output_buffer = ''
    end
    
    describe "should output any tags with tag_name" do
      it "br" do
        ec.any(:tag_name => 'br')
        @template.output_buffer.should == "<br />"
      end

      it "input" do
        ec.any(:tag_name => 'input', :type => 'text', :name => 'address')
        @template.output_buffer.should(
          have_tag('input[type="text"][name="address"]'))
      end

      it "div" do
        ec.any(:tag_name => 'div', :class => 'contents'){ "Hello" }
        @template.output_buffer.should have_tag('div[class="contents"]') do
          include_text("Hello")
        end
      end
    end
    
    it "should raise ArgumentError without tag_name" do
      lambda{ ec.any }.should raise_error(EcRider::EcHelper::ArgumentError)
    end
    
    it "should raise ArgumentError without tag_name but with ec:attr of tag_name" do
      ec.any(:class => "greeting"){ ec.attr(:name => 'tag_name'){ "p" }; "ハロー" }
      @template.output_buffer.should have_tag('p[class="greeting"]') do
        include_text("ハロー")
      end
    end
    
    it "should raise ArgumenrError without ec:any tag" do
      lambda{ ec.attr }.should raise_error(EcRider::EcHelper::ArgumentError)
    end
    
    it "should append extra class" do
      lambda{ ec.attr }.should raise_error(EcRider::EcHelper::ArgumentError)
    end
    
    it "example in desc #1" do
      ec.stub!(:stock_count).and_return(3)
      ec.any(:tag_name => "div") do
        ec.attr(:name => "class") do 
          @template.concat("product_count_")
          @template.concat(ec.stock_count.to_s)
        end
        @template.concat("商品A")
      end
      @template.output_buffer.should have_tag('div[class="product_count_3"]') do
        include_text("商品A")
      end
    end
  end

  describe "ec:cycle" do
    before(:each) do
      @template.output_buffer = ''
    end
    
    it "even or odd class" do
      7.times do
        @template.concat("<p>")
        ec.cycle do
          @template.concat("even,odd")
        end
        @template.concat("</p>")
      end
      @template.output_buffer.should == 
        "<p>even</p><p>odd</p><p>even</p><p>odd</p><p>even</p><p>odd</p><p>even</p>"
    end

  end

  describe "ec:if" do
    before(:each) do
      @template.output_buffer = ''
    end

    describe "evaluate any Ruby script" do

      it "test will be false" do
        Date.stub!(:today).and_return(Date.new(2009, 5, 31))
        ec.if(:test => "Date.today >= Date.new(2009, 6, 1)") do
          @template.concat("Opened!")
        end
        @template.output_buffer.should == ''
      end

      it "test will be true" do
        Date.stub!(:today).and_return(Date.new(2009, 6, 1))
        ec.if(:test => "Date.today >= Date.new(2009, 6, 1)") do
          @template.concat("Opened!")
        end
        @template.output_buffer.should == 'Opened!'
      end

      it "index != 1" do
        ec.instance_variable_set(:@order_index, 2)
        ec.if(:test => "@order_index == 1") do
          @template.concat("index is 1")
        end
        @template.output_buffer.should == ""
      end

      it "index == 1" do
        ec.instance_variable_set(:@order_index, 1)
        ec.if(:test => "@order_index == 1") do
          @template.concat("index is 1")
        end
        @template.output_buffer.should == "index is 1"
      end
    end
    
  end


  describe "_each_with_index" do
    it "with enumerable object name" do
      ec.instance_variable_set(:@product_names, ['A', 'B', 'C'])
      # private メソッドなので、普通には呼び出せないことを確認
      lambda{ ec._each_with_index }.should raise_error(NoMethodError)
      instance_variables = []
      ec.send(:_each_with_index, :@product_names) do
        instance_variables << {
          :item_obj => ec.instance_variable_get(:@product_name),
          :item_index => ec.instance_variable_get(:@product_name_index)
        }
      end
      instance_variables.should == [
        {:item_index => 0, :item_obj => 'A'},
        {:item_index => 1, :item_obj => 'B'},
        {:item_index => 2, :item_obj => 'C'}
      ]
    end

    it "with enumerable object name and item object name" do
      ec.instance_variable_set(:@product_names, ['A', 'B', 'C'])
      # private メソッドなので、普通には呼び出せないことを確認
      lambda{ ec._each_with_index }.should raise_error(NoMethodError)
      instance_variables = []
      ec.send(:_each_with_index, :@product_names, :@product) do
        instance_variables << {
          :item_obj => ec.instance_variable_get(:@product),
          :item_index => ec.instance_variable_get(:@product_index)
        }
      end
      instance_variables.should == [
        {:item_index => 0, :item_obj => 'A'},
        {:item_index => 1, :item_obj => 'B'},
        {:item_index => 2, :item_obj => 'C'}
      ]
    end

    it "with enumerable object name and item object name and index_name" do
      ec.instance_variable_set(:@product_names, ['A', 'B', 'C'])
      # private メソッドなので、普通には呼び出せないことを確認
      lambda{ ec._each_with_index }.should raise_error(NoMethodError)
      instance_variables = []
      ec.send(:_each_with_index, :@product_names, :@product, :@product_list_index) do
        instance_variables << {
          :item_obj => ec.instance_variable_get(:@product),
          :item_index => ec.instance_variable_get(:@product_list_index)
        }
      end
      instance_variables.should == [
        {:item_index => 0, :item_obj => 'A'},
        {:item_index => 1, :item_obj => 'B'},
        {:item_index => 2, :item_obj => 'C'}
      ]
    end
  end

  describe "weekday" do
    it "weekday" do
      Time.stub!(:now).and_return(Time.local(2008,8,17))
      ec.weekday.should == "Sunday"
      Time.stub!(:now).and_return(Time.local(2008,8,18))
      ec.weekday.should == "Monday"
      Time.stub!(:now).and_return(Time.local(2008,8,19))
      ec.weekday.should == "Tuesday"
      Time.stub!(:now).and_return(Time.local(2008,8,20))
      ec.weekday.should == "Wednesday"
      Time.stub!(:now).and_return(Time.local(2008,8,21))
      ec.weekday.should == "Thursday"
      Time.stub!(:now).and_return(Time.local(2008,8,22))
      ec.weekday.should == "Friday"
      Time.stub!(:now).and_return(Time.local(2008,8,23))
      ec.weekday.should == "Saturday"
    end

    it "weekday format" do
      Time.stub!(:now).and_return(Time.local(2008,8,17))
      ec.weekday(:format => "ja").should == "日"
      Time.stub!(:now).and_return(Time.local(2008,8,18))
      ec.weekday(:format => "ja").should == "月"
      Time.stub!(:now).and_return(Time.local(2008,8,19))
      ec.weekday(:format => "ja").should == "火"
      Time.stub!(:now).and_return(Time.local(2008,8,20))
      ec.weekday(:format => "ja").should == "水"
      Time.stub!(:now).and_return(Time.local(2008,8,21))
      ec.weekday(:format => "ja").should == "木"
      Time.stub!(:now).and_return(Time.local(2008,8,22))
      ec.weekday(:format => "ja").should == "金"
      Time.stub!(:now).and_return(Time.local(2008,8,23))
      ec.weekday(:format => "ja").should == "土"
    end
  end




end
