# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
    fixtures :products, :product_images,
      :product_variations, :product_variation_items, :product_categories,
      :product_skus, :product_displays, :product_sku_inventories, :product_sku_secure_inventories

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @template.extend(ApplicationHelper)
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe '<%= ec:cart_link %>' do
    it '<%= ec:cart_link %>' do
      ec.cart_link(:src => "aaa.gif").should have_tag('a') do
        with_tag('img[src^=/images/aaa.gif]')
      end

      ec.cart_link(:text => "リンク").should have_tag('a')
      ec.cart_link(:text => "リンク").should_not have_tag('img')

      ec.cart_link.should have_tag('a')
      ec.cart_link.should_not have_tag('img')
    end
  end

  describe '<%= ec:cart_in_button %>' do
    fixtures :kbn_code_names, :kbn_codes, :product_skus, :product_sku_inventories, :product_sku_secure_inventories, :product_components
    
    it '選択肢を指定する。在庫数が設定されている場合。' do
      product = Product.find(100001)
      product_sku = product.primary_sku
      product_sku.sku_code.should == "0000000000001-00-00-00"
      product_sku.orderable_count.should == 24
      product_sku.order_limit.should == 99
      
      ec.cart_in_button(:product_code => "0000000000001", :values => "1,2,3").should have_tag('form[method="get"][name="form_0000000000001_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000001_00_00_00"][name="amount_0000000000001_00_00_00"]') do
          with_tag('option', :count => 3)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
          with_tag('option[value="3"]', "3")
        end
        with_tag('input[type="button"][value="カートに入れる"][class="button_cart1"][onclick^="cart_on_click"]')
      end
      
      ec.instance_variable_set(:@exist_cart_on_click_function, nil)
      cart_in_button_result = ec.cart_in_button(:product_code => "0000000000001-00-00-00", :values => "1,2,3")
      cart_in_button_result.should have_tag('script[type="text/javascript"]')
      script = cart_in_button_result.scan(
        Regexp.new(
          Regexp.escape("<script type='text/javascript'>") <<
          "(.*)" <<
          Regexp.escape("</script>"), Regexp::MULTILINE)
        ).flatten.join
      script_lines = script.split(/$/).map{|line| line.strip}.select{|line| !line.empty?}
      expected = <<-EOS
        function cart_on_click(sku_code, sku_code_for_name) {
          location.href = '/cart/index?sku_code=' + sku_code + '&qty=' + document.getElementById('amount_' + sku_code_for_name).value;
        }
      EOS
      expected_lines = expected.split(/$/).map{|line| line.strip}.select{|line| !line.empty?}
      script_lines.should == expected_lines
    end
    
    it '選択肢を指定する。在庫数が設定されている場合。（選択肢最大値<注文可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000006-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == 50
      
      ec.cart_in_button(:product_code => "0000000000006-00-00-00", :values => "1,2,3,4").should have_tag('form[method="get"][name="form_0000000000006_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000006_00_00_00"][name="amount_0000000000006_00_00_00"]') do
          with_tag('option', :count => 4)
          with_tag('option', :maximum  => 4)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
      end
    end
    
    it '選択肢を指定する。在庫数が設定されている場合。（選択肢最大値=注文可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000006-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == 50
      
      ec.cart_in_button(:product_code => "0000000000006-00-00-00", :values => "1,3,5").should have_tag('form[method="get"][name="form_0000000000006_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000006_00_00_00"][name="amount_0000000000006_00_00_00"]') do
          with_tag('option', :count => 3)
          with_tag('option', :maximum  => 5)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="3"]', "3")
          with_tag('option[value="5"]', "5")
        end
      end
    end
    
    it '選択肢を指定する。在庫数が設定されている場合。（選択肢最大値>注文可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000006-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == 50
      
      ec.cart_in_button(:product_code => "0000000000006-00-00-00", :values => "1,3,6").should have_tag('form[method="get"][name="form_0000000000006_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000006_00_00_00"][name="amount_0000000000006_00_00_00"]') do
          with_tag('option', :count => 2)
          with_tag('option', :maximum  => 3)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="3"]', "3")
        end
      end
    end
    
    it '選択肢を指定する。上限数が設定されている場合。（選択肢最大値<上限数）' do
      product_sku = ProductSku.find_by_sku_code("0000000100003-00-00-00")
      product_sku.orderable_count.should == 110
      product_sku.order_limit.should == 20
      
      ec.cart_in_button(:product_code => "0000000100003-00-00-00", :values => "10,19").should have_tag('form[method="get"][name="form_0000000100003_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000100003_00_00_00"][name="amount_0000000100003_00_00_00"]') do
          with_tag('option', :count => 2)
          with_tag('option', :maximum  => 19)
          with_tag('option[selected="selected"][value="10"]', "10")
          with_tag('option[value="19"]', "19")
        end
      end
    end
    
    it '選択肢を指定する。上限数が設定されている場合。（選択肢最大値=上限数）' do
      product_sku = ProductSku.find_by_sku_code("0000000100003-00-00-00")
      product_sku.orderable_count.should == 110
      product_sku.order_limit.should == 20
      
      ec.cart_in_button(:product_code => "0000000100003-00-00-00", :values => "10,20").should have_tag('form[method="get"][name="form_0000000100003_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000100003_00_00_00"][name="amount_0000000100003_00_00_00"]') do
          with_tag('option', :count => 2)
          with_tag('option', :maximum  => 20)
          with_tag('option[selected="selected"][value="10"]', "10")
          with_tag('option[value="20"]', "20")
        end
      end
    end
    
    it '選択肢を指定する。上限数が設定されている場合。（選択肢最大値>上限数）' do
      product_sku = ProductSku.find_by_sku_code("0000000100003-00-00-00")
      product_sku.orderable_count.should == 110
      product_sku.order_limit.should == 20
      
      ec.cart_in_button(:product_code => "0000000100003-00-00-00", :values => "10,21").should have_tag('form[method="get"][name="form_0000000100003_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000100003_00_00_00"][name="amount_0000000100003_00_00_00"]') do
          with_tag('option', :count => 1)
          with_tag('option', :maximum  => 10)
          with_tag('option[selected="selected"][value="10"]', "10")
        end
      end
    end
    
    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されていない場合。' do
      ProductSkuInventory.destroy_all
      ProductSku.update_all(:parent_stock_count => 0, :composable_stock_count => 0, :orderable_count => 0)
      product_sku = ProductSku.find_by_sku_code("0000000000003-00-00-00")
      product_sku.orderable_count.should == 0
      product_sku.order_limit.should == 20
      
      @template.instance_variable_set(:@product_sku_0000000000003_00_00_00, nil)
      ec.cart_in_button(:product_code => "0000000000003-00-00-00").should == "この商品は売り切れました"
    end
    
    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されている場合。（上限数<販売可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000007-00-00-00")
      product_sku.orderable_count.should == 51
      product_sku.order_limit.should == 50
      
      ec.cart_in_button(:product_code => "0000000000007-00-00-00").should have_tag('form[method="get"][name="form_0000000000007_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000007_00_00_00"][name="amount_0000000000007_00_00_00"]') do
          with_tag('option', :count => 50)
          with_tag('option', :maximum  => 50)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
      end
    end
    
    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されている場合。（上限数=在庫数>販売可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000008-00-00-00")
      product_sku.orderable_count.should == 49
      product_sku.stock_count.should == 50
      product_sku.order_limit.should == 50
      
      ec.cart_in_button(:product_code => "0000000000008-00-00-00").should have_tag('form[method="get"][name="form_0000000000008_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000008_00_00_00"][name="amount_0000000000008_00_00_00"]') do
          with_tag('option', :count => 49)
          with_tag('option', :maximum  => 49)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
      end
    end
    
    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されている場合。（上限数>販売可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000009-00-00-00")
      product_sku.orderable_count.should == 49
      product_sku.order_limit.should == 50
      
      ec.cart_in_button(:product_code => "0000000000009-00-00-00").should have_tag('form[method="get"][name="form_0000000000009_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000009_00_00_00"][name="amount_0000000000009_00_00_00"]') do
          with_tag('option', :count => 49)
          with_tag('option', :maximum  => 49)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
      end
    end
    
    it '選択肢を指定しない。上限数が設定されていなくて、在庫数が設定されていない場合。' do
      product_sku = ProductSku.find_by_sku_code("0000000000004-00-00-00")
      product_sku.orderable_count.should == 0
      product_sku.order_limit.should == nil
      
      ec.cart_in_button(:product_code => "0000000000004-00-00-00").should == "この商品は売り切れました"
    end
    
    it '選択肢を指定しない。上限数が設定されていなくて、在庫数が設定されている場合。' do
      product_sku = ProductSku.find_by_sku_code("0000000000010-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == nil
      
      ec.cart_in_button(:product_code => "0000000000010-00-00-00").should have_tag('form[method="get"][name="form_0000000000010_00_00_00"][action=""]') do
        with_tag('select[id="amount_0000000000010_00_00_00"][name="amount_0000000000010_00_00_00"]') do
          with_tag('option', :count => 5)
          with_tag('option', :maximum  => 5)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
      end
    end
    
    it 'ボタンのクラスを指定する。' do
      ec.cart_in_button(:product_code => "0000000000001-00-00-00", :class => "aaa").should have_tag('form[method="get"][name="form_0000000000001_00_00_00"][action=""]') do
        with_tag('input[type="button"][value="カートに入れる"][class="aaa"][onclick^="cart_on_click"]')
      end
    end
    
    it '在庫切れ時の表示 ja' do
      ec.cart_in_button(:product_code => "0000000000004-00-00-00").should == "この商品は売り切れました"
      ec.cart_in_button(:product_code => "0000000000052-00-00-00").should == "近日入荷予定"
    end
    
    it '在庫切れ時の表示 en' do
      I18n.locale = 'en'
      ec.cart_in_button(:product_code => "0000000000004-00-00-00").should == "This product is sold out"
      ec.cart_in_button(:product_code => "0000000000052-00-00-00").should == "Comming soon"
    end
    
    it 'カートに入れるボタンが呼び出すファンクションは、ボタンの個数に関わらず一度だけ表示される' do
      ec.instance_variable_set(:@exist_cart_on_click_function, nil)
      ec.cart_in_button(:product_code => "0000000000001-00-00-00").should have_tag('script[type="text/javascript"]')
      ec.cart_in_button(:product_code => "0000000000001-00-00-00").should_not have_tag('script[type="text/javascript"]')
      ec.cart_in_button(:product_code => "0000000000001-00-00-00").should_not have_tag('script[type="text/javascript"]')
    end
  end


  describe '<ec:cart_continue_shopping_button>' do
    it "options null" do
      button = ec.cart_continue_shopping_button()
      button.should have_tag('input') do
        with_tag("input[type=?][class=?][value=?]", 'button',' button_gray', '←お買い物を続ける')
      end
    end

    it "options not null" do
      button = ec.cart_continue_shopping_button(:id => 'test_id', :class => 'test_class')
      button.should have_tag('input') do
        with_tag("input[type=?][id=?][class=?][value=?]", 'button','test_id','test_class button_gray', '←お買い物を続ける')
      end
    end
  end

  describe '<ec:cart_error_messages>' do
    it "error_messages null" do
      result = ec.cart_error_messages
      result.should == nil
    end
    it "error_messages with stock_error" do
      @template.instance_variable_set(:@stock_error, 'test stock_error message')
      result = ec.cart_error_messages
      result.should == '<div class="alert_msg"><ul><li>test stock_error message</li></ul></div>'
    end
  end


  describe '<ec:if_empty_cart>～</ec:if_empty_cart>' do
    it "products null" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:item_list).and_return([])
      invoked = false
      ec.if_empty_cart{
        invoked = true
        @template.output_buffer
      }
      invoked.should be_true
    end

    it "products not null" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:item_list).and_return(['not null'])
      invoked = false
      ec.if_empty_cart{
        invoked = true
        @template.output_buffer
      }
      invoked.should be_false
    end
  end

  describe '<ec:if_not_empty_cart>～</ec:if_not_empty_cart>' do
    it "products null" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:item_list).and_return([])
      invoked = false
      ec.if_not_empty_cart{
        invoked = true
        @template.output_buffer
      }
      invoked.should be_false
    end

    it "products not null" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:item_list).and_return(['not null'])
      invoked = false
      ec.if_not_empty_cart{
        invoked = true
        @template.output_buffer
      }
      invoked.should be_true
    end
  end

  describe '<ec:cart_list_include>' do
    it "should render :partial => 'cart/cart_list'" do
      session[:cart] = mock(:cart)
      @template.should_receive(:render).with(:partial => "cart/cart_list")
      ec.cart_list_include
    end
  end

  describe "ec:cart_item_list" do
    before(:each) do
      product_sku = ProductSku.find_by_product_id(1)
      product_sku.stub(:price).and_return('8190')
      @cart_item1 = Cart::Item.new(:key => '1', :product_sku => product_sku, :course_div_key => :normal, :qty => 2)
      @cart_item1.key.should == '1'
      product_sku2 = ProductSku.find_by_product_id(2)
      product_sku2.stub(:price).and_return('12600')
      @cart_item2 = Cart::Item.new(:key => '2', :product_sku => product_sku2, :course_div_key => :normal, :qty => 1)
      @cart_item2.discount_amount = 1000
    end

    it "normal usage" do
      cart_list = [@cart_item1, @cart_item2]
      # @cart_item1.key = '2'

      methods = %w(cart_item_image cart_item_product_name cart_item_product_sku_unit_price) +
        %w(cart_item_unit_price cart_item_qty_select cart_item_delete_button) +
        %w(cart_item_subtotal_amount cart_item_discount_amount cart_item_total_amount)
      actuals = {}
      
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
      ec.cart_item_list do
        methods.each do |method|
          actuals[method] ||= []
          actuals[method] << ec.send(method)
        end
        ""
      end
      actuals = actuals.symbolize_keys
      actuals.values.each do |actual_array|
        actual_array.length.should == 2
      end
      actuals[:cart_item_product_name][0].should == '錠剤100粒'
      actuals[:cart_item_product_name][1].should == '栄養ドリンク10箱'
      actuals[:cart_item_product_sku_unit_price][0].should == '8,190'
      actuals[:cart_item_product_sku_unit_price][1].should == '12,600'
      actuals[:cart_item_unit_price][0].should == '8,190'
      actuals[:cart_item_unit_price][1].should == '12,600'
      # actuals[:cart_item_qty][0].should == '2'
      # actuals[:cart_item_qty][1].should == '1'
      actuals[:cart_item_qty_select][0].should have_tag('select[id=product_row_1_qty]')
      actuals[:cart_item_qty_select][1].should have_tag('select[id=product_row_2_qty]')
      actuals[:cart_item_delete_button][0].should have_tag('a')
      actuals[:cart_item_delete_button][1].should have_tag('a')
      actuals[:cart_item_subtotal_amount][0].should == '16,380'
      actuals[:cart_item_subtotal_amount][1].should == '12,600'
      actuals[:cart_item_discount_amount][0].should == ""
      actuals[:cart_item_discount_amount][1].should == '1,000'
      actuals[:cart_item_total_amount][0].should == '16,380'
      actuals[:cart_item_total_amount][1].should == '11,600'
    end

    describe "cart_item_qty_select" do
      it "if course_div_param = 2" do
        @cart_item1.key = '2'
        # @cart_item1.product_sku.product.course_div_key = :nomal

        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_qty_select)
          concat(ec.cart_item_delete_button)
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('select') do
          with_tag("select[id=?]", 'product_row_2_qty')
        end
        @template.output_buffer.should have_tag('option') do
          (1..10).each do |i|
            with_tag("option[value=?]", i.to_s)
          end
          with_tag("option[value=2][selected=?]", 'selected')
        end
      end

      it "if course_div_param = 5" do
        @cart_item1.key = '2'
        @cart_item1.course_div = 5
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_qty_select)
          concat(ec.cart_item_delete_button)
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('select') do
          with_tag("select[id=?]", 'product_row_2_qty')
        end
      end

      it "format" do
        @cart_item1.key = '2'
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_qty_select(:format => '円'))
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('select') do
          with_tag("select[id=?]", 'product_row_2_qty')
        end
      end
    end
    
    describe "cart_item_delete_button" do
      it "format" do
        @cart_item1.key = '2'
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_delete_button(:delete_button_name => '削除'))
          @template.output_buffer
        end
      end
    end
  end

  describe "cart_item attribute" do
    before(:each) do
      @cart_item = mock(:cart_item)
      ec.instance_variable_set(:@cart_item, @cart_item)
    end
    
    describe "<ec:cart_item_image>" do
      before do
        I18n.locale = 'ja'
        cart = Cart.new()
        cart.add(ProductSku.find_by_id(282))
        session[:cart] = cart
        ec.instance_variable_set(:@cart_item, cart.items["0000000009020-00-00-00_1"])
      end
      it "usual" do
        ec.cart_item_list do
          ec.cart_item_image.should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001.gif][alt="商品1の商品名1"]')
          ""
        end
      end
      it "medium" do
        ec.cart_item_list do
          ec.cart_item_image(:type => 'medium').should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_medium.gif][alt="商品1の商品名1"]')
          ""
        end
      end
      it "large" do
        ec.cart_item_list do
          ec.cart_item_image(:type => 'large').should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_large.gif][alt="商品1の商品名1"]')
          ""
        end
      end
      it "small" do
        ec.cart_item_list do
          ec.cart_item_image(:type => 'small').should have_tag('img[src=?][alt=?]', "/ec_rider/product_images/0000/0001/product00001_small.gif", "商品1の商品名1")
          ""
        end
      end
      it "Macではgifのファイルの拡張子がなぜかpngになる？ attachment_fuの:processorで:rmagickを指定していなかった" do
        @template.path_to_image("/ec_rider/product_images/0000/0001/product00001_small.gif").should == 
          "/ec_rider/product_images/0000/0001/product00001_small.gif"
      end

      it "no_image" do
        cart = Cart.new()
        cart.add(ProductSku.find_by_id(265))
        session[:cart] = cart
        ec.cart_item_list do
          ec.cart_item_image(:type => 'small').should == nil
          ""
        end
      end
    end

    describe "<ec:cart_item_product_name>" do
      it "usual" do
        @cart_item.should_receive(:name).and_return("モック商品名1")
        ec.cart_item_product_name.should == 'モック商品名1'
      end
    end

    describe "<ec:cart_item_product_sku_unit_price>" do
      it "usual" do
        mock_sku = mock(:product_sku)
        mock_sku.should_receive(:price).and_return(1000)
        @cart_item.should_receive(:product_sku).and_return(mock_sku)
        ec.cart_item_product_sku_unit_price.should == '1,000'
      end
    end

    describe "<ec:cart_item_unit_price>" do
      it "usual" do
        @cart_item.should_receive(:unit_price).and_return(2000)
        ec.cart_item_unit_price.should == '2,000'
      end
    end

    describe "<ec:cart_item_subtotal_amount>" do
      it "usual" do
        @cart_item.should_receive(:subtotal_amount).and_return(3000)
        ec.cart_item_subtotal_amount(:post_currency => '円').should == '3,000円'
      end
    end

    describe "<ec:cart_item_discount_amount>" do
      it "usual" do
        @cart_item.should_receive(:discount_amount).and_return(4000)
        ec.cart_item_discount_amount.should == '4,000'
      end
      
      it "zero" do
        @cart_item.should_receive(:discount_amount).and_return(0)
        ec.cart_item_discount_amount.should == ""
      end
    end

    describe "<ec:cart_item_total_amount>" do
      it "usual" do
        @cart_item.should_receive(:total_amount).and_return(5000)
        ec.cart_item_total_amount.should == '5,000'
      end
    end
  end


  describe '<ec:cart_sum_total_amount_include>' do
    it "render :partial => 'cart/cart_total_prices'" do
      session[:cart] = mock(:cart)
      @template.should_receive(:render).with(:partial => "cart/cart_total_prices")
      ec.cart_sum_total_amount_include
    end
  end


  describe '<ec:cart_sum_total_qty>' do
    it "qty" do
      # @template.instance_variable_set(:@normal_deli_qty, '2')
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:total_qty).and_return(2)
      ec.cart_sum_total_qty.should == "<span id=\"normal_deli_qty\">2</span>"
    end
  end

  describe '<ec:cart_sum_subtotal_amount>' do
    it "qty" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:subtotal_amount).and_return(5000)
      ec.cart_sum_subtotal_amount(:post_currency=> "円").should == "<span id=\"cart_subtotal_amount\">5,000</span>円"
    end
  end

  describe '<ec:cart_sum_discount_amount>' do
    it "qty" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:discount_amount).and_return(3000)
      ec.cart_sum_discount_amount.should == "<span id=\"cart_discount_amount\">3,000</span>"
    end
  end

  describe '<ec:cart_sum_total_amount>' do
    it "qty" do
      session[:cart] = mock(:cart)
      session[:cart].should_receive(:total_amount).and_return(2000)
      ec.cart_sum_total_amount(:pre_currency=> '￥').should == "￥<span id=\"total_amount\">2,000</span>"
    end
  end

  describe "ec:cart_login_form" do
    it "normal usage" do
      @template.instance_variable_set(:@user_login_form, UserLoginForm.new)
      ec.cart_login_form do
        concat(ec.cart_login_error_messages || '')
        concat(ec.user_login_form_email)
        concat(ec.user_login_form_password)
        concat(ec.user_login_form_remember_me)
        ec.submit{"ログインする"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/authenticate?completion_url%5Baction%5D=index&amp;completion_url%5Bcontroller%5D=cart") do
        with_tag("input#user_login_form_email")
        with_tag("input#user_login_form_password")
        with_tag("input#remember_me[type=?]", 'checkbox')
        with_tag("input[type=?][value=?]", 'submit', 'ログインする')
      end
    end
  end


  describe '<ec:cart_link_to_remider>' do
    before do
      @template.stub!(:send).with(:params).and_return({:controller => 'cart', :action => 'index'})
    end
    it "options null" do
      ec.cart_link_to_reminder{'test'}
      @template.output_buffer.should == "<a href=\"/reminder?back_to_url%5Baction%5D=index&amp;back_to_url%5Bcontroller%5D=cart\&amp;completion_url%5Baction%5D=index&amp;completion_url%5Bcontroller%5D=cart\">test</a>"
    end

    it "options not null" do
      ec.cart_link_to_reminder(:id => 'test_id', :class => 'test_class'){'test'}
      @template.output_buffer.should == "<a href=\"/reminder?back_to_url%5Baction%5D=index&amp;back_to_url%5Bcontroller%5D=cart\&amp;completion_url%5Baction%5D=index&amp;completion_url%5Bcontroller%5D=cart\" class=\"test_class\" id=\"test_id\">test</a>"
    end

    it "should work with text" do
      link = ec.cart_link_to_reminder(:text => "リンク")
      link.should have_tag('a')
      link.should include("リンク")
    end

    it "should work with block" do
      ec.cart_link_to_reminder(:text => "textは表示されない"){"使われる文字列"}
      link = @template.output_buffer
      link.should have_tag('a')
      link.should include("使われる文字列")
      link.should_not include("textは表示されない")
    end
  end

end
