# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  include EcRider::PluginModule::RSpecExt

   fixtures :products, :product_displays#, :name_masters, :ms_code_names, :ms_shop_settings, :users

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @template.extend(ApplicationHelper)
  end

  describe "<ec:stylesheet/>" do
    it "with href" do
      ec.stylesheet(:href => "xec.css").should == '<link href="/stylesheets/xec.css" media="screen" rel="stylesheet" type="text/css" />'
      ec.stylesheet(:href => "http://www.railsapplication.com/style.css").should == '<link href="http://www.railsapplication.com/style.css" media="screen" rel="stylesheet" type="text/css" />'
      ec.stylesheet(:href => "xec.css", :media => "print").should == '<link href="/stylesheets/xec.css" media="print" rel="stylesheet" type="text/css" />'
    end

    it "with src" do
      ec.stylesheet(:src => "xec.css").should == '<link href="/stylesheets/xec.css" media="screen" rel="stylesheet" type="text/css" />'
      ec.stylesheet(:src => "http://www.railsapplication.com/style.css").should == '<link href="http://www.railsapplication.com/style.css" media="screen" rel="stylesheet" type="text/css" />'
      ec.stylesheet(:src => "xec.css", :media => "print").should == '<link href="/stylesheets/xec.css" media="print" rel="stylesheet" type="text/css" />'
    end

    it "with href and src" do
      ec.stylesheet(:href => "xec.css", :src => "xec1.css", :media => "print").should == '<link href="/stylesheets/xec.css" media="print" rel="stylesheet" type="text/css" />'
    end
  end

  describe "<ec:javascript/>" do
    it "with src" do
      ec.javascript(:src => "xec.js").should == '<script src="/javascripts/xec.js" type="text/javascript"></script>'
      ec.javascript(:src => "http://www.railsapplication.com/xmlhr.js").should == '<script src="http://www.railsapplication.com/xmlhr.js" type="text/javascript"></script>'
    end

    it "with href" do
      ec.javascript(:href => "xec.js").should == '<script src="/javascripts/xec.js" type="text/javascript"></script>'
      ec.javascript(:href => "http://www.railsapplication.com/xmlhr.js").should == '<script src="http://www.railsapplication.com/xmlhr.js" type="text/javascript"></script>'
    end

    it "with src and href" do
      ec.javascript(:src => "xec.js", :href => "xec1.js").should == '<script src="/javascripts/xec.js" type="text/javascript"></script>'
      ec.javascript(:src => "http://www.railsapplication.com/xmlhr.js", :href => "http://www.railsapplication.com/xmlhr1.js").should == '<script src="http://www.railsapplication.com/xmlhr.js" type="text/javascript"></script>'
    end
  end

  describe "<ec:meta/>" do
    it "<ec:meta/>" do
      ec.meta(:name => 'description', :content => 'ホームページの紹介').should have_tag('meta[name="description"][content="ホームページの紹介"]')
    end
  end

  describe "<ec:keyword/>" do
    it "<ec:keyword/>" do
      ec.keyword(:content => 'キーワード１,キーワード２').should have_tag('meta[name="keywords"][content="キーワード１,キーワード２"]')
    end
  end

  describe "<ec:today/>" do
    it "<ec:today/>" do
      ec.today.should == Time.now.strftime('%y/%m/%d')
    end
  end

  describe "<ec:today_jpn/>" do
    it "<ec:today_jpn/>" do
      ec.today_jpn.should == Time.now.strftime('%y年%m月%d日')
    end
  end

  describe "<%= ec:image %> " do
    it "required arguments are input" do
      pending_if_loaded_plugin(:test_for_plugin_module)
      ec.image(:src => "logo.gif").should have_tag('img[alt="Logo"][src^=/images/logo.gif]')

      ec.image(:src => "/cms/logo.gif", :alt => 'ロゴ', :width => "50%", :height => "50%").should have_tag(
        'img[src^=/cms/logo.gif][alt="ロゴ"][height="50%"][width="50%"]')

      ec.image(:src => "/common/logo.gif", :alt => 'ロゴ', :width => "50%", :height => "50%").should have_tag(
        'img[src^=/common/logo.gif][alt="ロゴ"][height="50%"][width="50%"]')
    end

    it "required arguments aren't input" do
      pending_if_loaded_plugin(:test_for_plugin_module)
      ec.image.should be_nil
      ec.image(:badarg => "/cms/logo.gif").should be_nil
    end
  end

  describe "<%= ec:link %> " do
    it "with href" do
      ec.link(:text => "リンク", :href => "/list").should == '<a href="/list">リンク</a>'
      ec.link(:text => "リンク", :href => "/list", :class => 'linkClass').should == '<a href="/list" class="linkClass">リンク</a>'
    end

    it "with src" do
      ec.link(:text => "リンク", :src => "/list").should == '<a href="/list">リンク</a>'
      ec.link(:text => "リンク", :src => "/list", :class => 'linkClass').should == '<a href="/list" class="linkClass">リンク</a>'
    end

    it "with href and src" do
      ec.link(:text => "リンク", :href => "/list", :src => "/list1").should == '<a href="/list">リンク</a>'
      ec.link(:text => "リンク", :href => "/list", :src => "/list1", :class => 'linkClass').should == '<a href="/list" class="linkClass">リンク</a>'
    end

    it "required arguments aren't input" do
      lambda{ ec.link }.should raise_error(EcRider::EcHelper::ArgumentError)
      lambda{ ec.link(:text => "リンク") }.should raise_error(EcRider::EcHelper::ArgumentError)
      ec.link(:href => "/list").should == nil
      lambda{ ec.link(:text => "リンク", :badarg => "/list") }.should raise_error(EcRider::EcHelper::ArgumentError)
      ec.link(:href => "/list", :badarg => "wrong").should == nil
    end

    it "can nest other elements" do
      # ブロックが渡されたメソッド(空でない中身を持つecタグ)は <%= %> ではなく <% %>で出力されるので、
      # メソッドの戻り値ではなく、出力結果(@template.output_buffer)に対してテストを行います。
      @template.output_buffer = ""
      ec.link(:href => "/list"){ec.image(:src => "logo.gif") + "テスト メッセージ"}
      @template.output_buffer.should have_tag('a[href^=/list]') do
        have_tag('img[alt="Logo"][src^=/images/logo.gif]')
      end
    end

    it "ignore text if block given" do
      # ブロックが渡されたメソッド(空でない中身を持つecタグ)は <%= %> ではなく <% %>で出力されるので、
      # メソッドの戻り値ではなく、出力結果(@template.output_buffer)に対してテストを行います。
      @template.output_buffer = ""
      ec.link(:href => "/list", :text => "ignored_text"){ec.image(:src => "logo.gif") + "テスト メッセージ"}
      @template.output_buffer.should have_tag('a[href^=/list]') do
        have_tag('img[alt="Logo"][src^=/images/logo.gif]')
      end
    end

    describe "sanitized text" do
      it ">リンク" do
        ec.link(:text => "&gt;リンク", :href => "/list").should == '<a href="/list">&gt;リンク</a>'
        ec.link(:text => "&gt;リンク", :href => "/list", :class => 'linkClass').should == '<a href="/list" class="linkClass">&gt;リンク</a>'
      end

      it "<リンク" do
        ec.link(:text => "&lt;リンク", :href => "/list").should == '<a href="/list">&lt;リンク</a>'
        ec.link(:text => "&lt;リンク", :href => "/list", :class => 'linkClass').should == '<a href="/list" class="linkClass">&lt;リンク</a>'
      end

      it '"リンク"' do
        ec.link(:text => '&quot;リンク&quot;', :href => "/list").should == '<a href="/list">&quot;リンク&quot;</a>'
        ec.link(:text => '&quot;リンク&quot;', :href => "/list", :class => 'linkClass').should == '<a href="/list" class="linkClass">&quot;リンク&quot;</a>'
      end
    end

    it "required arguments aren't input" do
      lambda{ ec.link }.should raise_error(EcRider::EcHelper::ArgumentError)
      ec.link(:src => "logo.gif").should be_nil
      ec.link(:href => "/list").should be_nil
      ec.link(:src => "logo.gif", :badarg => "/list").should be_nil
      ec.link(:href => "/list", :badarg => "wrong").should be_nil
    end
  end

  describe "<%= ec:top_link %> " do
    before(:each) do
      @template.output_buffer = ""
    end
    it "ec:top_link デフォルトトップページの場合" do
      if EcRider::Runtime.ec_front?
        ec.top_link(:text => "リンク").should == '<a href="/">リンク</a>'
        ec.top_link(:text => "リンク", :class => 'linkClass').should ==
          '<a href="/" class="linkClass">リンク</a>'
        ec.top_link{"リンク"}
        @template.output_buffer.should == '<a href="/">リンク</a>'
        @template.output_buffer = ""
        ec.top_link(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should == '<a href="/" class="linkClass">リンク</a>'
      end
    end

    it "ec:top_link トップページアクション指定の場合" do
      if EcRider::Runtime.ec_front?
        ms = ShopSetting.new
        ms.setting_div = '0008'
        ms.value = 'test'
        ms.save
        ec.top_link(:text => "リンク").should == '<a href="/test">リンク</a>'
        ec.top_link(:text => "リンク", :class => 'linkClass').should ==
          '<a href="/test" class="linkClass">リンク</a>'
        ec.top_link{"リンク"}
        @template.output_buffer.should == '<a href="/test">リンク</a>'
        @template.output_buffer = ""
        ec.top_link(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should == '<a href="/test" class="linkClass">リンク</a>'
      end
    end
    it "required arguments aren't input" do
      ec.top_link.should be_nil
    end
  end

  describe "<%= ec:top_button %> " do
    it "ec:top_button デフォルトトップページの場合" do
      if EcRider::Runtime.ec_front?
        ec.top_button(:text => "リンク").should have_tag('input[onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        ec.top_button(:text => "リンク", :class => 'linkClass').should have_tag(
          'input[class="linkClass"][onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        ec.top_button{"リンク"}
        @template.output_buffer.should have_tag('input[onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        @template.output_buffer = ""
        ec.top_button(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should have_tag('input[class="linkClass"][onclick="location.href=\'/\';"][type="button"][value="リンク"]')
      end
    end

    it "ec:top_button トップページアクション指定の場合" do
      if EcRider::Runtime.ec_front?
        ms = ShopSetting.new
        ms.setting_div = '0008'
        ms.value = 'test'
        ms.save
        ec.top_button(:text => "リンク").should have_tag('input[onclick="location.href=\'/test\';"][type="button"][value="リンク"]')
        ec.top_button(:text => "リンク", :class => 'linkClass').should have_tag(
          'input[class="linkClass"][onclick="location.href=\'/test\';"][type="button"][value="リンク"]')
        ec.top_button{"リンク"}
        @template.output_buffer.should have_tag('input[onclick="location.href=\'/test\';"][type="button"][value="リンク"]')
        @template.output_buffer = ""
        ec.top_button(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should have_tag('input[class="linkClass"][onclick="location.href=\'/test\';"][type="button"][value="リンク"]')
      end
    end

  end

  describe "<%= ec:entrance_link %> " do
    it "ec:entrance_link デフォルトサイト入り口ページの場合" do
      if EcRider::Runtime.ec_front?
        pending_if_loaded_plugin(:test_for_plugin_module)
        ec.entrance_link(:text => "リンク").should == '<a href="/">リンク</a>'
        ec.entrance_link(:text => "リンク", :class => 'linkClass').should ==
          '<a href="/" class="linkClass">リンク</a>'
        ec.entrance_link{"リンク"}
        @template.output_buffer.should == '<a href="/">リンク</a>'
        @template.output_buffer =""
        ec.entrance_link(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should ==
          '<a href="/" class="linkClass">リンク</a>'
      end
    end

    it "ec:entrance_link サイト入り口アクション指定の場合" do
      if EcRider::Runtime.ec_front?
        pending_if_loaded_plugin(:test_for_plugin_module)
        ms = ShopSetting.new
        ms.setting_div = '0008'
        ms.value = 'test'
        ms.save
        ec.entrance_link(:text => "リンク").should == '<a href="/">リンク</a>'
        ec.entrance_link(:text => "リンク", :class => 'linkClass').should ==
          '<a href="/" class="linkClass">リンク</a>'
        ec.entrance_link{"リンク"}
        @template.output_buffer.should == '<a href="/">リンク</a>'
        @template.output_buffer = ""
        ec.entrance_link(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should ==
          '<a href="/" class="linkClass">リンク</a>'
      end
    end
  end

  describe "<%= ec:entrance_button %> " do
    it "ec:entrance_button デフォルトサイト入り口ページの場合" do
      if EcRider::Runtime.ec_front?
        ec.entrance_button(:text => "リンク").should have_tag('input[onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        ec.entrance_button(:text => "リンク", :class => 'linkClass').should
          have_tag('input[class="linkClass"][onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        ec.entrance_button{"リンク"}
        @template.output_buffer.should have_tag('input[onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        @template.output_buffer = ""
        ec.entrance_button(:class => 'linkClass'){"リンク"}
        @template.output_buffer.should have_tag(
          'input[class="linkClass"][onclick="location.href=\'/\';"][type="button"][value="リンク"]')
      end
    end

    it "ec:entrance_button サイト入り口アクション指定の場合" do
      if EcRider::Runtime.ec_front?
        ms = ShopSetting.new
        ms.setting_div = '0008'
        ms.value = 'test'
        ms.save
        ec.entrance_button(:text => "リンク").should have_tag('input[onclick="location.href=\'/\';"][type="button"][value="リンク"]')
        ec.entrance_button(:text => "リンク", :class => 'linkClass').should
          have_tag('input[class="linkClass"][onclick="location.href=\'/\';"][type="button"][value="リンク"]')
      end
    end
  end

  describe "<%= ec:image_link %> " do
    it "required values are input" do
      ec.image_link(:src => "logo.gif", :href => "/top").should have_tag('a[href="/top"]') do
        with_tag('img[alt="Logo"][src^=/images/logo.gif]')
      end

      ec.image_link(:src => "logo.gif", :href => "/top", :border => "1").should have_tag('a[href="/top"]') do
        with_tag('img[alt="Logo"][src^=/images/logo.gif][border="1"]')
      end
    end
  end


  describe "<%= ec:language_links %> " do
    it "<%= ec:language_links %> " do
      mock_locales = ['ja', 'zh_CN', 'zh_TW', 'en'].map{|locale| Locale::Tag.parse(locale) }
      mock_locales.each do |locale|
        locale.class.should == Locale::Tag::Simple
        locale.respond_to?(:to_simple).should == true
        locale.to_rfc.to_s.should =~ /[^_]/
      end
      I18n.should_receive(:supported_locales).once.and_return(mock_locales)
      links = ec.language_links
      links.should have_tag('a[href="/locale/update/ja"]', '日本語')
      links.should have_tag('a[href="/locale/update/zh_CN"]', '簡体中文')
      links.should have_tag('a[href="/locale/update/zh_TW"]', '繁體中文')
      links.should_not have_tag('a[href="/locale/update/zh-CN"]', '簡体中文')
      links.should_not have_tag('a[href="/locale/update/zh-TW"]', '繁體中文')
      links.should have_tag('a[href="/locale/update/en"]', 'English')
    end
  end


  describe "<%= ec:button %> " do
    before(:each) do
      @template.output_buffer = ''
    end

    it "should work without options" do
      button = ec.button
      button.should == nil
    end

    it "should work with href option" do
      ec.button(:href => "/cart/index"){ "カートへ" }
      button = @template.output_buffer
      button.should have_tag("input[type=?]", 'button')
      button.should include("カートへ")
      button.should == "<input onclick=\"location.href='/cart/index';\" type=\"button\" value=\"カートへ\" />"
    end

    it "should work with block" do
      ec.button{ "トップへ" }
      button = @template.output_buffer
      button.should have_tag("input[type=?]", 'button')
      button.should include("トップへ")
      # 秋間コメント
      # TODO ec_adminとec_frontで挙動が違っているので、要対応です。
      button.should =~ %r{<input onclick=\"location.href='/(?:top)?';\" type=\"button\" value=\"トップへ\" />}
    end

    it "should work with href option" do
      button = ec.button(:text => "買い物履歴へ")
      button.should have_tag("input[type=?]", 'button')
      button.should include("買い物履歴へ")
      # 秋間コメント
      # TODO ec_adminとec_frontで挙動が違っているので、要対応です。
      button.should =~ %r{<input onclick=\"location.href='/(?:top)?';\" type=\"button\" value=\"買い物履歴へ\" />}
    end
  end

  describe "<%= ec:link_to_howto_shopping %> " do
    it "should empty" do
      ec.link_to_howto_shopping.should == ""
    end
  end

  describe "ec tag like standard html tag" do
    before(:each) do
      @template.output_buffer = ""
    end

    %w(html head title script body div span br hr h1 h2 h3 tr td th table tbody thead tfoot).each do |tag_name|
      describe "ec:#{tag_name}" do
        it "should set attribute dynamically" do
          ec.send(tag_name) do
            @template.concat("AAA")
          end
          @template.output_buffer.should == "<#{tag_name}>AAA</#{tag_name}>"
        end

        it "set class dynamically" do
          ec.stub!(:stock_count).and_return(3)
          ec.send(tag_name) do
            ec.attr(:name => "class") do
              @template.concat("product_count_")
              @template.concat(ec.stock_count.to_s)
            end
            @template.concat("商品A")
          end
          @template.output_buffer.should have_tag("#{tag_name}[class=\"product_count_3\"]") do
            include_text("商品A")
          end
        end

        it "should set attribute dynamically with class" do
          ec.send(tag_name) do
            ec.attr(:name => 'class'){ @template.concat("foo") }
            @template.concat("AAA")
          end
          @template.output_buffer.should == "<#{tag_name} class=\"foo\">AAA</#{tag_name}>"
        end

        it "should set attribute dynamically with additional class" do
          ec.send(tag_name, :class => "foo") do
            ec.attr(:name => 'class'){ @template.concat("bar") }
            @template.concat("AAA")
          end
          @template.output_buffer.should == "<#{tag_name} class=\"foo bar\">AAA</#{tag_name}>"
        end

        it "should set attribute dynamically with two additional class" do
          ec.send(tag_name, :class => "foo") do
            ec.attr(:name => 'class'){ @template.concat("bar") }
            ec.attr(:name => 'class'){ @template.concat("baz") }
            @template.concat("AAA")
          end
          @template.output_buffer.should == "<#{tag_name} class=\"foo bar baz\">AAA</#{tag_name}>"
        end


        it "should set attribute dynamically with additional class and cycle" do
          5.times do
            ec.send(tag_name, :class => "foo") do
              ec.attr(:name => 'class'){ @template.concat("bar") }
              ec.attr_cycle(:name => 'class'){ @template.concat("even,odd") }
              @template.concat("AAA")
            end
          end
          @template.output_buffer.should == [
            "<#{tag_name} class=\"foo bar even\">AAA</#{tag_name}>",
            "<#{tag_name} class=\"foo bar odd\">AAA</#{tag_name}>",
            "<#{tag_name} class=\"foo bar even\">AAA</#{tag_name}>",
            "<#{tag_name} class=\"foo bar odd\">AAA</#{tag_name}>",
            "<#{tag_name} class=\"foo bar even\">AAA</#{tag_name}>"
            ].join
        end

        it "should set attribute dynamically with style" do
          ec.send(tag_name) do
            ec.attr(:name => 'style'){ @template.concat("color: #FFF") }
            @template.concat("AAA")
          end
          @template.output_buffer.should == "<#{tag_name} style=\"color: #FFF\">AAA</#{tag_name}>"
        end

        it "should set attribute dynamically with additional style" do
          ec.send(tag_name, :style => "color: #FFF") do
            ec.attr(:name => 'style'){ @template.concat("font-weight: bold; font-size: small") }
            @template.concat("AAA")
          end
          @template.output_buffer.should == "<#{tag_name} style=\"color: #FFF; font-weight: bold; font-size: small\">AAA</#{tag_name}>"
        end
      end
    end
  end

  describe 'display_with_character_limit' do
    it '指定文字数以上、ブロックは文字列' do
      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "10", :leader => "...") do
        "12345678901234567890"
      end
      @template.output_buffer.should == '1234567890...'
    end

    it '指定文字数以下、ブロックは文字列' do
      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "100", :leader => "...") do
        "12345678901234567890"
      end
      @template.output_buffer.should == '12345678901234567890'
    end

    it '指定文字数以上、ブロックはECタグ' do
      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "5", :leader => "..") do
        @template.concat(ec.product_name_1(:product_code => "0000000000001"))
        @template.output_buffer
      end
      
      pending_if_loaded_plugin(:c0001)
      @template.output_buffer.should == '商品1の商..'
    end

    it '指定文字数以下、ブロックはECタグ' do
      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "100", :leader => "..") do
        @template.concat(ec.product_name_1(:product_code => "0000000000001"))
        @template.output_buffer
      end
      
      pending_if_loaded_plugin(:c0001)
      @template.output_buffer.should == '商品1の商品名1'
    end

    it '指定文字数以上、文字列にHTMLタグを含む' do
      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "10", :leader => "...") do
        "12345<br/>678901234567890"
      end
      @template.output_buffer.should == '12345<br/>67890...'

      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "10", :leader => "...") do
        "1234567<br/>8901234567890"
      end
      @template.output_buffer.should == '1234567<br/>890...'

      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "10", :leader => "...") do
        "1234567890<br/>1234567890"
      end
      @template.output_buffer.should == '1234567890<br/>...'

      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "10", :leader => "...") do
        "12345678901234567890<br/>"
      end
      @template.output_buffer.should == '1234567890...'

      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "10", :leader => "...") do
        "12345<br/>6789<br/>01234567890"
      end
      @template.output_buffer.should == '12345<br/>6789<br/>0...'

      @template.output_buffer = ""
      str = <<-"EOS"
12345<br />
67890<br />
abcde<br />ABCDE
     EOS

      ec.display_with_character_limit(:limit => "13", :leader => "...") do
        str
      end
      @template.output_buffer.should == "12345<br />\n67890<br />\na..."
    end

    it '指定文字数以上、ECタグで取得した値にHTMLタグを含む' do
      ec.stub!(:product_name_1).and_return('productAの<br/>商品名1')
      @template.output_buffer = ""
      ec.display_with_character_limit(:limit => "11", :leader => "...") do
        @template.concat(ec.product_name_1(:product_code => "0000000000001"))
      end
      @template.output_buffer.should == 'productAの<br/>商品...'
    end
  end

  describe 'ec:banners_for banner' do
    fixtures :banners, :banner_displays
    describe '会員の場合' do
      before do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(false)
        @template.output_buffer = ""
        @template.extend(ApplicationHelper)
        user = User.find_by_id(1005)
        @template.stub(:logged_in?).and_return(true)
        @template.stub(:current_user).and_return(user)
      end
      it "画像バナーの場合 会員の場合" do
        I18n.stub(:locale).and_return('ja')

        ec.banners_for(:banner_code => "11111111", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
        @template.output_buffer.should have_tag('a') do
          with_tag('img[src^=/banner_images/banner_0000000001.png]')
        end
      end
      it "テキストバナーの場合 会員の場合" do
        I18n.stub(:locale).and_return('ja')
        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
      end
      it "多言語用バナーがない場合 会員の場合" do
        I18n.stub(:locale).and_return('ko')

        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should_not have_tag('a[href=?]', 'http://www.ec-one.com')
      end
    end
    describe '非会員の場合' do
      before do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(false)
        @template.output_buffer = ""
        @template.extend(ApplicationHelper)
        user = User.new()
        @template.stub(:logged_in?).and_return(true)
        @template.stub(:current_user).and_return(user)
      end
      it "画像バナーの場合 非会員の場合" do
        I18n.stub(:locale).and_return('ja')
        ec.banners_for(:banner_code => "11111111", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
        @template.output_buffer.should have_tag('a') do
          with_tag('img[src^=/banner_images/banner_0000000001.png]')
        end
      end
      it "テキストバナーの場合 非会員の場合" do
        I18n.stub(:locale).and_return('ja')
        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
      end
      it "多言語用バナーがない場合 非会員の場合" do
        I18n.stub(:locale).and_return('ko')
        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should_not have_tag('a[href=?]', 'http://www.ec-one.com')
      end
    end
  end
  describe "mail_to_tag" do
    it "mail_to_tag no mail_to_body" do
      ec.mail_to_tag(:mail_to => "inquiry@ec-rider.com", :subject => "メールタイトル") do
        "<font color=\"red\">リンクタイトル</font>"
      end
      @template.output_buffer.should have_tag('a[href=?]', "mailto:inquiry@ec-rider.com?body=&amp;subject=#{CGI.escape("メールタイトル".tosjis)}" ) do |a_tag|
        a_tag.should have_tag('font[color=?]',"red") 
      end
    end
    it "mail_to_tag" do
      ec.mail_to_tag(:mail_to => "inquiry@ec-rider.com", :subject => "メールタイトル") do
        ec.mail_to_body do
          "本文本文"
        end
        "<font color=\"red\">リンクタイトル</font>"
      end
      @template.output_buffer.should have_tag('a[href=?]', "mailto:inquiry@ec-rider.com?body=#{CGI.escape("本文本文".tosjis)}&amp;subject=#{CGI.escape("メールタイトル".tosjis)}") do |a_tag|
        a_tag.should have_tag('font[color=?]',"red")
      end
    end
  end
  describe "emoji_tag" do
    it "ec_adminから使用" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      ec.emoji(:code => "1").include?("<img alt=\"[emoji1]\" src=\"/ec_rider/images/mobile/emoji/1.gif").should be_true
    end
    it 'バイナリで指定' do
      pending if EcRider.config.ec_admin
      pending "privateメソッドを呼んでいて解決策不明のため保留"
      EcRider.config.stub!(:ec_admin).and_return(false)
      @template.request.mobile.class.stub(:name).and_return("Jpmobile::Mobile::Docomo")
      ec.emoji(:code => "0").should == ''
      ec.emoji(:code => "1").should == [0xE63E].pack('U')
      ec.emoji(:code => "252").should == [0xE757].pack('U')
      ec.emoji(:code => "253").should == ''
    end
    it 'Unicodeで指定' do
      pending if EcRider.config.ec_admin
      pending "privateメソッドを呼んでいて解決策不明のため保留"
      EcRider.config.stub!(:ec_admin).and_return(false)
      @template.request.mobile.class.stub(:name).and_return("Jpmobile::Mobile::Au")
      ec.emoji(:code => "0").should == ''
      ec.emoji(:code => "1").should == '&#xE63E;'
      ec.emoji(:code => "252").should == '&#xE757;'
      ec.emoji(:code => "253").should == ''
    end
  end
end
