# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :users, :orders, :user_addresses, :user_emails, :order_shippings, :order_items, :products, :product_skus,
      :point_settings, :user_points, :product_images, :campaigns

  def ec
    helper.ec
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  before(:each) do
    @template.output_buffer = ""
    @orders = Order.paginate(
      :per_page => 1, :page => 1,
      :conditions => {:user_id => 1005},
      :include => [{:order_shippings => :order_items}],
      :order => 'orders.id DESC')
    ec.instance_variable_set(:@orders, @orders)
    @template.instance_variable_set(:@ques, '10')
  end

  describe "ec:if_order_history_exits" do
    it "should be true" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.if_order_history_exits {invoked = true}
      invoked.should be_true
    end

    it "should be false" do
      @template.instance_variable_set(:@orders, [])
      @template.extend(ApplicationHelper)
      invoked = false
      ec.if_order_history_exits {invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:unless_order_history_exits" do
    it "should be false" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.unless_order_history_exits {invoked = true}
      invoked.should be_false
    end

    it "should be true" do
      @template.instance_variable_set(:@orders, [])
      @template.extend(ApplicationHelper)
      invoked = false
      ec.unless_order_history_exits {invoked = true}
      invoked.should be_true
    end
  end

  describe "ec:select_for_order_history_display" do
    it "should be false" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.select_for_order_history_display.should have_tag('select')
    end
  end

  describe "ec:order_history_total_count" do
    it "should be 6" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.order_history_total_count.should == "1"
    end
  end

  describe "ec:order_history_display_offset_from" do
    it "should be 1" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.order_history_display_offset_from.should == "1"
    end
  end

  describe "ec:order_history_display_offset_to" do
    it "should be 1" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.order_history_display_offset_to.should == "1"
    end
  end

  describe "ec:order_history_display_paginate" do
    it "should return paginate" do
#      @orders = Order.paginate(
#        :per_page => 1, :page => 1,
#        :conditions => {:user_id => 1002},
#        :include => [{:order_shippings => :order_items}],
#        :order => 'orders.id DESC')
#      @template.instance_variable_set(:@orders, @orders)
#      @template.instance_variable_set(:@ques, 1)
#      @template.extend(ApplicationHelper)
#      ec.order_history_display_paginate.should have_tag('a')
    end
  end

  describe "ec:button_to_order_detail" do
    it "should return button" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.order_history_list do
        ec.button_to_order_detail{"詳細"}
      end
      @template.output_buffer
      @template.output_buffer.should have_tag('form[action=?]', "/member/history_order_detail") do
        with_tag("input#id[type=?]", 'hidden')
        with_tag("input#page[type=?]", 'hidden')
        with_tag("input[type=?][value=?]", 'submit', '詳細')
      end
    end
  end

  describe "ec:button_for_order_cancel" do
    it "should return button" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.order_history_list do
        ec.button_for_order_cancel{"キャンセル依頼"}
      end
      @template.output_buffer
      @template.output_buffer.should have_tag('form[action=?]', "/member/request_cancel") do
        with_tag("input#id[type=?]", 'hidden')
        with_tag("input#ques[type=?]", 'hidden')
        with_tag("input[type=?][value=?]", 'submit', 'キャンセル依頼')
      end
    end
  end

  describe "ec:button_for_print_receipt" do
    it "should return button" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      ec.order_history_list do
        ec.addressee_form{
        ec.addressee_form_addressee
        }
      end
      @template.output_buffer
      @template.output_buffer.should have_tag('form[action=?]', "/member/print_receipt") do
        with_tag("input#id[type=?]", 'hidden')
        with_tag("input#addressee_form_addressee[type=?]", 'text')
      end
    end
  end

  describe "nomal usage" do
    it "about @order" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      if_order_cancel_possible_result = false
      unless_order_cancel_possible_result = false
      ec.order_history_list do
        ec.order_cancel_status
        ec.if_order_cancel_possible {if_order_cancel_possible_result = true}
        ec.unless_order_cancel_possible {unless_order_cancel_possible_result = true}
        ec.order_shipping_count.should == 2
        ec.order_shipping_given_point.should == 0
        ec.order_code.should == "00000000005"
        ec.order_datetime == "2007/12/10"
        ec.order_full_name == "山下 菜々子"
        @template.output_buffer
      end
      if_order_cancel_possible_result.should be_true
      unless_order_cancel_possible_result.should be_false
    end

    it "about @order_shipping" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      order_shipping_code_result = []
      shipping_full_name_result = []
      shipping_full_address_result = []
      order_shipping_total_price_result = []
      order_shipping_status_div_name_result = []
      order_shipping_delivery_code_result = []
      shipping_kana_last_name_result = []
      shipping_kana_first_name_result = []
      shipping_tel_num_result = []
      order_shipping_shipping_cost_result = []
      order_shipping_payment_type_div_name_result = []
      ec.order_history_list do
        ec.order_shipping_list do
          order_shipping_code_result << ec.order_shipping_code
          shipping_full_name_result << ec.shipping_full_name
          shipping_full_address_result << ec.shipping_full_address
          order_shipping_total_price_result << ec.order_shipping_total_price
          order_shipping_status_div_name_result << ec.order_shipping_status_div_name
          ec.if_delivery_code_input do
            order_shipping_delivery_code_result << ec.order_shipping_delivery_code
          end
          ec.if_japanese_shipping_format do
            shipping_kana_last_name_result << ec.shipping_kana_last_name
            shipping_kana_first_name_result << ec.shipping_kana_first_name
          end
          shipping_tel_num_result << ec.shipping_tel_num
          order_shipping_shipping_cost_result << ec.order_shipping_shipping_cost
          order_shipping_payment_type_div_name_result << ec.order_shipping_payment_type_div_name
          @template.output_buffer
          ""
        end
      end
      order_shipping_code_result.should == ["00000000005", "00000000006"]
      shipping_full_name_result.should == ["山下 菜々子", "山下 桃子"]
      shipping_full_address_result.should == ["〒8340023<br/>福岡県八女市 馬場１－２－３　熱田マンション１０２号", "〒4030003<br/>山梨県富士吉田市 大明見大字５３１号"]
      order_shipping_total_price_result.should == ["34,020", "11,340"]
      order_shipping_status_div_name_result.should == ["配送準備中", "配送準備中"]
      order_shipping_delivery_code_result.should == []
      shipping_kana_last_name_result.should == ["ヤマシタ", "ヤマシタ"]
      shipping_kana_first_name_result.should == ["ナナコ", "モモコ"]
      shipping_tel_num_result.should == ["000-111-2222", "000-111-2223"]
      order_shipping_shipping_cost_result = [0, 0]
      order_shipping_payment_type_div_name_result = ["クレジットカード", "クレジットカード"]
    end

    it "about @order_item" do
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      order_shipping_product_name_result = []
      order_shipping_product_qty_result = []
      order_shipping_product_image_result = []
      order_shipping_product_unit_price_result = []
      order_shipping_product_sub_total_price_result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.order_shipping_product_list do
            order_shipping_product_name_result << ec.order_shipping_product_name
            order_shipping_product_qty_result << ec.order_shipping_product_qty
            order_shipping_product_image_result << ec.order_shipping_product_image
            order_shipping_product_unit_price_result << ec.order_shipping_product_unit_price
            order_shipping_product_sub_total_price_result << ec.order_shipping_product_sub_total_price
            ""
          end
          ""
        end
      end
      order_shipping_product_name_result = ["栄養ドリンク10箱", "栄養ドリンク10箱"]
      order_shipping_product_qty_result = ["1","1"]
      order_shipping_product_unit_price_result = ["11,340", "11,340"]
      order_shipping_product_sub_total_price_result = ["11,340", "11,340"]
    end
  end

  describe "ec:unless_order_cancel_possible" do
    it "should be true" do
      @orders[0].stub!(:cancel_status).and_return(:impossible)
      @template.instance_variable_set(:@orders, @orders)
      @template.extend(ApplicationHelper)
      result = false
      ec.order_history_list do
        ec.order_shipping_list do
          ec.unless_order_cancel_possible {result = true}
          @template.output_buffer
        end
      end
      result.should be_true
    end
  end

  describe "ec:if_delivery_code_input" do
    it "should be 123" do
      @orders[0].order_shippings[0].delivery_code = "123"
      @orders[0].order_shippings[1].delivery_code = ""
      @template.instance_variable_set(:@orders, @orders)
      result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.if_delivery_code_input do
            result << ec.order_shipping_delivery_code
          end
          @template.output_buffer
        end
      end
      result.should == ["123"]
    end
  end

  describe "ec:unless_delivery_code_input" do
    it "should be blank" do
      @orders[0].order_shippings[0].delivery_code = "123"
      @orders[0].order_shippings[1].delivery_code = ""
      @template.instance_variable_set(:@orders, @orders)
      result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.unless_delivery_code_input do
            result << ec.order_shipping_delivery_code
          end
          @template.output_buffer
        end
      end
      result.should == [nil]
    end
  end

  describe "ec:button_to_history" do
    it "should return button" do
      @template.instance_variable_set(:@page_no, "1")
      @template.extend(ApplicationHelper)
      ec.button_to_history{"戻る"}.should have_tag("input[type=?][value=?][onclick=?]", 'button', '戻る', "location.href='/member/history?page=1';")
    end
  end

  describe "ec:if_include_order_parent" do
    it "should be true" do
      order = Order.find(2)
      order.include_order_id = 9
      order.save!
      @order = Order.find(9)
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_include_order_parent {invoked = true}
      invoked.should be_true
    end

    it "should be false" do
      @order = Order.find(1)
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_include_order_parent {invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:if_include_order_child" do
    it "should be true" do
      @order = Order.find(2)
      @order.include_order_id = 9
      @order.save!
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_include_order_child {invoked = true}
      invoked.should be_true
    end

    it "should be false" do
      @order = Order.find(2)
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_include_order_child {invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:if_history_gift" do
    it "should be false" do
      @order = Order.find(2)
      @order.gift_flg.should be_false
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_gift {invoked = true}
      invoked.should be_false
    end

    it "should be true" do
      @order = Order.find(2)
      @order.gift_flg = true
      @order.save!
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_gift {invoked = true}
      invoked.should be_true
    end
  end

  describe "ec:if_history_send_invoice_separately" do
    it "should be false" do
      @order = Order.find(2)
      @order.send_invoice_separately_flg.should be_false
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_send_invoice_separately {invoked = true}
      invoked.should be_false
    end

    it "should be true" do
      @order = Order.find(2)
      @order.send_invoice_separately_flg = true
      @order.save!
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_send_invoice_separately {invoked = true}
      invoked.should be_true
    end
  end

  describe "ec:if_history_history_write_sender" do
    it "should be false" do
      @order = Order.find(2)
      @order.write_sender_flg.should be_false
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_write_sender {invoked = true}
      invoked.should be_false
    end

    it "should be true" do
      @order = Order.find(2)
      @order.write_sender_flg = true
      @order.save!
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_write_sender {invoked = true}
      invoked.should be_true
    end
  end

  describe "ec:if_point_setting" do
    it "should be ture" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg_true?.should be_true
      invoked = false
      ec.if_point_setting {invoked = true}
      invoked.should be_true
    end
    it "should be false" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = false
      point_setting.save!
      point_setting.available_flg_true?.should be_false
      invoked = false
      ec.if_point_setting {invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:if_history_history_write_sender" do
    it "should be false" do
      @order = Order.find(2)
      @order.write_sender_flg.should be_false
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_write_sender {invoked = true}
      invoked.should be_false
    end

    it "should be true" do
      @order = Order.find(2)
      @order.write_sender_flg = true
      @order.save!
      ec.instance_variable_set(:@order, @order)
      invoked = false
      ec.if_history_write_sender {invoked = true}
      invoked.should be_true
    end
  end
  describe "with ec:order_product_image" do
    before do
      @template.output_buffer = ""
      order_item = OrderItem.find_by_id(1)
      order_item.product_sku_id = 282
      order_item.save
      @orders = Order.paginate(
        :per_page => 1, :page => 1,
        :conditions => {:user_id => 1001},
        :include => [{:order_shippings => :order_items}],
        :order => 'orders.id DESC')
      ec.instance_variable_set(:@orders, @orders)
      @template.instance_variable_set(:@ques, '10')
      @template.extend(ApplicationHelper)
    end
    it "nil" do
      order_shipping_product_image_result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.order_shipping_product_list do
            order_shipping_product_image_result << ec.order_shipping_product_image
            ""
          end
          ""
        end
      end
      order_shipping_product_image_result[0].should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001.gif][alt="錠剤100粒"]')
    end

    it "medium" do
      order_shipping_product_image_result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.order_shipping_product_list do
            order_shipping_product_image_result << ec.order_shipping_product_image(:type => 'medium')
            ""
          end
          ""
        end
      end
      order_shipping_product_image_result[0].should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_medium.gif][alt="錠剤100粒"]')
    end

    it "large" do
      order_shipping_product_image_result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.order_shipping_product_list do
            order_shipping_product_image_result << ec.order_shipping_product_image(:type => 'large')
            ""
          end
          ""
        end
      end
      order_shipping_product_image_result[0].should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_large.gif][alt="錠剤100粒"]')
    end

    it "small" do
      order_shipping_product_image_result = []
      ec.order_history_list do
        ec.order_shipping_list do
          ec.order_shipping_product_list do
            order_shipping_product_image_result << ec.order_shipping_product_image(:type => 'small')
            ""
          end
          ""
        end
      end
      order_shipping_product_image_result[0].should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_small.gif][alt="錠剤100粒"]')
    end
  end
  describe "if_order_cancel_error" do
    it "エラーがある場合" do
      @template.instance_variable_set(:@user_order_error, 'test')
      result = ""
      ec.if_order_cancel_error do
        result = "出力されています。"
      end
      result.should == "出力されています。"
    end
    it "エラーがない場合" do
      @template.instance_variable_set(:@user_order_error, nil)
      ec.if_order_cancel_error do
        flunk("出力されています。")
      end
    end
  end

  describe "ec:each_order_history_applied_campaigns" do
    it "適用されたキャンペーン一覧" do
      order = Order.find(1)
      campaign_a = Campaign.find(20)
      order.stub!(:applied_campaigns).and_return([campaign_a])
      @template.instance_variable_set(:@order, order)
      result = []
      ec.each_order_history_applied_campaigns do
        result << ec.order_applied_campaign_name
        ''  #ec:each_order_applied_campaignsはブロックで文字列が渡される想定なので文字列を返す
      end
      result.first.should == "T0008F"
    end
  end
end
