# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :users, :user_addresses, :user_emails, :countries, :prefectures, :prefecture_displays, :shipping_costs, :shop_settings, :product_skus, :campaigns, :products, :shipping_cost_boundaries

  def ec
    helper.ec
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "shipping_tag" do

    before(:each) do
      @template.output_buffer = ""
      @user = User.find(1006, :include => [:user_address, :user_email])
      @shipping_addresses = @user.user_and_shipping_addresses
      @template.instance_variable_set(:@shipping_addresses, @shipping_addresses)
      @template.extend(ApplicationHelper)
    end

    describe "ec:if_shipping_address_error" do
      it "エラーがある場合" do
        @template.instance_variable_set(:@error_flg, true)
        result = ""
        ec.if_shipping_address_error do
          result = "出力されています。"
        end
        result.should == "出力されています。"
      end
      it "エラーがない場合" do
        @template.instance_variable_set(:@error_flg, nil)
        ec.if_shipping_address_error do
          flunk("出力されています。")
        end
      end
    end

    describe "ec:shipping_list" do
      it "<ec:shipping_list>～</ec:shipping_list>" do
        ec.shipping_list do
          ec.shipping_fullname
        end
        @template.output_buffer.should =~ /空川 結衣/
        @template.output_buffer.should =~ /空川 助左衛門/
      end
    end

    describe "ec:shipping_fullname" do
      it "<ec:shipping_fullname/>(kana)" do
        ec.shipping_list do
          ec.shipping_fullname
        end
        @template.output_buffer.should =~ /空川 結衣/
        @template.output_buffer.should =~ /空川 助左衛門/
        @template.output_buffer.should =~ /ソラカワ/
      end
    end

    describe "with countries" do
      fixtures :countries

      describe "ec:shipping_countryname" do
        it "<ec:shipping_countryname/>" do
          ec.shipping_list do
            ec.shipping_countryname
          end
          @template.output_buffer.should =~ /Japan/
        end
      end

      describe "ec:shipping_zipcode" do
        it "<ec:shipping_zipcode/>" do
          ec.shipping_list do
            ec.shipping_zipcode
          end
          @template.output_buffer.should =~ /〒1210051/
          @template.output_buffer.should =~ /〒5300002/
        end
      end

      describe "ec:shipping_cost" do
        it "送料無料キャンペーンなし" do
          ShopSetting.stub!(:get_setting).and_return('5300002,100')
          cart = Cart.new()
          product12 = ProductSku.find_by_id(276)
          product13 = ProductSku.find_by_id(277)
          cart.add(product12, product12.product.course_div_key, 1)
          cart.add(product13, product13.product.course_div_key, 2)
          request.session[:cart] = cart
          result = []
          ec.shipping_list do
            result << ec.shipping_cost.should
            @template.output_buffer
          end
          result[0].should == 100
          result[1].should == 200
        end
        it "送料無料キャンペーンありたが、購入金額が条件指定された金額より下回る場合" do
          ShopSetting.stub!(:get_setting).and_return('5300002,100')
          cart = Cart.new()
          product12 = ProductSku.find_by_id(276)
          product13 = ProductSku.find_by_id(277)
          cart.add(product12, product12.product.course_div_key, 1)
          cart.add(product13, product13.product.course_div_key, 2)
          request.session[:cart] = cart
          cam1 = Campaign.find_by_id(18)
          cam2 = Campaign.find_by_id(19)
          request.session[:cart].stub!(:campaigns_for_cart).and_return([cam1, cam2])
          result = []
          ec.shipping_list do
            result << ec.shipping_cost.should
            @template.output_buffer
          end
          result[0].should == 100
          result[1].should == 200
        end
      end
    end

    describe "ec:shipping_address" do
      it "<ec:shipping_address/>" do
        ec.shipping_list do
          ec.shipping_address
        end
        @template.output_buffer.should == '東京都足立区 神明４－１－６　マリンマンション１４０５号' + '大阪府大阪市北区 曽根崎新地４－２－７３　田中マンション３０５号'
      end
    end

    describe "ec:shipping_telnum" do
      it "<ec:shipping_telnum/>" do
        ec.shipping_list do
          ec.shipping_telnum
        end
        @template.output_buffer.should =~ /000-111-2224/
        @template.output_buffer.should =~ /000-111-2225/
      end
    end

    describe "ec:button_to_shipping_edit" do
      it "<ec:button_to_shipping_edit/>" do
        ec.shipping_list do
          ec.button_to_shipping_edit{'編集'}
        end
        @template.output_buffer.should have_tag('form') do
          with_tag('input')
        end
      end
    end

    describe "ec:button_to_shipping_destoroy" do
      it "<ec:button_to_shipping_destoroy/>" do
        ec.shipping_list do
          ec.button_to_shipping_destroy{'削除'} || ''
        end
        @template.output_buffer.should have_tag('form') do
          with_tag('input')
        end
      end
    end

  end

end
