# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :kbn_codes, :kbn_code_names

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:include_policy_text" do
    it "should be render /app/views/cms/policy_text.html.erb" do
      pending
      @template.stub!(:_set_controller_content_type).and_return('text/html')
      ec.include_policy_text.should_not be_blank
    end
  end

  describe "ec:policy_form" do
    it "会員種別なし" do
      @template.extend(ApplicationHelper)
      ec.policy_form do
        concat(ec.member_category_error_messages || '')
        ec.button{ "←前へ戻る"}
        ec.submit{"次へ進む→"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/signup") do
        without_tag("input[id=?][type=?]",'member_category_div' , 'hidden')
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '次へ進む→')
      end
    end

    it "会員種別あり" do
      @template.extend(ApplicationHelper)
      @template.instance_variable_set(:@member_category_div, '001')
      ec.policy_form do
        concat(ec.member_category_error_messages || '')
        ec.button{ "←前へ戻る"}
        ec.submit{"次へ進む→"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/signup") do
        with_tag("input[id=?][type=?][value=?]",'member_category_div' , 'hidden', '001')
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '次へ進む→')
      end
    end
  end

  describe "ec:signup_form" do
    before(:each) do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
    end

    it "about @user_address" do
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_error_messages || '')
        concat(ec.signup_form_main_address_last_name)
        concat(ec.signup_form_main_address_first_name)
        concat(ec.signup_form_main_address_name_error_message)
        concat(ec.signup_form_main_address_kana_last_name)
        concat(ec.signup_form_main_address_kana_first_name)
        concat(ec.signup_form_main_address_kana_name_error_message)
        concat(ec.signup_form_main_address_country)
        concat(ec.signup_form_main_address_address_1)
        concat(ec.signup_form_main_address_address_2)
        concat(ec.signup_form_main_address_address_error_message)
        concat(ec.signup_form_main_address_tel_num)
        ec.button{ "←前へ戻る"}
        ec.submit{"次へ進む（登録内容の確認）→"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("input#user_address_name_last")
        with_tag("input#user_address_name_first")
        with_tag("span#name_msg")
        with_tag("input#user_address_kana_name_last")
        with_tag("input#user_address_kana_name_first")
        with_tag("span#kana_name_msg")
        with_tag("select#user_address_country_code")
        with_tag("input#user_address_address_2")
        with_tag("input#user_address_address_3")
        with_tag("span#address_msg")
        with_tag("input#user_address_tel_num")
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '次へ進む（登録内容の確認）→')
      end
    end

    it "about @user_en_address" do
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_error_messages || '')
        concat(ec.signup_form_english_address_last_name)
        concat(ec.signup_form_english_address_first_name)
        concat(ec.signup_form_english_address_prefecture)
        concat(ec.signup_form_english_address_address_1)
        concat(ec.signup_form_english_address_address_2)
        ec.button{ "←前へ戻る"}
        ec.submit{"次へ進む（登録内容の確認）→"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("input#user_en_address_name_first")
        with_tag("input#user_en_address_name_last")
        with_tag("input#user_en_address_address_2")
        with_tag("input#user_en_address_address_3")
        with_tag("input#user_en_address_address_1")
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '次へ進む（登録内容の確認）→')
      end
    end

    it "about @user_email" do
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_error_messages || '')
        concat(ec.signup_form_profile_email)
        concat(ec.signup_form_profile_email_error_message)
        ec.button{ "←前へ戻る"}
        ec.submit{"次へ進む（登録内容の確認）→"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("input#user_email_email")
        with_tag("span#email_msg")
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '次へ進む（登録内容の確認）→')
      end
    end

    it "about @user_profile" do
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_error_messages || '')
        ec.signup_form_profile_gender do
          @template.concat(ec.signup_form_profile_gender_radio)
          @template.concat(ec.signup_form_profile_gender_label)
        end
        concat(ec.signup_form_profile_birth_year)
        concat(ec.signup_form_profile_birth_month)
        concat(ec.signup_form_profile_birth_day)
        concat(ec.signup_form_profile_birth_error_message)
        concat(ec.signup_form_profile_job)
        concat(ec.signup_form_profile_password)
        concat(ec.signup_form_profile_password_confirmation)
        concat(ec.signup_form_profile_remember_question)
        concat(ec.signup_form_profile_remember_answer)
        ec.button{ "←前へ戻る"}
        ec.submit{"次へ進む（登録内容の確認）→"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("input#user_profile_sex_1[type=?][value=?]", 'radio', '1')
        with_tag("label[for=?]", 'user_profile_sex_1', '男性')
        with_tag("input#user_profile_sex_2[type=?][value=?]", 'radio', '2')
        with_tag("label[for=?]", 'user_profile_sex_2', '女性')
        with_tag("select#user_profile_birth_year")
        with_tag("select#user_profile_birth_month")
        with_tag("select#user_profile_birth_day")
        with_tag("span#birth_msg")
        with_tag("select#user_profile_job_div")
        with_tag("input#user_profile_password[type=?]", 'password')
        with_tag("input#user_profile_password_confirmation[type=?]", 'password')
        with_tag("input#user_profile_remember_question")
        with_tag("input#user_profile_remember_answer")
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '次へ進む（登録内容の確認）→')
      end
    end

    it "should raise error" do
      @template.instance_variable_set(:@user_address, nil)
      @template.extend(ApplicationHelper)
      lambda{ec.signup_form}.should raise_error(EcRider::EcHelper::UsageError)
    end
  end

  describe "ec:include_user_address_fields" do
    it "should render 'account/user_address'" do
      pending
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.include_user_address_fields)
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
      end
    end
  end

  describe "ec:signup_form_main_address_country" do
    it "shippableto_oversea is false" do
      ec.stub!(:shippable_to_oversea?).and_return(false)
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_main_address_country)
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("input#user_address_country_code[type=?]", 'hidden')
      end
    end
  end

  describe "ec:if_shippable_to_oversea" do
    it "should be true" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.signup_form do
        ec.if_shippable_to_oversea{invoked = true}
        @template.output_buffer
      end
      invoked.should be_true
    end
  end

  describe "ec:unless_shippable_to_oversea" do
    it "should be false" do
      ec.stub!(:shippable_to_oversea?).and_return(false)
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.signup_form do
        ec.unless_shippable_to_oversea{invoked = true}
        @template.output_buffer
      end
      invoked.should be_true
    end
  end

  describe "ec:signup_form_main_address_country_pane" do
    before(:each) do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
    end

    it "with option 'JP'" do
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_main_address_country)
        ec.signup_form_main_address_country_pane(:country => 'JP', :tag_name => 'tr', :id => "address_form_zipcode_tr") do
          @template.concat(ec.signup_form_main_address_japanese_zip_code)
          @template.concat(ec.signup_form_main_address_zip_to_address_button{"郵便番号で住所表示"})
          @template.concat(ec.signup_form_main_address_japanese_zip_code_error_message)
          @template.concat(ec.signup_form_main_address_japanese_prefecture)
        end
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("tr#address_form_zipcode_tr[class=?]", 'main_address_country_pane_JP ')
        with_tag("input#user_address_zip_code")
        with_tag("input[type=?][value=?]", 'button', '郵便番号で住所表示')
        with_tag("span#zip_code_msg")
        with_tag("select#user_address_address_1")
      end
    end

    it "with no option" do
      @template.extend(ApplicationHelper)
      ec.signup_form do
        concat(ec.signup_form_main_address_country)
        ec.signup_form_main_address_country_pane(:tag_name => 'tr', :id => "address_form_zipcode_tr") do
          @template.concat(ec.signup_form_main_address_zip_code)
          @template.concat(ec.signup_form_main_address_prefecture)
        end
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_signup") do
        with_tag("tr#address_form_zipcode_tr[class=?]", 'main_address_country_pane_others ')
        with_tag("input#user_address_zip_code")
        without_tag("input[type=?][value=?]", 'button', '郵便番号で住所表示')
        without_tag("span#zip_code_msg")
        with_tag("input#user_address_address_1")
        without_tag("select#user_address_address_1")
      end
    end
  end

  describe "ec:signup_confirmation_form" do
    it "normal usage" do
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト", :kana_name_first => "ジロウ", :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000", :address_1 => "福岡県", :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP', :locale => 'ja'}
      @user_email = {:email => "test@test.jp"}
      @user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970",
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A',
        :kana_name_last => "テスト",  :kana_name_first => "ジロウ"}
      @user_en_address = {:name_last => "test", :name_first => "jiro",
         :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}

      @template.instance_variable_set(:@user_address, UserAddress.new(@user_address))
      @template.instance_variable_set(:@user_en_address, UserAddress.new(@user_en_address))
      @template.instance_variable_set(:@user_email, UserEmail.new(@user_email))
      @template.instance_variable_set(:@user_profile, User.new(@user_profile.merge(:user_address => UserAddress.new(@user_address.merge(:country => Country.new(:code=>'JP', :name=>'日本'))))))
      @template.extend(ApplicationHelper)
      ec.signup_confirmation_form do
        ec.signup_confirmation_form_main_address_last_name.should == "てすと"
        ec.signup_confirmation_form_main_address_first_name.should == "じろう"
        ec.signup_confirmation_form_main_address_full_name.should == "てすと じろう"
        ec.signup_confirmation_form_main_address_kana_last_name.should == "テスト"
        ec.signup_confirmation_form_main_address_kana_first_name.should == "ジロウ"
        ec.signup_confirmation_form_main_address_country.should == "日本"
        ec.signup_confirmation_form_main_address_zip_code.should == "0000000"
        ec.signup_confirmation_form_main_address_prefecture.should == "福岡県"
        ec.signup_confirmation_form_main_address_address_1.should == "福岡市中央区"
        ec.signup_confirmation_form_main_address_address_2.should == "天神１－１－１"
        ec.signup_confirmation_form_main_address_full_address.should == "〒0000000<br/>福岡県福岡市中央区 天神１－１－１"
        ec.signup_confirmation_form_main_address_tel_num.should == "000-0000-0000"
        ec.signup_confirmation_form_english_address_last_name.should == "test"
        ec.signup_confirmation_form_english_address_first_name.should == "jiro"
        ec.signup_confirmation_form_english_address_full_name.should == "jiro test"
        ec.signup_confirmation_form_english_address_prefecture.should == "Fukuoka"
        ec.signup_confirmation_form_english_address_address_1.should == "1-1-1 Tenjin Chuo-ku Fukuoka-shi"
        ec.signup_confirmation_form_english_address_address_2.should == ""
        ec.signup_confirmation_form_english_address_full_address.should == "1-1-1 Tenjin Chuo-ku Fukuoka-shi<br/>Fukuoka"
        ec.signup_confirmation_form_profile_email.should == "test@test.jp"
        ec.signup_confirmation_form_profile_gender.should == "男性"
        ec.signup_confirmation_form_profile_birth_year.should == "1970"
        ec.signup_confirmation_form_profile_birth_month.should == "1"
        ec.signup_confirmation_form_profile_birth_day.should == "23"
        ec.signup_confirmation_form_profile_job.should == "会社員"
        ec.signup_confirmation_form_profile_remember_question.should == "Q"
        ec.signup_confirmation_form_profile_remember_answer.should == "A"
        @template.output_buffer
      end
    end

    it "should raise error" do
      @template.instance_variable_set(:@user_address, nil)
      @template.extend(ApplicationHelper)
      lambda{ec.signup_confirmation_form}.should raise_error(EcRider::EcHelper::UsageError)
    end
  end

  describe "ec:if_signup_address_japanese_format" do
    it "japanese_format" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP', :locale => 'ja'))
      @template.extend(ApplicationHelper)
      invoked = false
      ec.if_signup_address_japanese_format{invoked = true}
      invoked.should be_true
    end

    it "english_format" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'US', :locale => 'ja'))
      @template.extend(ApplicationHelper)
      invoked = false
      ec.if_signup_address_japanese_format{invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:unless_signup_address_japanese_format" do
    it "japanese_format" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP', :locale => 'ja'))
      @template.extend(ApplicationHelper)
      invoked = false
      ec.unless_signup_address_japanese_format{invoked = true}
      invoked.should be_false
    end

    it "english_format" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'US', :locale => 'ja'))
      @template.extend(ApplicationHelper)
      invoked = false
      ec.unless_signup_address_japanese_format{invoked = true}
      invoked.should be_true
    end
  end

  describe "ec:if_english_address_input" do
    it "should be true" do
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.if_english_address_input{invoked = true}
      invoked.should be_true
    end

    it "should be false" do
      @template.instance_variable_set(:@user_en_address, nil)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.if_english_address_input{invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:unless_english_address_input" do
    it "should be false" do
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.unless_english_address_input{invoked = true}
      invoked.should be_false
    end

    it "should be true" do
      @template.extend(ApplicationHelper)
      invoked = false
      ec.unless_english_address_input{invoked = true}
      invoked.should be_false
    end
  end

  describe "ec:if_birth_day_input" do
    before(:each) do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
    end

    it "should be true" do
      @template.instance_variable_set(:@user_profile, User.new(:birth_month => "1", :birth_day => "23", :birth_year => "1970"))
      @template.extend(ApplicationHelper)
      invoked = false
      ec.signup_confirmation_form do
        ec.if_birth_day_input{invoked = true}
        @template.output_buffer
      end
      invoked.should be_true
    end

    it "should be true" do
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.signup_confirmation_form do
        ec.if_birth_day_input{invoked = true}
        @template.output_buffer
      end
      invoked.should be_false
    end
  end

  describe "ec:unless_birth_day_input" do
    before(:each) do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
    end

    it "should be false" do
      @template.instance_variable_set(:@user_profile, User.new(:birth_month => "1", :birth_day => "23", :birth_year => "1970"))
      @template.extend(ApplicationHelper)
      invoked = false
      ec.signup_confirmation_form do
        ec.unless_birth_day_input{invoked = true}
        @template.output_buffer
      end
      invoked.should be_false
    end

    it "should be true" do
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.signup_confirmation_form do
        ec.unless_birth_day_input{invoked = true}
        @template.output_buffer
      end
      invoked.should be_true
    end
  end

  describe "ec:if_customized_member" do
    describe "ec:policy_formで使用する場合" do
      it "会員種別パラメータありの場合" do
        @template.instance_variable_set(:@member_category_name, "name")
        @template.instance_variable_set(:@member_category_div, "001")
        @template.extend(ApplicationHelper)
        expect_true = false
        category_name = nil
        ec.policy_form do
          ec.if_customized_member do
            expect_true = true
            category_name = ec.member_category_name
            @template.output_buffer
          end
          @template.output_buffer
        end
        expect_true.should be_true
        category_name.should == "name"
      end
      it "会員種別パラメータなしの場合" do
        @template.instance_variable_set(:@member_category_name, nil)
        @template.instance_variable_set(:@member_category_div, nil)
        @template.extend(ApplicationHelper)
        expect_true = true
        ec.policy_form do
          ec.if_customized_member do
            expect_true = false
            @template.output_buffer
          end
          @template.output_buffer
        end
        expect_true.should be_true
      end
    end
    describe "ec:signup_formで使用する場合" do
      it "会員種別パラメータありの場合" do
        @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
        @template.instance_variable_set(:@user_en_address, UserAddress.new)
        @template.instance_variable_set(:@user_email, UserEmail.new)
        @template.instance_variable_set(:@user_profile, User.new(:member_category_div => '001'))
        @template.extend(ApplicationHelper)

        expect_true = false
        category_name = nil
        ec.signup_form do
          ec.if_customized_member do
            expect_true = true
            category_name = ec.member_category_name
            @template.output_buffer
          end
          @template.output_buffer
        end
        expect_true.should be_true
      end
      it "会員種別パラメータなしの場合" do
        @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
        @template.instance_variable_set(:@user_en_address, UserAddress.new)
        @template.instance_variable_set(:@user_email, UserEmail.new)
        @template.instance_variable_set(:@user_profile, User.new)
        @template.extend(ApplicationHelper)

        expect_true = true
        ec.signup_form do
          ec.if_customized_member do
            expect_true = false
            @template.output_buffer
          end
          @template.output_buffer
        end
        expect_true.should be_true
      end
    end
    describe "ec:signup_confirmation_formで使用する場合" do
      describe "ec:signup_formで使用する場合" do
        it "会員種別パラメータありの場合" do
          @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
          @template.instance_variable_set(:@user_en_address, UserAddress.new)
          @template.instance_variable_set(:@user_email, UserEmail.new)
          @template.instance_variable_set(:@user_profile, User.new(:member_category_div => '001'))
          @template.extend(ApplicationHelper)

          expect_true = false
          category_name = nil
          ec.signup_confirmation_form do
            ec.if_customized_member do
              expect_true = true
              @template.concat(ec.member_category_name)
              @template.output_buffer
            end
            @template.output_buffer
          end
          expect_true.should be_true
        end
        it "会員種別パラメータなしの場合" do
          @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
          @template.instance_variable_set(:@user_en_address, UserAddress.new)
          @template.instance_variable_set(:@user_email, UserEmail.new)
          @template.instance_variable_set(:@user_profile, User.new)
          @template.extend(ApplicationHelper)

          expect_true = true
          ec.signup_confirmation_form do
            ec.if_customized_member do
              expect_true = false
              @template.output_buffer
            end
            @template.output_buffer
          end
          expect_true.should be_true
        end
      end
    end
  end
  
  describe "ec:unless_customized_member" do
    it "ec:policy_formで使用する場合" do
      @template.instance_variable_set(:@member_category_name, "name")
      @template.instance_variable_set(:@member_category_div, "001")
      @template.extend(ApplicationHelper)
      expect_true = true
      ec.policy_form do
        ec.unless_customized_member do
          expect_true = false
          @template.output_buffer
        end
        @template.output_buffer
      end
      expect_true.should be_true
    end
    it "ec:signup_formで使用する場合" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      expect_true = true
      ec.signup_form do
        ec.unless_customized_member do
          expect_true = false
          @template.output_buffer
        end
        @template.output_buffer
      end
      expect_true.should be_true
    end
    it "ec:signup_confirmation_formで使用する場合" do
      @template.instance_variable_set(:@user_address, UserAddress.new(:country_code => 'JP'))
      @template.instance_variable_set(:@user_en_address, UserAddress.new)
      @template.instance_variable_set(:@user_email, UserEmail.new)
      @template.instance_variable_set(:@user_profile, User.new)
      @template.extend(ApplicationHelper)
      expect_true = true
      ec.signup_confirmation_form do
        ec.unless_customized_member do
          expect_true = false
          @template.output_buffer
        end
        @template.output_buffer
      end
      expect_true.should be_true
    end
  end
  
end
