# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
   fixtures :products, :user_points, :users, :orders
  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:user_point" do
    it "user_point_point_list単体で利用する場合 ポイント一覧は全件取得できること" do
      @template.extend(ApplicationHelper)
      @template.stub!(:current_user).and_return(User.find(1003))
      ec.user_point_total_point.should == 1400
      ec.user_point_term_point.should == 0 # pending
      order_code_list = []
      order_datetime_list = []
      order_product_name_list = []
      order_price_list = []
      given_point_list = []
      used_point_list = []
      ec.user_point_point_list do
        order_code_list << ec.user_point_order_code
        order_datetime_list << ec.user_point_order_datetime(:format => "%Y年%m月%d日 %X")
        ec.user_point_order_product_list do
          order_product_name_list << ec.user_point_order_product_name
          @template.concat("")
        end
        order_price_list << ec.user_point_order_price
        given_point_list << ec.user_point_given_point
        used_point_list << ec.user_point_used_point
        @template.concat("")
      end
      order_code_list[0].size.should == 11
    end
    it "user_point_point_list_areaを利用する場合 ポイント一覧は指定件数のみ取得できること" do
      UserPoint.delete_all
      20.times do |index|
        user_point = UserPoint.new(:given_point => 1, :order_id => index, :user_id => 1003)
        user_point.save
      end
      UserPoint.find(:all).size.should == 20
      @template.extend(ApplicationHelper)
      @template.stub!(:current_user).and_return(User.find(1003))
      result = 0
      ec.user_point_list_area(:product_separate_value => 7) do
        ec.user_point_pagination_count_per_page
        ec.user_point_point_list do
          result += 1
          @template.output_buffer
        end
        @template.output_buffer
      end
      result.should == 7
      @template.output_buffer.should have_tag('form[action=?]', "/member/point_history") do
        with_tag("select#condition_per_page")
      end
    end
    describe "user_point_term_date" do
      it "日付フォーマットがデフォルトの場合 00/00/00形式で表示されること" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1001))
        ec.user_point_term_date.should == "08/01/02"
      end
      it "日付フォーマット指定の場合 指定されたフォーマットで表示されること" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1001))
        ec.user_point_term_date(:format => "%Y年%m月%d日").should == "2008年01月02日"
      end
    end

    describe "user_point_order_datetime" do
      describe "with fixtures" do
        fixtures :orders

        it "日付フォーマットがデフォルトの場合" do
          @template.extend(ApplicationHelper)
          @template.stub!(:current_user).and_return(User.find(1001))
          order_datetime_list = []
          ec.user_point_point_list do
            order_datetime_list << ec.user_point_order_datetime
            @template.concat("")
          end
          order_datetime_list[0].should == "07/12/10"
          order_datetime_list[1].should == "07/12/10"
        end
        it "日付フォーマット指定の場合" do
          @template.extend(ApplicationHelper)
          @template.stub!(:current_user).and_return(User.find(1001))
          order_datetime_list = []
          ec.user_point_point_list do
            order_datetime_list << ec.user_point_order_datetime(:format => "%Y年%m月%d日")
            @template.concat("")
          end
          order_datetime_list[0].should == "2007年12月10日"
        end
      end
      
      it "注文がない場合 empty_formatの値が適用される" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        order_datetime_list = []
        ec.user_point_point_list do
          order_datetime_list << ec.user_point_order_datetime(:empty_format => "-")
          @template.concat("")
        end
        order_datetime_list[0].should == "-"
      end
    end
    describe "user_point_order_code" do
      it "注文がない場合の場合 empty_formatの値が適用される" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        order_code_list = []
        ec.user_point_point_list do
          order_code_list << ec.user_point_order_code(:empty_format => "-")
          @template.concat("")
        end
        order_code_list[0].should == ["-", {}]
      end      
    end
    describe "user_point_order_price" do
      it "ポイントがあるがポイントに紐づく注文がない場合 empty_formatの値が適用されること" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        order_price_list = []
        ec.user_point_point_list do
          order_price_list << ec.user_point_order_price(:empty_format => "-")
          @template.concat("")
        end
        order_price_list[0].should == "-"
      end
    end
    describe "user_point_order_product_list" do
      it "ポイントに紐づく注文がない場合 user_point_order_product_list内が実行されないこと" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        flag = true
        ec.user_point_point_list do
          ec.user_point_order_product_list do
            @template.concat("")
            flag = false
          end
          @template.concat("")
        end
        flag.should == true
      end
    end

    describe "user_point_pagination_link" do
      before do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1003))
      end
      it "user_point_list_area内で定義された場合　ページ遷移リンクが表示されること" do
        pending "willpagenate内でprivateメソッドを呼んでいて解決策不明のため保留にしています。(画面確認で動作確認要。)"
        ec.user_point_list_area(:product_separate_value => 10) do
          ec.user_point_pagination_link
        end
        @template.output_buffer.should have_tag('div[class=?]', "pagination")
      end
      it "user_point_list_area外で定義された場合　エラーになること" do
        lambda{ec.user_point_pagination_link}.should raise_error(EcRider::EcHelper::UsageError)
      end
    end
    describe "product_pagination_count_per_page" do
      before do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1003))
      end
      it "user_point_list_area内で定義された場合　ページ遷移リンクが表示されること" do
        ec.user_point_list_area(:product_separate_value => 10) do
          ec.user_point_pagination_count_per_page
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('form[action=?]', "/member/point_history") do
          with_tag("select#condition_per_page")
        end
      end
      it "user_point_list_area外で定義された場合　エラーになること" do
        lambda{ec.user_point_pagination_count_per_page}.should raise_error(EcRider::EcHelper::UsageError)
      end
    end

    describe "product_pagination_total_count" do
      it "@search_productsが1以上の場合" do
        points = UserPoint.paginate(:all, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@points, points)
        ec.user_point_pagination_total_count.should == points.total_entries
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        ec.user_point_pagination_total_count.should_not == nil
      end
    end

    describe "product_pagination_start_index" do
      it "@search_productsが1以上の場合" do
        points = UserPoint.paginate(:all, :limit => 7, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@points, points)
        ec.user_point_pagination_start_index.should == 3
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        ec.user_point_pagination_start_index.should_not == nil
      end
    end
    describe "product_pagination_last_index" do
      it "@search_productsが1以上の場合" do
        points = UserPoint.paginate(:all, :limit => 7, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@points, points)
        ec.user_point_pagination_last_index.should == 4
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        ec.user_point_pagination_last_index.should_not == nil
      end
    end

  end
end
