# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe OrderHelper do
  describe "order_count_list" do
    it "１から連番の数値が入った配列が返される" do
      product_sku = mock_model(ProductSku)
      product_sku.stub!('orderable_count').and_return(9)
      helper.order_count_list(product_sku).should == [1, 2, 3, 4, 5, 6, 7, 8, 9]
    end
  end

  describe :mail_template_list do
    it "MailTemplate#mail_type_div の:order_reply(注文問い合わせ回答メール)と:free(その他のメール)のみの選択肢" do
      order_shipping = OrderShipping.new # order_shippingはメソッド内で全く使われていません。
      helper.mail_template_list(order_shipping).should == 
        [["注文問い合わせ回答メール", "8"], ["その他のメール", "99"]]
    end
  end

  describe :shop_logo do
    fixtures :shop_settings

    it "画像あり" do
      ShopSetting.should_receive(:get_setting).with(:shop_logo, 'ja', :value => :url).and_return("http://ecrider.com/logo.png")
      ShopSetting.should_receive(:get_setting).with(:shop_name, 'ja').and_return("イーシーライダー")
      helper.shop_logo('ja').should == "<img alt=\"イーシーライダー\" src=\"http://ecrider.com/logo.png\" />"
    end
    it "画像なし" do
      ShopSetting.should_receive(:get_setting).with(:shop_logo, 'ja', :value => :url).and_return("")
      helper.shop_logo('ja').should == ""
    end
  end

  describe :grouped_products do
    fixtures :orders, :order_shippings, :order_items, 
      :product_skus, :products, :product_categories, :product_category_displays, 
      :countries
    
    it "注文に含まれる商品情報をカテゴリごとにまとめる" do
      order = Order.find(1)
      helper.grouped_products(order.order_shippings.first).should == 
        {1=>{:weight=>100, :count=>1, :amount=>1000}}
    end
  end

  describe :origin_list do
    fixtures :orders, :order_shippings, :order_items, 
      :product_skus, :product_skus, :products, :countries
    
    it "原産国のリストを生成する" do
      order = Order.find(1)
      helper.origin_list(order.order_shippings.first).should == ["Japan"]
    end
  end

  describe :product_list do
    fixtures :products, :product_displays # , :product_skus, :product_categories, :countries
    
    it "商品名のリストを生成する" do
      actual = helper.product_list
      actual.should include(["POPULAR ITEM TシャツA（白）Mサイズ （英語[en]）", "0000000000011"])
      actual.should include(["POPULAR ITEM TシャツA（白）Mサイズ （日本語[ja]）", "0000000000011"])
    end
  end

  describe "print_campaigns" do
    it "カートの場合" do
      campaigns = []
      campaigns << Campaign.new(:code => '11111', :name => 'お中元フェア')
      campaigns << Campaign.new(:code => '12345', :name => '夏を乗り切れキャンペーン')
      helper.print_campaigns(campaigns).should == '[11111] お中元フェア<br/>[12345] 夏を乗り切れキャンペーン<br/>'
    end

    it "注文の場合" do
      campaigns = []
      campaigns << OrderItemCampaign.new(:campaign_code => '11111', :campaign_name => 'お中元フェア')
      campaigns << OrderCampaign.new(:campaign_code => '12345', :campaign_name => '夏を乗り切れキャンペーン')
      helper.print_campaigns(campaigns).should == '[11111] お中元フェア<br/>[12345] 夏を乗り切れキャンペーン<br/>'
    end
  end

end
