# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductCategoriesHelper do
  fixtures :product_categories
  
  before{ login_as('admin') }

  #Delete this example and add some real ones or delete this file
  it "is included in the helper object" do
    included_modules = (class << helper; self; end).send :included_modules
    included_modules.should include(ProductCategoriesHelper)
    included_modules.should include(EcRider::ProductCategoriesHelper)
  end

  def build_test_tree
    fashion = ProductCategory.create!(:display_order => 1, :name_for_admin => 'ファッション', :url => "product_list", :code => "fashion")
    fashion.save_displays(:ja => 'ファッション', :en => 'fashion')
    fashion.children.create(:display_order => 1, :name_for_admin => 'トップス', :url => "product_list", :code => "tops").save_displays(:ja => 'トップス', :en => 'tops')
    fashion.children.create(:display_order => 2, :name_for_admin => 'ボトムス', :code => 'bottoms').save_displays(:ja => 'ボトムス', :en => 'bottoms')
    fashion.children.create(:display_order => 3, :name_for_admin => 'ゆかた', :code => 'yukata').save_displays(:ja => 'ゆかた', :en => 'yukata')
    fashion.children.create(:display_order => 4, :name_for_admin => 'バッグ', :code => 'bag').save_displays(:ja => 'バッグ', :en => 'bag')
    fashion.children.create(:display_order => 5, :name_for_admin => 'アクセサリー', :code => 'accessory').save_displays(:ja => 'アクセサリー', :en => 'accessory')
    fashion.children.create(:display_order => 6, :name_for_admin => 'ウィッグ', :code => 'wig').save_displays(:ja => 'ウィッグ', :en => 'wig')
    art = ProductCategory.create!(:display_order => 2, :name_for_admin => 'アート', :code => 'art')
    art.save_displays(:ja => 'アート', :en => 'art')
    art.children.create(:display_order => 1, :name_for_admin => 'フィギュア', :code => 'figure').save_displays(:ja => 'フィギュア', :en => 'figure')
    art.children.create(:display_order => 2, :name_for_admin => 'イラスト', :code => 'illust').save_displays(:ja => 'イラスト', :en => 'illustrations')
  end

  before(:each) do
    ProductCategory.destroy_all
    # childrenは:dependent => :destroyが設定されているので、子要素は全て削除されます。
    ProductCategory.count.should == 0
    build_test_tree
    @product_categories = ProductCategory.find(:all, :conditions => ["parent_id IS NULL"], :order => :display_order)
  end

  describe "list_trees" do
    it "カテゴリが取得できない場合は、ルート要素のみ作成される" do
      html = helper.list_trees([], :show_root => true)
      html.should have_tag('ul>li', "カテゴリTOP&nbsp;&nbsp;子要素を作成")
      html.should have_tag('a[href=?]', "/product_categories/new")
    end

    it "カテゴリのツリーを作成する" do
      html = helper.list_trees(@product_categories, :show_root => true)
      html.should have_tag('ul>li', "カテゴリTOP&nbsp;&nbsp;子要素を作成")
      html.should have_tag('ul>li>ul>li', "[tops] トップス&nbsp;&nbsp;（表示順：1）\n&nbsp;&nbsp;編集&nbsp;&nbsp;削除&nbsp;&nbsp;子要素を作成")
    end
  end

  describe "root" do
    it "カテゴリのルート要素を作成する" do
      html = helper.root
      html.should have_tag('ul>li', "カテゴリTOP&nbsp;&nbsp;子要素を作成")
      html.should have_tag('a[href=?]', "/product_categories/new")
    end
  end
end
