# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Accounting do
  fixtures :kbn_codes, :kbn_code_names,
    :order_shippings, :orders, :users, 
    :user_emails, :order_items, :products, :product_skus, :user_addresses, 
    :product_sku_inventories,:product_sku_secure_inventories,
    :kbn_code_names, :kbn_codes, :shop_settings, :mail_deliveries, :mail_templates,
    :user_points, :point_settings, :accountings
  
  describe "create_for_order" do
    it "銀行振り込み　獲得ポイントなし　利用ポイントなし" do
      order_shipping = OrderShipping.find(4)
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :order_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :order_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 0
    end
    
    it "銀行振り込み　獲得ポイントなし　利用ポイントあり" do
      Accounting.create!(
        :order_id => 1,
        :debt_sales => 10000,
        :debt_account_div_key => :order_cash,
        :credit_sales => 10000,
        :credit_account_div_key => :order_cash,
        :given_point => 300,
        :used_point => 0,
        :previous_point => 300
      )
      order_shipping = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :order_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :order_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 100
      accounting[0].previous_point.should == 200
    end
  
    it "クレジット　獲得ポイントあり　利用ポイントなし" do
      order_shipping = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :order_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :order_receivable
      accounting[0].given_point.should == 1000
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 1000
    end
  
    it "クレジット　獲得ポイントあり　利用ポイントあり" do
      order_shipping = OrderShipping.find(2)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :order_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :order_receivable
      accounting[0].given_point.should == 1000
      accounting[0].used_point.should == 100
      accounting[0].previous_point.should == 900
    end
  
    it "テスト注文は登録しない" do
      order_shipping = OrderShipping.find(2)
      order_shipping.order.test_flg = true
      order_shipping.order.save!
      lambda {
        Accounting.create_for_order(order_shipping)
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_cancel" do
    it "銀行振り込み　獲得ポイントなし　ポイント利用なし" do
      UserPoint.destroy(4)
      order_shipping = OrderShipping.find(4)
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :cancel_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :cancel_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 0
    end
    
    it "銀行振り込み　獲得ポイントなし　ポイント利用あり" do
      Accounting.create!(
        :order_id => 1,
        :debt_sales => 10000,
        :debt_account_div_key => :cancel_receivable,
        :credit_sales => 10000,
        :credit_account_div_key => :cancel_receivable,
        :given_point => 300,
        :used_point => 0,
        :previous_point => 300
      )
      order_shipping = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :cancel_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :cancel_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == -100
      accounting[0].previous_point.should == 400
    end
  
    it "クレジット　ポ獲得ポイントあり　イント利用なし" do
      order_shipping = OrderShipping.find(2)
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :cancel_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :cancel_receivable
      accounting[0].given_point.should == -1000
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == -1000
    end
  
    it "クレジット　獲得ポイントあり　ポイント利用あり" do
      order_shipping = OrderShipping.find(2)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :cancel_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :cancel_receivable
      accounting[0].given_point.should == -1000
      accounting[0].used_point.should == -100
      accounting[0].previous_point.should == -900
    end
  
    it "テスト注文は登録しない" do
      order_shipping = OrderShipping.find(2)
      order_shipping.order.test_flg = true
      order_shipping.order.save!
      lambda {
        Accounting.create_for_order(order_shipping)
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_point_summary" do
    it "キャンセル含まない" do
      Time.stub!(:now).and_return(Time.local(2010, 1, 1, 0, 10, 00))
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      order_shipping_3 = OrderShipping.find(3)
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 1000, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_3)
      order_shipping_4 = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping_4.order.user_id, :order_id => order_shipping_4.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 400, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_4)
      order_shipping_9 = OrderShipping.find(9)
      UserPoint.create!(
        :user_id => order_shipping_9.order.user_id, :order_id => order_shipping_9.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 600, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_9)
      Accounting.count.should == 4
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      Accounting.create_for_point_summary
      Accounting.count.should == 5
      expected_point_summary = 1000 + 1000 + 400 + 600
      accounting = Accounting.find(:last, :order => "id")
      accounting.debt_sales.should == expected_point_summary * 10
      accounting.debt_account_div_key.should == :point_summary
      accounting.credit_sales.should == expected_point_summary * 10
      accounting.credit_account_div_key.should == :point_summary
      accounting.given_point.should == 0
      accounting.used_point.should == 0
      accounting.previous_point.should == expected_point_summary
    end
    
    it "キャンセル含む" do
      Time.stub!(:now).and_return(Time.local(2010, 1, 1, 0, 10, 00))
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      order_shipping_3 = OrderShipping.find(3)
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 1000, :used_point => 300
      )
      Accounting.create_for_order(order_shipping_3)
      order_shipping_4 = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping_4.order.user_id, :order_id => order_shipping_4.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 400, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_4)
      order_shipping_9 = OrderShipping.find(9)
      UserPoint.create!(
        :user_id => order_shipping_9.order.user_id, :order_id => order_shipping_9.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 600, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_9)
      Accounting.create_for_cancel(order_shipping_3)
      Accounting.count.should == 5
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      Accounting.create_for_point_summary
      expected_point_summary = 1000 + 1000 - 300 + 400 + 600 - 1000 + 300
      accounting = Accounting.find(:last, :order => "id")
      accounting.debt_sales.should == expected_point_summary * 10
      accounting.debt_account_div_key.should == :point_summary
      accounting.credit_sales.should == expected_point_summary * 10
      accounting.credit_account_div_key.should == :point_summary
      accounting.given_point.should == 0
      accounting.used_point.should == 0
      accounting.previous_point.should == expected_point_summary
    end
    
    it "キャンセル含む　一度マイナスになる" do
      Time.stub!(:now).and_return(Time.local(2010, 1, 1, 0, 10, 00))
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      order_shipping_3 = OrderShipping.find(3)
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 0, :used_point => 300
      )
      Accounting.create_for_order(order_shipping_3)
      Accounting.find(:first, :order => 'id desc').previous_point.should == 700
      Accounting.create_for_cancel(order_shipping_2)
      Accounting.find(:first, :order => 'id desc').previous_point.should == -300
      order_shipping_9 = OrderShipping.find(9)
      UserPoint.create!(
        :user_id => order_shipping_9.order.user_id, :order_id => order_shipping_9.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 600, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_9)
      Accounting.count.should == 4
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      Accounting.create_for_point_summary
      expected_point_summary = 600
      accounting = Accounting.find(:last, :order => "id")
      accounting.debt_sales.should == expected_point_summary * 10
      accounting.debt_account_div_key.should == :point_summary
      accounting.credit_sales.should == expected_point_summary * 10
      accounting.credit_account_div_key.should == :point_summary
      accounting.given_point.should == 0
      accounting.used_point.should == 0
      accounting.previous_point.should == expected_point_summary
    end
    
    it "ポイント制度を利用していない場合は登録しない" do
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = false
      point_setting.save!
      lambda {
        Accounting.create_for_point_summary
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_point_expire" do
    it "ポイント期限切れ" do
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.create!(:user_id => 1001, :order_id => 2, :term_date => Date.today, :given_point => -1000, :used_point => 0)
      Accounting.create_for_point_expire(user_point)
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 2
      accounting[1].debt_sales.should == 1000 * 10
      accounting[1].debt_account_div_key.should == :point_expired
      accounting[1].credit_sales.should == 1000 * 10
      accounting[1].credit_account_div_key.should == :point_expired
      accounting[1].given_point.should == -1000
      accounting[1].used_point.should == 0
      accounting[1].previous_point.should == 0
    end
  end
  
  describe "create_for_point_create" do
    it "ポイント追加" do
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.create!(:user_id => 1001, :term_date => Date.today, :given_point => 500)
      Accounting.create_for_point_create(user_point)
      accounting = Accounting.find(:first, :order => "id desc")
      accounting.debt_sales.should == 500 * 10
      accounting.debt_account_div_key.should == :point_created
      accounting.credit_sales.should == 500 * 10
      accounting.credit_account_div_key.should == :point_created
      accounting.given_point.should == 500
      accounting.used_point.should == 0
      accounting.previous_point.should == 1500
    end
  end
  
  describe "create_for_point_update" do
    it "ポイント変更　付与ポイントを増やす" do
      order_shipping_2 = OrderShipping.find(2)
      order_shipping_2.status_div_key = :delivered
      order_shipping_2.save!
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      user_point.given_point = 1100
      user_point.used_point = 0
      user_point.save
      Accounting.create_for_point_update(user_point)
      expected_point_summary = 1100 - 1000 - 0
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 2
      accounting[1].order_id.should == 2
      accounting[1].debt_sales.should == expected_point_summary * 10
      accounting[1].debt_account_div_key.should == :point_updated
      accounting[1].credit_sales.should == expected_point_summary * 10
      accounting[1].credit_account_div_key.should == :point_updated
      accounting[1].given_point.should == 100
      accounting[1].used_point.should == 0
      accounting[1].previous_point.should == 1100
    end
    
    it "ポイント追加　付与ポイントを減らす" do
      order_shipping_2 = OrderShipping.find(2)
      order_shipping_2.status_div_key = :delivered
      order_shipping_2.save!
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      user_point.given_point = 800
      user_point.used_point = 100
      user_point.save
      Accounting.create_for_point_update(user_point)
      expected_point_summary = 800 - 1000 - 100
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 2
      accounting[1].order_id.should == 2
      accounting[1].debt_sales.should == expected_point_summary * 10
      accounting[1].debt_account_div_key.should == :point_updated
      accounting[1].credit_sales.should == expected_point_summary * 10
      accounting[1].credit_account_div_key.should == :point_updated
      accounting[1].given_point.should == -200
      accounting[1].used_point.should == 100
      accounting[1].previous_point.should == 700
    end
    
    it "キャンセルの場合は登録しない" do
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      accounting = Accounting.new(
        :order_id => 2,
        :user_point_id => user_point.id,
        :debt_account_div_key => :cancel_cash,
        :debt_sales => 1000,
        :credit_account_div_key => :cancel_cash,
        :credit_sales => 1000,
        :given_point => -100,
        :used_point => -20
      )
      accounting.save!
      user_point.given_point = 1100
      user_point.used_point = 0
      user_point.save
      lambda {
        Accounting.create_for_point_update(user_point)
      }.should change(Accounting, :count).by(0)
    end
    it "ポイント期限切れのポイント編集の場合は登録しない" do
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      user_point.given_point = 1100
      user_point.used_point = 0
      user_point.expired_flg = true
      user_point.save
      accounting = Accounting.new(
        :order_id => 2,
        :user_point_id => user_point.id,
        :debt_account_div_key => :order_cash,
        :debt_sales => 1000,
        :credit_account_div_key => :order_cash,
        :credit_sales => 1000,
        :given_point => -100,
        :used_point => -20
      )
      lambda {
        Accounting.create_for_point_update(user_point)
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_point_destroy" do
    it "ポイント削除 注文に紐づくポイントの場合" do
      order_shipping_2 = OrderShipping.find(2)
      order_shipping_2.status_div_key = :delivered
      order_shipping_2.save!
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      Accounting.create_for_point_destroy(user_point)
      accounting = Accounting.find(:all, :conditions => ["order_id = 2"], :order => "id")
      accounting.length.should == 2
      accounting[0].order_id.should == 2
      accounting[1].debt_sales.should == -(1000 * 10)
      accounting[1].debt_account_div_key.should == :point_destroyed
      accounting[1].credit_sales.should == -(1000 * 10)
      accounting[1].credit_account_div_key.should == :point_destroyed
      accounting[1].given_point.should == -1000
      accounting[1].used_point.should == 0
      accounting[1].previous_point.should == 0
    end
    
    it "ポイント削除 注文に紐づかないポイントの場合 ポイントがAccountingにマイナスポイントで登録されること" do
      user_point = UserPoint.find_by_id(14)
      Accounting.create_for_point_create(user_point)
      Accounting.create_for_point_destroy(user_point)
      accounting = Accounting.find(:all, :order => "id")
      accounting.length.should == 2
      accounting[1].order_id.should == nil
      accounting[1].debt_sales.should == -(200 * 10)
      accounting[1].debt_account_div_key.should == :point_destroyed
      accounting[1].credit_sales.should == -(200 * 10)
      accounting[1].credit_account_div_key.should == :point_destroyed
      accounting[1].given_point.should == -200
      accounting[1].used_point.should == 0
      accounting[1].previous_point.should == 0
    end

    it "ポイント削除 前データが登録されていない場合は登録しない" do
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      Accounting.create_for_point_destroy(user_point)
      accounting = Accounting.find(:all, :conditions => ["order_id = 2"], :order => "id")
      accounting.length.should == 0
    end

    it "ポイント削除 前データが注文キャンセルの場合は登録しない" do
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      accounting = Accounting.new(
        :order_id => 2,
        :user_point_id => user_point.id,
        :debt_account_div_key => :cancel_cash,
        :debt_sales => 1000,
        :credit_account_div_key => :cancel_cash,
        :credit_sales => 1000,
        :given_point => -100,
        :used_point => -20
      )
      accounting.save!
      lambda {
        Accounting.create_for_point_destroy(user_point)
      }.should change(Accounting, :count).by(0)
    end

    it "ポイント削除 削除するポイントが有効期限切れのもの場合は登録しない" do
      user_point = UserPoint.find(:first)
      user_point.expired_flg = true
      accounting = Accounting.new(
        :order_id => 2,
        :user_point_id => user_point.id,
        :debt_account_div_key => :order_cash,
        :debt_sales => 1000,
        :credit_account_div_key => :order_cash,
        :credit_sales => 1000,
        :given_point => -100,
        :used_point => -20
      )
      accounting.save!
      lambda {
        Accounting.create_for_point_destroy(user_point)
      }.should change(Accounting, :count).by(0)
    end
  end
  describe "to_find_options" do
    it "to_find_options" do
      condition = Accounting::SearchInfo.new(:start_date => '2010/2/16', :end_date => '2010/2/26', :none_output_data => "1", :previous_point => "1")
      option = {:page => 1, :per_page => 10}
      c = condition.to_find_options(option)
      c[:page].should == 1
      c[:per_page].should == 10
      c[:conditions].should == ["accountings.created_at >= ? AND accountings.created_at <= ? AND accountings.output_data_flg != ? AND (accountings.credit_account_div != ? AND accountings.credit_account_div != ? AND accountings.credit_account_div != ? AND accountings.credit_account_div != ?)",
      Time.parse('2010/2/16 00:00:00'),Time.parse('2010/2/26 23:59:59'), 1, "06", "07", "08", "09"
      ]
    end
    it "to_find_options date for nil" do
      condition = Accounting::SearchInfo.new(:start_date => "", :end_date => "", :none_output_data => "0", :previous_point => "0")
      option = {:page => 1, :per_page => 10}
      c = condition.to_find_options(option)
      c[:page].should == 1
      c[:per_page].should == 10
      c[:conditions].should == ["accountings.credit_account_div != ? AND (accountings.credit_account_div != ? AND accountings.credit_account_div != ? AND accountings.credit_account_div != ? AND accountings.credit_account_div != ?)", "05", "06", "07", "08", "09"]
    end
  end

  describe "account_div_code, account_div_name" do
    it "debt" do
      result = Accounting.new(:debt_account_div => Accounting.debt_account_div_id_by_key(:order_cash))
      result.debt_account_div_code.should == 100
      result.debt_account_div_output_name.should == "現金"
      result.debt_account_div = Accounting.debt_account_div_id_by_key(:order_receivable)
      result.debt_account_div_code.should == 135
      result.debt_account_div_output_name.should == "売掛金"
      result.debt_account_div = Accounting.debt_account_div_id_by_key(:cancel_cash)
      result.debt_account_div_code.should == 500
      result.debt_account_div_output_name.should == "売上高"
      result.debt_account_div = Accounting.debt_account_div_id_by_key(:cancel_receivable)
      result.debt_account_div_code.should == 500
      result.debt_account_div_output_name.should == "売上高"
      result.debt_account_div = Accounting.debt_account_div_id_by_key(:point_summary)
      result.debt_account_div_code.should == 999
      result.debt_account_div_output_name.should == "引当金繰入額"
      result.debt_account_div = Accounting.debt_account_div_id_by_key(:pay_back_cash)
      result.debt_account_div_code.should == 500
      result.debt_account_div_output_name.should == "売上高"
      result.debt_account_div = Accounting.debt_account_div_id_by_key(:pay_back_receivable)
      result.debt_account_div_code.should == 500
      result.debt_account_div_output_name.should == "売上高"
    end
    it "credit" do
      result = Accounting.new(:credit_account_div => Accounting.credit_account_div_id_by_key(:order_cash))
      result.credit_account_div_code.should == 500
      result.credit_account_div_output_name.should == "売上高"
      result.credit_account_div = Accounting.debt_account_div_id_by_key(:order_receivable)
      result.credit_account_div_code.should == 500
      result.credit_account_div_output_name.should == "売上高"
      result.credit_account_div = Accounting.debt_account_div_id_by_key(:cancel_cash)
      result.credit_account_div_code.should == 100
      result.credit_account_div_output_name.should == "現金"
      result.credit_account_div = Accounting.debt_account_div_id_by_key(:cancel_receivable)
      result.credit_account_div_code.should == 135
      result.credit_account_div_output_name.should == "売掛金"
      result.credit_account_div = Accounting.debt_account_div_id_by_key(:point_summary)
      result.credit_account_div_code.should == 999
      result.credit_account_div_output_name.should == "ポイント引当金"
      result.credit_account_div = Accounting.debt_account_div_id_by_key(:pay_back_cash)
      result.credit_account_div_code.should == 100
      result.credit_account_div_output_name.should == "現金"
      result.credit_account_div = Accounting.debt_account_div_id_by_key(:pay_back_receivable)
      result.credit_account_div_code.should == 135
      result.credit_account_div_output_name.should == "売掛金"
    end
    it "remark" do
      order = Order.new(:ec_order_code => "testcode")
      order.order_shippings << OrderShipping.new(:status_div => OrderShipping.status_div_id_by_key(:delivered))
      account = Accounting.new(:debt_account_div => "01",:debt_sales => 100, :credit_account_div => "01",:credit_sales => 100)
      account.order = order
      account.remark.should == "testcode(注文)"

      order = Order.new(:ec_order_code => "testcode")
      order.order_shippings << OrderShipping.new(:status_div => OrderShipping.status_div_id_by_key(:deliver_advised))
      account = Accounting.new(:debt_account_div => "01",:debt_sales => 100, :credit_account_div => "01",:credit_sales => 100)
      account.order = order
      account.remark.should == "testcode(注文)"

      order = Order.new(:ec_order_code => "testcode")
      order.order_shippings << OrderShipping.new(:status_div => OrderShipping.status_div_id_by_key(:canceled))
      account = Accounting.new(:debt_account_div => "01",:debt_sales => 100, :credit_account_div => "01",:credit_sales => 100)
      account.order = order
      account.remark.should == "testcode(キャンセル)"

      account = Accounting.new(:debt_account_div => "05",:debt_sales => 100, :credit_account_div => "05",:credit_sales => 100)
      account.remark.should == "ポイント集計"
    end
  end
end
