require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe BannerCondition do
  before(:each) do
    @valid_attributes = {
      :banner_id => 1,
      :type_cd => "value for type_cd",
      :operator_cd => "value for operator_cd",
      :value => "value for value",
      :weight_point => 1
    }
  end

  it "should create a new instance given valid attributes" do
    pending
    BannerCondition.create!(@valid_attributes)
  end

  describe "calc_weight_point" do
    it "calc_weight_point by eql" do
      cond = BannerCondition.new(:type_key => :user_gender, :operator_key => :eql, :value => '1', :weight_point => '10')
      cond.calc_weight_point(User.new(:sex_key => :male)).should == 10
      cond.calc_weight_point(User.new(:sex_key => :female)).should == 0
    end

    it "calc_weight_point by not" do
      cond = BannerCondition.new(:type_key => :user_gender, :operator_key => :not_eql, :value => '2', :weight_point => '10')
      cond.calc_weight_point(User.new(:sex_key => :male)).should == 10
      cond.calc_weight_point(User.new(:sex_key => :female)).should == 0
    end
  end
  describe "build_script" do
    # 性別
    describe "for user_gender" do
      it "build_script by eql" do
        cond = BannerCondition.new(:type_key => :user_gender, :operator_key => :eql, :value => '1')
        cond.evaluation_banner_condition(User.new(:sex_key => :male)).should == true
        cond.evaluation_banner_condition(User.new(:sex_key => :female)).should == false
      end
      it "build_script by not" do
        cond = BannerCondition.new(:type_key => :user_gender, :operator_key => :not_eql, :value => '1')
        cond.evaluation_banner_condition(User.new(:sex_key => :male)).should == false
        cond.evaluation_banner_condition(User.new(:sex_key => :female)).should == true
      end
    end
    # 年齢
    describe "for :user_age" do
      it "build_script by lte user_age" do
        cond = BannerCondition.new(:type_key => :user_age, :operator_key => :lte, :value => '10')
        user = User.new()
        user.stub!(:birthday).and_return('2000/03/22')
        user.stub!(:age).and_return(9)
        cond.evaluation_banner_condition(user).should == true
        user.stub!(:age).and_return(10)
        cond.evaluation_banner_condition(user).should == true
        user.stub!(:age).and_return(11)
        cond.evaluation_banner_condition(user).should == false
      end
      it "build_script by gte user_age" do
        cond = BannerCondition.new(:type_key => :user_age, :operator_key => :gte, :value => '10')
        user = User.new()
        user.stub!(:birthday).and_return('2000/03/22')
        user.stub!(:age).and_return(9)
        cond.evaluation_banner_condition(user).should == false
        user.stub!(:age).and_return(10)
        cond.evaluation_banner_condition(user).should == true
        user.stub!(:age).and_return(11)
        cond.evaluation_banner_condition(user).should == true
      end
    end
    # 職業
    describe "for :user_occupation" do
      it "build_script by eql user_occupation" do
        cond = BannerCondition.new(:type_key => :user_occupation, :operator_key => :eql, :value => '1')
        cond.evaluation_banner_condition(User.new(:job_div => '1')).should == true
        cond.evaluation_banner_condition(User.new(:job_div => '2')).should == false
      end
      it "build_script by not user_occupation" do
        cond = BannerCondition.new(:type_key => :user_occupation, :operator_key => :not_eql, :value => '1')
        cond.evaluation_banner_condition(User.new(:job_div => '1')).should == false
        cond.evaluation_banner_condition(User.new(:job_div => '2')).should == true
      end
    end
    # 生年月日
    describe "for :user_occupation" do
      fixtures :users
      it "build_script by eql user_occupation" do
        cond = BannerCondition.new(:type_key => :user_birthday, :operator_key => :eql, :value => '1981/11/15')
        cond.evaluation_banner_condition(User.find_by_id(1019)).should == true
        cond.evaluation_banner_condition(User.find_by_id(1018)).should == false
      end
      it "build_script by not user_occupation" do
        cond = BannerCondition.new(:type_key => :user_birthday, :operator_key => :not_eql, :value => '1981/11/15')
        cond.evaluation_banner_condition(User.find_by_id(1019)).should == false
        cond.evaluation_banner_condition(User.find_by_id(1018)).should == true
      end
    end
    # 住所
    describe "for user_prefecture" do
      fixtures :users, :user_addresses, :prefectures
      it "build_script by eql user_prefecture" do
        cond = BannerCondition.new(:type_key => :user_prefecture, :operator_key => :eql, :value => '福岡県')
        cond.evaluation_banner_condition(User.find_by_id(1000)).should == true
        cond.evaluation_banner_condition(User.find_by_id(1002)).should == false
      end
      it "build_script by not user_prefecture" do
        cond = BannerCondition.new(:type_key => :user_prefecture, :operator_key => :not_eql, :value => '福岡県')
        cond.evaluation_banner_condition(User.find_by_id(1000)).should == false
        cond.evaluation_banner_condition(User.find_by_id(1002)).should == true
      end
    end

    # 保有ポイント
    describe "for user_point" do
      fixtures :users, :user_points
      it "build_script by lt user_point" do
        cond = BannerCondition.new(:type_key => :user_point, :operator_key => :lt, :value => '500')
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == false
        cond.evaluation_banner_condition(User.find_by_id(1002)).should == true
      end
      it "build_script by gt user_point" do
        cond = BannerCondition.new(:type_key => :user_point, :operator_key => :gt, :value => '600')
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == true
        cond.evaluation_banner_condition(User.find_by_id(1021)).should == false
      end
    end
    # 国
    describe "for user_country" do
      fixtures :users, :user_addresses
      it "build_script by eql user_country" do
        cond = BannerCondition.new(:type_key => :user_country, :operator_key => :eql, :value => 'JP')
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == false
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == true
      end
      it "build_script by not_eql user_country" do
        cond = BannerCondition.new(:type_key => :user_country, :operator_key => :not_eql, :value => 'JP')
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == false
      end
    end

    # 発送日
    describe "for :shipping_date" do
      fixtures :users, :user_addresses
      it "build_script by eql :shipping_date" do
      pending "配送日時の仕様決定後まで保留"
        cond = BannerCondition.new(:type_key => :shipping_date, :operator_key => :eql, :value => 'JP')
        cond.build_script.should == "user.user_address.country_code == 'JP'"
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == false
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == true
      end
      it "build_script by not_eql :shipping_date" do
      pending "配送日時の仕様決定後まで保留"
        cond = BannerCondition.new(:type_key => :shipping_date, :operator_key => :not_eql, :value => 'JP')
        cond.build_script.should == "user.user_address.country_code != 'JP'"
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == false
      end
    end

    # システム日付
    describe "for :system_date" do
      fixtures :users
      it "build_script by lte :system_date" do
        cond = BannerCondition.new(:type_key => :system_date, :operator_key => :lte, :value => '2009/03/26')
        Date.stub!(:today).and_return(Date.new(2009,3,25))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        Date.stub!(:today).and_return(Date.new(2009,3,26))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        Date.stub!(:today).and_return(Date.new(2009,3,27))
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == false
      end
      it "build_script by gte :system_date" do
        cond = BannerCondition.new(:type_key => :system_date, :operator_key => :gte, :value => '2009/03/26')
        Date.stub!(:today).and_return(Date.new(2009,3,25))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == false
        Date.stub!(:today).and_return(Date.new(2009,3,26))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        Date.stub!(:today).and_return(Date.new(2009,3,27))
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == true
      end
      it "build_script by eql :system_date" do
        cond = BannerCondition.new(:type_key => :system_date, :operator_key => :eql, :value => '2009/03/26')
        Date.stub!(:today).and_return(Date.new(2009,3,25))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == false
        Date.stub!(:today).and_return(Date.new(2009,3,26))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        Date.stub!(:today).and_return(Date.new(2009,3,27))
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == false
      end
      it "build_script by not_eql :system_date" do
        cond = BannerCondition.new(:type_key => :system_date, :operator_key => :not_eql, :value => '2009/03/26')
        Date.stub!(:today).and_return(Date.new(2009,3,25))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == true
        Date.stub!(:today).and_return(Date.new(2009,3,26))
        cond.evaluation_banner_condition(User.find_by_id(1024)).should == false
        Date.stub!(:today).and_return(Date.new(2009,3,27))
        cond.evaluation_banner_condition(User.find_by_id(1001)).should == true
      end
    end
  end

  describe "display_value_for_type" do
    fixtures :kbn_codes, :kbn_code_names, :countries
    it "職業の場合" do
      banner_cond = BannerCondition.new(:type_key => :user_occupation, :value => '1')
      banner_cond.display_value_for_type.should == '会社員'
    end

    it "性別の場合" do
      banner_cond = BannerCondition.new(:type_key => :user_gender, :value => '1')
      banner_cond.display_value_for_type.should == '男性'
    end

    it "国の場合" do
      banner_cond = BannerCondition.new(:type_key => :user_country, :value => 'AF')
      banner_cond.display_value_for_type.should == 'Afghanistan'
    end

    it "その他の場合" do
      banner_cond = BannerCondition.new(:type_key => :user_age, :value => '11')
      banner_cond.display_value_for_type.should == '11'
      banner_cond.type_key = :user_birthday
      banner_cond.display_value_for_type.should == '11'
      banner_cond.type_key = :user_prefecture
      banner_cond.display_value_for_type.should == '11'
      banner_cond.type_key = :user_point
      banner_cond.display_value_for_type.should == '11'
      banner_cond.type_key = :system_date
      banner_cond.display_value_for_type.should == '11'
      banner_cond.type_key = :random
      banner_cond.display_value_for_type.should == '11'
    end
  end
end
