# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
describe Banner do
  fixtures :banners, :banner_conditions, :banner_displays, :users, :user_points, :cms_publishes

  describe "validate" do
    
    it "新規登録画面で何も入力しないで作成した場合" do
      banner = Banner.new(
        "code"=>"", 
        "type_cd"=>"1", 
        "link_url"=>"", 
        "display_order"=>"", 
        "publish_started_at(1i)"=>"2009", 
        "publish_started_at(2i)"=>"9", 
        "publish_started_at(3i)"=>"3", 
        "publish_started_at(4i)"=>"19", 
        "publish_started_at(5i)"=>"49", 
        "publish_finished_at(1i)"=>"2009", 
        "publish_finished_at(2i)"=>"9", 
        "publish_finished_at(3i)"=>"3", 
        "publish_finished_at(4i)"=>"19", 
        "publish_finished_at(5i)"=>"50", 
        "displays_attributes"=>{
          "0"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ja"}, 
          "1"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ko"}, 
          "2"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-CN"}, 
          "3"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-TW"}, 
          "4"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"en"}, 
          "5"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"th"}
        }, 
        "conditions_attributes"=>{
          "6"=>{
            "value"=>"", "operator_cd"=>"01", "type_cd"=>"01"}
        }
      )
      banner.valid?.should == false
      attrs = banner.errors.map{|(attr, msg)| attr.to_s}.sort
      attrs.should == %w(code displays_image_file conditions_value).sort
      banner.errors.on(:code).should == "を入力してください。"
      banner.errors.on(:link_url).should_not == "を入力してください。"
      banner.errors.on(:displays_image_file).should == "を入力してください。"
    end

    it "新規登録画面で何も入力しないで作成した場合(type_key == :image)" do
      banner = Banner.new(
        "code"=>"12345", 
        "type_cd"=>"1", 
        "link_url"=>"http://www.ecrider.ec-one.com", 
        "display_order"=>"", 
        "publish_started_at(1i)"=>"2009", 
        "publish_started_at(2i)"=>"9", 
        "publish_started_at(3i)"=>"3", 
        "publish_started_at(4i)"=>"19", 
        "publish_started_at(5i)"=>"49", 
        "publish_finished_at(1i)"=>"2009", 
        "publish_finished_at(2i)"=>"9", 
        "publish_finished_at(3i)"=>"3", 
        "publish_finished_at(4i)"=>"19", 
        "publish_finished_at(5i)"=>"50", 
        "displays_attributes"=>{
          "0"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ja"}, 
          "1"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ko"}, 
          "2"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-CN"}, 
          "3"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-TW"}, 
          "4"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"en"}, 
          "5"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"th"}
        }, 
        "conditions_attributes"=>{
        }
      )
      banner.valid?.should == false
      attrs = banner.errors.map{|(attr, msg)| attr.to_s}.sort
      attrs.should == %w(displays_image_file).sort
      banner.errors.on(:displays_image_file).should == "を入力してください。"
    end

    it "新規登録画面で何も入力しないで作成した場合(type_key == :text)" do
      banner = Banner.new(
        "code"=>"12345", 
        "type_cd"=>"2", 
        "link_url"=>"http://www.ecrider.ec-one.com", 
        "display_order"=>"", 
        "publish_started_at(1i)"=>"2009", 
        "publish_started_at(2i)"=>"9", 
        "publish_started_at(3i)"=>"3", 
        "publish_started_at(4i)"=>"19", 
        "publish_started_at(5i)"=>"49", 
        "publish_finished_at(1i)"=>"2009", 
        "publish_finished_at(2i)"=>"9", 
        "publish_finished_at(3i)"=>"3", 
        "publish_finished_at(4i)"=>"19", 
        "publish_finished_at(5i)"=>"50", 
        "displays_attributes"=>{
          "0"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ja"}, 
          "1"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ko"}, 
          "2"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-CN"}, 
          "3"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-TW"}, 
          "4"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"en"}, 
          "5"=>{
            "text_description"=>"", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"th"}
        }, 
        "conditions_attributes"=>{
        }
      )
      banner.valid?.should == false
      attrs = banner.errors.map{|(attr, msg)| attr.to_s}.sort
      attrs.should == %w(displays_text_description).sort
      banner.errors.on(:displays_text_description).should == "を入力してください。"
    end

    it "正しく保存できる場合" do
      banner = Banner.new(
        "code"=>"12345", 
        "type_cd"=>"2", 
        "link_url"=>"http://www.ec-one.com", 
        "display_order"=>"", 
        "publish_started_at(1i)"=>"2009", 
        "publish_started_at(2i)"=>"9", 
        "publish_started_at(3i)"=>"3", 
        "publish_started_at(4i)"=>"19", 
        "publish_started_at(5i)"=>"49", 
        "publish_finished_at(1i)"=>"2009", 
        "publish_finished_at(2i)"=>"9", 
        "publish_finished_at(3i)"=>"5", 
        "publish_finished_at(4i)"=>"19", 
        "publish_finished_at(5i)"=>"50", 
        "displays_attributes"=>{
          "0"=>{
            "text_description"=>"日本語バナー記述", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ja"}, 
          "1"=>{
            "text_description"=>"韓国語バナー記述", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"ko"}, 
          "2"=>{
            "text_description"=>"繁体語バナー記述", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-CN"}, 
          "3"=>{
            "text_description"=>"簡体語バナー記述", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"zh-TW"}, 
          "4"=>{
            "text_description"=>"英語バナー記述", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"en"}, 
          "5"=>{
            "text_description"=>"タイ語バナー記述", "image_file"=>"", "text_title"=>"01",
            "image_alt"=>"", "locale"=>"th"}
        }, 
        "conditions_attributes"=>{
          "6"=>{
            "value"=>"12", "operator_cd"=>"01", "type_cd"=>"01"}
        }
      )
      banner.valid?.should == true
      banner.errors.empty?.should == true
    end

  end

  describe Banner::SearchForm do
    describe "to_paginate_options" do
      after(:each) do
        # エラーが出ないかどうかを確認します
        Banner.paginate(@form.to_paginate_options)
      end
      
      it "default without params" do
        @form = Banner::SearchForm.new
        @form.to_paginate_options.should == {
          :per_page => 20,
          :page => 1,
          :order => "code asc, type_cd asc, created_at asc"
        }
      end

      it "default with params" do
        @form = Banner::SearchForm.new("disp_count" => "20", 
          "code" => "", 
          "link_url" => "",
          "publish_started_at" => "",  
          "publish_finished_at" => "")
        @form.to_paginate_options.should == {
          :per_page => 20,
          :page => 1,
          :order => "code asc, type_cd asc, created_at asc"
        }
      end

      it "all params" do
        @form = Banner::SearchForm.new("disp_count" => "50",
          "code" => "123",
          "publish_started_at" => "2009/9/1",
          "publish_finished_at" => "2009/9/3",
          "link_url" => "http://campaign.ecrider.com")
        
        @form.to_paginate_options.should == {
          :conditions => [
            "code like ? AND publish_started_at >= ? AND publish_finished_at <= ? AND link_url like ?",
            "%123%",
            Time.parse("2009/09/01 00:00:00"),
            Time.parse("2009/09/03 23:59:59"),
            "%http://campaign.ecrider.com%"
          ],
          :per_page => 50,
          :page => 1,
          :order => "code asc, type_cd asc, created_at asc"
        }
      end
    end


  end
  describe "calc_total_weight_point" do
    it "calc_total_weight_point" do
      user = User.new(:sex_key => :female, :job_div => '1', :birth_year => 1926, :birth_month => 3, :birth_day => 1)
      banner = Banner.find_by_id(2)
      result = banner.calc_total_weight_point(user)
      result.should == 40
    end
  end
  describe "conditions_in_random?" do
    it "conditions_in_random?" do
      banner = Banner.find_by_id(1)
      banner.conditions_in_random?.should == false
      banner = Banner.find_by_id(3)
      banner.conditions_in_random?.should == true
    end
  end

  describe "banner_group" do
    it "バナー登録件数が表示件数よりすくない場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,11,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000001", user, 3)
      result.size.should == 2
      result[0].id.should == 2
      result[1].id.should == 4
    end
    it "バナー登録件数が表示件数より多い場合 ランダムバナーのみ" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,11,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000004", user, 3)
      result.size.should == 3
    end
    it "ソート + ランダムバナー" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,11,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000001", user, 3)
      result.size.should == 2
    end
    it "指定したコードが存在しない場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,11,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000xx1", user, 3)
      result.size.should == 0
    end
    it "並び順テスト" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,12,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000002", user, 10)
      result.size.should == 6
      result[0].id.should == 10
      result[1].id.should == 13
      result[2].id.should == 12
      result[3].id.should == 11
      result[4].id.should == 9
      result[5].id.should == 14
    end
    it "テスト環境 テスト対象日で検索されること" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      Time.stub!(:now).and_return(Time.local(2009,11,12,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000002", user, 10)
      result.size.should == 5
    end
    it "ec_admin 全バナー検索されること" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,12,17,05,00))
      user = User.find_by_id(1013)
      result = Banner.banner_group("00000002", user, 10)
      result.size.should == 6
    end
  end
end
