# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

ActiveSupport::Deprecation.debug = true

describe Cart do

  describe "キャンペーンなし" do

    fixtures :products, #:product_discounts, :shipping_costs,
    :product_displays, :product_skus, :product_sku_inventories, :product_sku_secure_inventories,
      :user_points, :point_settings, :product_prices, :users, :user_emails, :user_addresses

    before(:each) do
      @cart = Cart.new

      @product12 = ProductSku.find(276)
      @key12 = Cart::Item.key_for(@product12.sku_code, @product12.product.course_div_key)
      @product13 = ProductSku.find(277)
      @key13 = Cart::Item.key_for(@product13.sku_code, @product13.product.course_div_key)
    end

    describe "deta test" do
      it "12" do
        @product12.should == ProductSku.find(276)
      end
      it "12" do
        @product13.should == ProductSku.find(277)
      end
    end
    describe "empty?" do
      it "should be return ture when products is empty." do
        @cart.empty?.should be_true
      end
    end

    describe "normal_deli_total_price" do
      it "通常送付の総額(送料は含みません)" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 2)
        @cart.normal_deli_total_price.should == @product12.price.to_i + @product13.price.to_i * 2
      end
    end

    describe "total_qty" do
      it "総数量を返します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 2)
        @cart.total_qty.should == 3
      end
    end

    describe "total_shipping_cost_point" do
      it "総数量を返します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 2)
        @cart.total_shipping_cost_point.should == 500
      end
    end

    describe "campaign_type_include?" do
      it "該当するキャンペーンが含まれる場合" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 2)
        @cart.campaign_type_include?(:shipping_cost).should == false
      end
      it "該当するキャンペーンが含まれない場合" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 2)
        @cart.campaign_type_include?(:shipping_cost).should == false
      end
    end

    describe "item_for" do
      it "同一の商品SKUがすでにカートに存在する場合その商品SKUを返します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.item_for(@product12.sku_code, @product12.product.course_div_key).product_sku.sku_code.should == '0000000009558-00-00-00'
        @cart.item_for('0', '0').should == nil
      end
    end

    describe "total_amount" do
      it "総額は、総金額 + 総送料" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 2)
        @cart.calculate
        @cart.total_amount.should == @product12.price.to_i + @product13.price.to_i * 2
      end
    end

    describe "total_qty" do
      it "通常送付の数量" do
        @cart.total_qty.should == 0
      end
    end

    describe "add" do
      it "SKUコードを指定してカートに商品を追加します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product12.sku_code, @product12.product.course_div_key, 1)
        @cart.items[@key12].qty.should == 2

        @cart.add(@product13.sku_code, @product13.product.course_div_key, 1)
        @cart.items[@key12].qty.should == 2
        @cart.items[@key13].qty.should == 1
      end

      it "SKUを指定してカートに商品を追加します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.items[@key12].qty.should == 2

        @cart.add(@product13, @product13.product.course_div_key, 1)
        @cart.items[@key12].qty.should == 2
        @cart.items[@key13].qty.should == 1
      end
    end

    describe "has_item_key?" do
      it "同一キーの商品がカート内に存在する場合true" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.has_item_key?(@key12).should be_true
        @cart.has_item_key?('0').should be_false
      end
    end

    describe "remove_by_item_key" do
      it "指定のキーの商品SKUをカートから全て取り除きます" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.remove_by_item_key(@key12)
        @cart.items.size.should == 0
      end

      it "何個追加されていても、なかったことになります。" do
        @cart.add(@product12, @product12.product.course_div_key, 50)
        @cart.remove_by_item_key(@key12)
        @cart.has_item_key?(@key12).should == false
        @cart.items.size.should == 0
      end
    end

    describe "remove" do
      it "指定のキーの商品SKUをカートから全て取り除きます" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.remove(@product12.sku_code, @product12.product.course_div_key) # SKUコードを指定
        @cart.items.size.should == 0
      end

      it "何個追加されていても、なかったことになります。" do
        @cart.add(@product12, @product12.product.course_div_key, 50)
        @cart.remove(@product12, @product12.product.course_div_key) # SKUコードではなくSKUを指定
        @cart.has_item_key?(@key12).should == false
        @cart.items.size.should == 0
      end
    end

    describe "replace" do
      it "指定の商品コードに合致する、商品グループを指定の条件の商品グループに変更します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.replace(@product12.sku_code, @product12.product.course_div_key, :periodic, 5)
        items = @cart.items.values
        items[0].course_div.should == '2'
        items[0].course_div_key.should == :periodic
        items[0].qty.should == 5
      end
    end

    describe "change_qty" do
      it "数量を変更します" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.change_qty(@key12, 5)
        @cart.items[@key12].qty.should == 5
      end
    end

    describe "reduce_qty" do
      it "数量を減らします。" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @price = @product12.price.to_i
        @cart.items[@key12].reduce_qty(1)
      end
    end

    describe "item_list" do
      it "カートにある商品をカートに追加した順に並べます" do
        @cart.add(@product12, @product12.product.course_div_key, 1)
        @cart.add(@product13, @product13.product.course_div_key, 1)
        # itemsをHashとして意識する必要はありません。
        # @cart.items[@key12] =  Cart::Item.new(:key => @key12, :product_sku => @product12, :course_div_key => :normal, :qty => 1)
        # @cart.items[@key13] =  Cart::Item.new(:key => @key13, :product_sku => @product13, :course_div_key => :normal, :qty => 1)

        list = @cart.item_list

        list.size.should == 2
        list[1].product_sku.sku_code.should == @product12.sku_code
        list[0].product_sku.sku_code.should == @product13.sku_code
      end
    end

    describe Cart::Item do
      #fixtures :product_prices

      describe "orderable_count" do
        it "注文可能数を返します。(購入数上限 || 99)と在庫数のうち、小さい方の値" do
          @cart.add(@product12, @product12.product.course_div_key, 1)
          @cart.items[@key12].orderable_count.should == 28
        end
      end

      describe "subtotal_amount" do
        it "SKUの単価 * 数量" do
          @product12.price.should == 1680
          @product13.price.should == 3150
          Cart::Item.new(:key => @key12, :product_sku => @product12, :course_div_key => @product12.product.course_div_key, :qty => 3).subtotal_amount == 1680 * 3
          Cart::Item.new(:key => @key13, :product_sku => @product13, :course_div_key => @product13.product.course_div_key, :qty => 4).subtotal_amount == 3150 * 4
        end
      end

      describe "change_qty" do
        it "指定された数に変更する" do
          @product12.price.should == 1680
          item = Cart::Item.new(:key => @key12, :product_sku => @product12, :course_div_key => @product12.product.course_div_key, :qty => 3)
          item.change_qty(4)
          item.qty.should == 4
          item.subtotal_amount.should == 1680 * 4
        end
      end

      describe "add_qty" do
        it "指定された数だけ増やす" do
          @product12.price.should == 1680
          item = Cart::Item.new(:key => @key12, :product_sku => @product12, :course_div_key => @product12.product.course_div_key, :qty => 3)
          item.add_qty(4)
          item.qty.should == 7
          item.subtotal_amount.should == 1680 * 7
        end
      end

      describe "point" do
        before(:each) do
          @item_12 = Cart::Item.new(:key => '12', :product_sku => @product12, :course_div_key => @product12.product.course_div_key, :qty => 3)
          @item_13 = Cart::Item.new(:key => '13', :product_sku => @product13, :course_div_key => @product13.product.course_div_key, :qty => 1)
          item_list = [@item_12, @item_13]
          @cart = Cart.new()
          @item_12.cart = @cart
          @item_13.cart = @cart
          @cart.stub!(:item_list).and_return(item_list)
        end
        it "キャンペーンが適用されていない場合" do
          @cart.point.should == 8
        end
        it "明細に対するキャンペーンが適用されている場合" do
          @item_13.stub!(:given_point).and_return(10)
          @item_13.stub!(:assign_point_campaign?).and_return(true)
          @cart.point.should == 15
        end
        it "カートに対する割引キャンペーンが適用されている場合" do
          @cart.stub!(:total_amount).and_return(1000)
          @cart.point.should == 1
        end
      end

      describe "given_point" do
        it "カートの中に商品が存在する場合、その獲得ポイントを返します" do
          @cart.add(@product12, @product12.product.course_div_key, 3)
          item = @cart.item_for(@product12.sku_code, @product12.product.course_div_key)
          @cart.point_setting_available?.should == true
          @cart.point_setting_issue_price.should == 1000
          item.total_amount.should > 5000
          item.point.should == 5
          item.given_point.should == 5
        end

        it "カートが存在する場合、その合計獲得ポイントを返します" do
          @cart.add(@product12, @product12.product.course_div_key, 3)
          @cart.add(@product12.sku_code, @product12.product.course_div_key, 4)
          @cart.given_point.should == 11
        end
      end

      describe "present?" do
        it "プレゼント品の場合、tureを返します" do
          Product.stub!(:find).and_return(mock_model(Product, {:present_flg => true}))
          item = Cart::Item.new(:product_sku => @product12)
          item.present?.should == true
        end

        it "プレゼント品ではない場合、falseを返します" do
          Product.stub!(:find).and_return(mock_model(Product, {:present_flg => false}))
          item = Cart::Item.new(:product_sku => @product12)
          item.present?.should == false
        end
      end

    end

  end

  # 以下の資料に基づいています。
  #  xec/doc/２．基本設計/step2/キャンペーン補足.xls
  describe "キャンペーンあり" do
    fixtures :users, :user_ranks, :user_rank_displays, :user_rank_conditions,
      :user_points, :point_settings, :product_prices, :users, :user_emails, :user_addresses,
      :products, :product_skus, :product_prices,
      :product_displays, :product_sku_inventories, :product_sku_secure_inventories,
      :campaigns, :campaign_conditions, :campaign_effects, :campaign_target_product_skus

    before(:each) do
      @user_rank = UserRank.new(:name => 'testadmin_name', :code => 'test001', :available_div => "01", :connector_cd => "01")
      @user_rank.displays << UserRankDisplay.new(:locale => "ja", :name => "testname")
      @user_rank.conditions << UserRankCondition.new(:type_cd => "01", :operator_cd => "01", :value => 1)
      @user_rank.save!
      @user = users(:users_00010) # 会員二十三
      @user.user_rank_id = @user_rank.id
      @user.save!
      @user.reload
      @product_skus = {}
      @product_skus[:A] = products(:products_00003).primary_sku # 商品A インテリア家具
      @product_skus[:B] = products(:products_00005).primary_sku # 商品B タンブラー
      @product_skus[:C] = products(:products_00006).primary_sku # 商品C ガム10箱
      prices = {:A => 100000, :B => 10000, :C => 1000}
      prices.each do |name, price|
        ProductPrice.create(
          :product_sku_id => @product_skus[name].id,
          :user_rank_id => @user_rank.id,
          :standard_price => price,
          :regular_purchase_price => price - 200  # 定期購入売価を標準売価より安くしておいて、テスト対象としてく区別する
        )
      end
      @cart = Cart.new
      @cart.user = @user
    end

    # 条件タイプとは？
    #
    # 条件タイプ：１
    #   対象商品：特定の商品
    #   商品詳細画面でのキャンペーン適用価格に反映される。
    #   キャンペーンの効果は、商品明細の小計に対して計算される。

    # 条件タイプ：２
    #   対象商品：全体（特定の商品ではなく）
    #   商品詳細画面でのキャンペーン適用価格には反映されない。
    #   キャンペーンの効果は、注文金額合計に対して計算される。

    # = T0001 =
    #   2009-01-01 --- 2009-01-31
    # 条件タイプ２のキャンペーンはカート画面で適用される
    #    * キャンペーンA    注文合計10,000円以上で10%引き（条件タイプ：２）、優先度100
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |      0     | 100,000
    # 商品B |  10,000 |  10,000          | → |   10,000 |      0     |  10,000
    # 商品C |   1,000 |   1,000          | → |    1,000 |      0     |   1,000
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    | 111,000
    #                                                          割引   |  11,100 (*1)
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |  99,900
    # (*1) 合計が10,000を超えるため、キャンペーンAが適用される。
    #
    it "T0001 条件タイプ２のキャンペーンはカート画面で適用される" do
      time_now = Time.parse("2009/01/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 1
      campaign_a = campaigns.first
      campaign_a.code.should == "T0001A"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.subtotal_amount.should == 111000
      @cart.discount_amount.should ==  11100
      @cart.total_amount.should    ==  99900
      @cart.applied_campaigns.map(&:code).should == ["T0001A"]
    end


    # = T0002 =
    #   2009-02-01 --- 2009-02-28
    # 条件タイプ１のキャンペーンは商品詳細画面で適用される
    #     キャンペーンA    注文合計10,000円以上で10%引き（条件タイプ：２）、優先度100
    #     キャンペーンB    商品Aが50%引き（条件タイプ：１）、優先度80
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 |  50,000 (*1)     | → |  100,000 | 50,000     |  50,000
    # 商品B |  10,000 |  10,000          | → |   10,000 |      0     |  10,000 <--
    # 商品C |   1,000 |   1,000          | → |    1,000 |      0     |   1,000 <-- あわせて11,000
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    |  61,000
    #                                                          割引   |   1,100 (*2)
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |  59,900
    # (*1) 商品AにキャンペーンBが適用される。
    # (*2) 商品Aを除く注文金額合計が10,000を超えるため、キャンペーンAが適用される。
    #      ただし商品Aには割引適用済みのため商品Bと商品Cの金額の合計が対象となる。
    it "T0002" do
      time_now = Time.parse("2009/02/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0002A"
      campaign_b = campaigns.last
      campaign_b.code.should == "T0002B"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==  50000
      @cart.item_for(@product_skus[:A]).total_amount.should     ==  50000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0002B"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.subtotal_amount.should ==  61000
      @cart.discount_amount.should ==   1100
      @cart.total_amount.should    ==  59900
      @cart.applied_campaigns.map(&:code).should == ["T0002A"]
      @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0002B", "T0002A"]
    end

    # = T0003 =
    #   2009-03-01 --- 2009-03-31
    # 同じ条件タイプのキャンペーンの場合、優先度の高いものが適用される
    #     キャンペーンA    商品A、商品B、商品Cが10%引き（条件タイプ：１）、優先度100
    #     キャンペーンB    商品Aが50%引き（条件タイプ：１）、優先度80
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 |  90,000          | → |  100,000 | 10,000     |  90,000 (*1)
    # 商品B |  10,000 |   9,000          | → |   10,000 |  1,000     |   9,000 (*1)
    # 商品C |   1,000 |     900          | → |    1,000 |    100     |     900 (*1)
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    |  99,900
    #                                                          割引   |       0
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |  99,900
    # (*1) 商品A、商品B、商品CにキャンペーンAが適用される。
    it "T0003" do
      time_now = Time.parse("2009/03/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0003A"
      campaign_b = campaigns.last
      campaign_b.code.should == "T0003B"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==  10000
      @cart.item_for(@product_skus[:A]).total_amount.should     ==  90000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0003A"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==   1000
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   9000
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0003A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==    100
      @cart.item_for(@product_skus[:C]).total_amount.should     ==    900
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0003A"]
      @cart.subtotal_amount.should ==  99900
      @cart.discount_amount.should ==      0
      @cart.total_amount.should    ==  99900
      @cart.applied_campaigns.map(&:code).should == []
      @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0003A"]
    end


    # = T0004 =
    #   2009-04-01 --- 2009-04-30
    # 同じ条件タイプのキャンペーンの場合、優先度の高いものが適用される
    #     キャンペーンA    商品A、商品B、商品Cが300円引き（条件タイプ：１）、優先度80
    #     キャンペーンB    商品Aが50%引き（条件タイプ：１）、優先度100
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 |  50,000          | → |  100,000 | 50,000 (*1)|  50,000
    # 商品B |  10,000 |   9,700          | → |   10,000 |    300     |   9,700 (*2)
    # 商品C |   1,000 |     700          | → |    1,000 |    300     |     700 (*2)
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    |  60,400
    #                                                          割引   |       0
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |  60,400
    # (*1) 商品AにキャンペーンBが適用される。
    # (*2) 商品B、商品CにキャンペーンAが適用される。
    it "T0004" do
      time_now = Time.parse("2009/04/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_b = campaigns.first
      campaign_b.code.should == "T0004B"
      campaign_a = campaigns.last
      campaign_a.code.should == "T0004A"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==  50000
      @cart.item_for(@product_skus[:A]).total_amount.should     ==  50000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0004B"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==    300
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   9700
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0004A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==    300
      @cart.item_for(@product_skus[:C]).total_amount.should     ==    700
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0004A"]
      @cart.subtotal_amount.should ==  60400
      @cart.discount_amount.should ==      0
      @cart.total_amount.should    ==  60400
      @cart.applied_campaigns.map(&:code).should == []
      @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0004B", "T0004A"]
    end


    # = T0005 =
    #   2009-05-01 --- 2009-05-31
    # 条件タイプと優先度が同じ場合、優先度が高いものが適用される
    #     キャンペーンA    小計額が5000円以上で商品A、商品B、商品Cが100円引き（条件タイプ：１）、優先度80
    #     キャンペーンB    商品Aが50%引き（条件タイプ：１）、優先度100
    # -------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     |    割引価格 |    小計
    # ------|---------|------------------|----|----------|-------------|-------------
    # 商品A | 100,000 |  50,000          | → |  100,000 |  50,000 (*1)|  50,000
    # 商品B |  10,000 |   9,900          | → |   10,000 |     100     |   9,900 (*2)
    # 商品C |   1,000 |   1,000          | → |    1,000 |       0     |   1,000 (*3)
    # ------|---------|------------------|----|----------|-------------|-------------
    #                                                          合計    |  60,900
    #                                                           割引   |       0
    # -----------------------------------------------------------------|-------------
    #                                                         総合計   |  60,900
    # (*1) 商品Aには、優先度の高いキャンペーンBが適用される。
    # (*2) 商品Bは、小計額が5000円以上なのでキャンペーンAが適用される。
    # (*3) 商品Cは、小計額が5000円以上でないためキャンペーンAは適用されない。
    it "T0005" do
      time_now = Time.parse("2009/05/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0005B"
      campaign_b = campaigns.last
      campaign_b.code.should == "T0005A"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==  50000
      @cart.item_for(@product_skus[:A]).total_amount.should     ==  50000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0005B"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==    100
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   9900
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0005A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.subtotal_amount.should ==  60900
      @cart.discount_amount.should ==      0
      @cart.total_amount.should    ==  60900
      @cart.applied_campaigns.map(&:code).should == []
      @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0005B", "T0005A"]
    end


    # = T0006 =
    #   2009-06-01 --- 2009-06-30
    # 条件タイプ１で割引が適用された商品は、条件タイプ２の割引の適用対象にならない
    #     キャンペーンA    商品A、商品B、商品Cが100円引き（条件タイプ：１）、優先度100
    #     キャンペーンB    200円引き（条件タイプ：２）、優先度60
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 |  99,900          | → |  100,000 |    100     |  99,900 (*1)
    # 商品B |  10,000 |   9,900          | → |   10,000 |    100     |   9,900 (*1)
    # 商品C |   1,000 |     900          | → |    1,000 |    100     |     900 (*1)
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    | 110,700
    #                                                          割引   |       0
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |  110,700
    # (*1) 商品A、商品B、商品CにキャンペーンAが適用される。
    it "T0006" do
      time_now = Time.parse("2009/06/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0006A"
      campaign_b = campaigns.last
      campaign_b.code.should == "T0006B"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==    100
      @cart.item_for(@product_skus[:A]).total_amount.should     ==  99900
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0006A"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==    100
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   9900
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0006A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==    100
      @cart.item_for(@product_skus[:C]).total_amount.should     ==    900
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0006A"]
      @cart.subtotal_amount.should == 110700
      @cart.discount_amount.should ==      0
      @cart.total_amount.should    == 110700
      @cart.applied_campaigns.map(&:code).should == []
      @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0006A"]
    end




    # = T0007 =
    #   2009-07-01 --- 2009-07-31
    # 条件タイプ１は条件タイプ２よりも優先される
    #     キャンペーンA    商品B、商品Cで10%引き（条件タイプ：１）、優先度80
    #     キャンペーンB    200円引き（条件タイプ：２）、優先度100
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |      0     | 100,000
    # 商品B |  10,000 |   9,000          | → |   10,000 |  1,000     |   9,000 (*1)
    # 商品C |   1,000 |     900          | → |    1,000 |    100     |     900 (*1)
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    | 109,900
    #                                                          割引   |     200 (*2)
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |  109,700
    # (*1) 商品B、商品CにキャンペーンAが適用される。
    # (*2) カート全体にキャンペーンBが適用される
    it "T0007" do
      time_now = Time.parse("2009/07/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_b = campaigns.first
      campaign_b.code.should == "T0007B"
      campaign_a = campaigns.last
      campaign_a.code.should == "T0007A"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==   1000
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   9000
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0007A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==    100
      @cart.item_for(@product_skus[:C]).total_amount.should     ==    900
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0007A"]
      @cart.subtotal_amount.should == 109900
      @cart.discount_amount.should ==    200
      @cart.total_amount.should    == 109700
      @cart.applied_campaigns.map(&:code).should == ["T0007B"]
      @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0007A", "T0007B"]
    end



    # = T0008 =
    #   2009-08-01 --- 2009-08-31
    # 割引以外のキャンペーン効果
    #     キャンペーンB    商品Cで石鹸プレゼント（条件タイプ：１）、優先度100 (*1)
    #     キャンペーンC    注文金額合計が１２０００円以上で化粧ポーチプレゼント（条件タイプ：２）、優先度20
    #     キャンペーンD    注文金額合計が１００００円以上で送料無料（条件タイプ：２）、優先度80
    #     キャンペーンE    商品B、商品Cで５０％OFF（条件タイプ：１）、優先度70
    #     キャンペーンF    商品Aで１０％OFF（条件タイプ：１）、優先度90
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   |    小計
    # ------|---------|------------------|----|----------|------------|--------
    # 商品A | 100,000 |  90,000          | → |  100,000 | 10,000     |  90,000 (*2)
    # 商品B |  10,000 |   5,000          | → |   10,000 |  5,000     |   5,000 (*3)
    # 商品C |   1,000 |     900          | → |    1,000 |    500     |     500 (*3)
    # ------|---------|------------------|----|----------|------------|--------
    #                                                         合計    |  95,500 (*4)(*5)
    #                                                          割引   |       0
    # ----------------------------------------------------------------|---------
    #                                                        総合計   |   95,500
    #
    # (*1) 商品CにプレゼントキャンペーンBが適用される（石鹸プレゼント）
    # (*2) 商品Aに金額キャンペーンFが適用される
    # (*3) 商品B、商品Cに金額キャンペーンEが適用される
    # (*4) 注文金額合計が１００００円を超えるため送料キャンペーンDが適用される（送料無料）
    # (*5) 注文金額合計が１２０００円を超えるため、プレゼントキャンペーンCが適用される（化粧ポーチプレゼント）
    describe "T0008" do

      before do
        # キャンペーンBのプレゼント用の「石鹸」。フィクスチャ上は「お酒」
        @present_for_b = ProductSku.find(277)
        @present_for_b.product.present_flg = true # ここだけでプレゼント品にしちゃいます
        @present_for_b.product.save!

        # キャンペーンCのプレゼント用の「化粧ポーチ」。フィクスチャ上は「ミニカー」
        @present_for_c = ProductSku.find(281)
        @present_for_c.product.present_flg = true # ここでだけプレゼント品にしちゃいます
        @present_for_c.product.save!

        time_now = Time.parse("2009/08/01 12:00:00")
        Time.stub!(:now).and_return(time_now)
        #
        campaigns = Campaign.availables
        campaigns.map(&:code).should == ["T0008B", "T0008F", "T0008D", "T0008E", "T0008C"]
        #
        # 先にここで呼び出しておかないと、CampaignConditions#match_to_conditions?内のinstance_eval内でclass_evalを行ってSecurityErrorになるので要注意。
        UserAddress.new.is_japanese_format?
      end

      it "プレゼント品の在庫あり" do
        @present_for_c.orderable_count.should_not == 0

        @cart.add(@product_skus[:A])
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])


        lambda{
          @cart.calculate
        }.should change(@cart.items, :length).by(2)

        3.times do # 3回calculateしても同じ結果になるべき
          @cart.calculate
          @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
          @cart.item_for(@product_skus[:A]).discount_amount.should  ==  10000
          @cart.item_for(@product_skus[:A]).total_amount.should     ==  90000
          @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0008F"]
          @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
          @cart.item_for(@product_skus[:B]).discount_amount.should  ==   5000
          @cart.item_for(@product_skus[:B]).total_amount.should     ==   5000
          @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0008E"]
          @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
          @cart.item_for(@product_skus[:C]).discount_amount.should  ==    500
          @cart.item_for(@product_skus[:C]).total_amount.should     ==    500
          @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0008B", "T0008E"]

          # キャンペーンで適用されたプレゼントが追加されています。
          @cart.item_for(@present_for_b, :present).should_not == nil
          @cart.item_for(@present_for_b, :present).subtotal_amount.should == 0
          @cart.item_for(@present_for_b, :present).discount_amount.should == 0
          @cart.item_for(@present_for_b, :present).total_amount.should    == 0
          @cart.item_for(@present_for_c, :present).should_not == nil
          @cart.item_for(@present_for_c, :present).qty.should == 1
          @cart.item_for(@present_for_c, :present).subtotal_amount.should == 0
          @cart.item_for(@present_for_c, :present).discount_amount.should == 0
          @cart.item_for(@present_for_c, :present).total_amount.should    == 0

          @cart.subtotal_amount.should ==  95500
          @cart.discount_amount.should ==      0
          @cart.total_amount.should    ==  95500
          @cart.applied_campaigns.map(&:code).should == ["T0008D", "T0008C"]
          @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0008B", "T0008F", "T0008E", "T0008D", "T0008C"]
        end
      end


      it "refs #1163 プレゼント品の在庫なしの場合はキャンペーンを適用しません" do
        # １つのキャンペーンには１つの効果しか設定できないため、
        # プレゼント品の在庫が切れていた場合は、そのキャンペーン自体が適用されません
        
        @present_for_c.inventories.each{|inventory| inventory.destroy}
        @present_for_c.parent_stock_count = 0
        @present_for_c.composable_stock_count = 0
        @present_for_c.orderable_count = 0
        @present_for_c.save!
        @present_for_c.orderable_count.should == 0

        @cart.add(@product_skus[:A])
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])

        lambda{
          @cart.calculate
        }.should change(@cart.items, :length).by(1)

        3.times do # 3回calculateしても同じ結果になるべき
          @cart.calculate
          @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
          @cart.item_for(@product_skus[:A]).discount_amount.should  ==  10000
          @cart.item_for(@product_skus[:A]).total_amount.should     ==  90000
          @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
          @cart.item_for(@product_skus[:B]).discount_amount.should  ==   5000
          @cart.item_for(@product_skus[:B]).total_amount.should     ==   5000
          @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
          @cart.item_for(@product_skus[:C]).discount_amount.should  ==    500
          @cart.item_for(@product_skus[:C]).total_amount.should     ==    500

          # キャンペーンで適用されたプレゼントが追加されています。
          @cart.item_for(@present_for_b, :present).should_not == nil
          @cart.item_for(@present_for_b, :present).subtotal_amount.should == 0
          @cart.item_for(@present_for_b, :present).discount_amount.should == 0
          @cart.item_for(@present_for_b, :present).total_amount.should    == 0
          @cart.item_for(@present_for_c, :present).should == nil

          @cart.subtotal_amount.should ==  95500
          @cart.discount_amount.should ==      0
          @cart.total_amount.should    ==  95500
          @cart.applied_campaigns.map(&:code).should == ["T0008D"]
        end
      end
    end

    # = T0009 =
    #   2009-09-01 --- 2009-09-28
    #     キャンペーンA    商品B、商品Cでポイント２倍（条件タイプ：１）、優先度80
    # ----------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | ポイント|    小計
    # ------|---------|------------------|----|----------|------------|---------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |      0     |   100   | 100,000 (*1)
    # 商品B |  10,000 |  10,000          | → |   10,000 |      0     |    20   |  10,000 (*2)
    # 商品C |   1,000 |   1,000          | → |    1,000 |      0     |     2   |   1,000 (*2)
    # ------|---------|------------------|----|----------|------------|---------|--------
    #                                                                  合計     | 111,000
    #                                                                  割引     |       0
    #                                                                  ポイント |     122
    # --------------------------------------------------------------------------|---------
    #                                                                  総合計   | 111,000
    # (*1) 商品Aに通常のポイント設定が適用される（1000円で1ポイント）。
    # (*2) 商品B、商品CにキャンペーンAが適用される（1000円で2ポイント）。
    it "T0009" do
      time_now = Time.parse("2009/09/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 1
      campaign_a = campaigns.first
      campaign_a.code.should == "T0009A"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).given_point.should      ==     20
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0009A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).given_point.should      ==      2
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0009A"]
      @cart.total_given_point.should ==    122
      @cart.subtotal_amount.should   == 111000
      @cart.discount_amount.should   ==      0
      @cart.total_amount.should      == 111000
      @cart.applied_campaigns.map(&:code).should == []
    end

    # = T0010-1 =
    #   2009-10-01 --- 2009-10-10
    #     キャンペーンB    ポイント３倍（条件タイプ：２）、優先度80
    # -----------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | ポイント|    小計
    # ------|---------|------------------|----|----------|------------|---------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |      0     |   100   | 100,000
    # 商品B |  10,000 |  10,000          | → |   10,000 |      0     |    10   |  10,000
    # 商品C |   1,000 |   1,000          | → |    1,000 |      0     |     1   |   1,000
    # ------|---------|------------------|----|----------|------------|---------|--------
    #                                                                  合計     | 111,000
    #                                                                  割引     |       0
    #                                                                  ポイント |     333 (*1)
    # --------------------------------------------------------------------------|---------
    #                                                                  総合計   | 111,000
    # (*1) キャンペーンBが適用される
    it "T0010" do
      time_now = Time.parse("2009/10/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 1
      campaign_b = campaigns.first
      campaign_b.code.should == "T0010B"

      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      # キャンペーン適用前
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:B]).given_point.should      ==     10
      @cart.item_for(@product_skus[:C]).given_point.should      ==      1
      #
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).given_point.should      ==     10
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).given_point.should      ==      1
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.total_given_point.should ==    333
      @cart.subtotal_amount.should   == 111000
      @cart.discount_amount.should   ==      0
      @cart.total_amount.should      == 111000
      @cart.applied_campaigns.map(&:code).should == ["T0010B"]
    end

    # = T0014 =
    #   2009-10-11 --- 2009-10-20
    #     キャンペーンB    合計額500000円以上でポイント２倍（条件タイプ：２）、優先度80
    # ----------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | ポイント|    小計
    # ------|---------|------------------|----|----------|------------|---------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |       0    |   100   | 100,000
    # 商品B |  10,000 |  10,000          | → |   10,000 |       0    |    10   |  10,000
    # 商品C |   1,000 |   1,000          | → |    1,000 |       0    |     1   |   1,000
    # ------|---------|------------------|----|----------|------------|---------|--------
    #                                                                  合計     | 111,000
    #                                                                  割引     |       0
    #                                                                  ポイント |     111 (*1)
    # --------------------------------------------------------------------------|---------
    #                                                                  総合計   | 111,000
    # (*1) 注文金額合計が500000円に満たないためキャンペーンBは適用されない。
    it "T0014" do
      time_now = Time.parse("2009/10/11 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 1
      campaign_b = campaigns.first
      campaign_b.code.should == "T0014B"

      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).given_point.should      ==     10
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).given_point.should      ==      1
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.total_given_point.should ==    111
      @cart.subtotal_amount.should   == 111000
      @cart.discount_amount.should   ==      0
      @cart.total_amount.should      == 111000
      @cart.applied_campaigns.map(&:code).should == []
    end



    # = T0011 =
    #   2009-11-01 --- 2009-11-10
    #     キャンペーンA    商品B、商品Cで５０％OFF（条件タイプ：１）、優先度90
    #     キャンペーンB    ポイント２倍（条件タイプ：２）、優先度80
    # ----------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | ポイント|    小計
    # ------|---------|------------------|----|----------|------------|---------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |       0    |   100   | 100,000
    # 商品B |  10,000 |  10,000          | → |   10,000 |   5,000    |     5   |   5,000 (*1)
    # 商品C |   1,000 |   1,000          | → |    1,000 |     500    |     0   |     500 (*1)
    # ------|---------|------------------|----|----------|------------|---------|--------
    #                                                                  合計     | 105,500
    #                                                                  割引     |       0
    #                                                                  ポイント |     210 (*2)
    # --------------------------------------------------------------------------|---------
    #                                                                  総合計   | 105,500
    # (*1) 商品B、商品CにキャンペーンAが適用される。
    # (*2) ポイントは注文金額合計に対して付与される
    #
    it "T0011" do
      time_now = Time.parse("2009/11/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0011A"
      campaign_a.targets.length.should == 2
      campaign_b = campaigns.last
      campaign_b.code.should == "T0011B"
      point_effect = campaign_b.effects.first
      point_effect.type_key.should == :point
      point_effect.operator_key.should == :multiply
      point_effect.value.should == "2"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      # キャンペーン適用前
      @cart.item_for(@product_skus[:A]).point.should            ==    100
      @cart.item_for(@product_skus[:B]).point.should            ==     10
      @cart.item_for(@product_skus[:C]).point.should            ==      1
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:B]).given_point.should      ==     10
      @cart.item_for(@product_skus[:C]).given_point.should      ==      1
      #
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).point.should            ==    100
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==   5000
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   5000
      @cart.item_for(@product_skus[:B]).point.should            ==      5
      @cart.item_for(@product_skus[:B]).given_point.should      ==      5
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0011A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==    500
      @cart.item_for(@product_skus[:C]).total_amount.should     ==    500
      @cart.item_for(@product_skus[:C]).point.should            ==      0 # 0.5 を小数点切り捨てで 0
      @cart.item_for(@product_skus[:C]).given_point.should      ==      0
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0011A"]
      @cart.total_given_point.should ==    210
      @cart.subtotal_amount.should   == 105500
      @cart.discount_amount.should   ==      0
      @cart.total_amount.should      == 105500
      @cart.applied_campaigns.map(&:code).should == ["T0011B"]
    end

    # = T0015 =
    #   2009-11-11 --- 2009-11-20
    #     キャンペーンA    商品B、商品Cで５０％OFF（条件タイプ：１）、優先度90
    #     キャンペーンB    商品A、ポイント２倍（条件タイプ：１）、優先度80
    # ----------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | ポイント|    小計
    # ------|---------|------------------|----|----------|------------|---------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |       0    |   200   | 100,000 (*1)
    # 商品B |  10,000 |  10,000          | → |   10,000 |   5,000    |     5   |   5,000 (*2)
    # 商品C |   1,000 |   1,000          | → |    1,000 |     500    |     0   |     500 (*2)
    # ------|---------|------------------|----|----------|------------|---------|--------
    #                                                                  合計     | 105,500
    #                                                                  割引     |   5,500
    #                                                                  ポイント |     205
    # --------------------------------------------------------------------------|---------
    #                                                                  総合計   | 105,500
    # (*1) 商品AにキャンペーンBが適用される。
    # (*2) 商品B、商品CにキャンペーンAが適用される。
    #      ポイントは割引後の金額に付与される
    it "T0015" do
      time_now = Time.parse("2009/11/11 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0015A"
      campaign_b = campaigns.last
      campaign_b.code.should == "T0015B"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).given_point.should      ==    200
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0015B"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==   5000
      @cart.item_for(@product_skus[:B]).total_amount.should     ==   5000
      @cart.item_for(@product_skus[:B]).given_point.should      ==      5
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == ["T0015A"]
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==    500
      @cart.item_for(@product_skus[:C]).total_amount.should     ==    500
      @cart.item_for(@product_skus[:C]).given_point.should      ==      0
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == ["T0015A"]
      @cart.total_given_point.should ==    205
      @cart.subtotal_amount.should   == 105500
      @cart.discount_amount.should   ==      0
      @cart.total_amount.should      == 105500
      @cart.applied_campaigns.map(&:code).should == []
    end

    # = T0016 =
    #   2010-01-01 --- 2010-01-10
    #     キャンペーンA    ５０％OFF（条件タイプ：２）、優先度90
    #     キャンペーンB    ポイント２倍（条件タイプ：２）、優先度80
    # ----------------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | ポイント|    小計
    # ------|---------|------------------|----|----------|------------|---------|--------
    # 商品A | 100,000 | 100,000          | → |  100,000 |       0    |   100   | 100,000
    # 商品B |  10,000 |  10,000          | → |   10,000 |       0    |    10   |  10,000
    # 商品C |   1,000 |   1,000          | → |    1,000 |       0    |     1   |   1,000
    # ------|---------|------------------|----|----------|------------|---------|--------
    #                                                                  合計     | 111,000
    #                                                                  割引     |  55,500 (*1)
    #                                                                  ポイント |     110 (*2)
    # --------------------------------------------------------------------------|---------
    #                                                                  総合計   |  55,500
    # (*1) キャンペーンAが適用される。
    # (*2) ポイントは注文金額合計に対して付与される
    #
    it "T0016" do
      time_now = Time.parse("2010/01/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0016A"
      campaign_a.targets.length.should == 0
      campaign_b = campaigns.last
      campaign_b.code.should == "T0016B"
      point_effect = campaign_b.effects.first
      point_effect.type_key.should == :point
      point_effect.operator_key.should == :multiply
      point_effect.value.should == "2"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      # キャンペーン適用前
      @cart.item_for(@product_skus[:A]).point.should            ==    100
      @cart.item_for(@product_skus[:B]).point.should            ==     10
      @cart.item_for(@product_skus[:C]).point.should            ==      1
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:B]).given_point.should      ==     10
      @cart.item_for(@product_skus[:C]).given_point.should      ==      1
      #
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).point.should            ==    100
      @cart.item_for(@product_skus[:A]).given_point.should      ==    100
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).point.should            ==     10
      @cart.item_for(@product_skus[:B]).given_point.should      ==     10
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).point.should            ==      1
      @cart.item_for(@product_skus[:C]).given_point.should      ==      1
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.total_given_point.should ==    110
      @cart.subtotal_amount.should   == 111000
      @cart.discount_amount.should   ==  55500
      @cart.total_amount.should      ==  55500
      @cart.applied_campaigns.map(&:code).should == ["T0016A", "T0016B"]
    end

    # = T0012 =
    #   2009-12-01 --- 2009-12-10
    #       [条件タイプ１で送料無料が適用される商品があれば、その注文は送料無料]
    #     キャンペーンA  商品Aで送料無料（条件タイプ：１）、優先度１
    #               
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | 小計
    # ------|---------|------------------|----|----------|-----------|---------
    # 商品A | 100,000 | 100,000          | → | 100,000  |         0 |  100,000 (*1)
    # 商品B |  10,000 |  10,000          | → |  10,000  |         0 |   10,000
    # 商品C |   1,000 |   1,000          | → |   1,000  |         0 |    1,000
    # ------|---------|------------------|----|----------|-----------|---------
    #                                                           合計 |  111,000
    #                                                           割引 |        0
    # ---------------------------------------------------------------|---------
    #                                                         総合計 |  111,000
    # (*1) 商品AにキャンペーンAが適用される（この注文自体が送料無料）
    it "T0012" do
      time_now = Time.parse("2009/12/01 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 1
      campaign_a = campaigns.first
      campaign_a.code.should == "T0012A"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0012A"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.subtotal_amount.should == 111000
      @cart.discount_amount.should ==      0
      @cart.total_amount.should    == 111000
      @cart.free_shipping_cost?.should be_true
      @cart.applied_campaigns.map(&:code).should == []
    end
    

    # = T0013 =
    #   2009-12-11 --- 2009-12-20
    #       [条件タイプ１で送料無料が適用される商品があれば、その注文は送料無料]
    #    キャンペーンA  商品Aが50%OFF（条件タイプ：１）、優先度２
    #    キャンペーンB  商品Aが80,000円以上で送料無料（条件タイプ：１）、優先度１
    #               
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | 小計
    # ------|---------|------------------|----|----------|-----------|---------
    # 商品A | 100,000 |  50,000          | → | 100,000  |    50,000 |   50,000 (*1)
    # 商品B |  10,000 |  10,000          | → |  10,000  |         0 |   10,000
    # 商品C |   1,000 |   1,000          | → |   1,000  |         0 |    1,000
    # ------|---------|------------------|----|----------|-----------|---------
    #                                                           合計 |   61,000
    #                                                           割引 |        0
    # ---------------------------------------------------------------|---------
    #                                                         総合計 |   61,000 (*2)
    # (*1) 商品Aが50%OFF（キャンペーンA）
    # (*2) 合計額が80,000円を超えないので送料無料にならない
    it "T0013" do
      time_now = Time.parse("2009/12/11 12:00:00")
      Time.stub!(:now).and_return(time_now)
      #
      campaigns = Campaign.availables
      campaigns.length.should == 2
      campaign_a = campaigns.first
      campaign_a.code.should == "T0013A"
      campaign_b = campaigns.last
      campaign_b.code.should == "T0013B"
      #
      @cart.add(@product_skus[:A])
      @cart.add(@product_skus[:B])
      @cart.add(@product_skus[:C])
      @cart.calculate
      @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
      @cart.item_for(@product_skus[:A]).discount_amount.should  ==  50000
      @cart.item_for(@product_skus[:A]).total_amount.should     ==  50000
      @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == ["T0013A"]
      @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
      @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
      @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
      @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
      @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
      @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
      @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []
      @cart.subtotal_amount.should ==  61000
      @cart.discount_amount.should ==      0
      @cart.total_amount.should    ==  61000
      @cart.free_shipping_cost?.should be_false
      @cart.applied_campaigns.map(&:code).should == []
    end

    # = T0017 =
    #   2010-01-11 --- 2010-01-20
    # 割引以外のキャンペーン効果
    #     キャンペーンA　　化粧ポーチプレゼント（条件タイプ：２）、優先度100 (*1)
    #     キャンペーンB    石鹸プレゼント（条件タイプ：２）、優先度90 (*2)
    # -------------------------------------------------------------------------
    #       | 商品詳細画面               | → |   カート画面
    #       | 定価    | キャンペーン価格 | → | 定価     | 割引価格   | 小計
    # ------|---------|------------------|----|----------|-----------|---------
    # 商品A | 100,000 | 100,000          | → | 100,000  |         0 |  100,000
    # 商品B |  10,000 |  10,000          | → |  10,000  |         0 |   10,000
    # 商品C |   1,000 |   1,000          | → |   1,000  |         0 |    1,000
    # ------|---------|------------------|----|----------|-----------|---------
    #                                                           合計 |  111,000
    #                                                           割引 |        0
    # ---------------------------------------------------------------|---------
    #                                                         総合計 |  111,000
    #
    # (*1) キャンペーンAが適用される
    # (*2) プレゼントキャンペーンは重複可能なのでキャンペーンBも適用される
    describe "T0017" do
      before do
        # キャンペーンBのプレゼント用の「化粧ポーチ」。フィクスチャ上は「お酒」
        @present_for_a = ProductSku.find(277)
        @present_for_a.product.present_flg = true # ここだけでプレゼント品にしちゃいます
        @present_for_a.product.save!

        # キャンペーンCのプレゼント用の「石鹸」。フィクスチャ上は「ミニカー」
        @present_for_b = ProductSku.find(281)
        @present_for_b.product.present_flg = true # ここでだけプレゼント品にしちゃいます
        @present_for_b.product.save!

        @cart.campaign_attached_at = Time.parse("2010/01/11 12:00:00")
        
        campaigns = Campaign.availables(@cart.campaign_attached_at)
        campaigns.map(&:code).should == ["T0017A", "T0017B"]
        #
        # 先にここで呼び出しておかないと、CampaignConditions#match_to_conditions?内のinstance_eval内でclass_evalを行ってSecurityErrorになるので要注意。
        UserAddress.new.is_japanese_format?
      end

      it "プレゼント品の在庫あり" do
        @present_for_a.orderable_count.should_not == 0
        @present_for_b.orderable_count.should_not == 0

        @cart.add(@product_skus[:A])
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])


        lambda{
          @cart.calculate
        }.should change(@cart.items, :length).by(2)

        3.times do # 3回calculateしても同じ結果になるべき
          @cart.calculate
          @cart.item_for(@product_skus[:A]).subtotal_amount.should  == 100000
          @cart.item_for(@product_skus[:A]).discount_amount.should  ==      0
          @cart.item_for(@product_skus[:A]).total_amount.should     == 100000
          @cart.item_for(@product_skus[:A]).applied_campaigns.map(&:code).should == []
          @cart.item_for(@product_skus[:B]).subtotal_amount.should  ==  10000
          @cart.item_for(@product_skus[:B]).discount_amount.should  ==      0
          @cart.item_for(@product_skus[:B]).total_amount.should     ==  10000
          @cart.item_for(@product_skus[:B]).applied_campaigns.map(&:code).should == []
          @cart.item_for(@product_skus[:C]).subtotal_amount.should  ==   1000
          @cart.item_for(@product_skus[:C]).discount_amount.should  ==      0
          @cart.item_for(@product_skus[:C]).total_amount.should     ==   1000
          @cart.item_for(@product_skus[:C]).applied_campaigns.map(&:code).should == []

          # キャンペーンで適用されたプレゼントが追加されています。
          @cart.item_for(@present_for_a, :present).should_not == nil
          @cart.item_for(@present_for_a, :present).qty.should == 1
          @cart.item_for(@present_for_a, :present).subtotal_amount.should == 0
          @cart.item_for(@present_for_a, :present).discount_amount.should == 0
          @cart.item_for(@present_for_a, :present).total_amount.should    == 0
          @cart.item_for(@present_for_b, :present).should_not == nil
          @cart.item_for(@present_for_b, :present).qty.should == 1
          @cart.item_for(@present_for_b, :present).subtotal_amount.should == 0
          @cart.item_for(@present_for_b, :present).discount_amount.should == 0
          @cart.item_for(@present_for_b, :present).total_amount.should    == 0

          @cart.subtotal_amount.should == 111000
          @cart.discount_amount.should ==      0
          @cart.total_amount.should    == 111000
          @cart.applied_campaigns.map(&:code).should == ["T0017A", "T0017B"]
          @cart.applied_campaigns_for_cart_and_items.map(&:code).should == ["T0017A", "T0017B"]
        end
      end
    end

    describe "#912 セッション上のCartのデータの削減" do
      before(:each) do
        # T0007
        time_now = Time.parse("2009/07/01 12:00:00")
        Time.stub!(:now).and_return(time_now)
        #
        campaigns = Campaign.availables
        campaigns.length.should == 2
        @campaign_b = campaigns.first
        @campaign_b.code.should == "T0007B"
        @campaign_a = campaigns.last
        @campaign_a.code.should == "T0007A"
        #
        @cart.add(@product_skus[:A])
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
      end

      it "Cart::Item#marshal_dump and Cart::Item#marshal_load" do
        target_item = @cart.item_for(@product_skus[:B])

        expected = {
          '@cart' => @cart,
          '@product_sku' => @product_skus[:B],
          # '@applied_effects' => # [#<CampaignEffect id: 13 ...>],
          '@name' => "タンブラー",
          '@unit_price' => 10000,
          '@key' => "0000000009030-00-00-00_1",
          '@discount_amount' => 1000,
          '@course_div' => "1",
          '@qty' => 1,
          # '@user_rank_id' => 92
        }
        expected.each do |key, value|
          target_item.instance_variable_get(key).should == value
        end
        target_item.instance_variable_get("@user_rank_id").class.should == Fixnum
        applied_effects = target_item.instance_variable_get("@applied_effects")
        applied_effects.class.should == Array
        applied_effects.first.class.should == CampaignEffect
        target_item.instance_variables.sort.should == (expected.keys + ['@given_point', '@applied_effects', '@user_rank_id']).sort

        loaded_item = Marshal.load(Marshal.dump(target_item))
        expected_loaded =  expected.dup.update('@cart' => nil, '@product_sku' => nil)
        expected_loaded.each do |key, value|
          loaded_item.instance_variable_get(key).should == value
        end
        target_item.instance_variables.sort.should == (expected_loaded.keys + ['@given_point', '@applied_effects', '@user_rank_id']).sort
        loaded_item.product_sku.should_not be_nil
        loaded_item.product_sku.id.should == @product_skus[:B].id
        applied_effects = target_item.applied_effects
        applied_effects.class.should == Array
        applied_effects.first.class.should == CampaignEffect
      end

      it "Cart#marshal_dump and Cart#marshal_load" do
        # @cart.instance_variable_get(:@campaigns).should_not be_nil
        @cart.item_list.length.should == 3
        @cart.item_for(@product_skus[:A]).should_not be_nil
        @cart.item_for(@product_skus[:B]).should_not be_nil
        @cart.item_for(@product_skus[:C]).should_not be_nil

        expected = {
          # "@applied_effects" => nil,
          # "@campaigns_for_cart" => nil,
          # "@campaigns_for_items" => nil,
          "@discount_amount" => 200,
          "@given_point" => nil,
          # "@item_list" => nil,
          # "@items" => nil,
          "@subtotal_amount" => 109900,
          "@subtotal_amount_for_discount" => 100000,
          "@total_amount" => 0,
          # "@user" => nil,
          # "@user_rank" => nil
        }
        expected.each do |attr, value|
          if value
            @cart.instance_variable_get(attr).should == value
          else
            @cart.instance_variable_get(attr).should be_nil
          end
        end
        applied_effects = @cart.instance_variable_get("@applied_effects")
        applied_effects.class.should == Array
        applied_effects.first.class.should == CampaignEffect
        campaigns_for_cart = @cart.instance_variable_get("@campaigns_for_cart")
        campaigns_for_cart.class.should == Array
        campaigns_for_cart.first.class.should == Campaign
        campaigns_for_items = @cart.instance_variable_get("@campaigns_for_items")
        campaigns_for_items.class.should == Array
        campaigns_for_items.first.class.should == Campaign
        item_list = @cart.instance_variable_get("@item_list")
        item_list.class.should == Array
        item_list.first.class.should == Cart::Item
        items = @cart.instance_variable_get("@items")
        items.class.should == Hash
        user = @cart.instance_variable_get("@user")
        user.class.should == User
        user = @cart.instance_variable_get("@user_rank")
        user.class.should == UserRank
        @cart.instance_variables.sort.should == (
          expected.keys + 
          %w(@applied_effects @campaigns_for_cart @campaigns_for_items) + 
          %w(@item_list @items @shipping_cost_effect_id @user @user_rank) + 
          %w(@point_setting_available @point_setting_issue_price) + 
          %w(@campaign_ids_for_cart @campaign_ids_for_items )).sort
          

        # Marshal.dumpでシリアライズして、Marshal.loadでデシリアライズして復元します。
        loaded_cart = Marshal.load(Marshal.dump(@cart))

        loaded_cart.instance_variable_get(:@applied_effects).should be_nil
        loaded_cart.instance_variable_get(:@campaigns_for_cart).should be_nil
        loaded_cart.instance_variable_get(:@campaigns_for_items).should be_nil
        loaded_cart.instance_variable_get(:@user).should be_nil
        loaded_cart.instance_variable_get(:@user_rank).should be_nil

        loaded_cart.instance_variable_get(:@item_list).should_not be_nil
        loaded_cart.instance_variable_get(:@items).should_not be_nil

        loaded_cart.instance_variable_get(:@applied_effect_ids).should == [@campaign_b.effects.first.id]
        loaded_cart.instance_variable_get(:@campaign_ids_for_cart).should == [@campaign_b.id]
        loaded_cart.instance_variable_get(:@campaign_ids_for_items).should == [@campaign_a.id]
        loaded_cart.instance_variable_get(:@user_id).should == @user.id

        loaded_cart.item_list.length.should == 3
        loaded_cart.item_for(@product_skus[:A]).should_not be_nil
        loaded_cart.item_for(@product_skus[:B]).should_not be_nil
        loaded_cart.item_for(@product_skus[:C]).should_not be_nil

        loaded_cart.user.class.should == User
        loaded_cart.user_rank.class.should == UserRank
        loaded_cart.applied_effects.class.should == Array
        loaded_cart.applied_effects.first.class.should == CampaignEffect
        loaded_cart.applied_effects.map(&:id).should == [@campaign_b.effects.first.id]
        loaded_cart.send(:campaigns_for_cart).class.should == Array
        loaded_cart.send(:campaigns_for_cart).first.class.should == Campaign
        loaded_cart.send(:campaigns_for_cart).map(&:id).should == [@campaign_b.id]
        loaded_cart.send(:campaigns_for_items).class.should == Array
        loaded_cart.send(:campaigns_for_items).first.class.should == Campaign
        loaded_cart.send(:campaigns_for_items).map(&:id).should == [@campaign_a.id]
      end
    end
  end

  describe "プレゼント品在庫切れ" do
    fixtures :products, :product_displays, :product_skus, :product_sku_inventories, :product_sku_secure_inventories, :product_prices
    before(:each) do
      @cart = Cart.new

      @product12 = ProductSku.find(276)
      @key12 = Cart::Item.key_for(@product12.sku_code, :present)
      @product13 = ProductSku.find(277)
      @key13 = Cart::Item.key_for(@product13.sku_code, :normal)
    end

    describe "delete_out_of_stock_present" do
      it "プレゼント品の数量を変更する" do
        @cart.add(@product12, :present, 50)
        @cart.add(@product13, :normal, 50)
        @product12.stub!(:orderable_count).and_return(3)
        @product13.stub!(:orderable_count).and_return(3)
        
        @cart.delete_out_of_stock_present
        
        @cart.items[@key12].qty.should == 3
        @cart.items[@key13].qty.should == 50
      end
      
      it "プレゼント品を削除する" do
        @cart.add(@product12, :present, 50)
        @cart.add(@product13, :normal, 50)
        @product12.stub!(:orderable_count).and_return(0)
        @product13.stub!(:orderable_count).and_return(0)
        
        @cart.delete_out_of_stock_present
        
        @cart.items[@key12].should be_nil
        @cart.items[@key13].qty.should == 50
      end
    end
    describe "item_list_order_by_present" do
      it "item_list_order_by_present" do
        sku274 = ProductSku.find(274)
        sku274.product.present_flg = false
        @cart.add(sku274, :normal, 50)
        sku277 = ProductSku.find(277)
        sku277.product.present_flg = true
        @cart.add(sku277, :normal, 50)
        sku276 = ProductSku.find(276)
        sku276.product.present_flg = false
        @cart.add(sku276, :normal, 50)

        result = @cart.item_list_order_by_present
        result[0].product_sku.product.present_flg.should be_false
        result[1].product_sku.product.present_flg.should be_false
        result[2].product_sku.product.present_flg.should be_true
      end
    end
  end

end
