# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe CmsDeadLinkCheck do
  fixtures :cms_bind_contents, :cms_bind_items, :cms_binds, :cms_page_titles, :cms_pages, :cms_projects,
    :cms_templates, :cms_template_contents, :cms_files, :cms_publishes, :cms_published_projects, 
    :products, :product_displays, :product_images, :banners, :banner_displays, :shop_settings,
    :cms_dead_link_checks, :cms_dead_link_resources, :cms_dead_links, :cms_default_templates

  before(:each) do
    EcRider.config.stub!(:ec_test_url).and_return('http://ec.test.com/')
  end

  describe "status_name_for_test" do
    it ":waiting" do
      CmsDeadLinkCheck.new(:status_key => :waiting).status_name_for_test.should == 'チェック中'
    end
    it ":processing" do
      CmsDeadLinkCheck.new(:status_key => :processing).status_name_for_test.should == 'チェック中'
    end
    it ":checked" do
      CmsDeadLinkCheck.new(:status_key => :checked).status_name_for_test.should == 'チェック完了'
    end
    it ":waiting" do
      CmsDeadLinkCheck.new(:status_key => :failed).status_name_for_test.should == 'チェック失敗'
    end
    it ":canceled" do
      CmsDeadLinkCheck.new(:status_key => :canceled).status_name_for_test.should == 'キャンセル'
    end
  end
  
  describe "sorted_resources" do
    it "cms_bind" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :cms_bind, :cms_bind_id => 46, :locale => 'ja')
      check.resources.create(:resource_key => :cms_bind, :cms_bind_id => 46, :locale => 'en')
      check.resources.create(:resource_key => :cms_bind, :cms_bind_id => 46, :locale => 'zh_CN')
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, {:cms_bind => {:cms_page => :titles}}]})
      check.sorted_resources.map{|r| r.locale}.should == ['en', 'ja', 'zh_CN']
    end
    
    it "cms_template" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :cms_template, :cms_template_id => 1, :locale => 'ja')
      check.resources.create(:resource_key => :cms_template, :cms_template_id => 1, :locale => 'en')
      check.resources.create(:resource_key => :cms_template, :cms_template_id => 1, :locale => 'zh_CN')
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, :cms_template]})
      check.sorted_resources.map{|r| r.locale}.should == ['en', 'ja', 'zh_CN']
    end
    
    it "cms_file" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :cms_file, :cms_file_id => 59)
      check.resources.create(:resource_key => :cms_file, :cms_file_id => 64)
      check.resources.create(:resource_key => :cms_file, :cms_file_id => 61)
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, :cms_file]})
      check.sorted_resources.map{|r| r.cms_file.public_file_path}.should == 
        ["public/stylesheets/application.css", "public/themes/ec_ver2/style.css", "public/themes/ec_ver2/theme/local.css"]
    end
    
    it "product" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :product, :product_id => 100011)
      check.resources.create(:resource_key => :product, :product_id => 1)
      check.resources.create(:resource_key => :product, :product_id => 100047)
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, {:product => :product_displays}]})
      check.sorted_resources.map{|r| r.product.product_code}.should == ["0000000000011", "0000000000047", "0000000009051"]
    end
    
    it "product_image" do
      check = CmsDeadLinkCheck.new
    end
    
    it "product_display" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :product_display, :product_display_id => 3)
      check.resources.create(:resource_key => :product_display, :product_display_id => 4)
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, {:product_display => :product}]})
      check.sorted_resources.map{|r| r.product_display.product.product_code}.should == ["0000000000013", "0000000000013"]
    end
    
    it "banner" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :banner, :banner_id => 1)
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, :banner]})
      check.sorted_resources.map{|r| r.banner_id}.should == [1]
    end
    
    it "banner_display" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :banner_display, :banner_display_id => 1)
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, {:banner_display => :banner}]})
      check.sorted_resources.map{|r| r.banner_display_id}.should == [1]
    end
    
    it "shop_setting" do
      check = CmsDeadLinkCheck.create
      check.resources.create(:resource_key => :shop_setting, :shop_setting_id => 11)
      check = CmsDeadLinkCheck.find(check.id, :include => {:resources => [:links, :shop_setting]})
      check.sorted_resources.map{|r| r.shop_setting_id}.should == [11]
    end
  end
  
  describe "extract_target_path_in_xml" do
    it "<ec:stylesheet/>" do
      content = <<-"EOS"
        <ec:stylesheet href="xecprint.css" media="print"/>
        <ec:stylesheet href="xecprint" media="print"/>
        <ec:stylesheet href="/style/xecprint.css" media="print"/>
        <ec:stylesheet href="http://xecprint" media="print"/>
        <ec:stylesheet src="xecprint.css" media="print"/>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "public/stylesheets/xecprint.css"
      paths[0].original_path.should == "xecprint.css"
      paths[1].check_path.should == "public/stylesheets/xecprint.css"
      paths[1].original_path.should == "xecprint"
      paths[2].check_path.should == "public/style/xecprint.css"
      paths[2].original_path.should == "/style/xecprint.css"
      paths[3].check_path.should == "http://xecprint.css"
      paths[3].original_path.should =="http://xecprint"
      paths[4].check_path.should == "public/stylesheets/xecprint.css"
      paths[4].original_path.should == "xecprint.css"
    end
    
    it "<ec:javascript/>" do
      content = <<-"EOS"
        <ec:javascript_in_page>
          <ec:javascript href="xec.js"/>
          <ec:javascript href="xec"/>
          <ec:javascript href="/js/xec.js"/>
          <ec:javascript href="http://xec"/>
          <ec:javascript src="xec.js"/>
        </ec:javascript_in_page>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "public/javascripts/xec.js"
      paths[0].original_path.should == "xec.js"
      paths[1].check_path.should == "public/javascripts/xec.js"
      paths[1].original_path.should == "xec"
      paths[2].check_path.should == "public/js/xec.js"
      paths[2].original_path.should == "/js/xec.js"
      paths[3].check_path.should == "http://xec.js"
      paths[3].original_path.should == "http://xec"
      paths[4].check_path.should == "public/javascripts/xec.js"
      paths[4].original_path.should == "xec.js"
    end
    
    it "<ec:image/>" do
      content = <<-"EOS"
        <ec:image src="logo.gif" alt="ロゴ" width="50%" height="50%"/>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "public/images/logo.gif"
      paths[0].original_path.should == "logo.gif"
    end
    
    it "<ec:image_link/>" do
      content = <<-"EOS"
        詳細はこちら→<ec:image_link src="to_detail.gif" href="/product/detail?product_code=12345" border="1"/>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == '/product/detail?product_code=12345'
      paths[0].original_path.should == "/product/detail?product_code=12345"
      paths[1].check_path.should == "public/images/to_detail.gif"
      paths[1].original_path.should == "to_detail.gif"
    end
    
    it "<a><img/></a>" do
      content = <<-"EOS"
        <a href="http://external.site.co.jp">
          <img src="link_image.png"/>
        </a>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "http://external.site.co.jp"
      paths[0].original_path.should == "http://external.site.co.jp"
      paths[1].check_path.should == "public/images/link_image.png"
      paths[1].original_path.should == "link_image.png"
    end
    
    it "styleタグ　background: url" do
      content = <<-"EOS"
        <style>
          p, td {
            font-size:18px;
            color:#0000FF;
            background: url(/images/common/td_bg.gif) no-repeat scroll -34px 0;
          }
          double {background: url("/images/common/double_bg.gif") no-repeat scroll -34px 0;}
          single {background: url('/images/common/single_bg.gif');}
          space {background: url( /images/common/space_bg.gif );}
        </style>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "public/images/common/td_bg.gif"
      paths[0].original_path.should == "/images/common/td_bg.gif"
      paths[1].check_path.should == "public/images/common/double_bg.gif"
      paths[1].original_path.should == "/images/common/double_bg.gif"
      paths[2].check_path.should == "public/images/common/single_bg.gif"
      paths[2].original_path.should == "/images/common/single_bg.gif"
      paths[3].check_path.should == "public/images/common/space_bg.gif"
      paths[3].original_path.should == "/images/common/space_bg.gif"
    end
    
    it "styleタグ　@import" do
      content = <<-"EOS"
        <style>
          @import url("url_double.css");
          @import url('url_single.CSS');
          @import url(url_nospace.css);
          @import url( url_space.css );
          @import "double.css";
          @import 'single.css';
        </style>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "public/stylesheets/url_double.css"
      paths[0].original_path.should == "url_double.css"
      paths[1].check_path.should == "public/stylesheets/url_single.CSS"
      paths[1].original_path.should == "url_single.CSS"
      paths[2].check_path.should == "public/stylesheets/url_nospace.css"
      paths[2].original_path.should == "url_nospace.css"
      paths[3].check_path.should == "public/stylesheets/url_space.css"
      paths[3].original_path.should == "url_space.css"
      paths[4].check_path.should == "public/stylesheets/double.css"
      paths[4].original_path.should == "double.css"
      paths[5].check_path.should == "public/stylesheets/single.css"
      paths[5].original_path.should == "single.css"
    end
    
    it "styleタグ　中身なし" do
      content = <<-"EOS"
        <style></style>
        <style></style>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths.length.should == 0
    end
    
    it "styleタグ　コメントあり" do
      content = <<-"EOS"
        <style>
          /*
          p, td {
            font-size:18px;
            color:#0000FF;
            background: url(/images/common/td_bg1.gif) no-repeat scroll -34px 0;
          }
          */
          
          p, td {
            font-size:18px;
            color:#0000FF;
          //  background: url(/images/common/td_bg2.gif) no-repeat scroll -34px 0;
          }
          
          /* 
           * small {
           *  font-size:12px;
           *  background: url(/images/common/small_bg1.gif); //背景画像
           * }
           */
           
          small {
            font-size:12px;
            background: url(/images/common/small_bg2.gif); //背景画像
          }
        </style>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths.length.should == 1
      paths[0].check_path.should == "public/images/common/small_bg2.gif"
      paths[0].original_path.should == "/images/common/small_bg2.gif"
    end
    
    it "styleタグ　チェック対象なし" do
      content = <<-"EOS"
        <style>
          p, td {
            font-size:18px;
            color:#0000FF;
          }
        </style>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths.length.should == 0
    end
    
    it "style属性で指定されたurl" do
      content = <<-"EOS"
        <h1 style="background: url(/images/common/h1_bg.gif) no-repeat scroll -134px 0;">
          タイトル
        </h1>
      EOS
      
      link_check = CmsDeadLinkCheck::LinkMatrixSetup.new
      paths = link_check.send("extract_target_path_in_xml", [content], :cms_bind, 1)
      paths[0].check_path.should == "public/images/common/h1_bg.gif"
      paths[0].original_path.should == "/images/common/h1_bg.gif"
    end
  end
  
  describe "execute_if_waiting" do
    if EcRider.config.ec_admin
      before(:each) do
        tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "ec_test"))
        EcRider.config.stub!(:ec_test_root).and_return(tmp_dir)
        Dir.glob(File.join(EcRider.config.ec_test_root, '*')) .each{|f| FileUtils.rm_rf(f) }
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public'))
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'app', 'views'))
        Net::HTTP.stub!(:get_response).and_return(Net::HTTPNotFound)
        Net::HTTP.stub!(:start)
      end

      it '商品画像、商品表示、商品詳細URL' do
        Product.destroy_all('id <> 1')
        Banner.stub!(:find).and_return([])
        ShopSetting.stub!(:find).and_return([])
        Dir.stub!(:glob).and_return([])
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public', 'product_images'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'public', 'product_images', 'product1.png'))
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public', 'images', 'common'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'public', 'images', 'common', 'bn_up.gif'))

        product = Product.find(:first, :include => [:images, :product_displays])
        product_img = ProductImage.create!(
          :product_id => product.id, :filename => "product1.png", :content_type => "image/png", :size => 777)
        ProductImage.create!(
          :parent_id => product_img.id, :thumbnail => :thumb, :filename => "product1_thumb.png", :content_type => "image/png", :size => 77)
        product.product_displays.each do |display|
          display.product_outline = "<ec:link href='/'/>"
          display.product_explanation = "<a href='/'>link to top</a>"
          display.normal_explanation = "<ec:image src='/product_images/product1.png'/>"
          display.recommendation = "<image src='/product_images/product1'/>"
          display.recommendation_2 = <<-"EOS"
            <a href='http://www.google.co.jp/' style="background:transparent url(common/bn_up.gif)">
              link to google.co.jp
            </a>
          EOS
          display.save!
        end
        product.save!

        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 2
        invoke_img = false
        invoke_disp = false
        invoke = false
        check.resources.each do |r|
          if r.resource_key == :product_image
            r.product_image_id.should == product_img.id
            r.links.length.should == 1
            r.links.first.url.index("product1_thumb.png").should > 0
            invoke_img = true
          elsif r.resource_key == :product_display
            r.product_display_id.should_not be_nil
            r.links.map{|l| l.url}.sort.should == ["/product_images/product1", "http://www.google.co.jp/"]
            invoke_disp = true
          elsif r.resource_key == :product
            invoke = true
          end
        end
        invoke_img.should be_true
        invoke_disp.should be_true
        invoke.should be_false
      end

      it 'バナー画像、バナー表示' do
        Product.stub!(:find).and_return([])
        ShopSetting.stub!(:find).and_return([])
        Dir.stub!(:glob).and_return([])
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public', 'banner_images'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'public', 'banner_images', 'banner_0000000001.png'))

        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 9
        invoke_disp = false
        invoke = false
        check.resources.each do |r|
          if r.resource_key == :banner
            r.links.length.should == 1
            r.links.map{|l| l.url}.sort.should == ["http://www.ec-one.com"]
            invoke = true
          elsif r.resource_key == :banner_display
            invoke_disp = true
          end
        end
        invoke_disp.should be_false
        invoke.should be_true
      end

      it "商品表示、バナー画像" do
        Product.destroy_all('id <> 1')
        ShopSetting.stub!(:find).and_return([])
        Dir.stub!(:glob).and_return([])
        Banner.destroy_all('id <> 1')
        banner = Banner.find(:first)
        banner.type_key = :text
        banner.displays.each do |display|
          display.text_title = "title"
          display.text_description = "<a herf=aaaaa<br>"
          display.save!
        end
        banner.save!

        product = Product.find(:first, :include => [:images, :product_displays])
        product.product_displays[0].product_outline = "<a herf=aaaaa<br>"
        product.product_displays[0].save!
        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 3
        invoke_product_disp = false
        invoke_banner_disp = false
        check.resources.each do |r|
          if r.resource_key == :product_display
            r.product_display_id.should_not be_nil
            r.links.map{|l| l.response_status}.sort.should == ["XML記述が不正です。"]
            invoke_product_disp = true
          elsif r.resource_key == :banner_display
            r.links.map{|l| l.response_status}.sort.should == ["XML記述が不正です。"]
            invoke_banner_disp = true
          end
        end
        invoke_product_disp.should be_true
        invoke_banner_disp.should be_true
      end

      it '店舗設定' do
        Product.stub!(:find).and_return([])
        Banner.stub!(:find).and_return([])
        ShopSetting.destroy_all
        ShopSetting.create!(:setting_div_key => :shop_logo, :value => "/images/logo.png")
        ShopSetting.create!(:setting_div_key => :top_action, :value => "/index", :locale => 'ja')
        ShopSetting.create!(:setting_div_key => :top_action, :value => "/top", :locale => 'en')
        ShopSetting.create!(:setting_div_key => :continue_shopping_action, :value => "/list")
        Dir.stub!(:glob).and_return([])
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public', 'images'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'public', 'images', 'logo.png'))

        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 0
  #      invoke = false
  #      check.resources.each do |r|
  #        if r.resource_key == :shop_setting
  #          r.shop_setting_id.should_not be_nil
  #          r.links.map{|l| l.url}.compact.length.should == 1
  #          invoke = true
  #        end
  #      end
  #      invoke.should be_true
      end

      it 'ページ' do
        Product.stub!(:find).and_return([])
        Banner.stub!(:find).and_return([])
        ShopSetting.stub!(:find).and_return([])
        src = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_pages', 'index.xml')
        content = CmsTemplateContent.find(25)
        content.descriptions = IO.read(src)
        content.save!
        src = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_pages', 'index_en.xml')
        content = CmsTemplateContent.find(26)
        content.descriptions = IO.read(src)
        content.save!
        src = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_pages', 'index_zh_CN.xml')
        content = CmsTemplateContent.find(27)
        content.descriptions = IO.read(src)
        content.save!
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'app', 'views', 'cms'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'app', 'views', 'cms', 'index.html.erb'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'app', 'views', 'cms', 'index_en.html.erb'))
        FileUtils.touch(File.join(EcRider.config.ec_test_root, 'app', 'views', 'cms', 'index_zh_CN.html.erb'))

        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 3
        invoke = false
        check.resources.each do |r|
          if r.resource_key == :cms_bind
            r.locale.should_not be_nil
            r.cms_bind_id.should_not be_nil
            r.links.map{|l| l.url}.compact.length.should >= 1
            invoke = true
          end
        end
        invoke.should be_true
      end

      it 'デフォルトテンプレート' do
        Product.stub!(:find).and_return([])
        Banner.stub!(:find).and_return([])
        ShopSetting.stub!(:find).and_return([])
        Dir.stub!(:glob).and_return([])
        src = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_layouts', 'top.xml')
        content = CmsTemplateContent.find(1)
        content.descriptions = IO.read(src)
        content.save!
        src = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_layouts', 'top_ko.xml')
        content = CmsTemplateContent.find(5)
        content.descriptions = IO.read(src)
        content.save!

        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 2
        invoke = false
        check.resources.each do |r|
          if r.resource_key == :cms_template
            r.locale.should_not be_nil
            r.cms_bind_id.should be_nil
            r.links.map{|l| l.url}.compact.length.should == 2
            invoke = true
          end
        end
        invoke.should be_true
      end

      it 'css' do
        Product.stub!(:find).and_return([])
        Banner.stub!(:find).and_return([])
        ShopSetting.stub!(:find).and_return([])
        src = File.join(EcRider.config.ec_admin_root, 'test', 'fixtures', 'cms_files', 'common', 'ec-rider.css')
        FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public', 'stylesheets'))
        dest = File.join(EcRider.config.ec_test_root, 'public', 'stylesheets', 'application.css')
        FileUtils.cp_r(src, dest)
        CmsFile.create!(:public_file_path => 'public/stylesheets/application.css')

        CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
        CmsDeadLinkCheck.execute_if_waiting
        check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
        check.status_key.should == :checked
        check.resources.length.should == 1
        invoke = false
        check.resources.each do |r|
          if r.resource_key == :cms_file
            r.cms_file_id.should_not be_nil
            r.links.map{|l| l.url}.compact.length.should == 6
            invoke = true
          end
        end
        invoke.should be_true
      end

      it 'css　@import' do
          Product.stub!(:find).and_return([])
          Banner.stub!(:find).and_return([])
          ShopSetting.stub!(:find).and_return([])
          src = File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'themes', 'ec_ver2', 'style.css')
          FileUtils.mkdir_p(File.join(EcRider.config.ec_test_root, 'public', 'stylesheets', 'layout'))
          dest = File.join(EcRider.config.ec_test_root, 'public', 'stylesheets', 'style.css')
          FileUtils.cp(src, dest)
          FileUtils.touch(File.join(EcRider.config.ec_test_root, 'public', 'stylesheets', 'layout', 'common.css'))
          CmsFile.create!(:public_file_path => 'public/stylesheets/style.css')
          CmsFile.create!(:public_file_path => 'public/stylesheets/layout/common.css')

          CmsDeadLinkCheck.create!(:cms_publish_id => 1, :status_key => :waiting)
          CmsDeadLinkCheck.execute_if_waiting
          check = CmsDeadLinkCheck.find(:first, :include => {:resources => :links})
          check.status_key.should == :checked
          check.resources.length.should == 1
          invoke = false
          check.resources.each do |r|
            if r.resource_key == :cms_file
              r.cms_file_id.should_not be_nil
              r.links.map{|l| l.url}.compact.include?("layout/common.css").should be_false
              invoke = true
            end
          end
          invoke.should be_true
      end
    end
  end
end
