# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CmsTemplate do
  fixtures :cms_templates, :cms_projects, :cms_template_contents, :cms_default_templates

  describe "primary_default_template_codes" do
    it "上下左右のそれぞれについて、最も優先度の高いプロジェクトで指定されているテンプレートコードを返します。" do
      if EcRider.config.ec_admin
        template = CmsTemplate.find(11)
        template.update_attributes(:delete_at_publish => false)
        CmsDefaultTemplate.create_or_update_default_template(template.cms_project_id, template.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template.template_code)
        
        CmsDefaultTemplate.count(:all).should == 3

        template_codes = CmsDefaultTemplate.primary_default_template_codes([1,2,3,4])

        template_codes.is_a?(CmsDefaultTemplate::CmsDefaultTemplateList).should be_true
        template_codes.values.length.should == 2
        template_codes.default_template_hash.is_a?(Hash).should be_true

        template_codes.default_template_hash[:pc][:top].should == '2009082119101338424590'
        template_codes.default_template_hash[:pc][:bottom].should == '2009082119101338424900'
        template_codes.default_template_hash[:pc][:left].should be_nil
        template_codes.default_template_hash[:pc][:right].should be_nil
      end
    end
    it "優先順位が同じプロジェクトがある場合 公開開始日の遅いプロジェクトで編集されたテンプレートが取得できること" do
      if EcRider.config.ec_admin
        cms_projects_00001 = CmsProject.find(cms_projects(:cms_projects_00001).id)
        cms_projects_00001.update_attributes(:priority => 100, :started_at => "2010-02-14 00:00:00")
        cms_projects_00002 = CmsProject.find(2)
        cms_projects_00002.update_attributes(:priority => 100, :started_at => "2010-02-13 00:00:00")

        # project1で編集したデフォルトテンプレート
        template1 = CmsTemplate.find(1)
        template1.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template1.cms_project_id, template1.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template1.template_code)

        # project2で編集したデフォルトテンプレート
        template15 = CmsTemplate.find(15)
        template15.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template15.cms_project_id, template15.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template15.template_code)

        template_codes = CmsDefaultTemplate.primary_default_template_codes([1,2,3,4])

        template_codes.is_a?(CmsDefaultTemplate::CmsDefaultTemplateList).should be_true
        template_codes.values.length.should == 2
        template_codes.default_template_hash.is_a?(Hash).should be_true

        template_codes.default_template_hash[:pc][:top].should == '2009082119101338424990'
        template_codes.default_template_hash[:pc][:bottom].should == '2009082119101338424900'
        template_codes.default_template_hash[:pc][:left].should be_nil
        template_codes.default_template_hash[:pc][:right].should be_nil
      end
    end
    it "優先順位が同じ かつ 公開開始日が同じプロジェクトがある場合 プロジェクトIDの大きい方で編集されたテンプレートが取得できること" do
      if EcRider.config.ec_admin
        cms_projects_00001 = CmsProject.find(cms_projects(:cms_projects_00001).id)
        cms_projects_00001.update_attributes(:priority => 100, :started_at => "2010-02-14 00:00:00")
        cms_projects_00002 = CmsProject.find(cms_projects(:cms_projects_00002).id)
        cms_projects_00002.update_attributes(:priority => 100, :started_at => "2010-02-14 00:00:00")

        # project1で編集したデフォルトテンプレート
        template1 = CmsTemplate.find(1)
        template1.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template1.cms_project_id, template1.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template1.template_code)

        # project2で編集したデフォルトテンプレート
        template15 = CmsTemplate.find(15)
        template15.update_attributes(:display_position_key => :top)
        CmsDefaultTemplate.create_or_update_default_template(template15.cms_project_id, template15.terminal, CmsDefaultTemplate.default_template_position_id_by_key(:top), template15.template_code)

        template_codes = CmsDefaultTemplate.primary_default_template_codes([1,2,3,4])

        template_codes.is_a?(CmsDefaultTemplate::CmsDefaultTemplateList).should be_true
        template_codes.values.is_a?(Array).should be_true
        template_codes.values.length.should == 2
        template_codes.default_template_hash.is_a?(Hash).should be_true

        template_codes.default_template_hash[:pc][:top].should == '2009082119101338424420'
        template_codes.default_template_hash[:pc][:bottom].should == '2009082119101338424900'
        template_codes.default_template_hash[:pc][:left].should be_nil
        template_codes.default_template_hash[:pc][:right].should be_nil
      end
    end
  end
  describe "CmsDefaultTemplateList" do
    it "上下左右のそれぞれについて、最も優先度の高いプロジェクトで指定されているテンプレートコードを返します。" do
      if EcRider.config.ec_admin
        template_codes = CmsDefaultTemplate.primary_default_template_codes([1,2,3,4])

        template_codes.is_a?(CmsDefaultTemplate::CmsDefaultTemplateList).should be_true
        template_codes.values.length.should == 2
        template_codes.default_template_hash.is_a?(Hash).should be_true

        template_codes.default_template_hash[:pc][:bottom].should == '2009082119101338424900'

        template_codes.position_id('2009082119101338424900').should == '1'
        template_codes.position_key('2009082119101338424900').should == :bottom
        template_codes.position_name('2009082119101338424900').should == '下'
        template_codes.editable?('2009082119101338424900').should be_false
      end
    end
    it "other terminal" do
      if EcRider.config.ec_admin
        CmsTemplate.update(1, :terminal_key => :mobile)
        CmsDefaultTemplate.update(1, :terminal_key => :mobile)
        template_codes = CmsDefaultTemplate.primary_default_template_codes([1,2,3,4])

        template_codes.position_id('2009082119101338424990').should be_nil
        template_codes.position_key('2009082119101338424990').should be_nil
        template_codes.position_name('2009082119101338424990').should be_nil
        template_codes.editable?('2009082119101338424990').should be_true
        template_codes.position_id('2009082119101338424990', :mobile).should == '0'
        template_codes.position_key('2009082119101338424990', :mobile).should == :top
        template_codes.position_name('2009082119101338424990', :mobile).should == '上'
        template_codes.editable?('2009082119101338424990', :mobile).should be_false
      end
    end
  end
end
