# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'
require 'pathname'

describe EcRider::CmsSetup do
  fixtures :cms_bind_contents, :cms_bind_items, :cms_binds, :cms_page_titles, :cms_pages, :cms_projects,
    :cms_templates, :cms_template_contents, :cms_files, :cms_default_templates

  include EcRider::CmsSetup

  describe "create_ecrider_cms_pages" do
    if EcRider.config.ec_admin
      it "ec-rider提供ページ" do
        create_ecrider_cms_pages

        cms_pages = CmsPage.find(:all, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles, :cms_project],
          :conditions => ["controller = ?", 'account'])
        cms_pages.length.should == 12
        cms_pages.each do |page|
          page.cms_project_id.should == 1
          page.action.should_not be_nil
          page.binds.length.should == 5
          page.binds.each do |bind|
            if bind.position_key == :center
              bind.cms_template.name.should_not be_nil
              bind.cms_template.cms_project_id.should == 1
              bind.cms_template.development_code.should == "account/#{page.action}"
              bind.cms_template.contents.length.should == 6
              bind.cms_template.contents.each do |content|
                content.locale.should_not be_nil
                content.descriptions.should_not be_nil
              end
            end
          end
          page.titles.length.should == 6
          page.titles.each do |title|
            title.locale.should_not be_nil
            title.name.should_not be_nil
          end
        end
      end

      it "DBに無いロケールを登録" do
        create_ecrider_cms_pages
        CmsTemplateContent.destroy_all(:locale => 'th')
        CmsTemplateContent.count(:conditions => ['locale = ? ', 'th']).should == 0
        create_ecrider_cms_pages
        CmsTemplateContent.count(:conditions => ['locale = ? ', 'th']).should > 0
      end
    end
  end

  describe "create_shop_base_cms_pages" do
    if EcRider.config.ec_admin
      it "店舗独自ページ" do
        create_shop_base_cms_pages

        cms_pages = CmsPage.find(:all, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles, :cms_project],
          :conditions => [" cms_project_id = ? and controller = ? and action = ?", CmsProject::BASE_PROJECT_ID, 'cms', 'index'])
        cms_pages.length.should == 1
        cms_pages.each do |page|
          page.cms_project_id.should == 1
          page.action.should_not be_nil
          page.binds.length.should == 5
          page.binds.each do |bind|
            if bind.position_key == :center
              bind.cms_template.name.should_not be_nil
              bind.cms_template.cms_project_id.should == 1
              bind.cms_template.development_code.should == "cms/index"
              bind.cms_template.contents.length.should == 6
              bind.cms_template.contents.each do |content|
                content.locale.should_not be_nil
                content.descriptions.should_not be_nil
              end
            end
          end
          page.titles.length.should == 6
          page.titles.each do |title|
            title.locale.should_not be_nil
            title.name.should_not be_nil
          end
        end
      end
    end
  end

  describe "create_shop_base_cms_templates" do
    if EcRider.config.ec_admin
      it "中央以外のテンプレート" do
        create_shop_base_cms_templates

        cms_templates = CmsTemplate.find(:all, :conditions => ["development_code = ?", 'cms/top'])
        cms_templates.length.should == 1
        cms_templates.each do |template|
          template.cms_project_id.should == 1
          template.contents.length.should == 6
          template.contents.each do |content|
            content.locale.should_not be_nil
            content.descriptions.should_not be_nil
          end
        end
      end
    end
  end

  describe "create_base_cms_file" do
    before(:each) do
      tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_cms_projects"))
      Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      FileUtils.mkdir_p(File.join(tmp_dir, 'admin', 'public', 'projects', '1', 'public'))
      EcRider.config.stub!(:ec_admin_root).and_return(File.join(tmp_dir, 'admin'))
      # EcRider.config.stub!(:ec_front_root).and_return(File.join(tmp_dir, 'front'))
    end
    
    it "既存ディレクトリ、新規ファイル" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'images', 'buttons'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'images', 'buttons', 'botton_cart.gif'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsFile, :count).by(2)
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/images/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/images/buttons/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?' ,CmsProject::BASE_PROJECT_ID, 'public/images/buttons/botton_cart.gif']).should == 1
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'images', 'buttons', 'botton_cart.gif')).should be_true
    end
    
    it "新規ディレクトリ、新規ファイル" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'newdir', 'style'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'newdir', 'style', 'product.css'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'newdir', 'style', 'common.css'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsFile, :count).by(4)
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/newdir/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/newdir/style/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?', CmsProject::BASE_PROJECT_ID, 'public/newdir/style/product.css']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?', CmsProject::BASE_PROJECT_ID, 'public/newdir/style/common.css']).should == 1
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'newdir', 'style', 'product.css')).should be_true
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'newdir', 'style', 'common.css')).should be_true
    end
    
    it "既存ディレクトリ、既存ファイル" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', '500.html'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsTemplate, :count).by(0)
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?', CmsProject::BASE_PROJECT_ID, 'public/500.html']).should == 1
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', '500.html')).should be_true
    end
    
    it "対象外" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', '.ignorefile'))
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'javascripts'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'javascripts', '.ignorefile'))
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'ignoredir'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'ignoredir', '.ignorefile'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsTemplate, :count).by(0)
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', '.ignorefile')).should be_false
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'javascripts', '.ignorefile')).should be_false
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'ignoredir', '.ignorefile')).should be_false
    end
  end

  describe "InitialData" do
    fixtures :cms_templates, :cms_template_contents, :cms_pages, :cms_page_titles, :cms_binds
    before do
      @templates_count = CmsTemplate.count
      @templte_contents_count = CmsTemplateContent.count
      @page_count = CmsPage.count
      @page_titels_count = CmsPageTitle.count
      @binds_count = CmsBind.count
      
      @xml_path = 'testpath'
      File.stub!(:exist?).with(@xml_path).and_return(true)
      IO.stub!(:read).with(@xml_path).and_return('<p>test</p>')
    end
    it "setup ja_center" do
      EcRider::CmsSetup::InitialData.setup(:pc, 'cms', 'test', 'test_page',  "test001", @xml_path, 'ja', :center).should be_true

      CmsTemplate.count.should == @templates_count + 1
      CmsTemplateContent.count.should == @templte_contents_count + 1
      CmsPage.count.should == @page_count + 1
      CmsPageTitle.count.should == @page_titels_count + 1
      CmsBind.count.should == @binds_count + 5

      EcRider::CmsSetup::InitialData.add_locale(:pc, 'cms', 'test', 'test_page',  "test001", @xml_path, 'en').should be_true

      CmsTemplate.count.should == @templates_count + 1
      CmsTemplateContent.count.should == @templte_contents_count + 2
      CmsPage.count.should == @page_count + 1
      CmsPageTitle.count.should == @page_titels_count + 2
      CmsBind.count.should == @binds_count + 5
    end
    it "setup en_center" do
      EcRider::CmsSetup::InitialData.setup(:pc, 'cms', 'test', 'test_page',  "test001", @xml_path, 'en', :center).should be_true

      CmsTemplate.count.should == @templates_count + 1
      CmsTemplateContent.count.should == @templte_contents_count + 1
      CmsPage.count.should == @page_count + 1
      CmsPageTitle.count.should == @page_titels_count + 1
      CmsBind.count.should == @binds_count + 5

      EcRider::CmsSetup::InitialData.add_locale(:pc, 'cms', 'test', 'test_page',  "test001", @xml_path, 'th').should be_true

      CmsTemplate.count.should == @templates_count + 1
      CmsTemplateContent.count.should == @templte_contents_count + 2
      CmsPage.count.should == @page_count + 1
      CmsPageTitle.count.should == @page_titels_count + 2
      CmsBind.count.should == @binds_count + 5
    end
    it "setup ja_top" do
      EcRider::CmsSetup::InitialData.setup(:pc, 'cms', 'test', 'test_page',  "test001", @xml_path, 'ja', :top).should be_true

      CmsTemplate.count.should == @templates_count + 1
      CmsTemplateContent.count.should == @templte_contents_count + 1
      CmsPage.count.should == @page_count
      CmsPageTitle.count.should == @page_titels_count
      CmsBind.count.should == @binds_count

      EcRider::CmsSetup::InitialData.add_locale(:pc, 'cms', 'test', 'test_page',  "test001", @xml_path, 'th').should be_true

      CmsTemplate.count.should == @templates_count + 1
      CmsTemplateContent.count.should == @templte_contents_count + 2
      CmsPage.count.should == @page_count
      CmsPageTitle.count.should == @page_titels_count
      CmsBind.count.should == @binds_count
    end
  end
end
