# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe Country do
  fixtures :countries, :shop_settings, :shipping_cost_boundaries, :shipping_costs
  
  # 配送可能区分
  it "ship_restriction_div means shipping restrictions" do
    Country.ship_restriction_div_options.should == [["配送可能", '0'],["配送不可", '1']]
    Country.ship_restriction_div_name_by_id('0').should == "配送可能"
    Country.ship_restriction_div_name_by_id('1').should == "配送不可"
    Country.ship_restriction_div_id_by_key(:able).should == "0"
    Country.ship_restriction_div_id_by_key(:disable).should == "1"

    u1 = Country.new
    u1.ship_restriction_div.should be_nil
    u1.ship_restriction_div_key.should be_nil
    u1.ship_restriction_div_name.should be_nil
    u1.ship_restriction_div = '0'
    u1.ship_restriction_div.should == '0'
    u1.ship_restriction_div_key.should == :able
    u1.ship_restriction_div_name.should == '配送可能'
    u1.ship_restriction_div = '1'
    u1.ship_restriction_div.should == '1'
    u1.ship_restriction_div_key.should == :disable
    u1.ship_restriction_div_name.should == '配送不可'
  end
  
  it "should get country list" do
    country_list = Country.country_list
    country_list.size.should == 246
    country_list.first.should == ['Afghanistan', 'AF']
  end
  
  it "should get country list for ship_to" do
    country_list = Country.ship_to_country_list
    country_list.size.should == 241
    country_list.first.should == ['Afghanistan', 'AF']
  end

  it "default_country" do
    Country.default_country.should == 'JP'
    c = Country.find(:first, :conditions => {:code => 'JP'})
    c.ship_restriction_div = '1'
    c.save
    Country.default_country.should == 'AF'
  end
  it "set_ship_restriction_able" do
    result = Country.set_ship_restriction_able(['JP'])
    result.each do |res|
      if res.code == 'JP'
        res.ship_restriction_div.should == "0"
      else
        res.ship_restriction_div.should == "1"
      end
    end
    result = Country.set_ship_restriction_able(['CN'])
    result.each do |res|
      if res.code == 'CN'
        res.ship_restriction_div.should == "0"
      else
        res.ship_restriction_div.should == "1"
      end
    end
  end

  describe "get_finder_sql" do
    before do
      @select = "SELECT t.id, t.area_type_div, t.name, t.distributor_name, t.use_language_div, t.ship_restriction_div, t.code, "
      @from = <<-EOS
        FROM (
              SELECT
                countries.id,
                '1' AS area_type_div,
                countries.name,
                distributors.name AS distributor_name,
                countries.use_language_div,
                countries.ship_restriction_div ,
                countries.code
              FROM countries LEFT OUTER JOIN distributors ON countries.distributor_id = distributors.id
              UNION
              SELECT
                '' AS id,
                '2' AS area_type_div,
                prefecture_displays.name,
                '' AS distributor_name,
                '' AS use_language_div,
                '0' AS ship_restriction_div,
                prefectures.code
              FROM shipping_costs
              LEFT OUTER JOIN prefectures ON shipping_costs.area_code = prefectures.code
              LEFT OUTER JOIN prefecture_displays ON (prefectures.id = prefecture_displays.prefecture_id AND prefecture_displays.locale = 'ja')
              WHERE shipping_costs.area_type_div = '2'
        ) t
EOS
      join_column = ""
      join = ""
      ShippingCostBoundary.all.each_with_index do |boundary, index|
        join_column = join_column + "sc#{index}.shipping_cost AS shipping_cost_#{index}, "
        join = join + " LEFT OUTER JOIN shipping_costs AS sc#{index} ON (#{boundary.id} = sc#{index}.shipping_cost_boundary_id AND t.code = sc#{index}.area_code) "
      end
      join_column = join_column.slice(0, join_column.size-2)
      @select = @select + join_column
      @join = join
      @condition = Country::SearchCondition.new()
    end
    it "閾値マスタにデータがない場合" do
      ShippingCostBoundary.destroy_all
      result = @condition.get_finder_sql
      where = " WHERE t.area_type_div = ?"
      select = "SELECT t.id, t.area_type_div, t.name, t.distributor_name, t.use_language_div, t.ship_restriction_div, t.code"
      sql = select + @from + where
      result.should == [sql, "1"]
    end
    it "検索条件無し" do
      result = @condition.get_finder_sql
      where = " WHERE t.area_type_div = ?"
      sql = @select + @from + @join + where
      result.should == [sql, "1"]
    end
    it "国名" do
      @condition.name = "Ja"
      result = @condition.get_finder_sql
      #where = " WHERE t.area_type_div = '1' AND name like ? AND t.area_type_div = ?"
      where = " WHERE name like ? AND t.area_type_div = ?"
      sql = @select + @from + @join + where
      result.should == [sql,"%Ja%", "1"]
    end
    it "国名頭文字" do
      @condition.name_top = "J"
      result = @condition.get_finder_sql
      #where = " WHERE t.area_type_div = '1' AND name like ? AND t.area_type_div = ?"
      where = " WHERE name like ? AND t.area_type_div = ?"
      sql = @select + @from + @join + where
      result.should == [sql,"J%", "1"]
    end
    it "都道府県名（部分一致）" do
      @condition.prefecture_name = "愛知"
      result = @condition.get_finder_sql
      #where = " WHERE t.area_type_div = '2' AND name like ? AND t.area_type_div = ?"
      where = " WHERE name like ? AND t.area_type_div = ?"
      sql = @select + @from + @join + where
      result.should == [sql,"%愛知%", "1"]
    end
    it "物流名（部分一致）" do
      @condition.distributor_name = "愛知"
      result = @condition.get_finder_sql
      #where = " WHERE distributor_name like ? AND t.area_type_div = ?"
      where = " WHERE t.area_type_div = ? AND distributor_name like ?"
      sql = @select + @from + @join + where
      result.should == [sql, "1", "%愛知%"]
    end
    it "対応言語（選択肢：英数、英数簡、英数日）" do
      @condition.use_language_div = "2"
      result = @condition.get_finder_sql
     # where = " WHERE use_language_div = ? AND t.area_type_div = ?"
      where = " WHERE t.area_type_div = ? AND use_language_div = ?"
      sql = @select + @from + @join + where
      result.should == [sql, "1", "2"]
    end
    it "配送可能区分（選択肢：配送可能、配送不可）" do
      @condition.ship_restriction_div = "1"
      result = @condition.get_finder_sql
      #where = " WHERE ship_restriction_div = ? AND t.area_type_div = ?"
      where = " WHERE t.area_type_div = ? AND ship_restriction_div = ?"
      sql = @select + @from + @join + where
      result.should == [sql, "1", "1"]
    end
    it "都道府県名を表示する" do
      @condition.prefecture_display_flg = "1"
      result = @condition.get_finder_sql
      where = " WHERE ((t.area_type_div = ? OR name = ?) OR (t.area_type_div = ?))"
      sql = @select + @from + @join + where
      result.should == [sql, "2", "japan", "1"]
    end
  end
  describe "get_shipping_costs_sql_for_prefectures" do
    before do
      @select = <<-EOS
      SELECT
        prefecture_displays.name,
        prefectures.code,
    EOS
    end
    it "idがある場合" do
      join_column = "sc0.shipping_cost AS shipping_cost_0, sc1.shipping_cost AS shipping_cost_1, sc2.shipping_cost AS shipping_cost_2, sc3.shipping_cost AS shipping_cost_3"
      join = " LEFT OUTER JOIN shipping_costs AS sc0 ON (? = sc0.shipping_cost_boundary_id AND prefectures.code = sc0.area_code)  LEFT OUTER JOIN shipping_costs AS sc1 ON (? = sc1.shipping_cost_boundary_id AND prefectures.code = sc1.area_code)  LEFT OUTER JOIN shipping_costs AS sc2 ON (? = sc2.shipping_cost_boundary_id AND prefectures.code = sc2.area_code)  LEFT OUTER JOIN shipping_costs AS sc3 ON (? = sc3.shipping_cost_boundary_id AND prefectures.code = sc3.area_code) "
      from = " FROM prefectures LEFT OUTER JOIN prefecture_displays ON (prefectures.id = prefecture_displays.prefecture_id AND prefecture_displays.locale = 'ja')"
      ids = [26,27,28,29]
      result = Country.get_shipping_costs_sql_for_prefectures(ids)
      query = [@select + join_column + from + join + "order by code"] + ids
      result.should == query
    end
    it "idが空白込みの場合" do
      join_column = "sc0.shipping_cost AS shipping_cost_0, sc1.shipping_cost AS shipping_cost_1, sc2.shipping_cost AS shipping_cost_2, sc3.shipping_cost AS shipping_cost_3"
      join = " LEFT OUTER JOIN shipping_costs AS sc0 ON (? = sc0.shipping_cost_boundary_id AND prefectures.code = sc0.area_code)  LEFT OUTER JOIN shipping_costs AS sc1 ON (? = sc1.shipping_cost_boundary_id AND prefectures.code = sc1.area_code)  LEFT OUTER JOIN shipping_costs AS sc2 ON (? = sc2.shipping_cost_boundary_id AND prefectures.code = sc2.area_code)  LEFT OUTER JOIN shipping_costs AS sc3 ON (? = sc3.shipping_cost_boundary_id AND prefectures.code = sc3.area_code) "
      from = " FROM prefectures LEFT OUTER JOIN prefecture_displays ON (prefectures.id = prefecture_displays.prefecture_id AND prefecture_displays.locale = 'ja')"
      ids = [26,"",28, ""]
      result = Country.get_shipping_costs_sql_for_prefectures(ids)
      query = [@select + join_column + from + join + "order by code"] + ids
      result.should == query
    end
    it "idがない場合県" do
      select = "      SELECT\n        prefecture_displays.name,\n        prefectures.code"
      join = " LEFT OUTER JOIN shipping_costs AS sc0 ON (? = sc0.shipping_cost_boundary_id AND prefectures.code = sc0.area_code)  LEFT OUTER JOIN shipping_costs AS sc1 ON (? = sc1.shipping_cost_boundary_id AND prefectures.code = sc1.area_code)  LEFT OUTER JOIN shipping_costs AS sc2 ON (? = sc2.shipping_cost_boundary_id AND prefectures.code = sc2.area_code)  LEFT OUTER JOIN shipping_costs AS sc3 ON (? = sc3.shipping_cost_boundary_id AND prefectures.code = sc3.area_code) "
      from = " FROM prefectures LEFT OUTER JOIN prefecture_displays ON (prefectures.id = prefecture_displays.prefecture_id AND prefecture_displays.locale = 'ja')"
      ids = []
      result = Country.get_shipping_costs_sql_for_prefectures(ids)
      query = [select + from +"order by code"] + ids
      result.should == query
    end
  end
  describe "get_sql_for_shipping_costs_countries" do
    it "idがある場合" do
      select = "SELECT countries.name, countries.code, "
      join_column = "sc0.shipping_cost AS shipping_cost_0, sc1.shipping_cost AS shipping_cost_1, sc2.shipping_cost AS shipping_cost_2, sc3.shipping_cost AS shipping_cost_3"
      join = " LEFT OUTER JOIN shipping_costs AS sc0 ON (? = sc0.shipping_cost_boundary_id AND countries.code = sc0.area_code)  LEFT OUTER JOIN shipping_costs AS sc1 ON (? = sc1.shipping_cost_boundary_id AND countries.code = sc1.area_code)  LEFT OUTER JOIN shipping_costs AS sc2 ON (? = sc2.shipping_cost_boundary_id AND countries.code = sc2.area_code)  LEFT OUTER JOIN shipping_costs AS sc3 ON (? = sc3.shipping_cost_boundary_id AND countries.code = sc3.area_code) "
      from = " FROM countries LEFT OUTER JOIN distributors ON countries.distributor_id = distributors.id "
      where = "where code in (?)"
      ids = [26,27,28,29]
      country_codes = ['CN', 'JP']
      result = Country.get_sql_for_shipping_costs_countries(ids, country_codes)
      query = [select + join_column + from + join + where] + ids
      query << country_codes
      result.should == query
    end
    it "idが空白込みの場合" do
      select = "SELECT countries.name, countries.code, "
      join_column = "sc0.shipping_cost AS shipping_cost_0, sc1.shipping_cost AS shipping_cost_1, sc2.shipping_cost AS shipping_cost_2, sc3.shipping_cost AS shipping_cost_3"
      join = " LEFT OUTER JOIN shipping_costs AS sc0 ON (? = sc0.shipping_cost_boundary_id AND countries.code = sc0.area_code)  LEFT OUTER JOIN shipping_costs AS sc1 ON (? = sc1.shipping_cost_boundary_id AND countries.code = sc1.area_code)  LEFT OUTER JOIN shipping_costs AS sc2 ON (? = sc2.shipping_cost_boundary_id AND countries.code = sc2.area_code)  LEFT OUTER JOIN shipping_costs AS sc3 ON (? = sc3.shipping_cost_boundary_id AND countries.code = sc3.area_code) "
      from = " FROM countries LEFT OUTER JOIN distributors ON countries.distributor_id = distributors.id "
      where = "where code in (?)"
      ids = [26,27,28,29]
      country_codes = ['CN', 'JP']
      result = Country.get_sql_for_shipping_costs_countries(ids, country_codes)
      query = [select + join_column + from + join + where] + ids
      query << country_codes
      result.should == query
    end
    it "idがない場合" do
      select = "SELECT countries.name, countries.code"
      join = " LEFT OUTER JOIN shipping_costs AS sc0 ON (? = sc0.shipping_cost_boundary_id AND countries.code = sc0.area_code)  LEFT OUTER JOIN shipping_costs AS sc1 ON (? = sc1.shipping_cost_boundary_id AND countries.code = sc1.area_code)  LEFT OUTER JOIN shipping_costs AS sc2 ON (? = sc2.shipping_cost_boundary_id AND countries.code = sc2.area_code)  LEFT OUTER JOIN shipping_costs AS sc3 ON (? = sc3.shipping_cost_boundary_id AND countries.code = sc3.area_code) "
      from = " FROM countries LEFT OUTER JOIN distributors ON countries.distributor_id = distributors.id "
      where = "where code in (?)"
      ids = []
      country_codes = ['CN', 'JP']
      result = Country.get_sql_for_shipping_costs_countries(ids, country_codes)
      query = [select + from + where] + ids
      query << country_codes
      result.should == query
    end
  end
end
