require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CsvDownloadPatternItem do
  before(:each) do
  end
  
  describe "name_with_remark" do 
    fixtures :kbn_codes, :kbn_code_names
    it "凡例あり" do
      item = CsvDownloadPatternItem.new(:code => "123")
      item.name_with_remark.should == '会員種別（1:WEB会員、3:オフライン専用会員、0:非会員、2:退会者）'
    end
    
    it "凡例なし" do
      item = CsvDownloadPatternItem.new(:code => "101")
      item.name_with_remark.should == '会員コード'
    end
  end
  
  describe "options_by_type" do 
    it "会員情報（会員単位）" do
      options = CsvDownloadPatternItem.options_by_type(:user).flatten
      options.include?('101').should be_true
      options.include?('253').should be_false
    end
    it "会員情報（会員単位）" do
      options = CsvDownloadPatternItem.options_by_type(:user_address).flatten
      options.include?('101').should be_true
      options.include?('301').should be_false
    end
    it "商品情報（商品単位）" do
      options = CsvDownloadPatternItem.options_by_type(:product).flatten
      options.include?('201').should be_true
      options.include?('253').should be_false
    end
    it "商品情報（SKU単位）" do
      options = CsvDownloadPatternItem.options_by_type(:product_sku).flatten
      options.include?('201').should be_true
      options.include?('253').should be_false
    end
    it "商品情報（言語単位）" do
      options = CsvDownloadPatternItem.options_by_type(:product_display).flatten
      options.include?('201').should be_true
      options.include?('253').should be_false
    end
    it "商品情報（売価単位）" do
      options = CsvDownloadPatternItem.options_by_type(:product_price).flatten
      options.include?('201').should be_true
      options.include?('253').should be_false
    end
    it "商品情報（構成単位）" do
      options = CsvDownloadPatternItem.options_by_type(:product_component).flatten
      options.include?('201').should be_true
      options.include?('253').should be_true
    end
    it "注文情報（注文単位）" do
      options = CsvDownloadPatternItem.options_by_type(:order).flatten
      options.include?('301').should be_true
      options.include?('350').should be_false
      options.include?('370').should be_false
    end
    it "注文情報（配送単位）" do
      options = CsvDownloadPatternItem.options_by_type(:order_shipping).flatten
      options.include?('301').should be_true
      options.include?('350').should be_true
      options.include?('370').should be_false
    end
    it "注文情報（商品単位）" do
      options = CsvDownloadPatternItem.options_by_type(:order_item).flatten
      options.include?('301').should be_true
      options.include?('350').should be_true
      options.include?('370').should be_true
    end
    it "注文情報（構成単位）" do
      options = CsvDownloadPatternItem.options_by_type(:order_item_component).flatten
      options.include?('300').should be_false
      options.include?('301').should be_true
      options.include?('409').should be_true
      options.include?('410').should be_false
    end
    it "経理情報" do
      options = CsvDownloadPatternItem.options_by_type(:accounting).flatten
      options.include?('505').should be_true
      options.include?('507').should be_true
      options.include?('508').should be_true
    end
  end
end
