require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CsvDownloadPattern do
  before(:each) do
    @pattern = CsvDownloadPattern.new(:download_type => '1', :name => 'new pattern', :character_code => '1', :file_name => "new_csv")
  end
  
  describe "validate" do 
    describe "valid" do 
      it "出力項目１個（内追加項目０個）＆固定値０個" do
        @pattern.items_attributes = {:fixed_value_list => "", :item_list => "101:会員コード"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "会員コード"
        item.code.should == "101"
        item.value.should be_nil
        item.display_order.should == 0
        
        @pattern.items_attributes = {:fixed_value_list => nil, :item_list => "101:会員コード"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "会員コード"
        item.code.should == "101"
        item.value.should be_nil
        item.display_order.should == 0
      end
      
      it "出力項目１個（内追加項目１個）＆固定値１個" do
        @pattern.items_attributes = {:fixed_value_list => '"項目名１:値１"', :item_list => "000:項目名１"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "項目名１"
        item.code.should == "000"
        item.value.should == "値１"
        item.display_order.should == 0
      end
      
      it "出力項目３個（内追加項目２個）＆固定値３つ" do
        @pattern.items_attributes = {:fixed_value_list => '"会員コード:xxx""item_3:3""item 4:4"', 
          :item_list => "101:会員コード,106:電話番号,000:item_3,000:item 4"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "会員コード"
        item.code.should == "101"
        item.value.should == "xxx"
        item.display_order.should == 0
        item = @pattern.items[1]
        item.name== "電話番号"
        item.code.should == "106"
        item.value.should be_nil
        item.display_order.should == 1
        item = @pattern.items[2]
        item.name== "item_3"
        item.code.should == "000"
        item.value.should == "3"
        item.display_order.should == 2
        item = @pattern.items[3]
        item.name== "item 4"
        item.code.should == "000"
        item.value.should == "4"
        item.display_order.should == 3
      end
      
      it "出力項目２個（内追加項目０個）＆固定値２個（改行あり）" do
      fixed_values = <<-"EOS"
"会員コード:xxx",
"電話番号:yyy"
EOS
        @pattern.items_attributes = {:fixed_value_list => fixed_values,
          :item_list => "101:会員コード,106:電話番号"}
          
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "会員コード"
        item.code.should == "101"
        item.value.should == "xxx"
        item.display_order.should == 0
        item = @pattern.items[1]
        item.name== "電話番号"
        item.code.should == "106"
        item.value.should == "yyy"
        item.display_order.should == 1
      end
      
      it "出力項目３個（内追加項目３個）＆固定値２個（空白あり）" do
        @pattern.items_attributes = {:fixed_value_list => '" 先頭スペース: 1""末尾スペース :2 "" 先頭と末尾スペース : 3 "',
          :item_list => "000:先頭スペース,000:末尾スペース,000:先頭と末尾スペース"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "先頭スペース"
        item.code.should == "000"
        item.value.should == " 1"
        item.display_order.should == 0
        item = @pattern.items[1]
        item.name== "末尾スペース"
        item.code.should == "000"
        item.value.should == "2 "
        item.display_order.should == 1
        item = @pattern.items[2]
        item.name== "先頭と末尾スペース"
        item.code.should == "000"
        item.value.should == " 3 "
        item.display_order.should == 2
      end
      
      it "固定値のコロンが多い" do
        @pattern.items_attributes = {:fixed_value_list => '"項目名１:値:１""項目名２:値２"', :item_list => "000:項目名１,000:項目名２"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "項目名１"
        item.code.should == "000"
        item.value.should == "値:１"
        item.display_order.should == 0
        item = @pattern.items[1]
        item.name== "項目名２"
        item.code.should == "000"
        item.value.should == "値２"
        item.display_order.should == 1
      end
      
      it "固定値の値未入力" do
        @pattern.items_attributes = {:fixed_value_list => '"項目名１:"', :item_list => "000:項目名１"}
        @pattern.valid?.should be_true
        item = @pattern.items[0]
        item.name== "項目名１"
        item.code.should == "000"
        item.value.should == ""
        item.display_order.should == 0
      end
    end
  
    describe "invalid" do 
      it "出力項目未入力" do
        @pattern.items_attributes = {:fixed_value_list => "", :item_list => ""}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "出力項目を選択してください。"
      end
      
      it "固定値のダブルクオートの囲み方が不正" do
        @pattern.items_attributes = {:fixed_value_list => '"項目名１:値１"""項目名２:値２"', :item_list => "000:項目名１,000:項目名２"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値のフォーマットが不正です。"
        @pattern.errors.clear
        @pattern.items_attributes = {:fixed_value_list => '項目名１:""', :item_list => "000:項目名１"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値のフォーマットが不正です。"
        @pattern.errors.clear
        @pattern.items_attributes = {:fixed_value_list => '"項目名１":""', :item_list => "000:項目名１"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値のフォーマットが不正です。"
      end
      
      it "固定値のコロンが足りない" do
        @pattern.items_attributes = {:fixed_value_list => '"項目名１値１""項目名２:値２"', :item_list => "000:項目名１,000:項目名２"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値のフォーマットが不正です。"
      end
      
      it "固定値の項目名未入力" do
        @pattern.items_attributes = {:fixed_value_list => '":値１"', :item_list => "000:項目名１"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値のフォーマットが不正です。"
      end
      
      it "固定値の項目名が出力項目名と合っていない" do
        @pattern.items_attributes = {:fixed_value_list => '"おかしな項目名:xxx"', :item_list => "101:会員コード"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値の項目名と出力項目名が合っていません。 おかしな項目名"
        @pattern.errors.clear
        @pattern.items_attributes = {:fixed_value_list => '"おかしな項目名１:xxx""おかしな項目名２:yyy"', :item_list => "101:会員コード"}
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == "固定値の項目名と出力項目名が合っていません。 おかしな項目名１,おかしな項目名２"
      end
      
      it "会員検索条件の日付不正" do
        @pattern.items_attributes = {:fixed_value_list => "", :item_list => "101:会員コード"}
        @pattern.set_condition({:regist_date_from => "abcdefg"})
        @pattern.valid?.should be_false
        @pattern.errors.on('base').should == ["検索開始日付 はYYYY/MM/DDで入力してください。", "検索開始日付 は正しくありません。"]
      end
    end
  end
  
  describe "set_condition" do 
    it "会員情報（住所単位）" do
      pattern = CsvDownloadPattern.new(:download_type => '1')
      pattern.set_condition({})
      pattern.user_condition.is_a?(User::SearchConditionForm)
    end
    
    it "商品情報（商品単位）" do
      pattern = CsvDownloadPattern.new(:download_type => '3')
      pattern.set_condition({})
      pattern.product_condition.is_a?(Product::SearchCondition)
    end
    
    it "注文情報（注文単位）" do
      pattern = CsvDownloadPattern.new(:download_type => '8')
      pattern.set_condition({})
      pattern.product_condition.is_a?(Order::SearchInfo)
    end

    it "経理情報" do
      pattern = CsvDownloadPattern.new(:download_type => '11')
      pattern.set_condition({})
      pattern.product_condition.is_a?(Accounting::SearchInfo)
    end
  end
  
  describe "uneditable?" do 
    it "変更可" do
      pattern = CsvDownloadPattern.new(:file_name => 'editable_file_name')
      pattern.uneditable?.should be_false
    end
    
    it "変更不可" do
      pattern = CsvDownloadPattern.new(:file_name => 'product_sku_bulk_upload_template')
      pattern.uneditable?.should be_true
    end
  end
  
  describe "name_options" do 
    fixtures :csv_download_patterns, :csv_download_pattern_items, :csv_download_pattern_user_conditions
    
    it "会員情報登録済みパターン" do
      @pattern.name_options.should == [["全会員", 1], ["会員一括アップロード雛形", 20]]
    end
  end
  
  describe "output_selects" do 
    fixtures :csv_download_patterns, :csv_download_pattern_items, :csv_download_pattern_user_conditions
    
    it "登録前" do
        @pattern.items_attributes = {:item_list => "101:会員コード,106:電話番号"}
        @pattern.output_selects.should == [["会員コード", "101:会員コード"], ["電話番号", "106:電話番号"]]
    end
    
    it "登録後" do
      pattern = CsvDownloadPattern.find(1, :include => [:items, :user_condition])
      pattern.output_selects.should == [["会員コード", "101:会員コード"], ["氏名漢字（姓）", "102:氏名漢字（姓）"], 
        ["氏名フリガナ（姓）", "103:氏名フリガナ（姓）"], ["氏名漢字（名）", "104:氏名漢字（名）"], ["氏名フリガナ（名）", "105:氏名フリガナ（名）"], 
        ["電話番号", "106:電話番号"], ["国", "107:国"], ["郵便番号", "108:郵便番号"], ["都道府県（住所１）", "109:都道府県（住所１）"], 
        ["市区町村（住所２）", "110:市区町村（住所２）"], ["番地・マンション名（住所３）", "111:番地・マンション名（住所３）"], 
        ["会員種別", "121:会員種別"], ["追加項目１", "000:追加項目１"]]
    end
  end
  
  describe "set_detail_for_edit" do 
    fixtures :csv_download_patterns, :csv_download_pattern_items,
      :csv_download_pattern_user_conditions, :csv_download_pattern_product_conditions, :csv_download_pattern_order_conditions
    
    it "会員情報（住所単位）" do
      pattern = CsvDownloadPattern.find(1, :include => [:items, :user_condition])
      pattern.items_attributes.should be_nil
      condition = pattern.set_detail_for_edit
      pattern.items_attributes[:item_list].should_not == ""
      pattern.items_attributes[:fixed_value_list].should_not == ""
      condition.is_a?(User::SearchConditionForm)
    end
    
    it "商品情報（売価単位）" do
      pattern = CsvDownloadPattern.find(29, :include => [:items, :product_condition])
      pattern.items_attributes.should be_nil
      condition = pattern.set_detail_for_edit
      pattern.items_attributes[:item_list].should_not == ""
      pattern.items_attributes[:fixed_value_list].should == ""
      condition.is_a?(Product::SearchCondition)
    end
    
    it "注文情報（商品単位）" do
      pattern = CsvDownloadPattern.find(39, :include => [:items, :order_condition])
      pattern.items_attributes.should be_nil
      condition = pattern.set_detail_for_edit
      pattern.items_attributes[:item_list].should_not == ""
      pattern.items_attributes[:fixed_value_list].should == "\"数量単位:PCS\"\"通貨単位:CNY\""
      condition.is_a?(Order::SearchInfo)
    end

    it "経理情報" do
      pattern = CsvDownloadPattern.find(44, :include => [:items, :accounting_condition])
      pattern.items_attributes.should be_nil
      condition = pattern.set_detail_for_edit
      pattern.items_attributes[:item_list].should_not == ""
      condition.is_a?(Accounting::SearchInfo)
    end
  end
  
  describe "destroy_item_and_condition" do 
    fixtures :csv_download_patterns, :csv_download_pattern_items,
      :csv_download_pattern_user_conditions, :csv_download_pattern_product_conditions, 
      :csv_download_pattern_order_conditions, :csv_download_pattern_accounting_conditions
    
    it "会員情報（住所単位）" do
      before_count_pattern = CsvDownloadPattern.count
      before_count_item = CsvDownloadPatternItem.count
      before_count_cond = CsvDownloadPatternUserCondition.count
      pattern = CsvDownloadPattern.find(1, :include => [:items, :user_condition])
      pattern.destroy_item_and_condition
      CsvDownloadPattern.count.should == before_count_pattern
      CsvDownloadPatternItem.count.should < before_count_item
      CsvDownloadPatternUserCondition.count.should < before_count_cond
    end
    
    it "商品情報（売価単位）" do
      before_count_pattern = CsvDownloadPattern.count
      before_count_item = CsvDownloadPatternItem.count
      before_count_cond = CsvDownloadPatternProductCondition.count
      pattern = CsvDownloadPattern.find(29, :include => [:items, :product_condition])
      pattern.destroy_item_and_condition
      CsvDownloadPattern.count.should == before_count_pattern
      CsvDownloadPatternItem.count.should < before_count_item
      CsvDownloadPatternProductCondition.count.should < before_count_cond
    end
    
    it "注文情報（商品単位）" do
      before_count_pattern = CsvDownloadPattern.count
      before_count_item = CsvDownloadPatternItem.count
      before_count_cond = CsvDownloadPatternOrderCondition.count
      pattern = CsvDownloadPattern.find(39, :include => [:items, :order_condition])
      pattern.destroy_item_and_condition
      CsvDownloadPattern.count.should == before_count_pattern
      CsvDownloadPatternItem.count.should < before_count_item
      CsvDownloadPatternOrderCondition.count.should < before_count_cond
    end

    it "経理情報" do
      before_count_pattern = CsvDownloadPattern.count
      before_count_item = CsvDownloadPatternItem.count
      before_count_cond = CsvDownloadPatternAccountingCondition.count
      pattern = CsvDownloadPattern.find(44, :include => [:items, :accounting_condition])
      pattern.destroy_item_and_condition
      CsvDownloadPattern.count.should == before_count_pattern
      CsvDownloadPatternItem.count.should < before_count_item
      CsvDownloadPatternAccountingCondition.count.should < before_count_cond
    end
  end
  
  describe "generate" do 
    fixtures :csv_download_patterns, :csv_download_pattern_items, 
      :csv_download_pattern_user_conditions, :csv_download_pattern_product_conditions, :csv_download_pattern_order_conditions,
      :users, :user_addresses, :user_emails, :kbn_codes,
      :products, :product_displays, :product_categories, :product_prices, :product_skus, :user_ranks, :product_variations, :product_components,
      :orders, :order_shippings, :order_items, :csv_download_pattern_accounting_conditions, :accountings
    
    it "会員情報（住所単位）" do
      pattern = CsvDownloadPattern.find(30, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should > 2
    end
    
    it "商品情報（SKU単位）" do
      pattern = CsvDownloadPattern.find(28, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should > 2
    end
    
    it "商品情報（言語単位）" do
      pattern = CsvDownloadPattern.find(31, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should > 2
    end
    
    it "商品情報（売価単位）" do
      pattern = CsvDownloadPattern.find(29, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should > 2
    end
    
    it "商品情報（構成単位）" do
      ProductComponent.create!(:product_sku_id => 291, :count => 1, :component_sku_id => 282)
      ProductComponent.create!(:product_sku_id => 291, :count => 1, :component_sku_id => 283)
      ProductComponent.create!(:product_sku_id => 291, :count => 1, :component_sku_id => 284)
      pattern = CsvDownloadPattern.find(32, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should == 4
    end
    
    it "注文情報（配送単位）" do
      pattern = CsvDownloadPattern.find(38, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should > 2
    end
    
    it "注文情報（商品単位）" do
      pattern = CsvDownloadPattern.find(39, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should == 17
    end

    it "注文情報（構成単位）" do
      # 注文情報（商品単位）のパターンをもとに、注文情報（構成単位）のパターンを作成してテストを行う。
      OrderItemComponent.create!(:order_item_id => 1, :component_sku_id => 282)
      OrderItemComponent.create!(:order_item_id => 1, :component_sku_id => 283)
      pattern = CsvDownloadPattern.find(39, :include => [:items, :user_condition, :order_condition, :product_condition])
      pattern.download_type = 12
      csv = pattern.generate
      csv.to_a.length.should == 18
    end

    it "経理情報" do
      pattern = CsvDownloadPattern.find(44, :include => [:items, :user_condition, :order_condition, :product_condition, :accounting_condition])
      csv = pattern.generate
      csv.to_a.length.should == 2
    end
    
    it "検索結果0件" do
      User.stub!(:find).and_return([])
      pattern = CsvDownloadPattern.find(30, :include => [:items, :user_condition, :order_condition, :product_condition])
      csv = pattern.generate
      csv.to_a.length.should == 2
    end
  end

  describe "format_value" do
    before do
      @csv_download_pattern = CsvDownloadPattern.new()
    end
    it ":date" do
      @csv_download_pattern.format_value(:date, Date.new(2009,3,26)).should == "2009/03/26"
    end
    it ":date_join" do
      @csv_download_pattern.format_value(:date_join, Date.new(2009,3,26)).should == "20090326"
    end
    it ":date_join_short" do
      @csv_download_pattern.format_value(:date_join_short, Date.new(2009,3,26)).should == "090326"
    end
    it ":date_time" do
      @csv_download_pattern.format_value(:date_time, Time.local(2009,3,26, 00,00,00)).should == "2009/03/26 00:00"
    end
    it ":onebyte_kana" do
      @csv_download_pattern.format_value(:onebyte_kana, "カナ").should == "ｶﾅ"
    end
  end
end
