# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe MailDelivery do
  class TestMailer < ActionMailer::Base
    def test_mail
      from       "test_from@test.com"
      recipients "test_to@test.com"
      subject    "テストメール"
      body       "テストメール本文"
    end
  end

  it "site_div means which site mail will be deliveried from." do
    MailDelivery.site_div_options.should == [
      ["自社", '1'],["楽天", '2'],["YAHOO", '3']
    ]
    MailDelivery.site_div_id_by_key(:jisha).should == "1"
    MailDelivery.site_div_id_by_key(:rakuten).should == "2"
    MailDelivery.site_div_id_by_key(:yahoo).should == "3"
  end

  describe "SearchConditionForm check" do
    it "test deliveried_at_from_unempty?" do
      m1 = MailDelivery::SearchConditionForm.new
      m1.deliveried_at_from_unempty?.should == false

      m1.deliveried_at_from = '2009/01/01'
      m1.deliveried_at_from_unempty?.should == true
    end
    it "test deliveried_at_to_unempty?" do
      m1 = MailDelivery::SearchConditionForm.new
      m1.deliveried_at_to_unempty?.should == false

      m1.deliveried_at_to = '2009/01/01'
      m1.deliveried_at_to_unempty?.should == true
    end
    it "test validate with date empty" do
      m1 = MailDelivery::SearchConditionForm.new
      m1.validate
      m1.errors.length.should == 0
    end
    it "test validate with valid date" do
      m1 = MailDelivery::SearchConditionForm.new(:deliveried_at_from => '2009/01/01', :deliveried_at_to => '2009/01/02')
      m1.validate
      m1.errors.length.should == 0
    end
    it "test validate with invalid date" do
      m1 = MailDelivery::SearchConditionForm.new(:deliveried_at_from => '2009/01/32', :deliveried_at_to => '2009/01/33')
      m1.validate
      m1.errors.length.should == 2
    end
    it "test deliveried_at_from_time" do
      m1 = MailDelivery::SearchConditionForm.new(:deliveried_at_from => '2009/01/01', :deliveried_at_to => '2009/01/02')
      m1.deliveried_at_from_time.should == Time.parse('2009/01/01 00:00:00')
    end
    it "test deliveried_at_to_time" do
      m1 = MailDelivery::SearchConditionForm.new(:deliveried_at_from => '2009/01/01', :deliveried_at_to => '2009/01/02')
      m1.deliveried_at_to_time.should == Time.parse('2009/01/02 23:59:59')
    end
  end

  it "test create mail history" do
    tmail = TestMailer.create_test_mail
    tmail.to = ['test_to@test.com']
    m1 = MailDelivery.create_front_mail_history(tmail, nil, nil, 1, :free, Time.parse('2009/01/01 10:00:00'))
    m1.user_id.should == 1
    m1.mail_div_key.should == :free
    m1.deliveried_at.should == Time.parse('2009/01/01 10:00:00')
    m1.header.should == ''
    m1.to_address.should == 'test_to@test.com'
    m1.from_address.should == 'test_from@test.com'
    m1.message_id.should == tmail.message_id
    m1.subject.should == NKF::nkf("-w", tmail.subject)
    m1.mail_body.should == NKF::nkf("-w", tmail.body)
  end

  it "test create mail history あて先が複数ある場合" do
    tmail = TestMailer.create_test_mail
    tmail.to = ['m-user@ec-one.com', 'm-user@ec-one.com', 'm-user@ec-one.com']
    m1 = MailDelivery.create_front_mail_history(tmail, nil, nil, 1, :free, Time.parse('2009/01/01 10:00:00'))
    m1.user_id.should == 1
    m1.mail_div_key.should == :free
    m1.deliveried_at.should == Time.parse('2009/01/01 10:00:00')
    m1.header.should == ''
    m1.to_address.should == 'm-user@ec-one.com,m-user@ec-one.com,m-user@ec-one.com'
    m1.from_address.should == 'test_from@test.com'
    m1.message_id.should == tmail.message_id
    m1.subject.should == NKF::nkf("-w", tmail.subject)
    m1.mail_body.should == NKF::nkf("-w", tmail.body)
  end

  it "test create mail history with subject and body" do
    tmail = TestMailer.create_test_mail

    m1 = MailDelivery.create_front_mail_history(tmail, 'さぶじぇくと', 'ぼでぃ', 1, :free, Time.parse('2009/01/01 10:00:00'))
    m1.user_id.should == 1
    m1.mail_div_key.should == :free
    m1.deliveried_at.should == Time.parse('2009/01/01 10:00:00')
    m1.header.should == ''
    m1.to_address.should == 'test_to@test.com'
    m1.from_address.should == 'test_from@test.com'
    m1.message_id.should == tmail.message_id
    m1.subject.should == 'さぶじぇくと'
    m1.mail_body.should == 'ぼでぃ'
  end

  it "test create_condition" do
    search_info = MailDelivery::SearchConditionForm.new(:mail_div => '0', :deliveried_at_from => '2009/01/01', :deliveried_at_to => '2009/01/02',
      :email => 'testest@test.com', :name => 'タムラ', :member_div => '', :user_code => '')
    c1 = MailDelivery.create_condition(search_info, Time.parse('2009/01/20 10:00:00'))
    c1.should ==
      ["(CONCAT(users.name_last, users.name_first) like ? OR CONCAT(users.name_first, users.name_last) like ? OR CONCAT(users.kana_name_last, users.kana_name_first) like ? " + 
       "OR CONCAT(users.kana_name_first, users.kana_name_last) like ? OR CONCAT(inquiries.name_last, inquiries.name_first) like ? OR CONCAT(inquiries.name_first, inquiries.name_last) like ? " +
       "OR CONCAT(inquiries.kana_name_last, inquiries.kana_name_first) like ? OR CONCAT(inquiries.kana_name_first, inquiries.kana_name_last) like ?) AND to_address like ? " +
        "AND mail_div = ? AND mail_deliveries.deliveried_at >= ? AND mail_deliveries.deliveried_at <= ? AND (mail_deliveries.deleted_at IS NULL OR mail_deliveries.deleted_at > ?)",
       "%タムラ%", "%タムラ%", "%タムラ%", "%タムラ%", "%タムラ%", "%タムラ%", "%タムラ%", "%タムラ%", "%testest@test.com%", "0", Time.parse('2009/01/01 00:00:00'),
        Time.parse('2009/01/02 23:59:59'), "2009-01-20 10:00:00"]
    MailDelivery.find(:all, :conditions => c1, :include => [:user, :inquiry]).size.should == 0
  end
end
