# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe OrderShipping do
  include EcRider::PluginModule::RSpecExt
  fixtures :kbn_codes, :kbn_code_names, :countries,
    :distributors, :distributor_delivery_times, :distributor_delivery_time_displays,
    :orders, :order_shippings, :order_items,
    :users, :user_emails, :user_addresses,
    :products, :product_skus, :product_sku_inventories, :product_sku_secure_inventories,
    :kbn_code_names, :kbn_codes, :shop_settings, 
    :mail_deliveries, :mail_templates,
    :accountings

  describe "A division value check" do
    it "arrival_hour means hour of arrival items." do
      OrderShipping.arrival_hour_options.should == [
        ["配送時間を指定しない", '0']]
      OrderShipping.arrival_hour_name_by_id('0').should == "配送時間を指定しない"

      oc1 = OrderShipping.new
      oc1.arrival_hour.should be_nil
      oc1.arrival_hour_key.should be_nil
      oc1.arrival_hour_name.should be_nil
      oc1.arrival_hour = '0'
      oc1.arrival_hour.should == '0'
      oc1.arrival_hour_key.should == :none
      oc1.arrival_hour_name.should == '配送時間を指定しない'
      oc1.arrival_hour = '1'
      oc1.arrival_hour.should == '1'
    end

    it "payment_type_div means how to pay for the ordered items." do
      OrderShipping.payment_type_div_options.should == [["代金引換", '1'],["クレジットカード", '2'],['アリペイ決済', '3'],["銀行口座振込", '4'],["コンビニ決済", '5'],["PayPal決済", '6'],["携帯キャリア決済", '7']]
      OrderShipping.payment_type_div_name_by_id('1').should == "代金引換"
      OrderShipping.payment_type_div_name_by_id('2').should == "クレジットカード"
      OrderShipping.payment_type_div_name_by_id('3').should == "アリペイ決済"
      OrderShipping.payment_type_div_name_by_id('4').should == "銀行口座振込"
      OrderShipping.payment_type_div_name_by_id('5').should == "コンビニ決済"
      OrderShipping.payment_type_div_name_by_id('6').should == "PayPal決済"
      OrderShipping.payment_type_div_name_by_id('7').should == "携帯キャリア決済"

      oc1 = OrderShipping.new
      oc1.payment_type_div.should be_nil
      oc1.payment_type_div_key.should be_nil
      oc1.payment_type_div_name.should be_nil
      oc1.payment_type_div = '1'
      oc1.payment_type_div.should == '1'
      oc1.payment_type_div_key.should == :cod
      oc1.payment_type_div_name.should == '代金引換'
      oc1.payment_type_div_entry[:pay_in_advance].should == true
      oc1.payment_type_div_entry[:default_status].should == :undelivery
      oc1.payment_type_div = '2'
      oc1.payment_type_div.should == '2'
      oc1.payment_type_div_key.should == :credit
      oc1.payment_type_div_name.should == 'クレジットカード'
      oc1.payment_type_div_entry[:pay_in_advance].should == true
      oc1.payment_type_div_entry[:default_status].should == :undelivery
      oc1.payment_type_div = '3'
      oc1.payment_type_div.should == '3'
      oc1.payment_type_div_key.should == :alipay
      oc1.payment_type_div_name.should == 'アリペイ決済'
      oc1.payment_type_div_entry[:pay_in_advance].should == true
      oc1.payment_type_div_entry[:default_status].should == :undelivery_settlement
      oc1.payment_type_div = '4'
      oc1.payment_type_div.should == '4'
      oc1.payment_type_div_key.should == :bank_transfer
      oc1.payment_type_div_name.should == '銀行口座振込'
      oc1.payment_type_div_entry[:default_status].should == :undelivery_unpaid
    end

    it "payment_type_code_yamato" do
      os1 = OrderShipping.new(:payment_type_div_key => :cod)
      os1.payment_type_code_yamato.should == 2
      os2 = OrderShipping.new(:payment_type_div_key => :credit)
      os2.payment_type_code_yamato.should == 0
    end

    it "delivery_div means how to delivery the ordered items." do
      OrderShipping.delivery_div_options.should == [["通常便", '0'],["メール便", '1']]
      OrderShipping.delivery_div_name_by_id('0').should == "通常便"
      OrderShipping.delivery_div_name_by_id('1').should == "メール便"

      oc1 = OrderShipping.new
      oc1.delivery_div.should be_nil
      oc1.delivery_div_key.should be_nil
      oc1.delivery_div_name.should be_nil
      oc1.delivery_div = '0'
      oc1.delivery_div.should == '0'
      oc1.delivery_div_key.should == :normal
      oc1.delivery_div_name.should == '通常便'
      oc1.delivery_div = '1'
      oc1.delivery_div.should == '1'
      oc1.delivery_div_key.should == :mail
      oc1.delivery_div_name.should == 'メール便'
    end

    it "status_div means which the online payment is first time or again." do
      OrderShipping.status_div_options.should == [
        ["決済前", "14"], 
        ["決済中", "12"], 
        ["入金待ち", "11"], 
        ["配送準備中", "0"], 
        ["配送指示済", "1"], 
        ["配送済", "2"], 
        ["配送案内済", "3"], 
        ["キャンセル依頼中", "10"], 
        ["キャンセル完了", "8"],
        ["決済NG", "13"], 
        ["在庫不足エラー", "15"], 
        ["内部エラー", "16"], 
        ["外部エラー", "17"],
        ["返品完了", "18"],
        ["販売期間エラー", "19"]
      ]
      OrderShipping.status_div_id_by_key(:undelivery_unpaid).should == "11"
      OrderShipping.status_div_id_by_key(:undelivery).should == "0"
      OrderShipping.status_div_id_by_key(:deliver_request).should == "1"
      OrderShipping.status_div_entry_by_key(:deliver_request)[:name_for_front].should == 'order_shippings.status_div.name_for_front.deliver_request'
      I18n.t(OrderShipping.status_div_entry_by_key(:deliver_request)[:name_for_front]).should == '発送手配中'
      OrderShipping.status_div_id_by_key(:delivered).should == "2"
      OrderShipping.status_div_entry_by_key(:delivered)[:name_for_front].should == 'order_shippings.status_div.name_for_front.delivered'
      OrderShipping.status_div_id_by_key(:deliver_advised).should == "3"
      OrderShipping.status_div_entry_by_key(:deliver_advised)[:name_for_front].should == 'order_shippings.status_div.name_for_front.deliver_advised'
      OrderShipping.status_div_id_by_key(:canceled).should == "8"
      OrderShipping.status_div_id_by_key(:cancel_request).should == "10"
    end

    it "csv_output_div means csv output status." do
      OrderShipping.csv_output_div_options.should == [["未出力", '0'],["出力済", '1']]
      OrderShipping.csv_output_div_name_by_id(0).should == "未出力"
      OrderShipping.csv_output_div_name_by_id(1).should == "出力済"
      OrderShipping.csv_output_div_id_by_key(:not_yet).should == '0'
      OrderShipping.csv_output_div_id_by_key(:done).should == '1'

      u1 = OrderShipping.new
      u1.csv_output_div.should be_nil
      u1.csv_output_div_key.should be_nil
      u1.csv_output_div_name.should be_nil
      u1.csv_output_div = '0'
      u1.csv_output_div.should == '0'
      u1.csv_output_div_key.should == :not_yet
      u1.csv_output_div_name.should == '未出力'
      u1.csv_output_div = '1'
      u1.csv_output_div.should == '1'
      u1.csv_output_div_key.should == :done
      u1.csv_output_div_name.should == '出力済'
    end
  end
  
  describe "A number of the figures check" do
    before(:each) do
      @delivery_content = ""
    end

    it "An error does not occur by a figure number of delivery_content check" do
      2.times {@delivery_content += "A"}
      order_shipping = OrderShipping.new(:delivery_content => @delivery_content)
      # 桁数チェック結果
      order_shipping.should be_valid
    end

    it "An error occurs by a figure number of inquiry check" do
      20001.times {@delivery_content += "A"}
      order_shipping = OrderShipping.new(:delivery_content => @delivery_content)
      # 桁数チェック結果
      order_shipping.errors_on(:delivery_content).should == ["は10000文字以内で入力してください。"]
    end
  end

  describe "test_update_order_items" do
    it "test_update_order_items" do
      pending
      order_shipping = OrderShipping.find(1)
      order_item = order_shipping.order_items[0]
  
      product_sku = ProductSku.find(265)
      order_shipping.order_items.length.should == 1
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 19
  
      order_item.qty = 2
      order_shipping.update_order_items([order_item])
      order_items_count = OrderItem.count(:conditions => {:order_shipping_id => order_shipping.id})
  
      product_sku = ProductSku.find(265)
      order_shipping.order_items.length.should == 1
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 18
  
      attributes = order_item.attributes.dup
      attributes.delete(:id)
      new_order_item = OrderItem.new(attributes)
      order_shipping.update_order_items([order_item, new_order_item])
      order_items_count = OrderItem.count(:conditions => {:order_shipping_id => order_shipping.id})
  
      product_sku = ProductSku.find(265)
      order_shipping.order_items.length.should == 1
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 16
  
      order_shipping.update_order_items([new_order_item])
      order_items_count = OrderItem.count(:conditions => {:order_shipping_id => order_shipping.id})
  
      product_sku = ProductSku.find(265)
      order_shipping.order_items.length.should == 1
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 18
  
      new_order_item.qty = 30
      assert_raise(OrderItem::OutOfStockError){
        order_shipping.update_order_items([new_order_item])
      }
    end
    
    it "新注文数量 > 現在注文可能数量　但し　(新注文数量 - 旧注文数量) < 現在注文可能数量" do
      # 更新前注文数量 = 20、在庫引当 = 1
      order_shipping = OrderShipping.find(1)
      order_item = order_shipping.order_items[0]
      order_item.qty = 20
      order_item.save!
      secure_stocks = order_item.product_sku_secure_inventories[0]
      secure_stocks.count = 15
      secure_stocks.save!
      ProductSku.update(265, :secure_stock_count => 15, :orderable_count => 5)
  
      # 更新前注文可能数量 = 5
      product_sku = ProductSku.find(265)
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 5
      before_sku_lock_version = product_sku.lock_version
      
      # 注文数量 = 10で更新
      attributes = order_item.attributes.dup
      attributes.delete(:id)
      new_order_item = OrderItem.new(attributes)
      new_order_item.qty = 10
      order_shipping.update_order_items([new_order_item], true)
      
      # 更新後注文数量 = 10、在庫引当 = 10、注文可能数量 = 10
      order_shipping = OrderShipping.find(1)
      order_shipping.order_items[0].qty.should == 10
      order_shipping.order_items[0].product_sku_secure_inventories.length.should == 1
      order_shipping.order_items[0].product_sku_secure_inventories[0].count.should == 10
      product_sku = ProductSku.find(265)
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 10
      product_sku.lock_version.should_not == before_sku_lock_version #排他制御チェック
    end
    
    it "(新注文数量 - 旧注文数量) > 現在注文可能数量" do
      # 更新前注文数量 = 1、在庫引当 = 1
      order_shipping = OrderShipping.find(1)
      order_item = order_shipping.order_items[0]
      order_item.qty = 1
      order_item.save!
  
      # 更新前注文可能数量 = 19
      product_sku = ProductSku.find(265)
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 19
      before_sku_lock_version = product_sku.lock_version
  
      # 注文数量 = 21で更新
      attributes = order_item.attributes.dup
      attributes.delete(:id)
      new_order_item = OrderItem.new(attributes)
      new_order_item.qty = 21
      
      # 在庫エラー
      lambda {order_shipping.update_order_items([new_order_item], true)}.should raise_error(OrderItem::OutOfStockError)
      # 更新しない
      order_shipping = OrderShipping.find(1)
      order_shipping.order_items[0].qty.should == 1
      order_shipping.order_items[0].product_sku_secure_inventories[0].count.should == 1
      product_sku = ProductSku.find(265)
      product_sku.stock_count.should == 20
      product_sku.orderable_count.should == 19
      product_sku.lock_version.should == before_sku_lock_version #排他制御チェック
    end
  end

  #
  # = secure_stocksのテスト =
  #
  it "test_secure_stocks　仮引き当て" do
    order_shipping = OrderShipping.find(1)

    product_sku = ProductSku.find(265)
    order_shipping.order_items.length.should == 1
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 19

    order_shipping.order_items[0].qty = 11
    order_shipping.secure_stocks(:before_pay)

    product_sku = ProductSku.find(265)
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 9

    order_shipping.order_items[0].save!

    order_shipping.order_items[0].qty = 1
    order_shipping.secure_stocks(:before_pay)

    product_sku = ProductSku.find(265)
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 19

    attributes = order_shipping.order_items[0].attributes.dup
    attributes.delete(:id)
    new_order_item = OrderItem.new(attributes)
    new_order_item.qty = 6
    new_order_shipping = OrderShipping.new()
    new_order_shipping.order_items << new_order_item
    new_order_shipping.order = Order.new
    new_order_shipping.save!
    new_order_shipping.secure_stocks(:before_pay)

    product_sku = ProductSku.find(265)
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 13

    new_order_shipping.order_items[0].qty = 23
    assert_raise(OrderItem::OutOfStockError){
      new_order_shipping.secure_stocks(:before_pay)
    }
  end

  #
  # = unsecure_stocksのテスト =
  #
  it "test_destroy_with_stock_control" do
    order_shipping = OrderShipping.find(1)

    product_sku = ProductSku.find(265)
    order_shipping.order_items.length.should == 1
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 19

    order_shipping.unsecure_stocks

    product_sku = ProductSku.find(265)
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 20
  end
  
  #
  # = delete_send_addressのテスト =
  #
  it "test_delete_send_address" do
    order_shipping = OrderShipping.find(10)
    assert_equal('田村', order_shipping.name_last)
    assert_equal('拓哉', order_shipping.name_first)
    assert_equal('タムラ', order_shipping.kana_name_last)
    assert_equal('タクヤ', order_shipping.kana_name_first)
    assert_equal('東京都', order_shipping.address_1)
    assert_equal('千代田区', order_shipping.address_2)
    assert_equal('東田３００－１００　千代田生命ビル１１１号', order_shipping.address_3)
    assert_equal('1000001', order_shipping.zip_code)
    assert(!order_shipping.deleted_send_address?)

    order_shipping.delete_send_address
    assert_nil(order_shipping.name_last)
    assert_nil(order_shipping.name_first)
    assert_nil(order_shipping.kana_name_last)
    assert_nil(order_shipping.kana_name_first)
    assert_nil(order_shipping.address_1)
    assert_nil(order_shipping.address_2)
    assert_nil(order_shipping.address_3)
    assert_nil(order_shipping.zip_code)
    assert(order_shipping.deleted_send_address?)
  end

  it "test_check_sales_period admin" do
    EcRider.config.stub!(:ec_admin).and_return(true)
    order_shipping = OrderShipping.find(2)
    order_shipping.order.ordered_at.should == Time.parse('2007-12-10 11:06:54')
    order_items = order_shipping.order_items
    order_items.length.should == 2
    product1 = order_items[0].product_sku.product
    product2 = order_items[1].product_sku.product
    product1.sale_started_at.should be_nil
    product1.sale_finished_at.should be_nil
    product2.sale_started_at.should be_nil
    product2.sale_finished_at.should be_nil
    
    lambda { order_shipping.check_sales_period }.should_not raise_error()

    product1.sale_started_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should_not raise_error()

    product1.sale_started_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should_not raise_error()

    product1.sale_started_at = nil
    product1.sale_finished_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should_not raise_error()
    
    product1.sale_finished_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should_not raise_error()
  end

  it "test_check_sales_period no admin" do
    EcRider.config.stub!(:ec_admin).and_return(false)
    order_shipping = OrderShipping.find(2)
    order_shipping.order.ordered_at.should == Time.parse('2007-12-10 11:06:54')
    order_items = order_shipping.order_items
    order_items.length.should == 2
    product1 = order_items[0].product_sku.product
    product2 = order_items[1].product_sku.product
    product1.sale_started_at.should be_nil
    product1.sale_finished_at.should be_nil
    product2.sale_started_at.should be_nil
    product2.sale_finished_at.should be_nil
    
    lambda { order_shipping.check_sales_period }.should_not raise_error()

    product1.sale_started_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should raise_error(OrderItem::NonPeriodicSalesError)

    product1.sale_started_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should_not raise_error()

    product1.sale_started_at = nil
    product1.sale_finished_at = Time.parse('2008-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should_not raise_error()
    
    product1.sale_finished_at = Time.parse('2006-12-10 11:06:54')
    product1.save!
    lambda { order_shipping.check_sales_period }.should raise_error(OrderItem::NonPeriodicSalesError)
  end

  it "test_check_stocks" do
    order_shipping = OrderShipping.find(1)

    product_sku = ProductSku.find(265)
    order_shipping.order_items.length.should == 1
    product_sku.stock_count.should == 20
    product_sku.orderable_count.should == 19

    lambda { order_shipping.check_stocks }.should_not raise_error()
    product_sku.orderable_count.should == 19

    order_shipping.order_items[0].qty = 11
    lambda { order_shipping.check_stocks }.should_not raise_error()
    product_sku.orderable_count.should == 19

    order_shipping.order_items[0].qty = 0
    lambda { order_shipping.check_stocks }.should_not raise_error()
    product_sku.orderable_count.should == 19

    order_shipping.order_items[0].qty = 21
    lambda { order_shipping.check_stocks }.should raise_error(OrderItem::OutOfStockError)
    product_sku.orderable_count.should == 19
  end


  it "generate_order_code method test." do
    order_shipping = OrderShipping.create
    order_shipping.order_code.should == order_shipping.id.to_s.rjust(11, '0')
  end

  it "select_form_list method test." do
    EcRider.config.stub!(:ecr_payment_types).and_return(['cod','bank_transfer','paypal','credit'])
    OrderShipping.select_form_list().should == [["代金引換", "1"], ["銀行口座振込", "4"], ["PayPal決済", "6"], ["クレジットカード", "2"]]
  end

  it "test fullname english" do
    oc1 = OrderShipping.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP')
    I18n.stub!('locale').and_return('en')
    oc1.fullname.should == 'name2 name1'
  end
  it "test fullname japanese" do
    oc1 = OrderShipping.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP')
    I18n.stub!('locale').and_return('ja')
    oc1.fullname.should == 'name1 name2'
  end
  it "test fulladdress english" do
    oc1 = OrderShipping.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP')
    I18n.stub!('locale').and_return('en')
    oc1.fulladdress.should == 'add3, add2'
  end
  it "test fulladdress japanese" do
    oc1 = OrderShipping.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP')
    I18n.stub!('locale').and_return('ja')
    oc1.fulladdress.should == 'add2 add3'
  end
  it "test fullname english with order" do
    oc1 = OrderShipping.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP')
    oc1.stub!(:order).and_return(true)
    oc1.order.stub!('locale').and_return('en')
    oc1.fullname.should == 'name2 name1'
  end
  it "test fullname japanese with order" do
    oc1 = OrderShipping.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP')
    oc1.stub!(:order).and_return(true)
    oc1.order.stub!('locale').and_return('ja')
    oc1.fullname.should == 'name1 name2'
  end
  it "test fulladdress english with order" do
    oc1 = OrderShipping.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP')
    oc1.stub!(:order).and_return(true)
    oc1.order.stub!('locale').and_return('en')
    oc1.fulladdress.should == 'add3, add2'
  end
  it "test fulladdress japanese with order" do
    oc1 = OrderShipping.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP')
    oc1.stub!(:order).and_return(true)
    oc1.order.stub!('locale').and_return('ja')
    oc1.fulladdress.should == 'add2 add3'
  end

  describe "find_includable_shipping" do
    it "同送の指定が可能な配送先一覧を取得する" do
      user_id = 1002
      order_shippings = OrderShipping.find_includable_shipping(user_id)
      order_shippings.size.should == 1
      order_shippings.each do |order_shipping|
        order_shipping.status_div.should == OrderShipping.status_div_id_by_key(:undelivery)
      end
    end
  end
  
  describe "total_weight" do
    it "重量の合計を返す" do
      order_shipping = OrderShipping.find(2)
      order_shipping.total_weight.should == 200
    end
    
    it "複数商品複数個" do
      order_shipping = OrderShipping.find(2)
      order_shipping.order_items[0].qty = 3
      order_shipping.order_items[1].qty = 5
      order_shipping.total_weight.should == 800
    end
  end
  
  describe "send_receipt_payment_mail" do
    it "入金確認メール送信" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(2)
        order_shipping.send_receipt_payment_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "receipt payment pc!"
        NKF.nkf('-w', mail.body).should == "山田 太郎様  \n\n入金を確認致しました。\n\n商品の出荷までしばらくお待ちください。"
      end
    end
    
    it "zh_CN" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(7)
        order_shipping.order.locale = 'zh_CN'
        order_shipping.order.save!
        order_shipping.shipment_content = 'ピッキング情報'
        order_shipping.invoice_content = '添付ファイルの内容'
        order_shipping.send_receipt_payment_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["email_address_01006@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "receipt payment pc!(zh-CN) 衷心感谢您对我们的支持参加"
        NKF.nkf('-w', mail.body).should == "空川 結衣  \n\npayment receipt completed.\n\n衷心感谢您对我们的支持参加"
      end
    end
  end
  
  describe "send_delivery_request_mail" do
    it "配送指示メール送信" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(2)
        order_shipping.shipment_content = 'ピッキング情報'
        order_shipping.invoice_content = '添付ファイルの内容'
        order_shipping.send_delivery_request_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test_delivery_service@ecrider.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "商品の出荷をお願いします"
        NKF.nkf('-w', mail.body).should == "ピッキング情報Attachment: invoice.html\n"
      end
    end
  end
  
  describe "send_shipment_complete_for_shop_mail" do
    fixtures :shop_settings
    it "出荷完了メール（店舗宛て）送信" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(2)
        order_shipping.send_shipment_complete_for_shop_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test_shipment_complete_for_shop@ecrider.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "出荷完了しました"
        NKF.nkf('-w', mail.body).should == "00000000002 の出荷が完了しました。"
      end
    end
  end
  
  describe "send_shipment_complete_mail" do
    it "出荷完了メール（顧客宛て）送信" do
      pending_if_loaded_plugin(:c0002)
      if  EcRider.config.ec_admin
        mail_template = MailTemplate.find(10)
        mail_template.pc_text = <<-EOF
<%=user_name%>様
出荷が完了しました。
商品の到着までしばらくお待ちください。

配送伝票番号：<%=delivery_code%>
配送状況問い合わせURL：<%=delivery_status_url%>
        EOF
        mail_template.mobile_subject_template = 'test'
        mail_template.mobile_text = 'test'
        mail_template.save!

        order_shipping = OrderShipping.find(2)
        order_shipping.delivery_code = 'A12345'
        order_shipping.country_code = 'JP'
        order_shipping.save!

        ActionMailer::Base.deliveries.clear

        order_shipping.send_shipment_complete_mail

        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "出荷完了しました"

        mail_body = NKF.nkf('-w', mail.body)
        mail_body.gsub(/\s|\r|\n/,'').should == "山田太郎様出荷が完了しました。商品の到着までしばらくお待ちください。配送伝票番号：A12345配送状況問い合わせURL：http://ec-one.com"
      end
    end
    
    it "zh_CN" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(7)
        order_shipping.order.locale = 'zh_CN'
        order_shipping.order.save!
        order_shipping.send_shipment_complete_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["email_address_01006@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "shipped pc! sample  衷心感谢您对我们的支持参加"
        NKF.nkf('-w', mail.body).should == "空川 結衣\n shipping completed. 衷心感谢您对我们的支持参加"
      end
    end
  end
  
  describe "send_cancel_request_mail" do
    it "キャンセル依頼メール送信" do
      unless EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(2)
        order_shipping.send_cancel_request_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "cancel pc!"
        NKF.nkf('-w', mail.body).should == "山田 太郎 様\n\nキャンセル依頼を行いました。\n\nありがとうございました。\n"
      end
    end
    
    it "zh_CN" do
      unless EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        I18n.stub!(:locale).and_return("zh_CN")
        order_shipping = OrderShipping.find(7)
        order_shipping.send_cancel_request_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["email_address_01006@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "cancel pc! sample  衷心感谢您对我们的支持参加"
        NKF.nkf('-w', mail.body).should == "空川 結衣\n cancel completed. 衷心感谢您对我们的支持参加\n"
      end
    end
  end

  describe "send_returned_mail" do
    it "返品完了メール送信" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(2)
        order_shipping.send_returned_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "返品処理完了のご連絡"
        NKF.nkf('-w', mail.body).should == "山田 太郎 様\n\n返品処理が完了しました。\n\n今後も宜しくお願い致します。\n"
      end
    end
  end

  describe "send_cancel_complete_mail" do
    it "キャンセル処理完了メール送信" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(2)
        order_shipping.send_cancel_complete_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["test@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "cancel complete mail subject pc!"
        NKF.nkf('-w', mail.body).should == "山田 太郎 様\n\n受注商品のキャンセル処理が完了しました。\n\n今後も宜しくお願い致します。"
      end
    end
    
    it "zh_CN" do
      pending_if_loaded_plugin(:c0002)
      if EcRider.config.ec_admin
        ActionMailer::Base.deliveries.clear
        order_shipping = OrderShipping.find(7)
        order_shipping.order.locale = 'zh_CN'
        order_shipping.order.save!
        order_shipping.send_cancel_complete_mail
        ActionMailer::Base.deliveries.size.should == 1
        mail = ActionMailer::Base.deliveries[0]
        mail.to.map{|to| NKF.nkf('-w', to)}.sort.should == ["email_address_01006@ec-one.com"]
        mail.from.map{|from| NKF.nkf('-w', from)}.sort.should == ["test_webadministrator@ecrider.com"]
        NKF.nkf('-w', mail.subject).should == "cancel complete mail subject pc! sample  衷心感谢您对我们的支持参加"
        NKF.nkf('-w', mail.body).should == "空川 結衣\n cancel completed.衷心感谢您对我们的支持参加"
      end
    end
  end
  
  describe "auto_cancelable?" do
    it "自動キャンセル可" do
      OrderShipping.new(:status_div_key => :before_settlement).auto_cancelable?.should be_true
      OrderShipping.new(:status_div_key => :undelivery_settlement).auto_cancelable?.should be_true
      OrderShipping.new(:status_div_key => :undelivery_unpaid).auto_cancelable?.should be_true
      OrderShipping.new(:status_div_key => :undelivery).auto_cancelable?.should be_true
    end
    it "自動キャンセル不可" do
      OrderShipping.new(:status_div_key => :deliver_request).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :delivered).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :deliver_advised).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :cancel_request).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :canceled).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :settlement_failed).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :stock_error).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :internal_error).auto_cancelable?.should be_false
      OrderShipping.new(:status_div_key => :external_error).auto_cancelable?.should be_false
    end
  end
  
  describe "cancelable?" do
    it "キャンセル可" do
      order_shipping = OrderShipping.new(:status_div_key => :undelivery)
      order_shipping.cancelable?.should be_true
    end
    it "キャンセル不可" do
      order_shipping = OrderShipping.new(:status_div_key => :delivered)
      order_shipping.cancelable?.should be_false
    end
  end
  
  describe "returnable?" do
    it "返品可" do
      order_shipping = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping)
      order_shipping.returnable?.should be_true
    end
    it "返品不可" do
      order_shipping = OrderShipping.find(2)
      order_shipping.returnable?.should be_nil
      order_shipping = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping)
      Accounting.create_for_cancel(order_shipping)
      order_shipping.returnable?.should be_false
    end
  end
  
  describe "arrival_time_display" do
    fixtures :distributors, :distributor_delivery_times, :distributor_delivery_time_displays, :countries
    it "表示名を取得" do
      order_shipping = OrderShipping.new(:country_code => 'JP', :arrival_hour => '1')
      order_shipping.arrival_time_display('ja').should == '日本語0001'
    end
  end

  describe "Order#amountを使う処理" do
    before do
      @order = Order.find(1)
      @order.amount.should == 8190
      @order_shipping = @order.order_shippings.first
    end
    
    describe :paid_order_amount do
      it "前払いの注文では合計の請求金額を返す" do
        @order_shipping.paid_order_amount.should == 8190
      end
      it "後払いの注文では合計の0を返す" do
        @order_shipping.should_receive(:pay_in_advance?).and_return(false)
        @order_shipping.paid_order_amount.should == 0
      end
    end

    describe :due_order_amount do
      it "前払いの注文では合計の請求金額を返す" do
        @order_shipping.due_order_amount.should == 0
      end
      it "後払いの注文では合計の0を返す" do
        @order_shipping.should_receive(:pay_in_advance?).and_return(false)
        @order_shipping.due_order_amount.should == 8190
      end
    end

    describe "update_membership" do
      it "新規会員を登録できる場合、会員区分が会員になること" do
        user = User.find(:first)
        user.member_div_key = :no_membership
        @order_shipping.save_user_as_no_membership_target = user
        @order_shipping.send_thanks_mail_args = [user, false]
        @order_shipping.update_membership
        @order_shipping.save_user_as_no_membership_target.member_div_key.should == :membership
        @order_shipping.send_thanks_mail_args[0].regist_error.should_not be_true
        result = User.find(:first)
        result.member_div_key.should == :membership
      end
      it "新規会員登録できない場合、会員区分が非会員になること" do
        user = User.find(:first)
        user.member_div_key = :no_membership
        user.save!
        new_user = User.new(user.attributes)
        new_user.member_div_key = :membership
        new_user.user_email = UserEmail.new(:email => user.user_email.email)
        new_user.save!
        @order_shipping.save_user_as_no_membership_target = user
        @order_shipping.send_thanks_mail_args = [user, false]
        @order_shipping.update_membership
        @order_shipping.save_user_as_no_membership_target.member_div_key.should == :no_membership
        @order_shipping.send_thanks_mail_args[0].regist_error.should be_true
        result = User.find(:first)
        result.member_div_key.should == :no_membership
        new_user = User.find_by_id(new_user.id)
        new_user.member_div_key.should == :membership
      end
    end

  end

end
