# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Product::CategorySearchForm do
  fixtures :products, :product_displays, :product_categories, :product_category_displays, :product_prices,
    :kbn_codes, :kbn_code_names, :product_skus, :user_ranks,
    :product_variation_displays, :product_variation_item_displays, :product_variation_items, :product_variations, :cms_publishes

  before(:each) do
    @now = Time.local(2009, 1, 10, 12, 00, 00)
    Time.stub!(:now).and_return(@now)
  end

  describe "カテゴリ検索クエリ生成" do
    it "全カテゴリ選択の場合 ec_front" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "", :condition_div => '0')
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 45
    end
    
    it "全カテゴリ選択の場合 ec_admin 公開期間を見ないこと" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "", :condition_div => '0')
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ?",
          'ja', 1]
      }
      Product.find(:all, form.to_find_options).size.should == 46
    end
    
    it "全カテゴリ選択の場合 ec_test" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      I18n.stub!(:locale).and_return('ja')
      test_now = Time.local(2009,9,05, 11,34,57)
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "", :condition_div => '0')
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?)",
          'ja', 1, test_now, test_now]
      }
      Product.find(:all, form.to_find_options).size.should == 45
    end

    it "親カテゴリ選択の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code14", :keyword => "", :condition_div => '0')
      serch_cond = [16, 18, 19, 20, 21, 22, 23, 14]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", 
          serch_cond, serch_cond, serch_cond, serch_cond, serch_cond, 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 5
    end

    it "子カテゴリ選択の場合(さらに子カテゴリが存在する場合)" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code20", :keyword => "", :condition_div => '0')
      serch_cond = [16,18,19,20]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) "<<
            "OR products.category_5_id IN (?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", 
          serch_cond, serch_cond, serch_cond, serch_cond, serch_cond, 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 5
    end

    it "子カテゴリ選択の場合(一番下の子カテゴリの場合)" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "", :condition_div => '0')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", 
          serch_cond, serch_cond, serch_cond, serch_cond, serch_cond, 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 2
    end
  end
  
  describe "キーワード検索クエリ生成" do
    it "キーワードをひとつだけ入力した場合（すべて）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "shirt", :condition_div => '0')
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", 
          "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 44
    end
    
    it "キーワードをひとつだけ入力した場合（いづれか）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "shirt", :condition_div => '1')
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions=>["((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", 
          "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 44
    end

    it "カテゴリを選択してキーワードをひとつだけ入力した場合（すべて）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '0')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order=>"products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)",
          serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 2
    end
    
    it "カテゴリを選択してキーワードをひとつだけ入力した場合（いづれか）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)",
          serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
          "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 2
    end
    
    it "カテゴリを選択してキーワードを複数入力した場合（すべて）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt pants shoes", :condition_div => '0')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order=>"products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) AND " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) AND " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)",
            serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%",
            "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%",
            "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%",
            'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 0
    end
    
    it "カテゴリを選択してキーワードを複数入力した場合（いづれか）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt pants shoes", :condition_div => '1')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) OR " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) OR " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%",
            "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%",
            "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%",
            'ja', 1, @now, @now],
        :order => "products.display_order is null, products.display_order, products.product_code",
      }
      Product.find(:all, form.to_find_options).size.should == 2
    end
  end
  
  describe "並び替え" do
    it "並び替えの条件が金額の安い順の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1', :order => '1', :user_rank_id => '1')#'product_prices.standard_price, products.id')
      serch_cond = [16]
      form.to_find_options.should == {
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 1, @now, @now],
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "product_prices.standard_price, products.display_order is null, products.display_order, products.product_code"
      }
      Product.find(:all, form.to_find_options).size.should == 2
    end
    
    it "並び替えの条件が金額の高い順の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1', :order => '2', :user_rank_id => '2')#'product_prices.standard_price desc, products.id')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 2, @now, @now],
          :order => "product_prices.standard_price desc, products.display_order is null, products.display_order, products.product_code"
       }
      Product.find(:all, form.to_find_options).size.should == 2
    end
    
    it "並び替えの条件が新着順の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1', :order => '3')
      serch_cond = [16]
      form.to_find_options.should == {
        :include => [:product_displays, :main_image, {:primary_sku => :prices}],
        :order => "products.publish_started_at desc, products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "product_skus.id IS NOT NULL AND product_displays.id IS NOT NULL AND product_prices.id IS NOT NULL AND " <<
            "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL AND "<<
            "product_displays.locale = ? AND "<<
            "(present_flg IS NULL OR present_flg = 0) AND product_prices.user_rank_id = ? AND " <<
            "(publish_started_at IS NULL OR publish_started_at <= ?) AND (publish_finished_at IS NULL OR publish_finished_at >= ?) AND " <<
            "(invisible_flg IS NULL OR invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', 1, @now, @now]
      }
      Product.find(:all, form.to_find_options).size.should == 2
    end
  end
end
