# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Product::FinderForm do
  fixtures :products, :product_displays, :product_categories, :product_category_displays, :product_prices,
    #:shipping_costs,
  :kbn_codes, :kbn_code_names, :product_skus, :user_ranks,
    :product_variation_displays, :product_variation_item_displays, :product_variation_items, :product_variations

  before(:each) do
    @now = Time.local(2009, 1, 10, 12, 00, 00)
    Time.stub!(:now).and_return(@now)
  end
  def new_form(*args)
    @form = Product::FinderForm.new(*args)
  end

  after(:each) do
    lambda{ Product.find(:all, @form.to_find_options)}.should_not raise_error
  end

  it "no attribute, no options" do
    new_form.to_find_options.should == {
      :conditions => ["(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
          "(invisible_flg is null)", @now, @now],
      :order=>"products.display_order is null, products.display_order, products.product_code"
    }
  end

  %w(category category_name_).each do |attr_base|
    (1..3).each do |index|
      category_attr = "#{attr_base}#{index}"
      alias_table = "nm4category#{index}"
      name_div_key = "product_category_#{index}".to_sym
      describe category_attr do
        describe "single value" do
          it "implictatly partial_match" do
            new_form(category_attr.to_sym => "服").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND (#{alias_table}.kbn_name like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "explictatly partial_match" do
            new_form(category_attr.to_sym => "*服*").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND (#{alias_table}.kbn_name like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code",
            }
          end

          it "head_match" do
            new_form(category_attr.to_sym => "服*").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND (#{alias_table}.kbn_name like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "服%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "tail_match" do
            new_form(category_attr.to_sym => "*服").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND (#{alias_table}.kbn_name like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "exact_match" do
            new_form(category_attr.to_sym => "=服").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND (#{alias_table}.kbn_name = ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "服", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end

        describe "multiple values" do
          it "implictatly partial_match" do
            new_form(category_attr.to_sym => "服,タオル,グッズ").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND ((#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", "%タオル%", "%グッズ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "implictatly partial_match with empty value pattern1" do
            new_form(category_attr.to_sym => "服,タオル,グッズ,").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND ((#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", "%タオル%", "%グッズ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "implictatly partial_match with empty value pattern1 with space" do
            new_form(category_attr.to_sym => "服, タオル , グッズ , ").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND ((#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", "%タオル%", "%グッズ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "implictatly partial_match with empty value pattern2" do
            new_form(category_attr.to_sym => "服,,タオル,,グッズ").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND ((#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", "%タオル%", "%グッズ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "complex multiple match" do
            new_form(category_attr.to_sym => "*服*,*タオル,グッズ*").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND ((#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", "%タオル", "グッズ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "complex multiple match with exact match" do
            new_form(category_attr.to_sym => "*服*,=タオル,グッズ*").to_find_options.should == {
              :joins => "inner join kbn_codes #{alias_table} on #{alias_table}.code = products.category_#{index}_id",
              :conditions => ["(#{alias_table}.kbn = ?) AND ((#{alias_table}.kbn_name like ?) OR (#{alias_table}.kbn_name = ?) OR (#{alias_table}.kbn_name like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)",
                KbnCode.name_div_id_by_key(name_div_key), "%服%", "タオル", "グッズ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end
      end
    end
  end

  %w(name product_name_).each do |attr_base|
    (1..2).each do |index|
      name_attr = "#{attr_base}#{index}"
      column = "product_name_#{index}"

      describe name_attr do
        describe "single value" do
          it "implictatly partial_match" do
            new_form(name_attr.to_sym => "フィギュア").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["(cond_product_displays.#{column} like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "%フィギュア%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "explictatly partial_match" do
            new_form(name_attr.to_sym => "*フィギュア*").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["(cond_product_displays.#{column} like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "%フィギュア%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "head_match" do
            new_form(name_attr.to_sym => "フィギュア*").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["(cond_product_displays.#{column} like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "フィギュア%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "tail_match" do
            new_form(name_attr.to_sym => "*フィギュア").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["(cond_product_displays.#{column} like ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "%フィギュア", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "exact_match" do
            new_form(name_attr.to_sym => "=フィギュア").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["(cond_product_displays.#{column} = ?) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "フィギュア", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end

        describe "multiple values" do
          it "implictatly partial_match" do
            new_form(name_attr.to_sym => "フィギュア,ガンプラ").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["((cond_product_displays.#{column} like ?) OR (cond_product_displays.#{column} like ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "%フィギュア%", "%ガンプラ%", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "explictatly partial_match" do
            new_form(name_attr.to_sym => "*フィギュア,=ガンプラ").to_find_options.should == {
              :joins => "inner join product_displays cond_product_displays on cond_product_displays.product_id = products.id",
              :conditions => ["((cond_product_displays.#{column} like ?) OR (cond_product_displays.#{column} = ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", "%フィギュア", "ガンプラ", @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end

      end
    end
  end

  describe "product_code" do
    describe "single value" do
      it "implictatly partial_match" do
        new_form(:product_code => "12345678").to_find_options.should == {
          :conditions => ["(products.product_code like ?) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "%12345678%", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end

      it "explictatly partial_match" do
        new_form(:product_code => "*12345678*").to_find_options.should == {
          :conditions => ["(products.product_code like ?) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "%12345678%", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end

      it "head_match" do
        new_form(:product_code => "12345678*").to_find_options.should == {
          :conditions => ["(products.product_code like ?) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "12345678%", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end

      it "tail_match" do
        new_form(:product_code => "*12345678").to_find_options.should == {
          :conditions => ["(products.product_code like ?) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "%12345678", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end

      it "exact_match" do
        new_form(:product_code => "=12345678").to_find_options.should == {
          :conditions => ["(products.product_code = ?) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "12345678", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end
    end

    describe "multiple values" do
      it "implictatly partial_match" do
        new_form(:product_code => "12345678*,*234*").to_find_options.should == {
          :conditions => ["((products.product_code like ?) OR (products.product_code like ?)) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "12345678%", "%234%", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end

      it "explictatly partial_match" do
        new_form(:product_code => "=1234567890123,=2345678901234,").to_find_options.should == {
          :conditions => ["((products.product_code = ?) OR (products.product_code = ?)) AND " <<
              "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
              "(invisible_flg is null)", "1234567890123", "2345678901234", @now, @now],
          :order=>"products.display_order is null, products.display_order, products.product_code"
        }
      end

    end
  end

  #    {:price => "products.price", :product_price => "products.price"}.each do |attr, column|
  {}.each do |attr, column|
    describe attr do
      describe "single value" do
        it "implicitly extract_match" do
          new_form(attr => "1000").to_find_options.should == {
            :conditions => ["(#{column} = ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "explicitly extract_match" do
          new_form(attr => "=1000").to_find_options.should == {
            :conditions => ["(#{column} = ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "greater than" do
          new_form(attr => ">1000").to_find_options.should == {
            :conditions => ["(#{column} > ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "greater equal" do
          new_form(attr => ">=1000").to_find_options.should == {
            :conditions => ["(#{column} >= ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "greater equal2" do
          new_form(attr => "=>1000").to_find_options.should == {
            :conditions => ["(#{column} >= ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "lesser than" do
          new_form(attr => "<1000").to_find_options.should == {
            :conditions => ["(#{column} < ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "lesser equal" do
          new_form(attr => "<=1000").to_find_options.should == {
            :conditions => ["(#{column} <= ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        it "lesser equal2" do
          new_form(attr => "=<1000").to_find_options.should == {
            :conditions => ["(#{column} <= ?) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        describe "without space" do
          it "between greater and lesser" do
            new_form(attr => ">1000<2000").to_find_options.should == {
              :conditions => ["((#{column} > ?) AND (#{column} < ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "between greater equal and lesser equal" do
            new_form(attr => ">=1000<=2000").to_find_options.should == {
              :conditions => ["((#{column} >= ?) AND (#{column} <= ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end

        describe "with space" do
          it "between greater and lesser" do
            new_form(attr => " > 1000 < 2000 ").to_find_options.should == {
              :conditions => ["((#{column} > ?) AND (#{column} < ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end

          it "between greater equal and lesser equal" do
            new_form(attr => " >= 1000 <= 2000 ").to_find_options.should == {
              :conditions => ["((#{column} >= ?) AND (#{column} <= ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end
      end

      describe "multiple values" do
        it "pattern1" do
          new_form(attr => "=1000,=2000").to_find_options.should == {
            :conditions => ["((#{column} = ?) OR (#{column} = ?)) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, 2000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end
        it "pattern2" do
          new_form(attr => "<1000,>2000").to_find_options.should == {
            :conditions => ["((#{column} < ?) OR (#{column} > ?)) AND " <<
                "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                "(invisible_flg is null)", 1000, 2000, @now, @now],
            :order=>"products.display_order is null, products.display_order, products.product_code"
          }
        end

        describe "without space" do
          it "pattern3" do
            new_form(attr => "<1000,>2000,=1500").to_find_options.should == {
              :conditions => ["((#{column} < ?) OR (#{column} > ?) OR (#{column} = ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, 1500, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
          it "pattern4" do
            new_form(attr => "<=1000,>=2000,=1500").to_find_options.should == {
              :conditions => ["((#{column} <= ?) OR (#{column} >= ?) OR (#{column} = ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, 1500, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end

        describe "with space" do
          it "pattern3 with space" do
            new_form(attr => " < 1000 , > 2000 , = 1500 ").to_find_options.should == {
              :conditions => ["((#{column} < ?) OR (#{column} > ?) OR (#{column} = ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, 1500, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
          it "pattern4 with space" do
            new_form(attr => " <= 1000 , >= 2000 , = 1500 ").to_find_options.should == {
              :conditions => ["((#{column} <= ?) OR (#{column} >= ?) OR (#{column} = ?)) AND " <<
                  "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
                  "(invisible_flg is null)", 1000, 2000, 1500, @now, @now],
              :order=>"products.display_order is null, products.display_order, products.product_code"
            }
          end
        end
      end
    end

  end
end
