require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductGroup do
  fixtures :products, :product_skus

  before(:each) do
    ProductGroup.destroy_all()
    ProductGroupSku.destroy_all()
    
    @valid_attributes = {
      :name => "関連商品１",
      :priority => 1,
      :skus_attributes => {
        "0" => {:product_sku_id => 80},
        "1" => {:product_sku_id => 6},
        "2" => {:product_sku_id => 15},
      }
    }
  end

  it "インスタンスが正しく生成されることを確認する" do
    ProductGroup.create!(@valid_attributes)
    product_group = ProductGroup.find(:first)
    product_group.name.should == "関連商品１"
    product_group.priority.should == 1
    product_group.skus.size.should == 3
    product_group.skus.map(&:product_sku_id).should == [6, 15, 80]
  end
  describe "SearchCondition" do
    describe "to_find_options" do
      before(:each) do
        ProductGroup.create!({
            :name => "おすすめ商品",
            :priority => 1,
            :skus_attributes => {
              "0" => {:product_sku_id => 265},
              "1" => {:product_sku_id => 266},
              "2" => {:product_sku_id => 267},
            }
          })
        ProductGroup.create!({
            :name => "人気商品",
            :priority => 2,
            :skus_attributes => {
              "0" => {:product_sku_id => 268},
              "1" => {:product_sku_id => 269},
            }
          })
      end

      def find_by_param(param)
        condition = ProductGroup::SearchCondition.new(param)
        ProductGroup.paginate(condition.to_paginate_options(:order => "id DESC"))
      end

      it "検索なし" do
        product_groups = find_by_param({})
        product_groups.size.should == 2
      end

      it "関連商品グループ名で検索" do
        product_groups = find_by_param({:name => "人気"})
        product_groups.size.should == 1
        product_groups[0].name.should == "人気商品"
      end

      it "SKUコードで検索" do
        product_groups = find_by_param({:sku_code => "0000000009030-00-00-00"})
        product_groups.size.should == 1

        product_sku_ids = product_groups[0].skus.map(&:product_sku_id)
        product_skus = ProductSku.find(:all, :conditions => ["id in (?)", product_sku_ids])
        product_skus.map(&:sku_code).include?("0000000009030-00-00-00").should be_true
      end

      it "商品名で検索" do
        product_groups = find_by_param({:product_name => "錠剤"})
        product_groups.size.should == 1

        product_sku_ids = product_groups[0].skus.map(&:product_sku_id)
        product_skus = ProductSku.find(:all, :conditions => ["id in (?)", product_sku_ids])
        product_ids = product_skus.map(&:product_id)
        products = Product.find(:all, :conditions => ["id in (?)", product_ids])
        products.map(&:order_product_name_1).include?("錠剤100粒").should be_true
      end
    end
  end
end
