# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductSku do
  describe "注文と関係しないテスト" do

    fixtures :products, :product_skus, :product_sku_inventories, :product_sku_secure_inventories, :product_components,
     :product_prices, :product_variations, :product_variation_displays, :product_variation_items, :product_variation_item_displays, :user_ranks

    before(:each) do
      @valid_attributes = {
        :product_id => 1,
        :variation_item_1_id => 1,
        :variation_item_2_id => 1,
        :weight => 1,
        :variation_item_3_id => 1,
        :safety_stock_count => 1,
        :shipping_cost_point => 10,
        :stockout_display_cd => '01'
      }
    end

    it "should create a new instance given valid attributes" do
      p = ProductSku.create!(@valid_attributes)
      ProductSkuInventory.create(:product_sku_id => p.id, :count => 1)
    end

    it "new" do
      p = ProductSku.new()
      p.parent_stock_count.should == 0
      p.secure_stock_count.should == 0
    end

    describe "sku_code" do
      it 'is "#{sku.product.product_code}-#{variation_item_1.code || "00"}-#{variation_item_2.code || "00"}-#{variation_item_2.code || "00"}" ' do
        product1 = Product.new(:product_code => '1234567890')
        sku1 = ProductSku.new
        sku1.product = product1
        sku1.should_receive(:branch_code).once.and_return("11-22-33")
        sku1.valid? # valid?でsku_codeが作成されます。
        sku1.sku_code.should == '1234567890-11-22-33'
      end
    end

    describe "stockout_display_cd" do
      it "valid fail" do
        p = ProductSku.new(@valid_attributes)
        p.stockout_display_cd = nil
        p.valid?.should be_false
      end
    end

    describe "branch_code, name_for_admin and variation_item_names_for_admin" do
      def name_for_admin_mock(code, name)
        result = ProductVariationItem.new(:code => code)
        result.should_receive(:name_for_admin).twice.and_return(name)
        result
      end

      it "without variation" do
        sku = ProductSku.new
        sku.name_for_admin.should == 'バリエーションなし'
        sku.branch_code.should == '00-00-00'
      end

      describe "wit a variation" do
        it "of 1" do
          sku = ProductSku.new(:variation_item_1 => name_for_admin_mock("L2", "LL"))
          sku.variation_item_names_for_admin.should == ['LL', nil, nil]
          sku.branch_code.should == 'L2-00-00'
          sku.name_for_admin.should == 'LL'
        end

        it "of 2" do
          sku = ProductSku.new(:variation_item_2 => name_for_admin_mock("R1", "赤"))
          sku.variation_item_names_for_admin.should == [nil, '赤', nil]
          sku.branch_code.should == '00-R1-00'
          sku.name_for_admin.should == '赤'
        end

        it "of 3" do
          sku = ProductSku.new(:variation_item_3 => name_for_admin_mock('10', "10枚"))
          sku.variation_item_names_for_admin.should == [nil, nil, '10枚']
          sku.branch_code.should == '00-00-10'
          sku.name_for_admin.should == '10枚'
        end
      end

      describe "wit two variation" do
        it "of 1, 2" do
          sku = ProductSku.new(
            :variation_item_1 => name_for_admin_mock('L2', "LL"),
            :variation_item_2 => name_for_admin_mock('R1', "赤"))
          sku.variation_item_names_for_admin.should == ["LL", "赤", nil]
          sku.branch_code.should == 'L2-R1-00'
          sku.name_for_admin.should == 'LL-赤'
        end

        it "of 1,3" do
          sku = ProductSku.new(
            :variation_item_1 => name_for_admin_mock('L2', "LL"),
            :variation_item_3 => name_for_admin_mock('10', "10枚"))
          sku.variation_item_names_for_admin.should == ["LL", nil, '10枚']
          sku.branch_code.should == 'L2-00-10'
          sku.name_for_admin.should == 'LL-10枚'
        end

        it "of 2,3" do
          sku = ProductSku.new(
            :variation_item_2 => name_for_admin_mock('R1', "赤"),
            :variation_item_3 => name_for_admin_mock('10', "10枚"))
          sku.variation_item_names_for_admin.should == [nil, "赤", "10枚"]
          sku.branch_code.should == '00-R1-10'
          sku.name_for_admin.should == '赤-10枚'
        end
      end

      it "wit three variation" do
        sku = ProductSku.new(
          :variation_item_1 => name_for_admin_mock('L2', "LL"),
          :variation_item_2 => name_for_admin_mock('R1', "赤"),
          :variation_item_3 => name_for_admin_mock('10', "10枚"))
        sku.variation_item_names_for_admin.should == ["LL", "赤", "10枚"]
        sku.branch_code.should == 'L2-R1-10'
        sku.name_for_admin.should == 'LL-赤-10枚'
      end

    end

    describe "stock_count" do
      it "get value" do
        sku = ProductSku.find(265)
        sku.stock_count.should == 20
      end
      
      it "get orderable_count" do
        sku = ProductSku.find(265)
        sku.stock_count(:orderable_count).should == 19
      end

      it "get orderable_count with component" do
        ProductComponent.create(:product_sku_id => 265, :component_sku_id => 266, :count => 2)
        sku = ProductSku.find(265)
        sku.stock_count(:orderable_count).should == 11
      end
    end

    describe "secure_stock_count" do
      it "get all_secure_stock_count" do
        sku = ProductSku.find(266)
        sku.get_secure_stock_count(:all).should == 27
      end
      it "get secure_stock_count" do
        sku = ProductSku.find(266)
        sku.get_secure_stock_count(:secure_stock_count).should == 26
      end
      it "get unpaid_secure_stock_count" do
        sku = ProductSku.find(266)
        sku.get_secure_stock_count(:unpaid_secure_stock_count).should == 1
      end
    end
    
    describe "price" do
      it "get value" do
        sku = ProductSku.find(265)
        sku.price.should == 8190
      end
    end

    describe "price_for" do
      before(:each) do
        @sku = ProductSku.find(265)
        @std_rank = UserRank.find(1) # スタンダード会員
        @test_rank = UserRank.create(:name => "テスト会員", :no_membership_flg => false)
        @test_price = ProductPrice.create(
          :product_sku_id => @sku.id, :user_rank_id => @test_rank.id, 
          :standard_price => 8000, 
          :regular_purchase_price => 7500)
      end

      it "get value" do
        @sku.price_for(@std_rank).should == 8190
        @sku.price_for(@test_rank).should == 8000
      end
    end

    describe "parent_skus" do
      before(:each) do
        ProductComponent.create!(:component_sku_id => "265", :product_sku_id => "1", :count => 1)
        ProductComponent.create!(:component_sku_id => "265", :product_sku_id => "1", :count => 2)
      end

      it "親商品SKUが返却されること" do
        sku = ProductSku.find(265)
        sku.parent_skus.size.should == 2
      end
    end
  end

  describe "注文と関係するテスト" do
    # 決済プラグイン内に移動しました。
    fixtures(:users, :user_emails, :user_addresses, :countries, :shipping_costs, :shipping_cost_boundaries, :prefectures, :prefecture_displays,
      :products, :product_displays, :product_skus, :product_prices, :product_components,
      :product_sku_inventories, :product_sku_secure_inventories,
      :orders, :order_shippings, :order_items,
      :point_settings, :user_points, :user_ranks,
      :kbn_codes,:kbn_code_names, :shop_settings)

    # 想定している注文とその状態と明細
    # 配送指示済
    # 00000000001 0000000009051-00-00-00 錠剤100粒 （錠剤100粒）
    # 00000000002 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000004 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000005 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000006 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000007 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000002 0000000009570-00-00-00 お酒 （お酒）
    # 00000000003 0000000009558-00-00-00 パワフル錠剤62粒 （パワフル錠剤62粒）
    # 00000000006 0000000009110-00-00-00 バスケットボール（2個入り） （バスケットボール（2個入り））
    # 00000000006 0000000009010-00-00-00 インテリア家具 （インテリア家具）
    # 00000000006 0000000009040-00-00-00 ガム10箱 （ガム10箱）

    # 配送指示済
    # 00000000009 0000000009558-00-00-00 パワフル錠剤62粒 （パワフル錠剤62粒）
    # 00000000009 0000000009040-00-00-00 ガム10箱 （ガム10箱）

    describe "before_destroy" do
      it "削除できる場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :delivered
        order_shiping.save!
        product1 = ProductSku.find(265)
        product1.error_if_unable_to_delete_product_sku.should be_true
        product1.errors.size.should == 0
      end

      it "１つ以上の商品SKUが発送前の注文に含まれている場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :undelivery
        order_shiping.save!
        product1 = ProductSku.find(265)
        product1.error_if_unable_to_delete_product_sku.should be_false
        product1.errors.size.should == 1
        product1.errors.full_messages.should == ["配送前の注文に含まれているため商品SKUを削除できません。"]
      end

      it "１つ以上の商品SKUが商品構成の子に含まれている場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :delivered
        order_shiping.save!
        product_component = ProductComponent.new(:product_sku_id => 266, :component_sku_id => 265, :count => 1)
        product_component.save!
        product1 = ProductSku.find(265)
        product1.error_if_unable_to_delete_product_sku.should be_false
        product1.errors.size.should == 1
        product1.errors.full_messages.should == ["商品構成の子になっているため商品SKUを削除できません。"]
      end

      it "発送前の注文に含まれ、かつ、商品構成の子にも含まれている場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :undelivery
        order_shiping.save!
        product_component = ProductComponent.new(:product_sku_id => 266, :component_sku_id => 265, :count => 1)
        product_component.save!
        product1 = ProductSku.find(265)
        product1.error_if_unable_to_delete_product_sku.should be_false
        product1.errors.size.should == 2
        product1.errors.full_messages.should == ["配送前の注文に含まれているため商品SKUを削除できません。","商品構成の子になっているため商品SKUを削除できません。"]
      end
    end

    describe "発送前の注文に含まれているSKUは削除できません" do
      before_delivery_status_div_keys = [
        :before_settlement    ,
        :undelivery_settlement,
        :undelivery_unpaid    ,
        :undelivery           ,
        :deliver_request      ,
        :cancel_request
      ]
      before_delivery_status_div_keys.each do |status_div_key|
        it "#{status_div_key.inspect} #{OrderShipping.status_div_name_by_key(status_div_key)}" do
          order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
          order_shiping = order.order_shippings.first
          order_shiping.status_div_key = status_div_key
          order_shiping.save!
          sku1 = ProductSku.find(:first, :conditions => {:sku_code => '0000000009051-00-00-00'})
          sku1.destroy.should == false
          sku1.errors.full_messages.should == ["配送前の注文に含まれているため商品SKUを削除できません。"]
        end
        
      end
    end

    describe "発送前の注文に含まれているSKUは削除できます" do
      product_deleteable_status_div_keys = [
        :delivered,
        :deliver_advised, 
        :canceled, 
        :settlement_failed,
        :stock_error,
        :internal_error,
        :external_error,
      ]
      product_deleteable_status_div_keys.each do |status_div_key|
        it "#{status_div_key.inspect} #{OrderShipping.status_div_name_by_key(status_div_key)}" do
          order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
          order_shiping = order.order_shippings.first
          order_shiping.status_div_key = status_div_key
          order_shiping.save!
          sku1 = ProductSku.find(:first, :conditions => {:sku_code => '0000000009051-00-00-00'})
          sku1.destroy.should == sku1
          sku1.errors.full_messages.should == []
        end
      end
    end
  end
end
