# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe Product do
  fixtures :products, :product_displays, :product_categories, :product_category_displays, :product_prices,
    #:shipping_costs,
  :kbn_codes, :kbn_code_names, :product_skus, :user_ranks,
    :product_variation_displays, :product_variation_item_displays, :product_variation_items, :product_variations
    :product_images

  
  #
  # = find_product_list_name_and_codeのテストケース =
  # 商品リストの０番目の商品名がPOPULAR ITEM TシャツA（白）Mサイズであること
  it "test_find_product_list_name_and_code" do
    result_list = Product.find_product_list_name_and_code
    result_list.first['order_product_name_1'].should == "POPULAR ITEM TシャツA（白）Mサイズ"
  end
  
  
  it "test product_list" do
    # 2009/08/13 商品割引の機能は削除されました。
    #    attributes = ProductDiscount.find(14).attributes
    #    attributes.delete(:id)
    #    attributes[:discount_price] = 5000
    #    ProductDiscount.create(attributes)

    list = Product.product_list(Time.parse('2009/01/20 10:00:00'))
    list.length.should == 46

    p = list[0]
    p.id.should == 100011
    #p.price.should == 5145
    #p.unit_price.should == 5145
    #p.regular_price.should == 5145

    p = list[6]
    p.id.should == 100023
    #p.price.should == 5145
    #p.unit_price.should == 5145
    #p.regular_price.should == 5145
  end

  it "test product_list with campaign" do
    #    ProductDiscount.update_all("end_date = '2030/12/31 12:00:00'", "discount_div = '1'")
    #    attributes = ProductDiscount.find(14).attributes
    #    attributes.delete(:id)
    #    attributes[:discount_price] = 5000
    #    attributes[:discount_div_key] = :campaign
    #    ProductDiscount.create(attributes)

    list = Product.product_list(Time.parse('2009/01/20 10:00:00'))
    list.length.should == 46

    p = list[0]
    p.id.should == 100011
    #p.price.should == 5145
    #p.unit_price.should == 5145
    #p.regular_price.should == 5145

    p = list[6]
    p.id.should == 100023
    #p.price.should == 5145
    #p.unit_price.should == 5145
    #p.regular_price.should == 5145
  end

  describe "with countries" do
    fixtures :countries

    it "test origin_name" do
      p = Product.new()
      p.origin_name.should be_nil

      p.origin = 'JP'
      p.origin_name.should == 'Japan'
    end
  end
  
  
  
  
  
  describe "has_sku?" do
    before(:each) do
      ProductSku.delete_all
      ProductSku.create!(:product_id => 1, :weight => 30, :safety_stock_count => 1, :shipping_cost_point => 100, :stockout_display_cd => '01')
    end

    it "商品SKUが1つでも登録されている場合、trueが返却される" do
      Product.find(1).has_sku?.should == true
    end

    it "商品SKUが1つも登録されていない場合、falseが返却される" do
      Product.find(2).has_sku?.should == false
    end
  end
  
  describe "image_sources" do
    before(:each) do
      ProductImage.create!(:id => 1, :product_id => 1, :filename => "image.jpg", :display_order => 1, :content_type => "image/jpeg", :size => 777)
      ProductImage.create!(:parent_id => 1, :thumbnail => "small", :filename => "image_small.jpg", :display_order => 1, :content_type => "image/jpeg", :size => 777)
      ProductImage.create!(:parent_id => 1, :thumbnail => "thumb", :filename => "image_thumb.jpg", :display_order => 1, :content_type => "image/jpeg", :size => 777)
      ProductImage.create!(:parent_id => 1, :thumbnail => "large", :filename => "image_large.jpg", :display_order => 1, :content_type => "image/jpeg", :size => 777)
      ProductImage.create!(:parent_id => 1, :thumbnail => "medium", :filename => "image_medium.jpg", :display_order => 1, :content_type => "image/jpeg", :size => 777)
    end

    it " CloudFrontへアップロードする商品画像のパスが返却される" do
      product = Product.find(1)
      product.image_sources.size.should == 5
      product.image_sources.each do |image|
        image.should match(/image.jpg||image_thumb.jpg||image_small.jpg||image_large.jpg||image_medium.jpg/)
      end
    end
  end
  
  describe "product_skus" do
    it "バリエーションに「サイズ」が登録されている" do
      product = Product.find(1)
      product.update_attributes(:variation_1_id => 1)
      product.product_skus.size.should == 3
    end

    it "バリエーションに「サイズ」と「色」が登録されている" do
      product = Product.find(2)
      product.update_attributes(:variation_1_id => 1, :variation_2_id => 2)
      product.product_skus.size.should == 9
    end

    it "バリエーションに「サイズ」と「色」、「セット」が登録されている" do
      product = Product.find(4)
      product.update_attributes(:variation_1_id => 1, :variation_2_id => 2, :variation_3_id => 3)
      product.product_skus.size.should == 18
    end

    it "既にSKUが登録されている" do
      product = Product.find(3)
      product.update_attributes(:variation_1_id => 1, :variation_2_id => 2)
      # 「(商品コード)-S-RD-00」のSKUコードを持つ、商品SKUを作成しています。
      product.skus[0].update_attributes(:variation_item_1_id => 1, :variation_item_2_id => 4)

      skus = product.product_skus
      skus.size.should == 9
      sku_codes = skus.map(&:sku_code)
      sku_codes.should_not == [nil, nil, nil, nil, nil, nil, nil, nil, nil]
    end
  end

  describe "price" do
    it "代表SKUの会員ランクが一番低い会員の標準売価が返却される" do
      product = Product.find(1)
      product.has_sku?.should be_true
      product.price.should == 8190
    end
  end
  
  describe "detail_url_outside_flg_name" do
    it "nil" do
      result = Product.new(:detail_url_outside_flg => nil)
      result.detail_url_outside_flg_name.should == "内部ページを利用する"
    end
    it "true" do
      result = Product.new(:detail_url_outside_flg => true)
      result.detail_url_outside_flg_name.should == "外部ページを利用する"
    end
    it "false" do
      result = Product.new(:detail_url_outside_flg => false)
      result.detail_url_outside_flg_name.should == "内部ページを利用する"
    end

  end
  describe "set_attributes_and_detail_url" do
    it "get detail_url_action" do
      p = Product.new(:detail_url => '/show_product?product_code=00000000001', :detail_url_outside_flg => false)
      p.detail_url_action.should == '/show_product'
    end
    it "set detail_url_action= false" do
      p = Product.new()
      p.set_attributes_and_detail_url(:product_code => '001',:detail_url_outside_flg => false, :detail_url_action => '/show_product')
      p.detail_url_action.should == '/show_product'
      p.detail_url.should == '/show_product?product_code=001'
    end
    it "set detail_url_action= true" do
      p = Product.new()
      p.set_attributes_and_detail_url(:product_code => '001',:detail_url_outside_flg => true, :detail_url => '/show_product')
      p.detail_url_action.should == '/show_product'
      p.detail_url.should == '/show_product'
    end
    it "set detail_url_action= nil detail_url_action= true" do
      p = Product.new()
      p.set_attributes_and_detail_url(:product_code => '001',:detail_url_outside_flg => true, :detail_url_action => '')
      p.detail_url_action.should == nil
      p.detail_url.should == nil
    end
    it "set detail_url_action= nil detail_url_action= false" do
      p = Product.new()
      p.set_attributes_and_detail_url(:product_code => '001',:detail_url_outside_flg => false, :detail_url => '')
      p.detail_url_action.should == nil
      p.detail_url.should == ''
    end
  end
  describe "check_sales_period" do
    it "check_sales_period = true" do
      p = Product.new()
      p.check_sales_period?.should == true
    end
    it "check_sales_period = false" do
      time_now = Time.parse("2009/02/01 00:00:00")
      Time.stub!(:now).and_return(time_now)
      p = Product.new(:sale_started_at => Time.parse('2010/01/20 10:00:00'))
      p.check_sales_period?.should == false
    end
    it "check_sales_period = false" do
      time_now = Time.parse("2009/12/01 00:00:00")
      Time.stub!(:now).and_return(time_now)
      p = Product.new(:sale_finished_at => Time.parse('2009/11/20 10:00:00'))
      p.check_sales_period?.should == false
    end
  end

  
  describe "注文と関係するテスト" do
    fixtures(:users, :user_emails, :user_addresses, :countries, :shipping_costs, :shipping_cost_boundaries, :prefectures, :prefecture_displays,
      :products, :product_displays, :product_skus, :product_prices, :product_components,
      :product_sku_inventories, :product_sku_secure_inventories,
      :orders, :order_shippings, :order_items,
      :point_settings, :user_points, :user_ranks,
      :kbn_codes,:kbn_code_names, :shop_settings)

    # 想定している注文とその状態と明細
    # 配送指示済
    # 00000000001 0000000009051-00-00-00 錠剤100粒 （錠剤100粒）
    # 00000000002 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000004 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000005 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000006 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000007 0000000009050-00-00-00 栄養ドリンク10箱 （栄養ドリンク10箱）
    # 00000000002 0000000009570-00-00-00 お酒 （お酒）
    # 00000000003 0000000009558-00-00-00 パワフル錠剤62粒 （パワフル錠剤62粒）
    # 00000000006 0000000009110-00-00-00 バスケットボール（2個入り） （バスケットボール（2個入り））
    # 00000000006 0000000009010-00-00-00 インテリア家具 （インテリア家具）
    # 00000000006 0000000009040-00-00-00 ガム10箱 （ガム10箱）

    # 配送指示済
    # 00000000009 0000000009558-00-00-00 パワフル錠剤62粒 （パワフル錠剤62粒）
    # 00000000009 0000000009040-00-00-00 ガム10箱 （ガム10箱）

    describe "before_destroy" do
      it "削除できる場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :delivered
        order_shiping.save!
        product1 = Product.find(:first, :conditions => {:product_code => '0000000009051'})
        product1.error_if_unable_to_delete_product.should be_true
        product1.errors.size.should == 0
      end

      it "１つ以上の商品SKUが発送前の注文に含まれている場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :undelivery
        order_shiping.save!
        product1 = Product.find(:first, :conditions => {:product_code => '0000000009051'})
        product1.error_if_unable_to_delete_product.should be_false
        product1.errors.size.should == 1
        product1.errors.full_messages.should == ["配送前の注文に含まれているため商品を削除できません。"]
      end

      it "１つ以上の商品SKUが商品構成の子に含まれている場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :delivered
        order_shiping.save!
        product_component = ProductComponent.new(:product_sku_id => 266, :component_sku_id => 265, :count => 1)
        product_component.save!
        product1 = Product.find(:first, :conditions => {:product_code => '0000000009051'})
        product1.error_if_unable_to_delete_product.should be_false
        product1.errors.size.should == 1
        product1.errors.full_messages.should == ["商品構成の子になっているため商品を削除できません。"]
      end

      it "発送前の注文に含まれ、かつ、商品構成の子にも含まれている場合" do
        order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
        order_shiping = order.order_shippings.first
        order_shiping.status_div_key = :undelivery
        order_shiping.save!
        product_component = ProductComponent.new(:product_sku_id => 266, :component_sku_id => 265, :count => 1)
        product_component.save!
        product1 = Product.find(:first, :conditions => {:product_code => '0000000009051'})
        product1.error_if_unable_to_delete_product.should be_false
        product1.errors.size.should == 2
        product1.errors.full_messages.should == ["配送前の注文に含まれているため商品を削除できません。","商品構成の子になっているため商品を削除できません。"]
      end
    end

    describe "発送前の注文に含まれている商品は削除できません" do
      before_delivery_status_div_keys = [
        :before_settlement    ,
        :undelivery_settlement,
        :undelivery_unpaid    ,
        :undelivery           ,
        :deliver_request      ,
        :cancel_request
      ]
      before_delivery_status_div_keys.each do |status_div_key|
        it "#{status_div_key.inspect} #{OrderShipping.status_div_name_by_key(status_div_key)}" do
          order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
          order_shiping = order.order_shippings.first
          order_shiping.status_div_key = status_div_key
          order_shiping.save!
          product1 = Product.find(:first, :conditions => {:product_code => '0000000009051'})
          product1.destroy.should == false
          product1.errors.full_messages.should == ["配送前の注文に含まれているため商品を削除できません。"]
        end

      end
    end

    describe "発送後の注文に含まれている商品は削除できます" do
      product_deleteable_status_div_keys = [
        :delivered,
        :deliver_advised,
        :canceled,
        :settlement_failed,
        :stock_error,
        :internal_error,
        :external_error,
      ]
      product_deleteable_status_div_keys.each do |status_div_key|
        it "#{status_div_key.inspect} #{OrderShipping.status_div_name_by_key(status_div_key)}" do
          order = Order.find(:first, :conditions => {:ec_order_code => "00000000001"})
          order_shiping = order.order_shippings.first
          order_shiping.status_div_key = status_div_key
          order_shiping.save!
          product1 = Product.find(:first, :conditions => {:product_code => '0000000009051'})
          product1.destroy.should == product1
          product1.errors.full_messages.should == []
        end
      end
    end
  end
  
end

