# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductVariation do
  fixtures :product_variations

  before(:each) do
    @valid_attributes = {
      :name_for_admin => "value for name_for_admin"
    }
  end

  it "should create a new instance given valid attributes" do
    ProductVariation.create!(@valid_attributes)
  end

  describe "options" do
    it "[バリエーション名, バリエーションID]の配列が返却される" do
      ProductVariation.options.size.should == 3
      
      ProductVariation.all.each do |v|
        ProductVariation.options.should include([v.name_for_admin, v.id])
      end
    end
  end
end

describe ProductVariation::SearchCondition do
  fixtures :product_variations, :product_variation_items

  describe "to_find_options" do
    it "test to_find_options with name_for_admin" do
      search_info = ProductVariation::SearchCondition.new(:variation_name => "サイズ")
      search_info.to_find_options[:conditions].should == [
        "product_variations.name_for_admin like ?", "%サイズ%"]
      product_variations = ProductVariation.find(:all, search_info.to_find_options)
      product_variations.length.should == 1
    end

    it "test to_find_options with code" do
      search_info = ProductVariation::SearchCondition.new(:variation_item => "RD")
      product_variations = ProductVariation.find(:all, search_info.to_find_options(:include => :items))
      product_variations.length.should == 1
    end

    it "test to_find_options with empty attributes" do
      search_info = ProductVariation::SearchCondition.new
      product_variations = ProductVariation.find(:all, search_info.to_find_options)
      product_variations.length.should == 3
    end
  end
end
