# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe ReminderForm do
  describe "in Japanese." do
    describe "Validate check." do
      before(:each) do
        @param = {:email => "email_address_01002@ec-one.com",
          :remember_question => "question", :remember_answer => "answer",
          :password => "password1", :password2 => "passpass1"}
      end
      it "The required check of the email" do
        @param[:email] = ""
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:email).should == "を入力してください。"
      end
      it "The required check of the remember_question" do
        @param[:remember_question] = ""
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:remember_question).should == "は正しくありません。"
      end
      it "The required check of the remember_answer" do
        @param[:remember_answer] = ""
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:remember_answer).should == "は正しくありません。"
      end
      it "The length check of the password" do
        @param[:password] = "aa1"
        @param[:password_confirmation] = "aa1"
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:password).should == "４文字以上で入力してください"
      end
      it "The length check of the password_confirmation" do
        @param[:password] = "aaa"
        @param[:password_confirmation] = "aaa"
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:password_confirmation).should == "４文字以上で入力してください"
      end
      it "The confirm check of the password" do
        @param[:password_confirmation] = "passpasspass"
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:password).should == "が一致していません。"
      end
      it "The format check of the email" do
        @param[:email] = "ec-one.com"
        test_reminder_form = ReminderForm.new(@param)
        test_reminder_form.should_not be_valid
        test_reminder_form.errors.on(:email).should == "は正しくありません。"
      end
    end
    describe "set_completion_url test" do
      it "set_completion_url success" do
        test_reminder_form = ReminderForm.new()
        url = {:controller => 'cart', :action => 'index'}
        test_reminder_form.set_completion_url(url)
        test_reminder_form.completion_url_controller.should == 'cart'
        test_reminder_form.completion_url_action.should == 'index'
      end
      it "completion_url nil" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.set_completion_url(nil)
        test_reminder_form.completion_url_controller.should == nil
        test_reminder_form.completion_url_action.should == nil
      end

      it "completion_url controller nil" do
        test_reminder_form = ReminderForm.new()
        url = {:controller => nil, :action => 'index'}
        test_reminder_form.set_completion_url(url)
        test_reminder_form.completion_url_controller.should == nil
        test_reminder_form.completion_url_action.should == nil
      end

      it "completion_url action nil" do
        test_reminder_form = ReminderForm.new()
        url = {:controller => 'cart', :action => nil}
        test_reminder_form.set_completion_url(url)
        test_reminder_form.completion_url_controller.should == nil
        test_reminder_form.completion_url_action.should == nil
      end

      it "completion_url controller empty" do
        test_reminder_form = ReminderForm.new()
        url = {:controller => '', :action => 'index'}
        test_reminder_form.set_completion_url(url)
        test_reminder_form.completion_url_controller.should == nil
        test_reminder_form.completion_url_action.should == nil
      end

      it "completion_url action empty" do
        test_reminder_form = ReminderForm.new()
        url = {:controller => 'cart', :action => ''}
        test_reminder_form.set_completion_url(url)
        test_reminder_form.completion_url_controller.should == nil
        test_reminder_form.completion_url_action.should == nil
      end
    end

    describe "other_redirect? test" do
      it "other_redirect? success" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = 'cart'
        test_reminder_form.completion_url_action = 'index'
        test_reminder_form.other_redirect?.should == true
      end
      it "completion_url nil" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = nil
        test_reminder_form.completion_url_action = nil
      end

      it "completion_url controller nil" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = nil
        test_reminder_form.completion_url_action = 'index'
        test_reminder_form.other_redirect?.should == false
      end

      it "completion_url action nil" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = ''
        test_reminder_form.completion_url_action = nil
        test_reminder_form.other_redirect?.should == false
      end

      it "completion_url controller empty" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = ''
        test_reminder_form.completion_url_action = 'index'
        test_reminder_form.other_redirect?.should == false
      end

      it "completion_url action empty" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = 'cart'
        test_reminder_form.completion_url_action = ''
        test_reminder_form.other_redirect?.should == false
      end

      it "completion_url controller empty" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = ""
        test_reminder_form.completion_url_action = 'index'
        test_reminder_form.other_redirect?.should == false
      end

      it "completion_url action empty" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.completion_url_controller = 'cart'
        test_reminder_form.completion_url_action = ""
        test_reminder_form.other_redirect?.should == false
      end
    end
    describe "back_to_url test" do
      it "back_to_url success" do
        test_reminder_form = ReminderForm.new()
        test_reminder_form.back_to_url = '/account/login'
        test_reminder_form.back_to_url.should == '/account/login'
      end
    end
  end
end
