# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe ShippingCostBoundary do
  fixtures :shipping_costs, :shipping_cost_boundaries
  describe "sort_boundaries" do
    it "ソートの場合" do
      input_boundaries = ["100", "250", "10", "150"]
      result = ShippingCostBoundary.sort_boundaries(input_boundaries)
      result[0].should == 10
      result[1].should == 100
      result[2].should == 150
      result[3].should == 250
    end
    it "重複の場合" do
      input_boundaries = ["10", "100", "10", "100", "25"]
      result = ShippingCostBoundary.sort_boundaries(input_boundaries)
      result.size.should == 3
      result[0].should == 10
      result[1].should == 25
      result[2].should == 100
    end
    it "閾値に空欄がある場合" do
      ShippingCostBoundary.create(:point => 100)
      ShippingCostBoundary.create(:point => 200)
      ShippingCostBoundary.create(:point => 300)
      input_boundaries = ["10", "", "100", ""]
      result = ShippingCostBoundary.sort_boundaries(input_boundaries)
      result.size.should == 2
      result[0].should == 10
      result[1].should == 100
    end
  end
  describe "get_boundary_ids" do
    it "既存のデータに同pointがある場合" do
      input_boundaries = ["1000", "1100", "1200"]
      result = ShippingCostBoundary.get_boundary_ids(input_boundaries)
      result.size.should == input_boundaries.size + 1
      result[0].should == 26
      result[1].should == 27
      result[2].should == 28
      result[3].should == 49
    end
    it "既存のデータに同pointがない場合" do
      input_boundaries = ["15", "25", "100"]
      result = ShippingCostBoundary.get_boundary_ids(input_boundaries)
      result.size.should == input_boundaries.size + 1
      result[0].should == ""
      result[1].should == ""
      result[2].should == ""
      result[3].should == 49
    end
  end
  describe "replace_boundary_points" do
    it "既存の閾値データより更新する閾値が多い場合" do
      size = ShippingCostBoundary.all.size
      points = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19]
      shipping_costs = [
        ShippingCost.new(:boundary_index => 0, :shipping_cost => 100), ShippingCost.new(:boundary_index => 1, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 2, :shipping_cost => 100), ShippingCost.new(:boundary_index => 3, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 4, :shipping_cost => 100), ShippingCost.new(:boundary_index => 5, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 6, :shipping_cost => 100), ShippingCost.new(:boundary_index => 7, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 8, :shipping_cost => 100), ShippingCost.new(:boundary_index => 9, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 10, :shipping_cost => 100), ShippingCost.new(:boundary_index => 11, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 12, :shipping_cost => 100), ShippingCost.new(:boundary_index => 13, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 14, :shipping_cost => 100), ShippingCost.new(:boundary_index => 15, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 16, :shipping_cost => 100), ShippingCost.new(:boundary_index => 17, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 18, :shipping_cost => 100)
      ]
      ShippingCostBoundary.replace_boundary_points(points, shipping_costs)
      ShippingCostBoundary.all.size.should > size
      ShippingCostBoundary.all.size.should == 19
      ShippingCost.all.size.should == 19

    end
    it "既存の閾値データより更新する閾値が少ない場合" do
      size = ShippingCostBoundary.all.size
      points = [1,2,3,""]
      shipping_costs = [
        ShippingCost.new(:boundary_index => 0, :shipping_cost => 100), ShippingCost.new(:boundary_index => 1, :shipping_cost => 100),
        ShippingCost.new(:boundary_index => 2, :shipping_cost => 100), ShippingCost.new(:boundary_index => 3, :shipping_cost => 100)
      ]
      ShippingCostBoundary.replace_boundary_points(points, shipping_costs)
      ShippingCostBoundary.all.size.should < size
      ShippingCostBoundary.all.size.should == 4
      ShippingCost.all.size.should == 4
    end
  end
end
