# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe UserAddress do
  fixtures :user_addresses,:users

  before(:each) do
    @address = UserAddress.new
  end


  describe "Validation Japanese" do
    before do
      @param = UserAddress.find(1011).attributes.dup
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the name_last" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the name_first" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the name_last" do
      @param[:name_last] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the name_first" do
      @param[:name_first] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the kana_name_last" do
      @param[:kana_name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kana_name_first" do
      @param[:kana_name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kana_name_last" do
      @param[:kana_name_last] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kana_name_first" do
      @param[:kana_name_first] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = "abcdefg"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the name_last" do
      @param[:name_last] = "あいう}えお"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
      test_address.errors.on(:name_last).should == 'は正しくありません。'
    end
    it "The format check of the name_first" do
      @param[:name_first] = "かきく}けこ"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
      test_address.errors.on(:name_first).should == 'は正しくありません。'
    end
    it "The format check of the kana_name_last" do
      @param[:kana_name_last] = "アイうエオ"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
      test_address.errors.on(:kana_name_last).should == 'は全角カタカナで入力してください。'
    end
    it "The format check of the kana_name_first" do
      @param[:kana_name_first] = "カキくケコ"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
      test_address.errors.on(:kana_name_first).should == 'は全角カタカナで入力してください。'
    end
    it "The length check of the fullname" do
      @param[:name_last] = "あいうえお1234567890"
      @param[:name_first] = "かきくけこ123456789"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The length check of the fullname" do
      @param[:name_last] = "あいうえお1234567890"
      @param[:name_first] = "かきくけこ1234567890"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
      test_address.errors.on(:base).should == 'お名前（姓＋名）は長すぎます。'
    end
    it "The length check of the fullname_kana" do
      @param[:kana_name_last] = "アイウエオアイウエオ"
      @param[:kana_name_first] = "カキクケコカキクケ"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The length check of the fullname_kana" do
      @param[:kana_name_last] = "アイウエオアイウエオ"
      @param[:kana_name_first] = "カキクケコカキクケコ"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
      test_address.errors.on(:base).should == 'フリガナ（姓＋名）は長すぎます。'
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = "+81-00-0000-0000"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
  end

  describe "Validation English" do
    before do
      address = UserAddress.find(1012)
      address.address_div_key = :user
      @param = address.attributes.dup
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The required check of the name_last" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_last" do
      @param[:name_last] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the zip_code at oversea" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the tel_num at oversea" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the address_1" do
      @param[:address_1] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
  end

  describe "Validation Japanese user's English address" do
    before do
      address = UserAddress.find(1012)
      address.address_div_key = :user_en
      @param = address.attributes.dup
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The required check of the name_last" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_last" do
      @param[:name_last] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the zip_code at oversea" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the tel_num at oversea" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the address_1" do
      @param[:address_1] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
  end

  describe "Validation Chinese" do
    before do
      @param = UserAddress.find(1011).attributes.dup
      @param[:country_code] = 'CN'
      @param[:name_last] = '支付宝'
      @param[:name_first] = '支付宝'
      @param[:address_1] = '北京市'
      @param[:address_2] = '朝阳区东三环北路38号'
      @param[:address_3] = '泰康金融大厦1座25层'
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the kanji_family_name" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kanji_name" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the kanji_family_name with japanese" do
      @param[:name_last] = "中国語"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the kanji_name with japanese" do
      @param[:name_first] = "労働"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the kanji_family_name with english" do
      @param[:name_last] = "yamada"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The format check of the kanji_name with english" do
      @param[:name_first] = "taro"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = "abcdefg"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = "+81-00-0000-0000"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
  end

  describe "fullname" do
    it "test fullname english" do
      test_address = UserAddress.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP', :locale => 'en')
      test_address.fullname.should == 'name2 name1'
    end
    it "test fullname japanese" do
      test_address = UserAddress.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP', :locale => 'ja')
      test_address.fullname.should == 'name1 name2'
    end
  end

  describe "fullname_kana" do
    it "test fullname_kana english" do
      test_address = UserAddress.new(:kana_name_last => 'kana_name1', :kana_name_first => 'kana_name2', :country_code => 'JP', :locale => 'en')
      test_address.fullname_kana.should == ''
    end
    it "test fullname_kana japanese" do
      test_address = UserAddress.new(:kana_name_last => 'kana_name1', :kana_name_first => 'kana_name2', :country_code => 'JP', :locale => 'ja')
      test_address.fullname_kana.should == 'kana_name1 kana_name2'
    end
  end

  describe "fulladdress" do
    describe "without address_1" do
      it "test fulladdress english" do
        test_address = UserAddress.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'en')
        test_address.fulladdress.should == 'add3, add2'
      end
      it "test fulladdress japanese" do
        test_address = UserAddress.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'ja')
        test_address.fulladdress.should == 'add2 add3'
      end
    end

    describe "with address_1" do
      it "test fulladdress english" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'en')
        test_address.fulladdress(true).should == 'add3, add2, add1'
      end
      it "test fulladdress japanese" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'ja')
        test_address.fulladdress(true).should == 'add1 add2 add3'
      end
    end

    describe "with address_1 and :no_space => true" do
      it "test fulladdress english" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'en')
        test_address.fulladdress(true, :delimeter => ',').should == 'add3,add2,add1'
      end
      it "test fulladdress japanese" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'ja')
        test_address.fulladdress(true, :delimeter => '').should == 'add1add2add3'
      end
    end
  end

  describe "validates_length_of test" do
    before do
      @param = UserAddress.find(1011).attributes.dup
    end
    it "name_last in japanese to be valid" do
      pending "フルネームのバイト数制限で検証エラーになるため全角での検証はペンディング"
      @param[:name_last] = "１２３４５６７８９０１２３４５６７８９０１２３４５６７８９０"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "name_last in japanese to be invalid" do
      @param[:name_last] = "１２３４５６７８９０１２３４５６７８９０１２３４５６７８９０１"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "name_last in english to be valid" do
      @param[:name_last] = "123456789012345678901234567890"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "name_last in english to be invalid" do
      @param[:name_last] = "1234567890123456789012345678901"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
  end

  describe "is_japanese_format?" do
    it "locale is ja, country is JP" do
      address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
      address.is_japanese_format?.should be_true
    end
    it "locale is ja, country is JP, sub_address" do
      address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
      address.is_japanese_format?.should be_true
    end
    it "locale is ja, country is JP" do
      address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
      address.is_japanese_format?.should be_false
    end
    it "locale is en, country is JP" do
      address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
      address.is_japanese_format?.should be_false
    end
    it "locale is zh-CN, country is JP" do
      address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
      address.is_japanese_format?.should be_false
    end
  end

  describe "is_chinese_format?" do
    describe "システム設定値　簡体字入力を許す場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(true)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is CN" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_true
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_true
      end
    end
    describe "システム設定値　簡体字入力を許さない場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(false)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
    end
  end

  describe "is_english_format?" do
    describe "システム設定値　簡体字入力を許す場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(true)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_english_format?.should be_true
      end
      it "locale is ja, country is CN" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
    end
    describe "システム設定値　簡体字入力を許さない場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(false)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_english_format?.should be_true
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
    end
  end

  describe "zip_to_address" do
    before do
      mock_add1 = JpAddress.new(:id => 1, :jiscode => '23106', :zipcode_old => '460', :zipcode => '4600002',
        :prefecture => '愛知県', :city => '名古屋市中区', :address => '丸の内')
      JpAddress.stub!(:find_by_zipcode).with('4600002').and_return(mock_add1)
      JpAddress.stub!(:find_by_zipcode).with('0000000').and_return(nil)
    end

    it "住所が取得できる場合" do
      address1, address2 = UserAddress.zip_to_address("4600002")
      address1.should == "愛知県"
      address2.should == "名古屋市中区丸の内"
    end

    it "住所が取得できない場合" do
      address1, address2 = UserAddress.zip_to_address("0000000")
      address1.should == ""
      address2.should == ""
    end
  end

end
