require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe UserRankCondition do
  before(:each) do
    @valid_attributes = {

    }
  end

  it "should create a new instance given valid attributes" do
    UserRankCondition.create!(@valid_attributes)
  end

  describe "not_all_inputed?" do
    it "blank" do
      result = UserRankCondition.new(:type_cd => "", :operator_cd => "", :value =>"")
      result.not_all_inputed?.should be_false
    end
    it "nil" do
      result = UserRankCondition.new()
      result.not_all_inputed?.should be_false
    end
    it "nil&blank" do
      result = UserRankCondition.new(:type_cd => "", :operator_cd => "")
      result.not_all_inputed?.should be_false
    end
    it "nil&blank&notnil" do
      result = UserRankCondition.new(:type_cd => "", :operator_cd => "3")
      result.not_all_inputed?.should be_true
    end
  end

  describe "judge" do
    before(:each) do
      @cond = UserRankCondition.new()
      @user = User.find_by_id(1003)
    end

    describe "年齢" do
      before(:each) do
        @cond.type_key = :user_age
        @user_value = @user.age
      end
      describe "年齢 < 設定値" do
        before(:each){@cond.operator_key = :lt}
        it "年齢が設定値より小さい場合 true " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "年齢が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "年齢が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "年齢 > 設定値" do
        before(:each){@cond.operator_key = :gt}
        it "年齢が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "年齢が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "年齢が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "年齢 <= 設定値" do
        before(:each){@cond.operator_key = :lte}
        it "年齢が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "年齢が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "年齢が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "年齢 >= 設定値" do
        before(:each){@cond.operator_key = :gte}
        it "年齢が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "年齢が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "年齢が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "年齢 == 設定値" do
        before(:each){@cond.operator_key = :eql}
        it "年齢が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "年齢が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "年齢が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "年齢 != 設定値" do
        before(:each){@cond.operator_key = :not_eql}
        it "年齢が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "年齢が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "年齢が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
    end

    describe "購入金額" do
      before(:each) do
        @cond.type_key = :order_amount
        @user_value = @user.total_purchase_for_distance
      end
      describe "購入金額 < 設定値" do
        before(:each){@cond.operator_key = :lt}
        it "購入金額が設定値より小さい場合 true " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "購入金額が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "購入金額が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "購入金額 > 設定値" do
        before(:each){@cond.operator_key = :gt}
        it "購入金額が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "購入金額が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "購入金額が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "購入金額 <= 設定値" do
        before(:each){@cond.operator_key = :lte}
        it "購入金額が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "購入金額が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "購入金額が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "購入金額 >= 設定値" do
        before(:each){@cond.operator_key = :gte}
        it "購入金額が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "購入金額が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "購入金額が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "購入金額 == 設定値" do
        before(:each){@cond.operator_key = :eql}
        it "購入金額が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "購入金額が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "購入金額が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "購入金額 != 設定値" do
        before(:each){@cond.operator_key = :not_eql}
        it "購入金額が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "購入金額が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "購入金額が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
    end

    describe "購入回数" do
      before(:each) do
        @cond.type_key = :order_count
        @user_value = @user.order_count_for_distance
      end
      describe "購入回数 < 設定値" do
        before(:each){@cond.operator_key = :lt}
        it "購入回数が設定値より小さい場合 true " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "購入回数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "購入回数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "購入回数 > 設定値" do
        before(:each){@cond.operator_key = :gt}
        it "購入回数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "購入回数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "購入回数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "購入回数 <= 設定値" do
        before(:each){@cond.operator_key = :lte}
        it "購入回数が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "購入回数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "購入回数が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "購入回数 >= 設定値" do
        before(:each){@cond.operator_key = :gte}
        it "購入回数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "購入回数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "購入回数が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "購入回数 == 設定値" do
        before(:each){@cond.operator_key = :eql}
        it "購入回数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "購入回数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "購入回数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "購入回数 != 設定値" do
        before(:each){@cond.operator_key = :not_eql}
        it "購入回数が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "購入回数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "購入回数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
    end

    describe "会員登録からの日数" do
      before(:each) do
        @cond.type_key = :member_elapsed_days
        @user_value = @user.count_day_of_member_created
      end
      describe "会員登録からの日数 < 設定値" do
        before(:each){@cond.operator_key = :lt}
        it "会員登録からの日数が設定値より小さい場合 true " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "会員登録からの日数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "会員登録からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "会員登録からの日数 > 設定値" do
        before(:each){@cond.operator_key = :gt}
        it "会員登録からの日数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "会員登録からの日数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "会員登録からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "会員登録からの日数 <= 設定値" do
        before(:each){@cond.operator_key = :lte}
        it "会員登録からの日数が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "会員登録からの日数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "会員登録からの日数が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "会員登録からの日数 >= 設定値" do
        before(:each){@cond.operator_key = :gte}
        it "会員登録からの日数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "会員登録からの日数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "会員登録からの日数が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "会員登録からの日数 == 設定値" do
        before(:each){@cond.operator_key = :eql}
        it "会員登録からの日数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "会員登録からの日数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "会員登録からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "会員登録からの日数 != 設定値" do
        before(:each){@cond.operator_key = :not_eql}
        it "会員登録からの日数が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "会員登録からの日数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "会員登録からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
    end

    describe "最終注文日からの日数" do
      before(:each) do
        @cond.type_key = :order_elapsed_days
        @user_value = @user.count_day_of_ordered
      end
      describe "最終注文日からの日数 < 設定値" do
        before(:each){@cond.operator_key = :lt}
        it "最終注文日からの日数が設定値より小さい場合 true " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "最終注文日からの日数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "最終注文日からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "最終注文日からの日数 > 設定値" do
        before(:each){@cond.operator_key = :gt}
        it "最終注文日からの日数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "最終注文日からの日数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "最終注文日からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
      describe "最終注文日からの日数 <= 設定値" do
        before(:each){@cond.operator_key = :lte}
        it "最終注文日からの日数が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "最終注文日からの日数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "最終注文日からの日数が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "最終注文日からの日数 >= 設定値" do
        before(:each){@cond.operator_key = :gte}
        it "最終注文日からの日数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "最終注文日からの日数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "最終注文日からの日数が設定値と同じ場合 true" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "最終注文日からの日数 == 設定値" do
        before(:each){@cond.operator_key = :eql}
        it "最終注文日からの日数が設定値より小さい場合 false " do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_false
        end
        it "最終注文日からの日数が設定値より大きい場合 false" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_false
        end
        it "最終注文日からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_true
        end
      end
      describe "最終注文日からの日数 != 設定値" do
        before(:each){@cond.operator_key = :not_eql}
        it "最終注文日からの日数が設定値より小さい場合 true" do
          @cond.value = @user_value + 1
          @cond.judge(@user).should be_true
        end
        it "最終注文日からの日数が設定値より大きい場合 true" do
          @cond.value = @user_value - 1
          @cond.judge(@user).should be_true
        end
        it "最終注文日からの日数が設定値と同じ場合 false" do
          @cond.value = @user_value
          @cond.judge(@user).should be_false
        end
      end
    end
  end
end
