# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe User do
  #include AuthenticatedTestHelper

  fixtures :users, :user_addresses, :user_emails, :kbn_codes, 
    :orders, :order_shippings

  describe "in Japanese." do
    # 顧客区分
    it "member_div means memberships and resignation." do
      User.member_div_options.should == [["WEB会員", "1"], ["オフライン専用会員", "3"], ["非会員", "0"], ["退会者", "2"]]
      User.member_div_name_by_id('0').should == "非会員"
      User.member_div_name_by_id('1').should == "WEB会員"
      User.member_div_name_by_id('2').should == "退会者"
      User.member_div_name_by_id('3').should == "オフライン専用会員"
      User.member_div_id_by_key(:no_membership).should == "0"
      User.member_div_id_by_key(:membership).should == "1"
      User.member_div_id_by_key(:resigned).should == "2"
      User.member_div_id_by_key(:offline_membership).should == "3"

      u1 = User.new
      #u1.member_div.should == nil
      u1.member_div_key.should be_nil
      u1.member_div_name.should be_nil
      u1.member_div = '0'
      u1.member_div.should == '0'
      u1.member_div_key.should == :no_membership
      u1.member_div_name.should == '非会員'
      u1.member_div = '1'
      u1.member_div.should == '1'
      u1.member_div_key.should == :membership
      u1.member_div_name.should == 'WEB会員'
      u1.member_div = '2'
      u1.member_div.should == '2'
      u1.member_div_key.should == :resigned
      u1.member_div_name.should == '退会者'
      u1.member_div = '3'
      u1.member_div.should == '3'
      u1.member_div_key.should == :offline_membership
      u1.member_div_name.should == 'オフライン専用会員'
    end

    # PCメールマガジン購読区分
    it "pc_mail_magazine_send_div means if user like to subscribe mail magazine." do
      User.pc_mail_magazine_send_div_options.should == [["HTML(絵や写真入のメール)を受け取る", 0],["テキストメール(文字だけのメール)を受け取る", 1],["受け取らない", 99]]
      User.pc_mail_magazine_send_div_name_by_id(0).should == "HTML(絵や写真入のメール)を受け取る"
      User.pc_mail_magazine_send_div_name_by_id(1).should == "テキストメール(文字だけのメール)を受け取る"
      User.pc_mail_magazine_send_div_name_by_id(99).should == "受け取らない"
      User.pc_mail_magazine_send_div_id_by_key(:prefer_html).should == 0
      User.pc_mail_magazine_send_div_id_by_key(:prefer_text).should == 1
      User.pc_mail_magazine_send_div_id_by_key(:not_prefer).should == 99

      u1 = User.new
      u1.pc_mail_magazine_send_div.should be_nil
      u1.pc_mail_magazine_send_div_key.should be_nil
      u1.pc_mail_magazine_send_div_name.should be_nil
      u1.pc_mail_magazine_send_div = '0'
      u1.pc_mail_magazine_send_div.should == "0"
      u1.pc_mail_magazine_send_div_key.should == :prefer_html
      u1.pc_mail_magazine_send_div_name.should == 'HTML(絵や写真入のメール)を受け取る'
      u1.pc_mail_magazine_send_div = '1'
      u1.pc_mail_magazine_send_div.should == "1"
      u1.pc_mail_magazine_send_div_key.should == :prefer_text
      u1.pc_mail_magazine_send_div_name.should == 'テキストメール(文字だけのメール)を受け取る'
      u1.pc_mail_magazine_send_div = '99'
      u1.pc_mail_magazine_send_div.should == "99"
      u1.pc_mail_magazine_send_div_key.should == :not_prefer
      u1.pc_mail_magazine_send_div_name.should == '受け取らない'
    end

    # モバイルメールマガジン購読区分
    it "mobile_mail_magazine_send_div means if user like to subscribe mail magazine." do
      User.mobile_mail_magazine_send_div_options.should == [["HTML(絵や写真入のメール)を受け取る", 0],["テキストメール(文字だけのメール)を受け取る", 1],["受け取らない", 99]]
      User.mobile_mail_magazine_send_div_name_by_id(0).should == "HTML(絵や写真入のメール)を受け取る"
      User.mobile_mail_magazine_send_div_name_by_id(1).should == "テキストメール(文字だけのメール)を受け取る"
      User.mobile_mail_magazine_send_div_name_by_id(99).should == "受け取らない"
      User.mobile_mail_magazine_send_div_id_by_key(:prefer_html).should == 0
      User.mobile_mail_magazine_send_div_id_by_key(:prefer_text).should == 1
      User.mobile_mail_magazine_send_div_id_by_key(:not_prefer).should == 99

      u1 = User.new
      u1.mobile_mail_magazine_send_div.should be_nil
      u1.mobile_mail_magazine_send_div_key.should be_nil
      u1.mobile_mail_magazine_send_div_name.should be_nil
      u1.mobile_mail_magazine_send_div = '0'
      u1.mobile_mail_magazine_send_div.should == "0"
      u1.mobile_mail_magazine_send_div_key.should == :prefer_html
      u1.mobile_mail_magazine_send_div_name.should == 'HTML(絵や写真入のメール)を受け取る'
      u1.mobile_mail_magazine_send_div = '1'
      u1.mobile_mail_magazine_send_div.should == "1"
      u1.mobile_mail_magazine_send_div_key.should == :prefer_text
      u1.mobile_mail_magazine_send_div_name.should == 'テキストメール(文字だけのメール)を受け取る'
      u1.mobile_mail_magazine_send_div = '99'
      u1.mobile_mail_magazine_send_div.should == "99"
      u1.mobile_mail_magazine_send_div_key.should == :not_prefer
      u1.mobile_mail_magazine_send_div_name.should == '受け取らない'
    end

    # DM送付希望区分
    it "dm_send_div means if user like to receive direct mail." do
      User.dm_send_div_options.should == [["希望しない", 0],["希望する", 1]]
      User.dm_send_div_name_by_id(0).should == "希望しない"
      User.dm_send_div_name_by_id(1).should == "希望する"
      User.dm_send_div_id_by_key(:not_prefer).should == 0
      User.dm_send_div_id_by_key(:prefer).should == 1

      u1 = User.new
      u1.dm_send_div.should be_nil
      u1.dm_send_div_key.should be_nil
      u1.dm_send_div_name.should be_nil
      u1.dm_send_div = '0'
      u1.dm_send_div.should == 0
      u1.dm_send_div_key.should == :not_prefer
      u1.dm_send_div_name.should == '希望しない'
      u1.dm_send_div = '1'
      u1.dm_send_div.should == 1
      u1.dm_send_div_key.should == :prefer
      u1.dm_send_div_name.should == '希望する'
    end

    # 性別区分
    it "sex means sex status." do
      User.sex_options.should == [["男性", '1'],["女性", '2']]
      User.sex_name_by_id('1').should == "男性"
      User.sex_name_by_id('2').should == "女性"
      User.sex_id_by_key(:male).should == '1'
      User.sex_id_by_key(:female).should == '2'

      u1 = User.new
      u1.sex.should be_nil
      u1.sex_key.should be_nil
      u1.sex_name.should be_nil
      u1.sex = '1'
      u1.sex.should == '1'
      u1.sex_key.should == :male
      u1.sex_name.should == '男性'
      u1.sex = '2'
      u1.sex.should == '2'
      u1.sex_key.should == :female
      u1.sex_name.should == '女性'
    end

    # 職業区分
    it "job_div means job status." do
      # NAME_MASTERから取得する為、テスト未実施
    end

    # 登録サイト区分
    it "regist_div means regist status." do
      User.regist_div_options.should == [["PC", '1'],["モバイル", '2']]
      User.regist_div_name_by_id('1').should == "PC"
      User.regist_div_name_by_id('2').should == "モバイル"
      User.regist_div_id_by_key(:pc).should == '1'
      User.regist_div_id_by_key(:mobile).should == '2'

      u1 = User.new
      u1.regist_div.should be_nil
      u1.regist_div_key.should be_nil
      u1.regist_div_name.should be_nil
      u1.regist_div = '1'
      u1.regist_div.should == '1'
      u1.regist_div_key.should == :pc
      u1.regist_div_name.should == 'PC'
      u1.regist_div = '2'
      u1.regist_div.should == '2'
      u1.regist_div_key.should == :mobile
      u1.regist_div_name.should == 'モバイル'
    end

    it "user_code is added automatically." do
      org_user = User.find(:first)
      new_user = User.new(org_user.attributes)
      new_user.member_div_key = :no_membership
      new_user.user_code = nil
      new_user.save

      new_user.user_code.should == sprintf('%012d', new_user.id)
    end

    it "member_div_is_membership" do
      test_user = User.new(:member_div_key => :membership)
      test_user.is_membership?.should be_true
    end

    it "member_div_not_is_membership" do
      test_user = User.new(:member_div_key => :no_membership)
      test_user.is_membership?.should be_false
    end

    it "member_div_offline_membership" do
      test_user = User.new(:member_div_key => :offline_membership)
      test_user.is_offline_membership?.should be_true
    end

    it "member_div_not_offline_membership" do
      test_user = User.new(:member_div_key => :no_membership)
      test_user.is_offline_membership?.should be_false
    end

    it "tel_num_not_hyphen method check." do
      test_user = User.new(:tel_num => '1234-56-7890')
      test_user.tel_num_not_hyphen.should == "1234567890"
    end

    it "birth_day_yyyy_mm_dd method check." do
      test_user = User.new
      test_user.birth_day_yyyy_mm_dd.should == nil
      test_user.birth_year = 2009
      test_user.birth_month = 10
      test_user.birth_day = 30
      test_user.birth_day_yyyy_mm_dd.should == "2009/10/30"
    end

    it "new_user method check" do
      user_param = {:user_code => '123456789012'}
      address_param = {:zip_code => '1234567'}
      email_param = {:email => 'email_address_01002@ec-one.com'}
      sub_address = {:name_last => 'testtest'}
      test_user = nil
      User.transaction do
        test_user = User.new_user(user_param, address_param, email_param, sub_address)
      end
      test_user.user_code.should == '123456789012'
      test_user.user_address.zip_code.should == '1234567'
      test_user.user_email.email.should == 'email_address_01002@ec-one.com'
      test_user.user_email_mobile.should be_nil
      test_user.sub_address.name_last.should == 'testtest'
    end

    it "new_user method check (site_type_div == :pc)" do
      user_param = {:user_code => '123456789012'}
      address_param = {:zip_code => '1234567'}
      email_param = {:email => 'email_address_01002@ec-one.com'}
      sub_address = {:name_last => 'testtest'}
      test_user = nil
      User.transaction do
        test_user = User.new_user(user_param, address_param, email_param, sub_address, :pc)
      end
      test_user.user_code.should == '123456789012'
      test_user.user_address.zip_code.should == '1234567'
      test_user.user_email.email.should == 'email_address_01002@ec-one.com'
      test_user.user_email_mobile.should be_nil
      test_user.sub_address.name_last.should == 'testtest'
    end

    it "new_user method check (site_type_div == :mobile)" do
      user_param = {:user_code => '123456789012'}
      address_param = {:zip_code => '1234567'}
      email_param = {:email => 'email_address_01002@ec-one.com'}
      sub_address = {:name_last => 'testtest'}
      test_user = nil
      User.transaction do
        test_user = User.new_user(user_param, address_param, email_param, sub_address, :mobile)
      end
      test_user.user_code.should == '123456789012'
      test_user.user_address.zip_code.should == '1234567'
      test_user.user_email.should be_nil
      test_user.user_email_mobile.email.should == 'email_address_01002@ec-one.com'
      test_user.sub_address.name_last.should == 'testtest'
    end

    # バリデーションチェック
    describe "Validation check" do
      before(:each) do
        @param = {:name_last => "意思湾", :name_first => '太郎',
          :kana_name_last => 'カイモノ', :kana_name_first => 'タロウ', :tel_num => '1234-56-7890',
          :sex => '1', :password => 'pass', :password_confirmation => 'pass', :remember_answer => 'テスト',
          :member_div => '1'
        }
      end
      it "性別　ショップ設定がない場合" do
        @param[:sex] = ""
        ShopSetting.stub!(:get_setting).and_return(nil)
        test_user = User.new(@param)
        test_user.errors_on(:sex).should_not == ["を入力してください。"]
      end
      it "性別　ショップ設定がある(true)場合" do
        @param[:sex] = ""
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:sex).should == ["を入力してください。"]
      end
      it "性別　ショップ設定がある(false)場合" do
        @param[:sex] = ""
        ShopSetting.stub!(:get_setting).and_return("false")
        test_user = User.new(@param)
        test_user.errors_on(:sex).should_not == ["を入力してください。"]
      end
      it "The required check of the password" do
        @param[:password] = ""
        @param[:password_confirmation] = ""
        test_user = User.new(@param)
        test_user.errors_on(:password).should == ["を入力してください。"]
      end
      it "The required check of the password_confirmation" do
        @param[:password] = ""
        @param[:password_confirmation] = ""
        test_user = User.new(@param)
        test_user.errors_on(:password_confirmation).should == ["を入力してください。"]
      end
      it "The required check of the remember_answer" do
        @param[:password] = ""
        @param[:password_confirmation] = ""
        @param[:remember_answer] = ""
        test_user = User.new(@param)
        test_user.errors_on(:remember_answer).should == ["を入力してください。"]
      end
      it "The length check of the password" do
        pending
        @param[:password] = "pas"
        test_user = User.new(@param)
        test_user.valid?
        test_user.errors_on(:password).should == ["は4文字以上で入力してください。"]
      end
      it "The password confirm check of the password" do
        pending
        @param[:password] = "password"
        @param[:password_confirmation] = "passpass"
        test_user = User.new(@param)
        test_user.valid?
        test_user.errors_on(:password).should == ["が一致しません。"]
      end
      it "職業　ショップ設定がない場合" do
        @param[:job_div] = ""
        ShopSetting.stub!(:get_setting).and_return(nil)
        test_user = User.new(@param)
        test_user.errors_on(:job_div).should_not == ["を入力してください。"]
      end
      it "職業　ショップ設定がある(true)場合" do
        @param[:job_div] = ""
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:job_div).should == ["を入力してください。"]
      end
      it "職業　ショップ設定がある(false)場合" do
        @param[:job_div] = ""
        ShopSetting.stub!(:get_setting).and_return("false")
        test_user = User.new(@param)
        test_user.errors_on(:job_div).should_not == ["を入力してください。"]
      end
      it "生年月日　ショップ設定がない場合" do
        @param[:birth_year] = ""
        @param[:birth_month] = ""
        @param[:birth_day] = ""
        ShopSetting.stub!(:get_setting).and_return(nil)
        test_user = User.new(@param)
        test_user.errors_on(:birth_year).should_not == ["を入力してください。"]
        test_user.errors_on(:birth_month).should_not == ["を入力してください。"]
        test_user.errors_on(:birth_month).should_not == ["を入力してください。"]
      end
      it "生年月日　ショップ設定がある(true)場合" do
        @param[:birth_year] = ""
        @param[:birth_month] = ""
        @param[:birth_day] = ""
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:birth_year).should == ["を入力してください。"]
        test_user.errors_on(:birth_month).should == ["を入力してください。"]
        test_user.errors_on(:birth_month).should == ["を入力してください。"]
      end
      it "生年月日　ショップ設定がある(false)場合" do
        @param[:birth_year] = ""
        @param[:birth_month] = ""
        @param[:birth_day] = ""
        ShopSetting.stub!(:get_setting).and_return("false")
        test_user = User.new(@param)
        test_user.errors_on(:birth_year).should_not == ["を入力してください。"]
        test_user.errors_on(:birth_month).should_not == ["を入力してください。"]
        test_user.errors_on(:birth_month).should_not == ["を入力してください。"]
      end
      it "ログインID必須チェック ショップ設定がない場合" do
        @param[:login_id] = ""
        ShopSetting.stub!(:get_setting).and_return(nil)
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == []
      end
      it "ログインID必須チェック ショップ設定がある(true)場合" do
        @param[:login_id] = ""
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == ["を入力してください。"]
      end
      it "ログインID必須チェック ショップ設定がある(false)場合" do
        @param[:login_id] = ""
        ShopSetting.stub!(:get_setting).and_return("false")
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == []
      end
      it "ログインIDフォーマットチェック 全角文字を入力した場合" do
        @param[:login_id] = "あいう"
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == ["はアルファベットで入力してください。"]
      end
      it "ログインIDフォーマットチェック 記号を入力した場合" do
        @param[:login_id] = "abc@def"
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == ["はアルファベットで入力してください。"]
      end
      it "ログインIDフォーマットチェック 英数字とアンダースコアのみの場合" do
        @param[:login_id] = "abc_012"
        ShopSetting.stub!(:get_setting).and_return("true")
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == []
      end
      it "ログインID重複チェック ショップ設定がない場合" do
        ShopSetting.stub!(:get_setting).and_return(nil)
        @param[:login_id] = "login1008"
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == []
      end
      it "ログインID重複チェック ショップ設定がある(false)場合" do
        ShopSetting.stub!(:get_setting).and_return("false")
        @param[:login_id] = "login1008"
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == []
      end
      it "ログインID重複チェック 重複している場合" do
        ShopSetting.stub!(:get_setting).and_return("true")
        @param[:login_id] = "login1008"
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == ["は、他のお客様が既にご使用されています。"]
      end
      it "ログインID重複チェック 重複していない場合" do
        ShopSetting.stub!(:get_setting).and_return("true")
        @param[:login_id] = "login1008xxx"
        test_user = User.new(@param)
        test_user.errors_on(:login_id).should == []
      end
    end

    describe 'is_unique_login_id?' do
      it "ec_frontの場合　重複あり　本番会員" do
        user = User.new(:login_id => 'login1008', :member_div => '1')
        user.is_unique_login_id?.should be_false
      end
      it "ec_testの場合　重複あり　テスト会員" do
        EcRider.config.stub!(:ec_test).and_return(true)
        User.find(1008).save
        User.find(1008).test_flg.should be_true
        user = User.new(:login_id => 'login1008', :member_div => '1')
        user.is_unique_login_id?.should be_false
      end
      it "ec_testの場合　本番会員とテスト会員の重複は許可" do
        EcRider.config.stub!(:ec_test).and_return(true)
        user = User.new(:login_id => 'login1008', :member_div => '1')
        user.is_unique_login_id?.should be_true
      end
      it "オフライン専用会員の場合の重複は許可" do
        user = User.new(:login_id => 'login1008', :member_div => '1')
        User.update(1008, :member_div_key => :offline_membership)
        user.is_unique_login_id?.should be_true
      end
      it "非会員の場合の重複は許可" do
        user = User.new(:login_id => 'login1008', :member_div => '1')
        User.update(1008, :member_div_key => :no_membership)
        user.is_unique_login_id?.should be_true
      end
      it "退会会員の場合の重複は許可" do
        user = User.new(:login_id => 'login1008', :member_div => '1')
        User.update(1008, :member_div_key => :resigned)
        user.is_unique_login_id?.should be_true
      end
      it "管理画面新規登録でユーザ区分が決まっていない場合は許可" do
        user = User.new(:login_id => 'login1008', :member_div => nil)
        User.update(1008, :member_div_key => :resigned)
        user.is_unique_login_id?.should be_true
      end
    end
  end

  describe "check password complexity policy" do
    before(:each) do
      @param = {:password => "password", :password_confirmation => "password",
        :remember_question => "question", :remember_answer => "answer",
        :birth_year => "2009", :birth_month => "12", :birth_day => "31", :member_div => '2'}
    end
    it "complexity check : all alphabet" do
      @param[:password] = "password"
      @param[:password_confirmation] = "password"
      test_user_form = User.new(@param)
      test_user_form.should_not be_valid
      test_user_form.errors.on(:password).should == "はアルファベットと数字を混在させて入力してください。"
    end
    it "complexity check : all number" do
      @param[:password] = "12345"
      @param[:password_confirmation] = "12345"
      test_user_form = User.new(@param)
      test_user_form.should_not be_valid
      test_user_form.errors.on(:password).should == "はアルファベットと数字を混在させて入力してください。"
    end
    it "complexity check : number + alphabet" do
      @param[:password] = "123ab"
      @param[:password_confirmation] = "123ab"
      test_user_form = User.new(@param)
      test_user_form.should be_valid
    end
    it "complexity check : alphabet + number" do
      @param[:password] = "abc12"
      @param[:password_confirmation] = "abc12"
      test_user_form = User.new(@param)
      test_user_form.should be_valid
    end
    it "complexity check : alphabet + number + alphabet" do
      @param[:password] = "ab3de"
      @param[:password_confirmation] = "ab3de"
      test_user_form = User.new(@param)
      test_user_form.should be_valid
    end
    it "complexity check : alphabet + number + alphabet" do
      @param[:password] = "ab3de"
      @param[:password_confirmation] = "ab3de"
      test_user_form = User.new(@param)
      test_user_form.should be_valid
    end
    it "complaxity check : include symbol" do
      @param[:password] = "ab3d@"
      @param[:password_confirmation] = "ab3d@"
      test_user_form = User.new(@param)
      test_user_form.should_not be_valid
      test_user_form.errors.on(:password).should == "はアルファベットと数字を混在させて入力してください。"
    end
  end

  it "resigned method check." do
    test_user = User.find(1002)
    test_user.resigned(ResignedReason.new(:user_id => 1002))
    test_user.resigned_reason.sex.should == '1'
    test_user.resigned_reason.job_div.should == '2'
    test_user.member_div.should == '2'
  end

  it "remind method check." do
    User.remind("email_address_01002@ec-one.com", "1", "ヤマダ", "yamada1", "yamada1").should be_true
  end

  it "remind method check with wrong email_address." do
    User.remind("wrongaddress@ec-one.com", "1", "ヤマダ", "yamada1", "yamada1").should be_false
  end


  describe "age" do
    it "original test" do
      # 年が経ってもチェックが正常終了するように対応
      test_year = 2009
      now_date = Time.now
      now_year = now_date.year
      add_year = now_year.to_i - test_year
      test_user = User.find(1002)
      test_user.birth_year += add_year
      # user.id=1002の誕生日は1/23。現在が1/23より前なら51歳、以降なら52歳
      test_user.age.should == (now_date.strftime("%m%d").to_i < 123 ? 51 : 52)
    end

    describe "akima's birthday" do
      before(:each) do
        @user = User.new(:birth_year => 1973, :birth_month => 12, :birth_day => 31)
      end

      it "on 2009/07/08" do
        Time.stub!(:now).and_return(Time.parse("2009/7/6 12:00:00"))
        @user.age.should == 35
      end
      it "on 2009/07/08 by argument" do
        @user.age(Time.parse("2009/7/6 12:00:00")).should == 35
      end

      it "on 2009/12/31" do
        Time.stub!(:now).and_return(Time.parse("2009/12/31 12:00:00"))
        @user.age.should == 36
      end
      it "on 2009/12/31 by argument" do
        @user.age(Time.parse("2009/12/31 12:00:00")).should == 36
      end

      it "on 2010/01/01" do
        Time.stub!(:now).and_return(Time.parse("2010/1/1 12:00:00"))
        @user.age.should == 36
      end
      it "on 2010/01/01 by argument" do
        @user.age(Time.parse("2010/1/1 12:00:00")).should == 36
      end
    end
  end

  it "total_purchase method check." do
    test_user = User.find(1002)
    test_user.total_purchase.should == 17750
  end

  it "total_purchase_for_distance" do
    test_user = User.find(1003)
    ordered_at = Time.local(2007,12,10,11,9,38)
    test_user.orders[0].ordered_at.should == ordered_at
    test_user.total_purchase.should == 3360
    test_user.user_rank_updated_at = Time.local(2010,3,26,0,0,0)
    test_user.user_rank_check_end_date = Time.local(2010,9,26,0,0,0)
    test_user.total_purchase_for_distance.should_not == 3360
    test_user.user_rank_updated_at = ordered_at
    test_user.user_rank_check_end_date = Time.local(2010,12,10,11,9,38)
    test_user.total_purchase_for_distance.should == 3360
    test_user.user_rank_updated_at = Time.local(2006,3,26,0,0,0)
    test_user.user_rank_check_end_date = ordered_at
    test_user.total_purchase_for_distance.should == 3360
  end

  it "order_count_for_distance" do
    test_user = User.find(1003)
    test_user.user_rank_updated_at = Time.local(2010,3,26,0,0,0)
    test_user.user_rank_check_end_date = Time.local(2010,9,26,0,0,0)
    test_user.order_count_for_distance.should == 0
    test_user.user_rank_updated_at = Time.local(2006,3,26,0,0,0)
    test_user.user_rank_check_end_date = Time.local(2010,9,26,0,0,0)
    test_user.order_count_for_distance.should == 1
  end

  it "count_day_of_member_created" do
    Time.stub!(:now).and_return(Time.parse("2008/12/10 10:00:00"))
    test_user = User.find(1003)
    test_user.created_at.should == Time.local(2007,12,10,11,9,43)
    test_user.count_day_of_member_created.should == 365
  end

  it "count_day_of_ordered" do
    Time.stub!(:now).and_return(Time.parse("2008/12/10 10:00:00"))
    test_user = User.find(1003)
    test_user.orders[0].ordered_at.should == Time.local(2007,12,10,11,9,38)
    test_user.count_day_of_ordered.should == 365
  end

  it "deletable? method check." do
    test_user = User.find(1002)
    test_user.deletable?.should be_false
  end

  it "deletable? 全ステータスで確認" do
    OrderShipping.status_div_keys.each do |key|
      test_user = User.new()
      order_shipping = OrderShipping.new(:status_div_key => key)
      order = Order.new()
      order.stub(:order_shippings).and_return([order_shipping])
      test_user.stub!(:orders).and_return([order])

      case key
      when :before_settlement;     test_user.deletable?.should be_false
      when :undelivery_settlement; test_user.deletable?.should be_false
      when :undelivery_unpaid;     test_user.deletable?.should be_false
      when :undelivery;            test_user.deletable?.should be_false
      when :deliver_request;       test_user.deletable?.should be_false
      when :delivered;             test_user.deletable?.should be_true
      when :deliver_advised;       test_user.deletable?.should be_true
      when :cancel_request;        test_user.deletable?.should be_false
      when :canceled;              test_user.deletable?.should be_true
      when :settlement_failed;     test_user.deletable?.should be_true
      when :stock_error;           test_user.deletable?.should be_true
      when :internal_error;        test_user.deletable?.should be_true
      when :external_error;        test_user.deletable?.should be_true
      when :returned;              test_user.deletable?.should be_true
      when :sales_period_error;    test_user.deletable?.should be_true
      else violated '想定外のステータスです'
      end
    end
  end

  describe 'set_test_flg' do
    it "ec_adminの場合　本番データ" do
      u1 = User.new
      u1.set_test_flg
      u1.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      u1 = User.new(:test_flg => true)
      u1.set_test_flg
      u1.test_flg.should be_true
    end

    it "ec_frontの場合" do
      u1 = User.new
      u1.set_test_flg
      u1.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      u1 = User.new
      u1.set_test_flg
      u1.test_flg.should be_true
    end
  end

  describe 'find_membership_user and authenticate' do
    describe "本番環境" do
      describe "本番会員" do
        describe "WEB会員" do
          it "ログイン成功" do
            User.find_membership_user('email_address_01008@test.com').should == User.find(1008)
            User.authenticate('email_address_01008@test.com', 'pass1').should == User.find(1008)
          end
          it "不正なパスワードではログインできない" do
            User.find_membership_user('email_address_01008@test.com').should == User.find(1008)
            User.authenticate('email_address_01008@test.com', 'wrongpassword').should be_nil
          end
          it "不正な（未登録の）メールアドレスではログインできない" do
            User.find_membership_user('wrongmailaddress@wrongmailaddress.com').should be_nil
            User.authenticate('wrongmailaddress@wrongmailaddress.com', 'password').should be_nil
          end
          it "メールアドレスが空ではログインできない" do
            User.find_membership_user('').should be_nil
            User.authenticate('', 'password').should be_nil
          end
          it "ログインIDでログイン成功" do
            User.find_membership_user('login1008').should == User.find(1008)
            User.authenticate('login1008', 'pass1').should == User.find(1008)
          end
          it "ログインIDで不正なパスワードではログインできない" do
            User.find_membership_user('login1008').should == User.find(1008)
            User.authenticate('login1008', 'wrongpassword').should be_nil
          end
          it "モバイル用メールアドレスでログイン成功" do
            UserEmail.create(
              :user_id => 1008,
              :email => 'email_address_01008_mobile@test_mobile.com',
              :pc_mobile_div_key => :mobile,
              :mail_address_pc_mobile_div_key => :mobile)
            User.find_membership_user('email_address_01008_mobile@test_mobile.com').should == User.find(1008)
            User.authenticate('email_address_01008_mobile@test_mobile.com', 'pass1').should == User.find(1008)
          end
        end
        describe "退会済み" do
          before do
            User.update(1008, :member_div_key => :resigned)
          end
          it "ログインできない（メールアドレス）" do
            User.find_membership_user('email_address_01008@test.com').should be_nil
            User.authenticate('email_address_01008@test.com', 'pass1').should be_nil
          end
          it "ログインできない（ログインID）" do
            User.find_membership_user('login1008').should be_nil
            User.authenticate('login1008', 'pass1').should be_nil
          end
        end
        describe "非会員" do
          before do
            User.update(1008, :member_div_key => :no_membership)
          end
          it "ログインできない（メールアドレス）" do
            User.find_membership_user('email_address_01008@test.com').should be_nil
            User.authenticate('email_address_01008@test.com', 'pass1').should be_nil
          end
          it "ログインできない（ログインID）" do
            User.find_membership_user('login1008').should be_nil
            User.authenticate('login1008', 'pass1').should be_nil
          end
        end
        describe "オフライン会員" do
          before do
            User.update(1008, :member_div_key => :offline_membership)
          end
          it "ログインできない（メールアドレス）" do
            User.find_membership_user('email_address_01008@test.com').should be_nil
            User.authenticate('email_address_01008@test.com', 'pass1').should be_nil
          end
          it "ログインできない（ログインID）" do
            User.find_membership_user('login1008').should be_nil
            User.authenticate('login1008', 'pass1').should be_nil
          end
        end
      end
      describe "テスト会員" do
        before do
          User.update(1008, :test_flg => true)
        end
        it "ログインできない（メールアドレス）" do
          User.find_membership_user('email_address_01008@test.com').should be_nil
          User.authenticate('email_address_01008@test.com', 'pass1').should be_nil
        end
        it "ログインできない（ログインID）" do
          User.find_membership_user('login1008').should be_nil
          User.authenticate('email_address_01008@test.com', 'pass1').should be_nil
        end
      end
    end
    describe "テスト環境" do
      before do
        EcRider.config.stub!(:ec_test).and_return(true)
      end
      describe "本番会員" do
        it "ログインできない（メールアドレス）" do
          User.find_membership_user('email_address_01008@test.com').should be_nil
          User.authenticate('email_address_01008@test.com', 'pass1').should be_nil
        end
        it "ログインできない（ログインID）" do
          User.find_membership_user('login1008').should be_nil
          User.authenticate('login1008', 'pass1').should be_nil
        end
      end
      describe "テスト会員" do
        before do
          User.update(1008, :test_flg => true)
        end
        it "ログインできる（メールアドレス）" do
          User.find_membership_user('email_address_01008@test.com').should == User.find(1008)
          User.authenticate('email_address_01008@test.com', 'pass1').should == User.find(1008)
        end
        it "ログインできる（ログインID）" do
          User.find_membership_user('login1008').should == User.find(1008)
          User.authenticate('login1008', 'pass1').should == User.find(1008)
        end
      end
    end
  end

  it "test remember_token?" do
    u1 = User.new()
    u1.remember_token?.should == nil
    u1.remember_token_expires_at = Time.now()
    u1.remember_token?.should == u1.remember_token_expires_at
  end

  it "test remember_me" do
    u1 = User.find(1008)
    before_remember_token = u1.remember_token
    before_expires_at = u1.remember_token_expires_at
    u1.remember_me
    u1.remember_token.should_not == before_remember_token
    u1.remember_token_expires_at.should_not == before_expires_at
  end

  it "test forget_me" do
    u1 = User.find(1008)
    u1.forget_me
    u1.remember_token.should be_nil
    u1.remember_token_expires_at.should be_nil
  end

  it "test shipping_addresses" do
    u1 = User.find(1007)
    u1.user_addresses.length.should == 2
    u1.user_addresses.first.id.should == 1009
    u1.user_addresses.first.address_div.should == '0'
    u1.user_addresses.last.id.should == 1010
    u1.user_addresses.last.address_div.should == '1'

    u1.shipping_addresses.length.should == 1

    u2 = User.find(1008)
    u2.user_addresses.length.should == 2
    u2.shipping_addresses.length.should == 0
  end

  it "test has_many addresses" do
    u1 = User.find(1008)
    u1.user_addresses.length.should == 2
    u1.user_address.id.should == 1011

    # 会員住所の情報を別のレコードに作り直す
    add1 = UserAddress.find(1011)
    add2 = UserAddress.new(add1.attributes)
    add1.destroy
    add2.save!

    u2 = User.find(1008)
    u2.user_addresses.length.should == 2
    u2.user_address.id.should == add2.id
  end

  describe "test shippable_to_oversea?" do
    it "test japanese user" do
      u1 = User.find(1002)
      u1.shippable_to_oversea?.should == false
    end

    it "test other language user" do
      u1 = User.find(1002)
      u1.user_address.locale = 'en'
      u1.shippable_to_oversea?.should == true
    end

    it "test other country user" do
      u1 = User.find(1002)
      u1.user_address.country_code = 'US'
      u1.shippable_to_oversea?.should == true
    end

    it "test japanese user with sub_address" do
      u1 = User.find(1002)
      hash = {:name_last => 'a', :name_first => 'a', :address_1 => 'a', :address_2 => 'a'}
      new_address = UserAddress.hash_to_sub_address(hash)
      new_address.set_sub_address_from_address(u1.user_address)
      u1.user_addresses << new_address

      u1.shippable_to_oversea?.should == true
    end
  end

  describe "test user_and_shipping_addresses" do
    it "test only user" do
      u1 = User.find(1002)
      u1.user_and_shipping_addresses.length.should == 1
    end

    it "test only user with sub_address" do
      u1 = User.find(1002)
      hash = {:name_last => 'a', :name_first => 'a', :address_1 => 'a', :address_2 => 'a'}
      new_address = UserAddress.hash_to_sub_address(hash)
      new_address.set_sub_address_from_address(u1.user_address)
      u1.user_addresses << new_address

      u1.user_and_shipping_addresses.length.should == 1
    end

    it "test user with shipping_address" do
      u1 = User.find(1007)
      u1.user_and_shipping_addresses.length.should == 2

      hash = {:name_last => 'a', :name_first => 'a', :address_1 => 'a', :address_2 => 'a'}
      new_address = UserAddress.hash_to_sub_address(hash)
      new_address.set_sub_address_from_address(u1.user_address)
      u1.user_addresses << new_address

      u1.user_and_shipping_addresses.length.should == 2
    end
  end

  describe "test honorific_prefix" do
    it "test Mr" do
      u1 = User.find(1008)
      u1.honorific_prefix.should == 'Mr.'
    end
    it "test Ms" do
      u1 = User.find(1006)
      u1.honorific_prefix.should == 'Ms.'
    end
  end

  describe "test change_password" do
    it "test validate error" do
      u1 = User.find(1002)
      u1.change_password('', '').should == false
    end

    it "test save error" do
      u1 = User.find(1002)
      u1.change_password('', '').should == false
    end

    it "test save success" do
      u1 = User.find(1002)
      u1.change_password('abc123', 'abc123').should == true
    end
  end

  describe "Validation for user edit." do
    describe "When locale is Japanese," do

      it "User.human_name should be resouce in Japanese." do
        User.human_name().should == "会員"
      end

      it "User.human_attribute_name should be resouce in Japanese." do
        User.human_attribute_name("birth").should == "生年月日"
        User.human_attribute_name("birth_year").should == "誕生日（年）"
        User.human_attribute_name("birth_month").should == "誕生日（月）"
        User.human_attribute_name("birth_day").should == "誕生日（日）"
        User.human_attribute_name("password").should == "パスワード"
        User.human_attribute_name("password_confirmation").should == "パスワード確認"
        User.human_attribute_name("remember_question").should == "秘密の質問"
        User.human_attribute_name("remember_answer").should == "秘密の質問（答え）"
      end

      describe "Validation check." do
        before(:each) do
          @param = {:password => "password", :password_confirmation => "password",
            :remember_question => "question", :remember_answer => "answer",
            :birth_year => "2009", :birth_month => "12", :birth_day => "31", :member_div => '1'}
        end
        it "The required check of the password" do
          @param[:password] = ""
          @param[:password_confirmation] = ""
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:password).should == "を入力してください。"
        end
        it "The required check of the password_confirmation" do
          @param[:password] = ""
          @param[:password_confirmation] = ""
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:password_confirmation).should == "を入力してください。"
        end
        it "The required check of the remember_question" do
          @param[:password] = ""
          @param[:remember_question] = ""
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:remember_question).should == "を入力してください。"
        end
        it "The required check of the remember_answer" do
          @param[:password] = ""
          @param[:remember_answer] = ""
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:remember_answer).should == "を入力してください。"
        end
        it "The required check of the birth_year" do
          @param[:birth_year] = ""
          test_user_form = User.new(@param.merge(:birth_check => true))
          test_user_form.should_not be_valid
          test_user_form.errors.on(:birth_year).should == "を入力してください。"
        end
        it "The required check of the birth_month" do
          @param[:birth_month] = ""
          test_user_form = User.new(@param.merge(:birth_check => true))
          test_user_form.should_not be_valid
          test_user_form.errors.on(:birth_month).should == "を入力してください。"
        end
        it "The required check of the birth_day" do
          @param[:birth_day] = ""
          test_user_form = User.new(@param.merge(:birth_check => true))
          test_user_form.should_not be_valid
          test_user_form.errors.on(:birth_day).should == "を入力してください。"
        end
        it "The confirmation check of the password" do
          @param[:password] = "passwor1"
          @param[:password_confirmation] = "passpass"
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:password).should == "が一致しません。"
        end
        it "The length check of the password" do
          @param[:password] = "p1"
          @param[:password_confirmation] = "p1"
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:password).should == "は4文字以上で入力してください。"
        end
        it "The length check of the password_confirmation" do
          @param[:password] = "1p"
          @param[:password_confirmation] = "1p"
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:password_confirmation).should == "は4文字以上で入力してください。"
        end
        it "The date validation check of the birth" do
          @param[:birth_day] = "32"
          test_user_form = User.new(@param)
          test_user_form.should_not be_valid
          test_user_form.errors.on(:birth).should == "は正しくありません。"
        end
      end
    end

    it "sub_address should return nil" do
      test_user_form = User.new()
      test_user_form.sub_address.should be_nil
    end
  end

  describe "birthday" do
    it "nil" do
      User.new.birthday.should == nil
    end

    it "valid" do
      User.new(:birth_year => 1973, :birth_month => 12, :birth_day => 31).birthday.should == Date.parse("1973/12/31")
    end

    it "invalid" do
      lambda{ User.new(:birth_year => 1973, :birth_month => 13, :birth_day => 77).birthday}.should raise_error(ArgumentError)
    end
  end

  describe "birthday_of_year" do
    it "nil" do
      User.new.birthday_of_year.should == nil
    end

    it "valid" do
      User.new(:birth_year => 1973, :birth_month => 2, :birth_day => 3).birthday_of_year.should == "02/03"
      User.new(:birth_year => 1973, :birth_month => 12, :birth_day => 31).birthday_of_year.should == "12/31"
    end

    it "invalid" do
      lambda{ User.new(:birth_year => 1973, :birth_month => 13, :birth_day => 77).birthday_of_year}.should raise_error(ArgumentError)
    end
  end

  describe "test?" do
    it "テスト会員" do
      User.new(:test_flg => true).test?.should be_true
    end

    it "本番会員" do
      User.new().test?.should be_false
      User.new(:test_flg => "").test?.should be_false
      User.new(:test_flg => nil).test?.should be_false
      User.new(:test_flg => false).test?.should be_false
    end
  end
  
  describe "test locked?" do
    it "test with login_challenge_count and locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = nil
      u.locked_at = nil
      u.save!
      u.locked?.should == false
    end
    it "test with login_challenge_count is nil" do
      u = User.find(1002)
      u.login_challenge_count = nil
      u.locked_at = Time.now
      u.save!
      u.locked?.should == false
    end
    it "test with locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = 100
      u.locked_at = nil
      u.save!
      u.locked?.should == false
    end
    it "test with login_challenge_count < MAX_LOGIN_CHALLENGE_COUNT, and locked_at isn't elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT - 1
      u.locked_at = Time.now
      u.save!
      u.locked?.should == false
    end
    it "test with login_challenge_count < MAX_LOGIN_CHALLENGE_COUNT, and locked_at was elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT - 1
      u.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).ago(1.second)
      u.save!
      u.locked?.should == false
    end
    it "test with login_challenge_count == MAX_LOGIN_CHALLENGE_COUNT, and locked_at isn't elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at = Time.now
      u.save!
      u.locked?.should == true
    end
    it "test with login_challenge_count == MAX_LOGIN_CHALLENGE_COUNT, and locked_at was elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).ago(1.second)
      u.save!
      u.locked?.should == false
    end
    it "test with login_challenge_count > MAX_LOGIN_CHALLENGE_COUNT, and locked_at isn't elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT + 1
      u.locked_at = Time.now
      u.save!
      u.locked?.should == true
    end
    it "test with login_challenge_count > MAX_LOGIN_CHALLENGE_COUNT, and locked_at is elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT + 1
      u.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).ago(1.second)
      u.save!
      u.locked?.should == false
    end
  end

  describe "test increment_login_challenge_count" do
    it "test with login_challenge_count and locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = nil
      u.locked_at = nil
      u.save!
      u.increment_login_challenge_count
      u = User.find(1002)
      u.login_challenge_count.should == 1
      u.locked_at.should == nil
    end
    it "test with login_challenge_count < (MAX_LOGIN_CHALLENGE_COUNT -1), and locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT - 2
      u.locked_at = nil
      u.save!
      u.increment_login_challenge_count
      u = User.find(1002)
      u.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT - 1
      u.locked_at.should == nil
    end
    it "test with login_challenge_count == (MAX_LOGIN_CHALLENGE_COUNT - 1), and locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT - 1
      u.locked_at = nil
      u.save!
      u.increment_login_challenge_count
      u = User.find(1002)
      #ロック情報が記録される
      u.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at.should <= Time.now
      u.locked_at.should > Time.now.ago(1.minutes)
    end
    it "test with login_challenge_count == (MAX_LOGIN_CHALLENGE_COUNT - 1), and locked_at isn't nil" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT - 1
      u.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).since(1.minute) #ロック終了１分前
      u.save!
      u.increment_login_challenge_count
      u = User.find(1002)
      #ロック情報が記録される
      u.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at.should <= Time.now
      u.locked_at.should > Time.now.ago(1.minutes)
    end
    it "test with login_challenge_count == MAX_LOGIN_CHALLENGE_COUNT, and locked_at isn't elasped USER_LOCK_PERIOD_SECONDS" do
      locked_time = Time.now
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at = locked_time
      u.save!
      u.increment_login_challenge_count
      u = User.find(1002)
      #ロック中は何も実行されない
      u.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at.to_s.should == locked_time.to_s
    end
    it "test with login_challenge_count > MAX_LOGIN_CHALLENGE_COUNT, and locked_at isn't elasped USER_LOCK_PERIOD_SECONDS" do
      locked_time = Time.now
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT + 1
      u.locked_at = locked_time
      u.save!
      u.increment_login_challenge_count
      #ロック中は何も実行されない
      u = User.find(1002)
      u.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT + 1
      u.locked_at.to_s.should == locked_time.to_s
    end
    it "test with login_challenge_count == MAX_LOGIN_CHALLENGE_COUNT, and locked_at was elasped USER_LOCK_PERIOD_SECONDS" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).ago(1.second)
      u.save!
      u.increment_login_challenge_count
      u = User.find(1002)
      #ロック期間終了すると、いったん初期化されてから再カウントされる
      u.login_challenge_count.should == 1
      u.locked_at.should == nil
    end
  end

  describe "test clear_login_challenge_count" do
    it "test with login_challenge_count and locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = nil
      u.locked_at = nil
      u.save!
      u.clear_login_challenge_count
      u = User.find(1002)
      u.login_challenge_count == 0
      u.locked_at == nil
    end
    it "test with login_challenge_count is not nil and locked_at is nil" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at = nil
      u.save!
      u.clear_login_challenge_count
      u = User.find(1002)
      u.login_challenge_count == 0
      u.locked_at == nil
    end
    it "test with login_challenge_count is nil and locked_at is not nil" do
      u = User.find(1002)
      u.login_challenge_count = nil
      u.locked_at = Time.now
      u.save!
      u.clear_login_challenge_count
      u = User.find(1002)
      u.login_challenge_count == 0
      u.locked_at == nil
    end
    it "test with login_challenge_count and locked_at is not nil" do
      u = User.find(1002)
      u.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      u.locked_at = Time.now
      u.save!
      u.clear_login_challenge_count
      u = User.find(1002)
      u.login_challenge_count == 0
      u.locked_at == nil
    end
  end

  describe "address_isolated?" do
    it "離島" do
      ShopSetting.stub!(:shipping_cost_outlier_or_nil).with("8700001").and_return(100)
      u = User.find(1002)
      u.address_isolated?.should be_true
    end

    it "離島でない" do
      ShopSetting.stub!(:shipping_cost_outlier_or_nil).with("8700001").and_return(nil)
      u = User.find(1002)
      u.address_isolated?.should be_false
    end
  end

  describe "fullname" do
    it "user_address != nil" do
      u = User.find(1002)
      u.fullname.should == "山田 太郎"
    end
    it "user_address == nil" do
      u = User.find(1002)
      u.user_address = nil
      u.fullname.should == ""
    end
  end

  describe "fullname_kana" do
    it "user_address != nil" do
      u = User.find(1002)
      u.fullname_kana.should == "ヤマダ タロウ"
    end
    it "user_address == nil" do
      u = User.find(1002)
      u.user_address = nil
      u.fullname_kana.should == ""
    end
  end

end
