# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Goldberg::Role do
  # fixtures :roles # spec/spec_helper.rb の config.global_fixtures でロードされています。


  describe "has_users?" do
    it "should return false without user" do
      role = Goldberg::Role.new
      role.has_users?.should == false
    end

    it "should return true with user" do
      role = Goldberg::Role.find(1)
      role.users.count.should > 0
      role.has_users?.should == true
    end
  end


  describe "can_access?" do
    before(:each) do
      @action_group = Goldberg::SiteActionGroup.find_by_name("countries")
      @update_action = @action_group[:update]
    end

    it "administrator can't access nil action" do
      admin = Goldberg::Role.find(1) # システム管理者
      admin.can_access?(nil).should == false
    end

    it "administrator can access countries/update" do
      admin = Goldberg::Role.find(1) # システム管理者
      admin.can_access?(@update_action).should == true
    end

    it "normal_user can't access countries/update" do
      user = Goldberg::Role.find(3) # 一般管理者
      user.can_access?(@update_action).should == false
    end
  end

  describe "action_ids" do
    # # before(:each) で書こうと思ったけど、フィクスチャがロードされていないらしく正しく動作しなかったので、仕方なくメソッドに。
    # def prepare_action_ids
    before(:each) do
      action_paths = %w(countries/index countries/update inquiry/index inquiry/delete_memo inquiry/regist_inquiry_memo)
      @action_ids = action_paths.inject({}) do |dest, path|
        action = Goldberg::SiteAction.find_by_path(path)
        puts "path '#{path}' not found" unless action
        action.should_not be_nil
        dest[path] = action.id
        dest
      end
    end

    it "create時に一緒に保存します" do
      test_action_ids = [ @action_ids["countries/index"], @action_ids["countries/update"] ].sort
      test_action_ids.each{|action_id| action_id.should_not be_nil }
      role = Goldberg::Role.new(:name => "new role", :action_ids => test_action_ids)
      role.save.should == true
      role = Goldberg::Role.find(role.id)
      role.role_actions.count.should == 2
      role.role_actions.map(&:site_action_id).sort.should == test_action_ids
    end

    it "update時は、元々のrole_actionsを全て削除して、渡されたものを一緒に保存します" do
      test_action_ids = [ @action_ids["countries/index"], @action_ids["countries/update"] ].sort
      role = Goldberg::Role.new(:name => "new role", :action_ids => test_action_ids)
      role.save.should == true
      role = Goldberg::Role.find(role.id)
      role.role_actions.count.should == 2
      role.role_actions.map(&:site_action_id).sort.should == test_action_ids

      test_action_ids = [ 
        @action_ids["countries/index"],
        @action_ids["inquiry/index"], @action_ids["inquiry/delete_memo"], @action_ids["inquiry/regist_inquiry_memo"],
      ].sort
      role.update_attributes(:name => "updated role", :action_ids => test_action_ids)
      role = Goldberg::Role.find(role.id)
      role.role_actions.count.should == 4
      role.role_actions.map(&:site_action_id).sort.should == test_action_ids
    end
  end

end
