# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Goldberg::SiteActionGroup do

  it "test_invalid_without_name" do
    site_controller = Goldberg::SiteActionGroup.new
    site_controller.valid?.should_not be_true
    site_controller.errors.invalid?(:name).should be_true
    site_controller.save.should_not be_true
  end


  describe "allow_update?" do
    it "should return true for normal controllers" do
      action_groups = Goldberg::SiteActionGroup.find(:all, 
        :conditions => ["name not like ?", 'goldberg/%']) # NOT LIKEです
      action_groups.each do |action_group|
        action_group.allow_update?.should == true
      end
    end

    it "should return false for goldberg controllers" do
      action_groups = Goldberg::SiteActionGroup.find(:all, 
        :conditions => ["name like ?", 'goldberg/%']) # LIKEです
      action_groups.each do |action_group|
        action_group.allow_update?.should == false
      end
    end
  end


  describe "about restful actions" do
    before(:each) do
      @restful_controller = Goldberg::SiteActionGroup.create(:name => "test/restful")
      %w(index show new create edit update destroy).each do |action_name|
        @restful_controller.actions.create(:name => action_name)
      end
    end


    describe "[]" do
      it "should return Goldberg::SiteAction by action_name" do
        [:index, 'index'].each do |action_name|
          index = @restful_controller[action_name]
          index.should_not == nil
          index.name.should == 'index'
          index.site_action_group_id.should == @restful_controller.id
        end
      end

      it "should return nil for unexist_action_name" do
        index = @restful_controller['absolutely_unexist_action']
        index.should == nil
      end
    end
  end

  describe "categorised_actions" do
    before(:each) do
      @example_controller = Goldberg::SiteActionGroup.create(:name => "test/example")
      @example_controller.actions.create(:category_name => 'index', :name => 'init_index')
      @example_controller.actions.create(:category_name => 'index', :name => 'index')
      @example_controller.actions.create(:category_name => 'index', :name => 'detail')
      @example_controller.actions.create(:category_name => 'create', :name => 'new')
      @example_controller.actions.create(:category_name => 'create', :name => 'create')
      @example_controller.actions.create(:category_name => 'create', :name => 'create_item')
      @example_controller.actions.create(:category_name => 'destroy', :name => 'delete')
      @example_controller.actions.create(:category_name => 'other', :name => 'open')
      @example_controller.actions.create(:category_name => 'other', :name => 'close')
    end
    
    it "has_category? should" do
      %w(index create destroy other).each do |category_name|
        @example_controller.has_category?(category_name).should == true
        @example_controller.has_category?(category_name.to_sym).should == true
      end
      @example_controller.has_category?('update').should == false
      @example_controller.has_category?(:update).should == false
    end
    
    it "should devide actions" do
      expected_action_names = {
        :index => %w(detail index init_index),
        :create => %w(create create_item new),
        :update => [],
        :destroy =>  %w(delete),
        :other =>  %w(close open)
      }
      expected_action_names.each do |category, action_names|
        @example_controller.categorised_actions[category.to_s].map(&:name).sort.should == action_names
        @example_controller.categorised_actions[category.to_sym].map(&:name).sort.should == action_names
      end
    end
    
    describe "with update actions" do
      before(:each) do
        @example_controller.actions.create(:category_name => 'update', :name => 'edit')
        @example_controller.actions.create(:category_name => 'update', :name => 'update')
      end

      it "should return true for all" do
        %w(index create update destroy).each do |category_name|
          @example_controller.has_category?(category_name).should == true
          @example_controller.has_category?(category_name.to_sym).should == true
        end
      end
      
      it "should devide actions" do
        @example_controller.categorised_actions[:update].map(&:name).sort.should == %w(edit update)
      end
    end
  end

  describe "base_name" do
    it "should remove '管理'" do
      Goldberg::SiteActionGroup.new(:display_name => 'キャンペーン管理').base_name.should == 'キャンペーン'
      Goldberg::SiteActionGroup.new(:display_name => 'ファイルマネージャ').base_name.should == 'ファイルマネージャ'
    end
  end



#   it "test_uniqueness_of_name" do
#     name_1 = 'THIS_IS_NAME_1'
#     name_2 = 'THIS_IS_NAME_2'
#     name_1.freeze
#     name_2.freeze

#     site_controller_1 = Goldberg::SiteActionGroup.new
#     site_controller_1.permission = @p
#     site_controller_2 = Goldberg::SiteActionGroup.new
#     site_controller_2.permission = @p

#     site_controller_1.name = name_1
#     site_controller_2.name = name_2

#     (site_controller_1.save).should be_true
#     (site_controller_2.save).should be_true

#     site_controller_2.name = name_1

#     # 権限毎にコントローラを登録するようにしたので、一意でなくても登録できます
#     (site_controller_2.save).should be_true
#     (site_controller_2.errors.invalid?(:name)).should(nil)
#   end

  
#  #create so the classes method can be tested...
#  class ThereShouldBeNoSuchRealClassNameGoldbergTestController < ApplicationController
#  end

#   describe "classes" do
#     it "コントローラのcontroller_pathとコントローラのクラスを返します" do
#       controller_classes = Goldberg::SiteActionGroup.classes
#       controller_classes.class.should == Hash
#       controller_classes['application'].should == nil
#       controller_classes.values.should_not include(::ApplicationController)
#       controller_classes.values.should_not include(ThereShouldBeNoSuchRealClassNameGoldbergTestController)
#       controller_classes['user'].should == ::UserController
#       controller_classes['cms'].should == ::CmsController
#       controller_classes['goldberg/users'].should == ::Goldberg::UsersController
#     end
#   end


  
end
