# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Goldberg::SiteAction do
  include EcRider::PluginModule::RSpecExt
  describe "find_by_path" do
    it "文字列で指定した場合" do
      action = Goldberg::SiteAction.find_by_path("cms_pages/index")
      action.new_record?.should == false
      action.name.should == 'index'
      action.controller.name = "cms_pages"
    end

    it "配列で指定した場合" do
      action = Goldberg::SiteAction.find_by_path(["countries", "update"])
      action.new_record?.should == false
      action.name.should == 'update'
      action.controller.name = "countries"
    end

    it "複数の引数で指定した場合" do
      action = Goldberg::SiteAction.find_by_path("user", "delete")
      action.new_record?.should == false
      action.name.should == 'delete'
      action.controller.name = "user"
    end

    it "Hashで指定した場合" do
      action = Goldberg::SiteAction.find_by_path(:controller => "product", :action => "create")
      action.new_record?.should == false
      action.name.should == 'create'
      action.controller.name = "product"
    end
  end

  before do
    # Some associated records (two of each will do)
    (@sc1, @sc2) = Goldberg::SiteActionGroup.find(:all)
    # (@p1, @p2) = Goldberg::Permission.find(:all)
  end
  
  it "test_no_fields" do
    ca = Goldberg::SiteAction.new
    ca.valid?.should_not be_true
    # name, and site_site_action_group_id are compulsory
    [:name, :site_action_group_id].each do |attr|
      ca.errors.on(attr).should be_true
    end
  end

  it "invalid cateogy_name" do
    ca = Goldberg::SiteAction.new(:name => 'test', :category_name => 'invalid')
    ca.controller = @sc1
    # ca.save.should_not be_true
    # ca.errors.on(:category_name).should == "はindex、create、update、destroy、otherのいずれかでなければなりません。"
    ca.save.should be_true
  end

  it "test_minimal_fields" do
    ca = Goldberg::SiteAction.new(:name => 'test', :category_name_key => :index)
    ca.controller = @sc1
    ca.save.should be_true
  end

  it "test_all_fields" do
    ca = Goldberg::SiteAction.new(:name => 'test', :category_name_key => :index)
    ca.controller = @sc1
    ca.save.should be_true
  end    
  
  # The name of the controller action should be unique within the
  # scope of the site controller.
  it "test_name_unique_within_scope_of_controller" do
    ca1 = Goldberg::SiteAction.new(:name => 'test', :category_name_key => :index)
    ca2 = Goldberg::SiteAction.new(:name => 'test', :category_name_key => :index)
    ca1.controller = @sc1
    ca2.controller = @sc1
    ca1.save

    # コントローラとアクションの組み合わせは必ず一意にならなくてはなりません。
    ca2.save.should_not be_true
    ca2.errors.on(:name).should == "はすでに存在します。"

    # Should work: same name, different controllers
    ca2.controller = @sc2
    ca2.save.should be_true

    # Should work: different names, same controller
    ca2.name = 'test2'
    ca2.controller = @sc1
    ca2.save.should be_true
  end


  describe "fullname" do
    it "with controller" do
      controller_action = Goldberg::SiteAction.find_by_path("countries/index")
      controller_action.site_action_group_id.should_not be_nil
      controller_action.controller.should_not be_nil
      controller_action.fullname.should == "countries: index"
    end

    it "without controller" do
      controller_action = Goldberg::SiteAction.find_by_path("countries/index")
      controller_action.site_action_group_id = nil
      controller_action.save.should == false
      controller_action.errors.on(:site_action_group_id).should_not be_empty
      # site_action_group_idがnilになることはないので、controllerなしのfullnameの
      # 実装は必要ない。
    end
  end

  describe "url" do
    it "should return url for this action" do
      controller_action = Goldberg::SiteAction.find_by_path("countries/index")
      controller_action.site_action_group_id.should_not be_nil
      controller_action.controller.should_not be_nil
      controller_action.url.should == "/countries/index"
    end
  end

  describe "allowed_for?" do
    before(:each) do
      @action_group = Goldberg::SiteActionGroup.find_by_name("countries")
      @update_action = @action_group[:update]
    end

    it "administrator can access countries/update" do
      admin = Goldberg::Role.find(1) # システム管理者
      @update_action.allowed_for?(admin).should == true
    end

    it "normal_user can't access countries/update" do
      user = Goldberg::Role.find(3) # 一般管理者
      @update_action.allowed_for?(user).should == false
    end
  end

  describe "append_actions_from_desc" do
    it "with append_action_groups" do
      pending_if_loaded_plugin(:c0002)
      Goldberg::SiteAction.delete_all
      Goldberg::SiteActionGroup.delete_all
      Goldberg::SiteAction.count.should == 0
      Goldberg::SiteActionGroup.count.should == 0

      Goldberg::SiteAction.append_actions_from_desc

      Goldberg::SiteAction.count.should_not == 0
      Goldberg::SiteActionGroup.count.should_not == 0
    end

    it "with append_action_groups_by_controller" do
      pending_if_loaded_plugin(:c0002)
      site_action_count = Goldberg::SiteAction.count
      site_action_group_count = Goldberg::SiteActionGroup.count

      site_action_group_id = Goldberg::SiteActionGroup.find_by_name('product')
      Goldberg::SiteActionGroup.delete_all(:id => site_action_group_id)
      Goldberg::SiteAction.delete_all(:site_action_group_id => site_action_group_id)

      Goldberg::SiteAction.count.should_not == site_action_count
      Goldberg::SiteActionGroup.count.should_not == site_action_group_count

      Goldberg::SiteAction.append_actions_from_desc_by_controller(ProductController)

      Goldberg::SiteAction.count.should == site_action_count
      Goldberg::SiteActionGroup.count.should == site_action_group_count
    end
  end

  describe "short_name" do
    describe "CMSページ管理（「管理」つきのSiteActionGroup。base_nameは「CMSページ」）の場合" do
      before(:each) do
        @action_group = Goldberg::SiteActionGroup.create(
          :name => 'cms_pages', :display_name => 'CMSページ管理')
        @index = @action_group.actions.create(:name => 'index', :display_name => "CMSページ検索")
        @deploy = @action_group.actions.create(:name => 'deploy', :display_name => "ページ公開")
      end

      it "base_nameを取り除く" do
        @index.short_name.should == "検索"
        @deploy.short_name.should == "ページ公開"
      end
    end

    describe "ファイルマネージャ（「管理」なしのSiteActionGroup。base_nameは「ファイルマネージャ」）の場合" do
      before(:each) do
        @action_group = Goldberg::SiteActionGroup.create(
          :name => 'cms_real_files', :display_name => 'ファイルマネージャ')
        @index = @action_group.actions.create(:name => 'index', :display_name => "ファイル一覧")
        @create = @action_group.actions.create(:name => 'create', :display_name => "アップロード")
      end

      it "base_nameを取り除けない" do
        @index.short_name.should == "ファイル一覧"
        @create.short_name.should == "アップロード"
      end
    end

  end

end
