# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe AdminMailer do
  fixtures :mail_templates, :users, :user_addresses, :orders, :mail_deliveries, :shop_settings,
    :product_skus, :order_shippings, :order_items, :inquiries, :distributor_delivery_times, :distributor_delivery_time_displays

  before(:each) do
    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []
    @from_email = 'ec-one_from@ec-one.com'
    @to_email = 'ec-one_to@ec-one.com'
    @to_email_mobile = 'ec-one_to@docomo.ne.jp'
    @subject = 'test subject'
    @body = 'test body'
    @order = Order.find_by_id(1)
    @order_en = Order.find_by_id(12)
    @enc_test_subject_en = 'thank you.'
    @enc_test_subject_zh_CN = '衷心感谢您对我们的支持参加'
    @enc_test_subject_zh_TW = '衷心感謝您對我們的支持參加'
    @enc_test_body_en = 'thank you.(body)'
    @enc_test_body_zh_CN = '衷心感谢您对我们的支持参加(body)'
    @enc_test_body_zh_TW = '衷心感謝您對我們的支持參加(body)'
  end

  # Cancel Complete Mail
  it "cancel complete pc mail empty test" do
    cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "cancel complete pc mail size test" do
    cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "cancel complete pc mail from_address test" do
    cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email)
    cancel_complete_pc_mail.from.should == [ShopSetting[:shop_mail]]
  end

  it "cancel complete pc mail to_address test" do
    cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email)
    cancel_complete_pc_mail.to.should == [@to_email]
  end

  it "cancel complete pc mail subject test" do
    cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email)
    cancel_complete_pc_mail.subject.should == "test subject"
  end

  it "cancel complete mobile mail empty test" do
    cancel_complete_mobile_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "cancel complete mobile mail size test" do
    cancel_complete_mobile_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "cancel complete mobile mail from_address test" do
    cancel_complete_mobile_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email_mobile)
    cancel_complete_mobile_mail.from.should == [ShopSetting[:shop_mail]]
  end

  it "cancel complete mobile mail to_address test" do
    cancel_complete_mobile_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email_mobile)
    cancel_complete_mobile_mail.to.should == [@to_email_mobile]
  end

  it "cancel complete mobile mail subject test" do
    cancel_complete_mobile_mail = AdminMailer.deliver_cancel_complete(@order, @subject, @body, 'ja', @to_email_mobile)
    cancel_complete_mobile_mail.subject.should == "test subject"
  end

  # Free Mail
  it "free mail empty test" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @subject, @body, 'ja')
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "free mail size test" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, '@subjectあああ', '@bodyいいい', 'ja')
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "free mail from_address test" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @subject, @body, 'ja')
    free_mail.from.should == [@from_email]
  end

  it "free mail to_address test" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @subject, @body, 'ja')
    free_mail.to.should == [@to_email]
  end

  it "free mail subject test" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @subject, @body, 'ja')
    free_mail.subject.should == @subject
  end

  it "free mail body test" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @subject, @body, 'ja')
    free_mail.body.should == @body
  end

  it "free mail encoding test locale:en" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @enc_test_subject_en, @enc_test_body_en, 'en')
    free_mail.quoted_subject.should == '=?iso-8859-1?B?dGhhbmsgeW91Lg==?='
    free_mail.content_transfer_encoding.should == '7bit'
    free_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', @enc_test_body_en);
  end

  it "free mail encoding test locale:zh_CN" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @enc_test_subject_zh_CN, @enc_test_body_zh_CN, 'zh_CN')
    free_mail.quoted_subject.should == '=?gb2312?B?1tTQxLjQ0LvE+rbUztLDx7XE1qez1rLOvNM=?='
    free_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', @enc_test_body_zh_CN);
  end

  it "free mail encoding test locale:zh_TW" do
    free_mail = AdminMailer.deliver_free_mail(@from_email, @to_email, @enc_test_subject_zh_TW, @enc_test_body_zh_TW, 'zh_TW')
    free_mail.quoted_subject.should == '=?big5?B?sEqk37dQwcKxernvp9qtzKq6pOSr+bDRpVs=?='
    free_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', @enc_test_body_zh_TW);
  end

  # 出荷指示メール
  it "shipment mail test" do
    shipment_mail = AdminMailer.deliver_shipment_mail(@from_email, @to_email, @subject, @body, 'ja')
    ActionMailer::Base.deliveries.should_not be_empty
    ActionMailer::Base.deliveries.size.should == 1
    shipment_mail.from.should == [@from_email]
    shipment_mail.to.should == [@to_email]
    shipment_mail.subject.should == @subject
    shipment_mail.body.should == "#{@body}Attachment: attachment\n"
  end

  it "shipment mail encoding test locale:en" do
    shipment_mail = AdminMailer.deliver_shipment_mail(@from_email, @to_email, @enc_test_subject_en, @enc_test_body_en, 'en')
    shipment_mail.quoted_subject.should == '=?iso-8859-1?B?dGhhbmsgeW91Lg==?=' #添付メール形式なのでSubjectのみチェック
  end

  it "shipment mail encoding test locale:zh_CN" do
    shipment_mail = AdminMailer.deliver_shipment_mail(@from_email, @to_email, @enc_test_subject_zh_CN, @enc_test_body_zh_CN, 'zh_CN')
    shipment_mail.quoted_subject.should == '=?gb2312?B?1tTQxLjQ0LvE+rbUztLDx7XE1qez1rLOvNM=?=' #添付メール形式なのでSubjectのみチェック
  end

  it "shipment mail encoding test locale:zh_TW" do
    shipment_mail = AdminMailer.deliver_shipment_mail(@from_email, @to_email, @enc_test_subject_zh_TW, @enc_test_body_zh_TW, 'zh_TW')
    shipment_mail.quoted_subject.should == '=?big5?B?sEqk37dQwcKxernvp9qtzKq6pOSr+bDRpVs=?=' #添付メール形式なのでSubjectのみチェック
  end

  it "send_shipment_mail method check" do
    from = "send_free_mail_from@ec-one.com"
    to  = "send_free_mail_to@ec-one.com"
    subject = "send_free_mail_test-subject"
    body = "send_free_mail_test-body"
    content = "test_attachment_file_content"
    order = Order.find(1)
    AdminMailer.send_shipment_mail('9', from, to, subject, body, order.user, order, content)
    mail_delivery = MailDelivery.find(:all, :conditions => ["from_address = ?", from])
    mail_delivery.size.should == 1
  end

  it "send_free_mail method check" do
    from = "send_free_mail_from@ec-one.com"
    to  = "send_free_mail_to@ec-one.com"
    subject = "send_free_mail_test-subject"
    body = "send_free_mail_test-body"
    order = Order.find(1)
    AdminMailer.send_free_mail('99', from, to, subject, body, order.user, order, nil)
    mail_delivery = MailDelivery.find(:all, :conditions => ["from_address = ?", from])
    mail_delivery.size.should == 1
  end

  it "send_free_mail method check inquiry" do
    from = "send_free_mail_from@ec-one.com"
    to  = "send_free_mail_to@ec-one.com"
    subject = "send_free_mail_test-subject"
    body = "send_free_mail_test-body"
    order = Order.find(1)
    inquiry = Inquiry.find(1)
    AdminMailer.send_free_mail('99', from, to, subject, body, order.user, nil, inquiry)
    mail_delivery = MailDelivery.find(:all, :conditions => ["from_address = ?", from])
    mail_delivery.size.should == 1
  end


  it "send_cancel_complete_mail method check" do
    to = "send_cancel_complete_mail_to@ec-one.com"
    order = Order.find(1)
    AdminMailer.send_cancel_complete_mail(order, to)
    mail_delivery = MailDelivery.find(:all, :conditions => ["from_address = ?", ShopSetting[:shop_mail]])
    mail_delivery.size.should == 1
  end

  describe 'project_approval_mail' do
    fixtures :cms_projects, :cms_project_assigns
    before(:each) do
      CmsProjectAssign.create!(:role_key => :checker, :goldberg_user_id => 1, :cms_project_id => cms_projects(:cms_projects_00002).id)
      @cms_project = CmsProject.find(cms_projects(:cms_projects_00002).id, :include => :assigns)
    end

    it "確認依頼メール" do
      @cms_project.status_key = :requesting_check
      project_approval_mail = AdminMailer.deliver_project_approval_mail(@cms_project, '修正しました。')
      ActionMailer::Base.deliveries.should_not be_empty
      project_approval_mail.to.sort.should == ['test_admin@ec-one.com', 'test_net_admin@ec-one.com']
      project_approval_mail.from.should == ['test_webadministrator@ecrider.com']
      NKF::nkf("-w", project_approval_mail.subject).should == 'プロジェクト確認依頼'
      NKF::nkf("-w", project_approval_mail.body).should == "project2 \n確認をお願いします。\n修正しました。"
    end

    it "承認依頼メール" do
      @cms_project.status_key = :requesting_approval
      project_approval_mail = AdminMailer.deliver_project_approval_mail(@cms_project, '明日までにお願いします。')
      ActionMailer::Base.deliveries.should_not be_empty
      project_approval_mail.to.sort.should == ['test_admin@ec-one.com']
      project_approval_mail.from.should == ['test_webadministrator@ecrider.com']
      NKF::nkf("-w", project_approval_mail.subject).should == 'プロジェクト承認依頼'
      NKF::nkf("-w", project_approval_mail.body).should == "project2 \n承認をお願いします。\n明日までにお願いします。"
    end

    it "差し戻しメール" do
      @cms_project.status_key = :editing
      project_approval_mail = AdminMailer.deliver_project_approval_mail(@cms_project, '画像が間違っています。')
      ActionMailer::Base.deliveries.should_not be_empty
      project_approval_mail.to.sort.should == ['public@ec-one.com']
      project_approval_mail.from.should == ['test_webadministrator@ecrider.com']
      NKF::nkf("-w", project_approval_mail.subject).should == 'プロジェクト差し戻し'
      NKF::nkf("-w", project_approval_mail.body).should == "project2 \n修正をお願いします。\n画像が間違っています。"
    end
  end

  describe "中国宛てメール文字コードテスト" do
    it "cancel complete mail encoding test locale:en" do
      user = User.find(@order_en.user_id)
      user.user_address.name_first = 'Taro'
      user.user_address.name_last = 'Yamada'
      user.user_address.save!
      cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order_en, @enc_test_subject_en, @enc_test_body_en, 'en', @to_email)
      cancel_complete_pc_mail.quoted_subject.should == '=?iso-8859-1?B?dGhhbmsgeW91Lg==?='
      cancel_complete_pc_mail.content_transfer_encoding.should == '7bit'
      cancel_complete_pc_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', @enc_test_body_en);
    end

    it "cancel complete mail encoding test locale:zh_CN" do
      @order_en.locale = 'zh_CN'
      user = User.find(@order_en.user_id)
      user.user_address.name_first = 'Taro'
      user.user_address.name_last = 'Yamada'
      user.user_address.save!
      cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order_en, @enc_test_subject_zh_CN, @enc_test_body_zh_CN, 'zh_CN', @to_email)
      cancel_complete_pc_mail.quoted_subject.should == '=?gb2312?B?1tTQxLjQ0LvE+rbUztLDx7XE1qez1rLOvNM=?='
      cancel_complete_pc_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', @enc_test_body_zh_CN);
    end

    it "cancel complete mail encoding test locale:zh_TW" do
      @order_en.locale = 'zh_TW'
      user = User.find(@order_en.user_id)
      user.user_address.name_first = 'Taro'
      user.user_address.name_last = 'Yamada'
      user.user_address.save!
      cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order_en, @enc_test_subject_zh_TW, @enc_test_body_zh_TW, 'zh_TW', @to_email)
      cancel_complete_pc_mail.quoted_subject.should == '=?big5?B?sEqk37dQwcKxernvp9qtzKq6pOSr+bDRpVs=?='
      cancel_complete_pc_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', @enc_test_body_zh_TW);
    end

    it "cancel complete  mail subject encoding error retry test(utf-8)" do
      #メール内容によって変わるような処理でないためこのメール送信処理でのみ実施
      @order_en.locale = 'zh_CN'
      user = User.find(@order_en.user_id)
      user.user_address.name_first = 'Taro'
      user.user_address.name_last = 'Yamada'
      user.user_address.save!
      #文字コード変換に失敗するようにテンプレートのロケールを入れ替える
      zhcn_template = MailTemplate.find(30)
      zhcn_template.locale = 'zh_TW'
      zhcn_template.save!
      zhtw_template = MailTemplate.find(41)
      zhtw_template.locale = 'zh_CN'
      zhtw_template.save!
      #タイトルの文字コードがロケールと異なる場合
      cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order_en, @enc_test_subject_zh_CN, @enc_test_body_zh_CN, 'zh-TW', @to_email)
      #zh_CNのSubjectエンコードに失敗してUTF-8で設定されていること
      cancel_complete_pc_mail.quoted_subject.should == "=?utf-8?B?6KG35b+D5oSf6LCi5oKo5a+55oiR5Lus55qE5pSv5oyB5Y+C5Yqg?="
      cancel_complete_pc_mail.quoted_body.should == @enc_test_body_zh_CN
    end

    it "cancel complete  mail body encoding error retry test(utf-8)" do
      #メール内容によって変わるような処理でないためこのメール送信処理でのみ実施
      @order_en.locale = 'zh_CN'
      user = User.find(@order_en.user_id)
      user.user_address.name_first = 'Taro'
      user.user_address.name_last = 'Yamada'
      user.user_address.save!
      cancel_complete_pc_mail = AdminMailer.deliver_cancel_complete(@order_en, @enc_test_subject_zh_CN, @enc_test_body_zh_TW, 'zh-CN', @to_email)
      #zh_CNの本文エンコードに失敗してUTF-8で設定されていること
      cancel_complete_pc_mail.quoted_subject.should == "=?utf-8?B?6KG35b+D5oSf6LCi5oKo5a+55oiR5Lus55qE5pSv5oyB5Y+C5Yqg?="
      cancel_complete_pc_mail.quoted_body.should == @enc_test_body_zh_TW
    end

  end
end
