module DirFixtures
  class Definition
    attr_accessor :root_path

    def initialize(root_path)
      @root_path = root_path
    end

    def target(value = nil)
      @target = value unless value.nil?
      @target
    end
    
    def backup_cache(value = nil)
      @backup_cache = value unless value.nil?
      @backup_cache
    end

    def backup_dir(value = nil)
      @backup_dir = value unless value.nil?
      @backup_dir
    end

    def protected_paths
      @protected_paths ||= []
    end

    def protect(*paths)
      dest = protected_paths
      paths.flatten.each{|path| dest << path}
      dest
    end

    def after_initialize
      cleared_dir = self.backup_dir
      if self.backup_cache && cleared_dir && !cleared_dir.empty?
        FileUtils.rm_rf(cleared_dir, :verbose => true) if File.exist?(cleared_dir)
        FileUtils.mkdir_p(cleared_dir, :verbose => true)
      end
    end

    def process(fixture_name, &block)
      Backup.transaction(target, :cache => backup_cache, :dir => backup_dir) do
        Fixture.setup(target, File.join(root_path, fixture_name), :protected_paths => protected_paths)
        yield
      end
    end

  end
end
