# -*- coding: utf-8 -*-
class InquiryController < ApplicationController
  include EcRider::SetDefaultLayout
  include EcRider::PluginModule::Extendable

  #
  # お問合せ画面表示
  #
  def index
    # バリデーションエラーで戻った場合
    if flash[:inquiry]
      @inquiry = flash[:inquiry]
      @inquiry_product = session[:inquiry_product]
    # 戻るボタンで戻った場合
    elsif session[:inquiry]
      @inquiry = session[:inquiry]
      @inquiry_product = session[:inquiry_product]
    # 初期表示
    else
      @inquiry = Inquiry.new(:country_code => 'JP')

      # 商品コードがわたってきた場合、問合せ区分を｢商品お問合せ｣にする
      if params[:product_code]
        @inquiry_product = Product.find(:first, :conditions => ["product_code = ?", params[:product_code]])
        session[:inquiry_product] = @inquiry_product
        @inquiry.inquiry_div_key = :product
      end

      if logged_in?
        @inquiry.email= get_email
        @inquiry.merge_only_need(self.current_user.user_address.attributes)
      end
    end

    render :action => 'inquiry'
  end

  def inquiry
    redirect_to :action => 'index'
  end

  #
  # お問合せ確認画面表示
  #
  def confirm
    return redirect_to(:action => 'index') unless params[:inquiry]

    @inquiry = Inquiry.new(params[:inquiry])
    @inquiry.user = self.current_user if logged_in?
    @inquiry.country_code ||= (logged_in? ? self.current_user.user_address.country_code : 'JP')
    @inquiry.locale = I18n.locale.to_s

    # ファイルが送られてくるため、別インスタンスに保存する
    tmp_image_dir = params[:tmp_image_dir].nil? ? "#{DateTime.now.strftime("%d%H%M%S")}#{@inquiry.object_id}" : params[:tmp_image_dir]
    tmp_image_dir_path = "#{EcRider.config.tmp_inquiry_images_dir}/#{tmp_image_dir}"
    @tmp_images = {:tmp_image_dir => tmp_image_dir}
    Dir.mkdir(tmp_image_dir_path) unless File.exists?(tmp_image_dir_path)

    (1..5).each do |index|
      base_name = "image_url_#{index}".to_sym
      next unless params[:inquiry][base_name]

      @tmp_images[base_name] = params[:inquiry][base_name]
      tmp_image_filename = "#{index}_#{@tmp_images[base_name].original_filename}"
      File.open("#{tmp_image_dir_path}/#{tmp_image_filename}", 'wb', 9999) do |of|
        of.write(@tmp_images[base_name].read)
      end
      @inquiry.send("#{base_name}=", tmp_image_filename)
    end

    #このときにファイルサイズのvalidが実施されます。
    unless @inquiry.valid? && Inquiry.file_validate(@tmp_images)
      redirect_to :action=> :index
      flash[:inquiry] = @inquiry
      return
    end

    @inquiry_product = session[:inquiry_product]
    session[:inquiry] = @inquiry
  end


  #
  # お問合せ登録
  #
  def update_information
    return redirect_to(:action => 'index') unless session[:inquiry]

    inquiry = session[:inquiry]
    inquiry.status_div_key = :not_yet

    # 会員の場合は会員番号も登録する
    inquiry.user_id = session[:user] if session[:user]

    Inquiry.transaction do
      inquiry.save!
      # 商品お問合せの場合は対象商品テーブルに登録する
      if inquiry.inquiry_div_key == :product && !session[:inquiry_product].nil?
        inquiry_product = InquiryProduct.new()
        inquiry_product.inquiry_id = inquiry.id
        inquiry_product.product_id = session[:inquiry_product].id
        inquiry_product.save!
      end
      # public/temp_inquiry_images/inquiryテーブルidのディレクトリを作成
      # 上記のディレクトリに、tmp_inquiry_imagesへ保存されたファイルすべてを保存します。
      # 最大5つのファイルが保存されます。
      unless params[:tmp_image_dir].nil? || !File.exist?("#{EcRider.config.tmp_inquiry_images_dir}/#{params[:tmp_image_dir]}")
        FileUtils.move("#{EcRider.config.tmp_inquiry_images_dir}/#{params[:tmp_image_dir]}/", "#{EcRider.config.ec_admin_root}/public/ec_rider/inquiry_images/#{inquiry.id}/")
        FileUtils.rm_f("#{EcRider.config.tmp_inquiry_images_dir}/#{params[:tmp_image_dir]}")
      end
      
      # 問合せ受付メール送信
      FrontSiteMailer.send_notice_inquiry_mail(inquiry)
      @inquiry_product = session[:inquiry_product]
      session[:inquiry] = nil
      session[:inquiry_product] = nil
    end

    render :action => :completion
  end

  #
  # お問合せ完了画面表示
  #
  def completion

  end

  def get_email
    return nil unless self.current_user.user_email
    self.current_user.user_email.email
  end

end


