# -*- coding: utf-8 -*-
require 'uri'

class LocaleController < ApplicationController
  
  def update
    lang = params[:id].gsub('-', '_')
    cookies["lang"] = lang
    uri = URI.parse(request.env["HTTP_REFERER"] || '/')
    if has_locale?(uri)
      uri.path.sub!(@regexp, "/#{lang}")
    else
      uri.path = "/#{lang}#{uri.path}"
    end
    respond_to do |format|
      format.html { redirect_to(uri.to_s) }
    end
  end

  def destroy
    cookies["lang"] = nil
    uri = URI.parse(request.env["HTTP_REFERER"] || '/')
    uri.path.sub!(@regexp, '') if has_locale?(uri)
    respond_to do |format|
      format.html { redirect_to(uri.to_s) }
    end
  end

  protected
  
  def has_locale?(uri)
    @regexp ||= Regexp.union(*I18n.supported_locales.map{|locale| /^\/#{locale.to_s}/i })
    !!(@regexp =~ uri.path)
  end


  # このlocaleコントローラだけは、処理を行っても不正な処理が動く訳ではないので、
  # セッションの form_authenticity_tokenとパラメータのトークンが一致しなくても処理を行うようにします。
  def protect_against_forgery?
    false
  end
end
