# -*- coding: utf-8 -*-

class MemberController < ApplicationController
  include EcRider::SetDefaultLayout
  include EcRider::PluginModule::Extendable

  before_filter :check_go_to_front

  #
  # 会員専用ページ
  #
  def index
    session[:address] = nil
  end

  #
  # 購入履歴
  #
  def history
    @user_order_error = flash[:user_order_error] || nil
    history_ready(params[:page])
  end

  #
  # 注文詳細
  #
  def history_order_detail
    return redirect_to(:action => 'history') unless params[:id]

    _order = Order.find(:all, :conditions => ["id = ?", [params[:id]]])

    redirect_to_404 unless _order.size == 1
    if current_user.users_data?("order", params[:id])
      flash[:user_order_error] = true
      return redirect_to :action => :history
    end

    @order = _order[0]
    @page_no = params[:page]
    @shipping_cost = 0

    @order.order_shippings.each do |order_shipping|
      @shipping_cost += order_shipping.shipping_cost
      @order_items = OrderItem.find(:all, :conditions => ["orders.id = ?",@order], :with_deleted => "true",
        :select=> "product_name,course_div,unit_price, sum(qty) AS qty",
        :joins=>  "JOIN order_shippings ON order_shippings.id = order_items.order_shipping_id
                   JOIN orders ON orders.id = order_shippings.order_id",
        :group => "product_name,unit_price, course_div")
    end
  end

  private
  def history_ready(page)
    @ques = params[:ques].blank? ? '10' : params[:ques]
    @user = self.current_user
      
    find_conditions = Order::SearchInfo.new(:user_id => @user.id, :successful_only_flg => true)
    @orders = Order.paginate(
      :per_page => @ques.to_i, 
      :page => page, 
      :conditions => find_conditions.create_condition, 
      :include => [{:order_shippings => :order_items}], 
      :order => 'orders.id DESC')
  end

  public
  #
  # = キャンセルの確認 =
  #
  def confirm_cancel
    @order = Order.find(params[:id])
  end

  #
  # キャンセル依頼
  #
  def request_cancel
    # セキュリティ対策
    # パラメータ改ざんされ、別会員の注文キャンセルが実行できないように自分の注文した注文IDかをチェックします。
    if current_user.users_data?("order", params[:id])
      flash[:user_order_error] = true
      return redirect_to :action => :history
    end
    order = Order.find(params[:id], :include => :order_shippings)
    order_shipping = order.order_shippings.first
    order_shipping.order = order
    order_shipping.request_cancel
    history_ready(params[:page])
    redirect_to :action => :history
  end

  #
  # 配送先追加
  #
  def show_addresses
    session[:change_address] = nil
    @error_flg = flash[:address_error] || nil
    @user_address = flash[:address]
    @user_address ||= session[:address]
    @user_address ||= UserAddress.new(:country_code => 'JP')
    @shipping_addresses = current_user.user_and_shipping_addresses
    @user_profile = current_user
  end

  #
  # 配送先追加確認
  #
  def confirm_address
    return redirect_to(:show_addresses) unless request.post?

    @user_address = UserAddress.new(params[:user_address])
    @user_address.locale = I18n.locale.to_s
    unless @user_address.valid?
      flash[:error] = @user_address.errors.zip
      flash[:address] = @user_address
      redirect_back_or_default(:action => :show_addresses)
      return
    end
    session[:address] = @user_address
    @user_profile = current_user
  end

  #
  # 配送先編集
  #
  def change_address
    return redirect_to(:action => 'show_addresses') unless params[:id]
    # セキュリティ対策
    # パラメータ改ざんされ、別会員の配送先住所が閲覧できないように自分の登録した配送先情報かをチェックします。
    if current_user.users_data?("user_address", params[:id])
      flash[:address_error] = true
      return redirect_to :action => :show_addresses
    end
    if flash[:user_address]
      # 戻るまたは、バリデーションエラーで遷移してきたケース
      @user_address = flash[:user_address]
    else
      @user_address = session[:change_address]
      @user_address ||= UserAddress.find(params[:id])
      session[:change_address] = @user_address
    end
    @user_profile = current_user
  end

  #
  # 配送先編集確認
  #
  def confirm_modify_change
    return redirect_to(:action => 'show_addresses') unless request.post?

    address_form = session[:change_address]
    address_form.merge_only_need(params[:user_address])
    unless address_form.valid?
      flash[:address] = address_form
      flash[:error] = address_form.errors.zip
      redirect_back_or_default(:action => :change_address, :id => address_form.id)
      return
    end
    session[:change_address] = address_form
    @user_address =  address_form
  end

  #
  # 配送先確認
  #
  def confirm_shipping
    return redirect_to(:action => 'show_addresses') unless request.post?

    if session[:change_address]
      confirm_modify_change
      render :action => 'confirm_modify_change' unless performed?
    else
      confirm_address
      render :action => 'confirm_address' unless performed?
    end

  end

  #
  # 配送先登録
  #
  def update_shipping
    return redirect_to(:action => 'show_addresses') unless request.post? || request.put?

    #追加処理
    if session[:address]
      address = session[:address]
      address.user_id= session[:user]
      address.address_div_key = :ship_to
      address.locale = I18n.locale.to_s
      address.save!
    end
    session[:address] = nil

    #変更処理
    if session[:change_address]
      address = session[:change_address]
      address.save!
      # メイン配送先を更新する場合、ユーザー情報も更新する
      if address.address_div_key == :user
        user = self.current_user
        user.mirror_from_address(address)
        user.save!
        if user.sub_address
          user.sub_address.set_sub_address_from_address(address)
          user.sub_address.save!
        end
      end
    end
    session[:change_address] = nil
    
    redirect_to(:action => :show_addresses)
  end

  #
  # 配送先削除
  #
  def delete_address
    return redirect_to(:action => 'show_addresses') unless params[:user_address][:id]
    # セキュリティ対策
    # パラメータ改ざんされ、別会員の配送先住所が閲覧できないように自分の登録した配送先情報かをチェックします。
    if current_user.users_data?("user_address", params[:user_address][:id])
      flash[:address_error] = true
      return redirect_to :action => :show_addresses
    end

    UserAddress.find(params[:user_address][:id]).destroy
    redirect_to(:action => :show_addresses)
  end

  #
  # 領収書
  #
  def print_receipt
    return redirect_to(:action => 'history') unless params[:id]
    # セキュリティ対策
    # パラメータ改ざんされ、別会員の配送先住所が閲覧できないように自分の登録した配送先情報かをチェックします。
    # パラメータにlangがある場合（管理サイトからのアクセスの場合）は
    if params[:from_admin].nil? && current_user.users_data?("order", params[:id])
      flash[:user_order_error] = true
      return redirect_to :action => :history
    end
    lang = params[:lang] unless params[:lang].nil?
    lang ||= (I18n.locale || I18n.default_locale.to_s).to_s
    @order = Order.find(params[:id], :include => [{:order_shippings => :order_items}])
    @orders = @order
    @order_shipping = @orders['order_shipping']
    
    unless params[:addressee_form].nil?
      @addressee = params[:addressee_form][:addressee]
    else
      @addressee = ""
    end
    render :layout => false
  end

  # ポイント履歴
  def point_hitory
  end

  protected
  #
  # セッションチェック
  #
  def render_error_if_invalid_session
    error_text = <<-"EOF"
      <html>
        <head>
          <title>403 Forbidden</title>
        </head>
        <body>
          <h1>403 Forbidden</h1>
          <p>
            Session validation failed.
          </p>
        </body>
      </html>
    EOF
    if params[:session_id_validation] == security_token
    return true
  else
    render(:text => error_text, :status => "403 Forbidden")
    return false
  end
end
end

