# -*- coding: utf-8 -*-
class OrderController < ApplicationController
  include EcRider::SetDefaultLayout
  include EcRider::PluginModule::Extendable

  before_filter :cart_in?, :except =>[:create, :not_user_info_error, :register_order_error, :re_order_error, :render_order_complete]
  before_filter :redirect_to_user_error_if_invalid, :except => [:agreement, :confirmation_cart, :payment, :not_user_info_error, :register_order_error, :re_order_error]

  #
  # 個人情報規約
  #
  def agreement
    clear_sessions
  end

  #
  # 個人情報登録
  #
  def confirmation_cart
    return redirect_to(:action => 'index') if session[:user]

    # バリデーションエラーで戻った場合
    if flash[:user_profile] || flash[:user_address] || flash[:user_address] || flash[:user_en_address]
      @user_profile = flash[:user_profile]
      @user_address = flash[:user_address]
      @user_email = flash[:user_email]
      @user_en_address = flash[:user_en_address]
    # 戻るボタンで戻った場合
    elsif session[:cart_tmp_user]
      @user_profile =   session[:cart_tmp_user]
      @user_address = session[:cart_tmp_user].user_address
      @user_email = session[:cart_tmp_user].user_email
      @user_en_address = session[:cart_tmp_user].sub_address
    # 初期表示
    else
      @user_profile = User.new(:sex_key => :male, :dm_send_div_key => :prefer)
      @user_address = UserAddress.new(:country_code => 'JP')
      @user_email = UserEmail.new
      @user_en_address = UserAddress.new(:country_code => 'JP')
    end
    @user_profile.not_decided_address = true
  end

  #
  # = 初期表示 =
  #
  def index
    session[:order_form] = nil
    redirect_to :action => 'payment'
  end

  #
  # = お届け先指定・お支払方法 =
  #
  # 事前条件：カートに一つ以上の商品が入っていること
  def payment
    # 会員情報入力画面から遷移してきた場合
    if params[:user_profile]
      user = User.new(params[:user_profile])
      user.remember_question_check = !user.password.blank?
      user.member_div_key = user.password.blank? ? :no_membership : :membership #メールアドレス重複チェックで使用する
      user_address = UserAddress.new(params[:user_address])
      user_address.locale = I18n.locale.to_s
      user_email = UserEmail.new(params[:user_email])
      user_email.user = user
      user.user_email = user_email
      user.mirror_from_address(user_address)

      # 新規の為、ユーザーIDなしを渡す(この処理はメールアドレス重複チェックに必要です。user_email.userの値を使用するため)
      user_email.user = user

      user.valid?
      user_address.valid?
      user_email.valid?

      if sub_address = UserAddress.hash_to_sub_address(params[:user_en_address])
        sub_address.set_sub_address_from_address(user_address)
        sub_address.valid?
      end

      error_count = user.errors.count + user_address.errors.count + user_email.errors.count
      error_count += sub_address.errors.count if sub_address

      if error_count > 0
        flash[:user_profile] = user
        flash[:user_address] = user_address
        flash[:user_email] = user_email
        flash[:user_en_address] = sub_address
        redirect_back_or_default :action => :confirmation_cart
        return
      end
      session[:cart_tmp_user] = User.new_user(params[:user_profile], params[:user_address], params[:user_email], params[:user_en_address])
    end

    @user = get_user
    # 配送先リストを取得する
    @shipping_addresses = session[:shipping_addresses]
    if @shipping_addresses.nil?
      @shipping_addresses = session[:user] ? current_user.user_and_shipping_addresses : session[:cart_tmp_user].user_and_shipping_addresses
    else
      @shipping_addresses[0] = session[:cart_tmp_user].user_and_shipping_addresses[0] unless session[:user]
    end
    session[:shipping_addresses] = @shipping_addresses

    # 同送先リストを取得します
    @included_shipping_addresses = OrderShipping.find_includable_shipping(current_user.id)
    session[:included_shipping_addresses] = @included_shipping_addresses

    # フォーム情報を取得
    @order_form = flash[:cart_send_info] || session[:order_form] || OrderForm.new(:send_address_index => 0)
    session[:order_form] = @order_form

    # 配送先入力フォームに必要なインスタンスを作成する
    @user_profile = @user
    @user_email = user_email
    @user_address = UserAddress.new(:country_code => 'JP')
    @user_en_address = UserAddress.new(:country_code => 'JP')

    # カートを再計算(カート内で適用できるキャンペーンを取得するために実施します。)
    session[:cart].calculate
  end

  #
  # 注文内容確認
  #
  def confirmation_order
    order_form = nil
    @stock_error = nil
    @user = get_user
    @user_address = @user.user_address
    @user_email = get_user_email
    
    # 注文情報登録処理からプレゼントの在庫エラーで戻った場合
    if flash[:stock_error]
      @stock_error = flash[:stock_error]
      order_form = session[:order_form]
      session[:cart].delete_out_of_stock_present
    elsif flash[:sales_period_error]
      @sales_period_error = flash[:sales_period_error]
      order_form = session[:order_form]
      session[:cart].delete_nonperiodic_sale_present
      
       # 支払方法選択画面からの遷移の場合
    else
      # 不正な画面遷移
      unless params[:order_form]
        return redirect_to(:controller => 'order', :action => 'not_user_info_error') unless performed?
      end
      order_form = OrderForm.new(params[:order_form])
      total_point = UserPoint.total_point_for_user(current_user.id)
      order_form.before_total_point = total_point
      order_form.after_total_point = total_point - order_form.use_point.to_i + session[:cart].total_given_point.to_i
      # 支払方法入力のバリデーションチェック
      if params[:order_form]
        flash[:cart_send_info] = order_form
        return redirect_to(:action => :payment) unless order_form.valid?
      end
      session[:order_form] = session[:cart_send_info] = order_form
    end

    # 配送先住所を取得する
    address = session[:shipping_addresses][order_form.send_address_index.to_i]
    # 同送指定された場合、同送指定可能な住所一覧から住所を設定する
    unless order_form.include_order_id.blank?
      session[:included_shipping_addresses].each do |order_shipping|
        address = order_shipping if order_shipping.order_id == order_form.include_order_id.to_i
      end
    end

    @order = get_order(@user, address, order_form, session[:cart])
    # コンビニ決済時の金額チェック
    order_shipping = @order.order_shippings.first
    total_price = order_shipping.total_amount + order_shipping.shipping_cost
    return redirect_to(:action => :payment) unless order_form.total_price_valid?(total_price)
    session[:cart_tmp_order] = @order
  end

  # モバイルプラグインで上書きされます。
  def get_user_email
    @user.user_email
  end

  # モバイルプラグインで上書きされます。
  def get_order(user, address, order_form, cart)
    Order.create_new_order(user, address, order_form, cart)
  end

  #
  # 注文
  #
  def create
    session[:settlement_error] = nil

    return redirect_to(:controller => 'order', :action => 'not_user_info_error') unless request.post?
    return redirect_to(:controller => 'order', :action => 're_order_error') unless session[:cart_tmp_order]
    return unless session[:cart]

    # TODO alipay決済修正時にプラグインに移動してください。
    if session[:cart_send_info] && session[:cart_send_info].normal_payment_type == OrderShipping.payment_type_div_id_by_key(:alipay)
      flash[:from_order_create] = true
      redirect_to :controller => 'sps_payment', :action => 'start_register'
      return
    end

    order = session[:cart_tmp_order]
    order_form = session[:cart_send_info]

    begin
      # 注文情報の登録後、ログインアカウントを返却する
      email, password = create_order(order)

      order_shipping = order.order_shippings.first
      order_shipping.order = order # この代入が重要です。代入しないとorder_shippingから参照するorderが不定になってしまいます。

      # Thanksメール送信用の引数
      order_shipping.send_thanks_mail_args = [session[:cart_tmp_user] ? @user : current_user, !!session[:user]]

      # 各決済サービスに決済要求
      order_shipping.payment_arg = order_form, @user
      # order_shipping.credit_card_payment if order_shipping.payment_type_div_key == :credit

      # 例外発生時に保存する非会員の情報を予め設定しておきます。
      order_shipping.save_user_as_no_membership_target = @user unless logged_in?

      # ポイント用の引数
      order_shipping.given_point = session[:cart].total_given_point if session[:cart]
      order_shipping.used_point = session[:order_form].use_point if session[:order_form]

      context = order_shipping.process_status_div

      case order_shipping.status_div_key
      when :settlement_failed then
        # クレジット処理結果がエラーの場合
        # order.update_status(:settlement_failed)
        # @user.save_as_no_membership unless logged_in?
        session[:settlement_error] = context.exceptions.last.message
        redirect_to :action => :payment
        return
      when :sales_period_error then
        # 販売期間外の商品がある場合
        nonperiod_item = OrderItem.find(context.exceptions.last.order_item_id)
        if nonperiod_item.course_div_key == :present
          flash[:sales_period_error] = context.exceptions.last.product_name
          redirect_to :controller => 'order', :action => 'confirmation_order'

        else
          flash[:sales_period_error] = context.exceptions.last.message
          redirect_to :controller => 'cart', :action => 'index'
        end
        return
      when :stock_error then
        # 在庫数が不足している場合
        # order.update_status(:stock_error)
        # OrderItem.update(stock_error.order_item_id, :out_of_stock_flg => true)
        # @user.save_as_no_membership unless logged_in?

        # プレゼント品が在庫不足の場合は注文確認画面へ
        out_of_stock_item = OrderItem.find(context.exceptions.last.order_item_id)
        if out_of_stock_item.course_div_key == :present
          flash[:stock_error] = context.exceptions.last.product_name
          redirect_to :controller => 'order', :action => 'confirmation_order'

          # 注文商品が在庫不足の場合はカートへ
        else
          flash[:stock_error] = context.exceptions.last.message
          redirect_to :controller => 'cart', :action => 'index'
        end
        return
      when :internal_error then
        # その他の例外
        # order.update_status(:internal_error)
        # @user.save_as_no_membership unless logged_in?
        clear_sessions
        session[:cart] = nil
        deliver_exception_notification(context.exceptions.last)
        redirect_to :controller => 'order', :action => 'register_order_error'
        return
      else
        # 上記以外の場合
        flash[:order_id] = order.id
        flash[:user_code] = @user ? @user.user_code : session[:cart_tmp_user].user_code
        flash[:sex] = @user ? @user.sex : session[:cart_tmp_user].sex

        # すでに購入が済んでいるケース(アンケート画面からの戻るボタンで戻ってきたケースなどは、フラッシュから取得)
        order_id = order ? order.id : flash[:order_id]

        unless logged_in?
          # アンケート画面へ遷移
          self.current_user = User.authenticate(email, password)
          reset_session_with_keep_content # セッションを作り直しています。中身は一緒
        end
        # 遷移先をプラグイン拡張しています。
        render_order_complete(order)
      end
    rescue Exception
      logger.fatal("ここにくることはないはずです。")
      raise
    end
  end

  # 注文作成後の遷移先をrenderします。
  # プラグインで拡張されています
  private
  def render_order_complete(order = nil)
    clear_sessions
    session[:cart] = nil
    session[:order_form] = nil
    render 'thanks/index'
  end
  public
  
  def create_order(order)
    Order.transaction do
      order_user = get_user
      if logged_in?
        @user = order_user
      else
        @user = order_user.clone(:include => [:user_addresses, :user_email, :user_email_mobile])
        @user.password = order_user.password
        @user.password_confirmation = order_user.password_confirmation
        @user.remember_question_check = false
      end
    
      # 会員の場合
      if session[:user]
        order.user_id = session[:user]
        order.save!
        @user.save!
        if session[:shipping_addresses]
          session[:shipping_addresses].each do |address|
            address.user_id = @user.id
            address.save!
          end
        end
        @user.sub_address.save! if @user.sub_address
        after_create_order_for_member(order)
        # 非会員の場合
      else
        @user.orders << order
        @user.save!
  
        session[:shipping_addresses].each_with_index do |address, index|
          unless index == 0 #会員自身の住所は@user.save!で登録済み。
            address.user_id = @user.id
            address.save!
          end
        end
        after_create_order_for_non_member(order)
      end
    end

    #ログインするための情報を取得
    email ||= order.site_type_div_key == :pc ? @user.user_email.email : @user.user_email_mobile.email
    password ||= @user.password
    
    return email, password
  end

  # 会員が注文した後に呼び出されます
  # プラグインで拡張するためのメソッド
  def after_create_order_for_member(order)
  end
  
  # 非会員が注文した後に呼び出されます
  # プラグインで拡張するためのメソッド
  def after_create_order_for_non_member(order)
  end
  

  #
  # = 新規配送先の追加フォームを表示します。 =
  #
  def new_shipping_address
    @user_address = UserAddress.new(:country_code => 'JP')

    render :update do |page|
      # 配送先入力フォームをクリアします
      page[:user_address_name_first].value = ''
      page[:user_address_name_last].value = ''
      if I18n.locale.to_s == 'ja'
        page[:user_address_kana_name_first].value = ''
        page[:user_address_kana_name_last].value = ''
      end
      page[:user_address_country_code].value = 'JP'
      page[:user_address_zip_code].value = ''
      page[:user_address_address_1].value = ''
      page[:user_address_address_2].value = ''
      if I18n.locale.to_s == 'ja'
        page[:user_address_address_3].value = ''
      end
      page[:user_address_tel_num].value = ''

      # アクションボタンなど
      page.select('.visible_when_new').each{|element| element.show}
      page.select('.visible_when_edit').each{|element| element.hide}

      # エラーメッセージの更新
      page[:shipping_address_error_messages].replace_html ''
      # 配送先入力フォームの表示
      page[:shipping_address_area].visual_effect(:appear)

      # 日本語の場合は住所入力フォームの要素を指定して値をセットします
      if I18n.locale.to_s == 'ja'

        js_func = <<-EOF
          function update_area_and_inputs(class_name, enabled) {
            if (enabled){
              $$("." + class_name).each(function(area){area.show();});
              $$("." + class_name + " input").each(function(input){input.enable();});
              $$("." + class_name + " select").each(function(select){select.enable();});
            }else{
              $$("." + class_name).each(function(area){area.hide();});
              $$("." + class_name + " input").each(function(input){input.disable();if(input.type!="button"){input.value = "";}});
              $$("." + class_name + " select").each(function(select){select.disable();});
            }
          }
          function update_addresses_by_country(){
            if ($("user_address_country_code").value == "JP"){
              update_area_and_inputs("main_address_country_pane_JP", true);
              update_area_and_inputs("main_address_country_pane_others", false);
            }else{
              update_area_and_inputs("main_address_country_pane_JP", false);
              update_area_and_inputs("main_address_country_pane_others", true);
            }
          }
        EOF

        page<< js_func
        page.call("update_addresses_by_country")
      end
    end
  end

  #
  # 配送先を新規追加します
  #
  def create_shipping_address
    # カートを再計算(カート内で適用できるキャンペーンを取得するために実施します。)
    session[:cart].calculate

    # 画面の配送先リストを取得する
    @shipping_addresses = session[:shipping_addresses]

    # 新しい配送先を作成する
    @shipping_address = UserAddress.new()
    @shipping_address.merge_only_need(params[:user_address])
    @shipping_address.address_div_key = :ship_to
    @shipping_address.locale = I18n.locale.to_s

    # 入力チェック
    unless  @shipping_address.valid?
      render_error_messages_for(@shipping_address)
      return
    end

    # 配送先リストに追加
    session[:shipping_addresses] << @shipping_address
    @shipping_addresses = session[:shipping_addresses]
    index = session[:shipping_addresses].length - 1

    response_text = get_update_shipping_list_script(index)
    render(:inline => response_text)
  end

  #
  # 配送先更新フォームを表示します。
  #
  def edit_shipping_address
    index = params[:id].to_i

    user_address = session[:shipping_addresses][index]
    render :update do |page|
      # 配送先入力フォームに、ここで編集データをセットします
      page[:user_address_name_first].value = user_address.name_first
      page[:user_address_name_last].value = user_address.name_last

      if I18n.locale.to_s == 'ja'
        page[:user_address_kana_name_first].value = user_address.kana_name_first
        page[:user_address_kana_name_last].value = user_address.kana_name_last
      end
      page[:user_address_country_code].value = user_address.country_code
      page[:user_address_zip_code].value = user_address.zip_code
      page[:user_address_address_1].value = user_address.address_1
      page[:user_address_address_2].value = user_address.address_2
      page[:user_address_address_3].value = user_address.address_3
      page[:user_address_tel_num].value = user_address.tel_num

      # 編集対象の配送先インデックスをhiddenに設定する
      page[:edit_target_index].value = index

      # アクションボタンの表示
      page.select('.visible_when_new').each{|element| element.hide}
      page.select('.visible_when_edit').each{|element| element.show}

      # エラーメッセージの更新
      page[:shipping_address_error_messages].replace_html ''
      # 配送先入力フォームの表示
      page[:shipping_address_area].visual_effect(:appear)

      # 日本語の場合は住所入力フォームの要素を指定して値をセットします
      if I18n.locale.to_s == 'ja'

        js_func = <<-EOF
          var country_select = $("user_address_country_code");
          function update_area_and_inputs(class_name, enabled) {
            if (enabled){
              $$("." + class_name).each(function(area){area.show();});
              $$("." + class_name + " input").each(function(input){input.enable();});
              $$("." + class_name + " select").each(function(select){select.enable();});
              $$("." + class_name + " #user_address_name_first").each(function(element){element.value = "#{user_address.name_first}";});
              $$("." + class_name + " #user_address_name_last").each(function(element){element.value = "#{user_address.name_last}";});
              $$("." + class_name + " #user_address_kana_name_first").each(function(element){element.value = "#{user_address.kana_name_first}";});
              $$("." + class_name + " #user_address_kana_name_last").each(function(element){element.value = "#{user_address.kana_name_last}";});
              $$("." + class_name + " #user_address_zip_code").each(function(element){element.value = "#{user_address.zip_code}";});
              $$("." + class_name + " #user_address_address_1").each(function(element){element.value = "#{user_address.address_1}";});
              $$("." + class_name + " #user_address_address_2").each(function(element){element.value = "#{user_address.address_2}";});
              $$("." + class_name + " #user_address_address_3").each(function(element){element.value = "#{user_address.address_3}";});
              $$("." + class_name + " #user_address_tel_num").each(function(element){element.value = "#{user_address.tel_num}";});
            }else{
              $$("." + class_name).each(function(area){area.hide();});
              $$("." + class_name + " input").each(function(input){input.disable();if(input.type!="button"){input.value = "";}});
              $$("." + class_name + " select").each(function(select){select.disable();});
            }
          }
          function update_addresses_by_country(){
            if (country_select.value == "JP"){
              update_area_and_inputs("main_address_country_pane_JP", true);
              update_area_and_inputs("main_address_country_pane_others", false);
            }else{
              update_area_and_inputs("main_address_country_pane_JP", false);
              update_area_and_inputs("main_address_country_pane_others", true);
            }
          }
        EOF

        page<< js_func
        page.call("update_addresses_by_country")
      end
    end
  end


  #
  # 配送先を更新します。
  #
  def update_shipping_address
    user = get_user
    index = params[:edit_target_index].to_i

    @shipping_address = session[:shipping_addresses][index]
    @shipping_address.merge_only_need(params[:user_address])
    @shipping_address.country.reload #変更された国コードで国情報を再取得する

    # 入力チェック
    unless  @shipping_address.valid?
      render_error_messages_for(@shipping_address)
      return
    end

    # 更新内容をセッションに反映する
    session[:shipping_addresses][index] = @shipping_address

    # 会員情報を変更した場合はセッション内の会員情報を更新する
    if index == 0
      user.user_address.merge_only_need(params[:user_address])
      user.mirror_from_address(@shipping_address)
      # 会員の英語住所が登録されている場合は、英語住所にも変更を反映（国、郵便番号、電話番号）
      if user.sub_address
        user.sub_address.set_sub_address_from_address(@shipping_address)
        user.sub_address.save!
      end
      session[:cart_tmp_user] = user
    end

    # カートを再計算(カート内で適用できるキャンペーンを取得するために実施します。)
    session[:cart].calculate

    @shipping_addresses = session[:shipping_addresses]
    response_text = get_update_shipping_list_script(index)
    render(:inline => response_text)
  end

  #会員情報が取得できないエラー画面
  def not_user_info_error
  end

  #注文時エラー画面
  def register_order_error
  end

  #二重注文エラー画面
  def re_order_error
  end

  private

  # カートにデータが存在しない場合にエラー画面へ遷移する
  def cart_in?
    if session[:cart] == nil || session[:cart].empty?
      redirect_to :controller=>'cart', :action => 'cart_no_input'
    end
  end

  #
  # 会員情報取得
  #
  def get_user
    session[:cart_tmp_user] =  (session[:user] && !session[:cart_tmp_user]) ? User.find(session[:user]) : session[:cart_tmp_user]
  end

  #
  # セッション情報初期化
  #
  def clear_sessions
    session[:cart_tmp_user] = nil
    session[:cart_tmp_order] = nil
    session[:cart_send_info] = nil
    session[:settlement_error] = nil
    session[:shipping_addresses] = nil
  end

  #
  # = Ajaxで配送先一覧を再表示するためのjavascript =
  #
  def get_update_shipping_list_script(index)
    # Ajaxのレスポンスとして返す内容は以下の２つです。
    #  * 新規に追加される行を保持するためのtable
    #  * それを表示しているテーブルへ移動させるJS
    #
    # trタグだけでなく、わざわざtableタグを返しているのは、
    # tableタグのinnerHTMLに代入することはJSでエラーになる恐れがあるためです。
    # see also http://d.hatena.ne.jp/akm/20071004
    result = <<-EOF
      <table id="edit_shipping_rows" style="">
        <%= render_partial_for_ajax_response('shipping_address_rows') %>
      </table>

      <script type="text/javascript">
      //<![CDATA[
        var shipping_address_button_row = $("address_buttom");
        var dest_table = shipping_address_button_row.parentNode;

        <%# trのみの指定だとヘッダーまで消してしまうので、oddとevenを指定して削除しています  %>
        $$('#shipping_address_list tr.odd').each(function(row){
          row.parentNode.removeChild(row);
        })
        $$('#shipping_address_list tr.even').each(function(row){
          row.parentNode.removeChild(row);
        })

        $$("#edit_shipping_rows tr").each(function(row){
          row.parentNode.removeChild(row);
          dest_table.insertBefore(row, shipping_address_button_row);
        });
        new Effect.Fade("shipping_address_area");
        $("order_form_send_address_index_#{index}").checked = true;
      //]]>
      </script>
    EOF
    result.split(/$/).map(&:strip).join("\n")
  end

  protected

  # トークンエラー時に、リダイレクトして同じページを再表示するアクション名。
  # send_one_placeはSPS（Alipay）決済のエラー/キャンセル時遷移により
  # 外部サイトからPOSTでアクセスされるため、トークンチェックで失敗しても
  # 画面を表示する。
  REDIRECT_MYSELF_ON_AUTHENTICITY_TOKEN_ERROR = ['payment']

  # = POSTアクセスによるトークンチェックを回避する =
  # POSTアクセスによるトークンチェックで失敗した際に、同じアクションにリダイレクトする。
  # ただし、POSTされたパラメータは破棄される。
  def verify_authenticity_token
    if request.get? || !REDIRECT_MYSELF_ON_AUTHENTICITY_TOKEN_ERROR.include?(params[:action])
      return super
    end
    begin
      return super
    rescue ActionController::InvalidAuthenticityToken
      #トークンチェック失敗時にリダイレクトで再表示
      redirect_to :action => params[:action]
    end
  end

  #
  # ユーザー情報がセッション上に存在するかをチェックします。
  #
  def redirect_to_user_error_if_invalid
    if !session[:user] && !session[:cart_tmp_user]
      redirect_to :controller => 'order', :action => 'not_user_info_error' unless performed?
      return
    end
  end
end
