# -*- coding: utf-8 -*-
# Methods added to this helper will be available to all templates in the application.
require "date"
module ApplicationHelper
  include EcRider::EcHelper

  # エラーメッセージなどのi18nサポート
  include EcRider::I18nSupport

  # JSサポート付きの入力フィールドなどのモジュール
  include EcRider::JsValidationHelper

  # プラグインで拡張されるヘルパー
  include ApplicationHelperExtention

  #
  # link_to_remoteのbutton版ヘルパ
  # 使い方はlink_to_remoteと同じ
  #
  def submit_to(name, id, url)
    button_to_function(name,"$('#{id}').action='#{url_for(url)}';$('#{id}').submit();")
  end

  # 数値に３桁ごとにカンマを挟む
  def fix_num_format(number)
    number.to_s.reverse.gsub(/[0-9]{3}/){|s| s + ","}.gsub(/,$/ ,"").reverse
  end

  def format_date(date)
    date.strftime(I18n.t('date.formats')[:default]) unless date.nil?
  end

  #
  # バリデーションエラーメッセージ
  # error_messages_forの代替に使用
  #
  def template_error_messages_for(*params)
    #    options = params.last.is_a?(Hash) ? params.pop.symbolize_keys : {}
    objects = params.collect {|object_name| instance_variable_get("@#{object_name}") }.compact
    count   = objects.inject(0) {|sum, object| sum + object.errors.count }
    unless count.zero?
      error_messages = objects.map {|object| object.errors.full_messages.map {|msg| content_tag(:li, msg) } }
      content_tag(:div, content_tag(:ul, error_messages), :class=>'alert_msg')
    end
  end

  #
  # セレクトボックス用の都道府県名リストを返す
  #
  def prefecture_names_for_select
    [[I18n.t("activerecord.errors.messages.EM990040"),""]] + Prefecture.prefecture_names(I18n.locale.to_s).each{|name| [name, name]}
  end

  #
  # 現在のページ情報を表示する
  #
  def paginate_info(collection)
    #"全<strong>#{collection.total_entries}</strong>件中<strong>#{collection.offset + 1}</strong>～<strong>#{collection.offset + collection.length}</strong>件を表示しています。"
    I18n.t("activerecord.errors.messages.NM040002", :total_entries => collection.total_entries, :offset_from => collection.offset + 1, :offset_to => collection.offset + collection.length)
  end

  #
  # デフォルトのレイアウトファイル名を取得します。
  #
  def default_layout(position, dir = 'cms')
    if (position != :top) and (position != :bottom)
      raise(ArgumentError, "Unsupported position #{position.inspect} it must be :top or :bottom")
    end
    layout = CmsLayout.send("default_layout_#{position.to_s}")
    path = layout.nil? ? nil : layout.path_to_render(dir)
    path = nil if path.nil? || !File.exist?(File.join(RAILS_ROOT, layout.deploy_path_from_root.split('/')))
    return path || CmsLayout::DEFAULT_LAYOUT_PATHS[position]
  end

  #
  # 住所を表示します
  #   :include_country => true 海外の場合には国を表示する
  #   :include_zipcode => true 郵便番号を表示する
  #   :inline => true 一行で表示する
  #
  def print_address(model, options = {})
    result = ''
    if model.is_japanese_format?
      separator = options[:inline] ? ' '   : '<br/>'
      result << "〒#{h(model.zip_code)}#{separator}" if options[:include_zipcode]
      result << h(model.address_1)
      result << h(model.fulladdress)
    else
      separator = options[:inline] ? ', ' : '<br/>'
      result << h(model.fulladdress)
      result << separator
      result << h(model.address_1)
      result << ", #{model.country.name}" if options[:include_country]
      result << "#{separator}zipcode:#{h(model.zip_code)}" if options[:include_zipcode] && !model.zip_code.blank?
    end
    return result
  end

  def display_zip_code_mark(name)
    return name.downcase.eql?("japan")? "〒" : ""
  end

  #
  # = 改行コード(\n)を<br/>に置換します。 =
  #
  def add_br_tag(str)
    return nil unless str
    str.split(/$/).map{|line| h(line)}.join("<br/>")
  end

  def link_to_without_ecr_omission(*args, &block)
     link_to(*args, &block)
  end

  # 店舗情報からSEO用のmetaタグを取得して出力します。
  def meta_tags_for_seo
    ShopSetting.get_setting(:meta_tags_for_seo, I18n.locale) if EcRider.config.ec_front
  end
  
  # 店舗情報からアクセス解析用のタグを取得して出力します。
  def last_tags_in_front_body
    ShopSetting.get_setting(:last_tags_in_front_body, I18n.locale) if EcRider.config.ec_front
  end
  
end
