# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module FavoriteTag
      extend MethodDescription


      desc <<-EOS
        [概要:お気に入り登録商品一覧を表示する場合に使用します。]
      EOS
      def favorites(*args, &block)
        return unless @template.logged_in?

        params = @template.send(:params)
        user = @template.current_user
        @per_page = params[:condition][:per_page] if params[:condition]
        @per_page ||= 3 # TODO 仮でデフォルト3
        page = params[:page]
        @favorites = user.favorites.paginate(:page => page, :per_page => @per_page)

        begin
          yield
        ensure
          @favorites = nil
        end
      end


      desc <<-EOS
        [概要:お気に入り登録商品がある場合に中身を実行します。]
      EOS
      def if_favorite_exists(&block)
        raise UsageError, "ec:if_favorite_existsはec:favoritesの中で使用してください" unless @favorites
        yield if !@favorites.empty? && block_given?
      end


      desc <<-EOS
        [概要:お気に入り登録商品がない場合に中身を実行します。]
      EOS
      def unless_favorite_exists(&block)
        raise UsageError, "ec:unless_favorite_existsはec:favoritesの中で使用してください" unless @favorites
        yield if @favorites.empty? && block_given?
      end


      desc <<-EOS
        [概要:お気に入り登録商品の総登録数を表示します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:favorite_pagination_total_count/>件中
            <ec:favorite_pagination_start_index/>～<ec:favorite_pagination_last_index/>件を表示しています。
          }}}
        ]
      EOS
      def favorite_pagination_total_count(options = {})
        raise UsageError, "ec:favorite_pagination_total_countはec:favoritesの中で使用してください" unless @favorites
        @favorites.total_entries
      end


      desc <<-EOS
        [概要:お気に入り登録商品一覧を表示する時に表示開始数を表示します。使用する場合はec:favoritesの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:favorite_pagination_total_count/>件中
            <ec:favorite_pagination_start_index/>～<ec:favorite_pagination_last_index/>件を表示しています。
          }}}
        ]
      EOS
      def favorite_pagination_start_index(options = {})
        raise UsageError, "ec:favorite_pagination_total_countはec:favoritesの中で使用してください" unless @favorites
        @favorites.offset + 1
      end


      desc <<-EOS
        [概要:お気に入り登録商品一覧を表示する時に表示終了数を表示します。使用する場合はec:favoritesの中で記述します。]
        [引数:HTMLオプションが使用できます。]
        [使用例:
          {{{
            <ec:favorite_pagination_total_count/>件中
            <ec:favorite_pagination_start_index/>～<ec:favorite_pagination_last_index/>件を表示しています。
          }}}
        ]
      EOS
      def favorite_pagination_last_index(options = {})
        raise UsageError, "ec:favorite_pagination_total_countはec:favoritesの中で使用してください" unless @favorites
        @favorites.offset + @favorites.length
      end


      desc <<-EOS
        [概要:お気に入り登録商品一覧の表示件数を切り替えるセレクトボックスを表示します。]
        [引数:render_name(必須)お気に入り登録商品一覧表示後のページ名を設定します。]
        [引数:select_count(任意)セレクトボックスで選ぶ表示件数の件数を｢○,○,○｣形式で設定します。何も指定しなかった場合、10,20,30を選択肢にします。]
        [使用例:
          {{{
          <ec:favorite_pagination_count_per_page/>
          <ec:favorite_pagination_count_per_page select_count="20,50,100"/>
          }}}
        ]
      EOS
      def favorite_pagination_count_per_page(*args)
        raise UsageError, "ec:favorite_pagination_count_per_pageはec:favoritesの中で使用してください" unless @favorites
        select_count, options = extract(args, :select_count => "10, 20, 30")

        # TODO もっといい方法がありそう
        selected = @template.send(:params)[:condition][:per_page] if @template.send(:params)[:condition]

        @template.form_for(:condition, :url => {:controller =>'favorites', :action => 'index'},:html => {:method => :get, :id => 'count_per_page_form'}) do |f|
          condition_page = @template.instance_variable_get(:@condition_page)
          @template.concat(@template.hidden_field_tag(:page, condition_page)) unless condition_page.nil?
          @template.concat(f.select(:per_page,select_count.split(/\s*,\s*/), {:selected => selected}, {:onChange => "this.form.submit();"}))
        end
        ""
      end

      
      desc <<-EOS
        [概要:お気に入り登録商品一覧が複数ページある場合に、ページ遷移リンクを表示します。ec:searched_productsの中で使用します。]
        [引数:previous_label(任意)前のページのリンクボタンの名前を指定できます。デフォルトは｢前へ｣になります。]
        [引数:next_label(任意)前のページのリンクボタンの名前を指定できます。デフォルトは｢次へ｣になります。]
        [使用例:
          {{{
            <ec:favorite_pagination_link previous_label="前のページ" next_label="次のページ"/>
            →　<a herf="">前のページ</a><a herf="">1</a><a herf="">2</a><a herf="">3</a><a herf="">次のページ</a>
          }}}
        ]
      EOS
      def favorite_pagination_link(*args)
        raise UsageError, "ec:favorite_pagination_linkはec:favoritesの中で使用してください" unless @favorites
        next_label, previous_label, option = extract(args, :next_label =>'次へ' , :previous_label => '前へ')
        # lang[]のパラメータが入ってしまいCMSコントローラでエラーになってしまうのでパラメータを書き換え
        # 言語対応に支障が出るようであれば別方法を検討する必要があります。
        @template.will_paginate(@favorites, :previous_label => previous_label, :next_label => next_label, :params=>{:lang=>nil})
      end


      desc <<-EOS
        [概要:取得しているお気に入り登録商品の分だけ繰り返しを行います。]
        [引数:なし]
        [戻り値:なし]
        [使用例:
          {{{
            <ec:each_favorites>
              <ec:favorite_name />
            </ec:each_products>
            ⇒ 全てのお気に入り登録商品の商品名を表示
          }}}
        ]
      EOS
      def each_favorites(*args, &block)
        raise UsageError, "ec:each_favoritesはec:favoritesの中で使用してください" unless @favorites
        _each_with_index(:@favorites) do
          # 商品関連のタグを使用するために、@each_productに商品を、@each_product_skuに商品SKUをセットする
          @template.instance_variable_set(:@each_product, @favorite.product_sku.product)
          @template.instance_variable_set(:@each_product_sku, @favorite.product_sku)
          yield
        end
      end


      # TODO 整理する
#      desc <<-EOS
#        [概要:お気に入り登録商品をカートに追加するボタンを表示します。]
#      EOS
#      def favorite_cart_in_button(&block)
#        # TODO 仮です
#        cart_in_button
#      end


      desc <<-EOS
        [概要:お気に入り登録商品を削除するボタンを表示します。]
      EOS
      def favorite_delete_button(*args, &block)
        # TODO 文言をymlから取得
#        button_name, class_name = *extract(args, :button_name => I18n.t(:"activerecord.errors.messages.NM990008"), :class => "button_cart1")
        button_name, class_name, options = extract(args, :button_name => "お気に入り削除", :class => :default)
        options = {:class=>class_name, :onclick=>"if (confirm('お気に入りから商品を削除します。よろしいですか？')==false){return}"}.update(options || {})

        @template.button_to_url(button_name, {:controller=>'favorites', :action=>'delete', :favorite_id=>@favorite.id, 'condition[per_page]' => @per_page}, options)
      end


      # TODO カートタグに移動する？
      desc <<-EOS
        [概要:ショッピングカート内の商品をお気に入りに登録するボタンを表示します。カートからは商品が削除されます。cart_item_list内で使用できます。]
        [引数 (オプション):move_link_name 削除ボタンの名前を設定できます。何も設定しない場合はmoveを表示します。]
        [使用例:{{{<ec:cart_item_delete_link move_link_name="お気に入りに移動する" />}}}]
      EOS
      def favorite_move_link(options = {}, &block)
        raise UsageError, "favorite_delete_buttonはec:cart_item_listの中で使用してください" unless @cart_item

        # ログイン中でお気に入りに未登録の場合のみボタンを表示する
        return unless @template.logged_in?

        user = @template.current_user
        favorite = Favorite.find_by_user_id_and_product_sku_id(@template.current_user.id, @cart_item.product_sku.id)
        return if favorite

        move_link_name = options[:move_link_name] || "move"
        link_to(move_link_name, :controller => :cart, :action => :cart_to_favorite, :id => @cart_item.key, :sku_id => @cart_item.product_sku.id)
      end


      desc <<-EOS
        [概要:お気に入り登録ボタンを表示します。]
      EOS
      def favorite_add_button

        # 


      end


    end
  end
end