# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module IncludeShippingTag
      extend MethodDescription

      desc <<-EOS
        [概要:ご注文日を出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:ご注文日]
        [使用例:{{{<ec:included_shipping_name_ordered_at/>}}}]
      EOS
      def included_shipping_ordered_at(*args)
        raise UsageError, "ec:included_shipping_order_dateはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        format= extract(args, :format => '%y/%m/%d')
        @included_shipping_address.ordered_at.to_datetime.strftime("#{format}")
      end

      desc <<-EOS
        [概要:同送先の漢字名（姓）を出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:同送先の漢字名（姓）]
        [使用例:{{{<ec:included_shipping_name_first/>}}}]
      EOS
      def included_shipping_name_first
        raise UsageError, "ec:included_shipping_name_firstはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        ERB::Util.h(@included_shipping_address.name_first)
      end

      desc <<-EOS
        [概要:同送先の漢字名（名）を出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:同送先の漢字名（名）]
        [使用例:{{{<ec:included_shipping_name_last/>}}}]
      EOS
      def included_shipping_name_last
        raise UsageError, "ec:included_shipping_name_lastはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        ERB::Util.h(@included_shipping_address.name_last)
      end

      desc <<-EOS
        [概要:同送先の郵便番号を出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:同送先の郵便番号]
        [使用例:{{{<ec:included_shipping_name_zipcode/>}}}]
      EOS
      def included_shipping_zipcode
        raise UsageError, "ec:included_shipping_zipcodeはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        ERB::Util.h("〒#{@included_shipping_address.zip_code}")
      end

      desc <<-EOS
        [概要:同送先の住所を出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:同送先の住所]
        [使用例:{{{<ec:included_shipping_address/>}}}]
      EOS
      def included_shipping_address
        raise UsageError, "ec:included_shipping_addressはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        ERB::Util.h("#{@included_shipping_address.address_1}#{@included_shipping_address.address_2}#{@included_shipping_address.address_3}")
      end

      desc <<-EOS
        [概要:同送先の電話番号を出力します。ec:each_included_shippings内で使用してください]
        [引数:なし]
        [戻り値:同送先の電話番号]
        [使用例:{{{<ec:included_shipping_telnum/>}}}]
      EOS
      def included_shipping_telnum
        raise UsageError, "ec:included_shipping_telnumはec:each_included_shippingsの中で使用してください" unless @included_shipping_address
        ERB::Util.h(@included_shipping_address.tel_num)
      end

      desc <<-EOS
        [概要:同送先の一覧を表示するボタンを出力します。]
        [使用例:
           {{{
           <ec:button_to_included_shipping_address>同送を指定する</ec:button_to_included_shipping_address>
           }}}
        ]
      EOS
      def button_to_included_shipping_address(*args, &block)
        button_name = @template.capture(&block) if block_given?
        result = @template.button_to_function(button_name,
          :onclick => "new Effect.Appear('included_shipping_address_area');")
        @template.concat(result) if result && block_given?
      end

      desc <<-EOS
        [概要:同送先の一覧を非表示にするボタンを出力します。]
        [使用例:
           {{{
           <ec:button_to_close_included_shipping_address>×閉じる</ec:button_to_close_included_shipping_address>
           }}}
        ]
      EOS
      def button_to_close_included_shipping_address(*args, &block)
        button_name = @template.capture(&block) if block_given?
        result = @template.button_to_function(button_name,
          :onclick => "new Effect.Fade('included_shipping_address_area');")
        @template.concat(result) if result && block_given?
      end

    end
  end
end
