# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module MethodDescription
      def self.extended(obj)
        @extended_objects ||= []
        @extended_objects << obj
      end

      def self.all_descriptions(reload = false)
        if @all_descriptions.nil? || reload
          @all_descriptions = {}
          @extended_objects.each do |obj|
            @all_descriptions.update(obj.descriptions)
          end
        end
        @all_descriptions
      end

      def desc(description)
        @last_description = (description || '').scan(/\[(.*?)\]/m).flatten.map do |s|
          th, td = *s.split(/[:：]/, 2)
          td ||= ''
          spaces = td.scan(/^\s*/).flatten.first
          td = td.split(/$/).map{|line|
            line.gsub(/^#{Regexp.new(Regexp.escape(spaces))}/, '') }.join("\n")
          td = ERB::Util.h(td)
          td.gsub!(/\n\n/m, "\n")
          td.gsub!(/\{\{\{/, '')
          td.gsub!(/\}\}\}/, '')
          td.gsub!(/\n\s*\Z/, "\n")
          td.gsub!(/\n/,"<br/>")
          "<tr><th>#{th}: </th><td>#{td}</td></tr>"
        end.join("\n") << "\n"
      end

      def descriptions
        @descriptions ||= {}
      end

      def method_added(method)
        descriptions[method.to_sym] = @last_description
        @last_description = nil
      end

      def desc_sort
        descriptions.to_a.sort do |a, b|
          a[0].to_s <=> b[0].to_s
        end
      end

      def self.bind_code_options
        options = []
        EcRider::EcHelper::ProductTag::PRODUCT_TAG_BIND_OPTION_METHODS.each do |method|
          options << [bind_code_name(method), method.to_s]
        end
        EcRider::EcHelper::ShopTag::SHOP_TAG_BIND_OPTION_METHODS.each do |method|
          options << [bind_code_name(method), method.to_s]
        end
        EcRider::EcHelper::UserLoginTag::USER_LOGIN_TAG_BIND_OPTION_METHODS.each do |method|
          options << [bind_code_name(method), method.to_s]
        end
        options
      end

      def self.bind_code_name(method)
        all_desc = EcRider::EcHelper::MethodDescription.all_descriptions(false)
        desc = all_desc[method.to_sym]
        desc.scan(/\<th\>概要\:\s.*td\>(.*?)を表示します。.*?/m).first if desc
      end
    end
  end
end
