# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module OrderHistoryTag
      extend MethodDescription

      desc <<-EOS
        [概要:購入履歴がある場合は中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_order_history_exits>購入履歴<ec:if_order_history_exits/>
           }}}
        ]
      EOS
      def if_order_history_exits(&block)
        unless @template.instance_variable_get(:@orders).empty?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:購入履歴がない場合は中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:unless_order_history_exits>購入履歴がありません<ec:unless_order_history_exits/>
           }}}
        ]
      EOS
      def unless_order_history_exits(&block)
        if @template.instance_variable_get(:@orders).empty?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:表示件数切り替えのセレクトボックスを出力します。購入履歴画面内で使用してください。]
        [引数 count_options(任意):選択肢を指定します。複数指定する場合はカンマ区切りにしてください。指定しない場合は"5,10,20,50"が指定されます。]
        [引数 セレクトボックスのHTML属性(任意):]
        [戻り値:表示件数のセレクトボックス]
        [使用例:
           {{{
           <ec:select_for_display/>
           ⇒ <select id="ques_count">
                <option selected="selected" value="5">5</option>
                <option value="10">10</option>
                <option value="20">20</option>
                <option value="50">50</option>
              </select>
           }}}
        ]
        [使用例:
           {{{
           <ec:select_for_display count_options="10,20"/>
           ⇒ <select id="ques_count">
                <option selected="selected" value="10">10</option>
                <option value="20">20</option>
              </select>
           }}}
        ]
      EOS
      def select_for_order_history_display(*args, &block)
        @orders = @template.instance_variable_get(:@orders)
        raise UsageError, "ここでは使えません" unless @orders
        display_number, options = extract(args, :count_options => "5,10,20,50")
        diaplay_numbers = display_number.split(/\s*,\s*/)
        history_disp_cnt_list = []
        diaplay_numbers.each do |num|
          history_disp_cnt_list << [num, num]
        end
        onchange = "window.location = '/member/history?page=1&ques=' + $('ques_count').value;"
        @template.select('', '', history_disp_cnt_list, {:selected => @template.instance_variable_get(:@ques)},
          {:name => 'ques', :id => 'ques_count', :onchange => onchange}.update(options || {}))
      end

      desc <<-EOS
        [概要:購入履歴の全件数を表示します。購入履歴画面内で使用してください。]
        [引数:なし]
        [概要:購入履歴の全件数。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           全<ec:order_history_total_count/>件中<ec:order_history_display_offset_from/>～
             <ec:order_history_display_offset_to/>を表示しています。
           ⇒全30件中10～20件を表示しています。
           }}}
        ]
      EOS
      def order_history_total_count
        @orders = @template.instance_variable_get(:@orders)
        raise UsageError, "ここでは使えません" unless @orders
        @template.number_with_delimiter(@orders.total_entries)
      end

      desc <<-EOS
        [概要:購入履歴一覧で全購入履歴中何件目からを表示しているかを出力します。購入履歴画面内で使用してください。]
        [引数:なし]
        [概要:現在表示中の購入履歴一覧の表示開始件数。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           全<ec:order_history_total_count/>件中<ec:order_history_display_offset_from/>～
             <ec:order_history_display_offset_to/>を表示しています。
           ⇒ 全30件中10～20件を表示しています。
           }}}
        ]
      EOS
      def order_history_display_offset_from
        @orders = @template.instance_variable_get(:@orders)
        raise UsageError, "ここでは使えません" unless @orders
        @template.number_with_delimiter(@orders.offset + 1)
      end

      desc <<-EOS
        [概要:履歴表示で何件までを表示しているかを出力します。購入履歴画面内で使用してください。]
        [引数:なし]
        [概要:現在表示中の購入履歴一覧の表示終了件数。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           全<ec:order_history_total_count/>件中<ec:order_history_display_offset_from/>～
             <ec:order_history_display_offset_to/>を表示しています。
           ⇒ 全30件中10～20件を表示しています。
           }}}
        ]
      EOS
      def order_history_display_offset_to
        @orders = @template.instance_variable_get(:@orders)
        raise UsageError, "ここでは使えません" unless @orders
        @template.number_with_delimiter(@orders.offset + @orders.length)
      end

      desc <<-EOS
        [概要:履歴一覧の前ページ・次ページへのリンクを出力します。購入履歴画面内で使用してください。]
        [引数 class(任意):ページネーションを囲むdivタグ（コンテナ）につくCSSクラス名を指定します。指定しない場合は"pagination"が指定されます。]
        [引数 prev_label(任意):前のページへのリンクのラベル を指定します。指定しない場合は"≪ Previous"が指定されます。]
        [引数 next_label(任意):次のページへのリンクのラベルを指定します。指定しない場合は"Next ≫"が指定されます。]
        [引数 separator(任意):ページとページの間のセパレータを指定します。指定しない場合は” ” （半角スペース）が指定されます。]
        [引数 page_links(任意):各ページへのリンクを表示するか否かを指定します。falseにすると各ページへのリンクは表示されず、前と次のページへのリンクのみ表示されます。]
        [戻り値:履歴一覧の前ページ・次ページへのリンク]
        [使用例:
           {{{
           <ec:order_history_display_paginate/>
           ⇒ ≪ Previous 1 2 Next ≫
           }}}
        ]
        [使用例:
           {{{
           <ec:order_history_display_paginate prev_label="前へ" next_label="次へ"/>
           ⇒ 前へ 1 2 次へ
           }}}
        ]
      EOS
      def order_history_display_paginate(options = {})
        @orders = @template.instance_variable_get(:@orders)
        @ques = @template.instance_variable_get(:@ques)
        raise UsageError, "ここでは使えません" unless @orders
        options = {:params => {:contoroller => 'member', :action => 'history', :ques => @ques, :lang => I18n.locale.to_s}}.update(options || {})
        @template.will_paginate(@orders, options)
      end


      desc <<-EOS
        [概要:購入履歴の数だけ繰り返します。購入履歴画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_history_list>AAA</ec:order_history_list>
           }}}
        ]
      EOS
      def order_history_list(&block)
        _each_with_index(:@orders, :@order, :@order_history_index) do
          @template.concat(@template.capture(&block))
        end
      end

      desc <<-EOS
        [概要:配送先の数だけ繰り返します。ec:order_history_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:order_shipping_list>AAA</ec:order_shipping_list>
           </ec:order_history_list>
           }}}
        ]
      EOS
      def order_shipping_list(&block)
        @order ||= @template.instance_variable_get(:@orders)
        raise UsageError, "ec:order_shipping_listはec:order_history_list購入履歴詳細画面の中で使用してください" unless @order
        @order_shippings = @order.order_shippings
        _each_with_index(:@order_shippings) do
          @template.concat(@template.capture(&block))
        end
        ""
      end

      desc <<-EOS
        [概要:注文商品の数だけ繰り返します。ec:order_shipping_list内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:order_shipping_list>
               <ec:order_shipping_product_list>AAA</ec:order_shipping_product_list>
             </ec:order_shipping_list>
           </ec:order_history_list>
           }}}
        ]
      EOS
      def order_shipping_product_list(&block)
        raise UsageError, "ec:order_shipping_listはec:order_shipping_listの中で使用してください" unless @order_shipping
        items = []
        present_items = []
        # プレゼント商品は最後列に並び替え
        @order_shipping.order_items.each do |item|
          if !item.product_sku.nil? && !item.product_sku.product.nil? && item.product_sku.product.present_flg
            present_items << item
          else
            items << item
          end
        end
        @order_items = items + present_items
        _each_with_index(:@order_items, :@order_item, :@order_shipping_product_index) do
          @template.concat(@template.capture(&block))
        end
        ""
      end

      desc <<-EOS
        [概要:購入履歴詳細ボタンを出力します。ボタンとして表示したい文字列や画像を囲んで使用します。ec:order_history_list内で使用してください。]
        [引数 inputタグのHTML属性(任意)]
        [戻り値:購入履歴詳細画面へ遷移するsubmitボタン]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:button_to_order_detail>"詳細"</ec:button_to_order_detail>
           </ec:order_history_list>
           ⇒ <input type="submit" value="詳細" name="commit"/>
           }}}
        ]
      EOS
      def button_to_order_detail(options = {}, &block)
        raise UsageError, "ec:order_cancel_statusはec:order_history_listの中で使用してください" unless @order
        @template.form_tag(:controller => 'member', :action => :history_order_detail) do
          @template.concat(@template.hidden_field_tag(:id, @order.id))
          @template.concat(@template.hidden_field_tag(:page, @orders.current_page))

          options ||= {}
          result = @template.submit_tag(@template.capture(&block), options) if block_given?
          @template.concat(result) if result && block_given?
        end
      end

      desc <<-EOS
        [概要:不正アクセスで注文へのアクセス時に中身を表示します。]
        [使用例:
           {{{
           <ec:if_order_cancel_error>
             <font color="red">ご指定の注文は存在しません。</font>
           </ec:if_order_cancel_error>
           ⇒ <font color="red">ご指定の注文は存在しません。</font>
           }}}
        ]
      EOS
      def if_order_cancel_error(&block)
        @template.concat(@template.capture(&block)) unless @template.instance_variable_get(:@user_order_error).nil?
      end

      desc <<-EOS
        [概要:キャンセル依頼ボタンを出力します。ボタンとして表示したい文字列や画像を囲んで使用します。ec:order_history_list内で使用してください。
        [引数 confirm_message(任意):ボタンを押したときに表示する確認メッセージを指定します。
          指定しない場合は「本当にキャンセル依頼してよろしいですか？」と表示されます。]
        [引数 inputタグのHTML属性(任意)]
        [戻り値:キャンセル処理を行うsubmitボタン]]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:button_for_order_cancel/>
           </ec:order_history_list>
           ⇒ <input type="submit" value="キャンセル依頼" onclick="return cancel_confirm()" name="commit"/>
           }}}
        ]
      EOS
      def button_for_order_cancel(*args, &block)
        raise UsageError, "ec:order_cancel_statusはec:order_history_listの中で使用してください" unless @order
        confirm_message, options = extract(args, :confirm_message => I18n.t(:"activerecord.errors.messages.NM040003"))
        text = @template.capture(&block) if block_given?
        js_func = @template.javascript_tag(<<-"MSG")
          function cancel_confirm(){
            if (confirm('#{confirm_message}')) {
              $('ques').value = $('ques_count').value;
              return true;
            }else{
              return false;
            }
          };
        MSG

        @template.form_tag(:controller => 'member', :action => :request_cancel) do
          @template.concat(@template.hidden_field_tag(:id, @order.id))
          @template.concat(@template.hidden_field_tag(:ques, @template.instance_variable_get(:@ques)))
          options = {:onclick => "return cancel_confirm()"}.update(options || {})
          result = @template.submit_tag(text, options) << js_func if block_given?
          @template.concat(result) if result && block_given?
        end
      end

      desc <<-EOS
        [概要:キャンセル可能かどうかを表示します。ec:order_history_list内で使用してください。]
        [引数:なし]
        [戻り値:]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:order_cancel_status/>
           </ec:order_history_list>
           }}}
        ]
      EOS
      def order_cancel_status
        raise UsageError, "ec:order_cancel_statusはec:order_history_listの中で使用してください" unless @order
        @order.cancel_status
      end

      desc <<-EOS
        [概要:キャンセル可能な場合は中身を実行します。ec:order_history_list内で使用してください。]
        [引数 なし]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:if_order_cancel_possible>キャンセル可能<ec:if_order_cancel_possible/>
           </ec:order_history_list>
           }}}
        ]
      EOS
      def if_order_cancel_possible(&block)
        raise UsageError, "ec:if_order_cancel_possibleはec:order_history_listの中で使用してください" unless @order
        if @order.cancel_status == :possible
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:キャンセル可能でない場合は中身を実行します。ec:order_history_list内で使用してください。]
        [引数 なし]
        [使用例:
           {{{
           <ec:order_history_list>
             <ec:unless_order_cancel_possible>キャンセル不可<ec:unless_order_cancel_possible/>
           </ec:order_history_list>
           }}}
        ]
      EOS
      def unless_order_cancel_possible(&block)
        raise UsageError, "ec:unless_order_cancel_possibleはec:order_history_listの中で使用してください" unless @order
        unless @order.cancel_status == :possible
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:一つの注文に対する配送先の件数を表示します。ec:order_history_list内で使用してください。]
        [引数 なし]
        [戻り値 配送先の件数]
        [使用例:
           {{{
           <ec:order_shipping_count/>
           }}}
        ]
      EOS
      def order_shipping_count
        raise UsageError, "ec:order_shipping_countはec:order_history_listの中で使用してください" unless @order
        @order.order_shippings.size
      end

      desc <<-EOS
        [概要:オンライン注文番号を表示します。ec:order_history_listまたは注文履歴詳細画面内で使用してください。]
        [引数 なし]
        [戻り値 オンライン注文番号]
        [使用例:
           {{{
           <ec:order_shipping_count/>
           }}}
        ]
      EOS
      def order_code
        @order ||= @template.instance_variable_get(:@order)
        raise UsageError, "ec:order_codeはec:order_history_listまたは注文履歴詳細画面の中で使用してください" unless @order
        @order.ec_order_code
      end

      desc <<-EOS
        [概要:注文日を出力します。]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [使用例:
           {{{
           <ec:order_datetime format="%y/%m/%d %H:%M"/>
           ⇒ 2009/03/01 15:30
           }}}
        ]
        [使用例:
           {{{
           <ec:order_date format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def order_datetime(*args)
        raise UsageError, "ec:order_dateはec:order_history_listまたは注文履歴詳細画面の中で使用してください" unless @order
        format= extract(args, :format => '%y/%m/%d')
        @order.ordered_at.strftime("#{format}")
      end

      desc <<-EOS
        [概要:注文の氏名をフルネームで表示します。ec:order_history_listまたは注文履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:注文氏名。配送先の国が日本の場合は"姓 名"で、日本以外の場合は"名 姓"で表示されます。]
        [使用例:
           {{{
           <ec:order_full_name/>
           }}}
        ]
      EOS
      def order_full_name
        @order ||= @template.instance_variable_get(:@order)
        raise UsageError, "ec:shipping_full_nameはec:order_history_listまたは注文履歴詳細画面の中で使用してください" unless @order
        ERB::Util.h(@order.user.fullname)
      end

      desc <<-EOS
        [概要:配送番号を表示します。ec:order_shipping_list内または注文履歴詳細画面で使用してください。]
        [引数:なし]
        [戻り値:配送番号]
        [使用例:
           {{{
           <ec:order_shipping_count/>
           }}}
        ]
      EOS
      def order_shipping_code
        raise UsageError, "ec:order_shipping_codeはec:order_shipping_listまたは注文履歴詳細画面の中で使用してください" unless @order_shipping
        @order_shipping.order_code
      end

      desc <<-EOS
        [概要:配送先の氏名をフルネームで表示します。ec:order_shipping_listまたは注文履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:配送先氏名。配送先の国が日本の場合は"姓 名"で、日本以外の場合は"名 姓"で表示されます。]
        [使用例:
           {{{
           <ec:shipping_full_name/>
           }}}
        ]
      EOS
      def shipping_full_name
        raise UsageError, "ec:shipping_full_nameはec:order_shipping_listまたは注文履歴詳細画面の中で使用してください" unless @order_shipping
        ERB::Util.h(@order_shipping.fullname)
      end

      desc <<-EOS
        [概要:配送先のカナ姓を表示します。ec:order_shipping_listまたは注文履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:カナ姓。]
        [使用例:
           {{{
           <ec:shipping_kana_last_name/>
           }}}
        ]
      EOS
      def shipping_kana_last_name
        raise UsageError, "ec:shipping_kana_last_nameはec:order_shipping_listまたは注文履歴詳細画面の中で使用してください" unless @order_shipping
        ERB::Util.h(@order_shipping.kana_name_last)
      end

      desc <<-EOS
        [概要:配送先のカナ名を表示します。ec:order_shipping_listまたは注文履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:カナ名。]
        [使用例:
           {{{
           <ec:shipping_kana_first_name/>
           }}}
        ]
      EOS
      def shipping_kana_first_name
        raise UsageError, "ec:shipping_kana_first_nameはec:order_shipping_listまたは注文履歴詳細画面の中で使用してください" unless @order_shipping
        ERB::Util.h(@order_shipping.kana_name_first)
      end

      desc <<-EOS
        [概要:配送先の住所を表示します。ec:order_shipping_list内で使用してください。]
        [引数:なし]
        [戻り値:配送先住所。郵便番号を含めた住所が表示されます。]
        [使用例:
           {{{
           <ec:shipping_full_address/>
           }}}
        ]
      EOS
      def shipping_full_address
        raise UsageError, "ec:shipping_full_addressはec:order_shipping_listの中で使用してください" unless @order_shipping
        @template.print_address(@order_shipping, :include_zipcode => true)
      end

      desc <<-EOS
        [概要:配送先の電話番号表示します。ec:order_shipping_listまたは注文履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:配送先の電話番号。配送先の国が日本の場合は"姓 名"で、日本以外の場合は"名 姓"で表示されます。]
        [使用例:
           {{{
           <ec:shipping_tel_num/>
           }}}
        ]
      EOS
      def shipping_tel_num
        raise UsageError, "ec:shipping_full_nameはec:order_shipping_listまたは注文履歴詳細画面の中で使用してください" unless @order_shipping
        @order_shipping.tel_num
      end

      desc <<-EOS
        [概要:送料+手数料を表示します。ec:order_shipping_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:送料+手数料。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           <ec:order_shipping_shipping_cost/>
           }}}
        ]
      EOS
      def order_shipping_shipping_cost
        raise UsageError, "ec:order_shipping_total_priceはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください" unless @order_shipping
        @template.fix_num_format(@order_shipping.shipping_cost)
      end

      desc <<-EOS
        [概要:配送先の合計金額を表示します。ec:order_shipping_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:配送先の合計金額。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           <ec:order_shipping_total_price/>
           }}}
        ]
      EOS
      def order_shipping_total_price
        raise UsageError, "ec:order_shipping_total_priceはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください" unless @order_shipping
        @template.fix_num_format(@order_shipping.total_amount + @order_shipping.shipping_cost)
      end

      desc <<-EOS
        [概要:支払い方法を表示します。ec:order_shipping_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:支払い方法。]
        [使用例:
           {{{
           <ec:order_shipping_payment_type_div_name/>
           }}}
        ]
      EOS
      def order_shipping_payment_type_div_name
        raise UsageError, "ec:order_shipping_total_priceはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください" unless @order_shipping
        @order_shipping.payment_type_div_name
      end

      desc <<-EOS
        [概要:注文のステータスを表示します。ec:order_shipping_list内で使用してください。]
        [引数:なし]
        [戻り値:注文のステータス。]
        [使用例:
           {{{
           <ec:order_shipping_status_div_name/>
           }}}
        ]
      EOS
      def order_shipping_status_div_name
        raise UsageError, "ec:order_shipping_status_div_nameはec:order_shipping_listの中で使用してください" unless @order_shipping
        @order_shipping.status_div_entry[:name_for_front].blank? ? @order_shipping.status_div_name : I18n.t( @order_shipping.status_div_entry[:name_for_front])
      end

      desc <<-EOS
        [概要:配送伝票番号を表示します。ec:order_shipping_list内で使用してください。]
        [引数:なし]
        [戻り値:配送伝票番号。]
        [使用例:
           {{{
           <ec:order_shipping_delivery_code/>
           }}}
        ]
      EOS
      def order_shipping_delivery_code
        raise UsageError, "ec:order_shipping_delivery_codeはec:order_shipping_listの中で使用してください" unless @order_shipping
        @order_shipping.delivery_code unless @order_shipping.delivery_code.blank?
      end

      unless defined?(SUPPORTED_STATUS)
        SUPPORTED_STATUS = %w(before_settlement undelivery_settlement undelivery_unpaid undelivery deliver_request delivered
          deliver_advised cancel_request canceled settlement_failed internal_error external_error).freeze
        #if_before_settlement if_undelivery_settlement if_undelivery_unpaid if_undelivery if_deliver_request if_delivered
        #if_deliver_advised if_cancel_request if_canceled if_settlement_failed if_internal_error if_external_errorメソッドを生成します
        SUPPORTED_STATUS.each do |status|
          desc <<-EOS
          [概要:注文ステータスが#{OrderShipping.status_div_name_by_key(status.intern)}の場合は中身を実行します。ec:order_shipping_list内で使用してください。]
          [引数:なし]
          [使用例:
             {{{
             <ec:if_#{status}>AAA</ec:if_#{status}>
             }}}
          ]
          EOS
          module_eval(<<-"EOS")
            def if_#{status}(&block)
              raise UsageError, 'ec:order_shipping_status_div_nameはec:order_shipping_listの中で使用してください' unless @order_shipping
              if @order_shipping.status_div_key == :#{status}
                yield
              end
            end
          EOS
        end

        #unless_before_settlement unless_undelivery_settlement unless_undelivery_unpaid unless_undelivery unless_deliver_request unless_delivered
        #unless_deliver_advised unless_cancel_request unless_canceled unless_settlement_failed unless_internal_error unless_external_errorメソッドを生成します
        SUPPORTED_STATUS.each do |status|
          desc <<-EOS
          [概要:注文ステータスが#{OrderShipping.status_div_name_by_key(status.intern)}以外の場合は中身を実行します。ec:order_shipping_list内で使用してください。]
          [引数:なし]
          [使用例:
             {{{
             <ec:unless_#{status}>AAA</ec:unless_#{status}>
             }}}
          ]
          EOS
          module_eval(<<-"EOS")
            def unless_#{status}(&block)
              raise UsageError, 'ec:order_shipping_status_div_nameはec:order_shipping_listの中で使用してください' unless @order_shipping
              unless @order_shipping.status_div_key == :#{status}
                yield
              end
            end
          EOS
        end
      end

      unless defined?(TARGET_COLUMNS)
        TARGET_COLUMNS = %w(delivery_code).freeze
        #if_delivery_code_inputメソッドを生成します
        TARGET_COLUMNS.each do |column|
          desc <<-EOS
          [概要:columnにデータがある場合は中身を実行します。ec:order_shipping_list内で使用してください。]
          [引数:なし]
          [使用例:
             {{{
             <ec:if_#{column}_input>AAA</ec:if_#{column}_input>
             }}}
          ]
          EOS
          module_eval(<<-"EOS")
            def if_#{column}_input(&block)
              raise UsageError, 'ec:if_#{column}_inputはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください' unless @order_shipping
              unless @order_shipping.#{column}.blank?
                yield
              end
            end
          EOS
        end

        #unless_delivery_code_inputメソッドを生成します
        TARGET_COLUMNS.each do |column|
          desc <<-EOS
          [概要:columnにデータがない場合は中身を実行します。ec:order_shipping_list内で使用してください。]
          [引数:なし]
          [使用例:
             {{{
             <ec:unless_#{column}_input>AAA</ec:if_#{column}_input>
             }}}
          ]
          EOS
          module_eval(<<-"EOS")
            def unless_#{column}_input(&block)
              raise UsageError, 'ec:unless_#{column}_inputはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください' unless @order_shipping
              if @order_shipping.#{column}.blank?
                yield
              end
            end
          EOS
        end
      end
      desc <<-EOS
        [概要:配送先住所が日本語の場合は中身を実行します。ec:order_shipping_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_japanese_shipping_format/>
           }}}
        ]
      EOS
      def if_japanese_shipping_format(&block)
        raise UsageError, "ec:if_japanese_shipping_formatはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください" unless @order_shipping
        if @order_shipping.is_japanese_format?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:注文商品の画像を表示します。ec:order_shipping_product_listまたは購入履歴詳細画面内で使用してください。]
        [引数 type(任意):商品画像のサイズを指定します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数:imgタグのHTML属性(任意)]
        [戻り値:注文商品の画像。]
        [使用例:
           {{{
           <ec:order_shipping_product_image/>
           }}}
        ]
      EOS
      def order_shipping_product_image(*args)
        raise UsageError, "ec:order_shipping_product_qtyはec:order_shipping_product_listまたは購入履歴詳細画面の中で使用してください" unless @order_item
        product_sku = ProductSku.find(:first, :conditions => ["id = ?", @order_item.product_sku_id])
        unless product_sku.nil?
          product = product_sku.product
          type, options = extract(args, :type => nil)
          options["alt"] ||= @order_item.product_name
          return @template.image_tag(product.main_image.public_filename(type), options) if !product.nil? && !product.main_image.nil?
        end
      end

      desc <<-EOS
        [概要:注文商品の名前を表示します。ec:order_shipping_product_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:注文商品の名前。]
        [使用例:
           {{{
           <ec:order_shipping_product_name/>
           }}}
        ]
      EOS
      def order_shipping_product_name
        raise UsageError, "ec:order_shipping_product_nameはec:order_shipping_product_listまたは購入履歴詳細画面の中で使用してください" unless @order_item
        @order_item.product_name
      end

      desc <<-EOS
        [概要:注文商品の点数を表示します。ec:order_shipping_product_list内または購入履歴詳細画面で使用してください。]
        [引数:なし]
        [戻り値:注文商品の点数。]
        [使用例:
           {{{
           <ec:order_shipping_product_qty/>
           }}}
        ]
      EOS
      def order_shipping_product_qty
        raise UsageError, "ec:order_shipping_product_qtyはec:order_shipping_product_listまたは購入履歴詳細画面の中で使用してください" unless @order_item
        @order_item.qty
      end

      desc <<-EOS
        [概要:注文商品の単価を表示します。ec:order_shipping_product_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:注文商品の単価。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           <ec:order_shipping_product_unit_price/>
           }}}
        ]
      EOS
      def order_shipping_product_unit_price
        raise UsageError, "ec:order_shipping_product_qtyはec:order_shipping_product_listまたは購入履歴詳細画面の中で使用してください" unless @order_item
        @template.fix_num_format(@order_item.unit_price)
      end

      desc <<-EOS
        [概要:注文商品の小計を表示します。ec:order_shipping_product_listまたは購入履歴詳細画面内で使用してください。]
        [引数:なし]
        [戻り値:注文商品の小計。3桁カンマ区切りで表示されます。]
        [使用例:
           {{{
           <ec:order_shipping_product_sub_total_price/>
           }}}
        ]
      EOS
      def order_shipping_product_sub_total_price
        raise UsageError, "ec:order_shipping_product_qtyはec:order_shipping_product_listまたは購入履歴詳細画面の中で使用してください" unless @order_item
        @template.fix_num_format(@order_item.total_amount)
      end

      desc <<-EOS
        [概要:購入履歴画面へ戻るボタンを出力します。ボタンとして表示したい文字列や画像を囲んで使用します。購入履歴詳細画面内で使用してください。]
        [引数 inputタグのHTML属性（任意:]
        [戻り値:inputボタン]
        [使用例:
           {{{
           <ec:button_to_history>戻る</ec:button_to_history>
           }}}
        ]
      EOS
      def button_to_history(*args, &block)
        page_no = @template.instance_variable_get(:@page_no)
        raise UsageError, "ec:button_to_historyは注文履歴詳細画面の中で使用してください" unless page_no
        options = args.shift || {}
        href = "/member/history?page=#{page_no}"
        result = @template.button_to_url(@template.capture(&block), href, options) if block_given?
        @template.concat(result) if result && block_given?
        result
      end


      desc <<-EOS
        [概要:領収書印刷ページに遷移する、領収書用宛名入力フォームを出力します。]
        [引数:なし]
        [戻り値:formタグ]
        [使用例:
           {{{
           <ec:addressee_form>
             宛名入力<ec:addressee_form_addressee/>
             <ec:submit>領収書印刷</ec:submit>
           </ec:addressee_form>
           }}}
        ]
      EOS
      def addressee_form(&block)
        raise UsageError, "ここでは使用できません。" unless @order
        @template.form_for(:addressee_form, :url => {:controller => 'member', :action => 'print_receipt'},
            :html=>{:target => '_blank'}) do |f|
          @addressee_form = f
          begin
            @template.concat(@template.hidden_field_tag(:id, @order.id))
            @template.concat(@template.capture(&block))
          ensure
            @addressee_form = nil
          end
        end
      end


      desc <<-EOS
        [概要:宛名を入力するinputタグを出力します。領収書用宛名入力フォーム内でのみ使用できます。]
        [引数:なし]
        [戻り値:inputタグ]
        [使用例:
           {{{
           <ec:addressee_form>
             宛名入力<ec:addressee_form_addressee/>
           </ec:addressee_form>
           }}}
        ]
      EOS
      def addressee_form_addressee(options = {})
        raise UsageError, "ec:order_shipping_total_priceはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください" unless @order
        options = {:maxLength => 50, :size => 35,
          :class => :default, :style => "ime-mode: inactive"}.update(options || {})
        @addressee_form.text_field(:addressee, options)
      end


      desc <<-EOS
        [概要:商品金額の合計を出力します。]
        [引数:なし]
        [戻り値:商品金額の合計]
        [使用例:
           {{{
           <ec:order_shipping_total_item_price/>
           }}}
           ⇒ 194,040
        ]
      EOS
      def order_shipping_total_item_price
        raise UsageError, "ec:order_shipping_total_priceはec:order_shipping_listまたは購入履歴詳細画面の中で使用してください" unless @order
        @template.fix_num_format(@order_shipping.total_amount)
      end

      desc <<-EOS
        [概要:宛名を表示します。]
        [引数:なし]
        [戻り値:宛名]
        [使用例:
           {{{
           <ec:addressee/>
           }}}
           ⇒ 木村 拓哉
        ]
      EOS
      def addressee
        addressee = @template.instance_variable_get(:@addressee)
        raise UsageError, "ec:order_codeはec:order_history_listまたは注文履歴詳細画面の中で使用してください" unless @order
        unless addressee.nil?
          return ERB::Util.h(addressee)
        else
          return "&nbsp;"
        end
      end

      desc <<-EOS
        [概要:同送があり、親である場合中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_include_order_parent><image_tag src="parent.gif"><ec:if_include_order_parent/>
           }}}
        ]
      EOS
      def if_include_order_parent(&block)
        raise UsageError, "ec:if_include_order_parent?はec:order_history_list購入履歴画面の中で使用してください" unless @order
        if  @order.include_order_id.nil? && @order.includable_shipping_parent?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:同送があり、子である場合中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_include_order_child><image_tag src="child.gif"><ec:if_include_order_child/>
           }}}
        ]
      EOS
      def if_include_order_child(&block)
        raise UsageError, "ec:if_include_order_child?はec:order_history_list購入履歴画面の中で使用してください" unless @order
        unless @order.include_order_id.nil?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:ギフトの場合中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_history_gift>ギフト&nbsp;<ec:if_history_gift/>
           }}}
        ]
      EOS
      def if_history_gift(&block)
        raise UsageError, "ec:if_history_giftはec:order_history_list購入履歴画面の中で使用してください" unless @order
        if @order.gift_flg
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:商品に納品書、請求書を同梱しない場合中身を実行します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_history_send_invoice_separately>商品に納品書、請求書を同梱しない&nbsp;<ec:if_history_send_invoice_separately/>
           }}}
        ]
      EOS
      def if_history_send_invoice_separately(&block)
        raise UsageError, "ec:if_history_send_invoice_separatelyはec:order_history_list購入履歴画面の中で使用してください" unless @order
        if @order.send_invoice_separately_flg
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:送状に購入者名を表示する場合中身を実行します。。]
        [引数:なし]
        [使用例:
           {{{
           <ec:history_write_sender>送状に購入者名を表示する&nbsp;<ec:history_write_sender/>
           }}}
        ]
      EOS
      def if_history_write_sender(&block)
        raise UsageError, "ec:if_history_write_senderはec:order_history_list購入履歴画面の中で使用してください" unless @order
        if @order.write_sender_flg
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:ポイント制度が利用されている場合、中身を実行します。。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_point_setting>獲得ポイント</ec:if_point_setting>
           }}}
        ]
      EOS
      def if_point_setting(&block)
        point_setting = PointSetting.find(:first)
        if !point_setting.blank? && point_setting.available_flg_true?
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:ポイント制度が利用されている場合、発行ポイントを表示します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:if_point_setting><ec:order_shippping_given_point/></ec:if_point_setting>
           }}}
        ]
      EOS
      def order_shipping_given_point
        @order ||= @template.instance_variable_get(:@order)
        raise UsageError, "ec:given_pointはec:order_history_list購入履歴画面の中で使用してください" unless @order
        point_setting = PointSetting.find(:first)
        if !point_setting.blank? && point_setting.available_flg_true?
          user_points = @order.user_points
          given_point = 0
          user_points.each {|point| return given_point = point.given_point if point.given_point}
          @template.number_with_delimiter(given_point)
        end
      end

      desc <<-EOS
        [概要:適用されているキャンペーン一覧を返します]
        [引数 display_qty(任意): 最大表示数を設定します。設定しない場合は適用されるキャンペーン分すべてを繰り返し実行します。]
        [使用例:
          {{{
          <ec:each_order_history_applied_campaigns>
            <ec:order_applied_campaign_link>
              <ec:order_applied_campaign_name />
            </ec:order_applied_campaign_link>
          </ec:each_order_history_applied_campaigns>
             →適用されているキャンペーン名称リンクを繰り返し表示します。
          }}}
        ]
      EOS
      def each_order_history_applied_campaigns(*args, &block)
        @order ||= @template.instance_variable_get(:@order)
        raise UsageError, "ec:given_pointはec:order_history_list購入履歴画面の中で使用してください" unless @order
        display_qty, options = extract(args, :display_qty => nil)
        @campaigns = @order.applied_campaigns
        begin
          _each_with_index(:@campaigns, :@campaign, :@index) do
            if display_qty.nil? || display_qty.to_i > @index.to_i
              @template.concat(@template.capture(&block))
            else
              break
            end
          end
        ensure
          @campaigns = nil
        end
      end
    end
  end
end
