# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module ReminderFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:リマインダー変更用のフォームを出力します。]
      EOS
      def reminder_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@reminder_form)
        @template.form_for(:reminder_form, :url => {:controller => 'reminder', :action => :reminder_completion}) do |f|
          @reminder_form = f
          @template.concat(@reminder_form.hidden_field(:completion_url_controller))
          @template.concat(@reminder_form.hidden_field(:completion_url_action))
          @template.concat(@reminder_form.hidden_field(:back_to_url))
          begin
            @template.concat(@template.capture(&block))
          ensure
            @reminder_form = nil
          end
        end
      end

      desc <<-EOS
        [概要:リマインダー変更時のエラーメッセージがあればそれを表示します。リマインダー変更用のフォーム内で使用してください。]
      EOS
      def reminder_form_error_messages
        @template.template_error_messages_for(:reminder_form)
      end

      desc <<-EOS
        [概要:会員情報に登録しているメールアドレスを入力するinputタグを出力します。リマインダー変更用のフォーム内でのみ使用できます。]
      EOS
      def reminder_form_email(options = {})
        raise UsageError, "ec:reminder_form_emailはec:reminder_formの中で使用してください" unless @reminder_form
        options = {:maxLength => 50,:size => 35,:class=>:default, :style => "ime-mode: inactive", :id => "reminder_form_email"}.update(options || {})
        validation_option = { :empty => true, :email => true, :realtime => true, :update => 'email_msg', :error => "メールアドレス"}

        url = "/reminder/get_remember_question"
        email_msg = "登録されていないメールアドレスです。"
        js_func = @template.javascript_tag(<<-"SCRIPT")
                               Event.observe('reminder_form_email', 'change', function(event){
                                    var email_field = Event.element(event);
                                    if (email_field.value == ""){
                                        $('remember_question').innerHTML = "";
                                        $('reminder_form_remember_question').value = "";
                                    } else {
                                        new Ajax.Updater("remember_question",
                                        "#{url }?email=" +
                                        encodeURIComponent(email_field.value),
                                        { method: 'get',
                                            onSuccess: function(request) {
                                                var question = request.responseText;
                                                if (question == '') { error_msg('email_msg','#{email_msg }'); }
                                                    else {error_msg('email_msg','');}
                                                    $('reminder_form_remember_question').value = request.responseText;
                                                }
                                            }
                                        );
                                   }
                               });
                          SCRIPT
        @reminder_form.text_field_valid(:email, options, validation_option) << js_func

      end

      desc <<-EOS
        [概要:秘密の質問を表示します。リマインダー変更用のフォーム内でのみ使用できます。]
      EOS
      def reminder_form_remember_question(options = {})
        raise UsageError, "ec:reminder_form_remember_questionはec:reminder_formの中で使用してください" unless @reminder_form
        remember_question =  @template.instance_variable_get(:@reminder_form).remember_question
        @template.concat("<span id='remember_question'>")
        @template.concat(remember_question) unless remember_question.nil?
        @template.concat("</span>")
        @reminder_form.hidden_field(:remember_question)
      end

      desc <<-EOS
        [概要:秘密の質問の回答を入力するinputタグを出力します。リマインダー変更用のフォーム内でのみ使用できます。]
      EOS
      def reminder_form_remember_answer(options = {})
        raise UsageError, "ec:reminder_form_remember_answerはec:reminder_formの中で使用してください" unless @reminder_form
        options = {:maxLength => 20,:size => 30,:class=>:default,:style => "ime-mode: active"}.update(options || {})
        validation_option = { :empty => true, :remember_answer => true, :realtime => true, :update => 'remember_answer_msg', :error => "答え"}
        @reminder_form.text_field_valid(:remember_answer, options, validation_option)
      end

      desc <<-EOS
        [概要:新しいパスワードを入力するinputタグを出力します。リマインダー変更用のフォーム内でのみ使用できます。]
      EOS
      def reminder_form_password(options = {})
        raise UsageError, "ec:user_reminder_form_passwordはec:reminder_formの中で使用してください" unless @reminder_form
        options = {:maxLength => 20, :size => 50, :autocomplete => "off"}.update(options || {})
        @reminder_form.password_field(:password, options)
      end

      desc <<-EOS
        [概要:新しいパスワード(確認用)を入力するinputタグを出力します。リマインダー変更用のフォーム内でのみ使用できます。]
      EOS
      def reminder_form_password_confirmation(options = {})
        raise UsageError, "ec:user_reminder_form_password_confirmationはec:reminder_formの中で使用してください" unless @reminder_form
        options = {:maxLength => 20, :size => 50}.update(options || {})
        @reminder_form.password_field(:password_confirmation, options)
      end
      
      desc <<-EOS
        [概要:遷移元の画面に戻るボタンを表示します。]
        [戻り値:画面遷移リンク。]
        [使用例:{{{<ec:reminder_form_button_to_back/>}}}]
      EOS
      def reminder_form_button_to_back(*args, &block)
        raise UsageError, "ec:reminder_form_button_to_backはec:reminder_formの中で使用してください" unless @reminder_form
        options = args.shift || {}
        href = @template.instance_variable_get(:@reminder_form).back_to_url
        result = @template.button_to_url(@template.capture(&block), href, options) if block_given?
        @template.concat(result) if result && block_given?
      end
    end
  end
end
