# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module ShippingFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:配送先入力用のフォームを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_form>
             <ec:shipping_form_error_messages/>
             <ec:include_user_address_fields/>
           </ec:shipping_form>
           }}}
        ]
      EOS
      def shipping_form(&block)
        unless @template.instance_variable_get(:@user_address)
          raise UsageError, "ここでは使用できません。"
        end
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => 'member', :action => :confirm_shipping}, :html => {:id => "shipping_form" }) do |f|
          @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f2|
            @user_address = f
            @user_profile = f2
            begin
              @template.concat(@template.capture(&block))
            ensure
              @user_address = nil
              @user_profile = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:配送先入力のエラーメッセージがあればそれを表示します。ec:shipping_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_form>
             <ec:shipping_form_error_messages/>
             <ec:include_user_address_fields/>
           </ec:shipping_form>
           }}}
        ]
      EOS
      def shipping_form_error_messages
        @template.template_error_messages_for(:user_address)
      end

      desc <<-EOS
        [概要:配送先入力確認用のフォームを出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_full_name/>
             <ec:if_address_japanese_format>
               <ec:shipping_confirmation_form_kana_last_name/>
               <ec:shipping_confirmation_form_kana_first_name/>
             </ec:if_address_japanese_format>
             <ec:shipping_confirmation_form_country/>
             <ec:shipping_confirmation_form_zip_code/>
             <ec:shipping_confirmation_form_prefecture/>
             <ec:shipping_confirmation_form_full_address/>
             <ec:shipping_confirmation_form_tel_num/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form(&block)
        unless @template.instance_variable_get(:@user_address)
          raise UsageError, "ここでは使用できません。"
        end
        @template.form_for(:user_address, @template.instance_variable_get(:@user_address), :url => {:controller => 'member', :action => :update_shipping}, :html => {:id => "shipping_confirmation_form" }) do |f|
          @template.fields_for(:user_profile, @template.instance_variable_get(:@user_profile)) do |f2|
            @user_address = f
            @user_profile = f2
            begin
              @template.concat(@template.capture(&block))
            ensure
              @user_address = nil
              @user_profile = nil
            end
          end
        end
      end

      desc <<-EOS
        [概要:配送先住所の姓名の入力内容をフルネームで出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_full_name/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_full_name
        raise UsageError, "ec:shipping_confirmation_form_full_nameはec:shipping_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.fullname)
      end

      desc <<-EOS
        [概要:配送先住所のカナ姓の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_kana_last_name/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_kana_last_name
        raise UsageError, "ec:shipping_confirmation_form_kana_last_nameはec:shipping_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.kana_name_last)
      end

      desc <<-EOS
        [概要:配送先住所のカナ名の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_kana_first_name/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_kana_first_name
        raise UsageError, "ec:shipping_confirmation_form_kana_first_nameはec:shipping_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.kana_name_first)
      end

      desc <<-EOS
        [概要:配送先住所の国の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_country/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_country
        raise UsageError, "ec:shipping_confirmation_form_countryはec:shipping_confirmation_formの中で使用してください" unless @user_address
        @user_address.object.country.name
      end

      desc <<-EOS
        [概要:配送先住所の郵便番号の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_zip_code/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_zip_code
        raise UsageError, "ec:shipping_confirmation_form_japanese_zip_codeはec:shipping_confirmation_formの中で使用してください" unless @user_address
        zip_code_mark = @user_address.object.country.name.downcase.eql?("japan")? "〒" : ""
        ERB::Util.h("#{zip_code_mark}#{@user_address.object.zip_code}")
      end

      desc <<-EOS
        [概要:配送先住所の都道府県の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_prefecture/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_prefecture
        raise UsageError, "ec:shipping_confirmation_form_japanese_prefectureはec:shipping_confirmation_formの中で使用してください" unless @user_address
        @user_address.object.address_1
      end

      desc <<-EOS
        [概要:配送先住所の住所の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_full_address/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_full_address
        raise UsageError, "ec:shipping_confirmation_form_address_2はec:shipping_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.fulladdress)
      end

      desc <<-EOS
        [概要:配送先住所の電話番号の入力内容を出力します。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:shipping_confirmation_form_tel_num/>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def shipping_confirmation_form_tel_num
        raise UsageError, "ec:shipping_confirmation_form_tel_numはec:shipping_confirmation_formの中で使用してください" unless @user_address
        ERB::Util.h(@user_address.object.tel_num)
      end

      desc <<-EOS
        [概要:配送先住所が日本フォーマットによる入力ならば、中身を実行します。。ec:shipping_confirmation_form内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:shipping_confirmation_form>
             <ec:if_address_japanese_format>
               <ec:shipping_confirmation_form_kana_last_name/>
               <ec:shipping_confirmation_form_kana_first_name/>
             </ec:if_address_japanese_format>
           </ec:shipping_confirmation_form>
           }}}
        ]
      EOS
      def if_address_japanese_format
        if @user_address.object.is_japanese_format?
          yield if block_given?
        end
      end

      # 別名を定義します。
      method_names = %w(full_name kana_last_name kana_first_name country zip_code prefecture tel_num)

      src_form_name = 'shipping_confirmation_form'
      alias_form_names = %w(order_confirmation_form)
      alias_form_names.each do |alias_form_name|
        method_names.each do |method_name|
          alias_method("#{alias_form_name}_#{method_name}", "#{src_form_name}_#{method_name}")
        end
      end

    end
  end
end
