# -*- coding: utf-8 -*-
require "rexml/document"

# EC-RiderのCMS用のXMLとerbの相互変換を行うためのクラスです。
class CmsXml

  attr_accessor :xml, :erb

  def initialize(attributes = {})
    attributes.each do |attr, value|
      send("#{attr}=", value)
    end
  end

  class << self

    def generate_erb(xml)
      unless xml.include?("<ec xmlns:ec=\"http://dummy\">#{xml}</ec>")
        xml = "<ec xmlns:ec=\"http://dummy\">#{xml}</ec>"
      end
      self.new(:xml => xml).to_erb
    end

    # XMLのタグとしての文法チェック
    def validate_tags(tags_text)
      self.generate_erb(tags_text)
      return true
    rescue REXML::ParseException
      return false
    end

  end

  def to_erb
    @erb = ""
    doc = REXML::Document.new(xml)
    process_xml(doc.elements.first)
    @erb
  end

  HTML_TAGS = 
    %w(a abbr acronym address applet area b base basefont bdo bgsound big blink blockquote body br button) +
    %w(caption center cite code col colgroup comment dd del dfn dir div dl dt em embed fieldset font form frame frameset) +
    %w(h1 h2 h3 h4 h5 h6 head hr html i iframe ilayer img input ins isindex kbd keygen label layer legend li link listing) +
    %w(map marquee menu meta multicol nextid nobr noembed noframes nolayer noscript object ol optgroup option p) +
    %w(param plaintext pre q rb rp rt ruby s samp script select server small spacer span strike strong style sub sup) +
    %w(table tbody td textarea tfoot th thead title tr tt ul var wbr xmp)
  
  SELF_CLOSING_TAGS = %w(input img br hr)

  OPENABLE_TAGS = HTML_TAGS - SELF_CLOSING_TAGS

  def process_xml(parent)
    parent.children.each do |child|
      case child
      when REXML::Element then
        if child.prefix == 'ec'
          # ec:yield は app/views/layouts/application.html.erb などで使用されます。
          if child.name == 'yield'
            tag_content = "yield"
            args = (child.attributes['args'] || '').split(',')
            unless args.empty?
              tag_content << '(' << args.map{|arg| ":#{arg}"}.join(', ') << ')'
            end
            @erb << "<%= #{tag_content} -%>" 
          else
            attr_array = []
            attrs = child.attributes
            attrs.keys.sort.each do |attr|
              attr_array << ":#{attr}=>\"#{attrs[attr]}\""
            end
            tag_content = "ec.#{child.name}"
            unless attr_array.empty?
              tag_content << '(' << attr_array.join(', ') << ")"
            end
            if child.children.empty?
              @erb << "<%= #{tag_content} -%>" 
            else
              @erb << "<% #{tag_content} do -%>" 
              process_xml(child)
              @erb << "<% end -%>" 
            end
          end
        else
          # prefixが空ならHTMLとして扱います。
          self_closing = child.prefix.blank? ? 
            # HTMLのタグなら自己終了タグとなるべきかどうかで判断します。
            SELF_CLOSING_TAGS.include?(child.name) : 
            # 非HTMLのタグの場合は、子要素を持っているかどうかで判断します。
            child.children.empty?
          if self_closing
            @erb << child.to_s.gsub(/'/, '"')
          else
            tag_name = (child.prefix.blank? ? '' : "#{child.prefix}:" ) << child.name
            tag_content = ''
            tag_content << tag_name
            child.attributes.each do |attr, value|
              tag_content << " #{attr}=\"#{value}\""
            end
            @erb << "<#{tag_content}>"
            process_xml(child)
            @erb << "</#{tag_name}>"
          end
        end
      when REXML::Comment
        @erb << "<%# #{child.to_s.strip} %>"
      else
        @erb << child.to_s
      end

    end
  end

end
