require 'fastercsv'
class CsvDownloadPatternItem < ActiveRecord::Base
  belongs_to :pattern, :class_name => "CsvDownloadPattern", :foreign_key => 'csv_download_pattern_id'
  
  selectable_attr :code do
    entry '000', :extra_column, '任意のカラム'
    
    with_options(:type => :user) do |g|
      g.entry '101', :users_user_code, '会員コード',                     :reader => "user_code"
      g.entry '102', :users_name_last, '氏名漢字（姓）',                 :reader => "name_last"
      g.entry '103', :users_kana_name_last, '氏名フリガナ（姓）',        :reader => "kana_name_last"
      g.entry '104', :users_name_first, '氏名漢字（名）',                :reader => "name_first"
      g.entry '105', :users_kana_name_last_first, '氏名フリガナ（名）',  :reader => "kana_name_first"
      g.entry '106', :users_tel_num, '電話番号',                         :reader => "tel_num"
      g.entry '107', :user_addresses_country_code, '国',                 :reader => "user_address.country_code"
      g.entry '108', :user_addresses_zip_code, '郵便番号',               :reader => "user_address.zip_code"
      g.entry '109', :user_addresses_address_1, '都道府県（住所１）',    :reader => "user_address.address_1"
      g.entry '110', :user_addresses_address_2, '市区町村（住所２）',    :reader => "user_address.address_2"
      g.entry '111', :user_addresses_address_3, '番地・マンション名（住所３）', :reader => "user_address.address_3"
      g.entry '117', :user_emails_email, 'メールアドレス',               :reader => "user_email.email"
      g.entry '118', :users_sex, '性別',                                 :reader => "sex", :remark => [User, :sex]
      g.entry '119', :users_birth_year, '生年月日（西暦）',              :reader => "birth_year"
      g.entry '120', :users_birth_month, '生年月日（月）',               :reader => "birth_month"
      g.entry '121', :users_birth_day, '生年月日（日）',                 :reader => "birth_day"
      g.entry '122', :users_job_div, '職業',                             :reader => "job_div", :remark => [User, :job_div]
      g.entry '123', :users_member_div, '会員種別',                      :reader => "member_div", :remark => [User, :member_div]
      g.entry '124', :users_resigened_at, '退会日',                      :reader => "resigened_at", :format => :date_time
      g.entry '125', :users_remember_answer, '秘密の質問の答え',         :reader => "remember_answer"
      g.entry '126', :users_regist_div, '登録サイト区分',                :reader => "regist_div", :remark => [User, :regist_div]
      g.entry '127', :users_dm_send_div, 'DM配信区分',                   :reader => "dm_send_div", :remark => [User, :dm_send_div]
      g.entry '128', :users_remember_question, '秘密の質問',             :reader => "remember_question"
      g.entry '130', :users_pc_mail_magazine_send_div, 'PCメールマガジン配信区分',           :reader => "pc_mail_magazine_send_div", :remark => [User, :pc_mail_magazine_send_div]
      g.entry '131', :users_mobile_mail_magazine_send_div, 'モバイルメールマガジン配信区分', :reader => "mobile_mail_magazine_send_div", :remark => [User, :mobile_mail_magazine_send_div]
      g.entry '132', :users_test_flg, 'テスト会員フラグ',                :reader => "test_flg", :remark => "1:テスト会員、0または空欄:本番会員"
      g.entry '133', :users_member_category_div, '会員種別',             :reader => "member_category_div", :remark => [User, :member_category_div]
      g.entry '134', :users_login_challenge_count, 'ログイン試行回数',   :reader => "login_challenge_count"
      g.entry '135', :users_locked_at, 'ログインロック日時',             :reader => "locked_at", :format => :date_time
      g.entry '138', :user_ranks_code, '会員ランクコード',               :reader => "rank.code", :remark => UserRank.rank_remark
    end
    
    with_options(:type => :product) do |g|
      g.entry '201', :products_product_code, '商品コード',                     :reader => "product_code"
      g.entry '202', :products_order_product_name_1, '管理用商品名（日本語）', :reader => "order_product_name_1"
      g.entry '203', :products_order_product_name_2, '管理用商品名（英語）',   :reader => "order_product_name_2"
      g.entry '204', :products_kana_product_name, '管理用商品名（カナ）',      :reader => "kana_product_name"
      g.entry '205', :products_course_div, 'コース区分',                       :reader => "course_div", :remark => [Product, :course_div]
      g.entry '206', :products_detail_url, '詳細画面URL',                      :reader => "detail_url"
      g.entry '207', :products_display_order, '表示順',                        :reader => "display_order"
      g.entry '208', :products_origin, '原産国コード',                               :reader => "origin"
      g.entry '209', :product_categories_1_code, 'カテゴリ1',                  :reader => "category_1.code"
      g.entry '210', :product_categories_2_code, 'カテゴリ2',                  :reader => "category_2.code"
      g.entry '211', :product_categories_3_code, 'カテゴリ3',                  :reader => "category_3.code"
      g.entry '212', :product_categories_4_code, 'カテゴリ4',                  :reader => "category_4.code"
      g.entry '213', :product_categories_5_code, 'カテゴリ5',                  :reader => "category_5.code"
      g.entry '214', :product_variations_1_name, 'バリエーション1',            :reader => "variation_1.name_for_admin"
      g.entry '215', :product_variations_2_name, 'バリエーション2',            :reader => "variation_2.name_for_admin"
      g.entry '216', :product_variations_3_name, 'バリエーション3',            :reader => "variation_3.name_for_admin"
      g.entry '217', :products_present_flg, 'プレゼント品フラグ',              :reader => "present_flg"
      g.entry '218', :products_sale_started_at, '販売開始日時',                :reader => "sale_started_at", :format => :date_time
      g.entry '259', :products_sale_finished_at, '販売終了日時',               :reader => "sale_finished_at", :format => :date_time
      g.entry '219', :products_publish_started_at, '公開開始日時',             :reader => "publish_started_at", :format => :date_time
      g.entry '220', :products_publish_finished_at, '公開終了日時',            :reader => "publish_finished_at", :format => :date_time
      g.entry '221', :products_invisible_flg, '公開フラグ',                    :reader => "invisible_flg", :remark => "1:非公開、0または空欄:公開"
      g.entry '222', :product_sku_variations_1_code, 'SKUバリエーション項目1', :reader => "primary_sku.variation_item_1.code"
      g.entry '223', :product_sku_variations_2_code, 'SKUバリエーション項目2', :reader => "primary_sku.variation_item_2.code"
      g.entry '224', :product_sku_variations_3_code, 'SKUバリエーション項目3', :reader => "primary_sku.variation_item_3.code"
      g.entry '225', :product_skus_sku_code, 'SKUコード',                      :reader => "primary_sku.sku_code", :remark => "商品コード-枝番1-枝番2-枝番3"
      g.entry '226', :product_skus_primary_sku_flg, '代表SKUフラグ',           :reader => "primary_sku.primary_sku_flg"
      g.entry '227', :product_skus_jan_code, 'JANコード',                      :reader => "primary_sku.jan_code"
      g.entry '228', :product_skus_shelf_no, '棚番',       :reader => "primary_sku.shelf_no"
      g.entry '229', :product_skus_maker_price, 'メーカー希望価格',            :reader => "primary_sku.maker_price"
      g.entry '230', :product_skus_lead_time, '出荷リードタイム',              :reader => "primary_sku.lead_time"
      g.entry '231', :product_skus_order_limit, '購入数上限',                  :reader => "primary_sku.order_limit"
      g.entry '232', :product_skus_weight, '重量（ｇ）',                       :reader => "primary_sku.weight"
      g.entry '233', :product_skus_usage_days, '何日分', :reader => "primary_sku.usage_days"
      g.entry '234', :product_skus_safety_stock_count, '安全在庫数',           :reader => "primary_sku.safety_stock_count"
      g.entry '235', :product_skus_shipping_cost_point, '送料ポイント',       :reader => "primary_sku.shipping_cost_point"
      g.entry '236', :product_skus_includable_count, '同梱可能数',             :reader => "primary_sku.includable_count"
      g.entry '237', :product_skus_regular_purchase_flg, '定期購入可能フラグ', :reader => "primary_sku.regular_purchase_flg", :remark => "1:可、0または空欄:不可"
      g.entry '238', :product_skus_stockout_display_cd, '在庫切れ時の表示コード', :reader => "primary_sku.stockout_display_cd"
      g.entry '239', :product_skus_parent_stockable_flg, '親商品に在庫を持つかどうか', :reader => "primary_sku.parent_stockable_flg", :remark => "1:持つ、0または空欄:持たない"
      g.entry '240', :product_displays_locale, '言語コード',                   :reader => "current_display.locale"
      g.entry '241', :product_displays_product_name_1, '商品名1',              :reader => "current_display.product_name_1"
      g.entry '242', :product_displays_product_name_2, '商品名2',              :reader => "current_display.product_name_2"
      g.entry '243', :product_displays_product_outline, '商品概要',            :reader => "current_display.product_outline"
      g.entry '244', :product_displays_product_explanation, '商品説明',        :reader => "current_display.product_explanation"
      g.entry '247', :product_displays_recommendation, '推奨メッセージ',       :reader => "current_display.recommendation"
      g.entry '248', :product_displays_recommendation_2, '推奨メッセージ2',    :reader => "current_display.recommendation_2"
      g.entry '249', :product_displays_product_keyword, '検索キーワード',      :reader => "current_display.product_keyword"
      g.entry '250', :product_price_user_ranks_code, '会員ランクコード',       :reader => "primary_sku.primary_price.user_rank.code"
      g.entry '251', :product_prices_standard_price, '標準売価',               :reader => "primary_sku.primary_price.standard_price"
      g.entry '252', :product_prices_regular_purchase_price, '定期購入売価',   :reader => "primary_sku.primary_price.regular_purchase_price"
      g.entry '257', :product_skus_stock_count, '在庫数',                      :reader => "primary_sku.parent_stock_count"
      g.entry '258', :product_skus_cost, '商品原価',                           :reader => "primary_sku.cost"
    end
    
    with_options(:type => :product_component) do |g|
      g.entry '253', :product_components_count, '子商品数量',                  :reader => "primary_sku.primary_component.count"
      g.entry '254', :product_component_skus_code, '子商品SKUコード',          :reader => "primary_sku.primary_component.component_sku.sku_code"
    end
    
    with_options(:type => :order) do |g|
      g.entry '301', :orders_ec_order_code, '受注コード',                             :reader => "ec_order_code"
      g.entry '302', :order_users_user_code, '会員コード',                            :reader => "user.user_code"
      g.entry '303', :orders_ordered_at, '注文日時',                                  :reader => "ordered_at", :format => :date_time
      g.entry '304', :orders_ordered_date, '注文日',                                  :reader => "ordered_at", :format => :date
      g.entry '305', :orders_site_type_div, '購入サイト区分',                         :reader => "site_type_div", :remark => [Order, :site_type_div]
      g.entry '306', :orders_free_request, '自由記入内容',                            :reader => "free_request"
      g.entry '307', :orders_on_line_payment_exe_div, 'クレジット実行区分',           :reader => "on_line_payment_exe_div", :remark => [Order, :on_line_payment_exe_div]
      g.entry '308', :orders_locale, '言語コード',                                    :reader => "locale"
      g.entry '309', :orders_send_invoice_separately_flg, '請求書・納品書別送フラグ', :reader => "send_invoice_separately_flg"
      g.entry '310', :orders_write_sender_flg, '送状への送り主明記フラグ',            :reader => "write_sender_flg"
      g.entry '311', :order_include_order_ec_order_code, '同送先の受注番号',          :reader => "include_order.ec_order_code"
      g.entry '312', :orders_gift_flg, 'ギフトフラグ',                                :reader => "gift_flg"
      g.entry '313', :order_users_name_last, '注文者氏名漢字（姓）',                  :reader => "user.name_last"
      g.entry '314', :order_users_name_first, '注文者氏名漢字（名）',                 :reader => "user.name_first"
      g.entry '315', :order_users_fullname, '注文者氏名漢字（姓 名）',                :reader => "user.fullname"
      g.entry '316', :order_users_kana_name_last, '注文者氏名フリガナ（姓）',         :reader => "user.kana_name_last"
      g.entry '317', :order_users_kana_name_first, '注文者氏名フリガナ（名）',        :reader => "user.kana_name_first"
      g.entry '319', :order_users_onebyte_kana_name_last, '注文者氏名ﾌﾘｶﾞﾅ（姓）',    :reader => "user.kana_name_last", :format => :onebyte_kana
      g.entry '320', :order_users_onebyte_kana_name_first, '注文者氏名ﾌﾘｶﾞﾅ（名）',   :reader => "user.kana_name_first", :format => :onebyte_kana
      g.entry '322', :order_user_addresses_country_code, '注文者国コード',            :reader => "user.user_address.country_code"
      g.entry '323', :order_user_addresses_zip_code, '注文者郵便番号',                :reader => "user.user_address.zip_code"
      g.entry '324', :order_user_addresses_zip_code_jp, '注文者郵便番号（000-0000）', :reader => "user.user_address.zip_code_jp"
      g.entry '325', :order_user_addresses_address_1, '注文者都道府県（住所１）',     :reader => "user.user_address.address_1"
      g.entry '326', :order_user_addresses_address_2, '注文者市区町村（住所２）',     :reader => "user.user_address.address_2"
      g.entry '327', :order_user_addresses_address_3, '注文者番地・マンション名（住所３）',     :reader => "user.user_address.address_3"
      g.entry '329', :order_users_tel_num, '注文者電話番号',                                    :reader => "user.tel_num"
      g.entry '374', :order_payment_order_id, '決済オーダーID',                       :reader => "payment_order_id"
    end
    
    with_options(:type => :order_shipping) do |g|
      g.entry '330', :order_shippings_order_code, '伝票番号',                              :reader => "order_shipping.order_code"
      g.entry '331', :order_shippings_status_div, '注文ステータス',                        :reader => "order_shipping.status_div", :remark => [OrderShipping, :status_div]
      g.entry '332', :order_shippings_name_last, '配送先氏名漢字（姓）',                   :reader => "order_shipping.name_last"
      g.entry '333', :order_shippings_name_first, '配送先氏名漢字（名）',                  :reader => "order_shipping.name_first"
      g.entry '334', :order_shippings_fullname, '配送先氏名漢字（姓 名）',                 :reader => "order_shipping.fullname"
      g.entry '335', :order_shippings_kana_name_last, '配送先氏名フリガナ（姓）',          :reader => "order_shipping.kana_name_last"
      g.entry '336', :order_shippings_kana_name_first, '配送先氏名フリガナ（名）',         :reader => "order_shipping.kana_name_first"
#      g.entry '337', :order_shippings_kana_fullname, '配送先氏名フリガナ（姓 名）',       :reader => "order_shipping.kana_fullname"
      g.entry '338', :order_shippings_onebyte_kana_name_last, '配送先氏名ﾌﾘｶﾞﾅ（姓）',     :reader => "order_shipping.kana_name_last", :format => :onebyte_kana
      g.entry '339', :order_shippings_onebyte_kana_name_first, '配送先氏名ﾌﾘｶﾞﾅ（名）',    :reader => "order_shipping.kana_name_first", :format => :onebyte_kana
#      g.entry '340', :order_shippings_onebyte_kana_fullname, '配送先氏名ﾌﾘｶﾞﾅ（姓 名）',   :reader => "order_shipping.kana_fullname", :format => :onebyte_kana
      g.entry '341', :order_shippings_country_code, '配送先国コード',                      :reader => "order_shipping.country_code"
      g.entry '342', :order_shippings_zip_code, '配送先郵便番号',                          :reader => "order_shipping.zip_code"
      g.entry '343', :order_shippings_zip_code_jp, '配送先郵便番号（000-0000）',           :reader => "order_shipping.zip_code_jp"
      g.entry '344', :order_shippings_address_1, '配送先都道府県（住所１）',               :reader => "order_shipping.address_1"
      g.entry '345', :order_shippings_address_2, '配送先市区町村（住所２）',               :reader => "order_shipping.address_2"
      g.entry '346', :order_shippings_address_3, '配送先番地・マンション名（住所３）',     :reader => "order_shipping.address_3"
#      g.entry '347', :order_shippings_fulladdress, '配送先住所（住所１＋住所２＋住所３）', :reader => "order_shipping.fulladdress"
      g.entry '348', :order_shippings_tel_num, '配送先電話番号',                           :reader => "order_shipping.tel_num"
      g.entry '349', :order_shippings_email, '注文者メールアドレス',                       :reader => "order_shipping.email"
      g.entry '350', :order_shippings_arrival_date, '配送日',                              :reader => "order_shipping.arrival_date", :format => :date
      g.entry '351', :order_shippings_arrival_hour, '配達時間',                            :reader => "order_shipping.arrival_hour", :remark => [OrderShipping, :arrival_hour]
      g.entry '352', :order_shippings_payment_type_div, '支払い方法区分',                  :reader => "order_shipping.payment_type_div", :remark => [OrderShipping, :payment_type_div]
      g.entry '375', :order_shippings_payment_type_code_yamato, '支払い方法区分（ヤマト）', :reader => "order_shipping.payment_type_code_yamato", :remark => "0:発払い、2:代引き" 
      g.entry '353', :order_shippings_delivery_div, '送付区分',                            :reader => "order_shipping.delivery_div", :remark => [OrderShipping, :delivery_div]
      g.entry '354', :order_shippings_complete_sent_mail_flg, 'メール送信完了フラグ',      :reader => "order_shipping.complete_sent_mail_flg"
      g.entry '355', :order_shippings_delivery_code, '発送伝票番号',                       :reader => "order_shipping.delivery_code"
      g.entry '356', :order_shippings_delivery_content, '配送に関する備考',                :reader => "order_shipping.delivery_content"
      g.entry '357', :order_shippings_subtotal_amount, '小計（明細合計金額の合計）',       :reader => "order_shipping.subtotal_amount"
      g.entry '358', :order_shippings_shipping_cost, '送料',                               :reader => "order_shipping.shipping_cost"
      g.entry '359', :order_shippings_discount_amount, '割引額',                           :reader => "order_shipping.discount_amount"
      g.entry '360', :order_shippings_total_amount, '合計金額（小計＋送料－割引額）',      :reader => "order_shipping.total_amount"
    end
    
    with_options(:type => :order_item) do |g|
      g.entry '361', :order_item_product_skus_sku_code, 'SKUコード',                      :reader => "order_shipping.order_item.product_sku.sku_code"
      g.entry '362', :order_items_product_name, '商品名（注文した言語）',                 :reader => "order_shipping.order_item.product_name"
      g.entry '363', :order_items_order_product_name_1, '商品名（日本語）',               :reader => "order_shipping.order_item.order_product_name_1"
      g.entry '364', :order_items_order_product_name_2, '商品名（英語）',                 :reader => "order_shipping.order_item.order_product_name_2"
      g.entry '365', :order_item_products_origin, '原産国',                               :reader => "order_shipping.order_item.product_sku.product.origin"
      g.entry '366', :order_items_qty, '数量',                                            :reader => "order_shipping.order_item.qty"
#      g.entry '367', :order_items_qty_times_made_num, '商品数量（数量×商品の構成数）',   :reader => "order_shipping.order_item.qty_times_made_num"
      g.entry '368', :order_items_unit_price, '単価',                                     :reader => "order_shipping.order_item.unit_price"
#      g.entry '369', :order_items_price_per_made_num, '商品単価（単価÷商品の構成数）',   :reader => "order_shipping.order_item.price_per_made_num"
      g.entry '370', :order_items_subtotal_amount, '明細小計（単価×数量）',              :reader => "order_shipping.order_item.subtotal_amount"
      g.entry '371', :order_items_discount_amount, '明細割引額',                          :reader => "order_shipping.order_item.discount_amount"
      g.entry '372', :order_items_total_amount, '明細合計金額（単価×数量－明細割引額）', :reader => "order_shipping.order_item.total_amount"
      g.entry '373', :order_items_course_div, 'コース区分',                               :reader => "order_shipping.order_item.discount_amount", :remark => [OrderItem, :course_div]
      g.entry '380', :order_items_jan_code, 'JANコード',                                 :reader => "order_shipping.order_item.get_sku.jan_code" 
      g.entry '381', :order_items_shelf_no, '棚番',                                      :reader => "order_shipping.order_item.get_sku.shelf_no" 
      g.entry '382', :order_items_lead_time, '出荷リードタイム',                         :reader => "order_shipping.order_item.get_sku.lead_time" 
      g.entry '383', :order_items_weight, '重量（ｇ）',                                  :reader => "order_shipping.order_item.get_sku.weight" 
      g.entry '384', :order_items_includable_count, '同梱可能数',                        :reader => "order_shipping.order_item.get_sku.includable_count"
      g.entry '385', :order_items_cost, '商品原価',                                      :reader => "order_shipping.order_item.get_sku.cost"
    end

    with_options(:type => :order_item_component) do |g|
      g.entry '386', :order_item_components_product_skus_sku_code, '子商品SKUコード',         :reader => "order_shipping.order_item.order_item_component.self_if_component.product_sku.sku_code"
      g.entry '387', :order_item_components_product_name, '子商品商品名（注文した言語）',     :reader => "order_shipping.order_item.order_item_component.self_if_component.product_name"
      g.entry '388', :order_item_components_order_product_name_1, '子商品商品名（日本語）',   :reader => "order_shipping.order_item.order_item_component.self_if_component.order_product_name_1"
      g.entry '389', :order_item_components_order_product_name_2, '子商品商品名（英語）',     :reader => "order_shipping.order_item.order_item_component.self_if_component.order_product_name_2"
      g.entry '390', :order_item_components_products_origin, '子商品原産国',                  :reader => "order_shipping.order_item.order_item_component.self_if_component.product_sku.product.origin"
      g.entry '391', :order_item_components_qty, '子商品数量',                                :reader => "order_shipping.order_item.order_item_component.self_if_component.qty"
      g.entry '392', :order_item_components_jan_code, '子商品JANコード',                      :reader => "order_shipping.order_item.order_item_component.self_if_component.get_sku.jan_code"
      g.entry '393', :order_item_components_shelf_no, '子商品棚番',                           :reader => "order_shipping.order_item.order_item_component.self_if_component.get_sku.shelf_no"
      g.entry '394', :order_item_components_lead_time, '子商品出荷リードタイム',              :reader => "order_shipping.order_item.order_item_component.self_if_component.get_sku.lead_time"
      g.entry '395', :order_item_components_weight, '子商品重量（ｇ）',                       :reader => "order_shipping.order_item.order_item_component.self_if_component.get_sku.weight"
      g.entry '396', :order_item_components_includable_count, '子商品同梱可能数',             :reader => "order_shipping.order_item.order_item_component.self_if_component.get_sku.includable_count"
      g.entry '397', :order_item_components_cost, '子商品商品原価',                           :reader => "order_shipping.order_item.order_item_component.self_if_component.get_sku.cost"

      g.entry '398', :order_item_components_product_skus_sku_code, '構成商品SKUコード',         :reader => "order_shipping.order_item.order_item_component.product_sku.sku_code"
      g.entry '399', :order_item_components_product_name, '構成商品商品名（注文した言語）',     :reader => "order_shipping.order_item.order_item_component.product_name"
      g.entry '400', :order_item_components_order_product_name_1, '構成商品商品名（日本語）',   :reader => "order_shipping.order_item.order_item_component.order_product_name_1"
      g.entry '401', :order_item_components_order_product_name_2, '構成商品商品名（英語）',     :reader => "order_shipping.order_item.order_item_component.order_product_name_2"
      g.entry '402', :order_item_components_products_origin, '構成商品原産国',                  :reader => "order_shipping.order_item.order_item_component.product_sku.product.origin"
      g.entry '403', :order_item_components_qty, '構成商品数量',                                :reader => "order_shipping.order_item.order_item_component.qty"
      g.entry '404', :order_item_components_jan_code, '構成商品JANコード',                      :reader => "order_shipping.order_item.order_item_component.get_sku.jan_code"
      g.entry '405', :order_item_components_shelf_no, '構成商品棚番',                           :reader => "order_shipping.order_item.order_item_component.get_sku.shelf_no"
      g.entry '406', :order_item_components_lead_time, '構成商品出荷リードタイム',              :reader => "order_shipping.order_item.order_item_component.get_sku.lead_time"
      g.entry '407', :order_item_components_weight, '構成商品重量（ｇ）',                       :reader => "order_shipping.order_item.order_item_component.get_sku.weight"
      g.entry '408', :order_item_components_includable_count, '構成商品同梱可能数',             :reader => "order_shipping.order_item.order_item_component.get_sku.includable_count"
      g.entry '409', :order_item_components_cost, '構成商品商品原価',                           :reader => "order_shipping.order_item.order_item_component.get_sku.cost"
    end

    with_options(:type => :accounting) do |g|
      g.entry '500', :accounting_date, '伝票日付(配送済み・キャンセル完了日付(yyyy/mm/dd))',      :reader => "created_at", :format => :date
      g.entry '501', :accounting_date, '伝票日付(配送済み・キャンセル完了日付(yymmdd))',          :reader => "created_at", :format => :date_join_short
      g.entry '502', :accounting_date, '伝票日付(配送済み・キャンセル完了日付(yyyymmdd))',        :reader => "created_at", :format => :date_join
      g.entry '503', :accounting_debtor_account_code, '借方勘定科目コード(勘定奉行)',     :reader => "debt_account_div_code"
      g.entry '504', :accounting_debtor_account_name, '借方勘定科目名称（弥生）',         :reader => "debt_account_div_output_name"
      g.entry '505', :accounting_debt_sales, '借方金額',                                  :reader => "debt_sales"
      g.entry '506', :accounting_debtor_account_code, '貸方勘定科目コード(勘定奉行)',     :reader => "credit_account_div_code"
      g.entry '507', :accounting_debtor_account_name, '貸方勘定科目名称（弥生）',         :reader => "credit_account_div_output_name"
      g.entry '508', :accounting_credit_sales, '貸方金額',                                :reader => "credit_sales"
      g.entry '509', :accounting_remark, '摘要',                                          :reader => "remark"
    end
  end
  
  # 凡例つきの項目名を返します。ヘッダで使用します
  def name_with_remark
    entry = CsvDownloadPatternItem.code_entry_by_id(code)
    result = name.blank? ? entry[:name] : name
    if remark = entry[:remark]
      remark_str = remark.is_a?(String) ? remark : remark.first.send("#{remark.last.to_s}_entries").map{|e| "#{e[:id]}:#{e[:name]}"}.join("、")
      result = "#{result}（#{remark_str}）"
    end
    return result
  end
  
  def self.options_by_type(type)
    return [] unless type
    case type
      when :user, :user_address
        call_type = [:user]
      when :product, :product_sku, :product_display, :product_price
        call_type = [:product]
      when :product_component
        call_type = [:product, :product_component]
      when :order
        call_type = [:order]
      when :order_shipping
        call_type = [:order, :order_shipping]
      when :order_item
        call_type = [:order, :order_shipping, :order_item]
      when :order_item_component
        call_type = [:order, :order_shipping, :order_item, :order_item_component]
      when :accounting
        call_type = [:accounting]
    end
    codes = CsvDownloadPatternItem.code_hash_array
    codes.find_all{|entry| call_type.include?(entry[:type])}.map{|code| [code[:name], code[:id]]}
  end
  
end
