# -*- coding: utf-8 -*-
class Distributor < ActiveRecord::Base
  has_many :delivery_times, :class_name => "DistributorDeliveryTime", :foreign_key => "distributor_id"
  accepts_nested_attributes_for :delivery_times

  # バリデーション
  validates_presence_of :name
  validates_numericality_of :deliverable_term_start, :only_integer =>true, :message => :EM990003, :if => :deliverable_term_check?
  validates_numericality_of :deliverable_term_end, :only_integer =>true, :message => :EM990003, :if => :deliverable_term_check?

  def validate
    if deliverable_term_check?
      errors.add(:deliverable_term_start, "は0以上を入力してください") if deliverable_term_start.nil? || deliverable_term_start < 0
      errors.add(:deliverable_term_end, "は0以上を入力してください") if deliverable_term_end.nil? || deliverable_term_end < 0
      if !deliverable_term_start.nil? && !deliverable_term_end.nil? && deliverable_term_start > deliverable_term_end
        errors.add(:deliverable_term_end, "は範囲が不正です。")
      end
    end
  end

  #お届け日指定範囲チェックするしない
  def deliverable_term_check?
    return deliverable_term_start || deliverable_term_end
  end

  # お届け時間名称の日本語を配列で返却します。
  def delivery_time_names_for_locale(locale = "")
    delivery_time_names = []
    self.delivery_times.each do |delivery_time|
      delivery_time.delivery_time_displays.each do |display|
        if display.locale == locale
          delivery_time_names << display.name
        end
      end
    end
    return delivery_time_names
  end

  #
  # = 国コードを指定して、配送希望日を配列で返却します。 =
  #
  def self.delivery_date_list(country_code, time_now = Time.now())
    delivery_date_list = []
    country = Country.find_by_code(country_code)

    country ? (distributor_id = country.distributor_id) : (return delivery_date_list)
    distributor_id ? (distributor = self.find(distributor_id)) : (return delivery_date_list)

    # 本来は正しい値が設定されているはずですが、念のためデフォルト値を設定しておきます
    s, e = (distributor.deliverable_term_start ||= 10), (distributor.deliverable_term_end ||= 20)

    # 配送希望日の配列を作成します
    (time_now.to_date + s..time_now.to_date + e).each do |date|
      delivery_date_list << date.strftime("%Y/%m/%d")
    end

    return delivery_date_list
  end
  
  #
  # = 物流管理検索入力フォーム(管理画面用) =
  #
  class SearchInfo < EcRider::ConditionForm
    attr_accessor :distributor_name
    #
    # = 検索条件を作成します。 =
    #
    def to_find_options(options = nil)
      joins = []
      where = Condition.block do |c|
        c.filter(:ignore_blank)
        #物流名称条件
        c.and("distributors.name", "like", "%#{distributor_name}%" )
      end
      result = {:conditions => where}
      result[:joins] = joins.uniq.join(" ")
      result.update(options) if options
      return result
    end
  end
end
