# -*- coding: utf-8 -*-
class Product < ActiveRecord::Base

  class SearchCondition < EcRider::ConditionForm
    SEARCH_ATTRIBUTE_NAMES = [:product_code, :product_name, :variation]
    ATTRIBUTE_NAMES = [:min_stock_count, :max_stock_count, :stock_count, :order, :user_rank_id].concat(SEARCH_ATTRIBUTE_NAMES)
    attr_accessor *ATTRIBUTE_NAMES

    # CSVダウンロード機能で使用しています
    def to_find_options(options = nil)
      joins = []
      where = Condition.block do |c|
        # 商品コード
        c.and("products.product_code", "like", "%#{product_code}%") unless product_code.blank?
        # 商品名
        unless product_name.blank?
          c.and { |c_name|
            c_name.or("products.order_product_name_1", "like", "%#{product_name}%")
            c_name.or("cond_product_displays.product_name_1", "like", "%#{product_name}%")
            joins << "left outer join product_displays cond_product_displays on cond_product_displays.product_id = products.id"
          }
        end
        # バリエーション
        unless variation.blank?
          c.and { |d|
            d.or("variation_1_id", variation.to_i)
            d.or("variation_2_id", variation.to_i)
            d.or("variation_3_id", variation.to_i)
          }
        end
      end
      unless order.nil?
        if user_rank_id.nil? || user_rank_id.blank?
          user_rank = UserRank.find(:first, :conditions=>['no_membership_flg = 1'])
          _user_rank_id = user_rank.id unless user_rank.nil?
        else
          _user_rank_id = user_rank_id.to_i
        end
        joins << "left outer join product_skus cond_product_skus on cond_product_skus.product_id = products.id"
        joins << "left outer join product_prices cond_product_prices on cond_product_prices.product_sku_id = cond_product_skus.id and cond_product_prices.user_rank_id = #{_user_rank_id}"
      end

      #ソート条件
      #標準
      order_condition  = 'products.display_order is null, products.display_order, products.product_code'
      #追加条件
      case  Product::CategorySearchForm.order_key_by_id(order)
        when :low_to_high #価格の安い順
          order_condition = "cond_product_prices.standard_price, #{order_condition}"
        when :high_to_low #価格の高い順
          order_condition = "cond_product_prices.standard_price desc, #{order_condition}"
        when :new_to_old #新着順
          order_condition = "products.publish_started_at desc, #{order_condition}"
        end

      result = {:conditions => where}
      result[:joins] = joins.uniq.join(" ")
      result[:order] = order_condition
      result.update(options) if options
      return result
    end


    # 商品管理検索機能で使用しています
    def to_find_options_for_products_index(options = nil)
      where = Condition.block do |c|
        # 商品コード
        c.and("products.product_code", "like", "%#{product_code}%") unless product_code.blank?
        # 商品名
        unless product_name.blank?
          c.and { |c_name|
            c_name.or("products.order_product_name_1", "like", "%#{product_name}%")
          }
        end
        # バリエーション
        unless variation.blank?
          c.and { |d|
            d.or("variation_1_id", variation.to_i)
            d.or("variation_2_id", variation.to_i)
            d.or("variation_3_id", variation.to_i)
          }
        end
      end

      # 標準順
      order_condition = 'products.display_order is null, products.display_order, products.product_code'

      result = {:conditions => where}
      result[:order] = order_condition
      result.update(options) if options
      return result
    end
  end
end
