class ProductGroupSku < ActiveRecord::Base
  versioned
  belongs_to :product_sku
  belongs_to :product_group
  
  # フロントサイトに表示するための関連商品の一覧を取得する
  #   all : "all"ならば指定されたsku_idを含むかどうかを気にしない
  #   group_name : 指定されていたら、該当する関連商品グループのみを対象にする
  def self.find_by_include_sku_id(sku_id = nil, all = nil, group_name = nil, limit = nil, user_rank_id = nil)
    return [] if all != 'all' && sku_id.nil?
    # 関連商品グループを取得する
    g_where = []
    g_param = []
    # 対象の商品を含む関連商品グループを取得する
    if all != 'all' && sku_id
      g_where << "EXISTS (SELECT * FROM product_group_skus gs WHERE gs.product_group_id = product_groups.id AND gs.product_sku_id = ?)"
      g_param << sku_id
    end
    # 指定した関連商品名の商品グループを取得する
    if group_name
      g_where << "product_groups.name = ?"
      g_param << group_name
    end
    g_order = "product_groups.priority desc, product_groups.id"
    product_groups = ProductGroup.find(:all, :select => 'id', :conditions => [g_where.join(" AND ")] + g_param, :order => g_order)

    where = []
    param = []

    # 環境に従って取得する商品の条件を制御する
    unless EcRider.config.ec_admin
      time_now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
      where << "(products.publish_started_at is null or products.publish_started_at <= ?)"
      where << "(products.publish_finished_at is null or products.publish_finished_at >= ?)"
      param << time_now
      param << time_now
    end
    unless (EcRider.config.ec_test ||  EcRider.config.ec_admin)
      where << '(products.invisible_flg is null or products.invisible_flg = 0)'
    end
    where << "(products.present_flg is null or products.present_flg = 0)"
    where << "products.deleted_at IS NULL AND product_skus.deleted_at IS NULL"
    where << "product_groups.id IS NOT NULL AND product_skus.id IS NOT NULL AND products.id IS NOT NULL"
    where << "product_displays.locale = ?"
    param << "#{I18n.locale || 'ja'}"

    # ユーザランクを取得
    unless user_rank_id
      user_rank = UserRank.find(:first, :conditions=>['no_membership_flg = 1'])
      user_rank_id = user_rank.id unless user_rank.nil?
    end
    where << "product_prices.user_rank_id = ?"
    param << user_rank_id

    include = [:product_group, {:product_sku => [{:product => [:product_displays, :main_image]}, :prices]}]
    s_order = "products.display_order is null, products.display_order, products.product_code"

    result = []
    product_sku_ids = (all != 'all' && sku_id) ? [sku_id] : []
    product_groups.each do |product_group|
      break if limit && (limit.to_i <= result.size)
      s_where = where.clone
      s_param = param.clone
      s_where << "product_groups.id = ?"
      s_param << product_group.id
      # 自身とすでに取得したSKUは含まない
      unless product_sku_ids.empty?
        s_where << "product_group_skus.product_sku_id NOT IN (?)"
        s_param << product_sku_ids
      end
      # 取得する
      s_limit = limit ? (limit.to_i - result.size) : nil
      product_group_skus = ProductGroupSku.find(:all, :include => include, :conditions => [s_where.join(" AND ")] + s_param, :limit => s_limit, :order => s_order)
      result = result + product_group_skus
      product_sku_ids = product_sku_ids + product_group_skus.map(&:product_sku_id)
    end

    # current_displayをセットする
    result.each do |product_group_sku|
      product_group_sku.product_sku.product.current_display = product_group_sku.product_sku.product.product_displays.first
    end

    return result
  end
end
