# -*- coding: utf-8 -*-
class ProductSkuSecureInventory < ActiveRecord::Base
  versioned

  acts_as_paranoid
  
  belongs_to :product_sku
  belongs_to :order_item

  # 注文に紐付く引き当てを取得する
  def self.find_by_order(order_id)
    order_item_ids = OrderItem.find(:all, :select => 'order_items.id', :include => :order_shipping, :conditions => ["order_shippings.order_id = ?", order_id])
    ProductSkuSecureInventory.find(:all, :conditions => ["order_item_id in (?)", order_item_ids])
  end

  # 渡されたSKUのリストに対して仮引き当て数を取得してセットする
  def self.set_unpaid_secure_stock_count(product_skus)
    sku_ids = product_skus.map{|sku| sku.id }
    unpaid_secure_inventories = sum('count', :conditions => ['(paid_flg IS NULL OR paid_flg = 0) AND product_sku_id IN (?)', sku_ids], :group => 'product_sku_id')
    product_skus.each do |sku|
      sku.sku_unpaid_secure_stock_count = unpaid_secure_inventories[sku.id] || 0
    end
    return product_skus
  end
end
