# -*- coding: utf-8 -*-
module SafeEval
  
  def safe_instance_eval(script, locals, safe_level = 3)
    local_variables = locals.keys.map{|key| "#{key.to_s} = locals[#{key.inspect}]"}.join(';')
    instance_eval(local_variables) 
    result = nil

    # 
    # この内部で、モデルの属性に初めてアクセスすると、
    #   activerecord-2.3.2/lib/active_record/attribute_methods.rb:211
    # でclass_evalが実行され、Securityエラーが発生します。
    # これを防ぐために、Thread内でclass_evalを実行しうる処理を回避させるメソッド prepare_to_safe_eval をここで呼び出します。
    #
    locals.each do |key, value|
      value.send(:prepare_to_safe_eval) if value.respond_to?(:prepare_to_safe_eval)
    end
    
    Thread.start do
      $SAFE = safe_level
      begin
        result = !!instance_eval(script, __FILE__, __LINE__)
      rescue Exception
        puts "\n\n\n-----script:\n" << script << "\n-----"
        puts $!.class.name
        puts $!
        puts $!.backtrace.join("\n")
        raise
      end
    end.join
    result
  end
  
  module ArModel
    def prepare_to_safe_eval
      unless self.class.instance_variable_get(:@safty_eval_prepared)
        self.class.columns.each{|col| send(col.name) }
        self.class.instance_variable_set(:@safty_eval_prepared, true)
      end
    end
  end


end
