# -*- coding: utf-8 -*-
class ShippingCost < ActiveRecord::Base
  versioned
  belongs_to :shipping_cost_boundary
  
  ################
  # バリデーション定義
  ################
  validates_numericality_of :shipping_cost, :greater_than_or_equal_to => 0, :only_integer => true, :allow_nil => true
  validates_presence_of :shipping_cost
  # エリア区分
  selectable_attr :area_type_div do
    entry "1", :country, '国'
    entry "2", :prefecture, '都道府県'
  end

  # 送料管理で使用
  attr_accessor :boundary_index

  # user_address
  #   具体的には UserAddress か OrderShipping を期待しています。
  # 
  # 住所情報、商品の合計重量から送料を計算し、返却します。
  #
  # 送料キャンペーンが適用できる場合、
  # 　・国が日本であり、店舗情報で｢遠隔地・離島の送料設定｣が設定されていない場合、
  # 　  キャンペーンの送料を適用します。(現時点では送料キャンペーンは送料無料のみ)
  # 　・国が日本であり、店舗情報で｢遠隔地・離島の送料設定｣が設定されている場合、
  #     キャンペーンの送料　+　遠隔地・離島の送料　を送料として返却します。
  # 送料キャンペーンがない場合、
  # 　・国が日本であり、店舗情報で｢遠隔地・離島の送料設定｣が設定されていない場合、
  # 　　　住所の県から送料を取得し、商品SKUの送料ポイントから該当する送料を返却します。
  # 　・国が日本であり、店舗情報で｢遠隔地・離島の送料設定｣が設定されている場合、
  # 　　送料に設定値を加算します。
  #
  def self.calc_shipping_cost(user_address = nil, point = nil, usable_campaign = false)
    shipping_cost = 0
    #　離島追加料金がある場合、追加料金を取得する
    outlier_cost = ShopSetting.shipping_cost_outlier(user_address.zip_code)
    unless user_address.nil? || point.nil?
      if usable_campaign
        # 離島 離島のみ送料追加
        shipping_cost = outlier_cost if ShopSetting.get_setting(:campaign_shipping_cost, I18n.locale) == "1"
      else
        # 日本の場合、都道府県から送料を取得
        if user_address.country_code == "JP" && !user_address.address_1.nil? && !user_address.address_1.blank?
          shipping_costs = Prefecture.find_by_name(user_address.address_1).shipping_costs
        else
          shipping_costs = user_address.country.shipping_costs
        end
        # 商品重量より送料を計算
        shipping_costs.each do |cost|
          if cost.shipping_cost_boundary.point.blank?
            shipping_cost = cost.shipping_cost
          else
            if point.to_i < cost.shipping_cost_boundary.point.to_i
              shipping_cost = cost.shipping_cost
              break
            end
          end
        end
        # 送料 + 離島追加送料を合算
        shipping_cost = (shipping_cost || 0) + (outlier_cost || 0)
        # TODO 同送の場合の処理が未実装
      end
    end
    return shipping_cost
  end
end
