# -*- coding: utf-8 -*-
class ShopSetting < ActiveRecord::Base
  versioned

  set_table_name 'shop_settings'

  validates_uniqueness_of :setting_div, :scope => :locale
  validates_presence_of :setting_div
  validates_length_of :value, :maximum => 2000, :allow_nil => true

  selectable_attr :setting_div do
    entry '0000', :shop_name    , '店舗名称'    , :required => true, :hint => <<-"END_OF_HINT" do
      ※ 16文字以内で指定してください。
      入力例：　EC-Rider商会
    END_OF_HINT
      def validate(shop_setting)
        # コンビニ決済処理でも使用するため、文字数を16文字以内に制限
        if shop_setting.value.split(//u).length > 16
          shop_setting.errors.add(:value, :too_long, :count => 16)
        end
      end
    end
    entry '0001', :shop_zip_code, '店舗郵便番号', :required => true, :hint => "入力例：　810-0001"
    entry '0002', :shop_address , '店舗住所'    , :required => true, :hint => <<-"END_OF_HINT"
      入力例：
      　名古屋市中区丸の内二丁目9-40
      　EC_RIDERビル101階
    END_OF_HINT
    entry '0003', :shop_tel_num, '店舗電話番号', :required => true, :hint => "入力例：　092-123-1234"
    entry '0004', :shop_logo   , '店舗ロゴ'    , :type => :file do
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
      end
    end
    entry '0005', :shop_mail   , '店舗メールアドレス', :required => true, :hint => "入力例：　tenpo_address@ecrider.com" do
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/
          shop_setting.errors.add(:value, :EM990007)
        end
        if shop_setting.value.length > 255
          shop_setting.errors.add(:value, :too_long, :count => 255)
        end
      end
    end
    entry '0007', :shipment_instruction_mail, '出荷指示メール送信先メールアドレス', :required => true, :hint => <<-"END_OF_HINT" do
      入力例：　shukka_shiji1@deliver.com, shukka_shiji2@deliver.com, shukka_shiji3@deliver.com"
      END_OF_HINT
 		  def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+((\s)*,(\s)*([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+))*$/
          shop_setting.errors.add(:value, :EM990007)
        end
        if shop_setting.value.length > 255
          shop_setting.errors.add(:value, :too_long, :count => 255)
        end
      end
    end
    entry '0008', :top_action, 'トップページアクション', :default => 'index', :type => :action, :hint => <<-"END_OF_HINT"
      ※ ページ管理で登録したページの「URL上の名前」を指定します。
      ※ 指定したページが、ショッピングサイトのトップページとして表示されます。
    END_OF_HINT
    entry '0011', :continue_shopping_action, '買い物を続けるアクション', :default => 'list', :type => :action, :hint => <<-"END_OF_HINT"
      ※ ページ管理で登録したページの「URL上の名前」を指定します。
      ※ 指定したページが、カートの中で「買い物を続ける」ボタンを押したときに表示されます。
    END_OF_HINT
    entry '0012', :policy_confirmation_page , '会員登録前の個人情報ポリシーのCMSのテキストページ', :default => 'policy_text', :type => :action, :hint => <<-"END_OF_HINT"
      ※ ページ管理で登録したページの「URL上の名前」を指定します。
      ※ 指定したファイルが「個人情報の収集についてのお知らせ」画面で、テキストエリアに表示されます。
      ※ アンダースコア( _ )で始まるページを選択してください。
    END_OF_HINT
    entry '0013', :site_name, 'サイト名称', :required => true, :hint => "入力例：　EC-Riderショッピングサイト"
    entry '0015', :exchange_rate, '換算レート', :hint => <<-"END_OF_HINT" do
      ※ 日本円からの変換レートを入力します。
      入力例：　0.0735
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^\d+(\.\d+)?$/
          shop_setting.errors.add(:value, :EM990003)
        end
      end
    end
    entry '0016', :rounding_processing, '換算後の金額の端数の処理', :type => :rounding_processing, :default => 'round_up', :hint => <<-"END_OF_HINT" do
      ※ 端数の処理方法を指定します。
      ※ 切り上げ：「round_up」を指定します。
      ※ 切り捨て：「round_down」を指定します。
      ※ 四捨五入：「round_off」を指定します。
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
      end
    end
    entry '0017', :aliyap_product_name, 'アリペイ画面の商品名称', :hint => <<-"END_OF_HINT" do
      ※ アリペイ口座側画面で表示される商品名称を指定します。
      ※ 簡体字で入力してください。
      ※ 指定しない場合は「Web－Shopping」となります。
    END_OF_HINT
      def check_convert_to_gbk(str)
        begin
          Iconv.iconv('GBK', 'UTF-8', str).to_s
          # 入力か出力の文字が指示された文字集合に含まれないために変換が停止した場合
          rescue Iconv::IllegalSequence => illegal_sequence
            return false
        end
        true
      end

      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        return shop_setting.errors.add(:value, 'は簡体字で入力してください。') unless check_convert_to_gbk(shop_setting.value)
        # GBK変換後にBASE64変換した文字数が1500を超えてはいけないが、255文字のバリデーションがあるため超えることは無いのでチェックは不要。
        if shop_setting.value.length > 255
          shop_setting.errors.add(:value, :too_long, :count => 255)
        end
      end
    end
    entry '0018', :payment_cancel_mail, '決済取り消し依頼メール送信先メールアドレス', :required => true, :hint => <<-"END_OF_HINT" do
      ※ アリペイ決済エラー時に送信する、決済取り消し依頼メールの送信先を指定します。
      入力例：　payment_cancel1@deliver.com, payment_cancel2@deliver.com, payment_cancel3@deliver.com"
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+((\s)*,(\s)*([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+))*$/
          shop_setting.errors.add(:value, :EM990007)
        end
        if shop_setting.value.length > 255
          shop_setting.errors.add(:value, :too_long, :count => 255)
        end
      end
    end
    entry '0020', :shipment_complete_mail, '配送完了メール（店舗向け）送信先メールアドレス', :required => true, :hint => <<-"END_OF_HINT" do
      入力例：　shukka_kanryou1@deliver.com, shukka_kanryou2@deliver.com, shukka_kanryou3@deliver.com
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+((\s)*,(\s)*([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+))*$/
          shop_setting.errors.add(:value, :EM990007)
        end
        if shop_setting.value.length > 255
          shop_setting.errors.add(:value, :too_long, :count => 255)
        end
      end
    end
    entry '0021', :entrance_action, 'サイト入り口アクション', :default => 'index', :type => :action, :hint => <<-"END_OF_HINT"
      ※ ページ管理で登録したページの「URL上の名前」を指定します。
      ※ 指定したページが、ショッピングサイトのサイト入り口ページとして表示されます。
      ※ サイト入り口アクションを登録した場合、トップページアクション名も登録してください。
    END_OF_HINT
    entry '0022', :user_form_gender, '会員情報性別入力必須', :type => :switch, :hint => '会員情報入力フォームで性別を必須入力にするかを設定します。trueの場合必須入力になります。'
    entry '0023', :user_form_birthday, '会員情報生年月日入力必須', :type => :switch, :hint => '会員情報入力フォームで生年月日を必須入力にするかを設定します。trueの場合必須入力になります。'
    entry '0024', :user_form_job, '会員情報職業入力必須', :type => :switch, :hint => '会員情報入力フォームで職業を必須入力にするかを設定します。trueの場合必須入力になります。'
    entry '0025', :shop_mail_test   , 'テスト環境用店舗メールアドレス', :required => true, :hint => "入力例：　test_tenpo_address@ecrider.com" do
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/
          shop_setting.errors.add(:value, :EM990007)
        end
        if shop_setting.value.length > 255
          shop_setting.errors.add(:value, :too_long, :count => 255)
        end
      end
    end
    entry '0026', :default_address_country   , '住所登録時の国選択セレクトボックスの初期値', :default => 'JP', :type => :country, :hint => '会員情報入力フォームで国選択の初期に表示する国を設定します。'
    entry '0027', :campaign_outlier   , '遠隔地・離島の送料設定', :required => true, :hint => <<-"EOS"
      ※ 遠隔地・離島の送料を指定します。
      ※ 郵便番号,送料(円)　の書式で入力します。
      ※ 複数の地域を登録する場合は、改行し列挙してください。
      入力例： 1002101,450[改行]
               1002211,500
    EOS
    entry '0028', :campaign_shipping_cost   , '送料無料キャンペーン対応方法', :default => 1, :hint => <<-"EOS"
      ※ 送料無料キャンペーンの遠隔地・離島の送料設定を行います。
      ※ 遠隔地・離島の送料も無料にする場合⇒０（ゼロ）
      ※ 遠隔地・離島の別途送料のみ請求する場合⇒１

      ※ 設定しない場合は「１」が設定されます
    EOS
    entry '0029', :cod_charge, '代金引換手数料', :hint => <<-"END_OF_HINT" do
      ※ 代金引換の手数料を日本円で入力します。
      入力例：　315
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^\d+(\d+)?$/
          shop_setting.errors.add(:value, "は整数で入力してください。")
        end
      end
    end
    
    entry '0030', :last_tags_in_front_body, "Google Analyticsなど、bodyタグに必ず付加するタグ", :hint => <<-"EOS" do
      ※ Google Analyticsなどアクセス解析のためのスクリプトを設定できます。アクセス解析以外の用途では使用はお控えください。
      ※ (Google Analyticsについては http://www.google.com/analytics/ をご参照ください)
      ※ ここで設定されたものは本番環境でのみ出力され、テスト環境では出力されません。
      ※ 2000文字まで入力可能です。
      ※ <script type="text/javascript"> から </script> までを入力してください。
    EOS
      def validate(shop_setting)
        unless shop_setting.value.blank?
          unless CmsXml.validate_tags(shop_setting.value)
            shop_setting.errors.add(:value, "のXMLの記述が不正です。")
          end
        end
      end
    end
    
    entry '0031', :meta_tags_for_seo, "GoogleウェブマスターなどのSEO対策用のMETAタグ", :hint => <<-"EOS" do
      ※ Googleウェブマスターなど。アクセス解析以外の用途では使用はお控えください。
      ※ (Googleウェブマスターについては http://www.google.com/webmasters をご参照ください)
      ※ ここで設定されたものは本番環境でのみ出力され、テスト環境では出力されません。
      ※ 2000文字まで入力可能です。
      例）
      <meta name="xxxxx1" content="xxxxxxxx"/>
      <meta name="xxxxx2" content="xxxxxxxx"/>

      ※ 各metaタグは /> で閉じるようにしてください。
      ○ <meta name="xxxxx" content="xxxxxxxx"/>
      × <meta name="xxxxx" content="xxxxxxxx">
    EOS
      def validate(shop_setting)
        unless shop_setting.value.blank?
          unless CmsXml.validate_tags(shop_setting.value)
            shop_setting.errors.add(:value, "のXMLの記述が不正です。タグが正しく閉じられているかご確認ください。")
          end
        end
      end
    end
    entry '0032', :user_form_pc_mail_magazine, '会員情報メールマガジン入力必須', :type => :switch, :hint => '会員情報入力フォームでメールマガジン送信の有無を必須入力にするかを設定します。trueの場合必須入力になります。'
    entry '0033', :settlement_inquiries, '決済時お問合せ先', :required => true, :hint => <<-"END_OF_HINT" do
      電子決済時、決済代行会社に送信されるお問合せ先です。店舗名、部署名等を指定してください。
      ※ 半角記号「^ ` { | } ~ & < > " '」は使用できません。
      ※ 42文字以内（全角の場合は21文字以内）で指定してください。
      入力例：　お客様サポートセンター
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^[^\^\`\{\|\}\~\&\<\>\"\']+$/
          shop_setting.errors.add(:value, :EM990007)
        end
        # Shift_JISで42Byte以内
        if Iconv.iconv('Shift_JIS', 'UTF-8', shop_setting.value).to_s.length > 42
          shop_setting.errors.add(:value, :EM990030)
        end
      end
    end
    entry '0034', :settlement_inquiries_tel_num, '決済時お問合せ先電話番号', :required => true, :hint => <<-"END_OF_HINT" do
      電子決済時、決済代行会社に送信されるお問合せ先電話番号です。
      ※ 半角数字とハイフン( - )のみ使用可能です。
      入力例：　092-123-1234
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^[\d,\-]+$/
          shop_setting.errors.add(:value, :EM990007)
        end
        if shop_setting.value.length > 12
          shop_setting.errors.add(:value, :too_long, :count => 12)
        end
      end
    end
    entry '0035', :settlement_inquiries_time_in, '決済時お問合せ先受付時間', :required => true, :hint => <<-"END_OF_HINT" do
      電子決済時、決済代行会社に送信されるお問合せ先受付時間です。
      ※ 「2桁:2桁-2桁:2桁」書式で指定してください。
      入力例：
      09:00-18:00
    END_OF_HINT
      def validate(shop_setting)
        return shop_setting.errors.add(:value, :blank) if shop_setting.value.blank?
        unless shop_setting.value =~ /^\d{2}:\d{2}\-\d{2}:\d{2}$/
          shop_setting.errors.add(:value, :EM990007)
        end
      end
    end
    entry '0036', :security_code, 'クレジットカード決済 セキュリティコードを利用する', :type => :switch, :hint => <<-"END_OF_HINT" do
      クレジットカード決済時に、セキュリティコード(クレジットカード裏面に印字されている末尾3桁または4桁の数字)の入力をするか否かを設定します。
      設定のない場合はセキュリティコードを利用しない決済となります。
    END_OF_HINT
    end

    entry '0037', :user_form_login_id, 'ログインIDの使用', :type => :switch, :default => 'false', :hint => <<-"END_OF_HINT" do
      ログイン時のIDとしてログインIDを使用するかどうかを指定します。
      ※ ログインIDを使用する場合は「true」を、使用しない場合は「false」を指定してください。
    END_OF_HINT
    end
  end

  selectable_attr :rounding_processing_list do
    entry 'round_up', :round_up, '切り上げ'
    entry 'round_down', :round_down, '切り捨て'
    entry 'round_off', :round_off, '四捨五入'
  end

  def validate
    super
    if errors.on(:value).nil? or errors.on(:value).empty?
      if entry = setting_div_entry
        entry.validate(self) if entry.respond_to?(:validate)
      end
    end
  end



  def self.[](setting_div_key)
    get_setting(setting_div_key, I18n.locale)
  end

  #
  # 設定区分と言語に紐づく値を取得する
  #
  def self.get_setting(setting_div_key, locale, options = {})
    seting_div_entry = setting_div_entry_by_key(setting_div_key)
    raise ArgumentError, "Unknown setting_div_key: #{setting_div_key.inspect}" if seting_div_entry.null?
    setting = find(:first,
      :order => 'locale desc', # localeがnullのレコードが最後に来るようにdesc。
      :conditions => [
        'setting_div = ? AND (locale = ? OR locale is null OR locale = \'\')',
        setting_div_id_by_key(setting_div_key),
        locale.to_s])

    # urlを取得する
    if options[:value] == :url
      setting ? setting.url : nil
      # valueを取得する
    else
      setting ? setting.value : seting_div_entry[:default]
    end
  end

  #
  # =SearchInfo=
  #
  # 検索条件を保持するためのクラス
  # 下記のアクセッサーを持つ
  #
  class SearchInfo < ActiveForm
    attr_accessor :setting_div, :locale, :disp_count

    def to_paginate_options(options = {})
      {
        :per_page => disp_count || 20,
        :conditions => create_condition,
        :order => 'setting_div, locale'
      }.update(options || {})
    end

    #
    # 検索条件生成
    # 
    def create_condition
      where = Condition.block do |c|
        # 項目の条件
        c.and("setting_div", "#{self.setting_div}") unless self.setting_div.blank?
        # 言語の条件
        c.and("locale", "#{self.locale}") unless self.locale.blank?
      end
      return where
    end
  end

  #
  # ページ管理でアップロードされた画像の一覧を返す
  #
  def self.image_list
    # 商品画像として選択可能な画像の拡張子
    selectable_extensions = ['.gif', '.jpg', '.jpeg', '.png']

    base_path = Pathname.new(File.join(EcRider.config.ec_front_root, 'public')).realpath
    base_url  = EcRider.config.ec_front_url

    real_file_pathes = Dir.glob(File.join(base_path, 'cms', '**', '*')).sort

    result = []
    real_file_pathes.each do |path|
      if selectable_extensions.include?(Pathname.new(path).extname.downcase) && /^\S*\-\d{14}\.\S*$/ !~ path
        result << path.gsub(base_path, '')
      end
    end
    result
  end

  #
  # setting_div_entry[:type]がfileのデータに対して、登録されている相対パスを絶対パスのURLにして返す
  #
  def url
    if value.nil? || setting_div_entry[:type] != :file
      return nil
    end

    return File.join(EcRider.config.ec_front_url, value)
  end

  #
  # = 個人情報ポリシーのCMSのテキストページファイルのパスを返します。 =
  #
  def self.policy_confirmation_page_path
    policy_page = ShopSetting.get_setting(:policy_confirmation_page, I18n.locale.to_s)
    if I18n.locale.to_s == 'ja'
      file_name = '%s.%s' % [policy_page, 'html.erb']
    else
      file_name = '%s_%s.%s' % [policy_page, I18n.locale.to_s.gsub("-", "_"), 'html.erb']
    end
    root_dir = EcRider.config.ec_test ? EcRider.config.ec_test_root : EcRider.config.ec_front_root
    unless File.exist?(File.join(root_dir, 'app', 'views', 'cms', file_name))
      file_name = '_policy_text.html.erb'
    end
    File.join('/cms', file_name)
  end

  #
  # 換算値を掛けた値を返します（小数点第2位まで）
  #
  def self.exchange(value)
    rate = ShopSetting.get_setting(:exchange_rate, I18n.locale)
    type = ShopSetting.get_setting(:rounding_processing, I18n.locale)
    return nil if !(value.is_a? Numeric) || rate.nil?

    result = value * rate.to_f
    case type
    when 'round_up'
      result = result.ceil.to_f
    when 'round_down'
      result = result.truncate.to_f
    when 'round_off'
      result = result.round.to_f
    end
    result.to_i
  end

  # 郵便番号から離島の追加送料設定を返却します。
  # 離島の追加設定が無い場合はnilを返却します
  def self.shipping_cost_outlier_or_nil(zipcode = "")
    setting = ShopSetting[:campaign_outlier]
    return nil unless setting
    
    setting.split(/\n/).compact.each do |values|
      params = values.split(',')
      return params[1].strip.to_i if params[0].strip == zipcode
    end
    return nil
  end
  # 離島の追加設定がない場合は0を返却します。
  def self.shipping_cost_outlier(zipcode = "")
    shipping_cost_outlier_or_nil(zipcode).to_i
  end
end
