# -*- coding: utf-8 -*-

require 'ec_rider/config'

class SitemapPage

  # ActiveRecord::BaseやActionFormを継承するようにしたらこのinitializeメソッドは消してください。
  def initialize(attributes = {})
    attributes.each do |attr, value|
      self.send("#{attr}=", value)
    end
  end

  # 以下の属性については、下記のサイトを参照してください
  # http://www.tafcue.com/2009/05/seo-xml-sitemap001.html
  attr_accessor :location         # urlset/url/loc        httpなどから始まるURI
  attr_accessor :last_modified_on # urlset/url/lastmod    YYYY-MM-DD
  attr_accessor :change_freq      # urlset/url/changefreq always/hourly/daily/weekly/monthly/yearly/never のいずれか
  attr_accessor :priority         # urlset/url/priority   0.0 〜 1.0 までの実数値の10進文字列


#
# cms以外のページ
#
  def self.default_pages(options = {},&url_for)
    unless @default_pages
      @default_pages = []
      other_pages = []
      locale_ignoreds = {}
      public_pages = CmsPage.primary_pages(options || {})
      # thanksはビューのみ存在していてコントローラは存在しないため除外する
      public_pages.each {|public_page| other_pages << public_page unless public_page.controller == 'cms' || public_page.controller == 'thanks' ||
        public_page.action.nil? || public_page.action.empty? || public_page.action =~ /^_/}
      other_pages.each do |other_page|
        locale_ignored_key = [other_page.controller, other_page.action]
        locale_ignoreds[ locale_ignored_key ] ||= {}
        I18n.supported_locales.map(&:to_s).each do |locale|
          locale_ignoreds[ locale_ignored_key ][locale.to_s] = other_page
          @default_pages << SitemapPage.new(
            # url_forを使いたいところですが、これはモデルでは使えないメソッドなのでブロックで渡してもらうことにします
            :location => url_for.call(:controller => other_page.controller, :action => other_page.action, :lang => locale),
            :last_modified_on => Date.new(2009, 5, 14), # TODO 本当は固定値じゃまずいと思うんですが・・・・
            :change_freq => :never, # 非CMSページはクローラーに拾ってほしくないので、neverにします
            :priority => 0.3
            )
        end
      end
    end
    @default_pages
  end

  #
  #cms_pageのコントローラがcmsでindexアクション以外の場合
  #
  def self.cms_pages(options = {}, &url_for)

    locale_ignoreds = {} # :langなしでurlを作るためにCmsPageオブジェクトを覚えておくHash
    result = []
    cms_pages = []
    top_action = ShopSetting.get_setting(:top_action,  I18n.locale)
    public_pages = CmsPage.primary_pages(options || {})
    public_pages.each {|public_page| cms_pages << public_page if public_page.controller == 'cms' &&
      !(public_page.action.nil? || public_page.action.empty? || public_page.action =~ /^_/)}
      
    cms_pages.each do |cms_page|
      locale_ignored_key = [cms_page.controller, cms_page.action]
      locale_ignoreds[ locale_ignored_key ] ||= {}
      I18n.supported_locales.map(&:to_s).each do |locale|
        locale_ignoreds[ locale_ignored_key ][locale.to_s] = cms_page
        if cms_page.controller == 'cms' && cms_page.action != top_action
          result << SitemapPage.new(
            # url_forを使いたいところですが、これはモデルでは使えないメソッドなのでブロックで渡してもらうことにします
            :location => url_for.call(:controller => cms_page.controller || 'cms',
            :action => cms_page.action, :lang => locale.to_s),
            :last_modified_on => cms_page.cms_project.started_at, 
            :change_freq => :weekly,
            :priority => 0.5
            )
        else
          #サイトトップページ
          result << SitemapPage.new(
            # url_forを使いたいところですが、これはモデルでは使えないメソッドなのでブロックで渡してもらうことにします
            :location => url_for.call(:controller => 'cms', :action => top_action, :lang=>locale),
            :last_modified_on => Date.today,
            :change_freq => :daily, # 非CMSページはクローラーに拾ってほしくないので、neverにします
            :priority => 1.0
            )
        end
      end
    end
    result

#言語コードを持たないURLもSitemapに載せる場合はコメントアウトを外す
#    locale_ignored_saitemap_pages = locale_ignoreds.map do |(key, cms_page_hash)|
#      primary_locale = locales.detect{|locale| !!cms_page_hash[locale.to_s] }
#      cms_page = cms_page_hash[ primary_locale ]
#      cms_page.nil? ? nil :
#        SitemapPage.new(
#          # url_forを使いたいところですが、これはモデルでは使えないメソッドなのでブロックで渡してもらうことにします
#          :location => url_for.call(:controller => cms_page.controller_name || 'cms',
#            :action => cms_page.action_name),
#          :last_modified_on => Date.new(2009, 5, 14), # TODO 本当は固定値じゃまずいと思うんですが・・・・
#          :change_freq => :weekly,
#          :priority => 0.5
#          )
#    end
#    locale_ignored_saitemap_pages.compact + result

  end

  #
  # ecrider提供ページ
  #
  def self.ecrider_pages(&url_for)
    unless @default_pages
      @default_pages = []

      controller_names = []
      controllers = Dir[File.join(RAILS_ROOT, 'app', 'controllers', '*.rb')]
      controllers.each do |path|
        controller_names << File.basename(path).gsub(/_controller\.rb$/, '')
      end
      controller_names.each do |controller_name|
        controller = "#{controller_name.camelize}Controller".constantize
        action_names = controller.action_methods.map
        action_names.each do |action_name|
          I18n.supported_locales.each do |locale|
            @default_pages << SitemapPage.new(
              # url_forを使いたいところですが、これはモデルでは使えないメソッドなのでブロックで渡してもらうことにします
              :location => url_for.call(:controller => controller_name.to_sym, :action => action_name, :lang => locale.to_s),
              :last_modified_on => Date.new(2009, 5, 14), # TODO 本当は固定値じゃまずいと思うんですが・・・・
              :change_freq => :never, # 非CMSページはクローラーに拾ってほしくないので、neverにします
              :priority => 0.3
            )
          end
        end
      end
    end
    @default_pages
  end
end
