# -*- coding: utf-8 -*-
class UserEmail < ActiveRecord::Base
  include EcRider::PluginModule::Extendable
  versioned

  belongs_to :user

  # メールアドレスサイト区分：メールアドレスがPCサイト用のものかモバイルサイト用のものかを判別する区分
  # ※PCサイト用のメールアドレスとして、モバイルのドメインのアドレスも登録可能。
  selectable_attr :pc_mobile_div do
    entry '1', :pc, 'PC'
    entry '2', :mobile, 'モバイル'
  end

  # メールアドレスドメイン区分：メールアドレスのドメインがPCメールのものかモバイルメールのものかを判別する区分
  selectable_attr :mail_address_pc_mobile_div do
    entry '1', :pc, 'PC'
    entry '2', :mobile, 'モバイル'
  end

  validates_presence_of :email,  :message => :EM990001, :if => :email_check?
  validates_format_of   :email,  :with => /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/,   :message => :EM990007, :if => :email_check?

  # メールアドレスのアカウント(@の前)を返す
  def account
    email[0..email.index("@")-1]
  end
  
  # メールチェックする場合、trueを返す
  def email_check?
    # フロントサイトから登録された場合は常に必須項目。
    # 管理サイトから登録された場合はWEB会員のみ必須。
    EcRider.config.ec_front || EcRider.config.ec_test || !self.email.nil? && !self.email.blank? || !self.user.nil? && self.user.is_membership?
  end
  # メールアドレスのドメイン(@の後ろ)を返す
  def career
    email[email.index("@")..email.size]
  end

  def validate
    errors.add(:email, :NM990009) unless is_unique_email?
  end

  def before_save
    self.mail_address_pc_mobile_div = UserEmail.check_by_emall(email)
  end

  # メールアドレスがユニークな場合はtrueを返す
  #　会員ユーザーのみが対象。非会員、退会会員の重複は許可します。
  #　本番会員とテスト会員の重複は許可します。
  def is_unique_email?
    if EcRider.config.ec_test
      conditions = ['user_emails.email = ? and member_div = ? and user_emails.user_id <> ? and test_flg = ?', self.email, User.member_div_id_by_key(:membership), self.user_id || '', true]
    else
      conditions = ['user_emails.email = ? and member_div = ? and user_emails.user_id <> ? and test_flg is null', self.email, User.member_div_id_by_key(:membership), self.user_id || '']
    end
    user_count = User.count(
      :joins => "LEFT OUTER JOIN user_emails ON user_emails.user_id = users.id  AND (user_emails.deleted_at IS NULL OR user_emails.deleted_at > '#{Time.now.strftime('%Y-%m-%d %H:%M:%S')}')" ,
      :conditions => conditions
    )
    return (user_count <= 0) || (self.user.nil?) || !(self.user.is_membership?)
  end


  # 指定ユーザーのメールアドレス情報を返す
  def self.find_by_user_id(user_id)
    raise ArgumentError, "Illegal arguments! user_id is nil" unless user_id
    self.find(:first, :conditions => ['user_id = ?', user_id])
  end

  MOBILE_DOMAIN_PATTERN = /^[^@]+@(docomo|ezweb|softbank|[^\.]+\.pdx|[^\.]+\.vodafone).ne.jp$/

  # メールアドレスが携帯のアドレスかどうかを判定しPCモバイル区分で返す。
  def self.check_by_emall(email)
    UserEmail.mail_address_pc_mobile_div_id_by_key(
      email =~ MOBILE_DOMAIN_PATTERN ? :mobile : :pc)
  end

end
