# -*- coding: utf-8 -*-

#
# = ec_testかどうかの判断 =
# 以下の条件のどちらかに当てはまればec_testへのデプロイとして動きます。
#  * 環境変数ECR_TEST_ENVがtrueが設定されている
#   * .bash_profileに設定する以外にも、以下のような実行でも環境変数は設定されます。
#    * ECR_TEST_ENV=true cap deploy
#  * config/ec_front.yml に以下が設定されている
#   * ec_test: true
#
ec_test = false
if (ENV['ECR_TEST_ENV'] || 'false') != 'false'
  ec_test = true
elsif File.exist?(File.join('.', 'config', 'ec_front.yml'))
  ec_front_setting = YAML::load_file(File.join('.', 'config', 'ec_front.yml'))
  ec_test = ec_front_setting["ec_test"]
end
$ecr_application_name_in_repository = "ec_front".freeze
$ecr_application_name = (ec_test ? "ec_test" : "ec_front").freeze

# config/deploy_common.rbの読み込み
# requireを使うとスクリプトを実行するコンテキストが変わってしまうのでNGです。
deploy_common = File.join(File.dirname(__FILE__), 'deploy', 'deploy_common.rb')
instance_eval(IO.read(deploy_common), deploy_common, 0)

# ec_fron の deploy時の問題
# capistranoがsvnからデータを取得する際にまずec_frontの最新のリビジョンを
# 取得してそのリビジョンの更新を取得しますが
# ファイルの更新は、ec_frontのリビジョンとして認識されないので、その場合ec_front内の
# ファイルに空行を加えるなどの変更を行いec_frontの最新のリビジョンを更新する必要があります。

namespace :mongrel do
  desc "Setup mongrel_cluster.yml to /etc/mongrel_cluster"
  task :copy_mongrel_cluster_yml do
    sudo "mkdir -p /etc/mongrel_cluster"
    sudo "cp config/mongrel_cluster_#{$ecr_application_name}.yml /etc/mongrel_cluster/#{$ecr_application_name}.yml"
  end
end
after "deploy:setup", "mongrel:copy_mongrel_cluster_yml"




namespace :cms do

  path_to_tmp_inquiry_images = "public/tmp_inquiry_images"

  if ec_test

    #
    # ec_test 固有の設定
    #

    desc "Setup shared dir"
    task :setup_shared_dir do
      run("mkdir -p #{shared_path}/#{path_to_tmp_inquiry_images}")
    end

    desc "Create ec_front.yml"
    task :create_ec_front_yml do
      run("mkdir -p #{shared_path}/config && echo ec_test: true > #{shared_path}/config/ec_front.yml")
    end

    desc "Update symlink to shared, versioned etc"
    task :update_symlinks do
      # ※注意
      # ec_testのsymlinkを作成する処理はcms_publishにもあります。
      # ここを変更する場合はそちらも同時に変更してください。（deploy.rbには必要ないかも）
      
      # #{release_path}/public/ec_rider/product_images のsymlinkを貼る前に、#{release_path}/public/ec_rider を作っておきます。
      run("mkdir -p #{release_path}/public/ec_rider")
      rm_rf_and_ln_nfs([ # 順番を保持するために一つのHashにまとめないでください。
        # product_imagesはプラグインの制約でpublicに対してec_adminもec_frontお同じパスでなければなりません。
        {"$ECR_CMS_GIT_DIR/public/ec_rider/product_images" => "#{release_path}/public/ec_rider/product_images"},
        {"$ECR_CMS_GIT_DIR/public/banner_images"  => "#{release_path}/public/banner_images"},
        {"#{shared_path}/#{path_to_tmp_inquiry_images}"   => "#{release_path}/#{path_to_tmp_inquiry_images}"},
        # 2009/11/24リリースでは未実装
        # {"$ECR_EC_ADMIN_SHARED/public/ec_rider/product_images_for_preview" => "#{release_path}/public/ec_rider/product_images_for_preview"},
      ])
    end

  else

    #
    # ec_front 固有の設定
    #

    desc "Setup shared dir"
    task :setup_shared_dir do
      run("mkdir -p #{shared_path}/#{path_to_tmp_inquiry_images}")
    end

    desc "Create ec_front.yml"
    task :create_ec_front_yml do
      run("mkdir -p #{shared_path}/config && echo ec_test: false > #{shared_path}/config/ec_front.yml")
    end

    desc "Update symlink to shared, versioned etc"
    task :update_symlinks do
      # 以下のディレクトリ群はgit管理下のec_front/public以下に必要ですが、
      # svn管理下に置いておきたいので、一旦避難してec_front/publicへのsymlinkを貼った後、
      # 元に戻します。
      # ec_front/app/viewsやec_front/publicには これらのディレクトリや
      # /tmp_inquiry_imagesをgitに追加しないように
      # cap deploy:setupによって.gitignoreが設定されています。
      evacuate_and_restore(release_path,
        'app/views/error_mailer',
        'app/views/front_site_mailer',
        'app/views/sitemap',
        'app/views/layouts/application*.html.erb', # 注意: wildcardを使ってます。
        'app/views/layouts/cms*.html.erb',  # 注意: wildcardを使ってます。
        'app/views/layouts/_double_submitt_prevention*.html.erb',  # 注意: wildcardを使ってます。
        'public/ecrider_javascripts',
        :delete_evacuation_dir => true) do
        # git管理下のディレクトリや、shared以下のディレクトリへのsymlinkを貼ります。
        rm_rf_and_ln_nfs([ # 順番を保持するために一つのHashにまとめないでください。
          {"$ECR_CMS_GIT_DIR/app/views" => "#{release_path}/app/views"},
          {"$ECR_CMS_GIT_DIR/public"    => "#{release_path}/public"},
          # "#{release_path}/public"のsymlinkを貼ったあとにtmp_inquiry_imagesのsymlinkを貼ります
          {"#{shared_path}/#{path_to_tmp_inquiry_images}" => "#{release_path}/#{path_to_tmp_inquiry_images}"},
        ])
      end
    end

  end

  #
  # ec_front / ec_test 共通の設定
  #

  desc "Update symlink to ec_front.yml"
  task :update_symlink_to_ec_front_yml do
    src = "#{shared_path}/config/ec_front.yml"
    dest = "#{release_path}/config/ec_front.yml"
    run("ln -s #{src} #{dest}")
  end

end

after "deploy:setup", "cms:setup_shared_dir"
after "deploy:setup", "cms:create_ec_front_yml"
after "deploy:update_code", "cms:update_symlinks"
after "deploy:update_code", "cms:update_symlink_to_ec_front_yml"
