# -*- coding: utf-8 -*-
# Be sure to restart your web server when you modify this file.

# Uncomment below to force Rails into production mode when
# you don't control web/app server and can't set it the proper way
# ENV['RAILS_ENV'] ||= 'production'

# Specifies gem version of Rails to use when vendor/rails is not present
RAILS_GEM_VERSION = '2.3.4' unless defined? RAILS_GEM_VERSION

# Bootstrap the Rails environment, frameworks, and default configuration
require File.join(File.dirname(__FILE__), 'boot')

Rails::Initializer.run do |config|
  # Settings in config/environments/* take precedence over those specified here.
  # Application configuration should go into files in config/initializers
  # -- all .rb files in that directory are automatically loaded.
  # See Rails::Configuration for more options.

  # Skip frameworks you're not going to use. To use Rails without a database
  # you must remove the Active Record framework.
  # config.frameworks -= [ :active_record, :active_resource, :action_mailer ]

  # Specify gems that this application depends on.
  # They can then be installed with "rake gems:install" on new installations.
  # You have to specify the <tt>:lib</tt> option for libraries, where the Gem name (<em>sqlite3-ruby</em>) differs from the file itself (_sqlite3_)
  # config.gem "bj"
  # config.gem "hpricot", :version => '0.6', :source => "http://code.whytheluckystiff.net"
  # config.gem "sqlite3-ruby", :lib => "sqlite3"
  # config.gem "aws-s3", :lib => "aws/s3"

  config.gem "mongrel", :version => '1.1.5'
  config.gem "locale", :version => '2.0.3'
  config.gem "locale_rails", :version => '2.0.3'
  config.gem "mislav-will_paginate", :version => '2.3.11', :lib => "will_paginate"
  config.gem "kakutani-yaml_waml", :version => '0.3.0', :lib => 'yaml_waml'
  config.gem "fastercsv", :version => '1.4.0' # for jp_address
  config.gem "ar-extensions", :version => '0.9.2'  # for jp_address
  config.gem "log4r", :version => '1.1.2'
  config.gem "roxml", :version => '2.5.3' # libxmk2関連のライブラリが必要です。インストール例: $ sudo apt-get install libxml2 libxml2-dev
  config.gem "selectable_attr", :version => "0.3.7"
  config.gem "selectable_attr_rails", :version => "0.3.9"
  config.gem "state_flow" , :version => "0.2.1"
  config.gem "schema_comments" , :version => "0.1.1"
  config.gem 'magic_userstamp', :version => '0.1.1'
  config.gem 'vestal_versions', :version => '0.8.3'
  #config.gem 'rmagick', :version => '2.12.2'

  # # development用
  # config.gem 'bullet', :source => 'http://gemcutter.org' # バージョンは指定しません

  # Only load the plugins named here, in the order given. By default, all plugins
  # in vendor/plugins are loaded in alphabetical order.
  # :all can be used as a placeholder for all plugins not explicitly named
  # config.plugins = [ :exception_notification, :ssl_requirement, :all ]

  # Add additional load paths for your own custom dirs
  # config.load_paths += %W( #{RAILS_ROOT}/extras )

  # Force all environments to use the same logger level
  # (by default production uses :info, the others :debug)

  if RAILS_ENV == 'production'
    config.logger = Logger.new("#{RAILS_ROOT}/log/#{(RAILS_ENV.downcase rescue nil)}_rotate.log", 22, 1*1024*1024)
    config.logger.level = eval("Logger::#{(ENV['ECR_LOG_LEVEL'] || 'debug').upcase}")
  else
    config.log_level = (ENV['ECR_LOG_LEVEL'] || 'debug').downcase.to_sym
  end

  # Make Time.zone default to the specified zone, and make Active Record store time values
  # in the database in UTC, and return them converted to the specified local zone.
  # Run "rake -D time" for a list of tasks for finding time zone names. Comment line to use default local time.
  # config.time_zone = 'Tokyo'

  # Use SQL instead of Active Record's schema dumper when creating the test database.
  # This is necessary if your schema can't be completely dumped by the schema dumper,
  # like if you have constraints or database-specific column types
  # config.active_record.schema_format = :sql

  # Activate observers that should always be running
  # config.active_record.observers = :cacher, :garbage_collector

  # # 連番でマイグレーションの採番をします。
  # config.active_record.timestamped_migrations = false

  # config.i18n.default_locale = 'ja'
end

# Goldberg's libraries
if RAILS_ENV == 'test'
  require "#{RAILS_ROOT}/lib/goldberg.rb"
  Dir["#{RAILS_ROOT}/lib/goldberg/*.rb"].each do |lib|
    require lib
  end
end

# Add new inflection rules using the following format
# (all these examples are active by default):
# Inflector.inflections do |inflect|
#   inflect.plural /^(ox)$/i, '\1en'
#   inflect.singular /^(ox)en/i, '\1'
#   inflect.irregular 'person', 'people'
#   inflect.uncountable %w( fish sheep )
# end

# Add new mime types for use in respond_to blocks:
# Mime::Type.register "text/richtext", :rtf
# Mime::Type.register "application/x-mobile", :mobile

# Include your application configuration below

#
# エラーメッセージをソートするためのロジック
#
#module ActiveSupport
#  class OrderedHash
#    def each_key
#      each { |key, value| yield key }
#    end
#  end
#end
#
#module ActiveRecord
#  class Errors
#    def initialize(base) # :nodoc:
#      @base, @errors = base, ActiveSupport::OrderedHash.new
#    end
#
#    def clear
#      @errors = ActiveSupport::OrderedHash.new
#    end
#  end
#
#  module Validations
#    module ClassMethods
#      def write_inheritable_set(key, methods)
#        existing_methods = read_inheritable_attribute(key) || []
#        write_inheritable_attribute(key, existing_methods | methods)
#      end
#    end
#  end
#end

ActiveSupport::Dependencies.load_paths << File.join(RAILS_ROOT, 'app', 'mailers')
ExceptionNotifier.exception_recipients = EcRider.config.error_mail_from_address
ExceptionNotifier.sender_address = EcRider.config.error_mail_from_address


# 将来的にlocale, locale_railsの変更によって挙動が変わるかもしれませんが、2.0.4時点では
# この設定がないとI18n.localeが"ja"や"en"といった文字列を返すため、この設定は必須です。
Locale.default = "ja-JP" if defined?(Locale)

ActionView::Base.class_eval do
  include ::EcRider::CloudFrontHelper
  include ::EcRider::HelperExtention
end

ActionView::Helpers::FormBuilder.class_eval do
  include EcRider::JsValidationHelper::FormBuilderExt
end

SchemaComments::SchemaComment.class_eval do
  include EcRider::SchemaCommentsExt::SchemaComment
end
SchemaComments::ConnectionAdapters::Adapter.class_eval do
  include EcRider::SchemaCommentsExt::Adapter
end