# -*- coding: utf-8 -*-
ActionController::Routing::Routes.draw do |map|
  # The priority is based upon order of creation: first created -> highest priority.

  # Sample of regular route:
  # map.connect 'products/:id', :controller => 'catalog', :action => 'view'
  # Keep in mind you can assign values other than :controller and :action

  # Sample of named route:
  # map.purchase 'products/:id/purchase', :controller => 'catalog', :action => 'purchase'
  # This route can be invoked with purchase_url(:id => product.id)

  # You can have the root of your site routed by hooking up ''
  # -- just remember to delete public/index.html.
  map.connect '', :controller => 'cms', :action=> "index"

  map.connect 'sitemap.:format', :controller => 'sitemap', :action=> "index"
  map.connect 'sitemap'        , :controller => 'sitemap', :action=> "index"

  map.connect 'default_pages.:format', :controller => 'sitemap', :action=> "default_pages"
  map.connect 'default_pages'        , :controller => 'sitemap', :action=> "default_pages"

  # map.connect 'sitemap.xml', :controller => 'sitemap', :action=> "index"

  # Allow downloading Web Service WSDL as a file with an extension
  # instead of a file named 'wsdl'
  map.connect ':controller/service.wsdl', :action => 'wsdl'

  # Install the default route as the lowest priority.
  pc_controllers = []
  mobile_controllers = []
  controllers = Dir[File.join(RAILS_ROOT, 'app', 'controllers', '*.rb')]
  plugin_controllers = Dir[File.join(RAILS_ROOT, 'vendor', 'plugins', 'ecr_*', 'app', 'controllers', '*.rb')]
  [controllers, plugin_controllers].flatten.each do |path|
    controller_name = File.basename(path).gsub(/_controller\.rb$/, '')
    if controller_name =~ /^mobile_/
      mobile_controllers << controller_name.sub(/^mobile_/, '')
    else
      pc_controllers << controller_name
    end
  end
  # extra pc_controllers
  pc_controllers << 'exception_notification'

  # ignored pc_controllers
  pc_controllers.delete('cms')
  pc_controllers.delete('sitemap')
  pc_controllers.delete('pc') # これは抽象クラスのようなものです。実際にはアクセスできません。
  pc_controllers.delete('mobile') # これは抽象クラスのようなものです。実際にはアクセスできません。

  # STEP2では携帯電話ブラウザ対応は日本のみ
  # :lang=>"ja" の指定は、内部的にMobileControllerで行います。
  map.connect 'i/', :controller => 'mobile_cms', :action => 'index'
  mobile_controllers.each do |controller|
    map.connect "i/#{controller}/:action/:id", :controller => "mobile_#{controller}"
  end
  map.connect 'i/:action/:id', :controller => 'mobile_cms'
  map.connect 'i/:action'    , :controller => 'mobile_cms'

  pc_controllers.each do |controller|
    map.connect "#{controller}/:action/:id.:format", :controller => controller
    map.connect "#{controller}/:action/:id", :controller => controller
  end

  I18n.supported_locales.each do |locale|
    pc_controllers.each do |controller|
      # map.connect "#{locale.to_s}/#{controller}/:action/:id.:format", :controller => controller, :lang => locale.to_s
      map.connect "#{locale.to_s}/#{controller}/:action/:id", :controller => controller, :lang => locale.to_s
    end
  end

  I18n.supported_locales.each do |locale|
    # map.connect "#{locale.to_s}/:action/:id.:format", :controller => 'cms', :lang => locale.to_s
    map.connect "#{locale.to_s}/:action/:id"        , :controller => 'cms', :lang => locale.to_s
    # map.connect "#{locale.to_s}/:id.:format", :controller => 'cms', :action => 'index', :lang => locale.to_s
    map.connect "#{locale.to_s}/:id"        , :controller => 'cms', :action => 'index', :lang => locale.to_s
  end

  # Access for CMS.
  map.connect ':action/:id', :controller => 'cms'

end
