class CreateTablePaymentErrorHitories < ActiveRecord::Migration
  class PaymentErrorHistory < ActiveRecord::Base
  end
  class ViewPaymentErrorHistory < ActiveRecord::Base
  end

  def self.up
    exist_view = CreateTablePaymentErrorHitories::PaymentErrorHistory.table_exists?

    # 既存のビューをリネーム
    if exist_view
      rename_view('payment_error_histories', 'view_payment_error_histories')
    end

    # テーブルを作成
    create_table :payment_error_histories, :comment => "決済エラー履歴" do |t|
      t.integer  :order_id, :comment => '注文ID'
      t.string   :payment_code, :comment => '決済番号'
      t.string   :tran_id, :comment => 'トランザクションID'
      t.string   :tran_date, :comment => '決済日時（YYYYmmddHHMMSS）'
      t.string   :err_code, :comment => 'エラーコード'
      t.string   :err_info, :comment => '詳細エラーコード'
      t.string   :status_div, :length => 3, :comment => 'ステータス区分'
      t.string   :payment_system_div, :length => 3, :comment => '支払い方法区分'
      t.string   :payment_div, :length => 3, :comment => '決済会社区分'
      t.integer  :updatable_id, :comment => '決済履歴のID'
      t.string   :updatable_type, :comment => '決済履歴のモデル名'
      t.integer  :sps_user_temporary_id, :comment => 'テンポラリ会員ID'
      t.integer  :test_flg, :comment => 'テストフラグ'
      t.datetime :created_at, :comment => ""
      t.datetime :updated_at, :comment => ""
      t.datetime :deleted_at, :comment => ""
      t.integer  :lock_version, :default => 0, :comment => ""
    end

    # データ移行
    if exist_view
      CreateTablePaymentErrorHitories::PaymentErrorHistory.transaction do
        CreateTablePaymentErrorHitories::ViewPaymentErrorHistory.find(:all).each do |history|
          new_payment_error_history(history).save!
        end
      end
    end
  end

  def self.down
    exist_view = CreateTablePaymentErrorHitories::ViewPaymentErrorHistory.table_exists?
    
    drop_table :payment_error_histories
    
    if exist_view
      rename_view('view_payment_error_histories', 'payment_error_histories')
    end
  end

  private
  def self.rename_view(name, new_name)
    # 現在のビューを作るSQLを取得する
    query = ''
    result = ActiveRecord::Base.connection.execute("show create view #{name};")
    result.each do |v|
      query << v[1]
    end
    # 元のビューを削除
    if "CreateTablePaymentErrorHitories::#{name.classify}".constantize.table_exists?
      ActiveRecord::Base.connection.execute("drop view #{name};")
    end
    # 新しい名前でビューを作成
    if "CreateTablePaymentErrorHitories::#{new_name.classify}".constantize.table_exists?
      ActiveRecord::Base.connection.execute("drop view #{new_name};")
    end
    ActiveRecord::Base.connection.execute(query.gsub("`#{name}`", "`#{new_name}`"))
  end

  def self.new_payment_error_history(obj)
    payment_div_hash = {'1' => '2', '2' => '3', '3' => '6'}
    CreateTablePaymentErrorHitories::PaymentErrorHistory.new(
      :order_id => obj.order_id,
      :payment_code => obj.payment_code,
      :tran_id => obj.tran_id,
      :tran_date => obj.tran_date,
      :err_code => obj.err_code,
      :err_info => obj.err_info,
      :status_div => obj.status_div,
      :payment_system_div => obj.payment_system_div,
      :payment_div => payment_div_hash[obj.payment_div],
      :updatable_id => obj.updatable_id,
      :updatable_type => obj.updatable_type,
      :created_at => obj.created_at,
      :updated_at => obj.updated_at,
      :sps_user_temporary_id => obj.sps_user_temporary_id,
      :test_flg => obj.test_flg
    )
  end
end
