class AddColumnStockCountAtProductSkus < ActiveRecord::Migration
  class ProductSku < ActiveRecord::Base
  end
  def self.up
    add_column :product_skus, :parent_stock_count, :integer, :null => false, :default => 0, :comment => '自身の在庫数(商品構成を考慮しない)'
    add_column :product_skus, :secure_stock_count, :integer, :null => false, :default => 0, :comment => '自身の引き当て数(商品構成を考慮しない)'
    query = <<-EOS
      UPDATE product_skus s
      INNER JOIN product_sku_current_inventories i ON i.product_sku_id = s.id
      SET s.parent_stock_count = i.stock_count, s.secure_stock_count = (i.secure_stock_count + i.unpaid_secure_stock_count)
    EOS
    AddColumnStockCountAtProductSkus::ProductSku.transaction do
      ActiveRecord::Base::connection::execute(query)
      skus = AddColumnStockCountAtProductSkus::ProductSku.find(:all, :conditions => ['parent_stock_count IS NULL OR secure_stock_count IS NULL'])
      skus.each do |sku|
        sku.parent_stock_count ||= 0
        sku.secure_stock_count ||= 0
        sku.save!
      end
    end
  end

  def self.down
    remove_column :product_skus, :parent_stock_count
    remove_column :product_skus, :secure_stock_count
  end
end
