class AddColumnOrderableCountAtProductSkus < ActiveRecord::Migration
  class ProductSku < ActiveRecord::Base
    has_many :components, :class_name => "AddColumnOrderableCountAtProductSkus::ProductComponent", :foreign_key => 'product_sku_id'
    def get_orderable_count
      # 親の在庫数
      parent_orderable_count = self.parent_stock_count - self.secure_stock_count
      child_count = nil
      child_orderable_count = nil
      # 子の在庫数
      self.components.each do |component|
        component_sku = component.component_sku
        if component_sku
          composable_count = (component_sku.parent_stock_count / component.count).to_i
          composable_orderable_count = ((component_sku.parent_stock_count - component_sku.secure_stock_count) / component.count).to_i
          child_count = child_count ? [composable_count, child_count].min : composable_count
          child_orderable_count = child_orderable_count ? [composable_orderable_count, child_orderable_count].min : composable_orderable_count
        end
      end
      # 親子を考慮した在庫数
      count = self.parent_stock_count
      orderable_count = parent_orderable_count
      if child_count
        count = child_count
        orderable_count = child_orderable_count
        if self.parent_stockable_flg
          count = [self.parent_stock_count, child_count].min
          orderable_count = [parent_orderable_count, child_orderable_count].min
        end
      end
      return {:stock_count => (count ? count : 0), :orderable_count => (orderable_count ? orderable_count : 0)}
    end
  end
  class ProductComponent < ActiveRecord::Base
    belongs_to :component_sku, :class_name => 'AddColumnOrderableCountAtProductSkus::ProductSku', :foreign_key => 'component_sku_id'
  end
  
  def self.up
    add_column :product_skus, :composable_stock_count, :integer, :null => false, :default => 0, :comment => '商品構成を考慮した在庫数'
    add_column :product_skus, :orderable_count, :integer, :null => false, :default => 0, :comment => '商品構成を考慮した販売可能数'

    AddColumnOrderableCountAtProductSkus::ProductSku.transaction do
      # 商品構成を考慮した在庫数=自身の在庫数、商品構成を考慮した販売可能数=(自身の在庫数-自身の引き当て数)をセットする
      ActiveRecord::Base::connection::execute(
        "UPDATE product_skus s set s.orderable_count = (s.parent_stock_count - s.secure_stock_count), s.composable_stock_count = s.parent_stock_count")

      # 商品構成の親になっているSKUについて、商品構成を考慮して販売可能数を再設定する
      components = AddColumnOrderableCountAtProductSkus::ProductComponent.find(:all, :select => 'product_sku_id', :group => 'product_sku_id')
      components.each do |component|
        sku = AddColumnOrderableCountAtProductSkus::ProductSku.find(:first, :conditions => ['id = ?', component.product_sku_id])
        composable_count = sku.get_orderable_count
        sku.composable_stock_count = composable_count[:stock_count]
        sku.orderable_count = composable_count[:orderable_count]
        sku.save!
      end
    end
  end

  def self.down
    remove_column :product_skus, :composable_stock_count
    remove_column :product_skus, :orderable_count
  end
end
