# -*- coding: utf-8 -*-
module EcrMobile
  module EcHelper

    # PC、モバイル両方に対するタグは～Tagという名前にします
    module CartTag
      unloadable
      extend ::EcRider::EcHelper::MethodDescription

      desc <<-EOS
        [概要:数値選択セレクトボックスとカートに入れるボタンを表示します。]
        [引数 product_code(準必須):商品コードを指定します。&lt;ec:each_products&gt&lt;/ec:each_products&gtの中では不要です。]
        [引数 values(オプション):購入数選択セレクトボックスの選択肢をカンマ区切りで指定します。
                       指定しないなければ1～購入上限数設定値を設定します。
                       購入上限数が設定されていなければ1～99を設定します。
                       在庫数が設定されていれば在庫数以下の値を設定します。]
        [引数 class(オプション):ボタンのクラスを指定します。
                       指定しなければbutton_cart1が適用されます。]
        [引数 label(オプション):セレクトボックスの前に表示する文字列を指定します。]
        [引数 label2(オプション):セレクトボックスの後ろに表示する文字列を指定します。]
        [引数 button_name(オプション):ボタンの名称を指定します。]
        [戻り値:数値選択セレクトボックスとカートに入れるボタン。]
        [使用例:
          {{{
          <ec:cart_in_button product_code="001" values="1,2,3"/> （購入数の選択肢を指定）
          }}}
        ]
      EOS
      def cart_in_button(*args, &block)
        button_name, class_name, label, label2, product_code, values = *extract(args, :button_name => I18n.t(:"activerecord.errors.messages.NM990008"), :class => "button_cart1",  :label => "", :label2 => "", :product_code => "", :values => "")
        amount_options = []
        orderable_count = product_sku_info(product_code, 'orderable_count').to_i

        # 販売期間内でない場合
        sale_started_at = product_info(product_code, 'sale_started_at')
        sale_finished_at = product_info(product_code, 'sale_finished_at')
        time_now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
        unless (sale_started_at.nil? || sale_started_at.blank? || sale_started_at <= time_now) &&
            (sale_finished_at.nil? || sale_finished_at.blank? || sale_finished_at >= time_now)
          return "<div>#{product_info(product_code, 'product_name_1')}#{I18n.t(:"activerecord.errors.messages.EM990044")}</div>"
        end
        if orderable_count > 0
          order_limit = product_sku_info(product_code, 'order_limit').to_i
          order_limit = 99 if order_limit == 0 || order_limit > 99
          limit = [order_limit, orderable_count].min
          value_array = []
          if values.blank?
            value_array = (1..limit).to_a
          else
            values.split(',').each{|i| value_array << i.to_i if i.to_i <= limit}
          end
          value_array.each do |v|
            amount_options << ((v == value_array.first) ? %(<option value="#{v}" selected="selected">#{v}</option>\n) : %(<option value="#{v}">#{v}</option>\n))
          end

          product_code = product_info(product_code, 'product_code') if product_code.blank?
          sku_code = (product_sku_info(product_code, 'sku_code') || get_sku_code(product_code))
          text = %(<form method="get" action="cart/index" name="form_#{sku_code.gsub(/-/, '_')}">\n)
          text << @template.hidden_field_tag("sku_code", sku_code)
          text << @template.hidden_field_tag("course_div", product_info(product_code, 'course_div'))
          text << '<div class="number_select_label">'
          text << label
          text << '</div>'
          text << '<div class="number_select_div">'
          text << @template.select_tag(:qty, amount_options, {})
          text << '</div>'
          text << '<div class="number_select_label">'
          text << label2
          text << '</div>'
          text << '<div class="cart_button_div">'
          text << @template.submit_tag(button_name, :class => class_name)
          text << '</div>'
          text << '</form>'
        else
          text = product_sku_info(product_code, 'stockout_display_name')
        end
        return text
      end

      desc <<-EOS
        [概要:ショッピングカート内の数量を変更するセレクトボックスを表示します。cart_item_list内で使用できます。]
        [引数 format(任意):個数の単位を指定します。個数、または個数セレクトボックスの後ろに表示されます]
        [戻り値:数量を変更するセレクトボックス]
      EOS
      def cart_item_qty_select(options = {}, &block)
        raise UsageError, "cart_item_qty_selectはec:cart_item_listの中で使用してください" unless @cart_item

        key = @cart_item.key
        order_limit = @cart_item.product_sku.order_limit
        order_limit = 99 if order_limit.nil? || order_limit > 99
        limit = [@cart_item.product_sku.orderable_count, order_limit].min
        order_count_list = (1..limit).to_a

        # 現在カートに入っている個数をセレクトボックスに追加する
        # 商品をカートに入れてから実在庫が変更された場合に、実際と表示が食い違ってしまうことを防ぐため
        order_count_list = (order_count_list << @cart_item.qty).sort.uniq

        qty_format = options[:format] || ""
        result = ''

        # プレゼント品ならば、selectタグではなく文字列を出力する
        # パラメータの構造としてkey_and_qtysの配列として動きます
        # key_and_qtys[{key1=>qty1,key2=>qty2}]
        # key,qtyはCart::Itemのkey,qtyに対応します
        if @cart_item.present?
          result << "<span id='#{"key_and_qtys[]#{key}"}'>#{@cart_item.qty.to_s}</span>"
          result << qty_format
        else
          result << @template.select_tag("key_and_qtys[]#{key}", @template.options_for_select(order_count_list, @cart_item.qty), {:id => "key_and_qtys[]#{key}"})
          result << qty_format
        end
        result
      end

      desc <<-EOS
        [概要:カートに入っている商品を再計算するためのフォームを表示します。]
        [引数 (オプション):なし]
        [戻り値:カートに入っている商品再計算用フォーム]
        [使用例:
           {{{
           <ec:cart_list_form>
             <ec:cart_item_list>
               <ec:cart_item_image/>
               <ec:cart_item_product_name/>
               <ec:cart_item_product_sku_unit_price/>
             </ec:cart_item_list>
           </ec:cart_item_form>
           }}}
        ]
      EOS
      def cart_list_form(*args, &block)
        cart = session[:cart]
        raise UsageError, "ここでは使用できません。" unless cart
        @template.form_tag({:controller =>'cart', :action => 'recalculate'}, {:method => :get}) do
          begin
            @template.concat(@template.capture(&block))
          ensure
            ""
          end
        end
      end

      desc <<-EOS
        [概要:ショッピングカート内の商品を削除するボタンを表示します。cart_item_list内で使用できます。]
        [引数 (オプション):delete_link_name 削除ボタンの名前を設定できます。何も設定しない場合はdeleteを表示します。]
        [使用例:{{{<ec:cart_item_delete_link delete_link_name="削除" />}}}]
      EOS
      def cart_item_delete_link(options = {}, &block)
        raise UsageError, "cart_item_delete_linkはec:cart_item_listの中で使用してください" unless @cart_item
        # プレゼント品ならば、「削除」ボタンを表示しない
        return if @cart_item.present?
        delete_link_name = options[:delete_link_name] || "delete"
        link_to(delete_link_name, {:controller => 'cart', :action=> 'remove', :id => @cart_item.key})
      end

      desc <<-EOS
        [概要: ｢お買い物を続ける｣ボタンを表示します。戻り先は商品一覧に設定してあるページになります。カート用のページ内で使用してください。]
        [引数 (オプション): class ボタンのクラス名を指定できます。]
        [使用例:
          {{{
          <ec:cart_continue_shopping_button class="aaa"/>
          }}}
          　　==> <input class="aaa button_gray" type="button" value="←お買い物を続ける" onclick="location.href='設定した商品一覧';"/>
        ]
      EOS
      def cart_continue_shopping_button(options = {})
        options[:class] = "#{options[:class]} button_gray"
        shop_setting = ShopSetting.get_setting(:continue_shopping_action, I18n.locale)
        shop_setting.sub!(/\//, '') if shop_setting
        @template.button_to(I18n.t(:"activerecord.errors.messages.NM990012"), {:controller => 'cms', :action => (shop_setting || 'index') }, options)
      end

    end
  end
end