# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'
describe EcRider::EcHelper do
    fixtures :products, :product_images,
      :product_variations, :product_variation_items, :product_categories,
      :product_skus, :product_displays, :product_sku_inventories, :product_sku_secure_inventories


  before(:each) do
    request.stub!(:mobile?).and_return(true)
    request.stub_chain(:mobile, :supports_cookie?).and_return(false)
    ActionController::Base.session_options[:key] = "session_id"
    @template.extend(ApplicationHelper)
    @template.output_buffer = ""
  end

  def ec
    helper.ec(:mobile)
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe '<%= ec:cart_in_button %>' do
    fixtures :kbn_code_names, :kbn_codes, :product_skus, :product_sku_inventories, :product_sku_secure_inventories, :product_components
    it '選択肢を指定する。在庫数が設定されている場合。' do
      product = Product.find(100001)
      product_sku = product.primary_sku
      product_sku.sku_code.should == "0000000000001-00-00-00"
      product_sku.orderable_count.should == 24
      product_sku.order_limit.should == 99

      ec.cart_in_button(:product_code => "0000000000001", :values => "1,2,3").should have_tag('form[method="get"][name="form_0000000000001_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000001-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"][value="5"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 3)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
          with_tag('option[value="3"]', "3")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end

      ec.cart_in_button(:product_code => "0000000000001-00-00-00", :values => "1,2,3").should have_tag('form[method="get"][name="form_0000000000001_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000001-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"][value="5"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 3)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
          with_tag('option[value="3"]', "3")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定する。在庫数が設定されている場合。（選択肢最大値<注文可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000006-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == 50

      ec.cart_in_button(:product_code => "0000000000006-00-00-00", :values => "1,2,3,4").should have_tag('form[method="get"][name="form_0000000000006_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000006-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"][value="5"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 4)
          with_tag('option', :maximum  => 4)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定する。在庫数が設定されている場合。（選択肢最大値=注文可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000006-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == 50

      ec.cart_in_button(:product_code => "0000000000006-00-00-00", :values => "1,3,5").should have_tag('form[method="get"][name="form_0000000000006_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000006-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"][value="5"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 3)
          with_tag('option', :maximum  => 5)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="3"]', "3")
          with_tag('option[value="5"]', "5")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定する。在庫数が設定されている場合。（選択肢最大値>注文可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000006-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == 50

      ec.cart_in_button(:product_code => "0000000000006-00-00-00", :values => "1,3,6").should have_tag('form[method="get"][name="form_0000000000006_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000006-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"][value="5"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 2)
          with_tag('option', :maximum  => 3)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="3"]', "3")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定する。上限数が設定されている場合。（選択肢最大値<上限数）' do
      product_sku = ProductSku.find_by_sku_code("0000000100003-00-00-00")
      product_sku.orderable_count.should == 110
      product_sku.order_limit.should == 20

      ec.cart_in_button(:product_code => "0000000100003-00-00-00", :values => "10,19").should have_tag('form[method="get"][name="form_0000000100003_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000100003-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 2)
          with_tag('option', :maximum  => 19)
          with_tag('option[selected="selected"][value="10"]', "10")
          with_tag('option[value="19"]', "19")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定する。上限数が設定されている場合。（選択肢最大値=上限数）' do
      product_sku = ProductSku.find_by_sku_code("0000000100003-00-00-00")
      product_sku.orderable_count.should == 110
      product_sku.order_limit.should == 20

      ec.cart_in_button(:product_code => "0000000100003-00-00-00", :values => "10,20").should have_tag('form[method="get"][name="form_0000000100003_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000100003-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 2)
          with_tag('option', :maximum  => 20)
          with_tag('option[selected="selected"][value="10"]', "10")
          with_tag('option[value="20"]', "20")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定する。上限数が設定されている場合。（選択肢最大値>上限数）' do
      product_sku = ProductSku.find_by_sku_code("0000000100003-00-00-00")
      product_sku.orderable_count.should == 110
      product_sku.order_limit.should == 20

      ec.cart_in_button(:product_code => "0000000100003-00-00-00", :values => "10,21").should have_tag('form[method="get"][name="form_0000000100003_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000100003-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 1)
          with_tag('option', :maximum  => 10)
          with_tag('option[selected="selected"][value="10"]', "10")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されていない場合。' do
      ProductSkuInventory.destroy_all
      ProductSku.update_all(:parent_stock_count => 0)
      product_sku = ProductSku.find_by_sku_code("0000000000003-00-00-00")
      product_sku.orderable_count.should == 0
      product_sku.order_limit.should == 20

      ec.cart_in_button(:product_code => "0000000000003-00-00-00").should == "この商品は売り切れました"
    end

    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されている場合。（上限数<販売可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000007-00-00-00")
      product_sku.orderable_count.should == 51
      product_sku.order_limit.should == 50

      ec.cart_in_button(:product_code => "0000000000007-00-00-00").should have_tag('form[method="get"][name="form_0000000000007_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000007-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 50)
          with_tag('option', :maximum  => 50)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されている場合。（上限数=在庫数>販売可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000008-00-00-00")
      product_sku.orderable_count.should == 49
      product_sku.stock_count.should == 50
      product_sku.order_limit.should == 50

      ec.cart_in_button(:product_code => "0000000000008-00-00-00").should have_tag('form[method="get"][name="form_0000000000008_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000008-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 49)
          with_tag('option', :maximum  => 49)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定しない。上限数が設定されていて、在庫数が設定されている場合。（上限数>販売可能数）' do
      product_sku = ProductSku.find_by_sku_code("0000000000009-00-00-00")
      product_sku.orderable_count.should == 49
      product_sku.order_limit.should == 50

      ec.cart_in_button(:product_code => "0000000000009-00-00-00").should have_tag('form[method="get"][name="form_0000000000009_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000009-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 49)
          with_tag('option', :maximum  => 49)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it '選択肢を指定しない。上限数が設定されていなくて、在庫数が設定されていない場合。' do
      product_sku = ProductSku.find_by_sku_code("0000000000004-00-00-00")
      product_sku.orderable_count.should == 0
      product_sku.order_limit.should == nil

      ec.cart_in_button(:product_code => "0000000000004-00-00-00").should == "この商品は売り切れました"
    end

    it '選択肢を指定しない。上限数が設定されていなくて、在庫数が設定されている場合。' do
      product_sku = ProductSku.find_by_sku_code("0000000000010-00-00-00")
      product_sku.orderable_count.should == 5
      product_sku.order_limit.should == nil

      ec.cart_in_button(:product_code => "0000000000010-00-00-00").should have_tag('form[method="get"][name="form_0000000000010_00_00_00"][action="cart/index"]') do
        with_tag('input[id="sku_code"][name="sku_code"][type="hidden"][value="0000000000010-00-00-00"]')
        with_tag('input[id="course_div"][name="course_div"][type="hidden"]')
        with_tag('select[id="qty"][name="qty"]') do
          with_tag('option', :count => 5)
          with_tag('option', :maximum  => 5)
          with_tag('option[selected="selected"][value="1"]', "1")
          with_tag('option[value="2"]', "2")
        end
        with_tag('input[type="submit"][value="カートに入れる"][class="button_cart1"]')
      end
    end

    it 'ボタンのクラスを指定する。' do
      ec.cart_in_button(:product_code => "0000000000001-00-00-00", :class => "aaa").should have_tag('form[method="get"][name="form_0000000000001_00_00_00"][action="cart/index"]') do
        with_tag('input[type="submit"][value="カートに入れる"][class="aaa"]')
      end
    end

    it '在庫切れ時の表示 ja' do
      ec.cart_in_button(:product_code => "0000000000004-00-00-00").should == "この商品は売り切れました"
      ec.cart_in_button(:product_code => "0000000000052-00-00-00").should == "近日入荷予定"
    end

    it '在庫切れ時の表示 en' do
      I18n.locale = 'en'
      ec.cart_in_button(:product_code => "0000000000004-00-00-00").should == "This product is sold out"
      ec.cart_in_button(:product_code => "0000000000052-00-00-00").should == "Comming soon"
    end
  end

  describe "cart_list_form" do
    before(:each) do
      product_sku = ProductSku.find_by_product_id(1)
      product_sku.stub(:price).and_return('8190')
      @cart_item1 = Cart::Item.new(:key => '1', :product_sku => product_sku, :course_div_key => :normal, :qty => 2)
      @cart_item1.key.should == '1'
      product_sku2 = ProductSku.find_by_product_id(2)
      product_sku2.stub(:price).and_return('12600')
      @cart_item2 = Cart::Item.new(:key => '2', :product_sku => product_sku2, :course_div_key => :normal, :qty => 1)
      @cart_item2.discount_amount = 1000
    end
    describe "cart_item_form" do
      it "cart_item_form" do
        @cart_item1.key = '2'
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        ec.cart_list_form do
          @template.output_buffer
        end
        if EcRider.config.ec_admin
          @template.output_buffer.should == "<form action=\"/cart/recalculate\" method=\"get\"></form>"
        else
          @template.output_buffer.should == "<form action=\"/cart/recalculate?session_id=#{request.session_options[:id]}\" method=\"get\"></form>"
        end
      end
    end
    describe "cart_item_qty_select" do
      it "if course_div_param = 2" do
        @cart_item1.key = '2'
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_qty_select)
          concat(ec.cart_item_delete_button)
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('select') do
          cart_list.each do |cart|
            with_tag("select[id=?]", "key_and_qtys[]#{cart.key}")
          end
        end
        @template.output_buffer.should have_tag('option') do
          (1..10).each do |i|
            with_tag("option[value=?]", i.to_s)
          end
          with_tag("option[value=2][selected=?]", 'selected')
        end
      end

      it "if course_div_param = 5" do
        @cart_item1.key = '2'
        @cart_item1.course_div = 5
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_qty_select)
          concat(ec.cart_item_delete_button)
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('select') do
          cart_list.each do |cart|
            with_tag("select[id=?]", "key_and_qtys[]#{cart.key}")
          end
        end
      end

      it "format" do
        @cart_item1.key = '2'
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_qty_select(:format => '円'))
          @template.output_buffer
        end
        @template.output_buffer.should have_tag('select') do
          cart_list.each do |cart|
            with_tag("select[id=?]", "key_and_qtys[]#{cart.key}")
          end
        end
      end
    end

    describe "cart_item_delete_button" do
      it "delete_button" do
        @cart_item1.key = '2'
        cart_list = [@cart_item1]
        session[:cart] = mock(:cart)
        session[:cart].should_receive(:item_list_order_by_present).and_return(cart_list)
        ec.cart_item_list do
          concat(ec.cart_item_delete_link(:delete_button_name => '削除'))
          @template.output_buffer
        end
        if EcRider.config.ec_admin
          @template.output_buffer.should == "<a href=\"/cart/remove/2\">delete</a>"
        else
          @template.output_buffer.should == "<a href=\"/cart/remove/2?#{
            ActionController::Base.session_options[:key]}=#{request.session_options[:id]}\">delete</a>"
        end
      end
    end

    describe "cart_continue_shopping_button" do
      it "continue_button" do
        if EcRider.config.ec_admin
          ec.cart_continue_shopping_button.should == "<form method=\"post\" action=\"/cms/list\" class=\"button-to\"><div><input class=\" button_gray\" type=\"submit\" value=\"←お買い物を続ける\" /></div></form>"
        else
          ec.cart_continue_shopping_button.should == "<form method=\"post\" action=\"/list?#{ActionController::Base.session_options[:key]
          }=#{request.session_options[:id]}\" class=\"button-to\"><div><input class=\" button_gray\" type=\"submit\" value=\"←お買い物を続ける\" /></div></form>"
        end
      end
    end
  end

end