# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'
describe EcRider::EcHelper do

  def ec
    helper.ec(:mobile)
  end

  before(:each) do
    request.stub!(:mobile?).and_return(true)
    request.stub_chain(:mobile, :supports_cookie?).and_return(true) # テストを簡略化するため、クッキーを使用する設定にする。
    @template.extend(ApplicationHelper)
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:ezlogin_form" do
    it "normal usage" do
      ec.ezlogin_form do
        ec.submit do
          @template.concat("かんたんログイン")
        end
        @template.output_buffer
      end

      @template.output_buffer.should have_tag('form[action=?][utn=?]', "/account/authenticate_ez", "utn") do
        with_tag('input[type="submit"][value="かんたんログイン"]')
      end
    end
  end

  describe "ec:ezlogin_config_form" do
    it "normal usage" do
      ec.ezlogin_config_form do
        ec.submit do
          @template.concat("設定する")
        end
        @template.output_buffer
      end
      
      @template.output_buffer.should have_tag('form[action=?][utn=?]', "/member/complete_config_ezlogin", "utn") do
        with_tag('input[type="submit"][value="設定する"]')
        with_tag('input#mobile_unique_id[type="hidden"]')
      end
    end
  end

  describe "ec:if_mobile_unique_id_get" do
    it "normal usage when @mobile_unique_id is variable" do
      @template.instance_variable_set(:@mobile_unique_id, "mobile_unique_id")
      str = "携帯電話の端末識別番号が取得できた場合に表示"
      ec.ezlogin_config_form do
        ec.if_mobile_unique_id_get do
          @template.concat(str)
        end
        @template.output_buffer
      end

      @template.output_buffer.include?(str).should be_true
    end

    it "normal usage when @mobile_unique_id is not variable" do
      @template.instance_variable_set(:@mobile_unique_id, nil)
      str = "携帯電話の端末識別番号が取得できた場合に表示"
      ec.ezlogin_config_form do
        ec.if_mobile_unique_id_get do
          @template.concat(str)
        end
        @template.output_buffer
      end

      @template.output_buffer.include?(str).should be_false
    end

    it "should raise error" do
      lambda{ec.if_mobile_unique_id_get}.should raise_error(EcRider::EcHelper::UsageError)
    end
  end

  describe "ec:if_mobile_unique_id_unknown" do
    it "normal usage when @mobile_unique_id is variable" do
      @template.instance_variable_set(:@mobile_unique_id, "mobile_unique_id")
      str = "携帯電話の端末識別番号が取得できなかった場合に表示"
      ec.ezlogin_config_form do
        ec.if_mobile_unique_id_unknown do
          @template.concat(str)
        end
        @template.output_buffer
      end

      @template.output_buffer.include?(str).should be_false
    end

    it "normal usage when @mobile_unique_id is not variable" do
      @template.instance_variable_set(:@mobile_unique_id, nil)
      str = "携帯電話の端末識別番号が取得できなかった場合に表示"
      ec.ezlogin_config_form do
        ec.if_mobile_unique_id_unknown do
          @template.concat(str)
        end
        @template.output_buffer
      end

      @template.output_buffer.include?(str).should be_true
    end

    it "should raise error" do
      lambda{ec.if_mobile_unique_id_get}.should raise_error(EcRider::EcHelper::UsageError)
    end
  end

  describe "ec:ezlogin_config_cancel_form" do
    it "normal usage" do
      ec.ezlogin_config_cancel_form do
        ec.submit do
          @template.concat("解除する")
        end
        @template.output_buffer
      end

      @template.output_buffer.should have_tag('form[action=?][utn=?]', "/member/complete_cancel_config_ezlogin", "utn") do
        with_tag('input[type="submit"][value="解除する"]')
        with_tag('input#mobile_unique_id[type="hidden"]')
      end
    end
  end

end
