# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :users, :user_addresses, :user_emails, :distributor_delivery_times, :distributor_delivery_time_displays, :countries,
    :products, :product_skus, :product_sku_inventories, :product_sku_secure_inventories
  def ec
    helper.ec(:mobile)
  end

  before(:each) do
    request.stub!(:mobile?).and_return(true)
    request.stub_chain(:mobile, :supports_cookie?).and_return(false)
    ActionController::Base.session_options[:key] = "session_id"
    @template.extend(ApplicationHelper)
    @template.output_buffer = ""
    @order_form = OrderForm.new(:send_address_index => 0)
    @template.instance_variable_set(:@order_form, @order_form)

    @shipping_address = UserAddress.new()
    @template.instance_variable_set(:@shipping_address, @shipping_address)
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end


#  describe "ec.order_form_card_kind" do
#    it "カード種類を選択するラジオボタンが出力されること" do
#      ec.order_form do
#        ec.order_form_card_kind do
#          concat(ec.order_form_card_kind_radio)
#          concat(ec.order_form_card_kind_label)
#        end
#      end
#      p @template.output_buffer
#
#    end
#  end


  describe "ec.order_form_payment_type" do
    it "お支払い方法を選択するセレクトボックスが出力されること" do
      ec.order_form do
        ec.order_form_payment_type
      end
      @template.output_buffer.should have_tag("select[name=?]","order_form[normal_payment_type]") do
        with_tag("option", "代金引換")
        with_tag("option", "クレジットカード")
        with_tag("option", "銀行口座振込")
      end
    end
  end


  describe "order_form_hope_delivery_time - 配送希望時間のセレクトボックスを表示させるタグのテスト" do
    it "セレクトボックス用のタグ出力、が取得できればOK" do
      ec.order_form do
        concat(ec.order_form_delivery_time_select)
        @template.output_buffer
      end
      if EcRider.config.ec_admin
        @template.output_buffer.should have_tag("form[action=?]", "/order/confirmation_order")
      else
        @template.output_buffer.should have_tag("form[action=?]", "/order/confirmation_order?session_id=#{request.session_options[:id]}")
      end
      @template.output_buffer.should have_tag("select[id=?]","order_form_delivery_time")
      @template.output_buffer.should have_tag("select[name=?]","order_form[delivery_time]") do
        with_tag("option[value=?]","1")
        with_tag("option[value=?]","2")
        with_tag("option[value=?]","3")
        with_tag("option[value=?]","4")
        with_tag("option[value=?]","5")
      end
    end
  end
  describe "shipping_tag" do

    before(:each) do
      @template.output_buffer = ""
      @user = User.find(1006, :include => [:user_address, :user_email])
      @shipping_addresses = @user.user_and_shipping_addresses
      @template.instance_variable_set(:@shipping_addresses, @shipping_addresses)
      @template.extend(ApplicationHelper)
      @template.instance_variable_set(:@send_address_index, 1)
    end

    describe "ec:shipping_list" do
      def set_cart_to_session
        cart = Cart.new
        p1 = product_skus(:product_skus_00012)
        cart.add(p1, p1.product.course_div_key, 1)
        session[:cart] = cart

      end

      describe "order_shipping_select_link" do
        it "order_shipping_select_link no_display" do
          @template.instance_variable_set(:@send_address_index, nil)
          helper.stub!(:action_name).and_return('show_addresses')
          ec.shipping_list do
            ec.order_shipping_select_link
          end
          @template.output_buffer.should  == ""
        end
        it "order_shipping_select_link display" do
          set_cart_to_session
          params[:controller] = 'order'
          ec.shipping_list do
            ec.order_shipping_select_link(){"この配送先にする"}
          end
          if EcRider.config.ec_admin
            @template.output_buffer.should have_tag("a[href=?]", "/order/payment?send_address_index=1")
          else
            @template.output_buffer.should have_tag("a[href=?]", "/order/payment?send_address_index=1&session_id=#{request.session_options[:id]}")
          end
        end
      end
    end
  end
end