module EcrMobileFront
  module ApplicationControllerExt
    unloadable
    
    def self.included(mod)
      mod.module_eval do
        EcrMobileFront.setup
        # モバイル用のフィルターを追加します。
        trans_sid
        mobile_filter
        after_filter :change_response_header_for_mobile
        alias_method_chain :reset_session, :mobile
      end
    end

    protected
      # モバイル用のレスポンスフィルターメソッド
      def change_response_header_for_mobile
        response.content_type  = 'application/xhtml+xml' if request.mobile?
      end

      # reset_session用のパッチを当てます。
      # 参照：http://groups.google.com/group/jpmobile/browse_thread/thread/155a37c0136c72c5/9dc83bfc1fb5818a
      def reset_session_with_mobile
        reset_session_without_mobile
        request.session_options[:id] = ActiveSupport::SecureRandom.hex(16) if request.mobile?
      end

      # 各キャリア毎の有効なIPのみに制限します。
      # 不正なIPからのリクエストは、
      def check_valid_ip
        if EcRider.config.valid_ip_check && request.mobile? && !request.mobile.valid_ip?
          # 調査のためのログを出力します。
          logger.error "#{'-'*80}"
          logger.error "Request Headers"
          logger.error "  HTTP_USER_AGENT : #{request.headers["HTTP_USER_AGENT"]}"
          logger.error "  REMOTE_ADD      : #{request.headers["REMOTE_ADDR"]}"
          logger.error "#{'-'*80}"
          render "cms/invalid_ip", :status => 400
        end
      end
  end
end