module EcrMobileFront
  module OrderControllerExt
    unloadable

    #
    # 個人情報規約
    #
    def agreement_with_ecr_mobile_front
      return agreement_without_ecr_mobile_front unless request.mobile?

      # セッションクリアは行わない。
    end


    #
    # = お届け先指定・お支払方法 =
    #
    # 事前条件：カートに一つ以上の商品が入っていること
    def payment_with_ecr_mobile_front
      return payment_without_ecr_mobile_front unless request.mobile?

      # 注文者情報登録画面から遷移してきた場合
      if params[:user_profile]
        # 注文者情報登録からの遷移の場合は、POST
        return redirect_to(:action => 'confirmation_cart') unless request.post?

        # 「郵便番号で住所表示」ボタンが押された場合
        if params.include?("zip_to_address")
          params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
          submit_flg = false
        end

        @user = User.new(params[:user_profile])
        @user.remember_question_check = !@user.password.blank?
        @user.member_div_key = @user.password.blank? ? :no_membership : :membership #メールアドレス重複チェックで使用する
        # モバイルは日本語しか対応しない
        params[:user_address]["country_code"] = "JP"
        @user_address = UserAddress.new(params[:user_address])
        @user_address.locale = I18n.locale.to_s
        @user_address.address_div_key = :user
        @user_email = UserEmail.new(params[:user_email])
        @user_email.pc_mobile_div_key = :mobile
        @user_email.user = @user
        @user.user_email = @user_email
        @user.mirror_from_address(@user_address)

        # 新規の為、ユーザーIDなしを渡す(この処理はメールアドレス重複チェックに必要です。user_email.userの値を使用するため)
        @user_email.user = @user

        submit_flg = true

        # 「郵便番号で住所表示」ボタンが押された場合
        if params.include?("zip_to_address")
          flash[:user_profile] = @user
          flash[:user_address] = @user_address
          flash[:user_email] = @user_email
          redirect_to(:action => 'confirmation_cart')
          return
        end

        # 「入会規約を読む」ボタンが押された場合
        if params.include?("agreement")
          submit_flg = false
          redirect_to(:action => 'agreement')
        end

        if submit_flg
          @user.valid?
          @user_address.valid?
          @user_email.valid?

          if @sub_address = UserAddress.hash_to_sub_address(params[:user_en_address])
            @sub_address.set_sub_address_from_address(@user_address)
            @sub_address.valid?
          end

          error_count = @user.errors.count + @user_address.errors.count + @user_email.errors.count
          error_count += @sub_address.errors.count if @sub_address

          if error_count > 0
            flash[:user_profile] = @user
            flash[:user_address] = @user_address
            flash[:user_email] = @user_email
            flash[:user_en_address] = @sub_address
            redirect_back_or_default :action => :confirmation_cart
            return
          end
        end
        site_type_div = request.mobile? ? :mobile : :pc
        session[:cart_tmp_user] = User.new_user(params[:user_profile], params[:user_address], params[:user_email], params[:user_en_address], site_type_div)
      end

      # 配送先リストを取得する
      @shipping_addresses = session[:shipping_addresses]
      if @shipping_addresses.nil?
        @shipping_addresses = session[:user] ? current_user.user_and_shipping_addresses : session[:cart_tmp_user].user_and_shipping_addresses
      else
        @shipping_addresses[0] = session[:cart_tmp_user].user_and_shipping_addresses[0] unless session[:user]
      end
      session[:shipping_addresses] = @shipping_addresses

      #配送先指定 1件分表示
      @send_address_index  = (params[:send_address_index] || 0).to_i

      # フォーム情報を取得
      @order_form = flash[:cart_send_info] || session[:order_form] || OrderForm.new(:send_address_index =>@send_address_index)
      session[:order_form] = @order_form

      # カートを再計算(カート内で適用できるキャンペーンを取得するために実施します。)
      session[:cart].calculate
    end

    #
    # 注文内容確認
    #
    def confirmation_order_with_ecr_mobile_front
      @site_type_div = :pc
      return confirmation_order_without_ecr_mobile_front unless request.mobile?
      params[:order_form][:send_address_index] = params[:send_address_index] || 0
      @site_type_div = :mobile
      return confirmation_order_without_ecr_mobile_front
    end

    def get_user_email_with_ecr_mobile_front
      @site_type_div == :pc ? @user.user_email : @user.user_email_mobile
    end

    def get_order_with_ecr_mobile_front(user, address, order_form, cart)
      Order.create_new_order(user, address, order_form, cart, @site_type_div)
    end


    #
    # 配送先一覧
    #
    def show_addresses
      session[:change_address] = nil
      @error_flg = flash[:address_error] || nil
      @user_address = flash[:address]
      @user_address ||= session[:address]
      @user_address ||= UserAddress.new(:country_code => 'JP')
      @shipping_addresses = session[:shipping_addresses]
      if @shipping_addresses.nil?
        @shipping_addresses = session[:user] ? current_user.user_and_shipping_addresses : session[:cart_tmp_user].user_and_shipping_addresses
      else
        @shipping_addresses[0] = session[:cart_tmp_user].user_and_shipping_addresses[0] unless session[:user]
      end
      session[:shipping_addresses] = @shipping_addresses

      @user_profile = session[:cart_tmp_user]
    end

    #
    # 配送先新規登録画面表示
    #
    def new_shipping_address_with_ecr_mobile_front
      return new_shipping_address_without_ecr_mobile_front unless request.mobile?
      session[:change_address] = nil
      @error_flg = flash[:address_error] || nil
      @user_address = flash[:address]
      @user_address ||= session[:address]
      @user_address ||= UserAddress.new
      @user_address[:country_code] = 'JP'
      @user_profile = (session[:user] ? current_user : User.new())
    end

    #
    # 配送先登録更新
    #
    def update_shipping
      return redirect_to(:action => 'show_addresses') unless request.post? || request.put?
      user = get_user

      #変更処理
      if session[:change_address]
        index = params[:edit_target_index].to_i
        @shipping_address = session[:shipping_addresses][index]
        @shipping_address.country.reload #変更された国コードで国情報を再取得する
     else
        #追加処理
        @shipping_address = session[:address]
        @shipping_address.address_div_key = :ship_to
        @shipping_address.locale = I18n.locale.to_s
      end

      #配送先変更
      if session[:change_address]
        # 更新内容をセッションに反映する
        session[:shipping_addresses][index] = @shipping_address

        # 会員情報を変更した場合はセッション内の会員情報を更新する
        if index == 0
          user.user_address.merge_only_need(session[:change_address].attributes)
          user.mirror_from_address(@shipping_address)
          # 会員の英語住所が登録されている場合は、英語住所にも変更を反映（国、郵便番号、電話番号）
          if user.sub_address
            user.sub_address.set_sub_address_from_address(@shipping_address)
            user.sub_address.save!
          end
          session[:cart_tmp_user] = user
        end
        session[:change_address] = nil
        @shipping_addresses = session[:shipping_addresses]
        return redirect_to(:action => 'show_addresses')
      #配送先追加
      else
        # 配送先リストに追加
        session[:shipping_addresses] << @shipping_address
        @shipping_addresses = session[:shipping_addresses]
        index = session[:shipping_addresses].length - 1
        session[:address] = nil
        return redirect_to(:action => 'show_addresses')
      end
    end

    #
    # 配送先編集画面表示
    #
    def change_address
      session[:change_address] = nil
      @error_flg = flash[:address_error] || nil
      @user_address = flash[:address]
      index = params[:id].to_i
      @user_address ||= session[:shipping_addresses][index]
      @user_address ||= UserAddress.new(:country_code => 'JP')
      @user_profile = current_user
      @edit_target_index = params[:id].to_i
      session[:change_address] = @user_address
    end

    #
    # 配送先削除
    #
    def delete_address
      return redirect_to(:action => 'show_addresses') unless params[:id]
      session[:shipping_addresses].delete_at(params[:id].to_i)
      redirect_to(:action => 'show_addresses')
    end

    #
    # 配送先追加確認
    #
    def confirm_address
      return redirect_to(:action => 'show_addresses') unless request.post?
      # モバイルは日本語しか対応しない
      params[:user_address]["country_code"] = "JP"

      # 「確認」ボタンが押されたかどうかを判別するフラグ
      submit_flg = true

      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        submit_flg = false
        params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
        flash[:address] = UserAddress.new(params[:user_address])
        redirect_to(:action => 'new_shipping_address')
        return
      end

      if submit_flg
        @user_address = UserAddress.new(params[:user_address])
        @user_address.locale = I18n.locale.to_s
        unless @user_address.valid?
          flash[:error] = @user_address.errors.zip
          flash[:address] = @user_address
          redirect_back_or_default(:action => :new_shipping_address)
          return
        end
        session[:address] = @user_address
      end
      @user_profile = current_user
    end


    #
    # 配送先編集確認
    #
    def confirm_modify_change
      return redirect_to(:action => 'show_addresses') unless request.post? || params[:user_address]
      # モバイルは日本語しか対応しない
      params[:user_address]["country_code"] = "JP"
      # 「確認」ボタンが押されたかどうかを判別するフラグ
      submit_flg = true

      # 編集対象のindexを取得する
      @edit_target_index = (params[:edit_target_index] || 0).to_i
      # 「郵便番号で住所表示」ボタンが押された場合
      if params.include?("zip_to_address")
        submit_flg = false
        params[:user_address][:address_1], params[:user_address][:address_2] = UserAddress.zip_to_address(params[:user_address][:zip_code])
        address_form = session[:change_address]
        address_form.merge_only_need(params[:user_address])
        flash[:address] = address_form
        redirect_to(:action => :change_address, :id => @edit_target_index)
      end
      if submit_flg
        address_form = session[:change_address]
        address_form.merge_only_need(params[:user_address])
        unless address_form.valid?
          flash[:address] = address_form
          flash[:error] = address_form.errors.zip
          redirect_to(:action => :change_address, :id => @edit_target_index)
          return
        end
        session[:change_address] = address_form
      end
      @user_address =  address_form
    end
  end
end