# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe "ApplicationControllerExt" do
  class AppTestController < ApplicationController
    before_filter :check_valid_ip, :only => [:check_valid_ip_test]
    def index; render :text => "テストボディ"; end
    def check_valid_ip_test; render :text => "valid_ip"; end
    def check_redirect; redirect_to :action => :index; end
  end
  controller_name :app_test

  before(:all) do
    ActionController::Routing::Routes.draw do |map|
      map.connect 'app_test/index', :controller => 'app_test', :action=> "index"
      map.connect 'app_test/check_valid_ip_test', :controller => 'app_test', :action=> "check_valid_ip_test"
      map.connect 'app_test/check_redirect', :controller => 'app_test', :action=> "check_redirect"
    end
  end

  describe 'mobile_filter' do
    describe 'モバイルリクエストの場合' do
      before(:each) do
        request.user_agent = 'DoCoMo/2.0 SH902i(c100;TB;W24H12)'
      end
      it 'trans_sid_modeが:mobileである。' do
        controller.trans_sid_mode.should == :mobile
      end
      it 'レスポンスヘッダーのContentTypeに「application/xhtml+xml」が入っている。' do
        get :index
        response.should be_success
        response.headers["Content-Type"].should =~ /application\/xhtml\+xml/
      end
      it 'レスポンスヘッダーの文字コードが「Shift_JIS」である。(DoCoMo)' do
        request.mobile.class.should == Jpmobile::Mobile::Docomo
        get :index
        response.should be_success
        response.charset.should == "Shift_JIS"
      end
      it 'レスポンスヘッダーの文字コードが「Shift_JIS」である。(AU)' do
        request.user_agent = 'KDDI-CA39 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0'
        request.mobile.class.should == Jpmobile::Mobile::Au
        get :index
        response.should be_success
        response.charset.should == "Shift_JIS"
      end
      it 'レスポンスヘッダーの文字コードが「UTF-8」である。(Softbank)' do
        request.user_agent = 'SoftBank/1.0/910T/TJ001/SN000000000000000 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1'
        request.mobile.class.should == Jpmobile::Mobile::Softbank
        get :index
        response.should be_success
        response.charset.should == "utf-8"
      end
    end

    describe 'PCリクエストの場合' do
      it 'レスポンスヘッダーのContentTypeに「text/html」が入っている。' do
        get :index
        response.should be_success
        response.headers["Content-Type"].should =~ /text\/html/
      end
      it 'レスポンスヘッダーの文字コードが「UTF-8」である。' do
        get :index
        response.should be_success
        response.headers["Content-Type"].should =~ /utf-8/
        response.charset.should == "utf-8"
      end
    end
  end

  describe "check_valid_ipフィルターが有効なとき、" do
    before(:each) do
      EcRider.config.stub!(:valid_ip_check).and_return(true)
    end
    describe "不正なIPからアクセスがあると、" do
      it "エラー画面（レスポンスコード=400）に遷移する。" do
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo,
          {:ident => 'unique1008', :valid_ip? => false, :supports_cookie? => false}))
        post :check_valid_ip_test
        response.should_not be_success
        response.code.should == "400"
        response.should render_template("cms/invalid_ip")
      end
    end
    describe "有効なIPからアクセスがあると、" do
      it "正常なレスポンスを返す。" do
        request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo,
          {:ident => 'unique1008', :valid_ip? => true, :supports_cookie? => false}))
        post :check_valid_ip_test
        response.should be_success
      end
    end
  end

  describe "customized jpmobile" do
    it "すべての機種でtrans_sidを使用する（au）" do
      request.user_agent = 'KDDI-CA39 UP.Browser/6.2.0.13.1.5 (GUI) MMP/2.0'
      request.session_options[:key] = "session_id"
      request.mobile.class.should == Jpmobile::Mobile::Au
      get :check_redirect
      response.should redirect_to(:controller => :app_test, :action => :index, :lang => 'ja', :session_id => request.session_options[:id])
    end
    it "すべての機種でtrans_sidを使用する（softbank）" do
      request.user_agent = 'SoftBank/1.0/910T/TJ001/SN000000000000000 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1'
      request.session_options[:key] = "session_id"
      request.mobile.class.should == Jpmobile::Mobile::Softbank
      get :check_redirect
      response.should redirect_to(:controller => :app_test, :action => :index, :lang => 'ja', :session_id => request.session_options[:id])
    end
    it "すべての機種でtrans_sidを使用する（docomo）" do
      request.user_agent = 'DoCoMo/2.0 SH902i(c100;TB;W24H12)'
      request.session_options[:key] = "session_id"
      request.mobile.class.should == Jpmobile::Mobile::Docomo
      get :check_redirect
      response.should redirect_to(:controller => :app_test, :action => :index, :lang => 'ja', :session_id => request.session_options[:id])
    end
    it "PCではtrans_idを使用しない" do
      request.stub!(:mobile?).and_return(false)
      get :check_redirect
      response.should redirect_to(:controller => :app_test, :action => :index, :lang => 'ja')
    end
  end
end