# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe OrderController do

  fixtures :users, :user_addresses, :user_emails, :user_points,
    :prefectures, :countries, :prefecture_displays,
    :order_items, :order_shippings, :orders,
    :products, :product_skus, :product_sku_inventories, :product_sku_secure_inventories

  def login_as_user
    user = User.find(1004, :include => :user_addresses)
    user.user_email = UserEmail.new(:email => 'email_address_01008@test.com', :pc_mobile_div_key => :pc)
    @request.session[:cart_tmp_user] = user
    @request.session[:user] = user.id
    @request.session[:shipping_addresses] = user.user_addresses
  end

  def login_as_mobile_user
    user = User.find(1004, :include => :user_addresses)
    user.user_email = nil
    user.user_email_mobile = UserEmail.new(:email => 'email_address_01008@test.com', :pc_mobile_div_key => :mobile)
    @request.session[:cart_tmp_user] = user
    @request.session[:user] = user.id
    @request.session[:shipping_addresses] = user.user_addresses
  end

  def set_cart_to_session
    cart = Cart.new
    p1 = product_skus(:product_skus_00012)
    cart.add(p1, p1.product.course_div_key, 1)
    session[:cart] = cart
  end

  describe "POST confirmation_order" do
    it "正常系（PCユーザー）" do
      set_cart_to_session
      login_as_user

      post :confirmation_order, :order_form => {:send_address_index => "0", :normal_payment_type => "1"}
      order = controller.instance_variable_get(:@order)
      order.should_not be_nil
      order.site_type_div_key.should == :pc
      user = controller.instance_variable_get(:@user)
      user.should_not be_nil
      user.user_email.should_not be_nil
      user.user_email_mobile.should be_nil
      controller.instance_variable_get(:@user_email).pc_mobile_div_key.should == :pc
      response.should render_template(:confirmation_order)
    end

    it "正常系（モバイルユーザー）" do
      request.stub!(:mobile?).and_return(true)
      request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo, {:supports_cookie? => false}))
      request.session_options[:key] = "session_id"
      ActionController::Base.session_options[:key] = "session_id"

      set_cart_to_session
      login_as_mobile_user

      post :confirmation_order, :order_form => {:send_address_index => "0", :normal_payment_type => "1"}
      order = controller.instance_variable_get(:@order)
      order.should_not be_nil
      order.site_type_div_key.should == :mobile
      user = controller.instance_variable_get(:@user)
      user.should_not be_nil
      user.user_email.should be_nil
      user.user_email_mobile.should_not be_nil
      controller.instance_variable_get(:@user_email).pc_mobile_div_key.should == :mobile
      response.should render_template(:confirmation_order_mobile)
    end
  end

  describe "POST payment" do
    before do
      request.stub!(:mobile?).and_return(true)
      request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo, {:supports_cookie? => false}))
      request.session_options[:key] = "session_id"
      ActionController::Base.session_options[:key] = "session_id"
      @user_profile = {
        :password => "pass1word",
        :password_confirmation => "pass1word",
        :remember_question => "A",
        :remember_answer => "A",
        :birth_year => "2000",
        :birth_month => "3",
        :birth_day => "11",
        :sex => "1"
      }
      @user_address = {
        :zip_code => "1210051",
        :tel_num => "000-111-2224",
        :address_1 => Iconv.conv('Shift_JIS', 'UTF-8', "東京都"),
        :address_2 => Iconv.conv('Shift_JIS', 'UTF-8', "足立区"),
        :address_3 => Iconv.conv('Shift_JIS', 'UTF-8', "神明4-1-6 マリンマンション1405号"),
        :name_first => Iconv.conv('Shift_JIS', 'UTF-8', "結衣"),
        :name_last => Iconv.conv('Shift_JIS', 'UTF-8', "空川"),
        :kana_name_first => Iconv.conv('Shift_JIS', 'UTF-8', "ユイ"),
        :kana_name_last => Iconv.conv('Shift_JIS', 'UTF-8', "ソラカワ")
      }
    end

    it "no user_form" do
      set_cart_to_session
      login_as_mobile_user
      post :payment
      response.should be_success
    end

    it "with user_form to no error." do
      set_cart_to_session
      login_as_mobile_user
      post :payment, :user_profile => @user_profile,
        :user_address => @user_address,
        :user_en_address => @user_en_address,
        :user_email => {
        :email => 'email_address_0100q@test.com'
      }
      response.should be_success
      response.should render_template('order/payment_mobile')
    end

    it "payment with zip_to_address" do
      set_cart_to_session
      login_as_mobile_user
      post :payment, :zip_to_address => 'test', :user_profile => @user_profile,
        :user_address => @user_address,
        :user_en_address => @user_en_address,
        :user_email => {
        :email => 'email_address_0100q@test.com'
      }
      response.should redirect_to(:action => :confirmation_cart, :lang => 'ja', :session_id => request.session_options[:id])
    end
  end

  def new_order
    org_order = Order.find(4)
    order = Order.new(org_order.attributes)
    org_order.order_shippings.each do |org_order_shipping|
      new_order_shipping = OrderShipping.new(org_order_shipping.attributes)
      org_order_shipping.order_items.each do |org_order_item|
        new_order_shipping.order_items << OrderItem.new(org_order_item.attributes)
      end
      new_order_shipping.status_div_key = :before_settlement
      order.order_shippings << new_order_shipping
    end
    order.site_type_div_key = :mobile # モバイルサイトで購入
    order
  end

  describe "POST create" do
    before do
      request.stub!(:mobile?).and_return(true)
      request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo, {:supports_cookie? => false}))
      request.session_options[:key] = "session_id"
      ActionController::Base.session_options[:key] = "session_id"
    end

    it "[正常系]モバイル会員　銀行振り込み" do
      login_as_mobile_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count

      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)

      order = Order.find(:first, :order => 'id desc')

      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid

      # 在庫は仮引当
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 1
      order_item.product_sku_secure_inventories.first.paid_flg.should be_false
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 22
      ProductSkuSecureInventory.count.should == before_secure_count + 1

      # ポイントは付与する
      order.user_points.length.should == 1
      order.user_points.first.given_point.should == 100

      # 顧客区分は会員のまま
      order.user.member_div_key.should == :membership
      session[:user].should_not be_nil

      # メールは1通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count

      response.should render_template('thanks/index')
    end
  end

  describe 'GET select_shipping_address' do
    before do
      request.stub!(:mobile?).and_return(true)
      request.stub!(:mobile).and_return(mock_model(Jpmobile::Mobile::Docomo, {:supports_cookie? => false}))
      request.session_options[:key] = "session_id"
      ActionController::Base.session_options[:key] = "session_id"
      @user_profile = {
        :password => "pass1word",
        :password_confirmation => "pass1word",
        :remember_question => "A",
        :remember_answer => "A",
        :birth_year => "2000",
        :birth_month => "3",
        :birth_day => "11",
        :sex => "1"
      }
      @user_address = {
        :zip_code => "1210051",
        :tel_num => "000-111-2224",
        :address_1 => Iconv.conv('Shift_JIS', 'UTF-8', "東京都"),
        :address_2 => Iconv.conv('Shift_JIS', 'UTF-8', "足立区"),
        :address_3 => Iconv.conv('Shift_JIS', 'UTF-8', "神明4-1-6 マリンマンション1405号"),
        :name_first => Iconv.conv('Shift_JIS', 'UTF-8', "結衣"),
        :name_last => Iconv.conv('Shift_JIS', 'UTF-8', "空川"),
        :kana_name_first => Iconv.conv('Shift_JIS', 'UTF-8', "ユイ"),
        :kana_name_last => Iconv.conv('Shift_JIS', 'UTF-8', "ソラカワ")
      }
    end

    it "show_addresses" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(:name_first=>'b'), UserAddress.new(:name_first=>'c')]
      get :show_addresses
      shipping_addresses = controller.instance_variable_get(:@shipping_addresses)
      shipping_addresses[0].name_first.should == 'a'
      response.should render_template('order/show_addresses_mobile')
    end

    it "select shipping" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(:name_first=>'b'), UserAddress.new(:name_first=>'c')]
      get(:payment, :send_address_index => 1)
      shipping_addresses = controller.instance_variable_get(:@shipping_addresses)
      shipping_addresses[0].name_first.should == "a"
      controller.instance_variable_get(:@send_address_index).should == 1
    end

    it "new_shipping_address" do
      login_as_user
      set_cart_to_session
      get :new_shipping_address
      response.should render_template('order/new_shipping_address_mobile')
    end

    it "edit_shipping_address" do
      login_as_user
      set_cart_to_session
      get :change_address, :edit_target_index => 1
      response.should render_template('order/change_address_mobile')
    end

    describe "confirm_address" do
      it "confirm_address valid error" do
        login_as_user
        set_cart_to_session
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(:name_first=>'c')]
        @user_address[:name_first] = ''
        post :confirm_address, :user_address=> @user_address
        session[:shipping_addresses].size.should == 2
        response.should redirect_to( :controller => :order, :action => :new_shipping_address, :session_id => request.session_options[:id], :lang=>'ja')
      end
      it "confirm_address success" do
        login_as_user
        set_cart_to_session
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(:name_first=>'c')]
        post :confirm_address, :user_address=>@user_address
        session[:shipping_addresses].size.should == 2
        session[:address].should_not be_nil
        response.should render_template('order/confirm_address_mobile')
      end
      it "confirm_address zip_to_address" do
        login_as_user
        set_cart_to_session
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(:name_first=>'c')]
        UserAddress.stub!(:zip_to_address).and_return(["愛知県", "名古屋市中区丸の内"])
        post :confirm_address,  :user_address => @user_address,:zip_to_address=>"郵便番号で住所表示"
        response.should redirect_to( :controller => :order, :action => :new_shipping_address, :session_id => request.session_options[:id], :lang=>'ja')
      end
    end

    describe "confirm_modify_change" do
      it "confirm_modify_change valid error" do
        login_as_user
        set_cart_to_session
        user_address = UserAddress.new(@user_address)
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), user_address, UserAddress.new(:name_first=>'c')]
        session[:change_address] = user_address
        @user_address[:name_first] = 'test'
        post :confirm_modify_change, :edit_target_index=>1, :user_address=> @user_address
        session[:shipping_addresses].size.should == 3
        response.should redirect_to( :controller => :order, :action => :change_address, :id => 1, :session_id => request.session_options[:id], :lang=>'ja')
      end
      it "confirm_modify_change success" do
        login_as_user
        set_cart_to_session
        @user_address[:country_code] = "JP"
        @user_address[:locale] = "ja"
        user_address = UserAddress.new(@user_address)
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), user_address, UserAddress.new(:name_first=>'c')]
        session[:change_address] = user_address
        @user_address[:name_first] = 'test'
        post :confirm_modify_change, :edit_target_index=>1, :user_address=> @user_address
        session[:shipping_addresses].size.should == 3
        response.should render_template('order/confirm_modify_change_mobile')
      end
      it "confirm_modify_change zip_to_address" do
        login_as_user
        set_cart_to_session
        @user_address[:country_code] = "JP"
        @user_address[:locale] = "ja"
        user_address = UserAddress.new(@user_address)
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), user_address, UserAddress.new(:name_first=>'c')]
        session[:change_address] = user_address
        @user_address[:name_first] = 'test'
        UserAddress.stub!(:zip_to_address).and_return(["愛知県", "名古屋市中区丸の内"])
        post :confirm_modify_change, :edit_target_index=>1, :user_address => @user_address,:zip_to_address=>"郵便番号で住所表示"
        response.should redirect_to( :controller => :order, :action => :change_address, :id => 1, :session_id => request.session_options[:id], :lang=>'ja')
      end
    end

    describe "update_shipping_address" do
      it "create update_shipping_address" do
        login_as_user
        set_cart_to_session
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(:name_first=>'c')]
        @user_address[:country_code] = "JP"
        @user_address[:locale] = "ja"
        session[:address] = UserAddress.new(@user_address)
        post :update_shipping
        response.should redirect_to( :controller => :order, :action => :show_addresses, :session_id => request.session_options[:id], :lang=>'ja')
      end

      it "update update_shipping_address" do
        login_as_user
        set_cart_to_session
        @user_address[:name_first] = 'test'
        @user_address[:country_code] = "JP"
        @user_address[:locale] = "ja"
        user_address = UserAddress.new(@user_address)
        session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), user_address, UserAddress.new(:name_first=>'c')]
        session[:change_address] = user_address
        post :update_shipping, :edit_target_index => 1
        response.should redirect_to( :controller => :order, :action => :show_addresses, :session_id => request.session_options[:id], :lang=>'ja')
      end
    end

    it "delete_address" do
      login_as_user
      set_cart_to_session

      @user_address[:name_first] = 'test'
      @user_address[:country_code] = "JP"
      @user_address[:locale] = "ja"
      session[:shipping_addresses] = [UserAddress.new(:name_first=>'a'), UserAddress.new(@user_address), UserAddress.new(:name_first=>'c')]
      post :delete_address, :id => 1
      response.should redirect_to( :controller => :order, :action => :show_addresses, :session_id => request.session_options[:id], :lang=>'ja')
      session[:shipping_addresses].size.should == 2
      session[:shipping_addresses][0].name_first.should == 'a'
      session[:shipping_addresses][1].name_first.should == 'c'
    end
  end
end