module EcRider
  module CloudFrontHelper
    def self.included(mod)
      mod.module_eval do
        include DispatcherMethods
      end
    end

    # include ::ActionView::Helpers::AssetTagHelper
    include ::EcRider::CloudFront

    DEFAULT_IMAGES_PATH = '/images'.freeze
    DEFAULT_JAVASCRIPTS_PATH = '/javascripts'.freeze
    DEFAULT_STYLESHEETS_PATH = '/stylesheets'.freeze

    def image_path_for_cloud_front(source)
      asset_url_for_cloud_front(source, DEFAULT_IMAGES_PATH)
    end

    def image_tag_for_cloud_front(source, options = {})
      actual_source = asset_url_for_cloud_front(source, DEFAULT_IMAGES_PATH)
      options.symbolize_keys!
      if options[:mouseover]
        options[:mouseover] = asset_url_for_cloud_front(options[:mouseover], DEFAULT_IMAGES_PATH)
      end
      options[:alt] ||= File.basename(path_to_image(source), '.*').split('.').first.to_s.capitalize
      image_tag(actual_source, options)
    end

    def javascript_path_for_cloud_front(source)
      source += ".js" unless source =~ /\.js$|\.javascript$/
      asset_url_for_cloud_front(source, DEFAULT_JAVASCRIPTS_PATH)
    end

    def javascript_include_tag_for_cloud_front(*source)
      if idx = source.index(:defaults)
        defaults_paths = ActionView::Helpers::AssetTagHelper::JAVASCRIPT_DEFAULT_SOURCES.map{|src| "#{src}.js"}
        source[idx, 1] = defaults_paths
      end
      javascript_include_tag(
        *args_map_without_options(*source){|src|
          src += ".js" unless src =~ /\.js$|\.javascript$/
          asset_url_for_cloud_front(src, DEFAULT_JAVASCRIPTS_PATH)})
    end

    def stylesheet_path_for_cloud_front(source)
      asset_url_for_cloud_front(source, DEFAULT_STYLESHEETS_PATH)
    end

    def stylesheet_link_tag_for_cloud_front(*source)
      stylesheet_link_tag(
        *args_map_without_options(*source){|src|
          asset_url_for_cloud_front(src, DEFAULT_STYLESHEETS_PATH)})
    end

    private
    def append_path_from_root_if_need(source, base_path)
      (/^(\/|http|ftp)/ =~ source) ? source : "#{base_path}/#{source}"
    end

    def asset_url_for_cloud_front(source, base_path)
      url_for_cloud_front(append_path_from_root_if_need(source, base_path))
    end

    def args_map_without_options(*args, &block)
      options = args.extract_options!
      args.map(&block) << options
    end


    public

    module DispatcherMethods
      DISPATHING_METHODS = [
        :image_path, :image_tag,
        :javascript_path, :javascript_include_tag,
        :stylesheet_path, :stylesheet_link_tag
      ]

      def self.included(mod)
        return if EcRider.config.cloud_front_url.blank?
        mod.module_eval do
          DISPATHING_METHODS.each do |m|
            alias_method_chain(m, :cloud_front)
          end
        end
      end

      private
      # ec_frontかつsslページ以外の場合cloud_frontを使用する
      def use_cloud_front?
        !(request.ssl? || @_cloud_front_method_dispatching) && EcRider.config.ec_front
      end

      public

      DISPATHING_METHODS.each do |m|
        module_eval(<<-EOS, __FILE__, __LINE__)
          def #{m}_with_cloud_front(*args, &block)
            method_dispatching_backup = @_cloud_front_method_dispatching
            begin
              method = use_cloud_front? ? :#{m}_for_cloud_front : :#{m}_without_cloud_front
              @_cloud_front_method_dispatching = true
              send(method, *args, &block)
            ensure
              @_cloud_front_method_dispatching = method_dispatching_backup
            end
          end
        EOS
      end

    end

  end
end
