# -*- coding: utf-8 -*-
module EcRider
  module I18nSupport
    # ApplicationController,OrderController,ApplicationHelper,CmsPage,CmsTemplateでインクルードしています

    private

    # i18nのメッセージリソースから文字列を取得します。
    def i18n_message(msg_symbol)
      I18n.t("activerecord.errors.messages.#{msg_symbol.to_s}")
    end

    # i18nのロケールを一時的に変更し処理を行います。
    #
    # 例. ロケールが'ja'であるとき、一時的に英語をのメッセージを取得したい場合。
    #  i18n_tmp_block('en'){ i18n_message(:inclusion) }
    #   => "is not included in the list"
    #
    def i18n_tmp_block(locale, &block)
      default_locale = I18n.locale
      begin
        I18n.locale = locale
        yield
      ensure
        I18n.locale = default_locale
      end
    end

    #
    # 言語の表示用文字列を返します。
    #
    def locale_for_display(locale)
      return '' if locale.blank?
      '%s[%s]' % [I18n.t(locale.to_s.gsub("-", "_")), locale.to_s]
    end
    
    #
    # 言語に合わせたpartialを描画します。
    # Ajaxの返り値としてrender partialを含むERBを返した際に、日本語のpartialが表示されてしまうため専用のメソッドにする
    #
    def render_partial_for_ajax_response(partial)
      locale = I18n.locale.to_s == 'ja' ? '' : "_#{I18n.locale.to_s.gsub("-", "_")}"
      render(:partial => "#{partial}#{locale}")
    end
  end
end


# locale_railsの
# lib/locale_rails/i18n.rb の内容を上書きします。
# 
module I18n
  module_function

  # Sets the locale.
  #  I18n.locale = "ja-JP"
  def locale=(tag)
    Locale.clear
    tag = Locale::Tag::Simple.parse(tag.to_s)
    Locale.current = tag
    Thread.current[:locale] = Locale.candidates(:type => :simple)[0]
  end

  # Sets the default locale.
  #  I18n.default_locale = "ja"
  def default_locale=(tag)
    tag = Locale::Tag::Simple.parse(tag.to_s)
    Locale.default = tag
    @@default_locale = tag
  end
  
  class << self

    # MissingTranslationData is overrided to fallback messages in candidate locales.
    def locale_rails_exception_handler(exception, locale, key, options) #:nodoc:
      ret = nil
      Locale.candidates(:type => :simple).each do |loc|
        begin
          ret = backend.translate(loc, key, options)
          break
        rescue I18n::MissingTranslationData 
          ret = I18n.default_exception_handler(exception, locale, key, options)
        end
      end
      ret
    end
    I18n.exception_handler = :locale_rails_exception_handler
  end

  def regularize(locale)
    return nil unless locale
    tag = Locale::Tag::Simple.parse(locale.to_s)
    tag ? tag.to_s : locale.to_s
  end

end

