# -*- coding: utf-8 -*-
module EcRider
  module SchemaCommentsExt
    TYPE_DIV_TABLE = 'table'
    TYPE_DIV_COLUMN = 'column'

    class SchemaCommentTable < ActiveRecord::Base
      set_table_name "table_column_names"
    end
    
    # schema_comments.ymlをDBへ登録する
    def self.load_schema_comments
      # schema_commentsテーブルが無い、または、schema_comments.ymlが無い場合はなにもしない
      return unless SchemaCommentTable.table_exists?
      return unless File.exists?(SchemaComments.yaml_path)

      schema_comments_hash = YAML.load_file(SchemaComments.yaml_path)
      SchemaCommentTable.transaction do
        SchemaCommentTable.destroy_all
        schema_comments_hash[SchemaComments::SchemaComment::TABLE_KEY].each do |table_name, name|
          SchemaCommentTable.new(:type_div => TYPE_DIV_TABLE, :table_name => table_name, :name => name).save!
        end
        schema_comments_hash[SchemaComments::SchemaComment::COLUMN_KEY].each do |table_name, column_comments|
          next unless column_comments
          column_comments.each do |column_name, name|
            SchemaCommentTable.new(:type_div => TYPE_DIV_COLUMN, :table_name => table_name, :column_name => column_name, :name => name).save!
          end
        end
      end
    end

    # DBからschema_comments.ymlを書き出す
    def self.dump_schema_comments
      return unless SchemaCommentTable.table_exists?
      table_comments, column_comments = {}, {}
      SchemaCommentTable.find(:all).each do |comment|
        if comment.type_div == TYPE_DIV_TABLE
          table_comments[comment.table_name] = comment.name
        elsif comment.type_div == TYPE_DIV_COLUMN
          column_comments[comment.table_name] ||= {}
          column_comments[comment.table_name][comment.column_name] = comment.name
        end
      end
      db = SchemaComments::SchemaComment::SortedStore.new(SchemaComments.yaml_path)
      db.transaction do
        db[SchemaComments::SchemaComment::TABLE_KEY] = table_comments
        db[SchemaComments::SchemaComment::COLUMN_KEY] = column_comments
      end
    end

    # SchemaComments::SchemaCommentにテーブルを使わせないようにする
    module SchemaComment
      def self.included(mod)
        mod.module_eval do
          set_table_name('dummy_schema_comments')
        end
      end
    end

    # SchemaComments::ConnectionAdapters::Adapterに対する修正パッチ
    module Adapter
      def self.included(mod)
        mod.module_eval do
          alias_method_chain :column_comments, :patch
        end
      end
      # 下のように呼び出すと、引数がcontentsがHashではなくString型になってしまい、カラムのコメントが登録できない。
      #   column_comments(:table_name => {:column_name => "name"})
      # そのため、下記のような形式でcolumn_commentsを呼び出せるようにcolumn_commentsを変更します。
      #   column_comments(:table_name, {:column_name => "name})
      def column_comments_with_patch(contents, columns = nil)
        if contents.is_a?(String) && columns.is_a?(Hash)
          columns.each_pair do |col, comment|
            column_comment(contents, col, comment) unless ::SchemaComments.quiet
          end
        else
          column_comments_without_patch(contents)
        end
      end
    end
  end
end

