require 'fileutils'

# キャッシュの制御を行うクラス
module EcRider
  class ViewCacheCtrl
    # 以前キャッシュがクリアされた日付
    cattr_accessor :before_cache_clear_time
    # 公開された日付を格納するディレクトリ
    PUBLISHED_DATE_DIR = File.join(RAILS_ROOT, 'tmp', 'published_date')
    # 公開された日付を検索するための正規表現文字列
    PUBLISHED_DATE_DIR_REG = PUBLISHED_DATE_DIR.to_s + '/*'

    class << self
      # ビューのキャッシュをクリアします。
      def view_cache_clear
        view_methods = (ActionView::Base::CompiledTemplates.instance_methods.sort - Object.methods).
          select{|method_name| method_name =~ /_run_erb_app47views47/}
        ActionView::Base::CompiledTemplates.module_eval do
          view_methods.each do |method_name|
            ActionView::Base.logger.debug "ActionView::Base::CompiledTemplates.remove_method(#{method_name})"
            remove_method(method_name)
          end
        end
        # インスタンスを作り直すので、こちらの方法は取らないようにします。
        # ActionController::Base.view_paths = [File.expand_path(File.join(RAILS_ROOT, 'app/views'))]
        ActionController::Base.view_paths.each do |path|
          path.instance_variable_set(:@loaded, false)
          path.instance_variable_set(:@paths, nil)
          path.load!
        end
        # ActionController::Base.view_paths.load! # 各pathでload!してるので不要
        # ActionMailer::Base.view_paths.load!
      end

      # 公開された日付が変更されたかどうかチェックします。
      def published_check
        Dir.glob(PUBLISHED_DATE_DIR_REG).each do |file|
          published_time = Time.parse(File::basename(file))
          if !@@before_cache_clear_time.nil? && @@before_cache_clear_time < published_time
            view_cache_clear
            ActiveRecord::Base.logger.debug "View cache clear!"
            @@before_cache_clear_time = published_time
          end
          @@before_cache_clear_time = published_time if @@before_cache_clear_time.nil?
        end
        @@before_cache_clear_time = Time.now if @@before_cache_clear_time.nil?
      end

      # テンプレートの再読み込みを行います。
      # ※ mongrelのように各プロセス毎にTCPポートをListenしている場合は、
      #    CommonController#update_viewsを使用してください。
      def remove_and_create_published_date
        Dir.glob(PUBLISHED_DATE_DIR_REG).each do |file|
          FileUtils.rm_f file
          ActiveRecord::Base.logger.debug "rm #{file}"
        end
        published_time = File.join(PUBLISHED_DATE_DIR, Time.now.strftime("%Y%m%d%H%M%S"))
        FileUtils.touch published_time
        ActiveRecord::Base.logger.debug "touch #{published_time}"
      end

      # 公開された日付を格納するフォルダを作成します。
      def create_tmp_dir
        unless File.exists?(PUBLISHED_DATE_DIR)
          FileUtils.mkdir_p(PUBLISHED_DATE_DIR)
          ActiveRecord::Base.logger.debug "mkdir #{PUBLISHED_DATE_DIR}"
        end
      end
    end
  end
end