# -*- coding: utf-8 -*-
require 'fileutils'

namespace :aws do

  desc "upload static files to cloud front"
  task :upload_static_files => :environment do
    files = ::EcRider::CloudFront.files_to_upload(true)

    puts "target_files are....\n  %s" % files.join("\n  ") << "\nTotal #{files.length} files."

    timestamp = Time.now.strftime(ENV['ECR_TIMESTAMP'] || ::EcRider::CloudFront::CLOUD_FRONT_VERSION_FORMAT)
    puts "upload timestamp is %s" % timestamp.inspect

    require 'aws/s3'
    s3 = Object.new
    s3.extend(::EcRider::CloudFront)
    targets = files.map{|file| file[file.index("\/public\/")+8..-1]}
    s3.create_and_upload_to_s3(targets, :timestamp => timestamp)
  end

  desc "create timestamped files from untimestamped files"
  task :create_timestamped_files => :environment do
    files = ::EcRider::CloudFront.files_to_upload

    puts "target_files are....\n  %s" % files.join("\n  ") << "\nTotal #{files.length} files."

    timestamp = Time.now.strftime(ENV['ECR_TIMESTAMP'] || ::EcRider::CloudFront::CLOUD_FRONT_VERSION_FORMAT)
    puts "upload timestamp is %s" % timestamp.inspect

    s3 = Object.new
    s3.extend(::EcRider::CloudFront)
    s3.create_timestamped_files(files, :timestamp => timestamp)
  end

  desc "download cached files from CloudFront to local public"
  task :download_cache => :environment do
    if EcRider.config.cloud_front_url
      require 'aws/s3'
      AWS::S3::Base.establish_connection!(
        :access_key_id     => EcRider.config.s3_access_key_id,
        :secret_access_key => EcRider.config.s3_secret_access_key)
      bucket_for_public =  AWS::S3::Bucket.find(EcRider.config.s3_bucket_name)
      obj = bucket_for_public
      puts (obj.methods.sort - Object.methods).inspect

      require 'net/http'
      require 'uri'
      uri = URI(EcRider.config.cloud_front_url)
      Net::HTTP.start(uri.host, uri.port) do |http|
        bucket_for_public.find_all do |obj|
          base_path = obj.path.gsub(/^.*#{EcRider.config.s3_bucket_name}/, '')
          http.request_get(base_path) do |res|
            filepath = File.join(RAILS_ROOT, 'public', base_path)
            puts "#{obj.path} ===> #{res.code} #{res.message}"
            case res.code.to_s
            when /20\d/ then
              open(filepath, 'wb') do |f|
                res.read_body{|data| f.write(data)}
              end
            else
              # ignore redirect and error
            end
          end
        end
      end

    end
  end
end
