namespace :db do
  namespace :countries do
    # countriesテーブルに
    # lib/countries.txtのデータを挿入します。
    desc "Insert country code data into countries to the current environment's database"
    task :load => :environment do
      require 'iconv'
      File.open("#{RAILS_ROOT}/lib/countries.txt") do |io|
        i = 0
        io.each do |line|
          i += 1
          if i > 2
            name_and_code = line.chomp!("\n").split(/;/)
            country = Country.new(
              :name => Iconv.iconv("UTF-8", "ISO8859-1", name_and_code[0].humanize).to_s,
              :code => name_and_code[1],
              :shipping_cost => 0
            )
            country.save!
            puts "#{country.name}, #{country.code}, #{country.shipping_cost}"
          else
            puts "[skip] " + line
          end
        end
      end
    end

    desc "Insert country code data into countries to the current environment's database"
    task :recreate => [:drop, :create, :load]
    
    desc "Create the countries table for the current environment's database."
    task :create => :environment do
      require_create_countries
      CreateCountries.up
    end

    desc "Drops the countries table for the current environment's database."
    task :drop => :environment do
      require_create_countries
      CreateCountries.down
    end

    # countriesテーブルのデータをコンソールに表示します。
    desc "Print all countries data from the current environment's database"
    task :print => :environment do
      require 'iconv'
      countries = Country.find(:all)
      puts "------------------------------------------------------------------------"
      puts "English Name, Code, Shipping Cost"
      puts "------------------------------------------------------------------------"
      countries.each { |country| puts "#{country.name}, #{country.code}, #{country.shipping_cost}" }
      puts "------------------------------------------------------------------------"
      puts "Count: #{countries.size}"
      puts "------------------------------------------------------------------------"
    end
    
    #
    # RAILS_ROOT/db/migrate/*_create_jp_address.rb を require します。
    #
    def require_create_countries
      Dir.glob(File.join(RAILS_ROOT, 'db', 'migrate', '*_create_countries.rb')) { |filename| require filename }
    end

  end
end

