require 'find'

def load_reflections
  model_file_names = []
  Find.find(File.join(RAILS_ROOT, "app", "models")) do |f|
    Find.prune if ".svn" == f
    model_file_names << f if /\.rb$/ =~ f
  end
  model_file_names.each{|f| require f}
  models = []
  ObjectSpace.each_object(Class) do |klass|
    models << klass if klass < ActiveRecord::Base
  end
  result = []
  models.each do |model|
    model.reflections.values.each do |reflection|
      ref_options = (reflection.options || {}).merge(
        reflection.instance_variable_get(:@optinos) || {})
      table_name = (ref_options[:class_name] ?
        ref_options[:class_name].constantize.table_name : nil) ||
        (reflection.macro == :belongs_to) ? 
        model.table_name : nil
      unless table_name
        # $stderr.puts reflection.inspect
        next
      end
      fk_key = ref_options[:foreign_key] || (
        (reflection.macro == :belongs_to) ? 
          "#{reflection.name}_id" :
          "#{model.table_name.singularize}_id")
      unless fk_key
        $stderr.puts reflection.inspect
        next 
      end
      result << {
        :table_name => table_name, 
        :foreign_key => fk_key }
    end
  end
  result.uniq
end

def show_lines(template)
  lines = []
  load_reflections.each do |ref|
    table_name = ref[:table_name]
    fk_name = ref[:foreign_key]
    next if fk_name == "id"
    lines << instance_eval('"' + template + '"')
  end
  puts lines.uniq.sort.join("\n")
end

namespace :gen_migrate do
  desc "generate add_index lines from model's reflections"
  task :add_index => :environment do
    show_lines('    add_index \'#{table_name}\', \'#{fk_name}\'')
  end
  
  desc "generate remove_index lines from model's reflections"
  task :remove_index => :environment do
    show_lines('    remove_index \'#{table_name}\', \'#{fk_name}\'')
  end
end
