namespace :git do

  namespace :symlinks do
    require 'pathname'

    roots = %w(design_sample ec_admin ec_front).inject({}) do |dest, name|
      dest[name.to_sym] = File.join(RAILS_ROOT, '..', name)
      dest
    end


    link_defs = { 
      :ec_admin => [
        'db',
        'app/models',
        'app/helpers/ec_rider',
        'config/initializers',
        'config/locales',
        'lib/ec_rider',
        'spec/models',
        'spec/fixtures',
        'spec/helpers/ec_rider',
        'test/unit',
        'test/fixtures',
        'test/functional/ec_rider',
        'vendor/plugins',
      ], 

      :design_sample => [
        'public/themes',
      ]
    }

    desc "setup symlinks for svn:externals"
    task :setup do
      dest_root = roots[:ec_front]
      link_defs.each do |src_key, links|
        src_root = roots[src_key]
        links.each do |link|
          link_src = File.join(src_root, *link.split('/'))
          link_dest = File.join(dest_root, *link.split('/'))
          next if File.exist?(link_dest)
          FileUtils.ln_s(link_src, link_dest,
            :verbose => true, :force => true, :noop => false)
        end
      end
    end

    desc "remove symlinks for svn:externals" 
    task :remove do
      dest_root = roots[:ec_front]
      link_defs.each do |src_key, links|
        src_root = roots[src_key]
        links.each do |link|
          link_src = File.join(src_root, *link.split('/'))
          link_dest = File.join(dest_root, *link.split('/'))
          next if File.exist?(link_dest)
          FileUtils.rm(link_src, link_dest,
            :verbose => true, :force => true, :noop => true)
        end
      end
    end
  end

end
