# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe AccountController do
  fixtures :users, :user_emails, :user_addresses, :mail_templates, :mail_deliveries, :kbn_codes, :kbn_code_names

  describe "ssl_required?" do
    it "should return true in production mode" do
      controller.stub!(:behave_as_production?).and_return(true)
      EcRider.config.stub!(:ssl_ignored).and_return(false)
      get :index
    end

    it "should return true in production mode but ssl ignored" do
      controller.stub!(:behave_as_production?).and_return(true)
      EcRider.config.stub!(:ssl_ignored).and_return(true)
      get :index
    end
  end
  
  
  describe "test index" do
    it "index with logged_in" do
      session[:user] = 1001
      get :index
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
    end

    it "index without logged_in" do
      session[:user] = nil
      get :index
      response.should redirect_to(:action => :login, :lang => 'ja')
    end
  end
  
  describe "test policy" do
    it "policy with logged_in  no forward_url" do
      session[:user] = 1001
      get :policy
      response.should redirect_to(:controller => :member, :action=> :index, :lang => 'ja')
    end

    it "policy with logged_in forward_url" do
      session[:user] = 1001
      request.env["HTTP_REFERER"] = "http://127.0.0.1:3001/ja/cart"
      get :policy
      response.should redirect_to "http://127.0.0.1:3001/ja/cart"
      session[:forward_url].should be_nil
    end

    it "policy without logged_in" do
      session[:user] = nil
      get :policy
      response.should render_template(:policy_information)
    end
  end
  
  describe "test entry_signup" do
    it "entry_signup with logged_in" do
      session[:user] = 1001
      get :entry_signup
      response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja')
    end

    it "entry_signup without logged_in" do
      session[:user] = nil
      get :entry_signup
      response.should render_template(:policy_information)
      controller.instance_variable_get(:@member_category_error).should_not be_nil
    end

    it "entry_signup with member_category_code" do
      session[:user] = nil
      get :entry_signup, :member_category_code => '001'
      response.should render_template(:policy_information)
      controller.instance_variable_get(:@member_category_div).should == '001'
      controller.instance_variable_get(:@member_category_name).should == 'ParentsChina会員'
    end

    it "entry_signup with general member's member_category_code" do
      session[:user] = nil
      get :entry_signup, :member_category_code => '000'
      response.should render_template(:policy_information)
      controller.instance_variable_get(:@member_category_error).should_not be_nil
    end

    it "entry_signup with wrong member_category_code" do
      session[:user] = nil
      get :entry_signup, :member_category_code => '0'
      response.should render_template(:policy_information)
      controller.instance_variable_get(:@member_category_error).should_not be_nil
    end
  end
  
  describe "test signup" do
    it "signup with logged_in and no forward_url" do
      session[:user] = 1001
      get :signup
      response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja')
    end

    it "signup with logged_in and forward_url" do
      session[:forward_url] = "http://127.0.0.1:3001/ja/cart"
      session[:user] = 1001
      get :signup
      response.should redirect_to "http://127.0.0.1:3001/ja/cart"
      session[:forward_url].should be_nil
    end


    it "signup with flash" do
      mock_user = flash[:user_profile] = mock_model(User)
      mock_main_address = flash[:user_address] = mock_model(UserAddress)
      mock_en_address = flash[:user_en_address] = mock_model(UserAddress)
      mock_email = flash[:user_email] = mock_model(UserEmail)
      post :signup
      response.should be_success
      response.should render_template(:signup)
      controller.instance_variable_get(:@user_profile).should == mock_user
      controller.instance_variable_get(:@user_email).should == mock_email
      controller.instance_variable_get(:@user_address).should == mock_main_address
      controller.instance_variable_get(:@user_en_address).should == mock_en_address
    end

    it "signup with session" do
      mock_user = User.find(1002)
      session[:signup] = mock_user
      post :signup
      response.should be_success
      response.should render_template(:signup)
      controller.instance_variable_get(:@user_profile).should == mock_user
      controller.instance_variable_get(:@user_email).should == mock_user.user_email
      controller.instance_variable_get(:@user_address).should == mock_user.user_address
      controller.instance_variable_get(:@user_en_address).should == mock_user.sub_address
    end

    it "signup init" do
      get :signup
      response.should be_success
      response.should render_template(:signup)
      controller.instance_variable_get(:@user_profile).sex_key.should == :male
      controller.instance_variable_get(:@user_profile).dm_send_div_key.should == :prefer
      controller.instance_variable_get(:@user_email).class.should == UserEmail
      controller.instance_variable_get(:@user_address).country_code.should == 'JP'
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
      controller.instance_variable_get(:@user_profile).member_category_div.should be_nil
      controller.instance_variable_get(:@user_email).class.should == UserEmail
      controller.instance_variable_get(:@user_address).country_code.should == 'JP'
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
    end

    it "signup init with member_category_div" do
      post :signup, :member_category_div => '001'
      response.should be_success
      response.should render_template(:signup)
      controller.instance_variable_get(:@user_profile).sex_key.should == :male
      controller.instance_variable_get(:@user_profile).dm_send_div_key.should == :prefer
      controller.instance_variable_get(:@user_profile).member_category_div.should == '001'
      controller.instance_variable_get(:@user_email).class.should == UserEmail
      controller.instance_variable_get(:@user_address).country_code.should == 'JP'
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
    end

    it "signup init with member_category_div by get" do
      get :signup, :member_category_div => '001'
      response.should be_success
      response.should render_template(:signup)
      controller.instance_variable_get(:@user_profile).sex_key.should == :male
      controller.instance_variable_get(:@user_profile).dm_send_div_key.should == :prefer
      controller.instance_variable_get(:@user_profile).member_category_div.should be_nil
    end
  end
  
  describe "test confirm_signup" do
    before do
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",  :kana_name_first => "ジロウ",   :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000",  :address_1 => "福岡県",  :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP'}
      @user_email = {:email => "test@test.jp", :pc_mobile_div => "1"}
      @user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970",
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A'}
      @user_en_address = {:name_last => "test", :name_first => "jiro",
        :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}
    end

    it "confirm_signup with get method" do
      get :confirm_signup
      response.should redirect_to(:action => 'signup', :lang => 'ja')
    end

    it "confirm_signup invalid" do
      post :confirm_signup
      response.should redirect_to(:action => 'signup', :lang => 'ja')
      flash[:user_profile].should_not be_nil
      flash[:user_profile].errors.length.should == 5
      flash[:user_address].should_not be_nil
      flash[:user_address].errors.length.should == 12
      flash[:user_email].should_not be_nil
      flash[:user_email].errors.length.should == 2
      flash[:user_en_address].should be_nil
    end

    it "confirm_signup invalid with sub_address" do
      @user_address[:kana_name_last] = nil
      post :confirm_signup, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should redirect_to(:action => 'signup', :lang => 'ja')

      flash[:user_profile].errors.length.should == 0
      flash[:user_address].errors.length.should == 2
      flash[:user_email].errors.length.should  == 0
      flash[:user_en_address].errors.length.should  == 0

      flash[:user_profile].attributes.should == User.new(@user_profile.merge(:member_div_key => :membership, :regist_div_key => :pc, :tel_num => "000-0000-0000",  :kana_name_last => nil,
          :kana_name_first => "ジロウ",   :name_last => "てすと", :name_first => "じろう", :login => 'てすと じろう', :pc_mail_magazine_send_div => 99)).attributes
      flash[:user_address].attributes.should == UserAddress.new(@user_address.merge(:locale => 'ja', :address_div_key => :user)).attributes
      flash[:user_email].attributes.should == UserEmail.new(@user_email.merge(:user_id => nil, :pc_mobile_div => "1")).attributes
      flash[:user_en_address].attributes.should == UserAddress.new(@user_en_address.merge(:address_div_key => :user_en, :country_code => 'JP',
          :tel_num => "000-0000-0000", :zip_code => "0000000", :locale => 'en')).attributes
    end

    it "confirm_signup valid" do
      post :confirm_signup, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      controller.instance_variable_get(:@user_profile).attributes.should ==
        User.new(@user_profile.merge(:member_div_key => :membership, :regist_div_key => :pc, :tel_num => "000-0000-0000",  :kana_name_last => "テスト",
          :kana_name_first => "ジロウ",   :name_last => "てすと", :name_first => "じろう", :login => 'てすと じろう', :pc_mail_magazine_send_div => 99)).attributes
      controller.instance_variable_get(:@user_email).attributes.should ==
        UserEmail.new(@user_email.merge(:user_id => nil, :pc_mobile_div => "1")).attributes
      controller.instance_variable_get(:@user_address).attributes.should ==
        UserAddress.new(@user_address.merge(:locale => 'ja', :address_div_key => :user)).attributes
      controller.instance_variable_get(:@user_en_address).attributes.should ==
        UserAddress.new(@user_en_address.merge(:address_div_key => :user_en, :country_code => 'JP',
          :tel_num => "000-0000-0000", :zip_code => "0000000", :locale => 'en')).attributes
      response.should be_success
      response.should render_template(:confirm_signup)
    end
  end
  
  describe "test create_user" do
    fixtures :shop_settings

    before do
      @address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",  :kana_name_first => "ジロウ",   :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000",  :address_1 => "福岡県",  :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP', :locale => 'ja'}
      @user_email = {:email => "test@test.jp", :pc_mobile_div => '1'}
      @user = {:birth_month => "1", :dm_send_div => "0", :job_div => "1", :birth_day => "23", :sex => "1",
        :birth_year => "1970", :member_div_key => :membership, :regist_div_key => :pc, :tel_num => "000-0000-0000",  :kana_name_last => "テスト",
        :kana_name_first => "ジロウ",   :name_last => "てすと", :name_first => "じろう", :login => 'じろう てすと',
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A'}
      @sub_address = {:name_last => "test", :name_first => "jiro", :address_div_key => :user_en, :country_code => 'JP', :locale => 'en',
        :tel_num => "000-0000-0000", :zip_code => "0000000", :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}
    end

    it "create_user with get method" do
      get :create_user
      response.should redirect_to(:action => 'signup', :lang => 'ja')
    end

    it "create_user should be success" do

      session[:signup] = User.new(@user)
      session[:signup].user_address = UserAddress.new(@address)
      session[:signup].user_addresses = [session[:signup].user_address, UserAddress.new(@sub_address)]
      session[:signup].user_email = UserEmail.new(@user_email)

      user_count = User.count()
      email_count = UserEmail.count()
      address_count = UserAddress.count()

      session[:forward_url] = "http://127.0.0.1:3001/show?product_code=0000000000011"

      post :create_user
      User.count().should == user_count + 1
      UserEmail.count().should == email_count + 1
      UserAddress.count().should == address_count + 2
      response.should redirect_to "http://127.0.0.1:3001/show?product_code=0000000000011"
      session[:forward_url].should be_nil
      #response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja')
      user = User.find(:first, :include => :user_email, :conditions => ["user_emails.email = ?", 'test@test.jp'])
      user.test_flg.should be_nil
      MailDelivery.count(:conditions => ["user_id = ? and mail_div = ?", user.id, MailTemplate.mail_type_div_id_by_key(:regist_user)]).should == 1
    end

    it "create_user should be success with member_category_div" do
      @user[:member_category_div] = 'abc'
      session[:signup] = User.new(@user)
      session[:signup].user_address = UserAddress.new(@address)
      session[:signup].user_addresses = [session[:signup].user_address, UserAddress.new(@sub_address)]
      session[:signup].user_email = UserEmail.new(@user_email)

      session[:forward_url] = "http://127.0.0.1:3001/show?product_code=0000000000011"

      post :create_user

      user = User.find(:first, :order => 'id desc')
      user.member_category_div.should == 'abc'
    end
    it "create_user should be success" do

      session[:signup] = User.new(@user)
      session[:signup].user_address = UserAddress.new(@address)
      session[:signup].user_addresses = [session[:signup].user_address, UserAddress.new(@sub_address)]
      session[:signup].user_email = UserEmail.new(@user_email)

      user_count = User.count()
      email_count = UserEmail.count()
      address_count = UserAddress.count()

      @template.stub(:logged_in?).and_return(true)
      post :create_user
      response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja', :id => '000')
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",  :kana_name_first => "ジロウ",   :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000",  :address_1 => "福岡県",  :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP', :locale => 'ja'}
      user_email = {:email => "test@test.jp", :pc_mobile_div => '1'}
      user = {:birth_month => "1", :dm_send_div => "0", :job_div => "1", :birth_day => "23", :sex => "1",
        :birth_year => "1970", :member_div_key => :membership, :regist_div_key => :pc, :tel_num => "000-0000-0000",  :kana_name_last => "テスト",
        :kana_name_first => "ジロウ",   :name_last => "てすと", :name_first => "じろう", :login => 'じろう てすと',
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A'}
      sub_address = {:name_last => "test", :name_first => "jiro", :address_div_key => :user_en, :country_code => 'JP', :locale => 'en',
        :tel_num => "000-0000-0000", :zip_code => "0000000", :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}

      session[:signup] = User.new(user)
      session[:signup].user_address = UserAddress.new(address)
      session[:signup].user_addresses = [session[:signup].user_address, UserAddress.new(sub_address)]
      session[:signup].user_email = UserEmail.new(user_email)

      user_count = User.count
      email_count = UserEmail.count
      address_count = UserAddress.count

      post :create_user
      User.count.should == user_count + 1
      UserEmail.count.should == email_count + 1
      UserAddress.count.should == address_count + 2
      response.should redirect_to(:controller => 'member', :action => 'index', :lang => 'ja', :id => '000')
      User.find(:first, :include => :user_email, :conditions => ["user_emails.email = ?", 'test@test.jp']).test_flg.should be_true
    end
  end
  
  # 各言語の分だけテストを実行したかったんですが、できませんでした・・・・
  # I18n.supported_locales.each do |test_locale|
  #   describe "test with #{test_locale}" do
  
  # before(:each) do
  #   Locale.default = test_locale.to_s
  #   I18n.locale = test_locale.to_s
  # end
  
  describe "test login" do
    it "login during logged_in no forward_url" do
      session[:user] = 1001
      get :login
      response.should redirect_to(:controller => :member, :action=> :index, :lang => 'ja')
    end

    it "login during logged_in forward_url" do
      session[:forward_url] =  "http://127.0.0.1:3001/show?product_code=0000000000011"
      session[:user] = 1001
      get :login
      response.should redirect_to "http://127.0.0.1:3001/show?product_code=0000000000011"
      session[:forward_url].should be_nil
    end

    it "login with flash" do
      flash[:user_login_form] = UserLoginForm.new(:email => 'wrong_address@test.com')
      get :login
      response.should be_success
      response.should render_template(:login)
      controller.instance_variable_get(:@user_login_form).password.should be_nil
      controller.instance_variable_get(:@user_login_form).email.should == 'wrong_address@test.com'
    end

    it "login should be success" do
      get :login
      response.should be_success
      response.should render_template(:login)
    end
  end
  
  describe "test authenticate" do
    it "authenticate with get method" do
      get :authenticate
      response.should  redirect_to(:action => 'login', :lang => 'ja')
    end

    it "authenticate valid with remember_me no forward_url" do
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1'
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
      controller.instance_variable_get(:@remember_me).should == '1'
      # Rails2.3.2でcookiesの仕様が変わったぽい。以前はCookieオブジェクトを返していたが、文字列を返すようになっている。
      # cookies[:auth_token].value.should_not be_nil
      # cookies[:auth_token].expires.should_not be_nil
      cookies[:auth_token].should_not be_nil
    end

    it "authenticate valid with remember_me forwar_url" do
      session[:forward_url] =  "http://127.0.0.1:3001/show?product_code=0000000000011"
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1'
      response.should redirect_to "http://127.0.0.1:3001/show?product_code=0000000000011"
      controller.instance_variable_get(:@remember_me).should == '1'
      session[:forward_url].should be_nil
      # Rails2.3.2でcookiesの仕様が変わったぽい。以前はCookieオブジェクトを返していたが、文字列を返すようになっている。
      # cookies[:auth_token].value.should_not be_nil
      # cookies[:auth_token].expires.should_not be_nil
      cookies[:auth_token].should_not be_nil
    end

    it "authenticate invalid" do
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'wrongpassword'}, :remember_me => '1'
      response.should redirect_to(:controller => 'account', :action => 'login', :lang => 'ja')
      flash[:user_login_form].attributes.should == {'password' => '', 'email' => 'member23@ecrider.com'}
    end

    it "other_redirect case success" do
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => 'cart', :action => 'index'}
      response.should redirect_to(:controller => 'cart', :action => 'index', :lang => 'ja')
    end

    it "other_redirect case invalid" do
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'wrongpassword'}, :remember_me => '1', :completion_url => {:controller => 'cart', :action => 'index'}
      response.should redirect_to(:controller => 'cart', :action => 'index', :lang => 'ja')
      flash[:user_login_form].attributes.should == {'password' => '', 'email' => 'member23@ecrider.com'}
      flash[:login_error_notice].should == "メールアドレスかパスワードが間違っています。"
    end

    it "other_redirect invalid" do
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => nil, :action => nil}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => nil, :action => 'index'}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => 'cart', :action => nil}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => "", :action => ""}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => "", :action => "index"}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => "cart", :action => ""}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => '', :action => ''}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => '', :action => 'index'}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')

      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1', :completion_url => {:controller => "cart", :action => ""}
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
    end

  end
  
  describe "test logout" do
    it "logout should be success" do
      session[:user] = 1001
      get :logout
      response.should redirect_to(:controller => 'cms', :action => 'index', :lang => 'ja')
      flash[:notice].should == nil
    end


  end
  #   end
  # end
  
  describe "test change_password" do
    it "change_password with current_user" do
      session[:user] = 1001
      get :change_password
      controller.instance_variable_get(:@user).id.should == 1001
      controller.instance_variable_get(:@user).password.should be_blank
    end
    it "change_password with flash" do
      session[:user] = 1001
      #      flash[:user] = User.new()
      get :change_password
      controller.instance_variable_get(:@user).id.should == 1001
      controller.instance_variable_get(:@user).password.should be_blank
    end
  end
  
  describe "test update_password" do
    before do
      session[:user] = 1001
    end

    it "update_password with get method" do
      get :update_password
      response.should redirect_to(:action => 'change_password', :lang => 'ja')
    end

    it "update_password invalid" do
      post :update_password, :user => {:password => 'pass1', :password_confirmation => 'pass2'}
      response.should redirect_to(:action => :change_password, :lang => 'ja')
      flash[:user].password.should == 'pass1'
      flash[:user].password_confirmation == 'pass2'
      User.find(1001).test_flg.should be_nil
    end

    it "update_password valid" do
      post :update_password, :user => {:password => 'pass2', :password_confirmation => 'pass2'}
      response.should redirect_to(:action => :index, :lang => 'ja')
      controller.instance_variable_get(:@user).id = 1001
      controller.instance_variable_get(:@user).password.should == 'pass2'
      User.find(1001).test_flg.should be_nil
    end

    it "update_password valid false" do
      controller.stub!(:current_user).and_return(User.new(:id => '不正なID'))
      post :update_password, :user => {:password => 'pass2', :password_confirmation => 'pass2'}
      response.should redirect_to(:action => :change_password, :lang => 'ja')
      controller.instance_variable_get(:@user).id = 1001
      controller.instance_variable_get(:@user).password.should == 'pass2'
      User.find(1001).test_flg.should be_nil
    end

    it "ec_testの場合" do
      User.update(1001, :test_flg => true)
      post :update_password, :user => {:password => 'pass2', :password_confirmation => 'pass2'}
      response.should redirect_to(:action => :index, :lang => 'ja')
      controller.instance_variable_get(:@user).id = 1001
      controller.instance_variable_get(:@user).password.should == 'pass2'
      User.find(1001).test_flg.should be_true
    end
  end
  
  describe "test change_information" do
    before do
      session[:user] = 1001
    end

    it "change_information with flash" do
      flash[:user_profile] = mock_user = mock_model(User)
      flash[:user_address] = mock_address = mock_model(UserAddress)
      flash[:user_email] = mock_email = mock_model(UserEmail)
      flash[:user_en_address] = mock_sub_address = mock_model(UserAddress)

      mock_user.should_receive(:not_decided_address=).with(true).once

      get :change_information
      response.should render_template('change_information')
      session[:account].should == nil
      controller.instance_variable_get(:@user_profile).should == mock_user
      controller.instance_variable_get(:@user_address).should == mock_address
      controller.instance_variable_get(:@user_email).should == mock_email
      controller.instance_variable_get(:@user_en_address).should == mock_sub_address
      #controller.instance_variable_get(:@user_emails).email.should == User.find(1001).user_email.email
    end

    it "change_information without flash" do
      mock_user = User.find(1001)
      get :change_information
      response.should render_template('change_information')
      session[:account].should == nil
      controller.instance_variable_get(:@user_profile).id.should == 1001
      controller.instance_variable_get(:@user_address).id.should == mock_user.user_address.id
      controller.instance_variable_get(:@user_email).id.should == mock_user.user_email.id
      controller.instance_variable_get(:@user_en_address).should be_nil
      #controller.instance_variable_get(:@user_emails).email.should == mock_user.user_email.email
    end
  end
  
  describe "test confirm_update_information" do
    before do
      session[:user] = 1001
      @mock_user = User.find(1001)
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト",  :kana_name_first => "ジロウ",   :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000",  :address_1 => "福岡県",  :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP'}
      @user_email = {:email => "test@test.jp", :pc_mobile_div => "1"}
      @user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970"}
      @user_en_address = {:name_last => "test", :name_first => "jiro",
        :address_1 => "Fukuoka",  :address_2 => "1-1-1 Tenjin Chuo-ku Fukuoka-shi", :address_3 => ""}
    end

    it "confirm_update_information with get method" do
      get :confirm_update_information
      response.should redirect_to(:action => 'change_information', :lang => 'ja')
    end

    it "confirm_update_information invalid" do
      @user_address[:name_first] = ''
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should redirect_to(:action => 'change_information', :lang => 'ja')
      session[:account][:user_profile].id.should == @mock_user.id
      session[:account][:user_email].id.should == @mock_user.user_email.id
      session[:account][:user_address].id.should == @mock_user.user_address.id
      session[:account][:user_en_address].id.should == @mock_user.sub_address.id
      flash[:user_address].errors.length.should == 2
    end

    it "confirm_update_information add sub_address" do
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should render_template('confirm_update_information')
      @mock_user.attributes = @user_profile.merge(:tel_num => "000-0000-0000",  :kana_name_last => "テスト", :kana_name_first => "ジロウ",
        :name_last => "てすと", :name_first => "じろう")
      controller.instance_variable_get(:@user_profile).attributes.should == @mock_user.attributes
      mock_email = @mock_user.user_email
      mock_email.attributes = @user_email
      controller.instance_variable_get(:@user_email).attributes.should == mock_email.attributes
      mock_address = @mock_user.user_address
      mock_address.attributes = @user_address
      controller.instance_variable_get(:@user_address).attributes.should == mock_address.attributes
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
      controller.instance_variable_get(:@user_en_address).name_last.should == "test"
      controller.instance_variable_get(:@user_en_address).tel_num.should == "000-0000-0000"
    end

    it "confirm_update_information delete sub_address" do
      session[:user] = 1008
      @mock_user = User.find(1008)
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email
      response.should render_template('confirm_update_information')
      @mock_user.attributes = @user_profile.merge(:tel_num => "000-0000-0000",  :kana_name_last => "テスト", :kana_name_first => "ジロウ",
        :name_last => "てすと", :name_first => "じろう")
      controller.instance_variable_get(:@user_profile).attributes.should == @mock_user.attributes
      mock_email = @mock_user.user_email
      mock_email.attributes = @user_email
      controller.instance_variable_get(:@user_email).attributes.should == mock_email.attributes
      mock_address = @mock_user.user_address
      mock_address.attributes = @user_address
      controller.instance_variable_get(:@user_address).attributes.should == mock_address.attributes
      controller.instance_variable_get(:@user_en_address).should be_nil
    end

    it "confirm_update_information change sub_address" do
      session[:user] = 1008
      @mock_user = User.find(1008)
      post :confirm_update_information, :user_profile => @user_profile, :user_address => @user_address, :user_email => @user_email, :user_en_address => @user_en_address
      response.should render_template('confirm_update_information')
      @mock_user.attributes = @user_profile.merge(:tel_num => "000-0000-0000",  :kana_name_last => "テスト", :kana_name_first => "ジロウ",
        :name_last => "てすと", :name_first => "じろう")
      controller.instance_variable_get(:@user_profile).attributes.should == @mock_user.attributes
      mock_email = @mock_user.user_email
      mock_email.attributes = @user_email
      controller.instance_variable_get(:@user_email).attributes.should == mock_email.attributes
      mock_address = @mock_user.user_address
      mock_address.attributes = @user_address
      controller.instance_variable_get(:@user_address).attributes.should == mock_address.attributes
      controller.instance_variable_get(:@user_en_address).class.should == UserAddress
      controller.instance_variable_get(:@user_en_address).name_last.should == "test"
      controller.instance_variable_get(:@user_en_address).tel_num.should == "000-0000-0000"
    end
  end
  
  describe "test update_information" do
    before do
      session[:user] = 1001
    end

    it "update_information with get method" do
      get :update_information
      response.should redirect_to(:action => 'change_information', :lang => 'ja')
    end

    it "update_information add sub_address" do
      mock_user = User.find(1001)
      mock_user.name_first = 'test1'
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email
      session[:account][:user_address] = mock_user.user_address
      mock_sub_address = UserAddress.new(UserAddress.find(1012).attributes)
      mock_sub_address.user_id = 1001
      session[:account][:user_en_address] = mock_sub_address
      address_count = UserAddress.count(:conditions => {:user_id => 1001})
      post :update_information
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1001}).should == address_count + 1
      User.find(1001).name_first.should == 'test1'
      User.find(1001).sub_address.should_not be_nil
      User.find(1008).test_flg.should be_nil
    end

    it "update_information delete sub_address" do
      mock_user = User.find(1008)
      mock_user.name_first = 'test1'
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email
      session[:account][:user_address] = mock_user.user_address
      session[:account][:user_en_address] = mock_user.sub_address
      session[:delete_en_address] = true

      address_count = UserAddress.count(:conditions => {:user_id => 1008})
      post :update_information
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1008}).should == address_count - 1
      User.find(1008).name_first.should == 'test1'
      User.find(1008).sub_address.should be_nil
      User.find(1008).test_flg.should be_nil
    end

    it "update_information change sub_address" do
      mock_user = User.find(1008)
      mock_user.name_first = 'test1'
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email
      session[:account][:user_address] = mock_user.user_address
      session[:account][:user_en_address] = mock_user.sub_address

      address_count = UserAddress.count(:conditions => {:user_id => 1008})
      post :update_information
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1008}).should == address_count
      User.find(1008).name_first.should == 'test1'
      User.find(1008).sub_address.should_not be_nil
      User.find(1008).test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      mock_user = User.find(1008)
      mock_user.name_first = 'test1'
      mock_user.test_flg = true
      session[:account] = {}
      session[:account][:user_profile] = mock_user
      session[:account][:user_email] = mock_user.user_email
      session[:account][:user_address] = mock_user.user_address
      session[:account][:user_en_address] = mock_user.sub_address

      address_count = UserAddress.count(:conditions => {:user_id => 1008})
      post :update_information
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
      session[:account].should be_nil
      UserAddress.count(:conditions => {:user_id => 1008}).should == address_count
      User.find(1008).name_first.should == 'test1'
      User.find(1008).sub_address.should_not be_nil
      User.find(1008).test_flg.should be_true
    end
  end
  
  describe "test change_security_answer" do
    before do
      session[:user] = 1001
    end

    it "change_security_answer with flash" do
      flash[:user] = User.find(1001)
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      get :change_security_answer
      response.should render_template('change_security_answer')
      session[:change_security_answer].should be_nil
    end

    it "change_recrity_answer without flash" do
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      get :change_security_answer
      response.should render_template('change_security_answer')
      session[:change_security_answer].should be_nil
    end
  end
  
  describe "test confirm_security_answer" do
    before do
      session[:user] = 1001
    end

    it "confirm_security_answer with get method" do
      get :confirm_security_answer
      response.should redirect_to(:action => 'change_security_answer', :lang => 'ja')
    end

    it "confirm_security_answer invalid" do
      post :confirm_security_answer, :user => {:remember_question => 'QQQ', :remember_answer => ''}
      response.should redirect_to(:action => :change_security_answer, :lang => 'ja')
      flash[:user].remember_question.should == 'QQQ'
      flash[:user].remember_answer.should == ''
    end

    it "confirm_security_answer valid" do
      post :confirm_security_answer, :user => {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      response.should render_template('confirm_security_answer')
      session[:change_security_answer].should == {"remember_question" => 'QQQ', "remember_answer" => 'AAA'}
    end
  end
  
  describe "test update_security_answer" do
    before do
      session[:user] = 1001
    end

    it "update_security_answer with get method" do
      get :update_security_answer
      response.should redirect_to(:action => 'change_security_answer', :lang => 'ja')
    end

    it "update_security_answer should be success" do
      session[:user] = 1001
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      post :update_security_answer
      response.should redirect_to(:action => :index, :lang => 'ja')
      User.find(1001).remember_question.should == 'QQQ'
      User.find(1001).remember_answer.should == 'AAA'
      User.find(1001).test_flg.should be_nil
    end

    it "update_security_answer should be success false" do
      session[:user] = 1001
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      controller.stub!(:current_user).and_return(User.new(:id => '不正なID'))
      post :update_security_answer
      response.should redirect_to(:action => :change_security_answer, :lang => 'ja')
      User.find(1001).test_flg.should be_nil
    end

    it "ec_testの場合" do
      User.update(1001, :test_flg => true)
      session[:user] = 1001
      session[:change_security_answer] = {:remember_question => 'QQQ', :remember_answer => 'AAA'}
      post :update_security_answer
      response.should redirect_to(:action => :index, :lang => 'ja')
      User.find(1001).remember_question.should == 'QQQ'
      User.find(1001).remember_answer.should == 'AAA'
      User.find(1001).test_flg.should be_true
    end
  end
  
  describe "ユーザロック制御テスト" do
    it "login_challenge_count was increment, by authenticate invalid" do
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'wrongpassword'}, :remember_me => '1'
      response.should redirect_to(:controller => 'account', :action => 'login', :lang => 'ja')
      user = User.find 1009
      user.login_challenge_count.should == 1
      user.locked_at.should be_nil
      flash[:login_error_notice].should be_nil
    end
    it "test to lock, when login failed User::MAX_LOGIN_CHALLENGE_COUNT times, by authenticate invalid" do
      user = User.find 1009
      user.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT - 1
      user.locked_at = nil
      user.save!
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'wrongpassword'}, :remember_me => '1'
      response.should redirect_to(:controller => 'account', :action => 'login', :lang => 'ja')
      actual = User.find 1009
      actual.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT
      actual.locked_at.should <= Time.now
      actual.locked_at.should > Time.now.ago(1.minute)
      flash[:login_error_notice].should == I18n.t("activerecord.errors.messages.EM030004")
    end
    it "invalid authenticate when user locked" do
      user = User.find 1009
      locked_time = Time.now
      user.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      user.locked_at = locked_time
      user.save!
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'wrongpassword'}, :remember_me => '1'
      response.should redirect_to(:controller => 'account', :action => 'login', :lang => 'ja')
      actual = User.find 1009
      actual.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT
      actual.locked_at.to_s.should == locked_time.to_s
      flash[:login_error_notice].should == I18n.t("activerecord.errors.messages.EM030004")
    end
    it "valid authenticate when user locked" do
      user = User.find 1009
      locked_time = Time.now
      user.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      user.locked_at = locked_time
      user.save!
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1'
      #ロック中は正しいID/パスワードでもエラーとする
      response.should redirect_to(:controller => 'account', :action => 'login', :lang => 'ja')
      actual = User.find 1009
      actual.login_challenge_count.should == User::MAX_LOGIN_CHALLENGE_COUNT
      actual.locked_at.to_s.should == locked_time.to_s
      flash[:login_error_notice].should == I18n.t("activerecord.errors.messages.EM030004")
    end
    it "locked user elasped more than User::USER_LOCK_PERIOD_SECONDS, and authenticate invalid" do
      user = User.find 1009
      user.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      user.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).ago(1.second)
      user.save!
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'wrongpassword'}, :remember_me => '1'
      response.should redirect_to(:controller => 'account', :action => 'login', :lang => 'ja')
      actual = User.find 1009
      actual.login_challenge_count.should == 1
      actual.locked_at.should be_nil
      flash[:login_error_notice].should be_nil
    end
    it "locked user elasped more than User::USER_LOCK_PERIOD_SECONDS, and authenticate valid" do
      user = User.find 1009
      user.login_challenge_count = User::MAX_LOGIN_CHALLENGE_COUNT
      user.locked_at = Time.now.ago(User::USER_LOCK_PERIOD_SECONDS.second).ago(1.second)
      user.save!
      post :authenticate, :user_login_form => {:email => 'member23@ecrider.com', :password => 'pass1'}, :remember_me => '1'
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
      actual = User.find 1009
      actual.login_challenge_count.should == 0
      actual.locked_at.should be_nil
    end
  end

  describe "ログイン後のリダイレクト" do
    def mock_login_form(stubs={})
      @mock_login_form ||= mock_model(UserLoginForm, stubs)
    end

    def mock_user(stubs={})
      @mock_user = mock_model(User, stubs)
    end

    it "メールアドレスが登録されていなければ、メールアドレス登録画面へ遷移する" do
      UserLoginForm.stub!(:new).and_return(
        mock_login_form(
          :email => 'hoge@ecrider.com',
          :password => 'pass1',
          :authenticate => mock_user(:clear_login_challenge_count => true)
        )
      )
      controller.stub!(:valid_mailaddress).with(@mock_user).and_return(false)
      
      post :authenticate
      response.should redirect_to(:controller => :account, :action => :add_mailaddress, :lang => 'ja')
    end

    it "メールアドレスが登録されていれば、会員画面に遷移する" do
      UserLoginForm.stub!(:new).and_return(
        mock_login_form(
          :email => 'member23@ecrider.com',
          :password => 'pass1',
          :authenticate => mock_user(:clear_login_challenge_count => true)
        )
      )
      controller.stub!(:valid_mailaddress).with(@mock_user).and_return(true)
      
      post :authenticate
      response.should redirect_to(:controller => :member, :action => :index, :lang => 'ja')
    end
  end

end
