# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe "ApplicationController" do
  class AppTestController < ApplicationController
    def index; render :text => "テストボディ"; end
  end
  controller_name :app_test

  before(:all) do
    ActionController::Routing::Routes.draw do |map|
      map.connect 'app_test/index', :controller => 'app_test', :action=> "index"
    end
  end
  
  it 'productionモードかつ環境変数ECR_ENABLE_PASSENGERが設定されている場合、公開チェックを行う。' do
    with_constants :RAILS_ENV => 'production' do
      EcRider.config.stub!(:enable_passenger).and_return(true)
      EcRider::ViewCacheCtrl.should_receive(:published_check).and_return(true)
      post :index
      response.should be_success
    end
  end
  it 'productionモードかつ環境変数ECR_ENABLE_PASSENGERが設定されていない場合、公開チェックは行われない。' do
    with_constants :RAILS_ENV => 'production' do
      EcRider::ViewCacheCtrl.should_not_receive(:published_check)
      post :index
      response.should be_success
    end
  end
  it 'testモードかつ環境変数ECR_ENABLE_PASSENGERが設定されている場合、公開チェックは行われない。' do
    EcRider.config.stub!(:enable_passenger).and_return(true)
    EcRider::ViewCacheCtrl.should_not_receive(:published_check)
    post :index
    response.should be_success
  end
  it 'developmentモードかつ環境変数ECR_ENABLE_PASSENGERが設定されている場合、公開チェックは行われない。' do
    with_constants :RAILS_ENV => 'development' do
      EcRider.config.stub!(:enable_passenger).and_return(true)
      EcRider::ViewCacheCtrl.should_not_receive(:published_check)
      post :index
      response.should be_success
    end
  end
end