# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe CartController do
  fixtures :users, :user_emails, :products, :product_skus

  describe 'GET index' do
    it 'session[:cart].products.size.should == 0' do
      get 'index'
      response.should be_success
      session[:cart].items.size.should == 0
    end

    it 'session[:cart].products.size.should == 1' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1
      nomal_products[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      nomal_products[0].qty.should == 1
      items = session[:cart].items.values
      items.size.should == 1
      items[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      items[0].qty.should == 1

      get 'index', {:sku_code => '0000000000002-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      session[:cart].items.size.should == 2
    end
  end

  describe 'DELETE remove' do
    it 'should render_template empty_cart' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1

      request.stub!(:xml_http_request?).and_return(true)
      delete 'remove', {:id => nomal_products[0].key}

      response.should be_success
      response.should render_template('cart/_cart_list')
      session[:cart].item_list.should be_empty
    end

    it 'should render_template normal_deli_form' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      get 'index', {:sku_code => '0000000000002-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 2

      request.stub!(:xml_http_request?).and_return(true)
      delete 'remove', {:id => nomal_products[0].key}

      response.should be_success
      response.should render_template('cart/_cart_list')
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1
      nomal_products[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      nomal_products[0].qty.should == 1
      items = session[:cart].items.values
      items.size.should == 1
      items[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      items[0].qty.should == 1
    end

    it 'should redirect_to error_404' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1

      delete 'remove', {:id => nomal_products[0].key}
      response.should redirect_to(:controller => 'cms', :action => 'error_404', :lang => 'ja')
    end
  end

  describe 'PUT recalculate' do
    it 'should render_template normal_deli_form' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1

      request.stub!(:xml_http_request?).and_return(true)
      put 'recalculate', {:id => nomal_products[0].key, :qty => '2', :cource_div => '1'}

      response.should be_success
      response.should render_template('cart/_cart_list')
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1
      nomal_products[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      nomal_products[0].qty.should == 2
      items = session[:cart].items.values
      items.size.should == 1
      items[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      items[0].qty.should == 2
    end

    it 'should not recalculat' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1

      request.stub!(:xml_http_request?).and_return(true)
      put 'recalculate', {:cource_div => '1'}

      response.should be_success
      response.should render_template('cart/_cart_list')
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1
      nomal_products[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      nomal_products[0].qty.should == 1
      items = session[:cart].items.values
      items.size.should == 1
      items[0].product_sku.sku_code.should == '0000000000001-00-00-00'
      items[0].qty.should == 1
    end

    it 'should redirect_to error_404' do
      get 'index', {:sku_code => '0000000000001-00-00-00', :qty => '1'}
      response.should redirect_to(:action => 'index', :lang => 'ja')

      get 'index'
      response.should be_success
      nomal_products = session[:cart].item_list
      nomal_products.size.should == 1

      put 'recalculate', {:id => nomal_products[0].key, :qty => '2', :cource_div => '1'}
      response.should redirect_to(:controller => 'cms', :action => 'error_404', :lang => 'ja')
    end
  end

  describe "test load_cart_info" do
    fixtures :products, :product_skus, :product_prices, :user_ranks
    
    before do
      Product.update(100001, {:course_div_key => :normal})
      Product.update(100002, {:course_div_key => :normal})
      Product.update(100003, {:course_div_key => :normal})

      session[:cart] = Cart.new()
      session[:cart].add('0000000000001-00-00-00')
      session[:cart].add('0000000000002-00-00-00')
      session[:cart].add('0000000000003-00-00-00')
      session[:cart].user_rank = UserRank.find(1)
    end

    it "get index for test load_cart_info" do
      get 'index'
      response.should be_success
      response.should render_template('cart_index')
      session[:cart].item_list.size.should == 3
      session[:cart].items.size.should == 3
      session[:cart].normal_deli_total_price.should == 24000
      session[:cart].total_qty.should == 3
    end
  end
  
  describe "error" do
    it "cart_no_input" do
      get :cart_no_input
      response.should be_success
    end
  end
end

