require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe CommonController do
  fixtures :products

  describe "GET change_country" do
    it "change JP" do
      request.stub!(:xml_http_request?).and_return(true)
      get :change_country, :address_form => {
        :country_code => 'JP'
      }, :sub_address => {
      }
      response.should be_success
    end
    it "change EN" do
      request.stub!(:xml_http_request?).and_return(true)
      get :change_country, :address_form => {
      }, :sub_address => {
      }
      response.should be_success
    end
  end

  describe "zip" do
    before do
      mock_add1 = JpAddress.new(:id => 1, :jiscode => '23106', :zipcode_old => '460', :zipcode => '4600002',
        :prefecture => '愛知県', :city => '名古屋市中区', :address => '丸の内')
      JpAddress.stub!(:find_by_zipcode).with('4600002').and_return(mock_add1)
      JpAddress.stub!(:find_by_zipcode).with('0000000').and_return(nil)
    end

    it "POST to_address with id is '4600002'" do
      post :to_address, :id=>'4600002'
      response.should have_text("['愛知県','名古屋市中区','丸の内']")
    end

    it "POST to_address with id is '0000000'" do
      post :to_address, :id=>'0000000'
      response.should have_text("['','','']")
    end
  end

  describe 'update_views' do
    it 'productionモードの場合、キャッシュをクリアして成功する。' do
      with_constants :RAILS_ENV => 'production' do
        EcRider::ViewCacheCtrl.should_receive(:view_cache_clear)
        get :update_views
        response.should be_success
        response.body.should == 'OK!'
      end
    end
    it 'productionモードでない場合、キャッシュはクリアされない。' do
      EcRider::ViewCacheCtrl.should_not_receive(:view_cache_clear)
      get :update_views_for_passenger
      response.should be_success
      response.body.should == 'OK!'
    end
  end

  describe 'update_views_for_passenger' do
    it 'productionモードの場合、公開日付ファイルの削除と作成が行われる。' do
      with_constants :RAILS_ENV => 'production' do
        EcRider::ViewCacheCtrl.should_receive(:remove_and_create_published_date)
        get :update_views_for_passenger
        response.should be_success
        response.body.should == 'OK!'
      end
    end
    it 'productionモードでない場合、何も行われず成功する。' do
      EcRider::ViewCacheCtrl.should_not_receive(:remove_and_create_published_date)
      get :update_views_for_passenger
      response.should be_success
      response.body.should == 'OK!'
    end
  end
end