# -*- coding: utf-8 -*-
# To change this template, choose Tools | Templates
# and open the template in the editor.

require File.dirname(__FILE__) + '/../spec_helper'

class ExceptionNotification < ApplicationController
  def raise_unknown_action
    raise ActionController::UnknownAction
  end
  def raise_argument_error
    raise ArgumentError
  end
end

describe ExceptionNotification,  "In exception_notification plugin," do
  describe "ExceptionNotifier" do
    before(:each) do
      ActionMailer::Base.deliveries = []
      ExceptionNotifier.sender_address = %("Exception Notifier" <exception.notifier@default.com>)
      ExceptionNotifier.exception_recipients = ["test@ec-one.com"]
      ExceptionNotifier.exception_recipients = %w(test@ec-one.com)
      ExceptionNotifier.email_prefix = "[ERROR] "
      ExceptionNotifier.sections = %w(request session environment backtrace)
    end

    it "should default settigs is true" do
      ExceptionNotifier.sender_address.should == %("Exception Notifier" <exception.notifier@default.com>)
      ExceptionNotifier.exception_recipients.should == ["test@ec-one.com"]
      ExceptionNotifier.email_prefix.should == "[ERROR] "
      ExceptionNotifier.sections.should == %w(request session environment backtrace)
    end

    it "should send error mail for raise_argument_error." do
      get :raise_unknown_action
      ActionMailer::Base.deliveries.size.should == 0
      response.should_not be_success
      response.should render_template("#{RAILS_ROOT}/public/404.html")
    end

    describe "should send error mail for raise_argument_error." do
      before(:all) do
        @paths = %w(public/500.html public/cms/500.html).map{|path| File.join(RAILS_ROOT, path)}
        @backup_files = @paths.inject({}) do |dest, path|
          if File.exist?(path)
            bak_path = path + ".bak"
            dest[path] = bak_path
            FileUtils.cp(path, bak_path)
          end
          dest
        end
      end

      after(:all) do
        @backup_files.each do |path, bak_path|
          FileUtils.rm(path) if File.exist?(path)
          FileUtils.mv(bak_path, path)
        end
      end


      it "public/500.html と public/cms/500.html が両方存在する" do
        @paths.each do |path|
          unless File.exist?(path)
            FileUtils.mkdir_p(File.dirname(path))
            FileUtils.touch(path)
          end
        end
        get :raise_argument_error
        response.should_not be_success
        response.should render_template("#{RAILS_ROOT}/public/cms/500.html")
      end

      it "public/cms/500.html だけ両方存在する" do
        path = File.join(RAILS_ROOT, "public/cms/500.html")
        unless File.exist?(path)
          FileUtils.mkdir_p(File.dirname(path))
          FileUtils.touch(path)
        end
        path = File.join(RAILS_ROOT, "public/500.html")
        FileUtils.rm(path) if File.exist?(path)
        
        get :raise_argument_error
        response.should_not be_success
        response.should render_template("#{RAILS_ROOT}/public/cms/500.html")
      end

      it "public/500.html だけ両方存在する" do
        path = File.join(RAILS_ROOT, "public/500.html")
        unless File.exist?(path)
          FileUtils.mkdir_p(File.dirname(path))
          FileUtils.touch(path)
        end
        path = File.join(RAILS_ROOT, "public/cms/500.html")
        FileUtils.rm(path) if File.exist?(path)
        
        get :raise_argument_error
        response.should_not be_success
        response.should render_template("#{RAILS_ROOT}/public/500.html")
      end

    end
  end
end

