# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe InquiryController do
  include EcRider::PluginModule::RSpecExt
  
  fixtures :users, :user_addresses, :user_emails, :inquiries, :shop_settings,
    :products, :product_displays, :inquiry_products

  #Delete this example and add some real ones
  it "should use InquiryController" do
    controller.should be_an_instance_of(InquiryController)
  end

  describe "Get 'index'" do
    it "#210 attributes set." do
      pending_if_loaded_plugin(:mobile)
      session[:user] = users(:users_00003).id
      flash[:inquiry] = nil
      session[:inquiry] = nil
      get(:index)
      controller.instance_variable_get(:@inquiry).email.should == "email_address_01002@ec-one.com"
      controller.instance_variable_get(:@inquiry).name_last.should == "山田"
      response.should be_success
    end

    it "flash[:inquiry] is not null" do
      flash[:inquiry] = Inquiry.new({
          :inquiry => "INQ",
          :email => "m-user@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "東京都",
          :address_2 => "足立区",
          :address_3 => "神明４－１－６　マリンマンション１４０５号",
          :name_first => "結衣",
          :name_last => "空川",
          :kana_name_first => "ユイ",
          :kana_name_last => "AAA"
      })
      session[:inquiry] = nil
      get(:index)
      controller.instance_variable_get(:@inquiry).email.should == "m-user@ec-one.com"
      response.should be_success
    end

    it "flash[:inquiry] is not null" do
      session[:inquiry] = Inquiry.new({
          :inquiry => "INQ",
          :email => "m-user@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "東京都",
          :address_2 => "足立区",
          :address_3 => "神明４－１－６　マリンマンション１４０５号",
          :name_first => "結衣",
          :name_last => "空川",
          :kana_name_first => "ユイ",
          :kana_name_last => "ソラカワ"
      })
      flash[:inquiry] = nil
      get(:index)
      controller.instance_variable_get(:@inquiry).email.should == "m-user@ec-one.com"
      response.should be_success
    end
    
    it "#210 attributes set. inquiry type product" do
      pending_if_loaded_plugin(:mobile)
      session[:user] = users(:users_00003).id
      flash[:inquiry] = nil
      session[:inquiry] = nil
      get(:index, :product_code => '0000000009051')
      controller.instance_variable_get(:@inquiry_product).should_not == nil
      controller.instance_variable_get(:@inquiry_product).id.should == 1
      controller.instance_variable_get(:@inquiry).email.should == "email_address_01002@ec-one.com"
      controller.instance_variable_get(:@inquiry).name_last.should == "山田"
      response.should be_success
    end

  end

  describe "POST confirm'" do
    it "redirect no parameter inquiry." do
      post :confirm
      response.should redirect_to(:action => 'index', :lang => 'ja')
    end

    it "confirm with valid parameter" do
      session[:inquiry] = nil
      flash[:inquiry] = nil
      post :confirm, :inquiry => {
          :inquiry => "INQ",
          :email => "m-user@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "東京都",
          :address_2 => "足立区",
          :address_3 => "神明４－１－６　マリンマンション１４０５号",
          :name_first => "結衣",
          :name_last => "空川",
          :kana_name_first => "ユイ",
          :kana_name_last => "ソラカワ"
      }, :address_form => {}
      session[:inquiry].should_not == nil
      response.should be_success
    end

    it "confirm with invalid parameter" do
      session[:inquiry] = nil
      flash[:inquiry] = nil
      post :confirm, :inquiry => { :inquiry => "INQ"}
      flash[:inquiry].should_not == nil
      response.should redirect_to(:action => 'index', :lang => 'ja')
    end

    it "confirm with valid parameter for product" do
      flash[:inquiry] = nil
      session[:inquiry] = Inquiry.new({
          :inquiry => "INQ",
          :country_code => 'JP',
          :email => "m-user@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "ADDR",
          :address_2 => "ADDRESS2",
          :address_3 => "ADDRESS3",
          :name_first => "TRER",
          :name_last => "HGFD",
          :kana_name_first => "YTREW",
          :kana_name_last => "ABCDE"
      })
      pro = Product.find(1)
      post :confirm, :inquiry => { :inquiry => "INQ"} , :inquiry_product => pro

      flash[:inquiry].should_not == nil
      response.should redirect_to(:action => 'index', :lang => 'ja')
    end
  end

  describe "POST completion'" do
    it "completion" do
      post :completion
      response.should be_success
    end
  end

  describe "POST update_information'" do

    it "update_information with valid session" do
      session[:inquiry] = Inquiry.new({
          :inquiry => "INQ",
          :country_code => 'JP',
          :email => "m-user@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "ADDR",
          :address_2 => "ADDRESS2",
          :address_3 => "ADDRESS3",
          :name_first => "TRER",
          :name_last => "HGFD",
          :kana_name_first => "YTREW",
          :kana_name_last => "ABCDE",
          :image_url_1 => "test.jpg"
      })
      post :update_information
      response.should be_success
      Inquiry.find(:first, :conditions => ['email = ?','m-user@ec-one.com']).test_flg.should be_nil
    end

    it "update_information with valid session for product inquiry" do
      session[:inquiry] = Inquiry.new({
          :inquiry => "INQ",
          :inquiry_div => '6',
          :country_code => 'JP',
          :email => "m-user@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "ADDR",
          :address_2 => "ADDRESS2",
          :address_3 => "ADDRESS3",
          :name_first => "TRER",
          :name_last => "HGFD",
          :kana_name_first => "YTREW",
          :kana_name_last => "ABCDE"
      })

      session[:inquiry_product] = Product.find(1)
      post(:update_information)
      response.should be_success

      ip = Inquiry.find(:first, :conditions => ['email = ?','m-user@ec-one.com'])
      ip.inquiry_products.size.should == 1
      ip.inquiry_products[0].product_id.should == 1
      ip.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      session[:inquiry] = Inquiry.new({
          :inquiry => "INQ",
          :country_code => 'JP',
          :email => "testuser@ec-one.com",
          :tel_num => "000-111-2224",
          :address_1 => "ADDR",
          :address_2 => "ADDRESS2",
          :address_3 => "ADDRESS3",
          :name_first => "TRER",
          :name_last => "HGFD",
          :kana_name_first => "YTREW",
          :kana_name_last => "ABCDE"
      })
      post :update_information
      response.should be_success
      Inquiry.find(:first, :conditions => ['email = ?','testuser@ec-one.com']).test_flg.should be_true
    end
  end
end
