# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe OrderController do

  fixtures :all

  before(:each) do
    setting = ShopSetting.find(:first, :conditions => ["setting_div = ?", ShopSetting.setting_div_id_by_key(:security_code)])
    setting.value = "false"
    setting.save!
    ActiveRecord::Base.logger.debug("*" * 50)
    ActiveRecord::Base.logger.debug(self.description.to_s)
    ActiveRecord::Base.logger.debug("*" * 50)
    EcRider.config.stub!(:payment_system_div).and_return("1")
  end

  def login_as_user
    user = User.find(1004, :include => :user_addresses)
    user.user_email = UserEmail.new(:email => 'email_address_01008@test.com')
    @request.session[:cart_tmp_user] = user
    @request.session[:user] = user.id
    @request.session[:shipping_addresses] = user.user_addresses
  end

  def login_as_user_with_sub_address
    user = User.find(1015)
    user.user_email = UserEmail.new(:email => 'email_address_x1015@test.com')
    @request.session[:cart_tmp_user] = user
    @request.session[:user] = user.id
  end

  def tmp_user
    org_user = User.find(1004)
    user = User.new(org_user.attributes)
    user.password = nil
    user.member_div_key = :no_membership
    user.user_email = UserEmail.new(:email => 'test1@test.com')
    user.user_address = UserAddress.new(org_user.user_address.attributes)
    @request.session[:cart_tmp_user] = user
    @request.session[:user] = nil
    @request.session[:shipping_addresses] = []
  end

  def new_user
    org_user = User.find(1004)
    user = User.new(org_user.attributes)
    user.password = "pass0"
    user.password_confirmation = "pass0"
    user.member_div_key = :membership
    user.user_email = UserEmail.new(:email => 'test2@test.com')
    user.user_address = UserAddress.new(org_user.user_address.attributes)
    @request.session[:cart_tmp_user] = user
    @request.session[:user] = nil
    @request.session[:shipping_addresses] = []
  end

  def new_order
    org_order = Order.find(4)
    order = Order.new(org_order.attributes)
    org_order.order_shippings.each do |org_order_shipping|
      new_order_shipping = OrderShipping.new(org_order_shipping.attributes)
      org_order_shipping.order_items.each do |org_order_item|
        new_order_shipping.order_items << OrderItem.new(org_order_item.attributes)
      end
      new_order_shipping.status_div_key = :before_settlement
      order.order_shippings << new_order_shipping
    end
    order
  end

  def set_cart_to_session
    cart = Cart.new
    p1 = ProductSku.find(265)
    cart.add(p1, p1.product.course_div_key, 1)
    session[:cart] = cart
  end

  describe 'GET agreement' do
    it "should be success" do
      login_as_user
      set_cart_to_session
      get 'agreement'
      response.should be_success
    end
  end
  
  describe 'GET confirmation_cart' do
    it "redirect to index" do
      login_as_user
      set_cart_to_session
      get 'confirmation_cart'
      response.should redirect_to(:controller => 'order', :action => 'index', :lang => 'ja')
    end
  
    it "flash[:error]" do
      set_cart_to_session
      flash[:user_profile] = User.new
      flash[:user_address] = UserAddress.new
      flash[:user_email] = UserEmail.new
      get 'confirmation_cart'
      controller.instance_variable_get(:@user_profile).should_not be_nil
      controller.instance_variable_get(:@user_address).should_not be_nil
      controller.instance_variable_get(:@user_email).should_not be_nil
    end
  
    it "session[:cart_tmp_user]" do
      set_cart_to_session
      tmp_user = User.new
      tmp_user.user_address = UserAddress.new
      tmp_user.user_email = UserEmail.new
      session[:cart_tmp_user] = tmp_user
      get 'confirmation_cart'
      controller.instance_variable_get(:@user_profile).should_not be_nil
    end
  
    it "session[:cart_tmp_user]" do
      set_cart_to_session
      get 'confirmation_cart'
      controller.instance_variable_get(:@user_profile).sex_key.should == :male
      controller.instance_variable_get(:@user_address).country_code == 'JP'
      controller.instance_variable_get(:@user_email).should_not be_nil
    end
  end
  
  describe 'GET index' do
    it "should be success" do
      login_as_user
      set_cart_to_session
      get 'index'
      response.should redirect_to(:controller => 'order', :action => 'payment', :lang => 'ja')
      session[:order_form].should be_nil
    end
  end
  
  describe "POST create" do
    it "[正常系]会員　銀行振り込み" do
      login_as_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      order = Order.find(:first, :order => 'id desc')
      
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid
      
      # 在庫は仮引当
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 1
      order_item.product_sku_secure_inventories.first.paid_flg.should be_false
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 22
      ProductSkuSecureInventory.count.should == before_secure_count + 1
      
      # ポイントは付与する
      order.user_points.length.should == 1
      order.user_points.first.given_point.should == 100

      # 顧客区分は会員のまま
      order.user.member_div_key.should == :membership
      session[:user].should_not be_nil
      
      # メールは1通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should render_template('thanks/index')
    end

    it "[正常系]会員　銀行振り込み（キャンペーンあり）" do
      login_as_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)
      @request.session[:cart_tmp_order].order_campaigns = [OrderCampaign.new(:campaign_id => 1, :campaign_code => '123', :campaign_name => '123')]

      lambda{
        post :create
      }.should change(OrderCampaign, :count).by(1)

      response.should render_template('thanks/index')
    end
    
    it "[正常系]非会員　会員登録なし　銀行振り込み　※ec_front" do
      tmp_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      # 本番注文
      order = Order.find(:first, :order => 'id desc')
      order.test?.should be_false
      
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid

      # 在庫は仮引当
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 1
      order_item.product_sku_secure_inventories.first.paid_flg.should be_false
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 22
      ProductSkuSecureInventory.count.should == before_secure_count + 1
      
      # ポイントは付与しない
      order.user_points.length.should == 0

      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      session[:user].should be_nil
      
      # メールは1通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should render_template('thanks/index')
    end

    it "[正常系]非会員　会員登録なし　銀行振り込み　※ec_test" do
      EcRider.config.stub!(:ec_test).and_return(true)
      tmp_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      # テスト注文
      order = Order.find(:first, :order => 'id desc')
      order.test?.should be_true
      
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid
      
      # 在庫は引当しない
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      session[:user].should be_nil
      
      # メールは1通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count

      response.should render_template('thanks/index')
    end
    
    it "[正常系]非会員　会員登録あり　銀行振り込み　※ec_front" do
      new_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      # 本番注文
      order = Order.find(:first, :order => 'id desc')
      order.test?.should be_false
      
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid
      
      # 在庫は仮引当
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 1
      order_item.product_sku_secure_inventories.first.paid_flg.should be_false
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 22
      ProductSkuSecureInventory.count.should == before_secure_count + 1
      
      # ポイントは付与する
      order.user_points.length.should == 1
      order.user_points.first.given_point.should == 100
      
      # 顧客区分は会員
      order.user.member_div_key.should == :membership
      session[:user].should_not be_nil
      
      # メールは2通
      session[:user].should_not be_nil
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count + 1
      
      response.should render_template('thanks/index')
    end

    it "[正常系]非会員　会員登録あり　銀行振り込み　※ec_test" do
      EcRider.config.stub!(:ec_test).and_return(true)
      new_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count

      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)
      
      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      # テスト注文
      order = Order.find(:first, :order => 'id desc')
      order.test?.should be_true
      
      # ステータスは入金待ち
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :undelivery_unpaid
      
      # 在庫は仮引当しない
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与する
      order.user_points.length.should == 1
      order.user_points.first.given_point.should == 100
      
      # 顧客区分は会員
      order.user.member_div_key.should == :membership
      session[:user].should_not be_nil

      # メールは2通
      session[:user].should_not be_nil
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count + 1
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count + 1
      
      response.should render_template('thanks/index')
    end


    it "[異常系]会員　セッションが空　redirect to re_order_error" do
      login_as_user
      @request.session[:cart_tmp_order] = nil

      lambda{
        post :create
      }.should change(Order, :count).by(0)
      
      response.should redirect_to(:controller => 'order', :action => 're_order_error', :lang => 'ja')
      session[:user].should_not be_nil
    end
    
    it "[異常系]在庫不足エラー　会員" do
      login_as_user
      order = new_order
      order.order_shippings.first.order_items.first.qty = 100
      @request.session[:cart_tmp_order] = order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      order = Order.find(:first, :order => 'id desc')
      
      # ステータスは在庫エラー
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :stock_error
      # 注文商品は在庫不足で登録
      order_item = order_shipping.order_items.first
      order_item.out_of_stock_flg.should be_true
      
      # 在庫は引当しない
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は会員のまま
      order.user.member_div_key.should == :membership
      session[:user].should_not be_nil
      
      # メールは0通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should redirect_to(:controller => 'cart', :action => 'index', :lang => 'ja')
    end
    
    it "[異常系]プレゼント品在庫不足エラー　会員" do
      login_as_user
      order = new_order
      order.order_shippings.first.order_items.first.qty = 100
      order.order_shippings.first.order_items.first.course_div_key = :present
      @request.session[:cart_tmp_order] = order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      order = Order.find(:first, :order => 'id desc')
      
      # ステータスは在庫エラー
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :stock_error
      # 注文商品は在庫不足で登録
      order_item = order_shipping.order_items.first
      order_item.out_of_stock_flg.should be_true
      
      # 在庫は引当しない
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は会員のまま
      order.user.member_div_key.should == :membership
      session[:user].should_not be_nil
      
      # メールは0通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should redirect_to(:controller => 'order', :action => 'confirmation_order', :lang => 'ja')
    end

    it "[異常系]在庫不足エラー　非会員　会員登録なし" do
      tmp_user
      order = new_order
      order.order_shippings.first.order_items.first.qty = 100
      @request.session[:cart_tmp_order] = order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      order = Order.find(:first, :order => 'id desc')
      
      # ステータスは在庫エラー
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :stock_error
      # 注文商品は在庫不足で登録
      order_item = order_shipping.order_items.first
      order_item.out_of_stock_flg.should be_true
      
      # 在庫は引当しない
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      session[:user].should be_nil
      
      # メールは0通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count

      response.should redirect_to(:controller => 'cart', :action => 'index', :lang => 'ja')
    end
    
    it "[異常系]在庫不足エラー　非会員　会員登録あり(在庫エラーのため非会員で登録)" do
      new_user
      order = new_order
      order.order_shippings.first.order_items.first.qty = 100
      @request.session[:cart_tmp_order] = order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)
      
      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      order = Order.find(:first, :order => 'id desc')
      
      # ステータスは在庫エラー
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :stock_error
      # 注文商品は在庫不足で登録
      order_item = order_shipping.order_items.first
      order_item.out_of_stock_flg.should be_true
      
      # 在庫は引当しない
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      session[:user].should be_nil

      # メールは0通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should redirect_to(:controller => 'cart', :action => 'index', :lang => 'ja')
    end
    
    it "[異常系]内部エラー　非会員　会員登録あり(内部エラーのため非会員で登録)" do
      new_user
      @request.session[:cart_tmp_order] = new_order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      
      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)
      
      FrontSiteMailer.stub!(:send_thanks_mail).and_raise(StandardError.new("メール送信時に内部エラーで失敗しました"))
      
      lambda{
        post :create
      }.should change(Order, :count).by(1)

      order = Order.find(:first, :order => 'id desc')
      
      # ステータスは内部エラー
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :internal_error
      
      # 在庫は引当しない
      order_item = order_shipping.order_items.first
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      session[:user].should be_nil

      session[:cart_tmp_user].should be_nil
      session[:cart_tmp_order].should be_nil
      session[:cart_send_info].should be_nil
      session[:cart].should be_nil

      # メールは0通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should redirect_to(:controller=>'order', :action => 'register_order_error', :lang => 'ja')
    end

    it "[異常系]非会員　会員登録なし　販売期間外　※ec_front" do
      tmp_user
      time_now = Time.parse("2009/02/01 00:00:00")
      Time.stub!(:now).and_return(time_now)
      order = new_order
      order.order_shippings.first.order_items.first.qty = 100
      @request.session[:cart_tmp_order] = order
      @request.session[:cart_send_info] = nil
      @request.session[:cart] = Cart.new
      @request.session[:cart].stub!(:total_given_point).and_return(100)

      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      before_secure_count = ProductSkuSecureInventory.count
      product = product_sku.product
      product.sale_started_at = Time.parse('2010-12-10 11:00:00')
      product.save!

      thanks_mail_condition = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:thanks)]
      before_thanks_mail_count = MailDelivery.count(:conditions => thanks_mail_condition)
      regist_mail_conditions = ["mail_div = ?", MailTemplate.mail_type_div_id_by_key(:regist_user)]
      before_regist_mail_count = MailDelivery.count(:conditions => regist_mail_conditions)

      lambda{
        post :create
      }.should change(Order, :count).by(1)
      
      # ステータスは販売期間エラー
      order_shipping = order.order_shippings.first
      order_shipping.status_div_key.should == :sales_period_error
      # 注文商品は販売期間エラーで登録
      order_item = order_shipping.order_items.first
      order_item.out_of_stock_flg.should be_false
      order_item.nonperiodic_sale_flg.should be_true
      
      # 在庫は引当しない
      order_item.product_sku_secure_inventories.length.should == 0
      order_item.product_sku_id.should == 266
      product_sku = ProductSku.find(266)
      product_sku.stock_count.should == 50
      product_sku.orderable_count == 23
      ProductSkuSecureInventory.count.should == before_secure_count
      
      # ポイントは付与しない
      order.user_points.length.should == 0
      
      # 顧客区分は非会員
      order.user.member_div_key.should == :no_membership
      session[:user].should be_nil

      # メールは0通
      MailDelivery.count(:conditions => thanks_mail_condition).should == before_thanks_mail_count
      MailDelivery.count(:conditions => regist_mail_conditions).should == before_regist_mail_count
      
      response.should redirect_to(:controller => 'cart', :action => 'index', :lang => 'ja')
    end
  end
  
  describe "cart_in? unit test." do
    it "cart_in? in index 1: session is null" do
      session[:cart] = nil
      get :index
      response.should redirect_to(:controller=>'cart', :action => 'cart_no_input', :lang => 'ja')
    end

    it "cart_in? in index 2: session.products is null" do
      session[:cart] = Cart.new
      session[:cart].clear
      get :index
      response.should redirect_to(:controller=>'cart', :action => 'cart_no_input', :lang => 'ja')
    end
  end
  
  describe "POST payment" do
    before do
      @user_profile = {
        :password => "pass1word",
        :password_confirmation => "pass1word",
        :remember_question => "A",
        :remember_answer => "A",
        :birth_year => "2000",
        :birth_month => "3",
        :birth_day => "11",
        :sex => "1"
      }
      @user_address = {
        :country_code => "JP",
        :zip_code => "1210051",
        :tel_num => "000-111-2224",
        :address_1 => "東京都",
        :address_2 => "足立区",
        :address_3 => "神明４－１－６　マリンマンション１４０５号",
        :name_first => "結衣",
        :name_last => "空川",
        :kana_name_first => "ユイ",
        :kana_name_last => "ソラカワ"
      }
      @user_en_address = {
        :address_1 => "ADDR",
        :address_2 => "ADDRESS2",
        :address_3 => "ADDRESS3",
        :name_first => "TRER",
        :name_last => "HGFD",
        :kana_name_first => "YTREW",
        :kana_name_last => "ABCDE"
      }
    end

    it "no user_form" do
      set_cart_to_session
      login_as_user
      post :payment
      response.should be_success
    end

    it "with user_form to no error." do
      set_cart_to_session
      login_as_user
      post :payment, :user_profile => @user_profile,
        :user_address => @user_address,
        :user_en_address => @user_en_address,
        :user_email => {
        :email => 'email_address_0100q@test.com'
      }
      response.should be_success
      response.should_not redirect_to( :action => :confirmation_cart, :lang => 'ja')
    end

    it "メールアドレス重複" do
      set_cart_to_session
      login_as_user
      post :payment, :user_profile => @user_profile,
        :user_address => @user_address,
        :user_en_address => @user_en_address,
        :user_email => {
        :email => 'member23@ecrider.com'
      }
      response.should redirect_to( :action => :confirmation_cart, :lang => 'ja')
    end

    it "メールアドレス未入力" do
      set_cart_to_session
      login_as_user
      post :payment, :user_profile => @user_profile,
        :user_address => @user_address,
        :user_en_address => @user_en_address,
        :user_email => {
        :email => ''
      }
      response.should redirect_to( :action => :confirmation_cart, :lang => 'ja')
    end

    it "with user_form error." do
      set_cart_to_session
      login_as_user
      post :payment, :user_profile => {
        :password => "pass1word",
        :password_confirmation => "pass1word",
        :remember_question => "A",
        :remember_answer => "A",
        :birth_year => "2000",
        :birth_month => "3",
        :birth_day => "11"
      }
      response.should redirect_to( :action => :confirmation_cart, :lang => 'ja')
    end

    it "no login." do
      set_cart_to_session
      login_as_user
      session[:user] = nil
      post :payment
      response.should be_success
    end

    it "from redirect(validation error)" do
      set_cart_to_session
      login_as_user
      flash[:cart_send_info] = OrderForm.new(:send_address_index => 0)
      post :payment
      response.should be_success
    end

    it ":order_form" do
      set_cart_to_session
      login_as_user
      session[:order_form] = true
      post :payment
      controller.instance_variable_get(:@order_form).should == true
      response.should be_success
    end

  end
  
  describe "POST confirmation_order" do

    it "プレゼント品の在庫切れで戻った場合" do
      set_cart_to_session
      login_as_user
      flash[:stock_error] = true;
      session[:order_form] = OrderForm.new(:send_address_index => "0", :normal_payment_type => "0")
      post :confirmation_order
      controller.instance_variable_get(:@order).should_not be_nil
      controller.instance_variable_get(:@user).should_not be_nil
      response.should be_success
    end

    it "no parameter :order_form" do
      set_cart_to_session
      login_as_user
      post :confirmation_order
      response.should redirect_to(:controller => 'order', :action => 'not_user_info_error', :lang => 'ja')
    end

    it "input valid parameter order_form" do
      pending
      set_cart_to_session
      login_as_user
      post :confirmation_order, :order_form => {
        :send_address_index => "0",
        :card_kind => :visa,
        :no_1 => "4111",
        :no_2 => "1111",
        :no_3 => "1111",
        :no_4 => "1111",
        :expire_year => "10",
        :expire_month => "10",
        :card_name => "Yuuichi Miura",
        :normal_payment_type => "1",
      }
      response.should be_success
    end

    it "input parameter order_form with error" do
      set_cart_to_session
      login_as_user
      post :confirmation_order, :order_form => {
        :send_address_index => "0",
        :normal_payment_type => "2",
        :no_1 => "11657511"
      }
      response.should redirect_to(:action => :payment, :lang => 'ja')
    end
  end
  
  describe 'GET new_shipping_address' do
    it "should be success" do
      login_as_user
      set_cart_to_session
      get 'new_shipping_address'
      response.should be_success
    end
  end
  
  describe 'GET create_shipping_address' do
    it "should be success" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new, UserAddress.new, UserAddress.new]
      get :create_shipping_address, :user_address => UserAddress.find(1011).attributes.dup
      response.should be_success
    end

    it "valid error" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new, UserAddress.new, UserAddress.new]
      user_address = UserAddress.find(1011).attributes.dup
      user_address["name_first"] = ""

      get :create_shipping_address, :user_address => user_address
      shipping_address = controller.instance_variable_get(:@shipping_address)

      (shipping_address.errors.count).should == 2
      response.should be_success
    end
  end
  
  describe 'GET edit_shipping_address' do
    it "should be success" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new, UserAddress.new, UserAddress.new]
      get :edit_shipping_address, :order_form => {
        :id => "0"
      }
      response.should be_success
    end
  end
  
  describe 'GET update_shipping_address' do
    
    it "should be success" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new, UserAddress.new, UserAddress.new]
      get(:update_shipping_address,
        :edit_target_index => "0",
        :user_address => UserAddress.find(1011).attributes.dup)

      response.should be_success
    end

    it "valid error" do
      login_as_user
      set_cart_to_session
      session[:shipping_addresses] = [UserAddress.new, UserAddress.new, UserAddress.new]
      user_address = UserAddress.find(1011).attributes.dup
      user_address["name_first"] = ""

      get(:update_shipping_address,
        :edit_target_index => "0",
        :user_address => user_address)
      shipping_address = controller.instance_variable_get(:@shipping_address)

      (shipping_address.errors.count).should == 2
      response.should be_success
    end
  end
  
  describe "error" do
    it "not_user_info_error" do
      get :not_user_info_error
      response.should be_success
    end

    it "register_order_error" do
      get :register_order_error
      response.should be_success
    end

    it "re_order_error" do
      get :re_order_error
      response.should be_success
    end
  end
 
  describe "サンクスメールテンプレート振り分けテスト" do
    it "銀行振り込み注文以外の場合" do
      pending "標準機能としては未実装"
    end

    it "銀行振り込み注文の場合" do
      login_as_user
      new_order = new_order()
      new_order.order_shippings.each do |os|
        os.payment_type_div_key = :bank_transfer
      end
      session[:cart_tmp_order] = new_order
      session[:cart_send_info] = nil
      session[:cart] = Cart.new
      session[:shipping_addresses] = []
      post :create
      order = Order.find(:first, :order => 'id desc')
      response.should render_template('thanks/index')
      #サンクスメール（銀行振り込み以外）
      ActionMailer::Base.deliveries.size.should == 1
      ActionMailer::Base.deliveries[0].subject.should == 'thanks pc! bank_transfer'
    end
  end

end
