# -*- coding: utf-8 -*-
# To change this template, choose Tools | Templates
# and open the template in the editor.

require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ResignController do
  fixtures :resigned_reasons, :users, :user_emails, :user_addresses, :shop_settings,
    :mail_deliveries

  before do
    user = User.find(1007)
    user.user_email = UserEmail.new(:email => 'email_address_01008@test.com')
    session[:cart_tmp_user] = user
    session[:user] = user.id
  end

  describe "Get 'index'" do
    before do
      get :index
    end
    it "[no parameter] index session[:resigned_reason] is nil" do
      session[:resigned_reason].should == nil
    end
    it "[no parameter] index redirect to resigned_reason" do
      assert_redirected_to({:action => :enter_reason})
    end
  end

  describe "Get 'enter_reason'" do
    before do
      flash[:resigned_reason] = nil
      session[:resigned_reason] = nil
    end
    it "[no parameter] resigned_reason session[:resigned_reason] is nil" do
      get :enter_reason
      session[:resigned_reason].should == nil
    end
    it "[no parameter] resigned_reason flash[:resigned_reason] is nil" do
      get :enter_reason
      flash[:resigned_reason].should == nil
    end
    it "[no parameter] resigned_reason ms_resigned_reason_id is '1'" do
      get :enter_reason
      assigns[:resigned_reason].ms_resigned_reason_id.should == 1
    end
    it "[no parameter] resigned_reason flash[:resigned_reason] is not nil" do
      flash[:resigned_reason] = "flash"
      get :enter_reason
      assigns[:resigned_reason].should == "flash"
    end
    it "[no parameter] resigned_reason session[:resigned_reason] is not nil" do
      session[:resigned_reason] = "session"
      get :enter_reason
      assigns[:resigned_reason].should == "session"
    end
    after do
      flash[:resigned_reason] = nil
      session[:resigned_reason] = nil
    end
  end

  describe "Post 'confirm'" do
    before do
      @session_resigned_reason = ResignedReason.find 1
    end
    it "[no parameter] confirm redirect to cms/index" do
      post :confirm
      assert_redirected_to(:action => 'enter_reason')
    end
    it "[validation error] confirm flash[:resigned_reason] is not nil" do
      resigned_reason = ResignedReason.find 2
      session[:resigned_reason] = @session_resigned_reason
      post :confirm , :resigned_reason => {:ms_resigned_reason_id =>resigned_reason.ms_resigned_reason_id, :reason => resigned_reason.reason}
      flash[:resigned_reason].should_not be_nil
    end
    it "[validation ok] confirm ms_resigned_reason_id, reason check" do
      resigned_reason = ResignedReason.find 3
      session[:resigned_reason] = @session_resigned_reason
      post :confirm , :resigned_reason => {:ms_resigned_reason_id =>resigned_reason.ms_resigned_reason_id, :reason => resigned_reason.reason}
      # 退会理由、自由記入の一致チェック
      session[:resigned_reason].ms_resigned_reason_id.should == resigned_reason.ms_resigned_reason_id
      session[:resigned_reason].reason.should == resigned_reason.reason
      assigns[:resigned_reason].ms_resigned_reason_id.should == resigned_reason.ms_resigned_reason_id
      assigns[:resigned_reason].reason.should == resigned_reason.reason
    end
  end

  describe "Post 'update_information'" do
    before do
      MailDelivery.delete_all
      @session_resigned_reason = ResignedReason.find(1)
      @session_user = User.find(1002)
    end
    it "[no parameter] update_information redirect to cms/index" do
      post :update_information
      assert_redirected_to(:action => 'enter_reason')
    end
    it "[update] update_information session, user clear" do
      session[:resigned_reason] = @session_resigned_reason
      session[:user] = @session_user
      post :update_information , :resigned_reason => {:ms_resigned_reason_id =>@session_resigned_reason.ms_resigned_reason_id, :reason => @session_resigned_reason.reason}
      session[:resigned_reason].should == nil
      session[:user].should == nil
      mail_result = MailDelivery.find(:all)
      mail_result.size.should == 1
    end
    it "[update] 会員がPCサイト用メールアドレスのみ登録している場合PCサイト用のメールアドレスにメールが送信されること" do
      @session_user.user_emails = []
      @session_user.user_emails << UserEmail.new(:email => "email_address_005@ec-rider.com", :pc_mobile_div => "1", :mail_address_pc_mobile_div => "1")
      session[:resigned_reason] = @session_resigned_reason
      session[:user] = @session_user
      post :update_information , :resigned_reason => {:ms_resigned_reason_id =>@session_resigned_reason.ms_resigned_reason_id, :reason => @session_resigned_reason.reason}
      session[:resigned_reason].should == nil
      session[:user].should == nil
      mail_result = MailDelivery.find(:all)
      mail_result.size.should == 1
      mail_result[0].subject.should == "UserInfo resign pc!"
      mail_result[0].to_address.should == "email_address_005@ec-rider.com"
    end
    it "[update] 会員がモバイルサイト用メールアドレスのみ登録している場合モバイルサイト用のメールアドレスにメールが送信されること" do
      @session_user.user_emails = []
      @session_user.user_emails << UserEmail.new(:email => "email_address_005@docomo.ne.jp", :pc_mobile_div => "2", :mail_address_pc_mobile_div => "2")
      session[:resigned_reason] = @session_resigned_reason
      session[:user] = @session_user
      post :update_information , :resigned_reason => {:ms_resigned_reason_id =>@session_resigned_reason.ms_resigned_reason_id, :reason => @session_resigned_reason.reason}
      session[:resigned_reason].should == nil
      session[:user].should == nil
      mail_result = MailDelivery.find(:all)
      mail_result.size.should == 1
      mail_result[0].subject.should == "UserInfo resign mobile!"
      mail_result[0].to_address.should == "email_address_005@docomo.ne.jp"
    end
    it "[update] 会員がPCサイト用メールアドレス かつ モバイルサイト用メールアドレスを登録している場合PCサイト用、モバイルサイト用の二つのメールアドレスにメールが送信されること" do
      @session_user.user_emails = []
      @session_user.user_emails << UserEmail.new(:email => "email_address_005@ec-rider.com", :pc_mobile_div => "1", :mail_address_pc_mobile_div => "1")
      @session_user.user_emails << UserEmail.new(:email => "email_address_005@docomo.ne.jp", :pc_mobile_div => "2", :mail_address_pc_mobile_div => "2")
      session[:resigned_reason] = @session_resigned_reason
      session[:user] = @session_user
      post :update_information , :resigned_reason => {:ms_resigned_reason_id =>@session_resigned_reason.ms_resigned_reason_id, :reason => @session_resigned_reason.reason}
      session[:resigned_reason].should == nil
      session[:user].should == nil
      mail_result = MailDelivery.find(:all)
      mail_result.size.should == 2
      mail_result[0].subject.should == "UserInfo resign pc!"
      mail_result[0].to_address.should == "email_address_005@ec-rider.com"
      mail_result[1].subject.should == "UserInfo resign mobile!"
      mail_result[1].to_address.should == "email_address_005@docomo.ne.jp"
    end
  end

  describe "Post 'completion'" do
    it "[no parameter] completion ok" do
      post :completion
      response.should be_success
    end
  end
end
