# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :products, :product_displays, :inquiry_products, 
    :users, :user_emails,
    :kbn_codes, :kbn_code_names, :shop_settings

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:inquiry_form_country_ja" do
    it "normal usage" do
      I18n.locale = 'ja'
      inquiry = Inquiry.new
      inquiry.inquiry_div = '6'
      @template.instance_variable_set(:@inquiry_product, Product.find(1))
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.extend(ApplicationHelper)
      ec.inquiry_form do
        concat(ec.inquiry_form_error_messages || '')
        concat(ec.inquiry_form_email)
        concat(ec.inquiry_form_name_last)
        concat(ec.inquiry_form_name_first)
        concat(ec.inquiry_form_tel_num)
        concat(ec.inquiry_form_inquiry)
        concat(ec.inquiry_form_inquiry_type_select)
        concat(ec.inquiry_form_image(:display_number => 1))
        concat(ec.inquiry_form_image(:display_number => 2))
        concat(ec.inquiry_form_image(:display_number => 3))
        concat(ec.inquiry_form_image(:display_number => 4))
        concat(ec.inquiry_form_image(:display_number => 5))
        ec.button{ "←前へ戻る"}
        ec.submit{"確認画面へ進む→"}
        @template.output_buffer
      end

      @template.output_buffer.should have_tag('form[action=?]', "/inquiry/confirm") do
        with_tag("input#inquiry_email")
        with_tag("input#inquiry_name_last")
        with_tag("input#inquiry_name_first")
        with_tag("input#inquiry_tel_num")
        with_tag("textarea#inquiry_inquiry")
        with_tag("select#inquiry_inquiry_div")
        with_tag("input#inquiry_image_url_1")
        with_tag("input#inquiry_image_url_2")
        with_tag("input#inquiry_image_url_3")
        with_tag("input#inquiry_image_url_4")
        with_tag("input#inquiry_image_url_5")
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '確認画面へ進む→')
      end
    end
  end

  describe "ec:inquiry_form_country_en" do
    it "normal usage" do
      I18n.locale = 'en'
      @template.instance_variable_set(:@inquiry, Inquiry.new)
      @template.extend(ApplicationHelper)
      ec.inquiry_form do
        concat(ec.inquiry_form_error_messages || '')
        concat(ec.inquiry_form_email)
        concat(ec.inquiry_form_name_last)
        concat(ec.inquiry_form_name_first)
        concat(ec.inquiry_form_tel_num)
        concat(ec.inquiry_form_inquiry)
        concat(ec.inquiry_form_image(:display_number => 1))
        concat(ec.inquiry_form_image(:display_number => 2))
        concat(ec.inquiry_form_image(:display_number => 3))
        concat(ec.inquiry_form_image(:display_number => 4))
        concat(ec.inquiry_form_image(:display_number => 5))
        ec.button{ "←前へ戻る"}
        ec.submit{"確認画面へ進む→"}
        @template.output_buffer
      end

      @template.output_buffer.should have_tag('form[action=?]', "/inquiry/confirm") do
        with_tag("input#inquiry_email")
        with_tag("input#inquiry_name_last")
        with_tag("input#inquiry_name_first")
        without_tag("input#inquiry_kana_name_last")
        without_tag("input#inquiry_kana_name_first")
        with_tag("input#inquiry_tel_num")
        with_tag("textarea#inquiry_inquiry")
        with_tag("input#inquiry_image_url_1")
        with_tag("input#inquiry_image_url_2")
        with_tag("input#inquiry_image_url_3")
        with_tag("input#inquiry_image_url_4")
        with_tag("input#inquiry_image_url_5")
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '確認画面へ進む→')
      end
    end
  end

  describe "ec:inquiry_confirmation_form_country_ja" do
    it "normal usage" do
      I18n.locale = 'ja'
      inquiry = Inquiry.new
      inquiry.inquiry_div_key = :other
      inquiry.email = 'm-user@ec-one.com'
      inquiry.name_last = 'テスト姓'
      inquiry.name_first = 'テスト名'
      inquiry.kana_name_last = 'テストセイ'
      inquiry.kana_name_first = 'テストメイ'
      inquiry.tel_num = '000-0000-0000'
      inquiry.inquiry = 'お問い合わせ'
      inquiry.country_code = 'JP'
      inquiry.locale = 'ja'
      inquiry.image_url_1 = 'Winter.jpg'
      tmp_image_dir = 'test091112001111'
      fixture_file_path = "#{EcRider.config.ec_admin_root}/test/fixtures/inquiry_images/Winter.jpg"
      upload_file_path = "#{EcRider.config.ec_front_root}/public/tmp_inquiry_images/#{tmp_image_dir}/"
      FileUtils.rm_r(upload_file_path) if File.exist?(upload_file_path)
      FileUtils.mkdir_p(upload_file_path)
      FileUtils.cp_r(fixture_file_path, upload_file_path)
      tmp_images = {:tmp_image_dir => tmp_image_dir, :image_url_1 => 'Winter.jpg'}
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@tmp_images, tmp_images)
      @template.extend(ApplicationHelper)
      output = ec.inquiry_confirmation_form do
        ec.inquiry_confirmation_form_email.should == 'm-user@ec-one.com'
        ec.inquiry_confirmation_form_fullname.should == 'テスト姓 テスト名'
        ec.inquiry_confirmation_form_kana_name_last.should == 'テストセイ'
        ec.inquiry_confirmation_form_kana_name_first.should == 'テストメイ'
        ec.inquiry_confirmation_form_tel_num.should == '000-0000-0000'
        ec.inquiry_confirmation_form_inquiry.should == 'お問い合わせ'
        ec.inquiry_confirmation_form_inquiry_type.should == 'その他のお問合せ'
        ec.inquiry_confirmation_form_image(:display_number => 1).should have_tag('img')
        ec.button{ "←前へ戻る"}
        ec.submit{"以上の内容で問い合わせする→"}
        @template.output_buffer
      end
      output.should have_tag("input[type=?][value=?]", 'hidden', tmp_image_dir)
      @template.output_buffer.should have_tag('form[action=?]', "/inquiry/update_information") do
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '以上の内容で問い合わせする→')
      end
      FileUtils.rm_r(upload_file_path)
    end
  end
  describe "ec:inquiry_confirmation_form_country_en" do
    it "normal usage" do
      I18n.locale = 'en'
      inquiry = Inquiry.new
      inquiry.inquiry_div_key = :product
      inquiry.email = 'm-user@ec-one.com'
      inquiry.name_last = 'テスト姓'
      inquiry.name_first = 'テスト名'
      inquiry.kana_name_last = 'テストセイ'
      inquiry.kana_name_first = 'テストメイ'
      inquiry.tel_num = '000-0000-0000'
      inquiry.inquiry = 'お問い合わせ'
      inquiry.country_code = 'JP'
      inquiry.locale = 'en'
      inquiry.image_url_1 = 'Winter.jpg'
      tmp_image_dir = 'test091112001111'
      fixture_file_path = "#{EcRider.config.ec_front_root}/test/fixtures/inquiry_images/Winter.jpg"
      upload_file_path = "#{EcRider.config.ec_front_root}/public/tmp_inquiry_images/#{tmp_image_dir}/"
      FileUtils.rm_r(upload_file_path)  if File.exist?(upload_file_path)
      FileUtils.mkdir_p(upload_file_path)
      FileUtils.cp_r(fixture_file_path, upload_file_path)
      tmp_images = {:tmp_image_dir => tmp_image_dir, :image_url_1 => 'Winter.jpg'}
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@tmp_images, tmp_images)
      @template.extend(ApplicationHelper)
      output = ec.inquiry_confirmation_form do
        ec.inquiry_confirmation_form_email.should == 'm-user@ec-one.com'
        ec.inquiry_confirmation_form_fullname.should == 'テスト名 テスト姓'
        ec.inquiry_confirmation_form_kana_name_last.should == 'テストセイ'
        ec.inquiry_confirmation_form_kana_name_first.should == 'テストメイ'
        ec.inquiry_confirmation_form_tel_num.should == '000-0000-0000'
        ec.inquiry_confirmation_form_inquiry.should == 'お問い合わせ'
        ec.inquiry_confirmation_form_inquiry_type.should == '商品について'
        ec.inquiry_confirmation_form_image(:display_number => 1).should have_tag('img')
        ec.button{ "←前へ戻る"}
        ec.submit{"以上の内容で問い合わせする→"}
        @template.output_buffer
      end
      output.should have_tag("input[type=?][value=?]", 'hidden', tmp_image_dir)
      @template.output_buffer.should have_tag('form[action=?]', "/inquiry/update_information") do
        with_tag("input[type=?][value=?]", 'button', '←前へ戻る')
        with_tag("input[type=?][value=?]", 'submit', '以上の内容で問い合わせする→')
      end
      FileUtils.rm_r(upload_file_path)
    end
  end

  describe "ec:if_product_inquiry" do
    I18n.locale = 'en'
    inquiry = Inquiry.new
    inquiry.email = 'm-user@ec-one.com'
    inquiry.name_last = 'テスト姓'
    inquiry.name_first = 'テスト名'
    inquiry.kana_name_last = 'テストセイ'
    inquiry.kana_name_first = 'テストメイ'
    inquiry.tel_num = '000-0000-0000'
    inquiry.inquiry = 'お問い合わせ'
    inquiry.country_code = 'JP'
    inquiry.locale = 'en'

    it "normal usage inquiry_div true" do
      product = Product.find(1)
      inquiry.inquiry_div = '6'
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.inquiry_form do
        ec.if_product_inquiry do
          invoked = true
          @template.output_buffer
        end
        @template.output_buffer
      end
      invoked.should be_true
    end

    it "normal usage inquiry_div false" do
      product = Product.find(1)
      inquiry.inquiry_div = '5'
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.inquiry_form do
        ec.if_product_inquiry do
          invoked = true
          @template.output_buffer
        end
        @template.output_buffer
      end
      invoked.should_not be_true
    end

    it "not normal usage" do
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, nil)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.inquiry_form do
        ec.if_product_inquiry do
          invoked = true
          ""
          @template.output_buffer
        end
        ""
        @template.output_buffer
      end
      invoked.should_not be_true
    end
  end

  describe "ec:unless_product_inquiry" do
    I18n.locale = 'en'
    inquiry = Inquiry.new
    inquiry.email = 'm-shouji@ec-one.com'
    inquiry.name_last = 'テスト姓'
    inquiry.name_first = 'テスト名'
    inquiry.kana_name_last = 'テストセイ'
    inquiry.kana_name_first = 'テストメイ'
    inquiry.tel_num = '000-0000-0000'
    inquiry.inquiry = 'お問い合わせ'
    inquiry.country_code = 'JP'
    inquiry.locale = 'en'

    it "normal usage inquiry_div false" do
      product = Product.find(1)
      inquiry.inquiry_div = '6'
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.inquiry_form do
        ec.unless_product_inquiry do
          invoked = true
          @template.output_buffer
        end
        @template.output_buffer
      end
      invoked.should be_false
    end

    it "normal usage inquiry_div false" do
      product = Product.find(1)
      inquiry.inquiry_div = '5'
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.inquiry_form do
        ec.unless_product_inquiry do
          invoked = true
          @template.output_buffer
        end
        @template.output_buffer
      end
      invoked.should be_true
    end

    it "not normal usage" do
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, nil)
      @template.extend(ApplicationHelper)
      invoked = false
      ec.inquiry_form do
        ec.unless_product_inquiry do
          invoked = true
          ""
          @template.output_buffer
        end
        ""
        @template.output_buffer
      end
      invoked.should be_true
    end
  end

  describe "ec:inquiry_form_product_code" do
    I18n.locale = 'en'
    inquiry = Inquiry.new
    inquiry.inquiry_div = '6'
    inquiry.email = 'm-user@ec-one.com'
    inquiry.name_last = 'テスト姓'
    inquiry.name_first = 'テスト名'
    inquiry.kana_name_last = 'テストセイ'
    inquiry.kana_name_first = 'テストメイ'
    inquiry.tel_num = '000-0000-0000'
    inquiry.inquiry = 'お問い合わせ'
    inquiry.country_code = 'JP'
    inquiry.locale = 'en'

    it "normal usage" do
      product = Product.find(1)
      @template.instance_variable_set(:@inquiry, inquiry)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      ec.inquiry_form do
        ec.if_product_inquiry do
          ec.inquiry_form_product_code.should == '0000000009051'
          ec.inquiry_form_product_name.should == '錠剤100粒'
          @template.output_buffer
        end
        @template.output_buffer
      end
    end
  end

  describe "ec:button_to_product_detail" do
    it "options null" do
      product = Product.find(1)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      ec.button_to_product_detail{'test'}
      button = @template.output_buffer
      except_str = "location.href=\'/show?product_code=0000000009051\';"
      button.should have_tag("input[type=?][value=?][onclick=?]", 'button','test', except_str)
    end
    it "options not null" do
      product = Product.find(1)
      @template.instance_variable_set(:@inquiry_product, product)
      @template.extend(ApplicationHelper)
      ec.button_to_product_detail(:id => 'test_id', :class => 'test_class'){'test'}
      button = @template.output_buffer
      except_str = "location.href=\'/show?product_code=0000000009051\';"
      button.should have_tag("input[class=?][id=?][type=?][value=?][onclick=?]",'test_class','test_id', 'button','test', except_str)
    end
    it "product null" do
      @template.instance_variable_set(:@inquiry_product, nil)
      @template.extend(ApplicationHelper)
      ec.button_to_product_detail(:id => 'test_id', :class => 'test_class'){'test'}
      button = @template.output_buffer
      button.should == ""
    end
  end


end
