# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  include EcRider::PluginModule::RSpecExt
  
  fixtures :products, :product_displays, :product_prices, :product_groups, :product_group_skus,
  :product_skus, :product_categories, :product_category_displays, :product_images, 
  :product_variations, :product_variation_items, :product_variation_item_displays, 
  :product_sku_inventories, :product_sku_secure_inventories,
  :users, :countries, 
  :user_ranks, :user_rank_displays, :user_rank_conditions,
  :point_settings, :shop_settings,
  :campaigns, :campaign_target_product_skus, :campaign_effects, :campaign_conditions,
  :product_faqs, :product_faq_displays, :product_faq_relation_products, 
  :cms_publishes

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @template.extend(ApplicationHelper)
    @template.stub(:logged_in?).and_return(true)
    @template.stub_chain(:current_user, :user_rank_id).and_return(1)
    helper.instance_variable_set("@user_rank_id", nil)
  end

  describe "product_info" do
    it "引数を元にDBから商品情報を取得する" do
      @template.instance_variable_set("@product0000000009051", nil)
      ec.send(:product_info, '0000000009051', 'id').should == 1
      @template.instance_variable_get("@product0000000009051").id.should == 1
    end
    it "URLパラメータは引数よりも優先される" do
      @template.instance_variable_set("@product0000000009050", nil)
      params[:product_code] ='0000000009050'
      ec.send(:product_info, '0000000009051', 'id').should == 2
      @template.instance_variable_get("@product0000000009050").id.should == 2
    end
    it "@product_group_sku、@each_productが無ければ、引数またはパラメータに渡されたproduct_codeを元にキャッシュから取得する" do
      @template.instance_variable_set("@product0000000009051", mock_model(Product, :id => 4))
      ec.send(:product_info, '0000000009051', 'id').should == 4
    end
    it "@product_group_skuが無く、@each_productがあれば、@each_productを商品情報として使用する" do
      @template.instance_variable_set("@product0000000009051", mock_model(Product, :id => 4))
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      ec.send(:product_info, '0000000009051', 'id').should == 5
    end
    it "@product_group_skuがあれば、@product_group_sku.productを商品情報として使用する" do
      @template.instance_variable_set("@product0000000009051", mock_model(Product, :id => 4))
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      @template.instance_variable_set("@product_group_sku", ProductSku.find(270))
      ec.send(:product_info, '0000000009051', 'id').should == 6
    end
  end

  describe "product_sku_info" do
    it "引数を元にDBから商品情報を取得する" do
      @template.instance_variable_set("@product_sku0000000009051", nil)
      ec.send(:product_sku_info, '0000000009051', 'id').should == 265
      @template.instance_variable_get("@product_sku0000000009051").id.should == 265
    end
    it "引数(sku_code)を元にDBから商品情報を取得する" do
      @template.instance_variable_set("@product_sku0000000009051_00_00_00", nil)
      ec.send(:product_sku_info, '0000000009051-00-00-00', 'id').should == 265
      @template.instance_variable_get("@product_sku0000000009051_00_00_00").id.should == 265
    end
    it "URLパラメータは引数よりも優先される" do
      @template.instance_variable_set("@product_sku0000000009050", nil)
      params[:product_code] ='0000000009050'
      ec.send(:product_sku_info, '0000000009051', 'id').should == 266
      @template.instance_variable_get("@product_sku0000000009050").id.should == 266
    end
    it "@each_product_sku、@product_group_sku、@each_productが無ければ、引数またはパラメータに渡されたproduct_codeを元にキャッシュから取得する" do
      @template.instance_variable_set("@product_sku0000000009051", mock_model(ProductSku, :id => 4))
      ec.send(:product_sku_info, '0000000009051', 'id').should == 4
    end
    it "@each_product_sku、@product_group_sku、@each_productが無ければ、引数またはパラメータに渡されたsku_codeを元にキャッシュから取得する" do
      @template.instance_variable_set("@product_sku0000000009051", mock_model(ProductSku, :id => 4))
      @template.instance_variable_set("@product_sku0000000009051_00_00_00", mock_model(ProductSku, :id => 5))
      ec.send(:product_sku_info, '0000000009051-00-00-00', 'id').should == 5
    end
    it "@each_product_sku、@product_group_skuが無く、@each_productがあれば、@each_productを商品情報として使用する" do
      @template.instance_variable_set("@product_sku0000000009051", mock_model(ProductSku, :id => 4))
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      ec.send(:product_sku_info, '0000000009051', 'id').should == 269
    end
    it "@each_product_skuが無く、@product_group_skuがあれば、@product_group_skuを商品情報として使用する" do
      @template.instance_variable_set("@product_sku0000000009051", mock_model(ProductSku, :id => 4))
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      @template.instance_variable_set("@product_group_sku", ProductSku.find(270))
      ec.send(:product_sku_info, '0000000009051', 'id').should == 270
    end
    it "@each_product_skuがあれば、@each_product_skuを商品情報として使用する" do
      @template.instance_variable_set("@product_sku0000000009051", mock_model(ProductSku, :id => 4))
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      @template.instance_variable_set("@product_group_sku", ProductSku.find(270))
      @template.instance_variable_set("@each_product_sku", ProductSku.find(271))
      ec.send(:product_sku_info, '0000000009051', 'id').should == 271
    end
  end

  describe "product_price_info" do
    before do
      @template.instance_variable_set("@user_rank_id", 1)
    end
    it "引数を元にDBから商品情報を取得する" do
      ec.send(:product_price_info, '0000000009051', 'id').should == 1
    end
    it "引数を元にDBから商品情報を取得する(会員ランクid = 2)" do
      @template.instance_variable_set("@user_rank_id", 2)
      ec.send(:product_price_info, '0000000009051', 'id').should == 2
    end
    it "引数(sku_code)を元にDBから商品情報を取得する" do
      ec.send(:product_price_info, '0000000009051-00-00-00', 'id').should == 1
    end
    it "URLパラメータは引数よりも優先される" do
      params[:product_code] ='0000000009050'
      ec.send(:product_price_info, '0000000009051', 'id').should == 5
    end
    it "@each_product_sku、@product_group_skuが無く、@each_productがあれば、@each_productを商品情報として使用する" do
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      ec.send(:product_price_info, '0000000009051', 'id').should == 17
    end
    it "@each_product_skuが無く、@product_group_skuがあれば、@product_group_skuを商品情報として使用する" do
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      @template.instance_variable_set("@product_group_sku", ProductSku.find(270))
      ec.send(:product_price_info, '0000000009051', 'id').should == 21
    end
    it "@each_product_skuがあれば、@each_product_skuを商品情報として使用する" do
      @template.instance_variable_set("@each_product", Product.find_by_product_code('0000000009030'))
      @template.instance_variable_set("@product_group_sku", ProductSku.find(270))
      @template.instance_variable_set("@each_product_sku", ProductSku.find(271))
      ec.send(:product_price_info, '0000000009051', 'id').should == 25
    end
  end

  describe "get_user_rank_id" do
    it "キャッシュがあればキャッシュから返す" do
      @template.instance_variable_set("@user_rank_id", 9)
      ec.send(:get_user_rank_id).should == 9
    end
    it "キャッシュが無ければカレントユーザから返す" do
      @template.instance_variable_set("@user_rank_id", nil)
      ec.send(:get_user_rank_id).should == 1
      @template.instance_variable_get("@user_rank_id").should == 1
    end
    it "キャッシュが無く、ログインしていなければ非会員" do
      @template.instance_variable_set("@user_rank_id", nil)
      UserRank.update_all(:no_membership_flg => 0)
      user = UserRank.update(3, :no_membership_flg => true)
      @template.stub(:logged_in?).and_return(false)

      ec.send(:get_user_rank_id).should == 3
      @template.instance_variable_get("@user_rank_id").should == 3
    end
  end

  describe "ec:each_product_skus" do
    it "<ec:each_product_skus>～</ec:each_product_skus> 取得している商品を順にループします" do
      product_sku = ProductSku.find(279)
      product_sku.product_id = 100002
      product_sku.save!
      product = Product.find(100002)
      product.skus.size.should == 2
      product_skus = []
      ec.each_product_skus(:product_code => "0000000000002") do
        product_skus << ec.product_name_1
      end
      product_skus.size.should == 2
    end
  end

  describe  'get_all_product' do
    before(:each) do
      EcRider.config.stub!(:ec_admin).and_return(false)
      helper.instance_variable_set(:@products, nil)
      ec.product_count.should == 44
      helper.instance_variable_set(:@products, nil)
      Time.stub!(:now).and_return(Time.local(2009, 1, 10, 12, 00, 00))
      CmsPublish.stub!(:latest_test_publish).and_return(CmsPublish.new(:designated_at => Time.local(2009, 5, 10, 12, 00, 00)))
    end

    it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_frontは現在日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      product = Product.find(100011)
      product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
      product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
      product.save!
      EcRider.config.ec_test.should be_false
      ec.product_count.should == 45
    end

    it "公開開始日時<=現在日時<=公開終了日時の商品を取得できない　ec_testはテスト対象日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      product = Product.find(100011)
      product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
      product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
      product.save!
      ec.product_count.should == 45
    end

    it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_adminは公開日に関わらず取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      product = Product.find(100011)
      product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
      product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
      product.save!
      ec.product_count.should == 46
    end

    it "現在日時<公開開始日時の商品は外す　ec_frontは現在日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      product = Product.find(100011)
      product.publish_started_at = Time.local(2019, 1, 10, 13, 00, 00)
      product.save!
      EcRider.config.ec_test.should be_false
      ec.product_count.should == 44
    end

    it "公開終了日時<現在日時の商品は外す　ec_frontは現在日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      product = Product.find(100011)
      product.publish_finished_at = Time.local(2009, 1, 10, 11, 00, 00)
      product.save!
      EcRider.config.ec_test.should be_false
      ec.product_count.should == 44
    end

    it "現在日時<公開開始日時の商品は外す　ec_testはテスト対象日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      product = Product.find(100011)
      product.publish_started_at = Time.local(2009, 6, 10, 12, 00, 00)
      product.save!
      ec.product_count.should == 45
    end

    it "公開終了日時<現在日時の商品は外す　ec_testはテスト対象日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      product = Product.find(100011)
      product.publish_finished_at = Time.local(2009, 3, 10, 12, 00, 00)
      product.save!
      ec.product_count.should == 45
    end

    it "現在日時<公開開始日時の商品も取得　ec_adminは公開日に関わらず取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      product = Product.find(100011)
      product.publish_started_at = Time.local(2019, 6, 10, 12, 00, 00)
      product.save!
      EcRider.config.ec_test.should be_false
      ec.product_count.should == 46
    end

    it "公開終了日時<現在日時の商品も取得　ec_adminは公開日に関わらず取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      product = Product.find(100011)
      product.publish_finished_at = Time.local(2009, 1, 10, 11, 00, 00)
      product.save!
      ec.product_count.should == 46
    end

    it "ec_frontでは非表示フラグがtrueの商品は外す" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      product = Product.find(100011)
      product.invisible_flg = true
      product.save!
      EcRider.config.ec_test.should be_false
      ec.product_count.should == 44
    end

    it "ec_testでは非表示フラグがtrueの商品も取得" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      product = Product.find(100011)
      product.invisible_flg = true
      product.save!
      ec.product_count.should == 46
    end

    it "ec_adminでは非表示フラグがtrueの商品も取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      product = Product.find(100011)
      product.invisible_flg = true
      product.save!
      ec.product_count.should == 46
    end
  end

  describe  '<ec:if_equal value="3" key="index">～</ec:if_equal>' do
    it "price=value" do
      invoked = false
      ec.if_equal(:key => "price", :product_code => "0000000000006", :value => "7000"){ invoked = true }
      invoked.should be_false
      invoked = false
      ec.if_equal(:key => "price", :product_code => "0000000000006", :value => "8000"){ invoked = true }
      invoked.should be_true
    end

    describe "required arguments aren't input" do
      it "no key and no value" do
        lambda{ ec.if_equal{ "dummy block"  } }.should raise_error(EcRider::EcHelper::ArgumentError)
      end

      it "no value" do
        lambda{ ec.if_equal(:key => "kana_product_name", :product_code => "0000000000010"){ "dummy block"  } }.should raise_error(EcRider::EcHelper::ArgumentError)
      end

      it "no key" do
        lambda{ ec.if_equal(:value => "5"){ "dummy block" } }.should raise_error(EcRider::EcHelper::ArgumentError)
      end
    end
  end

  describe "ec:if_greater_equal" do
    it '<ec:if_greater_equal value="3" key="index">～</ec:if_greater_equal>' do
      # 在庫数>=valueの場合
      invoked = false
      ec.if_greater_equal(:key => "price", :product_code => "0000000000006", :value => "7999"){ invoked = true }
      invoked.should be_true
      invoked = false
      ec.if_greater_equal(:key => "price", :product_code => "0000000000006", :value => "8000"){ invoked = true }
      invoked.should be_true
      invoked = false
      ec.if_greater_equal(:key => "price", :product_code => "0000000000006", :value => "8001"){ invoked = true }
      invoked.should be_false
    end
  end
  describe "ec:if_greater_than" do
    it '<ec:if_greater_than value="3" key="index">～</ec:if_greater_than> ' do
      # 在庫数>=valueの場合
      invoked = false
      ec.if_greater_than(:key => "price", :product_code => "0000000000006", :value => "7999"){ invoked = true }
      invoked.should be_true
      invoked = false
      ec.if_greater_than(:key => "price", :product_code => "0000000000006", :value => "8000"){ invoked = true }
      invoked.should be_false
      invoked = false
      ec.if_greater_than(:key => "price", :product_code => "0000000000006", :value => "8001"){ invoked = true }
      invoked.should be_false
    end
  end

  describe "ec:if_greater_equal" do
    it '<ec:if_less_equal value="3" key="index">～</ec:if_less_equal>' do
      # 在庫数<=valueの場合
      invoked = false
      ec.if_less_equal(:key => "price", :product_code => "0000000000006", :value => "7999"){ invoked = true }
      invoked.should be_false
      invoked = false
      ec.if_less_equal(:key => "price", :product_code => "0000000000006", :value => "8000"){ invoked = true }
      invoked.should be_true
      invoked = false
      ec.if_less_equal(:key => "price", :product_code => "0000000000006", :value => "8001"){ invoked = true }
      invoked.should be_true
    end
  end

  describe "ec:if_greater_than" do
    it '<ec:if_less_than value="3" key="index">～</ec:if_less_than> ' do
      # 在庫数<valueの場合
      invoked = false
      ec.if_less_than(:key => "price", :product_code => "0000000000006", :value => "7999"){ invoked = true }
      invoked.should be_false
      invoked = false
      ec.if_less_than(:key => "price", :product_code => "0000000000006", :value => "8000"){ invoked = true }
      invoked.should be_false
      invoked = false
      ec.if_less_than(:key => "price", :product_code => "0000000000006", :value => "8001"){ invoked = true }
      invoked.should be_true
    end
  end

  describe "ec:product_count" do
    it "<ec:product_count/>" do
      helper.instance_variable_set(:@products, [
          Product.new(:product_code => "C0001"),
          Product.new(:product_code => "C0002")])
      ec.product_count.should == 2
    end

    it "<%= ec:product_count %> 商品の総数を返します" do
      I18n.locale="ja"
      ec.product_count.should == 2
      helper.instance_variable_set(:@products, [])
      ec.product_count.should == 0
    end
  end

  describe "ec:each_product" do
    it "<ec:each_products>～</ec:each_products> 取得している商品を順にループします" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      passed_products = []
      ec.each_products do
        passed_products << ec.product_name_1
      end
      passed_products.size.should == 44
    end
    it "検索商品を表示する場合" do
      passed_products = []
      products = [Product.find(1),Product.find(2),Product.find(3)]
      @template.instance_variable_set(:@products, products)
      ec.each_products do
        passed_products << ec.product_name_1
      end
      passed_products.size.should == 3
    end
  end

  # TODO 未実装
#  describe "<ec:fetch_products/>" do
#    it "引数なしなら全ての商品を取得します" do
#      now = Time.local(2009, 1, 10, 12, 00, 00)
#      Time.stub!(:now).and_return(now)
#      product1 = mock_model(Product, :product_code => "1234567890123")
#      product_display1 = mock_model(ProductDisplay, :product_id => product1.id)
#      Product.should_receive(:find).with(:all,
#        :conditions => ["(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
#            "(invisible_flg is null)", now, now],
#      :order=>"products.display_order is null, products.display_order, products.product_code").
#        and_return([product1])
#      ec.fetch_products
#    end
#
#    it "ec_testはテスト対象日時で検索します" do
#      EcRider.config.stub!(:ec_test).and_return(true)
#      now = Time.local(2009, 1, 10, 12, 00, 00)
#      Time.stub!(:now).and_return(now)
#      designated_now = Time.local(2009, 5, 10, 12, 00, 00)
#      CmsPublish.stub!(:latest_test_publish).and_return(CmsPublish.new(:designated_at => designated_now))
#      product1 = mock_model(Product, :product_code => "1234567890123")
#      product_display1 = mock_model(ProductDisplay, :product_id => product1.id)
#      Product.should_receive(:find).with(:all,
#        :conditions => ["(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)))",
#          designated_now, designated_now],
#        :order=>"products.display_order is null, products.display_order, products.product_code").
#        and_return([product1])
#      ec.fetch_products
#    end
#
#    it "引数なしなら全ての商品を取得します" do
#      now = Time.local(2009, 1, 10, 12, 00, 00)
#      Time.stub!(:now).and_return(now)
#      product1 = mock_model(Product, :product_code => "1234567890123")
#      Product.should_receive(:find).
#        with(:all,
#        :joins => "inner join kbn_codes nm4category1 on nm4category1.code = products.category_1_id",
#        :conditions => ["(nm4category1.kbn = ?) AND (nm4category1.kbn_name like ?) AND " <<
#            "(((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))) AND " <<
#            "(invisible_flg is null)",
#          "01", "%服%", now, now],
#        :order=>"products.display_order is null, products.display_order, products.product_code").
#        and_return([product1])
#      ec.fetch_products(:category1 => "服")
#    end
#  end

  describe "ec:product_xxx" do
    describe "with fixtures :all" do
      fixtures :products, :product_categories, :product_category_displays, :product_components,
        :product_displays, :product_groups, :product_group_skus, :product_images,
        :product_prices, :product_skus, :product_sku_inventories, :product_sku_secure_inventories,
        :product_variations, :product_variation_displays, :product_variation_items, :product_variation_item_displays

      it "価格_非会員" do
        UserRank.update_all(:no_membership_flg => 0)
        UserRank.update(3, :no_membership_flg => 1)
        @template.stub(:logged_in?).and_return(false)

        ec.product_standard_price(:product_code => '0000000000052-00-00-00').should == "8,000"
      end

      it "価格_会員" do
        ec.product_standard_price(:product_code => '0000000000052-00-00-00').should == "16,800"
      end

      it '<%= ec:product_xxx(:product_code=>～) %> 商品コードを指定してデータを返します product_name_1' do
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == "商品1の商品名1"
      end
      
      it '<%= ec:product_xxx(:product_code=>～) %> 商品コードを指定してデータを返します' do
        ec.product_name_2(:product_code => "0000000000001").should == "商品1の商品名2"
        
        # 価格
        ec.product_price(:product_code => "0000000000001").should == "8,000"
        ec.product_price(:product_code => "0000000000001", :pre_currency => "￥", :post_currency => "").should == "￥8,000"
        ec.product_price(:product_code => "0000000000001", :pre_currency => "", :post_currency => "円").should == "8,000円"
        ec.product_price(:product_code => "wrongcode").should == "0"

        # 換算価格（換算レートが登録されていない場合）
        ec.exchange_price(:product_code => "0000000000001").should == ""
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == ""
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == ""

        # 送料込み換算価格
        ec.exchange_price_include_shipping_cost(:product_code => "0000000000001").should == ""
        ec.exchange_price_include_shipping_cost(:product_code => "0000000000001", :pre_currency => "中国").should == ""
        ec.exchange_price_include_shipping_cost(:product_code => "0000000000001", :post_currency => "元").should == ""

        setting = ShopSetting.new(:setting_div => ShopSetting.setting_div_id_by_key(:exchange_rate), :value => "0.52222312")
        setting.save!

        # 換算価格（換算後の金額の端数の処理が登録されていない場合（切り上げ））
        ec.exchange_price(:product_code => "0000000000001").should == "4,178"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,178"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,178元"
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222313"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,178"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,178"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,178元"

        # 換算価格（切り上げ）
        setting = ShopSetting.new(:setting_div => ShopSetting.setting_div_id_by_key(:rounding_processing), :value => "round_up")
        setting.save!
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222312"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,178"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,178"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,178元"
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222313"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,178"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,178"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,178元"

        # 換算価格（切り捨て）
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:rounding_processing))
        setting.value = "round_down"
        setting.save!
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222312"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,177"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,177"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,177元"
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222313"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,177"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,177"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,177元"

        # 換算価格（四捨五入）
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:rounding_processing))
        setting.value = "round_off"
        setting.save!
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222312"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,178"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,178"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,178元"
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.52222313"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "4,178"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国4,178"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "4,178元"

        # 換算価格（丸めが行われない場合）
        setting = ShopSetting.find_by_setting_div(ShopSetting.setting_div_id_by_key(:exchange_rate))
        setting.value = "0.0733"
        setting.save!
        ec.exchange_price(:product_code => "0000000000001").should == "586"
        ec.exchange_price(:product_code => "0000000000001", :pre_currency => "中国").should == "中国586"
        ec.exchange_price(:product_code => "0000000000001", :post_currency => "元").should == "586元"

        # 数量
        ProductSku.update(282, :order_limit => 99)

        # 原産国
        ec.product_origin(:product_code => "0000000000011").should == 'Japan'

        # 画像
        ec.product_image(:product_code => "0000000000001").should have_tag('img[src*=/product00001.gif][alt="商品1の商品名1"]')
        ec.product_image(:product_code => "0000000000001", :src => "wrong_src.gif").should have_tag('img[src^=/images/wrong_src.gif][alt="商品1の商品名1"]')
        ec.product_image(:product_code => "0000000000001", :alt => '商品１').should have_tag('img[src*=/product00001.gif][alt="商品１"]')
        ec.product_image(:product_code => "0000000000001", :width => "50", :height => "50").should have_tag('img[src*=/product00001.gif][height="50"][width="50"][alt="商品1の商品名1"]')
        ec.product_image(:product_code => "0000000000001", :type => "large").should have_tag('img[src*=/product00001_large.gif][alt="商品1の商品名1"]')
        ec.product_image(:product_code => "0000000000001", :type => "small").should have_tag('img[src*=/product00001_small.gif][alt="商品1の商品名1"]')
        ec.product_image(:product_code => "0000000000001", :type => "medium").should have_tag('img[src*=/product00001_medium.gif][alt="商品1の商品名1"]')
       
        ec.product_image(:index => 2, :product_code => "0000000000001").should have_tag('img[src*=/product00002.gif][alt="商品1の商品名1"]')
        ec.product_image(:index => 2, :product_code => "0000000000001", :src => "wrong_src.gif").should have_tag('img[src^=/images/wrong_src.gif][alt="商品1の商品名1"]')
        ec.product_image(:index => 2, :product_code => "0000000000001", :alt => '商品１').should have_tag('img[src*=/product00002.gif][alt="商品１"]')
        ec.product_image(:index => 2, :product_code => "0000000000001", :width => "50", :height => "50").should have_tag('img[src*=/product00002.gif][height="50"][width="50"][alt="商品1の商品名1"]')
        ec.product_image(:index => 2, :product_code => "0000000000001", :type => "large").should have_tag('img[src*=/product00002_large.gif][alt="商品1の商品名1"]')
        ec.product_image(:index => 2, :product_code => "0000000000001", :type => "small").should have_tag('img[src*=/product00002_small.gif][alt="商品1の商品名1"]')
        ec.product_image(:index => 2, :product_code => "0000000000001", :type => "medium").should have_tag('img[src*=/product00002_medium.gif][alt="商品1の商品名1"]')

        # リンク
        ec.product_detail_link(:product_code => "0000000000001", :text => "詳細を見る").should == '<a href="/show?product_code=0000000000001">詳細を見る</a>'

        # 画像リンク
        ec.product_image_link(:product_code => "0000000000001").should have_tag('a[href="/show?product_code=0000000000001"]') do
          with_tag('img[src*=/product00001.gif][alt="商品1の商品名1"]')
        end
        ec.product_image_link(:product_code => "0000000000001", :index => 2).should have_tag('a[href="/show?product_code=0000000000001"]') do
          with_tag('img[src*=/product00002.gif][alt="商品1の商品名1"]')
        end
        ec.product_image_link(:product_code => "0000000000001", :alt => "商品1").should have_tag('a[href="/show?product_code=0000000000001"]') do
          with_tag('img[src*=/product00001.gif][alt="商品1"]')
        end
        ec.product_image_link(:product_code => "0000000000001", :type => "large").should have_tag('a[href="/show?product_code=0000000000001"]') do
          with_tag('img[src*=/product00001_large.gif][alt="商品1の商品名1"]')
        end

        # タイトル、概要
        pending_if_loaded_plugin(:c0001) do
          ec.product_outline(:product_code => "0000000000001").should == "商品1の商品概要"
        end
        ec.product_explanation(:product_code => "0000000000001").should == "商品1の商品説明"
        ec.recommendation(:product_code => "0000000000001").should == "商品1の推奨メッセージ"
        ec.recommendation_2(:product_code => "0000000000001").should == "商品1の推奨メッセージ2"

        #METAタグ(keywords) localが設定されていない場合
        ec.product_keyword(:product_code => "0000000000011") == '<meta content="shirt" name="keywords">'
        #METAタグ(keywords) localが設定されている場合
        ec.product_keyword(:product_code => "0000000000011", :locale => "en") == '<meta content="shirt" locale="en" name="keywords">'

        pending_if_loaded_plugin(:c0001) do
          # 存在しない商品コードを指定
          ec.product_outline(:product_code => "wrongcode").should == ""
        end
      end

      it '<%= ec:product_xxx(:product_code=>～) %> 商品SKUコードを指定してデータを返します product_name_1' do
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == "商品1の商品名1"
      end
      
      it '<%= ec:product_xxx(:product_code=>～) %> 商品SKUコードを指定してデータを返します product_name_2' do
        ec.product_name_2(:product_code => "0000000000001-00-00-00").should == "商品1の商品名2"
      end
    end

    it "<%= ec:product_sku_xxx(:product_sku_code=>～) %> 商品SKUコードを指定してorder_limitを返します" do
      ec.order_limit(:product_code => "0000000000001").should == "99"
    end

    it "<%= ec:product_sku_xxx(:product_sku_code=>～) %> 商品SKUコードを指定してweightを返します" do
      ec.product_weight(:product_code => "0000000000001").should == "100"
    end

    describe "ec_frontとec_testの表示制御 product_codeから取得" do
      before(:each) do
        helper.instance_variable_set(:@product0000000000001, nil)
        Time.stub!(:now).and_return(Time.local(2009, 1, 10, 12, 00, 00))
        CmsPublish.stub!(:latest_test_publish).and_return(CmsPublish.new(:designated_at => Time.local(2009, 5, 10, 12, 00, 00)))
      end

      it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_frontは現在日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == "商品1の商品名1"
      end

      it "公開開始日時<=現在日時<=公開終了日時の商品を取得できない　ec_testはテスト対象日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == ''
      end

      it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_adminは公開日時に拘らず取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == "商品1の商品名1"
      end

      it "現在日時<公開開始日時の商品は外す　ec_frontは現在日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2019, 1, 10, 13, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == ''
      end

      it "公開終了日時<現在日時の商品は外す　ec_frontは現在日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.publish_finished_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == ''
      end

      it "現在日時<公開開始日時の商品は外す　ec_testはテスト対象日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2019, 6, 10, 12, 00, 00)
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == ''
      end

      it "公開終了日時<現在日時の商品は外す　ec_testはテスト対象日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(true)
        product = Product.find(100001)
        product.publish_finished_at = Time.local(2009, 3, 10, 12, 00, 00)
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == ''
      end

      it "現在日時<公開開始日時の商品も取得　ec_adminは公開日に関わらず取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2019, 6, 10, 12, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == '商品1の商品名1'
      end
  
      it "公開終了日時<現在日時の商品も取得　ec_adminは公開日に関わらず取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 6, 10, 12, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == '商品1の商品名1'
      end

      it "ec_frontでは非表示フラグがtrueの商品は外す" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.invisible_flg = true
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == ''
      end

      it "ec_testでは非表示フラグがtrueの商品も取得" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(true)
        product = Product.find(100001)
        product.invisible_flg = true
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == "商品1の商品名1"
      end

      it "ec_adminでは非表示フラグがtrueの商品も取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.invisible_flg = true
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001").should == "商品1の商品名1"
      end
    end

    describe "ec_frontとec_testの表示制御 product_sku_codeから取得" do
      before(:each) do
        helper.instance_variable_set(:@product0000000000001_00_00_00, nil)
        Time.stub!(:now).and_return(Time.local(2009, 1, 10, 12, 00, 00))
        CmsPublish.stub!(:latest_test_publish).and_return(CmsPublish.new(:designated_at => Time.local(2009, 5, 10, 12, 00, 00)))
      end

      it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_frontは現在日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false

        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == "商品1の商品名1"
      end

      it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_testはテスト対象日時で検索" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == ''
      end

      it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_adminは公開日時に拘らず取得 " do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.publish_finished_at = Time.local(2009, 1, 15, 11, 00, 00)
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == "商品1の商品名1"
      end

      it "現在日時<公開開始日時の商品は外す" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.publish_started_at = Time.local(2019, 1, 10, 13, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == ''
      end

      it "公開終了日時<現在日時の商品は外す" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.publish_finished_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == ''
      end

      it "現在日時<公開開始日時の商品も取得　ec_adminは公開日に関わらず取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.publish_finished_at = Time.local(2019, 1, 10, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == '商品1の商品名1'
      end
  
      it "公開終了日時<現在日時の商品も取得　ec_adminは公開日に関わらず取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.publish_finished_at = Time.local(2009, 1, 10, 11, 00, 00)
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == '商品1の商品名1'
      end

      it "ec_frontでは非表示フラグがtrueの商品は外す" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        product = Product.find(100001)
        product.invisible_flg = true
        product.save!
        EcRider.config.ec_test.should be_false
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == ''
      end

      it "ec_testでは非表示フラグがtrueの商品も取得" do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(true)
        product = Product.find(100001)
        product.invisible_flg = true
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == "商品1の商品名1"
      end

      it "ec_adminでは非表示フラグがtrueの商品も取得" do
        EcRider.config.stub!(:ec_admin).and_return(true)
        product = Product.find(100001)
        product.invisible_flg = true
        product.save!
        
        pending_if_loaded_plugin(:c0001)
        ec.product_name_1(:product_code => "0000000000001-00-00-00").should == "商品1の商品名1"
      end
    end
  end

  describe "<%= ec:button_to_product_inquiry %> " do
    it "options null" do
      ec.button_to_product_inquiry(:product_code => "0000000000011"){'test'}
      button = @template.output_buffer
      except_str = "location.href=\'/inquiry?product_code=0000000000011\';"
      button.should have_tag("input[type=?][value=?][onclick=?]", 'button','test',except_str)
    end

    it "options not null" do
      ec.button_to_product_inquiry(:product_code => "0000000000011", :id => 'test_id', :class => 'test_class'){'test'}
      button = @template.output_buffer
      except_str = "location.href=\'/inquiry?product_code=0000000000011\';"
      button.should have_tag("input[class=?][id=?][type=?][value=?][onclick=?]",'test_class','test_id', 'button','test', except_str)
    end
  end

  describe "ec:product_crumbs_display" do
    it "category_code がない場合" do
      ec.product_crumbs_display.should == nil
    end
    it "category_code がある場合(店舗側が指定した場合)" do
      ec.product_crumbs_display(:category_code =>'code14').should == '<a href="/list?category_code=code14">インテリア・生活雑貨</a>'
    end
    it "category_code がある場合(商品検索結果のページで使用している場合)" do
      @template.instance_variable_set(:@condition, Product::CategorySearchForm.new(:category_code => "code15"))
      ec.product_crumbs_display.should == '<a href="/list?category_code=code13">電化製品・カメラ</a> > <a href="/list?category_code=code15">パソコン</a>'
    end
  end

  describe "ec:product_crumbs" do
    it "option null" do
      # 秋間コメント
      # TO 廣瀬君
      # 前に指摘した商品コードを指定するべきところが直っていませんでした。
      ec.product_crumbs(:product_code => "0000000000048" , :display_number => 4).should ==
        '<a href="/list?category_code=code14">インテリア・生活雑貨</a> > <a href="/list?category_code=code16">その他</a>' <<
        ' > <a href="/list?category_code=code20">キッチン</a>'
      ec.product_crumbs(:product_code => "0000000000048" , :display_number => 2).should ==
        '<a href="/list?category_code=code14">インテリア・生活雑貨</a>'
    end
    it "option not null" do
      # 秋間コメント
      # TO 廣瀬君
      # 前に指摘した商品コードを指定するべきところが直っていませんでした。
      ec.product_crumbs(:product_code => "0000000000048" , :display_number => 4 , :separator =>' | ').should ==
        '<a href="/list?category_code=code14">インテリア・生活雑貨</a> | <a href="/list?category_code=code16">その他</a>' <<
        ' | <a href="/list?category_code=code20">キッチン</a>'
    end
  end

  describe "product_search_form" do

    def concat(*args)
      args.each{|arg| @template.concat(arg)}
    end

    describe "product_search_form" do
      it ":render_nameの引数がある場合" do
        if EcRider::Runtime.ec_front?
          ec.product_search_form(:render_name => 'product_search') do
            ec.submit{"test"}
            @template.output_buffer
          end

          @template.output_buffer.should have_tag('form[action=?]', "/product_search") do
            with_tag("input[type=?][name=?]", 'hidden', 'condition[per_page]')
            with_tag("input[type=?][value=?]", 'submit', 'test')
          end
        end
      end
      it ":render_name,:product_separate_valueの引数がある場合" do
        if EcRider::Runtime.ec_front?
          ec.product_search_form(:render_name => 'product_search', :product_separate_value => "20" ) do
            ec.submit{"test"}
            @template.output_buffer
          end
          @template.output_buffer.should have_tag('form[action=?]', "/product_search") do
            with_tag("input[type=?][name=?]", 'hidden', 'condition[per_page]')
            with_tag("input[type=?][value=?]", 'submit', 'test')
          end
        end
      end
    end

    describe "product_search_form_category_select" do
      it ":include_nameの引数がない場合" do
        ec.product_search_form(:render_name => 'testpage') do
          concat(ec.product_search_form_category_select)
          ec.submit{"test"}
          @template.output_buffer
        end
        @template.output_buffer.should have_tag("select[name=?]","condition[category_code]") do |select|
          select.should_not have_tag("option[value=?]","all")
        end
      end
      it ":include_nameの引数がある場合" do
        ec.product_search_form(:render_name => 'testpage') do
          concat(ec.product_search_form_category_select(:include_name => 'testtest'))
          ec.submit{"test"}
          @template.output_buffer
        end
        @template.output_buffer.should have_tag("select[name=?]","condition[category_code]") do |select|
          select.should have_tag("option[value=?]","")
        end
      end
    end

    describe "product_search_form_keyword" do
      it "usage" do
        if EcRider::Runtime.ec_front?
          ec.product_search_form(:render_name => 'product_search') do
            concat(ec.product_search_form_keyword)
            ec.submit{"test"}
            @template.output_buffer
          end

          @template.output_buffer.should have_tag('form[action=?]', "/product_search") do
            with_tag("input[name=?]", 'condition[keyword]')
          end
        end
      end
    end

    describe "product_search_form_condition_select" do
      it "usage" do
        ec.product_search_form(:render_name => 'product_search') do
          concat(ec.product_search_form_condition_select(:and_text => "and_test", :or_text => "or_test", :class => "testclass"))
          ec.submit{"test"}
          @template.output_buffer
        end
        @template.output_buffer.should have_tag("select[name=?][class=?]","condition[condition_div]", "testclass") do |select|
          select.should have_tag("option[value=?]","0")
          select.should have_tag("option[value=?]","1")
        end
      end
    end

    describe "カテゴリ検索 カテゴリツリータグ" do
      describe "カテゴリ検索 カテゴリツリータグ" do
        it "全カテゴリを取得する場合" do
          ul_index = 0
          @template.concat("<div>")
          ec.product_category_tree() do
            @template.concat("<ul>")
            @template.output_buffer
            ec.category_item() do
              @template.concat("<li>")
              @template.concat(ec.descendant_link(:render_name => 'search_result'))
              @template.concat("</li>")
              @template.output_buffer
            end
            @template.concat("</ul>")
            @template.output_buffer
          end
          @template.concat("</div>")
          @template.output_buffer.should have_tag("div") do |div|
            div.should have_tag("ul") do |ul_1|
              ul_1.should have_tag("li") do |ul_1_li_1|
                ul_1_li_1.should have_tag("a")
              end
              ul_1.should have_tag("ul") do |ul_1_1|
                ul_1_1.should have_tag("li") do |ul_1_1_li_1|
                  ul_1_1_li_1.should have_tag("a")
                end
              end
              ul_1.should have_tag("ul") do |ul_1_2|
                ul_1_2.should have_tag("li") do |ul_1_2_li_1|
                  ul_1_2_li_1.should have_tag("a")
                end
              end
            end
            div.should have_tag("ul") do |ul_2|
              ul_2.should have_tag("li") do |ul_2_li_1|
                ul_2_li_1.should have_tag("a")
              end
            end
          end
        end
      end
    end

    describe "searched_product_empty" do
      it "@search_productsが1以上の場合" do
        @template.instance_variable_set(:@search_products, Product.find(:all))
        flg = "false"
        ec.searched_product_empty do
          flg = "true"
        end
        flg.should == "false"
      end
      it "@search_productsが0の場合" do
        @template.instance_variable_set(:@search_products, [])
        flg = "false"
        ec.searched_product_empty do
          flg = "true"
        end
        flg.should == "true"
      end
    end

    describe "searched_product_not_empty" do
      it "@search_productsが1以上の場合" do
        @template.instance_variable_set(:@search_products, Product.find(:all))
        flg = "false"
        ec.searched_product_not_empty do
          flg = "true"
        end
        flg.should == "true"
      end
      it "@search_productsが0の場合" do
        @template.instance_variable_set(:@search_products, [])
        flg = "false"
        ec.searched_product_not_empty do
          flg = "true"
        end
        flg.should == "false"
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        flg = "false"
        ec.searched_product_not_empty do
          flg = "true"
        end
        flg.should == "false"
      end
    end

    describe "product_pagination_total_count" do
      it "@search_productsが1以上の場合" do
        products = Product.paginate(:all, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@search_products, products)
        ec.product_pagination_total_count.should == products.total_entries
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        ec.product_pagination_total_count.should_not == nil
      end
    end

    describe "product_pagination_start_index" do
      it "@search_productsが1以上の場合" do
        products = Product.paginate(:all, :limit => 7, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@search_products, products)
        ec.product_pagination_start_index.should == 3
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        ec.product_pagination_start_index.should_not == nil
      end
    end
    describe "product_pagination_last_index" do
      it "@search_productsが1以上の場合" do
        products = Product.paginate(:all, :limit => 7, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@search_products, products)
        ec.product_pagination_last_index.should == 4
      end
      it "@search_productsが存在しない(検索されていない)場合" do
        ec.product_pagination_last_index.should_not == nil
      end
    end
    describe "product_pagination_link" do
      it "検索結果がない場合" do
        products = Product.paginate(:all, :conditions => ['product_code = ?', 'testtestproduct'],:page => 1, :per_page => 1)
        @template.instance_variable_set(:@search_products, products)
        ec.product_pagination_link.should == nil
      end
      it "デフォルトの場合" do
        pending "willpagenate内でprivateメソッドを呼んでいて解決策不明のため保留"
        request.stub!(:controller).and_return('cms')
        renderer.stub!(:to_html).and_return('html')
        products = Product.paginate(:all, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@search_products, products)
        ec.product_pagination_link.shoud have_tag("a") do |a_tag|
          with_tag("a[class = ?]", "testclass")
          a_tags << a_tag
        end
      end
      it "リンク名を指定した場合" do
        pending "willpagenate内でprivateメソッドを呼んでいて解決策不明のため保留"
        products = Product.paginate(:all, :page => 2, :per_page => 2)
        @template.instance_variable_set(:@search_products, products)
        ec.product_pagination_link.shoud have_tag("a") do |a_tag|
          with_tag("a[class = ?]", "testclass")
          a_tags << a_tag
        end
      end
    end

    describe "product_pagination_count_per_page" do
      it "デフォルトの場合" do
        @template.instance_variable_set(:@search_products, nil)
        @template.concat(ec.product_pagination_count_per_page(:render_name => 'testpage'))
        @template.output_buffer.should have_tag("select") do |select|
          with_tag("option[value=?]", "10")
          with_tag("option[value=?]", "20")
          with_tag("option[value=?]", "30")
        end
      end
      it "表示件数指定の場合" do
        @template.instance_variable_set(:@search_products, nil)
        ec.product_pagination_count_per_page(:render_name => 'testpage', :select_count => "2,4,6")
        @template.output_buffer.should have_tag("select") do |select|
          with_tag("option[value=?]", "2")
          with_tag("option[value=?]", "4")
          with_tag("option[value=?]", "6")
        end
      end
    end

    describe "product_search_word_not_empty" do
      it "if @conditionがある場合" do
        @template.instance_variable_set(:@condition, Product::CategorySearchForm.new(:keyword => 'シャツ'))
        flg = "false"
        ec.if_search_condition_exits do
          flg = "true"
          ec.search_condition.should == 'シャツ'
        end
        flg.should == "true"
      end

      it "if @conditionがある場合" do
        @template.instance_variable_set(:@condition, Product::CategorySearchForm.new(:keyword => 'シャツ'))
        flg = "false"
        ec.unless_search_condition_exits do
          flg = "true"
        end
        flg.should == "false"

      end

      it "if @conditionがない場合" do
        @template.instance_variable_set(:@condition, nil)
        flg = "false"
        ec.if_search_condition_exits do
          flg = "true"
        end
        flg.should == "false"
      end

      it "if @conditionがない場合" do
        @template.instance_variable_set(:@condition, nil)
        flg = "false"
        ec.unless_search_condition_exits do
          flg = "true"
        end
        flg.should == "true"
      end
    end

    describe "if_sku_exist" do
      it "SKUが存在する" do
        @template.instance_variable_set(:@product_sku, ProductSku.find(:first))
        ec.if_sku_exist do
          true.should be_true
        end
        ec.unless_sku_exist do
          #ここは通ってはいけません
          true.should be_false
        end
      end
      
      it "SKUが存在しない" do
        @template.instance_variable_set(:@product_sku, nil)
        ec.if_sku_exist do
          #ここは通ってはいけません
          true.should be_false
        end
        ec.unless_sku_exist do
          true.should be_true
        end
      end
    end
    
    describe "searched_products" do
      before(:each) do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(false)
        I18n.stub!(:locale).and_return('ja')
        Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      end
      it "未入力検索" do
        params[:condition] = {:category_code => '',:keyword => '',:condition_div => '0', :per_page => '10'}
        ec.searched_products do
          @template.instance_variable_get(:@products).should_not == nil
        end
        @template.instance_variable_get(:@products).should == nil
      end
      it "カテゴリ検索(検索フォームからの場合)" do
        params[:condition] = {:category_code => 'code13',:keyword => '',:condition_div => '0', :per_page => '10'}
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 3
        end
        @template.instance_variable_get(:@products).should == nil
      end
      it "カテゴリ検索(リンクタグからの場合)" do
        params[:category_code] = 'code13'
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 3
        end
      end
      it "キーワード検索" do
        params[:condition] = {:category_code => '',:keyword => 'keytest',:condition_div => '0', :per_page => '10'}
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 1
          @template.instance_variable_get(:@products)[0].id.should == 100052
        end
      end
      it "条件検索" do
        params[:condition] = {:category_code => '',:keyword => 'keytest wrongkeyword',:condition_div => '1', :per_page => '10'}
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 1
        end
        params[:condition] = {:category_code => '',:keyword => 'keytest wrongkeyword',:condition_div => '0', :per_page => '10'}
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 0
        end
      end
      it "表示レコード数検索" do
        params[:condition] = {:category_code => '',:keyword => '',:condition_div => '0', :per_page => '13'}
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 13
        end
      end
      it "次ページ検索" do
        params[:condition] = {:category_code => '',:keyword => '',:condition_div => '0', :per_page => '13'}
        params[:page] = '2'
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 13
          @template.instance_variable_get(:@products).previous_page.should == 1
        end
      end

      it "並べ替え カテゴリ検索　標準" do
        params[:category_code] = 'code14'
        request.session[:user] = User.find(1000)
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 4
          @template.instance_variable_get(:@products)[0].display_order.should == 26
          @template.instance_variable_get(:@products)[1].display_order.should == 26
          @template.instance_variable_get(:@products)[2].display_order.should == 26
          @template.instance_variable_get(:@products)[3].display_order.should == 26
          @template.instance_variable_get(:@products)[0].product_code.should == '0000000000048'
          @template.instance_variable_get(:@products)[1].product_code.should == '0000000000049'
          @template.instance_variable_get(:@products)[2].product_code.should == '0000000000050'
          @template.instance_variable_get(:@products)[3].product_code.should == '0000000000051'
        end
      end

      it "並べ替え カテゴリ検索　価格の安い順" do
        params[:category_code] = 'code14'
        params[:order] = '1'
        request.session[:user] = User.find(1000)
        ec.searched_products do
          @products = @template.instance_variable_get(:@products)
          @products.size.should == 4
          unit_price = 0
          @products.each do |product|
            product.price.should >= unit_price
            unit_price = product.price
          end
        end
      end

      it "並べ替え カテゴリ検索　価格の高い順" do
        params[:category_code] = 'code14'
        params[:order] = '2'
        request.session[:user] = User.find(1000)
        ec.searched_products do
          @products = @template.instance_variable_get(:@products)
          @products.size.should == 4
          unit_price = 0
          @products.each do |product|
            if unit_price == 0
              unit_price = product.price
            else
              product.price.should <= unit_price
            end
            unit_price = product.price
          end
        end
      end

      it "並べ替え カテゴリ検索　新着順" do
        params[:category_code] = 'code14'
        params[:order] = '3'
        request.session[:user] = User.find(1000)
        ec.searched_products do
          @products = @template.instance_variable_get(:@products)
          @products.size.should == 4
          id = 0
          @products.each do |product|
            id = product.id if id == 0
            product.id.should >= id
            id = product.id
          end
        end
      end

      it "条件検索 標準順" do
        params[:condition] = {:category_code => 'code14',:keyword => '', :per_page => '10'}
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 4
          @products = @template.instance_variable_get(:@products)
          display_order = 0
          @products.each do |product|
            display_order = product.display_order if display_order == 0
            product.display_order.should >= display_order
            display_order = product.display_order
          end
        end
      end

      it "条件検索 価格の高い順" do
        params[:condition] = {:category_code => 'code14',:keyword => '', :per_page => '10'}
        params[:order] = '2'
        request.session[:user] = User.find(1000)
        ec.searched_products do
          @template.instance_variable_get(:@products).size.should == 4
          @products = @template.instance_variable_get(:@products)
          unit_price = 0
          @products.each do |product|
            if unit_price == 0
              unit_price = product.price
            else
              product.price.should <= unit_price
            end
            unit_price = product.price
          end
        end
      end
    end

    describe "ec:product_sort_link" do
      before do
        params[:controller] = 'cms'
        params[:action] = 'test_action'
      end
      it "価格の安い順" do
        ec.product_sort_link(:order => '1'){"リンク"}
        if EcRider::Runtime.ec_admin?
          @template.output_buffer.should == '<a href="/cms/test_action?order=1">リンク</a>'
        else
          @template.output_buffer.should == '<a href="/test_action?order=1">リンク</a>'
        end
      end
      it "標準" do
        ec.product_sort_link(){"リンク"}
        if EcRider::Runtime.ec_admin?
          @template.output_buffer.should == '<a href="/cms/test_action?order=0">リンク</a>'
        else
          @template.output_buffer.should == '<a href="/test_action?order=0">リンク</a>'
        end
      end
    end
  end

  describe "condition_products" do
    before(:each) do
      pending_if_loaded_plugin(:c0001)
    end
    it "引数がない場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      ec.condition_products do
        @template.instance_variable_get(:@products).should == nil
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:product_separate_value => 20) do
        @template.instance_variable_get(:@products).should == nil
      end
      @template.instance_variable_get(:@products).should == nil
    end
    it "存在しないカテゴリーコードの場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      ec.condition_products(:category_codes => "code18 code19") do
        @template.instance_variable_get(:@products).should_not == nil
      end
      @template.instance_variable_get(:@products).should == nil
    end
    it "存在するカテゴリーコードの場合 ja" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:category_codes => "code18, code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:category_codes => "code18,code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:category_codes => "code16") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 2
      end
      @template.instance_variable_get(:@products).should == nil
    end
    it "存在するカテゴリーコードの場合 ko" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ko')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 1
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:category_codes => "code18, code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 1
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:category_codes => "code18,code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 1
      end
      @template.instance_variable_get(:@products).should == nil
      ec.condition_products(:category_codes => "code16") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 0
      end
      @template.instance_variable_get(:@products).should == nil
    end

    it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_frontは現在日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
    end

    it "公開開始日時<=現在日時<=公開終了日時の商品を取得できない　ec_testはテスト対象日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 5
      end
    end

    it "公開開始日時<=現在日時<=公開終了日時の商品を取得　ec_adminは公開日に関わらず取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 6
      end
    end

    it "現在日時<公開開始日時の商品は外す　ec_frontは現在日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
    end

    it "公開終了日時<現在日時の商品は外す　ec_frontは現在日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
    end

    it "現在日時<公開開始日時の商品は外す　ec_testはテスト対象日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_true
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 5
      end
    end

    it "公開終了日時<現在日時の商品は外す　ec_testはテスト対象日時で検索" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_true
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 5
      end
    end

    it "現在日時<公開開始日時の商品も取得　ec_adminは公開日に関わらず取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 6
      end
    end

    it "公開終了日時<現在日時の商品も取得　ec_adminは公開日に関わらず取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 6
      end
    end

    it "ec_frontでは非表示フラグがtrueの商品は外す" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 4
      end
    end

    it "ec_testでは非表示フラグがtrueの商品も取得" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_true
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 5
      end
    end

    it "ec_adminでは非表示フラグがtrueの商品も取得" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      Time.stub!(:now).and_return(Time.local(2009,11,18, 11,56,29))
      EcRider.config.ec_test.should be_false
      ec.condition_products(:category_codes => "code20") do
        products = @template.instance_variable_get(:@products)
        products.size.should == 6
      end
    end
  end

  describe "ec:product_category_name_X" do
    before(:each) do
      Product.stub(:find).and_return(
        mock_model(Product, {:category_code_1_name => "カテゴリ1", :category_code_2_name => "カテゴリ2", :category_code_3_name => "カテゴリ3"})
      )
    end

    it "カテゴリ名が表示されること" do
      (1..3).each do |i|
        eval("ec.product_category_name_#{i}.should == 'カテゴリ#{i}'")
      end
    end
  end

  describe "ec:product_variation_name_X" do
    before(:each) do
      Product.stub(:find).and_return(
        mock_model(Product, {:variation_1_name => "バリエーション1", :variation_2_name => "バリエーション2", :variation_3_name => "バリエーション3"})
      )
    end

    it "バリエーション名が表示されること" do
      (1..3).each do |i|
        eval("ec.product_variation_name_#{i}.should == 'バリエーション#{i}'")
      end
    end
  end

  describe "ec:product_orderable_count" do
    before(:each) do
      ProductSku.stub(:find).and_return(
        mock_model(ProductSku, {:orderable_count => 100})
      )
    end

    it "在庫数が表示されること" do
      ec.product_orderable_count.should == 100
    end
  end

  describe "ec:product_sku_code" do
    before(:each) do
      ProductSku.stub(:find).and_return(
        mock_model(ProductSku, {:sku_code => "00000000000001-S-00-00"})
      )
    end

    it "SKUコードが表示されること" do
      ec.product_sku_code.should == "00000000000001-S-00-00"
    end
  end

  describe "ec:product_standard_price" do
    before(:each) do
      UserRank.stub(:find).and_return(
        mock_model(UserRank, {:user_rank_id => 1})
      )
      ProductPrice.stub(:find).and_return(
        mock_model(ProductPrice, {:standard_price => 1500})
      )
    end

    it "商品価格が表示されること" do
      ec.product_standard_price(:pre_currency => "", :post_currency => "").should == "1,500"
      ec.product_standard_price(:post_currency => "円").should == "1,500円"
      ec.product_standard_price.should == "1,500"
    end
  end

  describe "ec:product_regular_purchase_price" do
    before(:each) do
      UserRank.stub(:find).and_return(
        mock_model(UserRank, {:user_rank_id => 1})
      )
      ProductPrice.stub(:find).and_return(
        mock_model(ProductPrice, {:regular_purchase_price => 1200})
      )
    end

    it "定期購入価格が表示されること" do
      ec.product_regular_purchase_price(:pre_currency => "", :post_currency => "").should == "1,200"
      ec.product_regular_purchase_price(:pre_currency => "￥").should == "￥1,200"
      ec.product_regular_purchase_price.should == "1,200"
    end
  end

  describe "ec:product_lead_time" do
    before(:each) do
      ProductSku.stub(:find).and_return(
        mock_model(ProductSku, {:lead_time => 3})
      )
    end

    it "出荷可能日数が表示されること" do
      ec.product_lead_time.should == "3"
    end
  end

  describe "ec:product_maker_price" do
    before(:each) do
      ProductSku.stub(:find).and_return(
        mock_model(ProductSku, {:maker_price => 1500})
      )
    end

    it "メーカー希望小売価格が表示されること" do
      ec.product_maker_price(:pre_currency => "", :post_currency => "").should == "1,500"
      ec.product_maker_price(:post_currency => "円").should == "1,500円"
      ec.product_maker_price.should == "1,500"
    end
  end

  describe "ec:product_jan_code" do
    before(:each) do
      ProductSku.stub(:find).and_return(
        mock_model(ProductSku, {:jan_code => "T00001"})
      )
    end

    it "JANコードが表示されること" do
      ec.product_jan_code.should == "T00001"
    end
  end

  describe "each_product_images" do
    before(:each) do
      ec.stub(:instance_variable_get_with_code).and_return(nil)
      Product.stub(:find).and_return(mock_model(Product, {:images =>
              [
              mock_model(ProductImage, {:public_filename => "/test_1.gif"}),
              mock_model(ProductImage, {:public_filename => "/test_2.gif"}),
              mock_model(ProductImage, {:public_filename => "/test_3.gif"}),
              mock_model(ProductImage, {:public_filename => "/test_4.gif"}),
              mock_model(ProductImage, {:public_filename => "/test_5.gif"}),
            ],
            :product_name_1 => 'テスト商品'
          })
      )
    end

    it "商品画像の繰り返しタグが行われること、表示数の指定がない場合はすべて" do
      targets = []
      ec.each_product_images do
        one_row = []
        one_row << ec.product_image
        targets << one_row
      end
      targets.size.should == 5
      targets.each do |target|
        # refs http://d.hatena.ne.jp/hs9587/20080625/1214380280
        # src属性の値が「/test_」で始まる
        target[0].should have_tag('img[src^=/test_]')
      end
    end

    it "商品画像の繰り返しタグが、指定した表示数分だけ行われること" do
      targets = []
      ec.each_product_images(:display => 2) do
        one_row = []
        one_row << ec.product_image
        targets << one_row
      end
      targets.size.should == 2
    end
  end

  describe "ec:each_product_groups" do
    fixtures :products, :product_displays, :product_skus, :product_groups, :product_group_skus, :product_prices

    it "関連商品の繰り返しタグが行われること" do
      target = []
      ec.each_product_groups(:all => "all") do
        target << ec.product_sku_code
      end
      target.should == ["0000000000001-00-00-00", "0000000000002-00-00-00", "0000000000004-00-00-00", "0000000000003-00-00-00"]
    end
    
    it "関連商品グループ指定" do
      target = []
      ec.each_product_groups(:all => "all", :group_name => 'group_2') do
        target << ec.product_sku_code
      end
      target.should == ["0000000000001-00-00-00", "0000000000002-00-00-00", "0000000000003-00-00-00"]
    end
    
    it "商品指定" do
      target = []
      ec.each_product_groups(:product_code => '0000000000001-00-00-00') do
        target << ec.product_sku_code
      end
      target.should == ["0000000000002-00-00-00", "0000000000004-00-00-00", "0000000000003-00-00-00"]
    end

    it "関連商品の繰り返しタグが、指定した表示数分だけ行われること" do
      target = []
      ec.each_product_groups(:all => "all", :display => 2) do
        target << ec.product_sku_code
      end
      target.should == ["0000000000001-00-00-00", "0000000000002-00-00-00"]
    end
    
    it "関連商品情報を表示できること" do
      targets = []
      ec.each_product_groups(:all => "all") do
        one_row = []
        one_row << ec.product_group_product_name
        one_row << ec.product_group_price
        targets << one_row
      end
      targets.size.should == 4
      sku_1 = targets[0]
      sku_1[0].should == "商品1の商品名1"
      sku_1[1].should == "8,000"
    end

    it "関連商品価格_会員" do
      helper.instance_variable_set(:@product_group_sku, ProductSku.find(313))
      ec.product_group_price.should == "16,800"
    end
    
    it "関連商品メーカー希望小売価格" do
      product_sku = ProductSku.find(282)
      product_sku.maker_price = 3500
      product_sku.save
      
      helper.instance_variable_set(:@product_group_sku, product_sku)
      ec.product_group_maker_price.should == "3,500"
    end
    
    it "関連商品画像" do
      helper.instance_variable_set(:@product_grou_sku, ProductSku.find(282))
      ec.product_group_image_link.should have_tag('a[href="/show?product_code=0000000000001"]') do
        have_tag('img[src*=/product00001.gif][alt="商品1の商品名1"]')
      end
    end
  end

  describe "ec:each_product_components" do
    before(:each) do
      ProductSku.stub(:find).and_return(mock_model(ProductSku, {:components =>
              [
              mock_model(ProductComponent, {:component_sku_id => 1, :count => 10}),
              mock_model(ProductComponent, {:component_sku_id => 2, :count => 15}),
              mock_model(ProductComponent, {:component_sku_id => 3, :count => 5})
            ]
          })
      )

      ProductSku.stub(:find).with(1).and_return(mock_model(ProductSku, {:product_name_1 => "商品SKU1"}))
      ProductSku.stub(:find).with(2).and_return(mock_model(ProductSku, {:product_name_1 => "商品SKU2"}))
      ProductSku.stub(:find).with(3).and_return(mock_model(ProductSku, {:product_name_1 => "商品SKU3"}))
    end

    it "関連構成の繰り返しタグが行われること" do
      targets = []
      ec.each_product_components do
        one_row = []
        one_row << ec.product_component_name
        one_row << ec.product_component_count
        targets << one_row
      end
      targets.size.should == 3
      targets.each do |target|
        ["商品SKU1", "商品SKU2", "商品SKU3"].should be_include(target[0])
        [5, 10, 15].should be_include(target[1])
      end
    end
  end

  describe "ec:product_variation_item_select_X" do
    it "商品バリエーションのセレクトボックスを表示します。" do
      ec.product_variation_item_select_1(:product_code => "0000000000002").should have_tag("select") do
        with_tag("option", "S")
      end
      ec.product_variation_item_select_2(:product_code => "0000000000002").should have_tag("select") do
        with_tag("option", "赤")
      end
    end

    it "商品バリエーションが未設定の場合は、hidden_fieldを出力します。" do
      ec.product_variation_item_select_3.should have_tag("input[id='variation_3_item'][type='hidden'][value='00']")
    end
  end

  describe "campaign" do
    before(:each) do
      @user = users(:users_00010) # 会員二十三
      @template.stub(:current_user).and_return(@user)
      @template.stub(:current_user_id).and_return(@user.id)
    end

    it "display_qtyでちゃんと制限されるかどうかのテスト" do
      Time.stub!(:now).and_return(Time.parse("2009/01/01 12:00:00"))
      result_price = ec.product_campaign_price(:product_code => "0000000009050-00-00-00")
      result_price.should == "11,340"
      ec.product_available_campaign_each(:product_code => "0000000009050-00-00-00", :display_qty => 0) do
        flunk("ここが呼び出されたら:disply_qtyが効いていないってことなのでNG")
      end
    end

    it "キャンペーンが見つからなかった場合は何も出力されません" do
      Time.stub!(:now).and_return(Time.parse("1999/01/01 12:00:00"))
      result_price = ec.product_campaign_price(:product_code => "0000000009050-00-00-00")
      result_price.should == "12,600"
      ec.product_available_campaign_each(:product_code => "0000000009050-00-00-00") do
        flunk("ここが呼び出されたらキャンペーンが見つかってしまっているのでNG")
      end
    end

    it "商品に対して条件タイプ２のキャンペーンが出力される" do
      Time.stub!(:now).and_return(Time.parse("2009/01/01 12:00:00"))
      result_price = ec.product_campaign_price(:product_code => "0000000009050-00-00-00")
      result_price.should == "11,340"
      result = []
      ec.product_available_campaign_each(:product_code => "0000000009050-00-00-00") do
        ec.product_campaign_page_link do
          result << ec.product_available_campaign_name
          ec.product_available_campaign_name
        end
        @template.output_buffer
      end
      result.should == ["T0001A"]
    end

    describe "#1131 商品に対して条件タイプ１のキャンペーンも出力される" do
      it "T0006 商品A" do
        Time.stub!(:now).and_return(Time.parse("2009/06/01 12:00:00"))
        result_price = ec.product_campaign_price(:product_code => "0000000009010-00-00-00") # 商品A インテリア家具
        result_price.should == "7,700"
        result = []
        ec.product_available_campaign_each(:product_code => "0000000009010-00-00-00") do
          ec.product_campaign_page_link do
            result << ec.product_available_campaign_name
            ec.product_available_campaign_name
          end
          @template.output_buffer
        end
        result.should == ["T0006A"]
      end


      describe "T0008 商品A" do
        #     キャンペーンC    １２０００円以上で化粧ポーチプレゼント（条件タイプ：２）、優先度20
        #     キャンペーンD    １００００円以上で送料無料（条件タイプ：２）、優先度80
        # なので、合計金額が変わるように単価を変更してテストしています。
        patterns = [
          {:name => "キャンペーンC,Dの対象外", :price =>  7800, :campaign_price =>  "7,020", :applied_campaigns => ["T0008F"]},
          {:name => "キャンペーンC,Dの対象外", :price => 11000, :campaign_price =>  "9,900", :applied_campaigns => ["T0008F"]},
          {:name => "キャンペーンDのみ対象",   :price => 12000, :campaign_price => "10,800", :applied_campaigns => ["T0008F", "T0008D"]},
          {:name => "キャンペーンC,Dが対象",   :price => 15000, :campaign_price => "13,500", :applied_campaigns => ["T0008F", "T0008D", "T0008C"]},
        ]

        patterns.each do |pattern|
          it pattern[:name] do
            Time.stub!(:now).and_return(Time.parse("2009/08/01 12:00:00"))
            product_sku = ProductSku.find_by_sku_code("0000000009010-00-00-00")
            product_price = ProductPrice.find(:first, :conditions => ["product_sku_id = ? AND user_rank_id = ?", 
              product_sku.id, @user.user_rank_id])
            product_price.standard_price = pattern[:price]
            product_price.save!
            result_price = ec.product_campaign_price(:product_code => product_sku.sku_code) # 商品A インテリア家具
            result_price.should == pattern[:campaign_price]
            result = []
            ec.product_available_campaign_each(:product_code => product_sku.sku_code) do
              ec.product_campaign_page_link do
                result << ec.product_available_campaign_name
                ec.product_available_campaign_name
              end
              @template.output_buffer
            end
            result.should == pattern[:applied_campaigns]
          end
        end

      end

      it "T0008 商品C" do
        Time.stub!(:now).and_return(Time.parse("2009/08/01 12:00:00"))
        result_price = ec.product_campaign_price(:product_code => "0000000009040-00-00-00") # 商品A インテリア家具
        result_price.should == "9,450"
        result = []
        ec.product_available_campaign_each(:product_code => "0000000009040-00-00-00") do
          ec.product_campaign_page_link do
            result << ec.product_available_campaign_name
            ec.product_available_campaign_name
          end
          @template.output_buffer
        end
        result.should == ["T0008B", "T0008E"] # 合計金額が10,000円未満なので、CとDは適用されません。
      end
    end

  end

  describe "商品Q&AECタグテスト" do
    describe "if_product_faq_exists" do
      it "日本　商品Q&Aがない場合" do
        I18n.stub!(:locale).and_return('ja')
        ec.if_product_faq_exists(:product_code =>"00000001") do
          @template.concat("OK")
        end
        @template.output_buffer.should_not == "OK"
      end
      it "日本　商品Q&Aがある場合" do
        I18n.stub!(:locale).and_return('ja')
        ec.if_product_faq_exists(:product_code =>"0000000009010") do
          @template.concat("OK")
        end
        @template.output_buffer.should == "OK"
      end
      it "繁体字　商品Q&Aがない場合" do
        I18n.stub!(:locale).and_return('zh_TW')
        ec.if_product_faq_exists(:product_code =>"00000001") do
          @template.concat("OK")
        end
        @template.output_buffer.should_not == "OK"
      end
      it "繁体字　商品Q&Aがある場合" do
        I18n.stub!(:locale).and_return('zh_TW')
        ec.if_product_faq_exists(:product_code =>"0000000009010") do
          @template.concat("OK")
        end
        @template.output_buffer.should == "OK"
      end
    end
    describe "each_product_faqs" do
      it "表示件数＞商品Q&A件数の場合　商品Q&A件数まで繰り返し実行されること" do
        I18n.stub!(:locale).and_return('ja')
        result = 0
        ec.each_product_faqs(:product_code =>"0000000009010", :display_qty => "4") do
          result += 1
          ""
        end
        result.should == 3
      end
      it "表示件数＜商品Q&A件数の場合　表示件数まで繰り返し実行されること" do
        I18n.stub!(:locale).and_return('ja')
        result = 0
        ec.each_product_faqs(:product_code =>"0000000009010", :display_qty => "2") do
          result += 1
          ""
        end
        result.should == 2
      end
      it "表示件数を設定しないとき　商品Q&A件数まで繰り返し実行されること" do
        I18n.stub!(:locale).and_return('ja')
        result = 0
        ec.each_product_faqs(:product_code =>"0000000009010") do
          result += 1
          ""
        end
        result.should == 3
      end
      it "表示順 日本語" do
        I18n.stub!(:locale).and_return('ja')
        inquiry_users = []
        inquiries = []
        answer_users = []
        answers = []
        answer_dates = []

        ec.each_product_faqs(:product_code =>"0000000009010") do
          inquiry_users << ec.product_faq_inquiry_user
          inquiries << ec.product_faq_inquiry
          answer_users << ec.product_faq_answer_user
          answers << ec.product_faq_answer
          answer_dates << ec.product_faq_answer_date
          ""
        end
        inquiry_users[0].should == "会社員Aさん_日本語"
        inquiry_users[1].should == "質問"
        inquiry_users[2].should == "会社員Bさん_日本語"
        inquiries[0].should == "【日本語】\\r\\nお問合せです。\\r\\nお問合せです。\\r\\nお問合せです。\\r\\n"
        answer_users[0].should == "武井_日本語"
        answers[0].should == "【日本語】\\r\\n回答します。\\r\\n回答します。\\r\\n回答します。\\r\\n"
        answer_dates[0].should == "09/11/27"
      end

      it "表示順 繁体字" do
        I18n.stub!(:locale).and_return('zh_TW')
        result = 0
        inquiry_users = []
        inquiries = []
        answer_users = []
        answers = []
        answer_dates = []

        ec.each_product_faqs(:product_code =>"0000000009010") do
          inquiry_users << ec.product_faq_inquiry_user
          inquiries << ec.product_faq_inquiry
          answer_users << ec.product_faq_answer_user
          answers << ec.product_faq_answer
          answer_dates << ec.product_faq_answer_date
          result += 1
          ""
        end
        result.should == 1
        inquiry_users[0].should == "会社員Aさん_繁体字"
        inquiries[0].should == "【繁体字】\\r\\nお問合せです。\\r\\nお問合せです。\\r\\nお問合せです。\\r\\n"
        answer_users[0].should == "武井_繁体字"
        answers[0].should == "【繁体字】\\r\\n回答します。\\r\\n回答します。\\r\\n回答します。\\r\\n"
        answer_dates[0].should == "09/11/27"
      end
    end
  end

  describe "コード付きDIVタグテスト" do
    fixtures :products, :product_skus
    it "div_with_product_code" do
      params[:product_code] = '0000000009051'
      ec.div_with_product_code(:class=>"styleA") do
        @template.output_buffer
      end
      @template.output_buffer.should have_tag( "div[class=?]", '0000000009051 styleA')
    end

    it "div_with_category_code" do
      params[:category_code] = '1000'
      ec.div_with_category_code(:class=>"styleB") do
        @template.output_buffer
      end
      @template.output_buffer.should have_tag( "div[class=?]", '1000 styleB')
    end

    it "div_with_sku_code" do
      params[:product_code] = '0000000009051'
      ec.div_with_sku_code(:class=>"styleC") do
        @template.output_buffer
      end
      @template.output_buffer.should have_tag( "div[class=?]", '0000000009051-00-00-00 styleC')
    end
  end
end
