# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  # fixtures :products, :product_displays, :name_masters, :ms_code_names, :ms_shop_settings, :users

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:user_login_form" do
    it "normal usage" do
      @template.instance_variable_set(:@user_login_form, UserLoginForm.new)
      @template.extend(ApplicationHelper)
      ec.user_login_form do
        concat(ec.user_login_form_error_messages || '')
        concat(ec.user_login_form_email)
        concat(ec.user_login_form_password)
        concat(ec.user_login_form_remember_me)
        ec.button{ "戻る"} # ネストするecタグは自動でconcatします。
        ec.submit{"ログインする"} 
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/authenticate") do
        with_tag("input#user_login_form_email")
        with_tag("input#user_login_form_password")
        with_tag("input#remember_me[type=?]", 'checkbox')
        with_tag("input[type=?][value=?]", 'button', '戻る')
        with_tag("input[type=?][value=?]", 'submit', 'ログインする')
      end
    end
  end
  
  describe "ec:add_mailaddress_form" do
    it "normal usage" do
      user_email = UserEmail.new(:email => 'aaa')
      user_email.valid?
      @template.instance_variable_set(:@user_email, user_email)
      @template.instance_variable_set(:@back_to_url, {:controller => '/account', :action => 'index'})
      @template.extend(ApplicationHelper)
      @template.should_receive(:template_error_messages_for).and_return('')
      ec.add_mailaddress_form do
        concat(ec.add_mailaddress_form_error_messages)
        concat(ec.add_mailaddress_form_email)
        ec.add_mailaddress_form_button_to_back{"戻る"} # ネストするecタグは自動でconcatします。
        ec.submit{"進む"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/confirm_mailaddress") do
        with_tag("input#user_email_email")
        with_tag("input[type=?][value=?][onclick=?]", 'button', '戻る', "location.href='/account';")
        with_tag("input[type=?][value=?]", 'submit', '進む')
      end
    end

    it "if_reminder_to_add_mailaddress true" do
      @template.instance_variable_set(:@from_reminder, true)
      result = true
      ec.if_reminder_to_add_mailaddress do
        result = false
      end
      result.should be_false
    end

    it "if_reminder_to_add_mailaddress false" do
      @template.instance_variable_set(:@from_reminder, nil)
      result = true
      ec.if_reminder_to_add_mailaddress do
        result = false
      end
      result.should be_true
    end
  end

  describe "ec:add_mailaddress_confirmation_form" do
    it "normal usage" do
      user_email = UserEmail.new(:email => 'aaa@ecrider.com')
      user_email.valid?
      @template.instance_variable_set(:@user_email, user_email)
      @template.instance_variable_set(:@back_to_url, {:controller => '/account', :action => 'index'})
      @template.extend(ApplicationHelper)
      ec.add_mailaddress_confirmation_form do
        concat(ec.add_mailaddress_confirmation_form_email)
        ec.button(:href => '/account/add_mailaddress'){"戻る"} # ネストするecタグは自動でconcatします。
        ec.submit{"進む"}
        @template.output_buffer
      end
      @template.output_buffer.should have_tag('form[action=?]', "/account/update_mailaddress") do
        with_tag("input[type=?][value=?][onclick=?]", 'button', '戻る', "location.href='/account/add_mailaddress';")
        with_tag("input[type=?][value=?]", 'submit', '進む')
      end
      @template.output_buffer.should =~ /aaa@ecrider\.com/
    end
  end

end
