# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  # fixtures :products, :product_displays, :name_masters, :ms_code_names, :ms_shop_settings, :users

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @template.extend(ApplicationHelper)
  end

  describe "tags which works with current_user" do
    describe "ec:if_logged_in" do
      it "if_logged_in true" do
        @template.stub!(:logged_in?).and_return(true)
        invoked = false
        ec.if_logged_in{ invoked = true }
        invoked.should be_true
      end
      
      it "if_logged_in false" do
        @template.stub!(:logged_in?).and_return(false)
        invoked = false
        ec.if_logged_in{ invoked = true }
        invoked.should be_false
      end

      it "if_not_logged_in false" do
        @template.stub!(:logged_in?).and_return(true)
        invoked = false
        ec.if_not_logged_in{ invoked = true }
        invoked.should be_false
      end

      it "if_not_logged_in true" do
        @template.stub!(:logged_in?).and_return(false)
        invoked = false
        ec.if_not_logged_in{ invoked = true }
        invoked.should be_true
      end
    end

    describe "display current_user" do
      before do
        current_user = User.new(
          :name_last => "漢字名（姓）",
          :name_first => "漢字名（名）",
          :kana_name_last => "カナメイ（セイ）",
          :kana_name_first => "カナメイ（メイ）",
          :sex => "1"
        )
        user_address = UserAddress.new(
          :locale => 'ja',
          :country_code => 'JP',
          :name_last => "漢字名（姓）",
          :name_first => "漢字名（名）"
        )
        user_rank = UserRank.new(
          :name => 'ゴールド会員',
          :code => '111'
        )
        user_rank.displays << UserRankDisplay.new(:name => "ゴールド会員(日本語)",:locale => 'ja')
        user_rank.displays << UserRankDisplay.new(:name => "ゴールド会員(英語)",:locale => 'en')
        current_user.user_address = user_address
        current_user.rank = user_rank
        @template.stub!(:current_user).and_return(current_user)
      end
      
      it '<%= ec:name_last %>' do
        ec.name_last.should == "漢字名（姓）"
      end

      it '<%= ec:kanji_family_name %>' do
        ec.kanji_family_name.should == "漢字名（姓）"
      end

      it '<%= ec:name_first %>' do
        ec.kanji_name.should == "漢字名（名）"
      end

      it '<%= ec:kanji_name %>' do
        ec.kanji_name.should == "漢字名（名）"
      end

      it '<%= ec:kana_name_last %>' do
        ec.kana_family_name.should == "カナメイ（セイ）"
      end

      it '<%= ec:kana_family_name %>' do
        ec.kana_family_name.should == "カナメイ（セイ）"
      end

      it '<%= ec:kana_name_first %>' do
        ec.kana_name.should == "カナメイ（メイ）"
      end

      it '<%= ec:kana_name %>' do
        ec.kana_name.should == "カナメイ（メイ）"
      end

      it '<%= ec:full_name %>' do
        ec.full_name.should == "漢字名（姓） 漢字名（名）"
      end

      it '<%= ec:honorific_prefix %>' do
        ec.honorific_prefix.should == "Mr."
      end

      it '<%= ec:user_rank %> locale for ja' do
        I18n.stub!(:locale).and_return("ja")
        ec.user_rank.should == "ゴールド会員(日本語)"
      end

      it '<%= ec:user_rank %> locale for en' do
        I18n.stub!(:locale).and_return("en")
        ec.user_rank.should == "ゴールド会員(英語)"
      end

      it '<%= ec:user_rank %> locale for th' do
        I18n.stub!(:locale).and_return("th")
        ec.user_rank.should be_nil
      end
    end
  end


  describe '<%= ec:member_registration_link %>' do
    it "should work without options" do
      ec.member_registration_link.should have_tag('a')
      ec.member_registration_link.should_not have_tag('img')
    end

    it "should work with src" do
      ec.member_registration_link(:src => "aaa.gif").should have_tag('a') do
        with_tag('img[src^=/images/aaa.gif]')
      end
    end

    it "should work with text" do
      link = ec.member_registration_link(:text => "リンク")
      link.should have_tag('a')
      link.should include("リンク")
      link.should_not have_tag('img')
    end

    it "should work with block" do
      ec.member_registration_link(:text => "textは無視される"){"使われる文字列"}
      link = @template.output_buffer
      link.should have_tag('a')
      link.should include("使われる文字列")
      link.should_not include("textは無視される")
    end
  end

  it '<%= ec:login_link %>' do
    ec.login_link(:src => "aaa.gif").should have_tag('a') do
      with_tag('img[src^=/images/aaa.gif]')
    end

    ec.login_link(:text => "リンク").should have_tag('a')
    ec.login_link(:text => "リンク").should_not have_tag('img')

    ec.login_link.should have_tag('a')
    ec.login_link.should_not have_tag('img')
  end

  it '<%= ec:logout_link %>' do
    ec.logout_link(:src => "aaa.gif").should have_tag('a') do
      with_tag('img[src^=/images/aaa.gif]')
    end

    ec.logout_link(:text => "リンク").should have_tag('a')
    ec.logout_link(:text => "リンク").should_not have_tag('img')

    ec.logout_link.should have_tag('a')
    ec.logout_link.should_not have_tag('img')
  end

  describe '<%= ec:link_to_reminder %>' do
    it "should not work without options" do
      ec.link_to_reminder.should be_nil
    end

    it "should work with text" do
      link = ec.link_to_reminder(:text => "リンク")
      link.should have_tag('a')
      link.should include("リンク")
    end

    it "should work with block" do
      ec.link_to_reminder(:text => "textは表示されない"){"使われる文字列"}
      link = @template.output_buffer
      link.should have_tag('a')
      link.should include("使われる文字列")
      link.should_not include("textは表示されない")
    end

    it "should work with block" do
      @template.stub!(:send).with(:params).and_return({:controller => 'cart', :action => 'index'})
      ec.link_to_reminder(:text => "text")
      @template.output_buffer.should == '<a href="/reminder?back_to_url%5Baction%5D=index&amp;back_to_url%5Bcontroller%5D=cart">text</a>'
    end
  end

end
