# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe EcRider::JsValidationHelper, :type => :helper do
  include ApplicationHelper

  class TestModel
    attr_accessor :foo
  end


  describe "text_field_valid" do
    describe "return same text as text_field" do
      before(:each) do
        test_model = TestModel.new
        test_model.foo = "FOO"
        @template.instance_variable_set(:@test_model, test_model)
      end

      it "without no options and no validate options" do
        text_field_valid(:test_model, :foo).should == text_field(:test_model, :foo)
      end

      it "without no validate options" do
        text_field_valid(:test_model, :foo, :size => 50).should == text_field(:test_model, :foo, :size => 50)
      end

      it "with empty validate options" do
        text_field_valid(:test_model, :foo, {:size => 50}, {}).should == text_field(:test_model, :foo, :size => 50)
      end
    end

    describe "normal usage" do
      describe "with empty option" do
        it "with realtime option" do
          validation_option = {:empty => true, :realtime => true, :update => 'name_msg', :error => I18n.t(:"activerecord.attributes.user_address.name_last")}
          text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == [
            '<input id="test_model_foo" name="test_model[foo]" size="50" type="text" onblur="', 
            "if((($F('test_model_foo') != '') || error_msg('name_msg','お名前（姓）を入力してください。'))){", 
              "Element.addClassName($('test_model_foo'),'default');Element.removeClassName($('test_model_foo'),'error');$('name_msg').innerHTML = '';return true;", 
            "}else{", 
            "Element.addClassName($('test_model_foo'),'error');Element.removeClassName($('test_model_foo'),'default');return false;}", '" />', 
            '<span id="validate_test_model_foo" ></span>'].join
        end

        # realtimeオプション要らないじゃん！！
        it "without realtime option" do
          validation_option = {:empty => true, :update => 'name_msg', :error => I18n.t(:"activerecord.attributes.user_address.name_last")}
          text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == [
            '<input id="test_model_foo" name="test_model[foo]" size="50" type="text" onblur="',
            "if((($F('test_model_foo') != '') || error_msg('name_msg','お名前（姓）を入力してください。'))){",
              "Element.addClassName($('test_model_foo'),'default');Element.removeClassName($('test_model_foo'),'error');$('name_msg').innerHTML = '';return true;",
            "}else{",
              "Element.addClassName($('test_model_foo'),'error');Element.removeClassName($('test_model_foo'),'default');return false;}",
            '" />',
            '<span id="validate_test_model_foo" ></span>'].join
        end

        # alphaオプションは使われているんだけど、指定しない場合と結果は同じらしい？
        it "with alpha option" do
          validation_option = {:empty => true, :alpha => true, :update => 'name_msg', :error => I18n.t(:"activerecord.attributes.user_address.name_last")}
          text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == 
            text_field_valid(:test_model, :foo, {:size => 50}, {
              :empty => true, :update => 'name_msg', :error => I18n.t(:"activerecord.attributes.user_address.name_last")})
          text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == [
            '<input id="test_model_foo" name="test_model[foo]" size="50" type="text" onblur="',
            "if((($F('test_model_foo') != '') || error_msg('name_msg','お名前（姓）を入力してください。'))){",
              "Element.addClassName($('test_model_foo'),'default');Element.removeClassName($('test_model_foo'),'error');$('name_msg').innerHTML = '';return true;",
            "}else{",
              "Element.addClassName($('test_model_foo'),'error');Element.removeClassName($('test_model_foo'),'default');return false;}",
            '" />',
            '<span id="validate_test_model_foo" ></span>'].join
        end

      end
      
      describe "with numeric option" do
        it "without min option" do
          validation_option = {:numeric => true, :update => 'zip_code_msg', :error =>I18n.t(:"activerecord.attributes.user_address.zip_code")}
          text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == [
            '<input id="test_model_foo" name="test_model[foo]" size="50" type="text" onblur="',
            "if((validate.isNum($F('test_model_foo')) || error_msg('zip_code_msg','郵便番号は数値で入力してください。'))){",
              "Element.addClassName($('test_model_foo'),'default');Element.removeClassName($('test_model_foo'),'error');$('zip_code_msg').innerHTML = '';return true;",
            "}else{",
              "Element.addClassName($('test_model_foo'),'error');Element.removeClassName($('test_model_foo'),'default');return false;}", '" />',
            '<span id="validate_test_model_foo" ></span>'
          ].join
        end

        it "with min option" do
          validation_option = {:numeric => true, :min => 4, :update => 'zip_code_msg', :error =>I18n.t(:"activerecord.attributes.user_address.zip_code")}
          text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == [
            '<input id="test_model_foo" name="test_model[foo]" size="50" type="text" onblur="',
            "if((validate.isNum($F('test_model_foo')) || error_msg('zip_code_msg','郵便番号は数値で入力してください。'))", 
              " && (($F('test_model_foo').length >= 4) || error_msg('zip_code_msg','郵便番号は短すぎます。'))){",
              "Element.addClassName($('test_model_foo'),'default');Element.removeClassName($('test_model_foo'),'error');$('zip_code_msg').innerHTML = '';return true;",
            "}else{",
              "Element.addClassName($('test_model_foo'),'error');Element.removeClassName($('test_model_foo'),'default');return false;}", '" />',
            '<span id="validate_test_model_foo" ></span>'
          ].join
        end
      end

      it "with email option" do
        validation_option = {:empty => true, :email => true, :update => 'email_msg', :error => I18n.t(:"activerecord.attributes.user_email.email")}
        text_field_valid(:test_model, :foo, {:size => 50}, validation_option).should == [
          '<input id="test_model_foo" name="test_model[foo]" size="50" type="text" onblur="', 
          "if(", 
            "(($F('test_model_foo') != '') || error_msg('email_msg','メールアドレスを入力してください。')) && ",
             "(validate.isEMailAddr($F('test_model_foo')) || error_msg('email_msg','メールアドレスは正しくありません。'))){", 
            "Element.addClassName($('test_model_foo'),'default');Element.removeClassName($('test_model_foo'),'error');$('email_msg').innerHTML = '';return true;",
          "}else{",
            "Element.addClassName($('test_model_foo'),'error');Element.removeClassName($('test_model_foo'),'default');return false;}", '" />',
          '<span id="validate_test_model_foo" ></span>'
        ].join
      end
    end
  end

  describe "generate_function_generate_error_message" do
    describe "should return error message in Japanese" do
      it "for :empty" do
        generete_error_message = generate_function_generate_error_message("name", "名前")
        generete_error_message.call(:empty).should == " || error_msg('name','名前を入力してください。')"
      end

      it "should return for :numeric" do
        generete_error_message = generate_function_generate_error_message("zip_code", "郵便番号")
        generete_error_message.call(:numeric).should == " || error_msg('zip_code','郵便番号は数値で入力してください。')"
      end

      it "should return for :alpha" do
        generete_error_message = generate_function_generate_error_message("cardholder_name", "カード名義")
        generete_error_message.call(:alpha).should == " || error_msg('cardholder_name','カード名義はアルファベットで入力してください。')"
      end

      it "should return for :onebyte" do
        generete_error_message = generate_function_generate_error_message("tel", "電話番号")
        generete_error_message.call(:onebyte).should == " || error_msg('tel','電話番号は半角で入力してください。')"
      end

      it "should return for :towbyte" do
        generete_error_message = generate_function_generate_error_message("address", "住所")
        generete_error_message.call(:towbyte).should == " || error_msg('address','住所は全角で入力してください。')"
      end

      it "should return for :max" do
        generete_error_message = generate_function_generate_error_message("address", "住所")
        generete_error_message.call(:max).should == " || error_msg('address','住所は長すぎます。')"
      end

      it "should return for :min" do
        generete_error_message = generate_function_generate_error_message("tel", "電話番号")
        generete_error_message.call(:min).should == " || error_msg('tel','電話番号は短すぎます。')"
      end

      it "should return for :email" do
        generete_error_message = generate_function_generate_error_message("email", "メールアドレス")
        generete_error_message.call(:email).should == " || error_msg('email','メールアドレスは正しくありません。')"
      end

      it "should return for nil" do
        generete_error_message = generate_function_generate_error_message(nil, nil)
        generete_error_message.call(nil).should == ""
      end

      it "should return for message is nil" do
        generete_error_message = generate_function_generate_error_message("test", nil)
        generete_error_message.call(nil).should == ""
      end
    end
  end
  
end
