# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../../spec_helper')

describe EcRider::ProductCategoriesHelper do

  before(:each) do
    ProductCategory.destroy_all
    # childrenは:dependent => :destroyが設定されているので、子要素は全て削除されます。
    ProductCategory.count.should == 0
    build_test_tree
    @product_categories = ProductCategory.find(:all, :conditions => ["parent_id IS NULL"], :order => :display_order)
  end

  def build_test_tree
    fashion = ProductCategory.create!(:display_order => 1, :name_for_admin => 'ファッション', :url => "product_list", :code => "fashion")
    fashion.save_displays(:ja => 'ファッション', :en => 'fashion')
    fashion.children.create(:display_order => 1, :name_for_admin => 'トップス', :url => "product_list", :code => "tops").save_displays(:ja => 'トップス', :en => 'tops')
    fashion.children.create(:display_order => 2, :name_for_admin => 'ボトムス', :code => 'bottoms').save_displays(:ja => 'ボトムス', :en => 'bottoms')
    fashion.children.create(:display_order => 3, :name_for_admin => 'ゆかた', :code => 'yukata').save_displays(:ja => 'ゆかた', :en => 'yukata')
    fashion.children.create(:display_order => 4, :name_for_admin => 'バッグ', :code => 'bag').save_displays(:ja => 'バッグ', :en => 'bag')
    fashion.children.create(:display_order => 5, :name_for_admin => 'アクセサリー', :code => 'accessory').save_displays(:ja => 'アクセサリー', :en => 'accessory')
    fashion.children.create(:display_order => 6, :name_for_admin => 'ウィッグ', :code => 'wig').save_displays(:ja => 'ウィッグ', :en => 'wig')
    art = ProductCategory.create!(:display_order => 2, :name_for_admin => 'アート', :code => 'art')
    art.save_displays(:ja => 'アート', :en => 'art')
    art.children.create(:display_order => 1, :name_for_admin => 'フィギュア', :code => 'figure').save_displays(:ja => 'フィギュア', :en => 'figure')
    art.children.create(:display_order => 2, :name_for_admin => 'イラスト', :code => 'illust').save_displays(:ja => 'イラスト', :en => 'illustrations')
  end

  describe "カテゴリー階層を一行で表示する" do
    before(:each) do
      @category = ProductCategory.find_by_name_for_admin("トップス")
    end

    it "文字列で表示する" do
      html = helper.one_line_category_tree(@category)
      html.should == "ファッション > トップス"
    end

    it "リンクで表示する" do
      html = helper.one_line_category_tree(@category, {:link => true})
      links = html.split(/\s* > \s*/)
      if EcRider::Runtime.ec_admin?
        links[0].should have_tag('a', "ファッション")
        links[0].should have_tag('a[href=?]', "/cms/product_list?category=fashion")
        links[1].should have_tag('a', "トップス")
        links[1].should have_tag('a[href=?]', "/cms/product_list?category=tops")
      else
        links[0].should have_tag('a', "ファッション")
        links[0].should have_tag('a[href=?]', "/product_list?category=fashion")
        links[1].should have_tag('a', "トップス")
        links[1].should have_tag('a[href=?]', "/product_list?category=tops")
      end

    end

    it "区切り文字を変更する" do
      html = helper.one_line_category_tree(@category, {:separator => " + "})
      html.should == "ファッション + トップス"
    end
  end
end
