require File.join(File.dirname(__FILE__), '..', '..', 'spec_helper')

#require File.join(RAILS_ROOT, 'lib', 'ec_rider', 'view_cache_ctrl')
describe EcRider::ViewCacheCtrl do

  before(:each) do
    delete_dir
  end

  after(:all) do
    delete_dir
  end

  def delete_dir
    if File.exist?(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
      FileUtils.rm_rf(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
    end
  end

  # TODO テストモードでの、
  # キャッシュがクリアされたかどうかのテストの書き方が不明。
  it "view_cache_clearは、エラーが発生しない。" do
    EcRider::ViewCacheCtrl.view_cache_clear
  end
  
  it "remove_and_create_published_dateは、メソッドが呼ばれた時刻のファイルを作成する。" do
    FileUtils.mkdir_p(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
    FileUtils.touch(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR + "20090222123456")
    EcRider::ViewCacheCtrl.remove_and_create_published_date
    published_dates = Dir.glob(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR_REG)
    published_dates.should_not be_nil
    published_dates.size.should == 1
    published_dates.each do |date|
      date.should_not == "20090222123456"
    end
  end

  describe "published_checkは、" do
    it "EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIRに何も無い場合でも、" <<
      "EcRider::ViewCacheCtrl.before_cache_clear_timeには現在の日付が設定される。" do
      EcRider::ViewCacheCtrl.should_not_receive(:view_cache_clear)
      Time.stub!(:now).and_return(Time.parse("20100101000000"))
      FileUtils.mkdir_p(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
      EcRider::ViewCacheCtrl.before_cache_clear_time = nil
      EcRider::ViewCacheCtrl.published_check
      EcRider::ViewCacheCtrl.before_cache_clear_time.should == Time.parse("20100101000000")
    end
    it "EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIRに公開日付が入っており、" <<
      "published_checkが一度も呼ばれていない場合、キャッシュはクリアされず、" <<
      "EcRider::ViewCacheCtrl.before_cache_clear_timeに公開日付と同じ日付がセットされる。" do
      EcRider::ViewCacheCtrl.should_not_receive(:view_cache_clear)
      FileUtils.mkdir_p(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
      FileUtils.touch File.join(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR, "20101231123456")
      EcRider::ViewCacheCtrl.before_cache_clear_time = nil
      EcRider::ViewCacheCtrl.published_check
      EcRider::ViewCacheCtrl.before_cache_clear_time.should == Time.parse("20101231123456")
    end
    it "EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIRに公開日付が入っているが、" <<
      "以前の公開した日付から変わっていない場合、" <<
      "キャッシュはクリアされず、" <<
      "EcRider::ViewCacheCtrl.before_cache_clear_timeは同じ日付のままである。" do
      EcRider::ViewCacheCtrl.should_not_receive(:view_cache_clear)
      FileUtils.mkdir_p(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
      FileUtils.touch File.join(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR, "20101231123456")
      EcRider::ViewCacheCtrl.before_cache_clear_time = Time.parse("20101231123456")
      EcRider::ViewCacheCtrl.published_check
      EcRider::ViewCacheCtrl.before_cache_clear_time.should == Time.parse("20101231123456")
    end
    it "EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIRに公開日付が入っているが、" <<
      "以前の公開した日付より古い日付の場合、キャッシュはクリアされず、" <<
      "EcRider::ViewCacheCtrl.before_cache_clear_timeは同じ日付のままである。" do
      EcRider::ViewCacheCtrl.should_not_receive(:view_cache_clear)
      FileUtils.mkdir_p(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
      FileUtils.touch File.join(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR, "20090101000000")
      EcRider::ViewCacheCtrl.before_cache_clear_time = Time.parse("20101231123456")
      EcRider::ViewCacheCtrl.published_check
      EcRider::ViewCacheCtrl.before_cache_clear_time.should == Time.parse("20101231123456")
    end
    it "EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIRに公開日付が入っており、" <<
      "以前の公開した日付より新しい日付の場合、キャッシュをクリアし、" <<
      "EcRider::ViewCacheCtrl.before_cache_clear_timeは新しい日付になる。" do
      EcRider::ViewCacheCtrl.should_receive(:view_cache_clear).once.and_return(true)
      FileUtils.mkdir_p(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR)
      FileUtils.touch File.join(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR, "20101231123456")
      EcRider::ViewCacheCtrl.before_cache_clear_time = Time.parse("20090101000000")
      EcRider::ViewCacheCtrl.published_check
      EcRider::ViewCacheCtrl.before_cache_clear_time.should == Time.parse("20101231123456")
    end
  end

  it "create_tmp_dirは、EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIRを作成する。" do
    File.exist?(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR).should be_false
    EcRider::ViewCacheCtrl.create_tmp_dir
    File.exist?(EcRider::ViewCacheCtrl::PUBLISHED_DATE_DIR).should be_true
  end
end
