# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe BannerDisplay do
  before(:each) do
    @valid_attributes = {
      :banner_id => 1,
      :locale => "ja",
      :image_public_path => "path/to/banner/image",
      :image_alt => "value for image_alt",
      :text_title => "01",
      :text_description => "value for text_description"
    }
  end

  it "should create a new instance given valid attributes" do
    BannerDisplay.create!(@valid_attributes)
  end

  describe "useless?" do
    it "type_key = :text, text_descriptionがblankの場合" do
      banner_display = BannerDisplay.new(:text_description => "")
      banner_display.type_key = :text
      banner_display.useless?.should == true
    end
    it "type_key = :text, text_descriptionがblankではない場合" do
      banner_display = BannerDisplay.new(:text_description => "abc")
      banner_display.type_key = :text
      banner_display.useless?.should == false
    end

    it "type_key = :image, image_fileがblankの場合" do
      banner_display = BannerDisplay.new(:image_file => "")
      banner_display.type_key = :image
      banner_display.useless?.should == true
    end
    it "type_key = :text, image_fileがblankではない場合" do
      banner_display = BannerDisplay.new(:image_file => "abc")
      banner_display.type_key = :image
      banner_display.useless?.should == false
    end
  end

  describe "build_image_public_path" do
    fixtures :banners, :banner_displays

    before(:each) do
      @display = BannerDisplay.find(1)
      @display.banner.should_not be_nil
      @display.banner.id.should == 1
      @mock_file_io = mock(:file_io, :original_filename => "aaa.png")
      @display.image_file = @mock_file_io
    end
    
    it "with default locale ja" do
      I18n.default_locale.to_s.should == 'ja'
      @display.locale.to_s.should == "ja"
      @display.send(:build_image_public_path).should == "banner_images/banner_0000000001.png"
    end
    
    it "with default locale zh-TW" do
      I18n.default_locale.to_s.should == 'ja'
      @display.locale = "zh_TW"
      @display.send(:build_image_public_path).should == "banner_images/banner_0000000001_zh_TW.png"
    end
    
  end


end
