# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'
require 'pathname'

describe EcRider::CloudFront do
  fixtures :cms_files

  include EcRider::CloudFront

  describe "format_file_name" do
    describe ":add operation" do
      it "should insert timestamp to filepath" do
        format_file_name(:add, 'README', :timestamp => Time.local(2009, 3, 9, 12, 15, 29)).should == 'README-20090309121529'
        format_file_name(:add, 'logo.png', :timestamp => Time.local(2009, 3, 9, 12, 15, 29)).should == 'logo-20090309121529.png'
        format_file_name(:add, 'logo.png', :timestamp => '20090309121529').should == 'logo-20090309121529.png'
        format_file_name(:add, 'cms/top.html', :timestamp => '20090309121529').should == 'cms/top-20090309121529.html'
        format_file_name(:add, 'cms/top.backup.html', :timestamp => '20090309121529').should == 'cms/top.backup-20090309121529.html'
      end
    end

    describe ":cut operation" do
      it "should cut timestamp from filepath" do
        format_file_name(:cut, 'README-20090309121529').should == 'README'
        format_file_name(:cut, '404-20090309121529.html').should == '404.html'
        format_file_name(:cut, 'cms/top-20090309121529.html').should == 'cms/top.html'
        format_file_name(:cut, 'cms/top.backup-20090309121529.html').should == 'cms/top.backup.html'
      end

      it "should return file_path without timestamp" do
        format_file_name(:cut, 'README').should == 'README'
        format_file_name(:cut, '404.html').should == '404.html'
        format_file_name(:cut, 'cms/top.html').should == 'cms/top.html'
        format_file_name(:cut, 'cms/top.backup.html').should == 'cms/top.backup.html'
      end
    end

    describe ":find operation" do
      it "should insert search string for timestamp to filepath" do
        format_file_name(:find, 'README').should == 'README-??????????????'
        format_file_name(:find, '404.html').should == '404-??????????????.html'
        format_file_name(:find, 'cms/top.html').should == 'cms/top-??????????????.html'
        format_file_name(:find, 'cms/top.backup.html').should == 'cms/top.backup-??????????????.html'
      end
    end
  end

  describe "url_for_cloud_front" do
    before(:each) do
      EcRider.config.stub!(:cloud_front_url).and_return('http://0123456789.cloudfront.net')
      tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_url_for_cloud_front"))
      Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      FileUtils.mkdir_p(File.join(tmp_dir, 'cms'))
      FileUtils.mkdir_p(File.join(tmp_dir, 'javascripts'))
      @public_dir_for_cloud_front = tmp_dir
    end

    it "should return original file name if both original files and uploaded files exist. It's most usual case" do
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/404.html'), File.join(@public_dir_for_cloud_front, '404.html'))
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/404.html'), File.join(@public_dir_for_cloud_front, '404-20090309121529.html'))
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/javascripts/prototype.js'), File.join(@public_dir_for_cloud_front, 'javascripts/prototype.js'))
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/javascripts/prototype.js'), File.join(@public_dir_for_cloud_front, 'javascripts/prototype-20090309121529.js'))
      url_for_cloud_front('404.html').should == 'http://0123456789.cloudfront.net/404-20090309121529.html'
      url_for_cloud_front('/404.html').should == 'http://0123456789.cloudfront.net/404-20090309121529.html'
      url_for_cloud_front('javascripts/prototype.js').should == 'http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js'
      url_for_cloud_front('/javascripts/prototype.js').should == 'http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js'
    end

    it "should return original file name if original files don't exist but uploaded files exist. It's most unusual case" do
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/404.html'), File.join(@public_dir_for_cloud_front, '404-20090309121529.html'))
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/javascripts/prototype.js'), File.join(@public_dir_for_cloud_front, 'javascripts/prototype-20090309121529.js'))
      url_for_cloud_front('404.html').should == 'http://0123456789.cloudfront.net/404-20090309121529.html'
      url_for_cloud_front('/404.html').should == 'http://0123456789.cloudfront.net/404-20090309121529.html'
      url_for_cloud_front('javascripts/prototype.js').should == 'http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js'
      url_for_cloud_front('/javascripts/prototype.js').should == 'http://0123456789.cloudfront.net/javascripts/prototype-20090309121529.js'
    end

    it "should return original file name if uploaded files don't exist but orignal files exist." do
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/404.html'), File.join(@public_dir_for_cloud_front, '404.html'))
      FileUtils.cp(File.join(RAILS_ROOT, '../ec_admin/public/projects/1/public/javascripts/prototype.js'), File.join(@public_dir_for_cloud_front, 'javascripts'))
      url_for_cloud_front('404.html').should == '404.html'
      url_for_cloud_front('/404.html').should == '/404.html'
      url_for_cloud_front('javascripts/prototype.js').should == 'javascripts/prototype.js'
      url_for_cloud_front('/javascripts/prototype.js').should == '/javascripts/prototype.js'
    end

    it "should return original file name if neither original files nor uploaded files don't exist." do
      url_for_cloud_front('404.html').should == '404.html'
      url_for_cloud_front('/404.html').should == '/404.html'
      url_for_cloud_front('javascripts/prototype.js').should == 'javascripts/prototype.js'
      url_for_cloud_front('/javascripts/prototype.js').should == '/javascripts/prototype.js'
    end

    it "should return original file name if uploaded files don't exist but orignal files exist." do
#      FileUtils.cp_r(File.join(RAILS_ROOT, 'test/fixtures/cloud_front/cms'), File.join(@public_dir_for_cloud_front, 'cms'))
      url_for_cloud_front('404.html').should == '404.html'
      url_for_cloud_front('/404.html').should == '/404.html'
      url_for_cloud_front('javascripts/prototype.js').should == 'javascripts/prototype.js'
      url_for_cloud_front('/javascripts/prototype.js').should == '/javascripts/prototype.js'
    end

  end




  module LineInclusionMatchers
    class LineInclusionMatcher
      def initialize(expected)
        @expected = expected
      end
      def matches?(target)
        @target = target
        @target.to_s.each_line do |line|
          return true if line.include?(@expected)
        end
        return false
      end
      def failure_message
        "expected #{@target[0..20]} has line(s) including #{@expected.inspect}"
      end
      def negative_failure_message
        "expected #{@target.inspect} doesn't line including  {@expected.inspect}"
      end
    end

    def has_line(expected)
      LineInclusionMatcher.new(expected)
    end
  end

  include LineInclusionMatchers

  describe "create_timestamped_files" do
    before(:each) do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_front).and_return(true)
      EcRider.config.stub!(:cloud_front_url).and_return('http://0123456789.cloudfront.net')
      tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_create_timestamped_files"))
      Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      @public_dir_for_cloud_front = File.join(tmp_dir, 'public')
      FileUtils.mkdir_p(@public_dir_for_cloud_front)
    end

    def setup_copied_public_files(*paths)
      paths.each do |path|
        dest_dir_base = File.join(@public_dir_for_cloud_front, *(path.split('/')[0..-2]))
        cms_files = CmsFile.find(:all, :conditions => ['public_file_path like ? and (cms_project_id is null or cms_project_id = ?)', "public/#{path}%", CmsProject::BASE_PROJECT_ID], :order => 'public_file_path')

        cms_files.each do |cms_file|
          cp_dest = File.join(@public_dir_for_cloud_front, cms_file.public_file_path.sub('public/', ''))
          if cms_file.directory?
            FileUtils.mkdir_p(cp_dest)
          else
            FileUtils.cp(cms_file.pj_file_path, cp_dest)
          end
        end
      end
    end

    def setup_copied_fixture_files(*paths)
      src_base = '../ec_front/test/fixtures/cloud_front'
      paths.each do |path|
        dest_dir_base = File.join(@public_dir_for_cloud_front, *(path.split('/')[0..-2]))
        FileUtils.mkdir_p(dest_dir_base, :verbose => false)

        cp_src = File.join(RAILS_ROOT, src_base, *path.split('/'))
        cp_dest = File.join(@public_dir_for_cloud_front, *path.split('/'))
        FileUtils.cp_r(cp_src, cp_dest, :verbose => false)
      end
    end

    def tmp_file_name(name)
      File.join(@public_dir_for_cloud_front, *name.split('/'))
    end

    it "shoud copy same files if simple_files given" do
      setup_copied_public_files('cms/common')

      expectations = [
        tmp_file_name('cms/common/cart-20090311162012.png')
      ]
      expectations.each{|file| File.exist?(file).should == false}
      Dir.glob("#{@public_dir_for_cloud_front}/**/*-20090311162012.*").length == 0

      copied_files = create_timestamped_files(
        %w(cms/common/cart.png),
        :timestamp => '20090311162012')

      copied_files.should == [
        'cms/common/cart-20090311162012.png'
      ]
      expectations.each{|file| File.exist?(file).should == true}
      Dir.glob("#{@public_dir_for_cloud_front}/**/*-20090311162012.*").length == 1
    end


    it "shoud copy related files if css given" do
      setup_copied_public_files('javascripts', 'themes/ec_ver2', '404.html')

      expectations = [
        tmp_file_name('themes/ec_ver2/style-20090311162012.css'),
        tmp_file_name('themes/ec_ver2/layout/common-20090311162012.css'),
        tmp_file_name('themes/ec_ver2/layout/img/hr-20090311162012.gif'),
        tmp_file_name('themes/ec_ver2/layout/pagelayout-20090311162012.css'),
        tmp_file_name('themes/ec_ver2/layout/printlayout-20090311162012.css'),
        tmp_file_name('404-20090311162012.html'),
      ]
      expectations.each{|file| File.exist?(file).should == false}
      Dir.glob("#{@public_dir_for_cloud_front}/**/*-20090311162012.*").length == 0

      copied_files = create_timestamped_files(
        %w(404.html themes/ec_ver2/style.css),
        :timestamp => '20090311162012')

      [
        '404-20090311162012.html',
        'themes/ec_ver2/style-20090311162012.css',
        'themes/ec_ver2/layout/common-20090311162012.css',
        'themes/ec_ver2/layout/img/hr-20090311162012.gif',
        'themes/ec_ver2/layout/pagelayout-20090311162012.css',
        'themes/ec_ver2/layout/printlayout-20090311162012.css',
        # 他にもCSSから参照される画像が多数含まれます
      ].each do |copied|
        copied_files.should include(copied)
      end

      expectations.each{|file| File.exist?(file).should == true}
      Dir.glob("#{@public_dir_for_cloud_front}/**/*-20090311162012.*").length == expectations.length

      open(tmp_file_name('themes/ec_ver2/style-20090311162012.css')) do |f|
        text = f.read
        text.should has_line('@import "layout/common-20090311162012.css"')
        text.should has_line('@import "layout/pagelayout-20090311162012.css"')
        text.should has_line('@import "layout/printlayout-20090311162012.css"')
      end
    end

    it "#597 fix to copy files with timestamps" do
      setup_copied_fixture_files('cms')

      expectations = [
        tmp_file_name('cms/common/ec-rider-20090528185600.css'),
        tmp_file_name('cms/common/rightbox-20090528185600.png'),
        tmp_file_name('cms/common/button_cart-20090528185600.gif'),
        tmp_file_name('cms/top/shopinfo_back-20090528185600.png'),
        tmp_file_name('cms/top/headline-20090528185600.jpg'),
      ]
      expectations.each{|file| File.exist?(file).should == false}
      Dir.glob("#{@public_dir_for_cloud_front}/**/*-20090528185600.*").length == 0

      copied_files = create_timestamped_files('cms/common/ec-rider.css', :timestamp => '20090528185600')

      copied_files = copied_files.map{|filepath| Pathname.new(filepath).cleanpath.to_s}
      [
        'cms/common/ec-rider-20090528185600.css',
        'cms/common/rightbox-20090528185600.png',
        'cms/common/button_cart-20090528185600.gif',
        'cms/top/shopinfo_back-20090528185600.png',
        'cms/top/headline-20090528185600.jpg',
        # 他にもCSSから参照される画像が多数含まれます
      ].each do |copied|
        copied_files.should include(Pathname.new(copied).cleanpath.to_s)
      end

    end
  end

end
