# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe CmsFile do

  describe "パスのバリデーション" do
    it "should be valid." do
      csm_file = CmsFile.new(:public_file_path => 'public/images/product_images/')
      csm_file.should be_valid
      csm_file = CmsFile.new(:public_file_path => 'public/stylesheets/product_images_for_preview/')
      csm_file.should be_valid
      csm_file = CmsFile.new(:public_file_path => 'public/javascripts/ec_rider/')
      csm_file.should be_valid
      csm_file = CmsFile.new(:public_file_path => 'public/themes/projects/')
      csm_file.should be_valid
      csm_file = CmsFile.new(:public_file_path => 'public/cms/logo.GIF')
      csm_file.should be_valid
      csm_file = CmsFile.new(:public_file_path => 'public/public/product_images/')
      csm_file.should be_valid
    end

    it "should be invalid." do
      csm_file = CmsFile.new(:public_file_path => 'public/product_images/')
      csm_file.should_not be_valid
      csm_file.errors.on('base').should == "publicディレクトリの直下にはproduct_imagesという名前のファイルやディレクトリを作成できません。"
      csm_file.errors.clear
      csm_file = CmsFile.new(:public_file_path => 'public/product_images_for_preview/')
      csm_file.should_not be_valid
      csm_file.errors.on('base').should == "publicディレクトリの直下にはproduct_images_for_previewという名前のファイルやディレクトリを作成できません。"
      csm_file.errors.clear
      csm_file = CmsFile.new(:public_file_path => 'public/ec_rider/')
      csm_file.should_not be_valid
      csm_file.errors.on('base').should == "publicディレクトリの直下にはec_riderという名前のファイルやディレクトリを作成できません。"
      csm_file.errors.clear
      csm_file = CmsFile.new(:public_file_path => 'public/projects/')
      csm_file.should_not be_valid
      csm_file.errors.on('base').should == "publicディレクトリの直下にはprojectsという名前のファイルやディレクトリを作成できません。"
      csm_file.errors.clear
      csm_file = CmsFile.new(:public_file_path => 'public/product_images')
      csm_file.should_not be_valid
      csm_file.errors.on('base').should == "publicディレクトリの直下にはproduct_imagesという名前のファイルやディレクトリを作成できません。"
      csm_file.errors.clear
      csm_file = CmsFile.new(:public_file_path => 'public/ディレクトリ名')
      csm_file.should_not be_valid
      csm_file.errors.on('base').should == "ディレクトリ名に全角文字が含まれています。"
    end
  end

  describe "directory?" do
    it "should be true." do
      csm_file = CmsFile.new(:public_file_path => 'public/')
      csm_file.directory?.should be_true
      csm_file = CmsFile.new(:public_file_path => 'public/images/')
      csm_file.directory?.should be_true
    end

    it "should be false." do
      csm_file = CmsFile.new(:public_file_path => 'public/item01.jpg')
      csm_file.directory?.should be_false
      csm_file = CmsFile.new(:public_file_path => 'public/stylesheets/style01.css')
      csm_file.directory?.should be_false
      csm_file = CmsFile.new(:public_file_path => 'public/javascripts/prototype.js')
      csm_file.directory?.should be_false
    end
  end

  describe "public_dir_path" do
    it "case of directory." do
      csm_file = CmsFile.new(:public_file_path => 'public/')
      csm_file.public_dir_path.should == 'public/'
      csm_file = CmsFile.new(:public_file_path => 'public/images/')
      csm_file.public_dir_path.should == 'public/images/'
    end

    it "case of file." do
      csm_file = CmsFile.new(:public_file_path => 'public/item01.jpg')
      csm_file.public_dir_path.should == 'public/'
      csm_file = CmsFile.new(:public_file_path => 'public/stylesheets/style01.css')
      csm_file.public_dir_path.should == 'public/stylesheets/'
      csm_file = CmsFile.new(:public_file_path => 'public/javascripts/prototype.js')
      csm_file.public_dir_path.should == 'public/javascripts/'
    end
  end

  describe "pj_file_path" do
    it "cms_project_idあり" do
      csm_file = CmsFile.new(:public_file_path => 'public/item01.jpg', :cms_project_id => 10)
      csm_file.pj_file_path.should == File.expand_path("#{EcRider.config.ec_admin_root}/public/projects/10/public/item01.jpg")
    end

    it "cms_project_idなし" do
      csm_file = CmsFile.new(:public_file_path => 'public/item01.jpg')
      csm_file.cms_project_id.should == nil
      csm_file.pj_file_path.should == File.expand_path("#{EcRider.config.ec_admin_root}/public/projects/public/item01.jpg")
    end
  end

  describe "current_project?" do
    it "should be true." do
      csm_file = CmsFile.new(:public_file_path => 'public/item01.jpg', :cms_project_id => 10)
      csm_file.current_project?(10).should be_true
    end

    it "should be false." do
      csm_file = CmsFile.new(:public_file_path => 'public/stylesheets/style01.css', :cms_project_id => 10)
      csm_file.current_project?(20).should be_false
      csm_file = CmsFile.new(:public_file_path => 'public/stylesheets/style01.css')
      csm_file.current_project?(20).should be_false
    end
  end

  describe "create_condition" do
  fixtures :cms_projects
    before(:each) do
      CmsFile.delete_all
      CmsFile.create(:public_file_path => 'public/')
    end

    it "プロジェクト編集対象　かつ　編集中　かつ重複データなし" do
      #                                  公開期間
      # project2のpublic/aaa.jpg       (開始)------(終了)
      #                             ▲
      # project2のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00002).id
      cms_files[1].cms_project.status_cd.should == '0'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象　かつ　編集中　かつ重複データあり" do
      #                                  公開期間               優先順位
      # project1のpublic/aaa.jpg (開始)-------------(終了)         100
      # project4のpublic/aaa.jpg              (開始)------(終了)    14
      #                                ▲
      # project4のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00004).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00004).id
      cms_files[1].cms_project.status_cd.should == '3'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象外　かつ　編集中　かつ重複データなし" do
      #                                    公開期間
      # project2のpublic/aaa.jpg       (開始)------(終了)
      #                              ▲
      # project1のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00001).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 1
      cms_files[0].public_file_path.should == 'public/'
    end

    it "プロジェクト編集対象外　かつ　編集中　かつ重複データあり" do
      #                                  公開期間                優先順位
      # project1のpublic/aaa.jpg (開始)-------------(終了)         100
      # project4のpublic/aaa.jpg              (開始)------(終了)    14
      #                                 ▲
      # project1のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00001).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00001).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象外　かつ　公開中　かつ重複データなし" do
      #                                公開期間
      # project1のpublic/aaa.jpg   (開始)------(終了)
      #                                    ▲
      # project2のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00001).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象　かつ　公開中　かつ重複データあり" do
      #                                    公開期間             優先順位
      # project1のpublic/aaa.jpg (開始)-----------------(終了)     100
      # project4のpublic/aaa.jpg              (開始)------(終了)    14
      #                                              ▲
      # project4のファイル検索
      cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
      cms_project.status_cd = '4'
      cms_project.save!
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00004).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00004).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象外　かつ　公開中　かつ重複データあり　※優先度低いプロジェクトで検索" do
      #                                    公開期間              優先順位
      # project1のpublic/aaa.jpg (開始)-----------------(終了)     100
      # project4のpublic/aaa.jpg              (開始)------(終了)    14
      #                                              ▲
      # project1のファイル検索
      cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
      cms_project.status_cd = '4'
      cms_project.save!
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00001).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00001).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象外　かつ　公開中（公開時に削除）" do
      #                                     公開期間             優先順位  公開時削除
      # project1のpublic/aaa.jpg (開始)-------------(終了)         100
      # project4のpublic/aaa.jpg              (開始)------(終了)    14         true
      #                                  ▲
      # project4のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id, :delete_at_publish => true)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00004).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00001).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_true
    end

    it "プロジェクト編集対象外　かつ　公開中（削除中）" do
      #                                     公開期間         優先順位  公開時削除
      # project1のpublic/aaa.jpg (開始)-------------(終了)     100
      # project4のpublic/aaa.jpg          (開始)------(終了)    14         true
      #                                          ▲
      # project4のファイル検索
      cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
      cms_project.status_cd = '4'
      cms_project.save!
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id, :delete_at_publish => true)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00004).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00001).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_true
    end

    it "プロジェクト編集対象外　かつ　公開中（削除中）　※公開時削除したプロジェクト以外で検索" do
      #                                     公開期間         優先順位  公開時削除
      # project1のpublic/aaa.jpg (開始)-------------(終了)     100
      # project4のpublic/aaa.jpg          (開始)------(終了)    14         true
      #                                           ▲
      # project2のファイル検索
      cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
      cms_project.status_cd = '4'
      cms_project.save!
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00004).id, :delete_at_publish => true)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path')
      cms_files.length.should == 1
    end

    it "プロジェクト編集対象外　かつ　公開中　※公開時削除したプロジェクトは公開終了" do
      #                                     公開期間             優先順位  公開時削除
      # project1のpublic/aaa.jpg (開始)-------------------(終了)   100
      # project3のpublic/aaa.jpg      (開始)----(終了)              30         true
      #                                                 ▲
      # project3のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00001).id)
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00003).id, :delete_at_publish => true)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00003).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00001).id
      cms_files[1].cms_project.status_cd.should == '4'
      delete_ids.include?(cms_files[1].id).should be_true
    end

    it "プロジェクト編集対象　かつ　公開終了" do
      #                                  公開期間
      # project3のpublic/aaa.jpg  (開始)------(終了)
      #                                               ▲
      # project3のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00003).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00003).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[1].public_file_path.should == 'public/aaa.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00003).id
      cms_files[1].cms_project.status_cd.should == '5'
      delete_ids.include?(cms_files[1].id).should be_false
    end

    it "プロジェクト編集対象外　かつ　公開終了" do
      #                                  公開期間
      # project3のpublic/aaa.jpg  (開始)------(終了)
      #                                                ▲
      # project1のファイル検索
      CmsFile.create(:public_file_path => 'public/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00003).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00001).id)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 1
    end
    
    it "ファイル名検索" do
      CmsFile.create(:public_file_path => 'public/test.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test/', :cms_project_id => cms_projects(:cms_projects_00002).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id, :dir_or_file_name => 'test', :file_type_key => :file)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 1
      cms_files[0].public_file_path.should == 'public/test.jpg'
      cms_files[0].cms_project_id.should == cms_projects(:cms_projects_00002).id
    end

    it "ディレクトリ名検索" do
      CmsFile.create(:public_file_path => 'public/test.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test/', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test/aaa.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test/bbb.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id, :dir_or_file_name => 'test', :file_type_key => :dir)
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 3
      cms_files[0].public_file_path.should == 'public/test/'
      cms_files[0].cms_project_id.should == cms_projects(:cms_projects_00002).id
    end

    it "ファイル/ディレクトリ名検索" do
      CmsFile.create(:public_file_path => 'public/test_dir/', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test_dir/test.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id, :dir_or_file_name => 'test', :file_type_key => '')
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 2
      cms_files[0].public_file_path.should == 'public/test_dir/'
      cms_files[0].cms_project_id.should == cms_projects(:cms_projects_00002).id
      cms_files[1].public_file_path.should == 'public/test_dir/test.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00002).id
    end

    it "プロジェクト編集対象検索" do
      CmsFile.create(:public_file_path => 'public/test_dir/', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test_dir/test2.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      CmsFile.create(:public_file_path => 'public/test_dir/test.jpg', :cms_project_id => cms_projects(:cms_projects_00002).id)
      condition =CmsFile::SearchCondition.new(:cms_project_id => cms_projects(:cms_projects_00002).id, :dir_or_file_name => '',
        :file_type_key => '', :edit_target => '1')
      cond, delete_ids = condition.create_condition
      cms_files = CmsFile.find(:all, :include => :cms_project, :conditions => cond, :order => 'public_file_path, cms_files.id')
      cms_files.length.should == 3
      cms_files[0].public_file_path.should == 'public/test_dir/'
      cms_files[0].cms_project_id.should == cms_projects(:cms_projects_00002).id
      cms_files[1].public_file_path.should == 'public/test_dir/test.jpg'
      cms_files[1].cms_project_id.should == cms_projects(:cms_projects_00002).id
      cms_files[2].public_file_path.should == 'public/test_dir/test2.jpg'
      cms_files[2].cms_project_id.should == cms_projects(:cms_projects_00002).id
    end
  end

  describe "overwrite" do
    fixtures :cms_projects, :cms_files

    before(:each) do
      tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_cms_files"))
      Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      FileUtils.mkdir_p(File.join(tmp_dir, 'admin', 'public'))
      FileUtils.mkdir_p(File.join(tmp_dir, 'front', 'public'))
      EcRider.config.stub!(:ec_admin_root).and_return(File.join(tmp_dir, 'admin'))
      EcRider.config.stub!(:ec_front_root).and_return(File.join(tmp_dir, 'front'))
      @publish = CmsPublish.create!(
        :target => CmsPublish.target_id_by_key(:production),
        :status_cd => CmsPublish.status_id_by_key(:processing),
        :designated_at => 1.minute.ago,
        :default_template_paths => {})
    end

    it "ファイルコピー" do
      cms_file = CmsFile.new(:cms_project_id => 1, :public_file_path => 'public/zzz/abc.png')
      file_action = CmsPublish::FileAction.new(@publish, cms_file.public_file_path)
      file_action.targets << cms_file
      
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'zzz'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'zzz', 'abc.png'))
      File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should == false
      cms_file.overwrite(file_action)
      File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should == true
    end
  end

  describe "delete" do
    fixtures :cms_projects, :cms_files

    before(:each) do
      tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_cms_files"))
      Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      FileUtils.mkdir_p(File.join(tmp_dir, 'admin', 'public'))
      FileUtils.mkdir_p(File.join(tmp_dir, 'front', 'public'))
      EcRider.config.stub!(:ec_admin_root).and_return(File.join(tmp_dir, 'admin'))
      EcRider.config.stub!(:ec_front_root).and_return(File.join(tmp_dir, 'front'))
      @publish = CmsPublish.create!(
        :target => CmsPublish.target_id_by_key(:production),
        :status_cd => CmsPublish.status_id_by_key(:processing),
        :designated_at => 1.minute.ago,
        :default_template_paths => {})
    end

    it "ファイルを削除" do
      cms_file = CmsFile.new(:cms_project_id => 1, :public_file_path => 'public/zzz/abc.png')
      file_action = CmsPublish::FileAction.new(@publish, cms_file.public_file_path)
      file_action.targets << cms_file
      
      FileUtils.mkdir_p(File.join(EcRider.config.ec_front_root, 'public', 'zzz'))
      FileUtils.touch(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png'))
      File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should == true
      cms_file.delete(file_action)
      File.exist?(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png')).should == false
    end
  end

end
