# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe CmsPage do
  fixtures :cms_projects, :cms_pages, :cms_page_titles,
    :cms_binds,   :cms_bind_items, :cms_bind_contents, 
    :cms_templates, :cms_template_contents, :cms_files, :cms_default_templates

  before(:each) do
  end

  def set_binds_attributes
    result = {"0"=>{:position=>"0",
        :items_attributes=>{"0"=>{:no=>"01", :code=>"shop_tel_num",:input_type=>"select"},
          "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"}},
        :cms_template_id=>"15"},
      "1"=>{:position=>"1",:cms_template_id=>"2"},
      "2"=>{:position=>"2",:cms_template_id=>"3"},
      "3"=>{:position=>"3",:cms_template_id=>""},
      "4"=>{:position=>"4",
        :items_attributes=>{"0"=>{:no=>"01",:code=>"shop_tel_num",:input_type=>"tab",
            :contents_attributes=>{"zh_CN"=>{:descriptions=>"再入荷しました！！cn",:locale=>"zh_CN"},
              "ja"=>{:descriptions=>"再入荷しました！！ja",:locale=>"ja"},
              "zh_TW"=>{:descriptions=>"再入荷しました！！tw", :locale=>"zh_TW"},
              "ko"=>{:descriptions=>"再入荷しました！！ko",:locale=>"ko"},
              "th"=>{:descriptions=>"再入荷しました！！th",:locale=>"th"},
              "en"=>{:descriptions=>"再入荷しました！！en",:locale=>"en"}}},
          "1"=>{:no=>"02",:code=>"shop_tel_num",:input_type=>"select"},
          "2"=>{:no=>"03",:code=>"shop_tel_num",:input_type=>"tab",
            :contents_attributes=>{"zh_CN"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh_CN"},
              "ja"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ja"},
              "zh_TW"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"zh_TW"},
              "ko"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"ko"},
              "th"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"th"},
              "en"=>{:descriptions=>"<ec:shop_logo/>",:locale=>"en"}}}},
        :cms_template_id=>"16"}}
    return result
  end

  def set_titles_attributes
    result = {"zh_TW"=>{"name"=>"","locale"=>"zh_TW"},
      "ja"=>{"name"=>"フィギュア商品詳細ja","locale"=>"ja"},
      "ko"=>{"name"=>"フィギュア商品詳細ko","locale"=>"ko"},
      "zh_CN"=>{"name"=>"","locale"=>"zh_CN"},
      "th"=>{"name"=>"","locale"=>"th"},
      "en"=>{"name"=>"","locale"=>"en"}}
    return result
  end

  if EcRider.config.ec_admin
    describe "validate" do
      before(:each) do
        @cms_page = CmsPage.new()
        @cms_page.cms_project_id = '1'
        @cms_page.action = 'new_action'
        @cms_page.terminal = '0'
        @cms_page.binds_attributes = set_binds_attributes
        @cms_page.titles_attributes = set_titles_attributes
        @cms_page.errors.clear
        @cms_page.stub!(:request).and_return(mock_model(Object, {:mobile? => false}))
      end

      it "should be valid" do
        @cms_page.should be_valid
      end

      it "URLが重複している場合" do
        @cms_page.controller = 'cms'
        @cms_page.action = 'show_fashion'
        @cms_page.cms_project_id = 2
        @cms_page.should_not be_valid

        @cms_page.controller = ''
        @cms_page.action = 'show_fashion'
        @cms_page.cms_project_id = 2
        @cms_page.should_not be_valid

        @cms_page.controller = nil
        @cms_page.action = 'show_fashion'
        @cms_page.cms_project_id = 2
        @cms_page.should_not be_valid
      end

      it "URLが重複していてもOKの場合" do
        @cms_page.controller = 'cms'
        @cms_page.action = 'show_fashion'
        @cms_page.should be_valid

        @cms_page.controller = ''
        @cms_page.action = 'show_fashion'
        @cms_page.should be_valid

        @cms_page.controller = nil
        @cms_page.action = 'show_fashion'
        @cms_page.should be_valid
      end

      it "中央テンプレートが未選択" do
        @cms_page.binds_attributes["4"][:cms_template_id] = nil
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "中央テンプレートは必須です。"
      end

      it "バインドの内容が未選択" do
        @cms_page.binds_attributes["0"][:items_attributes]["0"][:input_type] = nil
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "上テンプレート バインド01の内容を入力してください。"
      end

      it "直接入力で日本語の内容が未入力" do
        @cms_page.binds_attributes["4"][:items_attributes]["0"][:contents_attributes]["ja"][:descriptions] = ''
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "中央テンプレート バインド01 日本語の内容は必須です。"
      end

      it "文字数オーバー（日本語）" do
        @cms_page.binds_attributes["4"][:items_attributes]["0"][:contents_attributes]["ja"][:descriptions] = 'あ'*30000
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "中央テンプレート バインド01 日本語[ja] 65535バイト以内で入力してください。"
      end

      it "文字数オーバー（英語）" do
        @cms_page.binds_attributes["4"][:items_attributes]["0"][:contents_attributes]["en"][:descriptions] = 'a'*65536
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "中央テンプレート バインド01 英語[en] 65535バイト以内で入力してください。"
      end

      it "XMLが文法的に不正" do
        @cms_page.binds_attributes["4"][:items_attributes]["0"][:contents_attributes]["zh_CN"][:descriptions] = '<ec:today>'
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "中央テンプレート バインド01 中国語(簡体字)[zh_CN] XMLの記述が不正です。"
      end

      it "存在しないecタグを入力" do
        @cms_page.binds_attributes["4"][:items_attributes]["0"][:contents_attributes]["zh_TW"][:descriptions] = '<ec:wrongtag/>'
        @cms_page.should_not be_valid
        @cms_page.errors.on('base').should == "中央テンプレート バインド01 中国語(繁体字)[zh_TW] ec:wrongtag は存在しないタグです。"
      end

      describe "validates_format_of :action" do
        it "英数字と_" do
          @cms_page.action = 'character'
          @cms_page.should be_valid

          @cms_page.action = 'c_h_a_r_a_c_t_e_r'
          @cms_page.should be_valid

          @cms_page.action = 'character123'
          @cms_page.should be_valid

          @cms_page.action = 'c'
          @cms_page.should be_valid

          @cms_page.action = 'character/'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"

          @cms_page.errors.clear
          @cms_page.action = '/symbol'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"

          @cms_page.errors.clear
          @cms_page.action = 'sym.bol'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"

          @cms_page.errors.clear
          @cms_page.action = 'symbol?'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"

          @cms_page.errors.clear
          @cms_page.action = '!'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"
        end

        it "大文字" do
          @cms_page.action = 'CAPITAL'
          @cms_page.should be_valid
          @cms_page.action.should == 'capital'

          @cms_page.action = 'C_A_P_I_T_A_L'
          @cms_page.should be_valid
          @cms_page.action.should == 'c_a_p_i_t_a_l'

          @cms_page.action = 'CAPITAL123'
          @cms_page.should be_valid
          @cms_page.action.should == 'capital123'

          @cms_page.action = 'C'
          @cms_page.should be_valid
        end

        it "先頭文字" do
          @cms_page.action = 'initial'
          @cms_page.should be_valid

          @cms_page.errors.clear
          @cms_page.action = '123initial'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"
        end

        # 2009/11/14、CmsPageのactionの先頭に_(アンダースコア)を使えるようにしました。
        # 本来のCmsPageの仕様としては矛盾していますが、partial用のテンプレートを扱うために仕方なくそのようになっています。
        # 2009/11/24のリリース後、再び_を先頭に使用できないように戻します。
        it "先頭文字にアンダースコア" do
          pending("2009/11/24向けの暫定的ペンディング。リリース後にペンディングを外します")
          @cms_page.action = '_initial'
          @cms_page.should_not be_valid
          @cms_page.errors.on(:action).should == "は不正な値です。"
        end

        it "コントローラがCMS以外" do
          @cms_page.controller = 'cart'
          @cms_page.action = 'cart_index'
          @cms_page.cms_project_id = 50
          @cms_page.should be_valid

          @cms_page.controller = 'cart'
          @cms_page.action = '_cart_list'
          @cms_page.cms_project_id = 50
          @cms_page.should be_valid
        end
      end
    end

    describe "validate_for_setup?" do
      before(:each) do
        @cms_page = CmsPage.new
        @cms_page.cms_project_id = '1'
        @cms_page.action = 'new_action'
        @cms_page.terminal = '0'
      end

      it "should be valid" do
        @cms_page.validate_for_setup?.should be_true
      end

      it "ページ名[ja]未入力の場合" do
        @cms_page.titles << CmsPageTitle.new("name"=>"","locale"=>"ja")
        @cms_page.validate_for_setup?.should be_false

        @cms_page.titles.clear
        @cms_page.titles << CmsPageTitle.new("name"=>nil,"locale"=>"ja")
        @cms_page.validate_for_setup?.should be_false
      end

      it "ページ名[docomo]未入力の場合" do
        @cms_page.terminal = '1'
        @cms_page.titles << CmsPageTitle.new("name"=>"","locale"=>"docomo")
        @cms_page.validate_for_setup?.should be_false

        @cms_page.titles.clear
        @cms_page.titles << CmsPageTitle.new("name"=>nil,"locale"=>"docomo")
        @cms_page.validate_for_setup?.should be_false
      end
    end

    describe "find" do
      before do
        # テスト用に想定しているのは、
        # project1のcms/policy_textとcms以外のすべてのアクションを削除した状態なので、
        # 不要なものを削除します。
        CmsPage.destroy_all(
          "cms_project_id = 1 AND (" <<
          "(controller = 'cms' AND action in ('list', 'show', '_policy_text')) OR controller <> 'cms')")
      end

      it "プロジェクト編集対象　かつ　編集中　かつ重複データなし" do
        #                                  公開期間
        # project2: show_fashion<id: 4>       (開始)------(終了)
        #                                  ▲
        CmsProject.find(cms_projects(:cms_projects_00001).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_002).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        # project2のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 2)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [4]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象　かつ　編集中　かつ重複データあり" do
        #                                  公開期間             優先順位
        # project1: index<id:14> (開始)-------------(終了)         100
        # project4: index<id:16>              (開始)------(終了)    14
        #                                  ▲
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project4のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 6)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [16]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象外　かつ　編集中　かつ重複データなし" do
        #                                  公開期間
        # project2: show_fashion<id: 4>       (開始)------(終了)
        #                                  ▲
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_002).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project1のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 1)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.should == []
      end

      it "プロジェクト編集対象外　かつ　編集中　かつ重複データあり" do
        #                                  公開期間             優先順位
        # project1: index<id:14> (開始)-------------(終了)         100
        # project4: index<id:16>              (開始)------(終了)    14
        #                                ▲
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project1のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 1)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [14]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象外　かつ　公開中　かつ重複データなし" do
        #                            公開期間
        # project1: index<id:14>   (開始)------(終了)
        #                                  ▲
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project2のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 2)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [14]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象　かつ　公開中　かつ重複データあり" do
        #                                  公開期間             優先順位
        # project1: index<id:14> (開始)-----------------(終了)     100
        # project4: index<id:16>              (開始)------(終了)    14
        #                                             ▲
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
        cms_project.status_cd = '4'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project4のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 4)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [16]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象外　かつ　公開中　かつ重複データあり　※優先度低いプロジェクトで検索" do
        #                                  公開期間             優先順位
        # project1: index<id:14> (開始)-----------------(終了)     100
        # project4: index<id:16>              (開始)------(終了)    14
        #                                             ▲
        # 期待値：優先順位の高いproject1のテンプレートが取得できること
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00004).id)
        cms_project.status_cd = '4'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project1のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 1)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [14]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象外　かつ　公開中（公開時に削除）" do
        #                                  公開期間             優先順位  公開時削除
        # project1: index<id:14> (開始)-------------(終了)         100
        # project3: index<id:15>              (開始)------(終了)    30         true
        #                                ▲
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00003).id)
        cms_project.status_cd = '0'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project3のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 3)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [14]
        delete_ids.include?(cms_pages[0].id).should be_true
      end

      it "プロジェクト編集対象外　かつ　公開中（削除中）" do
        #                                  公開期間         優先順位  公開時削除
        # project1: index<id:14> (開始)-------------(終了)     100
        # project3: index<id:15>          (開始)------(終了)    30         true
        #                                          ▲
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00003).id)
        cms_project.status_cd = '4'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project3のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 3)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [14]
        delete_ids.include?(cms_pages[0].id).should be_true
      end

      it "プロジェクト編集対象外　かつ　公開中（削除中）　※公開時削除したプロジェクト以外で検索" do
        #                                  公開期間         優先順位  公開時削除
        # project1: index<id:14> (開始)-------------(終了)     100
        # project3: index<id:15>          (開始)------(終了)    30         true
        # priject2: index<id: 2>                                14
        #                                          ▲
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00003).id)
        cms_project.status_cd = '4'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project2のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 2)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.should == []
      end

      it "プロジェクト編集対象外　かつ　公開中　※公開時削除したプロジェクトは公開終了" do
        #                                  公開期間             優先順位  公開時削除
        # project1: index<id:14> (開始)-------------------(終了)   100
        # project3: index<id:15>      (開始)----(終了)              30         true
        #                                               ▲
        CmsProject.find(cms_projects(:cms_projects_00002).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_001).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project3のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 3)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [14]
        delete_ids.include?(cms_pages[0].id).should be_true
      end

      it "プロジェクト編集対象　かつ　公開終了" do
        #                                  公開期間
        # project2: show_fashion<id: 4>  (開始)------(終了)
        #                                                     ▲
        CmsProject.find(cms_projects(:cms_projects_00001).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00002).id)
        cms_project.status_cd = '5'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_002).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy
        # project2のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 2)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.map(&:id).should == [4]
        delete_ids.include?(cms_pages[0].id).should be_false
      end

      it "プロジェクト編集対象外　かつ　公開終了" do
        #                                  公開期間
        # project2: show_fashion<id: 4>  (開始)------(終了)
        #                                                      ▲
        CmsProject.find(cms_projects(:cms_projects_00001).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00003).id).destroy
        CmsProject.find(cms_projects(:cms_projects_00004).id).destroy
        cms_project = CmsProject.find(cms_projects(:cms_projects_00002).id)
        cms_project.status_cd = '5'
        cms_project.save!
        CmsPage.find(cms_pages(:cms_pages_002).id).destroy
        CmsPage.find(cms_pages(:cms_pages_003).id).destroy
        CmsPage.find(cms_pages(:cms_pages_004).id).destroy
        CmsPage.find(cms_pages(:cms_pages_00049).id).destroy

        # project1のファイル検索
        condition = CmsPage::SearchCondition.new(:cms_project_id => 1)
        cond, delete_ids = condition.create_query
        cms_pages = CmsPage.find_by_sql(cond)
        cms_pages.should == []
      end
    end

    describe "create search condition" do
      it "should create search condition" do
        condition = CmsPage::SearchCondition.new(:cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 46

        condition = CmsPage::SearchCondition.new(:page_name => 'ファッション商品詳細', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1

        condition = CmsPage::SearchCondition.new(:page_name => 'フィギュア商品詳細', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 0

        condition = CmsPage::SearchCondition.new(:front_url => 'fashion', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1

        condition = CmsPage::SearchCondition.new(:front_url => 'figure', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 0

        condition = CmsPage::SearchCondition.new(:template_name => '商品詳細テンプレート', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1

        condition = CmsPage::SearchCondition.new(:template_name => '左', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 4

        condition = CmsPage::SearchCondition.new(:started_at => '2008/01/01', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 2 + 44

        condition = CmsPage::SearchCondition.new(:finished_at => '2020/05/01', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 2 + 44

        condition = CmsPage::SearchCondition.new(:finished_at => '2020/01/01', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1 + 44

        condition = CmsPage::SearchCondition.new(:edit_target => '1', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1

        condition = CmsPage::SearchCondition.new(:edit_target => '1', :cms_project_id => 3)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1

        condition = CmsPage::SearchCondition.new(:terminal => '0', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 2 + 43

        cms_page = CmsPage.find(cms_pages(:cms_pages_001).id)
        cms_page.terminal = nil
        cms_page.save!
        condition = CmsPage::SearchCondition.new(:terminal => '0', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1 + 43

        cms_page = CmsPage.find(cms_pages(:cms_pages_001).id)
        cms_page.terminal = '2'
        cms_page.save!
        condition = CmsPage::SearchCondition.new(:terminal => '0', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 1 + 43

        cms_page = CmsPage.find(cms_pages(:cms_pages_001).id)
        cms_page.terminal = '1'
        cms_page.save!
        condition = CmsPage::SearchCondition.new(:terminal => '1', :cms_project_id => 2)
        cms_pages = CmsPage.find_by_sql(condition.create_query[0])
        cms_pages.size.should == 2
      end
    end

    describe "new_create" do
      it "should create object" do
        cms_page = CmsPage.new(:action => 'new_action', :terminal => "0", :binds_attributes => set_binds_attributes, :titles_attributes => set_titles_attributes)
        cms_page.set_values_for_create

        cms_page.titles.class.should == Array
        cms_page.titles.length.should == I18n.supported_locales.length
        cms_page.titles[0].class.should == CmsPageTitle

        items_length = 0
        contents_length = 0
        cms_page.binds.class.should == Array
        cms_page.binds.length.should == 5
        cms_page.binds[0].class.should == CmsBind
        cms_page.binds.each do |bind|
          if bind.items
            bind.items.class.should == Array
            unless bind.items.empty?
              bind.items[0].class.should == CmsBindItem
              bind.items.each do |item|
                items_length += 1
                item.contents.class.should == Array
                contents_length += item.contents.length
                item.contents[0].class.should == CmsBindContent unless item.contents.empty?
              end
            end
          end
        end
        items_length.should == 5
        contents_length.should == 12
      end
    end

    describe "previewable?" do
      it "should be true" do
        cms_page = CmsPage.find(4, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles])
        cms_page.previewable?('ja').should be_true
      end
    end

    describe "copy_public_files_for_preview" do
      before(:each) do
        tmp_dir = File.expand_path(File.join(RAILS_ROOT, "tmp", "test_cms_pages"))
        Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
        FileUtils.mkdir_p(File.join(tmp_dir, 'admin', 'public'))
        FileUtils.mkdir_p(File.join(tmp_dir, 'front', 'public'))
        EcRider.config.stub!(:ec_admin_root).and_return(File.join(tmp_dir, 'admin'))
        EcRider.config.stub!(:ec_front_root).and_return(File.join(tmp_dir, 'front'))
      end

      it "project1のファイル" do
        # tempディレクトリに空のファイルを作成しておいてそれをコピーする
        mock_cms_file = CmsFile.new(:cms_project_id => 1, :public_file_path => 'public/abc.png')
        mock_cms_file.pj_file_path.should =~ %r{projects/1/public/abc.png}
        CmsFile.stub!(:find).and_return([mock_cms_file])
        FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public'))
        FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'abc.png'))
        File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'abc.png')).should == false
        cms_page = CmsPage.find(4, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles])
        cms_page.cms_project_id.should == 2
        cms_page.copy_public_files_for_preview
        File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'abc.png')).should == true
      end

      it "ec_front/publicのファイル" do
        # tempディレクトリに空のファイルを作成しておいてそれをコピーする
        CmsFile.stub!(:find).and_return([])
        FileUtils.mkdir_p(File.join(EcRider.config.ec_front_root, 'public', 'zzz'))
        FileUtils.touch(File.join(EcRider.config.ec_front_root, 'public', 'zzz', 'abc.png'))
        File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'zzz', 'abc.png')).should == false
        cms_page = CmsPage.find(4, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles])
        cms_page.cms_project_id.should == 2
        cms_page.copy_public_files_for_preview
        File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'zzz', 'abc.png')).should == true
      end

      it "robots.txtはコピーしない" do
        # tempディレクトリに空のファイルを作成しておいてそれをコピーする
        CmsFile.stub!(:find).and_return([])
        FileUtils.mkdir_p(File.join(EcRider.config.ec_front_root, 'public', 'projects', '1', 'public'))
        FileUtils.touch(File.join(EcRider.config.ec_front_root, 'public', 'projects', '1', 'public', 'robots.txt'))
        File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'robots.txt')).should == false
        cms_page = CmsPage.find(4, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles])
        cms_page.cms_project_id.should == 2
        cms_page.copy_public_files_for_preview
        File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'robots.txt')).should == false
      end
    end

    describe "action_options" do
      it "セレクトボックスに表示するCMSページのアクション配列が取得できる" do
        options = CmsPage.action_options
        options.size.should == 6
        options.should == [["/index", "/index"], ["/list", "/list"], ["/search_list", "/search_list"], ["/show", "/show"], ["/show_fashion", "/show_fashion"], ["/_policy_text", "/_policy_text"]]
      end
    end

    describe "published_filename" do
      it "cms/show_fashionの場合" do
        proj2 = CmsProject.find(2)
        page = proj2.pages.find(:first, :conditions => {:controller => 'cms', :action => 'show_fashion'})
        page.published_filename(:ja).should     == "cms/show_fashion.html.erb"
        page.published_filename(:en).should     == "cms/show_fashion_en.html.erb"
        page.published_filename('zh_CN').should == "cms/show_fashion_zh_CN.html.erb"
        page.published_filename(:docomo).should     == "cms/show_fashion_mobile.html.erb"
        page.published_filename(:au).should     == "cms/show_fashion_mobile_au.html.erb"
        page.published_filename(:softbank).should == "cms/show_fashion_mobile_softbank.html.erb"
      end

      it "account/loginの場合" do
        proj1 = CmsProject.find(1)
        page = proj1.pages.find(:first, :conditions => {:controller => 'account', :action => 'login'})
        page.published_filename(:ja).should     == "account/login.html.erb"
        page.published_filename(:en).should     == "account/login_en.html.erb"
        page.published_filename('zh_CN').should == "account/login_zh_CN.html.erb"
        page.published_filename(:docomo).should     == "account/login_mobile.html.erb"
        page.published_filename(:au).should     == "account/login_mobile_au.html.erb"
        page.published_filename(:softbank).should == "account/login_mobile_softbank.html.erb"
      end
    end

    describe "primary_pages" do
      it "フィクスチャベース" do
        # mysql> select cms_pages.id, cms_pages.cms_project_id, cms_projects.priority
        #        from cms_pages inner join cms_projects on cms_projects.id = cms_pages.cms_project_id
        #        where controller = "cms" AND action = "index";
        # +----+----------------+----------+
        # | id | cms_project_id | priority |
        # +----+----------------+----------+
        # | 14 |              1 |      100 |
        # | 15 |              3 |        3 |
        # | 16 |              6 |       14 |
        # +----+----------------+----------+
        # 3 rows in set (0.00 sec)
        #
        # 2009/08/20現在、プロジェクト間で重複するCmsPageは "cms/index"のみ。
        # ただし、オープンされているプロジェクトだけだと重複がないので、全てのステータスを対象にします。
        primary_pages = CmsPage.primary_pages(({:project_status_keys => [*CmsProject.status_keys]}))
        primary_pages.length.should == CmsPage.count - 2
        primary_pages.detect{|page| page.controller == 'cms' && page.action == 'index'}.id.should == 15
      end

      it "ステータスとプロジェクトID指定" do
        primary_pages = CmsPage.primary_pages({:project_status_keys => [:opened], :project_ids => [2,3,6]})
        primary_pages.length.should == CmsPage.count - 2
        primary_pages.detect{|page| page.controller == 'cms' && page.action == 'index'}.id.should == 15
      end

      it "プロジェクトID指定" do
        primary_pages = CmsPage.primary_pages({:project_ids => [6]})
        primary_pages.length.should == CmsPage.count - 3
        primary_pages.detect{|page| page.controller == 'cms' && page.action == 'index'}.id.should == 16
      end
    end

    describe "page_list_ids" do
      before do
        CmsProject.delete_all
        CmsPage.delete_all
        @cms_projects = {}
        @cms_projects["100"] = CmsProject.new(:started_at => '2008-10-10 00:00:00', :priority => 100, :status_cd => 4, :acceptance_type => '0', :project_name => "test100", :project_code => "test100")
        @cms_projects["101"] = CmsProject.new(:started_at => '2009-10-10 00:00:00', :priority => 101, :status_cd => 4, :acceptance_type => '0', :project_name => "test101", :project_code => "test101")
        @cms_projects["102"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 102, :status_cd => 4, :acceptance_type => '0', :project_name => "test102", :project_code => "test102")
        @cms_projects["103"] = CmsProject.new(:started_at => '2008-10-10 00:00:00', :priority => 103, :status_cd => 4, :acceptance_type => '0', :project_name => "test103", :project_code => "test103")
        @cms_projects["104"] = CmsProject.new(:started_at => '2009-10-10 00:00:00', :priority => 103, :status_cd => 4, :acceptance_type => '0', :project_name => "test104", :project_code => "test104")
        @cms_projects["105"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 103, :status_cd => 4, :acceptance_type => '0', :project_name => "test105", :project_code => "test105")
        @cms_projects["106"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 104, :status_cd => 4, :acceptance_type => '0', :project_name => "test106", :project_code => "test106")
        @cms_projects["107"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 104, :status_cd => 4, :acceptance_type => '0', :project_name => "test107", :project_code => "test107")
        @cms_projects["108"] = CmsProject.new(:started_at => '2010-10-10 00:00:00', :priority => 104, :status_cd => 4, :acceptance_type => '0', :project_name => "test108", :project_code => "test108")

        @cms_pages = {}
        @cms_pages["1000"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1001"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1002"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1003"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1004"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1005"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1006"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1007"] = CmsPage.new(:controller => "controller1", :action => "action")
        @cms_pages["1008"] = CmsPage.new(:controller => "controller1", :action => "action")

        @cms_pages["1000"].cms_project = @cms_projects["100"]
        @cms_pages["1001"].cms_project = @cms_projects["101"]
        @cms_pages["1002"].cms_project = @cms_projects["102"]
        @cms_pages["1003"].cms_project = @cms_projects["103"]
        @cms_pages["1004"].cms_project = @cms_projects["104"]
        @cms_pages["1005"].cms_project = @cms_projects["105"]
        @cms_pages["1006"].cms_project = @cms_projects["106"]
        @cms_pages["1007"].cms_project = @cms_projects["107"]
        @cms_pages["1008"].cms_project = @cms_projects["108"]

        @cms_pages["1009"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1010"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1011"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1012"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1013"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1014"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1015"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1016"] = CmsPage.new(:controller => "controller2", :action => "action")
        @cms_pages["1017"] = CmsPage.new(:controller => "controller2", :action => "action")

        @cms_pages["1009"].cms_project = @cms_projects["100"]
        @cms_pages["1010"].cms_project = @cms_projects["101"]
        @cms_pages["1011"].cms_project = @cms_projects["102"]
        @cms_pages["1012"].cms_project = @cms_projects["103"]
        @cms_pages["1013"].cms_project = @cms_projects["104"]
        @cms_pages["1014"].cms_project = @cms_projects["105"]
        @cms_pages["1015"].cms_project = @cms_projects["106"]
        @cms_pages["1016"].cms_project = @cms_projects["107"]
        @cms_pages["1017"].cms_project = @cms_projects["108"]
      end

      describe "ターゲットのプロジェクトで編集されたページの場合" do
        it "ターゲットのプロジェクト 公開中でない場合　ターゲットプロジェクトで編集されたページが取得できること" do
          @cms_pages.keys.each do |key|
            @cms_pages[key].save!
          end

          condition =CmsPage::SearchCondition.new(:cms_project_id => @cms_projects["101"].id)
          page_ids, delete_ids = condition.page_list_ids(@cms_projects["101"].id)
          page_ids.size.should == 2
          CmsPage.find_by_id(page_ids[0]).cms_project.id.should == @cms_projects["101"].id
          CmsPage.find_by_id(page_ids[1]).cms_project.id.should == @cms_projects["101"].id
        end
        it "ターゲットのプロジェクト 公開中の場合　ターゲットプロジェクトで編集されたページが取得できること" do
          @cms_pages.keys.each do |key|
            @cms_pages[key].save!
          end

          condition =CmsPage::SearchCondition.new(:cms_project_id => @cms_projects["101"].id)
          page_ids, delete_ids = condition.page_list_ids(@cms_projects["101"].id)
          page_ids.size.should == 2
          CmsPage.find_by_id(page_ids[0]).cms_project.id.should == @cms_projects["101"].id
          CmsPage.find_by_id(page_ids[1]).cms_project.id.should == @cms_projects["101"].id
        end
      end
      describe "ターゲットのプロジェクトで未編集のページの場合" do
        it "優先順位が同順、かつ公開開始日も同順の場合、プロジェクトIDの大きいプロジェクトのページが取得できること" do
          @cms_pages["1006"].save!
          @cms_pages["1007"].save!
          @cms_pages["1008"].save!
          condition =CmsPage::SearchCondition.new(:cms_project_id => 9999)
          page_ids, delete_ids = condition.page_list_ids("9999")
          CmsPage.find_by_id(page_ids[0]).cms_project.id.should == @cms_projects["108"].id
        end
        it "優先順位が同順、の場合、公開開始日の大きいプロジェクトのページが取得できること" do
          @cms_pages["1003"].save!
          @cms_pages["1004"].save!
          @cms_pages["1005"].save!
          condition =CmsPage::SearchCondition.new(:cms_project_id => 9999)
          page_ids, delete_ids = condition.page_list_ids("9999")
          CmsPage.find_by_id(page_ids[0]).cms_project.id.should == @cms_projects["105"].id
        end
        it "優先順位の高いプロジェクトのページが取得できること" do
          @cms_pages["1000"].save!
          @cms_pages["1001"].save!
          @cms_pages["1002"].save!
          condition =CmsPage::SearchCondition.new(:cms_project_id => 9999)
          page_ids, delete_ids = condition.page_list_ids("9999")
          CmsPage.find_by_id(page_ids[0]).cms_project.id.should == @cms_projects["102"].id
        end
        it "公開終了のプロジェクトがある場合、公開中の優先順位の高いプロジェクトのページが取得できること" do
          @cms_pages["1000"].save!
          @cms_pages["1001"].save!
          @cms_pages["1002"].cms_project.status_cd = 5
          @cms_pages["1002"].save!
          condition =CmsPage::SearchCondition.new(:cms_project_id => 9999)
          page_ids, delete_ids = condition.page_list_ids("9999")
          CmsPage.find_by_id(page_ids[0]).cms_project.id.should == @cms_projects["101"].id
        end
      end
    end
  end
end
