# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe EcRider::Config do

  before(:each) do
    @config = EcRider::Config.new
    # ENVをHashとしてコピーしておく
    @env_backup = ENV.to_hash.dup
  end

  after(:each) do
    @env_backup.each do |key, value|
      ENV[key] = value unless ENV[key] == value
    end
    ENV.to_hash.each do |key, value|
      next if @env_backup.key?(key)
      ENV.delete(key)
    end
  end


  it "should override value with override method" do
    @config.default_ec_admin_url = "http://localhost:9000"
    @config.default_ec_admin_url.should == "http://localhost:9000"
    @config.override(:default_ec_admin_url, "http://localhost:9999") do
      @config.default_ec_admin_url.should == "http://localhost:9999"
    end
    @config.default_ec_admin_url.should == "http://localhost:9000"
  end

  %w(ec_admin ec_front).each do |app_name|
    env_base = "EC_RIDER_%s" % app_name.gsub(/^ec_/, '').upcase

    instance_eval(<<-"EOS", __FILE__, __LINE__)
      describe 'access #{app_name}_url' do

        it "normal access" do
          @config.#{app_name}_url = "http://localhost:4000"
          @config.#{app_name}_url.should  == "http://localhost:4000"
        end

        it "should return default_#{app_name}_url value" do
          ENV['#{env_base}_URL'] = nil
          # @config.#{app_name}_url = nil
          @config.default_#{app_name}_url = "http://localhost:4000"
          @config.#{app_name}_url.should  == "http://localhost:4000"
        end

        it "should return ENV['#{env_base}_URL'] value" do
          ENV['#{env_base}_URL'] = "http://localhost:4000"
          @config.#{app_name}_root = nil
          @config.default_#{app_name}_url = nil
          @config.#{app_name}_url.should  == "http://localhost:4000"
        end

        it "should raise CondigurationError if no ENV['#{env_base}_URL'] and no default_#{app_name}_url" do
          ENV['#{env_base}_URL'] = nil
          @config.#{app_name}_root = nil
          @config.default_#{app_name}_url = nil
          lambda{ @config.#{app_name}_url }.should raise_error(EcRider::ConfigurationError)
        end
      end

      describe "#{app_name}_root" do
        it "should be able to access with #{env_base}_ROOT" do
          ENV['#{env_base}_ROOT'] = '/home/xec/#{app_name}'
          ENV['EC_RIDER_HOME'] = nil
          @config.#{app_name}_root.should == '/home/xec/#{app_name}'
        end

        it "should be able to access with EC_RIDER_HOME" do
          ENV['#{env_base}_ROOT'] = nil
          ENV['EC_RIDER_HOME'] = '/home/xec/'
          @config.#{app_name}_root.should == '/home/xec/#{app_name}'
        end

        it "should be able to access without ENV" do
          ENV['#{env_base}_ROOT'] = nil
          ENV['EC_RIDER_HOME'] = nil
          @config.#{app_name}_root.should == File.join(RAILS_ROOT, '..', '#{app_name}')
        end
      end
    EOS
  end

  csv_dir_names = %w(users orders).each do |csv_dir_name|

    describe "access csv_#{csv_dir_name}_dir" do

      csv_dir = File.join(RAILS_ROOT, "tmp", "csv")
      tmp_dir = File.join(RAILS_ROOT, "tmp", "csv", csv_dir_name)

      env_dir = "ECR_CSV_#{csv_dir_name.upcase}_DIR"

      instance_eval(<<-"EOS", __FILE__, __LINE__)
        it "normal access" do
          @config.csv_#{csv_dir_name}_dir = "#{tmp_dir}"
          @config.csv_#{csv_dir_name}_dir.should  == "#{tmp_dir}"
        end

        it "should return csv_dir/ #{csv_dir_name}" do
          ENV['#{env_dir}'] = nil
          @config.csv_dir = '#{csv_dir}'
          @config.csv_#{csv_dir_name}_dir.should  == '#{csv_dir}/#{csv_dir_name}'
        end

        it "should return ENV['#{env_dir}}']" do
          ENV['#{env_dir}'] = "/path/to/dir/by/env/csv/#{csv_dir_name}_dir"
          @config.csv_#{csv_dir_name}_dir.should  == "/path/to/dir/by/env/csv/#{csv_dir_name}_dir"
        end

        it "should return ENV['ECR_CSV_DIR']/ #{csv_dir_name}" do
          ENV['#{env_dir}'] = nil
          ENV['ECR_CSV_DIR'] = "/path/to/dir/by/env/csv_dir"
          @config.csv_#{csv_dir_name}_dir.should  == "/path/to/dir/by/env/csv_dir/#{csv_dir_name}"
        end
      EOS
    end
  end



end
