# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe Distributor do
  fixtures :distributors, :distributor_delivery_times, :distributor_delivery_time_displays
  it "test to_find_options" do
    condition = Distributor::SearchInfo.new(:distributor_name => '物流')
    option = {:page => 1, :per_page => 10}
    c = condition.to_find_options(option)
    c[:page].should == 1
    c[:per_page].should == 10
    c[:conditions].should == ["distributors.name like ?", '%物流%']
  end
  describe "validation" do
    it "名称は必須条件であること" do
      dist = Distributor.new(:name => nil)
      dist.valid?.should == false
    end
    
    it "お届け日指定範囲が入力されている場合、両方必須チェックされること" do
      dist = Distributor.new(:deliverable_term_start => 4)
      dist.valid?.should == false
    end
    
    it "お届け日指定範囲が入力されている場合、数値チェックされること" do
      dist = Distributor.new(:deliverable_term_start => "ttt")
      dist.valid?.should == false
    end
    
    it "お届け日指定範囲が入力されている場合、値チェックされること" do
      dist = Distributor.new(:deliverable_term_start => -100)
      dist.valid?.should == false
    end
    
    it "お届け日指定範囲が入力されている場合、範囲チェックされること" do
      dist = Distributor.new(:deliverable_term_start => 3, :deliverable_term_end => 1)
      dist.valid?.should == false
    end
  end

  describe "delivery_time_names_for_locale" do
    it "日本語を取得" do
      dist = Distributor.find(8)
      dist.delivery_time_names_for_locale('ja').should == ["日本語1", "日本語2", "日本語3", "日本語4", "日本語5", "日本語6", "日本語7", "日本語8", "日本語9", "日本語10"]
    end
    it "英語を取得" do
      dist = Distributor.find(8)
      dist.delivery_time_names_for_locale('en').should == ["英語1", "英語2", "英語3", "英語4", "英語5", "英語6", "英語7", "英語8", "英語9", "英語10"]
    end
  end
end
