# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Order::SearchInfo do

  fixtures(:users, :user_emails, :user_addresses, :countries,
    :products, :product_displays, :product_skus, :product_prices,
    :product_sku_inventories, :product_sku_secure_inventories,
    :orders, :order_shippings, :order_items,
    :kbn_codes,:kbn_code_names, :shop_settings)

  it "should rise validation error when total_amount > upper_total_amount." do
    order = Order::SearchInfo.new(:total_amount => 1000, :upper_total_amount => 999)
    order.should_not be_valid
    order.errors.on(:upper_total_amount).should == "の指定に矛盾があります。"
  end

  it "should rise validation error when wrong date." do
    order = Order::SearchInfo.new(
      :order_date_start => '2009/01/32',
      :order_date_end => '2009/13/30'
    )
    order.should_not be_valid
    order.errors.on(:order_date_start).should == "は正しくありません。"
    order.errors.on(:order_date_end).should == "は正しくありません。"
  end

  it "should rise validation error when qty_from > qty_to." do
    order = Order::SearchInfo.new(:qty_from => '1000', :qty_to => '999')
    order.should_not be_valid
    order.errors.on(:qty_to).should == "の指定に矛盾があります。"
  end

  it "配送コード" do
    condition = Order::SearchInfo.new(:order_shipping_code => '000002')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 1
  end

  it "受注金額" do
    condition = Order::SearchInfo.new(:total_amount => '10000')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 5
    condition = Order::SearchInfo.new(:upper_total_amount => '10000')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 3
    condition = Order::SearchInfo.new(:total_amount => '10000', :upper_total_amount => '20000')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 3
  end

  it "配送区分" do
    condition = Order::SearchInfo.new(:delivery_to_native_flg => '1')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 7
    condition = Order::SearchInfo.new(:delivery_to_foreign_flg => '1')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 1
    condition = Order::SearchInfo.new(:delivery_to_native_flg => '1', :delivery_to_foreign_flg => '1')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 8
  end

  it "配送予定日" do
    condition = Order::SearchInfo.new(:shipping_date_start => '2007/12/23')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 5
    condition = Order::SearchInfo.new(:shipping_date_end => '2007/12/24')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')    
    orders.length.should == 8
    condition = Order::SearchInfo.new(:shipping_date_start => '2007/12/23', :shipping_date_end => '2007/12/24')
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 3
  end

  it "テスト注文" do
    Order.update(orders(:orders_00007).id, :test_flg => true)
    condition = Order::SearchInfo.new(:test_flg => CmsPublish.target_id_by_key(:test))
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 1
  end

  it "本番注文" do
    Order.update(orders(:orders_00007).id, :test_flg => true)
    condition = Order::SearchInfo.new(:test_flg => CmsPublish.target_id_by_key(:production))
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 7
  end

  it "本番注文とテスト注文" do
    Order.update(orders(:orders_00007).id, :test_flg => true)
    condition = Order::SearchInfo.new
    orders = Order.find(:all, :include => Order::SearchInfo.create_include, :conditions => condition.create_condition, :order => 'orders.id DESC')
    orders.length.should == 8
  end
end
