# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe OrderForm do
  fixtures :distributors, :distributor_delivery_times, :distributor_delivery_time_displays, :countries, :shop_settings

  describe "in japanese." do
    describe "Validate check." do
      describe "The required check." do
        it "The required check of the send_address_index." do
          @param = {:send_address_index => "", :sent_one_place => "1"}
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:send_address_index).should == "を入力してください。"
        end
      end
      describe "A number of the figures check" do
        before(:each) do
          @free_request = ""
          @param = {:send_address_index => "1", :sent_one_place => "1"}
        end

        it "An error does not occur by a figure number of free_request check" do
          1000.times {@free_request += "A"}
          @param[:free_request] = @free_request
          order_form = OrderForm.new(@param)
          # 桁数チェック結果
          order_form.should be_valid
        end

        it "An error occurs by a figure number of free_request check" do
          1001.times {@free_request += "A"}
          @param[:free_request] = @free_request
          order_form = OrderForm.new(@param)
          # 桁数チェック結果
          order_form.should_not be_valid
          order_form.errors.on(:free_request).should == "は1000文字以内で入力してください。"
        end
      end

      describe "Credit card expire check" do
        before(:each) do
          setting = ShopSetting.find(:first, :conditions => ["setting_div = ?", ShopSetting.setting_div_id_by_key(:security_code)])
          setting.value = "false"
          setting.save!
          @param = {:send_address_index => "1", :sent_one_place => "1", :normal_payment_type => '2', :card_kind => 'visa', :card_name => "ECONE", :no_1 => "1111", :no_2 => "1111", :no_3 => "1111", :no_4 => "1111"}
        end

        it "The normal system check of the expiration date of the credit card" do
          @param[:expire_year] = "10"
          @param[:expire_month] = "09"
          order_form = OrderForm.new(@param)
          order_form.should be_valid
        end

        it "A system check of the length of the expiration date of the credit card" do
          @param[:expire_year] = "1"
          @param[:expire_month] = "9"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:expire_year).should == "は２桁の数値で入力してください。"
          order_form.errors.on(:expire_month).should == "は２桁の数値で入力してください。"
        end

        it "The system check of the number of the expiration date of the credit card" do
          @param[:expire_year] = "aa"
          @param[:expire_month] = "bb"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:expire_year).should == "は２桁の数値で入力してください。"
          order_form.errors.on(:expire_month).should == "は２桁の数値で入力してください。"
        end

        it "A system check of the validity of the expiration date of the credit card" do
          @param[:expire_year] = "10"
          @param[:expire_month] = "13"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:expire).should == "は正しくありません。"
        end

        it "amex test" do
          @param[:card_kind] = 'amex'
          model = OrderForm.new(@param)
          model.credit_card_not_amex?.should be_false
        end
      end
      describe "use_point" do
        it "整合性チェック" do
          @param = {:send_address_index => "1", :sent_one_place => "1", :normal_payment_type => '2', :card_kind => 'visa', :card_name => "ECONE", :no_1 => "1111", :no_2 => "1111", :no_3 => "1111", :no_4 => "1111"}
          @param[:use_point] = "200"
          @param[:before_total_point] = "100"
          @param[:use_point_flg] = "true"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:use_point).should == "は正しくありません。"
        end
      end
    end
  end
  
  describe "other method." do
    it "The assembling of the card number" do
      @param = {:no_1 => "1234",:no_2 => "1234",:no_3 => "1234",:no_4 => "1234"}
      test_order_form = OrderForm.new(@param)
      test_order_form.card_no.should == "1234123412341234"
    end
  end

  describe "配送希望日時" do
    before(:each) do
      @distributor = Distributor.first
      countory = Country.find_by_code("JP")
      countory.distributor_id = @distributor.id
      countory.save!
    end

    it "配送希望日の一覧を取得する（引数に国を指定していない場合は、「日本」の設定が返却される）" do
      delivery_start = @distributor.deliverable_term_start
      delivery_end = @distributor.deliverable_term_end
      available_delivery_date_count = (delivery_end - delivery_start) + 1

      order = OrderForm.new()
      available_delivery_date = order.available_delivery_date
      # 配送希望日を未選択とするブランクが追加さてているので「 + 1」しています
      available_delivery_date.count.should == available_delivery_date_count + 1
    end

    it "配送希望時間の一覧を表示する（引数に国を指定していない場合は、「日本」の設定が返却される）" do
      distributor_id = @distributor.id

      order = OrderForm.new()
      available_delivery_time = order.available_delivery_time
      available_delivery_time.each do |dt|
        dt.distributor_id.should == distributor_id
        dt.displays.should_not be_empty
      end
    end
  end
  describe "use_point?" do
    it "trueが入っている場合trueを返却する" do
      order_form = OrderForm.new(:use_point_flg => "true")
      order_form.use_point?.should be_true
    end
    it "falseが入っている場合falseを返却する" do
      order_form = OrderForm.new(:use_point_flg => "false")
      order_form.use_point?.should be_false
    end
    it "何も入っていない場合falseを返却する" do
      order_form = OrderForm.new(:use_point_flg => nil)
      order_form.use_point?.should be_false
    end
  end
  describe 'キャンペーン対象日時' do
    it "キャンペーンの対象日時が現在日時の場合trueを返す" do
      order = OrderForm.new()
      order.campaign_attached_div = OrderForm.campaign_attached_div_id_by_key(:now)
      order.campaign_attached_now?.should be_true
    end
    it "キャンペーンの対象日時が現在日時でない場合falseを返す" do
      order = OrderForm.new()
      order.campaign_attached_div = OrderForm.campaign_attached_div_id_by_key(:select)
      order.campaign_attached_now?.should be_false
    end
  end
  describe "security_code" do
    before do
      @order_form = OrderForm.new(
        :normal_payment_type_key => :credit , :card_kind => "1", :expire_year => "10",
        :expire_month => "01", :card_name => "test", :card_security_code => "1111",
        :no_1 => 1111, :no_2 => 1111, :no_3 => 1111, :no_4 => 1111
      )
    end
    describe "validates_numericality_of" do
      it "文字列　エラーになること" do
        @order_form.card_security_code = "aaaa"
        @order_form.should_not be_valid
        @order_form.errors.on(:card_security_code).should == "は数値で入力してください。"
      end
      it "数字　エラーにならないこと" do
        @order_form.card_security_code = "1111"
        @order_form.should be_valid
        @order_form.errors.on(:card_security_code).should_not == "は数値で入力してください。"
      end
    end
    describe "validates_length_of" do
      it "2桁　エラーになること" do
        @order_form.card_security_code = "11"
        @order_form.should_not be_valid
        @order_form.errors.on(:card_security_code).should == "は３桁または４桁の数値で入力してください。"
      end
      it "3桁　エラーにならないこと" do
        @order_form.card_security_code = "111"
        @order_form.should be_valid
        @order_form.errors.on(:card_security_code).should_not == "は３桁または４桁の数値で入力してください。"
      end
      it "4桁　エラーにならないこと" do
        @order_form.card_security_code = "1111"
        @order_form.should be_valid
        @order_form.errors.on(:card_security_code).should_not == "は３桁または４桁の数値で入力してください。"
      end
      it "5桁　エラーになること" do
        @order_form.card_security_code = "11111"
        @order_form.should_not be_valid
        @order_form.errors.on(:card_security_code).should == "は３桁または４桁の数値で入力してください。"
      end
    end
    describe "validates_presence_of" do
      before do
        ShopSetting.delete_all
      end
      it "クレジットカード以外の場合　セキュリティコード必須条件が無効になること" do
        @order_form.normal_payment_type_key = :cod
        @order_form.should be_valid
        @order_form.errors.on(:card_security_code).should_not == "を入力してください。"
      end
      it "クレジットカードの場合かつセキュリティコードを利用しない場合　セキュリティコード必須条件が無効になること" do
        setting = ShopSetting.new(:setting_div_key => :security_code, :value => "false")
        setting.save!
        @order_form.should be_valid
        @order_form.errors.on(:card_security_code).should_not == "を入力してください。"
      end
      it "クレジットカードの場合かつセキュリティコードを利用する場合　セキュリティコード必須条件が有効になること" do
        setting = ShopSetting.new(:setting_div_key => :security_code, :value => "true")
        setting.save!
        @order_form.card_security_code = nil
        @order_form.should_not be_valid
        @order_form.errors.on(:card_security_code).should == "を入力してください。"
      end
    end
    
    describe "no" do
      before(:each) do
        @order_form = OrderForm.new(
          :normal_payment_type_key => :credit , :card_kind => "1", :expire_year => "10",
          :expire_month => "01", :card_name => "test", :card_security_code => "1111",
          :no => nil
        )
      end

      it "必須チェック" do
        # 必須チェックNG
        @order_form.should_not be_valid

        @order_form.no = '1234111111111111'
        # 必須チェックOK
        @order_form.should be_valid
      end
    end
    
  end
end
