# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe PaymentErrorHistory do
  fixtures :orders, :order_shippings, :payment_error_histories, :users, :user_addresses
  
  describe "selectable_attr" do
    it "status_div" do
      PaymentErrorHistory.status_div_options.should == [
        ["対応必要なし", '0'], 
        ["未対応", '1'], 
        ["返答待ち", '2'], 
        ["対応完了", '3']
      ]
      PaymentErrorHistory.status_div_id_by_key(:no_need).should == "0"
      PaymentErrorHistory.status_div_id_by_key(:not_yet).should == "1"
      PaymentErrorHistory.status_div_id_by_key(:waiting).should == "2"
      PaymentErrorHistory.status_div_id_by_key(:complete).should == "3"
    end
  end

  describe "検索条件テスト" do
    it "本番決済のみ" do
      form = PaymentErrorHistory::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:production))
      result = PaymentErrorHistory.find(:all, :conditions => PaymentErrorHistory.create_condition(form))
      result.length.should == 1
    end

    it "テスト決済のみ" do
      form = PaymentErrorHistory::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:test))
      result = PaymentErrorHistory.find(:all, :conditions => PaymentErrorHistory.create_condition(form))
      result.length.should == 1
    end

    it "本番決済とテスト決済" do
      form = PaymentErrorHistory::SearchConditionForm.new
      result = PaymentErrorHistory.find(:all, :conditions => PaymentErrorHistory.create_condition(form))
      result.length.should == 2
    end
  end

  describe "getter" do
    it "user" do
      peh = PaymentErrorHistory.find(1)
      peh.user.id.should == 1001
    end

    it "user_id" do
      peh = PaymentErrorHistory.find(1)
      peh.user_id.should == 1001
    end

    it "user_name" do
      peh = PaymentErrorHistory.find(1)
      peh.user_name.should == '鈴木 幸隆'
    end
  end

end
