# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe PointSetting do
fixtures :point_settings, :user_points
  describe "区分値テスト" do
    it "ポイント制度利用区分 取得テスト" do
      PointSetting.available_flg_options.should == [
          ['使用しない', false],
          ['使用する', true]
      ]
    end

    it "有効期限有無区分 取得テスト" do
      PointSetting.terminable_flg_options.should == [
          ['なし', false],
          ['あり', true]
      ]
    end

    it "有効期限基準日区分 取得テスト" do
      PointSetting.term_div_options.should == [
          ['最終ポイント発行日', '0'],
          ['日付指定',  '99']
      ]
    end

    it "有効期間区分 取得テスト" do
      PointSetting.point_period_div_options.should == [
          ['1ヶ月', '1'],
          ['6ヶ月',  '6'],
          ['12ヶ月', '12'],
          ['24ヶ月',  '24']
      ]
    end
  end

  describe "バリデーションテスト" do
    it "必須項目テスト term_div" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.term_div = ""
      point_setting.should_not be_valid
      point_setting.errors.on(:term_div).should == "を入力してください。"
    end
    it "必須項目テスト term_div" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.point_period_div = ""
      point_setting.should_not be_valid
      point_setting.errors.on(:point_period_div).should == "を入力してください。"
    end
    it "必須項目テスト term_div" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.term_div = ""
      point_setting.should_not be_valid
      point_setting.errors.on(:term_div).should == "を入力してください。"
    end
  end
  describe "有効期限表示テスト" do
    it "ポイント発行なしの場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = false
      term_day = point_setting.create_term_date(Date.today)
      term_day.should == nil
    end
    it "有効期限無しの場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = false
      term_day = point_setting.create_term_date(Date.today)
      term_day.should == nil
    end
    it "最終ポイント付与の場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.point_period_div = 1
      point_setting.term_div = PointSetting.term_div_id_by_key(:last_point)
      day = Date.strptime('2008/03/26', "%Y/%m/%d")
      term_day = point_setting.create_term_date(day)
      term_day.strftime('%Y/%m/%d').should == '2008/04/26'
    end
    it "日付指定：期限が今年の場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.point_period_div = 1
      point_setting.term_div = PointSetting.term_div_id_by_key(:manual)
      point_setting.term_day = '31'
      point_setting.term_month = '03'
      day = Date.strptime('2008/03/26', "%Y/%m/%d")
      term_day = point_setting.create_term_date(day)
      term_day.strftime('%Y/%m/%d').should == '2008/03/31'
    end
    it "日付指定：期限が来年の場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.point_period_div = 1
      point_setting.term_div = PointSetting.term_div_id_by_key(:manual)
      point_setting.term_day = '31'
      point_setting.term_month = '03'
      day = Date.strptime('2008/04/26', "%Y/%m/%d")
      term_day = point_setting.create_term_date(day)
      term_day.strftime('%Y/%m/%d').should == '2009/03/31'
    end
  end
  
  describe "update_and_all_user_point_term_date" do    
    it "ポイント設定なし 期待値：有効期限が削除されること" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.save!
      point_setting.available_flg = false
      point_setting.update_and_all_user_point_term_date
      [1001,1002,1003,1004,1005].each do |user_id|
        results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
        results.each do |result|
          if result.expired_flg == true
            result.term_date.should_not == nil
          else
            result.term_date.should == nil
          end
        end
      end
    end
    
    it "ポイント設定あり 有効期限なし 期待値：有効期限切れ以外のポイントの有効期限が削除されること" do
      UserPoint.find(:all).each do |user_point|
        user_point.term_date.should_not == ""
      end
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = false
      point_setting.save!
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.terminable_flg = false
      point_setting.update_and_all_user_point_term_date
      [1001,1002,1003,1004,1005].each do |user_id|
        results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
        results.each do |result|
          if result.expired_flg == true
            result.term_date.should_not == nil
          else
            result.term_date.should == nil
          end
        end
      end
    end
    
    it "ポイント設定あり 有効期限あり 日付指定 期待値：有効期限が変更されること" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.term_div_key = :manual
      point_setting.term_month = 1
      point_setting.term_day = 1
      Date.stub!(:today).and_return(Date.new(2010,5,7))
      point_setting.update_and_all_user_point_term_date
      [1001,1002,1003,1004,1005].each do |user_id|
        results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
        results.each do |result|
          if result.expired_flg == true
            result.term_date.should_not == Time.local(2011,1,1,23,59,59)
          else
            result.term_date.should == Time.local(2011,1,1,23,59,59)
          end
        end
      end
    end
    
    it "ポイント設定あり 有効期限あり 最終ポイント発行日 24ヶ月⇒12ヶ月に変更 期待値：有効期限が増えること" do
      point_setting = PointSetting.find(:first)
      point_setting.point_period_div_key = 24
      point_setting.save!
      
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.term_div_key = :last_point
      point_setting.point_period_div_key = 12
      Date.stub!(:today).and_return(Date.new(2010,5,7))
      point_setting.update_and_all_user_point_term_date

      user_id = 1001
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2009,1,4,23,59,59)
      results[1].term_date.should == Time.local(2009,1,4,23,59,59)
      results[2].term_date.should == Time.local(2009,1,4,23,59,59)
      results[3].term_date.should == Time.local(2009,1,4,23,59,59)

      user_id = 1002
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2009,1,6,23,59,59)
      results[1].term_date.should == Time.local(2009,1,6,23,59,59)

      user_id = 1003
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2009,1,8,23,59,59)
      results[1].term_date.should == Time.local(2009,1,8,23,59,59)

      user_id = 1004
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should_not == Time.local(2009,1,12,23,59,59)
      results[0].expired_flg.should be_true
      results[1].term_date.should_not == Time.local(2009,1,12,23,59,59)
      results[1].expired_flg.should be_true
      results[2].term_date.should == Time.local(2009,1,12,23,59,59)
      results[3].term_date.should == Time.local(2009,1,12,23,59,59)

      user_id = 1005
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2009,1,13,23,59,59)
    end
    
    it "ポイント設定あり 有効期限あり 最終ポイント発行日 12ヶ月⇒24ヶ月に変更 期待値：有効期限が減ること 有効期限切れになったポイントは有効期限切れになること" do
      UserPoint.create(:given_point =>100,:used_point => 20, :user_id => 1001, :expired_flg =>true,
          :term_date => Time.local(2010,3,26,23,59,59))
      UserPoint.create(:given_point => -80,:used_point => 0, :user_id => 1001, :expired_flg =>true,
          :term_date => Time.local(2010,3,26,23,59,59))
      point_setting = PointSetting.find(:first)
      point_setting.point_period_div_key = 12
      point_setting.save!
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.term_div_key = :last_point
      point_setting.point_period_div_key = 24
      Date.stub!(:today).and_return(Date.new(2010,5,7))
      point_setting.update_and_all_user_point_term_date

      user_id = 1001
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2010,1,4,23,59,59)
      results[1].term_date.should == Time.local(2010,1,4,23,59,59)
      results[2].term_date.should == Time.local(2010,1,4,23,59,59)
      results[3].term_date.should == Time.local(2010,1,4,23,59,59)

      user_id = 1002
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2010,1,6,23,59,59)
      results[1].term_date.should == Time.local(2010,1,6,23,59,59)

      user_id = 1003
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2010,1,8,23,59,59)
      results[1].term_date.should == Time.local(2010,1,8,23,59,59)

      user_id = 1004
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should_not == Time.local(2010,1,12,23,59,59)
      results[0].expired_flg.should be_true
      results[1].term_date.should_not == Time.local(2010,1,12,23,59,59)
      results[1].expired_flg.should be_true
      results[2].term_date.should == Time.local(2010,1,12,23,59,59)
      results[3].term_date.should == Time.local(2010,1,12,23,59,59)

      user_id = 1005
      results = UserPoint.find(:all, :conditions => ["user_id = ?", user_id], :order => "id")
      results[0].term_date.should == Time.local(2010,1,13,23,59,59)
    end
  end
end
