# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Product::DialogForm do
  fixtures :products, :product_displays, :product_categories, :product_category_displays, :product_prices,
    #:shipping_costs,
  :kbn_codes, :kbn_code_names, :product_skus, :user_ranks,
    :product_variation_displays, :product_variation_item_displays, :product_variation_items, :product_variations

  it "with empty search_word" do
    dialog = Product::DialogForm.new(:search_word => "", :present_cd => "", :per_page => '20')
    dialog.to_paginate_options.should == {
      :page => 1,
      :per_page => 20,
      :order => 'products.display_order is null, products.display_order, products.product_code'
    }
    lambda{ Product.paginate(:all, dialog.to_paginate_options) }.should_not raise_error
  end

  it "with search_word" do
    dialog = Product::DialogForm.new(:search_word => "花火", :present_cd => "", :per_page => '20')
    dialog.to_paginate_options.should == {
      :page => 1,
      :per_page => 20,
      :conditions => [
        "(products.product_code like ? OR products.order_product_name_1 like ? OR products.order_product_name_2 like ? OR products.kana_product_name like ?)",
        "%花火%", "%花火%", "%花火%", "%花火%"],
      :order => 'products.display_order is null, products.display_order, products.product_code'
    }
    lambda{ Product.paginate(:all, dialog.to_paginate_options) }.should_not raise_error
  end

  it "with search_word with present only" do
    dialog = Product::DialogForm.new(:search_word => "花火", :present_cd => "1", :per_page => '20')
    dialog.to_paginate_options.should == {
      :page => 1,
      :per_page => 20,
      :conditions => [
        "(products.product_code like ? OR products.order_product_name_1 like ? OR products.order_product_name_2 like ? OR products.kana_product_name like ?) AND products.present_flg = ?",
        "%花火%", "%花火%", "%花火%", "%花火%", 1],
      :order => 'products.display_order is null, products.display_order, products.product_code'
    }
    lambda{ Product.paginate(:all, dialog.to_paginate_options) }.should_not raise_error
  end

  it "with search_word except present" do
    dialog = Product::DialogForm.new(:search_word => "花火", :present_cd => "2", :per_page => '20')
    dialog.to_paginate_options.should == {
      :page => 1,
      :per_page => 20,
      :conditions => [
        "(products.product_code like ? OR products.order_product_name_1 like ? OR products.order_product_name_2 like ? OR products.kana_product_name like ?) AND (products.present_flg = ? OR products.present_flg IS NULL)",
        "%花火%", "%花火%", "%花火%", "%花火%", 0],
      :order => 'products.display_order is null, products.display_order, products.product_code'
    }
    lambda{ Product.paginate(:all, dialog.to_paginate_options) }.should_not raise_error
  end

end
