# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Product::SearchCondition do
  fixtures :products, :product_displays, :product_categories, :product_category_displays, :product_prices,
    #:shipping_costs,
  :kbn_codes, :kbn_code_names, :product_skus, :user_ranks,
    :product_variation_displays, :product_variation_item_displays, :product_variation_items, :product_variations

  describe "to_find_options" do
    it "default" do
      condition = Product::SearchCondition.new
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 46
    end

    it "with product_code" do
      condition = Product::SearchCondition.new(:product_code => '0000000009040')
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 1
      products[0].id.should == 6
    end

    it "with product_name" do
      condition = Product::SearchCondition.new(:product_name => 'ミニカー')
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 1
      products[0].id.should == 17
    end

    it "with variation" do
      condition = Product::SearchCondition.new(:variation => '1')
      products = Product.find(:all, condition.to_find_options.merge(:order => 'id'))
      products.size.should == 4
      products[0].id.should == 100002
      products[1].id.should == 100052
    end

   it "with default order user_rank_id" do
      product = Product.find(100053)
      product.update_attribute("display_order", nil)
      #おすすめ順(通常)
      # display_orderの表示順に表示
      # display_orderに同じ値が登録されている商品は、product_code順に表示する
      # display_orderがnilの場合は一番最後に表示し、複数ある場合はproduct_code順に表示する
      condition = Product::SearchCondition.new(:variation => '1', :order => '0', :user_rank_id => '1')
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 5
      products[0].id.should == 100052
      products[0].display_order.should == 26
      products[0].product_code.should == "0000000000052"
      products[1].id.should == 100054
      products[1].display_order.should == 26
      products[1].product_code.should == "0000000000054"
      products[2].id.should == 100054
      products[2].display_order.should == 26
      products[3].id.should == 100002
      products[3].display_order.should == 53
    end

    it "with order user_rank_id" do
      #価格の安い順
      condition = Product::SearchCondition.new(:variation => '1', :order => '1', :user_rank_id => '1')
      condition.to_find_options[:order].should == "cond_product_prices.standard_price, products.display_order is null, products.display_order, products.product_code"
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 5
      products[0].id.should == 100053
      products[1].id.should == 100054
    end

    it "with order user_rank_id" do
      #価格の高い順
      condition = Product::SearchCondition.new(:variation => '1', :order => '2', :user_rank_id => '1')
      condition.to_find_options[:order].should == "cond_product_prices.standard_price desc, products.display_order is null, products.display_order, products.product_code"
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 5
      products[0].id.should == 100052
      products[1].id.should == 100053
    end

    it "with order user_rank_id" do
      #新着順
      condition = Product::SearchCondition.new(:variation => '1', :order => '3', :user_rank_id => '1')
      condition.to_find_options[:order].should == "products.publish_started_at desc, products.display_order is null, products.display_order, products.product_code"
      products = Product.find(:all, condition.to_find_options)
      products.size.should == 5
      products[0].id.should == 100053
      products[1].id.should == 100052
    end
  end
  
  describe "to_paginate_options" do
    it "default" do
      condition = Product::SearchCondition.new
      condition.disp_count.should == 20
      condition.per_page.should == 20
      condition.page.should == 1
      condition.to_paginate_options.should == {
        :page => 1,
        :per_page => 20,
        :conditions => nil, :joins => "",
        :order => 'products.display_order is null, products.display_order, products.product_code'
      }
    end

    it "with product_code" do
      condition = Product::SearchCondition.new(:product_code => '0000000009040', :page => 2)
      condition.to_paginate_options.should == {
        :per_page => 20, :page => 2,
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["products.product_code like ?", '%0000000009040%'],
        :joins => ""
      }
    end

    it "with variation" do
      condition = Product::SearchCondition.new(:variation => '1', :page => 2)
      condition.to_paginate_options.should == {
        :per_page => 20, :page => 2,
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(variation_1_id = ? OR variation_2_id = ? OR variation_3_id = ?)", 1, 1, 1],
        :joins => ""
      }
    end

    it "with product_code and per_page" do
      condition = Product::SearchCondition.new(:product_code => '0000000009040', :page => 2, :per_page => 30)
      condition.to_paginate_options.should == {
        :per_page => 30, :page => 2,
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["products.product_code like ?", '%0000000009040%'],
        :joins => ""
      }
    end

  end
end
