# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductCategory do

  before(:each) do
    @valid_attributes = {
      :parent_id => nil,
      :name_for_admin => "value for name_for_admin",
      :code => "code"
    }
  end

  it "should create a new instance given valid attributes" do
    ProductCategory.create!(@valid_attributes)
  end

  describe "without fixtures" do
    
    def build_test_tree
      fashion = ProductCategory.create!(:display_order => 1, :name_for_admin => 'ファッション', :code => 'fashion')
      fashion.save_displays(:ja => 'ファッション', :en => 'fashion')
      fashion.children.create(:display_order => 1, :name_for_admin => 'トップス', :code => 'tops').save_displays(:ja => 'トップス', :en => 'tops')
      fashion.children.create(:display_order => 2, :name_for_admin => 'ボトムス', :code => 'bottoms').save_displays(:ja => 'ボトムス', :en => 'bottoms')
      fashion.children.create(:display_order => 3, :name_for_admin => 'ゆかた', :code => 'yukata').save_displays(:ja => 'ゆかた', :en => 'yukata')
      fashion.children.create(:display_order => 4, :name_for_admin => 'バッグ', :code => 'bag').save_displays(:ja => 'バッグ', :en => 'bag')
      fashion.children.create(:display_order => 5, :name_for_admin => 'アクセサリー', :code => 'accessory').save_displays(:ja => 'アクセサリー', :en => 'accessory')
      fashion.children.create(:display_order => 6, :name_for_admin => 'ウィッグ', :code => 'wig').save_displays(:ja => 'ウィッグ', :en => 'wig')
      art = ProductCategory.create!(:display_order => 2, :name_for_admin => 'アート', :code => 'art')
      art.save_displays(:ja => 'アート', :en => 'art')
      art.children.create(:display_order => 1, :name_for_admin => 'フィギュア', :code => 'figure').save_displays(:ja => 'フィギュア', :en => 'figure')
      art.children.create(:display_order => 2, :name_for_admin => 'イラスト', :code => 'illust').save_displays(:ja => 'イラスト', :en => 'illustrations')
    end

    describe "parents" do
      before(:each) do
        ProductCategory.destroy_all
        ProductCategory.count.should == 0
        build_test_tree
      end

      it "親要素を配列で取得する" do
        pc1 = ProductCategory.find_by_name_for_admin("トップス")
        pc1.parents.should == [ProductCategory.find_by_name_for_admin("ファッション")]

        pc2 = ProductCategory.find_by_name_for_admin("フィギュア")
        pc2.parents.should == [ProductCategory.find_by_name_for_admin("アート")]
      end

      it "親要素が無い場合は空の配列" do
        pc1 = ProductCategory.find_by_name_for_admin("ファッション")
        pc1.parents.should == []
      end
    end

    describe "building category tree" do
      it "default" do
        ProductCategory.destroy_all("parent_id is null")
        # childrenは:dependent => :destroyが設定されているので、子要素は全て削除されます。
        ProductCategory.count.should == 0
        # displaysも:dependent => :destroyが設定されているので、子要素は全て削除されます。
        ProductCategoryDisplay.count.should == 0

        build_test_tree
        ProductCategory.count(:conditions => {:parent_id => nil}).should == 2

        fashion = ProductCategory.find_by_name_for_admin("ファッション")
        fashion.should_not be_nil
        fashion.children.count.should == 6
        fashion.displays.count.should == 2
        fashion.display_names.should == {'ja' => 'ファッション', 'en' => 'fashion'}
        I18n.locale = 'ja'
        fashion.display_name.should == 'ファッション'
        fashion.display_name('ja').should == 'ファッション'
        fashion.display_name('en').should == 'fashion'
        I18n.locale = 'en'
        fashion.display_name.should == 'fashion'
        fashion.display_name('ja').should == 'ファッション'
        fashion.display_name('en').should == 'fashion'
        I18n.locale = 'ko'
        fashion.display_name.should == 'ファッション'
        fashion.display_name('ja').should == 'ファッション'
        fashion.display_name('en').should == 'fashion'

        art = ProductCategory.find_by_name_for_admin("アート")
        art.should_not be_nil
        art.children.count.should == 2
        art.displays.count.should == 2
        art.display_names.should == {'ja' => 'アート', 'en' => 'art'}

        art.save_displays(:en => 'Art', :'zh_TW' => '藝術', :'zh_CN' => '芸術')
        art.displays.count.should == 3
        art.display_names.should == {'en' => 'Art', 'zh_TW' => '藝術', 'zh_CN' => '芸術'}
        I18n.locale = 'ja'
        art.display_name.should == nil
        art.display_name('ja').should == nil
        art.display_name('en').should == 'Art'
        art.display_name('zh-TW').should == '藝術'
        art.display_name('zh-CN').should == '芸術'
        I18n.locale = 'zh-CN'
        art.display_name.should == '芸術'
        art.display_name('ja').should == nil
        art.display_name('en').should == 'Art'
        art.display_name('zh-TW').should == '藝術'
        art.display_name('zh-CN').should == '芸術'
        I18n.locale = 'th'
        art.display_name.should == nil
        art.display_name('ja').should == nil
        art.display_name('en').should == 'Art'
        art.display_name('zh-TW').should == '藝術'
        art.display_name('zh-CN').should == '芸術'

      end
    end
  end

  describe "with fixtures" do
    # 秋間コメント
    # フィクスチャが足りていませんでした
    fixtures :products, :product_displays, :product_categories, :product_category_displays

    describe "get_staff_emails_for_cc" do
      it "all categoriy email not null" do
        product = Product.find(100048)
        bcc_list = ProductCategory.get_staff_emails_for_cc(product)
        bcc_list.should == ['m-user@ec-one.com', 'm-user2@ec-one.com', 'm-user3@ec-one.com', 'm-user4@ec-one.com', 'm-user5@ec-one.com']
      end
      it "one categoriy email null" do
        product = Product.find(100049)
        bcc_list = ProductCategory.get_staff_emails_for_cc(product)
        bcc_list.should == ['m-user@ec-one.com', 'm-user3@ec-one.com', 'm-user4@ec-one.com', 'm-user5@ec-one.com']
      end
      it "all categoriy email null" do
        product = Product.find(100050)
        bcc_list = ProductCategory.get_staff_emails_for_cc(product)
        bcc_list.should == ['m-user2@ec-one.com']
      end
      it "all parent categoriy email null" do
        product = Product.find(100051)
        bcc_list = ProductCategory.get_staff_emails_for_cc(product)
        bcc_list.should == []
      end
      it "all categoriy null" do
        product = Product.find(100046)
        bcc_list = ProductCategory.get_staff_emails_for_cc(product)
        bcc_list.should == []
      end
    end

    describe "ancestor_ids" do
      it "should not include self include_self = false" do
        category = ProductCategory.find(15)
        category.ancestor_ids.should == [13]
      end

      it "should not include self include_self = true" do
        category = ProductCategory.find(15)
        category.ancestor_ids(true).should == [13, 15]
      end
    end

    describe "get_pankuzu_list" do
      it "cateogry_id not null" do
        ProductCategory.find(15).should_not be_nil
        pankuzu_list = ProductCategory.get_pankuzu_list("code15")
        pankuzu_list.should_not be_nil
        pankuzu_list[0].name.should == '電化製品・カメラ'
        pankuzu_list[0].product_category.url.should == '/list'
        pankuzu_list[1].name.should == 'パソコン'
        pankuzu_list[1].product_category.url.should == '/list'
      end
      it "cateogry_id null" do
        pankuzu_list = ProductCategory.get_pankuzu_list(nil)
        pankuzu_list.should == nil
      end
    end
    describe "descendant_ids" do
      describe "最上層カテゴリの場合" do
        it "子孫をすべて取得" do
          product_category = ProductCategory.find(14)
          product_category.descendant_ids.sort.should == [16, 18, 19, 20, 21, 22, 23]
          ProductCategory.descendant_ids('code14').sort.should == [16, 18, 19, 20, 21, 22, 23]
        end
        it "子孫とそのカテゴリ自身を取得" do
          product_category = ProductCategory.find(14)
          product_category.descendant_ids(true).sort.should == [14, 16, 18, 19, 20, 21, 22, 23]
          ProductCategory.descendant_ids('code14', true).sort.should == [14, 16, 18, 19, 20, 21, 22, 23]
        end
      end
      describe "中層のカテゴリの場合" do
        it "子孫をすべて取得" do
          product_category = ProductCategory.find(20)
          product_category.descendant_ids.sort.should == [16,18,19]
          ProductCategory.descendant_ids('code20').sort.should == [16, 18, 19]
        end
        it "子孫とそのカテゴリ自身を取得" do
          product_category = ProductCategory.find(20)
          product_category.descendant_ids(true).sort.should == [16,18,19,20]
          ProductCategory.descendant_ids('code20', true).sort.should == [16,18,19,20]
        end
      end
      describe "最下層のカテゴリの場合" do
        it "子孫をすべて取得" do
          product_category = ProductCategory.find(16)
          product_category.descendant_ids.sort.should == []
          ProductCategory.descendant_ids('code16').sort.should == []
        end
        it "子孫とそのカテゴリ自身を取得" do
          product_category = ProductCategory.find(16)
          product_category.descendant_ids(true).sort.should == [16]
          ProductCategory.descendant_ids('code16', true).sort.should == [16]
        end
      end
    end

  end
  
  describe "test find_all_categories" do
    fixtures :product_categories, :product_category_displays
    it "全てのカテゴリを入れ子の配列で取得する" do
      categories = ProductCategory.find_all_categories
      
      categories.size.should == 7
      categories.flatten.size.should == 15
        
      categories[0].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1 = categories[0]
      category_1.id.should == 13
      category_1.code.should == "code13"
      category_1.name.should == "電化製品・カメラ"
      
      categories[1].is_a?(Array).should be_true
      categories[1].size.should == 3
      
      categories[1][0].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1_1 = categories[1][0]
      category_1_1.id.should == 15
      category_1_1.code.should == "code15"
      category_1_1.name.should == "パソコン"
      
      categories[1][1].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1_2 = categories[1][1]
      category_1_2.id.should == 17
      category_1_2.code.should == "code17"
      category_1_2.name.should == "カメラ"
      
      categories[1][2].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1_3 = categories[1][2]
      category_1_3.id.should == 24
      category_1_3.code.should == "code24"
      category_1_3.name.should == "その他"
      
      categories[2].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_2 = categories[2]
      category_2.id.should == 14
      category_2.code.should == "code14"
      category_2.name.should == "インテリア・生活雑貨"
    end
    
    it "全てのカテゴリをフラットな配列で取得する" do
      categories = ProductCategory.find_all_categories('ja', nil, '　　')
      
      categories.size.should == 15
      categories.flatten.size.should == 15
        
      categories[0].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1 = categories[0]
      category_1.id.should == 13
      category_1.code.should == "code13"
      category_1.name.should == "電化製品・カメラ"

      categories[1].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1_1 = categories[1]
      category_1_1.id.should == 15
      category_1_1.code.should == "code15"
      category_1_1.name.should == "　　パソコン"
      
      categories[2].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1_2 = categories[2]
      category_1_2.id.should == 17
      category_1_2.code.should == "code17"
      category_1_2.name.should == "　　カメラ"
      
      categories[3].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_1_3 = categories[3]
      category_1_3.id.should == 24
      category_1_3.code.should == "code24"
      category_1_3.name.should == "　　その他"
      
      categories[4].is_a?(ProductCategory::CategoryDisplayAtLocale).should be_true
      category_2 = categories[4]
      category_2.id.should == 14
      category_2.code.should == "code14"
      category_2.name.should == "インテリア・生活雑貨"
    end
  end

end
