# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe ProductFaq do
  fixtures :product_faqs, :product_faq_displays, :product_faq_relation_products, :products
  describe "invisible_flg_name" do
    it "invisible_flg ture" do
      pf = ProductFaq.new()
      pf.invisible_flg = true
      pf.invisible_flg_name.should == "非公開"
    end
    it "invisible_flg false" do
      pf = ProductFaq.new()
      pf.invisible_flg = false
      pf.invisible_flg_name.should == "公開"
    end
  end

  describe "product_codes" do
    it "product_codes" do
      pf = ProductFaq.find(1)
      pf.product_codes.should == ["0000000009051"]
    end
  end

  describe "product_names" do
    it "product_names" do
      pf = ProductFaq.find(1)
      pf.product_names.should == ["錠剤100粒"]
    end
  end

  describe "inquiry" do
    it "日本語の場合" do
      pf = ProductFaq.find(1)
      pf.inquiry.should == "この商品についてお問合せします。"
    end
    it "日本語以外" do
      pf = ProductFaq.find(2)
      pf.inquiry.should == "中国語(繁体字)この商品についてお問合せします。"
    end
  end

  describe "answer" do
    it "reration" do
      pf = ProductFaq.find(1)
      pf.answer.should == "回答します。"
    end
  end

  describe "locales" do
    it "reration" do
      pf = ProductFaq.find(2)
      pf.locales.should == ['zh_TW', 'en']
    end
  end
  it "test create_condition" do
    condition = ProductFaq::SearchInfo.new(:product_code => '0000000009051', :product_name => '錠剤', :inquiry => 'お問合せ', :answer => '回答します')
    option = {:page => 1, :per_page => 10, :include => [:inquiry_memos], :order => "product_faqs.created_at DESC"}
    c = condition.to_find_options(option)
    c[:page].should == 1
    c[:per_page].should == 10
    c[:include].should == [:inquiry_memos]
    c[:order].should == "product_faqs.created_at DESC"
    c[:conditions].should == ["products.product_code like ? AND products.order_product_name_1 like ? AND product_faq_displays.inquiry like ? AND product_faq_displays.answer like ?",
    '%0000000009051%','%錠剤%','%お問合せ%', '%回答します%'
    ]
  end

  describe "バリデーションテスト　validate_for_product_faq" do
    it "valid? false" do
      pf = ProductFaq.find(2)
      pf.display_order = "aa"
      flg, product_display = pf.validate_for_product_faq
      flg.should == false
    end
    it "言語別入力がすべて未入力の場合" do
      pf = ProductFaq.find(2)
      pf.displays.each do |display|
        display.inquiry = ""
        display.inquiry_user = ""
        display.answer = ""
        display.answer_user = ""
      end
      flg, product_display = pf.validate_for_product_faq
      flg.should == false
    end
    it "商品が１つも登録していない場合 すべて削除対象の場合" do
      pf = ProductFaq.find(2)
      pf.relation_products.each do |r_p|
        r_p.mark_for_destruction
      end
      flg, product_display = pf.validate_for_product_faq
      flg.should == false
    end
  end
  describe "old_product_mark_for_destruction" do
    it "引数なしの場合" do
      pf = ProductFaq.find(1)
      pf.old_product_mark_for_destruction
      pf.relation_products.each do |target|
        target.marked_for_destruction?.should == nil
      end
    end
    it "引数ありの場合" do
      pf = ProductFaq.find(1)
      pf.old_product_mark_for_destruction(1)
      pf.relation_products.each do |target|
        target.marked_for_destruction?.should == true
      end
    end
  end

  describe "to_find_options" do
    before do
      @condition = ProductFaq::SearchInfo.new()
    end
    it "全件取得" do
      result = ProductFaq.paginate(
        @condition.to_find_options(:page => 1, :per_page => 10, :order => "product_faqs.created_at DESC")
      )
      result.size.should == 6
    end
    it "商品コード条件" do
      @condition.product_code = "0000000009051"
      result = ProductFaq.paginate(
        @condition.to_find_options(:page => 1, :per_page => 10, :order => "product_faqs.created_at DESC")
      )
      result.size.should == 2
    end
    it "商品名条件" do
      @condition.product_name ="インテリア"
      result = ProductFaq.paginate(
        @condition.to_find_options(:page => 1, :per_page => 10, :order => "product_faqs.created_at DESC")
      )
      result.size.should == 4
    end
    it "問い合わせ内容条件" do
      @condition.inquiry = "非公開"
      result = ProductFaq.paginate(
        @condition.to_find_options(:page => 1, :per_page => 10, :order => "product_faqs.created_at DESC")
      )
      result.size.should == 1
    end
    it "回答内容条件" do
      @condition.answer = "【日本語】"
      result = ProductFaq.paginate(
        @condition.to_find_options(:page => 1, :per_page => 10, :order => "product_faqs.created_at DESC")
      )
      result.size.should == 1
    end
  end
end
