require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductGroupSku do
  fixtures :products, :product_skus, :product_groups, :product_group_skus, :product_displays, :product_prices, :user_ranks
  before do
     EcRider.config.stub!(:ec_admin).and_return(false)
     EcRider.config.stub!(:ec_test).and_return(false)
  end
  
  describe "find_by_include_sku_id" do
    it "すべて" do
      skus = ProductGroupSku.find_by_include_sku_id(nil, "all")
      skus.size.should == 4
      skus[0].product_sku_id.should == 282
      skus[1].product_sku_id.should == 283
      skus[2].product_sku_id.should == 285
      skus[3].product_sku_id.should == 284
    end
    
    it "自身を含むグループ" do
      skus = ProductGroupSku.find_by_include_sku_id(282)
      skus.size.should == 3
      skus[0].product_sku_id.should == 283
      skus[1].product_sku_id.should == 285
      skus[2].product_sku_id.should == 284
    end
    
    it "指定したグループ" do
      skus = ProductGroupSku.find_by_include_sku_id(282, "all", "group_1")
      skus.size.should == 3
      skus[0].product_sku_id.should == 282
      skus[1].product_sku_id.should == 283
      skus[2].product_sku_id.should == 285
    end
    
    it "指定したグループで自身を含むグループ" do
      skus = ProductGroupSku.find_by_include_sku_id(283, nil, "group_1")
      skus.size.should == 2
      skus[0].product_sku_id.should == 282
      skus[1].product_sku_id.should == 285
    end
    
    it "引数不正" do
      skus = ProductGroupSku.find_by_include_sku_id()
      skus.size.should == 0
    end
    
    it "sku_id不正" do
      skus = ProductGroupSku.find_by_include_sku_id(0)
      skus.size.should == 0
    end
  end
end
