# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductSkuInventory do
  fixtures :products, :product_displays, 
    :product_variation_displays,
    :product_variations,
    :product_variation_item_displays,
    :product_variation_items,
    :product_skus, 
    :product_sku_inventories, :product_sku_secure_inventories, :product_components

  describe "ProductSkuInventory::CreateForm" do
    describe "validation" do
      it "valid" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'out', :count => '23')
        form.valid?.should be_true
      end
      it "valid with comment" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'in', :count => '1', :comment => ('あ' * 255))
        form.valid?.should be_true
      end
      it "count is requred" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'in')
        form.valid?.should be_false
      end
      it "count can not equal 0" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'in', :count => '0')
        form.valid?.should be_false
      end
      it "count is greater than 0" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'in', :count => '-1')
        form.valid?.should be_false
      end
      it "comment is less than 255" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'in', :count => '1', :comment => ('あ' * 256))
        form.valid?.should be_false
      end
      it "在庫数を減らす場合に、引き当て分は確保しなければいけない" do
        form = ProductSkuInventory::CreateForm.new(:product_sku_id => 266, :in_out_div => 'out', :count => '24')
        form.valid?.should be_false
      end
    end
  end
  
  describe "ProductSkuInventory::SearchForm" do
    it "create_query without options" do
      conditions = ProductSkuInventory::SearchForm.new()
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 46
      
      product_sku = product_skus.first
      product_sku.id.should == 282
      product_sku.product_code.should == "0000000000001"
      product_sku.order_product_name_1.should == "商品1の注文商品名1"
      product_sku.parent_stock_count.should == 99
      product_sku.composable_stock_count.should == 25
      product_sku.secure_stock_count.should == 0
      product_sku.sku_unpaid_secure_stock_count.should == 0
      product_sku.orderable_count.should == 24
      
      assert_sku = ProductSku.find(282)
      assert_sku.parent_stock_count.should == product_sku.parent_stock_count.to_i
      assert_sku.stock_count.should == product_sku.composable_stock_count.to_i
      assert_sku.get_secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.get_secure_stock_count(:unpaid_secure_stock_count).should == product_sku.sku_unpaid_secure_stock_count.to_i
      assert_sku.get_secure_stock_count(:secure_stock_count).should == 0
      assert_sku.secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.orderable_count.should == product_sku.orderable_count.to_i
    end
    
    it "search by product_code" do
      conditions = ProductSkuInventory::SearchForm.new(:product_code => '0000000009050')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 1
      
      product_sku = product_skus.first
      product_sku.id.should == 266
      product_sku.parent_stock_count.should == 50
      product_sku.composable_stock_count.should == 50
      product_sku.secure_stock_count.should == 27
      product_sku.sku_unpaid_secure_stock_count.should == 1
      product_sku.orderable_count.should == 23
      
      assert_sku = ProductSku.find(266)
      assert_sku.parent_stock_count.should == product_sku.parent_stock_count.to_i
      assert_sku.stock_count.should == product_sku.composable_stock_count.to_i
      assert_sku.get_secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.get_secure_stock_count(:unpaid_secure_stock_count).should == product_sku.sku_unpaid_secure_stock_count.to_i
      assert_sku.get_secure_stock_count(:secure_stock_count).should == 26
      assert_sku.secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.orderable_count.should == product_sku.orderable_count.to_i
    end
    
    it "having product component" do
      i = ProductSkuInventory.find(18)
      i.count = 20
      i.save
      product_sku = ProductSku.find(282)
      product_sku.parent_stock_count = 20
      product_sku.composable_stock_count = product_sku.stock_count(:stock_count)
      product_sku.orderable_count = product_sku.stock_count(:orderable_count)
      product_sku.save!
      
      conditions = ProductSkuInventory::SearchForm.new(:product_code => '0000000000001')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 1
      
      product_sku = product_skus.first
      product_sku.id.should == 282
      product_sku.product_code.should == "0000000000001"
      product_sku.order_product_name_1.should == "商品1の注文商品名1"
      product_sku.parent_stock_count.should == 20
      product_sku.composable_stock_count.should == 25
      product_sku.secure_stock_count.should == 0
      product_sku.sku_unpaid_secure_stock_count.should == 0
      product_sku.orderable_count.should == 24
      
      assert_sku = ProductSku.find(282)
      assert_sku.parent_stock_count.should == product_sku.parent_stock_count.to_i
      assert_sku.stock_count.should == product_sku.composable_stock_count.to_i
      assert_sku.get_secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.get_secure_stock_count(:unpaid_secure_stock_count).should == product_sku.sku_unpaid_secure_stock_count.to_i
      assert_sku.get_secure_stock_count(:secure_stock_count).should == 0
      assert_sku.secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.orderable_count.should == product_sku.orderable_count.to_i
      
      p = ProductSku.find(282)
      p.parent_stockable_flg = true
      p.composable_stock_count = p.stock_count(:stock_count)
      p.orderable_count = p.stock_count(:orderable_count)
      p.save
      
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 1
      
      product_sku = product_skus.first
      product_sku.id.should == 282
      product_sku.product_code.should == "0000000000001"
      product_sku.order_product_name_1.should == "商品1の注文商品名1"
      product_sku.parent_stock_count.should == 20
      product_sku.composable_stock_count.should == 20
      product_sku.secure_stock_count.should == 0
      product_sku.sku_unpaid_secure_stock_count.should == 0
      product_sku.orderable_count.should == 20
      
      assert_sku = ProductSku.find(282)
      assert_sku.parent_stock_count.should == product_sku.parent_stock_count.to_i
      assert_sku.stock_count.should == product_sku.composable_stock_count.to_i
      assert_sku.get_secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.get_secure_stock_count(:unpaid_secure_stock_count).should == product_sku.sku_unpaid_secure_stock_count.to_i
      assert_sku.get_secure_stock_count(:secure_stock_count).should == 0
      assert_sku.secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.orderable_count.should == product_sku.orderable_count.to_i
    end
    
    it "having variations" do
      conditions = ProductSkuInventory::SearchForm.new(:product_name => '赤')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 1
      
      product_sku = product_skus.first
      product_sku.id.should == 283
      product_sku.product_code.should == "0000000000002"
      product_sku.order_product_name_1.should == "商品2の注文商品名1"
      product_sku.sku_variation_name.should == "S - 赤"
      product_sku.parent_stock_count.should == 0
      product_sku.composable_stock_count.should == 0
      product_sku.secure_stock_count.should == 0
      product_sku.sku_unpaid_secure_stock_count.should == 0
      product_sku.orderable_count.should == 0
      
      assert_sku = ProductSku.find(283)
      assert_sku.parent_stock_count.should == product_sku.parent_stock_count.to_i
      assert_sku.stock_count.should == product_sku.composable_stock_count.to_i
      assert_sku.get_secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.get_secure_stock_count(:unpaid_secure_stock_count).should == product_sku.sku_unpaid_secure_stock_count.to_i
      assert_sku.get_secure_stock_count(:secure_stock_count).should == 0
      assert_sku.secure_stock_count.should == product_sku.secure_stock_count
      assert_sku.orderable_count.should == product_sku.orderable_count.to_i
    end
    
    it "search by product_name" do
      conditions = ProductSkuInventory::SearchForm.new(:product_name => 'インテリア家具')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 1
      product_sku = product_skus.first
      product_sku.id.should == 267
    end
    
    it "search by stock_count" do
      conditions = ProductSkuInventory::SearchForm.new(:stock_count_min => '5', :stock_count_max => '5')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 2
    end
    
    it "search by orderable_count" do
      conditions = ProductSkuInventory::SearchForm.new(:orderable_count_min => '5', :orderable_count_max => '5')
      product_skus = ProductSku.find(:all,
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 2
    end
    
    it "search by out_of_stock_flg" do
      conditions = ProductSkuInventory::SearchForm.new(:out_of_stock_flg => '1')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 10
    end

    it "search by out_of_safety_stock_flg" do
      conditions = ProductSkuInventory::SearchForm.new(:out_of_safety_stock_flg => '1')
      product_skus = ProductSku.find(:all, 
        :select => ProductSkuInventory::SearchForm.create_selects('product_skus'),
        :joins => ProductSkuInventory::SearchForm.create_joins('product_skus'),
        :order => 'product_code, sku_variation_name',
        :conditions => conditions.to_paginate_condition)
      product_skus = ProductSkuSecureInventory.set_unpaid_secure_stock_count(product_skus)
      product_skus = ProductComponent.set_component_parent_sku_id(product_skus)
      product_skus.size.should == 11
    end
  end
  
  describe "get_stock_count" do
    it "get stock_count" do
      inventory = ProductSkuInventory.find(1)
      
      inventory_1 = ProductSkuInventory.new(:product_sku_id => inventory.product_sku_id, :count => 10)
      inventory_1.save
      
      inventory_2 = ProductSkuInventory.new(:product_sku_id => inventory.product_sku_id, :count => -5)
      inventory_2.save

      ProductSku.update(265, :parent_stock_count => 25, :composable_stock_count => 25, :orderable_count => 25)
      
      inventory.get_stock_count.should == 20
      inventory_1.get_stock_count.should == 30
      inventory_2.get_stock_count.should == 25
    end
  end
end
