# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe ShippingCost do
  fixtures :shipping_costs, :shipping_cost_boundaries, 
    :prefectures, :prefecture_displays, :user_addresses,
    :shop_settings, :countries
  describe "calc_shipping_cost" do
    before do
      @user_address = UserAddress.new(:country_code => 'JP', :zip_code => '333-0009', :address_1 => '愛知県')
      ShopSetting.stub!(:get_setting).and_return(@setting_outlier)
      @setting_outlier = <<-EOS
460-0002,200
440-0020,100
EOS
    end
    it "キャンペーンなし 国が日本 都道府県が愛知県 離島設定なし の場合" do
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, false)
      result.should == 300
    end
    it "キャンペーンなし 国が日本 都道府県が福岡県 離島設定なし の場合" do
      @user_address.address_1 = '福岡県'
      ShopSetting.stub!(:get_setting).and_return(@setting_outlier)
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, false)
      result.should == 310
    end
    it "キャンペーンなし 国が日本 都道府県が福岡県 離島設定あり の場合" do
      @user_address.address_1 = '福岡県'
      @user_address.zip_code = '460-0002'
      ShopSetting.stub!(:get_setting).and_return(@setting_outlier)
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, false)
      result.should == 510
    end
    it "キャンペーンなし 国がアフガニスタン の場合" do
      @user_address.country_code = 'AF'
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, false)
      result.should == 330
    end
    it "送料無料キャンペーンあり 国が日本 都道府県が愛知県 離島設定なし の場合" do
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, true)
      result.should == 0
    end
    it "送料無料キャンペーンあり 国が日本 都道府県が福岡県 離島設定なし の場合" do
      @user_address.address_1 = '福岡県'
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, true)
      result.should == 0
    end
    it "送料無料キャンペーンあり 国が日本 都道府県が福岡県 離島設定あり の場合" do
      @user_address.address_1 = '福岡県'
      @user_address.zip_code = '440-0020'
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, true)
      result.should == 0
    end
    it "送料無料キャンペーンあり 国が中国 の場合" do
      @user_address.country_code = 'AF'
      result = ShippingCost.calc_shipping_cost(@user_address, 1100, true)
      result.should == 0
    end
  end
end
