# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
describe SitemapPage do
  fixtures :cms_projects, :cms_pages

  before(:each) do
    SitemapPage.instance_variable_set(:@default_pages, nil)
    @valid_attributes = {
      :location => "value for location",
      :last_modified_on => Date.today,
      :change_freq => "value for change_freq",
      :priority => "value for priority",
      :cms_page_id => 1
    }
  end
  
  describe "default_pages" do
    it "should return SitemapPage object[]" do
      count_with_lang = 0
      count_without_lang = 0
      sitemap_pages = SitemapPage.default_pages(:order => 'controller asc, action asc') do |*args|
        args.length.should >= 1
        options = args.first
        if options[:controller] == 'sitemap'
          options[:controller].should == 'sitemap'
          options[:action].should == 'index' 
        end
        if options[:lang]
          count_with_lang += 1
        else
          count_without_lang += 1
        end
      end

      # lang抜きのURLは定義されているすべてのコントローラの全アクションの数
#      count_without_lang.should == 
#        SitemapPage::DEFAULT_PAGES.inject(0){|sum, (controller, actions)| sum += actions.length}
      count_without_lang.should == 0
      
#      count_with_lang.should == count_without_lang * target_langs.length
#      count_with_lang.should == SitemapPage::DEFAULT_PAGES.inject(0){|sum, (controller, actions)| sum += actions.length} * I18n.supported_locales.length
      
      sitemap_pages.length.should == count_with_lang + count_without_lang

      sitemap_page = sitemap_pages.first
      sitemap_page.last_modified_on.should == Date.new(2009, 5, 14)
      sitemap_page.change_freq.should == :never
      sitemap_page.priority.should == 0.3
    end
  end  
  
  describe "cms_pages" do
    describe "should return SitemapPage object[]" do
      def invoke_cms_pages
        count_with_lang = 0
        count_without_lang = 0
        sitemap_pages = SitemapPage.cms_pages(:order => 'controller asc, action asc') do |*args|
          args.length.should >= 1
          options = args.first
          if options[:lang]
            count_with_lang += 1
          else
            count_without_lang += 1
          end
        end
        locale_ignoreds = {}
        cms_pages = []
        public_pages = CmsPage.primary_pages
        public_pages.each {|public_page| cms_pages << public_page if public_page.controller == 'cms' &&
          !(public_page.action.nil? || public_page.action.empty? || public_page.action =~ /^_/)}
        cms_pages.each do |cms_page|
          locale_ignored_key = [cms_page.controller, cms_page.action]
          locale_ignoreds[ locale_ignored_key ] ||= {}
          I18n.supported_locales.map(&:to_s).each do |locale|
            locale_ignoreds[ locale_ignored_key ][locale.to_s] = cms_page
          end
        end

        
#        count_without_lang.should == locale_ignored.uniq.length
        count_without_lang.should == 0
        
#        count_with_lang.should == cms_pages.length  * I18n.supported_locales.map(&:to_s).length
        
#        sitemap_pages.length.should == count_with_lang + count_without_lang
        sitemap_pages.length.should == count_with_lang

        sitemap_page = sitemap_pages.first
        sitemap_page.change_freq.should == :daily
        sitemap_page.priority.should == 1.0
        sitemap_page = sitemap_pages.last
        sitemap_page.change_freq.should == :weekly
        sitemap_page.priority.should == 0.5
      end

      it "invoke cms pages" do
        invoke_cms_pages
      end

    end  
  end  
end
