# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe "SearchConditionForm class check" do

  fixtures :users, :user_addresses, :user_emails, :kbn_codes, 
    :orders, :order_shippings

  it "validate with regist_date_from, regist_date_to are empty" do
    u1 = User::SearchConditionForm.new()
    u1.validate
    u1.errors.length == 0
  end

  it "validate with regist_date_from, regist_date_to are not empty" do
    u1 = User::SearchConditionForm.new(:regist_date_from => '2009/01/01', :regist_date_to => '2009/01/02')
    u1.validate
    u1.errors.length == 0
  end

  it "validate with regist_date_from, regist_date_to are wrong" do
    u1 = User::SearchConditionForm.new(:regist_date_from => '2009/01/32', :regist_date_to => '2009/01/33')
    u1.validate
    u1.errors.length == 2
  end

  it "test regist_date_from_time" do
    u1 = User::SearchConditionForm.new(:regist_date_from => '2009/01/01', :regist_date_to => '2009/01/02')
    u1.regist_date_from_time.should == Time.parse('2009/01/01 00:00:00')
  end

  it "test regist_date_to_time" do
    u1 = User::SearchConditionForm.new(:regist_date_from => '2009/01/01', :regist_date_to => '2009/01/02')
    u1.regist_date_to_time.should == Time.parse('2009/01/02 23:59:59')
  end

  it "test tel_num_not_hyphen" do
    u1 = User::SearchConditionForm.new(:tel_num => '012-345-6789')
    u1.tel_num_not_hyphen.should == '0123456789'
  end

  describe "create_condition" do
    it "test create_condition with user_code" do
      c = User.create_condition(
        User::SearchConditionForm.new(:user_code => '1007'),
        Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id',
        :include => [:user_addresses, :user_email])
      u.length.should == 1
      u.first.id.should == 1006
    end

    it "test create_condition with name" do
      c = User.create_condition(User::SearchConditionForm.new(:name => 'てすと'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
      u.first.id.should == 1008
    end

    it "test create_condition with sex" do
      c = User.create_condition(User::SearchConditionForm.new(:sex => User.sex_id_by_key(:female)), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 6
      u[0].id.should == 1003
      u[1].id.should == 1004
      u[2].id.should == 1005
      u[3].id.should == 1006
      u[4].id.should == 1014
      u[5].id.should == 1024
    end

    it "test create_condition with address" do
      c = User.create_condition(User::SearchConditionForm.new(:address => '北海道'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
      u.first.id.should == 1008
    end

    it "test create_condition with email" do
      c = User.create_condition(User::SearchConditionForm.new(:email => 'test.com'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
      u.first.id.should == 1008
    end

    it "test create_condition with tel_num" do
      c = User.create_condition(User::SearchConditionForm.new(:tel_num => '0000-00-0000'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
      u.first.id.should == 1008
    end

    it "test create_condition with fax_num" do
      User.find(1009).update_attribute(:fax_num, '000-000-0000')
      User.find(1008).update_attribute(:tel_num, '000-000-0001')
      c = User.create_condition(User::SearchConditionForm.new(:tel_num => '0000-00-0000'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
      u.first.id.should == 1009
    end

    it "test create_condition with blacklist_div" do
      User.find(:first).update_attribute(:blacklist_div, User.blacklist_div_id_by_key(:entry_1))
      c = User.create_condition(User::SearchConditionForm.new(:blacklist_div => User.blacklist_div_id_by_key(:entry_1)), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
    end

    it "test create_condition with member_div" do
      c = User.create_condition(User::SearchConditionForm.new(:member_div => User.member_div_id_by_key(:membership)), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 35
    end

    it "test create_condition with regist_date_from" do
      c = User.create_condition(User::SearchConditionForm.new(:regist_date_from => '2009/01/01'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 27
      u.first.id.should == 1008
    end

    it "test create_condition with regist_date_to" do
      c = User.create_condition(User::SearchConditionForm.new(:regist_date_to => '2009/01/01'), Time.parse('2009/01/20 10:00:00'))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 8
    end

    it "本番会員のみ" do
      User.update(users(:users_00001).id, :test_flg => true)
      c = User.create_condition(User::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:production)))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 34
    end

    it "テスト会員のみ" do
      User.update(users(:users_00001).id, :test_flg => true)
      c = User.create_condition(User::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:test)))
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 1
    end

    it "本番会員とテスト会員" do
      User.update(users(:users_00001).id, :test_flg => true)
      c = User.create_condition(User::SearchConditionForm.new())
      u = User.find(:all, :conditions => c, :order => 'users.id', :include => [:user_addresses, :user_email])
      u.length.should == 35
    end
  end

end
