# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe UserPoint do
  #include AuthenticatedTestHelper
  fixtures :users, :orders, :user_points, :point_settings, :accountings

  describe "user_point_create" do
    it "true" do
      result = UserPoint.new(:new_flg => true)
      result.user_point_create.should be_true
    end

    it "false" do
      result = UserPoint.new(:new_flg => false)
      result.user_point_create.should be_false
    end

    it "false" do
      result = UserPoint.new
      result.user_point_create.should be_false
    end
  end

  describe "total_point_for_user" do
    it "仮ポイントなしの会員の場合" do
      total_point_1001 = UserPoint.total_point_for_user(1001)
      total_point_1001.should == 700
      total_point_1002 = UserPoint.total_point_for_user(1002)
      total_point_1002.should == 300
      total_point_1003 = UserPoint.total_point_for_user(1003)
      total_point_1003.should == 1400
    end

    it "仮ポイントありの会員の場合" do
      total_point_1001 = UserPoint.total_point_for_user(1001)
      total_point_1001.should == 700
      total_point_1002 = UserPoint.total_point_for_user(1002)
      total_point_1002.should == 300
      total_point_1003 = UserPoint.total_point_for_user(1003)
      total_point_1003.should == 1400
    end
  end

  describe "term_date_for_user" do
    it "" do
      term_date_1001 = UserPoint.term_date_for_user(1001)
      term_date_1001.strftime('%Y-%m-%d %H:%M:%S').should == "2008-01-02 16:15:21"
      term_date_1002 = UserPoint.term_date_for_user(1002)
      term_date_1002.strftime('%Y-%m-%d %H:%M:%S').should == "2008-01-06 16:15:21"
      term_date_1003 = UserPoint.term_date_for_user(1003)
      term_date_1003.strftime('%Y-%m-%d %H:%M:%S').should == "2008-01-07 16:15:21"
    end
  end

  describe "update_expired" do
    it "ポイントが最終付与時" do
      UserPoint.delete_all(['user_id = ?', 1004]) # テスト用に作成した実際ありえないデータのため削除
      UserPoint.update_expired
      expired_points = UserPoint.find(:all, :conditions => ['expired_flg = ?', 1], :order => 'id desc')
      expired_points[0].user_id.should == 1003
      expired_points[0].given_point.should == -700
      expired_points[1].user_id.should == 1002
      expired_points[1].given_point.should == -300
      expired_points[2].user_id.should == 1001
      expired_points[2].given_point.should == -700
      accountings = Accounting.find(:all, :conditions => ["user_point_id is not null"], :order => 'user_point_id desc')
      accountings.length.should == 0
    end
  end

  describe "term_date_update_for_last_point" do
    it "最終ポイント付与時の場合" do
      point = PointSetting.find(:first)
      point.available_flg = true
      point.terminable_flg = true
      point.term_div = "0"
      point.point_period_div = 6
      point.save!
      UserPoint.term_date_update_for_last_point(1001, "2007/03/26")
      after_points = UserPoint.find(:all, :conditions => ["expired_flg = 0 AND user_id = ?", 1001])
      after_points.each do |after_point|
        if after_point.given_point == 0
          # 使用ポイントは有効期限を付け替えない
          after_point.term_date.should be_nil
        else
          after_point.term_date.strftime('%Y-%m-%d').should == "2007-03-26"
        end
      end
    end
    it "最終ポイント付与時以外の場合" do
      point = PointSetting.find(:first)
      point.available_flg = true
      point.terminable_flg = true
      point.term_div = "99"
      point.point_period_div = 6
      point.save!
      UserPoint.term_date_update_for_last_point(1001, "2007/03/26")
      after_points = UserPoint.find(:all, :conditions => ["expired_flg = 0 AND user_id = ?", 1001])
      after_points.each do |after_point|
        if after_point.given_point == 0
          after_point.term_date.should be_nil
        else
          after_point.term_date.strftime('%Y-%m-%d').should_not == "2007-03-26"
        end
      end
    end
  end
  
  describe "SQL生成テスト" do
    it "test create_condition" do
      pending
      params = {
        :date_search_div => '1',
        :date_from => '2007/03/26',
        :date_to => '2008/03/26',
        :member_id => '1001',
        :member_name => 'member_name test',
        :total_point => '200',
        :terminable_flg => '1'
        }
      condition = UserPoint::SearchInfo.new(params)
      sql = condition.create_user_point_find_query
      sql.should == ["select * from user_points left outer join users on user_points.user_id = users.id left outer join orders on user_points.order_id = orders.id left outer join (select user_id, (sum(given_point) - sum(used_point)) as total_point from user_points GROUP BY user_id) total on user_points.user_id = total.user_id where orders.created_at >= ? AND orders.created_at <= ? AND user_points.user_id like ? AND (CONCAT(users.name_last, users.name_first) like ? OR CONCAT(users.name_first, users.name_last) like ? OR CONCAT(users.kana_name_last, users.kana_name_first) like ? OR CONCAT(users.kana_name_first, users.kana_name_last) like ?) AND total_point = ? AND user_points.expired_flg = ? ORDER BY user_points.created_at DESC", "Mon Mar 26 00:00:00 +0900 2007", "Wed Mar 26 23:59:59 +0900 2008", "%1001%", "%member_nametest%", "%member_nametest%", "%member_nametest%", "%member_nametest%", "200", "1"]
    end
  end

  describe "point_by_term_date" do
    it "指定した有効期限までに有効期限切れのポイントが含まれている場合" do
      total_point = UserPoint.point_to_term_date(1004, "3012/01/01")
      total_point.should == 400
    end
    it "指定した有効期限以降のポイントが存在する場合、以降のポイントを計算しない" do
      total_point = UserPoint.point_to_term_date(1004, "3011/01/01")
       total_point.should == 400
    end
  end

  describe "active_given_point" do
    it "正常処理 仮ポイントフラグがfalseになること" do
      order_id = 12
      user_point = UserPoint.find(:first, :conditions => ["order_id = ?", order_id])
      user_point.inactive_flg = true
      user_point.save!
      UserPoint.active_given_point(order_id)
      UserPoint.find(:first, :conditions => ["order_id = ?", order_id]).inactive_flg.should be_false
    end
  end

  describe "inactive_given_point" do
    it "正常処理 仮ポイントフラグがtrueになること" do
      order_id = 12
      user_point = UserPoint.find(:first, :conditions => ["order_id = ?", order_id])
      user_point.inactive_flg = false
      user_point.save!
      UserPoint.inactive_given_point(order_id)
      UserPoint.find(:first, :conditions => ["order_id = ?", order_id]).inactive_flg.should be_true
    end
  end

  describe "update_inactive_point" do
    it "仮ポイントを変更する" do
      user_point = UserPoint.find(1)
      user_point.inactive_flg = 1
      user_point.save!

      UserPoint.update_inactive_point(1, 1010)

      user_point_1 = UserPoint.find(1)
      user_point_1.order_id.should == 1
      user_point_1.given_point.should == 1010
      user_point_3 = UserPoint.find(3)
      user_point_3.order_id.should == 1
      user_point_3.given_point.should == 0
    end
  end
end
