require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe UserRank do
  fixtures :user_ranks
  describe "shold get user rank list" do
    it "should get user rank list" do
      rank_list = UserRank.user_rank_list()
      rank_list[0].should == ["非会員", 1]
      rank_list[1].should == ["ゴールド会員", 2]
      rank_list[2].should == ["スタンダード会員", 3]
    end
  end

  describe "setup_displays_for" do
    it "言語ごとのdisplayのインスタンスが生成されること" do
      result = UserRank.new()
      result.setup_displays_for
      result.displays.size.should == I18n.supported_locales.size
    end
  end

  describe "setup_conditions" do
    it "5つのconditionsのインスタンスが生成されること" do
      result = UserRank.new()
      result.setup_conditions
      result.conditions.size.should == 5
    end
  end
  
  describe UserRank::SearchForm do
    describe "to_paginate_options" do
      after(:each) do
        # エラーが出ないかどうかを確認します
        UserRank.paginate(@form.to_paginate_options)
      end
      
      it "default without params" do
        @form = UserRank::SearchForm.new
        @form.to_paginate_options.should == {
          :per_page => 20,
          :page => 1,
          :order => "code asc, created_at asc"
        }
      end

      it "default with params" do
        @form = UserRank::SearchForm.new("disp_count" => "20", 
          "code" => "", 
          "name" => "")
        @form.to_paginate_options.should == {
          :per_page => 20,
          :page => 1,
          :order => "code asc, created_at asc"
        }
      end

      it "all params" do
        @form = UserRank::SearchForm.new("disp_count" => "50",
          "code" => "01",
          "name" => "testname",
          "type_cd" => "02")
        
        @form.to_paginate_options.should == {
          :conditions => [
            "code like ? AND name like ? AND user_rank_conditions.type_cd = ?",
            "%01%",
            "%testname%",
            "02"
          ],
          :per_page => 50,
          :page => 1,
          :order => "code asc, created_at asc",
          :joins => ["inner join user_rank_conditions on user_rank_conditions.user_rank_id = user_ranks.id"],
          :group => "user_ranks.id"
        }
      end
      it "non type_cd params" do
        @form = UserRank::SearchForm.new("disp_count" => "50",
          "code" => "01",
          "name" => "testname"
        )

        @form.to_paginate_options.should == {
          :conditions => [
            "code like ? AND name like ?",
            "%01%",
            "%testname%"
          ],
          :per_page => 50,
          :page => 1,
          :order => "code asc, created_at asc"
        }
      end
    end
  end

  describe "upgrade? 会員ランク更新バッチ ランクアップ条件判定のテスト" do
    before do
      UserRank.delete_all

      ranks = []
      set_ranks = []
      # ベーシック会員 優先度 999 会員登録日からの日数 　０日 以上
      set_ranks << {:name => "ベーシック会員", :code => "01", :priority => "999", :condition_value => "0"}
      # スタンダード会員 優先度 3 会員登録日からの日数 ３０日 以上
      set_ranks << {:name => "スタンダード会員", :code => "02", :priority => "3", :condition_value => "30"}
      # シルバー会員 優先度 2 会員登録日からの日数 　　６０日 以上
      set_ranks << {:name => "シルバー会員", :code => "03", :priority => "2", :condition_value => "60"}
      # ゴールド会員 優先度 2 会員登録日からの日数　 　９０日 以上
      set_ranks << {:name => "ゴールド会員", :code => "04", :priority => "1", :condition_value => "90"}

      set_ranks.each do |set_rank|
        rank = UserRank.new(:name => set_rank[:name], :no_membership_flg => 0, :code => set_rank[:code], :available_div => "1",
          :connector_cd => "0", :grading_period_div => "0", :priority => set_rank[:priority])
        rank.conditions << UserRankCondition.new(:no => 1, :type_cd => "04", :operator_cd => "04", :value => set_rank[:condition_value])
        2..5.times do |i|
          rank.conditions << UserRankCondition.new(:no => i, :type_cd => "", :operator_cd => "", :value =>"")
        end
        I18n.supported_locales.each do |locale|
          rank.displays << UserRankDisplay.new(:locale => locale.to_s, :name => set_rank[:name])
        end
        rank.save!
        ranks << rank
      end
      @basic = ranks[0]
      @standard = ranks[1]
      @silver = ranks[2]
      @gold = ranks[3]
      @test_user = User.find_by_id(1001)
    end
    describe "会員ランク未使用判定" do
      describe "スタンダード会員を使用しない" do
        before do
          Time.stub!(:now).and_return(Time.local(2010,06,10, 00,00,00))
          @standard.available_div = "0"
          @standard.save!
        end
        it "判定期間無し ベーシック会員かつスタンダード会員にランクアップする条件に一致する → ベーシック会員をキープ" do
          @test_user.created_at = Time.local(2010,06,9, 00,00,00)
          @test_user.user_rank_id = @basic.id
          @test_user.save!
          UserRank.update_all_user_rank
          User.find_by_id(1001).rank.name.should == "ベーシック会員"
        end
        it "判定期間無しベーシック会員かつゴールド会員にランクアップする条件に一致する → ゴールド会員にランクアップすること" do
          @test_user.created_at = Time.local(2010,01,9, 00,00,00)
          @test_user.user_rank_id = @basic.id
          @test_user.save!
          UserRank.update_all_user_rank
          User.find_by_id(1001).rank.name.should == "ゴールド会員"
        end
        it "判定期間あり 判定期間中 シルバー会員かつシルバー会員にランクアップする条件に一致しない → ベーシック会員にランクダウンすること" do
          @test_user.created_at = Time.local(2010,06,9, 00,00,00)
          @test_user.user_rank_updated_at = Time.local(2010,03,9, 00,00,00)
          @test_user.user_rank_id = @silver.id
          @test_user.save!
          @silver.grading_period_div = "1"
          UserRank.update_all_user_rank
          User.find_by_id(1001).rank.name.should == "ベーシック会員"
        end
      end
    end
    describe "すべての会員ランクが判定期間無しの場合" do
      it "ベーシック会員かつどの会員ランクのランクアップする条件に一致しない → ベーシック会員をキープすること" do
        @test_user.created_at = Time.local(2010,06,9, 00,00,00)
        @test_user.user_rank_id = @basic.id
        @test_user.save!
        UserRank.update_all_user_rank
        User.find_by_id(1001).rank.name.should == "ベーシック会員"

      end
      it "ベーシック会員かつスタンダード会員にランクアップする条件に一致する → スタンダード会員にランクアップすること" do

      end
      it "スタンダード会員かつスタンダード会員にランクアップする条件に一致しない → ランクダウンしないこと" do

      end
      it "スタンダード会員かつゴールド会員にランクアップする条件に一致する → ゴールド会員にランクアップすること" do

      end
      it "ゴールド会員かつスタンダード会員にランクアップする条件に一致しない → ランクダウンしないこと" do

      end
    end

    describe "ベーシック以外の会員ランクが判定期間ありの場合(３ヶ月)" do
      describe "判定期間中" do
        it "判定期間中 スタンダード会員かつスタンダード会員にランクアップする条件に一致しない → スタンダード会員をキープすること" do

        end
        it "判定期間中 スタンダード会員かつスタンダード会員にランクアップする条件に一致する → スタンダード会員をキープすること" do

        end
        it "判定期間中 スタンダード会員かつシルバー会員にランクアップする条件に一致する → シルバー会員にランクアップすること" do

        end
      end
      describe "判定期間経過後" do
        it "判定期間経過後 スタンダード会員かつスタンダード会員にランクアップする条件に一致しない → ベーシック会員にランクダウン" do

        end
        it "判定期間経過後 スタンダード会員かつスタンダード会員にランクアップする条件に一致する → スタンダード会員をキープすること" do

        end
        it "判定期間経過後 スタンダード会員かつシルバー会員にランクアップする条件に一致する → シルバー会員にランクアップすること" do

        end
      end
    end

    describe "upgrade? 会員ランク更新バッチ 更新される会員が正しいかをテストします。" do
      it "ランクアップ条件に一致する" do
        @test_user.created_at = Time.local(2010,06,9, 00,00,00)
        @test_user.user_rank_id = @basic.id
        @test_user.save!
        result, user_rank_id = @basic.upgrade?(@test_user)
        result.should == true
        user_rank_id.should == @basic.id
      end
      it "ランクアップ条件に一致しない" do
        @test_user.created_at = Time.local(2010,06,9, 00,00,00)
        @test_user.user_rank_id = @basic.id
        @test_user.save!
        result, user_rank_id = @standard.upgrade?(@test_user)
        result.should == false
        user_rank_id.should_not == @basic.id
      end
      it "ランクアップ条件に一致する" do
        @test_user.created_at = Time.local(2010,06,9, 00,00,00)
        @test_user.user_rank_id = @basic.id
        @test_user.save!
        result, user_rank_id = @silver.upgrade?(@test_user)
        result.should == false
        user_rank_id.should == @silver.id
      end
    end
  end
end
