# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'
require 'nkf'
require 'iconv'

describe FrontSiteMailer do
  fixtures :user_addresses, :kbn_codes, :users, :countries, :user_emails,
    :mail_templates, :inquiries, :shop_settings, 
    :orders, :order_shippings, :order_items, :product_skus, :products

  before(:each) do
    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []
    @user_id = 1002
    @user_id_en = 1014
    @email = 'email_address_01002@ec-one.com'
    @email_mobile = 'ec-one@docomo.ne.jp'
    @last_name = "山田"
    @last_name_en = 'Yamada'
    @first_name = "太郎"
    @first_name_en = 'Taro'
    @user_name = '山田太郎'
    @user_name_en = 'YamadaTaro'
    @inquiry = Inquiry.find 1
    @inquiry_mobile = Inquiry.find 2
    @inquiry_mobile.email = @email_mobile
    @order_id = 2
    @order_id_en = 12
    @order = Order.find 2
    @user = User.find @order.user_id
    @before_test_locale = I18n.locale #テスト内でロケールを変更するため変更前のデータを退避
    @subject = 'test subject'
    @body = 'test body'
    @subject_en = 'this is the subject'
    @body_en = 'this is the body'
    @subject_zh_CN='衷心感谢您对我们的支持参加(subject)'
    @body_zh_CN='衷心感谢您对我们的支持参加(body)'
    @subject_zh_TW='衷心感謝您對我們的支持參加(subject)'
    @body_zh_TW='衷心感謝您對我們的支持參加(body)'
end

  after(:each) do
    I18n.locale = @before_test_locale #テスト内で変更したロケールを元に戻す 
  end

  it "signup mail encoding test locale:en" do
    I18n.locale = 'en'
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id_en, @email)
    signup_pc_mail.quoted_subject.should == '=?iso-8859-1?B?VXNlckluZm8gcmVnaXN0IHBjIShlbik=?='
    signup_pc_mail.content_transfer_encoding.should == '7bit'
    signup_pc_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', "Nijuhachi Kaiin\n registration completed. thank you.\n");
  end

  it "signup mail encoding test locale:zh_CN" do
    I18n.locale = 'zh_CN'
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id_en, @email)
    signup_pc_mail.quoted_subject.should == '=?gb2312?B?VXNlckluZm8gcmVnaXN0IHBjISBzYW1wbGUg1tTQxLjQ0LvE+rbUztLDx7XE1qez1rLOvNM=?='
    signup_pc_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', "Nijuhachi Kaiin\n registration completed. 衷心感谢您对我们的支持参加\n");
  end

  it "signup mail encoding test locale:zn_TW" do
    I18n.locale = 'zh_TW'
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id_en, @email)
    signup_pc_mail.quoted_subject.should == '=?big5?B?VXNlckluZm8gcmVnaXN0IHBjISBzYW1wbGUgsEqk37dQwcKxernvp9qtzKq6pOSr+bDRpVs=?='
    signup_pc_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', "Nijuhachi Kaiin\n registration completed. 衷心感謝您對我們的支持參加\n");
  end

  it "signup mail subject encoding error retry test(utf-8)" do
    #メール内容によって変わるような処理でないためこのメール送信処理でのみ実施
    I18n.locale = 'zh_CN'
    #文字コード変換に失敗するようにテンプレートのロケールを入れ替える
    zhcn_template = MailTemplate.find(27)
    zhcn_template.locale = 'zh_TW'
    zhcn_template.save!
    zhtw_template = MailTemplate.find(38)
    zhtw_template.locale = 'zh_CN'
    zhtw_template.save!
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id_en, @email)
    #zh_CNのSubjectエンコードに失敗してUTF-8で設定されていること
    signup_pc_mail.quoted_subject.should == '=?utf-8?B?VXNlckluZm8gcmVnaXN0IHBjISBzYW1wbGUg6KG35b+D5oSf6Kyd5oKo5bCN5oiR5YCR55qE5pSv5oyB5Y+D5Yqg?='
    signup_pc_mail.quoted_body.should == "Nijuhachi Kaiin\n registration completed. 衷心感謝您對我們的支持參加\n"
  end

  it "signup mail body encoding error retry test(utf-8)" do
    #メール内容によって変わるような処理でないためこのメール送信処理でのみ実施
    I18n.locale = 'zh_CN'
    #本文文字コード変換に失敗するようにテンプレートのロケールを入れ替えてSubjectは正常になるようにする
    zhcn_template = MailTemplate.find(27)
    zhcn_template.locale = 'zh_TW'
    zhcn_template.save!
    zhtw_template = MailTemplate.find(38)
    zhtw_template.locale = 'zh_CN'
    zhtw_template.pc_subject_template = zhcn_template.pc_subject_template
    zhtw_template.save!
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id_en, @email)
    #zh_CNの本文エンコードに失敗してUTF-8で設定されていること
    signup_pc_mail.quoted_subject.should == '=?utf-8?B?VXNlckluZm8gcmVnaXN0IHBjISBzYW1wbGUg6KG35b+D5oSf6LCi5oKo5a+55oiR5Lus55qE5pSv5oyB5Y+C5Yqg?='
    signup_pc_mail.quoted_body.should == "Nijuhachi Kaiin\n registration completed. 衷心感謝您對我們的支持參加\n"
  end


  # ユーザ登録メール
  it "signup pc mail empty test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id, @email)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "signup pc mail size test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id, @email)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "signup pc mail subject test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id, @email)
    signup_pc_mail.subject.should == "UserInfo regist pc!"
  end

  it "signup pc mail to_address test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup(@user_id, @email)
    signup_pc_mail.to.should == [@email]
  end

  it "signup mobile mail empty test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup(@user_id, @email_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "signup mobile mail size test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup(@user_id, @email_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "signup mobile mail subject test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup(@user_id, @email_mobile)
    signup_mobile_mail.subject.should == "UserInfo regist mobile!"
  end

  it "signup mobile mail to_address test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup(@user_id, @email_mobile)
    signup_mobile_mail.to.should == [@email_mobile]
  end

  # ユーザ登録エラーメール
  it "signup pc mail empty test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "signup pc mail size test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "signup pc mail subject test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email)
    NKF::nkf("-w", signup_pc_mail.subject).should == "会員登録できませんでした(pc)"
  end

  it "signup pc mail to_address test" do
    signup_pc_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email)
    signup_pc_mail.to.should == [@email]
  end

  it "signup mobile mail empty test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "signup mobile mail size test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "signup mobile mail subject test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email_mobile)
    NKF::nkf("-w", signup_mobile_mail.subject).should == "会員登録できませんでした(mobile)"
  end

  it "signup mobile mail to_address test" do
    signup_mobile_mail = FrontSiteMailer.deliver_signup_error(@user_id, @email_mobile)
    signup_mobile_mail.to.should == [@email_mobile]
  end

  # ユーザ退会メール
  it "resign pc mail empty test" do
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "resign pc mail size test" do
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "resign pc mail subject test" do
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email)
    resign_pc_mail.subject.should == "UserInfo resign pc!"
  end

  it "resign pc mail to_address test" do
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email)
    resign_pc_mail.to.should == [@email]
  end

  it "resign mobile mail empty test" do
    resign_mobile_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "resign mobile mail size test" do
    resign_mobile_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "resign mobile mail subject test" do
    resign_mobile_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email_mobile)
    resign_mobile_mail.subject.should == "UserInfo resign mobile!"
  end

  it "resign mobile mail to_address test" do
    resign_mobile_mail = FrontSiteMailer.deliver_resign(@last_name, @first_name, @user_name, @email_mobile)
    resign_mobile_mail.to.should == [@email_mobile]
  end

  it "resign mail encoding test locale:en" do
    I18n.locale = 'en'
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name_en, @first_name_en, @user_name_en, @email)
    resign_pc_mail.quoted_subject.should == '=?iso-8859-1?B?VXNlckluZm8gcmVzaWduIHBjIShlbik=?='
    resign_pc_mail.content_transfer_encoding.should == '7bit'
    resign_pc_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', "YamadaTaro\n resign completed. thank you.\n");
  end

  it "resign mail encoding test locale:zh_CN" do
    I18n.locale = 'zh_CN'
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name_en, @first_name_en, @user_name_en, @email)
    resign_pc_mail.quoted_subject.should == '=?gb2312?B?VXNlckluZm8gcmVzaWduIHBjISBzYW1wbGUgINbU0MS40NC7xPq21M7Sw8e1xNans9ayzrzT?='
    resign_pc_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', "YamadaTaro\n resign completed. 衷心感谢您对我们的支持参加\n");
  end

  it "resign mail encoding test locale:zh_TW" do
    I18n.locale = 'zh_TW'
    resign_pc_mail = FrontSiteMailer.deliver_resign(@last_name_en, @first_name_en, @user_name_en, @email)
    resign_pc_mail.quoted_subject.should == '=?big5?B?VXNlckluZm8gcmVzaWduIHBjISBzYW1wbGUgILBKpN+3UMHCsXq576farcyquqTkq/mw0aVb?='
    resign_pc_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', "YamadaTaro\n resign completed. 衷心感謝您對我們的支持參加\n");
  end


  # サンクスメール
  it "thanks pc mail empty test" do
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id, @email)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "thanks pc mail size test" do
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id, @email)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "thanks pc mail subject test" do
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id, @email)
    thanks_pc_mail.subject.should == "thanks pc!"
  end

  it "thanks pc mail to_address test" do
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id, @email)
    thanks_pc_mail.to.should == [@email]
  end

  it "thanks mobile mail empty test" do
    thanks_mobile_mail = FrontSiteMailer.deliver_thanks(@order_id, @email_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "thanks mobile mail size test" do
    thanks_mobile_mail = FrontSiteMailer.deliver_thanks(@order_id, @email_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "thanks mobile mail subject test" do
    thanks_mobile_mail = FrontSiteMailer.deliver_thanks(@order_id, @email_mobile)
    thanks_mobile_mail.subject.should == "thanks mobile!"
  end

  it "thanks mobile mail to_address test" do
    thanks_mobile_mail = FrontSiteMailer.deliver_thanks(@order_id, @email_mobile)
    thanks_mobile_mail.to.should == [@email_mobile]
  end

  it "thanks mail encoding test locale:en" do
    I18n.locale = 'en'
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id_en, @email)
    thanks_pc_mail.quoted_subject.should == '=?iso-8859-1?B?dGhhbmtzIHBjIShlbik=?='
    thanks_pc_mail.content_transfer_encoding.should == '7bit'
    thanks_pc_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', "received [Online order number] 00000000011\n=========================\
=======================================\nsubtotal                          0yen\nshipping                   0yen\n---------------------------------------\
-------------------------\ntotal                          0yen\n----------------------------------------------------------------\n\n thank you\n");
  end

  it "thanks mail encoding test locale:zh_CN" do
    I18n.locale = 'zh_CN'
    order = Order.find(@order_id_en)
    order.locale = 'zh_CN'
    order.save!
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id_en, @email)
    thanks_pc_mail.quoted_subject.should == '=?gb2312?B?dGhhbmtzIHBjISBzYW1wbGUg1tTQxLjQ0LvE+rbUztLDx7XE1qez1rLOvNM=?='
    thanks_pc_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', "received [在线购入编码] 00000000011\n=========================\
=======================================\n小计                          0日元\n邮费                   0日元\n---------------------------------------\
-------------------------\n合计                          0日元\n----------------------------------------------------------------\n\n 衷心感谢您\n");
  end

  it "thanks mail encoding test locale:zh_TW" do
    I18n.locale = 'zh_TW'
    order = Order.find(@order_id_en)
    order.locale = 'zh_TW'
    order.save!
    thanks_pc_mail = FrontSiteMailer.deliver_thanks(@order_id_en, @email)
    thanks_pc_mail.quoted_subject.should == '=?big5?B?dGhhbmtzIHBjISBzYW1wbGUgsEqk37dQwcKxernvp9qtzKq6pOSr+bDRpVs=?='
    thanks_pc_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', "received [Online order number] 00000000011\n=========================\
=======================================\nsubtotal                          0yen\nshipping                   0yen\n---------------------------------------\
-------------------------\ntotal                          0yen\n----------------------------------------------------------------\n\n 衷心感謝您對我們的支持參加\n");
  end


  # 問い合わせ通知メール
  it "notice_inquiry pc mail empty test" do
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "notice_inquiry pc mail size test" do
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "notice_inquiry pc mail subject test" do
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    notice_inquiry_pc_mail.subject.should == "inquiry pc!"
  end

  it "notice_inquiry pc mail to_address test" do
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    notice_inquiry_pc_mail.to.should == [@inquiry.email]
  end

  it "notice_inquiry mobile mail empty test" do
    notice_inquiry_mobile_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "notice_inquiry mobile mail size test" do
    notice_inquiry_mobile_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "notice_inquiry mobile mail subject test" do
    notice_inquiry_mobile_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry_mobile)
    notice_inquiry_mobile_mail.subject.should == "inquiry mobile!"
  end

  it "notice_inquiry pc mail to_address test" do
    notice_inquiry_mobile_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry_mobile)
    notice_inquiry_mobile_mail.to.should == [@inquiry_mobile.email]
  end

  it "notice_inquiry mail encoding test locale:en" do
    I18n.locale = 'en'
    @inquiry.inquiry = 'INQUIRY'
    @inquiry.name_first = 'Taro'
    @inquiry.name_last = 'Yamada'
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    notice_inquiry_pc_mail.quoted_subject.should == '=?iso-8859-1?B?aW5xdWlyeSBwYyEoZW4p?='
    notice_inquiry_pc_mail.content_transfer_encoding.should == '7bit'
    notice_inquiry_pc_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', "Yamada Taro\n [inquiry content] INQUIRY\n received.\n");
  end

  it "notice_inquiry mail encoding test locale:zh_CN" do
    I18n.locale = 'zh_CN'
    @inquiry.inquiry = 'INQUIRY'
    @inquiry.name_first = 'Taro'
    @inquiry.name_last = 'Yamada'
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    notice_inquiry_pc_mail.quoted_subject.should == '=?gb2312?B?aW5xdWlyeSBwYyEgc2FtcGxlICDW1NDEuNDQu8T6ttTO0sPHtcTWp7PWss680w==?='
    notice_inquiry_pc_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', "Yamada Taro\n [inquiry content] INQUIRY\n received. 衷心感谢您对我们的支持参加\n");
  end

  it "notice_inquiry mail encoding test locale:zh_TW" do
    I18n.locale = 'zh_TW'
    @inquiry.inquiry = 'INQUIRY'
    @inquiry.name_first = 'Taro'
    @inquiry.name_last = 'Yamada'
    notice_inquiry_pc_mail = FrontSiteMailer.deliver_notice_inquiry(@inquiry)
    notice_inquiry_pc_mail.quoted_subject.should == '=?big5?B?aW5xdWlyeSBwYyEgc2FtcGxlICCwSqTft1DBwrF6ue+n2q3Mqrqk5Kv5sNGlWw==?='
    notice_inquiry_pc_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', "Yamada Taro\n [inquiry content] INQUIRY\n received.衷心感謝您對我們的支持參加\n");
  end


  # キャンセル依頼メール
  it "cancel pc mail empty test" do
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "cancel pc mail size test" do
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "cancel pc mail subject test" do
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email)
    cancel_pc_mail.subject.should == "cancel pc!"
  end

  it "cancel pc mail to_address test" do
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email)
    cancel_pc_mail.to.should == [@email]
  end

  it "cancel mobile mail empty test" do
    cancel_mobile_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email_mobile)
    ActionMailer::Base.deliveries.should_not be_empty
  end

  it "cancel mobile mail size test" do
    cancel_mobile_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email_mobile)
    ActionMailer::Base.deliveries.size.should == 1
  end

  it "cancel mobile mail subject test" do
    cancel_mobile_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email_mobile)
    cancel_mobile_mail.subject.should == "cancel mobile!"
  end

  it "cancel mobile mail to_address test" do
    cancel_mobile_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@order, @user, @email_mobile)
    cancel_mobile_mail.to.should == [@email_mobile]
  end

  it "cancel mail encoding test locale:en" do
    I18n.locale = 'en'
    @user.user_address.name_first = 'Taro'
    @user.user_address.name_last = 'Yamada'
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@ec_order_chapter, @user, @email)
    cancel_pc_mail.quoted_subject.should == '=?iso-8859-1?B?Y2FuY2VsIHBjIShlbik=?='
    cancel_pc_mail.content_transfer_encoding.should == '7bit'
    cancel_pc_mail.quoted_body.should == Iconv.conv('iso-8859-1', 'utf-8', "Yamada Taro\n cancel completed. thank you.\n");
  end

  it "cancel mail encoding test locale:zh_CN" do
    I18n.locale = 'zh_CN'
    @user.user_address.name_first = 'Taro'
    @user.user_address.name_last = 'Yamada'
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@ec_order_chapter, @user, @email)
    cancel_pc_mail.quoted_subject.should == '=?gb2312?B?Y2FuY2VsIHBjISBzYW1wbGUgINbU0MS40NC7xPq21M7Sw8e1xNans9ayzrzT?='
    cancel_pc_mail.quoted_body.should == Iconv.conv('gb2312', 'utf-8', "Yamada Taro\n cancel completed. 衷心感谢您对我们的支持参加\n");
  end

  it "cancel mail encoding test locale:zh_TW" do
    I18n.locale = 'zh_TW'
    @user.user_address.name_first = 'Taro'
    @user.user_address.name_last = 'Yamada'
    cancel_pc_mail = FrontSiteMailer.deliver_notice_cancel_reqest(@ec_order_chapter, @user, @email)
    cancel_pc_mail.quoted_subject.should == '=?big5?B?Y2FuY2VsIHBjISBzYW1wbGUgILBKpN+3UMHCsXq576farcyquqTkq/mw0aVb?='
    cancel_pc_mail.quoted_body.should == Iconv.conv('big5', 'utf-8', "Yamada Taro\n cancel completed. 衷心感謝您對我們的支持參加\n");
  end

  it "send_notice_cancel_reqest_mail method test." do
    to = "send_notice_cancel_reqest_mail_test-to@ec-one.com"
    order = Order.find(1)
    FrontSiteMailer.send_notice_cancel_reqest_mail(order, order.user, to)
    mail_delivery = MailDelivery.find(:all, :conditions => ["from_address = ?", ShopSetting[:shop_mail]])
    mail_delivery.should_not == nil
  end

  it "send_notice_inquiry_mail method test." do
    inquiry = Inquiry.find(1)
    FrontSiteMailer.send_notice_inquiry_mail(inquiry)
    mail_deliveries = MailDelivery.find(:all, :conditions => ["from_address = ?", ShopSetting[:shop_mail]])
    mail_deliveries.should_not == nil
    mail_deliveries.each do |md|
      md.inquiry_id.should == 1
    end
  end

  it "send_notice_inquiry_mail for product method test." do
    inquiry = Inquiry.find(3)
    FrontSiteMailer.send_notice_inquiry_mail(inquiry)
    mail_deliveries = MailDelivery.find(:all, :conditions => ["from_address = ?", ShopSetting[:shop_mail]])
    mail_deliveries.should_not == nil
    mail_deliveries.each do |md|
      md.inquiry_id.should == 3
    end
  end

  describe "send_thanks_mail 注文完了後メール送信" do
    it "ログインユーザの場合 注文完了メールのみ" do
      MailDelivery.delete_all
      FrontSiteMailer.send_thanks_mail(User.find(:first), true, 4)
      result = MailDelivery.find(:all)
      result.size.should == 1
      result.first.mail_div.should == MailTemplate.mail_type_div_id_by_key(:thanks)
    end
    it "非会員ユーザの場合 注文完了メールのみ" do
      MailDelivery.delete_all
      user = User.find(:first)
      user.member_div_key = :no_membership
      FrontSiteMailer.send_thanks_mail(user, false, 4)
      result = MailDelivery.find(:all)
      result.size.should == 1
      result.first.mail_div.should == MailTemplate.mail_type_div_id_by_key(:thanks)
    end
    it "非会員ユーザ(会員登録)の場合 注文完了メール・会員登録" do
      user = User.find(:first)
      user.member_div_key = :membership
      MailDelivery.delete_all
      FrontSiteMailer.send_thanks_mail(user, false, 4)
      result = MailDelivery.find(:all, :order => "id")
      result.size.should == 2
      result[0].mail_div.should == MailTemplate.mail_type_div_id_by_key(:regist_user)
      result[1].mail_div.should == MailTemplate.mail_type_div_id_by_key(:thanks)
    end
    it "非会員ユーザ(会員登録失敗)の場合 注文完了メール・会員登録エラーメール" do
      user = User.find(:first)
      user.member_div_key = :no_membership
      user.regist_error = true
      MailDelivery.delete_all
      FrontSiteMailer.send_thanks_mail(user, false, 4)
      result = MailDelivery.find(:all, :order => "id")
      result.size.should == 2
      result[0].mail_div.should == MailTemplate.mail_type_div_id_by_key(:regist_user_error)
      result[1].mail_div.should == MailTemplate.mail_type_div_id_by_key(:thanks)
    end
  end
end
