module DirFixtures
  class Fixture
    class << self
      def setup(dest_dir, fixture_path, options = nil)
        options = {
          :protected_paths => nil
        }.update(options || {})
        (options[:protected_paths] || []).each do |protected_path|
          violated_dirs = Dir.glob(File.join(fixture_path, protected_path, '**/*'))
          unless violated_dirs.empty?
            raise ProtectViolationError, "#{protected_path} is protected, but fixture '#{fixture_path} includes #{violated_dirs.inspect}'"
          end
        end
        fixture_path_entries = Dir.entries(fixture_path) - %w(. .. .svn)
        
        src_list = fixture_path_entries.map{|path| File.join(fixture_path, path)}
        src_svn_exist = src_list.any?{|src| !Dir.glob(File.join(src, "**/.svn")).empty?}
        
        dest_list = fixture_path_entries.map{|path| File.join(dest_dir, path)}
        dest_svns = dest_list.map{|dest| Dir.glob(File.join(dest, "**/.svn"))}.flatten
        if src_svn_exist
          if !dest_svns.empty?
            raise SubversionViolationError, "descendant directory of #{dest_dir} has .svn directory. #{dest_svns.inspect}"
          end
        end
        FileUtils.cp_r(src_list, dest_dir, :verbose => false)
        if src_svn_exist
          copied_dest_svns = dest_list.map{|dest| Dir.glob(File.join(dest, "**/.svn"))}.flatten
          copied_dest_svns.each do |copied_dest_svn|
            FileUtils.rm_rf(copied_dest_svn, :verbose => false)
          end
        end
      end
    end
    

  end
end
