# -*- coding: utf-8 -*-
require 'yaml'
require 'erb'

module I18n
  module Backend
    class Erb < I18n::Backend::Simple
      protected
      def load_yml(filepath)
        result = load_yaml_impl(filepath)
        # EC-Riderプラグイン用のカスタマイズ
        ecr_plugin_locale_pattern = File.join(RAILS_ROOT, "vendor/plugins/ecr_*/locales/#{File.basename(filepath)}")
        ecr_plugin_yamls = Dir[ecr_plugin_locale_pattern]
        ecr_plugin_yamls.each do |ecr_plugin_yaml_path|
          deep_update(result, load_yaml_impl(ecr_plugin_yaml_path))
        end
        result
      end

      private
      def load_yaml_impl(filepath)
        erb = ERB.new(IO.read(filepath))
        erb.filename = filepath
        YAML.load(erb.result)
      end

      def deep_update(dest, src)
        src.each do |key, value|
          if value.is_a?(Hash) && dest[key].is_a?(Hash)
            deep_update(dest[key], value)
          else
            dest[key] = value
          end
        end
      end

    end
  end
end
